import React from 'react';
import PropTypes from 'prop-types';
import style from './styles.module.less';
import { DefaultPhoto } from 'COMPONENT';
import CallButton from './components/CallButton';
import {
  CALL_LOGIN_STATUS_ABNORMAL,
  CALL_LOGIN_STATUS_CALLING,
  CALL_LOGIN_STATUS_RINGING,
  CALL_WAY_WEB,
  callWayCfg,
} from 'CONST/callConfig';
import {
  LOCAL_CALL_STATUS_BEGIN,
  LOCAL_CALL_STATUS_CALLING,
  LOCAL_CALL_STATUS_HAND_UP,
  LOCAL_CALL_STATUS_END,
  LOCAL_CALL_STATUS_ABNORMAL,
  LOCAL_CALL_STATUS_END_ABNORMAL,
  localCallStatusCfg,
} from './config';
import { errorsHandle, runFunc } from 'UTIL/common';
import { makeCall, hangUp } from 'SERVICE/callCenter';

class CallOutSider extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.makeWebCall = this.makeWebCall.bind(this);
    this.makeCall = this.makeCall.bind(this);
    this.handUp = this.handUp.bind(this);
    this.listenAgentLoginStatus = this.listenAgentLoginStatus.bind(this);
    this.isCalling = this.isCalling.bind(this);
    this.isCallEnd = this.isCallEnd.bind(this);
    this.getCallId = this.getCallId.bind(this);
    this.updateCallStatus = this.updateCallStatus.bind(this);

    this.state = {
      localCallStatus: LOCAL_CALL_STATUS_BEGIN,
    };

    this.callId = null;
  }

  static propTypes = {
    phoneNum: PropTypes.oneOfType([PropTypes.number, PropTypes.string]).isRequired,
    name: PropTypes.string.isRequired,
    customerId: PropTypes.string.isRequired,
    studentId: PropTypes.string.isRequired,
    agentId: PropTypes.string.isRequired,
    agentLoginStatus: PropTypes.number,
    callWay: PropTypes.number,
    dispatch: PropTypes.func.isRequired,
  };

  componentDidMount() {
    this.init();
  }

  componentDidUpdate(prevProps, prevState, snapshot) {
    this.listenAgentLoginStatus(prevProps, this.props, this.state);
  }

  async init() {
    this.callId = null;
    this.setState({ localCallStatus: LOCAL_CALL_STATUS_BEGIN });
    if (this.props.callWay === CALL_WAY_WEB) {
      await new Promise(this.makeWebCall);
    }
    this.callId = await new Promise(this.makeCall);
    if (!this.callId) {
      this.setState({ localCallStatus: LOCAL_CALL_STATUS_ABNORMAL });
    }
  }

  async makeWebCall(resolve, reject) {
    this.props.dispatch({
      type: 'callCenter/makeCall',
      payload: {
        resolve,
        reject,
      },
    });
  }

  async makeCall(resolve) {
    const apiParams = {
      agentId: this.props.agentId,
      customerId: this.props.customerId,
      studentId: this.props.studentId,
    };
    const res = await makeCall(apiParams).catch(errorsHandle());
    if (res && res.data) {
      resolve((res.data || {}).callId);
    } else {
      resolve(null);
    }
  }

  async handUp(callback) {
    const apiParams = {
      agentId: this.props.agentId,
      callId: this.callId,
    };

    await this.setState({ localCallStatus: LOCAL_CALL_STATUS_HAND_UP });
    const res = await hangUp(apiParams).catch(errorsHandle());
    this.setState({ localCallStatus: res ? LOCAL_CALL_STATUS_END : LOCAL_CALL_STATUS_END_ABNORMAL });
    runFunc(callback);
  }

  listenAgentLoginStatus(prevProps, props, state) {
    // 当前电话关断后，本地通话状态不再受外部props影响
    if (state.localCallStatus === LOCAL_CALL_STATUS_END) {
      return;
    }
    // 当agentLoginStatus更新
    if (prevProps.agentLoginStatus !== props.agentLoginStatus) {
      if (prevProps.agentLoginStatus === CALL_LOGIN_STATUS_CALLING) {
        // 上个状态是`通话中`的，本地改为`通话结束`
        this.setState({ localCallStatus: LOCAL_CALL_STATUS_END });
      } else if (prevProps.agentLoginStatus === CALL_LOGIN_STATUS_RINGING) {
        // 上个状态是`接通中`的，本地改为`通话中`或`通话结束`
        if (props.agentLoginStatus === CALL_LOGIN_STATUS_CALLING) {
          this.setState({ localCallStatus: LOCAL_CALL_STATUS_CALLING });
        } else {
          this.setState({ localCallStatus: LOCAL_CALL_STATUS_END });
        }
      }
    }
  }

  isCalling() {
    return this.state.localCallStatus === LOCAL_CALL_STATUS_CALLING;
  }

  isCallEnd() {
    return this.state.localCallStatus === LOCAL_CALL_STATUS_END;
  }

  getCallId() {
    return this.callId;
  }

  updateCallStatus(resolve) {
    const _self = this;
    this.props.dispatch({
      type: 'callCenter/fetchToUpdAgentInfo',
      payload: {
        callback(agentObj) {
          const agentLoginStatus = (agentObj || {}).agentLoginStatus || CALL_LOGIN_STATUS_ABNORMAL;
          // 非响铃及通话中的，将本地状态改为通话结束
          if ([CALL_LOGIN_STATUS_CALLING, CALL_LOGIN_STATUS_RINGING].every((item) => item !== agentLoginStatus)) {
            _self.setState({ localCallStatus: LOCAL_CALL_STATUS_END });
          }
          resolve(agentLoginStatus);
        },
      },
    });
  }

  render() {
    return (
      <div className={style.callOutSider}>
        <div className={style.phoneNum}>{this.props.phoneNum || '-'}</div>

        <div className={style.photoWrapper}>
          <DefaultPhoto />
        </div>

        <div className={style.personName}>{this.props.name || '-'}</div>

        <div className={style.callStatus}>
          {localCallStatusCfg[this.state.localCallStatus]}
          {/* TODO: 通话计时 */}
        </div>

        <div className={style.callButtonWrapper}>
          {this.props.callWay === CALL_WAY_WEB ? (
            /* 后续可加入接听逻辑 */
            <CallButton
              type="HAND_UP"
              onClick={this.handUp}
              disabled={this.state.localCallStatus !== LOCAL_CALL_STATUS_CALLING}
            />
          ) : (
            <div className={style.callStatusTips}>
              {this.state.localCallStatus === LOCAL_CALL_STATUS_END
                ? '通话结束'
                : `${callWayCfg[this.props.callWay]}接听中...`}
            </div>
          )}
        </div>
      </div>
    );
  }
}

export default CallOutSider;
