import React from 'react';
import { Tree, Input } from 'antd';
import { Modal, NoData } from 'COMPONENT';
import { getResourcePoolWithVolume } from 'SERVICE/system';
import {
  filterTreeData,
  transformArrayToTree,
  filterDefaultExpandKeys,
  formatNodeName,
  getParentKey,
  getNodeKey,
} from './util';
import style from './index.less';

const TreeNode = Tree.TreeNode;
const Search = Input.Search;

/**
 * 客户资源池树弹窗
 * tips: singleChecked与checkLeafOnly结合使用，若非叶子节点允许选中则无法实现单选功能
 *
 * @param props.title {string} 弹窗标题
 * @param props.loading {boolean} 外层控制的loading(外层正在请求接口时使用)
 * @param props.singleChecked {boolean} 是否单选，默认true
 * @param props.checkLeafOnly {boolean} 是否只允许勾选叶子节点，默认true
 * @param props.initTreeParam {object} 初始化书树节点的请求参数
 * @param props.onOk {func} 确认回调，回调参数为勾选选中的树节点ID、勾选选中的树节点
 * @param props.onClose {func} 弹窗关闭
 */
class ModalCustomerPoolTree extends React.Component {
  static defaultProps = {
    title: '客户资源分配节点',
    loading: false,
    singleChecked: true,
    checkLeafOnly: true,
    initTreeParam: null,
    onOk: () => {},
    onClose: () => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      originData: null, // 一维的树节点数据(搜索使用)
      treeData: null, // 树型结构数据
      searchValue: null, // 输入框值
      selectedKeys: [], // 点击的节点key(id)
      selectedNodes: [], // 点击的节点
      checkedKeys: [], // 勾选选中的节点key(id)
      checkedNodes: [], // 勾选选中的节点
      expandedKeys: [], // 展开的节点key(id)
      autoExpandParent: true, // 自动展开父节点
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化资源树
   */
  initContent = async () => {
    this.toggleLoading();

    const params = this.props.initTreeParam || {};
    const response = await getResourcePoolWithVolume(params).catch(() => this.toggleLoading());
    const data = filterTreeData(response);
    const treeData = transformArrayToTree(data);
    const defaultExpandKeys = filterDefaultExpandKeys(treeData);

    this.setState({
      originData: data,
      treeData: treeData,
      expandedKeys: defaultExpandKeys,
    });
    this.toggleLoading();
  };

  /**
   * loading
   */
  toggleLoading = () => {
    this.setState((prevState) => {
      return {
        loading: !prevState.loading,
      };
    });
  };

  /**
   * 渲染树节点
   *
   * @param data 树节点数据
   * @return {<TreeNode>[]}
   */
  loopNode = (data = []) => {
    const { searchValue } = this.state;
    const { checkLeafOnly } = this.props;

    return data.map((item) => {
      const { id, name, children, isParent } = item;
      const title = formatNodeName(name, searchValue);

      if (children && children.length) {
        const disableCheckbox = isParent && checkLeafOnly;

        return (
          <TreeNode key={id} title={title} disableCheckbox={disableCheckbox}>
            {this.loopNode(children)}
          </TreeNode>
        );
      }

      return <TreeNode key={id} title={title} />;
    });
  };

  /**
   * 勾选中节点
   *
   * @param checkedKeys {array} 选中树节点key(id)
   * @param info
   */
  onCheck = (checkedKeys, info) => {
    if (this.props.singleChecked) {
      this.doCheckSingle(checkedKeys, info);
    } else {
      this.doCheckMultiple(checkedKeys, info);
    }
  };
  /**
   * 单选
   */
  doCheckSingle = (checkedKeys, info) => {
    this.setState((prevState) => {
      const { checkedNodes } = info;
      const { checkedKeys: oldCheckedKeys } = prevState;
      let resultCheckedKeys = [];
      let resultCheckedNodes = [];

      if (!oldCheckedKeys || oldCheckedKeys.length <= 0) {
        resultCheckedKeys = checkedKeys;
        resultCheckedNodes = checkedNodes.map((item) => {
          return {
            id: item.key,
            name: item.props.title,
          };
        });
      } else {
        // 找到最新选中那一个并保存
        checkedKeys.forEach((key, index) => {
          const isOldKey = oldCheckedKeys.some((k) => k === key);
          const item = checkedNodes[index];

          if (!isOldKey) {
            resultCheckedKeys.push(key);
            resultCheckedNodes.push({
              id: item.key,
              name: item.props.title,
            });
          }
        });
      }

      return {
        checkedKeys: resultCheckedKeys,
        checkedNodes: resultCheckedNodes,
      };
    });
  };
  /**
   * 多选
   */
  doCheckMultiple = (checkedKeys, info) => {
    const { checkedNodes } = info;
    const resultNodes = checkedNodes.map((item) => {
      return {
        id: item.key,
        name: item.props.title,
      };
    });

    this.setState({
      checkedKeys,
      checkedNodes: resultNodes,
    });
  };

  /**
   * 点击树节点，展开树节点
   *
   * @param selectedKeys {array} 树节点key(id)
   * @param info {object} 事件详情
   */
  onClick = (selectedKeys, info) => {
    this.doSelect(selectedKeys, info);
    this.doExpand(info);
  };
  doSelect = (selectedKeys, info) => {
    const { selectedNodes } = info;
    const resultNodes = selectedNodes.map((item) => {
      return {
        id: item.key,
        name: item.props.title,
      };
    });

    this.setState({
      selectedKeys,
      selectedNodes: resultNodes,
    });
  };
  doExpand = (info) => {
    const { node } = info;
    const { eventKey, children, expanded } = node.props;

    const isParent = children && children.length > 0;
    if (!isParent) {
      return;
    }

    this.setState((prevState) => {
      const oldExpandedKeys = prevState.expandedKeys || [];
      let resultExpandedKeys = [];

      // 展开状态不要被selected干扰
      if (expanded) {
        resultExpandedKeys = oldExpandedKeys.filter((oldKey) => oldKey !== eventKey);
      } else {
        resultExpandedKeys = oldExpandedKeys.concat([eventKey]);
      }

      return {
        expandedKeys: resultExpandedKeys,
        autoExpandParent: false,
      };
    });
  };

  /**
   * 展开
   *
   * @param expandedKeys {array} 展开的节点key(id)
   * @param info {object} node节点信息
   */
  onExpand = (expandedKeys, info) => {
    this.setState({
      expandedKeys,
      autoExpandParent: false,
    });
  };

  /**
   * 搜索
   *
   * @param e
   */
  onInputChange = (e) => {
    const { originData, treeData } = this.state;
    const value = e.target.value;
    const expandedKeys = originData
      .map((item) => {
        if (item.name.indexOf(value) > -1) {
          return getParentKey(getNodeKey(item), treeData);
        }

        return null;
      })
      .filter((item, i, self) => {
        return item && self.indexOf(item) === i;
      });

    this.setState({
      searchValue: value,
      expandedKeys,
      autoExpandParent: true,
    });
  };

  /**
   * 确认
   */
  onOk = () => {
    this.props.onOk(this.state.checkedKeys, this.state.checkedNodes);
  };

  render() {
    const { loading, treeData } = this.state;
    const hasData = treeData && treeData.length > 0;
    const showNoData = !loading && !hasData;

    return (
      <Modal
        title={this.props.title}
        width={600}
        loading={loading || this.props.loading}
        okBtnShow={true}
        cancelBtnShow={true}
        closable={true}
        onOk={this.onOk}
        onClose={this.props.onClose}
        className={style.modalWrap}
      >
        {hasData && [
          <Search key={1} style={{ marginBottom: 5 }} placeholder="输入名称搜索" onChange={this.onInputChange} />,
          <div key={2} className={style.treeWrap}>
            <Tree
              checkable={true}
              onCheck={this.onCheck}
              checkedKeys={this.state.checkedKeys}
              onSelect={this.onClick}
              onExpand={this.onExpand}
              expandedKeys={this.state.expandedKeys}
              autoExpandParent={this.state.autoExpandParent}
            >
              {this.loopNode(treeData)}
            </Tree>
          </div>,
        ]}

        {showNoData && <NoData />}
      </Modal>
    );
  }
}

export default ModalCustomerPoolTree;
