import React from 'react';
import PropTypes from 'prop-types';
import styles from './index.less';
import { Form, Input, Row, Col } from 'antd';
import { Modal, TextArea, Section, List, TextLink } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import { desensitizationPhoneNumber } from 'UTIL/common';
import * as loaders from './loader';
import * as utils from './util';
import { confirmStatusDict } from './constant';
import { saveCustomerContactChangeLog } from 'SERVICE/customer';
import { runFunc, errorsHandle } from 'UTIL/common';

const errorsHandler = errorsHandle();
const FormItem = Form.Item;
const formItemCol = {
  labelCol: { span: 5 },
  wrapperCol: { span: 18 },
};
const phoneRegExp = /^1\d{10}$/;
const { COL_WIDTH_M, COL_WIDTH_L } = List;

/**
 * 修改客户手机号码
 */
class Index extends React.Component {
  static propTypes = {
    customerId: PropTypes.string.isRequired,
    customerName: PropTypes.string.isRequired,
    customerContact: PropTypes.string.isRequired,
    loading: PropTypes.bool,
    onClose: PropTypes.func.isRequired,
    onOk: PropTypes.func,
  };

  static defaultProps = {
    loading: false,
  };

  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      contact: null, // 新手机号码
      remark: null, // 修改原因
      logList: null, // 历史修改记录
      okBtnDisabled: true, // 提交按钮禁用状态
    };

    this.logColumns = [
      { title: '发起时间', dataIndex: 'createTime', key: 'createTime', width: 160 },
      { title: '修改人', dataIndex: 'operatorName', key: 'operatorName', width: COL_WIDTH_M },
      { title: '原手机号', dataIndex: 'oldContact', key: 'oldContact', width: COL_WIDTH_M },
      { title: '新手机号', dataIndex: 'newContact', key: 'newContact', width: COL_WIDTH_M },
      { title: '修改原因', dataIndex: 'modifyReason', key: 'modifyReason', width: COL_WIDTH_M, lines: 2 },
      {
        title: '家长确认状态',
        dataIndex: 'confirmStatus',
        key: 'confirmStatus',
        lines: 2,
        width: COL_WIDTH_L,
        dict: confirmStatusDict,
      },
      { title: '修改时间', dataIndex: 'changeTime', key: 'changeTime', width: 160 },
      {
        title: '操作',
        dataIndex: 'action',
        key: 'action',
        width: COL_WIDTH_M,
        render: (text, record) => {
          const canDoReSend = utils.calcConfirmStatusIsWait(record.confirmStatus);

          return (
            canDoReSend && (
              <TextLink
                onClick={() => {
                  this.onReSendMessage(record);
                }}
              >
                重新发送
              </TextLink>
            )
          );
        },
      },
    ];
  }

  componentDidMount() {
    this.initForm();
    this.initApplyRecord();
  }

  /**
   * 初始化表单
   */
  initForm = () => {
    const { customerName, customerContact } = this.props;
    const { setFieldsValue } = this.props.form;

    setFieldsValue({
      customerName,
      customerContact,
    });
  };

  /**
   * 初始化申请记录
   */
  initApplyRecord = () => {
    loaders.loadApplyRecord({ customerId: this.props.customerId }).then((response) => {
      this.setState({
        logList: response.data,
      });
    });
  };

  /**
   * 手机号码改变
   */
  onContactChange = () => {
    this.props.form.validateFields((err, values) => {
      if (err && err.contact) {
        this.setState({ okBtnDisabled: true });

        return;
      }

      if (values.contact === this.props.customerContact) {
        showMessage('请输入与原号码不同的手机号码', 'error');
        this.setState({ okBtnDisabled: true });

        return;
      }

      this.checkContactApplyValid(values.contact);
    });
  };

  /**
   * 检查当前号码是否有申请记录
   *
   * @param contact {string} 手机号码
   */
  checkContactApplyValid = (contact) => {
    loaders.checkContactIsExist({ contact }).then(({ isExist }) => {
      if (isExist) {
        showAlert('由于修改后的客户号码在系统里已存在，不允许修改为该号码。');
        this.setState({ okBtnDisabled: true });

        return;
      }

      if (utils.calcIsExistApplyRecord(contact, this.state.logList)) {
        const dContact = desensitizationPhoneNumber(contact);
        const content = `系统中已有用户提交申请，将本客户号码更改为${dContact}，请勿重复提交申请，可在下方修改记录当中直接重发短信。`;

        showAlert(content);
        this.setState({ okBtnDisabled: true });

        return;
      }

      this.setState({ okBtnDisabled: false });
    });
  };

  /**
   * 表单提交校验
   */
  onSubmit = () => {
    this.props.form.validateFieldsAndScroll((err, values) => {
      if (err) {
        showMessage('请完善信息再进行操作!', 'error');
        return;
      }

      this.doSubmit(values);
    });
  };

  /**
   * 提交表单，发送确认短信
   *
   * @param values {Object} 表单参数
   */
  doSubmit = async (values) => {
    this.setState({ loading: true });
    const { customerId, customerContact } = this.props;
    const { contact, remark } = values;
    const apiParams = {
      cusId: customerId,
      oldContact: customerContact,
      newContact: contact,
      modifyReason: remark,
    };

    const res = await saveCustomerContactChangeLog(apiParams).catch(errorsHandler);

    if (res && res.data) {
      runFunc(this.props.onOk, res.data, contact);
    } else {
      this.setState({ loading: false });
    }
  };

  /**
   * 重新发送短信
   *
   * @param record {Object} 历史修改记录
   */
  onReSendMessage = (record) => {
    runFunc(this.props.onOk, record.id, record.newContact);
  };

  render() {
    const { getFieldDecorator } = this.props.form;
    const { logList } = this.state;

    return (
      <Modal
        title="修改联系方式"
        okText="发送确认信息"
        width={800}
        loading={this.state.loading || this.props.loading}
        okBtnDisabled={this.state.okBtnDisabled}
        okBtnShow={true}
        cancelBtnShow={true}
        onOk={this.onSubmit}
        onClose={this.props.onClose}
        className={styles.modalWrap}
      >
        <Form>
          <Row>
            <Col span={24}>
              <FormItem label="客户姓名" {...formItemCol}>
                {getFieldDecorator('customerName', {})(<Input disabled={true} />)}
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem label="客户手机号码(原)" {...formItemCol}>
                {getFieldDecorator('customerContact', {})(<Input disabled={true} />)}
              </FormItem>
            </Col>
            <Col span={24}>
              <div className={styles.line} />
            </Col>
            <Col span={24}>
              <FormItem label="客户手机号码(新)" {...formItemCol} required={true}>
                {getFieldDecorator('contact', {
                  rules: [
                    {
                      required: true,
                      pattern: phoneRegExp,
                      message: '请填写正确的联系方式',
                    },
                  ],
                })(<Input onBlur={this.onContactChange} />)}
                <p className={styles.tips}>
                  修改号码需发送确认信息，可发送短信或二维码海报，家长登录页面确认后才能成功修改
                </p>
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem label="修改原因" {...formItemCol} required={true}>
                {getFieldDecorator('remark', {
                  rules: [
                    {
                      required: true,
                      message: '请填写修改原因',
                    },
                  ],
                })(<TextArea maxLength={200} placeholder="请写明修改手机号码的原因" />)}
              </FormItem>
            </Col>
          </Row>
        </Form>

        <Section title="历史修改记录">
          <List columns={this.logColumns} dataSource={logList || []} />
        </Section>
      </Modal>
    );
  }
}

export default Form.create()(Index);
