import React from 'react';
import PropTypes from 'prop-types';
import style from './index.less';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import Modal from 'COMPONENT/Modal';
import Loading from 'COMPONENT/Loading';
import {
  findCourseByClassTwoId,
  getMiniClassCourseDetail,
  getSSChangeClsCourse,
  getSSChangeClsMiniClsCourse,
} from 'SERVICE/course';
import CourseTimeDesk from './CourseTimeDesk';
import { clone } from 'lodash';
import { runFunc } from 'UTIL/common';
import moment from 'moment';

// 产品类型只能为双师/小班
const TWO_TEACHER_VAL = 'TWO_TEACHER';
const SMALL_CLASS_VAL = 'SMALL_CLASS';

const momentFormat = 'YYYY-MM-DD HH:mm';

// 容器
const Container = (props) => {
  const classPrefix = 'Container';
  return (
    <div className={style[classPrefix]}>
      <Loading loading={props.loading} />
      {props.children}
    </div>
  );
};

// 无数据
const NoData = (props) => {
  return (
    <div className={style['NoData']}>
      <div className={style['NoData-tips']}>{props.text}</div>
    </div>
  );
};

// 双师/小班 选择产品课时数
class ModalSelectCourseTime extends React.Component {
  constructor(props) {
    super(props);

    this.checkBeforeInit = this.checkBeforeInit.bind(this);
    this.loadCourseList = this.loadCourseList.bind(this);
    this.buildCourseList = this.buildCourseList.bind(this);
    this.disabledAdapter = this.disabledAdapter.bind(this);
    this.beforeNowMaxCourseDisabledAdapter = this.beforeNowMaxCourseDisabledAdapter.bind(this);
    this.updateSelect = this.updateSelect.bind(this);
    this.confirm = this.confirm.bind(this);
    this.markAndSelectCourses = this.markAndSelectCourses.bind(this);

    this.state = {
      loading: false,
      modalVisible: false,
      list: [],
      selectedKeys: [],
      noDataText: '正在加载...',
      today: moment(new Date()).format(momentFormat),
    };

    this.isXh = props.productType === SMALL_CLASS_VAL;
    this.modalRef = null;
    this.courseDateReg = new RegExp(/(-)/g);
  }

  static classPrefix = 'ModalSelectCourseTime';

  static propTypes = {
    classId: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired,
    productType: PropTypes.string.isRequired,
    onCancel: PropTypes.func.isRequired,
    onConfirm: PropTypes.func.isRequired,
    onChange: PropTypes.func,
    title: PropTypes.string,
    multiSelect: PropTypes.bool,
    autoSelectAfter: PropTypes.bool,
    beforeNowDisabled: PropTypes.bool,
    beforeNowDays: PropTypes.number,
    noDataDirectCallback: PropTypes.func,
    footerTipsRender: PropTypes.func,
    showConfirmTip: PropTypes.bool,
    confirmTip: PropTypes.string,
    noSelectedDisabled: PropTypes.bool,
    noSelectedCourseConfirmTip: PropTypes.string,
    /**
     * 默认选中
     *
     * if function then @return {[number]}
     * */
    defaultSelect: PropTypes.oneOfType([PropTypes.func, PropTypes.arrayOf(PropTypes.number)]),
    disabledAdapter: PropTypes.func,
    beforeNowMaxCourse: PropTypes.number,
    // 最多可选择课时数
    availableCourseNumber: PropTypes.number,
    restNatureOfferNum: PropTypes.number,
    // 已购买未扣费小时
    totalUnConsumedNatureHours: PropTypes.number,
  };

  static defaultProps = {
    title: '选择上课时间',
    // autoSelectAfter优先级高于multiSelect
    autoSelectAfter: false,
    multiSelect: false,
    beforeNowDisabled: false,
    beforeNowDays: 0,
    showConfirmTip: false,
    confirmTip: '',
    noSelectedDisabled: false,
    beforeNowMaxCourse: 0,
    noSelectedCourseConfirmTip: '',
    totalUnConsumedNatureHours: undefined,
  };

  componentDidMount() {
    this.checkBeforeInit();
    this.loadCourseList();
  }

  // 未传classId/productType时报错
  checkBeforeInit() {
    if (!this.props.classId || this.props.classId === '') {
      throw new Error('ModalSelectCourseTime.props.classId为必传项！');
    } else if (!this.props.productType || this.props.productType === '') {
      throw new Error('ModalSelectCourseTime.props.productType为必传项！');
    }
  }

  async loadCourseList() {
    this.setState({ loading: true });
    let isTwoTeacher = this.props.productType === TWO_TEACHER_VAL;
    let fetchFn = isTwoTeacher ? findCourseByClassTwoId : getMiniClassCourseDetail;
    let fetchParams = isTwoTeacher ? { twoTeacherClassTwoId: this.props.classId } : { miniClassId: this.props.classId };
    if (this.props.modalType === 'changeClass') {
      // 转班，用新接口
      fetchFn = isTwoTeacher ? getSSChangeClsCourse : getSSChangeClsMiniClsCourse;
      fetchParams = {
        oldClassId: this.props.oldClassId,
        newClassId: this.props.classId,
        productType: this.props.productType,
        studentId: this.props.studentId,
      };
    }
    fetchParams.page = 1;
    fetchParams.rows = 100;

    let { rows, timeStamp } = await fetchFn(fetchParams, true);

    const today = moment(timeStamp).format(momentFormat);

    let payload = {
      list: [],
      loading: false,
      noDataText: '该班暂无课程',
      modalVisible: false,
      today,
      tagDay: today,
    };

    if (this.props.beforeNowDisabled) {
      payload.tagDay = moment(timeStamp).subtract(this.props.beforeNowDays, 'day').format(momentFormat);
    }

    if (rows.length > 0) {
      const courses = this.buildCourseList(rows, payload.today, payload.tagDay);
      const { list, selectedKeys } = this.markAndSelectCourses(courses, rows);
      payload.list = list;
      payload.selectedKeys = selectedKeys;
      payload.modalVisible = true;

      if (this.props.defaultSelect) {
        if (Array.isArray(this.props.defaultSelect)) {
          payload.selectedKeys = this.props.defaultSelect;
        } else if (typeof this.props.defaultSelect === 'function') {
          payload.selectedKeys = runFunc(this.props.defaultSelect, payload.list, payload.today, payload.tagDay);
          if (!Array.isArray(payload.selectedKeys)) {
            payload.selectedKeys = [];
          }
        }
      }
      if (this.props.modalType === 'changeClass') {
        // 转班 自动选择且不能修改
        let keys = [];
        payload.list.forEach((v, key) => {
          keys.push(key);
        });
        this.setState(payload);
        this.updateSelect(keys);
        return;
      }
    } else if (this.props.noDataDirectCallback) {
      return this.props.noDataDirectCallback();
    }

    this.setState(payload);
  }

  /**
   *
   * @param {*} list
   * @desc 标记从后往前选择的满足剩余课时之外的可选课程及初始化选择keys，没传剩余课时数totalUnConsumedNatureHours属性不做处理
   * @return Object { list: Array, selectedKeys: Array }
   */

  markAndSelectCourses(list, rows) {
    let { totalUnConsumedNatureHours } = this.props;
    const selectedKeys = [];
    const results = [];
    let i = list.length - 1;
    if (totalUnConsumedNatureHours === undefined) {
      return {
        list,
        selectedKeys,
      };
    }
    while (i >= 0) {
      const course = list[i];
      const dataSource = rows[i];
      console.log({ course });
      if (!course.disabled && course.courseStatusBool) {
        // 可选且未结课
        // 一讲课时数 默认为1
        const {
          _origin: { courseHours = 1 },
        } = course;
        totalUnConsumedNatureHours -= (dataSource.courseNatureMinutes || 60) / 60;
        if (totalUnConsumedNatureHours < 0) {
          course.insufficient = true;
        } else {
          selectedKeys.push(i);
        }
      }
      results.unshift(course);
      i--;
    }
    return {
      list: results,
      selectedKeys,
    };
  }

  buildCourseList(list, today, tagDay) {
    return this.disabledAdapter(
      list.map((item, index) => ({
        courseId: item.courseId,
        miniClassCourseId: item.miniClassCourseId,
        courseDate: item.courseDate,
        courseBeginTime: item.courseTime,
        courseEndTime: item.courseEndTime,
        courseDateText: item.courseDate.replace(this.courseDateReg, '.'),
        name: this.props.modalType === 'changeClass' ? `第 ${item.courseNum} 讲` : `第 ${index + 1} 讲`,
        courseStatusName: item.courseStatusName,
        courseStatusBool: item.courseStatus === 'NEW',
        courseDay: `${item.courseDate} ${item.courseTime}`,
        disabled: false,
        _origin: item,
      })),
      today,
      tagDay,
    );
  }

  disabledAdapter(list, today, tagDay) {
    if (typeof this.props.disabledAdapter === 'function') {
      return this.props.disabledAdapter(list, today, tagDay);
    }

    if (this.props.beforeNowDisabled) {
      if (this.props.beforeNowMaxCourse) {
        return this.beforeNowMaxCourseDisabledAdapter(list, today, tagDay);
      }

      return list.map((course) => {
        course.disabled = `${course.courseDate} ${course.courseBeginTime}` < tagDay;
        return course;
      });
    }

    return list;
  }

  beforeNowMaxCourseDisabledAdapter(list, today, tagDay) {
    if (today < list[0].courseDay) {
      return list;
    }

    const len = list.length;

    let baseIndex = -1;
    if (today < list[len - 1].courseDay) {
      baseIndex = list.findIndex((item) => today < item.courseDay);
    } else {
      baseIndex = len;
    }

    let tagIndex = list.findIndex((item) => tagDay < item.courseDay);
    if (tagIndex === -1) {
      if (tagDay > list[len - 1].courseDay) {
        tagIndex = len;
      } else {
        tagIndex = 0;
      }
    }

    const maxCourse = this.props.beforeNowMaxCourse;
    const end = baseIndex - tagIndex >= maxCourse ? baseIndex - maxCourse : tagIndex;
    for (let i = 0; i < end; i++) {
      list[i].disabled = true;
    }

    return list;
  }

  updateSelect(keys) {
    this.setState({ selectedKeys: keys }, () => {
      if (this.props.onChange) {
        this.props.onChange(this.state.selectedKeys, this.state.list);
      }
    });
  }

  confirm() {
    const { selectedKeys } = this.state;
    if (selectedKeys.length === 0) {
      const { title, noSelectedCourseConfirmTip } = this.props;
      const tip = noSelectedCourseConfirmTip
        ? noSelectedCourseConfirmTip
        : `请先${title.indexOf('选择') === -1 ? '选择' : ''}${title}！`;
      this.modalRef.showOkTips(tip);
      return;
    }
    let index = selectedKeys;
    let item = [];
    let list = clone(this.state.list);
    if (this.props.autoSelectAfter || this.props.multiSelect) {
      selectedKeys.forEach((key) => {
        item.push(list[key]);
      });
    } else {
      index = selectedKeys[0];
      item = list[index];
    }
    this.props.onConfirm(item, index, list);
  }

  renderTipText() {
    const { list, selectedKeys } = this.state;
    const { availableCourseNumber, restNatureOfferNum, totalUnConsumedNatureHours } = this.props;
    if (availableCourseNumber === undefined) return null;
    const courses = this.state.list.filter((course) => !course.disabled && course.courseStatusBool);
    const selectedCourses = list.filter((course, i) => !course.disabled && selectedKeys.includes(i));

    let leftCourse = 0;
    let selectedCourse = 0;
    const courseLength = courses.length;
    let tempAvailableCourseNumber = 0;

    if (courses.length) {
      // 最终要算每讲课时数
      let sumCourseHours = 0;
      let sumCourseMinutes = 0;
      while (courses.length) {
        const {
          _origin: { courseHours = 1, courseMinutes = 1 },
        } = courses.pop();
        sumCourseHours += courseHours;
        sumCourseMinutes += courseMinutes;
      }
      leftCourse = Number(formatCourseToHour(sumCourseHours, sumCourseMinutes / courseLength));
      tempAvailableCourseNumber = Number(formatCourseToHour(availableCourseNumber, sumCourseMinutes / courseLength));
    } else {
      tempAvailableCourseNumber = restNatureOfferNum;
    }

    if (selectedCourses.length) {
      let sumCourseMinutes = 0;
      while (selectedCourses.length) {
        const {
          _origin: { courseNatureMinutes = 60 },
        } = selectedCourses.pop();
        sumCourseMinutes += courseNatureMinutes;
      }
      selectedCourse = Number(sumCourseMinutes / 60);
    }

    // 学生本产品剩余小时4，本小班总小时6，学生最多可报读小时4，当前已勾选报读小时4
    let text = `学生本产品剩余小时${tempAvailableCourseNumber}、`;
    // if (typeof totalUnConsumedNatureHours !== 'undefined') {
    //   text = `学生本产品剩余小时${totalUnConsumedNatureHours}、`;
    // }
    return `${text}本小班总小时${
      totalUnConsumedNatureHours || 0
    }，学生最多可报读小时${leftCourse}，当前已勾选报读小时${selectedCourse}`;
  }

  render() {
    const prefix = ModalSelectCourseTime.classPrefix;
    const tipText = this.renderTipText();

    const ModalMain = (
      <Modal
        title={this.props.title}
        loading={this.state.loading}
        className={style[prefix]}
        widthSize="M"
        onCancel={this.props.onCancel}
        onOk={this.confirm}
        okBtnDisabled={this.props.noSelectedDisabled ? this.state.selectedKeys.length === 0 : false}
        showConfirmTip={this.props.showConfirmTip}
        confirmTip={this.props.confirmTip}
        ref={(ref) => (this.modalRef = getWrappedInstance(ref))}
        footerTipText={runFunc(this.props.footerTipsRender, this.state.selectedKeys, this.state.list) || ''}
      >
        {this.state.list.length > 0 ? (
          <div>
            {tipText && <p className={style.courseTip}>{tipText}</p>}
            <CourseTimeDesk
              dataSource={this.state.list}
              selectedKeys={this.state.selectedKeys}
              onKeysChange={this.updateSelect}
              multiSelect={this.props.multiSelect}
              autoSelectAfter={this.props.autoSelectAfter}
              beforeNowDisabled={this.props.beforeNowDisabled}
              modalType={this.props.modalType}
            />
          </div>
        ) : (
          <NoData text={this.state.noDataText} />
        )}
      </Modal>
    );

    let component = ModalMain;
    if (this.props.noDataDirectCallback) {
      component = <Container loading={this.state.loading}>{this.state.modalVisible && ModalMain}</Container>;
    }

    return component;
  }
}

export default ModalSelectCourseTime;
