import React from 'react';
import { Icon, Upload } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Modal as AntdModal } from 'antd';
import './styles.less';
import PropTypes from 'prop-types';

/**
 * 上传附件组件
 *
 * @param props.text {string} 按钮文字
 * @param props.size {number} 按钮大小
 * @param props.value {object} 阿里云附件结果
 * @param props.disabled {boolean} 是否禁用 {true: 是, false: 否} 默认false
 * @param props.viewOnly {boolean} 是否只读 {true: 是, false: 否} 默认false
 * @param props.params {object?}
 * @param props.uploadComplete {function} 上传成功回调 回调参数为file
 * @param props.deleteComplete {function} 删除成功回调 回调参数为file
 */
class UploadFile extends React.Component {
  static propTypes = {
    beforeUpload: PropTypes.func,
    validateFileTypes: PropTypes.array,
  };

  static defaultProps = {
    text: '上传',
    size: 32,
    value: null,
    disabled: false,
    viewOnly: false,
    params: null,
    uploadComplete: () => {},
    deleteComplete: () => {},
  };

  constructor(props) {
    super(props);

    const file = props.value || null;

    this.state = {
      file: file,
      btnCover: false, // 切换显示 按钮层
    };
  }

  componentWillReceiveProps = (nextProps) => {
    this.setState({ file: nextProps.value || null });
  };

  /**
   * 附件上传时不断调用
   * 当上传成功时，可获取到全部file数据
   * 调用父组件传递进来的callback
   */
  onUploadChange = (info) => {
    const { params, uploadComplete } = this.props;
    const file = info;

    if (file.src && file.src !== '') {
      let newFile = {
        ...file,
        fileId: file['x:fileid'],
        fileName: file['name'],
        ...params,
      };

      if (!newFile.fileId && !newFile.fileName) {
        showMessage('附件上传失败，请联系管理人员！', 'error');
        return;
      }

      this.setState({ file: newFile });
      uploadComplete(newFile);
      showMessage('附件上传成功！', 'success');
    }
  };

  /**
   * 附件上传前进行校验
   * 必需返回一个fileList [Array]
   */
  checkLength = (files) => {
    if (!files || files.length <= 0) {
      return [];
    }

    const file = files[0];
    const fileType = file.type || file.name.slice(file.name.lastIndexOf('.') + 1);
    const valid =
      fileType &&
      this.props.validateFileTypes.some((type) => {
        return fileType.indexOf(type) > -1;
      });

    if (!valid) {
      showMessage('附件格式不正确！', 'warning');
      return [];
    }

    return files;
  };

  /**
   * 设置插件大小
   */
  setSize = (size, type) => {
    let result = {};

    switch (type) {
      case 1:
        result.width = size;
        result.height = size;
        break;
      case 2:
        result.width = size - 20;
        result.height = size - 20;
        break;
      case 3:
        let icon = parseInt(size / 5);
        result.width = icon;
        result.height = icon;
        break;
      default:
        result.width = 100;
        result.height = 100;
        break;
    }

    return result;
  };

  setIconSize = (size) => {
    return parseInt(size / 5);
  };

  /**
   * 删除附件
   */
  deleteImg = (e) => {
    const { deleteComplete } = this.props;
    const { file } = this.state;
    const that = this;

    AntdModal.confirm({
      title: '是否删除此附件？',
      content: '',
      okType: 'danger',
      onOk() {
        deleteComplete(file.fileId);
        that.setState({ file: null, btnCover: false });
      },
    });

    this.setState((prevState) => {
      return { btnCover: false };
    });
  };

  beforeUpload = (files) => {
    if (this.props.beforeUpload) {
      return this.props.beforeUpload(files);
    }

    return this.checkLength(files);
  };

  render() {
    const { editing, validator, onChange, size, text, uploadComplete, disabled, viewOnly } = this.props;
    const { file, btnCover } = this.state;

    let size_1 = this.setSize(size, 1);

    let iconSize = this.setIconSize(size);

    return (
      <div className="UploadImg">
        <div className={!file ? 'hom-upload-img' : ''} style={!file ? size_1 : {}}>
          {!file && (
            <div className="hom-upload-img-picker">
              <Upload
                hasExtension={true}
                onChange={this.onUploadChange}
                multiple={false}
                beforeUpload={this.beforeUpload}
              >
                <div className="hom-upload-img-picker-btn">
                  <div>
                    <Icon type="plus" size={iconSize} />
                  </div>
                  <div>{text}</div>
                </div>
              </Upload>
            </div>
          )}
          {file && file.fileName !== '' && (
            <div
            // onMouseEnter={this.btnCoverToggle}
            // onMouseLeave={this.btnCoverToggle}
            >
              <div>
                <span>{file.fileName}</span>
                {!disabled && !viewOnly && (
                  <span className="deleteFile" onClick={(e) => this.deleteImg(e)}>
                    删除附件
                  </span>
                )}
              </div>
            </div>
          )}
        </div>
      </div>
    );
  }
}

export default UploadFile;
