import React from 'react';
import { Icon, Popover, QrcodeFileUpload, Upload, PreviewImgs } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Modal as AntdModal } from 'antd';
import './styles.less';
import PropTypes from 'prop-types';

const validateFileTypes = ['png', 'jpg', 'jpeg', 'bmp'];

/**
 * 上传图片组件
 *
 * @param props.text {string} 按钮文字
 * @param props.size {number} 按钮大小
 * @param props.value {object} 阿里云图片结果
 * @param props.disabled {boolean} 是否禁用 {true: 是, false: 否} 默认false
 * @param props.viewOnly {boolean} 是否只读 {true: 是, false: 否} 默认false
 * @param props.params {object?}
 * @param props.uploadComplete {function} 上传成功回调 回调参数为file
 * @param props.deleteComplete {function} 删除成功回调 回调参数为file
 */
class UploadImg extends React.Component {
  static propTypes = {
    beforeUpload: PropTypes.func,
  };

  static defaultProps = {
    text: '上传',
    size: 32,
    value: null,
    disabled: false,
    viewOnly: false,
    params: null,
    qrCodeUploadVisible: true,
    uploadComplete: () => {},
    deleteComplete: () => {},
  };

  constructor(props) {
    super(props);

    const file = props.value || null;

    this.state = {
      file: file,
      btnCover: false, // 切换显示 按钮层
      previewImg: false, // 图片预览
    };
  }

  componentWillReceiveProps = (nextProps) => {
    this.setState({ file: nextProps.value || null });
  };

  /**
   * 二维码传图回调
   *
   * @param info {array} [{filePath: '阿里云oss', fileSize: '7467B', fileid: '文件ID', realFileName: '文件名', type: ''}]
   */
  get2dUploadImgInfo = (info) => {
    const { params, uploadComplete } = this.props;
    const len = info.length;

    if (len > 0) {
      const file = info[len - 1];
      if (file.filePath && file.filePath !== '') {
        let newFile = {
          fileId: file.fileid,
          fileName: file.realFileName,
          src: file.filePath,
          ...params,
          fileSize: file.fileSize,
        };

        this.setState({ file: newFile });
        uploadComplete(newFile);
      }
    }
  };

  /**
   * 图片上传时不断调用
   * 当上传成功时，可获取到全部file数据
   * 调用父组件传递进来的callback
   */
  onUploadChange = (info) => {
    const { params, uploadComplete } = this.props;
    const file = info;

    if (file.src && file.src !== '') {
      let newFile = {
        ...file,
        fileId: file['x:fileid'],
        fileName: file['x:realname'],
        ...params,
      };

      if (!newFile.fileId && !newFile.fileName) {
        showMessage('图片上传失败，请联系管理人员！', 'error');
        return;
      }

      this.setState({ file: newFile });
      uploadComplete(newFile);
      showMessage('图片上传成功！', 'success');
    }
  };

  /**
   * 图片上传前进行校验
   * 必需返回一个fileList [Array]
   */
  checkLength = (files) => {
    if (!files || files.length <= 0) {
      return [];
    }

    const file = files[0];
    const fileType = file.type || file.name.slice(file.name.lastIndexOf('.') + 1);
    const valid =
      fileType &&
      validateFileTypes.some((type) => {
        return fileType.indexOf(type) > -1;
      });

    if (!valid) {
      showMessage('图片格式不正确！', 'warning');
      return [];
    }

    return files;
  };

  /**
   * 按钮遮罩层
   */
  btnCoverToggle = () => {
    this.setState((prevState, props) => ({
      btnCover: !prevState.btnCover,
    }));
  };

  /**
   * 设置插件大小
   */
  setSize = (size, type) => {
    let result = {};

    switch (type) {
      case 1:
        result.width = size;
        result.height = size;
        break;
      case 2:
        result.width = size - 20;
        result.height = size - 20;
        break;
      case 3:
        let icon = parseInt(size / 5);
        result.width = icon;
        result.height = icon;
        break;
      default:
        result.width = 100;
        result.height = 100;
        break;
    }

    return result;
  };

  setIconSize = (size) => {
    return parseInt(size / 5);
  };

  /**
   * 预览
   */
  previewToggle = (e) => {
    this.setState((prevState, props) => ({
      previewImg: !prevState.previewImg,
    }));
  };

  /**
   * 删除图片
   */
  deleteImg = (e) => {
    const { deleteComplete } = this.props;
    const { file } = this.state;
    const that = this;

    AntdModal.confirm({
      title: '是否删除此图片？',
      content: '',
      okType: 'danger',
      onOk() {
        deleteComplete(file.fileId);
        that.setState({ file: null, btnCover: false });
      },
    });

    this.setState((prevState) => {
      return { btnCover: false };
    });
  };

  /**
   * 生成预览图样式
   */
  setPreview = (img) => {
    return {
      backgroundImage: 'url(' + img + ')',
      backgroundSize: 'contain',
      backgroundPosition: 'center center',
      backgroundRepeat: 'no-repeat',
    };
  };

  beforeUpload = (files) => {
    if (this.props.beforeUpload) {
      return this.props.beforeUpload(files);
    }

    return this.checkLength(files);
  };

  render() {
    const { editing, validator, onChange, size, text, uploadComplete, disabled, viewOnly } = this.props;
    const { file, btnCover, previewImg } = this.state;

    let size_1 = this.setSize(size, 1);
    let size_2 = this.setSize(size, 2);
    let size_3 = this.setSize(size, 3);

    let iconSize = this.setIconSize(size);

    let imgSrc = file && file.src ? file.src : '';
    let imgList = [];
    imgList.push(imgSrc);

    return (
      <div className="UploadImg">
        <div className="hom-upload-img" style={size_1}>
          {this.props.qrCodeUploadVisible && (
            <div className="hom-upload-img-qr" style={size_3}>
              <Popover
                placement="topRight"
                content={
                  <div>
                    <QrcodeFileUpload height={160} width={160} callback={this.get2dUploadImgInfo} multiple />
                    <div style={{ textAlign: 'center', marginTop: 10 }}>手机扫码上传图片</div>
                  </div>
                }
              >
                <Icon type="2d_code_half_mc" size={iconSize} />
              </Popover>
            </div>
          )}
          <div className="hom-upload-img-picker">
            <Upload onChange={this.onUploadChange} multiple={false} beforeUpload={this.beforeUpload}>
              <div className="hom-upload-img-picker-btn">
                <div>
                  <Icon type="plus" size={iconSize} />
                </div>
                <div>{text}</div>
              </div>
            </Upload>
          </div>
          {(disabled || file || viewOnly) && <div className="hom-upload-img-disabled" />}
          {file && file.src !== '' && (
            <div
              className="hom-upload-img-view"
              style={size_2}
              // onMouseEnter={this.btnCoverToggle}
              // onMouseLeave={this.btnCoverToggle}
            >
              <div className="hom-upload-img-view-wrapper">
                <img className="hom-upload-img-view-img" src={file.src} />
              </div>

              <div className="hom-upload-img-cover">
                <div className="hom-upload-img-detail" onClick={(e) => this.previewToggle(e)} title="预览图片">
                  <Icon type="visible_mc" color="#000000" size={iconSize} />
                </div>
                {!viewOnly && (
                  <div className="hom-upload-img-delete" onClick={(e) => this.deleteImg(e)} title="删除图片">
                    <Icon type="delete_mc" color="#000000" size={iconSize} />
                  </div>
                )}
              </div>
            </div>
          )}
        </div>
        {previewImg && (
          <div className="hom-upload-img-preview">
            <div className="hom-upload-img-preview-wrapper">
              <PreviewImgs list={imgList} activeSrc={imgSrc} showThumbnail={false} />
            </div>
            <div className="hom-upload-img-preview-close" onClick={(e) => this.previewToggle(e)}>
              <Icon type="cross_circle" size={32} />
            </div>
          </div>
        )}
      </div>
    );
  }
}

export default UploadImg;
