import React from 'react';
import PropTypes from 'prop-types';
import style from './index.less';
import classNames from 'classnames';
import { Pagination } from 'antd';
import Loading from '../Loading';
import { runFunc } from '../utils/common';

const classPrefix = 'Deck';

const selectedWrap = (render, selectedKeys, onSelectChange, options = {}) => {
  return (props, index) => {
    return options.noFragment ?
      (
        render({
          ...props,
          onClick: () => { onSelectChange(index) },
          isSelected: selectedKeys.indexOf(index) !== -1
        }, index)
      )
      :
      (
        <div className={style[`${classPrefix}-SelectedWrap`]} onClick={() => { onSelectChange(index) }} key={index}>
          {
            render({...props, isSelected: selectedKeys.indexOf(index) !== -1 }, index)
          }
        </div>
      )
  };
};

class Deck extends React.Component {
  constructor(props) {
    super(props);
    this.onShowSizeChange  = this.onShowSizeChange.bind(this);
    this.onSelectChange  = this.onSelectChange.bind(this);
  }
  static propTypes = {
    dataSource: PropTypes.array.isRequired,
    render: PropTypes.func.isRequired,
    pagination: PropTypes.shape({
      total: PropTypes.number.isRequired,
      current: PropTypes.number.isRequired,
      size: PropTypes.number.isRequired,
      onChange: PropTypes.func.isRequired,
      showQuickJumper: PropTypes.bool,
      pageSizeOptions: PropTypes.arrayOf(PropTypes.string),
      showSizeChanger: PropTypes.bool,
      onShowSizeChange: PropTypes.func,
    }),
    rowSelection: PropTypes.shape({
      selectedRowKeys: PropTypes.array,
      onChange: PropTypes.func,
      singleSelect: PropTypes.bool,
      options: PropTypes.shape({
        noFragment: PropTypes.bool,
      }),
    }),
    className: PropTypes.string,
    bodyClassName: PropTypes.string,
    bodyStyle: PropTypes.object,
    loading: PropTypes.bool,
    loadingTip: PropTypes.string,
    fixedBodyHeight: PropTypes.bool,
  };
  static defaultProps = {
    pagination: null,
    defaultValue: {
      showQuickJumper: true,
      pageSizeOptions: ['10', '20', '50'],
      showSizeChanger: true,
    },
    bodyStyle: {},
    loading: false,
    loadingTip: '',
    fixedBodyHeight: false,
  };
  static showTotal(total, range) {
    return (
      <span>共 {total} 条记录</span>
    )
  }
  static stopPropagation(props) {
    return (
      <div className="Deck-stopPropagation" onClick={(e) => { e.stopPropagation() }}>{props.children}</div>
    );
  }
  onShowSizeChange(current, size) {
    this.props.pagination.onChange(1, size);
  }
  onSelectChange(key) {
    let keys = this.props.rowSelection.selectedRowKeys.slice();
    if (this.props.rowSelection.singleSelect) {
      keys = keys[0] === key ? [] : [key];
    } else {
      let checkKey = keys.indexOf(key);
      if (checkKey === -1) {
        keys.push(key);
        keys.reverse();
      } else {
        keys.splice(checkKey, 1);
      }
    }
    runFunc(this.props.rowSelection.onChange, keys);
  }

  render() {
    const { dataSource, pagination, render, rowSelection, defaultValue,
      className, bodyClassName, bodyStyle, loading, loadingTip } = this.props;
    return (
      <div className={classNames(
        style[classPrefix],
        this.props.fixedBodyHeight ? 'fixedBodyHeight' : '',
        className
      )}>
        <div className={classNames(
          style[`${classPrefix}-body`],
          this.props.fixedBodyHeight ? 'fixedBodyHeight' : '',
          bodyClassName
          )} style={bodyStyle}>
          {
            dataSource.map((item, index) => {
              return rowSelection && rowSelection.selectedRowKeys ?
                selectedWrap(
                  render,
                  rowSelection.selectedRowKeys,
                  this.onSelectChange,
                  rowSelection.options || {}
                )(item, index)
                :
                render(item, index)
            })
          }
        </div>
        {pagination && dataSource.length > 0 && (
          <div className={style[`${classPrefix}-footer`]}>
            <Pagination
              defaultCurrent={1}
              total={pagination.total}
              current={pagination.current}
              pageSize={pagination.size}
              onChange={pagination.onChange}
              showQuickJumper={pagination.showQuickJumper || defaultValue.showQuickJumper}
              pageSizeOptions={pagination.pageSizeOptions || defaultValue.pageSizeOptions}
              showSizeChanger={pagination.showSizeChanger || defaultValue.showSizeChanger}
              onShowSizeChange={pagination.onShowSizeChange || this.onShowSizeChange}
              showTotal={Deck.showTotal}/>
          </div>
        )}

        <Loading loading={loading} tip={loadingTip}/>
      </div>
    )
  }
}

export default Deck;
