import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import style from './styles.module.less';
import { Form } from 'antd';
import { ExpandHeight } from '../../Expand';
import Button from '../../Button';
import TextLink from '../../TextLink';
import { runFunc } from '../../utils/common';

const classPrefix = 'SearchFilterForm';

class _SearchFilterForm extends React.Component {
  constructor(props) {
    super(props);
    this.openExpand = this.openExpand.bind(this);
    this.closeExpand = this.closeExpand.bind(this);
    this.submit = this.submit.bind(this);
    this.reset = this.reset.bind(this);
    this.$expand = null;
  }

  static propTypes = {
    truncationNum: PropTypes.number,
    onSearch: PropTypes.func.isRequired,
    onSearchFail: PropTypes.func,
    onReset: PropTypes.func,

    searchLoading: PropTypes.bool,
    searchDisabled: PropTypes.bool,
    autoComplete: PropTypes.bool,
    resetDisabled: PropTypes.bool,

    mapPropsToFields: PropTypes.func,
    onFieldsChange: PropTypes.func,
    onValueChange: PropTypes.func,
  };

  static defaultProps = {
    truncationNum: 1,
    searchLoading: false,
    searchDisabled: false,
    autoComplete: false,
    resetDisabled: false,
  };

  static stopPropagation(e) {
    e.stopPropagation();
  }

  openExpand() {
    runFunc(this.$expand.open);
  }

  closeExpand() {
    if (this.$expand) {
      runFunc(this.$expand.close);
    }
  }

  submit(e) {
    e.preventDefault();
    this.props.form.validateFields((err, values) => {
      if (err) {
        runFunc(this.props.onSearchFail, err);
      } else {
        this.closeExpand();
        runFunc(this.props.onSearch, values);
      }
    });
  }

  reset() {
    this.props.form.resetFields();
    runFunc(this.props.onReset);
  }

  render() {
    let children = this.props.children || [];
    if (React.isValidElement(children)) {
      children = [children];
    }
    // 过滤掉非react对象
    const childrenAfterFilter = children.filter(child => React.isValidElement(child));
    const isNeedExpand = childrenAfterFilter.length > this.props.truncationNum;
    const allChildren = React.Children.map(
      childrenAfterFilter,
      child => React.cloneElement(child, { form: this.props.form })
    );
    const mainChildren = isNeedExpand ?
      allChildren.slice(0, this.props.truncationNum >= 1 ? this.props.truncationNum : 1) : allChildren;

    return (
      <div className={style[classPrefix]} onClick={_SearchFilterForm.stopPropagation}>
        <Form autoComplete={this.props.autoComplete ? 'on' : 'off' } onSubmit={this.submit}>
          <div className={classNames(style[`${classPrefix}-main`])}>
            {
              mainChildren
            }
            <div className={classNames(style[`${classPrefix}-main-tools`], { 'not-expand': !isNeedExpand, 'has-expand-toolbar-body': !!this.props.expandToolbarBody })}>
              {
                isNeedExpand &&
                <Button onClick={this.openExpand} style={{ width: 100 }}>更多筛选</Button>
              }
              <Button onClick={this.submit}
                      loading={this.props.searchLoading}
                      disabled={this.props.searchDisabled}
                      type="primary">查询</Button>
              <Button onClick={this.reset} disabled={this.props.resetDisabled}>重置</Button>
              {this.props.expandToolbarBody}
            </div>
            {this.props.appendToolbarBody}
          </div>
          {
            isNeedExpand &&
            <ExpandHeight className={style[`${classPrefix}-wrapper`]} ref={c => this.$expand = c}>
              <div className={style[`${classPrefix}-expand`]}>
                {
                  allChildren
                }
                <div className={style[`${classPrefix}-expand-footer`]}>
                  <TextLink onClick={this.closeExpand}>收起</TextLink>
                  <Button onClick={this.submit}
                          htmlType="submit"
                          loading={this.props.searchLoading}
                          disabled={this.props.searchDisabled}
                          type="primary">查询</Button>
                  <Button onClick={this.reset}>重置</Button>
                </div>
              </div>
            </ExpandHeight>
          }
        </Form>
      </div>
    )
  }
}

const NewForm = Form.create({
  mapPropsToFields(props) {
    return runFunc(props.mapPropsToFields);
  },
  onFieldsChange(props, fields) {
    runFunc(props.onFieldsChange, fields);
  },
  onValuesChange(props, values) {
    runFunc(props.onValueChange, values);
  },
})(_SearchFilterForm);

export default class SearchFilterForm extends React.Component {
  static propTypes = {
    truncationNum: PropTypes.number,
    onSearch: PropTypes.func.isRequired,
    onSearchFail: PropTypes.func,
    onReset: PropTypes.func,

    searchLoading: PropTypes.bool,
    searchDisabled: PropTypes.bool,
    autoComplete: PropTypes.bool,
    resetDisabled: PropTypes.bool,

    mapPropsToFields: PropTypes.func,
    onFieldsChange: PropTypes.func,
    onValueChange: PropTypes.func,
  };

  constructor(props) {
    super(props);
    this.setFieldsValue = this.setFieldsValue.bind(this);
    this.getFieldsValue = this.getFieldsValue.bind(this);
    this.getFieldValue = this.getFieldValue.bind(this);
    this.validateFields = this.validateFields.bind(this);
    this.form = null;
  }

  setFieldsValue(payload) {
    this.form.setFieldsValue(payload);
  }

  getFieldsValue(fieldKeyArr) {
    return this.form.getFieldsValue(fieldKeyArr);
  }

  getFieldValue(fieldKey) {
    return this.form.getFieldValue(fieldKey);
  }

  validateFields(...arg) {
    return this.form.validateFields(...arg);
  }

  resetFields() {
    this.form.resetFields();
  }

  render() {
    return (
      <NewForm wrappedComponentRef={c => this.form = c.props.form} {...this.props}/>
    );
  }
}
