const isIdCard = (id) => {
  const tempPattern = /^\d{17}(\d|X)$/;
  if (typeof id !== 'string') {
    return false
  }

  if (!tempPattern.test(id) || id.length !== 18) {
    return false
  }

  const areaCode = id.slice(0, 6);
  const birthCode = id.slice(6, 14);

  const areaStatus = areaValid(areaCode);
  const birthStatus = birthValid(birthCode);
  const endStatus = endValid(id);

  return areaStatus && birthStatus && endStatus
}

// 地区校验，只校验省级是否合规
const areaValid = (str) => {
  const format = /^(([1][1-5])|([2][1-3])|([3][1-7])|([4][1-6])|([5][0-4])|([6][1-5])|([7][1])|([8][1-3]))\d{4}$/;
  return format.test(str)
};

// 生日校验，校验出生年月日是否合法，还有是否时间小于今天
const birthValid = (str) => {
  try {
    const beforeCalculateYear = Number(str.slice(0, 4)),
      beforeCalculateMonth = Number(str.slice(4, 6)),
      beforeCalculateDay = Number(str.slice(6, 8));
    const beforeCalculateBirthTime = new Date(beforeCalculateYear, beforeCalculateMonth - 1, beforeCalculateDay);
    const afterCalculateYear = beforeCalculateBirthTime.getFullYear();
    const afterCalculateMonty = beforeCalculateBirthTime.getMonth() + 1;
    const afterCalculateDay = beforeCalculateBirthTime.getDate();
    const nowTime = new Date().getTime();
    return beforeCalculateYear === afterCalculateYear && beforeCalculateMonth === afterCalculateMonty && beforeCalculateDay === afterCalculateDay && nowTime > beforeCalculateBirthTime.getTime();
  }catch (e) {
    return false
  }
};

// 尾号校验
const endValid = (str) => {
  const c = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2]; // 系数
  const b = ['1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2']; // 校验码对照表
  const idArray = str.split('');
  const tempSum = c.reduce((prev, curr, index) => {
    return parseInt(idArray[index]) * curr + prev
  }, 0);
  return idArray[17].toUpperCase() === b[tempSum % 11].toUpperCase();
};

const checkRequired = value => {
  let e = null
  if (typeof value === 'number') {
    if (!value && value !== 0) {
      e = true
    }
  } else if (value instanceof Object) {
    if (!value || Object.keys(value).length === 0) {
      e = true
    }
  } else if (value instanceof Array) {
    if (!value || value.length === 0) {
      e = true
    }
  } else {
    if (!value) {
      e = true
    }
  }
  return e
}

const isCellPhone = value => {
  if(/^(?:13\d|15[89])-?\d{5}(\d{3}|\*{3})$/.test(value)){
    return true
  }
  return false
}

const isTelPhone = value => {
  if(/^(([0+]\d{2,3}-)?(0\d{2,3})-)(\d{7,8})(-(\d{3,}))?$/.test(value)){
    return true
  }
  return false
}

const checkPhone = value => {
  value = value + ''
  // 手机号只校验第一位为1，且长度为11
  const tel = /^1\d{10}$/;
  // const tel= /(^010\d{8}$)|(^010-\d{8}$)|(^02[0-9]\d{8}$)|(^02[0-9]-\d{8}$)|(^0[3-9]\d{2}-\d{7,8}$)|((^0[3-9]\d{9,10}$))|(^[2-9]\d{6,7}$)|(^13\d{9}$)|(^14[145678]\d{8}$)|(^15\d{9}$)|(^166\d{8}$)|(^17[01345678]\d{8}$)|(^18\d{9}$)|(^19[89]\d{8}$)/
  return tel.test(value) ? null : true
  //return (isCellPhone(value) || isTelPhone(value)) ? null : true
}

const checkNumber = (value, config = {}) => {
  const { min, max, excludeMin, excludeMax, tips, int } = config
  const hasMin = 'min' in config
  const hasMax = 'max' in config
  const hasExcludeMin = 'excludeMin' in config
  const hasExcludeMax = 'excludeMax' in config
  const isNumber = typeof value === 'number'
  const isInt = isNumber && value % 1 === 0
  const basicCheck = int ? isInt : isNumber
  const typeText = int ? '整数' : '数字'
  let error = null
  if (!hasMin && !hasMax && !hasExcludeMin && !hasExcludeMax) {
    if (!basicCheck) {
      error = `请输入${typeText}`
    }
  } else if (hasMin && !hasMax && !hasExcludeMin && !hasExcludeMax) {
    if (!basicCheck || value < min) {
      error = `请输入大于或等于${min}的${typeText}`
    }
  } else if (!hasMin && hasMax && !hasExcludeMin && !hasExcludeMax) {
    if (!basicCheck || value > max) {
      error = `请输入小于或等于${max}的${typeText}`
    }
  } else if (!hasMin && !hasMax && hasExcludeMin && !hasExcludeMax) {
    if (!basicCheck || value <= excludeMin) {
      error = `请输入大于${excludeMin}的${typeText}`
    }
  } else if (!hasMin && !hasMax && !hasExcludeMin && hasExcludeMax) {
    if (!basicCheck || value >= excludeMax) {
      error = `请输入小于${excludeMax}的${typeText}`
    }
  } else {
    if (hasMin && hasMax) {
      if (!basicCheck || value < min || value > max) {
        error = `请输入${min}-${max}之间的${typeText}`
      }
    } else if (hasExcludeMin && hasMax) {
      if (!basicCheck || value <= excludeMin || value > max) {
        error = `请输入${excludeMin}~${max}之间的${typeText}（不含${excludeMin}）`
      }
    } else if (hasMin && hasExcludeMax) {
      if (!basicCheck || value < min || value >= excludeMax) {
        error = `请输入${min}~${excludeMax}之间的${typeText}（不含${excludeMax}）`
      }
    } else if (hasExcludeMin && hasExcludeMax) {
      if (!basicCheck || value <= excludeMin || value >= excludeMax) {
        error = `请输入${excludeMin}~${excludeMax}之间的${typeText}（不含${excludeMin}和${excludeMax}）`
      }
    }
  }
  if (error && tips) {
    error = tips
  }
  return error
}

// 身份证
function checkIdentityNumber(id) {
  return isIdCard(id);
}

function checkIdentityNumber18(id) {
  return isIdCard(id);
}

function checkIdentityNumber15(id) {
  var year = id.substring(6, 8);
  var month = id.substring(8, 10);
  var day = id.substring(10, 12);
  var temp_date = new Date(year, parseFloat(month) - 1, parseFloat(day));
  if (
    temp_date.getYear() != parseFloat(year) ||
    temp_date.getMonth() != parseFloat(month) - 1 ||
    temp_date.getDate() != parseFloat(day)
  ) {
    return false;
  }
  return true;
}

export {
  checkRequired,
  checkPhone,
  checkNumber,
  checkIdentityNumber,
  checkIdentityNumber18,
  checkIdentityNumber15,
}
