import {
  CALL_LOGIN_STATUS_OFFLINE,
  CALL_AGENT_STATUS_ACTIVE,
  CALL_WAY_SIP,
  CALL_DIRECT_ANSWER,
  CALL_DIRECT_CALL,
} from 'CONST/callConfig';
import { runFunc, errorsHandle } from 'UTIL/common';
import api from 'SERVICE/callCenter';
import { SobotCallCenter, VertinhoClient } from './lib';
import { ENUM as VertoENUM } from './lib/vertinho';

export default {
  namespace: 'callCenter',
  state: {
    // 用户是否已绑定座席
    alreadyBind: false,
    // 使用坐席的用户 id
    occupantId: '',
    // 使用坐席的用户名
    occupantName: '',
    // 座席 id
    agentId: '',
    // 企业id
    companyId: '',
    // 应用id
    appId: '',
    // 座席当前呼叫方式
    callWay: CALL_WAY_SIP,
    // 用户座席绑定的手机号码
    phoneNumber: '',
    // 当前用户座席工作状态
    agentLoginStatus: CALL_LOGIN_STATUS_OFFLINE,
    // 座席全局工作状态
    uniqueAgentLoginStatus: CALL_LOGIN_STATUS_OFFLINE,
    // 座席管理状态
    agentStatus: CALL_AGENT_STATUS_ACTIVE,
    // 针对当前登录用户的座席启用状态
    enabled: false,
    // 座席是否可供当前用户使用（上下线）
    available: false,
    // 智齿访问令牌
    token: '',
    // VoIP号码，由14位数字组成
    voipAccount: '',
    // VoIP密码，由8位数字和字母组成
    voipPwd: '',
    // VoIP注册地址以及端口号
    serverIP: '',
    // 当前通话呼叫类型
    currentDirection: null,
    // 当前呼出/呼入 callId
    currentCallId: null,
    // 当前 verto 软电话 callId
    currentVertoCallId: null,
    // verto 软电话实例
    vertoInstance: null,
    // 呼叫中心实例
    callCenterInstance: null,
    // 到期时间
    expireTime: null,
  },
  reducers: {
    /**
     * 更新状态
     * @param  {object} payload 需要更新的状态键值对
     */
    updateState(state, { payload = {} }) {
      return { ...state, ...payload };
    },
    updateAgentInfo(state, { payload = {} }) {
      /**
       * alreadyBind, agentLoginStatus, callWay,
       * userId, userName,
       * agentId, companyId, appId, phoneNumber
       */
      const {
        loginUserId,
        userId: occupantId,
        userName: occupantName,
        agentLoginStatus: uniqueAgentLoginStatus,
        ...restData
      } = payload;

      /**
       * 判断座席是否启用
       * 1. 用户已绑定座席
       * 2. 座席管理状态为“启用”
       */
      const enabled = payload.alreadyBind && Number(payload.agentStatus) === CALL_AGENT_STATUS_ACTIVE;

      /**
       * 判断座席是否可供当前用户使用
       * 占用座席的用户与当前登录用户一致
       */
      const available = enabled && (!occupantId || occupantId === loginUserId);

      // 计算出的当前用户座席状态
      const agentLoginStatus = available ? Number(uniqueAgentLoginStatus) : CALL_LOGIN_STATUS_OFFLINE;

      const newPayload = {
        ...restData,
        occupantId,
        occupantName,
        callWay: Number(payload.callWay),
        agentLoginStatus,
        uniqueAgentLoginStatus: Number(uniqueAgentLoginStatus),
        agentStatus: Number(payload.agentStatus),
        enabled,
        available,
      };

      return {
        ...state,
        ...newPayload,
      };
    },
    updateCallClient(state, { payload = {} }) {
      const keyList = [
        {
          from: 'direction',
          to: 'currentDirection',
        },
        {
          from: 'callId',
          to: 'currentCallId',
        },
        {
          from: 'vertoCallId',
          to: 'currentVertoCallId',
        },
      ];
      const exactPayload = keyList.reduce((prev, config) => {
        const { from, to } = config;
        const value = payload[from];

        if (value !== undefined) {
          prev[to] = value;
        }

        return prev;
      }, {});

      return {
        ...state,
        ...exactPayload,
      };
    },
  },
  effects: {
    /**
     * 初始化座席
     * @param {Object} action
     * @param {Object} effects
     */
    *initAgent({ payload = {} }, { call, put, take, select }) {
      yield put({
        type: 'fetchToUpdAgentInfo',
      });

      // 等待 type 为 callCenter/fetchToUpdAgentInfoEnd 的 action 发出
      const agentEndAction = yield take('callCenter/fetchToUpdAgentInfoEnd');

      if (agentEndAction.payload && !agentEndAction.payload.successful) {
        return runFunc(payload.reject);
      }

      const agentId = yield select((state) => state.callCenter.agentId);

      if (!agentId) {
        return runFunc(payload.reject);
      }

      const response = yield api.getAgentEndpointInfo({ agentId }).catch(errorsHandle());

      if (!response) {
        return runFunc(payload.reject);
      }

      // voipAccount, voipPwd, serverIP
      const { data = {} } = response;

      yield put({
        type: 'updateState',
        payload: data,
      });

      const partState = yield select((state) => {
        const { callCenterInstance, vertoInstance, ...rest } = state.callCenter;

        return rest;
      });

      runFunc(payload.resolve, partState);
    },
    /**
     * 获取座席信息并更新
     * @param {Object} action
     * @param {Object} action.payload
     * @param {Object} action.payload.resolve 成功
     * @param {Object} action.payload.reject 失败
     * @param {Object} action.payload.callback 副作用回调
     * @param {Object} effects
     */
    *fetchToUpdAgentInfo({ payload = {} }, { call, put, select }) {
      const response = yield api.getAgentInfoByUser().catch(errorsHandle());

      if (!response) {
        yield put({
          type: 'callCenter/fetchToUpdAgentInfoEnd',
          payload: {
            successful: false,
          },
        });

        return runFunc(payload.reject);
      }

      const data = response.data || {};

      const loginUserId = yield select((state) => state.account.user.userId);

      yield put({
        type: 'updateAgentInfo',
        payload: {
          ...data,
          loginUserId,
        },
      });

      const partState = yield select((state) => {
        const { callCenterInstance, vertoInstance, ...rest } = state.callCenter;

        return rest;
      });

      runFunc(payload.resolve, partState);

      // 执行副作用 callback
      runFunc(payload.callback, partState);

      // 发出 type 为 callCenter/fetchToUpdAgentInfoEnd 的 action
      // 注意，发出的 action type 不在 {namespace}.effects 里面
      // take 的时候必须带上 namespace
      yield put({
        type: 'callCenter/fetchToUpdAgentInfoEnd',
        payload: {
          successful: true,
        },
      });
    },
    /**
     * 调接口更改座席上下线状态
     * @param {Object} action
     * @param {Object} action.payload
     * @param {Number} action.payload.agentLoginStatus 座席工作状态
     * @param {Number} action.payload.callWay 呼叫方式
     * @param {Function} action.payload.callback 回调
     * @param {Object} effects
     */
    *changeAgentOnlineStatus({ payload = {} }, { call, put, select }) {
      const agentId = yield select((state) => state.callCenter.agentId);
      const response = yield api
        .updateAgentStatus({
          agentId,
          agentLoginStatus: String(payload.agentLoginStatus),
          callWay: payload.callWay ? String(payload.callWay) : void 0,
        })
        .catch(errorsHandle());

      const successful = response && response.data ? response.data : false;

      if (successful) {
        const callwayPayload = payload.callWay ? { callWay: payload.callWay } : {};

        yield put({
          type: 'updateState',
          payload: {
            ...callwayPayload,
            agentLoginStatus: payload.agentLoginStatus,
            uniqueAgentLoginStatus: payload.agentLoginStatus,
          },
        });

        // 副作用回调，callback 可能是 promise 的 resolve
        runFunc(payload.resolve);
      } else {
        runFunc(payload.reject);
      }
    },
    /**
     * 操作座席上线，新建呼叫实例
     * @param {Object} action
     * @param {Object} action.payload
     * @param {Function} action.payload.onlineCallback 上线回调
     * @param {Function} action.payload.messageCallback 消息处理回调
     * @param {Object} effects
     */
    *putCallCenterOnline({ payload = {} }, { call, put, select }) {
      // 获取智齿 access token 并更新
      const TokenResponse = yield api.getSocketAuthToken().catch(errorsHandle());

      if (!TokenResponse || !TokenResponse.data) {
        runFunc(payload.reject);
      }

      yield put({
        type: 'updateState',
        payload: {
          token: TokenResponse.data.accessToken,
        },
      });

      // 取呼叫中心实例参数配置
      const { prevCallCenterInstance, ...callCenterParams } = yield select((state) => {
        const { agentId, companyId, appId, token, callCenterInstance } = state.callCenter;

        return {
          agentId,
          companyId,
          appId,
          token,
          prevCallCenterInstance: callCenterInstance,
        };
      });

      // 确保上一个呼叫中心实例关闭
      if (prevCallCenterInstance) {
        prevCallCenterInstance.goOffline();
      }

      const { onlineCallback, messageCallback } = payload;

      const callCenterInstance = new SobotCallCenter({
        ...callCenterParams,
        messageHandler: (result) => {
          switch (result.resultType) {
            case 'online':
              onlineCallback();
              break;
            case 'message':
              messageCallback(result.data);
              break;
          }
        },
      });

      yield put({
        type: 'updateState',
        payload: {
          callCenterInstance,
        },
      });

      callCenterInstance.goOnline();

      runFunc(payload.resolve);
    },
    /**
     * 在座席状态变更为下线后，关闭呼叫中心实例
     * @param {Object} action
     * @param {Object} effects
     */
    *takeCallCenterOffline(action, { put, select }) {
      const callCenterInstance = yield select((state) => state.callCenter.callCenterInstance);

      if (callCenterInstance) {
        callCenterInstance.goOffline();
      }

      const vertoInstance = yield select((state) => state.callCenter.vertoInstance);

      if (vertoInstance && vertoInstance instanceof VertinhoClient) {
        vertoInstance.destroy();
      }

      yield put({
        type: 'updateState',
        payload: {
          callCenterInstance: null,
          vertoInstance: null,
        },
      });
    },
    /**
     * 初始化 verto 软电话
     * @param {Object} action
     * @param {Object} effects
     */
    *bootstrapVerto({ payload = {} }, { put, select }) {
      const legacyVertoClient = yield select((state) => state.callCenter.vertoInstance);

      if (legacyVertoClient && legacyVertoClient instanceof VertinhoClient) {
        legacyVertoClient.destroy();
      }

      const { voipAccount, voipPwd, serverIP } = yield select((state) => {
        const { voipAccount, voipPwd, serverIP } = state.callCenter;

        return { voipAccount, voipPwd, serverIP };
      });

      const newVertoClient = new VertinhoClient(
        {
          webSocket: {
            login: `${voipAccount}@${serverIP}`,
            password: voipPwd,
            url: `wss://${serverIP}`,
          },
          deviceParams: {
            useMic: 'any',
            useSpeak: 'any',
            useCamera: 'none',
          },
          audioParams: {
            googAutoGainControl: true,
            // 禁用噪音消除
            googNoiseSuppression: false,
            googHighpassFilter: true,
            // 禁用回声消除
            googEchoCancellation: false,
          },
          remoteAudioId: 'cc-widget-call-audio',
          iceServers: [{ urls: ['stun:39.97.65.10:3478'] }],
        },
        {
          onWebSocketLoginSuccess: () => {},
          onClientReady: (feedback) => console.log('Client ready', feedback),
          onInfo: (info) => console.log('Info received', info),
          onDisplay: (display) => console.log('Displaying remote audio', display),
          onCallStateChange: (diff) => {
            console.log('Call state changed', diff);
            callStateCallback(diff);
          },
        },
      );

      newVertoClient.ringer = document.querySelector('#cc-widget-call-ringer');

      const { onCallChange, getDirection } = payload;

      const callStateCallback = (payload) => {
        const { callID, current: currentState } = payload;
        const callInstance = newVertoClient.calls[callID];

        if (!callInstance) {
          return;
        }

        const direction = getDirection();

        switch (currentState) {
          case VertoENUM.state.ringing:
            onCallChange({
              vertoCallId: callID,
            });

            if (direction === CALL_DIRECT_CALL) {
              callInstance.answer();
            }

            break;
          case VertoENUM.state.destroy:
            // 置 verto 软电话 CallId 为 null
            onCallChange({
              vertoCallId: null,
            });
            break;
          default:
            break;
        }
      };

      yield put({
        type: 'updateState',
        payload: {
          vertoInstance: newVertoClient,
        },
      });

      runFunc(payload.resolve);
    },
    /**
     * 拨打电话
     * @param {Object} action
     * @param {Object} effects
     */
    *makeCall({ payload = {} }, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          currentDirection: CALL_DIRECT_CALL,
        },
      });

      runFunc(payload.resolve);
    },
    /**
     * 挂断网络电话
     * @param {Object} action
     * @param {Object} effects
     */
    *hangUp({ payload = {} }, { select, put }) {
      const params = yield select((state) => {
        const { agentId, currentCallId } = state.callCenter;

        return {
          agentId,
          callId: currentCallId,
        };
      });

      if (!params.callId) {
        return runFunc(payload.reject);
      }

      const response = yield api.hangUp(params).catch(errorsHandle());
      const successful = response && response.data ? response.data : false;

      if (!successful) {
        return runFunc(payload.reject);
      }

      yield put({
        type: 'updateState',
        payload: {
          currentDirection: null,
          currentCallId: null,
        },
      });

      runFunc(payload.resolve);
    },
    /**
     * 接听网络电话
     * @param {Object} action
     * @param {Object} effects
     */
    *answerCall({ payload = {} }, { select, put }) {
      const { vertoCallId, vertoInstance } = yield select((state) => {
        const { currentVertoCallId, vertoInstance } = state.callCenter;

        return {
          vertoCallId: currentVertoCallId,
          vertoInstance,
        };
      });

      const callInstance = vertoCallId ? vertoInstance.calls[vertoCallId] : null;

      if (callInstance) {
        callInstance.answer();

        runFunc(payload.resolve);
      } else {
        runFunc(payload.reject);
      }
    },
  },
};
