import { isObject } from 'UTIL/lang/data-type';

// 默认存活检查间隔 30 秒
const DEFAULT_HEARTBEAT_INTERVAL = 30000;

class ExSocket {
  constructor(props) {
    // 存活检查 timer
    this.survivalSinalTimer = null;
    // 服务器无应答时连接关闭的 timer
    this.closeSignalTimer = null;

    const { url, onOpen, onMessage, onClose, onError } = props;
    const handlers = [
      {
        eventName: 'open',
        handler: this.getOpenHandler(onOpen),
      },
      {
        eventName: 'message',
        handler: this.getMessageHandler(onMessage),
      },
      {
        eventName: 'close',
        handler: onClose,
      },
      {
        eventName: 'error',
        handler: onError,
      },
    ];

    this.socket = new WebSocket(url);

    handlers.forEach((item) => {
      const { eventName, handler } = item;

      this.socket.addEventListener(eventName, handler);
    });
  }

  getOpenHandler(callback) {
    return (event) => {
      this.setSurvivalCheck();

      if (callback) {
        callback(event);
      }
    };
  }

  getMessageHandler(callback) {
    return (event) => {
      this.setSurvivalCheck();

      if (callback) {
        callback(event);
      }
    };
  }

  /**
   * 启用存活检查
   */
  setSurvivalCheck() {
    if (this.survivalSinalTimer) {
      clearTimeout(this.survivalSinalTimer);
    }

    if (this.closeSignalTimer) {
      clearTimeout(this.closeSignalTimer);
    }

    const interval = DEFAULT_HEARTBEAT_INTERVAL;

    // 重新设置存活检查 timer
    this.survivalSinalTimer = setTimeout(() => {
      // 发送存活检查消息
      this.send({
        messageType: 'SurvivalCheck',
        message: '存活检查',
      });

      // 重新设置服务器无应答时连接关闭的 timer
      this.closeSignalTimer = setTimeout(() => {
        this.close();
      }, interval);
    }, interval);
  }

  /**
   * 发送消息
   * @param {Object} data 发送的消息
   */
  send(data) {
    if (!this.socket) {
      return;
    }

    const message = isObject(data) ? JSON.stringify(data) : '';

    this.socket.send(message);
  }

  /**
   * 关闭 socket 连接
   */
  close() {
    if (!this.socket) {
      return;
    }

    if (this.survivalSinalTimer) {
      clearTimeout(this.survivalSinalTimer);
    }

    if (this.closeSignalTimer) {
      clearTimeout(this.closeSignalTimer);
    }

    this.socket.close();
  }
}

export default ExSocket;
