import React from 'react';
import styles from './index.less';
import { toggleTabItem, formatterValue } from './util';

/**
 * 按钮组tab
 * tips: 使用 <ButtonTab className="mt-50" tabs={dataTabs} size="middle" />
 * tips: tab属性 {name: 文字, value: 值, active: 是否激活, disabled: 是否允许点击, isButton: 是否只当作一个可点击的button(优先级高于disabled)}
 *
 * @param props.tabs {array} tab数组: [{name: 'tab', value: 1, active: false, disabled: false, isButton: false}]
 * @param props.activeValue {array} 选中的值
 * @param props.multiple {boolean} 是否多选，默认false
 * @param props.size {string} 按钮大小 small/middle/large，默认small
 * @param props.className {string} 样式
 * @param props.justify {string} center
 * @param props.onTabClick {func} 点击tab的回调 (出参1：点击的tab, 出参2：激活的tab数组)
 */
const ButtonTab = (props) => {
  const tabs = props.tabs || [];
  const activeValue = props.activeValue || [];
  const multiple = props.multiple || false;
  const size = props.size || 'small';
  const className = props.className || '';
  const justify = props.justify;
  const onTabClick = props.onTabClick || null;

  let stateTabs = [...tabs];
  stateTabs.forEach((item) => {
    const active = activeValue.some((value) => item.value === value);
    item.active = active;
  });

  /**
   * 点击tab
   *
   * @param tab 点击的tab
   * @param tabIndex 点击的tab的index
   */
  const onClick = (tab, tabIndex) => {
    if (!onTabClick) {
      return false;
    }

    if (tab.isButton) {
      onTabClick(tab);
      return false;
    }

    if (tab.disabled) {
      return false;
    }

    let activeTab = [];

    if (!multiple) {
      // 单选且必选
      if (tab.active) {
        return false;
      }

      stateTabs.forEach((item, index) => {
        if (index === tabIndex) {
          toggleTabItem(item);
          activeTab = [item];
        } else {
          item.active = false;
        }
      });
    } else {
      // 多选
      toggleTabItem(tabs[tabIndex]);

      let hasActive = false,
        activeIndex = 0;

      activeTab.forEach((item, index) => {
        if (item.value === tab.value) {
          hasActive = true;
          activeIndex = index;
          return false;
        }
      });

      if (hasActive) {
        if (!tab.active) {
          // 存在并且取消激活
          activeTab.splice(activeIndex, 1);
        }
      } else {
        if (tab.active) {
          // 不存在并且激活
          activeTab.push(tab);
        }
      }
    }

    const value = formatterValue(activeTab);

    onTabClick(tab, value);
  };

  let ulStyle = {};
  let liStyle = {};
  if (justify && justify === 'center') {
    ulStyle = { display: 'flex', justifyContent: 'center' };
    liStyle = { display: 'flex' };
  }

  return (
    <ul className={`${styles.button__tab} ${className}`} style={ulStyle} data-size={size}>
      {stateTabs.map((item, tabIndex) => {
        return (
          <li
            style={liStyle}
            data-value={item.value}
            data-disabled={item.disabled}
            data-active={item.active}
            key={tabIndex}
            onClick={(e) => {
              onClick(item, tabIndex);
            }}
          >
            {item.name || ''}
          </li>
        );
      })}
    </ul>
  );
};

export default ButtonTab;
