import React from 'react';
import { Icon, DatePicker } from 'antd';
import style from './index.less';
import { computeTime, computePreTime, computeNextTime, formatDatePickerValue } from './util';

/**
 * 时间分页器,无状态组件
 *
 * @param props.type {number} 时间类型 {1: 日, 2: 周, 3: 月, 4: 年, 5: 近三个月}
 * @param props.timeStamp {number} 当前时间
 * @param props.onChange {func} 时间改变回调
 * @param props.onClick {func} 点击时间
 * @param props.className {string} 样式
 * @param props.preDisabled {boolean} 上一个disabled
 * @param props.nextDisabled {boolean} 下一个disabled
 * @param props.allowPicker {boolean} 允许中间的时间选择器，与onClick互斥
 */
const TimePagination = (props) => {
  const { className, preDisabled, nextDisabled, onChange, onClick, allowPicker } = props;
  const { type, timeStamp } = props;

  const date = timeStamp ? new Date(timeStamp) : new Date();
  const timeResult = computeTime(type, date);
  const timeText = timeResult.timeText || '';
  const datePickerValue = formatDatePickerValue(date);

  /**
   * 上一个
   */
  const onLeftClick = () => {
    if (preDisabled) {
      return;
    }

    const preTime = computePreTime(type, date);
    onChange && onChange(preTime);
  };

  /**
   * 下一个
   */
  const onRightClick = () => {
    if (nextDisabled) {
      return;
    }

    const nextTime = computeNextTime(type, date);
    onChange && onChange(nextTime);
  };

  /**
   * 点击中间时间
   */
  const onCenterClick = (e) => {
    onClick && onClick(timeResult);
  };

  /**
   * 时间选择器切换，计算选中时间并赋值
   *
   * @param date moment对象, 需转为Date对象
   * @param dateString
   */
  const onDatePickerChange = (date, dateString) => {
    const timeResult = computeTime(type, date.toDate());
    onChange && onChange(timeResult);
  };

  return (
    <div className={`${style.timeWrap} ${className}`}>
      <div className={`${style.timeBtn} bg-icon-pre`} data-disabled={preDisabled} onClick={onLeftClick}>
        <Icon type={'left'} />
      </div>
      <div className={style.timeText} data-click={!!onClick} onClick={onCenterClick}>
        {timeText}
      </div>
      <div className={`${style.timeBtn} bg-icon-next`} data-disabled={nextDisabled} onClick={onRightClick}>
        <Icon type={'right'} />
      </div>

      {allowPicker && (
        <DatePicker
          format="YYYY-MM-DD"
          value={datePickerValue}
          onChange={onDatePickerChange}
          className={style.timeDatePicker}
        />
      )}
    </div>
  );
};

export default TimePagination;
