import moment from 'moment';
import { SEASON_TYPE, DATE_TYPE } from './constant';

// 时间类型 {1: 日，2：周，3：月，4：年, 5：近三个月}
const timeTypeArr = [1, 2, 3, 4, 5];
// 一天的毫秒数
const milliSecond = 1000 * 60 * 60 * 24;

/**
 * 计算时间
 *
 * @param type 时间类型
 * @param date 时间
 */
const computeTime = function (type = 1, date = new Date()) {
  type = formatterType(type);

  let timeResult = {
    timeType: type,
    timeTypeText: DATE_TYPE[type],
    timeStamp: null, // 当前时间戳，用于计算上/下一个时间节点 (可能是某天/某周某一天/某月某一天/某年某一天)
    timeText: '--', // 时间文本
    startTime: null, // 开始时间 时间戳
    startTimeText: '', // 开始时间 格式化
    endTime: null, // 结束时间 时间戳
    endTimeText: '', // 结束时间 格式化
  };
  let computeResult = {};

  switch (type) {
    case 1:
      computeResult = computeDay(date);
      break;
    case 2:
      computeResult = computeWeek(date);
      break;
    case 3:
      computeResult = computeMonth(date);
      break;
    case 4:
      computeResult = computeYear(date);
      break;
    case 5:
      computeResult = compute3month(date);
      break;
    default:
      computeResult = computeDay(date);
      break;
  }

  timeResult = Object.assign({}, timeResult, computeResult);

  return timeResult;
};

/**
 * 计算上一个时间
 * @param type
 * @param date
 */
const computePreTime = function (type = 1, date = new Date()) {
  let nowDate = null;

  switch (type) {
    case 1:
      nowDate = new Date(date.getTime() - milliSecond);
      break;
    case 2:
      nowDate = new Date(date.getTime() - 7 * milliSecond);
      break;
    case 3:
      nowDate = date;
      nowDate.setMonth(nowDate.getMonth() - 1);
      break;
    case 4:
      nowDate = date;
      nowDate.setFullYear(nowDate.getFullYear() - 1);
      break;
    case 5:
      nowDate = date;
      nowDate.setMonth(nowDate.getMonth() - 1);
      break;
    default:
      nowDate = new Date(date.getTime() - milliSecond);
      break;
  }

  return computeTime(type, nowDate);
};

/**
 * 计算下一个时间
 * @param type
 * @param date
 */
const computeNextTime = function (type = 1, date = new Date()) {
  let nowDate = null;

  switch (type) {
    case 1:
      nowDate = new Date(date.getTime() + milliSecond);
      break;
    case 2:
      nowDate = new Date(date.getTime() + 7 * milliSecond);
      break;
    case 3:
      nowDate = date;
      nowDate.setMonth(nowDate.getMonth() + 1);
      break;
    case 4:
      nowDate = date;
      nowDate.setFullYear(nowDate.getFullYear() + 1);
      break;
    case 5:
      nowDate = date;
      nowDate.setMonth(nowDate.getMonth() + 1);
      break;
    default:
      nowDate = new Date(date.getTime() + milliSecond);
      break;
  }

  return computeTime(type, nowDate);
};

/**
 * 格式化type，防止程序出错
 * @param type
 * @returns {number}
 */
function formatterType(type) {
  let hasType = timeTypeArr.some((value) => {
    return value === type;
  });

  if (!hasType) {
    type = 1;
  }

  return type;
}

/**
 * 计算日的起止时间, startTime === endTime
 * @param date
 */
function computeDay(date = new Date()) {
  let timeStamp = date.getTime();
  let timeText = moment(date).format('YYYY.MM.DD');

  return {
    timeStamp: timeStamp,
    timeText: `${timeText}`, // `${timeText}-${timeText}`
    startTime: timeStamp,
    startTimeText: moment(date).format('YYYY-MM-DD'),
    endTime: timeStamp,
    endTimeText: moment(date).format('YYYY-MM-DD'),
  };
}

/**
 * 计算周的起止时间, startTime < endTime
 * @param date
 */
function computeWeek(date = new Date()) {
  let firstDate = getFirstDayOfWeek(date);
  let lastDate = getLastDayOfWeek(date);
  let timeText = moment(firstDate).format('YYYY.MM.DD') + '-' + moment(lastDate).format('YYYY.MM.DD');

  return {
    timeStamp: date.getTime(),
    timeText: timeText,
    startTime: firstDate.getTime(),
    startTimeText: moment(firstDate).format('YYYY-MM-DD'),
    endTime: lastDate.getTime(),
    endTimeText: moment(lastDate).format('YYYY-MM-DD'),
  };
}
/**
 * 获取当前周的第一天
 * @param date
 * @return {Date}
 */
function getFirstDayOfWeek(date) {
  // 0(日)~6(六) -> 1(一)~7(日)
  let day = date.getDay() || 7;
  return new Date(date.getFullYear(), date.getMonth(), date.getDate() + 1 - day);
}
/**
 * 获取当前周的最后一天
 * @param date
 * @returns {Date}
 */
function getLastDayOfWeek(date) {
  const copyDate = new Date(date);
  const firstDate = getFirstDayOfWeek(copyDate);
  const lastDate = moment(firstDate).add(6, 'day');

  return new Date(lastDate);
}

/**
 * 计算月份的起止时间, startTime < endTime
 * 注意 先保存当前时间再计算起止日期
 * @param date
 */
function computeMonth(date = new Date()) {
  const timeStamp = date.getTime();
  let firstDate = getFirstDayOfMonth(date);
  let lastDate = getLastDayOfMonth(date);
  let timeText = moment(firstDate).format('YYYY.MM.DD') + '-' + moment(lastDate).format('YYYY.MM.DD');

  return {
    timeStamp: timeStamp,
    timeText: timeText,
    startTime: firstDate.getTime(),
    startTimeText: moment(firstDate).format('YYYY-MM-DD'),
    endTime: lastDate.getTime(),
    endTimeText: moment(lastDate).format('YYYY-MM-DD'),
  };
}
/**
 * 获取当前月第一天
 *
 * @param date
 * @returns {Date}
 */
function getFirstDayOfMonth(date) {
  const copyDate = new Date(date);
  copyDate.setDate(1);

  return copyDate;
}
/**
 * 获取当前月最后一天
 * @param date
 * @returns {Date}
 */
function getLastDayOfMonth(date) {
  // 0~11
  const month = date.getMonth();

  const nextMonth = month + 1;
  const nextMonthFirstDay = new Date(date.getFullYear(), nextMonth, 1);
  // 获取当月最后一天
  const lastDate = new Date(nextMonthFirstDay - milliSecond);

  return lastDate;
}

/**
 * 计算年份的起止时间, startTime === endTime
 * @param date
 */
function computeYear(date = new Date()) {
  const timeStamp = date.getTime();
  const year = date.getFullYear();

  return {
    timeStamp: timeStamp,
    timeText: year,
    startTime: timeStamp,
    startTimeText: moment(date).format('YYYY-MM-DD'),
    endTime: timeStamp,
    endTimeText: moment(date).format('YYYY-MM-DD'),
  };
}

/**
 * 计算近三个月起止时间, startTime < endTime
 * @param date
 */
function compute3month(date = new Date()) {
  let result = getDuringOf3Month(date);
  let firstDate = result.firstDate;
  let lastDate = result.lastDate;
  let timeText = moment(firstDate).format('YYYY.MM.DD') + '-' + moment(lastDate).format('YYYY.MM.DD');

  return {
    timeStamp: date.getTime(),
    timeText: timeText,
    startTime: firstDate.getTime(),
    startTimeText: moment(firstDate).format('YYYY-MM-DD'),
    endTime: lastDate.getTime(),
    endTimeText: moment(lastDate).format('YYYY-MM-DD'),
  };
}

/**
 * 获取当前时间的前2个月的第一天与当前月份最后一天
 * @param date
 * @returns {{firstDate: Date, lastDate: Date}}
 */
function getDuringOf3Month(date) {
  // 当前年
  let nowYear = date.getFullYear();
  // 当前年/月,注意 + 1
  let nowMonth = date.getMonth() + 1;

  // 前三个月年/月
  let preYear = nowYear;
  let preMonth = nowMonth;

  if (nowMonth <= 2) {
    let desc = nowMonth - 2;
    let trueMonth = 12 - Math.abs(desc);
    preMonth = trueMonth - 1; // 0~11

    preYear = nowYear - 1;
  } else {
    let desc = nowMonth - 2;
    preMonth = desc - 1;
  }

  // 前2个月第一天
  let firstDate = new Date(preYear, preMonth, 1);
  // 当前月份的最后一天
  let lastDate = getLastDayOfMonth(date);

  return {
    firstDate: firstDate,
    lastDate: lastDate,
  };
}

/**
 * 计算当前日期的季度
 * @param date
 * @returns {number}
 */
const computeSeason = function (date = new Date()) {
  let month = date.getMonth() + 1;
  let seasonValue = 1;

  Object.keys(SEASON_TYPE).forEach((key) => {
    if (SEASON_TYPE[key].indexOf(month) > -1) {
      seasonValue = key;
      return false;
    }
  });

  return parseInt(seasonValue);
};

/**
 * 计算时间选择器的值
 *
 * @param date
 * @return {any}
 */
const formatDatePickerValue = function (date = new Date()) {
  return date ? moment(date, 'YYYY-MM-DD') : null;
};

export { computeTime, computePreTime, computeNextTime, computeSeason, formatDatePickerValue };
