import React from 'react';
import { Col, Row } from 'antd';
import { MultiSelect, NoData, Loading } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import echarts from 'ASSET/js/echarts.min';
import styles from '../styles.less';
import ssStyle from './index.less';
import TopList from './TopList';
import ButtonTab from './ButtonTab';
import TimePagination from './TimePagination';
import { computeTime } from './TimePagination/util';
import { loadCommonDictByGroup } from 'UTIL/dict';
import { getTopRankForSsBossBy } from 'SERVICE/home';
import {
  PRODUCT_TYPE_SMALL_CLASS,
  PRODUCT_TYPE_TWO_TEACHER,
  productTypeDict,
  tempCourseSeriesDict,
  tempClassTypeDict,
} from './dict';
import { chartOption, calcDefaultTimeValue, calcDefaultDictValue, formatStateByProductType } from './util';

/**
 * 双师-运营业绩TOP10概览
 */
class TopTenSS extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      data: [], // 数据

      tempTimeTypeTab: null, // 时间类型tab
      tempTimeTypeTabValue: [], // 时间类型tab选中值
      tempTimeType: null, // 当前时间类型
      tempTimeStamp: null, // 当前时间戳
      tempCourseSeries: [], // 临时产品系列(标识)
      tempClassType: [], // 临时班级类型(标识)

      startDate: null, // 开始时间
      endDate: null, // 结束时间
      productType: [], // 产品类型
      productSeries: [], // 产品系列
      productCharacter: [], // 产品性质
      classType: [], // 班级类型
      statisticsRole: null, // 统计角色 课程顾问、分校负责人、运营部负责人
      rankings: 10, // 排名数

      commonDict: {
        productCharacterDict: [], // 产品性质字典
      },
    };
    this.twoTeacherDict = {
      courseSeriesDict: [], // 产品系列字典
      classTypeDict: [], // 班级类型字典
    };
    this.smallClassDict = {
      courseSeriesDict: [], // 产品系列字典
      classTypeDict: [], // 班级类型字典
    };

    this.$chartDom = null;
    this.chart = null;
  }

  componentDidMount() {
    this.initChart();
    this.initDict();
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.onResize, false);
  }

  /**
   * 初始化图表
   */
  initChart = () => {
    if (this.$chartDom) {
      this.chart = echarts.init(this.$chartDom);
    }

    window.addEventListener('resize', this.onResize, false);
  };

  /**
   * 初始化字典
   * tips: 由于category key相同，小班双师分开请求(产品系列、班级类型)
   */
  initDict = async () => {
    const categoryTwoTeacher = [
      { category: 'COURSE_SERIES', parentId: PRODUCT_TYPE_TWO_TEACHER },
      { category: 'CLASS_TYPE', parentId: PRODUCT_TYPE_TWO_TEACHER },
    ];
    let categorySmallClass = [
      { category: 'COURSE_SERIES', parentId: PRODUCT_TYPE_SMALL_CLASS },
      { category: 'CLASS_TYPE', parentId: PRODUCT_TYPE_SMALL_CLASS },
    ];
    const categoryCommon = [{ category: 'PRODUCT_CHARACTER', parentId: '' }];

    // 共用的字典拼接到小班一同请求, commonDict需放入state
    categorySmallClass = categorySmallClass.concat(categoryCommon);

    const [resTwoTeacher, resSmallClass] = await Promise.all([
      loadCommonDictByGroup(categoryTwoTeacher),
      loadCommonDictByGroup(categorySmallClass),
    ]);

    if (resTwoTeacher && resSmallClass) {
      const twoTeacherDict = {
        courseSeriesDict: resTwoTeacher['COURSE_SERIES'] || [],
        classTypeDict: resTwoTeacher['CLASS_TYPE'] || [],
      };
      const smallClassDict = {
        courseSeriesDict: resSmallClass['COURSE_SERIES'] || [],
        classTypeDict: resSmallClass['CLASS_TYPE'] || [],
      };
      const commonDict = {
        productCharacterDict: resSmallClass['PRODUCT_CHARACTER'] || [],
      };

      this.twoTeacherDict = twoTeacherDict;
      this.smallClassDict = smallClassDict;

      this.setState({ commonDict });
      this.initState();
    }
  };

  /**
   * 初始化状态
   */
  initState = () => {
    const defaultTimeValue = calcDefaultTimeValue();
    const defaultDictValue = calcDefaultDictValue(this.state.commonDict.productCharacterDict);

    this.setState(
      {
        ...defaultTimeValue,
        ...defaultDictValue,
      },
      () => {
        this.loadData();
      },
    );
  };

  onResize = () => {
    if (this.chart) {
      this.chart.resize();
    }
  };

  /**
   * 手动触发重绘
   */
  doResize = () => {
    const e = document.createEvent('Event');
    e.initEvent('resize', true, true);
    window.dispatchEvent(e);
  };

  /**
   * 获取统计数据
   *
   * @return {Promise<void>}
   */
  loadData = async () => {
    if (this.state.loading) {
      return;
    }

    const apiParam = this.formatSearchParam();
    const isOk = this.checkParamOk(apiParam);

    if (!isOk) {
      showMessage('筛选条件不能为空！', 'error');
      return;
    }

    this.setState({ loading: true });
    const data = await getTopRankForSsBossBy(apiParam).catch((e) => {
      this.setState({ loading: false });
      this.formatResponseData([]);
    });

    this.setState({ loading: false });
    this.formatResponseData(data);
    this.doResize();
  };
  /**
   * 检查请求参数是否完整
   *
   * @param param 请求参数
   * @return {boolean}
   */
  checkParamOk = (param) => {
    let flag = true;
    const { productType, productSeries, productCharacter, classType } = param;
    const validArr = [productType, productSeries, productCharacter, classType];

    validArr.forEach((item) => {
      if (!item || item.length <= 0) {
        flag = false;
        return false;
      }
    });

    return flag;
  };
  /**
   * 格式化请求参数，注意多选的处理
   * tips: productType影响产品系列及班级类型，分星火和双师处理
   *
   * @return {*}
   */
  formatSearchParam = () => {
    const { startDate, endDate, productType, productCharacter, rankings } = this.state;
    const { tempCourseSeries, tempClassType } = this.state;
    const twoTeacherDict = this.twoTeacherDict;
    const smallClassDict = this.smallClassDict;
    const param = { productType, tempCourseSeries, tempClassType, twoTeacherDict, smallClassDict };
    const result = formatStateByProductType(param);

    return {
      startDate,
      endDate,
      productType,
      productSeries: result.productSeries,
      productCharacter,
      classType: result.classType,
      rankings,
    };
  };
  /**
   * 格式化请求数据
   *
   * @param data
   */
  formatResponseData = (data = []) => {
    let xAxisData = [];
    let seriesData = [];

    if (data) {
      data.forEach((item, index) => {
        xAxisData.push(item.userName);
        seriesData.push(item.amount);
      });
    }

    if (this.chart) {
      let option = { ...chartOption };
      option.xAxis.data = xAxisData;
      option.series[0].data = seriesData;

      this.chart.setOption(option);
    }

    this.setState({ data });
  };

  /**
   * 时间维度切换
   *
   * @param tab 点击的tab
   * @param activeValue 激活的tab值
   */
  onTimeTabClick = (tab, activeValue) => {
    this.setState(
      {
        tempTimeTypeTabValue: activeValue,
        tempTimeType: activeValue[0],
      },
      () => {
        const result = computeTime(activeValue[0]);
        this.onTimePagerChange(result);
      },
    );
  };

  /**
   * 时间分页器切换
   *
   * @param result
   */
  onTimePagerChange = (result) => {
    this.setState(
      {
        tempTimeStamp: result.timeStamp,
        startDate: result.startTimeText,
        endDate: result.endTimeText,
      },
      () => {
        this.loadData();
      },
    );
  };

  /**
   * 选项改变
   *
   * @param val
   * @param stateKey
   */
  onSelectChange = (val, stateKey) => {
    this.setState(
      {
        [stateKey]: val,
      },
      () => {
        this.loadData();
      },
    );
  };

  render() {
    // 保留chartDom
    const isNoData = !this.state.data || this.state.data.length <= 0;
    const chartStyle = { display: isNoData ? 'none' : 'block' };
    const noDataStyle = { display: isNoData ? 'block' : 'none' };

    return (
      <div className={styles.section}>
        <Loading loading={this.state.loading} />

        <div className={styles.title}>
          <div className={styles.inner}>运营业绩TOP10概览</div>
        </div>

        <Row type="flex" justify="center" className={ssStyle.conditionWrap}>
          <Col span={3}>
            <ButtonTab
              tabs={this.state.tempTimeTypeTab}
              activeValue={this.state.tempTimeTypeTabValue}
              onTabClick={this.onTimeTabClick}
              justify="center"
            />

            <TimePagination
              type={this.state.tempTimeType}
              timeStamp={this.state.tempTimeStamp}
              onChange={this.onTimePagerChange}
              allowPicker={true}
            />
          </Col>
          <Col span={2}>
            <MultiSelect
              showSearch={false}
              allowClear={false}
              placeholder="产品类型"
              dict={productTypeDict}
              value={this.state.productType}
              onChange={(val) => this.onSelectChange(val, 'productType')}
            />
          </Col>
          <Col span={2}>
            <MultiSelect
              showSearch={false}
              allowClear={false}
              placeholder="产品系列"
              dict={tempCourseSeriesDict}
              value={this.state.tempCourseSeries}
              onChange={(val) => this.onSelectChange(val, 'tempCourseSeries')}
            />
          </Col>
          <Col span={2}>
            <MultiSelect
              showSearch={false}
              allowClear={false}
              placeholder="产品性质"
              dict={this.state.commonDict.productCharacterDict || []}
              value={this.state.productCharacter}
              onChange={(val) => this.onSelectChange(val, 'productCharacter')}
            />
          </Col>
          <Col span={2}>
            <MultiSelect
              showSearch={false}
              allowClear={false}
              placeholder="班级类型"
              dict={tempClassTypeDict}
              value={this.state.tempClassType}
              onChange={(val) => this.onSelectChange(val, 'tempClassType')}
            />
          </Col>
        </Row>

        <Row gutter={48} style={chartStyle}>
          <Col span={16}>
            <div
              className={styles.echarts}
              ref={(dom) => {
                this.$chartDom = dom;
              }}
            />
          </Col>
          <Col span={8}>
            <TopList data={this.state.data} />
          </Col>
        </Row>

        <Row gutter={48} className={ssStyle.noDataWrap} style={noDataStyle}>
          <NoData />
        </Row>
      </div>
    );
  }
}

export default TopTenSS;
