import {
  PRODUCT_TYPE_SMALL_CLASS,
  PRODUCT_TYPE_TWO_TEACHER,
  productTypeDict,
  TEMP_COURSE_SERIES_LONG,
  tempClassTypeDict,
  PRODUCT_CHARACTER_TEXT_NORMAL,
  tempCourseSeriesDict,
} from './dict';
import { getDictValByText, getDictTextByVal } from 'UTIL/dict';
import { computeTime } from './TimePagination/util';

/**
 * 初始化时间
 *
 * @return {*}
 */
export function calcDefaultTimeValue() {
  const timeTypeTab = [
    { name: '今', value: 1 },
    { name: '周', value: 2 },
    { name: '月', value: 3 },
  ];
  const timeType = 3;
  const result = computeTime(timeType, new Date());

  return {
    tempTimeTypeTab: timeTypeTab,
    tempTimeTypeTabValue: [timeType],
    tempTimeType: timeType,
    tempTimeStamp: result.timeStamp,
    startDate: result.startTimeText,
    endDate: result.endTimeText,
  };
}

/**
 * 计算除时间以外的初始state
 * 产品类型 可多选，默认全选
 * 产品系列 可多选，默认长期班
 * 产品性质 可多选，默认常规课程
 * 班级类型 可多选，默认全选
 *
 * @param productCharacterDict 产品性质字典
 * @return {*}
 */
export function calcDefaultDictValue(productCharacterDict = []) {
  const productCharacterNormal = getDictValByText(productCharacterDict, PRODUCT_CHARACTER_TEXT_NORMAL);
  const productType = productTypeDict.map((item) => item.val);
  const courseSeries = [TEMP_COURSE_SERIES_LONG];
  const productCharacter = (productCharacterNormal && [productCharacterNormal]) || [];
  const classType = tempClassTypeDict.map((item) => item.val);

  return {
    productType: productType,
    tempCourseSeries: courseSeries,
    productCharacter: productCharacter,
    tempClassType: classType,
  };
}

/**
 * 请求参数中，根据所选产品类型格式化产品系列及班级类型，最后拼接成数组
 * tips: 未选产品类型，但选了产品系列或班级类型，则返回`双师及小班`的`产品系列或班级类型`
 * tips: 选中双师或小班产品类型，返回`双师或小班`对应的`产品系列或班级类型`
 * tips: 班级类型用名称模糊搜索获取值
 * eg: 不选产品，返回双师及小班类型的产品系列及班级类型
 * eg: 选中双师产品类型，同时选中产品系列及班级类型-返回双师类型的产品系列及班级类型
 *
 * @param param.productType {array} 产品类型
 * @param param.tempCourseSeries {array} 选中的产品系列标识 [长期班，短期班]
 * @param param.tempClassType {array} 选中的班级类型标识 [收费班、0元班]
 * @param param.twoTeacherDict {object} 双师字典集
 * @param param.smallClassDict {object} 小班字典集
 *
 * @return {{productSeries: {array}, classType: {array}}}
 */
export function formatStateByProductType(param) {
  const { tempCourseSeries, tempClassType, twoTeacherDict, smallClassDict } = param;
  const productType = param.productType || [];
  // 是否模糊搜索
  const isFuzzySearch = true;

  const ssCourseSeriesDict = twoTeacherDict.courseSeriesDict || [];
  const ssClassTypeDict = twoTeacherDict.classTypeDict || [];
  const xbCourseSeriesDict = smallClassDict.courseSeriesDict || [];
  const xbClassTypeDict = smallClassDict.classTypeDict || [];

  // 产品系列
  let productSeries = [];
  // 班级类型
  let classType = [];

  const hasSSProduct = productType.some((type) => type === PRODUCT_TYPE_TWO_TEACHER);
  const hasXBProduct = productType.some((type) => type === PRODUCT_TYPE_SMALL_CLASS);
  const hasCourseSeries = tempCourseSeries && tempCourseSeries.length > 0;
  const hasClassType = tempClassType && tempClassType.length > 0;

  // 什么类型都不选则返回双师及小班的产品系列、班级类型(为了代码结构一致)
  if (!hasSSProduct && !hasXBProduct) {
    // 组装产品系列
    if (hasCourseSeries) {
      tempCourseSeries.forEach((val) => {
        const courseSeriesText = getDictTextByVal(tempCourseSeriesDict, val);
        const courseSeriesVal = getDictValByText(ssCourseSeriesDict, courseSeriesText);
        if (courseSeriesVal) {
          productSeries.push(courseSeriesVal);
        }
      });

      tempCourseSeries.forEach((val) => {
        const courseSeriesText = getDictTextByVal(tempCourseSeriesDict, val);
        const courseSeriesVal = getDictValByText(xbCourseSeriesDict, courseSeriesText);
        if (courseSeriesVal) {
          productSeries.push(courseSeriesVal);
        }
      });
    }

    // 组装班级类型
    if (hasClassType) {
      tempClassType.forEach((val) => {
        const classTypeText = getDictTextByVal(tempClassTypeDict, val);

        // 双师产品
        if (!hasSSProduct) {
          if (isFuzzySearch) {
            const classTypeValArr = findDictValByText(ssClassTypeDict, classTypeText);
            classType = classType.concat(classTypeValArr);
          } else {
            const classTypeVal = getDictValByText(ssClassTypeDict, classTypeText);
            if (classTypeVal) {
              classType.push(classTypeVal);
            }
          }
        }

        // 小班产品
        if (!hasXBProduct) {
          if (isFuzzySearch) {
            const classTypeValArr = findDictValByText(xbClassTypeDict, classTypeText);
            classType = classType.concat(classTypeValArr);
          } else {
            const classTypeVal = getDictValByText(xbClassTypeDict, classTypeText);
            if (classTypeVal) {
              classType.push(classTypeVal);
            }
          }
        }
      });
    }
  } else {
    // 组装产品系列
    if (hasCourseSeries) {
      tempCourseSeries.forEach((val) => {
        const courseSeriesText = getDictTextByVal(tempCourseSeriesDict, val);

        if (hasSSProduct) {
          const courseSeriesVal = getDictValByText(ssCourseSeriesDict, courseSeriesText);
          if (courseSeriesVal) {
            productSeries.push(courseSeriesVal);
          }
        }

        if (hasXBProduct) {
          const courseSeriesVal = getDictValByText(xbCourseSeriesDict, courseSeriesText);
          if (courseSeriesVal) {
            productSeries.push(courseSeriesVal);
          }
        }
      });
    }

    // 组装班级类型
    if (hasClassType) {
      tempClassType.forEach((val) => {
        const classTypeText = getDictTextByVal(tempClassTypeDict, val);

        // 双师产品
        if (hasSSProduct) {
          if (isFuzzySearch) {
            const classTypeValArr = findDictValByText(ssClassTypeDict, classTypeText);
            classType = classType.concat(classTypeValArr);
          } else {
            const classTypeVal = getDictValByText(ssClassTypeDict, classTypeText);
            if (classTypeVal) {
              classType.push(classTypeVal);
            }
          }
        }

        // 小班产品
        if (hasXBProduct) {
          if (isFuzzySearch) {
            const classTypeValArr = findDictValByText(xbClassTypeDict, classTypeText);
            classType = classType.concat(classTypeValArr);
          } else {
            const classTypeVal = getDictValByText(xbClassTypeDict, classTypeText);
            if (classTypeVal) {
              classType.push(classTypeVal);
            }
          }
        }
      });
    }
  }

  return {
    productSeries,
    classType,
  };
}

/**
 * 根据text模糊查找字典中对应的值
 *
 * @param dict 字典
 * @param text 需要搜索的文本
 * @return {Array} 值列表
 */
function findDictValByText(dict, text) {
  if (!dict || dict.length <= 0 || !text) {
    return [];
  }

  let optionResult = [];
  let valResult = [];
  const hasLabel = !!dict[0].label;

  if (hasLabel) {
    dict.forEach((item) => {
      if (item.options && item.options.length > 0) {
        item.options.forEach((option) => {
          if (option.text.indexOf(text) > -1) {
            optionResult.push(option);
          }
        });
      }
    });
  } else {
    dict.forEach((option) => {
      if (option.text.indexOf(text) > -1) {
        optionResult.push(option);
      }
    });
  }

  if (optionResult.length > 0) {
    valResult = optionResult.map((option) => option.val);
  }

  return valResult;
}

/**
 * 图表配置
 *
 * @type {*}
 */
export const chartOption = {
  color: ['#5B63FF', '#DD93F7', '#FDE9FF'],
  tooltip: {
    trigger: 'item',
    axisPointer: {
      type: 'line',
    },
    position: 'top',
    formatter: '{b} <br/>{c}',
  },
  legend: {
    data: [],
    itemWidth: 8,
    itemHeight: 8,
    borderRadius: 6,
    right: '10%',
  },
  grid: {
    top: '30px',
    left: '1%',
    right: '4%',
    bottom: '3%',
    containLabel: true,
  },
  yAxis: {
    type: 'value',
    name: '科次',
    axisLine: {
      show: false,
    },
    splitLine: {
      lineStyle: {
        opacity: 0.3,
      },
    },
    axisTick: {
      show: false,
    },
    minInterval: 1,
  },
  xAxis: {
    axisLine: {
      show: false,
    },
    axisTick: {
      show: false,
    },
    type: 'category',
    data: [],
  },
  series: [
    {
      type: 'bar',
      stack: '总量',
      barWidth: 16,
      data: [],
    },
  ],
};
