import React from 'react';
import moment from 'moment';
import { Form, DatePicker, TimePicker, Row, Col } from 'antd';
import { Modal, FormItem, Select, TextArea } from 'COMPONENT';
import { DATE_FORMAT, TIME_FORMAT } from 'COMPONENT/utils/form';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import * as dicts from '../../../../common/util/dicts';
import * as loaders from '../../loaders';
import * as utils from './util';

const FormItemNative = Form.Item;
const formItemStyle = { padding: 0 };
const labelWidth = 96;
const labelName = {
  intentionCampus: '意向学习中心',
  dealStatus: '外呼跟进状态',
  intention: '意向度',
  isAppointCome: '是否诺到',
  appointDate: '诺到日期',
  appointTime: '诺到时间',
  remark: '跟进内容',
};

/**
 * 添加跟进记录弹窗
 *
 * @param props.row {object} 跟进客户列表行数据
 * @param props.dict {object} 字典集
 * @param props.onClose {function} 弹窗关闭回调
 */
class ModalAddRecord extends React.Component {
  static defaultProps = {
    row: null,
    dict: {},
    onClose: () => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      dateRequire: false, // 表单项诺到时间是否必填（state管理才能动态更新表单校验规则）
      dealStatusDict: [], // 外呼跟进状态字典（兼容资源调转后跟进状态丢失问题）
    };
    this.$modal = {};
  }

  componentDidMount() {
    this.initLastRecord();
  }

  /**
   * 初始化上一次跟进记录
   */
  initLastRecord = () => {
    const { lastFollowUpId } = this.props.row;

    if (!lastFollowUpId) {
      this.handleDefault();
      return;
    }

    this.toggleLoading();
    loaders
      .loadLastFollowRecord({ lastFollowUpId })
      .then((data) => {
        this.toggleLoading();

        if (!data) {
          return;
        }

        const {
          form: { setFieldsValue },
        } = this.props;
        const { intentionCampusId, dealStatus, intentionOfTheCustomerId, remark } = data;
        const appointDate = data.appointDate || '';
        const dataArr = appointDate.split(' ');
        const date = dataArr[0] || null;
        const time = dataArr[1] || null;

        setFieldsValue({
          intentionCampus: intentionCampusId,
          dealStatus,
          intention: intentionOfTheCustomerId,
          appointDate: date ? moment(appointDate) : null,
          appointTime: time ? moment(appointDate) : null,
          remark,
        });

        const dealStatusDict = utils.concatCalloutDealStatusDict(dealStatus);
        this.setState({ dealStatusDict });
      })
      .catch(() => {
        this.toggleLoading();
        this.handleDefault();
      });
  };

  /**
   * 默认情况处理
   */
  handleDefault = () => {
    const dealStatusDict = utils.concatCalloutDealStatusDict();
    this.setState({ dealStatusDict });

    const { intentionCampusId, appointmentDate } = this.props.row;
    const {
      form: { setFieldsValue },
    } = this.props;
    setFieldsValue({
      intentionCampus: intentionCampusId || null,
      appointDate: appointmentDate ? moment(appointmentDate) : moment(),
    });
  };

  /**
   * 跟进状态变化改变表单校验项
   *
   * @param val {string} 跟进状态
   */
  onDealStatusChange = (val) => {
    const {
      form: { setFields, getFieldValue },
    } = this.props;
    const appointDate = getFieldValue('appointDate');
    const dateRequire = val === dicts.CALLOUT_DEAL_STATUS_FOLLOW_COMPLETED;
    const dateError = dateRequire && !appointDate ? [`请选择${labelName.appointDate}`] : null;

    this.setState({ dateRequire }, () => {
      setFields({
        appointDate: {
          value: appointDate,
          errors: dateError,
        },
      });
    });
  };

  /**
   * 禁用今天之前的时间
   *
   * @param value {string} 时间
   * @return {boolean}
   */
  disableDateBeforeToday = (value) => {
    if (!value) {
      return false;
    }

    return moment(value).valueOf() < moment(moment().format(DATE_FORMAT)).valueOf();
  };

  /**
   * 提交
   *
   * @param e
   */
  onSubmit = (e) => {
    e.preventDefault();

    this.props.form.validateFieldsAndScroll((err, values) => {
      if (err) {
        this.$modal.showOkTips('请完善信息再进行操作!');
        return;
      }

      this.doSubmit(values);
    });
  };

  /**
   * 执行提交
   *
   * @param values {object} 表单项值
   */
  doSubmit = (values) => {
    const params = this.formatSubmitParam(values);

    this.toggleLoading();
    loaders
      .saveLastFollowRecord(params)
      .then(() => {
        showMessage('跟进成功', 'success');
        this.toggleLoading();
        this.props.onClose('success');
      })
      .catch(() => {
        this.toggleLoading();
      });
  };

  /**
   * 格式化请求参数
   * tips: 已预约状态对应诺到日期必填
   *
   * @param values {object} 表单项值
   */
  formatSubmitParam = (values) => {
    const { appointDate, appointTime, dealStatus, intention, intentionCampus, remark } = values;
    const {
      row: { id },
    } = this.props;

    let resultDate = '';
    if (appointDate) {
      resultDate = appointDate.format(DATE_FORMAT);
    }
    if (appointTime) {
      resultDate += ' ' + appointTime.format(TIME_FORMAT);
    }

    return {
      id,
      appointDate: resultDate,
      dealStatus,
      intentionOfTheCustomerId: intention,
      intentionCampusId: intentionCampus,
      remark,
    };
  };

  toggleLoading = () => {
    this.setState((prevState) => {
      return {
        loading: !prevState.loading,
      };
    });
  };

  render() {
    const {
      form: { getFieldDecorator },
    } = this.props;

    return (
      <Modal
        title="跟进记录"
        width={800}
        loading={this.state.loading}
        onClose={this.props.onClose}
        onOk={this.onSubmit}
        ref={(c) => (this.$modal = getWrappedInstance(c))}
      >
        <Form>
          <Row gutter={15}>
            <Col span={8}>
              <FormItem labelWidth={labelWidth} name={labelName.intentionCampus} required style={formItemStyle}>
                <FormItemNative>
                  {getFieldDecorator('intentionCampus', {
                    rules: [{ required: true, message: `请选择${labelName.intentionCampus}` }],
                  })(<Select placeholder="请选择" dict={this.props.dict.blCampusDict || null} allowClear={false} />)}
                </FormItemNative>
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem labelWidth={labelWidth} name={labelName.dealStatus} required style={formItemStyle}>
                <FormItemNative>
                  {getFieldDecorator('dealStatus', {
                    rules: [{ required: true, message: `请选择${labelName.dealStatus}` }],
                  })(
                    <Select
                      placeholder="请选择"
                      dict={this.state.dealStatusDict}
                      allowClear={false}
                      onChange={this.onDealStatusChange}
                    />,
                  )}
                </FormItemNative>
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem labelWidth={labelWidth} name={labelName.intention} required style={formItemStyle}>
                <FormItemNative>
                  {getFieldDecorator('intention', {
                    rules: [{ required: true, message: `请选择${labelName.intention}` }],
                  })(<Select placeholder="请选择" dict={this.props.dict.intentionDict || null} allowClear={false} />)}
                </FormItemNative>
              </FormItem>
            </Col>
          </Row>

          <Row gutter={15}>
            <Col span={12}>
              <FormItem
                labelWidth={labelWidth}
                name={labelName.appointDate}
                required={this.state.dateRequire}
                style={formItemStyle}
              >
                <FormItemNative>
                  {getFieldDecorator('appointDate', {
                    rules: [{ required: this.state.dateRequire, message: `请选择${labelName.appointDate}` }],
                  })(<DatePicker />)}
                </FormItemNative>
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem labelWidth={10} name="" style={formItemStyle}>
                <FormItemNative>
                  {getFieldDecorator('appointTime', {
                    rules: [],
                  })(<TimePicker format={TIME_FORMAT} />)}
                </FormItemNative>
              </FormItem>
            </Col>
          </Row>

          <Row>
            <Col>
              <FormItem labelWidth={labelWidth} name={labelName.remark} required style={formItemStyle}>
                <FormItemNative>
                  {getFieldDecorator('remark', {
                    rules: [{ required: true, max: 1000, message: `${labelName.remark}不能超过1000字` }],
                  })(<TextArea rows={3} wordCount maxLength={1000} />)}
                </FormItemNative>
              </FormItem>
            </Col>
          </Row>
        </Form>
      </Modal>
    );
  }
}

export default Form.create()(ModalAddRecord);
