import React from 'react';
import { Tree, Input } from 'antd';
import { Modal, NoData, Select } from 'COMPONENT';
import {
  filterTreeData,
  transformArrayToTree,
  filterDefaultExpandKeys,
  formatNodeName,
  getParentKey,
  getNodeKey,
  findNodesFromOriginDataByIds,
} from './util';
import style from './index.less';
import * as loaders from '../../loaders';

const TreeNode = Tree.TreeNode;
const Search = Input.Search;
const roleCodeConsultor = 'CONSULTOR';
const roleCodeCallOut = 'OUTCALL_SPEC';
const roleCodeDict = [
  { text: '课程顾问', val: roleCodeConsultor },
  { text: '外呼专员', val: roleCodeCallOut },
];

/**
 * 资源调配树
 *
 * @param props.loading {boolean} 外层控制的loading(外层正在请求接口时使用)
 * @param props.onOk {func} 确认回调，回调参数为选中的树节点ID、选中的树节点
 * @param props.onClose {func} 弹窗关闭
 */
class ModalCalloutPoolTree extends React.Component {
  static defaultProps = {
    loading: false,
    onOk: () => {},
    onClose: () => {},
  };

  constructor(props) {
    super(props);

    this.defaultState = {
      searchValue: null, // 输入框值
      originData: null, // 一维的树节点数据(搜索使用)
      treeData: null, // 树型结构数据
      selectedKeys: [], // 点击的节点key(id)
      selectedNodes: [], // 点击的节点
      checkedKeys: [], // 勾选选中的节点key(id)
      checkedNodes: [], // 勾选选中的节点
      expandedKeys: [], // 展开的节点key(id)
      autoExpandParent: true, // 自动展开父节点
    };
    this.state = {
      loading: false,
      roleCode: roleCodeConsultor, // 角色
      ...this.defaultState,
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化资源树
   */
  initContent = async () => {
    this.toggleLoading();

    const { roleCode } = this.state;
    const response = await loaders.getCalloutResourceTree({ roleCode }).catch(() => this.toggleLoading());
    const data = filterTreeData(response);
    const treeData = transformArrayToTree(data);
    const defaultExpandKeys = filterDefaultExpandKeys(treeData);

    this.setState({
      originData: data,
      treeData: treeData,
      expandedKeys: defaultExpandKeys,
    });
    this.toggleLoading();
  };

  /**
   * 重置状态
   */
  initState = () => {
    this.setState(this.defaultState);
  };

  /**
   * loading
   */
  toggleLoading = () => {
    this.setState((prevState) => {
      return {
        loading: !prevState.loading,
      };
    });
  };

  /**
   * 渲染树节点
   *
   * @param data 树节点数据
   * @return {<TreeNode>[]}
   */
  loopNode = (data = []) => {
    const { searchValue } = this.state;
    const { checkLeafOnly } = this.props;

    return data.map((item) => {
      const { id, name, children, isParent } = item;
      const title = formatNodeName(name, searchValue);

      if (children && children.length) {
        const disableCheckbox = isParent && checkLeafOnly;

        return (
          <TreeNode key={id} title={title} disableCheckbox={disableCheckbox}>
            {this.loopNode(children)}
          </TreeNode>
        );
      }

      return <TreeNode key={id} title={title} />;
    });
  };

  /**
   * 点击树节点，保存点击选中的树节点
   *
   * @param selectedKeys {array} 选中的树节点ID
   * @param info {object} 当前树节点
   */
  onClick = (selectedKeys, info) => {
    const { selectedNodes } = info;
    const resultNodes = selectedNodes.map((item) => {
      return {
        id: item.key,
        name: item.props.title,
      };
    });

    this.setState({
      selectedKeys,
      selectedNodes: resultNodes,
    });
  };

  /**
   * 展开
   *
   * @param expandedKeys {array} 展开的节点key(id)
   * @param info {object} node节点信息
   */
  onExpand = (expandedKeys, info) => {
    this.setState({
      expandedKeys,
      autoExpandParent: false,
    });
  };

  /**
   * 搜索
   *
   * @param e
   */
  onInputChange = (e) => {
    const { originData, treeData } = this.state;
    const value = e.target.value;
    const expandedKeys = originData
      .map((item) => {
        if (item.name.indexOf(value) > -1) {
          return getParentKey(getNodeKey(item), treeData);
        }

        return null;
      })
      .filter((item, i, self) => {
        return item && self.indexOf(item) === i;
      });

    this.setState({
      searchValue: value,
      expandedKeys,
      autoExpandParent: true,
    });
  };

  /**
   * 切换咨询师树数据
   */
  onRoleCodeChange = (val) => {
    this.setState({ roleCode: val }, () => {
      this.initState();
      this.initContent();
    });
  };

  /**
   * 确认
   */
  onOk = () => {
    const { selectedKeys, selectedNodes, originData } = this.state;
    const originNodes = findNodesFromOriginDataByIds(selectedKeys, originData);

    this.props.onOk(selectedKeys, selectedNodes, originNodes);
  };

  render() {
    const { loading, treeData } = this.state;
    const hasData = treeData && treeData.length > 0;
    const showNoData = !loading && !hasData;

    return (
      <Modal
        title={
          <div className={style.modalTitle}>
            <span className={style.title}>已跟进资源调配</span>
            <Select
              placeholder="请选择角色"
              size="small"
              allowClear={false}
              value={this.state.roleCode}
              dict={roleCodeDict}
              onChange={this.onRoleCodeChange}
              className={style.select}
            />
          </div>
        }
        width={600}
        loading={loading || this.props.loading}
        okBtnShow={true}
        cancelBtnShow={true}
        closable={true}
        onOk={this.onOk}
        onClose={this.props.onClose}
        className={style.modalWrap}
      >
        {hasData && [
          <Search key={1} style={{ marginBottom: 5 }} placeholder="输入名称搜索" onChange={this.onInputChange} />,
          <div key={2} className={style.treeWrap}>
            <Tree
              onSelect={this.onClick}
              onExpand={this.onExpand}
              expandedKeys={this.state.expandedKeys}
              autoExpandParent={this.state.autoExpandParent}
            >
              {this.loopNode(treeData)}
            </Tree>
          </div>,
        ]}

        {showNoData && <NoData />}
      </Modal>
    );
  }
}

export default ModalCalloutPoolTree;
