import React from 'react';
import { NODE_KEY_ID, NODE_KEY_PARENT_ID, BRACKET_LEFT, BRACKET_RIGHT } from './constant';

/**
 * 将新的外呼工作台资源树数据转为旧接口资源树数据结构，方便代码复用
 *
 * @param data {Array} [{orgId, orgName, userList: [{userId, userName, roleCode}]}]
 * @return {Array} [{id, name, parentId, orgType, ...}]
 */
export function transformCalloutResourceToOldResource(data = []) {
  if (!data || data.length === 0) {
    return [];
  }

  let result = [];

  data.forEach((branch) => {
    const { orgId, orgName, userList } = branch;
    const childList = userList || [];
    const childLength = childList.length;
    const parentNode = {
      id: orgId,
      name: orgName,
      parent: true,
      parentId: null,
      orgType: 'BRENCH',
      resourcePoolStatus: 'INVALID',
      resourcePoolType: 'CUSTOMER_RESOURCE_POOL',
      roleCode: null,
      nodeState: 1,
      current: 0,
      total: 0,
      childNodeNums: childLength,
    };

    result.push(parentNode);

    childList.forEach((child, index) => {
      const { userId, userName, roleCode } = child;
      const childNode = {
        id: userId,
        name: userName,
        parent: false,
        parentId: orgId,
        orgType: null,
        resourcePoolStatus: null,
        resourcePoolType: 'PERSONAL_POOL',
        roleCode,
        nodeState: 1,
        current: index,
        total: childLength,
        childNodeNums: 0,
      };

      result.push(childNode);
    });
  });

  return result;
}

/**
 * 客户资源树请求返回的数据格式转换处理
 *
 * @param data {Array} 旧资源树数据 [{id, name, parentId, orgType, ...}]
 */
export function filterTreeData(data = []) {
  if (!Array.isArray(data)) {
    return [];
  }

  let result = [];
  const list = [...data];

  list.forEach((item) => {
    let node = { ...item };
    const { orgType } = node;
    const resourcePoolStatus = node.resourcePoolStatus || node.resourcePoolstatus;

    node.name = node.resourcePoolName || node.name;

    if (resourcePoolStatus === 'INVALID' || orgType === 'DEPARTMENT') {
      node.nocheck = true;
    } else {
      node.name = `${node.name}`;
    }

    const isOpen = orgType === 'GROUNP' || orgType === 'BRENCH' || orgType === 'DEPARTMENT' || orgType === 'CAMPUS';
    if (!node.parent && isOpen) {
      node.open = true;
    }

    if (parent.parent) {
      node.isParent = true;
    }

    if (node.nodeState === 1) {
      result.push(node);
    }
  });

  return result;
}

/**
 * 将有父子关系的平行数组转换成树形数据
 *
 * @param list {array} 平行数组数据
 * @return {Array}
 */
export function transformArrayToTree(list = []) {
  if (!Array.isArray(list) || list.length <= 0) {
    return [];
  }

  const data = [...list];
  const result = [];

  data.forEach((item) => {
    delete item.children;
  });

  const map = {};
  data.forEach((item) => {
    map[getNodeKey(item)] = item;
  });

  data.forEach((item) => {
    const parent = map[getNodeParentKey(item)];

    if (parent) {
      (parent.children || (parent.children = [])).push(item);
    } else {
      result.push(item);
    }
  });

  return result;
}

/**
 * 过滤默认展开的树节点ID，默认只展开一层
 *
 * @param treeData 树节点数据
 * @return {Array}
 */
export function filterDefaultExpandKeys(treeData = []) {
  let expandKeys = [];

  treeData.forEach((item) => {
    if (item.isParent) {
      expandKeys.push(getNodeKey(item));
    }
  });

  return expandKeys;
}

/**
 * 获取树节点key/ID
 *
 * @param node {object} 树节点
 * @param idKey {string} 树节点ID字段
 * @return {*}
 */
export function getNodeKey(node = {}, idKey = NODE_KEY_ID) {
  return node[idKey];
}

/**
 * 获取父节点key/ID
 * @param node {object} 树节点
 * @param pIdKey {string} 父节点ID字段
 * @return {*}
 */
export function getNodeParentKey(node = {}, pIdKey = NODE_KEY_PARENT_ID) {
  return node[pIdKey];
}

/**
 * 格式化节点名称，包含输入框搜索字符
 *
 * @param name 节点名称
 * @param searchValue 搜索搜索字符
 * @return {string|*}
 */
export function formatNodeName(name, searchValue) {
  if (!name) {
    return '';
  }

  if (!searchValue) {
    return name;
  }

  const index = name.indexOf(searchValue);
  if (index > -1) {
    const beforeStr = name.substr(0, index);
    const afterStr = name.substr(index + searchValue.length);

    return (
      <span>
        {beforeStr}
        <span style={{ color: 'red' }}>{searchValue}</span>
        {afterStr}
      </span>
    );
  }

  return name;
}

/**
 * 截取左右括号之前的字符串
 *
 * @param name 完整的节点名称
 * @return {string|*}
 */
export function formatTrueName(name) {
  if (!name) {
    return '';
  }

  const lastBracket = name.lastIndexOf(BRACKET_LEFT);
  const hasBracket = lastBracket > -1;

  if (!hasBracket) {
    return name;
  }

  return name.substr(0, lastBracket);
}

/**
 * 根据树节点key查找父节点
 *
 * @param key 树节点ID
 * @param tree 树型结构数据
 * @return {*}
 */
export const getParentKey = (key, tree) => {
  let parentKey;

  for (let i = 0; i < tree.length; i++) {
    const node = tree[i];
    if (node.children) {
      if (node.children.some((item) => getNodeKey(item) === key)) {
        parentKey = getNodeKey(node);
      } else if (getParentKey(key, node.children)) {
        parentKey = getParentKey(key, node.children);
      }
    }
  }

  return parentKey;
};

/**
 * 根据ID从原始一维数组查询树节点
 *
 * @param id {string?} 树节点ID
 * @param data {Array?} 一维树数组
 * @return {(undefined | *)|null}
 */
export function findNodeFormOriginDataById(id, data = []) {
  if (!id || !data || data.length <= 0) {
    return null;
  }

  return data.find((node) => node.id === id);
}

/**
 * 根据ids从原始一维数组查询树节点
 *
 * @param ids {Array?} 树节点IDs
 * @param data {Array?} 一维树数组
 * @return {Array} 树节点列表
 */
export function findNodesFromOriginDataByIds(ids, data = []) {
  if (!ids || ids.length <= 0 || !data || data.length <= 0) {
    return [];
  }

  const uniqueIds = Array.from(new Set(ids));
  let nodeList = [];

  uniqueIds.forEach((id) => {
    const node = data.find((node) => node.id === id);

    if (node) {
      nodeList.push(node);
    }
  });

  return nodeList;
}
