import React from 'react';
import { connect } from 'dva';
import styles from './styles.less';
import moment from 'moment';
import { Loading, DatePicker, List, SearchBar, Input, Select } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadCommonDictByGroup, loadCampusDict } from 'UTIL/dict';
import { formatDeliverTarget, splitDateString, trimParams } from '../utils';
import ModalCalloutPoolTree from './ModalCalloutPoolTree';
import * as dicts from 'PAGE/SSCRM/common/util/dicts';
import * as loaders from '../loaders';

const { COL_WIDTH_S, COL_WIDTH_M } = List;

/**
 * 签单客户tab
 */
class CusSignedList extends React.Component {
  constructor(props) {
    super(props);

    const { campusId } = this.props;

    this.initSearchParam = {
      studentName: null,
      contact: null,
      gradeId: null,
      resEntranceId: null,
      isAppointCome: null,
      deliverTargetId: null,
      intentionOfTheCustomerId: null,
      intentionCampusId: null,
      customerName: null,
      signTimeStart: null,
      signTimeEnd: null,
      signCampusId: campusId,
    };

    this.state = {
      loading: false,
      searchExpand: false, // 搜索栏展开状态

      studentName: null, // 查询条件 学生姓名
      contact: null, // 查询条件 联系电话
      gradeId: null, // 查询条件 年级
      resEntranceId: null, // 查询条件 资源来源
      isAppointCome: null, // 查询条件 是否诺到
      signCampusId: null, // 查询条件 签单学习中心
      deliverTargetId: null, // 查询条件 跟进人
      dealStatus: null, // 查询条件 跟进状态
      intentionOfTheCustomerId: null, // 查询条件 意向度
      intentionCampusId: null, // 查询条件 意向学习中心
      customerName: null, // 查询条件 客户姓名
      signTimeStart: null, // 查询条件 签单时间（开始）
      signTimeEnd: null, // 查询条件 签单时间（结束）

      gradeDict: [], // 字典 年级
      entranceDict: [], // 字典 资源入口
      blCampusDict: [], // 字典 学校
      intentionDict: [], // 字典 意向度
      deliverTargetDict: [], // 字典 跟进人

      list: [], // 列表数据
      totalRecords: 0, // 总记录数
      currentPage: 1, // 当前页码
      pageSize: 10, // 每页记录数
      pageSelectedKeys: null, // 选中key
      pageSelectedRows: null, // 选中row

      transformModalVisible: false, // 是否显示调转弹窗
      transformLoading: false, // 调转提交loading
    };
  }

  /**
   * 初始化
   *
   * @return {Promise<void>}
   */
  initContent = async () => {
    await this.initialBranch();

    const category = 'STUDENT_GRADE,RES_ENTRANCE,CUSTOMER_SATISFICING';
    const [listData, groupDict, deliverTargetDict] = await Promise.all([
      this.loadCusSignedList(this.formatSearchParams()),
      loadCommonDictByGroup(category),
      loaders.loadDeliverTargetDict(),
    ]);
    const { list, totalRecords } = listData;

    this.setState({
      gradeDict: groupDict['STUDENT_GRADE'],
      entranceDict: groupDict['RES_ENTRANCE'],
      intentionDict: groupDict['CUSTOMER_SATISFICING'],
      deliverTargetDict,
      list,
      totalRecords,
    });
  };

  /**
   * 初始化学习中心字典
   *
   * @return {Promise<void>}
   */
  async initialBranch() {
    const { branchId, campusId } = this.props;
    const blCampusDict = await loadCampusDict(branchId);

    this.initSearchParam['signCampusId'] = campusId;
    this.setState({
      signCampusId: campusId,
      blCampusDict,
    });
  }

  /**
   * 搜索
   */
  onSearch = () => {
    const param = this.formatSearchParams();
    const trimmedParam = trimParams(param);
    const params = { ...trimmedParam, currentPage: 1 };

    this.setState({ loading: true });
    this.loadCusSignedList(params).then(({ list, totalRecords }) => {
      this.setState({ loading: false, currentPage: 1, list, totalRecords });
    });
  };

  reset = () => {
    this.setState(this.initSearchParam);
  };

  /**
   * 翻页
   *
   * @param current {number} 页码
   * @param pageSize {number} 每页条数
   */
  onPaginationChange = ({ current, pageSize }) => {
    this.setState({ loading: true });
    const changed = {
      currentPage: this.state.pageSize !== pageSize ? 1 : current,
      pageSize,
    };
    const params = { ...this.formatSearchParams(), ...changed };

    this.loadCusSignedList(params).then(({ list, totalRecords }) => {
      this.setState({ loading: false, ...changed, list, totalRecords });
    });
  };

  /**
   * 获取完成跟进列表
   * tips: dealStatus 为已预约状态
   *
   * @param params
   * @return {Promise<{totalRecords: *, page: *, list: *}>}
   */
  loadCusSignedList = (params) => {
    this.onRowSelectChange(null, null);

    return loaders.loadCusList(params);
  };

  /**
   * 格式化搜索参数
   */
  formatSearchParams = () => {
    const {
      studentName,
      contact,
      gradeId,
      resEntranceId,
      isAppointCome,
      deliverTargetId,
      intentionOfTheCustomerId,
      intentionCampusId,
      customerName,
      signTimeStart,
      signTimeEnd,
      currentPage,
      pageSize,
    } = this.state;

    return {
      studentName,
      contact,
      gradeId,
      resEntranceId,
      isAppointCome,
      deliverTargetId,
      dealStatus: dicts.CALLOUT_DEAL_STATUS_FOLLOW_COMPLETED,
      intentionOfTheCustomerId,
      intentionCampusId,
      customerName,
      createDateStart: signTimeStart,
      createDateEnd: signTimeEnd,
      currentPage,
      pageSize,
    };
  };

  getDisabledSignTimeStartDate = (startDate) => {
    const { signTimeEnd: endDate } = this.state;

    if (!startDate || !endDate) {
      return false;
    }

    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledSignTimeEndDate = (endDate) => {
    const { signTimeStart: startDate } = this.state;

    if (!endDate || !startDate) {
      return false;
    }

    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  /**
   * 选择列表
   *
   * @param keys 选中key（非ID）
   * @param rows 选中列表
   */
  onRowSelectChange = (keys, rows) => {
    this.setState({
      pageSelectedKeys: keys,
      pageSelectedRows: rows,
    });
  };

  /**
   * 已签资源调配
   */
  onShowTransformModal = () => {
    const { pageSelectedRows } = this.state;

    if (!pageSelectedRows || pageSelectedRows.length <= 0) {
      showMessage('请选择客户资源分配对象', 'error');
      return;
    }

    this.toggleTransformModal();
  };

  /**
   * 显示弹窗
   */
  toggleTransformModal = () => {
    this.setState((prevState) => {
      return {
        transformModalVisible: !prevState.transformModalVisible,
      };
    });
  };

  /**
   * 确认调转
   * tips: 不允许自己调转为自己，即目标跟进人与客户的跟进人不允许是同一人
   *
   * @param checkedKeys {Array} 选中的树ID
   * @param checkedNodes {Array} 选中的树节点
   * @param originNodes {Array} 选中的原始树节点
   */
  onPoolTreeOK = async (checkedKeys, checkedNodes, originNodes) => {
    if (!checkedNodes || checkedNodes.length <= 0) {
      showMessage('请选择资源分配节点', 'error');
      return;
    }

    const { pageSelectedRows } = this.state;
    const newDeliverTarget = checkedKeys[0];
    const newDeliverType = originNodes[0].resourcePoolType;
    const isSameDeliverTarget = pageSelectedRows.some((record) => formatDeliverTarget(record) === newDeliverTarget);
    const sourceList = pageSelectedRows.map((record) => {
      return {
        cusId: record.cusId,
        studentId: record.studentId,
        deliverTarget: formatDeliverTarget(record),
        deliverType: record.deliverType,
      };
    });

    if (isSameDeliverTarget) {
      showMessage('新跟进人不能与原跟进人相同！', 'error');
      return;
    }

    const params = { deliverTarget: newDeliverTarget, deliverType: newDeliverType, sourceList };
    this.doTransform(params);
  };
  doTransform = async (params) => {
    this.setState({ transformLoading: true });
    const response = await loaders.doAllocateCustomerResource(params).catch((e) => {
      this.setState({ transformLoading: false });
      showMessage(e.text || '操作失败: 请重试！', 'error');
    });

    this.setState({ transformLoading: false });
    if (response) {
      if (response.resultCode === 0) {
        showMessage('操作成功!', 'success');
        this.onSearch();
        this.toggleTransformModal();
      } else {
        showMessage(response.resultMessage, 'error');
      }
    }
  };

  render() {
    const { loading, searchExpand, gradeDict, blCampusDict, list, totalRecords, currentPage, pageSize } = this.state;

    const columns = [
      { title: '客户姓名', dataIndex: 'cusName', key: 'cusName', width: COL_WIDTH_M },
      { title: '联系方式', dataIndex: 'contact', key: 'contact', width: 120 },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_M },
      { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: COL_WIDTH_S },
      { title: '学校', dataIndex: 'schoolName', key: 'schoolName', width: COL_WIDTH_M },
      { title: '资源所属学校', dataIndex: 'blCampusName', key: 'blCampusName', width: COL_WIDTH_M },
      { title: '意向学习中心', dataIndex: 'intentionCampusName', key: 'intentionCampusName', width: COL_WIDTH_M },
      { title: '资源来源', dataIndex: 'resEntranceName', key: 'resEntranceName', width: 80 },
      { title: '最新跟进记录', dataIndex: 'lastRemark', key: 'lastRemark', width: 180, lines: 2 },
      {
        title: '最后跟进时间',
        dataIndex: 'lastFollowDate',
        key: 'lastFollowDate',
        width: 120,
        render: (text) => {
          return splitDateString(text);
        },
      },
      { title: '外呼跟进状态', dataIndex: 'dealStatusName', key: 'dealStatusName', width: COL_WIDTH_M },
      { title: '是否诺到', dataIndex: 'isAppointCome', key: 'isAppointCome', width: 80 },
      { title: '诺到日期', dataIndex: 'appointmentDate', key: 'appointmentDate', width: 160 },
      { title: '意向度', dataIndex: 'intentionOfTheCustomerName', key: 'intentionOfTheCustomerName', width: 80 },
      { title: '跟进人', dataIndex: 'deliverTargetName', key: 'deliverTargetName', width: COL_WIDTH_M },
      { title: '创建时间', dataIndex: 'createDate', key: 'createDate', width: 160 },
    ];

    return (
      <div className={styles.wrap}>
        <SearchBar
          expand={searchExpand}
          onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
          onSearch={this.onSearch}
          onReset={this.reset}
        >
          <Input
            placeholder="学生姓名"
            value={this.state.studentName}
            onChange={(e) => this.setState({ studentName: e.target.value })}
          />
          <Input
            placeholder="联系方式"
            value={this.state.contact}
            onChange={(e) => this.setState({ contact: e.target.value })}
          />
          <Select
            placeholder="年级"
            dict={gradeDict}
            value={this.state.gradeId}
            onChange={(val) => this.setState({ gradeId: val })}
          />
          <Select
            placeholder="资源来源"
            dict={dicts.ss_CALLOUT_ENTRANCE_DICT}
            value={this.state.resEntranceId}
            onChange={(val) => this.setState({ resEntranceId: val })}
          />
          <Select
            placeholder="是否诺到"
            dict={dicts.SS_IS_APPOINT_COME_DICT}
            value={this.state.isAppointCome}
            onChange={(val) => this.setState({ isAppointCome: val })}
          />
          <Select
            placeholder="跟进人"
            dict={this.state.deliverTargetDict}
            value={this.state.deliverTargetId}
            onChange={(val) => this.setState({ deliverTargetId: val })}
          />
          <Select
            placeholder="意向度"
            dict={this.state.intentionDict}
            value={this.state.intentionOfTheCustomerId}
            onChange={(val) => this.setState({ intentionOfTheCustomerId: val })}
          />
          <Select
            placeholder="意向学习中心"
            showSearch={true}
            dict={blCampusDict}
            value={this.state.intentionCampusId}
            onChange={(val) => {
              this.setState({ intentionCampusId: val });
            }}
          />
          <Input
            placeholder="客户姓名"
            value={this.state.customerName}
            onChange={(e) => this.setState({ customerName: e.target.value })}
          />
          <DatePicker
            placeholder="创建日期（起）"
            disabledDate={this.getDisabledSignTimeStartDate}
            value={this.state.signTimeStart}
            onChange={(val) => this.setState({ signTimeStart: val })}
          />
          <DatePicker
            placeholder="创建日期（止）"
            disabledDate={this.getDisabledSignTimeEndDate}
            value={this.state.signTimeEnd}
            onChange={(val) => this.setState({ signTimeEnd: val })}
          />
        </SearchBar>

        <List
          bordered={true}
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
          className={styles.customList}
          rowSelection={{
            selectedRowKeys: this.state.pageSelectedKeys,
            onChange: (keys, rows) => {
              this.onRowSelectChange(keys, rows);
            },
          }}
        />

        {this.state.transformModalVisible && (
          <ModalCalloutPoolTree
            loading={this.state.transformLoading}
            onOk={this.onPoolTreeOK}
            onClose={this.toggleTransformModal}
          />
        )}

        <Loading loading={loading} />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps, null, null, { withRef: true })(CusSignedList);
