/**
 * 提取列表中的跟进人ID（不同接口字段不同）
 *
 * @param rowData 列表数据
 * @return {null|*}
 */
export function formatDeliverTarget(rowData) {
  if (!rowData) {
    return null;
  }

  return rowData.deliverTargetId || rowData.deliverTarget;
}

/**
 * 判断当前用户是否为跟进人
 *
 * @param record {object} 列表行数据
 * @param userId {string} 当前用户ID
 * @return {boolean}
 */
export function calcIsSameDeliverTarget(record, userId) {
  const deliverTarget = formatDeliverTarget(record);

  return deliverTarget === userId;
}

/**
 * 格式化搜索条件时间(接远艺接口-流失客户列表)
 *
 * @param timeType 时间类型 1: 最近跟进，2：下次跟进，3：诺到，4：上门，5：创建，6：流失
 * @param date string[] 时间段
 * @return {*}
 */
export function formatDuringTime(timeType = null, date = null) {
  let result = {
    recentlyTimeStart: null, // 最近跟进时间（起）
    recentlyTimeEnd: null, // 最近跟进时间（止）
    nextFollowTimeStart: null, // 下次跟进时间（起）
    nextFollowTimeEnd: null, // 下次跟进时间（止）
    meetingTimeStart: null, // 诺到(原预约上门)时间（起）
    meetingTimeEnd: null, // 诺到(原预约上门)时间（止）
    visitComeDateStart: null, // 上门时间（起）
    visitComeDateEnd: null, // 上门时间（止）
    createDateStart: null, // 创建时间（起）
    createDateEnd: null, // 创建时间（止）
    lostCustomerTimeStart: null, // 流失时间（起）
    lostCustomerTimeEnd: null, // 流失时间（止）
  };

  if (!timeType || !date || date.length <= 1) {
    return result;
  }

  const startTime = date[0];
  const endTime = date[1];
  let timeObj = {};

  switch (timeType) {
    case '1':
      timeObj = { recentlyTimeStart: startTime, recentlyTimeEnd: endTime };
      break;
    case '2':
      timeObj = { nextFollowTimeStart: startTime, nextFollowTimeEnd: endTime };
      break;
    case '3':
      timeObj = { meetingTimeStart: startTime, meetingTimeEnd: endTime };
      break;
    case '4':
      timeObj = { visitComeDateStart: startTime, visitComeDateEnd: endTime };
      break;
    case '5':
      timeObj = { createDateStart: startTime, createDateEnd: endTime };
      break;
    case '6':
      timeObj = { lostCustomerTimeStart: startTime, lostCustomerTimeEnd: endTime };
      break;
    default:
      break;
  }

  result = { ...result, ...timeObj };

  return result;
}

/**
 * 分离时间字符串的天数
 *
 * @param time {string} 时间
 * @param splitStr {string} 分割符
 * @return {string}
 */
export function splitDateString(time = '', splitStr = ' ') {
  if (!time) {
    return '';
  }

  const timeArr = time.split(splitStr);

  if (timeArr.length > 0) {
    return timeArr[0];
  }

  return '';
}

/**
 * 格式化跟进人字典
 *
 * @param data {array} [{orgId, orgName, userList: [{userId, userName, roleCode}]}]
 * @return {Array}
 */
export function formatDeliverTargetDict(data) {
  if (!data || data.length <= 0) {
    return [];
  }

  let dict = [];

  data.forEach((item) => {
    const { orgId, orgName, userList } = item;
    const options = (userList || []).map((user) => {
      return { text: user.userName, val: user.userId };
    });

    dict.push({
      text: orgName,
      val: orgId,
      label: orgName,
      options,
    });
  });

  return dict;
}

/**
 * 去除参数前后空格
 *
 * @param params {object}
 */
export function trimParams(params = {}) {
  if (!params || Object.prototype.toString.call(params) !== '[object Object]') {
    return params;
  }

  let result = {};

  Object.keys(params).forEach((key) => {
    const val = params[key];

    if (Object.prototype.toString.call(val) === '[object String]') {
      result[key] = val.trim();
    } else {
      result[key] = val;
    }
  });

  return result;
}
