import React from 'react';
import { connect } from 'dva';
import styles from './styles.less';
import { Row, Col, DatePicker } from 'antd';
import moment from 'moment';
import { Input, Select, FormItem, TextArea, Checkbox, Block, Button, TextLink } from 'COMPONENT';
import NewStudent from './NewStudent';
import { addCusFormCfg, addCusModalRowStyle, STUDENT_INFO, formatStudentList } from './utils';
import * as loaders from './loaders';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { checkPhone } from 'COMPONENT/Validator/checkers';
import { noop, linkFormCfg, WORK_BENCH } from './utils';
import { loadCommonDictByGroup, loadCampusDict } from 'UTIL/dict';
import CusInfoLayout from './CusInfoLayout';
import { SS_COMMON_ENTRANCE_DICT, SS_RECEPTION_ENTRANCE_DICT, CERTIFICATE_TYPE_ID_CARD } from '../util/dicts';
import { isIdCard } from 'UTIL/common';

// 身份证
const certificateTypeIdCard = CERTIFICATE_TYPE_ID_CARD;

/**
 * 录入客户
 *
 * @props.pageFromType 页面入口 Reception: 前台工作-资源录入, Consultant: 咨询师工作台-跟进客户, Resource: 资源管理-客户录入
 */
class AddCustomerMain extends React.Component {
  static defaultProps = {
    onLoadingStart: noop,
    onLoadingEnd: noop,
    onSaveSuccess: noop,
    onError: noop,
    pageFromType: '',
  };

  constructor(props) {
    super(props);

    const initCampus = props.userCampusId || null;

    this.initStates = {
      cusName: null, // 客户姓名
      customerId: null, // 客户ID
      phone: null, // 客户电话
      entrance: null, // 资源入口
      blCampusId: initCampus, // 资源所属学习中心
      intentionCampusId: initCampus, // 意向学习中心
      meetingTime: null, // 诺到时间
      remark: '', // 备注
      cusIdCardType: null, // 证件类型
      cusIdCardNo: '', // 客户身份证

      linkStudents: [], // 关联学生 {id, name, grade, schoolName, follower,gradeID, schoolId }
      checkedLinkStudents: [], // 选中的关联学生ID {canFollow, follower, gradeId, schoolId, gradeName, id, name, schoolName}
      newStudents: [{ id: '1' }], // 添加的学生 [{cusName, grade, cls, school, schoolName, sex, birthday, contract, address}]

      checking: false, // 提交校验开启状态
    };

    this.state = {
      ...this.initStates,
      userProvince: null, // 当前登录用户所属省份
      userCity: null, // 当前登录用户所属城市
      gradeDict: [], // 字典，年级
      entranceDict: [], // 字典，资源入口
      certificateTypeDict: [], // 字典，证件类型
      userCampusDict: [], // 字典，意向学习中心(用户所属)
    };

    this.errors = {};
    this.newStudentIdx = 2;
  }

  componentDidMount() {
    const { onLoadingStart, onLoadingEnd, userBranchId, isCampus, userCampusOption } = this.props;

    onLoadingStart();
    Promise.all([
      loadCommonDictByGroup('STUDENT_GRADE,RES_ENTRANCE,CERTIFICATE_TYPE'),
      loaders.loadLoginUserProvinceCity(),
      loadCampusDict(userBranchId),
    ])
      .then(([groupDict, provinceCity, campusDict]) => {
        onLoadingEnd();

        const { province, city } = provinceCity;
        const userCampusDict = isCampus ? userCampusOption : campusDict;

        this.setState({
          gradeDict: groupDict['STUDENT_GRADE'],
          entranceDict: groupDict['RES_ENTRANCE'],
          certificateTypeDict: groupDict['CERTIFICATE_TYPE'],
          userProvince: province,
          userCity: city,
          userCampusDict: userCampusDict || [],
        });
      })
      .catch(() => {
        onLoadingEnd();
      });
  }

  reset = () => {
    this.setState(this.initStates);
    this.newStudentIdx = 2;
  };

  /**
   * 保存
   */
  onOk = () => {
    const { onSaveSuccess, onError, onLoadingStart, onLoadingEnd, pageFromType } = this.props;
    const {
      customerId,
      cusName,
      phone,
      entrance,
      blCampusId,
      intentionCampusId,
      meetingTime,
      remark,
      cusIdCardType,
      cusIdCardNo,
      linkStudents,
      checkedLinkStudents,
      newStudents,
      checking,
    } = this.state;

    if (!checking) {
      this.setState({ checking: true });
    }

    const errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      onError(errorTips);
      return;
    }

    const newStudentList = newStudents.filter((s) => s.name && s.name.trim());
    if (newStudentList.length === 0 && checkedLinkStudents.length === 0) {
      onError('请至少选择/填写一个学生姓名');
      return;
    }

    const stuNameList = [...newStudentList, ...linkStudents.map((s) => s.name.trim())];
    const hasRepeatName = Array.from(new Set(stuNameList)).length < stuNameList.length;
    if (hasRepeatName) {
      onError('学生姓名不能重名');
      return;
    }

    const finalNewStudent = formatStudentList(newStudentList, true);
    const finalOldStudent = formatStudentList(checkedLinkStudents, false);
    const studentImportVos = [...finalNewStudent, ...finalOldStudent];

    // 证件类型、证件号码不同时为空时
    const shouldReturn = (cusIdCardType && !cusIdCardNo) || (!cusIdCardType && cusIdCardNo);
    if (shouldReturn) {
      const msg = !cusIdCardType ? '请选择证件类型！' : '请填写证件号码！';
      return onError(msg);
    }

    // 再校验身份证号码
    if (cusIdCardType) {
      if (cusIdCardType === certificateTypeIdCard && !isIdCard(cusIdCardNo)) {
        return onError('客户身份证格式错误');
      } else if (!cusIdCardNo.trim()) {
        return onError('客户证件号码错误');
      }
    }

    if (finalNewStudent.length) {
      let tempIndex = 0;
      for (let value of finalNewStudent) {
        ++tempIndex;
        if (value.idCardNo && !isIdCard(value.idCardNo)) {
          break;
        }
      }
      let tempStatus = finalNewStudent.every((value) => !value.idCardNo || isIdCard(value.idCardNo));
      if (!tempStatus) {
        return showMessage(`保存失败：学员${tempIndex}身份证格式错误`);
      }
    }

    const param = {
      workBench: WORK_BENCH[pageFromType],
      cusName,
      cusId: customerId,
      contact: phone,
      resEntranceId: entrance,
      blCampusId,
      intentionCampusId,
      meetingTime: meetingTime ? meetingTime.format('YYYY-MM-DD HH:mm:ss') : '',
      cusIdCardType,
      cusIdCardNo,
      remark,
      studentImportVos,
    };

    onLoadingStart();
    loaders.saveCustomer(param).then(
      () => {
        showMessage('保存成功', 'success');
        onLoadingEnd();
        onSaveSuccess();
      },
      (e) => {
        onLoadingEnd();

        if (e.text) {
          onError(e.text);
        } else {
          showMessage('保存失败');
        }
      },
    );
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  getLinkStudentsToolBar = () => {
    const { linkStudents, checkedLinkStudents } = this.state;
    const canFollowStus = linkStudents.filter((s) => s.canFollow);
    const checked = canFollowStus.length > 0 && canFollowStus.length === checkedLinkStudents.length;

    return (
      <Checkbox checked={checked} onChange={this.checkAllLinkStudentChange} disabled={canFollowStus.length === 0}>
        全部跟进
      </Checkbox>
    );
  };

  checkAllLinkStudentChange = (e) => {
    const { linkStudents } = this.state;
    const checked = e.target.checked;
    let checkedLinkStudents;

    if (checked) {
      checkedLinkStudents = linkStudents.filter((s) => s.canFollow);
    } else {
      checkedLinkStudents = [];
    }

    this.setState({ checkedLinkStudents });
  };

  checkLinkStudentChange = (e, stu) => {
    const { checkedLinkStudents } = this.state;
    const checked = e.target.checked;
    let newCheckedLinkStudents;
    if (checked) {
      if (checkedLinkStudents.indexOf(stu) < 0) {
        newCheckedLinkStudents = [...checkedLinkStudents, stu];
      }
    } else {
      newCheckedLinkStudents = checkedLinkStudents.filter((c) => c !== stu);
    }

    this.setState({ checkedLinkStudents: newCheckedLinkStudents });
  };

  /**
   * 学生信息表单修改
   * @param id 学生ID（保存提交时需转成stuId）
   * @param data 学生信息
   */
  newStudentChange = (id, data) => {
    const studentInfo = STUDENT_INFO;
    const studentList = this.state.newStudents.map((stu) => {
      if (stu.id === id) {
        return { ...studentInfo, ...data };
      }

      return stu;
    });

    this.setState({
      newStudents: studentList,
    });
  };

  /**
   * 删除学生
   * @param id 学生ID
   */
  deleteNewStudent = (id) => {
    const { newStudents } = this.state;

    this.setState({
      newStudents: newStudents.filter((stu) => stu.id !== id),
    });

    let newErrors = {};
    Object.keys(this.errors).forEach((key) => {
      if (key.indexOf(`newStu_${id}`) < 0) {
        newErrors[key] = this.errors[key];
      }
    });

    this.errors = newErrors;
  };

  /**
   * 添加学生
   */
  addNewStudent = () => {
    this.setState({
      newStudents: [...this.state.newStudents, { id: '' + this.newStudentIdx++ }],
    });
  };

  /**
   * 判断是否可以添加学生
   */
  checkAddNewStudent = () => {
    if (this.state.newStudents.length === 0) {
      this.addNewStudent();
    }
  };

  /**
   * 联系方式输入
   */
  onPhoneChange = (e) => {
    const { onLoadingStart, onLoadingEnd } = this.props;
    const val = e.target.value ? e.target.value.trim() : null;
    const phoneValidate = checkPhone(val) === null;

    this.setState({ phone: val });

    if (phoneValidate) {
      onLoadingStart();
      loaders.loadLinkStudents({ phone: val }).then(
        ({ list, customerId, parentInfo }) => {
          onLoadingEnd();
          this.setState({ linkStudents: list || [], checkedLinkStudents: [], customerId: customerId || null });
          this.setState({
            cusName: parentInfo ? parentInfo.customerName : this.state.cusName,
            entrance: parentInfo ? parentInfo.resEntranceId : this.state.entrance,
            intentionCampusId: parentInfo ? parentInfo.intentionCampusId : this.state.intentionCampusId,
            blCampusId: parentInfo ? parentInfo.blCampusId : this.state.blCampusId,
            cusIdCardType: parentInfo ? parentInfo.customerIdCardType : this.state.cusIdCardType,
            cusIdCardNo: parentInfo ? parentInfo.customerIdCardNo : this.state.cusIdCardNo,
          });

          if (!list || list.length === 0) {
            this.checkAddNewStudent();
          }
        },
        () => {
          onLoadingEnd();
          this.setState({ linkStudents: [], checkedLinkStudents: [], customerId: null });
          this.checkAddNewStudent();
        },
      );
    } else {
      this.setState({ linkStudents: [], checkedLinkStudents: [], customerId: null });
      this.checkAddNewStudent();
    }
  };

  render() {
    const { pageFromType } = this.props;
    const {
      cusName,
      phone,
      entrance,
      blCampusId,
      intentionCampusId,
      meetingTime,
      remark,
      checking,
      cusIdCardNo,
      linkStudents,
      checkedLinkStudents,
      newStudents,
      gradeDict,
      userProvince,
      userCity,
      userCampusDict,
    } = this.state;

    const entranceDict = pageFromType === 'Reception' ? SS_RECEPTION_ENTRANCE_DICT : SS_COMMON_ENTRANCE_DICT;

    return (
      <div className={styles.wrap}>
        <Block title="客户信息" style={{ marginTop: 20 }}>
          <CusInfoLayout>
            <FormItem {...addCusFormCfg} name="客户姓名" required>
              <Input
                value={cusName}
                onChange={(e) => this.setState({ cusName: e.target.value ? e.target.value.trim() : null })}
                validator={{
                  silent: !checking,
                  onErrorChange: (e) => this.errorChange('cusName', e),
                }}
              />
            </FormItem>
            <FormItem {...addCusFormCfg} name="联系方式" required>
              <Input
                value={phone}
                onChange={this.onPhoneChange}
                validator={{
                  rules: ['required', 'phone'],
                  silent: !checking,
                  onErrorChange: (e) => this.errorChange('phone', e),
                }}
              />
            </FormItem>
            <FormItem {...addCusFormCfg} name="资源来源" required>
              <Select
                dict={entranceDict}
                value={entrance}
                onChange={(val) => {
                  this.setState({ entrance: val });
                }}
                validator={{
                  silent: !checking,
                  onErrorChange: (e) => this.errorChange('entrance', e),
                }}
              />
            </FormItem>
            <FormItem {...addCusFormCfg} name="诺到时间">
              <DatePicker
                style={{ width: '100%' }}
                size="large"
                showTime={{ defaultValue: moment('00:00:00', 'HH:mm:ss') }}
                format="YYYY-MM-DD HH:mm:ss"
                value={meetingTime}
                onChange={(val) => this.setState({ meetingTime: val })}
              />
            </FormItem>
            <FormItem {...addCusFormCfg} name="资源所属学习中心" required>
              <Select
                dict={userCampusDict}
                value={blCampusId}
                onChange={(val) => this.setState({ blCampusId: val })}
                validator={{
                  silent: !checking,
                  onErrorChange: (e) => this.errorChange('blCampusId', e),
                }}
              />
            </FormItem>
            <FormItem {...addCusFormCfg} name="意向学习中心" required>
              <Select
                dict={userCampusDict}
                value={intentionCampusId}
                onChange={(val) => this.setState({ intentionCampusId: val })}
                validator={{
                  silent: !checking,
                  onErrorChange: (e) => this.errorChange('intentionCampusId', e),
                }}
              />
            </FormItem>
            <FormItem {...addCusFormCfg} name="证件类型">
              <Select
                dict={this.state.certificateTypeDict}
                value={this.state.cusIdCardType}
                onChange={(val) => this.setState({ cusIdCardType: val })}
              />
            </FormItem>
            <FormItem {...addCusFormCfg} name="证件号码">
              <Input
                value={cusIdCardNo}
                onChange={(e) => this.setState({ cusIdCardNo: e.target.value ? e.target.value.trim() : null })}
              />
            </FormItem>
          </CusInfoLayout>
          <Row style={addCusModalRowStyle}>
            <Col span={24}>
              <FormItem {...addCusFormCfg} name="备注">
                <TextArea rows={3} value={remark} onChange={(e) => this.setState({ remark: e.target.value })} />
              </FormItem>
            </Col>
          </Row>
        </Block>

        {linkStudents.length > 0 && (
          <Block title="关联学生" headerToolBar={this.getLinkStudentsToolBar()}>
            <div className={styles.tipInfo}>
              <span>同一学生，请勾选关联并保存即可</span>
            </div>
            {linkStudents.map((stu) => {
              const checked = checkedLinkStudents.indexOf(stu) >= 0;
              const tag = stu.studentType === 'ENROLLED' && !stu.canFollow && '在读学生';

              return (
                <div key={stu.id} className={styles.linkStuItem}>
                  <Row style={{ marginLeft: 20 }}>
                    <Col span={1} className={styles.xsCheckboxCol}>
                      <Checkbox
                        checked={checked}
                        onChange={(e) => this.checkLinkStudentChange(e, stu)}
                        disabled={!stu.canFollow}
                      />
                    </Col>
                    <Col span={8}>
                      <FormItem {...linkFormCfg} name="学生">
                        <div
                          style={{
                            wordBreak: 'break-word',
                            display: 'flex',
                            justifyContent: 'center',
                            alignItems: 'baseline',
                          }}
                        >
                          <span
                            style={{
                              wordBreak: 'break-word',
                              flex: '1',
                            }}
                          >
                            {stu.name}
                          </span>
                          {tag && (
                            <span
                              style={{
                                marginLeft: '10px',
                                border: '1px #ff8080 solid',
                                padding: '0 4px',
                                borderRadius: '4px',
                                color: '#ff8080',
                                display: 'inline-block',
                                lineHeight: '20px',
                                fontSize: '12px',
                              }}
                            >
                              {tag}
                            </span>
                          )}
                        </div>
                      </FormItem>
                    </Col>
                    <Col span={4}>
                      <FormItem {...linkFormCfg} name="年级">
                        {stu.gradeName}
                      </FormItem>
                    </Col>
                    <Col span={11}>
                      <FormItem {...linkFormCfg} name="学校">
                        {stu.schoolName}
                      </FormItem>
                    </Col>
                  </Row>
                </div>
              );
            })}
          </Block>
        )}

        <Block title="学生信息">
          {newStudents.map((stu, idx) => {
            return (
              <div key={stu.id} className={styles.newStu}>
                <div className={styles.newStuHeader}>
                  <h5 className={styles.newStuTitle}>{`学生${idx + 1}`}</h5>
                  {(linkStudents.length > 0 || newStudents.length > 1) && (
                    <div className={styles.newStuToolBar}>
                      <TextLink onClick={() => this.deleteNewStudent(stu.id)}>删除</TextLink>
                    </div>
                  )}
                </div>
                <NewStudent
                  key={stu.id}
                  data={stu}
                  onChange={(data) => this.newStudentChange(stu.id, data)}
                  onErrorChange={(fieldName, e) => this.errorChange(`newStu_${stu.id}_${fieldName}`, e)}
                  checking={checking}
                  gradeDict={gradeDict}
                  userProvince={userProvince}
                  userCity={userCity}
                />
              </div>
            );
          })}
          <div className={styles.newStudentsFooter}>
            <Button onClick={() => this.addNewStudent()}>添加学员</Button>
          </div>
        </Block>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const user = state.account.user || {};
  const { isBlBrench, isBlCampus, brenchId, campusId, campusName } = user;

  const isBranch = isBlBrench === 'true' || isBlBrench === true; // 是否分公司(双师-分校)
  const isCampus = isBlCampus === 'true' || isBlCampus === true; // 是否学习中心
  const userCampusOption = isCampus ? [{ val: campusId, text: campusName }] : [];

  return {
    userBranchId: brenchId,
    userCampusId: campusId,
    isBranch,
    isCampus,
    userCampusOption,
  };
};

export default connect(mapStateToProps, null, null, { withRef: true })(AddCustomerMain);
