import React from 'react';
import { connect } from 'dva';
import styles from './styles.less';
import { Block, FormItem, Section, TextLink } from 'COMPONENT';
import { getFieldWithFiller } from 'COMPONENT/utils/form';
import ModalEditCustomerContact from 'BLOC/ModalEditCustomerContact';
import * as loaders from '../../util/loaders';
import { Input, Select, TextArea } from 'COMPONENT';
import SelectSchoolModal from 'PAGE/common/SelectSchoolModal';
import { getDictTextByVal, loadCommonDict, loadCampusDict } from 'UTIL/dict';
import { findOrganizationById } from 'SERVICE/common';

const formCfg = {
  labelWidth: 100,
  size: 'small',
};

/**
 * 客户详情
 * @param props.cusId {string} 客户ID
 */
class CusDetailBlock extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      editing: false, // 是否编辑状态，false为编辑true为保存
      checking: false, // 校验开启状态
      studentName: null, // 学生姓名
      gradeId: null, // 年级
      schoolId: null, // 学校
      schoolName: null, // 学校名称
      cusName: null, // 客户姓名
      contact: null, // 客户电话
      resEntranceName: null, // 资源来源
      blCampusName: null, // 资源所属学习中心
      intentionCampusId: null, // 意向学习中心ID
      intentionCampusName: null, // 意向学习中心
      lastDeliverName: null, // 分配人
      deliverTargetName: null, // 跟进人
      dealStatusName: null, // 跟进状态名称
      remark: '', // 备注
      modalEditPhoneShow: false, // 显示修改手机号码弹窗
      cusIdCardNo: '', // 客户身份证
      studentIdCardNo: '', // 学生身份证
      saveTipsVis: false, // 保存按钮提示气泡显示状态
      saveTips: null, // 保存按钮提示信息
      selectSchoolModalVis: false, // 选择学校弹框显示状态
      gradeDict: [], // 字典，年级
      provinceId: null,
      cityId: null,
      initData: null, // 备份数据，用于取消时回退
      initCampusDict: [], // 初始所属学习中心字典，用于取消时回退
      campusDict: [], // 字典，所属学习中心
    };

    this.errors = {};
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = () => {
    const { cusId, studentId, deliverTarget } = this.props;
    const { organizationId, campusId, brenchId, isBlCampus, campusName } = this.props.user || {};

    // 判断是否学习中心
    const isCampus = isBlCampus === 'true' || isBlCampus === true;
    const userCampusOption = isCampus ? [{ val: campusId, text: campusName }] : [];

    Promise.all([
      loaders.loadCusDetail(cusId, studentId, deliverTarget),
      loadCommonDict('STUDENT_GRADE'),
      findOrganizationById({ id: campusId || brenchId }),
      loadCampusDict(brenchId),
    ])
      .then(([cusDetail, gradeDict, organizationInfo, campusDict]) => {
        let provinceId = '';
        let cityId = '';
        if (organizationId === '000001') {
          provinceId = '440000';
          cityId = '441900';
        } else {
          const { city, province } = organizationInfo;
          provinceId = province ? province.id : '';
          cityId = city ? city.id : '';
        }

        const userCampusDict = isCampus ? userCampusOption : campusDict;

        this.setState({
          ...cusDetail,
          initData: cusDetail,
          gradeDict,
          campusDict: userCampusDict,
          initCampusDict: userCampusDict,
          provinceId,
          cityId,
        });
      })
      .catch(() => {});
  };

  /**
   * 修改手机号码
   */
  onEditContact = () => {
    this.toggleModalEditContact();
  };

  /**
   * 显示修改手机号码弹窗
   */
  toggleModalEditContact = () => {
    this.setState((prevState) => {
      return {
        modalEditPhoneShow: !prevState.modalEditPhoneShow,
      };
    });
  };

  /**
   * 追加学习中心下拉选项，解决跨学习中心无权限问题
   *
   * @param campusDict {array} 权限学习中心字典
   * @param campusId {string} 追加的学习中心ID
   * @param campusName {string} 追加的学习中心名称
   */
  appendCampusDict = (campusDict = [], campusId = '', campusName = '') => {
    if (!campusId || !campusName) {
      return campusDict;
    }

    const appendDict = [{ val: campusId, text: campusName }];
    if (!campusDict || campusDict.length <= 0) {
      return appendDict;
    }

    const isExist = campusDict.some((item) => item.val === campusId);
    if (isExist) {
      return campusDict;
    }

    return campusDict.concat(appendDict);
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  showSaveTips = (text) => {
    this.setState({ saveTipsVis: true, saveTips: text || '请正确填写信息' });
  };

  render() {
    const {
      editing,
      checking,
      studentName,
      gradeId,
      schoolName,
      cusName,
      contact,
      resEntranceName,
      blCampusName,
      intentionCampusId,
      intentionCampusName,
      lastDeliverName,
      deliverTargetName,
      dealStatusName,
      remark,
      cusIdCardTypeName,
      cusIdCardNo,
      studentIdCardNo,
      gradeDict,
      campusDict,
      selectSchoolModalVis,
      provinceId,
      cityId,
    } = this.state;
    const intentionCampusDict = this.appendCampusDict(campusDict, intentionCampusId, intentionCampusName);

    return (
      <div className={styles.wrap}>
        <Block>
          <Section title="客户详情">
            <div className={styles.inner}>
              <FormItem {...formCfg} name="学生姓名">
                {editing ? (
                  <Input value={studentName} onChange={(e) => this.setState({ studentName: e.target.value })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(studentName)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="年级">
                {editing ? (
                  <Select dict={gradeDict} value={gradeId} onChange={(val) => this.setState({ gradeId: val })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getDictTextByVal(gradeDict, gradeId, true)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="学校">
                {editing ? (
                  <Input readOnly value={schoolName} onClick={() => this.setState({ selectSchoolModalVis: true })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(schoolName)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="客户姓名" required={editing}>
                {editing ? (
                  <Input
                    value={cusName}
                    onChange={(e) => this.setState({ cusName: e.target.value })}
                    validator={{
                      rules: ['required'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('cusName', e),
                    }}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(cusName)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="客户电话">
                <div style={{ wordBreak: 'break-word' }}>
                  {getFieldWithFiller(contact)}
                  {contact && this.props.btnAuth_editContact && (
                    <TextLink style={{ marginLeft: 10 }} onClick={this.onEditContact}>
                      修改
                    </TextLink>
                  )}
                </div>
              </FormItem>
              <FormItem {...formCfg} name="资源来源">
                <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(resEntranceName)}</div>
              </FormItem>
              <FormItem {...formCfg} name="资源所属学习中心">
                <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(blCampusName)}</div>
              </FormItem>
              <FormItem {...formCfg} name="意向学习中心">
                {editing ? (
                  <Select
                    dict={intentionCampusDict}
                    value={intentionCampusId}
                    onChange={(val) => this.setState({ intentionCampusId: val })}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>
                    {getDictTextByVal(intentionCampusDict, intentionCampusId, true)}
                  </div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="分配人">
                <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(lastDeliverName)}</div>
              </FormItem>
              <FormItem {...formCfg} name="跟进人">
                <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(deliverTargetName)}</div>
              </FormItem>
              <FormItem {...formCfg} name="跟进状态">
                <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(dealStatusName)}</div>
              </FormItem>
              <FormItem {...formCfg} name="备注">
                {editing ? (
                  <TextArea
                    style={{ wordBreak: 'break-word' }}
                    rows={3}
                    value={remark}
                    onChange={(e) => this.setState({ remark: e.target.value })}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(remark)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="家长证件类型">
                <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(cusIdCardTypeName)}</div>
              </FormItem>
              <FormItem {...formCfg} name="家长证件号码">
                {editing ? (
                  <Input value={cusIdCardNo} onChange={(e) => this.setState({ cusIdCardNo: e.target.value })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(cusIdCardNo)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="学生身份证">
                {editing ? (
                  <Input value={studentIdCardNo} onChange={(e) => this.setState({ studentIdCardNo: e.target.value })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(studentIdCardNo)}</div>
                )}
              </FormItem>
            </div>
          </Section>
        </Block>

        {this.state.modalEditPhoneShow && (
          <ModalEditCustomerContact
            customerId={this.state.cusId}
            customerName={cusName}
            customerContact={contact}
            onClose={this.toggleModalEditContact}
          />
        )}
        {selectSchoolModalVis && (
          <SelectSchoolModal
            onConfirm={(school, schoolName) =>
              this.setState({ selectSchoolModalVis: false, schoolId: school, schoolName })
            }
            onClose={() => this.setState({ selectSchoolModalVis: false })}
            defaultProvince={provinceId}
            defaultCity={cityId}
          />
        )}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const btnAuth = state.account.btnAuths || [];
  const btnAuth_editContact = btnAuth.indexOf('SSResourceCustomerEditContact') > -1;

  return {
    btnAuth_editContact,
  };
};

export default connect(mapStateToProps)(CusDetailBlock);
