import React from 'react';
import { connect } from 'dva';
import styles from './styles.less';
import { Loading, DatePicker, List, SearchBar, Input, Select, TextLink } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadCampusDict, loadCommonDictByGroup } from 'UTIL/dict';
import loaders from '../loaders';
import moment from 'moment';
import { Modal as AntdModal } from 'antd';
import { CUS_LIST_LOST } from '../dicts';
import { formatPhoneNumber, formatDeliverTarget } from '../utils';
import ModalCustomerPoolTree from 'SRC/BLoComponents/ModalCustomerPoolTree';
import classNames from 'classnames';

const { COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

/**
 * 签单客户tab
 */
class CusSignedList extends React.Component {
  constructor(props) {
    super(props);

    const { campusId } = this.props;

    this.showEmployeeTip = this.showEmployeeTip.bind(this);

    this.initSearchParam = {
      studentName: null,
      gradeId: null,
      signCampusId: campusId,
      resEntranceId: null,
      deliverTarget: null,
      signTimeStart: null,
      signTimeEnd: null,
    };

    this.state = {
      loading: false,
      searchExpand: false, // 搜索栏展开状态

      studentName: null, // 查询条件 学生姓名
      gradeId: null, // 查询条件 年级
      signCampusId: '', // 查询条件 签单学习中心
      resEntranceId: null, // 查询条件 资源来源
      deliverTarget: null, // 查询条件 跟进人
      signTimeStart: null, // 查询条件 签单时间（开始）
      signTimeEnd: null, // 查询条件 签单时间（结束）

      gradeDict: [], // 字典 年级
      entranceDict: [], // 字典 资源入口
      blCampusDict: [], // 字典 学校

      list: [], // 列表数据 [{id, studentName, gradeName, schoolName}]
      totalRecords: 0, // 总记录数
      currentPage: 1, // 当前页码
      pageSize: 10, // 每页记录数
      pageSelectedKeys: null, // 选中key
      pageSelectedRows: null, // 选中row

      transformModalVisible: false, // 是否显示调转弹窗
      transformLoading: false, // 调转提交loading
    };
  }

  async initialBranch() {
    const { branchId, campusId } = this.props;
    const blCampusDict = await loadCampusDict(branchId);

    this.initSearchParam['signCampusId'] = campusId;
    this.setState({
      signCampusId: campusId,
      blCampusDict,
    });
  }

  initContent = async () => {
    await this.initialBranch();

    const { userOrganizationId } = this.props;
    const category = 'STUDENT_GRADE,RES_ENTRANCE';
    const [listData, groupDict, deliverTargetDict] = await Promise.all([
      this.loadCusSignedList(this.formatSearchParams()),
      loadCommonDictByGroup(category),
      loaders.loadDeliverTargetDict({ roleCode: 'CONSULTOR', organizationId: userOrganizationId }),
    ]);
    const { list, totalRecords } = listData;
    this.setState({
      gradeDict: groupDict['STUDENT_GRADE'],
      entranceDict: groupDict['RES_ENTRANCE'],
      deliverTargetDict,
      list,
      totalRecords,
    });
  };

  search = () => {
    this.setState({ loading: true });
    const params = { ...this.formatSearchParams(), currentPage: 1 };
    this.loadCusSignedList(params)
      .then(({ list, totalRecords, page }) => {
        this.setState({ loading: false, currentPage: 1, list, totalRecords });
      })
      .catch(() => {
        this.setState({
          loading: false,
          currentPage: 1,
          list: [],
          totalRecords: 0,
        });
      });
  };

  reset = () => {
    this.setState(this.initSearchParam);
  };

  onPaginationChange = ({ current, pageSize }) => {
    this.setState({ loading: true });
    const changed = {
      currentPage: this.state.pageSize !== pageSize ? 1 : current,
      pageSize,
    };
    const params = { ...this.formatSearchParams(), ...changed };

    this.loadCusSignedList(params)
      .then(({ list, totalRecords }) => {
        this.setState({ loading: false, ...changed, list, totalRecords });
      })
      .catch(() => {
        this.setState({
          loading: false,
          ...changed,
          list: [],
          totalRecords: 0,
        });
      });
  };

  loadCusSignedList = (params) => {
    this.onRowSelectChange(null, null);

    return loaders.loadCusSignedList(params);
  };

  /**
   * 格式化搜索参数
   */
  formatSearchParams = () => {
    const {
      studentName,
      gradeId,
      resEntranceId,
      deliverTarget,
      signTimeStart,
      signTimeEnd,
      signCampusId,
      currentPage,
      pageSize,
    } = this.state;
    return {
      studentName,
      gradeId,
      resEntranceId,
      deliverTarget,
      signTimeStart,
      signTimeEnd,
      signCampusId,
      currentPage,
      pageSize,
    };
  };

  /**
   * 签单
   * tips: 合同学习中心（签单客户列表中取 签单学习中心`signBlCampusId`）
   * tips: 新生-跳转添加合同页面，合同类型为新签
   * tips: 老生-提示:当前学生已到达4次课消，请联系辅导老师进行签单, 若是学习中心配置了不限制签单开关，则允许签单
   */
  signNew = async (record) => {
    const { cusId, studentId, studentName, gradeId, schoolId, schoolName, signBlCampusId: blCampusId } = record;
    const query = {
      cusId,
      studentId,
      studentName,
      gradeId,
      schoolId,
      schoolName,
      blCampusId,
      contractType: 'NEW_CONTRACT',
    };

    const judgeResponse = await loaders.judgeIsOldStuForSsBoss({ studentId });
    const isOldStudent = judgeResponse && judgeResponse.data;

    const switchFlag = await loaders.getSwitchConfigFlag(blCampusId);
    const { isUnLimitSign } = switchFlag;

    if (isOldStudent && !isUnLimitSign) {
      showMessage('当前学生长期常规课程课消已达到4次，请联系辅导老师进行签单', 'error');
      return;
    }

    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: `/signContract/${JSON.stringify(query)}`,
        refresh: true,
      },
    });

    window.__refreshParentEntrance = (params, refresh) => {
      this.props.dispatch({
        type: 'main/removeTab',
        payload: { id: this.props.activeTabId },
      });

      if (!refresh) {
        this.props.dispatch({
          type: 'main/openPage',
          payload: {
            params: {
              initCusList: CUS_LIST_LOST,
            },
            refresh: true,
            path: '/SSConsultantFollowCustomer',
          },
        });

        return;
      }

      this.search();
      this.signNew({ ...record, ...params });
    };
  };

  /**
   * 跳转跟进详情
   *
   * @param params
   */
  onPageToDetail = (params) => {
    const { dispatch, type } = this.props;
    const path = 'SSCustomerDetailView';

    params = {
      ...params,
      type,
      onRefresh: this.search,
    };

    dispatch({
      type: 'main/openPage',
      payload: {
        path: `/${path}`,
        params,
        refresh: true,
      },
    });
  };

  getDisabledSignTimeStartDate = (startDate) => {
    const { signTimeEnd: endDate } = this.state;

    if (!startDate || !endDate) {
      return false;
    }

    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledSignTimeEndDate = (endDate) => {
    const { signTimeStart: startDate } = this.state;

    if (!endDate || !startDate) {
      return false;
    }

    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  /**
   * 选择列表
   *
   * @param keys 选中key（非ID）
   * @param rows 选中列表
   */
  onRowSelectChange = (keys, rows) => {
    this.setState({
      pageSelectedKeys: keys,
      pageSelectedRows: rows,
    });
  };

  /**
   * 已签资源调配
   */
  onShowTransformModal = () => {
    const { pageSelectedRows } = this.state;

    if (!pageSelectedRows || pageSelectedRows.length <= 0) {
      showMessage('请选择客户资源分配对象', 'error');
      return;
    }

    this.toggleTransformModal();
  };

  /**
   * 显示弹窗
   */
  toggleTransformModal = () => {
    this.setState((prevState) => {
      return {
        transformModalVisible: !prevState.transformModalVisible,
      };
    });
  };

  /**
   * 确认调转
   * tips: 只允许调转到个人资源池
   * tips: 不允许自己调转为自己，即目标跟进人与客户的跟进人不允许是同一人
   *
   * @param checkedKeys
   * @param checkedNodes
   */
  onPoolTreeOK = async (checkedKeys, checkedNodes) => {
    if (!checkedNodes || checkedNodes.length <= 0) {
      showMessage('请选择资源分配节点', 'error');
      return;
    }

    const { pageSelectedRows } = this.state;
    const newDeliverTarget = checkedKeys[0];
    const isSameDeliverTarget = pageSelectedRows.some((record) => {
      return formatDeliverTarget(record) === newDeliverTarget;
    });
    const sourceList = pageSelectedRows.map((record) => {
      return {
        cusId: record.cusId,
        studentId: record.studentId,
        deliverTarget: formatDeliverTarget(record),
        deliverType: record.deliverType,
      };
    });

    if (isSameDeliverTarget) {
      showMessage('新跟进人不能与原跟进人相同！', 'error');
      return;
    }

    const params = { isSignedResources: true, deliverTarget: newDeliverTarget, sourceList };
    this.doTransform(params);
  };
  doTransform = async (params) => {
    this.setState({ transformLoading: true });
    const response = await loaders.doAllocateCustomerResource(params).catch((e) => {
      this.setState({ transformLoading: false });
      showMessage(e.text || '操作失败: 请重试！', 'error');
    });

    this.setState({ transformLoading: false });
    if (response) {
      if (response.resultCode === 0) {
        showMessage('操作成功!', 'success');
        this.search();
        this.toggleTransformModal();
      } else {
        showMessage(response.resultMessage, 'error');
      }
    }
  };

  showEmployeeTip = () => {
    const tip =
      '该学生的客户号码属于员工号码，根据集团要求，内部员工号码不允许签单，请使用非员工号码录入资源然后签单。';
    AntdModal.info({
      title: tip,
      content: '',
      className: 'homAntdModal',
    });
  };

  render() {
    const { userId } = this.props;
    const {
      loading,
      searchExpand,
      studentName,
      gradeId,
      signCampusId,
      resEntranceId,
      signTimeStart,
      signTimeEnd,
      gradeDict,
      entranceDict,
      blCampusDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
    } = this.state;

    const columns = [
      { title: '客户姓名', dataIndex: 'cusName', key: 'cusName' },
      { title: '联系方式', dataIndex: 'contact', key: 'contact', width: COL_WIDTH_L },
      {
        title: '学生姓名',
        dataIndex: 'studentName',
        key: 'studentName',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          const { employee } = record;
          return (
            <div>
              {formatPhoneNumber(text)}
              {employee && (
                <div>
                  <span className={'employeeFlag'}>内部员工</span>
                </div>
              )}
            </div>
          );
        },
      },
      { title: '年级', dataIndex: 'gradeName', key: 'gradeName' },
      { title: '学校', dataIndex: 'schoolName', key: 'schoolName', width: COL_WIDTH_L },
      { title: '资源所属学校', dataIndex: 'blCampusName', key: 'blCampusName', width: COL_WIDTH_M },
      { title: '签单学习中心', dataIndex: 'signBlCampusName', key: 'signBlCampusName', width: COL_WIDTH_M },
      { title: '资源来源', dataIndex: 'resEntranceName', key: 'resEntranceName' },
      { title: '签单人', dataIndex: 'signStaffName', key: 'signStaffName' },
      { title: '跟进人', dataIndex: 'deliverTargetName', key: 'deliverTargetName' },
      { title: '跟进状态', dataIndex: 'dealStatusName', key: 'dealStatusName', width: 80 },
      { title: '最新签单时间', dataIndex: 'signTime', key: 'signTime', width: COL_WIDTH_XL },
      {
        title: '操作',
        key: 'action',
        fixed: 'right',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          const { cusId, studentId, employee } = record;
          const deliverTarget = formatDeliverTarget(record);
          const params = {
            cusId,
            studentId,
            detailEditable: true,
            followEditable: true,
            deliverTarget,
            followList: [],
          };
          const isSameUser = deliverTarget === userId;

          return (
            <span>
              <TextLink
                className={classNames({
                  [styles.textLink]: true,
                  [styles.disabled]: !isSameUser,
                })}
                style={{ marginRight: '16px' }}
                onClick={() => {
                  // 20221115版本去除内部员工无法签单限制
                  // 是否是内部员工
                  /*if (employee) {
                    this.showEmployeeTip()
                    return
                  }*/
                  if (!isSameUser) return;

                  this.signNew(record);
                }}
              >
                签单
              </TextLink>
              <TextLink onClick={() => this.onPageToDetail(params)}>跟进详情</TextLink>
            </span>
          );
        },
      },
    ];

    return (
      <div className={styles.wrap}>
        <SearchBar
          expand={searchExpand}
          onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
          onSearch={this.search}
          onReset={this.reset}
        >
          <Input
            placeholder="学生姓名"
            value={studentName}
            onChange={(e) => this.setState({ studentName: e.target.value })}
          />
          <Select
            placeholder="年级"
            dict={gradeDict}
            value={gradeId}
            onChange={(val) => this.setState({ gradeId: val })}
          />
          <Select
            placeholder="签单学习中心"
            showSearch
            allowClear={true}
            dict={blCampusDict}
            value={signCampusId}
            onChange={(val) => {
              this.setState({ signCampusId: val });
            }}
          />
          <Select
            placeholder="资源来源"
            dict={entranceDict}
            value={resEntranceId}
            onChange={(val) => this.setState({ resEntranceId: val })}
          />
          <Select
            placeholder="跟进人"
            dict={this.state.deliverTargetDict}
            value={this.state.deliverTarget}
            onChange={(val) => this.setState({ deliverTarget: val })}
          />
          <DatePicker
            placeholder="签单日期（起）"
            disabledDate={this.getDisabledSignTimeStartDate}
            value={signTimeStart}
            onChange={(val) => this.setState({ signTimeStart: val })}
          />
          <DatePicker
            placeholder="签单日期（止）"
            disabledDate={this.getDisabledSignTimeEndDate}
            value={signTimeEnd}
            onChange={(val) => this.setState({ signTimeEnd: val })}
          />
        </SearchBar>

        <List
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
          className={styles.customList}
          rowSelection={{
            selectedRowKeys: this.state.pageSelectedKeys,
            onChange: (keys, rows) => {
              this.onRowSelectChange(keys, rows);
            },
          }}
        />

        {this.state.transformModalVisible && (
          <ModalCustomerPoolTree
            initTreeParam={{ roleCodes: 'CONSULTOR' }}
            loading={this.state.transformLoading}
            onOk={this.onPoolTreeOK}
            onClose={this.toggleTransformModal}
          />
        )}

        <Loading loading={loading} />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const user = state.account.user;
  const { userId, organizationId } = user || {};

  return {
    userId,
    userOrganizationId: organizationId,
    activeTabId: state.main.activeTabId,
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps, null, null, { withRef: true })(CusSignedList);
