import { SS_DEAL_STATUS_EXAM_PASS } from 'PAGE/SSCRM/common/util/dicts';

/**
 * 过滤双师允许跟进的客户列表
 * tips: 未签单且用户ID==跟进人
 *
 * @param list 列表
 * @param userId 用户ID
 * @return {Array|*[]}
 */
export function filterUnFetchListBySS(list = [], userId) {
  if (!list) {
    return [];
  }

  return list.filter((record) => {
    const { customerStatus } = record;
    const deliverTarget = formatDeliverTarget(record);
    const isSigned = calcIsSigned(customerStatus);

    return deliverTarget === userId && !isSigned;
  });
}

/**
 * 判断客户是否已签单
 *
 * @param customerStatus 客户状态
 * @return {boolean}
 */
export function calcIsSigned(customerStatus) {
  return customerStatus === 'FLOW_AWAY';
}

/**
 * 判断是否可以签单
 * tips: 登录人=跟进人,跟进状态为"考试通过"
 *
 * @param record {object} 列表数据
 * @param userId {string} 用户ID
 * @return {boolean}
 */
export function calcCanDoSign(record, userId) {
  if (!record || !userId) {
    return false;
  }

  const deliverTarget = formatDeliverTarget(record);
  const dealStatus = record.dealStatus;

  const isSameUser = deliverTarget === userId;
  const isPass = dealStatus === SS_DEAL_STATUS_EXAM_PASS;

  return isSameUser && isPass;
}

/**
 * 提取列表中的跟进人ID（不同接口字段不同）
 *
 * @param rowData 列表数据
 * @return {null|*}
 */
export function formatDeliverTarget(rowData) {
  if (!rowData) {
    return null;
  }

  return rowData.deliverTargetId || rowData.deliverTarget;
}

/**
 * 格式化搜索条件时间(接远艺接口-流失客户列表)
 *
 * @param timeType 时间类型 1: 最近跟进，2：下次跟进，3：诺到，4：上门，5：创建，6：流失
 * @param date string[] 时间段
 * @return {*}
 */
export function formatDuringTime(timeType = null, date = null) {
  let result = {
    recentlyTimeStart: null, // 最近跟进时间（起）
    recentlyTimeEnd: null, // 最近跟进时间（止）
    nextFollowTimeStart: null, // 下次跟进时间（起）
    nextFollowTimeEnd: null, // 下次跟进时间（止）
    meetingTimeStart: null, // 诺到(原预约上门)时间（起）
    meetingTimeEnd: null, // 诺到(原预约上门)时间（止）
    visitComeDateStart: null, // 上门时间（起）
    visitComeDateEnd: null, // 上门时间（止）
    createDateStart: null, // 创建时间（起）
    createDateEnd: null, // 创建时间（止）
    lostCustomerTimeStart: null, // 流失时间（起）
    lostCustomerTimeEnd: null, // 流失时间（止）
  };

  if (!timeType || !date || date.length <= 1) {
    return result;
  }

  const startTime = date[0];
  const endTime = date[1];
  let timeObj = {};

  switch (timeType) {
    case '1':
      timeObj = { recentlyTimeStart: startTime, recentlyTimeEnd: endTime };
      break;
    case '2':
      timeObj = { nextFollowTimeStart: startTime, nextFollowTimeEnd: endTime };
      break;
    case '3':
      timeObj = { meetingTimeStart: startTime, meetingTimeEnd: endTime };
      break;
    case '4':
      timeObj = { visitComeDateStart: startTime, visitComeDateEnd: endTime };
      break;
    case '5':
      timeObj = { createDateStart: startTime, createDateEnd: endTime };
      break;
    case '6':
      timeObj = { lostCustomerTimeStart: startTime, lostCustomerTimeEnd: endTime };
      break;
    default:
      break;
  }

  result = { ...result, ...timeObj };

  return result;
}

/**
 * 格式化搜索条件时间(接叶聪接口-资源管理-客户列表)
 *
 * @param timeType 时间类型 1: 最近跟进，2：下次跟进，3：诺到，4：上门，5：创建，6：流失
 * @param date string[] 时间段
 * @return {*}
 */
export function formatDuringTime2(timeType = null, date = null) {
  let result = {
    lastFollowUpTimeStart: null, // 最近跟进时间（起）
    lastFollowUpTimeEnd: null, // 最近跟进时间（止）
    nextFollowTimeStart: null, // 下次跟进时间（起）
    nextFollowTimeEnd: null, // 下次跟进时间（止）
    appointmentTimeStart: null, // 诺到(原预约上门)时间（起）
    appointmentTimeEnd: null, // 诺到(原预约上门)时间（止）
    visitComeDateStart: null, // 上门时间（起）
    visitComeDateEnd: null, // 上门时间（止）
    createDateStart: null, // 创建时间（起）
    createDateEnd: null, // 创建时间（止）
    lostCustomerTimeStart: null, // 流失时间（起）
    lostCustomerTimeEnd: null, // 流失时间（止）
  };

  if (!timeType || !date || date.length <= 1) {
    return result;
  }

  const startTime = date[0];
  const endTime = date[1];
  let timeObj = {};

  switch (timeType) {
    case '1':
      timeObj = { lastFollowUpTimeStart: startTime, lastFollowUpTimeEnd: endTime };
      break;
    case '2':
      timeObj = { nextFollowTimeStart: startTime, nextFollowTimeEnd: endTime };
      break;
    case '3':
      timeObj = { appointmentTimeStart: startTime, appointmentTimeEnd: endTime };
      break;
    case '4':
      timeObj = { visitComeDateStart: startTime, visitComeDateEnd: endTime };
      break;
    case '5':
      timeObj = { createDateStart: startTime, createDateEnd: endTime };
      break;
    case '6':
      timeObj = { lostCustomerTimeStart: startTime, lostCustomerTimeEnd: endTime };
      break;
    default:
      break;
  }

  result = { ...result, ...timeObj };

  return result;
}

/**
 * 格式化开始结束时间
 *
 * @param date
 * @return {{startTime: *, endTime: *}|{startTime: string, endTime: string}}
 */
export function formatPeriodTime(date = null) {
  if (!date || date.length <= 1) {
    return { startTime: '', endTime: '' };
  }

  return { startTime: date[0], endTime: date[1] };
}

/**
 * 隐藏部分手机号码
 *
 * @param number
 * @return {string|*}
 */
export function formatPhoneNumber(number) {
  if (!number) {
    return '';
  }

  return number.replace(/(\d{3})(\d{4})(\d{4})/, '$1****$3');
}

/**
 * 分离时间字符串的天数
 *
 * @param time {string} 时间
 * @param splitStr {string} 分割符
 * @return {string}
 */
export function splitDateString(time = '', splitStr = ' ') {
  if (!time) {
    return '';
  }

  const timeArr = time.split(splitStr);

  if (timeArr.length > 0) {
    return timeArr[0];
  }

  return '';
}

/**
 * 格式化新签页面旧boss跳转URL
 *
 * @param params.cusId 客户ID
 * @param params.studentId 学生ID
 * @param params.studentName 学生姓名
 * @param params.gradeId 年级ID
 * @param params.schoolId 学校ID
 * @param params.schoolName 学校名称
 * @param params.blCampusId 合同学习中心
 * @param params.contractType 合同类型
 * @return {string}
 */
export function formatSignNewUrl(params) {
  const { cusId, studentId, studentName, gradeId, schoolId, schoolName, blCampusId, contractType } = params;
  const url =
    `` +
    `function/contract/editContractReactFeat.html` +
    `?customerId=${cusId}` +
    `&studentId=${studentId}` +
    `&consultorStudentName=${studentName}` +
    `&studentGrade=${gradeId}` +
    `&schoolId=${schoolId}` +
    `&schoolName=${schoolName}` +
    `&blCampusId=${blCampusId}` +
    `&pointialStudentIndex=` +
    `&studentType=POTENTIAL` +
    `&contractType=${contractType || 'NEW_CONTRACT'}` +
    `&source=following`;

  return url;
}

// 双师不限制签单按钮
export const SWITCH_CONFIG_UNLIMITED_NEW_SIGN = 'UNLIMITED_NEW_SIGN';
