import React from 'react';
import { connect } from 'dva';
import { DatePicker } from 'antd';
import { Page, SearchBar, Input, Select, List, TextLink, Button } from 'COMPONENT';
import {
  formatDuringTime2,
  formatPhoneNumber,
  splitDateString,
  formatDeliverTarget,
} from '../../consultant/FollowCustomer/utils';
import { SS_DEAL_STATUS_DICT, SS_IS_APPOINT_COME_DICT } from '../../common/util/dicts';
import dicts from '../../consultant/FollowCustomer/dicts';
import loaders from './loaders';
import * as commonLoader from 'PAGE/SSCRM/common/AddCustomerMain/loaders';
import { loadCampusDict, loadCommonDict } from 'UTIL/dict';
import HeadSearchContainer from '../common/component/HeadSearchContainer';
import SelectSchoolModal from 'PAGE/common/SelectSchoolModal';
import styles from './index.less';
import { dateToMoment } from 'COMPONENT/utils/form';
import ModalEditCustomerContact from 'BLOC/ModalEditCustomerContact';

const RangePicker = DatePicker.RangePicker;
const { COL_WIDTH_S, COL_WIDTH_M } = List;

/**
 * 资源管理-客户列表
 */
class CustomerList extends React.Component {
  constructor(props) {
    super(props);

    this.initSearchParam = {
      studentName: null,
      contact: null,
      gradeId: null,
      resEntranceId: null,
      isAppointCome: null,
      dealStatus: null,
      intentionOfTheCustomerId: null,
      intentionCampusId: null,
      schoolId: null,
      schoolName: null,
      customerName: null,
      tempTimeType: '1',
      tempTime: null,
    };

    this.state = {
      loading: false,

      studentName: null, // 查询条件 学生姓名
      contact: null, // 查询条件 联系方式
      gradeId: null, // 查询条件 年级
      resEntranceId: null, // 查询条件 资源入口/资源来源
      isAppointCome: null, // 查询条件 是否诺到/是否上门
      dealStatus: null, // 查询条件 跟进状态
      intentionOfTheCustomerId: null, // 查询条件 意向度
      intentionCampusId: '', // 查询条件 意向学习中心
      schoolId: null, // 查询条件 学校
      customerName: null, // 查询条件 客户姓名

      tempTimeType: '1', // 临时查询条件 时间类型
      tempTime: null, // 临时查询条件 时间(根据时间类型设置不同的起止时间)

      blCampusDict: [], // 字典 学校
      gradeDict: [], // 字典 年级
      entranceDict: [], // 字典 资源入口
      intentionDict: [], // 字典 意向度

      list: [], // 列表数据
      currentPage: 1, // 当前页码
      pageSize: 20, // 每页记录数
      totalRecords: 0, // 总记录数

      selectSchoolModalVis: false, // 选择学校弹框显示状态
      schoolName: null, // 学校名称（用于显示）
      userProvince: null, // 当前登录用户所属省份
      userCity: null, // 当前登录用户所属城市

      modalEditPhoneShow: false, // 显示修改手机号码弹窗
      selectedRow: null, // 当前操作的行数据
    };
  }

  componentDidMount() {
    this.initPage();
  }

  initBranch = async () => {
    const blCampusDict = await loadCampusDict();

    this.setState({ blCampusDict });
  };

  initPage = async () => {
    this.loadList();
    this.initBranch();

    const [gradeDict, entranceDict, intentionDict, provinceCity] = await Promise.all([
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('RES_ENTRANCE'),
      loadCommonDict('CUSTOMER_SATISFICING'),
      commonLoader.loadLoginUserProvinceCity(),
    ]);

    const { province, city } = provinceCity;

    this.setState({
      gradeDict,
      entranceDict,
      intentionDict,
      userProvince: province,
      userCity: city,
    });
  };

  onSearch = () => {
    this.loadList({ currentPage: 1 });
  };

  onReset = () => {
    this.setState(this.initSearchParam);
  };

  onRefresh = () => {
    this.loadList({});
  };

  onPaginationChange = ({ current, pageSize }) => {
    const changed = {
      currentPage: this.state.pageSize !== pageSize ? 1 : current,
      pageSize,
    };

    this.loadList(changed);
  };

  /**
   * 获取客户列表
   *
   * @param changeParam
   * @return {Promise<{totalRecords: *, page: *, list: *}>}
   */
  loadList = (changeParam = {}) => {
    const param = this.formatSearchParams();
    const params = { ...param, ...changeParam };
    const { currentPage, pageSize } = params;

    this.setState({ loading: true });

    loaders
      .loadCustomerList(params)
      .then(({ list, totalRecords }) => {
        this.setState({ loading: false, currentPage, pageSize, list, totalRecords });
      })
      .catch(() => {
        this.setState({ loading: false, currentPage, pageSize, list: [], totalRecords: 0 });
      });
  };

  /**
   * 格式化请求参数
   */
  formatSearchParams = () => {
    const {
      studentName,
      customerName,
      gradeId,
      schoolId,
      contact,
      resEntranceId,
      isAppointCome,
      dealStatus,
      intentionOfTheCustomerId,
      intentionCampusId,
      tempTimeType,
      tempTime,
      currentPage,
      pageSize,
    } = this.state;

    let param = {
      studentName,
      customerName,
      gradeId,
      schoolId,
      contact,
      resEntranceId,
      isAppointCome,
      dealStatus,
      intentionOfTheCustomerId,
      intentionCampusId,
      currentPage,
      pageSize,
    };
    const timeParam = formatDuringTime2(tempTimeType, tempTime);

    param = {
      ...param,
      ...timeParam,
    };

    return param;
  };

  /**
   * 表格列
   *
   * @return {*[]}
   */
  formatColumns = () => {
    return [
      { title: '客户姓名', dataIndex: 'customerName', key: 'customerName', width: COL_WIDTH_M },
      {
        title: '联系方式',
        dataIndex: 'contact',
        key: 'contact',
        width: COL_WIDTH_M,
        render: (text) => {
          return formatPhoneNumber(text);
        },
      },
      {
        title: '学生姓名',
        dataIndex: 'studentName',
        key: 'studentName',
        width: COL_WIDTH_M,
        render: (text, record) => {
          const { customerId, studentId, employee } = record;
          const deliverTarget = formatDeliverTarget(record);
          const params = {
            cusId: customerId,
            studentId,
            detailEditable: false,
            followEditable: false,
            deliverTarget,
            list: [],
          };

          return (
            <TextLink onClick={() => this.onPageToDetail(params)}>
              <div>
                {text}
                {employee && (
                  <div>
                    <span className={'employeeFlag'}>内部员工</span>
                  </div>
                )}
              </div>
            </TextLink>
          );
        },
      },
      { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: COL_WIDTH_S },
      { title: '学校', dataIndex: 'schoolName', key: 'schoolName', width: COL_WIDTH_M },
      { title: '资源所属学习中心', dataIndex: 'blCampusName', key: 'blCampusName', width: COL_WIDTH_M },
      { title: '意向学习中心', dataIndex: 'intentionCampusName', key: 'intentionCampusName', width: COL_WIDTH_M },
      { title: '资源来源', dataIndex: 'resEntranceName', key: 'resEntranceName', width: 80 },
      {
        title: '最后跟进时间',
        dataIndex: 'lastFollowUpTime',
        key: 'lastFollowUpTime',
        width: 120,
        render: (text) => {
          return splitDateString(text);
        },
      },
      { title: '跟进状态', dataIndex: 'dealStatusName', key: 'dealStatusName', width: 80 },
      {
        title: '是否诺到',
        dataIndex: 'isAppointCome',
        key: 'isAppointCome',
        width: 80,
        render: (text, record) => {
          const { appointmentDate } = record;

          return appointmentDate ? '是' : '否';
        },
      },
      { title: '诺到日期', dataIndex: 'appointmentDate', key: 'appointmentDate', width: 160 },
      { title: '意向度', dataIndex: 'intentionName', key: 'intentionName', width: 80 },
      { title: '未跟进天数', dataIndex: 'noopDay', key: 'noopDay', width: 120 },
      { title: '创建日期', dataIndex: 'createDate', key: 'createDate', width: 120 },
      { title: '录入人', dataIndex: 'createUserName', key: 'createUserName' },
      {
        title: '操作',
        dataIndex: 'action',
        key: 'action',
        width: COL_WIDTH_M,
        render: (text, record) => {
          return (
            <TextLink
              disabled={!this.props.btnAuth_editContact}
              onClick={() => {
                this.onEditContact(record);
              }}
            >
              修改联系方式
            </TextLink>
          );
        },
      },
    ];
  };

  /**
   * 跳转客户详情
   */
  onPageToDetail = (params) => {
    const { dispatch, type } = this.props;
    const path = 'SSCustomerDetailView';

    params = {
      ...params,
      type,
      onRefresh: this.search,
    };

    dispatch({
      type: 'main/openPage',
      payload: {
        path: `/${path}`,
        params,
        refresh: true,
      },
    });
  };

  /**
   * 学校输入值改变触发事件
   *
   * @param val
   */
  onSchoolNameChange = (val) => {
    if (!val) {
      this.setState({ schoolId: null, schoolName: null });
    }
  };

  /**
   * 修改手机号码
   *
   * @param record {Object}
   */
  onEditContact = (record) => {
    this.setState({ selectedRow: record });
    this.toggleModalEditContact();
  };

  /**
   * 显示修改手机号码弹窗
   */
  toggleModalEditContact = () => {
    this.setState((prevState) => {
      return {
        modalEditPhoneShow: !prevState.modalEditPhoneShow,
      };
    });
  };

  render() {
    const {
      studentName,
      contact,
      gradeId,
      resEntranceId,
      isAppointCome,
      dealStatus,
      intentionOfTheCustomerId,
      intentionCampusId,
      schoolName,
      customerName,
      tempTimeType,
      tempTime,
    } = this.state;
    const { gradeDict, entranceDict, intentionDict, blCampusDict } = this.state;
    const { loading, list, totalRecords, currentPage, pageSize } = this.state;
    const { userProvince, userCity, selectSchoolModalVis } = this.state;
    const { modalEditPhoneShow, selectedRow } = this.state;
    const columns = this.formatColumns();

    return (
      <Page
        fitWindowHeight={true}
        route={this.props.route}
        loading={loading}
        className={styles.pageWrap}
        subHeader={
          <HeadSearchContainer
            searchBar={
              <SearchBar onSearch={this.onSearch} onReset={this.onReset}>
                <Input
                  placeholder="学生姓名"
                  value={studentName}
                  onChange={(e) => this.setState({ studentName: e.target.value })}
                />
                <Input
                  placeholder="联系方式"
                  value={contact}
                  onChange={(e) => this.setState({ contact: e.target.value })}
                />
                <Select
                  placeholder="年级"
                  dict={gradeDict}
                  value={gradeId}
                  onChange={(val) => this.setState({ gradeId: val })}
                />
                <Select
                  placeholder="资源来源"
                  dict={entranceDict}
                  value={resEntranceId}
                  onChange={(val) => this.setState({ resEntranceId: val })}
                />
                <Select
                  placeholder="是否诺到"
                  dict={SS_IS_APPOINT_COME_DICT}
                  value={isAppointCome}
                  onChange={(val) => this.setState({ isAppointCome: val })}
                />
                <Select
                  placeholder="跟进状态"
                  dict={SS_DEAL_STATUS_DICT}
                  value={dealStatus}
                  onChange={(val) => this.setState({ dealStatus: val })}
                />
                <Select
                  placeholder="意向度"
                  dict={intentionDict}
                  value={intentionOfTheCustomerId}
                  onChange={(val) => this.setState({ intentionOfTheCustomerId: val })}
                />
                <Select
                  placeholder="意向学习中心"
                  showSearch
                  allowClear={true}
                  dict={blCampusDict}
                  value={intentionCampusId}
                  onChange={(val) => {
                    this.setState({ intentionCampusId: val });
                  }}
                />
                <Input
                  placeholder="学校"
                  readOnly
                  allowClear={true}
                  value={schoolName}
                  onClick={() => {
                    this.setState({ selectSchoolModalVis: true });
                  }}
                  onChange={(e) => {
                    this.onSchoolNameChange(e.target.value);
                  }}
                />
                <Input
                  placeholder="客户姓名"
                  value={customerName}
                  onChange={(e) => this.setState({ customerName: e.target.value })}
                />

                <Select
                  placeholder="时间类型"
                  dict={dicts.followTimeTypeDict}
                  value={tempTimeType}
                  onChange={(val) => this.setState({ tempTimeType: val })}
                />
                <RangePicker
                  placeholder={['日期(起)', '日期(止)']}
                  format="YYYY-MM-DD"
                  showTime={false}
                  allowClear={true}
                  size="large"
                  value={tempTime ? [dateToMoment(tempTime[0]), dateToMoment(tempTime[1])] : []}
                  onChange={(date, dateString) => {
                    this.setState({ tempTime: dateString });
                  }}
                />
              </SearchBar>
            }
            buttonGroup={<Button onClick={this.onRefresh}>刷新</Button>}
          />
        }
      >
        <List
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
          bordered={true}
        />

        {selectSchoolModalVis && (
          <SelectSchoolModal
            defaultProvince={userProvince}
            defaultCity={userCity}
            onConfirm={(school, schoolName) => {
              this.setState({ selectSchoolModalVis: false, schoolId: school, schoolName });
            }}
            onClose={() => this.setState({ selectSchoolModalVis: false })}
          />
        )}

        {modalEditPhoneShow && selectedRow && (
          <ModalEditCustomerContact
            customerId={selectedRow.customerId}
            customerName={selectedRow.customerName}
            customerContact={selectedRow.contact}
            onClose={this.toggleModalEditContact}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const btnAuth = state.account.btnAuths || [];
  const btnAuth_editContact = btnAuth.indexOf('SSResourceCustomerEditContact') > -1;

  return {
    btnAuth_editContact,
  };
};

export default connect(mapStateToProps)(CustomerList);
