import React from 'react';
import { connect } from 'dva';
import { Col, Row, Tooltip, Icon } from 'antd';
import { Modal, FormItem, Select, Input, TextLink, Popover } from 'COMPONENT';
import SearchTag from './SearchTag';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import ChooseFileButton from 'PAGE/personnelManage/TeacherIntroduction/ChooseFileButton';
import styles from './index.less';
import fileUtil from 'UTIL/file';
import { urlBase } from 'CONST/config';
import { labelWidth, FLAG_LOAD_CAMPUS_TAG_LIST } from './constant';
import loader from './loader';

class ImportResourceModal extends React.Component {
  static defaultProps = {
    campusId: null, // 资源学习中心ID
    campusDict: [], // 资源学习中心字典
  };

  constructor(props) {
    super(props);

    this.state = {
      loading: false,

      campusId: null, // 资源学习中心ID
      file: null, // 文件
      tag: null, // 自定义标签

      tagList: [], // 资源学习中心下的标签列表
    };

    this.errors = {};
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化资源学习中心
   */
  initContent = () => {
    const { campusId } = this.props;

    if (campusId) {
      this.setState({ campusId });
    }

    this.loadTagNameList(campusId);
  };

  /**
   * 获取资源学习中心下的标签列表
   *
   * @param campusId
   * @return {Promise<void>}
   */
  loadTagNameList = async (campusId) => {
    const data = await loader.getPoolTagList({ resourceCampusIds: campusId });
    this.setState({ tagList: data });
  };

  /**
   * 选择文件
   */
  onSelectFile = (file) => {
    this.setState({ file: file });
  };

  /**
   * 选择学习中心
   *
   * @param value
   */
  onCampusChange = (value) => {
    this.setState({ campusId: value });

    if (FLAG_LOAD_CAMPUS_TAG_LIST) {
      this.loadTagNameList(value);
    }
  };

  /**
   * 选中标签
   *
   * @param tagName
   */
  onTagClick = (tagName) => {
    this.setState({ tag: tagName });
  };

  /**
   * 校验自定义标签名称
   *
   * @param value {string} 标签名称
   * @return {null}
   */
  validTagName = (value) => {
    const reg = /^[\s+a-zA-Z0-9_\u4e00-\u9fa5]{0,10}$/;
    let msg = null;

    if (value && !reg.test(value)) {
      msg = '请输入中文、数字、英文或"_"(下划线)';
    }

    return msg;
  };

  /**
   * 记录表单校验结果
   *
   * @param key
   * @param error {string} 错误信息
   */
  onErrorChange = (key, error) => {
    if (error) {
      this.errors[key] = error;
    } else {
      delete this.errors[key];
    }
  };

  /**
   * 下载模版
   * 旧-http://xuebangsoft-eduboss.oss-cn-shenzhen.aliyuncs.com/客户资源导入模板.xlsx
   * 新-https://ssboss-test.oss-cn-shenzhen.aliyuncs.com/双师客户资源导入模板.xlsx
   */
  downloadTemplate = () => {
    const { ossPath } = this.props;
    const fileName = '双师客户资源导入模板.xlsx';
    const url = ossPath + fileName;

    fileUtil.downloadDirect(url, fileName);
  };

  /**
   * 提交
   */
  onSubmit = () => {
    const { file, campusId, tag } = this.state;

    if (!file) {
      showMessage('请选择文件');
      return false;
    }

    if (!campusId) {
      showMessage('请选择资源学习中心');
      return false;
    }

    if (Object.keys(this.errors).length > 0) {
      return false;
    }

    this.doImport({ blCampusId: campusId, tag: tag ? tag.trim() : '', file });
  };
  doImport = (param) => {
    this.toggleLoading();

    const url = urlBase + 'TwoTeacherCustomerAction/uploadCustomerResource.do';
    const formData = new FormData();

    for (let key in param) {
      formData.append(key, param[key]);
    }

    const xhr = new XMLHttpRequest();
    xhr.onload = () => {
      try {
        this.toggleLoading();

        let result = {};
        try {
          result = JSON.parse(xhr.responseText);
        } catch (e) {}

        if (xhr.status === 200 && result.resultCode === 0) {
          showMessage(result.resultMessage || '导入成功', 'success');
          this.onClose(true);
        } else {
          showMessage(result.resultMessage);
        }
      } catch (e) {
        showMessage('系统错误');
        this.toggleLoading();
      }
    };
    xhr.onerror = () => {
      showMessage('网络错误!');
      this.toggleLoading();
    };
    xhr.open('POST', url);
    xhr.send(formData);
  };

  toggleLoading = () => {
    this.setState((prevState) => {
      return {
        loading: !prevState.loading,
      };
    });
  };

  onClose = (isUploaded) => {
    this.props.onClose(isUploaded);
  };

  render() {
    const { campusDict } = this.props;
    const { loading, campusId, file } = this.state;

    return (
      <Modal
        loading={loading}
        title="资源导入"
        width={450}
        onOk={this.onSubmit}
        onClose={this.props.onClose}
        cancelBtnShow={false}
        okText="提交"
        className={styles.modalWrap}
      >
        <Row gutter={5}>
          <Col span={24}>
            <FormItem name="资源学习中心" labelWidth={labelWidth}>
              <Select placeholder="请选择" dict={campusDict} value={campusId} onChange={this.onCampusChange} />
            </FormItem>
            <FormItem name="" labelWidth={labelWidth}>
              <TextLink onClick={this.downloadTemplate}>下载模板</TextLink>
            </FormItem>
            <FormItem name="上传文件" labelWidth={labelWidth} className={styles.fileItem}>
              <ChooseFileButton
                key="upload"
                size="small"
                btnText="选择文件"
                accept="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet,application/vnd.ms-excel"
                fileType={['excel']}
                onChange={this.onSelectFile}
              />
              {file && (
                <Tooltip placement="top" title={file.name}>
                  <span style={{ marginLeft: '15px' }}>{file.name}</span>
                </Tooltip>
              )}
            </FormItem>
            <FormItem name="自定义标签" labelWidth={labelWidth} style={{ position: 'relative', marginTop: 5 }}>
              <Popover
                maxWidth={260}
                placement="bottom"
                trigger="click"
                overlayClassName={styles.popoverItem}
                content={
                  <SearchTag placeholder="输入历史标签搜索" tagList={this.state.tagList} onSelect={this.onTagClick} />
                }
              >
                <Input
                  placeholder="标签可用于资源池的搜索"
                  maxLength={10}
                  wordCount={true}
                  allowClear={true}
                  value={this.state.tag}
                  onChange={(e) => {
                    this.setState({ tag: e.target.value });
                  }}
                  validator={{
                    onErrorChange: (e) => this.onErrorChange('tag', e),
                    rules: [this.validTagName],
                    silent: false,
                  }}
                />
              </Popover>

              <span className={styles.infoIcon}>
                <Popover
                  icon="info"
                  maxWidth={200}
                  content={
                    <div className={styles.tips}>
                      <p>1、仅可输入中文、数字、英文及"_"(下划线)</p>
                      <p>2、不可超过10个中文字符</p>
                    </div>
                  }
                >
                  <Icon type="info-circle" size={14} style={{ color: 'rgb(65, 174, 255)' }} />
                </Popover>
              </span>
            </FormItem>
          </Col>
        </Row>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const user = state.account.user;
  const { aliPath } = user || {};

  return {
    ossPath: aliPath,
  };
};

export default connect(mapStateToProps)(ImportResourceModal);
