import React from 'react';
import { connect } from 'dva';
import { DatePicker } from 'antd';
import { Page, SearchBar, Input, Select, List, TextLink, Button } from 'COMPONENT';
import ShowConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import { formatPhoneNumber, formatPeriodTime, splitDateString } from '../../consultant/FollowCustomer/utils';
import {
  SS_IS_APPOINT_COME_DICT,
  SS_EXPORT_STATUS,
  SS_EXPORT_STATUS_SUCCESS,
  SS_EXPORT_STATUS_WAITING,
  SS_EXPORT_STATUS_ERROR,
} from '../../common/util/dicts';
import loaders from './loaders';
import { loadCampusDict, loadCommonDict } from 'UTIL/dict';
import ImportResourceModal from './component/ImportResourceModal';
import styles from './index.less';
import HeadSearchContainer from '../common/component/HeadSearchContainer';
import { dateToMoment } from 'COMPONENT/utils/form';

const RangePicker = DatePicker.RangePicker;
const { COL_WIDTH_M, COL_WIDTH_L } = List;

/**
 * 资源管理-资源列表导入
 */
class ResourceListImport extends React.Component {
  constructor(props) {
    super(props);

    this.initSearchParam = {
      contact: null,
      gradeId: null,
      isAppointCome: null,
      intentionCampusId: null,
      schoolName: null,
      customerName: null,
      cusStatus: null,
      tempTime: null,
    };

    this.state = {
      loading: false,

      contact: null, // 查询条件 联系方式
      gradeId: null, // 查询条件 年级
      isAppointCome: null, // 查询条件 是否诺到/是否上门
      intentionCampusId: '', // 查询条件 意向学习中心
      schoolName: null, // 查询条件 学校
      customerName: null, // 查询条件 客户姓名
      cusStatus: null, // 查询条件 导入状态

      tempTime: null, // 临时查询条件 时间

      blCampusDict: [], // 字典 学校
      gradeDict: [], // 字典 年级
      entranceDict: [], // 字典 资源入口

      list: [], // 列表数据
      currentPage: 1, // 当前页码
      pageSize: 20, // 每页记录数
      totalRecords: 0, // 总记录数
      nowCusStatus: null, // 当前查询列表的导入状态值，用于批量操作按钮显示判断

      importModalVisible: false, // 显示导入弹窗
      selectedRows: [], // 当前选中行数据
      selectedRowKeys: [], // 当前选中行的key
    };
  }

  componentDidMount() {
    this.initPage();
  }

  initBranch = async () => {
    const blCampusDict = await loadCampusDict();

    this.setState({ blCampusDict });
  };

  initPage = async () => {
    this.loadList();
    await this.initBranch();

    const [gradeDict, entranceDict] = await Promise.all([
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('RES_ENTRANCE'),
      this.loadList(this.formatSearchParams()),
    ]);

    this.setState({ gradeDict, entranceDict });
  };

  onSearch = () => {
    this.loadList({ currentPage: 1 });
  };

  onReset = () => {
    this.setState(this.initSearchParam);
  };

  onPaginationChange = ({ current, pageSize }) => {
    const changed = {
      currentPage: this.state.pageSize !== pageSize ? 1 : current,
      pageSize,
    };

    this.loadList(changed);
  };

  /**
   * 获取客户列表
   *
   * @param changeParam
   * @return {Promise<{totalRecords: *, page: *, list: *}>}
   */
  loadList = (changeParam = {}) => {
    const param = this.formatSearchParams();
    const params = { ...param, ...changeParam };

    this.setState({ loading: true });

    loaders
      .loadCustomerList(params)
      .then(({ list, totalRecords }) => {
        this.formatListResponse(params, { list, totalRecords });
      })
      .catch((e) => {
        this.formatListResponse(params, null);
      });
  };

  /**
   * 格式化请求参数
   */
  formatSearchParams = () => {
    const {
      customerName,
      gradeId,
      schoolName,
      contact,
      isAppointCome,
      intentionCampusId,
      cusStatus,
      tempTime,
      currentPage,
      pageSize,
    } = this.state;

    let param = {
      customerName,
      gradeId,
      schoolName,
      contact,
      isAppointCome,
      cusStatus,
      intentionCampusId,
      currentPage,
      pageSize,
    };
    const timeParam = formatPeriodTime(tempTime);

    param = {
      ...param,
      ...timeParam,
    };

    return param;
  };

  /**
   * 格式化请求结果
   *
   * @param params 请求参数集合
   * @param response 返回结果集合
   */
  formatListResponse = (params = {}, response = {}) => {
    this.setState({
      loading: false,
      currentPage: params.currentPage,
      pageSize: params.pageSize,
      list: response.list || [],
      totalRecords: response.totalRecords || 0,
      nowCusStatus: params.cusStatus,
      selectedRows: [],
      selectedRowKeys: [],
    });
  };

  /**
   * 表格列
   *
   * @return {*[]}
   */
  formatColumns = () => {
    return [
      { title: '客户姓名', dataIndex: 'cusName', key: 'cusName', width: COL_WIDTH_M },
      {
        title: '联系方式',
        dataIndex: 'contact',
        key: 'contact',
        width: COL_WIDTH_M,
        render: (text, record) => {
          return formatPhoneNumber(text);
        },
      },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_M },
      { title: '学生年级', dataIndex: 'gradeName', key: 'gradeName', width: COL_WIDTH_M },
      { title: '学校', dataIndex: 'schoolName', key: 'schoolName', width: COL_WIDTH_M },
      { title: '资源所属学习中心', dataIndex: 'blCampusName', key: 'blCampusName', width: COL_WIDTH_M },
      { title: '资源来源', dataIndex: 'resEntranceName', key: 'resEntranceName', width: 80 },
      {
        title: '是否诺到',
        dataIndex: 'isAppointCome',
        key: 'isAppointCome',
        width: 80,
        render: (text, record, index) => {
          const { appointmentTime } = record;

          return appointmentTime ? '是' : '否';
        },
      },
      { title: '诺到时间', dataIndex: 'appointmentDate', key: 'appointmentDate', width: 160 },
      { title: '跟进备注', dataIndex: 'lastRemark', key: 'lastRemark', width: 160, lines: 2 },
      { title: '资源分配节点', dataIndex: 'deliverTargetName', key: 'deliverTargetName', width: COL_WIDTH_M },
      { title: '导入状态', dataIndex: 'cusStatus', key: 'cusStatus', dict: SS_EXPORT_STATUS, width: 80 },
      { title: '失败原因', dataIndex: 'failReason', key: 'failReason', width: 120, lines: 2 },
      { title: '创建人', dataIndex: 'lastDeliverName', key: 'lastDeliverName' },
      {
        title: '创建日期',
        dataIndex: 'createDate',
        key: 'createDate',
        width: 120,
        render: (text) => {
          return splitDateString(text);
        },
      },
      {
        title: '操作',
        key: 'action',
        fixed: 'right',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          const { cusStatus } = record;
          const canDoDelete = cusStatus !== SS_EXPORT_STATUS_SUCCESS;
          const canDoImport = cusStatus === SS_EXPORT_STATUS_WAITING;
          const style = { marginRight: 15 };

          return (
            <span>
              <TextLink disabled={!canDoDelete} onClick={() => this.onDelete(record)} style={style}>
                删除
              </TextLink>
              <TextLink disabled={!canDoImport} onClick={() => this.onImport(record)}>
                导入
              </TextLink>
            </span>
          );
        },
      },
    ];
  };

  /**
   * 执行删除
   *
   * @param record 列表行数据
   */
  onDelete = (record) => {
    const ids = this.formatSelectedIds(record);

    if (!ids) {
      ShowMessage('请选择要删除的条目！', 'error');
      return;
    }

    ShowConfirm('该操作将删除选中的条目，是否继续？', async () => {
      const idKey = ids.indexOf(',') > -1 ? 'ws' : 'id';
      const params = { [idKey]: ids, oper: 'del' };

      await loaders.deleteCustomer(params).catch((e) => {
        ShowMessage('删除失败，请重试！', 'error');
      });

      ShowMessage('删除成功！', 'success');
      this.onSearch();
    });
  };

  /**
   * 执行导入
   *
   * @param record {object|null} 表格行数据
   */
  onImport = (record = null) => {
    const ids = this.formatSelectedIds(record);

    if (!ids) {
      ShowMessage('请选择要导入的条目！', 'error');
      return;
    }

    loaders
      .importCustomer({ ids })
      .then((result) => {
        ShowMessage(result.resultMessage || '导入成功！', 'success');
        this.onSearch();
      })
      .catch((e) => {
        ShowMessage(e.text || '导入失败！', 'error');
        this.onSearch();
      });
  };

  /**
   * 格式化选中列表的ids
   *
   * @param record 行数据（单个导入/删除时有效）
   */
  formatSelectedIds = (record = null) => {
    let ids = '';

    if (record) {
      ids = record.id;
    } else {
      const { selectedRows } = this.state;

      if (selectedRows) {
        ids = selectedRows.map((item) => item.id).join(',');
      }
    }

    return ids;
  };

  /**
   * 打开导入弹窗
   * 1、上传导入Excel先导入到中间表
   * 2、点击导入才真正导入到客户列表
   */
  onShowImportModal = () => {
    this.setState({ importModalVisible: true });
  };

  /**
   * 关闭导入弹窗
   *
   * @param isUploaded 是否导入成功
   */
  onImportModalClose = (isUploaded) => {
    this.setState({ importModalVisible: false });

    if (isUploaded) {
      this.onSearch();
    }
  };

  render() {
    const {
      contact,
      gradeId,
      isAppointCome,
      intentionCampusId,
      schoolName,
      customerName,
      cusStatus,
      importModalVisible,
      selectedRowKeys,
      tempTime,
    } = this.state;
    const { gradeDict, entranceDict, blCampusDict } = this.state;
    const { loading, list, totalRecords, currentPage, pageSize, nowCusStatus } = this.state;
    const columns = this.formatColumns();
    const showMultiDelete = nowCusStatus === SS_EXPORT_STATUS_WAITING || nowCusStatus === SS_EXPORT_STATUS_ERROR;
    const showMultiImport = nowCusStatus === SS_EXPORT_STATUS_WAITING;

    return (
      <Page
        fitWindowHeight={true}
        route={this.props.route}
        loading={loading}
        className={styles.pageWrap}
        subHeader={
          <HeadSearchContainer
            searchBar={
              <SearchBar onSearch={this.onSearch} onReset={this.onReset}>
                <RangePicker
                  placeholder={['日期(起)', '日期(止)']}
                  format="YYYY-MM-DD"
                  showTime={false}
                  allowClear={true}
                  size="large"
                  onChange={(date, dateString) => {
                    this.setState({ tempTime: dateString });
                  }}
                  value={tempTime ? [dateToMoment(tempTime[0]), dateToMoment(tempTime[1])] : []}
                />
                <Input
                  placeholder="客户姓名"
                  value={customerName}
                  onChange={(e) => this.setState({ customerName: e.target.value })}
                />
                <Input
                  placeholder="联系方式"
                  value={contact}
                  onChange={(e) => this.setState({ contact: e.target.value })}
                />
                <Input
                  placeholder="学校"
                  value={schoolName}
                  onChange={(e) => this.setState({ schoolName: e.target.value })}
                />
                <Select
                  placeholder="学生年级"
                  dict={gradeDict}
                  value={gradeId}
                  onChange={(val) => this.setState({ gradeId: val })}
                />
                <Select
                  placeholder="资源所属学习中心"
                  showSearch
                  allowClear={true}
                  dict={blCampusDict}
                  value={intentionCampusId}
                  onChange={(val) => {
                    this.setState({ intentionCampusId: val });
                  }}
                />
                <Select
                  placeholder="是否诺到"
                  dict={SS_IS_APPOINT_COME_DICT}
                  value={isAppointCome}
                  onChange={(val) => this.setState({ isAppointCome: val })}
                />
                <Select
                  placeholder="导入状态"
                  dict={SS_EXPORT_STATUS}
                  value={cusStatus}
                  onChange={(val) => this.setState({ cusStatus: val })}
                />
              </SearchBar>
            }
            buttonGroup={null}
          />
        }
        headerToolBar={
          <div>
            <Button type="primary" onClick={this.onShowImportModal} style={{ marginRight: 10 }}>
              资源导入
            </Button>
            {showMultiDelete && (
              <Button
                type="primary"
                onClick={(e) => {
                  this.onDelete(null);
                }}
                style={{ marginRight: 10 }}
              >
                批量删除
              </Button>
            )}
            {showMultiImport && (
              <Button
                type="primary"
                onClick={(e) => {
                  this.onImport(null);
                }}
              >
                批量导入
              </Button>
            )}
          </div>
        }
      >
        <List
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
          rowSelection={{
            selectedRowKeys: selectedRowKeys,
            onChange: (keys, rows) => {
              this.setState({ selectedRowKeys: keys, selectedRows: rows });
            },
          }}
          bordered={true}
        />

        {importModalVisible && (
          <ImportResourceModal campusId={null} campusDict={blCampusDict} onClose={this.onImportModalClose} />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(ResourceListImport);
