import React from 'react';
import moment from 'moment';
import { Page, List, SearchFilter, Button, Input, MultiSelect, Select, SearchField } from 'COMPONENT';
import { DatePicker } from 'antd';
import { getCampusByLoginUser } from 'SERVICE/common';
import { Columns } from './const';
import { getOriginReturnStatistics, exportOriginReturnStatistics } from 'SERVICE/audit';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const title = '原路退费统计';
const { RangePicker } = DatePicker;
const time = [
  moment().subtract(1, 'month').startOf('month'), //上个月
  moment().endOf('day'),
];
const searchParams = {
  time,
  branchId: '',
  campusIds: [],
  studentName: '',
};

export default class OriginReturnStatistics extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      dataList: [],
      searchParams,
      loading: false,
      branchDict: [],
      campusDict: [],
      pagination: {
        total: 0,
        current: 1,
        pageSize: 20,
      },
    };
    this.campusCache = {};
    this.initDict = this.initDict.bind(this);
    this.handleSearch = this.handleSearch.bind(this);
    this.handleReset = this.handleReset.bind(this);
    this.loadData = this.loadData.bind(this);
    this.onTimeChange = this.onTimeChange.bind(this);
    this.onPageChange = this.onPageChange.bind(this);
    this.onBranchChange = this.onBranchChange.bind(this);
    this.onStudentNameChange = this.onStudentNameChange.bind(this);
    this.onCampusChange = this.onCampusChange.bind(this);
    this.transformParams = this.transformParams.bind(this);
    this.exportData = this.exportData.bind(this);
  }

  componentDidMount() {
    this.initDict();
    this.loadData();
  }

  async initDict() {
    const data = await getCampusByLoginUser();
    const branchDict = [];
    data.forEach((group) => {
      const branch = group[0] || {};
      const campuses = group && group.length > 1 ? group.slice(1) : [];
      if (branch) {
        branchDict.push({
          val: branch.id,
          text: branch.name,
        });
        this.campusCache[branch.id] = campuses.map((campus) => {
          return {
            val: campus.id,
            text: campus.name,
          };
        });
      }
    });
    this.setState({ branchDict });
  }

  transformParams() {
    const { searchParams, pagination } = this.state;
    const { time, studentName, campusIds, branchId } = searchParams;
    const { pageSize, current: pageNo } = pagination;
    const [startTime, endTime] = time;
    return {
      startTime: startTime.valueOf(),
      endTime: endTime.valueOf(),
      studentName,
      campusIds,
      branchId,
      pageSize,
      pageNo,
    };
  }

  async loadData() {
    try {
      const { pagination } = this.state;
      this.setState({ loading: true });
      const { resultCode, records, rows } = await getOriginReturnStatistics(this.transformParams());
      if (resultCode === 0) {
        this.setState({ dataList: rows, pagination: { ...pagination, total: records } });
      }
    } finally {
      this.setState({ loading: false });
    }
  }

  async exportData() {
    if (this.state.pagination.total > 5000) {
      return showMessage('记录数大于5000条，请缩小范围查询，再点击导出', 'error');
    }
    const { resultCode, data, resultMessage } = await exportOriginReturnStatistics(this.transformParams());
    if (resultCode === 0) {
      if (data) {
        window.top.open(data);
        showMessage('导出成功', 'success');
      } else {
        showMessage('导出列表无数据,请重新选择筛选项!', 'error');
      }
    } else {
      showMessage(resultMessage || '导出失败', 'error');
    }
  }

  handleSearch() {
    const { pagination } = this.state;
    this.setState({ pagination: { ...pagination, current: 1 } }, this.loadData);
  }

  handleReset() {
    const { pagination } = this.state;
    this.setState({ searchParams, pagination: { ...pagination, current: 1, pageSize: 20 } }, this.loadData);
  }

  onTimeChange(value) {
    const params = this.state.searchParams;
    this.setState({ searchParams: { ...params, time: value } });
  }

  onPageChange({ current, pageSize }) {
    const { pagination } = this.state;
    this.setState({ pagination: { ...pagination, pageSize, current } }, this.loadData);
  }

  onStudentNameChange(e) {
    const { searchParams } = this.state;
    let studentName = e.target.value ? e.target.value.trim() : '';
    this.setState({ searchParams: { ...searchParams, studentName } });
  }

  onCampusChange(campusIds) {
    const { searchParams } = this.state;
    this.setState({ searchParams: { ...searchParams, campusIds } });
  }

  async onBranchChange(branchId = '') {
    const { searchParams } = this.state;
    this.setState({ campusDict: [] });
    if (branchId) {
      this.setState({ campusDict: this.campusCache[branchId] });
    }
    this.setState({ searchParams: { ...searchParams, branchId } });
  }

  render() {
    const { route } = this.props;
    const { loading, searchParams, pagination, dataList, campusDict, branchDict } = this.state;

    return (
      <Page
        name={title}
        route={route}
        fitWindowHeight
        loadingTip="正在加载数据"
        loading={loading}
        headerToolBar={<Button onClick={this.exportData}>导出excel</Button>}
        subHeader={
          <SearchFilter truncationNum={4} onSearch={this.handleSearch} onReset={this.handleReset}>
            <SearchField style={{ flex: '0 0 20%' }}>
              <RangePicker
                value={searchParams.time}
                format="YYYY-MM-DD"
                placeholder={['开始时间', '结束时间']}
                size="large"
                allowClear={false}
                onChange={this.onTimeChange}
              />
            </SearchField>
            <SearchField>
              <Select
                value={searchParams.branchId}
                placeholder="分公司"
                allowClear
                showSearch
                dict={branchDict}
                onChange={this.onBranchChange}
              />
            </SearchField>
            <SearchField>
              <MultiSelect
                value={searchParams.campusIds}
                placeholder="学习中心"
                allowClear
                showSearch
                dict={campusDict}
                onChange={this.onCampusChange}
              />
            </SearchField>
            <SearchField>
              <Input
                value={searchParams.studentName}
                placeholder="学生姓名"
                allowClear
                maxLength={50}
                onChange={this.onStudentNameChange}
              />
            </SearchField>
          </SearchFilter>
        }
      >
        <List dataSource={dataList} columns={Columns} pagination={pagination} onChange={this.onPageChange} />
      </Page>
    );
  }
}
