import React from 'react';
import FormItem from 'COMPONENT/FormItem';
import List from 'COMPONENT/List';
import { Row, Col } from 'antd';
import loaders from '../loaders';
import styles from './styles.less';
import { loadCommonDict } from 'UTIL/dict';
import Loading from 'COMPONENT/Loading';
import 'COMPONENT/Page/styles.less';

const dictToObjMapper = (dict) => {
  return dict.reduce((obj, ele) => {
    obj[ele.val] = ele.text;
    return obj;
  }, {});
};

const baseInfoCn = {
  studentName: '学生姓名',
  studentContact: '学生电话',
  customerName: '客户姓名',
  customerContact: '客户电话',
};
const courseInfoCn = {
  oneOnOneStatusName: '一对一状态',
  smallClassStatusName: '小班状态',
  oneOnManyStatusName: '一对多状态',
  statusName: '状态',
  blCampusName: '学籍学习中心',
  gradeName: '年级',
  enrolDate: '入学日期',
  studyManegerName: '学管师',
  icCardNo: 'IC卡编号',
  homeroomTeacherName: '学生班主任',
};
const familyInfoCn = {
  sex: '性别',
  birthday: '出生日期',
  fatherName: '父亲姓名',
  fatherPhone: '父亲电话',
  motherName: '母亲姓名',
  motherPhone: '母亲电话',
  address: '联系地址',
};

const renderItems = (itemsCn, items) => {
  const itemList = Object.keys(itemsCn);
  return itemList.map((ele) => {
    return (
      <Col key={ele} span={12} className={styles.customFormItem}>
        <FormItem name={itemsCn[ele] || '-'}>{items[ele] || '-'}</FormItem>
      </Col>
    );
  });
};

class StuInfoBlock extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: true,
      baseInfo: {},
      courseInfo: {},
      familyInfo: {},
      schoolInfo: [],
    };
  }

  componentDidMount() {
    this.setState({ loading: true });
    const { stuId } = this.props;
    if (!stuId) {
      return;
    }
    Promise.all([
      loaders.getStudentById(stuId),
      loadCommonDict('SCHOOL_LEVEL'),
      loadCommonDict('StudentSchoolAttendState'),
    ]).then(([info, schoolLevelDict, schoolAttendStateDict]) => {
      const schoolLevelObj = dictToObjMapper(schoolLevelDict);
      const schoolAttendStateObj = dictToObjMapper(schoolAttendStateDict);

      const baseInfo = {
        studentName: info.name || '-',
        studentContact: info.contact || '-',
        customerName: info.latestCustomerName || '-',
        customerContact: info.latestCustomerContact || '-',
      };
      const courseInfo = {
        oneOnOneStatusName: info.oneOnOneStatusName || '-',
        smallClassStatusName: info.smallClassStatusName || '-',
        oneOnManyStatusName: info.oneOnManyStatusName || '-',
        statusName: info.statusName || '-',
        gradeName: info.gradeName || '-',
        enrolDate: info.enrolDate || '-',
        studyManegerName: info.studyManegerName || '-',
        blCampusName: info.blCampusName || '-',
        icCardNo: info.icCardNo || '-',
        homeroomTeacherName: info.homeroomTeacherName || '-',
      };
      const familyInfo = {
        sex: info.sex === undefined || info.sex === null ? '-' : info.sex == 1 ? '男' : '女',
        birthday: info.bothday || '-',
        fatherName: info.fatherName || '-',
        fatherPhone: info.fatherPhone || '-',
        motherName: info.motherName || '-',
        motherPhone: info.motherPhone || '-',
        address: info.address || '-',
      };
      const schoolInfo = (info.attendSchools || []).map((ele) => {
        return {
          attendState: schoolAttendStateObj[ele.attendState] || '-',
          schoolType: schoolLevelObj[ele.schoolType] || '-',
          schoolName: ele.schoolName || '-',
        };
      });
      this.setState({
        loading: false,
        baseInfo,
        courseInfo,
        familyInfo,
        schoolInfo,
      });
    });
  }

  render() {
    const { loading, baseInfo, courseInfo, familyInfo, schoolInfo } = this.state;
    return (
      <div className={styles.wrap}>
        <Loading loading={loading} />
        <Row key="baseInfo">
          <Col span={24}>
            <h3 className={styles.header}>基础信息</h3>
          </Col>
          <Col span={12} className={styles.items}>
            {renderItems(baseInfoCn, baseInfo)}
          </Col>
        </Row>
        <Row key="courseInfo">
          <Col span={24}>
            <h3 className={styles.header}>课程相关</h3>
          </Col>
          <Col span={12} className={styles.items}>
            {renderItems(courseInfoCn, courseInfo)}
          </Col>
        </Row>
        <Row key="familyInfo">
          <Col span={24}>
            <h3 className={styles.header}>家庭信息</h3>
          </Col>
          <Col span={12} className={styles.items}>
            {renderItems(familyInfoCn, familyInfo)}
          </Col>
        </Row>
        <Row key="schoolInfo">
          <Col span={24}>
            <h3 className={styles.header}>就读学校</h3>
          </Col>
          <Col span={12} className={styles.items}>
            {!schoolInfo.length ? (
              <Col span={24} style={{ textAlign: 'center', color: '#999999' }}>
                空
              </Col>
            ) : (
              schoolInfo.map((ele) => {
                return (
                  <Col key={ele.schoolName} span={12} className={styles.customFormItem}>
                    <FormItem name="学校">{`${ele.schoolType}/${ele.attendState}/${ele.schoolName}`}</FormItem>
                  </Col>
                );
              })
            )}
          </Col>
        </Row>
      </div>
    );
  }
}
export default StuInfoBlock;
