import React from 'react';
import Modal from 'COMPONENT/Modal';
import styles from './styles.less';
import Select from 'COMPONENT/Select';
import FormItem from 'COMPONENT/FormItem';
import classnames from 'classnames';
import loaders from './loaders';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { loadSourceDetailDict } from 'PAGE/common/loaders';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadProvinceCityDict, getCityDictByProvince } from 'UTIL/dict';
import { Row, Col } from 'antd';
import { getDictTextByVal } from '../../../utils/dict/index';

const formCfg = {
  labelWidth: 80,
  style: {},
};

class SelectSchoolModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 等待状态

      province: props.defaultProvince || null, // {string} 省份
      city: props.defaultCity || null, // {string} 城市
      school: null, // {string} 学校
      schoolName: null, // {string} 学校名称

      provinceGroupDict: [], // {object[]} 字典，按省份分组的城市字典
      provinceDict: [], // {object[]} 字典，省份
      cityDict: [], // {object[]} 字典，城市
      schoolDict: [], // {object[]} 字典，学校

      checking: false, // {boolean} 校验开启状态
    };
    this.errors = {};
  }

  componentDidMount() {
    this.setState({ loading: true });
    let calls = [loadProvinceCityDict(true)];
    if (this.props.defaultCity) {
      calls.push(loaders.loadSchoolDict(this.props.defaultCity));
    }
    Promise.all([loadProvinceCityDict(true), loaders.loadSchoolDict(this.props.defaultCity)]).then(
      ([provinceGroupDict, schoolDict]) => {
        const provinceDict = provinceGroupDict.map((d) => {
          return { val: d.val, text: d.label };
        });
        this.setState({
          loading: false,
          provinceGroupDict,
          provinceDict,
          cityDict: getCityDictByProvince(provinceGroupDict, this.props.defaultProvince || null),
          schoolDict,
        });
      },
    );
  }

  onOk = () => {
    const { onConfirm } = this.props;
    const { checking, school, schoolName } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    const errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      this.modal.showOkTips(errorTips);
      return;
    }
    onConfirm(school, schoolName);
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  provinceChange = (val) => {
    const { provinceGroupDict } = this.state;
    this.setState({ province: val, city: null, cityDict: getCityDictByProvince(provinceGroupDict, val) });
  };

  cityChange = (val) => {
    this.setState({ city: val, school: null, schoolName: null });
    if (val) {
      this.setState({ loading: true });
      loaders.loadSchoolDict(val).then((schoolDict) => {
        this.setState({ loading: false, schoolDict });
      });
    } else {
      this.setState({ schoolDict: [] });
    }
  };

  render() {
    const { onClose } = this.props;
    const { loading, province, city, school, schoolName, provinceDict, cityDict, schoolDict, checking } = this.state;
    const modalCls = {
      [styles.wrap]: 1,
      [styles.modal]: 1,
    };
    return (
      <Modal
        className={classnames(modalCls)}
        loading={loading}
        title="选择学校"
        width={514}
        onOk={this.onOk}
        onClose={onClose}
        noLine
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        <div className={styles.main}>
          <Row>
            <Col span={12}>
              <FormItem {...formCfg} name="省份" required>
                <Select
                  showSearch
                  dict={provinceDict}
                  value={province}
                  onChange={this.provinceChange}
                  validator={{
                    rules: ['required'],
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('province', e),
                  }}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...formCfg} name="城市" required>
                <Select
                  showSearch
                  dict={cityDict}
                  value={city}
                  onChange={this.cityChange}
                  validator={{
                    rules: ['required'],
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('city', e),
                  }}
                />
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={24}>
              <FormItem {...formCfg} name="学校" required>
                <Select
                  showSearch
                  dict={schoolDict}
                  value={school}
                  onChange={(val) => this.setState({ school: val, schoolName: getDictTextByVal(schoolDict, val) })}
                  validator={{
                    rules: ['required'],
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('school', e),
                  }}
                />
              </FormItem>
            </Col>
          </Row>
        </div>
      </Modal>
    );
  }
}

export default SelectSchoolModal;
