import React from 'react';
import PropTypes from 'prop-types';
import style from './styles.module.less';
import { List } from 'COMPONENT';
import { errorsHandle, typeOf } from 'UTIL/common';
import { callFlagCfg, callDirectionCfg, callEndCfg } from 'CONST/callConfig';
import { durationToTime } from '../utils';
import { getCallRecords } from 'SERVICE/callCenter';
import moment from 'moment';

const setWidth = List.createSetWidth();

const momentFormat = 'YYYY-MM-DD HH:mm:ss';

const defaultText = '--';

class CallRecord extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.loadList = this.loadList.bind(this);
    this.search = this.search.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);

    this.state = {
      listLoading: false,
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: props.pagination ? props.defaultPageSize : 999,
    };

    this.columns = [
      { title: '通话ID', dataIndex: 'callId', key: 'callId', width: setWidth(6) },
      { title: '通话员工', dataIndex: 'userName', key: 'userName', width: setWidth(4) },
      {
        title: '呼叫时间',
        dataIndex: 'createTime',
        key: 'createTime',
        width: setWidth(5),
        render: (text) => moment(text).format(momentFormat),
      },
      {
        title: '客户姓名',
        dataIndex: 'customerName',
        key: 'customerName',
        width: setWidth(4),
        render: (text) => text || defaultText,
      },
      {
        title: '学生姓名',
        dataIndex: 'studentName',
        key: 'studentName',
        width: setWidth(4),
        render: (text) => text || defaultText,
      },
      { title: '学生年级', dataIndex: 'gradeName', key: 'gradeName', width: setWidth(4) },
      { title: '主叫号码', dataIndex: 'caller', key: 'caller', width: setWidth(6) },
      { title: '被叫号码', dataIndex: 'callee', key: 'callee', width: setWidth(6) },
      {
        title: '接听状态',
        dataIndex: 'callFlag',
        key: 'callFlag',
        width: setWidth(4),
        render: (text) => callFlagCfg[text],
      },
      {
        title: '通话时长',
        dataIndex: 'duration',
        key: 'duration',
        width: setWidth(4),
        render: (text) => durationToTime(text),
      },
      {
        title: '呼叫类型',
        dataIndex: 'callDirection',
        key: 'callDirection',
        width: setWidth(4),
        render: (text) => callDirectionCfg[text],
      },
      {
        title: '挂断方',
        dataIndex: 'ender',
        key: 'ender',
        width: setWidth(4),
        render: (text) => callEndCfg[text],
      },
    ];
    if (props.audioAuth) {
      this.columns.push({
        title: '通话录音',
        dataIndex: 'bossRecordUrl',
        key: 'bossRecordUrl',
        width: setWidth(4),
        render: (text, record) => {
          const source = text || record.recordUrl;
          return source ? (
            <div className={style.audioWrapper}>
              <audio controls controlsList="nodownload">
                <source src={source} type="audio/wav" />
              </audio>
            </div>
          ) : (
            defaultText
          );
        },
      });
    }
  }

  static propTypes = {
    searchKeys: PropTypes.object,
    loading: PropTypes.bool,
    audioAuth: PropTypes.bool,
    pagination: PropTypes.bool,
    defaultPageSize: PropTypes.oneOf([10, 20, 50]),
  };

  static defaultProps = {
    loading: false,
    audioAuth: false,
    pagination: false,
    defaultPageSize: 10,
  };

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ listLoading: true });
    await new Promise(this.loadList);
    this.setState({ listLoading: false });
  }

  async loadList(resolve) {
    let apiParams = {
      pageNum: this.state.pageCurrent,
      pageSize: this.state.pageSize,
    };
    if (typeOf(this.props.searchKeys) === 'object') {
      apiParams = {
        ...apiParams,
        ...this.props.searchKeys,
      };
    }
    const rsp = await getCallRecords(apiParams).catch(errorsHandle());
    if (rsp && rsp.resultCode === 0 && rsp.data) {
      this.setState({
        list: rsp.data.list || [],
        total: rsp.data.total,
      });
    }
    resolve();
  }

  search() {
    this.changePage({
      current: 1,
      pageSize: this.state.pageSize,
    });
  }

  updatePage(pageCurrent, pageSize) {
    return new Promise((resolve) => {
      this.setState(
        (prevState) => {
          let payload = {};
          if (pageSize && pageSize !== prevState.pageSize) {
            payload.pageCurrent = 1;
            payload.pageSize = pageSize;
          } else {
            payload.pageCurrent = pageCurrent;
          }
          return payload;
        },
        () => {
          resolve();
        },
      );
    });
  }

  async changePage({ current, pageSize }) {
    await this.updatePage(current, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  render() {
    return (
      <List
        loading={this.props.loading || this.state.listLoading}
        columns={this.columns}
        dataSource={this.state.list}
        pagination={
          this.props.pagination
            ? {
                total: this.state.total,
                current: this.state.pageCurrent,
                pageSize: this.state.pageSize,
              }
            : null
        }
        onChange={this.props.pagination ? this.changePage : null}
      />
    );
  }
}

export default CallRecord;
