import React from 'react';
import { connect } from 'dva';
import styles from './index.less';
import { Page } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import ChooseStudent from '../../component/ChooseStudent';
import ScoreInfo from '../../component/ScoreInfo';
import PaperInfo from '../../component/PaperInfo';
import { getPaperQuestionList, addPaperQuestionList } from 'SERVICE/entranceExam';
import { formatQuestionList, formatApiQuestionList } from '../../component/ScoreInfo/util';
import { formatApiPictureList } from '../../component/PaperInfo/util';
import { runFunc } from 'UTIL/common';

/**
 * 录入分数
 *
 * @param props.paperId 试卷ID，从父页面传入
 * @param props.pageTitle 试卷名称，从父页面传入
 * @param props.branchId 分公司ID，从父页面传入
 * @param props.onSaveSuccess 保存成功回调，从父页面传入
 */
class AddPage extends React.Component {
  static defaultProps = {
    paperId: null,
    pageTitle: null,
    branchId: null,
    onSaveSuccess: null,
  };

  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      studentInfo: null, // 学生
      questionList: null, // 答题
      fileList: null, // 答卷
    };
  }

  componentWillMount() {
    this.initContent();
  }

  /**
   * 录入分数默认满分
   */
  initContent = () => {
    const { paperId } = this.props;

    this.toggleLoading();
    getPaperQuestionList({ paperId })
      .then((response) => {
        const data = response.data || {};
        const questionList = formatQuestionList(data.questions, true);

        this.toggleLoading();
        this.setState({ questionList, fileList: [] });
      })
      .catch((e) => {
        this.toggleLoading();
        this.setState({ questionList: [], fileList: [] });
      });
  };

  /**
   * 选中学生
   *
   * @param student 学生信息
   */
  onStudentSelect = (student) => {
    this.setState({ studentInfo: student });
  };

  /**
   * 修改分数
   *
   * @param questionList 题目列表
   */
  onScoreChange = (questionList) => {
    this.setState({ questionList });
  };

  /**
   * 选择文件
   *
   * @param fileList 文件列表
   */
  onPaperChange = (fileList) => {
    this.setState({ fileList });
  };

  /**
   * 新增
   */
  handleOk = () => {
    const { studentInfo, questionList, fileList } = this.state;

    if (!studentInfo) {
      showMessage('请选择学生！', 'error');
      return;
    }

    if (!questionList) {
      showMessage('请填写得分！', 'error');
      return;
    }

    if (!fileList || fileList.length === 0) {
      showMessage('请上传答卷！', 'error');
      return;
    }

    showConfirm('该操作将会新增一份报告，是否继续？', () => {
      this.doSubmit();
    });
  };

  /**
   * 提交
   */
  doSubmit = () => {
    const { paperId, branchId, onSaveSuccess } = this.props;
    const { studentInfo, questionList, fileList } = this.state;
    const params = {
      paperId,
      studentId: studentInfo.studentId,
      studentName: studentInfo.studentName,
      branchId,
      questions: formatApiQuestionList(questionList),
      picList: formatApiPictureList(fileList),
    };

    this.toggleLoading();
    addPaperQuestionList(params)
      .then((res) => {
        this.toggleLoading();
        showMessage('录入成功！', 'success');

        runFunc(onSaveSuccess, res.data);
      })
      .catch((e) => {
        this.toggleLoading();
      });
  };

  toggleLoading = () => {
    this.setState((prevState) => {
      return {
        loading: !prevState.loading,
      };
    });
  };

  render() {
    const { questionList, fileList } = this.state;

    return (
      <Page
        name={this.props.pageTitle}
        footer={true}
        loading={this.state.loading}
        route={this.props.route}
        onOk={this.handleOk}
      >
        <section style={{ marginBottom: 50 }}>
          <h5 className={styles.blockTitle}>学生信息</h5>
          <ChooseStudent onSelect={this.onStudentSelect} />
        </section>

        <section style={{ marginBottom: 30 }}>
          <h5 className={styles.blockTitle}>答题信息</h5>
          {questionList && <ScoreInfo questionList={questionList} onChange={this.onScoreChange} />}
        </section>

        <section>
          <h5 className={styles.blockTitle}>学生答卷</h5>
          <PaperInfo fileList={fileList} onChange={this.onPaperChange} />
        </section>
      </Page>
    );
  }
}

const mapStateTopProps = (state) => {
  const pageParams = state.main.pageParams || {};
  const data = pageParams['/EntranceTestPaperManage/Add'] || {};
  const params = {
    paperId: data.paperId,
    pageTitle: data.title || '录入分数',
    branchId: data.branchId,
    onSaveSuccess: data.onSaveSuccess,
  };

  return {
    ...params,
  };
};

export default connect(mapStateTopProps)(AddPage);
