import React from 'react';
import { connect } from 'dva';
import styles from './index.less';
import { Page } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import StudentInfo from '../../component/StudentInfo';
import ScoreInfo from '../../component/ScoreInfo';
import PaperInfo from '../../component/PaperInfo';
import { getStudentPaperQuestion, editStudentPaperQuestion } from 'SERVICE/entranceExam';
import { calcHasChangeList, cloneJsonArray } from './util';
import { formatQuestionList, formatApiQuestionList } from '../../component/ScoreInfo/util';
import { formatFileList, formatApiPictureList } from '../../component/PaperInfo/util';
import { formatStudentInfo } from '../../component/StudentInfo/util';

/**
 * 修改分数
 *
 * @param props.examId 考试ID，从父页面传入(考试ID !== 试卷ID)
 * @param props.pageTitle 页面title，从父页面传入
 * @param props.studentId 学生ID，从父页面传入
 * @param props.branchId 分公司ID，从父页面传入
 * @param props.onSaveSuccess 保存成功回调，从父页面传入
 */
class EditPage extends React.Component {
  static defaultProps = {
    examId: null,
    pageTitle: null,
    studentId: null,
    branchId: null,
    onSaveSuccess: null,
  };

  constructor(props) {
    super(props);

    this.state = {
      paperId: null, // 试卷ID, 从详情接口获取

      originStudentInfo: null, // 原始学生信息
      originQuestionList: null, // 原始答题信息
      originFileList: null, // 原始答卷信息

      changeQuestionList: null, // 修改的答题
      changeFileList: null, // 修改的答卷
    };
  }

  componentWillMount() {
    this.initContent();
  }

  initContent = () => {
    this.loadPaperQuestion();
  };

  /**
   * 获取学生答卷信息
   */
  loadPaperQuestion = () => {
    const { examId, studentId } = this.props;

    this.toggleLoading();
    getStudentPaperQuestion({ examId, studentId })
      .then((response) => {
        const data = response.data || {};
        const { paperId, student, questions, picList } = data;

        const studentInfo = formatStudentInfo(student);
        const questionList = formatQuestionList(questions, false);
        const fileList = formatFileList(picList);

        this.toggleLoading();
        this.setState({
          paperId,
          originStudentInfo: studentInfo,
          originQuestionList: questionList,
          changeQuestionList: cloneJsonArray(questionList),
          originFileList: fileList,
          changeFileList: cloneJsonArray(fileList),
        });
      })
      .catch((e) => {
        this.toggleLoading();
        this.setState({
          paperId: null,
          originStudentInfo: null,
          originQuestionList: [],
          changeQuestionList: [],
          originFileList: [],
          changeFileList: [],
        });
      });
  };

  /**
   * 修改分数
   *
   * @param questionList 题目列表
   */
  onScoreChange = (questionList) => {
    this.setState({ changeQuestionList: questionList });
  };

  /**
   * 选择文件
   *
   * @param fileList 文件列表
   */
  onPaperChange = (fileList) => {
    this.setState({ changeFileList: fileList });
  };

  /**
   * 修改分数，教学接口要求添加一个是否修改信息的标识字段
   */
  handleOk = () => {
    const { originQuestionList, originFileList, changeQuestionList, changeFileList } = this.state;

    if (!changeQuestionList) {
      showMessage('请填写得分！', 'error');
      return;
    }

    if (!changeFileList || changeFileList.length === 0) {
      showMessage('请上传答卷！', 'error');
      return;
    }

    const hasChangeScore = calcHasChangeList(originQuestionList, changeQuestionList, 'questionId', 'score');
    const hasChangeFile = calcHasChangeList(originFileList, changeFileList, 'fileId', 'src');
    if (!hasChangeScore && !hasChangeFile) {
      showMessage('请先修改信息！', 'error');
      return;
    }

    showConfirm('该操作将会新增一份报告，是否继续？', () => {
      this.doSubmit();
    });
  };

  /**
   * 提交
   */
  doSubmit = () => {
    const { examId, studentId, branchId, onSaveSuccess } = this.props;
    const { paperId, originStudentInfo, changeQuestionList, changeFileList } = this.state;
    const params = {
      examId,
      paperId,
      studentId,
      studentName: originStudentInfo.studentName,
      branchId,
      questions: formatApiQuestionList(changeQuestionList),
      picList: formatApiPictureList(changeFileList),
    };

    this.toggleLoading();
    editStudentPaperQuestion(params)
      .then((result) => {
        this.toggleLoading();
        showMessage('修改成功！', 'success');

        if (onSaveSuccess) {
          onSaveSuccess();
        }
      })
      .catch((e) => {
        this.toggleLoading();
      });
  };

  toggleLoading = () => {
    this.setState((prevState) => {
      return {
        loading: !prevState.loading,
      };
    });
  };

  render() {
    const { originStudentInfo, originQuestionList, originFileList } = this.state;

    return (
      <Page
        name={this.props.pageTitle}
        footer={true}
        loading={this.state.loading}
        route={this.props.route}
        onOk={this.handleOk}
      >
        <section style={{ marginBottom: 50 }}>
          <h5 className={styles.blockTitle}>学生信息</h5>
          {originStudentInfo && <StudentInfo student={originStudentInfo} />}
        </section>

        <section style={{ marginBottom: 30 }}>
          <h5 className={styles.blockTitle}>答题信息</h5>
          {originQuestionList && <ScoreInfo questionList={originQuestionList} onChange={this.onScoreChange} />}
        </section>

        <section>
          <h5 className={styles.blockTitle}>学生答卷</h5>
          {originFileList && <PaperInfo fileList={originFileList} onChange={this.onPaperChange} />}
        </section>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const pageParams = state.main.pageParams || {};
  const data = pageParams['/EntranceTestReportManage/Edit'] || {};
  const params = {
    examId: data.examId,
    pageTitle: data.title || '修改分数',
    studentId: data.studentId,
    branchId: data.branchId,
    onSaveSuccess: data.onSaveSuccess,
  };

  return {
    ...params,
  };
};

export default connect(mapStateToProps)(EditPage);
