import React from 'react';
import { connect } from 'dva';
import styles from './index.less';
import { Row, Col, Radio } from 'antd';
import { Button, FormItem, Input } from 'COMPONENT';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import ModalConsultantAddStudent from 'PAGE/common/AddCustomerMain/Modal';
import ModalMasterAddStudent from 'PAGE/customer/AddTransferCustomer/Modal';
import { WORKBENCH_RECEPTION } from 'PAGE/common/utils';
import { getStudentListByCusContact } from 'SERVICE/consultant';
import { formatStudentStatusName } from './util';

const RadioGroup = Radio.Group;

/**
 * 学生信息
 *
 * @param props.onSelect {function} 选择学生回调，回调参数为学生信息
 */
class ChooseStudent extends React.Component {
  static defaultProps = {
    onSelect: (student) => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      checkSilent: true, // 关闭表单校验
      errors: {}, // 表单校验错误信息
      loading: false,
      phoneNumber: null, // 电话号码
      studentId: null, // 选中的学生ID
      studentList: [], // 学生列表
      modalAddStudentShow: false, // 显示录入学生弹窗
    };
  }

  /**
   * 查询学生
   */
  onSearch = () => {
    const contact = this.state.phoneNumber;

    if (!contact) {
      return;
    }

    if (this.state.checkSilent) {
      this.setState({ checkSilent: false });
    }

    const errorTips = getCommonErrorTips(this.state.errors);
    if (errorTips) {
      return;
    }

    this.toggleLoading();
    getStudentListByCusContact({ contact: contact.trim() })
      .then((result) => {
        const data = (result && result.data) || {};
        const { customerId, studentList } = data;

        this.toggleLoading();
        this.setState({
          studentId: null,
          studentList,
        });
        this.props.onSelect(null);
      })
      .catch((e) => {
        this.toggleLoading();
        this.setState({
          studentId: null,
          studentList: [],
        });
        this.props.onSelect(null);
      });
  };

  /**
   * 手机号输入
   *
   * @param e
   */
  onPhoneNumberChange = (e) => {
    const val = e.target.value;
    const checkSilent = !val;

    this.setState({ phoneNumber: val, checkSilent });
  };

  /**
   * 表单错误校验
   *
   * @param key
   * @param e
   */
  errorChange = (key, e) => {
    this.setState((prevState) => {
      const errors = {
        ...prevState.errors,
        [key]: e,
      };

      return {
        errors: { ...errors },
      };
    });
  };

  /**
   * 选择学生
   * tips: 这里会触发到click事件
   *
   * @param e
   */
  onRadioChange = (e) => {
    this.setState({ studentId: e.target.value });
  };

  /**
   * 学生列表行点击
   *
   * @param student
   */
  onRowCLick = (student) => {
    this.setState({ studentId: student.studentId });
    this.props.onSelect(student);
  };

  /**
   * 新增学生成功之后回调
   *
   * @param student {object?} 学生信息
   */
  onAddStudentComplete = (student) => {
    this.onSearch();
  };

  /**
   * 显示/关闭弹窗
   */
  toggleAddModal = () => {
    this.setState((prevState) => {
      return {
        modalAddStudentShow: !prevState.modalAddStudentShow,
      };
    });
  };

  toggleLoading = () => {
    this.setState((prevState) => {
      return {
        loading: !prevState.loading,
      };
    });
  };

  render() {
    const { userId } = this.props;
    const { phoneNumber, studentList } = this.state;
    const modalConsultantCusConf = {
      phone: {
        initialValue: phoneNumber,
        disabled: true,
      },
      follower: {
        initialValue: userId,
      },
    };
    const modalMasterConfig = {
      contact: {
        initialValue: phoneNumber,
        disabled: true,
      },
    };
    const errorTips = getCommonErrorTips(this.state.errors);
    const phoneAble = phoneNumber && !errorTips;

    return (
      <section>
        <Row style={{ marginBottom: 12 }}>
          <Col span={3} style={{ marginRight: 10 }}>
            <FormItem name="" labelWidth={0} style={{ padding: 0, marginBottom: 0 }}>
              <Input
                placeholder="手机号"
                value={phoneNumber}
                onChange={this.onPhoneNumberChange}
                validator={{
                  rules: ['phone'],
                  silent: this.state.checkSilent,
                  onErrorChange: (e) => this.errorChange('phoneNumber', e),
                }}
                allowClear={true}
                className={styles.phoneInput}
              />
            </FormItem>
          </Col>
          <Col span={1} style={{ minWidth: 90 }}>
            <Button type="primary" disabled={!phoneAble || this.state.loading} onClick={this.onSearch}>
              查询
            </Button>
          </Col>
          <Col span={2}>
            <Button type="primary" disabled={!phoneAble} onClick={this.toggleAddModal}>
              添加学生
            </Button>
          </Col>
        </Row>

        {studentList && studentList.length > 0 && (
          <RadioGroup value={this.state.studentId} onChange={this.onRadioChange} className={styles.radioGroup}>
            <table className={styles.table}>
              <thead>
                <tr>
                  <td className={styles.tdRadio}>&nbsp;</td>
                  <td>姓名</td>
                  <td>年级</td>
                  <td>学校</td>
                  <td>备注</td>
                </tr>
              </thead>
              <tbody>
                {studentList.map((student, index) => {
                  const { studentId, studentName, gradeName, schoolName, status } = student;
                  const statusName = formatStudentStatusName(status);

                  return (
                    <tr
                      key={index}
                      onClick={(e) => {
                        this.onRowCLick(student);
                      }}
                    >
                      <td className={styles.tdRadio}>
                        <Radio value={studentId} />
                      </td>
                      <td>{studentName}</td>
                      <td>{gradeName}</td>
                      <td>{schoolName}</td>
                      <td>{statusName && <span className={styles.statusName}>{statusName}</span>}</td>
                    </tr>
                  );
                })}
              </tbody>
            </table>
          </RadioGroup>
        )}

        {this.state.modalAddStudentShow ? (
          this.props.isConsultant ? (
            <ModalConsultantAddStudent
              title="新增学生"
              okText="保存"
              cancelBtnShow={true}
              workBench={WORKBENCH_RECEPTION}
              callType="FRONT_DESK"
              customerFormItemConf={modalConsultantCusConf}
              studentFormItemConf={null}
              btnAddStudentVisible={false}
              onCancel={this.toggleAddModal}
              afterConfirm={this.onAddStudentComplete}
            />
          ) : (
            <ModalMasterAddStudent
              formItemCfg={modalMasterConfig}
              onCancel={this.toggleAddModal}
              afterConfirm={this.onAddStudentComplete}
            />
          )
        ) : (
          ''
        )}
      </section>
    );
  }
}

const mapStateToProps = (state) => {
  const { user } = state.account;
  const { roleCode, userId } = user || {};

  // 咨询师(学习规划师)
  const isConsultant = roleCode.indexOf('CONSULTOR') > -1 || roleCode.indexOf('CONSULTOR_DIRECTOR') > -1;
  // 班主任/学管师
  const isMaster = roleCode.indexOf('STUDY_MANAGER') > -1 || roleCode.indexOf('STUDY_MANAGER_HEAD') > -1;

  return {
    isConsultant,
    isMaster,
    userId,
  };
};

export default connect(mapStateToProps)(ChooseStudent);
