import React from 'react';
import styles from './index.less';
import { InputNumber, Icon } from 'COMPONENT';
import { calcTotal, cloneJsonArray } from './util';

/**
 * 答题信息
 *
 * @param props.editable {boolean} 是否允许编，默认true
 * @param props.questionList {array} 原始题目列表 [{questionId: ID, questionTitle: '', score: 得分, totalScore: 总分}]
 * @param props.onChange {function} 分数修改回调方法，回调参数为分数信息
 */
class ScoreInfo extends React.Component {
  static defaultProps = {
    editable: true,
    questionList: null,
    onChange: (questionList) => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      questionList: [],
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化
   */
  initContent = () => {
    this.setState({ questionList: cloneJsonArray(this.props.questionList) });
  };

  /**
   * 点击icon自动填充分数
   *
   * @param iconType {number} {1: 完全正确, 2: 部分正确, 3: 错误}
   * @param index {number} 题目索引
   * @param maxScore {number} 题目最大可填分数
   */
  onIconClick = (iconType, index, maxScore) => {
    if (!this.props.editable) {
      return;
    }

    let val = 0;

    switch (iconType) {
      case 1:
        val = maxScore;
        break;
      case 2:
        val = Math.ceil(maxScore / 2);
        break;
      case 3:
        val = 0;
        break;
      default:
        break;
    }

    this.setScore(val, index);
  };

  /**
   * 分数修改
   *
   * @param val {number|string} 分数，注意：当清空input时，组件返回的值为null
   * @param index {number} 题目索引
   */
  onScoreChange = (val, index) => {
    this.setScore(val, index);
  };

  /**
   * 填充分数
   *
   * @param val {number} 值
   * @param index {number} 题目索引
   */
  setScore = (val, index) => {
    let list = [...this.state.questionList];
    list[index].score = val || 0;

    this.setState({ questionList: list });
    this.props.onChange(list);
  };

  render() {
    const { editable } = this.props;
    const questionList = this.state.questionList || [];
    const inputTotal = calcTotal(questionList, 'score', 10);
    const totalTotal = calcTotal(questionList, 'totalScore', 10);

    return (
      <table className={styles.table}>
        <thead>
          <tr>
            <td>题目</td>
            <td>完全正确</td>
            <td>部分正确</td>
            <td>错误</td>
            <td>得分/满分</td>
            <td>&nbsp;</td>
          </tr>
        </thead>
        <tbody>
          {questionList.map((item, index) => {
            const { questionTitle } = item;
            const score = item.score || 0;
            const maxScore = item.totalScore || 0;

            const fullActive = score === maxScore;
            const partActive = score > 0 && score < maxScore;
            const errorActive = score === 0;

            return (
              <tr key={index}>
                <td className={styles.tdTitle}>{questionTitle}</td>
                <td>
                  <Icon
                    type={fullActive ? 'check_circle_outlined_h' : 'check_circle_outlined'}
                    size={24}
                    onClick={(e) => {
                      this.onIconClick(1, index, maxScore);
                    }}
                    className={styles.icon}
                    data-icon-type="full"
                  />
                </td>
                <td>
                  <Icon
                    type={partActive ? 'check_circle_outlined_point_h' : 'check_circle_outlined_point'}
                    size={24}
                    onClick={(e) => {
                      this.onIconClick(2, index, maxScore);
                    }}
                    className={styles.icon}
                    data-icon-type="part"
                  />
                </td>
                <td>
                  <Icon
                    type={errorActive ? 'close_circle_outlined_h' : 'close_circle_outlined'}
                    size={24}
                    onClick={(e) => {
                      this.onIconClick(3, index, maxScore);
                    }}
                    className={styles.icon}
                    data-icon-type="error"
                  />
                </td>
                {editable
                  ? [
                      <td key={1} style={{ width: 120 }}>
                        <InputNumber
                          placeholder="请输入分数"
                          min={0}
                          max={maxScore}
                          precision={1}
                          value={score}
                          onChange={(val) => {
                            this.onScoreChange(val, index);
                          }}
                          style={{ width: '100%' }}
                        />
                      </td>,
                      <td key={2}>
                        <span className={styles.scoreText}>/{maxScore}</span>
                      </td>,
                    ]
                  : [
                      <td key={1} className={styles.scoreText} colSpan={2}>
                        {score}/{maxScore}
                      </td>,
                    ]}
              </tr>
            );
          })}
          <tr>
            <td colSpan={4} className={styles.tdTitle}>
              总分
            </td>
            {editable
              ? [
                  <td key={1} className={styles.scoreText} style={{ width: 120, textAlign: 'right' }}>
                    {inputTotal}
                  </td>,
                  <td key={2} className={styles.scoreText}>
                    /{totalTotal}
                  </td>,
                ]
              : [
                  <td key={1} className={styles.scoreText} colSpan={2}>
                    {inputTotal}/{totalTotal}
                  </td>,
                ]}
          </tr>
        </tbody>
      </table>
    );
  }
}

export default ScoreInfo;
