import React from 'react';
import styles from './styles.less';
import Loading from 'COMPONENT/Loading';
import dicts from '../dicts';
import DropdownMenu from 'COMPONENT/DropdownMenu';
import Icon from 'COMPONENT/Icon';
import Button from 'COMPONENT/Button';
import DatePicker from 'COMPONENT/DatePicker';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import TextLink from 'COMPONENT/TextLink';
import { loadCommonDict } from 'UTIL/dict';
import loaders from '../loaders';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const initListSearchKeys = {
  stuName: null,
  grade: null,
  school: null,
  dispatchDate: null,
  name: null,
  contract: null,
  entrance: null,
  status: null,
  dispatcher: null,
};
class Pool extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态

      stuName: null, // {string} 查询条件 学生姓名
      grade: null, // {string} 查询条件 学生年级
      school: null, // {string} 查询条件 所在学校
      dispatchDate: null, // {string} 查询条件 分配时间
      name: null, // {string} 查询条件 客户姓名
      contract: null, // {string} 查询条件 联系方式
      entrance: null, // {string} 查询条件 资源入口
      status: null, // {string} 查询条件 客户状态
      dispatcher: null, // {string} 查询条件 分配人

      gradeDict: [], // {[object]} 字典 年级
      entranceDict: dicts.entranceDict, // {[object]} 字典 资源入口
      statusDict: [], // {[object]} 字典 客户状态

      list: [], // {Array} 列表数据，[{ id, stuName, grade, schoolName, name, phone, entrance, status, remark, dispatchTime, dispatcher }]
      listSearchKeys: initListSearchKeys, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数

      selectedRowKeys: [], // {string} 选中的行
    };
  }

  initContent = async () => {
    this.setState({ loading: true });
    const [listData, gradeDict] = await Promise.all([
      loaders.loadPoolResList(this.getListParams()),
      loadCommonDict('STUDENT_GRADE'),
    ]);
    const { list, totalRecords } = listData;
    this.setState({ loading: false, gradeDict, list, totalRecords });
  };

  getListParams = () => {
    const {
      listSearchKeys: { stuName, grade, school, dispatchDate, name, contract, entrance, status, dispatcher },
      currentPage,
      pageSize,
    } = this.state;
    return {
      stuName,
      grade,
      school,
      dispatchDate,
      name,
      contract,
      entrance,
      status,
      dispatcher,
      currentPage,
      pageSize,
      poolId: this.props.poolId,
    };
  };

  search = () => {
    const { stuName, grade, school, dispatchDate, name, contract, entrance, status, dispatcher } = this.state;
    const listSearchKeys = { stuName, grade, school, dispatchDate, name, contract, entrance, status, dispatcher };
    this.setState({ loading: true });
    loaders
      .loadPoolResList({ ...this.getListParams(), ...listSearchKeys, currentPage: 1 })
      .then(({ list, totalRecords }) => {
        this.setState({ loading: false, listSearchKeys, currentPage: 1, list, totalRecords });
      });
  };

  reset = () => {
    this.setState(initListSearchKeys);
  };

  onPaginationChange = ({ current, pageSize }) => {
    this.setState({ loading: true });
    const changed = {
      currentPage: current,
      pageSize,
    };
    this.setState({ loading: true });
    loaders.loadPoolResList({ ...this.getListParams(), ...changed }).then(({ list, totalRecords }) => {
      this.setState({ loading: false, ...changed, list, totalRecords });
    });
  };

  goDetail = (id) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/CommonCusDetailPage',
        params: { cusId: id, onSaveSuccess: this.search },
        refresh: true,
        mirrorId: `ConsultantResourcePools/Pool/${this.props.poolId}/${id}`,
      },
    });
  };

  onRowSelectChange = (selectedRowKeys, selectedRows) => {
    this.setState({ selectedRowKeys });
  };

  render() {
    const {
      loading,
      searchExpand,
      stuName,
      grade,
      school,
      dispatchDate,
      name,
      contract,
      entrance,
      status,
      dispatcher,
      list,
      totalRecords,
      currentPage,
      pageSize,
      gradeDict,
      entranceDict,
      statusDict,
      selectedRowKeys,
    } = this.state;
    const columns = [
      { title: '学生姓名', dataIndex: 'stuName' },
      { title: '年级', dataIndex: 'grade', dict: gradeDict },
      { title: '学校', width: COL_WIDTH_L, dataIndex: 'schoolName' },
      { title: '客户姓名', dataIndex: 'name' },
      { title: '联系电话', width: COL_WIDTH_L, dataIndex: 'phone' },
      { title: '资源入口', dataIndex: 'entrance', dict: entranceDict },
      { title: '客户状态', dataIndex: 'status', dict: statusDict },
      { title: '备注', dataIndex: 'remark', width: COL_WIDTH_XL },
      {
        title: '最后分配时间',
        dataIndex: 'dispatchTime',
        render: (text) => {
          if (!text) {
            return null;
          }
          const a = text.split(' ');
          return (
            <span>
              {a[0]}
              <br />
              {a[1]}
            </span>
          );
        },
      },
      { title: '分配人', dataIndex: 'dispatcher' },
      {
        title: '操作',
        fixed: 'right',
        key: 'action',
        getActionConfig: (text, record, index) => {
          const { key } = record;
          return [
            { text: '详情', onClick: () => this.goDetail(key) },
            { text: '获取', onClick: null },
          ];
        },
      },
    ];
    return (
      <div className={styles.wrap}>
        <div className="flexWrap">
          <div className={styles.toolBar}>
            <div className={styles.toolBarLeft}>
              <SearchBar
                expand={searchExpand}
                onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
                onSearch={this.search}
                onReset={this.reset}
              >
                <Input
                  placeholder="学生姓名"
                  value={stuName}
                  onChange={(e) => this.setState({ stuName: e.target.value })}
                />
                <Select
                  placeholder="学生年级"
                  dict={gradeDict}
                  value={grade}
                  onChange={(val) => this.setState({ grade: val })}
                />
                <Input
                  placeholder="所在学校"
                  value={school}
                  onChange={(e) => this.setState({ school: e.target.value })}
                />
                <DatePicker
                  placeholder="分配时间"
                  value={dispatchDate}
                  onChange={(val) => this.setState({ dispatchDate: val })}
                />
                <Input placeholder="客户姓名" value={name} onChange={(e) => this.setState({ name: e.target.value })} />
                <Input
                  placeholder="联系方式"
                  value={contract}
                  onChange={(e) => this.setState({ contract: e.target.value })}
                />
                <Select
                  placeholder="资源入口"
                  dict={entranceDict}
                  value={entrance}
                  onChange={(val) => this.setState({ entrance: val })}
                />
                <Select
                  placeholder="客户状态"
                  dict={statusDict}
                  value={status}
                  onChange={(val) => this.setState({ status: val })}
                />
                <Input
                  placeholder="分配人"
                  value={dispatcher}
                  onChange={(e) => this.setState({ dispatcher: e.target.value })}
                />
              </SearchBar>
            </div>
            <div className={styles.toolBarRight}>
              <Button>批量获取</Button>
            </div>
          </div>
          <div className="flexInnerFit">
            <List
              dataSource={list}
              columns={columns}
              pagination={{
                current: currentPage,
                total: totalRecords,
                pageSize,
              }}
              onChange={this.onPaginationChange}
              rowSelection={{
                selectedRowKeys,
                onChange: this.onRowSelectChange,
              }}
            />
          </div>
        </div>
        <Loading loading={loading} />
      </div>
    );
  }
}
export default Pool;
