import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'dva';
import style from './styles.module.less';
import { Modal } from 'COMPONENT/ShoppingCart';
import Wrapper from 'COMPONENT/Wrapper';
import Deck from 'COMPONENT/Deck';
import NoDataPlus from 'COMPONENT/NoDataPlus';
import ToBeSelectedCard from './ToBeSelectedCard';
import SelectedCard from './SelectedCard';
import Search from './Search';
import { sKeyAdapter } from './utils';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { runFunc, errorsHandle } from 'UTIL/common';
import { getLinkClassList as queryClassList } from '../loaders';
import { TWO_TEACHER_VAL } from '../constants/config';

// 待选班render
function classCardRender({ onClick, isTwoTeacher }) {
  return (props, index) => {
    return <ToBeSelectedCard {...props} onSelect={onClick} index={index} key={index} isTwoTeacher={isTwoTeacher} />;
  };
}
// 已选班render
function selectedCardRender({ onDelete }) {
  return (props, index) => {
    return <SelectedCard {...props} onDelete={onDelete} index={index} key={index} />;
  };
}

const SearchResult = (props) => {
  return (
    <div className={style.SearchResult}>
      共筛选到<span className={style.resultNumber}>{props.number}</span>个结果
    </div>
  );
};

class ModalSelectClass extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initSKey = this.initSKey.bind(this);
    this.loadList = this.loadList.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.handleSelect = this.handleSelect.bind(this);
    this.handleDelete = this.handleDelete.bind(this);
    this.onOk = this.onOk.bind(this);
    this.state = {
      loading: false,
      // 筛选项
      searchKey: null,
      // 待选列表
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 10,
      // 已选列表
      selectedList: [],
    };
    this.isSearchAlready = false;
  }

  static propTypes = {
    // 弹窗标题
    title: PropTypes.string,
    // 需初始化的筛选项
    initSKey: PropTypes.object,
    // 关闭
    onClose: PropTypes.func.isRequired,
    // 提交
    onSubmit: PropTypes.func.isRequired,
  };

  static defaultProps = {
    title: '选择班级',
    initSKey: null,
    visible: false,
  };

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    this.isSearchAlready = true;
    await new Promise(this.initSKey);
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: [
          'twoTcMainTc',
          'twoTcAuxTc',
          'smallClassMainTc',
          'smallClassAuxTc',
          'branchCampus',
          'campus',
          'productTypeTree',
          'grade',
          'subject',
          'year',
          'quarter',
          'phase',
          'weekDay',
          'productGroup',
        ],
        callback() {
          resolve();
        },
      },
    });
  }

  initSKey(resolve) {
    let newSKey = sKeyAdapter(this.props.initSKey, this.props.yearDict, this.props.gradeDict);
    this.setState({ searchKey: newSKey }, () => {
      resolve();
    });
  }

  getListParams() {
    let searchKey = {
      page: this.state.pageCurrent,
      rows: this.state.pageSize,
      getTotal: false,
    };

    for (let key in this.state.searchKey) {
      searchKey[key] = Array.isArray(this.state.searchKey[key])
        ? this.state.searchKey[key].join(',')
        : this.state.searchKey[key];
    }

    return searchKey;
  }

  async loadList(resolve) {
    const params = this.getListParams();

    const { mainTeacherId, assistTeacherId, ...apiParams } = params;
    apiParams.mainTeacherName = mainTeacherId;
    apiParams.assistTeacherName = assistTeacherId;
    const isFirstPage = apiParams.page === 1;

    let fetchList = [];

    if (isFirstPage) {
      fetchList.unshift(queryClassList({ ...apiParams, getTotal: true }, true));
    }
    fetchList.unshift(queryClassList(apiParams, true));

    const rspAll = await Promise.all(fetchList).catch(errorsHandle());
    if (rspAll) {
      const [rspList, rspTotal] = rspAll;
      let payload = { list: rspList.rows || [] };
      if (rspTotal) {
        payload.total = rspTotal.records;
      }
      this.setState(payload);
    }

    resolve();
  }

  updatePage(pageCurrent, pageSize) {
    return new Promise((resolve) => {
      this.setState(
        (prevState) => {
          let payload = {};
          if (pageSize && pageSize !== prevState.pageSize) {
            payload.pageCurrent = 1;
            payload.pageSize = pageSize;
          } else {
            payload.pageCurrent = pageCurrent;
          }
          return payload;
        },
        () => {
          resolve();
        },
      );
    });
  }

  async changePage(pageCurrent, pageSize) {
    await this.updatePage(pageCurrent, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  search(params) {
    this.setState({ searchKey: params }, () => {
      this.changePage(1, this.state.pageSize);
    });
  }

  handleSelect(index) {
    let tagItem = this.state.list[index];
    if (this.state.selectedList.findIndex((item) => item.classId === tagItem.classId) === -1) {
      this.setState((prevState) => {
        prevState.selectedList.push(tagItem);
        return { selectedList: prevState.selectedList };
      });
    } else {
      showMessage('该班已存在，无需重复选择', 'error');
    }
  }

  handleDelete(index) {
    this.setState((prevState) => {
      prevState.selectedList.splice(index, 1);
      return { selectedList: prevState.selectedList };
    });
  }

  onOk() {
    runFunc(this.props.onSubmit, this.state.selectedList);
  }

  render() {
    const { productType } = this.state.searchKey || {};
    const isTwoTeacher = productType === TWO_TEACHER_VAL;
    return (
      <Modal
        loading={this.state.loading}
        onOk={this.onOk}
        okText="保存"
        onClose={this.props.onClose}
        contentOptions={{
          title: <span style={{ fontSize: 18 }}>{this.props.title}</span>,
          children: (
            <Wrapper
              header={
                <div>
                  <Search initSKey={this.state.searchKey} onSearch={this.search} isAlready={this.isSearchAlready} />
                  <SearchResult number={this.state.total} />
                </div>
              }
            >
              {this.state.list.length > 0 ? (
                <Deck
                  dataSource={this.state.list}
                  render={classCardRender({
                    onClick: this.handleSelect,
                    isTwoTeacher,
                  })}
                  pagination={{
                    total: this.state.total,
                    current: this.state.pageCurrent,
                    size: this.state.pageSize,
                    onChange: this.changePage,
                  }}
                  className={style.ToBeSelectCard}
                />
              ) : (
                <NoDataPlus />
              )}
            </Wrapper>
          ),
        }}
        sideOptions={{
          title: '已选班级',
          children: (
            <Deck
              dataSource={this.state.selectedList}
              render={selectedCardRender({
                onDelete: this.handleDelete,
              })}
              className={style.SelectedCard}
            />
          ),
        }}
      />
    );
  }
}

const mapStateToProps = (state) => {
  return {
    yearDict: state.dict.year,
    gradeDict: state.dict.grade,
  };
};

export default connect(mapStateToProps)(ModalSelectClass);
