import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import { Deck, Modal, Button, InputNumber, NoDataPlus } from 'COMPONENT';
import { Icon } from 'antd';
import ContractCard from './ContractCard';
import { typeOf, runFunc, errorsHandle } from 'UTIL/common';
import style from './styles.module.less';
import {
  calculateBatchPayArrangeMoney,
  deleteContractSegmentByContractId,
  getBranchSwitchConfigsByCodesAndCampusId,
} from 'SERVICE/contract';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import NP from 'number-precision';

const cardRender = ({ onChange, inputDisabled, inputVisible }) => {
  return (props) => (
    <ContractCard
      {...props}
      onChange={onChange}
      inputDisabled={inputDisabled}
      inputVisible={inputVisible}
      key={props.contractId}
    />
  );
};

class ContractProductPayment extends React.Component {
  static PropTypes = {
    // 一键分配金额
    autoAssignVisible: PropTypes.bool,
    // 多合同列表
    list: PropTypes.array,
    // 合同产品付款金额改变时
    onChange: PropTypes.func,
    // 最大高度，超过最大高度后内滚动
    maxBodyHeight: PropTypes.number,
    // 主体类名
    bodyClassName: PropTypes.string,
    // 头部插槽
    headerSlot: PropTypes.node,
    // 表格内的本次收款字段是否显示
    inputVisible: PropTypes.bool,
    // 是否输入框禁用
    disabled: PropTypes.bool,
    // 额外插槽
    subSlot: PropTypes.node,
    //
    headerTailSlot: PropTypes.node,
    //
    className: PropTypes.string,
    // 是否有全部收款按钮
    hasPayAllBtn: PropTypes.bool,
    onReload: PropTypes.func,
  };

  static defaultProps = {
    list: [],
    disabled: false,
    maxBodyHeight: 0,
    inputVisible: true,
    hasPayAllBtn: false,
    autoAssignVisible: false,
  };

  constructor(props) {
    super(props);

    this.refreshValue = this.refreshValue.bind(this);
    this.onAutoAssignChange = this.onAutoAssignChange.bind(this);
    this.onAssignClick = this.onAssignClick.bind(this);
    this.onEmptyClick = this.onEmptyClick.bind(this);
    this.onPayAllClick = this.onPayAllClick.bind(this);
    this.getTotalAmount = this.getTotalAmount.bind(this);
    this.setValue = this.setValue.bind(this);
    this.getValue = this.getValue.bind(this);
    this.validate = this.validate.bind(this);

    this.state = {
      autoAssign: 0,
      maxAmount: Infinity,
      isShowPayAllBtn: false,
      confirmPayAllModalVisible: false,
    };
  }

  componentDidMount() {
    // 是否显示全额收款
    if (this.props.hasPayAllBtn) {
      this.getBranchSwitchConfigsByCodesAndCampusId();
    }
  }

  refreshValue() {
    // 更新本次收款金额
    const total = this.props.list.reduce((accumulate, current) => NP.plus(accumulate, current.canPaidAmount), 0);
    this.setState({
      maxAmount: total,
      autoAssign: total,
    });
    if (total > 0) {
      this.onAssignClick(total);
    } else {
      this.onEmptyClick();
    }
  }

  onAutoAssignChange(val) {
    this.setState({ autoAssign: val });
  }

  async onAssignClick(val) {
    let value = typeof val == 'number' ? val : undefined;
    if (!value && this.state.autoAssign === 0) {
      return showMessage('请先输入本次收款金额！', 'error');
    }

    let list = this.props.list;

    const apiParams = {
      // 过滤掉无需支付的合同
      contractIds: this.props.list.filter((item) => item.canPaidAmount > 0).map((item) => item.contractId),
      totalPayAmount: value || this.state.autoAssign,
    };
    if (apiParams.contractIds.length === 0) {
      return;
    }
    const res = await calculateBatchPayArrangeMoney(apiParams).catch(errorsHandle());

    if (res && Array.isArray(res.data)) {
      this.setValue(
        res.data.map((item) => ({
          amount: item.arrangeMoney,
          contractId: item.contractId,
          contractProductId: item.contractProductId,
        })),
        this.cleanAmount(this.props.list),
      );
    }

    runFunc(this.props.onChange, list, this.calcTotal(list));
  }

  onEmptyClick() {
    this.setState({ autoAssign: 0 });

    runFunc(this.props.onChange, this.cleanAmount(this.props.list), 0);
  }

  onPayAllClick() {
    if (!this.props.list.length) {
      return;
    }
    const [first] = this.props.list;
    // 【全额收款】按钮增加判断，若该合同已经取消分段，则点击【全额收款】时，不再弹出左图的弹窗提示。此时点击【全额收款】只帮用户把剩余未收金额填入收款输入框
    if (first.segmentChargeConfigSnapshot) {
      this.setState({ confirmPayAllModalVisible: true });
    } else {
      this.refreshValue();
    }
  }

  getTotalAmount() {
    return this.props.list.reduce((result, current) => {
      return NP.plus(
        result,
        current.contractProductArr.reduce((total, item) => {
          return NP.plus(total, item.amount || 0);
        }, 0),
      );
    }, 0);
  }

  cleanAmount = (list) => {
    list.forEach((contract) => {
      contract.contractProductArr.forEach((item) => {
        delete item.amount;
      });
    });
    return list;
  };

  setValue(params, list) {
    list = list || this.props.list;

    const type = typeOf(params);
    if (type === 'object') {
      const contractIndex = list.findIndex((item) => item.contractId === params.contractId);
      if (contractIndex > -1) {
        const productIndex = list[contractIndex].contractProductArr.findIndex(
          (item) => item.contractProductId === params.contractProductId,
        );
        if (productIndex > -1) {
          list[contractIndex].contractProductArr[productIndex].amount = params.amount;
        }
      }
    } else if (type === 'array' && params.length > 0) {
      params.forEach((paramsItem) => {
        const contractIndex = list.findIndex((item) => item.contractId === paramsItem.contractId);
        if (contractIndex > -1) {
          const productIndex = list[contractIndex].contractProductArr.findIndex(
            (item) => item.contractProductId === paramsItem.contractProductId,
          );
          if (productIndex > -1) {
            list[contractIndex].contractProductArr[productIndex].amount = paramsItem.amount;
          }
        }
      });
    }

    const total = this.calcTotal(list);

    this.onAutoAssignChange(total);

    runFunc(this.props.onChange, list, total);
  }

  calcTotal = (list) => {
    return list.reduce((contractAmount, contractCurrent) => {
      return NP.plus(
        contractAmount,
        contractCurrent.contractProductArr.reduce((productAmount, productCurrent) => {
          return NP.plus(productAmount, productCurrent.amount || 0);
        }, 0),
      );
    }, 0);
  };

  getValue() {
    return this.props.list.reduce((result, current) => {
      return result.concat(
        current.contractProductArr.map((item) => ({
          contractId: current.contractId,
          contractProductId: item.contractProductId,
          paidAmount: item.amount || 0,
        })),
      );
    }, []);
  }

  validate() {
    return this.props.list.some((contract) =>
      contract.contractProductArr.some((product) => product.pendingAmount > 0 && product.amount > 0),
    );
  }

  getBranchSwitchConfigsByCodesAndCampusId = async () => {
    let isShowPayAllBtn = false;
    const campusId = this.props.list[0].blCampusId;
    const result = await getBranchSwitchConfigsByCodesAndCampusId({
      codes: 'SC_BRANCH_CANCEL_SEGMENT_CHARGE',
      campusId,
    }).catch(errorsHandle());
    if (result && result.resultCode == 0 && result.data && result.data.length) {
      isShowPayAllBtn = result.data[0].switchFlag;
    }
    this.setState({ isShowPayAllBtn });
  };

  handlePayAll = async () => {
    const contractId = this.props.list.map((item) => item.contractId);
    // const canPaidAmountContractId = this.props.list.filter(item => item.canPaidAmount > 0);
    const result = await deleteContractSegmentByContractId({
      contractId: contractId.join(','),
    }).catch(errorsHandle());
    this.handleClosePayAllConfirmModal();
    if (result && result.resultCode === 0) {
      this.props.onReload();
      /*if (canPaidAmountContractId.length) {
        this.refreshValue();
      } else {*/
      //}
    }
  };

  handleClosePayAllConfirmModal = () => {
    this.setState({ confirmPayAllModalVisible: false });
  };

  // 全部付款的二次确认弹框
  generateConfirmModal() {
    const props = {
      widthSize: 's',
      title: '温馨提示',
      okText: '确认取消',
      cancelText: '保留分段',
      onOk: this.handlePayAll,
      onCancel: this.handleClosePayAllConfirmModal,
    };

    return (
      this.state.confirmPayAllModalVisible && (
        <Modal {...props}>
          <div className={style.confirmTextWrapper}>
            <Icon
              type="exclamation-circle"
              style={{ color: '#ffbf00', fontSize: 22, marginRight: 5, position: 'relative', top: 2 }}
            />
            本操作将取消合同命中的分段收费规则，取消后合同可收取全款，请问是否取消？（确定后不可撤回）
          </div>
        </Modal>
      )
    );
  }

  render() {
    return (
      <div className={this.props.className}>
        <div className={style.header}>
          {this.props.headerSlot}

          {this.props.autoAssignVisible && (
            <span className={classNames(style.headerItem, style.autoAssignWrap)}>
              <span className={style.bold}>本次收款金额：</span>
              <InputNumber
                min={0}
                max={this.state.maxAmount}
                precision={2}
                value={this.state.autoAssign}
                disabled={this.props.disabled}
                onChange={this.onAutoAssignChange}
                size="xSmall"
              />
              <Button onClick={this.onAssignClick} type="primary" size="xSmall" disabled={this.props.disabled}>
                一键分配
              </Button>
              <Button onClick={this.onEmptyClick} size="xSmall" disabled={this.props.disabled}>
                清除分配
              </Button>
              {this.props.hasPayAllBtn && this.state.isShowPayAllBtn && (
                <Button size="xSmall" onClick={this.onPayAllClick} disabled={this.props.disabled}>
                  全额收款
                </Button>
              )}
              {this.props.headerTailSlot}
            </span>
          )}
        </div>

        {this.props.subSlot && <div className={style.subSlot}>{this.props.subSlot}</div>}

        <div>
          {this.props.list.length > 0 ? (
            <Deck
              bodyStyle={this.props.maxBodyHeight ? { maxHeight: this.props.maxBodyHeight } : {}}
              bodyClassName={classNames(style.deckBody, this.props.bodyClassName)}
              dataSource={this.props.list}
              render={cardRender({
                onChange: this.setValue,
                inputDisabled: this.props.disabled,
                inputVisible: this.props.inputVisible,
              })}
            />
          ) : (
            <NoDataPlus />
          )}
        </div>

        {this.generateConfirmModal()}
      </div>
    );
  }
}

ContractProductPayment.HeaderItem = (props) => {
  return (
    <span className={style.headerItem} style={props.style}>
      {props.children}
    </span>
  );
};

export default ContractProductPayment;
