import React from 'react';
import PropTypes from 'prop-types';
import styles from './styles.module.less';
import classNames from 'classnames';
import { MultiSelect, Button, PopoverWrap } from 'COMPONENT';
import NP from 'number-precision';
import { getDepositCouponsByStudent } from 'SERVICE/contract';
import { errorsHandle, runFunc } from 'UTIL/common';

const errorsHandler = errorsHandle();

class UseDeposit extends React.Component {
  static propTypes = {
    studentId: PropTypes.string.isRequired,
    amount: PropTypes.number,
    direction: PropTypes.oneOf(['row', 'column']),
    onChange: PropTypes.func,
    size: PropTypes.oneOf(['large', 'small', 'xSmall']),
    labelSlot: PropTypes.node,
    selectWidth: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
    disabledTips: PropTypes.string,
  };

  static defaultProps = {
    amount: 0,
    direction: 'row',
    default: 'large',
    selectWidth: 'auto',
    disabledTips: '金额必需大于0',
  };

  constructor(props) {
    super(props);

    this.initDict = this.initDict.bind(this);
    this.loadData = this.loadData.bind(this);
    this.onChange = this.onChange.bind(this);
    this.onClick = this.onClick.bind(this);
    this.reset = this.reset.bind(this);
    this.emitChange = this.emitChange.bind(this);
    this.getDepositList = this.getDepositList.bind(this);

    this.state = {
      selected: [],
      total: 0,
      count: 0,
      dict: [],
      maxAmount: 0,
    };
  }

  componentDidMount() {
    this.loadData();
  }

  componentDidUpdate(prevProps) {
    if (!prevProps.amount && this.props.amount) {
      this.initDict();
    } else if (prevProps.amount !== this.props.amount) {
      const _max = this.props.amount ? Math.min(this.props.amount, this.state.maxAmount) : this.state.maxAmount;
      if (this.state.total <= _max) {
        this.setState({
          dict: this.dictAdapter(this.state.selected, NP.minus(_max, this.state.total), this.state.dict || []),
        });
      } else {
        this.reset();
      }
    }
  }

  initDict() {
    if (this.state.dict) {
      this.reset();
    } else {
      const _self = this;
      setTimeout(function () {
        _self.initDict();
      }, 100);
    }
  }

  async loadData() {
    try {
      getDepositCouponsByStudent({ studentId: this.props.studentId }).then((res) => {
        this.setState({
          dict: (res.data.depositList || []).map((item) => ({
            text: `${item.highAmount}元 (${item.endDate}失效)`,
            val: item.couponId,
            disabled: false,
            highAmount: item.highAmount,
          })),
          maxAmount: res.data.maxAmount,
        });
      });
    } catch (e) {
      errorsHandler(e);
    }
  }

  onChange(selected) {
    if (!selected) {
      return this.reset();
    }

    this.setState((prevState, props) => {
      const _max = props.amount ? Math.min(props.amount, prevState.maxAmount) : prevState.maxAmount;

      prevState.total = this.totalAdapter(selected, prevState.dict);
      if (selected.length < prevState.selected.length || prevState.total <= _max) {
        prevState.selected = selected;
      } else {
        prevState.total = this.totalAdapter(prevState.selected, prevState.dict);
      }
      prevState.dict = this.dictAdapter(prevState.selected, NP.minus(_max, prevState.total), prevState.dict);
      prevState.count = prevState.selected.length;

      return prevState;
    }, this.emitChange);
  }

  dictAdapter = (selected, restAmount, dict) => {
    dict.forEach((item) => {
      if (selected.indexOf(item.val) === -1) {
        item.disabled = item.highAmount > restAmount;
      }
    });
    return dict;
  };

  totalAdapter = (selected, dict) => {
    return selected.reduce((accumulate, current) => {
      const tag = dict.find((item) => item.val === current);
      return tag ? NP.plus(accumulate, tag.highAmount) : accumulate;
    }, 0);
  };

  onClick() {
    this.setState((prevState, props) => {
      const _max = props.amount ? Math.min(props.amount, prevState.maxAmount) : prevState.maxAmount;

      prevState.selected = [];
      prevState.total = 0;
      prevState.count = 0;

      prevState.dict.forEach((item) => {
        const _total = NP.plus(prevState.total, item.highAmount);
        if (_total <= _max) {
          prevState.selected.push(item.val);
          prevState.total = _total;
          prevState.count++;

          item.disabled = false;
        } else {
          item.disabled = true;
        }
      });

      return prevState;
    }, this.emitChange);
  }

  reset() {
    this.setState((prevState, props) => {
      prevState.selected = [];
      prevState.total = 0;
      prevState.count = 0;

      if (prevState.dict.length > 0) {
        prevState.dict = this.dictAdapter(
          [],
          props.amount ? Math.min(props.amount, prevState.maxAmount) : prevState.maxAmount,
          prevState.dict,
        );
      }

      return prevState;
    }, this.emitChange);
  }

  emitChange() {
    runFunc(this.props.onChange, this.state.selected, this.state.total);
  }

  getDepositList() {
    return this.state.dict || [];
  }

  render() {
    const { props, state } = this;
    const noDict = state.dict.length === 0;
    const disabled = noDict || !props.amount;

    return (
      <div className={classNames(styles.useDeposit, styles[props.direction])}>
        <div className={styles.wrap}>
          {this.props.labelSlot}

          <PopoverWrap visible={disabled} content={noDict ? '没有可用的定金券' : props.disabledTips} type="warning">
            <div className={styles.selectWrap} style={{ width: this.props.selectWidth }}>
              <MultiSelect
                placeholder="请选择定金券"
                value={state.selected}
                dict={state.dict}
                selectAll={false}
                showSearch={false}
                size={props.size}
                onChange={this.onChange}
                disabled={disabled}
              />
            </div>
          </PopoverWrap>
        </div>
        <div className={styles.wrap}>
          {props.direction === 'column' && (
            <span>
              已选金额：{state.total}元（{state.count}张）
            </span>
          )}

          <Button onClick={this.onClick} type="primary" size="xSmall">
            一键选择
          </Button>

          {props.direction === 'row' && (
            <span style={{ marginLeft: 6 }}>
              已选金额：{state.total}元（{state.count}张）
            </span>
          )}
        </div>
      </div>
    );
  }
}

export default UseDeposit;
