import React from 'react';
import PropTypes from 'prop-types';
import styles from './styles.module.less';
import { Select } from 'COMPONENT';
import { performanceOwnerTypeDict, USER_USER, USER_CAMPUS, USER_BRANCH } from './config';
import { runFunc, errorsHandle, typeOf } from 'UTIL/common';
import { loadBranchDict, loadBranchCampusDict } from 'UTIL/dict';
import { debounce } from 'lodash';
import { getUserForPerformance } from 'SERVICE/contract';

const errorsHandler = errorsHandle();

class PerformanceOwnerSelect extends React.Component {
  static propTypes = {
    initialValue: PropTypes.arrayOf(PropTypes.string),
    value: PropTypes.arrayOf(PropTypes.string),
    onChange: PropTypes.func,
  };

  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.onTypeChange = this.onTypeChange.bind(this);
    this.onOwnerChange = this.onOwnerChange.bind(this);
    this.onPersonSearch = this.onPersonSearch.bind(this);

    this.state = {
      type: undefined,
      owner: undefined,
      personDict: [],
      campusDict: [],
      branchDict: [],
    };
    if (Array.isArray(props.initialValue)) {
      this.state.type = props.initialValue[0] || undefined;
      this.state.owner = props.initialValue[1] || undefined;
    }

    this.lastSelectRef = null;
  }

  componentDidMount() {
    this.init();
  }

  componentDidUpdate(prevProps, prevState, snapshot) {
    const { props, state } = this;
    const prevVal = prevProps.value || [];
    const curVal = props.value || [];
    if (
      // props有更新
      (typeOf(prevProps.value) !== typeOf(props.value) || prevVal[0] !== curVal[0] || prevVal[1] !== curVal[1]) &&
      // props值与state不一致
      (curVal[0] !== state.type || curVal[1] !== state.owner)
    ) {
      if (Array.isArray(props.value)) {
        this.setState({
          type: props.value[0] || state.type,
          owner: props.value[1] || state.owner,
        });
      } else {
        this.setState({
          type: undefined,
          owner: undefined,
        });
      }
    }
  }

  async init() {
    const [campusDict, branchDict] = await Promise.all([loadBranchCampusDict(), loadBranchDict()]).catch(errorsHandler);
    this.setState({
      campusDict,
      branchDict,
    });
  }

  onTypeChange(type) {
    this.lastSelectRef.clearSearchKey();
    this.setState(
      {
        type: type || undefined,
        owner: undefined,
        personDict: type === USER_USER ? [] : this.state.personDict,
      },
      () => {
        runFunc(this.props.onChange, []);
      },
    );
  }

  onOwnerChange(owner) {
    this.setState(
      {
        owner: owner || undefined,
      },
      () => {
        if (owner) {
          runFunc(this.props.onChange, [this.state.type, owner]);
        }
      },
    );
  }

  async onPersonSearch(val) {
    const { state } = this;

    let selectedOwner;
    if (state.owner) {
      selectedOwner = state.personDict.find((opt) => opt.val === state.owner);
    }
    let dict = [];
    if (val) {
      const data = await getUserForPerformance({ term: val }).catch(errorsHandler);
      dict = data.map((item) => ({
        val: item.value,
        text: item.label,
      }));
    }
    if (selectedOwner && dict.findIndex((opt) => opt.val === selectedOwner.val) === -1) {
      dict.unshift(selectedOwner);
    }
    this.setState({
      personDict: dict,
    });
  }

  debouncePersonSearch = (val) => {
    debounce(this.onPersonSearch, 600)(val);
  };

  getOwnerPlaceholder = (state) => {
    switch (state.type) {
      case USER_USER:
        return '业绩归属人';
      case USER_CAMPUS:
        return '业绩归属学习中心';
      case USER_BRANCH:
        return '业绩归属分公司';
      default:
        return '请先选择业绩归属类型';
    }
  };

  getOwnerDict = (state) => {
    switch (state.type) {
      case USER_USER:
        return state.personDict;
      case USER_CAMPUS:
        return state.campusDict;
      case USER_BRANCH:
        return state.branchDict;
    }
  };

  render() {
    const { state } = this;
    const isUser = state.type === USER_USER;
    return (
      <div className={styles.selectWrap}>
        <Select
          className={styles.selectFirst}
          placeholder="业绩归属类型"
          dict={performanceOwnerTypeDict}
          value={state.type}
          onChange={this.onTypeChange}
        />
        <Select
          ref={(c) => (this.lastSelectRef = c)}
          className={styles.selectLast}
          placeholder={this.getOwnerPlaceholder(state)}
          dict={this.getOwnerDict(state)}
          value={state.owner}
          onChange={this.onOwnerChange}
          showSearch
          onSearchKeyChange={isUser ? this.debouncePersonSearch : undefined}
          noOptionTips={isUser ? '请输入姓名进行查询' : undefined}
          disabled={!state.type}
        />
      </div>
    );
  }
}

export default PerformanceOwnerSelect;
