import { CONTRACT_TYPE_NEW, CONTRACT_TYPE_RE_NEW } from './constant';
import { isString, isNumber } from 'UTIL/lang/data-type';
import moment from 'moment';

/**
 * 格式化合同类型显示文案
 *
 * @param contractType {string} 合同类型
 * @return {boolean|string}
 */
export function formatContractTypeName(contractType) {
  return (contractType === CONTRACT_TYPE_NEW && '新签') || (contractType === CONTRACT_TYPE_RE_NEW && '续费') || '';
}

/**
 * 收集对象中值不同的key
 *
 * @param preData {Object} 修改前合同详情
 * @param nextData {Object} 修改后合同详情
 * @return {string} 用`,`分割的字符串key
 */
export function collectDifferentKey(preData, nextData) {
  if (!preData || !nextData) {
    return '';
  }

  let result = [];

  Object.keys(preData).forEach((key) => {
    const preValue = preData[key];
    const nextValue = nextData[key];

    if (!preValue || isString(preValue) || isNumber(preValue)) {
      if (preValue !== nextValue) {
        result.push(key);
      }
    }
  });

  return result.join(',');
}

/**
 * 判断是否应该标红字段
 *
 * @param key {string} 比较字段
 * @param differentKeys {string} 值不相同的key集合
 * @return {boolean}
 */
function calcShouldStandardRedKey(key, differentKeys) {
  if (!key || !differentKeys) {
    return false;
  }

  return differentKeys.indexOf(key) > -1;
}

/**
 * 格式化红色样式
 *
 * @param key {string} 比较字段
 * @param differentKeys {string} 值不相同的key集合
 * @return {{}|{color: string}}
 */
export function formatRedStyle(key, differentKeys) {
  if (calcShouldStandardRedKey(key, differentKeys)) {
    return { color: '#ff4d4f', fontWeight: 600 };
  }

  return {};
}

/**
 * 通过开始上课时间及总课时长，获取上课时间范围
 *
 * @param startTime {string} '14:00'
 * @param timeThrough {number} 90
 *
 * @return {string} '14:00~15:30'
 * */
export function getTimeRange(startTime, timeThrough) {
  if (!startTime) {
    return '';
  }

  if (!timeThrough) {
    return startTime;
  }

  return `${startTime}~${moment(startTime, 'HH:mm').add(timeThrough, 'm').format('HH:mm')}`;
}

/**
 * 分割开始上课时间
 *
 * @param startClassDate {string} 开始上课时间
 * @param splitStr {string} 时间分隔符
 * @return {string|*|string}
 */
export function splitStartClassDate(startClassDate, splitStr = ' ') {
  if (!startClassDate) {
    return '';
  }

  const timeArr = startClassDate.split(splitStr);

  return timeArr[0];
}

/**
 * 隐藏部分手机号码
 *
 * @param number
 * @return {string|*}
 */
export function formatPhoneNumber(number) {
  if (!number) {
    return '';
  }

  return number.replace(/(\d{3})(\d{4})(\d{4})/, '$1****$3');
}
