import React from 'react';
import { connect } from 'dva';
import { storeName, dateFormat, insightPayTypeDict, insightExtraBranch, insightExtraCampus } from './store/config';
import { runFunc } from 'UTIL/common';
import { SearchFilter, SearchField } from 'COMPONENT/SearchFilter';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import Validator from 'COMPONENT/Validator';
import { DatePicker } from 'antd';
import { getRelationsDictPlus, getRelationPayloadPlus } from 'MODEL/dict/utils/index';
import { branch, campus } from 'MODEL/dict/utils/config';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const RangePicker = DatePicker.RangePicker;

class Search extends React.Component {
  constructor(props) {
    super(props);
    this.updateProps = this.updateProps.bind(this);
    this.updateValue = this.updateValue.bind(this);
    this.updateDateRange = this.updateDateRange.bind(this);
    this.updateBranch = this.updateBranch.bind(this);
    this.errorChange = this.errorChange.bind(this);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.state = {
      isNeedValidator: false,
      error: {},
    };
    // 分公司关联关系type、key对应关系配置
    this.branchTreeMap = {
      [branch]: 'sKey_branch',
      [campus]: 'sKey_campus',
    };
  }

  updateProps(payload) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }

  updateValue(key, val) {
    this.updateProps({ [key]: val });
  }

  updateDateRange(moments, dates) {
    this.updateProps({
      sKey_momentRange: moments,
      sKey_dateRange: dates,
    });
  }

  updateBranch(val) {
    let payload = getRelationPayloadPlus('sKey_branch', val, {
      type: branch,
      dictTree: this.props.branchCampusTree,
      state: this.props,
      map: this.branchTreeMap,
    });
    this.updateProps(payload);
  }

  errorChange(err, keyName) {
    this.setState((prevState) => ({
      error: {
        ...prevState.error,
        [keyName]: err,
      },
    }));
  }

  search() {
    // 起始日期未选时
    if (!(this.props.sKey_momentRange[0] && this.props.sKey_momentRange[1])) {
      return showMessage('起始日期为必填项！', 'error');
    }
    runFunc(this.props.onSearch);
  }

  reset() {
    this.setState(
      {
        isNeedValidator: false,
        error: {},
      },
      () => {
        runFunc(this.props.onReset);
      },
    );
  }

  render() {
    return (
      <SearchFilter truncationNum={4} onSearch={this.search} onReset={this.reset} searchLoading={this.props.loading}>
        <SearchField span={2}>
          <Validator
            value={this.props.sKey_momentRange[0] && this.props.sKey_momentRange[1] ? 'true' : null}
            rules={['required']}
            silent={!Boolean(this.state.error.sKey_momentRange)}
            showStatus={true}
            showTips={false}
            onErrorChange={(err) => this.errorChange(err, 'sKey_momentRange')}
          >
            <RangePicker
              placeholder={['开始时间', '结束时间']}
              value={this.props.sKey_momentRange}
              onChange={this.updateDateRange}
              format={dateFormat}
              showTime={false}
              size="large"
            />
          </Validator>
        </SearchField>
        <SearchField>
          <Select
            placeholder="分公司"
            value={this.props.sKey_branch}
            dict={insightExtraBranch.concat(this.props.branchDict || [])}
            onChange={this.updateBranch}
            showSearch
          />
        </SearchField>
        <SearchField>
          <Select
            placeholder="学习中心"
            value={this.props.sKey_campus}
            dict={insightExtraCampus.concat(this.props.branchCampusDict || [])}
            onChange={(val) => this.updateValue('sKey_campus', val)}
            showSearch
          />
        </SearchField>
        <SearchField>
          <Select
            placeholder="类型"
            value={this.props.sKey_financeType}
            dict={this.props.financeTypeDict}
            onChange={(val) => this.updateValue('sKey_financeType', val)}
          />
        </SearchField>
        <SearchField>
          <Select
            placeholder="支付方式"
            value={this.props.sKey_channel}
            dict={insightPayTypeDict}
            onChange={(val) => this.updateValue('sKey_channel', val)}
          />
        </SearchField>
        <SearchField>
          <Select
            placeholder="活动类型"
            value={this.props.sKey_activityType}
            dict={this.props.activityTypeDict}
            onChange={(val) => this.updateValue('sKey_activityType', val)}
          />
        </SearchField>
        <SearchField>
          <Input
            placeholder="学生名字"
            value={this.props.sKey_studentName}
            onChange={(e) => this.updateValue('sKey_studentName', e.target.value)}
          />
        </SearchField>
        <SearchField>
          <Input
            placeholder="支付单号"
            value={this.props.sKey_orderNoInChannel}
            onChange={(e) => this.updateValue('sKey_orderNoInChannel', e.target.value)}
          />
        </SearchField>
      </SearchFilter>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    // 数据字典
    branchDict: state.dict.allBranchCampusTree,
    branchCampusDict: getRelationsDictPlus(campus)(state.dict.allBranchCampusTree, state[storeName].sKey_branch),
    branchCampusTree: state.dict.allBranchCampusTree,
    financeTypeDict: state.dict.insightFinanceType,
    activityTypeDict: state.dict.insightActivityType,
    // 筛选项
    sKey_momentRange: state[storeName].sKey_momentRange,
    sKey_dateRange: state[storeName].sKey_dateRange,
    sKey_studentName: state[storeName].sKey_studentName,
    sKey_orderNoInChannel: state[storeName].sKey_orderNoInChannel,
    sKey_branch: state[storeName].sKey_branch,
    sKey_campus: state[storeName].sKey_campus,
    sKey_financeType: state[storeName].sKey_financeType,
    sKey_channel: state[storeName].sKey_channel,
    sKey_activityType: state[storeName].sKey_activityType,
  };
};

export default connect(mapStateToProps)(Search);
