import React from 'react';
import { connect } from 'dva';
import Input from 'COMPONENT/Input';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import styles from './styles.less';
import Button from 'COMPONENT/Button';
import Tabs from 'COMPONENT/Tabs';
import { loadList } from '../loaders';
import {
  LIST_TYPE_MY,
  LIST_TYPE_OTHER,
  listTypeDict,
  CONTRACT_TYPE_NEW,
  CONTRACT_TYPE_AGAIN,
  contractTypeDict,
} from '../dicts';
import dicts from '../dicts';
import { sexDict } from 'PAGE/common/dicts';
import { loadCommonDict } from 'UTIL/dict';
import { getDictTextByVal } from '../../../../utils/dict/index';

const { Tab } = Tabs;
const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const defaultSearchKeys = {
  stuName: null, // {string} 查询条件 学生姓名
  cusPhone: null, // {string} 查询条件 客户电话
  cusName: null, // {string} 查询条件 客户姓名
};
const defaultCurrentPage = 1;
const defaultPageSize = 20;
let lists = {};
listTypeDict.forEach((d) => {
  lists[d.val] = {
    list: [], // {object[]} 列表数据
    totalRecords: 0, // {number} 总记录数
    currentPage: defaultCurrentPage, // {number} 当前页码
    pageSize: defaultPageSize, // {number} 每页记录数
  };
});

class ContractList extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      ...defaultSearchKeys,
      listSearchKeys: defaultSearchKeys, // {object} 列表数据的查询关键字
      activeTabKey: LIST_TYPE_MY, // {string} 激活的标签页ID
      lists, // {object} 列表数据集合
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = () => {
    const { lists: listsState } = this.state;
    this.setState({ loading: true });
    Promise.all([loadList(this.getListParams())]).then(([data]) => {
      let lists = {};
      Object.keys(listsState).forEach((key) => {
        lists[key] = { ...listsState[key], ...data[key] };
      });
      this.setState({ loading: false, lists });
    });
  };

  getListParams = (type) => {
    const {
      listSearchKeys: { stuName, cusPhone, cusName },
      lists,
    } = this.state;
    let currentPage = defaultCurrentPage;
    let pageSize = defaultPageSize;
    if (type) {
      const listData = lists[type];
      currentPage = listData.currentPage;
      pageSize = listData.pageSize;
    }
    return { stuName, cusPhone, cusName, currentPage, pageSize, type };
  };

  search = () => {
    const { stuName, cusPhone, cusName } = this.state;
    const listSearchKeys = { stuName, cusPhone, cusName };
    this.setState({ loading: true });
    loadList({ ...this.getListParams(), ...listSearchKeys }).then((data) => {
      let lists = {};
      Object.keys(data).forEach((key) => {
        lists[key] = { ...data[key], currentPage: defaultCurrentPage, pageSize: defaultPageSize };
      });
      this.setState({ loading: false, listSearchKeys, lists });
    });
  };

  reset = () => {
    this.setState(defaultSearchKeys);
  };

  onPaginationChange = ({ current, pageSize }, type) => {
    const pageInfo = {
      currentPage: current,
      pageSize,
    };
    this.setState({ loading: true });
    loadList({ ...this.getListParams(type), ...pageInfo }).then(
      (data) => {
        this.setState({
          loading: false,
          lists: { ...this.state.lists, [type]: { ...data, ...pageInfo } },
        });
      },
      () => {
        this.setState({ loading: false });
      },
    );
  };

  createStudent = () => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/AddContractList/CreateStudent',
        refresh: true,
      },
    });
  };

  reSign = ({ cusId, stuId, cusName, phone, stuName, stuSchool, gradeName, campusName }) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/AddContractList/Shop',
        params: {
          status: dicts.SHOP_STATUS_SELECT_SET,
          stuInfo: {
            cusId,
            stuId,
            contractType: CONTRACT_TYPE_AGAIN,
            cusName,
            contract: phone,
            campusName,
            stuName,
            gradeName,
            schoolName: stuSchool,
          },
        },
        refresh: true,
      },
    });
  };

  onTabChange = (key) => {
    this.setState({ activeTabKey: key });
  };

  render() {
    const { loading, stuName, cusPhone, cusName, activeTabKey, lists } = this.state;

    const columns = [
      { title: '客户姓名', dataIndex: 'cusName' },
      { title: '电话号码', dataIndex: 'phone', width: COL_WIDTH_L },
      { title: '学生编号', dataIndex: 'stuNo', width: COL_WIDTH_L },
      { title: '学生姓名', dataIndex: 'stuName' },
      { title: '性别', dataIndex: 'sex', dict: sexDict },
      { title: '学生学校', dataIndex: 'stuSchool', width: COL_WIDTH_L },
      { title: '年级', dataIndex: 'gradeName' },
      { title: '学管师', dataIndex: 'studyManager' },
      { title: '学习中心', dataIndex: 'campusName' },
      {
        title: '操作',
        key: 'action',
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          return [{ text: '续费', onClick: () => this.reSign(record) }];
        },
      },
    ];

    return (
      <Page fitWindowHeight={true} loading={loading} route={this.props.route}>
        <div className={styles.wrap}>
          <div className={styles.rightBtnWrap}>
            <Button onClick={this.createStudent}>创建新生</Button>
          </div>
          <div className="flexWrap">
            <div className={styles.searchBar}>
              <SearchBar onSearch={this.search} onReset={this.reset}>
                <Input
                  placeholder="学生姓名"
                  value={stuName}
                  onChange={(e) => this.setState({ stuName: e.target.value })}
                />
                <Input
                  placeholder="客户电话"
                  value={cusPhone}
                  onChange={(e) => this.setState({ cusPhone: e.target.value })}
                />
                <Input
                  placeholder="客户姓名"
                  value={cusName}
                  onChange={(e) => this.setState({ cusName: e.target.value })}
                />
              </SearchBar>
            </div>
            <div className="flexInnerFit">
              <Tabs onChange={this.onTabChange} activeKey={activeTabKey}>
                {Object.keys(lists).map((key) => {
                  const d = lists[key];
                  return (
                    <Tab key={key} title={getDictTextByVal(listTypeDict, key)} value={key}>
                      <List
                        columns={columns}
                        dataSource={d.list}
                        pagination={{
                          total: d.totalRecords,
                          current: d.currentPage,
                          pageSize: d.pageSize,
                        }}
                        onChange={(params) => this.onPaginationChange(params, key)}
                      />
                    </Tab>
                  );
                })}
              </Tabs>
            </div>
          </div>
        </div>
      </Page>
    );
  }
}

export default connect()(ContractList);
