/**
 * @param {boolean} loading - 页面加载状态
 * @param {object} stuInfo - 学生信息，{ cusId, stuId, contractType, cusName, contract, campusName, stuName, gradeName, schoolName }
 * @param {object[]} courseInfo - 课程信息，[{ type, name, grade, subject, teacher, address, time, status, endDate }]
 * @param {string} status - 当前状态，选择商品或者结账
 * @param {string[]} types - 选择产品时包含的产品类别
 * @param {object[]} cartList - 购物车中的产品列表，
  [{
    id {string} - ID
    productId {string} - 产品ID
    classId {string?} - 班级ID
    type {string} - 类型
    name {string} - 名称
    count {number} - 购买数量
    price {number} - 单价
    promotionInfo {object[]} - 参加的优惠信息
    infos {string[]} - 附加信息
    additionProducts {object[]} - 关联产品
    firstClassDate {string} - 首次上课日期（仅小班和双师类型的产品具有）
    selectedPromotion {string} - 选中的优惠ID
  }]
 * @param {string} checkedTicket - 当前选中的优惠券ID（对整个订单使用）
 * @param {string} code - 当前使用的促销码（对整个订单使用）
 */
import React from 'react';
import { connect } from 'dva';
import SelectProductSet from '../SelectProductSet';
import SelectProduct from '../SelectProduct';
import Checkout from '../Checkout';
import Page from 'COMPONENT/Page';
import styles from './styles.less';
import dicts from '../dicts';
import loaders from '../loaders';
import { getDictTextByVal } from '../../../../utils/dict/index';
import Button from 'COMPONENT/Button';
import showMessage from '../../../../components/utils/globalTips/showMessage';

class Shop extends React.Component {
  componentWillReceiveProps(nextProps) {
    if (this.props.activePageId === '/AddContractList/Shop' && nextProps.activePageId !== '/AddContractList/Shop') {
      this.closeShop();
    }
  }

  closeShop = () => {
    this.props.tabpanes.forEach((t) => {
      const { id } = t;
      if (id.indexOf('/AddContractMakeDiscount') === 0) {
        this.props.dispatch({
          type: 'main/removeTab',
          payload: { id },
        });
      }
    });
  };

  updateParams = (changed) => {
    const { dispatch, params } = this.props;
    dispatch({
      type: 'main/setPageParams',
      payload: {
        path: '/AddContractList/Shop',
        params: { ...params, ...changed },
      },
    });
  };

  saveContract = () => {
    const { dispatch, params } = this.props;
    const {
      cartList,
      stuInfo: { contractType, cusId, stuId },
    } = params;
    this.updateParams({ loading: true });
    loaders.saveContract({ cartList, contractType, cusId, stuId }).then(
      () => {
        dispatch({
          type: 'main/openPage',
          payload: {
            path: '/AddContractList',
            params: {},
            refresh: true,
          },
        });
        showMessage('保存成功', 'success');
        this.updateParams({ loading: false });
      },
      (e) => {
        showMessage(e.text || '保存失败');
        this.updateParams({ loading: false });
      },
    );
  };

  getFooter = () => {
    const {
      params: { status, loading, cartList },
    } = this.props;
    if (status === dicts.SHOP_STATUS_CHECK) {
      return (
        <div style={{ overflow: 'hidden' }}>
          <div className="Page-btn-wrap">
            <Button type="primary" onClick={this.saveContract} disabled={loading || cartList.length === 0}>
              保存并跳转到合同列表
            </Button>
          </div>
        </div>
      );
    }
    return false;
  };

  getContent = () => {
    const {
      user,
      params: { status, types, cartList, dynDicts, stuInfo, courseInfo },
    } = this.props;
    if (status === dicts.SHOP_STATUS_SELECT_SET) {
      return (
        <SelectProductSet onUpdateParams={this.updateParams} user={user} stuInfo={stuInfo} courseInfo={courseInfo} />
      );
    } else {
      return (
        <div>
          <div style={{ display: status === dicts.SHOP_STATUS_SELECT ? 'block' : 'none' }}>
            <SelectProduct
              types={types}
              cartList={cartList}
              onUpdateParams={this.updateParams}
              stuInfo={stuInfo}
              courseInfo={courseInfo}
            />
          </div>
          <div style={{ display: status === dicts.SHOP_STATUS_CHECK ? 'block' : 'none' }}>
            <Checkout
              types={types}
              user={user}
              onUpdateParams={this.updateParams}
              stuInfo={stuInfo}
              courseInfo={courseInfo}
            />
          </div>
        </div>
      );
    }
  };

  render() {
    const {
      params: { status, loading, cartList },
    } = this.props;
    return (
      <Page
        loading={loading}
        route={this.props.route}
        name={getDictTextByVal(dicts.shopStatusDict, status)}
        footer={this.getFooter()}
        className={styles.page}
      >
        {this.getContent()}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
    params: state.main.pageParams['/AddContractList/Shop'],
    activePageId: state.main.tabpanes.find((t) => t.id === '/AddContractList').activePageId,
    tabpanes: state.main.tabpanes,
  };
};

export default connect(mapStateToProps)(Shop);
