import React from 'react';

import { List, Input, Select, Button } from 'COMPONENT';
import EducationPlatformModal from '../EducationPlatformModal';

import _ from 'lodash';
import moment from 'moment';

import { errorsHandle } from 'SRC/utils/common';

import ShowConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import showMessage from 'SRC/components/utils/globalTips/showMessage';

import { selectOrgByIdAndTypeOption } from 'SERVICE/common';
import * as educationManagementPlatformApi from 'SERVICE/educationManagementPlatform';

import styles from './style.module.less';

const statusDict = [
  { val: '', text: '全部' },
  { val: 0, text: '无效' },
  { val: 1, text: '有效' },
];

export default class DataList extends React.Component {
  state = {
    // 搜索条件
    status: null,
    posName: null,
    branchId: null,
    posNumber: null,
    blCampusId: null,
    platformName: null,

    // 列表数据
    list: [],
    total: 0,
    pageSize: 30,
    currentPage: 1,

    // 数据字典
    campusDict: [],
    companyDict: [],

    visible: false,
    modelMode: 'add',
    initialValues: {},
  };

  constructor(props) {
    super(props);
  }

  componentDidMount() {
    this.init();
  }

  init() {
    this.loadData();
    this.loadDict();
  }

  // 加载公司数据字典
  async loadCompanyDict() {
    const result = await selectOrgByIdAndTypeOption({
      orgType: 'BRENCH',
    }).catch(errorsHandle());

    if (!result || result.resultCode !== 0 || !result.value) {
      this.setState({ companyDict: [] });
      return;
    }

    const companyDict = _.map(result.value, (text, val) => ({ val, text })).filter(({ val }) => val);
    this.setState({ companyDict: [{ val: '', text: '请选择' }, ...companyDict] });
  }

  // 加载学习中心
  async loadCampusDict(orgId) {
    this.setState({ campusDict: [] });

    const result = await selectOrgByIdAndTypeOption({
      orgId,
      orgType: 'CAMPUS',
    }).catch(errorsHandle());

    if (!result || result.resultCode !== 0 || !result.value) {
      return;
    }

    const campusDict = _.map(result.value, (text, val) => ({ val, text })).filter(({ val }) => val);
    this.setState({ campusDict: [{ val: '', text: '请选择' }, ...campusDict] });
  }

  async loadData() {
    this.props.onSetLoading(true);

    const {
      status,
      currentPage: pageNum,
      pageSize,
      branchId,
      blCampusId,
      posName,
      posNumber,
      platformName,
    } = this.state;
    const result = await educationManagementPlatformApi
      .getPageEducationPlatform({
        status,
        posName,
        pageNum,
        pageSize,
        branchId,
        posNumber,
        blCampusId,
        platformName,
      })
      .catch(errorsHandle());

    this.props.onSetLoading(false);
    if (!result || result.resultCode !== 0 || !result.data.list.length) {
      this.setState({ list: [], total: 0 });
      return;
    }

    this.setState({
      total: result.data.total,
      list: result.data.list.map((item) => ({ ...item, key: item.id })),
    });
  }

  loadDict() {
    this.loadCompanyDict();
  }

  getColumns() {
    return [
      {
        title: '序号',
        width: 40,
        dataIndex: 'index',
        render: (context, value, index) => {
          return (this.state.currentPage - 1) * this.state.pageSize + index + 1;
        },
      },
      { title: '平台名称', width: 120, dataIndex: 'platformName' },
      { title: '终端名称', width: 120, dataIndex: 'posName' },
      { title: '平台终端号', width: 100, dataIndex: 'posNumber' },
      { title: '分公司', width: 100, dataIndex: 'branchName' },
      { title: '学习中心', width: 120, dataIndex: 'blCampusName' },
      {
        title: '添加时间',
        width: 120,
        dataIndex: 'createTime',
        render: (value, index) => moment(value).format('YYYY-MM-DD HH:mm:ss'),
      },
      { title: '添加人', width: 80, dataIndex: 'createUserName' },
      {
        title: '状态',
        width: 40,
        dataIndex: 'status',
        render: (status) => (statusDict.find(({ val }) => val === status) || {}).text || '-',
      },
      {
        title: '操作',
        width: 100,
        fixed: 'right',
        key: 'action',
        getActionConfig: (text, record, index) => {
          const { status } = record;
          return status == 1
            ? [
                { text: '修改', onClick: () => this.handleEdit(record) },
                { text: '无效', onClick: () => this.handleChangeStatus(record) },
                { text: '详情', onClick: () => this.handleViewDetail(record) },
              ]
            : [
                { text: '有效', onClick: () => this.handleChangeStatus(record) },
                { text: '详情', onClick: () => this.handleViewDetail(record) },
              ];
        },
      },
    ];
  }

  handleAddSuccess = () => {
    this.loadData();
    this.setState({ visible: false });
  };

  handleCancelAdd = () => {
    this.setState({ visible: false });
  };

  // 选择公司
  handleSelectCompany = (branchId) => {
    this.loadCampusDict(branchId);
    this.setState({ branchId, blCampusId: null });
  };

  // 选择学习中心
  handleSelectCampus = (blCampusId) => {
    this.setState({ blCampusId });
  };

  // 选择状态
  handleSelectStatus = (status) => {
    this.setState({ status });
  };

  // 查询
  handleSearch = () => {
    this.setState(
      {
        currentPage: 1,
      },
      () => {
        this.loadData();
      },
    );
  };

  // 重置
  handleReset = () => {
    this.setState(
      {
        status: null,
        posName: null,
        currentPage: 1,
        branchId: null,
        campusDict: [],
        posNumber: null,
        blCampusId: null,
        platformName: null,
      },
      () => {
        this.loadData();
      },
    );
  };

  // 添加
  handleAdd = () => {
    this.setState({
      visible: true,
      modelMode: 'add',
      initialValues: {},
    });
  };

  // 修改
  handleEdit = (record = {}) => {
    this.setState({
      visible: true,
      modelMode: 'edit',
      initialValues: record,
    });
  };

  // 切换状态
  handleChangeStatus = (record) => {
    const { id, status } = record;
    ShowConfirm({ 1: '确定要设置无效吗？', 0: '确定要设置有效吗?' }[status], async () => {
      this.props.onSetLoading(true);
      const result = await educationManagementPlatformApi.updateEducationPlatformStatus({
        id,
        status: { 1: 0, 0: 1 }[status],
      });
      this.props.onSetLoading(false);
      if (!result || result.resultCode !== 0) {
        return;
      }
      showMessage('设置成功', 'success');
      this.loadData();
    });
  };

  // 查看详情
  handleViewDetail = (record = {}) => {
    this.setState({
      visible: true,
      modelMode: 'view',
      initialValues: record,
    });
  };

  handlePaginationChange = (pageInfo) => {
    const { pageSize, current: currentPage } = pageInfo;
    this.setState(
      {
        pageSize,
        currentPage,
      },
      () => {
        this.loadData();
      },
    );
  };

  // 搜索
  generateSearchBox() {
    const { status, blCampusId, branchId, campusDict, companyDict, posNumber, platformName, posName } = this.state;

    return (
      <div className={styles.searchWrapper}>
        <div className={styles.searchFields}>
          {/*平台名称*/}
          <Input
            value={platformName}
            maxLength="200"
            placeholder="请输入平台名称"
            onChange={(e) => this.setState({ platformName: e.target.value })}
          />
          {/*终端名称*/}
          <Input
            value={posName}
            maxLength="200"
            placeholder="请输入终端名称"
            onChange={(e) => this.setState({ posName: e.target.value })}
          />
          {/*终端编码*/}
          <Input
            value={posNumber}
            maxLength="64"
            placeholder="请输入终端号"
            onChange={(e) => this.setState({ posNumber: e.target.value })}
          />
          {/*分公司*/}
          <Select
            showSearch
            value={branchId}
            placeholder="分公司"
            onChange={this.handleSelectCompany}
            dict={companyDict}
          ></Select>
          {/*学习中心*/}
          <Select
            showSearch
            value={blCampusId}
            placeholder="学习中心"
            onChange={this.handleSelectCampus}
            dict={campusDict}
          ></Select>
          {/*状态*/}
          <Select value={status} placeholder="状态" onChange={this.handleSelectStatus} dict={statusDict}></Select>
        </div>
      </div>
    );
  }

  // 工具栏
  generateToolbar() {
    return (
      <div className={styles.searchToolbar}>
        <Button style={{ marginRight: '10px' }} type="primary" onClick={this.handleSearch}>
          查询
        </Button>
        <Button style={{ marginRight: '10px' }} onClick={this.handleReset}>
          重置
        </Button>
        <Button type="primary" onClick={this.handleAdd}>
          新增
        </Button>
      </div>
    );
  }

  // 列表
  generateList() {
    const { list, total, pageSize, currentPage } = this.state;

    return (
      <List
        dataSource={list}
        columns={this.getColumns()}
        pagination={{
          pageSize,
          total: total,
          current: currentPage,
          pageSizeOptions: ['10', '20', '30', '40', '50'],
        }}
        onChange={this.handlePaginationChange}
      />
    );
  }

  // 弹框
  generateModal() {
    const props = {
      modelMode: this.state.modelMode,
      onCancel: this.handleCancelAdd,
      onAddSuccess: this.handleAddSuccess,
      companyDict: this.state.companyDict,
      initialValues: this.state.initialValues,
    };
    return this.state.visible && <EducationPlatformModal {...props} />;
  }

  render() {
    return (
      <div>
        {this.generateSearchBox()}
        {this.generateToolbar()}
        {this.generateList()}
        {this.generateModal()}
      </div>
    );
  }
}
