import React from 'react';

import { DatePicker } from 'antd';
import { List, Input, Select, Button } from 'COMPONENT';

import ExportResultModal from '../ExportResultModal';
import ConfirmMatchModal from '../ConfirmMatchModal';
import ImportExcelModal from '../ImportExcelModal';

import _ from 'lodash';
import moment from 'moment';
import FileUtil from '../../../../../utils/file';

import showMessage from 'SRC/components/utils/globalTips/showMessage';
import { errorsHandle } from 'SRC/utils/common';
import { selectOrgByIdAndTypeOption } from 'SERVICE/common';
import * as educationPlatformPaymentApi from 'SERVICE/educationPlatformPayment';

import styles from './style.module.less';

const { RangePicker } = DatePicker;
const matchingStatusDict = [
  { val: '', text: '请选择' },
  { val: 'NOT_YET_MATCH', text: '未匹配' },
  { val: 'SYSTEM_MATCH', text: '系统匹配' },
  { val: 'ARTIFICIAL_MATCH', text: '人工匹配' },
];

export default class DataList extends React.Component {
  state = {
    list: [],
    total: 0,
    pageSize: 30,
    currentPage: 1,

    blCampusId: null, // 分公司
    branchId: null, // 学习中心
    posNumber: null, // 终端号
    posId: null, // 参考支付
    amount: null, // 支付金额
    merchantName: null, // 商户名称
    matchingStatus: null, // 匹配状态
    currentDate: [moment(new Date()).subtract(1, 'months').startOf('month'), moment()], // 日期

    record: {},

    companyDict: [], // 分公司数据字典
    campusDict: [], // 学习中心数据字典

    exportModalVisible: false,
    confirmMatchModalVisible: false,
    importExcelModalVisible: false,
  };

  constructor(props) {
    super(props);
  }

  componentDidMount() {
    this.init();
  }

  init() {
    this.loadData();
    this.loadDict();
  }

  async loadCompanyDict() {
    const result = await selectOrgByIdAndTypeOption({
      orgType: 'BRENCH',
    }).catch(errorsHandle());

    if (!result || result.resultCode !== 0 || !result.value) {
      this.setState({ companyDict: [] });
      return;
    }

    const companyDict = _.map(result.value, (text, val) => ({ val, text })).filter(({ val }) => val);
    this.setState({ companyDict: [{ val: '', text: '请选择' }, ...companyDict] });
  }

  async loadCampusDict(orgId) {
    this.setState({ campusDict: [] });
    const result = await selectOrgByIdAndTypeOption({
      orgId,
      orgType: 'CAMPUS',
    }).catch(errorsHandle());

    if (!result || result.resultCode !== 0 || !result.value) {
      this.setState({ campusDict: [] });
      return;
    }

    const campusDict = _.map(result.value, (text, val) => ({ val, text })).filter(({ val }) => val);
    this.setState({ campusDict: [{ val: '', text: '请选择' }, ...campusDict] });
  }

  async loadData() {
    const {
      posId,
      amount,
      pageSize,
      branchId,
      blCampusId,
      posNumber,
      currentDate,
      merchantName,
      matchingStatus,
      currentPage: pageNum,
    } = this.state;
    this.props.onSetLoading(true);
    const result = await educationPlatformPaymentApi
      .getEduPayDataList({
        posId,
        amount,
        pageNum,
        pageSize,
        branchId,
        posNumber,
        blCampusId,
        merchantName,
        matchingStatus,
        posTimeEnd: moment(currentDate[1]).format('YYYYMMDD'),
        posTimeBegin: moment(currentDate[0]).format('YYYYMMDD'),
      })
      .catch(errorsHandle());
    this.props.onSetLoading(false);
    if (!result || result.resultCode !== 0) {
      this.setState({ list: [], total: 0 });
      return;
    }
    const {
      data: { total, list },
    } = result;
    this.setState({ total, list: list.map((item) => ({ ...item, key: item.id })) });
  }

  loadDict() {
    this.loadCompanyDict();
  }

  // 人工匹配
  handleMatch = (record) => {
    this.setState({ record, confirmMatchModalVisible: true });
  };

  // 选择分公司
  handleChangeCompany = (branchId) => {
    this.loadCampusDict(branchId);
    this.setState({ branchId, blCampusId: null });
  };

  // 选择学习中心
  handleChangeCampus = (blCampusId) => {
    this.setState({ blCampusId });
  };

  // 选择匹配状态
  handleChangeMatchingStatus = (matchingStatus) => {
    this.setState({ matchingStatus });
  };

  // 分页处理
  handlePaginationChange = (pageInfo) => {
    const { pageSize, current: currentPage } = pageInfo;
    this.setState(
      {
        pageSize,
        currentPage,
      },
      () => {
        this.loadData();
      },
    );
  };

  // 查询
  handleSearch = () => {
    this.loadData();
  };

  // 重置
  handleReset = () => {
    this.setState(
      {
        total: 0,
        currentPage: 1,
        blCampusId: null, // 分公司
        branchId: null, // 学习中心
        posNumber: null, // 终端号
        posId: null, // 参考支付
        amount: null, // 支付金额
        merchantName: null, // 商户名称
        matchingStatus: null, // 匹配状态
        currentDate: [moment(new Date()).subtract(1, 'months').startOf('month'), moment()], // 日期
      },
      () => {
        this.loadData();
      },
    );
  };

  // 导入
  handleImport = () => {
    this.setState({ importExcelModalVisible: true });
  };

  // 导出结果
  handleExportResult = () => {
    this.setState({ exportModalVisible: true });
  };

  // 导出Excel
  handleExportExcel = async () => {
    if (!this.state.total) {
      showMessage('没有数据，无法导出', 'error');
      return;
    }
    if (this.state.total > 5000) {
      showMessage('数据量已超过最大导出数量5000条，请缩小范围查询再导出', 'error');
      return;
    }

    const { posId, amount, branchId, blCampusId, posNumber, currentDate, merchantName, matchingStatus } = this.state;
    const [posTimeBegin, posTimeEnd] = currentDate;

    this.props.onSetLoading(true);
    const result = await educationPlatformPaymentApi
      .exportEduPayPageDataExcel({
        posId,
        amount,
        branchId,
        blCampusId,
        posNumber,
        pageNum: 1,
        merchantName,
        matchingStatus,
        pageSize: 5000,
        posTimeEnd: moment(posTimeEnd).format('YYYYMMDD'),
        posTimeBegin: moment(posTimeBegin).format('YYYYMMDD'),
      })
      .catch(errorsHandle());
    this.props.onSetLoading(false);

    if (!result || result.resultCode !== 0) {
      return;
    }
    FileUtil.downloadFile(result.data);
  };

  handleCloseExportModal = () => {
    this.setState({ exportModalVisible: false });
  };

  handleCloseConfirmMatchModal = () => {
    this.setState({ confirmMatchModalVisible: false });
  };

  handleCloseImportExcelModal = () => {
    this.setState({ importExcelModalVisible: false });
  };

  handleImportSuccess = () => {
    this.loadData();
    this.setState({ importExcelModalVisible: false });
  };

  handleChangeDate = (currentDate) => {
    this.setState({ currentDate });
  };

  // 人工匹配成功
  handleConfirmMatchSuccess = () => {
    this.loadData();
    this.handleCloseConfirmMatchModal();
  };

  getColumns() {
    return [
      { title: '分公司', width: 120, dataIndex: 'branchName' },
      { title: '学习中心', width: 150, dataIndex: 'blCampusName' },
      { title: '终端号', width: 100, dataIndex: 'posNumber' },
      { title: '商户名称', width: 200, dataIndex: 'merchantName' },
      { title: '支付参考号', width: 100, dataIndex: 'posId' },
      { title: '支付金额（元）', width: 120, dataIndex: 'amount' },
      { title: '支付日期', width: 100, dataIndex: 'posTime', render: (value) => moment(value).format('YYYY-MM-DD') },
      {
        title: '导入时间',
        width: 150,
        dataIndex: 'createTime',
        render: (value) => moment(value).format('YYYY-MM-DD HH:mm:ss'),
      },
      { title: '导入人', width: 85, dataIndex: 'createUserName' },
      {
        title: '匹配状态',
        width: 85,
        dataIndex: 'matchingStatus',
        render: (value) => matchingStatusDict.find(({ val }) => val === value).text,
      },
      { title: '备注', width: 200, dataIndex: 'remark' },
      {
        title: '操作',
        fixed: 'right',
        key: 'action',
        getActionConfig: (text, record, index) => {
          // 匹配状态为未匹配显示人工匹配按钮
          return record.matchingStatus == 'NOT_YET_MATCH'
            ? [{ text: '人工匹配', onClick: () => this.handleMatch(record) }]
            : [];
        },
      },
    ];
  }

  // 搜索
  generateSearchBox() {
    return (
      <div className={styles.searchWrapper}>
        <div className={styles.searchFields}>
          {this.generateDate()}
          {this.generateCompanySelect()}
          {this.generateCampusSelect()}
          {this.generatePosNumber()}
          {this.generatePosId()}
          {this.generateAmount()}
          {this.generateMerchantName()}
        </div>
      </div>
    );
  }

  // 日期
  generateDate() {
    const props = {
      size: 'large',
      allowClear: false,
      style: { width: 300 },
      format: 'YYYY-MM-DD',
      value: this.state.currentDate,
      onChange: this.handleChangeDate,
      placeholder: ['开始时间', '结束时间'],
    };
    return <RangePicker {...props} />;
  }

  // 公司
  generateCompanySelect() {
    const props = {
      showSearch: true,
      style: { width: 200 },
      placeholder: '分公司',
      value: this.state.branchId,
      dict: this.state.companyDict,
      onChange: this.handleChangeCompany,
    };
    return <Select {...props} />;
  }

  // 学习中心
  generateCampusSelect() {
    const props = {
      showSearch: true,
      style: { width: 200 },
      placeholder: '学习中心',
      value: this.state.blCampusId,
      dict: this.state.campusDict,
      onChange: this.handleChangeCampus,
    };
    return <Select {...props} />;
  }

  // 终端号
  generatePosNumber() {
    return (
      <Input
        value={this.state.posNumber}
        placeholder="请输入终端号"
        onChange={(e) => this.setState({ posNumber: e.target.value })}
      />
    );
  }

  // 参考支付
  generatePosId() {
    return (
      <Input
        value={this.state.posId}
        placeholder="请输入支付参考"
        onChange={(e) => this.setState({ posId: e.target.value })}
      />
    );
  }

  // 支付金额
  generateAmount() {
    return (
      <Input
        value={this.state.amount}
        placeholder="请输入支付金额"
        onChange={(e) => this.setState({ amount: e.target.value })}
      />
    );
  }

  // 商户
  generateMerchantName() {
    return (
      <Input
        value={this.state.merchantName}
        placeholder="请输入商户名称"
        onChange={(e) => this.setState({ merchantName: e.target.value })}
      />
    );
  }

  // 匹配状态
  generateMatchingStatusSelect() {
    const props = {
      showSearch: true,
      dict: matchingStatusDict,
      placeholder: '匹配状态',
      value: this.state.matchingStatus,
      onChange: this.handleChangeMatchingStatus,
      style: { width: '13.9%' },
    };
    return <Select {...props} />;
  }

  // 工具栏
  generateToolbar() {
    return (
      <div className={styles.searchToolbar}>
        {this.generateMatchingStatusSelect()}
        <Button type="primary" onClick={this.handleSearch}>
          查询
        </Button>
        <Button onClick={this.handleReset}>重置</Button>
        <Button type="primary" onClick={this.handleImport}>
          数据导入
        </Button>
        <Button type="primary" onClick={this.handleExportResult}>
          导入结果
        </Button>
        <Button onClick={this.handleExportExcel}>导出Excel</Button>
      </div>
    );
  }

  // 列表
  generateList() {
    const { list, total, pageSize, currentPage } = this.state;

    return (
      <List
        dataSource={list}
        columns={this.getColumns()}
        pagination={{
          total,
          pageSize,
          current: currentPage,
          pageSizeOptions: ['10', '20', '30', '40', '50'],
        }}
        onChange={this.handlePaginationChange}
      />
    );
  }

  // 导出Excel弹框
  generateExportModal() {
    const props = {
      onCancel: this.handleCloseExportModal,
      visible: this.state.exportModalVisible,
    };
    return this.state.exportModalVisible && <ExportResultModal {...props} />;
  }

  // 确定人工匹配
  generateConfirmMatchModal() {
    const props = {
      id: this.state.record.id,
      onCancel: this.handleCloseConfirmMatchModal,
      visible: this.state.confirmMatchModalVisible,
      onConfirmMatchSuccess: this.handleConfirmMatchSuccess,
    };
    return this.state.confirmMatchModalVisible && <ConfirmMatchModal {...props} />;
  }

  generateImportExcelModal() {
    const props = {
      onImportSuccess: this.handleImportSuccess,
      onCancel: this.handleCloseImportExcelModal,
      visible: this.state.importExcelModalVisible,
    };
    return this.state.importExcelModalVisible && <ImportExcelModal {...props} />;
  }

  render() {
    return (
      <div>
        {this.generateSearchBox()}
        {this.generateToolbar()}
        {this.generateList()}
        {this.generateExportModal()}
        {this.generateConfirmMatchModal()}
        {this.generateImportExcelModal()}
      </div>
    );
  }
}
