import React from 'react';

import { Form, Input, Radio, Modal, Select, Checkbox } from 'COMPONENT';

import { runFunc, errorsHandle } from 'SRC/utils/common';

import showMessage from 'SRC/components/utils/globalTips/showMessage';

import { getSelectOption } from 'SERVICE/common';
import * as Api from 'SERVICE/industrialRelConfig';

const { Item: FormItem } = Form;
const { Group: RadioGroup } = Radio;
const { Group: CheckboxGroup } = Checkbox;

const formItemProps = {
  labelCol: { span: 6 },
  wrapperCol: { span: 16 },
};

const courseTypeInitValue = 'SMALL_CLASS'; // 课程类型初始值
const payInitValue = 'A'; // 支付模块初始值

const Utils = {
  map2Array(map, hasEmptyValue) {
    const array = [];

    for (const key in map) {
      if (key && Object.prototype.hasOwnProperty.call(map, key)) {
        array.push({ val: key, text: map[key] });
      }
    }

    if (hasEmptyValue) {
      array.unshift({ val: '', text: '全部' });
    }

    return array;
  },
};

export default class AddConfRelModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      treeData: [],
      loading: false,
      classTypeDict: [],
      courseSeriesDict: [],
      productGroupDict: [],
      selectedCourseType: 'SMALL_CLASS',
    };
    this.formRef = null;
  }

  componentDidMount() {
    this.loadDictByCourseType(courseTypeInitValue);
    this.formRef.setFieldsValue({
      classTypeId: '',
      courseSeriesId: '',
      category: courseTypeInitValue,
      contractPayCategory: payInitValue,
      campusName: this.props.campusName,
    });
  }

  get isShowCourseSeries() {
    return !!{ SMALL_CLASS: 1, ONE_ON_ONE_COURSE: 1, ONE_ON_MANY: 1 }[this.state.selectedCourseType];
  }

  get isShowClassType() {
    return !!{ SMALL_CLASS: 1 }[this.state.selectedCourseType];
  }

  get isShowProductGroup() {
    return !!{ ONE_ON_ONE_COURSE: 1, ONE_ON_MANY: 1 }[this.state.selectedCourseType];
  }

  loadDictByCourseType(courseType) {
    if (!courseType) {
      return this.clearDict();
    }
    Promise.all([
      getSelectOption({ parentId: courseType, selectOptionCategory: 'PRODUCT_GROUP' }),
      getSelectOption({ parentId: courseType, selectOptionCategory: 'COURSE_SERIES' }),
      getSelectOption({ parentId: courseType, selectOptionCategory: 'CLASS_TYPE' }),
    ])
      .then((res) => {
        const [productGroupDict, courseSeriesDict, classTypeDict] = res.map(({ value }) =>
          Utils.map2Array(value, true),
        );
        this.setState({
          classTypeDict,
          courseSeriesDict,
          productGroupDict: courseType === 'SMALL_CLASS' ? [] : productGroupDict,
        });
      })
      .catch(() => {
        this.clearDict();
      });
  }

  clearDict() {
    this.setState({
      classTypeDict: [],
      courseSeriesDict: [],
      productGroupDict: [],
    });
  }

  handleSubmit = () => {
    runFunc(this.formRef.submitFields);
  };

  handleSubmitFormFail = () => {};

  handleSubmitForm = async (values) => {
    this.setState({ loading: true });
    const result = await Api.savePayCategorys({
      ...values,
      campusId: this.props.campusId,
    }).catch(errorsHandle());
    this.setState({ loading: false });

    if (result && result.resultCode === 0) {
      showMessage('添加成功', 'success');
      this.props.onAddSuccess();
    }
  };

  handleSelectCourseType = (e) => {
    const selectedCourseType = e.target.value;
    this.setState({ selectedCourseType }, () => {
      const defaultValues = {};
      if (this.isShowClassType) {
        defaultValues.classTypeId = '';
      }
      if (this.isShowCourseSeries) {
        defaultValues.courseSeriesId = '';
      }
      if (this.isShowProductGroup) {
        defaultValues.productGroupId = '';
      }
      this.formRef.setFieldsValue(defaultValues);
    });
    this.loadDictByCourseType(selectedCourseType);
  };

  handleSelectAllGrade = (e) => {
    this.formRef.setFieldsValue({
      gradeIds: e.target.checked ? this.props.dicts.studentGradeDict.map(({ val }) => val) : null,
    });
  };

  generateModal() {
    const props = {
      width: 800,
      okBtnText: '保存',
      title: '工商主体关系配置',
      onOk: this.handleSubmit,
      visible: this.props.visible,
      loading: this.state.loading,
      onCancel: this.props.onCancel,
    };

    return <Modal {...props}>{this.generateForm()}</Modal>;
  }

  generateForm() {
    const props = {
      ref: (c) => (this.formRef = c),
      onSubmit: this.handleSubmitForm,
      onSubmitFail: this.handleSubmitFormFail,
    };
    return (
      <Form {...props}>
        {this.generateCampusFormItem()}
        {this.generateCourseTypeFormItem()}
        {this.generatePayFormItem()}
        {this.isShowCourseSeries && this.generateCourseSeriesFormItem()}
        {this.isShowClassType && this.generateClassTypeFormItem()}
        {this.isShowProductGroup && this.generateProductGroupFormItem()}
        {this.generateStudentGradeFormItem()}
      </Form>
    );
  }

  // 学习中心
  generateCampusFormItem() {
    const itemProps = {
      key: 'campusName',
      name: 'campusName',
      formItemProps: {
        colon: false,
        label: '学习中心',
        ...formItemProps,
      },
    };

    return (
      <FormItem {...itemProps}>
        <Input disabled={true} />
      </FormItem>
    );
  }

  // 课程类型
  generateCourseTypeFormItem() {
    const { courseTypeDict } = this.props.dicts;
    const itemProps = {
      key: 'category',
      name: 'category',
      formItemProps: {
        colon: false,
        label: '课程类型',
        ...formItemProps,
      },
    };

    return (
      <FormItem {...itemProps}>
        <RadioGroup onChange={this.handleSelectCourseType}>
          {courseTypeDict.map((item) => (
            <Radio value={item.val} key={item.val}>
              {item.text}
            </Radio>
          ))}
        </RadioGroup>
      </FormItem>
    );
  }

  // 支付模块
  generatePayFormItem() {
    const { payDict } = this.props.dicts;
    const itemProps = {
      key: 'contractPayCategory',
      name: 'contractPayCategory',
      formItemProps: {
        colon: false,
        label: '支付模块',
        ...formItemProps,
      },
    };

    return (
      <FormItem {...itemProps}>
        <RadioGroup>
          {payDict.map((item) => (
            <Radio value={item.val} key={item.val}>
              {item.text}
            </Radio>
          ))}
        </RadioGroup>
      </FormItem>
    );
  }

  // 产品系列
  generateCourseSeriesFormItem() {
    const { courseSeriesDict } = this.state;
    const itemProps = {
      key: 'courseSeriesId',
      name: 'courseSeriesId',
      formItemProps: {
        colon: false,
        label: '产品系列',
        ...formItemProps,
      },
    };

    return (
      <FormItem {...itemProps}>
        <Select placeholder="请选择" dict={courseSeriesDict} showSearch />
      </FormItem>
    );
  }

  // 产品类型
  generateClassTypeFormItem() {
    const { classTypeDict } = this.state;
    const itemProps = {
      key: 'classTypeId',
      name: 'classTypeId',
      formItemProps: {
        colon: false,
        label: '产品班级类型',
        ...formItemProps,
      },
    };

    return (
      <FormItem {...itemProps}>
        <Select placeholder="请选择" dict={classTypeDict} showSearch />
      </FormItem>
    );
  }

  // 产品类型
  generateProductGroupFormItem() {
    const { productGroupDict } = this.state;
    const itemProps = {
      key: 'productGroupId',
      name: 'productGroupId',
      formItemProps: {
        colon: false,
        label: '产品组',
        ...formItemProps,
      },
    };

    return (
      <FormItem {...itemProps}>
        <Select placeholder="请选择" dict={productGroupDict} showSearch />
      </FormItem>
    );
  }

  // 合同年级
  generateStudentGradeFormItem() {
    const { studentGradeDict } = this.props.dicts;
    const itemProps = {
      key: 'gradeIds',
      name: 'gradeIds',
      rules: [{ required: true, message: '合同年级不能为空' }],
      formItemProps: {
        colon: false,
        label: (
          <div style={{ display: 'inline-block' }}>
            合同年级<Checkbox onChange={this.handleSelectAllGrade}>全选</Checkbox>
          </div>
        ),
        ...formItemProps,
      },
    };

    return (
      <FormItem {...itemProps}>
        <CheckboxGroup>
          {studentGradeDict.map((item) => (
            <Checkbox value={item.val} key={item.val}>
              {item.text}
            </Checkbox>
          ))}
        </CheckboxGroup>
      </FormItem>
    );
  }

  render() {
    return this.generateModal();
  }
}
