import React from 'react';
import PropTypes from 'prop-types';
import { Modal, Button, PopoverWrap } from 'COMPONENT';
import { Radio } from 'antd';
import ContractProductPayment from '../../BLoComponets/ContractProductPayment';
import { invalidFilter } from '../../BLoComponets/ContractProductPayment/utils';
import {
  getContractByIds,
  batchConfirmingContractAndSharePay,
  batchConfirmingContract,
  batchDepositUsed,
  getSegmentChargeItemByContractIds,
} from 'SERVICE/contract';
import { errorsHandle, runFunc } from 'UTIL/common';
import style from './styles.module.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import UseDeposit from '../../BLoComponets/UseDeposit';
import NP from 'number-precision';
import Statistics from '../../BLoComponets/ContractProductPayment/Statistics';

const RadioGroup = Radio.Group;

class ConfirmContract extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.loadData = this.loadData.bind(this);
    this.onRadioChange = this.onRadioChange.bind(this);
    this.onPaymentChange = this.onPaymentChange.bind(this);
    this.submit = this.submit.bind(this);
    this.confirm = this.confirm.bind(this);
    this.confirmAndPay = this.confirmAndPay.bind(this);
    this.saveDepositThenConfirm = this.saveDepositThenConfirm.bind(this);
    this.onDepositChange = this.onDepositChange.bind(this);

    this.state = {
      loading: false,
      canPay: true,
      radio: true,
      list: [],
      transactionAmount: 0,
      depositAmount: 0,
      depositSelected: [],
      // 统计值
      totalStatistics: 0,
      paidStatistics: 0,
      promotionStatistics: 0,

      contractProductPaymentKey: 1,
    };

    this.useDepositRef = null;
    this.paymentRef = null;
  }

  static propTypes = {
    contractIds: PropTypes.arrayOf(PropTypes.string).isRequired,
    studentId: PropTypes.string.isRequired,
    onCancel: PropTypes.func.isRequired,
    onSuccess: PropTypes.func,
    useDeposit: PropTypes.bool,
  };

  static defaultProps = {
    useDeposit: false,
  };

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  async loadData(resolve) {
    const res = await getContractByIds({ contractIds: this.props.contractIds }).catch(errorsHandle());
    if (res && res.data) {
      const list = invalidFilter(res.data);

      // 待分配金额均为0才可支付
      let canPay = list.every((item) => item.availableAmount === 0);
      if (canPay) {
        // 待支付全部为0的，无需支付
        canPay = list.some((contract) => {
          return contract.contractProductArr.some((prd) => prd.pendingAmount);
        });
      }

      // 获取分段收费相关设置
      const response = await getSegmentChargeItemByContractIds({ contractIds: this.props.contractIds }).catch(
        errorsHandle(),
      );
      if (response && response.data) {
        const tempContractIds = Object.keys(response.data);
        list.forEach((item) => {
          if (tempContractIds.indexOf(item.contractId) > -1) {
            item.segmentChargeItems = response['data'][item.contractId];
          }
        });
      }

      this.setState(
        {
          list,
          canPay,
          radio: canPay,
          contractProductPaymentKey: this.state.contractProductPaymentKey + 1,
          ...this.calcStatistics(list),
        },
        () => {
          this.paymentRef.refreshValue();
        },
      );
    }
    resolve();
  }

  handleReload = () => {
    return new Promise(this.loadData);
  };

  calcStatistics = (list) => {
    let result = {
      totalStatistics: 0,
      paidStatistics: 0,
      promotionStatistics: 0,
    };

    list.forEach((contract) => {
      contract.contractProductArr.forEach((item) => {
        result.totalStatistics = NP.plus(result.totalStatistics, item.totalAmount);
        result.paidStatistics = NP.plus(result.paidStatistics, item.paidAmount);
        result.promotionStatistics = NP.plus(result.promotionStatistics, item.promotionAmount);
      });
    });

    return result;
  };

  onRadioChange(e) {
    this.setState({ radio: e.target.value }, () => {
      this.paymentRef.refreshValue();
    });
  }

  onPaymentChange(list, total) {
    this.setState(
      {
        list,
        transactionAmount: total,
      },
      () => {
        if (total === 0) {
          this.useDepositRef && this.useDepositRef.reset();
        }
      },
    );
  }

  submit() {
    if (this.state.radio) {
      if (this.state.depositAmount && NP.minus(this.state.transactionAmount, this.state.depositAmount) === 0) {
        return this.saveDepositThenConfirm();
      }
      this.confirmAndPay();
    } else {
      this.confirm();
    }
  }

  async confirm() {
    this.setState({ loading: true });

    let apiParams = {
      contractIds: this.state.list.map((item) => item.contractId),
    };

    const res = await batchConfirmingContract(apiParams).catch(errorsHandle());

    if (res && res.data) {
      return runFunc(this.props.onSuccess, res.data, 'confirm');
    }

    this.setState({ loading: false });
  }

  async confirmAndPay() {
    if (this.paymentRef.validate()) {
      this.setState({ loading: true });

      let apiParams = {
        studentId: this.props.studentId,
        depositIds: this.state.depositSelected,
        transactionAmount: this.paymentRef.getTotalAmount(),
        contractProductPaymentList: this.paymentRef.getValue(),
      };

      const res = await batchConfirmingContractAndSharePay(apiParams).catch(errorsHandle());

      if (res && res.data) {
        return runFunc(this.props.onSuccess, res.data, 'confirmPay');
      }

      this.setState({ loading: false });
    } else {
      showMessage('本次收款金额不能为0！');
    }
  }

  async saveDepositThenConfirm() {
    this.setState({ loading: true });

    let apiParams = {
      needConfirm: true,
      studentId: this.props.studentId,
      depositIds: this.state.depositSelected,
      transactionAmount: this.paymentRef.getTotalAmount(),
      contractProductPaymentList: this.paymentRef.getValue(),
    };

    const res = await batchDepositUsed(apiParams).catch(errorsHandle());

    if (res && res.data) {
      return runFunc(this.props.onSuccess, res.data.confirmBatchId, 'confirm');
    }

    this.setState({ loading: false });
  }

  onDepositChange(depositSelected, depositAmount) {
    this.setState({
      depositAmount,
      depositSelected,
    });
  }

  getOkBtnText = (radio, transactionAmount, depositAmount) => {
    if (radio) {
      return depositAmount && NP.minus(transactionAmount, depositAmount) === 0
        ? '保存支付并确认信息'
        : '确认信息并支付';
    } else {
      return '确认信息';
    }
  };

  render() {
    const { props, state } = this;

    return (
      <Modal
        title="确认信息"
        loading={state.loading}
        onCancel={props.onCancel}
        widthSize="XL"
        footer={
          <div className={style.footer}>
            <div className={style.footerMain}>
              {state.radio && (
                <Statistics
                  totalAmount={state.totalStatistics}
                  paidAmount={state.paidStatistics}
                  promotionAmount={state.promotionStatistics}
                  transactionAmount={state.transactionAmount}
                  depositAmount={state.depositAmount}
                />
              )}
            </div>

            <div className={style.footerCtrl}>
              <PopoverWrap
                visible={
                  state.transactionAmount > 0 &&
                  this.useDepositRef &&
                  this.useDepositRef.getDepositList().length > 0 &&
                  state.depositAmount === 0
                }
                content="该学生名下有未使用定金券，请确认本次支付是否要使用？"
                type="warning"
                overlayStyle={{ width: 220 }}
              >
                <Button onClick={this.submit} type="primary">
                  {this.getOkBtnText(state.radio, state.transactionAmount, state.depositAmount)}
                </Button>
              </PopoverWrap>
            </div>
          </div>
        }
      >
        <div>
          {state.canPay && (
            <div style={{ marginBottom: state.radio ? 0 : 12 }}>
              <RadioGroup value={state.radio} onChange={this.onRadioChange}>
                <Radio value={true}>支付尾款</Radio>
                <Radio value={false}>仅确认信息</Radio>
              </RadioGroup>
            </div>
          )}
        </div>

        <ContractProductPayment
          key={this.state.contractProductPaymentKey}
          className={style.contractProductPaymentTop}
          headerTailSlot={state.depositSelected.length > 0 && <span>【本次收款金额】已包含定金券金额</span>}
          subSlot={
            state.radio && props.useDeposit ? (
              <UseDeposit
                studentId={props.studentId}
                ref={(c) => (this.useDepositRef = c)}
                amount={state.transactionAmount}
                onChange={this.onDepositChange}
                labelSlot={<span className={style.depositLabel}>已预付定金</span>}
                selectWidth={260}
                size="xSmall"
                disabledTips="收款金额必需大于0"
              />
            ) : undefined
          }
          bodyClassName={style.maxHeight}
          ref={(c) => (this.paymentRef = c)}
          list={state.list}
          onReload={this.handleReload}
          onChange={this.onPaymentChange}
          radioVisible={state.radio}
          autoAssignVisible={state.radio}
          inputVisible={state.radio}
          hasPayAllBtn={(state.list || []).length === 1}
        />
      </Modal>
    );
  }
}

export default ConfirmContract;
