import React from 'react';
import { Row, Col } from 'antd';
import { debounce } from 'lodash';
import { List, Select, TextLink } from 'COMPONENT';
import { loginUser } from 'UTIL/common';
import { getUserForPerformance } from 'SERVICE/contract';
import { responsibleTypeDict } from '../constant';
import * as utils from './utils';

/**
 * 合同产品列表
 *
 * @param props.list {Array} 产品列表
 * @param props.onChange {function} 产品列表修改
 * @param props.onShowRefundModal {function} 打开课程扣费记录弹窗
 */
class ProductList extends React.Component {
  static defaultProps = {
    list: [],
    onChange: () => {},
    onShowRefundModal: () => {},
  };

  constructor(props) {
    super(props);

    this.isTwoTeacher = loginUser.isTwoTeacherSym();
    this.columns = this.formatColumns();
    this.dictVersion = 0; // 搜索计数

    this.state = {
      productList: [], // 产品列表
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化
   */
  initContent = () => {
    this.setState({
      productList: this.props.list || [],
    });
  };

  /**
   * 格式化表格列
   *
   * @return {*[]}
   */
  formatColumns = () => {
    let columns = [
      {
        title: '序号',
        dataIndex: 'index',
        width: 60,
        render: (text, record, index) => {
          return index + 1;
        },
      },
      { title: '产品名称', dataIndex: 'productionName', width: 100 },
      {
        title: '可退金额',
        dataIndex: 'refundableAmount',
        width: 80,
        render: (text, record) => {
          const disabled = utils.calcIsOthersProduct(record.productType);

          return (
            <TextLink
              disabled={disabled}
              onClick={() => {
                this.onShowRefundModal(record);
              }}
            >
              {text}
            </TextLink>
          );
        },
      },
    ];

    if (this.isTwoTeacher) {
      columns = columns.concat([
        {
          title: '退费科次责任人*',
          dataIndex: 'responsible',
          width: 520,
          render: (text, record, index) => {
            const responsibleIdDictKey = utils.formatPersonDictStateKey('responsibleId', index);
            const counselorIdDictKey = utils.formatPersonDictStateKey('counselorId', index);
            const disabled = utils.calcNotValidRecord(record);

            return (
              <Row gutter={10}>
                <Col span={5}>
                  <Select
                    placeholder="岗位"
                    dict={responsibleTypeDict}
                    value={record.responsibleType}
                    onChange={(val) => {
                      this.onSelectChange(val, index, 'responsibleType');
                    }}
                    disabled={disabled}
                  />
                </Col>
                <Col span={7}>
                  <Select
                    placeholder="责任人"
                    dict={this.state[responsibleIdDictKey] || []}
                    value={record.responsibleId}
                    onChange={(val) => {
                      this.onSelectChange(val, index, 'responsibleId');
                    }}
                    showSearch={true}
                    noOptionTips="没有找到"
                    searchPlaceholder="输入名字搜索"
                    onSearchKeyChange={(val) => this.onSearchUser(val, index, 'responsibleId')}
                    disabled={disabled}
                  />
                </Col>
                <Col span={5}>
                  <Select
                    placeholder="岗位"
                    dict={responsibleTypeDict}
                    value={record.counselorType}
                    onChange={(val) => {
                      this.onSelectChange(val, index, 'counselorType');
                    }}
                    disabled={disabled}
                  />
                </Col>
                <Col span={7}>
                  <Select
                    placeholder="责任人"
                    dict={this.state[counselorIdDictKey] || []}
                    value={record.counselorId}
                    onChange={(val) => {
                      this.onSelectChange(val, index, 'counselorId');
                    }}
                    showSearch={true}
                    noOptionTips="没有找到"
                    searchPlaceholder="输入名字搜索"
                    onSearchKeyChange={(val) => this.onSearchUser(val, index, 'counselorId')}
                    disabled={disabled}
                  />
                </Col>
              </Row>
            );
          },
        },
      ]);
    }

    return columns;
  };

  /**
   * 下拉选择框值修改
   *
   * @param val {string} 值
   * @param index {number} 产品列表索引
   * @param key {string} 保存字段
   */
  onSelectChange = (val, index, key) => {
    let list = this.state.productList;

    list[index][key] = val;

    this.setState({ productList: list });
    this.props.onChange(list);
  };

  /**
   * 搜索用户
   *
   * @param val {string}
   * @param index {number} 产品列表索引
   * @param key {string} 保存字段
   */
  onSearchUser = (val, index, key) => {
    debounce(this.doSearchUser, 300)(val, index, key);
  };

  /**
   * 执行搜索
   * tips: 每一行的用户下拉框一个dict，用 index 后缀区分
   *
   * @param val
   * @param index
   * @param key
   */
  doSearchUser = async (val, index, key) => {
    const personDictKey = utils.formatPersonDictStateKey(key, index);

    if (val) {
      const dictVersion = ++this.dictVersion;

      try {
        const dict = await getUserForPerformance({ term: val });

        if (this.dictVersion === dictVersion) {
          this.setState({
            [personDictKey]: dict.map((item) => ({ text: item.label, val: item.value, title: item.label })),
          });
        }
      } catch (e) {}
    } else {
      this.dictVersion++;
      this.setState({
        // [personDictKey]: [],
      });
    }
  };

  /**
   * 课程扣费记录详情弹窗
   *
   * @param record {Object} 产品
   */
  onShowRefundModal = (record) => {
    this.props.onShowRefundModal(record.id, record.productType);
  };

  render() {
    return <List dataSource={this.state.productList} columns={this.columns} />;
  }
}

export default ProductList;
