import React from 'react';
import { browserHistory } from 'dva/router';
import { Row, Col } from 'antd';
import { Modal, OldModal, TextArea, FormItem } from 'COMPONENT';
import { loginUser } from 'UTIL/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import styles from './index.less';
import * as loaders from './loaders';
import * as utils from './utils';
import ProductList from './ProductList';
import FooterTips from './FooterTips';
import { calcNotValidRecord } from './ProductList/utils';

/**
 * 结束合同弹窗
 *
 * @param props.contractId {string} 合同ID
 */
class EndContractModal extends React.Component {
  constructor(props) {
    super(props);

    this.isTwoTeacher = loginUser.isTwoTeacherSym();

    this.state = {
      loading: false,
      contractVo: null, // 合同详情
      productList: null, // 合同产品列表
      changeProductList: null, // 修改后的产品列表（双师使用，注意初始未修改的情况）
      reason: null, // 原因

      modalRefundShow: false, // 是否显示课程扣费详情弹窗
      modalProductId: null, // 弹窗产品ID
      modalProductType: null, // 弹窗产品类型
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化合同详情
   */
  initContent = async () => {
    const { contractId } = this.props.oldPageParams || {};

    if (!contractId) {
      return;
    }

    this.toggleLoading();
    const contractVo = await loaders.loadContractDetail(contractId).catch(() => this.toggleLoading());
    const productList = contractVo ? contractVo.contractProductVos : [];

    this.toggleLoading();
    this.setState({
      contractVo,
      productList,
    });
  };

  /**
   * 产品列表修改（双师系统会触发）
   *
   * @param list {Array} 修改后的产品列表
   */
  onProductListChange = (list) => {
    this.setState({ changeProductList: list });
  };

  /**
   * 显示课程扣费记录弹窗（打开旧boss弹窗）
   * tips: 需要注意旧弹窗z-index层级修改，避免旧弹窗被覆盖
   *
   * @param productId {string} 合同产品ID（不是产品ID）
   * @param productType {string} 产品类型
   */
  onShowRefundModal = (productId, productType) => {
    this.setState(
      {
        modalProductId: productId,
        modalProductType: productType,
      },
      () => {
        this.toggleModalRefundShow();
      },
    );
  };

  /**
   * 提交
   */
  onSubmit = () => {
    const params = this.formatParams();

    if (!params.remark) {
      showMessage('请填写原因！');
      return;
    }

    if (this.isTwoTeacher) {
      const { contractProducts } = params;
      const canSubmit = contractProducts.every((item) => {
        if (item.notValidate) {
          return true;
        }

        return (item.responsibleType && item.responsibleId) || (item.counselorType && item.counselorId);
      });

      if (!canSubmit) {
        showMessage('每个合同产品至少选择一个责任人与岗位!');
        return;
      }
    }

    this.doSubmit(params);
  };

  doSubmit = (params) => {
    this.toggleLoading();
    loaders
      .doFinishContract(params)
      .then(() => {
        this.toggleLoading();
        showMessage('操作成功！', 'success');
        this.props.onClose();
        this.refreshContractList();
      })
      .catch(() => this.toggleLoading());
  };

  /**
   * 格式化请求参数
   * tips: 岗位与责任人字段仅双师系统使用
   * tips: 岗位字段需要转成整型传给后端
   *
   * @return {{contractId, contractConfirmStatus, remark: (null|*), contractProducts: Array}}
   */
  formatParams = () => {
    const { contractId, contractConfirmStatus } = this.props.oldPageParams || {};
    const { productList, changeProductList, reason } = this.state;
    const resultList = this.isTwoTeacher ? changeProductList || productList : productList;
    const contractProducts = resultList.map((item) => {
      const { id, refundableAmount, responsibleType, responsibleId, counselorType, counselorId } = item;
      const notValidate = calcNotValidRecord(item);

      return {
        contractProductId: id,
        refundableAmount,
        responsibleType: Number(responsibleType),
        responsibleId,
        counselorType: Number(counselorType),
        counselorId,
        notValidate,
      };
    });

    return {
      contractId,
      contractConfirmStatus,
      contractProducts,
      remark: reason,
    };
  };

  /**
   * 暴力刷新合同列表页面
   */
  refreshContractList = () => {
    browserHistory.push('/oldPage?src=function/contract/listContract.html?t=' + Date.now());
  };

  /**
   * 显示/隐藏loading
   */
  toggleLoading = () => {
    this.setState((prevState) => {
      return {
        loading: !prevState.loading,
      };
    });
  };

  /**
   * 显示/隐藏课程扣费记录弹窗
   */
  toggleModalRefundShow = () => {
    this.setState((prevState) => {
      return {
        modalRefundShow: !prevState.modalRefundShow,
      };
    });
  };

  render() {
    const { productList } = this.state;

    return (
      <Modal
        title="结束合同（剩余资金退回电子账户）"
        loading={this.state.loading}
        width={860}
        onClose={this.props.onClose}
        onOk={this.onSubmit}
        className={styles.endContractModal}
      >
        <div className={styles.title}>
          <span>退回电子账户金额</span>
          <span className={styles.totalAmount}>{utils.calcTotalRefundableAmount(productList)}</span>
        </div>

        {productList && productList.length > 0 && (
          <ProductList
            list={productList}
            onChange={this.onProductListChange}
            onShowRefundModal={this.onShowRefundModal}
          />
        )}

        <Row className={styles.reason}>
          <Col span={24}>
            <FormItem name="原因:" required={true} labelWidth={80} style={{ padding: 0 }}>
              <TextArea
                rows={3}
                maxLength={200}
                value={this.state.reason}
                onChange={(e) => {
                  this.setState({ reason: e.target.value });
                }}
              />
            </FormItem>
          </Col>
        </Row>

        <FooterTips className={styles.footerTips} />

        {this.state.modalRefundShow && (
          <OldModal
            modalUrl="function/modals/refundAmountRulesModal.html"
            params={{ productId: this.state.modalProductId, productType: this.state.modalProductType }}
            onClose={this.toggleModalRefundShow}
          />
        )}
      </Modal>
    );
  }
}

export default EndContractModal;
