import React from 'react';
import PropTypes from 'prop-types';
import { Modal, Button, Select } from 'COMPONENT';
import { Form, Input } from 'antd';
import { sendSmsCode, confirmSavePayer } from 'SERVICE/contract';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { desensitizationPhoneNumber, validIdentityCardByType } from 'UTIL/common';
import { checkCardNo } from './util';
import styles from './styles.module.less';
import { getSelectOption } from 'SERVICE/common';

const FormItem = Form.Item;
const title = '确认付款人信息';
const formItemLayout = {
  labelCol: { span: 8 },
  wrapperCol: { span: 16 },
};

/**
 * 确认收款人信息弹窗
 */
class ConfirmPayerInfoModal extends React.Component {
  static propTypes = {
    payerName: PropTypes.string.isRequired,
    payerPhone: PropTypes.string.isRequired,
    payerIdCardNo: PropTypes.string.isRequired,
    customerConfirmStatus: PropTypes.number.isRequired,
    customerIdCardType: PropTypes.string.isRequired,
    onOk: PropTypes.func.isRequired,
    onCancel: PropTypes.func.isRequired,
  };

  static defaultProps = {
    onOk() {},
  };

  constructor(props) {
    super(props);

    this.state = {
      identityTypeList: [],
      // 姓名
      payerName: props.payerName,
      // 电话
      payerPhone: props.payerPhone,
      // 身份证号
      payerIdCardNo: props.payerIdCardNo,
      // 证件类型
      customerIdCardType: props.customerIdCardType,
      // 客户手机号验证状态
      customerConfirmStatus: props.customerConfirmStatus,
      // 验证码
      verifyCode: '',
      // 确认按钮
      confirmBtnDisabled: true,
      count: 0,
    };

    this.timer = null;
  }

  componentWillMount() {
    clearTimeout(this.timer);
  }

  componentDidMount() {
    this.fetchIdentityType();
  }

  fetchIdentityType = async () => {
    const res = await getSelectOption({ selectOptionCategory: 'CERTIFICATE_TYPE' });
    const identityType = res.value || {};
    const identityTypeList =
      this.state.customerIdCardType && this.state.customerIdCardType !== 'CT_IDENTITY_CARD'
        ? Object.keys(identityType)
            .filter((key) => key)
            .map((key) => {
              return {
                val: key,
                text: identityType[key],
              };
            })
        : [{ text: '身份证', val: 'CT_IDENTITY_CARD' }];
    this.setState({
      identityTypeList,
    });
  };

  /**
   * 计时
   *
   * @param count
   */
  countDown = (count) => {
    this.timer = setTimeout(() => {
      if (count > 0) {
        this.setState({ count: count - 1 });
        this.countDown(count - 1);
      }
    }, 1000);
  };

  /**
   * 发送验证码
   *
   * @return {Promise<void>}
   */
  sendVerifyCode = async () => {
    const messageType = 'CONTRACT_CONFIRM';
    await sendSmsCode({ phone: this.state.payerPhone, type: messageType });

    this.setState({ count: 60 });
    this.countDown(60);
  };

  /**
   * 确认按钮高亮条件：姓名已填写，验证码已填写，身份证号已填写（满足身份证号通用填写规则）
   *
   * @return {boolean}
   */
  checkConfirmBtn = () => {
    const { payerPhone, payerIdCardNo, payerName, verifyCode, customerConfirmStatus } = this.state;

    let check = payerPhone && payerIdCardNo && payerName;

    if (customerConfirmStatus === 2) {
      return check;
    } else {
      return check && verifyCode;
    }
  };

  confirmPayer = async () => {
    const params = {
      contractId: this.props.contractId,
      phone: this.state.payerPhone,
      name: this.state.payerName,
      verifyCode: this.state.verifyCode,
      cardNo: this.state.payerIdCardNo,
      cardType: this.state.customerIdCardType,
    };

    let validIdentityData = validIdentityCardByType(params.cardType, params.cardNo, true);
    if (!validIdentityData.valid) {
      return showMessage('证件格式有误，请检查证件号码', 'error');
    }

    confirmSavePayer(params)
      .then(() => {
        showMessage('操作成功！', 'success');
        clearTimeout(this.timer);
        this.props.onOk();
      })
      .catch((e) => {
        showMessage(e.text || '操作失败，请重试！', 'error');
      });
  };

  close = () => {
    clearTimeout(this.timer);
    this.setState({ count: 0 }, () => {
      this.props.onCancel();
    });
  };

  render() {
    return (
      <Modal
        title={title}
        width={500}
        loading={this.state.loading}
        onCancel={this.close}
        footer={
          <div className={styles.footer}>
            <Button type="default" onClick={this.close}>
              关闭
            </Button>
            <Button type="primary" disabled={!this.checkConfirmBtn()} onClick={this.confirmPayer}>
              确认
            </Button>
          </div>
        }
      >
        <Form className={styles.contractPayerInfoForm}>
          <FormItem {...formItemLayout} label="付款人姓名">
            <Input value={this.state.payerName} onChange={(e) => this.setState({ payerName: e.target.value })} />
          </FormItem>
          <FormItem {...formItemLayout} label="联系电话">
            <div>
              <span>{desensitizationPhoneNumber(this.state.payerPhone)}&nbsp;</span>
              {this.state.customerConfirmStatus === 2 && <span className={styles.verified}>客户手机号码已验证</span>}
            </div>
            {this.state.customerConfirmStatus === 1 && (
              <div className={styles.verify}>
                <div className={styles.verifyInput}>
                  <Input
                    size="large"
                    value={this.state.verifyCode}
                    onChange={(e) => this.setState({ verifyCode: e.target.value })}
                  />
                </div>
                <Button type="primary" size="middle" disabled={this.state.count > 0} onClick={this.sendVerifyCode}>
                  {this.state.count > 0 ? `重新发送(${this.state.count}s)` : '发送验证码'}
                </Button>
              </div>
            )}
          </FormItem>
          <FormItem {...formItemLayout} label="证件类型">
            <Select
              disabled={Boolean(this.props.customerIdCardType)}
              value={this.state.customerIdCardType}
              placeholder="请选择证件类型"
              dict={this.state.identityTypeList}
              onChange={(value) =>
                this.setState({
                  customerIdCardType: value,
                })
              }
            />
          </FormItem>
          <FormItem {...formItemLayout} label="证件号码">
            <Input
              disabled={Boolean(this.state.customerIdCardType && this.state.customerIdCardType !== 'CT_IDENTITY_CARD')}
              value={this.state.payerIdCardNo}
              onChange={(e) => this.setState({ payerIdCardNo: e.target.value })}
            />
          </FormItem>
        </Form>
      </Modal>
    );
  }
}

export default ConfirmPayerInfoModal;
