import React from 'react';
import PropTypes from 'prop-types';
import NP from 'number-precision';
import { debounce } from 'lodash';
import { List, Select, InputNumber } from 'COMPONENT';
import { isString } from 'UTIL/lang/data-type';
import {
  USERTYPE,
  CAMPUSTYPE,
  CAMPUS_BRANCH,
  CAMPUS_CAMPUS,
  campusTypeDict,
  fundStateMap,
  fundAllocatingMap,
} from './common';
import { createUid } from './utils';
import styles from './styles.less';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;

// 将数组项目按照指定字段排序 [{key: val}, {key: val}, ...]
function arrSortByItemKey(arr, keyName) {
  let _storage = {};
  arr.forEach((item) => {
    if (_storage[item[keyName]]) {
      _storage[item[keyName]].push(item);
    } else {
      _storage[item[keyName]] = [item];
    }
  });
  return Object.values(_storage).reduce((prevVal, currentVal) => prevVal.concat(currentVal), []);
}

// 科次业绩上限值
const MAX_AMOUNT_RATE = 1;
const MIN_AMOUNT_RATE = 0;

// 业绩归属大类 hashmap
const baseBizTypeMap = {
  [USERTYPE]: {
    value: USERTYPE,
    label: '个人',
    listName: 'personalList',
  },
  [CAMPUSTYPE]: {
    value: CAMPUSTYPE,
    label: '分公司',
    listName: 'campusList',
  },
};

// 学习中心业绩归属对象 hashmap
const campusBizTargetHashmap = {
  [CAMPUS_BRANCH]: {
    value: CAMPUS_BRANCH,
    label: '分公司',
    dictName: 'branchDict',
  },
  [CAMPUS_CAMPUS]: {
    value: CAMPUS_CAMPUS,
    label: '学习中心',
    dictName: 'campusDict',
  },
};

class OpsArea extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      personalList: [],
      campusList: [],
      /**
       * 用户字典 hashmap
       * key 为 personalList 每行的 rowId
       */
      userDictMap: {},
    };

    this.handleUserSearchChange = debounce(this.handleUserSearchEffect.bind(this), 1000);
  }

  componentDidMount() {
    const { personalList, campusList } = this.getClassifiedList();
    const userDictMap = this.initUserDictMap(personalList);

    this.setState({
      personalList,
      campusList,
      userDictMap,
    });
  }

  componentDidUpdate(prevProps) {
    if (this.props.sourceList !== prevProps.sourceList) {
      const { personalList, campusList } = this.getClassifiedList();
      const userDictMap = this.initUserDictMap(personalList);

      this.setState({
        personalList,
        campusList,
        userDictMap,
      });
    }
  }

  /**
   * 初始化个人业绩分配记录的业绩归属人字典 hashmap
   * @param {Array<Object>} personalList 个人业绩分配记录
   */
  initUserDictMap(personalList) {
    if (!Array.isArray(personalList)) {
      return {};
    }

    return personalList
      .filter((row) => isString(row.rowId))
      .reduce((prev, row) => {
        const { rowId, distributeBizId, distributeBizName } = row;

        prev[rowId] = [
          {
            val: distributeBizId,
            text: distributeBizName,
          },
        ];

        return prev;
      }, {});
  }

  /**
   * 将源数据分类成个人业绩列表、学习中心业绩列表
   */
  getClassifiedList() {
    const listMap = this.props.sourceList.reduce(
      (prev, item) => {
        const { personalList, campusList } = prev;

        if (item.baseDistributeType === USERTYPE) {
          prev.personalList = personalList.concat({ ...item });
        } else {
          prev.campusList = campusList.concat({ ...item });
        }

        return prev;
      },
      {
        personalList: [],
        campusList: [],
      },
    );

    // 按 contractProductId 分类
    const personalList = arrSortByItemKey(listMap.personalList, 'contractProductId');
    const campusList = arrSortByItemKey(listMap.campusList, 'contractProductId');

    return {
      personalList,
      campusList,
    };
  }

  /**
   * 获取当前可提交的数据
   */
  getCurrentResult() {
    const [personalList, campusList] = ['personalList', 'campusList'].map((name) => {
      return this.state[name].filter((row) => {
        const { dataEditable = false } = fundAllocatingMap[row.canDistribute] || {};

        return dataEditable;
      });
    });
    const isPersonalValid = this.validateOpsList(personalList);
    const isCampusValid = this.validateOpsList(campusList);
    const isDataValid = isPersonalValid && isCampusValid;

    if (!isDataValid) {
      return {
        isValid: false,
        result: [],
        errMsg: '请填写有效的业绩归属和科次业绩',
      };
    }

    const { sourceList } = this.props;
    const srcPersonalList = sourceList.filter((row) => {
      const isUserType = row.baseDistributeType === USERTYPE;
      const { dataEditable = false } = fundAllocatingMap[row.canDistribute] || {};

      return isUserType && dataEditable;
    });
    const srcCampusList = sourceList.filter((row) => {
      const isCampusType = row.baseDistributeType === CAMPUSTYPE;
      const { dataEditable = false } = fundAllocatingMap[row.canDistribute] || {};

      return isCampusType && dataEditable;
    });
    const finalPersonalList = this.filterOpsList(personalList, srcPersonalList);
    const finalCampusList = this.filterOpsList(campusList, srcCampusList);
    const finalFullList = [...finalPersonalList, ...finalCampusList];
    const isFinalValid = finalFullList.length > 0;

    return {
      isValid: isFinalValid,
      result: finalFullList,
      errMsg: isFinalValid ? '' : '没有可提交的数据',
    };
  }

  /**
   * 校验每行数据是否合法
   * @param {Array} result
   */
  validateOpsList(result) {
    const isValueValid = result.every((el) => {
      const isBizIdValid = !!el.distributeBizId;
      const isRateValid = el.amountRate > 0 && el.amountRate <= el.amountTotalRate;

      return isBizIdValid && isRateValid;
    });

    if (!isValueValid) {
      return false;
    }

    // 根据合同产品 id 归类为 hashmap
    const contrPrdHashmap = result.reduce((prev, row) => {
      const { contractProductId: id } = row;

      if (!prev[id]) {
        prev[id] = [];
      }

      prev[id] = prev[id].concat(row);

      return prev;
    }, {});

    const isLogicValid = Object.keys(contrPrdHashmap).every((contrPrdId) => {
      const contrPrdList = contrPrdHashmap[contrPrdId];
      const { amountTotalRate = 0 } = contrPrdList[0] || {};
      const rateSum = contrPrdList.reduce((prev, el) => {
        return NP.plus(prev, el.amountRate);
      }, 0);

      return rateSum === amountTotalRate;
    });

    return isValueValid && isLogicValid;
  }

  /**
   * 计算出可提交的列表
   * @param {Array} targetList 待对比列表
   * @param {Array} sourceList 源数据列表
   */
  filterOpsList(targetList, sourceList) {
    // 修改结果或者源数据为空列表，则表明不需要提交修改记录
    if (!targetList.length || !sourceList.length) {
      return [];
    }

    // 根据合同产品 id 归类为 hashmap
    const contrPrdHashmap = targetList.reduce((prev, row) => {
      const { contractProductId: id } = row;

      if (!prev[id]) {
        prev[id] = [];
      }

      prev[id] = prev[id].concat(row);

      return prev;
    }, {});

    // 过滤后的修改记录
    const filteredTarget = Object.keys(contrPrdHashmap).reduce((prev, id) => {
      const contrPrdTargetList = contrPrdHashmap[id];
      const contrPrdSrcList = sourceList.filter((row) => row.contractProductId === id);
      const srcHashmap = contrPrdSrcList.reduce((prev, row) => {
        const { rowId } = row;

        prev[rowId] = row;

        return prev;
      }, {});

      // 针对某个合同产品 id
      // 如果修改结果与源数据列表长度相同，并且修改结果中没有新的 rowId
      // 表明当前操作可能只是在原纪录上修改
      const maybeEdited =
        contrPrdTargetList.length === contrPrdSrcList.length &&
        contrPrdTargetList.every((row) => !!srcHashmap[row.rowId]);

      if (maybeEdited) {
        const isRealEdited = contrPrdTargetList.some((row) => {
          const srcRow = srcHashmap[row.rowId];

          const isRowEdited = ['subDistributeType', 'distributeBizId', 'amountRate'].some(
            (key) => row[key] !== srcRow[key],
          );

          return isRowEdited;
        });

        return isRealEdited ? prev.concat(contrPrdTargetList) : prev;
      } else {
        return prev.concat(contrPrdTargetList);
      }
    }, []);

    return filteredTarget;
  }

  /**
   * 获取操作列表名称
   * @param {string} type
   */
  getListName(type) {
    const { listName } = baseBizTypeMap[type];

    return listName;
  }

  /**
   * 获取表的列配置信息
   * @param {string} type 列表类型
   */
  getColumns(type) {
    const { editable } = this.props;
    const isCampus = type === CAMPUSTYPE;

    const columns = [
      { dataIndex: 'productName', title: '产品名称', width: COL_WIDTH_L },
      { dataIndex: 'productCharacterName', title: '产品性质', width: 80 },
      {
        dataIndex: 'subjectName',
        title: '科目',
        width: COL_WIDTH_S,
        render: (text) => text || '-',
      },
      { dataIndex: 'gradeName', title: '年级', width: COL_WIDTH_S },
      {
        dataIndex: 'fundsDistributeState',
        title: '资金分配状态',
        width: 100,
        render: (state, row, index) => {
          const { label = '-' } = fundStateMap[state] || {};

          return label;
        },
      },
      { dataIndex: 'contractProductStatusName', title: '合同产品状态', width: COL_WIDTH_M },
      {
        dataIndex: 'subDistributeType',
        title: '业绩归属类型',
        width: 120,
        render: (subType, row, index) => {
          const { dataEditable = false } = fundAllocatingMap[row.canDistribute] || {};
          const rowEditable = editable && dataEditable;

          if (!rowEditable) {
            return row.subDistributeTypeName;
          }

          return isCampus ? this.renderCampusBizType(subType, row) : row.subDistributeTypeName;
        },
      },
      {
        dataIndex: 'distributeBizId',
        title: '业绩归属',
        width: COL_WIDTH_L,
        render: (bizId, row, index) => {
          const { dataEditable = false } = fundAllocatingMap[row.canDistribute] || {};
          const rowEditable = editable && dataEditable;

          if (!rowEditable) {
            return row.distributeBizName;
          }

          return isCampus ? this.renderCampusBizTarget(bizId, row) : this.renderPersonalBizTarget(bizId, row);
        },
      },
      {
        dataIndex: 'amountRate',
        title: '分配业绩科次',
        width: 100,
        render: (rate, row, index) => {
          const { dataEditable = false } = fundAllocatingMap[row.canDistribute] || {};
          const rowEditable = editable && dataEditable;

          if (!rowEditable) {
            return row.amountRate;
          }

          return this.renderAmountRate(rate, row);
        },
      },
      {
        key: 'action',
        title: '操作',
        width: 100,
        getActionConfig: (text, row, index) => {
          const { dataEditable = false } = fundAllocatingMap[row.canDistribute] || {};
          const rowEditable = editable && dataEditable;

          return rowEditable ? this.renderActionUnit(row) : [];
        },
      },
    ];

    return columns;
  }

  /**
   * 渲染操作单元格
   * @param {Object} row 某行数据
   */
  renderActionUnit(row) {
    const listName = this.getListName(row.baseDistributeType);
    const contrPrdId = row.contractProductId;
    const list = this.state[listName].filter((row) => row.contractProductId === contrPrdId);

    const rowId = row.rowId;
    const subRowIndex = list.findIndex((row) => row.rowId === rowId);
    const isLastSubRow = subRowIndex === list.length - 1;
    const isOnlyOneRow = subRowIndex === 0 && list.length === 1;

    // 只有某个合同产品 id 的最后一行才有添加按钮
    const addingBtnInfo = isLastSubRow
      ? [
          {
            text: '添加',
            onClick: () => this.addListRow(row),
          },
        ]
      : [];

    // 某个合同产品 id 的记录只有一行时没有删除按钮
    const deletingBtnInfo = isOnlyOneRow
      ? []
      : [
          {
            text: '删除',
            onClick: () => this.deleteListRow(row),
          },
        ];

    return [...deletingBtnInfo, ...addingBtnInfo];
  }

  /**
   * 增加列表行
   * @param {Object} row 当前行
   */
  addListRow(row) {
    const { rowId: currentRowId, baseDistributeType: listType } = row;
    const listName = this.getListName(listType);

    const newRowId = createUid();
    const newRow = {
      ...row,
      distributeBizId: null,
      distributeBizName: null,
      distributeUserId: null,
      distributeUserName: null,
      distributeOrganizationId: null,
      distributeOrganizationName: null,
      amountRate: 0,
      rowId: newRowId,
      key: newRowId,
    };

    this.setState((prevState) => {
      const oldList = prevState[listName];
      const index = oldList.findIndex((row) => row.rowId === currentRowId);
      const newList = oldList.slice(0, index + 1).concat(newRow, oldList.slice(index + 1));

      return {
        [listName]: newList,
      };
    });
  }

  /**
   * 删除列表行
   * @param {Object} row 当前行
   */
  deleteListRow(row) {
    const { rowId: currentRowId, baseDistributeType: listType } = row;
    const listName = this.getListName(listType);

    this.setState((prevState) => {
      return {
        [listName]: prevState[listName].filter((row) => row.rowId !== currentRowId),
      };
    });
  }

  /**
   * 更新某行
   * @param {Object} row 当前行
   * @param {Object} content 修改内容
   */
  updateListRow(row, content) {
    const { rowId, baseDistributeType: listType } = row;

    const listName = this.getListName(listType);

    this.setState((prevState) => {
      const ret = prevState[listName].map((row) => {
        return row.rowId === rowId
          ? {
              ...row,
              ...content,
            }
          : row;
      });

      return {
        [listName]: ret,
      };
    });
  }

  /**
   * 渲染分公司/学习中心业绩归属类型
   * @param {string} subType 业绩归属类型
   * @param {object} row 行数据
   * @param {boolean} editable 是否可编辑
   */
  renderCampusBizType(subType, row) {
    return (
      <Select
        dict={campusTypeDict}
        value={subType}
        onChange={(newSubType) => {
          const newContent = {
            subDistributeType: newSubType,
            distributeBizId: null,
            distributeBizName: null,
            distributeUserId: null,
            distributeUserName: null,
            distributeOrganizationId: null,
            distributeOrganizationName: null,
          };

          this.updateListRow(row, newContent);
        }}
      />
    );
  }

  /**
   * 渲染分公司/学习中心业绩归属对象
   * @param {string} bizId 业绩归属对象 id
   * @param {Object} row 当前行
   */
  renderCampusBizTarget(bizId, row) {
    let dict = [];

    if (row.subDistributeType) {
      const { dictName } = campusBizTargetHashmap[row.subDistributeType];

      dict = this.props[dictName];
    }

    return (
      <Select
        value={bizId}
        dict={dict}
        showSearch
        noOptionTips="没有找到"
        searchPlaceholder="输入名字搜索"
        onChange={(newBizId) => {
          const [{ text: newBizName = '' } = {}] = dict.filter((item) => item.val === newBizId);

          const newContent = {
            distributeBizId: newBizId,
            distributeBizName: newBizName,
            distributeUserId: null,
            distributeUserName: null,
            distributeOrganizationId: null,
            distributeOrganizationName: null,
          };

          this.updateListRow(row, newContent);
        }}
      />
    );
  }

  /**
   * 渲染个人业绩归属对象
   * @param {string} bizId 业绩归属对象 id
   * @param {Object} row 当前行
   */
  renderPersonalBizTarget(bizId, row) {
    const { userDictMap } = this.state;
    const { rowId } = row;
    const dict = userDictMap[rowId];

    // Select 组件问题：当传入 onSearchKeyChange 时
    // 无法在 dict 中对 value 进行过滤
    // 只能手动构造和 searchValue 匹配的 dict 传入组件
    return (
      <Select
        value={bizId}
        dict={dict}
        showSearch
        searchPlaceholder="输入名字搜索"
        noOptionTips="没有找到"
        searchNotFoundTips="没有找到"
        onSearchKeyChange={(name) => {
          if (!name) {
            return;
          } else {
            this.handleUserSearchChange(name, row);
          }
        }}
        onChange={(newBizId) => {
          const matchList = newBizId ? dict.filter((item) => item.val === newBizId) : [];
          const [{ text: newBizName = '' } = {}] = matchList;

          const newContent = {
            distributeBizId: newBizId,
            distributeBizName: newBizName,
            distributeUserId: null,
            distributeUserName: null,
            distributeOrganizationId: null,
            distributeOrganizationName: null,
          };

          this.updateListRow(row, newContent);
        }}
      />
    );
  }

  /**
   * 按名称搜索用户
   * @param {string} name 输入的文字
   * @param {Object} row 某一行个人科次业绩分配记录
   */
  async handleUserSearchEffect(name, row) {
    const userDict = await this.props.onUserSearch(name);
    const matchList = userDict.filter((item) => item.val === row.distributeBizId);

    if (!matchList.length) {
      this.updateListRow(row, {
        distributeBizId: '',
        distributeBizName: '',
      });
    }

    this.updateUserDictMap(row, userDict);
  }

  /**
   * 更新用户字典 hashmap
   * @param {Object} row 某行业绩分配记录
   * @param {Array<Object>} dict 用户字典
   */
  updateUserDictMap(row, dict) {
    const { rowId } = row;

    this.setState((prevState) => {
      const prevUserDictMap = prevState.userDictMap;

      return {
        userDictMap: {
          ...prevUserDictMap,
          [rowId]: dict,
        },
      };
    });
  }

  /**
   * 渲染科次业绩值
   * @param {number} rate 业绩值
   * @param {Object} row 当前行
   */
  renderAmountRate(rate, row) {
    return (
      <InputNumber
        value={rate}
        min={MIN_AMOUNT_RATE}
        precision={2}
        step={0.01}
        max={Math.min(MAX_AMOUNT_RATE, row.amountTotalRate)}
        onBlur={(e) => {
          this.handleAmountRateBlur(row, e.target.value);
        }}
        onChange={(newRate) => {
          this.updateListRow(row, {
            amountRate: newRate,
          });
        }}
      />
    );
  }

  /**
   * 响应科次业绩修改
   * @param {Object} currentRow 当前行
   * @param {number} rate 业绩值
   */
  handleAmountRateBlur(row, rate) {
    const listName = this.getListName(row.baseDistributeType);
    const list = this.state[listName].filter((el) => el.contractProductId === row.contractProductId);
    const currentRateSum = list.reduce((prev, el) => {
      return el.rowId === row.rowId ? prev : NP.plus(prev, Number(el.amountRate));
    }, 0);

    // 剩余可分配业绩
    const leftRateValue = Math.max(MIN_AMOUNT_RATE, NP.minus(row.amountTotalRate, currentRateSum));

    // 重新给业绩单元格赋值
    // 有效值大于0，并且小于等于剩余可分配业绩
    const finalRate = Math.min(rate, leftRateValue);

    if (rate !== finalRate) {
      this.updateListRow(row, {
        amountRate: finalRate,
      });
    }
  }

  render() {
    const { campusList = [], personalList = [] } = this.state;

    return (
      <div className={styles.opsArea}>
        <div style={{ marginBottom: 30 }}>
          <div className={styles.listHeader}>
            <p className={styles.headerTitle}>
              <span>个人业绩分配</span>
            </p>
          </div>
          <List dataSource={personalList} columns={this.getColumns(USERTYPE)} />
        </div>
        <div>
          <div className={styles.listHeader}>
            <p className={styles.headerTitle}>
              <span>学习中心业绩分配</span>
            </p>
          </div>
          <List dataSource={campusList} columns={this.getColumns(CAMPUSTYPE)} />
        </div>
      </div>
    );
  }
}

OpsArea.propTypes = {
  editable: PropTypes.bool.isRequired,
  sourceList: PropTypes.array.isRequired,
  branchDict: PropTypes.array.isRequired,
  campusDict: PropTypes.array.isRequired,
  onUserSearch: PropTypes.func.isRequired,
};

export default OpsArea;
