import React from 'react';
import PropTypes from 'prop-types';
import {
  getXhSbjPerfAllocationList,
  updXhSbjPerfAllocationList,
  getXhSbjPerfAllocationHistList,
} from 'SERVICE/contract';
import { searchUser } from 'SERVICE/customer';
import { loadAllBranchCampusDict, loadAllBranchDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

import ModalWrapper from './Modal';
import Header from './Header';
import Footer from './Footer';
import OpsArea from './OpsArea';
import OpsHistory from './OpsHistory';
import { createUid } from './utils';
import { USERTYPE } from './common';

class AllocationWrapper extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      // 从旧页面列表过来的数据
      fundDetail: {
        contractId: props.oldPageParams.contractId,
        contractTotalMoney: props.oldPageParams.contractTotalMoney,
        contractCampus: props.oldPageParams.contractCampus,
        confirmStatusName: props.oldPageParams.confirmStatusName,
        stuName: props.oldPageParams.stuName,
        stuId: props.oldPageParams.stuId,
      },
      // 是否可以分配科次
      opsEditable: true,
      // 从服务端同步来的业绩分配结果列表
      opsResList: [],
      // 分配历史记录列表和总数
      opsHistory: {
        list: [],
        total: 0,
      },
      // 所有分公司字典
      branchDict: [],
      // 所有学习中心字典
      campusDict: [],
      // 加载状态
      loading: false,
    };
  }

  async componentDidMount() {
    this.setState({ loading: true });

    const { contractId } = this.state.fundDetail;

    try {
      const [branchDict, campusDict, opsResList, opsHistRes] = await Promise.all([
        loadAllBranchDict(),
        loadAllBranchCampusDict(),
        this.getOpsList({ contractId }),
        this.getOpsHistory({ contractId }),
      ]);

      this.setState({
        loading: false,
        branchDict,
        campusDict: campusDict.map((group) => {
          const newOptions = group.options.map((dict) => ({
            ...dict,
            title: dict.text,
          }));

          return {
            ...group,
            options: newOptions,
          };
        }),
        opsResList,
        opsHistory: {
          list: opsHistRes.list,
          total: opsHistRes.total,
        },
      });
    } catch (err) {
      this.setState({ loading: false });

      showMessage(err.text || err.message);
    }
  }

  async getOpsList({ contractId }) {
    const response = await getXhSbjPerfAllocationList({ contractId });
    const rawList = response.data;

    // 为数组项目生成 rowId，方便后续数据比对
    const rowIdPrefix = createUid();
    const opsResList = rawList.map((el, index) => {
      const rowId = rowIdPrefix + String(index);

      return {
        ...el,
        distributeBizName: el.baseDistributeType === USERTYPE ? el.distributeUserName : el.distributeOrganizationName,
        amountRate: Number(el.amountRate),
        amountTotalRate: Number(el.amountTotalRate),
        rowId,
        key: rowId,
      };
    });

    return opsResList;
  }

  /**
   * 获取操作历史
   * @param {Object} param0
   */
  async getOpsHistory({ contractId, pageNum = 1, pageSize = 10 }) {
    const response = await getXhSbjPerfAllocationHistList({
      pageNum,
      pageSize,
      contractId,
    });
    const { data } = response;

    const rowIdPrefix = createUid();

    return {
      list: data.list.map((el, index) => {
        const rowId = rowIdPrefix + String(index);

        return {
          ...el,
          distributeBizName: el.baseDistributeType === USERTYPE ? el.distributeUserName : el.distributeOrganizationName,
          amountRate: Number(el.amountRate),
          rowId,
          key: rowId,
        };
      }),
      total: data.total,
    };
  }

  /**
   * 响应操作记录页码等状态更新
   */
  onHistoryPageChange = async ({ pageNum = 1, pageSize = 10 }) => {
    this.setState({ loading: true });

    const { contractId } = this.state.fundDetail;

    try {
      const res = await this.getOpsHistory({ contractId, pageNum, pageSize });

      this.setState({
        loading: false,
        opsHistory: {
          list: res.list,
          total: res.total,
        },
      });
    } catch (e) {
      this.setState({ loading: false });
      showMessage(e.message || e.text);
    }
  };

  /**
   * 查询用户字典
   * @param {string} name 输入的名字
   */
  getUserDict = async (name) => {
    this.setState({ loading: true });

    let userDict = [];

    try {
      const response = await searchUser({ name });

      if (response && Array.isArray(response.data)) {
        userDict = response.data.map((item) => ({
          text: `${item.username}<${item.employeeNo}>`,
          subText: item.orgInfo,
          val: item.userId,
        }));
      }

      this.setState({ loading: false });
    } catch (err) {
      this.setState({ loading: false });

      showMessage(err.message);
    }

    return userDict;
  };

  /**
   * 保存提交
   * @param {Array} resultList 修改后的分配列表
   */
  onSave = async (resultList) => {
    this.setState({ loading: true });

    const requestParams = {
      contractId: this.state.fundDetail.contractId,
      itemList: resultList.map((row) => {
        const { contractProductId, subDistributeType, distributeBizId, amountRate } = row;

        return {
          contractProductId,
          subDistributeType,
          distributeBizId,
          amountRate,
        };
      }),
    };

    try {
      await updXhSbjPerfAllocationList(requestParams);

      this.setState({ loading: false }, () => {
        showMessage('保存成功！', 'success');
        this.props.onClose();
      });
    } catch (e) {
      // console.log(e);
      this.setState({ loading: false }, () => {
        showMessage(e.message || e.text, 'error');
      });
    }
  };

  render() {
    const { onClose } = this.props;
    const { loading, opsEditable, fundDetail, branchDict, campusDict, opsHistory, opsResList } = this.state;

    return (
      <ModalWrapper
        loading={loading}
        opsEditable={opsEditable}
        onClose={onClose}
        onSave={this.onSave}
        header={<Header fundDetail={fundDetail} />}
        footer={<Footer info={fundDetail} onLinkClick={onClose} />}
        opsHistory={
          <OpsHistory list={opsHistory.list} totalNum={opsHistory.total} onPageChange={this.onHistoryPageChange} />
        }
        renderOpsArea={({ refCb }) => (
          <OpsArea
            editable={opsEditable}
            sourceList={opsResList}
            branchDict={branchDict}
            campusDict={campusDict}
            onUserSearch={this.getUserDict}
            ref={refCb}
          />
        )}
      />
    );
  }
}

AllocationWrapper.propTypes = {
  oldPageParams: PropTypes.object.isRequired,
  onClose: PropTypes.func.isRequired,
};

export default AllocationWrapper;
