import React from 'react';
import Page from 'COMPONENT/Page';
import DatePicker from 'COMPONENT/DatePicker';
import moment from 'moment';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import { loadCommonDict, loadBranchCampusDict } from 'UTIL/dict';
import { connect } from 'dva';
import loaders from './loaders';
import List from 'COMPONENT/List';
import ProductDetail from './productDetail';
import styles from './index.less';
import { debounce } from 'lodash';

const paidStateDict = [
  { val: 1, text: '待分配资金为0' },
  { val: 2, text: '待分配资金不为0' },
];
const distributionStateDict = [
  { val: 1, text: '个人待分配科次为0' },
  { val: 2, text: '个人待分配科次不为0' },
  { val: 3, text: '学习中心待分配科次为0' },
  { val: 4, text: '学习中心待分配科次不为0' },
];
// const productTypeDict = [
//   { val: 'TWO_TEACHER', text: '双师'},
//   { val: 'SMALL_CLASS', text: '小班'},
//   { val: 'LECTURE', text: '讲座'},
//   { val: 'OTHERS', text: '其他'},
// ]

const user = Index.getLoginUser();

class performanceInquiry extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: true,

      contractStartDate: moment().subtract(1, 'month').date(1).format('YYYY-MM-DD'), // 开始日期
      contractEndDate: moment().format('YYYY-MM-DD'), // 结束日期
      queryParam: null, // 学生编号/学生姓名/学生id
      contractQueryParam: null, // 签单人,合同编号
      paidState: null, // 资金分配情况
      distributionState: null, // 分配科次情况
      grade: null, // 年级
      productType: null, // 产品类型
      contractBlCampus: user.isBlCampus === 'true' ? user.campusId : null, // 合同学习中心,默认登录角色学习中心,非学习中心角色为 null

      gradeDict: null, //数据字典 年级
      blCampusDict: null, //数据字典 学习中心
      productTypeDict: null, // 数据字典 产品类型

      dataSource: [], // 列表数据源
      pagination: {
        current: 1,
        pageSize: 20,
        total: 0,
      }, // 列表分页信息

      detailModal: false, // 是否打开详情模态框

      studentId: null, // 模态框学生 id
      contractProductDistributionInfoList: null, // 模态框 学生的合同产品列表（包括科次业绩分配统计）
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const [gradeDict, blCampusDict, productTypeDict] = await Promise.all([
      loadCommonDict('STUDENT_GRADE'),
      loadBranchCampusDict(),
      loadCommonDict('PRODUCT_TYPE'),
    ]);
    await this.search();
    this.setState({ gradeDict, blCampusDict, productTypeDict, loading: false });
  };

  /**
   * 获取要禁用的开始日期值
   */
  getDisabledStartDate = (contractStartDate) => {
    const { contractEndDate } = this.state;
    if (!contractStartDate || !contractEndDate) {
      return false;
    }
    return moment(contractStartDate).valueOf() > moment(contractEndDate).valueOf();
  };

  /**
   * 获取要禁用的结束日期值
   */
  getDisabledEndDate = (contractEndDate) => {
    const { contractStartDate } = this.state;
    if (!contractEndDate || !contractStartDate) {
      return false;
    }
    return moment(contractEndDate).valueOf() < moment(contractStartDate).valueOf();
  };

  /**
   * 搜索
   */
  search = async () => {
    // this.setState({loading: true})
    const {
      contractBlCampus,
      contractEndDate,
      contractQueryParam,
      contractStartDate,
      distributionState,
      grade,
      paidState,
      productType,
      queryParam,
      pagination: { current: page, pageSize: rows },
    } = this.state;
    const params = {
      contractBlCampus,
      contractEndDate,
      contractQueryParam,
      contractStartDate,
      distributionState,
      grade,
      paidState,
      productType,
      queryParam,
      page,
      rows,
    };
    let data = await loaders.getStudentContractProductDistributionListApi(params);
    this.setState({
      dataSource: data.rows,
      pagination: {
        current: data.page,
        pageSize: rows,
        total: data.records,
      },
      loading: false,
    });
  };
  /**
   * 重置
   */
  reset = () => {
    this.setState({
      contractStartDate: null, // 开始日期
      contractEndDate: null, // 结束日期
      queryParam: null, // 学生编号/学生姓名/学生id
      contractQueryParam: null, // 签单人,合同编号
      paidState: null, // 资金分配情况
      distributionState: null, // 分配科次情况
      grade: null, // 年级
      productType: null, // 产品类型
      contractBlCampus: user.isBlCampus === 'true' ? user.campusId : null, // 合同学习中心
    });
  };

  /**
   * 打开详情 Modal
   */
  openDetailModal = () => {
    this.setState({ detailModal: true });
  };

  /**
   * 关闭详情 Modal
   */
  closeDetailModal = () => {
    this.setState({ detailModal: false });
  };

  /**
   * 获取学生的合同产品列表（包括科次业绩分配统计）
   */
  getContractProductDistributionInfoListByStudentId = async (studentId) => {
    this.setState({ loading: true });
    let contractProductDistributionInfoList = await loaders.getContractProductDistributionInfoListByStudentIdApi({
      studentId,
    });
    this.setState({ loading: false, contractProductDistributionInfoList });
    this.openDetailModal();
  };

  render() {
    const {
      loading,
      contractStartDate,
      contractEndDate,
      queryParam,
      contractQueryParam,
      paidState,
      distributionState,
      productType,
      grade,
      contractBlCampus,
      gradeDict,
      blCampusDict,
      productTypeDict,
      pagination,
      dataSource,
      detailModal,
      studentId,
      contractProductDistributionInfoList,
    } = this.state;

    let getContractProductDistributionInfoListByStudentId = this.getContractProductDistributionInfoListByStudentId;
    let columns = [
      { title: '学生编号', dataIndex: 'studentId', width: 144 },
      { title: '学生姓名', dataIndex: 'studentName' },
      { title: '年级', dataIndex: 'gradeName' },
      { title: '学习中心', dataIndex: 'blCampusName' },
      { title: '合同学习中心', dataIndex: 'contractBlCampusName' },
      { title: '已分配资金', dataIndex: 'totalPaidAmount' },
      { title: '待分配资金', dataIndex: 'totalPendingAmount' },
      { title: '学习中心待分配金额', dataIndex: 'campusPendingDistributionAmount', width: 114 },
      { title: '学习中心待分配科次', dataIndex: 'campusPendingDistributionAmountRate', width: 114 },
      { title: '个人待分配金额', dataIndex: 'staffPendingDistributionAmount', width: 114 },
      { title: '个人待分配科次', dataIndex: 'staffPendingDistributionAmountRate', width: 114 },
      {
        title: '操作',
        dataIndex: '',
        render(currentValue, currentRowData) {
          console.log(this);
          return (
            <span
              onClick={() => {
                // this.setState({
                //   detailModal: true,
                //   studentId: currentRowData.studentId
                // })
                getContractProductDistributionInfoListByStudentId(currentRowData.studentId);
              }}
              className={styles.detailText}
            >
              详情
            </span>
          );
        },
        fixed: 'right',
      },
    ];

    return (
      <Page title={0} loading={loading}>
        <SearchBar
          onReset={this.reset}
          onSearch={() => {
            this.setState({ loading: true });
            debounce(this.search, 3000)();
          }}
        >
          <DatePicker
            placeholder="开始日期"
            value={contractStartDate}
            onChange={(contractStartDate) => {
              this.setState({ contractStartDate });
            }}
            disabledDate={this.getDisabledStartDate}
          />
          <DatePicker
            placeholder="结束日期"
            value={contractEndDate}
            onChange={(contractEndDate) => {
              this.setState({ contractEndDate });
            }}
            disabledDate={this.getDisabledEndDate}
          />
          <Input
            placeholder="学生编号/学生姓名/产品名称"
            value={queryParam}
            onChange={(event) => {
              this.setState({ queryParam: event.currentTarget.value });
            }}
          />
          <Input
            placeholder="签订人/合同编号"
            value={contractQueryParam}
            onChange={(event) => {
              this.setState({ contractQueryParam: event.currentTarget.value });
            }}
          />
          <Select
            placeholder="产品类型"
            dict={productTypeDict}
            value={productType}
            onChange={(productType) => {
              this.setState({ productType });
            }}
          />
          <Select
            placeholder="选择年级"
            dict={gradeDict}
            value={grade}
            onChange={(grade) => {
              this.setState({ grade });
            }}
          />
          <Select
            placeholder="选择合同学习中心"
            dict={blCampusDict}
            value={contractBlCampus}
            showSearch={true}
            onChange={(contractBlCampus) => {
              this.setState({ contractBlCampus });
            }}
          />
          <Select
            placeholder="资金分配情况"
            dict={paidStateDict}
            value={paidState}
            onChange={(paidState) => {
              this.setState({ paidState });
            }}
          />
          <Select
            placeholder="分配科次情况"
            dict={distributionStateDict}
            value={distributionState}
            onChange={(distributionState) => {
              this.setState({ distributionState });
            }}
          />
        </SearchBar>
        <List
          dataSource={dataSource}
          columns={columns}
          pagination={pagination}
          onChange={async (pagination) => {
            console.log('pagination');
            console.log(pagination);
            await this.setState({
              pagination: {
                current: pagination.current,
                pageSize: pagination.pageSize,
                total: this.state.pagination.total,
              },
            });
            this.search();
          }}
        />
        {detailModal && (
          <ProductDetail
            contractProductDistributionInfoList={contractProductDistributionInfoList}
            closeDetailModal={this.closeDetailModal}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    account: state.main.account,
  };
};
export default connect(mapStateToProps)(performanceInquiry);
