import React from 'react';
import moment from 'moment';
import styles from './index.less';
import { getDictTextByVal } from 'UTIL/dict';
import { runFunc } from 'UTIL/common';
import {
  NONE,
  PROMOTION_FULL_SALE_NORMAL,
  PROMOTION_FULL_SALE_EVERY,
  PROMOTION_DISCOUNT_CASH,
  PROMOTION_DISCOUNT_COURSE,
  PROMOTION_DISCOUNT_PRODUCT,
  PROMOTION_GIVE_HOURS,
  PRODUCT_NUMS,
  FullReducePromotion,
  DiscountPromotion,
  FullGivePromotion,
  conditionUnitCfg,
  continuityPromotionsTypeCfg,
  promotionsPrefixCfg,
  promotionsTypeCfg,
  promotionsUnitCfg,
  needRuleUnitList,
  continuityDict,
} from './config';
import { COUPON_RULE_VO_KEY, dateFormatter } from '../config';

const ruleVoKey = COUPON_RULE_VO_KEY;

/**
 * 选中背景样式
 *
 * @param selected
 * @param disabled
 * @return {string}
 */
export function formatSelectedStyle(selected, disabled) {
  let selectedStyle = '';

  if (disabled) {
    selectedStyle = styles.disabled;
  } else if (selected) {
    selectedStyle = styles.selected;
  }

  return selectedStyle;
}

/**
 * 格式化连报条件
 */
export function formatContinuityTypeName(continuityType) {
  if (!continuityType) {
    return '';
  }

  return getDictTextByVal(continuityDict, continuityType);
}

/**
 * 格式化所属分公司
 *
 * @param branchVos
 * @return {string}
 */
export function formatBranchName(branchVos = []) {
  if (branchVos && branchVos.length > 0) {
    const names = branchVos.map((item) => item.branchName || '');

    return names.join(',');
  }

  return '';
}

/**
 * 连报 且是 满减+产品数 或 产品折扣时，运行函数
 *
 * @param continuityType {string} required 连报条件
 * @param promotionType {string} required 优惠类型
 * @param ruleUnit {string} required 满减条件类型
 * @param func1 {function=} true时执行
 * @param func2 {function=} false时执行
 * @return {function,boolean}
 * */
export function checkContinuePromotion({ continuityType, promotionType, ruleUnit }, func1, func2) {
  const flag =
    continuityType !== NONE &&
    ((promotionType === PROMOTION_FULL_SALE_NORMAL && ruleUnit === PRODUCT_NUMS) ||
      promotionType === PROMOTION_DISCOUNT_PRODUCT);

  if (flag) {
    return (func1 && runFunc(func1)) || true;
  } else {
    return (func2 && runFunc(func2)) || false;
  }
}

/**
 * 格式化`连报条件`描述配置
 *
 * @param promotionType {string} 优惠类型
 * @return {Object} 文字描述配置
 * */
export function getContinuityPromotionsType(promotionType) {
  let result;

  switch (promotionType) {
    case PROMOTION_FULL_SALE_NORMAL:
    case PROMOTION_FULL_SALE_EVERY:
      result = continuityPromotionsTypeCfg[FullReducePromotion];
      break;
    case PROMOTION_DISCOUNT_CASH:
    case PROMOTION_DISCOUNT_COURSE:
    case PROMOTION_DISCOUNT_PRODUCT:
      result = continuityPromotionsTypeCfg[DiscountPromotion];
      break;
    case PROMOTION_GIVE_HOURS:
      result = continuityPromotionsTypeCfg[FullGivePromotion];
      break;
    default:
      result = continuityPromotionsTypeCfg['undefined'];
  }

  return result;
}

/**
 * 根据化优惠券数据格式化单个优惠规则描述
 *
 * @param params.promotionType 优惠券类型
 * @param params.continuityType 连报条件
 * @param params.rule 需要格式化的优惠规则
 */
export function formatPromotionRuleResult(params) {
  const { promotionType, rule } = params;

  if (!promotionType || !rule) {
    const defPreFixText = '';
    const defPrefixNum = '';
    const defPrefixUnit = '';
    const defResultText = '';
    const defResultNum = '';
    const defResultUnit = '';

    return {
      prefixText: defPreFixText,
      prefixNum: defPrefixNum,
      prefixUnit: defPrefixUnit,
      resultText: defResultText,
      resultNum: defResultNum,
      resultUnit: defResultUnit,
    };
  }

  const prefixText = promotionsPrefixCfg[promotionType];
  const prefixUnit = conditionUnitCfg[promotionType];
  const resultText = promotionsTypeCfg[promotionType];
  const resultUnit = promotionsUnitCfg[promotionType];

  let _prefixUnit;
  if (needRuleUnitList.includes(promotionType)) {
    _prefixUnit = prefixUnit[rule.ruleUnit];
  } else {
    _prefixUnit = prefixUnit;
  }
  if (typeof _prefixUnit !== 'string') {
    _prefixUnit = '';
  }

  return {
    prefixText: prefixText, // 满
    prefixNum: rule.promotionRule, // N (件)
    prefixUnit: _prefixUnit, // 件产品 (单位)
    resultText: resultText, // 减
    resultNum: rule.promotion, // N (元)
    resultUnit: resultUnit, // 元 (单位)
  };
}

/**
 * 仅根据优惠规则格式化该规则描述
 * tips: 规则单位`ruleUnit` {MONEY:金额, COURSE_HOURS:课时, PRODUCT_NUMS:产品数量}
 * tips: 优惠单位`unit` {MONEY:金额, COURSE_HOURS:课时, PRODUCT_NUMS:产品数量, DISCOUNT:折扣, GIFT:物品}
 *
 * @param rule {object} 优惠规则
 */
export function formatPromotionRuleResultByRule(rule = {}) {
  const { ruleUnit, unit, promotionRule, promotion } = rule;

  const prefixTextConfig = {
    MONEY: '满',
    COURSE_HOURS: '满',
    PRODUCT_NUMS: '满',
  };
  const prefixUnitConfig = {
    MONEY: '元',
    COURSE_HOURS: '课时',
    PRODUCT_NUMS: '件产品',
  };
  const resultTextConfig = {
    MONEY: '减',
    COURSE_HOURS: '赠',
    PRODUCT_NUMS: '减',
    GIFT: '赠',
    DISCOUNT: '打',
  };
  const resultUnitConfig = {
    MONEY: '元',
    COURSE_HOURS: '课时',
    PRODUCT_NUMS: '产品数量',
    GIFT: '物品',
    DISCOUNT: '折',
  };

  return {
    prefixText: prefixTextConfig[ruleUnit],
    prefixNum: promotionRule,
    prefixUnit: prefixUnitConfig[ruleUnit],
    resultText: resultTextConfig[unit],
    resultNum: promotion,
    resultUnit: resultUnitConfig[unit],
  };
}

/**
 * 将上面的优惠规则拼接为完整字符串
 *
 * @param ruleResult 格式化后的单个规则
 * @return {string} `满 N 件产品 减 N 元`
 */
export function formatRuleEntireText(ruleResult) {
  const { prefixText, prefixNum, prefixUnit, resultText, resultNum, resultUnit } = ruleResult;

  return prefixText + prefixNum + prefixUnit + resultText + resultNum + resultUnit;
}

/**
 * 格式化优惠券的单个使用条件/使用规则
 *
 * @param rule 单个使用规则
 * @param promotionType 优惠券规则
 * @param continuityType 连报类型
 */
function formatPromotionRule(rule, promotionType, continuityType) {
  const prefixText = promotionsPrefixCfg[promotionType];
  const prefixUnit = conditionUnitCfg[promotionType];
  const resultText = promotionsTypeCfg[promotionType];
  const resultUnit = promotionsUnitCfg[promotionType];

  let _prefixUnit;
  if (needRuleUnitList.includes(promotionType)) {
    _prefixUnit = prefixUnit[rule.ruleUnit];
  } else {
    _prefixUnit = prefixUnit;
  }
  if (typeof _prefixUnit !== 'string') {
    _prefixUnit = '';
  }

  let firstStr = `${prefixText}${rule.promotionRule}${_prefixUnit}`;
  let lastStr = '';

  if (
    checkContinuePromotion({ continuityType, promotionType, ruleUnit: rule.ruleUnit }) &&
    Array.isArray(item.detailList) &&
    !rule.promotion
  ) {
    lastStr = rule.detailList
      .map((detail, index) => {
        return `${getContinuityPromotionsType(promotionType)[continuityType][index]}${detail.promotion}${resultUnit}`;
      })
      .join('');
  } else {
    lastStr = `${resultText}${rule.promotion}${resultUnit}`;
  }

  return {
    firstStr,
    lastStr,
  };
}

/**
 * 格式化优惠券的单个使用条件
 *
 * @param data {Object} 优惠数据
 * @return {*}
 */
export function formatSingleUseCondition(data) {
  const { promotionTypeId: promotionType, continuityType, hightAmount } = data;
  const ruleVos = data[ruleVoKey] || [];
  const prefixText = promotionsPrefixCfg[promotionType];
  const prefixUnit = conditionUnitCfg[promotionType];
  const resultText = promotionsTypeCfg[promotionType];
  const resultUnit = promotionsUnitCfg[promotionType];

  let firstStr = '';
  let lastStr = '';
  // (未使用)查找可使用的规则
  const rule = ruleVos && ruleVos.length > 0 ? ruleVos[0] : {};

  // 满减促销-满减
  if (promotionType === PROMOTION_FULL_SALE_EVERY) {
    if (rule) {
      firstStr = `${prefixText}${item.promotionRule}${prefixUnit[item.ruleUnit]}${resultText}`;
      lastStr = `${item.promotion}${resultUnit}`;
    }
  } else {
    const { firstStr: first, lastStr: last } = formatPromotionRule(rule, promotionType, continuityType);

    firstStr = first;
    lastStr = last;
  }

  return {
    firstStr,
    lastStr,
  };
}

/**
 * 格式化优惠券所有使用条件
 *
 * @param data {Object} 优惠数据
 * @return {string} 使用条件
 * */
export function formatUseCondition(data) {
  const { promotionTypeId: promotionType, continuityType, hightAmount } = data;
  const ruleVos = data[ruleVoKey] || [];
  const prefixText = promotionsPrefixCfg[promotionType];
  const prefixUnit = conditionUnitCfg[promotionType];
  const resultText = promotionsTypeCfg[promotionType];
  const resultUnit = promotionsUnitCfg[promotionType];

  // 满减促销-满减
  if (promotionType === PROMOTION_FULL_SALE_EVERY) {
    return ruleVos
      .map((item) => {
        return `${prefixText}${item.promotionRule}${prefixUnit[item.ruleUnit]}${resultText}${
          item.promotion
        }${resultUnit}，最高减${hightAmount}${resultUnit}`;
      })
      .join('；');
  } else {
    return ruleVos
      .map((item) => {
        const { firstStr, lastStr } = formatPromotionRule(item, promotionType, continuityType);

        return firstStr + lastStr;
      })
      .join('；');
  }
}

/**
 * 活动时间
 *
 * @param record
 * @param splitStr
 */
export function renderActivityDate(record, splitStr = '-') {
  const { startDate, endDate } = record;

  return (
    <span>
      {moment(startDate).format(dateFormatter)} {splitStr} {moment(endDate).format(dateFormatter)}
    </span>
  );
}
