import React, { PropTypes } from 'react';
import moment from 'moment';
import { Row, Col, DatePicker, Pagination } from 'antd';
import { Modal, Wrapper, NoDataPlus, Input, Select, Button } from 'COMPONENT';
import styles from './index.less';
import CouponList from './CouponList';
import {
  initCouponList,
  initMathCouponList,
  formatListAfterSelect,
  formatCouponListApiParam,
  formatMultipleMatchCouponIds,
} from './util';
import { promiseMultipleMatchCouponList } from './loader';
import { continuityTypeDict, superPositionDict } from './dict';
import { dateTimeFormatter, defaultTimeEarly, defaultTimeLate } from './config';
import { getPromotionCouponsByStudent } from 'SERVICE/contract';

const { RangePicker } = DatePicker;
const showPager = false;

/**
 * 选择优惠券弹窗
 * 单选：{multiple: false} 或 {multiple: true, limit: 1}
 * 多选(限制0或N)：{multiple: true, limit: 大于等于0且不等于1的数}
 * 多选(无限制)：{multiple: true, limit: 小于0的数}
 *
 * @param props.multiple {boolean} 是否可多选，默认false
 * @param props.limit {number} 可选数量限制，仅当multiple=true时有效，不传或传入小于0的数则不限制，默认1
 * @param props.branchId {string} 合同(学生)分公司ID
 * @param props.studentId {string} 学生ID
 * @param props.couponList {array} 已选优惠券
 * @param props.productList {array} 已选产品(过滤后的小班和双师产品)
 * @param props.realAmount {number} 合同应收款合计(优惠后的总金额)
 * @param props.onCancel {function} 点击关闭
 * @param props.onOk {function} 确认选择，回调参数为selectedList(选中的优惠券)
 */
class Index extends React.Component {
  static propTypes = {
    multiple: PropTypes.bool,
    limit: PropTypes.number,
    branchId: PropTypes.string,
    studentId: PropTypes.string,
    couponList: PropTypes.array,
    productList: PropTypes.array,
    realAmount: PropTypes.number,
    onCancel: PropTypes.func,
    onOk: PropTypes.func,
  };

  static defaultProps = {
    multiple: false,
    limit: 1,
    branchId: null,
    studentId: null,
    couponList: [],
    productList: [],
    realAmount: 0,
    onCancel: () => {},
    onOk: (selectedList) => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      promotionName: null, // 券名称
      continuityType: null, // 连报条件
      superPosition: null, // 是否叠加
      tempDate: null, // 有效期

      loading: false, // loading
      list: [], // 学生可用优惠券列表
      pageNo: 1, // 页码
      pageSize: 50, // 每页条数
      totalRecords: 0, // 总条数

      selectedList: [], // 所有分页选中的优惠
    };

    this.initSearchParam = {
      promotionName: null,
      continuityType: null,
      superPosition: null,
      tempDate: null,
    };
  }

  componentDidMount() {
    this.initModal();
  }

  /**
   * 初始化弹窗的初始数据等
   */
  initModal = () => {
    const { couponList } = this.props;

    this.setState(
      {
        selectedList: couponList,
      },
      () => {
        this.loadList();
      },
    );
  };

  /**
   * (等待后端修改接口)
   * 一次性加载学生所有优惠券
   * 1、若未选择产品，则返回所有可用但不可选优惠券
   * 2、若选了产品，则返回所有可用与不可用的优惠券
   */
  loadListDisposable = (changeParam = {}) => {
    const { productList } = this.props;
    const { selectedList } = this.state;
    const params = { ...this.formatListParam(), ...changeParam };
    const apiParam = formatCouponListApiParam(params);

    this.setState({ loading: true });
    promiseMultipleMatchCouponList(apiParam)
      .then((response) => {
        let list = [];

        if (response && response.data) {
          const { suitablePromotion, unSuitablePromotion } = response.data;
          list = suitablePromotion.concat(unSuitablePromotion);
          console.log(list);
        }

        const couponList = initCouponList(list, selectedList, productList);
        const matchIds = formatMultipleMatchCouponIds({ response });
        const finalList = initMathCouponList(couponList, matchIds) || [];

        this.setState({
          loading: false,
          list: finalList,
          pageNo: params.pageNo,
          pageSize: params.pageSize,
          totalRecords: finalList.length,
        });
      })
      .catch((e) => {
        this.setState({
          loading: false,
          list: [],
          pageNo: params.pageNo,
          pageSize: params.pageSize,
          totalRecords: 0,
        });
      });
  };

  /**
   * 获取学生可用优惠券列表（现已改用批量方法：loadMultipleMatchList）
   * 1、先前端判断当前产品数量是否满足优惠券规则
   * 2、再后端判断满足产品数量的优惠券是否符合课程及连报条件
   */
  loadList = (changeParam = {}) => {
    const { productList } = this.props;
    const { selectedList } = this.state;
    const params = {
      promotionNature: 'COUPON_NORMAL',
      ...this.formatListParam(),
      ...changeParam,
    };
    const apiParam = formatCouponListApiParam(params);

    this.setState({ loading: true });
    getPromotionCouponsByStudent(apiParam)
      .then((response) => {
        const result = this.formatListResult(response);
        const couponList = initCouponList(result.list, selectedList, productList);

        this.loadMultipleMatchList(couponList, (finalList) => {
          this.setState({
            loading: false,
            list: finalList || [],
            pageNo: params.pageNo,
            pageSize: params.pageSize,
            totalRecords: result.total,
          });
        });
      })
      .catch((e) => {
        const result = this.formatListResult();
        const couponList = initCouponList(result.list, selectedList, productList);

        this.setState({
          loading: false,
          list: couponList,
          pageNo: params.pageNo,
          pageSize: params.pageSize,
          totalRecords: result.total,
        });
      });
  };

  /**
   * 批量查询优惠券是否可用（可以将loadList合成为一个接口）
   *
   * @param couponList 学生可用的全部优惠券列表
   * @param callback 回调,回调参数为最终优惠列表
   */
  loadMultipleMatchList = (couponList = [], callback) => {
    const { studentId, branchId, productList } = this.props;
    const shouldCheckList = couponList.filter((item) => item.disabled === false);
    const params = { studentId, branchId, productList };

    if (shouldCheckList.length <= 0) {
      callback(couponList);
    } else {
      promiseMultipleMatchCouponList(params)
        .then((response) => {
          const matchIds = formatMultipleMatchCouponIds({ response });
          const finalList = initMathCouponList(couponList, matchIds);

          callback(finalList);
        })
        .catch((e) => {
          callback([]);
        });
    }
  };

  /**
   * 格式化请求参数
   */
  formatListParam = () => {
    const { branchId, studentId } = this.props;
    const { promotionName, continuityType, superPosition, tempDate, pageNo, pageSize } = this.state;
    let startDate = '';
    let endDate = '';

    if (tempDate && tempDate.length >= 2) {
      startDate = tempDate[0].format(dateTimeFormatter);
      endDate = tempDate[1].format(dateTimeFormatter);
    }

    return {
      branchId,
      studentId,
      name: promotionName,
      continuityType,
      beginTime: startDate,
      endTime: endDate,
      superPosition: superPosition !== null ? Number(superPosition) : null,
      pageNo,
      pageSize,
    };
  };

  /**
   * 格式化请求结果
   *
   * @param response
   */
  formatListResult = (response = {}) => {
    const data = response.data || {};
    const { records, total } = data;

    return {
      list: records || [],
      total: total || 0,
    };
  };

  /**
   * 分页改变
   *
   * @param currentPage
   * @param pageSize
   */
  onPageChange = (currentPage, pageSize) => {
    this.loadList({ pageNo: currentPage, pageSize });
  };

  onSearch = () => {
    this.loadList({ pageNo: 1 });
  };

  onReset = () => {
    this.setState(this.initSearchParam);
  };

  /**
   * 确认选择(可以不使用优惠券)
   */
  onOk = () => {
    const { selectedList } = this.state;

    this.props.onOk(selectedList);
  };

  /**
   * 选择有效期
   *
   * @param moments []
   */
  onDateRangeChange = (moments) => {
    this.setState({ tempDate: moments });
  };

  /**
   * 选择/取消选择优惠券
   *
   * @param coupon 当前选择优惠券
   */
  onCouponSelect = (coupon) => {
    this.setState((prevState) => {
      const { resultList, resultSelectedList } = formatListAfterSelect({
        coupon,
        list: prevState.list || [],
        selectedList: prevState.selectedList,
        multiple: this.props.multiple,
        limit: this.props.limit,
      });

      return {
        list: resultList,
        selectedList: resultSelectedList,
      };
    });
  };

  render() {
    const { productList } = this.props;
    const { loading, promotionName, continuityType, superPosition, tempDate, pageNo, pageSize, totalRecords, list } =
      this.state;

    return (
      <Modal
        title="选择优惠券"
        cancelText="取消"
        okText="确认选择"
        okBtnShow={true}
        okBtnDisabled={false}
        loading={loading}
        onCancel={this.props.onCancel}
        onOk={this.onOk}
        className={styles.couponModal}
      >
        <Wrapper
          bodyFitHeight
          header={
            <Row gutter={8}>
              <Col span={4}>
                <Input
                  placeholder="券名称"
                  value={promotionName}
                  onChange={(e) => this.setState({ promotionName: e.target.value })}
                />
              </Col>
              <Col span={3}>
                <Select
                  placeholder="连报条件"
                  showSearch={true}
                  dict={continuityTypeDict}
                  value={continuityType}
                  onChange={(val) => this.setState({ continuityType: val })}
                />
              </Col>
              <Col span={3}>
                <Select
                  placeholder="是否叠加"
                  showSearch={true}
                  dict={superPositionDict}
                  value={superPosition}
                  onChange={(val) => this.setState({ superPosition: val })}
                />
              </Col>
              <Col span={8}>
                <RangePicker
                  size="large"
                  placeholder={['开始时间', '结束时间']}
                  format="YYYY-MM-DD HH:mm:ss"
                  showTime={{
                    hideDisabledOptions: true,
                    defaultValue: [moment(defaultTimeEarly, 'HH:mm:ss'), moment(defaultTimeLate, 'HH:mm:ss')],
                  }}
                  value={tempDate}
                  onChange={this.onDateRangeChange}
                  style={{ width: '100%' }}
                />
              </Col>
              <Col span={5}>
                <Button type="primary" onClick={this.onSearch} style={{ marginRight: 8 }}>
                  查询
                </Button>
                <Button onClick={this.onReset}>重置</Button>
              </Col>
            </Row>
          }
          footer={
            showPager ? (
              <Pagination
                current={pageNo}
                pageSize={pageSize}
                total={totalRecords}
                pageSizeOptions={['10', '20']}
                showSizeChanger
                showQuickJumper
                onChange={this.onPageChange}
                onShowSizeChange={this.onPageChange}
              />
            ) : (
              ''
            )
          }
        >
          {list && list.length > 0 ? (
            <CouponList list={list} productList={productList} onSelect={this.onCouponSelect} />
          ) : (
            <NoDataPlus text="暂无可用优惠券" />
          )}
        </Wrapper>
      </Modal>
    );
  }
}

export default Index;
