import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { SMALL_CLASS, TWO_TEACHER } from '../config';
import { COUPON_ID_KEY, COUPON_PROMOTION_RULE_NUM_KEY } from './config';

const idKey = COUPON_ID_KEY;
const ruleNumKey = COUPON_PROMOTION_RULE_NUM_KEY;

/**
 * 初始化请求的优惠券列表(暂时没有不可用状态优惠券)
 * 1、选中在前，禁用在后
 * 2、对于选中的券，下次打开弹窗将已选券前置到第一个
 * 3、排序后添加一个是否选中状态(selected)、是否可选状态属性(disabled)
 *
 * @param list 后端返回的学生能用优惠券列表
 * @param selectedList 合同列表选中的优惠券
 * @param productList {array} 已选产品
 * @return {Array}
 */
export function initCouponList(list = [], selectedList = [], productList = []) {
  let result = [];
  const hasList = list && list.length > 0;
  const hasSelected = selectedList && selectedList.length > 0;

  if (!hasList) {
    return result;
  }

  if (hasSelected) {
    const selectedIds = selectedList.map((item) => item[idKey]);

    result = list.map((item) => {
      const disabled = formatCouponDisableStatus(item, productList);

      return {
        ...item,
        disabled,
        selected: selectedIds.indexOf(item[idKey]) > -1,
      };
    });

    sortDisableCouponList(result);
  } else {
    result = list.map((item, index) => {
      const disabled = formatCouponDisableStatus(item, productList);

      return {
        ...item,
        disabled,
        selected: false,
      };
    });

    sortDisableCouponList(result);
  }

  return result;
}

/**
 * 前端判断完优惠券是否符合产品数量后，进行课程及连报规则的能用情况
 *
 * @param list {array} 前端初步处理后的学生能用优惠券列表（前端规则判断及排序）
 * @param mathIds {array<string>} 后端匹配的可用的优惠券ID列表
 */
export function initMathCouponList(list = [], mathIds = []) {
  const result = list.map((item) => {
    return {
      ...item,
      disabled: !(mathIds.indexOf(item[idKey]) > -1),
    };
  });

  sortDisableCouponList(result);

  return result;
}

/**
 * 选择优惠券时格式化选中的列表(只改变selected状态)
 *
 * @param coupon 当前选择的优惠券
 * @param list 所有优惠券
 * @param multiple 是否多选
 * @param limit 可选数量限制
 * @param selectedList 已经选中的优惠券
 * @return {*}
 */
export function formatListAfterSelect({ coupon, list, selectedList, multiple, limit }) {
  if (!coupon || !list || list.length <= 0) {
    return [];
  }

  // 当前点击的优惠券的状态
  // 当前是选中状态，需要取消选中（取消选中不需要判断是否可多选）
  // 当前是未选中状态，需要选中（选中则需要判断是否可多选）
  const { selected } = coupon;
  const oldSelectedList = selectedList || [];
  const oldSelectedLen = oldSelectedList.length;
  let resultSelectedList = [];

  // 取消选中
  if (selected) {
    list.forEach((item) => {
      if (item[idKey] === coupon[idKey]) {
        item.selected = false;
        return false;
      }
    });

    oldSelectedList.forEach((item) => {
      if (item[idKey] !== coupon[idKey]) {
        resultSelectedList.push(item);
      }
    });

    return {
      resultList: list,
      resultSelectedList: resultSelectedList,
    };
  }

  // 单选-选中 (不需要判断是否已有选中优惠)
  const isSingle = !multiple || limit === 1;

  if (isSingle) {
    list.forEach((item) => {
      item.selected = item[idKey] === coupon[idKey];
    });

    return {
      resultList: list,
      resultSelectedList: [coupon],
    };
  }

  // 多选-选中 (需要判断最大可选数量)
  const hasLimit = limit >= 0;

  if (hasLimit && oldSelectedLen >= limit) {
    showMessage(`最多可选${limit}个优惠券！`, 'error');
    return {
      resultList: list,
      resultSelectedList: oldSelectedList,
    };
  }

  // 不可叠加
  if (oldSelectedLen > 0 && (oldSelectedList[0].superPosition === 0 || coupon.superPosition === 0)) {
    showMessage(`不可叠加当前优惠券！`, 'error');
    return {
      resultList: list,
      resultSelectedList: oldSelectedList,
    };
  }

  list.forEach((item) => {
    if (item[idKey] === coupon[idKey]) {
      item.selected = true;
      return false;
    }
  });

  resultSelectedList = oldSelectedList.concat([coupon]);

  return {
    resultList: list,
    resultSelectedList: resultSelectedList,
  };
}

/**
 * 格式化(获取)当前分页选中的优惠券
 *
 * @param list 所有优惠券列表
 * @return {Array|*[]}
 */
export function formatPageSelectedList(list = []) {
  if (!list || list.length <= 0) {
    return [];
  }

  const selected = list.filter((item) => item.selected);

  return selected.map((item) => {
    let coupon = { ...item };

    delete coupon.selected;
    delete coupon.disabled;

    return coupon;
  });
}

/**
 * 优惠列表后端最终请求参数
 *
 * @param params 表单参数
 * @return {*}
 */
export function formatCouponListApiParam(params = {}) {
  const param = { ...params };

  param.current = params.pageNo;
  param.size = params.pageSize;

  delete param.pageNo;
  delete param.pageSize;

  return param;
}

/**
 * 格式化优惠券的禁用状态
 *
 * @param coupon 优惠券
 * @param productList 已选产品列表
 * @return {boolean} true: 禁用，false：可用
 */
export function formatCouponDisableStatus(coupon, productList = []) {
  const hasProduct = productList && productList.length > 0;

  if (!coupon || !hasProduct) {
    return true;
  }

  const ruleVo = coupon.promotionRuleVos || [];
  if (ruleVo.length <= 0) {
    return true;
  }

  const hitRule = findHitRule(ruleVo, productList);

  return !hitRule;
}

/**
 * 找到命中的规则
 * 根据产品列表找到命中的优惠券使用规则 (已选产品数量 >= promotionRule)
 *
 * @param ruleVos 优惠券使用规则集合
 * @param productList 产品列表
 */
export function findHitRule(ruleVos = [], productList = []) {
  return ruleVos.find((rule) => {
    return productList.length >= rule[ruleNumKey];
  });
}

/**
 * 优惠券排序
 * 1、截止时间越近的越往前
 * 2、将禁用的优惠券排在后面
 * 3、将选中的排在前面
 *
 * @param list 优惠券列表
 */
export function sortDisableCouponList(list = []) {
  list.sort((a, b) => {
    return new Date(a.endDate).getTime() - new Date(b.endDate).getTime();
  });

  list.sort((a, b) => {
    return a.disabled - b.disabled;
  });

  list.sort((a, b) => {
    return b.selected - a.selected;
  });
}

/**
 * 过滤小班与双师产品列表
 *
 * @param productList
 * @return {Array|*[]}
 */
export function filterValidProductList(productList = []) {
  if (!productList) {
    return [];
  }

  return productList.filter((item) => {
    return isValidProduct(item, true);
  });
}

/**
 * 过滤非小班、双师产品列表
 *
 * @param productList
 * @return {Array|*[]}
 */
export function filterUnValidProductList(productList = []) {
  if (!productList) {
    return [];
  }

  return productList.filter((item) => {
    return isValidProduct(item, false);
  });
}

/**
 * 判断产品是否为小班或双师产品
 *
 * @param product {*} 产品
 * @param valid {boolean} true: 小班或双师产品，false: 不是小班与双师产品
 */
function isValidProduct(product = {}, valid = true) {
  if (!product) {
    return false;
  }

  const { productType } = product;
  const isXB = productType === SMALL_CLASS || productType === TWO_TEACHER;

  return valid ? isXB : !isXB;
}

/**
 * 格式化报读接口需要的班级ID
 * 班级id: 小班请在班级id前拼接"XB-",双师请在班级id前拼接"SS-"
 *
 * @param productList 产品列表
 * @return {any[]}
 */
export function formatBDApiClassIds(productList = []) {
  return productList.map((item) => {
    if (item.productType === SMALL_CLASS) {
      return 'XB-' + item.classId;
    } else if (item.productType === TWO_TEACHER) {
      return 'SS-' + item.classId;
    }

    return false;
  });
}

/**
 * 将是否可用优惠券结果列表转换为优惠券ID字符串的匹配
 * 取巧：能用的优惠券ID push进一个数组
 *
 * @param couponList 需要检查的优惠券列表
 * @param responseList 结果列表集合（与需要检查的优惠券列表一一对应）
 */
export function formatMatchCouponIds({ couponList = [], responseList = [] }) {
  let matchIds = [];

  responseList.forEach((result, index) => {
    if (result && result.data === true) {
      matchIds.push(couponList[index][idKey]);
    }
  });

  return matchIds;
}

/**
 * 计算选中产品中，可使用优惠的产品(小班、双师)的实际支付金额总金额（优惠过后）
 *
 * @param productList {array} 选中产品，需过滤不是小班双师的产品
 * @return {number}
 */
export function formatBDRealAmount(productList = []) {
  if (!productList) {
    return 0;
  }

  return productList.reduce((total, currentItem) => {
    if (isValidProduct(currentItem, true)) {
      return total + currentItem.realAmount;
    }

    return total + 0;
  }, 0);
}

/**
 * 提取批量查询的优惠券列表中的可用ID
 *
 * @param response {*} {data: {suitablePromotion: [可用优惠券], unSuitablePromotion: [不可用优惠券]}}
 * @return {array<string>}
 */
export function formatMultipleMatchCouponIds({ response = {} }) {
  if (!response) {
    return [];
  }

  const data = response.data || {};
  const suitableList = data.suitablePromotion || [];

  return suitableList.map((item) => item[idKey]);
}
