import React from 'react';
import ReactDOM from 'react-dom';
import { connect } from 'dva';
import style from './styles.module.less';
import { Tabs } from 'antd';
import { List, InputNumber, MultiSelect, Popover, Loading, ListAction } from 'COMPONENT';
import Quarter from '../components/Quarter';
import TableSubject from './TableSubject';
import TableProduct from './TableProduct';
import TableClass from './TableClass';
import ModalChangeLinkPrdSS from '../ModalChangeLinkPrdSS';
import { storeName, TAB_TRIGGER } from '../store/storeCfg';
import { takeOutPromotionProperties } from '../store/storeUtils';
import { focusResize } from '../utils';
import { SMALL_CLASS, TWO_TEACHER, UNVALID, OTHERS, quarter } from '../config';
import { runFunc } from 'UTIL/common';
import { debounce } from 'lodash';
import { renderPromotionName, findHitParentProduct, filterPromotionByContractType } from './util';

const TabPane = Tabs.TabPane;
const setWidth = List.createSetWidth();

/**
 * 双师的产品列表
 */
class ProductTable extends React.Component {
  constructor(props) {
    super(props);

    this.debounceCalc = debounce(this.calcPrdPromotion, 600);
    this.state = {
      modalVisible: false,
      changeProductData: null,
      changeProductIndex: null,
    };
    this.columns = [
      {
        title: '季度',
        dataIndex: 'quarterId',
        key: 'quarterId',
        width: setWidth(4),
        render: (text) => {
          return text ? <Quarter id={text} isTwoTeacher={true} /> : '-';
        },
      },
      {
        title: '科目',
        dataIndex: 'subjectName',
        key: 'subjectName',
        width: setWidth(7),
        render: (text, record) => {
          let subjectName = text || '';

          if (record.productType === OTHERS) {
            const hitParentProduct = findHitParentProduct(this.props.selectedList, record);

            if (hitParentProduct) {
              subjectName = hitParentProduct.subjectName;
            }
          }

          return <TableSubject type={record.productTypeName} name={subjectName} />;
        },
      },
      {
        title: '产品名称',
        key: 'productInfo',
        width: setWidth(7),
        render: (text, record) => {
          return <TableProduct type={record.productTypeName} name={record.productName || ''} />;
        },
      },
      {
        title: '班级',
        key: 'classInfo',
        width: setWidth(9),
        render: (text, record) => {
          return record.classId ? <TableClass data={record} today={this.props.timeText} /> : '-';
        },
      },
      {
        title: '上课开始时间',
        key: 'startClassDateAndTime',
        width: setWidth(7),
        render: (text, record) => {
          return record.classId ? (
            <div>
              <p>{record._courseDate || record.startClassDate}</p>
              <p>{record.startClassTime}</p>
            </div>
          ) : (
            '-'
          );
        },
      },
      {
        title: '老师',
        key: 'teacherName',
        width: setWidth(4),
        render: (text, record) => {
          return (
            ([SMALL_CLASS, TWO_TEACHER].includes(record.productType)
              ? record.assistTeacherName
              : record.mainTeacherName) || '-'
          );
        },
      },
      { title: '单价', dataIndex: 'price', key: 'price', width: setWidth(4) },
    ];
    this.linkColumns = this.columns.concat([
      { title: '状态', dataIndex: 'status', key: 'status', width: setWidth(2) },
      {
        title: '操作',
        key: 'actions',
        width: setWidth(6),
        render: (text, record, index) => {
          let subTableLen = record._list.length;
          if (subTableLen > 0) {
            return (
              <ListAction
                config={[
                  {
                    text: `更改班级(${subTableLen})`,
                    onClick: () => {
                      this.switchModalVisible(record, index, 'changeLinkPrd');
                    },
                  },
                ]}
                direction="column"
              />
            );
          }
          return null;
        },
      },
    ]);
    this.selectedColumns = this.columns.concat([
      {
        title: '数量',
        dataIndex: 'quantity',
        key: 'quantity',
        width: setWidth(4),
        render: (text, record) => {
          return (
            <InputNumber
              value={text}
              onChange={(val) => this.updatePrdQuantity(val, record._originIndex)}
              min={0}
              disabled={record._editQuantityDisabled || (record._quantityDisabled && Boolean(record._hadInitQuantity))}
            />
          );
        },
      },
      { title: '产品总额', dataIndex: 'payment', key: 'payment', width: setWidth(5) },
      {
        title: '优惠',
        key: 'usedPromotion',
        width: 200,
        render: (text, record) => {
          if (Array.isArray(record._activePromotion) && record._activePromotion.length > 0) {
            return renderPromotionName(record._activePromotion);
          }

          let dict = null;
          const disabled = !record.promotions || record.promotions.length === 0;

          if (!disabled) {
            const tempDict =
              record.promotions[0].id === '-1'
                ? this.props.productPromotionDict
                : record.promotions.map((item) => {
                    return { text: item.name, val: item.id, properties: takeOutPromotionProperties(item) };
                  });

            // 优惠券过滤
            dict = filterPromotionByContractType(tempDict, this.props.contractType);
          }

          const multiSelect = (
            <MultiSelect
              placeholder="请选择可用优惠"
              dict={dict}
              value={record._prdPromotionVal}
              onChange={(val) => this.updatePrdPromotion(val, record._originIndex)}
              disabled={record._editPrdPromotionDisabled || !record.quantity}
              selectAll={false}
              noOptionTips="没有可用优惠"
              showSearch
            />
          );

          return record._warningBeforeEditPrdPromotion ? (
            <Popover
              content="修改原合同产品优惠将重置优惠款"
              placement="top"
              overlayClassName={style.warningOverlay}
              key={record.contractProductId}
            >
              <div>{multiSelect}</div>
            </Popover>
          ) : (
            multiSelect
          );
        },
      },
      { title: '优惠款', dataIndex: 'promotionAmount', key: 'promotionAmount', width: setWidth(4) },
      { title: '应收款', dataIndex: 'realAmount', key: 'realAmount', width: setWidth(4) },

      { title: '状态', dataIndex: 'status', key: 'status', width: setWidth(3) },
      {
        title: '操作',
        key: 'actions',
        width: setWidth(6),
        fixed: 'right',
        render: (text, record) => {
          let cfg = [];

          if (!record._btnDeleteDisabled) {
            cfg.push({
              text: '删除',
              onClick: () => {
                this.deleteSelectedPrd(record, record._originIndex);
              },
            });
          } else if (this.props.isEdit && record.contractProductId && !record._btnInvalidDisabled) {
            cfg.push({
              text: record._contractPrdStatus === UNVALID ? '撤销' : '取消',
              onClick: () => {
                this.switchContractPrdInvalid(record, record._originIndex);
              },
            });
          }

          if (record._btnClearPromotionVisible) {
            cfg.push({
              text: '清除优惠',
              onClick: () => {
                this.clearPromotion(record, record._originIndex);
              },
              needConfirm: true,
              confirmTips: '确认清除优惠吗？',
            });
          }

          if (record.hasOwnProperty('_list')) {
            const subTableLen = record._list.length;

            if (subTableLen > 0) {
              cfg.unshift({
                text: `更改班级(${subTableLen})`,
                onClick: () => {
                  this.switchModalVisible(record, record._originIndex, 'changeSelectedPrd');
                },
              });
            }
          }

          return <ListAction config={cfg} direction="column" />;
        },
      },
    ]);

    this.SelectedListRef = null;
  }

  componentDidMount() {
    // FIXME 在已选产品列表时，固定操作列无法更新宽度
    window.addEventListener('resize', this.setTableFixedWidth, false);
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.setTableFixedWidth, false);
  }

  // 补偿表格固定操作宽度列表(rc-table bug)
  setTableFixedWidth = () => {
    const listNode = ReactDOM.findDOMNode(this.SelectedListRef);
    let fixedCol = listNode.querySelector('.ant-table-fixed-right');
    if (fixedCol) {
      const tableWrap = listNode.querySelector('.ant-table-scroll');
      const thNodes = tableWrap.querySelector('.ant-table-header').querySelectorAll('th');
      const actionWidth = thNodes[thNodes.length - 1].offsetWidth;
      fixedCol.style['min-width'] = `${actionWidth + 7}px`;
      fixedCol.style['max-width'] = `${actionWidth + 7}px`;
    }
  };

  getSelectedColumns = (columns) => {
    let isInsetCol = columns.findIndex((item) => item.key === 'promotionConsumeAmount') > -1;
    if (this.props.isEdit && !isInsetCol) {
      columns.splice(-2, 0, {
        title: '已扣优惠',
        dataIndex: 'promotionConsumeAmount',
        key: 'promotionConsumeAmount',
        width: setWidth(5),
      });
    } else if (!this.props.isEdit && isInsetCol) {
      columns.splice(-3, 1);
    }
    return columns;
  };

  switchModalVisible = (record, index, type) => {
    this.setState((prevState) => {
      let payload = { modalVisible: !prevState.modalVisible };
      if (payload.modalVisible) {
        payload.changeProductData = record;
        payload.changeProductIndex = index;
        payload.changeProductType = type;
      } else {
        payload.changeProductData = null;
        payload.changeProductIndex = null;
        payload.changeProductType = null;
      }
      return payload;
    });
  };

  onLinkSelect = (record, selected, selectedRows) => {
    this.props.dispatch({
      type: `${storeName}/synLinkPrdToSelected`,
      payload: {
        selected,
        updateRows: [record],
      },
    });

    this.props.dispatch({
      type: `${storeName}/updateLinkSelect`,
      payload: { record, selected, selectedRows },
    });
  };

  onLinkSelectAll = (selected, selectedRows, changeRows) => {
    this.props.dispatch({
      type: `${storeName}/synLinkPrdToSelected`,
      payload: {
        selected,
        updateRows: changeRows,
      },
    });

    this.props.dispatch({
      type: `${storeName}/updateLinkSelectAll`,
      payload: { selected, selectedRows, changeRows },
    });
  };

  deleteSelectedPrd = (record, index) => {
    this.props.dispatch({
      type: `${storeName}/deleteSelectedPrd`,
      payload: { record, index },
    });
  };

  updatePrdQuantity = (val, index) => {
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/changePrdQuantity`,
      payload: {
        val,
        index,
        callback(payload) {
          runFunc(_self.debounceCalc, payload);
        },
      },
    });
  };

  calcPrdPromotion = (payload) => {
    this.props.dispatch({
      type: `${storeName}/calcPrdPromotion`,
      payload,
    });
  };

  updatePrdPromotion = (val, index) => {
    this.props.dispatch({
      type: `${storeName}/changePrdPromotion`,
      payload: { val, index },
    });
  };

  clearPromotion = (record, index) => {
    this.props.dispatch({
      type: `${storeName}/clearPromotion`,
      payload: { record, index },
    });
  };

  switchContractPrdInvalid = (record, index) => {
    this.props.dispatch({
      type: `${storeName}/switchContractPrdInvalid`,
      payload: { record, index },
    });
  };

  /**
   * 将产品列表排序，产品类型为`其他`的排在后面
   * tips: 仅针对双师的已选产品列表
   * tips: 不改变store源数组，添加`_originIndex`字段对应源数组index，用于操作产品
   *
   * @param list 已选产品
   * @return {*[]}
   */
  sortProductList = (list = []) => {
    const result = list.map((item, index) => {
      return {
        ...item,
        _originIndex: index,
      };
    });

    // 按照科目类型排序
    result.sort((a, b) => {
      const aOthers = a.productType === OTHERS;
      const bOthers = b.productType === OTHERS;

      return aOthers - bOthers;
    });
    // 按照季度排序
    return result.sort((a, b) => {
      const aQuater = quarter[a.quarterId] || 5;
      const bQuater = quarter[b.quarterId] || 5;
      return aQuater - bQuater;
    });
  };

  render() {
    const selectedProductList = this.sortProductList(this.props.selectedList);
    const ListSelected = (
      <List
        ref={(c) => (this.SelectedListRef = c)}
        dataSource={selectedProductList}
        columns={this.getSelectedColumns(this.selectedColumns)}
        bordered
      />
    );
    let ProductTable = ListSelected;

    if (this.props.funcAuth.includes(TAB_TRIGGER)) {
      ProductTable = (
        <Tabs defaultActiveKey="2" onTabClick={focusResize}>
          <TabPane tab="续报班级" key="1">
            <List
              dataSource={this.props.linkList}
              columns={this.linkColumns}
              rowSelection={{
                selectedRowKeys: this.props.linkRowKeys,
                onSelect: this.onLinkSelect,
                onSelectAll: this.onLinkSelectAll,
              }}
              bordered
            />
          </TabPane>
          <TabPane tab={`已选产品${this.props.count ? `(${this.props.count})` : ''}`} key="2">
            {ListSelected}
          </TabPane>
        </Tabs>
      );
    }

    return (
      <div id="ProductTable" className={style.ProductTable}>
        {ProductTable}
        <Loading loading={this.props.calcLoading} tip="优惠金额计算中..." />
        {this.state.modalVisible && (
          <ModalChangeLinkPrdSS
            data={this.state.changeProductData}
            index={this.state.changeProductIndex}
            type={this.state.changeProductType}
            onCancel={this.switchModalVisible}
          />
        )}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    contractType: state[storeName].contractType,
    timeText: state[storeName].timeText,
    isEdit: state[storeName].isEdit,
    funcAuth: state[storeName].funcAuth,
    isEcs: state[storeName].isEcs,
    calcLoading: state[storeName].calcLoading,
    linkExpandKeys: state[storeName].linkExpandKeys,
    linkRowKeys: state[storeName].linkRowKeys,
    linkList: state[storeName].linkList,
    selectedList: state[storeName].selectedList,
    count: state[storeName].selectedList.length,
    productPromotionDict: state[storeName].productPromotionDict,
  };
};

export default connect(mapStateToProps)(ProductTable);
