import NP from 'number-precision';
import {
  ECS_CLASS,
  contractTypeCfg,
  SMALL_CLASS,
  TWO_TEACHER,
  OTHERS,
  ENDED,
  CLOSE_PRODUCT,
  UNVALID,
  FROZEN,
  OTHER_CATEGORY_TYPE_MARKETING,
} from '../config';

function checkSamePrd(prd) {
  return prd.productId + (prd.classId || '');
}

const relation = {
  count: 0,
  set() {
    let ret = `${new Date().getTime()}${this.count}`;
    this.addCount();
    return ret;
  },
  addCount() {
    this.count++;
  },
};
/**
 * 获取产品关联标识
 * @return {String} _relation
 * */
function setPrdRelation() {
  return relation.set();
}

/**
 * 获取产品关联标识
 * @param prd {Object} 产品
 * @return {String} _relation
 * */
function getPrdRelation(prd) {
  return prd._relation || null;
}

/**
 * 添加产品及其子产品,建立产品与子产品的关联关系
 * tips: 将子产品从父产品分离，并添加父产品的`classId`、`productType`保存到后端（仅其他类型的产品需要）
 * tips: 父子产品建立relationId
 * tips: 添加`relateClassId`字段 后端使用
 * tips: 添加`relateProductType`字段 后端使用
 * tips: 修改此处字段时注意修改如下方法
 *  `/ProductTableSS/util.js/findHitParentProduct()`方法
 *  `/store/index/submit`参数
 *  下方的`contractInfoAdepter()`方法
 *
 * @param prd {Object} 新增产品
 * @return {Array} 产品列
 * */
function addProductWithSub(prd) {
  const _relation = setPrdRelation();

  prd = { ...prd, _relation };

  if (Array.isArray(prd.subProducts) && prd.subProducts.length > 0) {
    const isEcs = prd.productType === ECS_CLASS;
    const subPrdList = prd.subProducts.map((item) => {
      return {
        ...item,
        // 保存父级关联关系
        _relation,
        _btnDeleteDisabled: isEcs,
        // 子产品保存父班级id
        _relateClassId: prd.classId,
        // 子产品保存父产品类型（用于后端区分系统）
        _relateProductType: prd.productType,
        // 子产品数量从count取
        quantity: item.count,
      };
    });

    return [prd].concat(subPrdList);
  }

  return [prd];
}

/**
 * 批量添加产品及其子产品
 * @param list {Array} 新增产品列
 * @return {Array} 产品列
 * */
function batchAddProductWithSub(list) {
  return list.map((prd) => addProductWithSub(prd)).reduce((ret, cur) => ret.concat(cur), []);
}

// 合同详情数据设配器
function contractInfoAdepter(data) {
  let result = {
    contractId: data.contractId,
    contractType: data.contractType,
    contractTypeName: contractTypeCfg[data.contractType],
    isEcs: data.ecsContract,
    customerId: data.customerId,
    customerName: data.customerName,
    customerContact: data.customerContact,
    studentId: data.studentId,
    studentName: data.studentName,
    signDate: data.signDate,
    blCampusId: data.blCampusId,
    schoolName: data.schoolName,
    schoolId: data.isVirtualSchool ? null : data.schoolId,
    virtualSchool: data.isVirtualSchool ? { schoolName: data.schoolName } : null,
    gradeId: data.contractGradeId,
    remark: data.remark,
    transferIntroduceId: data.transferIntroduceId,
    totalAmount: data.totalAmount,
    promotionAmount: data.promotionAmount,
    realAmount: data.realAmount,
    couponList: [],
    selectedList: [],
  };

  const hadCoupon = Array.isArray(data.promotionCouponArr) && data.promotionCouponArr.length > 0;
  if (hadCoupon) {
    result.couponList = data.promotionCouponArr.map((item) => ({
      ...item,
      _couponName: item.promotionCouponName,
      _couponId: item.promotionCouponId,
      _ruleId: item.promotionCouponRuleId,
      _couponDiscount: item.promotionCouponAmount,
    }));
  }

  if (Array.isArray(data.contractProductArr) && data.contractProductArr.length > 0) {
    result.selectedList = data.contractProductArr.map((item) => {
      // 优惠金额大于0的默认有优惠(对于暑秋连报的优惠活动，可能存在某一季度的优惠是0的情况，所以加上是否有优惠活动的判断)
      const hadPromotionAmount =
        (item.promotionAmount && item.promotionAmount > 0) || (item.promotionActivityArr || []).length > 0;
      const hadUsedCoupon = hadCoupon && [SMALL_CLASS, TWO_TEACHER].includes(item.productType);

      return {
        ...item,
        quantity: item.courseHours,
        status: item.statusName,
        promotions: (item.optionalPromotions || []).map((item) => ({
          id: item.promotionId,
          name: item.promotionName,
          properties: takeOutPromotionProperties(item),
        })),
        payment: item.totalAmount,
        realAmount: item.realPayAmount,
        _contractPrdStatus: item.status,
        _statusCache: item.status,
        _quantityDisabled:
          item.productType === OTHERS && item.otherCategoryId === OTHER_CATEGORY_TYPE_MARKETING
            ? false
            : [SMALL_CLASS, TWO_TEACHER, OTHERS, ECS_CLASS].includes(item.productType), // 产品类型为"其他",并且费用类型为"市场",则产品数量可编辑
        _hadInitQuantity: typeof item.quantity === 'number' ? item.quantity : true,
        _prdPromotion: (item.promotionArr || []).map((item) => ({
          id: item.promotionId,
          name: item.promotionName,
          properties: takeOutPromotionProperties(item),
        })),
        _prdPromotionVal: (item.promotionArr || []).map((item) => item.promotionId),
        // 自动计算优惠才需要传优惠活动
        _activePromotion: (item.promotionActivityArr || []).map((item) => {
          return {
            name: item.promotionActivityName,
            id: item.promotionActivityId,
            rule: item.promotionActivityRuleId,
            type: item.promotionType || null,
            ruleDetail: item.rule || {},
          };
        }),
        /**
         * 这里这样处理是因为当存在赠送课时，产品总额会大于单价*数量，所以必需取接口返回的总额
         * 备注：之前产品数量输入未限制小数点后2位，而后端存在该限制，会导致总价也大于单价*数量，产生修改产品数量时，未能更新价格的bug
         * */
        _paymentWithPrdPromotion: item.totalAmount > NP.times(item.courseHours, item.price) ? item.totalAmount : 0,
        // 非无效的合同产品，优惠金额大于0并且优惠金额大于已扣优惠，可以清空优惠
        _btnClearPromotionVisible:
          hadPromotionAmount && item.status !== UNVALID && item.promotionAmount >= item.promotionConsumeAmount,
        // 修改合同产品的产品优惠时，是否需要提示
        _warningBeforeEditPrdPromotion: hadPromotionAmount && hadUsedCoupon,
        _relateClassId: item.relateClassId,
        _relateProductType: item.relateProductType,
        ...getContractPrdActions(item, result.isEcs),
      };
    });
  }
  // FIXME: 详情接口返回的数据缺少产品关联关系，无法为其添加`_relation`字段
  return result;
}

// 获取合同产品 更改/操作visible
function getContractPrdActions(contractProduct, isEcs) {
  if ([ENDED, CLOSE_PRODUCT, UNVALID, FROZEN].includes(contractProduct.status)) {
    // 不可无效、不可删除、不可改数量、不可改、不可清空优惠
    return {
      _editQuantityDisabled: true,
      _editPrdPromotionDisabled: true,
      _btnDeleteDisabled: true,
      _btnInvalidDisabled: true,
      _btnClearPromotionVisible: false,
    };
  } else if (contractProduct.consumeAmount > 0) {
    // 不可删除、不可无效
    return {
      _btnDeleteDisabled: true,
      _btnInvalidDisabled: true,
    };
  } else if (contractProduct.isWashed) {
    // 冲销的 不可删除
    // 冲销且已付款的 不可无效
    return {
      _btnDeleteDisabled: true,
      _btnInvalidDisabled: contractProduct.paidAmount > 0,
    };
  } else {
    // 不可无效
    // 修改合同时，(合同产品)精英班的其他类型产品不可删除(按照旧逻辑)
    return {
      _btnDeleteDisabled: isEcs && contractProduct.productType === OTHERS,
      _btnInvalidDisabled: true,
    };
  }
}

/**
 * 取出优惠券的优惠属性字段
 * tips: 不同接口返回字段不同 promotionProperties|propertiesId
 * tips: 前端组装的字典取 properties
 *
 * @param promotion
 * @return {null|*|null}
 */
function takeOutPromotionProperties(promotion) {
  if (!promotion) {
    return null;
  }

  return promotion.promotionProperties || promotion.propertiesId || promotion.properties || null;
}

export {
  checkSamePrd,
  setPrdRelation,
  getPrdRelation,
  addProductWithSub,
  batchAddProductWithSub,
  contractInfoAdepter,
  takeOutPromotionProperties,
};
