import React from 'react';
import PropTypes from 'prop-types';
import moment from 'moment';
import { Row, Col, DatePicker, Pagination } from 'antd';
import { Modal, Wrapper, NoDataPlus, Input, Select, Button } from 'COMPONENT';
import styles from './index.less';
import CouponList from './CouponListClassify';
import {
  initCouponList,
  initMathCouponList,
  formatListAfterSelect,
  formatCouponListApiParam,
  formatMultipleMatchCouponIds,
  formatBDRealAmount,
  judgeIsSatisfyCouponAmount,
  judgeIsSatisfyCouponContinueLimit,
  initMathSelectedList,
  formatClassInfoApiParam,
} from './util';
import {
  loadStudentPromotion,
  promiseMultipleMatchCouponList,
  loadMaxCouponPromotion,
  loadLimitForUseCoupon,
} from './loader';
import {
  continuityTypeDict,
  superPositionDict,
  promotionNatureDict,
  PROMOTION_NATURE_DEPOSIT,
  PROMOTION_NATURE_NORMAL,
} from './dict';
import { dateTimeFormatter, defaultTimeEarly, defaultTimeLate, DEFAULT_PAGE_SIZE, SHOW_PAGER } from './config';
import { CLASS_COURSE, ONE_ON_ONE_COURSE, ONE_ON_MANY } from '../config';

const { RangePicker } = DatePicker;
const pageSize = DEFAULT_PAGE_SIZE;
const showPager = SHOW_PAGER;

/**
 * 选择优惠券弹窗
 * 单选：{multiple: false} 或 {multiple: true, limit: 1}
 * 多选(限制0或N)：{multiple: true, limit: 大于等于0且不等于1的数}
 * 多选(无限制)：{multiple: true, limit: 小于0的数}
 *
 * @param props.multiple {boolean} 是否可多选，默认false
 * @param props.limit {number} 可选数量限制，仅当multiple=true时有效，传入小于0的数则不限制，默认1
 * @param props.branchId {string} 合同(学生)分公司ID
 * @param props.studentId {string} 学生ID
 * @param props.customerContact {string} 客户手机号码
 * @param props.couponList {array} 已选优惠券
 * @param props.productList {array} 已选产品(过滤后的小班和双师产品)
 * @param props.realAmount {number} 合同应收款合计(优惠后的总金额)
 * @param props.onCancel {function} 点击关闭
 * @param props.onOk {function} 确认选择，回调参数为selectedList(选中的优惠券)
 * @param props.groupType {string} 合同类型 (CLASS_COURSE: 班课, ONE_ON_ONE_COURSE: 1对1, ONE_ON_MANY: 1对多)
 * @param props.isYinLiu {boolean} 产品性质是否引流 (目前只有常规班课、常规个性化1对1的合同才可使用优惠券)
 */
class Index extends React.Component {
  static propTypes = {
    multiple: PropTypes.bool,
    limit: PropTypes.number,
    branchId: PropTypes.string,
    studentId: PropTypes.string,
    customerContact: PropTypes.string,
    contactList: PropTypes.arrayOf(PropTypes.string),
    couponList: PropTypes.array,
    productList: PropTypes.array,
    realAmount: PropTypes.number,
    onCancel: PropTypes.func,
    onOk: PropTypes.func,
    groupType: PropTypes.oneOf([CLASS_COURSE, ONE_ON_ONE_COURSE, ONE_ON_MANY]).isRequired,
    isYinLiu: PropTypes.bool,
    useCase: PropTypes.oneOf(['ALL', 'RE', 'NEW']).isRequired,
  };

  static defaultProps = {
    multiple: false,
    limit: 1,
    branchId: null,
    studentId: null,
    couponList: [],
    productList: [],
    realAmount: 0,
    onCancel: () => {},
    onOk: () => {},
    isYinLiu: false,
  };

  constructor(props) {
    super(props);

    this.state = {
      promotionName: null, // 券名称

      // 券类型
      tempDate: null, // 有效期
      continuityType: null, // 连报条件
      superPosition: null, // 是否叠加

      list: [], // 学生可用优惠券列表
      pageNo: 1, // 页码
      totalRecords: 0, // 总条数
      loading: false, // loading
      pageSize: pageSize, // 每页条数

      promotionNatureSelectDisabled: /*this.isOneToN || this.isBanKeYiLiu*/ false,
      promotionNature: this.isOneToN ? '' : this.isBanKeYiLiu ? /*PROMOTION_NATURE_NORMAL*/ '' : '',

      selectedList: [], // 所有分页选中的优惠
      continueLimitData: null, // 连报条件数据
    };

    this.initSearchParam = {
      promotionName: null,
      promotionNature: null,
      continuityType: null,
      superPosition: null,
      tempDate: null,
    };
  }

  componentDidMount() {
    this.initModal();
  }

  /**
   * 是否班课引流合同
   * @readonly
   * @memberof Index
   */
  get isBanKeYiLiu() {
    const { groupType, isYinLiu } = this.props;
    return groupType === CLASS_COURSE && isYinLiu;
  }

  get isOneToN() {
    return [ONE_ON_ONE_COURSE, ONE_ON_MANY].includes(this.props.groupType);
  }

  /**
   * 初始化弹窗的初始数据等
   */
  initModal = async () => {
    const { couponList } = this.props;

    if (couponList.length > 0) {
      const limitParam = this.formatLimitForUseCouponParam();
      const [continueLimitData] = await Promise.all([loadLimitForUseCoupon(limitParam)]);

      this.setState(
        {
          selectedList: couponList,
          continueLimitData,
        },
        () => {
          this.loadList();
        },
      );

      return;
    }

    if (this.props.productList.length === 0) {
      return this.loadList();
    }

    const maxCouponParam = this.formatAdjectiveCouponParam();
    const limitParam = this.formatLimitForUseCouponParam();
    const [maxCouponList, continueLimitData] = await Promise.all([
      loadMaxCouponPromotion(maxCouponParam),
      loadLimitForUseCoupon(limitParam),
    ]);

    this.setState(
      {
        selectedList: maxCouponList,
        continueLimitData,
      },
      () => {
        this.loadList();
      },
    );
  };

  /**
   * 获取学生所有优惠券列表
   * 1、先前端判断当前产品数量是否满足优惠券规则
   * 2、再后端判断满足产品数量的优惠券是否符合课程及连报条件
   */
  loadList = (changeParam = {}) => {
    const { productList } = this.props;
    const { selectedList } = this.state;
    const params = { ...this.formatListParam(), ...changeParam };
    const apiParam = formatCouponListApiParam(params);

    this.setState({ loading: true });
    loadStudentPromotion(apiParam)
      .then((response) => {
        const result = this.formatListResult(response);
        const couponList = initCouponList(result.list, selectedList, productList);

        this.loadMultipleMatchList(couponList, (finalList, finalSelectedList) => {
          this.setState({
            loading: false,
            list: finalList || [],
            pageNo: params.pageNo,
            pageSize: params.pageSize,
            totalRecords: result.total,
            selectedList: finalSelectedList,
          });
        });
      })
      .catch(() => {
        const result = this.formatListResult();
        const couponList = initCouponList(result.list, selectedList, productList);

        this.setState({
          loading: false,
          list: couponList,
          pageNo: params.pageNo,
          pageSize: params.pageSize,
          totalRecords: result.total,
        });
      });
  };

  /**
   * 根据所选产品批量查询优惠券是否可用
   * tips: 不使用拼接suitablePromotion与unSuitablePromotion数组的方式，采用匹配可用优惠券ID方式标识优惠券是否可用
   * tips: 优惠券添加是否可用状态(suitable)、是否可选状态属性(disabled)，suitable状态优先级高于disabled
   *
   * @param couponList 学生可用的全部优惠券列表
   * @param callback 回调,回调参数为最终优惠列表
   */
  loadMultipleMatchList = (couponList = [], callback) => {
    const { selectedList } = this.state;
    const { studentId, branchId, productList, useCase, contactList } = this.props;
    const needCheckList = couponList.filter((item) => item.disabled === false);
    const params = {
      studentId,
      branchId,
      productList,
      useCase,
      contactList,
    };

    // 无需要检查是否可用的优惠券
    if (needCheckList.length <= 0) {
      callback(couponList);
      return;
    }

    promiseMultipleMatchCouponList(params)
      .then((response) => {
        const matchIds = formatMultipleMatchCouponIds({ response });
        const finalList = initMathCouponList(couponList, matchIds);
        const finalSelectedList = initMathSelectedList(selectedList, matchIds);

        callback(finalList, finalSelectedList);
      })
      .catch(() => {
        callback([], []);
      });
  };

  /**
   * 格式化请求参数
   */
  formatListParam = () => {
    const { branchId, studentId, contactList } = this.props;
    const { pageNo, tempDate, pageSize, promotionName, promotionNature, continuityType, superPosition } = this.state;
    let startDate = '';
    let endDate = '';

    if (tempDate && tempDate.length >= 2) {
      startDate = tempDate[0].format(dateTimeFormatter);
      endDate = tempDate[1].format(dateTimeFormatter);
    }

    return {
      branchId,
      studentId,
      contactList,
      name: promotionName,
      // 支持多个券类型(select单选)
      promotionNatures: promotionNature ? [promotionNature] : null,
      continuityType,
      beginTime: startDate,
      endTime: endDate,
      superPosition: superPosition !== null ? Number(superPosition) : null,
      pageNo,
      pageSize,
    };
  };

  /**
   * 格式化请求结果
   *
   * @param response
   */
  formatListResult = (response = {}) => {
    const data = response.data || {};
    const { records, total } = data;

    return {
      list: records || [],
      total: total || 0,
    };
  };

  /**
   * 格式化获取最优的优惠券组合请求参数
   */
  formatAdjectiveCouponParam = () => {
    const { productList, studentId, branchId, contactList } = this.props;
    const totalRealAmount = formatBDRealAmount(productList);
    const classInfos = formatClassInfoApiParam(productList, this.props.useCase);

    return {
      contactList,
      studentId: studentId,
      branchId,
      amount: totalRealAmount,
      showFutureCoupon: false,
      classInfos,
    };
  };

  /**
   * 格式化统计传入的班级中连报科目数和单报科目数请求参数
   *
   * @return {{classId: string, useCase: string, productType: *}[]}
   */
  formatLimitForUseCouponParam = () => {
    return formatClassInfoApiParam(this.props.productList, this.props.useCase);
  };

  /**
   * 分页改变
   *
   * @param currentPage
   * @param pageSize
   */
  onPageChange = (currentPage, pageSize) => {
    this.loadList({ pageNo: currentPage, pageSize });
  };

  onSearch = () => {
    this.loadList({ pageNo: 1 });
  };

  onReset = () => {
    this.setState(this.initSearchParam);
  };

  /**
   * 确认选择(可以不使用优惠券)
   */
  onOk = () => {
    const { selectedList } = this.state;
    const isSatisfyCouponAmount = judgeIsSatisfyCouponAmount(selectedList, this.props.productList);
    const isSatisfyCouponContinueLimit = judgeIsSatisfyCouponContinueLimit(selectedList, this.state.continueLimitData);

    if (!isSatisfyCouponAmount || !isSatisfyCouponContinueLimit) {
      return;
    }

    this.props.onOk(selectedList || []);
  };

  /**
   * 选择有效期
   *
   * @param moments []
   */
  onDateRangeChange = (moments) => {
    this.setState({ tempDate: moments });
  };

  /**
   * 选择/取消选择优惠券
   *
   * @param coupon 当前选择优惠券
   */
  onCouponSelect = (coupon) => {
    this.setState((prevState) => {
      const { resultList, resultSelectedList } = formatListAfterSelect({
        coupon,
        list: prevState.list || [],
        selectedList: prevState.selectedList,
        multiple: this.props.multiple,
        limit: this.props.limit,
        continueLimitData: this.state.continueLimitData,
      });

      return {
        list: resultList,
        selectedList: resultSelectedList,
      };
    });
  };

  render() {
    const { productList } = this.props;
    const {
      list,
      pageNo,
      loading,
      tempDate,
      pageSize,
      totalRecords,
      promotionName,
      continuityType,
      superPosition,
      promotionNatureSelectDisabled,
    } = this.state;

    return (
      <Modal
        title="选择优惠券"
        cancelText="取消"
        okText="确认选择"
        okBtnShow={true}
        okBtnDisabled={false}
        loading={loading}
        onCancel={this.props.onCancel}
        onOk={this.onOk}
        className={styles.couponModal}
      >
        <Wrapper
          bodyFitHeight
          header={
            <Row gutter={8}>
              <Col span={3}>
                <Input
                  placeholder="券名称"
                  value={promotionName}
                  onChange={(e) => this.setState({ promotionName: e.target.value })}
                />
              </Col>
              {
                /** 一对一合同类型 只能选择抵扣券类型优惠券 班课引流合同类型 只能选择满减券型优惠券 */
                <Col span={3}>
                  <Select
                    placeholder="券类型"
                    dict={promotionNatureDict}
                    value={this.state.promotionNature}
                    disabled={promotionNatureSelectDisabled}
                    onChange={(val) => this.setState({ promotionNature: val })}
                  />
                </Col>
              }
              <Col span={3}>
                <Select
                  placeholder="连报条件"
                  showSearch={true}
                  dict={continuityTypeDict}
                  value={continuityType}
                  onChange={(val) => this.setState({ continuityType: val })}
                />
              </Col>
              <Col span={3}>
                <Select
                  placeholder="是否叠加"
                  showSearch={true}
                  dict={superPositionDict}
                  value={superPosition}
                  onChange={(val) => this.setState({ superPosition: val })}
                />
              </Col>
              <Col span={7}>
                <RangePicker
                  size="large"
                  placeholder={['开始时间', '结束时间']}
                  format="YYYY-MM-DD HH:mm:ss"
                  showTime={{
                    hideDisabledOptions: true,
                    defaultValue: [moment(defaultTimeEarly, 'HH:mm:ss'), moment(defaultTimeLate, 'HH:mm:ss')],
                  }}
                  value={tempDate}
                  onChange={this.onDateRangeChange}
                  style={{ width: '100%' }}
                />
              </Col>
              <Col span={5}>
                <Button type="primary" onClick={this.onSearch} style={{ marginRight: 8 }}>
                  查询
                </Button>
                <Button onClick={this.onReset}>重置</Button>
              </Col>
            </Row>
          }
          footer={
            showPager ? (
              <Pagination
                current={pageNo}
                pageSize={pageSize}
                total={totalRecords}
                pageSizeOptions={['10', '20']}
                showSizeChanger
                showQuickJumper
                onChange={this.onPageChange}
                onShowSizeChange={this.onPageChange}
              />
            ) : (
              ''
            )
          }
        >
          {list && list.length > 0 ? (
            <CouponList list={list} productList={productList} onSelect={this.onCouponSelect} />
          ) : (
            <NoDataPlus text="暂无可用优惠券" />
          )}
        </Wrapper>
      </Modal>
    );
  }
}

export default Index;
