import {
  getPromotionCouponsByStudent,
  getSuitablePromotion,
  getMaxCouponPromotion,
  getLimitForUseCoupon,
} from 'SERVICE/contract';
import { formatBDRealAmount, formatClassInfoApiParam } from './util';

/**
 * 获取学生所有优惠券
 *
 * @param params
 * @return {Promise<{total: number, records: Array}|*>}
 */
export function loadStudentPromotion(params) {
  return getPromotionCouponsByStudent(params);
}

/**
 * 订单结算时批量获取可选优惠券和不可选优惠券
 * 最后将结果组装成promiseAllMatchCouponList所返回结果
 *
 * @param studentId 学生ID
 * @param contact
 * @param branchId 学生分公司ID
 * @param productList 选中产品列表
 * @param useCase
 * @return {*} {data: {suitablePromotion: [可用优惠券], unSuitablePromotion: [不可用优惠券]}}
 */
export function promiseMultipleMatchCouponList({ studentId, contactList, branchId, productList, useCase }) {
  const params = {
    studentId,
    contactList,
    branchId,
    amount: formatBDRealAmount(productList),
    classInfos: formatClassInfoApiParam(productList, useCase),
  };

  return getSuitablePromotion(params);
}

/**
 * 获取最优优惠券组合
 * tips: 后端返回的最优用券组合包含不可用券，需要前端过滤不可用优惠券
 *
 * @param params.contact {string} 手机号非空
 * @param params.bossStudentId {string} boss学生id
 * @param params.branchId {string} 分公司ID
 * @param params.amount {Number} 结算金额，当前选中项结算后的价格，不包含资料费
 * @param params.showFutureCoupon {boolean} 是否展示未来的优惠券 true: 展示有效时间在将来和当前的优惠券, false: 展示当前有效的优惠券
 * @param params.classInfos {Array} [{productType: 班级类型, classId: 班级ID, useCase: 优惠活动优惠场景}]
 * @return {*}
 */
export async function loadMaxCouponPromotion(params) {
  try {
    const response = await getMaxCouponPromotion(params);
    const { data } = response || {};

    return data || [];
  } catch (e) {
    return [];
  }
}

/**
 * 统计传入的班级中连报科目数和单报科目数,用于判断预存抵扣券叠加券使用数量上限(连报一科能用一张连报券/两张单报券，单报一科能用一张单报券)
 * tips: 只针对抵扣券
 * tips: 连报券按科目计算，不连报券按照班级数量计算
 * tips: 此处有个坑，后端接口计算的不连报数量是按照科目计算(bug)。。因此拿到结果前端再计算正确的不连报数量
 *
 * @param params {Array} [{productType, classId, useCase}]
 * @return {Promise<{}|*>}
 */
export async function loadLimitForUseCoupon(params) {
  const defaultData = {
    sumAutNum: 0, // 暑秋连报数(科目数)
    winSprNum: 0, // 寒春连报数(科目数)
    mixingNum: 0, // 连报券混合情况下(同时使用连报券和不连报券)的单报数
    singleNum: 0, // 只使用单报券情况下的单报数(等于班级数量)
  };

  // 连报基数（班级数量）
  // 实际上每个产品的班级数量并不一定相同，应由后端计算，这里默认为2
  const SUBJECT_CARDINAL_NUMBER = 2;

  try {
    const response = await getLimitForUseCoupon(params);
    const { data } = response || {};
    const { sumAutNum, winSprNum, num } = data;
    const singleNum = (sumAutNum + winSprNum) * SUBJECT_CARDINAL_NUMBER + num;

    return {
      sumAutNum,
      winSprNum,
      mixingNum: num,
      singleNum,
    };
  } catch (e) {
    return defaultData;
  }
}
