import NP from 'number-precision';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { COUPON_ID_KEY, COUPON_RULE_VO_KEY, COUPON_PROMOTION_RULE_NUM_KEY } from '../config';
import { PROMOTION_NATURE_DEPOSIT, PROMOTION_NATURE_NORMAL, PROMOTION_NATURE_REDEEM } from '../dict';
import { RULE_KEY, RULE_UNIT_PRODUCT_NUMS, RULE_UNIT_COURSE_HOURS, RULE_UNIT_NATURE_MINUTES } from '../config';
import {
  calcIsWinContinue,
  calcIsSumContinue,
  calcIsSingleContinue,
  calcIsDepositCoupon,
  calcCouponSuitableStatus,
} from './judges';
import { formatBDRealAmount } from './formatters';

const idKey = COUPON_ID_KEY;
const promotionVoKey = COUPON_RULE_VO_KEY;
const ruleNumKey = COUPON_PROMOTION_RULE_NUM_KEY;

/**
 * 初始化请求的优惠券列表(暂时没有不可用状态优惠券)
 * 1、选中在前，禁用在后
 * 2、对于选中的券，下次打开弹窗将已选券前置到第一个
 * 3、排序后添加一个是否选中状态(selected)、是否可选状态属性(disabled)
 *
 * @param list 后端返回的学生能用优惠券列表
 * @param selectedList 合同列表选中的优惠券
 * @param productList {array} 已选产品
 * @return {Array}
 */
export function initCouponList(list = [], selectedList = [], productList = []) {
  let result = [];
  const hasList = list && list.length > 0;
  const hasSelected = selectedList && selectedList.length > 0;

  if (!hasList) {
    return result;
  }

  if (hasSelected) {
    const selectedIds = selectedList.map((item) => item[idKey]);

    result = list.map((item) => {
      const disabled = formatCouponDisableStatus(item, productList);
      const selected = !disabled && selectedIds.indexOf(item[idKey]) > -1;

      return {
        ...item,
        disabled,
        selected,
      };
    });

    sortDisableCouponList(result);
  } else {
    result = list.map((item) => {
      const disabled = formatCouponDisableStatus(item, productList);

      return {
        ...item,
        disabled,
        selected: false,
      };
    });

    sortDisableCouponList(result);
  }

  return result;
}

/**
 * 前端判断完优惠券是否符合产品数量后，进行课程及连报规则的能用情况
 * tips: 添加是否可选状态(suitable)、是否可选状态属性(disabled)
 *
 * @param list {array} 前端初步处理后的学生能用优惠券列表（前端规则判断及排序）
 * @param mathIds {array<string>} 后端匹配的可用的优惠券ID列表
 */
export function initMathCouponList(list = [], mathIds = []) {
  const result = list.map((item) => {
    const suitable = mathIds.indexOf(item[idKey]) > -1;
    const selected = item.selected && suitable;

    return {
      ...item,
      suitable,
      disabled: !suitable,
      selected,
    };
  });

  sortDisableCouponList(result);

  return result;
}

/**
 * 前端判断完优惠券是否符合产品数量后，过滤不可用的已选优惠券
 *
 * @param selectedList {Array} 已选优惠券
 * @param mathIds {Array<string>} 后端匹配的可用的优惠券ID列表
 * @return {Array}
 */
export function initMathSelectedList(selectedList = [], mathIds = []) {
  let result = [];

  selectedList.forEach((item) => {
    const suitable = mathIds.indexOf(item[idKey]) > -1;

    if (suitable) {
      result.push(item);
    }
  });

  return result;
}

/**
 * 选择优惠券时格式化选中的列表(只改变selected状态)
 * tips: 只限制是否可叠加
 * tips: 优惠券使用状态优先级: disabled(学生是否可用) > suitable(当前产品下是否可选)
 *
 * @param coupon 当前选择的优惠券
 * @param list 所有优惠券
 * @param selectedList 已经选中的优惠券
 * @param multiple 是否多选
 * @param limit 可选数量限制
 * @param continueLimitData {Object} 连报限制条件数据(只针对抵扣券)
 * @return {*}
 */
export function formatListAfterSelect({ coupon, list, selectedList, multiple, limit, continueLimitData }) {
  if (!coupon || !list || list.length <= 0) {
    return [];
  }

  // 当前点击的优惠券的状态
  // 当前是选中状态，需要取消选中（取消选中不需要判断是否可多选）
  // 当前是未选中状态，需要选中（选中则需要判断是否可多选）
  const { selected } = coupon;
  const oldSelectedList = selectedList || [];
  const oldSelectedLen = oldSelectedList.length;
  let resultSelectedList = [];

  // 取消选中
  if (selected) {
    list.forEach((item) => {
      if (item[idKey] === coupon[idKey]) {
        item.selected = false;
        return false;
      }
    });

    oldSelectedList.forEach((item) => {
      if (item[idKey] !== coupon[idKey]) {
        resultSelectedList.push(item);
      }
    });

    return {
      resultList: list,
      resultSelectedList: resultSelectedList,
    };
  }

  // 单选-选中 (不需要判断是否已有选中优惠)
  const isSingle = !multiple || limit === 1;
  if (isSingle) {
    list.forEach((item) => {
      item.selected = item[idKey] === coupon[idKey];
    });

    return {
      resultList: list,
      resultSelectedList: [coupon],
    };
  }

  // 多选-选中 (需要判断最大可选数量)
  const hasLimit = limit >= 0;
  if (hasLimit && oldSelectedLen >= limit) {
    showMessage(`最多可选${limit}个优惠券！`, 'error');
    return {
      resultList: list,
      resultSelectedList: oldSelectedList,
    };
  }

  // 不可叠加, superPosition: 是否可叠加 {0: 不可叠加, 1: 可叠加}
  // 分类已选列表
  const classifySelectedList = classifyCouponByPromotionNature(selectedList);
  const natureSelectedList = classifySelectedList[coupon.promotionNature] || [];
  const unableSuperPosition =
    natureSelectedList.length > 0 && (natureSelectedList[0].superPosition === 0 || coupon.superPosition === 0);
  if (unableSuperPosition) {
    showMessage('不可叠加当前优惠券！', 'error');
    return {
      resultList: list,
      resultSelectedList: oldSelectedList,
    };
  }

  // 可叠加，再判断连报情况
  const continueResult = calcCouponSuitableStatus(coupon, oldSelectedList, continueLimitData);
  if (!continueResult.suitable) {
    showMessage(continueResult.errorMsg, 'error');
    return {
      resultList: list,
      resultSelectedList: oldSelectedList,
    };
  }

  // 通过所有条件，最终选中
  list.forEach((item) => {
    if (item[idKey] === coupon[idKey]) {
      item.selected = true;
      return false;
    }
  });

  resultSelectedList = oldSelectedList.concat([coupon]);

  return {
    resultList: list,
    resultSelectedList: resultSelectedList,
  };
}

/**
 * 根据优惠券业务型态分类优惠券
 *
 * @param list {Array} 优惠券列表
 * @return {{}}
 */
export function classifyCouponByPromotionNature(list = []) {
  if (!list || list.length === 0) {
    return {};
  }

  let result = {};

  list.forEach((coupon) => {
    const { promotionNature } = coupon;

    if (result[promotionNature]) {
      result[promotionNature].push(coupon);
    } else {
      result[promotionNature] = [coupon];
    }
  });

  return result;
}

/**
 * 格式化优惠券的禁用状态
 *
 * @param coupon 优惠券
 * @param productList 已选产品列表
 * @return {boolean} true: 禁用，false：可用
 */
export function formatCouponDisableStatus(coupon, productList = []) {
  const hasProduct = productList && productList.length > 0;

  if (!coupon || !hasProduct) {
    return true;
  }
  // 代金券没有优惠规则
  if (coupon.promotionNature === PROMOTION_NATURE_REDEEM) {
    return false;
  }

  const ruleVo = coupon.promotionRuleVos || [];
  if (ruleVo.length <= 0) {
    return true;
  }

  const hitRule = findHitRule(ruleVo, productList);

  return !hitRule;
}

/**
 * 找到命中的规则
 * 根据产品列表找到命中的优惠券使用规则
 * tips：getSuitablePromotion 接口返回的是否可用判定条件优先级最高
 *
 * @param ruleVos {Array} 优惠券使用规则集合
 * @param productList {Array} 产品列表
 */
export function findHitRule(ruleVos = [], productList = []) {
  return ruleVos.find((rule) => {
    switch (rule[RULE_KEY]) {
      // 当规则单位为`产品数量`时
      case RULE_UNIT_PRODUCT_NUMS: {
        return productList.length >= rule[ruleNumKey];
      }
      // 当规则单位为`课时数`时
      case RULE_UNIT_COURSE_HOURS: {
        const quantityValid = productList.some((product) => product.quantity >= rule[ruleNumKey]);
        const totalQuantityValid =
          productList.map((r) => r.quantity).reduce((pre, cur) => pre + cur, 0) >= rule[ruleNumKey];

        return quantityValid || totalQuantityValid;
      }
      // 当规则单位为`课时分钟`时
      // 目前仅抵扣券使用该规则
      // 除了课时分钟符合条件，合同金额还需大于购券金额（该条件由优惠中心判断）
      case RULE_UNIT_NATURE_MINUTES: {
        const quantityValid = productList.some((product) => (product.quantityMinutes || 0) >= rule[ruleNumKey]);
        const totalQuantityValid =
          productList.map((r) => r.quantityMinutes || 0).reduce((pre, cur) => pre + cur, 0) >= rule[ruleNumKey];

        return quantityValid || totalQuantityValid;
      }
      default: {
        return true;
      }
    }
  });
}

/**
 * 优惠券排序
 * - 截止时间越近的越往前
 * - 将优惠券按照业务型态分类
 * - 将禁用的优惠券排在后面
 * - 将选中的排在前面
 *
 * @param list 优惠券列表
 */
export function sortDisableCouponList(list = []) {
  list.sort((a, b) => {
    return a.promotionNature - b.promotionNature;
  });

  list.sort((a, b) => {
    return new Date(a.endDate).getTime() - new Date(b.endDate).getTime();
  });

  list.sort((a, b) => {
    return a.disabled - b.disabled;
  });

  list.sort((a, b) => {
    return b.selected - a.selected;
  });
}

/**
 * 判断所选抵扣券组合是否符合金额使用条件
 * tips：该规则只针对抵扣券
 *
 * @param couponList {Array} 选中的优惠券
 * @param productList {Array} 产品列表
 */
export function judgeIsSatisfyCouponAmount(couponList, productList) {
  const classifyResult = classifyCouponByPromotionNature(couponList);
  const normalPromotion = classifyResult[PROMOTION_NATURE_NORMAL] || [];
  const depositPromotion = classifyResult[PROMOTION_NATURE_DEPOSIT] || [];

  // 未选择抵扣券则跳过校验
  if (depositPromotion.length === 0) {
    return true;
  }

  // 总合同产品金额（除去产品优惠及活动的实收总金额）
  let resultAmount = formatBDRealAmount(productList);

  // 先减去满减券的优惠金额
  normalPromotion.forEach((coupon) => {
    const hitRule = findHitRule(coupon[promotionVoKey], productList);
    if (hitRule) {
      resultAmount = NP.minus(resultAmount, hitRule.promotion);
    }
  });

  // 再计算抵扣券的买券金额（已收金额）
  const depositPayAmount = depositPromotion.reduce((total, coupon) => {
    return NP.plus(total, coupon.alreadyPayAmount || 0);
  }, 0);

  if (resultAmount < depositPayAmount) {
    showMessage('产品金额小于抵扣券已收金额，该抵扣券组合不可使用', 'error');
    return false;
  }

  return true;
}

/**
 * 判断所选抵扣券组合是否符合连报使用条件
 * tips: 只针对抵扣券的规则，需要过滤抵扣券
 *
 * @param allSelectedList {Array} 所有选中的优惠券
 * @param continueLimitData {Object} 抵扣券连报限制条件
 * @return {boolean}
 */
export function judgeIsSatisfyCouponContinueLimit(allSelectedList, continueLimitData) {
  if (!allSelectedList || allSelectedList.length === 0) {
    return true;
  }

  // 先过滤抵扣券
  const selectedList = allSelectedList.filter((item) => calcIsDepositCoupon(item));
  if (selectedList.length === 0) {
    return true;
  }

  const { sumAutNum, winSprNum, mixingNum, singleNum } = continueLimitData;

  // 已选连报优惠分类
  const singleSelected = selectedList.filter((item) => calcIsSingleContinue(item.continuityType));
  const sumContinueSelected = selectedList.filter((item) => calcIsSumContinue(item.continuityType));
  const winContinueSelected = selectedList.filter((item) => calcIsWinContinue(item.continuityType));

  // 判断已选抵扣券是否为组合使用券（连报、不连报券混合）
  const isMixingContinue =
    singleSelected.length > 0 && (sumContinueSelected.length > 0 || winContinueSelected.length > 0);

  // 单连报券的判断
  if (isMixingContinue) {
    if (singleSelected.length > mixingNum) {
      const msg =
        mixingNum > 0 ? `当前组合条件下单连报抵扣券数量不能超过${mixingNum}张` : '当前组合条件下单连报抵扣券不可用';
      showMessage(msg, 'error');
      return false;
    }
  } else {
    if (singleSelected.length > singleNum) {
      const msg =
        singleNum > 0 ? `当前组合条件下单连报抵扣券数量不能超过${singleNum}张` : '当前组合条件下单连报抵扣券不可用';
      showMessage(msg, 'error');
      return false;
    }
  }

  // 暑秋连报条件判断
  if (sumContinueSelected.length > sumAutNum) {
    const msg =
      sumAutNum > 0 ? `当前组合条件下暑秋连报抵扣券数量不能超过${sumAutNum}张` : '当前组合条件下暑秋连报抵扣券不可用';
    showMessage(msg, 'error');
    return false;
  }

  // 寒春连报条件判断
  if (winContinueSelected.length > winSprNum) {
    const msg =
      winSprNum > 0 ? `当前组合条件下寒春连报抵扣券数量不能超过${winSprNum}张` : '当前组合条件下寒春连报抵扣券不可用';
    showMessage(msg, 'error');
    return false;
  }

  return true;
}
