import { PROMOTION_NATURE_DEPOSIT } from '../dict';
import { ONE_ON_ONE_COURSE, ONE_ON_MANY } from '../../config';

/**
 * 判断是否为抵扣券
 *
 * @param coupon {Object} 优惠券
 * @return {boolean}
 */
export function calcIsDepositCoupon(coupon) {
  return coupon.promotionNature === PROMOTION_NATURE_DEPOSIT;
}

/**
 * 判断单报
 *
 * @param continuityType {string} 连报类型 { NONE: 不连报, SUM_AUT: 暑期连报, WIN_SPR: 寒春连报 }
 * @return {boolean}
 */
export function calcIsSingleContinue(continuityType) {
  return continuityType === 'NONE';
}

/**
 * 盘端暑秋连报
 *
 * @param continuityType {string} 连报类型 { NONE: 不连报, SUM_AUT: 暑期连报, WIN_SPR: 寒春连报 }
 * @return {boolean}
 */
export function calcIsSumContinue(continuityType) {
  return continuityType === 'SUM_AUT';
}

/**
 * 判断寒春连报
 *
 * @param continuityType {string} 连报类型 { NONE: 不连报, SUM_AUT: 暑期连报, WIN_SPR: 寒春连报 }
 * @return {boolean}
 */
export function calcIsWinContinue(continuityType) {
  return continuityType === 'WIN_SPR';
}

/**
 * 判断当前优惠券是否可选
 * tips: 只针对抵扣券的规则，需要过滤抵扣券
 *
 * @param coupon {*} 当前优惠券
 * @param allSelectedList {Array} 所有已选优惠券
 * @param continueLimitData {Object} 连报限制数量
 */
export function calcCouponSuitableStatus(coupon, allSelectedList, continueLimitData) {
  if (!calcIsDepositCoupon(coupon)) {
    return {
      suitable: true,
      errorMsg: '',
    };
  }

  // 先过滤抵扣券
  const selectedList = allSelectedList.filter((item) => calcIsDepositCoupon(item));
  if (selectedList.length === 0) {
    return {
      suitable: true,
      errorMsg: '',
    };
  }

  const { sumAutNum, winSprNum, mixingNum, singleNum } = continueLimitData;

  // 判断当前券类型
  // 单连报券
  const isSingleContinue = calcIsSingleContinue(coupon.continuityType);
  // 暑秋连报券
  const isSumContinue = calcIsSumContinue(coupon.continuityType);
  // 寒春连报券
  const isWinContinue = calcIsWinContinue(coupon.continuityType);

  // 已选连报优惠分类
  const singleSelected = selectedList.filter((item) => calcIsSingleContinue(item.continuityType));
  const sumContinueSelected = selectedList.filter((item) => calcIsSumContinue(item.continuityType));
  const winContinueSelected = selectedList.filter((item) => calcIsWinContinue(item.continuityType));

  // 判断当前选择的抵扣券是否即将造成混合使用（连报、不连报券混合）
  const willMixingContinue = isSingleContinue
    ? sumContinueSelected.length > 0 || winContinueSelected.length > 0
    : singleSelected.length > 0;

  if (willMixingContinue) {
    // 即将造成混合使用情况下
    // 需要区分先选了混合还是先选了单报抵扣券
    if (isSingleContinue) {
      if (singleSelected.length >= mixingNum) {
        const msg =
          mixingNum > 0 ? `当前组合条件下单连报抵扣券数量不能超过${mixingNum}张` : '当前组合条件下单连报抵扣券不可用';
        return {
          suitable: false,
          errorMsg: msg,
        };
      }
    } else if (isSumContinue) {
      if (sumContinueSelected.length >= sumAutNum) {
        const msg =
          sumAutNum > 0
            ? `当前组合条件下暑秋连报抵扣券数量不能超过${sumAutNum}张`
            : '当前组合条件下暑秋连报抵扣券不可用';
        return {
          suitable: false,
          errorMsg: msg,
        };
      } else if (singleSelected.length > mixingNum) {
        return {
          suitable: false,
          errorMsg: `请先取消${singleSelected.length - mixingNum}张单报抵扣券`,
        };
      }
    } else if (isWinContinue) {
      if (winContinueSelected.length >= winSprNum) {
        const msg =
          winSprNum > 0
            ? `当前组合条件下寒春连报抵扣券数量不能超过${winSprNum}张`
            : '当前组合条件下寒春连报抵扣券不可用';
        return {
          suitable: false,
          errorMsg: msg,
        };
      } else if (singleSelected.length > mixingNum) {
        return {
          suitable: false,
          errorMsg: `请先取消${singleSelected.length - mixingNum}张单报抵扣券`,
        };
      }
    }
  } else {
    if (isSingleContinue) {
      if (singleSelected.length >= singleNum) {
        const msg =
          singleNum > 0 ? `当前组合条件下单连报抵扣券数量不能超过${singleNum}张` : '当前组合条件下单连报抵扣券不可用';
        return {
          suitable: false,
          errorMsg: msg,
        };
      }
    } else if (isSumContinue) {
      if (sumContinueSelected.length >= sumAutNum) {
        const msg =
          sumAutNum > 0
            ? `当前组合条件下暑秋连报抵扣券数量不能超过${sumAutNum}张`
            : '当前组合条件下暑秋连报抵扣券不可用';
        return {
          suitable: false,
          errorMsg: msg,
        };
      }
    } else if (isWinContinue) {
      if (winContinueSelected.length >= winSprNum) {
        const msg =
          winSprNum > 0
            ? `当前组合条件下寒春连报抵扣券数量不能超过${winSprNum}张`
            : '当前组合条件下寒春连报抵扣券不可用';
        return {
          suitable: false,
          errorMsg: msg,
        };
      }
    }
  }

  return {
    suitable: true,
    errorMsg: '',
  };
}

/**
 * 判断是否个性化课程
 *
 * @param productType {string} 产品类型
 * @return {boolean}
 */
export function judgeIsPersonalizeCourse(productType) {
  return [ONE_ON_ONE_COURSE, ONE_ON_MANY].includes(productType);
}
