import NP from 'number-precision';
import {
  ECS_CLASS,
  contractTypeCfg,
  SMALL_CLASS,
  ONE_ON_ONE_COURSE,
  ONE_ON_MANY,
  TWO_TEACHER,
  OTHERS,
  ENDED,
  CLOSE_PRODUCT,
  UNVALID,
  FROZEN,
  contractGroupMap,
  contractGroupPriority,
  groupTypesMap,
  contractOtherGroupMap,
  productTypeNameCfg,
  CLASS_COURSE,
  useCouponGroupTypes,
  useCouponProductTypes,
  SPECIAL_PRODUCT_TYPE,
  CLASS_TYPE_BROADCAST_COOPERATE,
  PRODUCT_GROUP_BROADCAST_COOPERATE,
  OTHER_CATEGORY_TYPE_MARKETING,
} from '../config';
import { ENG_CHARSET, createRandomStr } from 'UTIL/uid';
import { clone } from 'lodash';
import { isMoreThanZero } from '../utils';
import { CATEGORY_A } from 'CONST/dicts/product';

/**
 * 创建 uid
 * @param {string} suffix 补充字符串
 */
export function createUid(suffix = '') {
  const prefix = createRandomStr(16, ENG_CHARSET);

  return prefix + String(suffix);
}

function checkSamePrd(prd) {
  return prd.productId + (prd.classId || '');
}

const relation = {
  count: 0,
  set() {
    let ret = `${new Date().getTime()}${this.count}`;
    this.addCount();
    return ret;
  },
  addCount() {
    this.count++;
  },
};
/**
 * 获取产品关联标识
 * @return {String} _relation
 * */
function setPrdRelation() {
  return relation.set();
}

/**
 * 获取产品关联标识
 * @param prd {Object} 产品
 * @return {String} _relation
 * */
function getPrdRelation(prd) {
  return prd._relation || null;
}

/**
 * 添加产品及其子产品,建立产品与子产品的关联关系
 * tips: 将子产品从父产品分离，并添加父产品的`classId`、`productType`保存到后端（仅其他类型的产品需要）
 * tips: 父子产品建立relationId
 * tips: 添加`relateClassId`字段 后端使用
 * tips: 添加`relateProductType`字段 后端使用
 * tips: 修改此处字段时注意修改如下方法
 *  `/ProductTableSS/util.js/findHitParentProduct()`方法
 *  `/store/index/submit`参数
 *  下方的`contractInfoAdepter()`方法
 *
 * @param prd {Object} 新增产品
 * @return {Array} 产品列
 * */
function addProductWithSub(prd) {
  const _relation = setPrdRelation();

  prd = {
    ...prd,
    _uid: createUid(),
    _relation,
  };

  if (Array.isArray(prd.subProducts) && prd.subProducts.length > 0) {
    const isEcs = prd.productType === ECS_CLASS;
    const subPrdList = prd.subProducts.map((item) => {
      return {
        ...item,
        _uid: createUid(),
        // 保存父级关联关系
        _relation,
        _btnDeleteDisabled: isEcs,
        // 子产品保存父班级id
        _relateClassId: prd.classId,
        // 子产品保存父产品类型（用于后端区分系统）
        _relateProductType: prd.productType,
        // 保存关联关系
        otherOfCategoryId: prd.otherOfCategoryId,
        // 子产品数量从count取
        quantity: item.count,
        // 子产品性质直接用父级赋值(方便分为同一组-兼容父子性质不一样场景)
        productCharacterId: prd.productCharacterId,
        // 子产品性质直接用父级赋值
        productCharacterName: prd.productCharacterName,
        // 关联产品的naturePrice，新增：取price，修改：取naturePrice
        naturePrice: item.naturePrice,
      };
    });

    return [prd].concat(subPrdList);
  }

  return [prd];
}

/**
 * 批量添加产品及其子产品
 * @param list {Array} 新增产品列
 * @return {Array} 产品列
 * */
function batchAddProductWithSub(list) {
  return list.map((prd) => addProductWithSub(prd)).reduce((ret, cur) => ret.concat(cur), []);
}

// 1对1、1对多 添加班级id
function addClassIdByUid(prd) {
  if ([ONE_ON_ONE_COURSE, ONE_ON_MANY].includes(prd.productType) && !prd.classId) {
    prd.classId = `${
      {
        ONE_ON_ONE_COURSE: 'oto',
        ONE_ON_MANY: 'otm',
      }[prd.productType]
    }_${createUid()}`;
  }
  return prd;
}

// 合同详情数据设配器
function contractInfoAdepter(data) {
  let result = {
    contractId: data.contractId,
    contractType: data.contractType,
    contractTypeName: contractTypeCfg[data.contractType],
    isEcs: data.ecsContract,
    customerId: data.customerId,
    customerName: data.customerName,
    customerContact: data.customerContact,
    studentId: data.studentId,
    studentName: data.studentName,
    signDate: data.signDate,
    blCampusId: data.blCampusId,
    schoolName: data.schoolName,
    schoolId: data.isVirtualSchool ? null : data.schoolId,
    virtualSchool: data.isVirtualSchool ? { schoolName: data.schoolName } : null,
    gradeId: data.contractGradeId,
    transferIntroduceId: data.transferIntroduceId,
    selectedList: [],
    groupList: [],
    totalAmount: data.totalAmount,
    promotionAmount: data.promotionAmount,
    realAmount: NP.minus(data.totalAmount, data.paidAmount, data.promotionAmount),
    paidAmount: data.paidAmount,
    customerIdCardNo: data.customerIdCardNo,
    contractPayCategory: null,
  };

  let couponList = [];
  let couponAmount = 0;
  const hadCoupon = Array.isArray(data.promotionCouponArr) && data.promotionCouponArr.length > 0;
  if (hadCoupon) {
    couponList = data.promotionCouponArr.map((item) => ({
      ...item,
      _couponName: item.promotionCouponName,
      _couponId: item.promotionCouponId,
      _ruleId: item.promotionCouponRuleId,
      _couponDiscount: item.promotionCouponAmount,
      _couponAlreadyPayAmount: item.alreadyPayAmount,
      _couponOrderNoInChannel: item.payOrderNum,
    }));
    couponAmount = data.promotionCouponArr.reduce((total, current) => NP.plus(total, current.promotionCouponAmount), 0);
  }

  if (Array.isArray(data.contractProductArr) && data.contractProductArr.length > 0) {
    result.selectedList = data.contractProductArr.map((item) => {
      const hadUsedActive = (item.promotionActivityArr || []).length > 0;
      const hadUsedCoupon = hadCoupon && useCouponProductTypes.includes(item.productType);

      // 优惠金额大于0的默认有优惠(对于暑秋连报的优惠活动，可能存在某一季度的优惠是0的情况，所以加上是否有优惠活动的判断)
      const hadPromotionAmount = (item.promotionAmount || 0) > 0 || hadUsedActive;
      const originPayment = NP.times(item.courseHours, item.price);
      // 详情接口返回的数据缺少产品关联关系，无法为其添加`_relation`字段
      item = {
        ...item,
        _uid: createUid(),
        quantity: item.courseHours,
        status: item.statusName,
        isProductNarrow: item.isProductNarrow, // 是否缩单
        promotions: (item.optionalPromotions || []).map((item) => ({
          id: item.promotionId,
          name: item.promotionName,
          properties: takeOutPromotionProperties(item),
        })),
        payment: item.totalAmount,
        realAmount: item.realPayAmount,
        // 产品已收金额
        _paidAmount: item.paidAmount,
        // 产品所有优惠金额
        _promotionAmount: item.promotionAmount,
        _contractPrdStatus: item.status,
        _statusCache: item.status,
        _quantityDisabled:
          item.productType === OTHERS && item.otherCategoryId === OTHER_CATEGORY_TYPE_MARKETING
            ? false
            : [SMALL_CLASS, TWO_TEACHER, OTHERS, ECS_CLASS].includes(item.productType), // 产品类型为"其他",并且费用类型为"市场",则产品数量可编辑
        _hadInitQuantity: typeof item.quantity === 'number' ? item.quantity : true,
        _prdPromotion: (item.promotionArr || []).map((item) => ({
          id: item.promotionId,
          name: item.promotionName,
          properties: takeOutPromotionProperties(item),
        })),
        _prdPromotionVal: (item.promotionArr || []).map((item) => item.promotionId),
        // 自动计算优惠才需要传优惠活动
        _activePromotion: (item.promotionActivityArr || []).map((item) => {
          return {
            name: item.promotionActivityName,
            id: item.promotionActivityId,
            rule: item.promotionActivityRuleId,
          };
        }),
        /**
         * 这里这样处理是因为当存在赠送课时，产品总额会大于单价*数量，所以必需取接口返回的总额
         * 备注：之前产品数量输入未限制小数点后2位，而后端存在该限制，会导致总价也大于单价*数量，产生修改产品数量时，未能更新价格的bug
         * */
        _paymentAddition: item.totalAmount > originPayment ? NP.minus(item.totalAmount, originPayment) : 0,
        // 非无效的合同产品，优惠金额大于0并且优惠金额大于已扣优惠，可以清空优惠
        _btnClearPromotionVisible:
          hadPromotionAmount && item.status !== UNVALID && item.promotionAmount >= item.promotionConsumeAmount,
        // 修改合同产品的产品优惠时，是否需要提示
        _warningBeforeEditPrdPromotion: hadPromotionAmount && (hadUsedActive || hadUsedCoupon),
        _relateClassId: item.relateClassId,
        _relateProductType: item.relateProductType,
        ...getContractPrdActions(item, result.isEcs),
      };

      item.payment = calcItemPayment(item);
      item.paidAmount = calcItemPaidAmount(item);
      item.promotionAmount = calcItemPromotionAmount(item);
      item.realAmount = calcItemRealAmount(item);

      return item;
    });
    // 小班、一对多合同
    if (SPECIAL_PRODUCT_TYPE[data.contractProductTag]) {
      const specialSelectedList = result.selectedList.filter(
        ({ productType }) => productType === data.contractProductTag,
      );
      result.isQualityContract =
        !!specialSelectedList.length &&
        !!specialSelectedList.filter(({ courseSeriesName }) => courseSeriesName === '素质').length; // 标识是否素质合同
      // 广电合同 小班产品：产品系列为素质+班级类型为合作项目 1对多产品：产品系列为素质+产品组为合作项目
      if (data.contractProductTag === SMALL_CLASS) {
        // 小班--广电合作合同
        result.isBroadcastCooperate =
          !!specialSelectedList.length &&
          !!specialSelectedList.filter(
            ({ classTypeId, courseSeriesName }) =>
              classTypeId === CLASS_TYPE_BROADCAST_COOPERATE && courseSeriesName === '素质',
          ).length; //  标识是否合作项目素质
      } else if (data.contractProductTag === ONE_ON_MANY) {
        // 1对多--广电合作合同
        result.isBroadcastCooperate =
          !!specialSelectedList.length &&
          !!specialSelectedList.filter(
            ({ productGroupId, courseSeriesName }) =>
              productGroupId === PRODUCT_GROUP_BROADCAST_COOPERATE && courseSeriesName === '素质',
          ).length; //  标识是否合作项目素质
      } else {
        result.isBroadcastCooperate = false;
      }
    } else {
      result.isQualityContract = false;
      result.isBroadcastCooperate = false;
    }
    const _noOtherList = result.selectedList.filter((item) => item.productType !== OTHERS);
    result.contractPayCategory =
      _noOtherList.length > 0 ? _noOtherList[0].contractPayCategory || CATEGORY_A : CATEGORY_A;
  } else {
    result.isQualityContract = false;
    result.isBroadcastCooperate = false;
  }

  const _isYinLiu = data.contractCharacter === 'YINLIU';
  const productCharacterName = _isYinLiu ? '引流' : '常规';

  result.groupList = getEditProductGroup(clone(result.selectedList), [
    {
      id: createUid(),
      _uid: createUid(),
      groupType: contractGroupMap[data.contractProductTag],
      data: [],
      couponList,
      couponAmount,
      totalAmount: data.totalAmount,
      promotionAmount: data.promotionAmount,
      realAmount: data.realAmount,
      remark: data.remark,
      title: `【${data.contractCharacter ? `${productCharacterName} ` : ''}】${
        productTypeNameCfg[data.contractProductTag]
      }`,
      productCharacterName,
      _isYinLiu,
      quarterType: data.quarterType,
    },
  ]);

  return result;
}

// 获取合同产品 更改/操作visible
function getContractPrdActions(contractProduct, isEcs) {
  if ([ENDED, CLOSE_PRODUCT, UNVALID, FROZEN].includes(contractProduct.status)) {
    // 不可无效、不可删除、不可改数量、不可改、不可清空优惠
    return {
      _editQuantityDisabled: true,
      _editPrdPromotionDisabled: true,
      _btnDeleteDisabled: true,
      _btnInvalidDisabled: true,
      _btnClearPromotionVisible: false,
    };
  } else if (contractProduct.consumeAmount > 0) {
    // 不可删除、不可无效
    return {
      _btnDeleteDisabled: true,
      _btnInvalidDisabled: true,
    };
  } else if (contractProduct.isWashed) {
    // 冲销的 不可删除
    // 冲销且已付款的 不可无效
    return {
      _btnDeleteDisabled: true,
      _btnInvalidDisabled: contractProduct.paidAmount > 0,
    };
  } else {
    // 不可无效
    // 修改合同时，(合同产品)精英班的其他类型产品不可删除(按照旧逻辑)
    return {
      _btnDeleteDisabled: isEcs && contractProduct.productType === OTHERS,
      _btnInvalidDisabled: true,
    };
  }
}

/**
 * 取出优惠券的优惠属性字段
 * tips: 不同接口返回字段不同 promotionProperties|propertiesId
 * tips: 前端组装的字典取 properties
 *
 * @param promotion
 * @return {null|*|null}
 */
function takeOutPromotionProperties(promotion) {
  if (!promotion) {
    return null;
  }

  return promotion.promotionProperties || promotion.propertiesId || promotion.properties || null;
}

// 兼容undefine 和 null 等空情况。
function productCharacterEq(sourceId, tartgetId) {
  return sourceId == tartgetId;
}

// 筛选分组 (1. 处理父子关联关系 -> 2. 没有关联关系则通过 groupType 进行分组 )
function groupFilterCallback(item, groupType) {
  const relateGroupType = contractGroupMap[item._relateProductType];
  // 判断是否有父子关联关系
  if (relateGroupType === groupType) {
    return true;
  }

  return item._groupType === groupType && !relateGroupType;
}

// 如果是其它类型，otherOfCategoryId 判断是否有 所属产品类型
function handleOtherGroup(productGroupList, characterItem) {
  const { productCharacterId } = characterItem;
  let otherGroup = productGroupList.find(
    (item) => item.groupType === OTHERS && productCharacterEq(item.productCharacterId, productCharacterId),
  );
  if (!otherGroup) return;

  let otherGroupData = otherGroup.data;
  otherGroupData.forEach((otherItem) => {
    if (!otherItem.otherOfCategoryId) return;

    const productGroupItem = productGroupList.find(
      (groupItem) =>
        groupItem.groupType === contractOtherGroupMap[otherItem.otherOfCategoryId] &&
        productCharacterEq(groupItem.productCharacterId, productCharacterId),
    );
    if (productGroupItem) {
      productGroupItem.data.push({ ...otherItem });
      otherItem.isDelete = true;
    } else {
      otherItem.isDelete = false;
    }
  });

  otherGroup.data = otherGroupData.filter((item) => !item.isDelete);
}

// 计算产品性质有几种    兼容可变
// 产品性质id   productCharacterId
function getProductCharacterDict(selectedList) {
  let dict = [];
  selectedList.forEach((selectedItem) => {
    let { productCharacterId, productCharacterName } = selectedItem;

    const characterItem = dict.find((item) => productCharacterEq(item.productCharacterId, productCharacterId));
    !characterItem &&
      dict.push({
        productCharacterId,
        productCharacterName,
      });
  });

  return dict;
}

// 计算合同明细
function calcGroupTotal(list = []) {
  let total = {
    totalAmount: 0,
    promotionAmount: 0,
    realAmount: 0,
    paidAmount: 0,
  };

  list.reduce((total, current) => {
    if (current._contractPrdStatus === UNVALID) {
      return total;
    }

    total.totalAmount = NP.plus(total.totalAmount, current.payment || 0);
    total.promotionAmount = NP.plus(total.promotionAmount, current.promotionAmount || 0);
    total.realAmount = NP.plus(total.realAmount, current.realAmount || 0);
    total.paidAmount = NP.plus(total.paidAmount, current.paidAmount || 0);

    return total;
  }, total);

  return total;
}

// 计算所有合同组明细
function calcGroupListTotal(groupList) {
  let list = [...groupList];
  list.forEach((productGroupItem) => {
    const total = calcGroupTotal(productGroupItem.data);
    Object.assign(productGroupItem, total);
  });
  return list;
}

/**
 * 对产品进行分类 1. 通过groupType 拿到当前分类 2. 当前分类再通过性质分类 3. 如果存在则更新 4. 分完类之后再计算总数
 */
function getProductGroup(productList = [], groupList = []) {
  let productGroupList = [];
  let characterDict = getProductCharacterDict(productList);

  contractGroupPriority.forEach((groupType) => {
    // 所有当前分类
    const allProductGroupData = productList.filter((item) => groupFilterCallback(item, groupType));
    if (!allProductGroupData.length) return;

    characterDict.forEach((characterItem) => {
      const { productCharacterId, productCharacterName } = characterItem;
      // 当前分类再通过性质分类
      const productGroupData = allProductGroupData.filter((item) =>
        productCharacterEq(item.productCharacterId, productCharacterId),
      );
      // 分组是否已经存在
      let productGroupItem = groupList.find(
        (item) => item.groupType === groupType && productCharacterEq(item.productCharacterId, productCharacterId),
      );

      if (productGroupItem) {
        productGroupItem.data = productGroupData;
        if (isNormalOneToN({ groupType })) {
          productGroupItem.isAutoCalc = true;
        } else {
          productGroupItem.isAutoCalc = productGroupData.every((item) => (item._prdPromotionAmount || 0) === 0);
        }
      } else {
        productGroupItem = {
          id: createUid(),
          _uid: createUid(),
          title: `【${productCharacterName || ''}】${groupTypesMap[groupType]}`,
          groupType,
          productCharacterId,
          productCharacterName,
          data: productGroupData,
          isAutoCalc: true,
          couponList: [],
          remark: '',
          _isYinLiu: /引流/.test(productCharacterName),
          quarterType: null,
        };
      }

      productGroupList.push(productGroupItem);
    });
  });

  // 处理其他类型分类
  characterDict.forEach((item) => {
    handleOtherGroup(productGroupList, item);
  });

  // 计算总额
  productGroupList = calcGroupListTotal(productGroupList);

  return productGroupList.filter((item) => item.data.length > 0);
}

function getEditProductGroup(productList, groupList) {
  const total = calcGroupTotal(productList);

  groupList[0].data = productList;
  groupList[0].totalAmount = total.totalAmount;
  groupList[0].promotionAmount = total.promotionAmount;
  groupList[0].realAmount = total.realAmount;
  groupList[0].paidAmount = total.paidAmount;
  groupList[0].isAutoCalc = false;

  return groupList;
}

function getGroupList(productList = [], groupList = [], isEdit = false) {
  if (isEdit) {
    return getEditProductGroup(productList, groupList);
  } else {
    return getProductGroup(productList, groupList);
  }
}

// 修改合同时限制产品只可以选择当前合同产品
function getLimitProduct(contractProductArr) {
  let productTypeArr = contractProductArr.map((prd) => prd.productType);
  if (productTypeArr.length > 1) {
    productTypeArr = productTypeArr.filter((item) => item !== OTHERS);
  }
  return {
    text: productTypeNameCfg[productTypeArr[0]],
    val: productTypeArr[0],
  };
}

// 计算产品总额
function calcItemPayment(item) {
  let payment = null;

  // 课程单价 * 课时数
  if (isMoreThanZero((item.quantity && Number(item.quantity)) || 0)) {
    payment = NP.times(item.price, item.quantity);
  }

  if (!payment) {
    return 0;
  }

  // 包含其他金额的总额(产品优惠内赠送的金额)
  if (item._paymentAddition > 0) {
    payment = NP.plus(payment, item._paymentAddition);
  }
  return payment;
}

// 计算产品已收
function calcItemPaidAmount(item) {
  let paidAmount = 0;

  if (item._paidAmount) {
    paidAmount = item._paidAmount;
  }

  const _discountInfo = item._discountInfo || {};

  if (_discountInfo._couponAlreadyPayAmount) {
    paidAmount = NP.plus(paidAmount, _discountInfo._couponAlreadyPayAmount);
  }

  return paidAmount;
}

// 计算产品优惠金额(券、活动、产品优惠)
function calcItemPromotionAmount(item) {
  let promotionAmount = item._promotionAmount || 0;

  // if (item._promotionAmount) {
  //   promotionAmount = item._promotionAmount;
  // // (修改合同)已扣优惠
  // } else if (item.promotionConsumeAmount) {
  //   promotionAmount = item.promotionConsumeAmount;
  // }

  const _discountInfo = item._discountInfo || {};

  if (_discountInfo._couponDiscount) {
    promotionAmount = NP.plus(promotionAmount, _discountInfo._couponDiscount);
  }
  // 优惠活动与产品优惠不共存
  if (_discountInfo._activeDiscount) {
    promotionAmount = NP.plus(promotionAmount, _discountInfo._activeDiscount);
  } else if (item._prdPromotionAmount) {
    promotionAmount = NP.plus(promotionAmount, item._prdPromotionAmount);
  }

  return promotionAmount || /*item.promotionConsumeAmount ||*/ 0;
}

// 计算产品应收(依赖payment)
function calcItemRealAmount(item) {
  let realAmount = 0;

  if (item.payment) {
    realAmount = item.payment;

    if (item.promotionAmount) {
      realAmount = NP.minus(realAmount, item.promotionAmount || 0);
    }

    if (item.paidAmount) {
      realAmount = NP.minus(realAmount, item.paidAmount || 0);
    }
  }
  return realAmount;
}

function removePrdPromotion(item) {
  delete item._prdPromotion;
  delete item._prdPromotionAmount;
  delete item._paymentAddition;
  item._prdPromotionVal = null;
}

function removeActivePromotion(item) {
  delete item._activePromotion;
  if (item._discountInfo) {
    delete item._discountInfo._activeDiscount;
  }
}

function getCouponTypeName(tag) {
  const couponTypeMap = {
    'COUPON_NORMAL-COUPON_FULL_SALE_NORMAL': '满减券',
    'COUPON_REDEEM-COUPON_CASH_REDEEM': '代金券',
    'COUPON_DEPOSIT-COUPON_FULL_SALE_NORMAL': '抵扣券',
    'COUPON_DEPOSIT_ONLY-COUPON_CASH_DEPOSIT ': '定金券',
  };

  return couponTypeMap[`${tag.promotionNature}-${tag.promotionTypeId}`] || '优惠券';
}

/** 20220704 引流合同可以选择产品优惠 */
function isNormalClassCourse(groupItem) {
  return groupItem.groupType === CLASS_COURSE /* && !groupItem._isYinLiu*/;
}

function isNormalOneToN(groupItem) {
  return [ONE_ON_ONE_COURSE, ONE_ON_MANY].includes(groupItem.groupType) /* && !groupItem._isYinLiu*/;
}

function isNormalCanUseCoupon(groupItem) {
  return useCouponGroupTypes.includes(groupItem.groupType) /* && !groupItem._isYinLiu*/;
}

// 是否班级(小班、精英班)
function isClass(groupItem) {
  if (!groupItem || !groupItem.hasOwnProperty('groupType')) {
    return false;
  }
  return [CLASS_COURSE, ECS_CLASS].includes(groupItem.groupType);
}

export {
  checkSamePrd,
  setPrdRelation,
  getPrdRelation,
  getProductGroup,
  addProductWithSub,
  batchAddProductWithSub,
  addClassIdByUid,
  contractInfoAdepter,
  takeOutPromotionProperties,
  getLimitProduct,
  getGroupList,
  calcItemPayment,
  calcItemPromotionAmount,
  calcItemRealAmount,
  calcItemPaidAmount,
  removePrdPromotion,
  removeActivePromotion,
  calcGroupTotal,
  getCouponTypeName,
  isNormalClassCourse,
  isNormalOneToN,
  isNormalCanUseCoupon,
  isClass,
};
