import React, { Component } from 'react';
import { Modal as AntdModal } from 'antd';
import {
  Page,
  List,
  Input,
  Select,
  Button,
  InputNumber,
  SearchFilterForm,
  SearchFieldForm,
  DateRangePicker,
} from 'COMPONENT';

import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import ImportExcelModal from './components/ImportExcelModal';
import ExportResultModal from './components/ExportResultModal';

import { errorsHandle } from 'UTIL/common';

import { connect } from 'dva';

import { getDataDictList } from 'SERVICE/system';
import * as thirdPlatformAuditListApi from 'SERVICE/thirdPlatformAuditList';

import styles from './styles.module.less';

import moment from 'moment';
import file from 'UTIL/file';

const errorsHandler = errorsHandle();
const preMonthFistDate = moment().subtract(1, 'months').startOf('month').startOf('day');
const defaultDateRange = [preMonthFistDate, moment().endOf('day')];

const matchingStatusDict = [
  {
    text: '未匹配',
    val: 'NOT_YET_MATCH',
  },
  {
    text: '系统匹配',
    val: 'SYSTEM_MATCH',
  },
  {
    text: '人工匹配',
    val: 'ARTIFICIAL_MATCH',
  },
];

class ThirdPlatformAuditList extends Component {
  state = {
    list: [],

    total: 0,
    pageNum: 1,
    pageSize: 100,

    record: {},

    isExporting: false,
    searchData: {
      createTime: defaultDateRange,
    },

    platformDict: [],

    loading: false,
    tableFixedHeight: 100,

    importExcelModalVisible: false,
    exportResultModalVisible: false,
    confirmMatchModalVisible: false,
  };

  constructor(props) {
    super(props);
    this.currentId = null;
    this.searchRef = null;
  }

  componentWillMount() {
    this.init();
    this.initColumns();
    this.setTableFixedHeight();
    window.addEventListener('resize', this.setTableFixedHeight, false);
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.setTableFixedHeight, false);
  }

  componentDidMount() {
    this.searchRef?.form?.setFieldsValue({
      createTime: defaultDateRange,
    });
  }

  init() {
    this.initDict();
    this.initData();
  }

  initColumns() {
    this.columns = [
      {
        width: 50,
        title: '序号',
        dataIndex: 'index',
        render: (value, record, index) => index + 1,
      },
      {
        width: 200,
        title: '平台名称',
        dataIndex: 'platformName',
        render: (value) => (value ? value : '-'),
      },
      {
        width: 200,
        title: '支付参考号',
        dataIndex: 'thirdPartyOrderNo',
        render: (value) => (value ? value : '-'),
      },
      {
        width: 150,
        title: '支付金额（元）',
        dataIndex: 'amount',
        render: (value) => (value == null ? '-' : value),
      },
      {
        width: 160,
        title: '导入时间',
        dataIndex: 'createTime',
        render: (value) => (value ? moment(value).format('YYYY-MM-DD hh:mm:ss') : '-'),
      },
      {
        width: 100,
        title: '导入人',
        dataIndex: 'createUserName',
        render: (value) => (value ? value : '-'),
      },
      {
        width: 100,
        title: '匹配状态',
        dataIndex: 'matchingStatus',
        render: (value) => (value != null ? matchingStatusDict.find(({ val }) => val == value)?.text || '-' : '-'),
      },
      {
        width: 120,
        title: '操作',
        fixed: 'right',
        key: 'action',
        getActionConfig: (_, record) => {
          return record.matchingStatus == 'NOT_YET_MATCH'
            ? [
                { text: '人工匹配', onClick: () => this.handleMatch(record) },
                { text: '删除', onClick: () => this.handleDelete(record) },
              ]
            : [];
        },
      },
    ];
  }

  initDict() {
    getDataDictList({ category: 'THIRD_PARTY_PAYMENT', rows: 999 })
      .then((res) => {
        if (res?.rows) {
          this.setState({ platformDict: res?.rows.map(({ id, name }) => ({ val: id, text: name })) });
        } else {
          this.setState({ platformDict: [] });
        }
      })
      .catch(() => {
        this.setState({ platformDict: [] });
      });
  }

  initData() {
    this.onSearch(this.state.searchData);
  }

  reload() {
    this.onSearch(this.state.searchData);
  }

  setParams(apiParams) {
    if (apiParams.createTime) {
      if (moment.isMoment(apiParams.createTime[0])) {
        apiParams.createTimeBegin = apiParams.createTime[0].startOf('day').valueOf();
        apiParams.createTimeEnd = apiParams.createTime[1].endOf('day').valueOf();
      } else {
        apiParams.createTimeBegin = moment(apiParams.createTime[0]).startOf('day').valueOf();
        apiParams.createTimeEnd = moment(apiParams.createTime[1]).endOf('day').valueOf();
      }

      delete apiParams.createTime;
    }
  }

  async fetchList(apiParams) {
    this.setParams(apiParams);
    const res = await thirdPlatformAuditListApi.getThirdPayDataApiList(apiParams).catch(errorsHandler);
    if (res && res.data) {
      this.setState({
        total: res.data.total,
        list: res.data.list.map((item, id) => ({ id, ...item })),
      });
    } else {
      this.setState({
        list: [],
        total: 0,
      });
    }
  }

  async loadList(apiParams) {
    this.setState({ loading: true });
    await this.fetchList(apiParams);
    this.setState({ loading: false });
  }

  setTableFixedHeight = () => {
    this.setState({
      tableFixedHeight: Math.max(window.document.body.clientHeight - 348, 100),
    });
  };

  // 搜索
  onSearch = (searchData) => {
    this.setState(
      () => {
        return {
          searchData,
          pageNum: 1,
        };
      },
      () => {
        this.loadList({
          ...this.state.searchData,
          pageNum: this.state.pageNum,
          pageSize: this.state.pageSize,
        });
      },
    );
  };

  // 重置
  onReset = () => {
    this.setState({
      searchData: {
        createTime: defaultDateRange,
      },
    });
    this.searchRef?.form?.setFieldsValue({
      createTime: defaultDateRange,
    });
  };

  // 分页
  onPageChange = ({ current: pageNum, pageSize }) => {
    this.setState(
      () => {
        return {
          pageNum,
          pageSize,
        };
      },
      () => {
        this.loadList({
          ...this.state.searchData,
          pageNum: this.state.pageNum,
          pageSize: this.state.pageSize,
        });
      },
    );
  };

  // 导出
  handleExport = () => {
    if (!this.state.total) {
      return showMessage('当前查询数据为空');
    }
    this.setState({ isExporting: true });
    const apiParams = {
      ...this.state.searchData,
      pageNum: 1,
      pageSize: this.state.total,
    };
    this.setParams(apiParams);
    thirdPlatformAuditListApi
      .exportThirdPayDataApiList(apiParams)
      .then((res) => {
        showMessage('导出Excel成功', 'success');
        file.downloadDirect(res.data, `第三方支付平台数据列表_${Date.now()}`);
      })
      .catch((e) => {
        showMessage(e.resultMessage || e.text || '导出数据失败，请稍后重试', 'error');
      })
      .finally(() => {
        this.setState({ isExporting: false });
      });
  };

  handleImportData = () => {
    this.setState({
      importExcelModalVisible: true,
    });
  };

  handleViewImportResult = () => {
    this.setState({
      exportResultModalVisible: true,
    });
  };

  handleImportSuccess = () => {
    this.reload();
    this.handleCloseImportExcelModal();
  };

  handleCloseImportExcelModal = () => {
    this.setState({
      importExcelModalVisible: false,
    });
  };

  handleCloseExportResultModal = () => {
    this.setState({
      exportResultModalVisible: false,
    });
  };

  handleCloseConfirmMatchModal = () => {
    this.setState({
      confirmMatchModalVisible: false,
    });
  };

  // 人工匹配成功
  handleConfirmMatchSuccess = () => {
    this.reload();
    this.handleCloseConfirmMatchModal();
  };

  // 人工匹配
  handleMatch = async (record) => {
    const result = await thirdPlatformAuditListApi
      .manualMatchThirdPayData({
        id: record.id,
      })
      .catch(errorsHandle());
    if (!result || result.resultCode !== 0) {
      return;
    }
    this.reload();
    showMessage('人工匹配成功', 'success');
  };

  // 删除
  handleDelete = (record) => {
    AntdModal.confirm({
      title: '温馨提醒',
      content: '是否删除此数据？',
      okType: 'danger',
      onOk: () => {
        thirdPlatformAuditListApi
          .deleteThirdPayDataById({
            id: record.id,
          })
          .then(() => {
            this.reload();
            showMessage('删除成功', 'success');
          })
          .catch(errorsHandle());
      },
    });
  };

  generateToolbar() {
    return (
      <div style={{ marginBottom: 16 }}>
        <Button type="primary" style={{ marginRight: '14px' }} onClick={this.handleImportData}>
          数据导入
        </Button>
        <Button type="primary" style={{ marginRight: '14px' }} onClick={this.handleViewImportResult}>
          导入结果
        </Button>
        <Button type="primary" onClick={this.handleExport} loading={this.state.isExporting}>
          导出Excel
        </Button>
      </div>
    );
  }

  generateList() {
    const { state } = this;

    return (
      <List
        rowKey="id"
        dataSource={state.list}
        columns={this.columns}
        bordered={false}
        fixedHeader={false}
        onChange={this.onPageChange}
        pagination={{
          showQuickJumper: true,
          showSizeChanger: true,
          total: state.total,
          current: state.pageNum,
          pageSize: state.pageSize,
          defaultPageSize: 100,
          pageSizeOptions: ['100', '300', '500'],
          showTotal: (total) => `共 ${total} 条记录`,
        }}
        scroll={{
          y: this.state.tableFixedHeight,
          x: _.reduce(this.columns, (sum, data) => (sum += data.width || 0), 0),
        }}
      />
    );
  }

  generateImportExcelModal() {
    const props = {
      onImportSuccess: this.handleImportSuccess,
      onCancel: this.handleCloseImportExcelModal,
    };
    return this.state.importExcelModalVisible && <ImportExcelModal {...props} />;
  }

  generateExportResultModal() {
    const props = {
      onCancel: this.handleCloseExportResultModal,
    };
    return this.state.exportResultModalVisible && <ExportResultModal {...props} />;
  }

  generateConfirmMatchModal() {
    const props = {
      id: this.state.record.id,
      onCancel: this.handleCloseConfirmMatchModal,
      visible: this.state.confirmMatchModalVisible,
      onConfirmMatchSuccess: this.handleConfirmMatchSuccess,
    };
    return this.state.confirmMatchModalVisible && <ConfirmMatchModal {...props} />;
  }

  render() {
    const { state, props } = this;
    return (
      <Page
        className={styles.thirdPlatformAuditListWrapper}
        route={props.route}
        loading={state.loading}
        subHeader={
          <div>
            <SearchFilterForm
              ref={(c) => (this.searchRef = c)}
              truncationNum={8}
              onSearch={this.onSearch}
              onReset={this.onReset}
              appendToolbarBody={null}
            >
              <SearchFieldForm name="createTime">
                <DateRangePicker placeholder={['开始日期', '结束日期']} allowClear={false} />
              </SearchFieldForm>
              <SearchFieldForm name="thirdPartyOrderNo">
                <Input placeholder="请输入支付参考号" maxLength={32} allowClear />
              </SearchFieldForm>
              <SearchFieldForm name="amount">
                <InputNumber placeholder="请输入支付金额" max={1000000} min={0} />
              </SearchFieldForm>
              <SearchFieldForm name="platformId">
                <Select dict={this.state.platformDict} placeholder="平台名称" />
              </SearchFieldForm>
              <SearchFieldForm name="matchingStatus">
                <Select dict={matchingStatusDict} placeholder="匹配状态" />
              </SearchFieldForm>
            </SearchFilterForm>
          </div>
        }
        fitWindowHeight
      >
        {this.generateToolbar()}
        {this.generateList()}
        {this.generateImportExcelModal()}
        {this.generateExportResultModal()}
      </Page>
    );
  }
}

const mapStoreToProps = (store) => {
  const {
    account: { btnAuths },
    dict: { branch: branchDict, year: yearDict, quarter: quarterDict },
  } = store;
  return {
    branchDict,
    quarterDict,
    btnAuth: btnAuths || [],
    yearDict: (yearDict || []).sort((a, b) => b.text - a.text),
  };
};

export default connect(mapStoreToProps)(ThirdPlatformAuditList);
