import React from 'react';
import PropTypes from 'prop-types';
import moment from 'moment';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { FormItem, Button, Select, Validator, Loading } from 'COMPONENT';
import { Modal, DatePicker, TimePicker } from 'antd';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import { getSmallClassTeacher } from 'SERVICE/common';
import { editBigClassCourse } from 'SERVICE/bigClass';

const formCfg = {
  labelWidth: 110,
};
class Appointment extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      form: {
        teacherId: '',
        courseDate: null,
        courseStartTime: null,
      },
      checking: false,
      loading: false,
      modalLoading: true,
      teacherList: [],
    };
    this.errors = {};
  }

  static propTypes = {
    data: PropTypes.object,
    onSuccess: PropTypes.func,
    onCancel: PropTypes.func,
  };

  static defaultProps = {
    onSuccess: function () {},
    onCancel: function () {},
  };

  componentDidMount() {
    let { brenchId, courseDate, courseStartTime } = this.props.data;
    this.setState({
      form: {
        ...this.props.data,
        courseDate: moment(courseDate, 'YYYY-MM-DD'),
        courseStartTime: moment(courseStartTime, 'HH:mm:ss'),
      },
    });
    this.getSelectableTeacher(brenchId);
  }

  // 获取小班老师
  getSelectableTeacher = (brenchId) => {
    getSmallClassTeacher({ brenchId: brenchId })
      .then((res) => {
        let teacherList = res || [];
        let campusMap = teacherList.reduce((pre, cur) => {
          if (!pre[cur.campusName]) {
            pre[cur.campusName] = {
              label: cur.campusName,
              val: cur.campusId,
              options: [],
            };
          }
          pre[cur.campusName].options.push({
            text: cur.name,
            val: cur.userId,
          });

          return pre;
        }, {});

        let teacherGroup = Object.keys(campusMap).map((k) => campusMap[k]);

        this.setState({
          teacherList: teacherGroup,
        });
      })
      .finally(() => {
        this.setState({
          modalLoading: false,
        });
      });
  };

  // 设置单个表单值
  setFormValue = (key, value) => {
    this.setState({
      form: {
        ...this.state.form,
        [key]: value,
      },
    });
  };

  resetForm() {
    this.setState({
      form: {
        teacherId: '',
        courseDate: null,
        courseStartTime: null,
      },
      loading: false,
      checking: false,
    });
  }

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  onSubmit = () => {
    const errors = { ...this.errors };
    const errorTips = getCommonErrorTips(errors);

    this.setState({ checking: true });

    if (errorTips) {
      return;
    }

    this.setState({ loading: true });
    let { courseDate, courseStartTime, teacherId, onlineClassCourseId } = this.state.form;

    let params = {
      teacherId,
      onlineClassCourseId,
      courseDate: courseDate.format('YYYY-MM-DD'),
      courseStartTime: courseStartTime.format('HH:mm'),
    };

    editBigClassCourse(params)
      .then(() => {
        showMessage('保存成功', 'success');
        this.props.onSuccess();
      })
      .catch((e) => {
        showMessage(e.text);
      })
      .finally(() => {
        this.setState({ loading: false });
      });
  };

  onCancel = () => {
    this.props.onCancel();
  };

  getDisabledDate = (date) => {
    let Day = new Date();
    let DayTime = `${Day.getFullYear()}-${Day.getMonth() + 1}-${Day.getDate()}`;
    let currentDate = new Date(DayTime).getTime();

    return new Date(date).getTime() < currentDate;
  };

  render() {
    const { onCancel, data } = this.props;
    const { checking, loading, teacherList, modalLoading } = this.state;
    let { teacherId, courseDate, courseStartTime } = this.state.form;

    return (
      <Modal
        title={`修改课程（第${data.courseNum}讲）`}
        width={600}
        onCancel={onCancel}
        maskClosable={false}
        footer={
          <div>
            <Button type="primary" loading={loading} onClick={this.onSubmit}>
              提交
            </Button>
            <Button onClick={this.onCancel}>取消</Button>
          </div>
        }
        visible={true}
      >
        <Loading loading={modalLoading} />
        <FormItem {...formCfg} name="上课日期" required>
          <Validator
            {...{
              value: courseDate,
              rules: ['required'],
              silent: !checking,
              onErrorChange: (e) => {
                this.errorChange('courseDate', e);
              },
            }}
          >
            <DatePicker
              size="large"
              value={courseDate}
              placeholder="请选择上课日期"
              disabledDate={(val) => this.getDisabledDate(val)}
              onChange={(value) => {
                this.setFormValue('courseDate', value);
              }}
            />
          </Validator>
        </FormItem>
        <FormItem {...formCfg} name="上课时间" required>
          <Validator
            {...{
              value: courseStartTime,
              rules: ['required'],
              silent: !checking,
              onErrorChange: (e) => {
                this.errorChange('courseStartTime', e);
              },
            }}
          >
            <TimePicker
              size="large"
              format="HH:mm"
              value={courseStartTime}
              disabledHours={() => {
                return Array(6)
                  .fill(null)
                  .map((item, i) => {
                    return i;
                  });
              }}
              disabledMinutes={(h) => {
                if (h <= 6) {
                  return [0];
                }
                return [];
              }}
              onChange={(value) => {
                this.setFormValue('courseStartTime', value);
              }}
            />
          </Validator>
        </FormItem>
        <FormItem {...formCfg} name="主讲老师" required>
          <Select
            showSearch
            dict={teacherList}
            value={teacherId}
            mode="multiple"
            onChange={(value) => {
              this.setFormValue('teacherId', value);
            }}
            validator={{
              rules: ['required'],
              silent: !checking,
              onErrorChange: (e) => this.errorChange('teacherId', e),
            }}
          />
        </FormItem>
      </Modal>
    );
  }
}

export default Appointment;
