import React from 'react';
import { connect } from 'dva';
import moment from 'moment';
import { Page, List, TextLink, Button } from 'COMPONENT';
import { Modal } from 'antd';
import Loading from 'COMPONENT/Loading';
import columns from './column';
import styles from './styles.module.less';
import { getBigClassCourse, getBisClassDetail, deleteBigClassCourse, addBigClassCourse } from 'SERVICE/bigClass';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

import EditCouse from './editCourse';
import AddCouse from './AddNewCourse';

class CourseDetail extends React.Component {
  constructor(props) {
    super(props);

    this.state = Object.assign({
      loading: false, // {boolean} 内容加载状态
      list: [], // {Array} 列表数据，
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
      courseData: {
        totalClassHours: '',
        arrangeCourseHours: '',
        notArrangeCourseHours: '',
      },
      showEditModal: false,
      showAddModal: false,
    });

    this.pageParams = props.pageParams[`/${props.route.path}`] || {};
  }

  componentDidMount() {
    this.fetchClassCourse();
    this.fetchClassCourseStatistic();
  }

  fetchClassCourseStatistic() {
    getBisClassDetail({ onlineClassId: this.pageParams.classId }).then((res) => {
      this.setState({
        courseData: res.data,
      });
    });
  }

  fetchClassCourse = () => {
    this.setRequestLoading(true);

    getBigClassCourse({ onlineClassId: this.pageParams.classId })
      .then((res) => {
        this.setState({
          list: res.data,
          loading: false,
        });
      })
      .catch((e) => {
        showMessage(e.text || '请求异常');
        this.setRequestLoading(false);
      });
  };

  onPaginationChange = (pageData) => {
    this.setState(
      {
        currentPage: pageData.current,
        pageSize: pageData.pageSize,
      },
      this.fetchClassCourse,
    );
  };

  setRequestLoading = (loading) => {
    this.setState({
      loading,
    });
  };

  onDelete = (row) => {
    Modal.confirm({
      content: '提示：确定要删除吗？',
      okText: '确认',
      cancelText: '取消',
      onOk: () => {
        return new Promise((reslove) => {
          return deleteBigClassCourse({
            onlineClassCourseId: row.onlineClassCourseId,
          })
            .then(() => {
              showMessage('删除成功', 'success');
              this.fetchClassCourse();
              this.fetchClassCourseStatistic();
            })
            .catch((e) => {
              showMessage(e.text || '删除失败');
            })
            .finally(() => {
              reslove();
            });
        });
      },
    });
  };

  onEdit = (row) => {
    this.setState({
      showEditModal: true,
      currentRow: {
        ...row,
        brenchId: this.pageParams.blBranchId,
      },
    });
  };

  onSubmit = (courseList) => {
    addBigClassCourse(courseList)
      .then((res) => {
        this.setState({
          showAddModal: false,
        });
        showMessage('保存成功', 'success');
        this.fetchClassCourse();
        this.fetchClassCourseStatistic();
      })
      .catch((e) => {
        showMessage(e.text);
      });
  };

  render() {
    const { route } = this.props;
    const { loading, list, courseData, showEditModal, showAddModal, currentRow } = this.state;

    return (
      <Page
        name={route.name}
        route={route}
        subHeader={
          <div>
            <div className={styles.countWrap}>
              <p className={styles.count}>
                <span>总课时：{courseData.totalClassHours}</span>
                <span>已排课时：{courseData.arrangeCourseHours}</span>
                <span>未排课时：{courseData.notArrangeCourseHours}</span>
              </p>
              <Button
                type="primary"
                onClick={() => {
                  this.setState({
                    showAddModal: true,
                  });
                }}
              >
                新建课程
              </Button>
            </div>

            <List
              dataSource={list}
              columns={columns.concat({
                title: '操作',
                dataIndex: 'operation',
                key: 'operation',
                fixed: 'right',
                width: 150,
                render: (text, row) => {
                  const { courseDate, courseStartTime } = row;
                  const time = moment(courseDate + courseStartTime, 'YYYY-MM-DD HH:mm').format('x');
                  const currentTime = Date.now();

                  if (time - currentTime < 20 * 60 * 1000) {
                    return null;
                  }

                  return (
                    <div>
                      <TextLink style={{ marginRight: '20px' }} onClick={() => this.onEdit(row)}>
                        修改
                      </TextLink>

                      <TextLink onClick={() => this.onDelete(row)}>删除</TextLink>
                    </div>
                  );
                },
              })}
              bordered={true}
              className={styles.customList}
            />

            {showEditModal && (
              <EditCouse
                data={currentRow}
                onCancel={() => {
                  this.setState({
                    showEditModal: false,
                  });
                }}
                onSuccess={() => {
                  this.setState({
                    showEditModal: false,
                  });
                  this.fetchClassCourse();
                  this.fetchClassCourseStatistic();
                }}
              />
            )}

            {showAddModal && (
              <AddCouse
                branchId={this.pageParams.blBranchId}
                classId={this.pageParams.classId}
                onClose={() => {
                  this.setState({
                    showAddModal: false,
                  });
                }}
                onSubmit={this.onSubmit}
              />
            )}

            <Loading loading={loading} />
          </div>
        }
      ></Page>
    );
  }
}

function mapStateToProps(state) {
  return {
    pageParams: state.main.pageParams || {},
  };
}

export default connect(mapStateToProps)(CourseDetail);
