import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'dva';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { FormItem, Button, Input, Validator, Select } from 'COMPONENT';
import { Modal } from 'antd';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import styles from './styles.module.less';

import { saveStudentReceiveInfo } from 'SERVICE/bigClass';
import { getRegionList, getAreaList } from 'SERVICE/common';

const formCfg = {
  labelWidth: 110,
};
let cityMap = {};

class AddressDialog extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      addressDetail: {
        recipient: '',
        recipientPhone: '',
        address: '',
        provinceId: '',
        cityId: '',
        areaId: '',
      },
      checking: false,
      loading: false,
      provinceList: [],
      cityList: [],
      areaList: [],
    };
    this.errors = {};
  }

  static propTypes = {
    data: PropTypes.object,
    visible: PropTypes.bool,
    id: PropTypes.number,
    onSuccess: PropTypes.func,
    onCancel: PropTypes.func,
  };

  static defaultProps = {
    data: {},
    onSuccess: function () {},
    onCancel: function () {},
  };

  componentDidMount() {
    this.fetchProvince();
  }

  componentWillReceiveProps(props) {
    if (props.visible === false) {
      this.resetForm();
    } else {
      let receiveInfoData = props.data || {};
      if (receiveInfoData.provinceId) {
        receiveInfoData.provinceId = String(receiveInfoData.provinceId);
        receiveInfoData.cityId = String(receiveInfoData.cityId);

        this.setState({
          cityList: cityMap[receiveInfoData.provinceId],
        });

        this.getAreaListByCity(receiveInfoData.cityId);
      }

      this.setState({
        addressDetail: receiveInfoData,
      });
    }
  }

  formatareaOption = (arr) => {
    return (arr || []).map((c) => {
      return {
        text: c.name,
        val: c.id,
      };
    });
  };

  // 设置单个表单值
  setFormValue = (key, value) => {
    this.setState({
      addressDetail: {
        ...this.state.addressDetail,
        [key]: value,
      },
    });
  };

  resetForm() {
    this.setState({
      addressDetail: {
        recipient: '',
        recipientPhone: '',
        address: '',
        provinceId: '',
        cityId: '',
        areaId: '',
      },
      loading: false,
      checking: false,
    });
  }

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  onSubmit = () => {
    const { recipient, recipientPhone, address, provinceId, cityId, areaId } = this.state.addressDetail;

    const { onSuccess } = this.props;
    const errors = { ...this.errors };
    const errorTips = getCommonErrorTips(errors);

    this.setState({ checking: true });

    if (errorTips) {
      return;
    }

    if (!provinceId || !cityId || !areaId) {
      return showMessage('请选择省市区');
    }

    let params = {
      studentId: this.props.data.studentId,
      contractProductId: this.props.data.contractProductId,
      provinceId: Number(provinceId),
      cityId: Number(cityId),
      areaId,
      address,
      recipient,
      recipientPhone,
    };

    this.setState({ loading: true });

    saveStudentReceiveInfo(params)
      .then(() => {
        this.setState({ loading: false });
        showMessage('保存成功', 'success');
        onSuccess();
      })
      .catch((e) => {
        console.log(e);
        showMessage(e.text);
        this.setState({ loading: false });
      });
  };

  onCancel = () => {
    this.props.onCancel();
  };

  // 获取省份
  fetchProvince = () => {
    getRegionList().then((res) => {
      const provinceList = this.formatareaOption(res);

      // 生成省份map用于获取city
      cityMap = res.reduce((pre, cur) => {
        pre[cur.id] = this.formatareaOption(cur.children);
        return pre;
      }, {});

      this.setState(
        {
          provinceList: provinceList,
        },
        () => {
          let provinceId = this.props.data && this.props.data.provinceId;
          if (provinceId) {
            this.setState({
              cityList: cityMap[provinceId],
            });
          }
        },
      );
    });
  };

  // 获取地区
  getAreaListByCity = (value) => {
    getAreaList({ cityId: value }).then((res) => {
      this.setState({
        areaList: this.formatareaOption(res),
      });
    });
  };

  render() {
    const { visible, onCancel, data } = this.props;
    const { checking, loading, provinceList, areaList, cityList } = this.state;
    const { address, recipient, recipientPhone, provinceId, cityId, areaId } = this.state.addressDetail;

    return (
      <Modal
        title="收件信息"
        width={800}
        onCancel={onCancel}
        footer={
          <div>
            <Button type="primary" loading={loading} onClick={this.onSubmit}>
              提交
            </Button>
            <Button onClick={this.onCancel}>取消</Button>
          </div>
        }
        visible={visible}
      >
        <p className={styles.studentInfo}>
          <span>学生姓名：{data.studentName}</span>
          <span>学生编号：{data.studentId}</span>
        </p>

        <FormItem {...formCfg} name="地区" required>
          <Select
            className={styles.area}
            placeholder="省"
            dict={provinceList}
            value={provinceId}
            onChange={(value) => {
              this.setFormValue('provinceId', value);
              this.setState({
                areaList: [],
                cityList: cityMap[value],
              });
            }}
          />
          <Select
            className={styles.area}
            placeholder="市"
            dict={cityList}
            value={cityId}
            onChange={(value) => {
              this.setFormValue('cityId', value);
              value && this.getAreaListByCity(value);
            }}
          />
          <Select
            className={styles.area}
            placeholder="区"
            dict={areaList}
            value={areaId}
            onChange={(value) => {
              this.setFormValue('areaId', value);
            }}
          />
        </FormItem>

        <FormItem {...formCfg} name="详细地址" required>
          <Input
            value={address}
            onChange={(e) => {
              this.setFormValue('address', e.target.value);
            }}
            validator={{
              rules: ['required'],
              silent: !checking,
              onErrorChange: (e) => this.errorChange('address', e),
            }}
            placeholder="请输入详情地址"
          />
        </FormItem>

        <FormItem {...formCfg} name="收件人" required>
          <Input
            value={recipient}
            onChange={(e) => {
              this.setFormValue('recipient', e.target.value);
            }}
            validator={{
              rules: ['required'],
              silent: !checking,
              onErrorChange: (e) => this.errorChange('recipient', e),
            }}
            placeholder="请输入收件人"
          />
        </FormItem>

        <FormItem {...formCfg} name="收件人电话" required>
          <Input
            value={recipientPhone}
            placeholder="请输入收件人电话"
            onChange={(e) => {
              this.setFormValue('recipientPhone', e.target.value);
            }}
            validator={{
              rules: ['required', 'phone'],
              silent: !checking,
              onErrorChange: (e) => this.errorChange('recipientPhone', e),
            }}
          />
        </FormItem>
      </Modal>
    );
  }
}

export default connect()(AddressDialog);
