import React from 'react';
import { connect } from 'dva';
import { Page, List, TextLink, Button } from 'COMPONENT';
import { Modal } from 'antd';
import Loading from 'COMPONENT/Loading';
import Search from './search';
import columns from './column';
import styles from './styles.module.less';
import { getBisClass, deleteBisClass } from 'SERVICE/bigClass';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const unStartStatus = 'PENDING_START';

class BigClassManage extends React.Component {
  constructor(props) {
    super(props);

    this.state = Object.assign({
      loading: false, // {boolean} 内容加载状态
      list: [], // {Array} 列表数据，
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
      searchParams: {},
    });
  }

  componentWillReceiveProps(props) {
    if (props.pageParams) {
      let params = props.pageParams[`/${props.route.path}`];
      if (params && params.refreshBySelf) {
        this.fetchBigClassList();
        params.refreshBySelf = false;
      }
    }
  }

  fetchBigClassList = () => {
    this.setRequestLoading(true);
    let searchParams = this.state.searchParams;
    let pagination = {
      pageSize: this.state.pageSize,
      pageNum: this.state.currentPage,
    };

    let params = {
      ...searchParams,
      ...pagination,
    };

    getBisClass(params)
      .then((res) => {
        this.setState({
          list: res.data.list,
          totalRecords: res.data.total,
          loading: false,
        });
      })
      .catch((e) => {
        showMessage(e.text || '请求异常');
        this.setRequestLoading(false);
      });
  };

  onSearch = (searchParams) => {
    this.setState(
      {
        searchParams,
        currentPage: 1,
      },
      this.fetchBigClassList,
    );
  };

  onPaginationChange = (pageData) => {
    this.setState(
      {
        currentPage: pageData.current,
        pageSize: pageData.pageSize,
      },
      this.fetchBigClassList,
    );
  };

  setRequestLoading = (loading) => {
    this.setState({
      loading,
    });
  };

  toDetail = (params) => {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/LocalNetworkClass/detail',
        refresh: true,
        params: params,
      },
    });
  };

  toCourseDetail = (params) => {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/LocalClassCourseDetail',
        refresh: true,
        params: params,
      },
    });
  };

  toCreate = (params) => {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/LocalNetworkClass/create',
        refresh: true,
        params: params,
      },
    });
  };

  onDelete = (row) => {
    Modal.confirm({
      content: '提示：确定要删除吗？',
      okText: '确认',
      cancelText: '取消',
      onOk: () => {
        return new Promise((reslove) => {
          return deleteBisClass({ onlineClassId: row.classId })
            .then(() => {
              showMessage('删除成功', 'success');
              this.fetchBigClassList();
            })
            .catch((e) => {
              showMessage(e.text || '删除失败');
            })
            .finally(() => {
              reslove();
            });
        });
      },
    });
  };

  render() {
    const { route, hasAddPermisison, hasEditPermisison, hasDeletePermisison } = this.props;
    const { loading, list, totalRecords, currentPage, pageSize } = this.state;

    return (
      <Page
        name={route.name}
        route={route}
        headerToolBar={
          hasAddPermisison ? (
            <Button type="primary" onClick={() => this.toCreate({ type: 'new' })}>
              创建本地网课
            </Button>
          ) : null
        }
        subHeader={
          <div className={styles.wrap}>
            <Search onSearch={this.onSearch} />

            <List
              dataSource={list}
              columns={columns.concat({
                title: '操作',
                dataIndex: 'operation',
                key: 'operation',
                fixed: 'right',
                width: 150,
                render: (text, row) => {
                  return (
                    <div>
                      <TextLink
                        style={{ marginRight: '20px' }}
                        onClick={() =>
                          this.toDetail({
                            type: 'detail',
                            classId: row.classId,
                            branchId: row.blBranchId,
                            data: row,
                          })
                        }
                      >
                        详情
                      </TextLink>
                      {hasEditPermisison && (
                        <TextLink
                          style={{ marginRight: '20px' }}
                          disabled={row.status !== unStartStatus}
                          onClick={() =>
                            this.toCreate({
                              type: 'edit',
                              classId: row.classId,
                              branchId: row.blBranchId,
                              data: row,
                            })
                          }
                        >
                          编辑
                        </TextLink>
                      )}
                      <TextLink
                        style={{ marginRight: '20px' }}
                        onClick={() =>
                          this.toCourseDetail({
                            ...row,
                          })
                        }
                      >
                        课程
                      </TextLink>
                      {hasDeletePermisison && (
                        <TextLink disabled={row.status !== unStartStatus} onClick={() => this.onDelete(row)}>
                          删除
                        </TextLink>
                      )}
                    </div>
                  );
                },
              })}
              pagination={{
                current: currentPage,
                total: totalRecords,
                pageSize,
              }}
              onChange={this.onPaginationChange}
              bordered={true}
              className={styles.customList}
            />

            <Loading loading={loading} />
          </div>
        }
      ></Page>
    );
  }
}

function mapStateToProps(state) {
  const { btnAuths } = state.account;

  const hasAddPermisison = btnAuths.indexOf('LOCAL_NETWORK_CLASS_ADD') > -1;
  const hasEditPermisison = btnAuths.indexOf('LOCAL_NETWORK_CLASS_EDIT') > -1;
  const hasDeletePermisison = btnAuths.indexOf('LOCAL_NETWORK_CLASS_DELETE') > -1;

  return {
    hasAddPermisison,
    hasEditPermisison,
    hasDeletePermisison,
    pageParams: state.main.pageParams || {},
  };
}

export default connect(mapStateToProps)(BigClassManage);
