import { storeName, messageModalTextMap } from './config';
import {
  listChangerClassFlowRecords,
  findChangerClassFlowRecordById,
  stopChangerClassByFlowId,
  sendMessageForChangerClass,
  doChangerClassByFlowId,
  listWorkflowDynamic,
} from 'SERVICE/course';
import { clone } from 'lodash';
import { errorsHandling, isEmpty } from './utils';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import moment from 'moment';

const defaultState = {
  sKey: {
    sKey_oaInstanceId: '',
    sKey_studentName: '',
    sKey_applicantId: '',
    sKey_changeType: '',
    sKey_operationType: '',
    sKey_oldCampusId: '',
    sKey_newCampusId: '',
    sKey_auditStatus: '',
    sKey_changeStatus: '',
    sKey_isConfirm: '',
    sKey_startTime: '',
    sKey_endTime: '',
  },
  list: {
    // 列表数据
    records: [],
    rowCount: 0,
    pageSize: 20,
    pageCurrent: 1,
    listLoading: false,
  },
  listData: {
    selectedRows: [],
    selectedRow: null,
  },
  messageModal: {
    messageModalVisible: false,
    messageModalText: '',
    messageModalType: '',
  },
  detailModal: {
    detailModalVisible: false,
    detailModalData: {},
    workflowTableRecords: [],
  },
  dict: {
    applicantDict: [],
  },
};
Object.freeze(defaultState);

export default {
  namespace: storeName,
  state: {
    // index
    ...clone(defaultState.sKey),
    ...clone(defaultState.list),
    ...clone(defaultState.listData),
    ...clone(defaultState.messageModal),
    ...clone(defaultState.detailModal),
    ...clone(defaultState.dict),
  },
  reducers: {
    initSearchDefault(state, action) {
      return {
        ...state,
        sKey_startTime: moment().format('YYYY-MM-DD'),
        sKey_endTime: moment().format('YYYY-MM-DD'),
      };
    },

    updateState(state, action) {
      const { payload } = action;
      return {
        ...state,
        ...payload,
      };
    },

    updateList(state, { payload: { records, rowCount } }) {
      const mapRecords = records.map((record) => ({
        ...record,
        // some extra data
      }));

      return {
        ...state,
        records: mapRecords,
        rowCount,
      };
    },

    openMessageModal(state, { payload: { messageModalType, selectedRow } }) {
      const { selectedRows } = state;
      if (!selectedRows.includes(selectedRow)) {
        selectedRows.push(selectedRow);
      }

      return {
        ...state,
        messageModalVisible: true,
        messageModalText: messageModalTextMap[messageModalType],
        messageModalType,
        selectedRow,
        selectedRows,
      };
    },

    closeMessageModal(state) {
      const { selectedRows, selectedRow } = state;
      const selectedRowIndex = selectedRows.indexOf(selectedRow);
      selectedRows.splice(selectedRowIndex, 1);

      return {
        ...state,
        ...clone(defaultState.messageModal),
        selectedRow: null,
        selectedRows,
      };
    },

    closeDetailModal(state) {
      return {
        ...state,
        ...clone(defaultState.detailModal),
      };
    },
  },
  effects: {
    *loadData({ payload: { resolve, reject } }, { select, put }) {
      const sKeys = yield select(searchValuesSelector);
      const apiParams = yield select((state) => {
        const { sKey_startTime, sKey_endTime } = state[storeName];
        return {
          ...sKeys,
          startTime: sKey_startTime ? sKey_startTime + ' 00:00:00' : '',
          endTime: sKey_endTime ? sKey_endTime + ' 23:59:59' : '',
          page: state[storeName].pageCurrent,
          rows: state[storeName].pageSize,
          getTotal: true,
        };
      });
      const rsp = yield listChangerClassFlowRecords(apiParams, true).catch(errorsHandling());
      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updateList',
          payload: {
            records: rsp.data || [],
            rowCount: rsp.rowCount,
          },
        });
        resolve();
      } else {
        reject();
      }
    },

    *resetPage(action, { put }) {
      const newDefaultState = Object.keys(defaultState).reduce((acc, key) => {
        Object.assign(acc, defaultState[key]);
        return acc;
      }, {});
      yield put({
        type: 'updateState',
        payload: {
          ...clone(newDefaultState),
        },
      });
    },

    *onMessageModalOk({ payload: { resolve } }, { put, select }) {
      const flowId = yield select(flowIdSelector);
      const requestFnName = yield select((state) => state[storeName].messageModalType);
      const requestFnNameMap = {
        stopChangerClassByFlowId,
        sendMessageForChangerClass,
        doChangerClassByFlowId,
      };

      const response = yield requestFnNameMap[requestFnName]({ flowId }).catch(errorsHandling());
      if (response) {
        showMessage('操作成功', 'success');
        resolve();
      }
      yield put({
        type: 'closeMessageModal',
      });
    },

    *openDetailModal({ payload: { selectedRow, messageModalType } }, { put, select }) {
      const flowId = selectedRow.id;
      const response = yield findChangerClassFlowRecordById({ flowId }).catch(errorsHandling());
      const WorkflowResponse = yield listWorkflowDynamic({ flowId, type: 5 }).catch(errorsHandling());

      if (response && WorkflowResponse && response.data && WorkflowResponse.data) {
        yield put({
          type: 'updateState',
          payload: {
            detailModalVisible: true,
            detailModalData: response.data,
            workflowTableRecords: WorkflowResponse.data,
          },
        });
      }
    },
  },
};

function searchValuesSelector(state) {
  const store = state[storeName];
  return Object.keys(defaultState.sKey)
    .filter((key) => !isEmpty(store[key]))
    .reduce((accumulate, key) => {
      const val = store[key];
      const newKey = key.replace('sKey_', '');
      accumulate[newKey] = val;
      return accumulate;
    }, {});
}

function flowIdSelector(state) {
  return state[storeName].selectedRow.id;
}
