import React from 'react';
import styles from './styles.module.less';
import classNames from 'classnames';
import { Modal, Button } from 'COMPONENT';
import { listMiniClassStudentCourseByCpId } from 'SERVICE/course';
import { getWeekNameByDate } from 'UTIL/time';

const LESSONS_COUNT = 20; // 固定20讲
const COMPACT_COUNT = 4; // 大于4个课程时用固定宽度显示
const LESSON_COL_WIDTH = 330; // 课程固定宽度330px

/* 列首/固定列 */
function StickyCol(props) {
  const { className = '' } = props;
  return (
    <div key="stickyCol" className={classNames([styles.col, styles.stickyCol, className])}>
      {props.children}
    </div>
  );
}

/* 固定行 */
function StickyRow(props) {
  const { className = '' } = props;
  return <div className={classNames([styles.row, styles.stickyRow, className])}>{props.children}</div>;
}

/* 课程列 */
function CourseCol(props) {
  const { className = '', children } = props;
  return <div className={classNames([styles.col, styles.courseCol, className])}>{children}</div>;
}

/* 讲次表头 */
function CourseLessonHeader(props) {
  return (
    <CourseCol {...props} className={styles.lessonCol}>
      <div className={styles.date}>课程日期</div>
      <div className={styles.week}>星期</div>
      <div className={styles.lessonStatus}>状态</div>
    </CourseCol>
  );
}

export default class ProductDetailModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      courseList: [],
    };
    this.isProductNormal = props.product.status === 'NORMAL'; // 合同正常
    this.currentCourse = props.product.miniClassId; // 当前在班id
  }

  componentDidMount() {
    this.initDetail();
  }

  /* 是否在班 */
  isCurrentCourse(course) {
    return this.currentCourse === course.miniClassId;
  }

  async initDetail() {
    this.setState({ loading: true });
    try {
      const res = await listMiniClassStudentCourseByCpId({ cpId: this.props.product.id });
      if (res.resultCode !== 0) {
        throw new Error(res.resultMessage);
      }
      const courseList = this.formatCourse(res.data);
      this.setState({ courseList });
    } catch (e) {
      console.log('initDetail', e);
    } finally {
      this.setState({ loading: false });
    }
  }

  /** 格式化班级列表
   * 1. 排序：在班的班级要排在最前面
   * 2. 填充：接口只返回了有排课的班级，如果在班的班级没有排课要填充到最前面;
   */
  formatCourse(courseList = []) {
    if (!this.currentCourse) return courseList;
    // 在班的课程放在最前面
    const curCourseIndex = courseList.findIndex((course) => this.isCurrentCourse(course));
    if (curCourseIndex !== -1) {
      // 排序
      const [curCourse] = courseList.splice(curCourseIndex, 1);
      courseList.unshift(curCourse);
    } else {
      // 填充
      const { product } = this.props;
      courseList.unshift({
        miniClassId: product.miniClassId,
        miniClassName: product.miniClassName,
        blCampusId: product.miniClassCampusId,
        blCampusName: product.miniClassCampusName,
        courseAttendentVos: [],
      });
    }
    return courseList;
  }

  /* 操作 */
  renderCourseActions = (course) => {
    // 非正常合同
    if (!this.isProductNormal) return null;
    const { auths, product, action } = this.props;
    const isCurrentCourse = this.isCurrentCourse(course);
    const hasUncharge = course.courseAttendentVos.some((item) => item.chargeStatus === 'UNCHARGE');
    // 非在班
    if (!isCurrentCourse) {
      return (
        <div className={styles.actions}>
          {auths.switch && hasUncharge && (
            <Button
              type="primary"
              size="xSmall"
              onClick={() => action({ type: 'SWITCH', data: { product, miniClassId: course.miniClassId } })}
            >
              调课
            </Button>
          )}
        </div>
      );
    }
    // 在班
    const { miniClassStudentStatus } = product;
    if (miniClassStudentStatus === 'PREPARE_DROP') {
      return (
        <div className={styles.actions}>
          {auths.cancelPreWithdraw && (
            <Button type="primary" size="xSmall" onClick={() => action({ type: 'CANCEL_PREWITHDRAW', data: product })}>
              取消预退
            </Button>
          )}
        </div>
      );
    }
    if (miniClassStudentStatus === 'NORMAL') {
      return (
        <div className={styles.actions}>
          {auths.switch && hasUncharge && (
            <Button type="primary" size="xSmall" onClick={() => action({ type: 'SWITCH', data: { product } })}>
              调课
            </Button>
          )}
          {auths.transfer && (
            <Button type="primary" size="xSmall" onClick={() => action({ type: 'TRANSFER', data: product })}>
              转班
            </Button>
          )}
          {auths.preWithdraw && (
            <Button type="primary" size="xSmall" onClick={() => action({ type: 'PREWITHDRAW', data: product })}>
              预退
            </Button>
          )}
          {auths.withdraw && (
            <Button size="xSmall" onClick={() => action({ type: 'WITHDRAW', data: product })}>
              退班
            </Button>
          )}
        </div>
      );
    }
    return <div className={styles.actions}></div>;
  };

  renderCourseCols = (rowindex, courseList) => {
    const courseNum = rowindex + 1;
    return (
      <div className={classNames([styles.colGroup])}>
        {courseList.map((course, courseIndex) => {
          const lesson = course.courseAttendentVos.find((item) => item.courseNum === courseNum);
          return lesson ? (
            <CourseCol key={courseIndex} className={styles.lessonCol}>
              <div className={styles.date}>
                {lesson.courseDate} {lesson.courseTime}
              </div>
              <div className={styles.week}>{getWeekNameByDate(lesson.courseDate)}</div>
              <div className={styles.lessonStatus}>{lesson.chargeStatus === 'CHARGED' ? '已扣费' : '未扣费'}</div>
            </CourseCol>
          ) : (
            <CourseCol key={courseIndex} className={styles.lessonCol} />
          );
        })}
      </div>
    );
  };

  renderLessonHeader = (props = {}) => {
    return (
      <CourseCol {...props} className={styles.lessonCol}>
        <div className={styles.date}>课程日期</div>
        <div className={styles.week}>星期</div>
        <div className={styles.lessonStatus}>状态</div>
      </CourseCol>
    );
  };

  renderTable = () => {
    const { product } = this.props;
    const { courseList } = this.state;
    return (
      <div
        className={styles.productTable}
        style={courseList.length < COMPACT_COUNT ? null : { width: 140 + courseList.length * LESSON_COL_WIDTH + 'px' }}
      >
        <StickyRow className={styles.dark}>
          <StickyCol>产品名称</StickyCol>
          <div className={classNames([styles.col, styles.name])}>
            {product.productName}（{product.statusName}）
          </div>
        </StickyRow>
        <StickyRow>
          <StickyCol>班级名称</StickyCol>
          {courseList.map((course, index) => (
            <CourseCol key={index}>
              {course.miniClassName}
              {this.isCurrentCourse(course) ? '（在班）' : ''}
            </CourseCol>
          ))}
        </StickyRow>
        <StickyRow>
          <StickyCol>班级学习中心</StickyCol>
          {courseList.map((course, index) => (
            <CourseCol key={index}>{course.blCampusName}</CourseCol>
          ))}
        </StickyRow>
        <StickyRow>
          <StickyCol>操作</StickyCol>
          {courseList.map((course, index) => (
            <CourseCol key={index}>{this.renderCourseActions(course)}</CourseCol>
          ))}
        </StickyRow>
        <StickyRow className={styles.dark}>
          <StickyCol>讲次(共{LESSONS_COUNT}讲)</StickyCol>
          {courseList.length ? (
            courseList.map((_, index) => <CourseLessonHeader key={index} />)
          ) : (
            <CourseLessonHeader />
          )}
        </StickyRow>
        {Array.from({ length: LESSONS_COUNT }).map((_, index) => (
          <div className={styles.row} key={index}>
            <StickyCol>第{index + 1}讲</StickyCol>
            {this.renderCourseCols(index, courseList)}
          </div>
        ))}
      </div>
    );
  };

  handleClose = () => {
    this.props.onClose();
  };

  render() {
    return (
      <Modal
        title="上课详情"
        width={1300}
        loading={this.state.loading}
        footer={null}
        bodyStyle={{ paddingBottom: '50px' }}
        onCancel={this.handleClose}
      >
        <div className={styles.tableWrapper}>{this.renderTable()}</div>
      </Modal>
    );
  }
}
