import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import MultiSelect from 'COMPONENT/MultiSelect';
import Button from 'COMPONENT/Button';
import DotButton from 'COMPONENT/DotButton';
import styles from './styles.less';
import loaders from '../loaders';
import dicts from '../dicts';
import utils from '../utils';
import { loadCommonDict } from 'UTIL/dict';
import moment from 'moment';
import Schedule from '../Schedule';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import DetailModal from '../DetailModal';
import { debounce } from 'lodash';
import { getDictTextByVal, loadBranchCampusDict } from 'UTIL/dict';

const { startDate: initStartDate, endDate: initEndDate } = utils.getWeekRange(moment());

const getToolbarLeft = () => {
  const samples = dicts.classStatusDict.map((d) => {
    return { text: d.text, color: utils.statusColors[d.val].primary };
  });
  return (
    <div className={styles.toolbarLeft}>
      {samples.map((sa) => {
        const { color, text } = sa;
        return (
          <div key={color} className={styles.sample}>
            <div className={styles.sampleColor} style={{ background: color }} />
            <div className={styles.sampleText}>{text}</div>
          </div>
        );
      })}
    </div>
  );
};

const dotBtnStyle = {
  height: 36,
  width: 28,
  lineHeight: '36px',
};

class ClassScheduleMain extends React.Component {
  constructor(props) {
    super(props);
    this.ownerDictVersion = 1;
    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态

      campus: [], // {string[]} 查询条件 所属学习中心
      grade: [], // {string[]} 查询条件 年级
      subject: [], // {string[]} 查询条件 科目
      productType: [], // {string[]} 查询条件 产品类型
      classType: [], // {string[]} 查询条件 班级类型
      owner: [], // {[{ val, text }]} 查询条件 课程表归属者，老师或学生
      className: null, // {string} 查询条件 班课名称

      campusDict: [], // {[object]} 字典 所属学习中心
      gradeDict: [], // {[object]} 字典 年级
      subjectDict: [], // {[object]} 字典 科目
      productTypeDict: [], // {[object]} 字典 产品类型
      classTypeDict: [], // {[object]} 字典 班级类型
      ownerDict: [], // {[object]} 字典 课程表归属者，老师或学生

      startDate: initStartDate, //  {string}  开始日期
      endDate: initEndDate, //  {string}  结束日期

      list: [], // {object[]} 列表数据
      defaultSearchKeys: null, // {object} 列表数据的默认查询关键字
      searchKeys: null, // {object} 列表数据的查询关键字

      detailModalVis: false, // {boolean} 详情弹框显示状态
      currentClass: null, // {string} 当前详情弹框的课程
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = () => {
    const { ownerType } = this.props;
    //const campusId = 'ORG0000000030'
    const campusId = 'ff808081482d0c3801482d380c9e0009';

    const { startDate, endDate } = this.state;
    this.setState({ loading: true });
    Promise.all([loaders.loadOwnerDict({ ownerType })]).then(([ownerDict]) => {
      const defaultCampus = [campusId];
      const defaultOwner = ownerDict.length > 0 ? [ownerDict[0]] : [];
      const defaultSearchKeys = {
        campus: defaultCampus,
        grade: [],
        subject: [],
        productType: [],
        classType: [],
        owner: defaultOwner,
        className: null,
      };
      Promise.all([
        loaders.loadList({ ...defaultSearchKeys, startDate, endDate, ownerType }),
        loadBranchCampusDict(),
        loadCommonDict('STUDENT_GRADE'),
        loadCommonDict('SUBJECT'),
        loadCommonDict('ProductClassType'),
        loadCommonDict('CLASS_TYPE'),
      ]).then(([list, campusDict, gradeDict, subjectDict, productTypeDict, classTypeDict]) => {
        this.setState({
          loading: false,
          list,
          campus: defaultCampus,
          owner: defaultOwner,
          campusDict,
          gradeDict,
          subjectDict,
          productTypeDict,
          classTypeDict,
          ownerDict,
          defaultSearchKeys,
          searchKeys: defaultSearchKeys,
        });
      });
    });
  };

  getListParams = () => {
    const { ownerType } = this.props;
    const {
      searchKeys: { campus, grade, subject, productType, classType, owner, className },
      startDate,
      endDate,
    } = this.state;
    return { campus, grade, subject, productType, classType, owner, className, startDate, endDate, ownerType };
  };

  search = () => {
    const { campus, grade, subject, productType, classType, owner, className } = this.state;
    const searchKeys = { campus, grade, subject, productType, classType, owner, className };
    this.setState({ loading: true });
    loaders.loadList({ ...this.getListParams(), ...searchKeys }).then(
      (list) => {
        this.setState({ loading: false, searchKeys, list });
      },
      (e) => {
        this.setState({ loading: false });
        showMessage(e.text || '加载失败');
      },
    );
  };

  reset = () => {
    this.setState(this.state.defaultSearchKeys);
  };

  goWeek = ({ startDate, endDate }) => {
    this.setState({ loading: true });
    loaders.loadList({ ...this.getListParams(), startDate, endDate }).then(
      (list) => {
        this.setState({ loading: false, startDate, endDate, list });
      },
      (e) => {
        this.setState({ loading: false });
        showMessage(e.text || '加载失败');
      },
    );
  };

  ownerChange = (val) => {
    const { ownerDict } = this.state;
    val = val ? val.slice(0, utils.panelsMaxCount) : val;
    let dict;
    if (val && val.length >= utils.panelsMaxCount) {
      dict = ownerDict.map((d) => {
        if (val.indexOf(d.val) < 0) {
          return { ...d, disabled: true };
        }
        return d;
      });
    } else {
      dict = ownerDict.map((d) => {
        return { val: d.val, text: d.text };
      });
    }
    this.setState({ owner: val, ownerDict: dict });
  };

  onOwnerSelectSearchKeyChange = (key) => {
    debounce(this.searchOwnerDict, 200)(key);
  };

  searchOwnerDict = (key) => {
    const { ownerType } = this.props;
    if (key) {
      const ownerDictVersion = ++this.ownerDictVersion;
      loaders.loadOwnerDict({ name: key, ownerType }).then(
        (dict) => {
          if (this.ownerDictVersion === ownerDictVersion) {
            this.setState({ ownerDict: dict });
          }
        },
        () => {
          this.setState({ ownerDict: [] });
        },
      );
    } else {
      this.ownerDictVersion++;
      this.setState({ ownerDict: [] });
    }
  };

  render() {
    const { ownerType } = this.props;
    const {
      loading,
      searchExpand,
      campus,
      grade,
      subject,
      productType,
      classType,
      owner,
      className,
      campusDict,
      gradeDict,
      subjectDict,
      productTypeDict,
      classTypeDict,
      ownerDict,
      startDate,
      endDate,
      list,
      detailModalVis,
      currentClass,
      searchKeys,
    } = this.state;

    return (
      <Page
        loading={loading}
        fitWindowHeight={true}
        route={this.props.route}
        subHeader={
          <div className={styles.searchBar}>
            <SearchBar
              expand={searchExpand}
              onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
              onSearch={this.search}
              onReset={this.reset}
            >
              <MultiSelect
                placeholder="所属学习中心"
                dict={campusDict}
                value={campus}
                onChange={(val) => this.setState({ campus: val })}
              />
              <MultiSelect
                placeholder="年级"
                dict={gradeDict}
                value={grade}
                onChange={(val) => this.setState({ grade: val })}
              />
              <MultiSelect
                placeholder="科目"
                dict={subjectDict}
                value={subject}
                onChange={(val) => this.setState({ subject: val })}
              />
              <MultiSelect
                placeholder="产品类型"
                dict={productTypeDict}
                value={productType}
                onChange={(val) => this.setState({ productType: val })}
              />
              <MultiSelect
                placeholder="班级类型"
                dict={classTypeDict}
                value={classType}
                onChange={(val) => this.setState({ classType: val })}
              />
              <MultiSelect
                placeholder={getDictTextByVal(dicts.ownerTypeDict, ownerType)}
                dict={ownerDict}
                value={owner}
                onChange={this.ownerChange}
                onSearchKeyChange={
                  ownerType === dicts.OWNER_TYPE_TEACHER ? undefined : this.onOwnerSelectSearchKeyChange
                }
                textInValue
              />
              <Input
                placeholder="班课名称"
                value={className}
                onChange={(e) => this.setState({ className: e.target.value })}
              />
            </SearchBar>
          </div>
        }
      >
        <div className={styles.wrap}>
          {searchKeys && searchKeys.owner && searchKeys.owner.length > 0 ? (
            <div style={{ height: '100%' }}>
              <div className={styles.toolbar}>
                {getToolbarLeft()}
                <div className={styles.toolbarRight}>
                  <DotButton
                    iconType="left"
                    className={styles.weekBtn}
                    shape={'square'}
                    style={dotBtnStyle}
                    onClick={() =>
                      this.goWeek({ startDate: utils.addDays(startDate, -7), endDate: utils.addDays(endDate, -7) })
                    }
                  />
                  <Button className={styles.weekBtn} onClick={() => this.goWeek(utils.getWeekRange(moment()))}>
                    本周
                  </Button>
                  <DotButton
                    iconType="right"
                    className={styles.weekBtn}
                    shape={'square'}
                    style={dotBtnStyle}
                    onClick={() =>
                      this.goWeek({ startDate: utils.addDays(startDate, 7), endDate: utils.addDays(endDate, 7) })
                    }
                  />
                </div>
                <div className={styles.toolbarDateRange}>
                  {startDate}~{endDate}
                </div>
              </div>
              <Schedule
                startDate={startDate}
                endDate={endDate}
                list={list}
                onCardClick={(data) => this.setState({ detailModalVis: true, currentClass: data })}
              />
            </div>
          ) : (
            <div className={styles.tips}>{`请先选择${getDictTextByVal(dicts.ownerTypeDict, ownerType)}查询课表`}</div>
          )}
        </div>
        {detailModalVis && (
          <DetailModal
            classId={currentClass.classId}
            productClassId={currentClass.productClassId}
            ownerType={ownerType}
            onClose={() => this.setState({ detailModalVis: false })}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
  };
};

export default connect(mapStateToProps)(ClassScheduleMain);
