import {
  getStudentClassCourseList,
  getTeacherClassCourseList,
  getClassCourseDetail,
  findStudentByCampus,
  getUserAutoComplateEnhanced,
} from 'SERVICE/course';
import dicts from './dicts';
import utils from './utils';
import { getDictTextByVal } from '../../../utils/dict/index';

const loadOwnerDict = async ({ name, ownerType }) => {
  if (ownerType === dicts.OWNER_TYPE_TEACHER) {
    const data = await getUserAutoComplateEnhanced({ roleCode: 'TEATCHER', term: '' }, true);
    return data.map((d) => {
      return { val: d.value, text: d.label };
    });
  } else {
    if (name) {
      const data = await findStudentByCampus({ name }, true);
      return data.map((d) => {
        return { val: d.value, text: d.label, title: d.label };
      });
    } else {
      return [];
    }
  }
};

const loadList = async ({
  campus,
  grade,
  subject,
  productType,
  classType,
  owner,
  className,
  startDate,
  endDate,
  ownerType,
}) => {
  if (!owner || owner.length === 0) {
    return [];
  }
  try {
    const api = ownerType === dicts.OWNER_TYPE_TEACHER ? getTeacherClassCourseList : getStudentClassCourseList;
    const params = {
      blCampusIds: campus,
      gradeIds: grade,
      subjectIds: subject,
      productClassTypes: productType,
      classTypes: classType,
      [ownerType === dicts.OWNER_TYPE_TEACHER ? 'teacherIds' : 'studentIds']: owner ? owner.map((o) => o.val) : null,
      productClassName: className,
      startTime: startDate,
      endTime: endDate,
    };
    let data = await api(params);
    data = data || [];
    const classList = data.map((d) => {
      const type = d.productClassType;
      return {
        classId: d.courseId,
        productClassId: d.productClassId,
        teacherId: d.teacherId,
        teacherName:
          type === dicts.PRODUCT_TYPE_TWO || type === dicts.PRODUCT_TYPE_TWO_ASSIST
            ? d.twoTeacherMain
            : ownerType === dicts.OWNER_TYPE_TEACHER
            ? d.teacherName
            : d.teacher,
        studentId: d.studentId,
        studentName: d.studentName,
        status: getStatus(d),
        type,
        date: d.courseDate,
        startTime: d.courseStartTime ? d.courseStartTime.slice(11, 16) : null,
        endTime: d.courseEndTime ? d.courseEndTime.slice(11, 16) : null,
        className: d.productClassName,
        gradeName: d.grade,
        subjectName: d.subject,
        campusName: d.blCampus,
        classroomName: d.classRoom,
        classNo: d.courseNum,
        comeCount: d.completeNumOfStudent,
        shouldComeCount: d.numberOfStudent,
        classStudents: d.studentInCourse,
        assistTeacherName: d.twoTeacherAssist,
      };
    });
    const cardList = classList
      .filter((c) => !(ownerType === dicts.OWNER_TYPE_STUDENT && c.type === dicts.PRODUCT_TYPE_TWO))
      .map((cls) => {
        const {
          classId,
          productClassId,
          teacherId,
          teacherName,
          studentId,
          studentName,
          status,
          type,
          date,
          startTime,
          endTime,
          className,
          gradeName,
          subjectName,
          campusName,
          classroomName,
          classNo,
          comeCount,
          shouldComeCount,
          classStudents,
          assistTeacherName,
        } = cls;
        return {
          panelId: ownerType === dicts.OWNER_TYPE_TEACHER ? teacherId : studentId,
          panelName: ownerType === dicts.OWNER_TYPE_TEACHER ? teacherName : studentName,
          date,
          startTime,
          endTime,
          status,
          type,
          headerInfo: `${startTime}-${endTime}`,
          classId,
          productClassId,
          title: getCardTitle(cls),
          infos: getCardInfos(cls, ownerType),
          moreInfos: getCardMoreInfos(cls, ownerType),
        };
      });
    //await new Promise(resolve => { setTimeout(resolve, 1000) })
    return getPanelList(cardList);
  } catch (e) {
    console.log(e);
  }
};

const getCardMoreInfos = (classInfo, ownerType) => {
  const {
    classId,
    teacherId,
    teacherName,
    studentId,
    studentName,
    status,
    type,
    date,
    startTime,
    endTime,
    className,
    gradeName,
    subjectName,
    campusName,
    classroomName,
    classNo,
    comeCount,
    shouldComeCount,
    classStudents,
    assistTeacherName,
  } = classInfo;
  let infos = [];
  if (type === dicts.PRODUCT_TYPE_ONE || type === dicts.PRODUCT_TYPE_OTM) {
    infos = [
      ...infos,
      { label: '授课老师', content: teacherName },
      { label: '时间', content: `${startTime}-${endTime}` },
      { label: '地点', content: campusName },
    ];
    infos.push({ label: '上课学生', content: classStudents });
  } else {
    if (type !== dicts.PRODUCT_TYPE_LECTURE) {
      infos.push({ label: '课程序号', content: classNo });
    }
    infos.push({
      label: type === dicts.PRODUCT_TYPE_TWO || type === dicts.PRODUCT_TYPE_TWO_ASSIST ? '主讲老师' : '授课老师',
      content: teacherName,
    });
    if (type === dicts.PRODUCT_TYPE_TWO_ASSIST) {
      infos.push({ label: '辅导老师', content: assistTeacherName });
    }
    infos = [
      ...infos,
      { label: '时间', content: `${startTime}-${endTime}` },
      { label: '地点', content: utils.getDotStr([campusName, classroomName]) },
    ];
    if (ownerType === dicts.OWNER_TYPE_TEACHER && type !== dicts.PRODUCT_TYPE_TWO) {
      infos.push({ label: '上课/应到', content: `${comeCount}/${shouldComeCount}` });
    }
  }
  return infos;
};

const getCardTitle = (classInfo) => {
  const { type, className, gradeName, subjectName } = classInfo;
  if (type === dicts.PRODUCT_TYPE_ONE || type === dicts.PRODUCT_TYPE_OTM) {
    return `${dicts.productTypeDict.find((d) => d.val === type).text}${utils.getDotStr([gradeName, subjectName])}`;
  }
  return className;
};

const getCardInfos = (classInfo, ownerType) => {
  const {
    classId,
    teacherId,
    teacherName,
    studentId,
    studentName,
    status,
    type,
    date,
    startTime,
    endTime,
    className,
    gradeName,
    subjectName,
    campusName,
    classroomName,
    classNo,
    comeCount,
    shouldComeCount,
    classStudents,
    assistTeacherName,
  } = classInfo;
  let infos = [];
  if (type === dicts.PRODUCT_TYPE_ONE || type === dicts.PRODUCT_TYPE_OTM) {
    infos.push(campusName);
  } else {
    infos.push(utils.getDotStr([campusName, classroomName]));
  }
  if (ownerType === dicts.OWNER_TYPE_STUDENT) {
    infos.push(type === dicts.PRODUCT_TYPE_TWO_ASSIST ? assistTeacherName : teacherName);
  }
  return infos;
};

const getPanelList = (cardList) => {
  let panelList = [];
  cardList.forEach((card) => {
    const { panelId, panelName, ...cardInfo } = card;
    let panel = panelList.find((p) => p.id === panelId);
    if (panel) {
      panel.cardList.push(cardInfo);
    } else {
      panelList.push({ id: panelId, name: panelName, cardList: [cardInfo] });
    }
  });
  return panelList;
};

const getStatus = (d) => {
  const courseStatus = d.courseStatus;
  let status;
  if (d.crashInd > 0) {
    status = dicts.CLASS_STATUS_CONFLICT;
  } else if (
    courseStatus === 'STUDENT_ATTENDANCE' ||
    courseStatus === 'TEACHER_ATTENDANCE' ||
    courseStatus === 'STUDY_MANAGER_AUDITED'
  ) {
    status = dicts.CLASS_STATUS_CHECKED;
  } else if (courseStatus === 'TEACHER_LEAVE' || courseStatus === 'STUDENT_LEAVE') {
    status = dicts.CLASS_STATUS_LEAVE;
  } else {
    status = courseStatus;
  }
  return status;
};

const loadDetail = async ({ classId, productClassId, ownerType }) => {
  try {
    const d = await getClassCourseDetail({ courseId: classId, productClassId });
    const { classRoomConflict, studentConflict, teacherConflict } = d;
    const type = d.productClassType;
    const classInfo = {
      classId,
      status: getStatus(d),
      type,
      date: d.courseDate,
      time: d.courseTime,
      className: d.productClassName,
      gradeName: d.grade,
      subjectName: d.subject,
      campusName: d.blCampus,
      teacherName:
        type === dicts.PRODUCT_TYPE_TWO || type === dicts.PRODUCT_TYPE_TWO_ASSIST ? d.twoTeacherMain : d.teacher,
      classroomName: d.classRoom,
      studentName: d.student,
      classNo: d.courseNum,
      comeCount: d.completeNumOfStudent,
      shouldComeCount: d.numberOfStudent,
      classStudents: d.studentInCourse,
      assistTeacherName: d.twoTeacherAssist,
    };
    let conflicts = [];
    if (teacherConflict && teacherConflict.length > 0) {
      conflicts.push({ title: '老师冲突', list: teacherConflict.map((c) => c.content) });
    }
    if (studentConflict && studentConflict.length > 0) {
      conflicts.push({ title: '学生冲突', list: studentConflict.map((c) => c.content) });
    }
    if (classRoomConflict && classRoomConflict.length > 0) {
      conflicts.push({ title: '教室/卡座冲突', list: classRoomConflict.map((c) => c.content) });
    }
    return {
      title: getDetailTitle(classInfo),
      className: getDetailClassName(classInfo),
      infos: getDetailInfos(classInfo),
      conflicts,
    };
  } catch (e) {
    console.log(e);
  }
};

const getDetailTitle = (classInfo) => {
  const { type, classNo } = classInfo;
  let title = '';
  title += dicts.productTypeDict.find((d) => d.val === type).text;
  title += '课程详情';
  if (classNo) {
    title += `（${classNo}）`;
  }
  return title;
};

const getDetailClassName = (classInfo) => {
  const {
    classId,
    productClassId,
    teacherId,
    teacherName,
    studentId,
    studentName,
    status,
    type,
    date,
    startTime,
    endTime,
    className,
    gradeName,
    subjectName,
    campusName,
    classroomName,
    classNo,
    comeCount,
    shouldComeCount,
    classStudents,
    assistTeacherName,
  } = classInfo;
  if (
    type === dicts.PRODUCT_TYPE_SMALL ||
    type === dicts.PRODUCT_TYPE_TWO ||
    type === dicts.PRODUCT_TYPE_TWO_ASSIST ||
    type === dicts.PRODUCT_TYPE_LECTURE
  ) {
    return className;
  } else if (type === dicts.PRODUCT_TYPE_ONE || type === dicts.PRODUCT_TYPE_OTM) {
    return utils.getDotStr([gradeName, subjectName, getDictTextByVal(dicts.productTypeDict, type)], '');
  }
  return null;
};

const getDetailInfos = (classInfo) => {
  const {
    classNo,
    type,
    className,
    status,
    comeCount,
    shouldComeCount,
    campusName,
    gradeName,
    subjectName,
    date,
    time,
    teacherName,
    assistTeacherName,
    classStudents,
    classroomName,
  } = classInfo;
  let statusSecondary;
  if (
    type === dicts.PRODUCT_TYPE_SMALL ||
    type === dicts.PRODUCT_TYPE_TWO_ASSIST ||
    type === dicts.PRODUCT_TYPE_LECTURE
  ) {
    statusSecondary = `（上课/应到：${comeCount}/${shouldComeCount}）`;
  }
  let infos = [];
  infos = [
    ...infos,
    {
      label: '课程状态',
      content: dicts.classStatusDict.find((d) => d.val === status).text,
      secondaryContent: statusSecondary,
    },
    { label: '所属学习中心', content: campusName },
  ];
  infos.push({
    label: type === dicts.PRODUCT_TYPE_TWO || type === dicts.PRODUCT_TYPE_TWO_ASSIST ? '主讲老师' : '授课老师',
    content: teacherName,
  });
  if (type === dicts.PRODUCT_TYPE_TWO_ASSIST) {
    infos.push({ label: '辅导老师', content: assistTeacherName });
  }
  if (type === dicts.PRODUCT_TYPE_ONE || type === dicts.PRODUCT_TYPE_OTM) {
    infos.push({ label: '上课学生', content: classStudents });
  }
  infos = [
    ...infos,
    { label: '年级', content: gradeName },
    { label: '科目', content: subjectName },
    { label: '上课日期', content: date },
    { label: '上课时间', content: time },
  ];
  if (
    type === dicts.PRODUCT_TYPE_SMALL ||
    type === dicts.PRODUCT_TYPE_TWO ||
    type === dicts.PRODUCT_TYPE_TWO_ASSIST ||
    type === dicts.PRODUCT_TYPE_LECTURE
  ) {
    infos.push({ label: '教室/卡座', content: classroomName });
  }
  return infos;
};

export default {
  loadOwnerDict,
  loadList,
  loadDetail,
};
