import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import { Row, Col } from 'antd';
import MultiSelect from 'COMPONENT/MultiSelect';
import moment from 'moment';
import { branch, campus } from 'MODEL/dict/utils/config';
import { getRelationsDictPlus, getRelationPayloadPlus } from 'MODEL/dict/utils/index';
import Button from 'COMPONENT/Button';
import Select from 'COMPONENT/Select';
import { exportSyncFailedRecordExcelApi, getLastUpdateTimeApi } from './loaders';
import styles from './index.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const startYear = moment().subtract(6, 'days').year();
const startMonth = moment().subtract(6, 'days').month() + 1;
const startDay = moment().subtract(6, 'days').date();
const endYear = moment().add(6, 'days').year();
const endMonth = moment().add(6, 'days').month() + 1;
const endDay = moment().add(6, 'days').date();

const formatTimestamp = (timestamp) => {
  return moment(timestamp).format('YYYY-MM-DD HH:mm:ss');
};

class ClassinFailedSyncRecordDownload extends React.Component {
  constructor(props) {
    super(props);
    this.updateBranch = this.updateBranch.bind(this);
    this.state = {
      loading: false,

      branchId: null,
      campusId: null,

      branchIds: [],
      campusIds: null,
      oneToOneLastUpdateTime: 0,
      miniClassLastUpdateTime: 0,
      oneToManyLastUpdateTime: 0,
      twoTeacherLastUpdateTime: 0,
    };

    // 分公司关联关系type、key对应关系配置
    this.branchTreeMap = {
      [branch]: 'branchId',
      [campus]: 'campusId',
    };
  }

  componentDidMount() {
    this.init();
  }

  init = async () => {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    const updateTime = await getLastUpdateTimeApi();
    const { oneToOneLastUpdateTime, miniClassLastUpdateTime, oneToManyLastUpdateTime, twoTeacherLastUpdateTime } =
      updateTime;
    this.initDefault();
    this.setState({
      loading: false,
      oneToOneLastUpdateTime,
      miniClassLastUpdateTime,
      oneToManyLastUpdateTime,
      twoTeacherLastUpdateTime,
    });
  };

  initDefault = () => {
    const { branchDict } = this.props;
    let branchId;
    let campusId;
    const { brenchId: userBranchId, campusId: userCampusId, orgType } = Index.getLoginUser();

    if (orgType === 'CAMPUS') {
      branchId = [userBranchId];
      campusId = [userCampusId];
      if (userBranchId === userCampusId) {
        campusId = null;
      }
    } else if (orgType === 'BRENCH') {
      const branchMap = branchDict.find((branch) => branch.val === userBranchId);
      branchId = [userBranchId];
      campusId = branchMap ? branchMap.campus.map((campus) => campus.val) : [];
    } else {
      //集团
      branchId = branchDict.map((branch) => branch.val);
      campusId = branchDict.reduce((arr, cur) => {
        if (Array.isArray(cur.campus)) {
          return arr.concat(cur.campus.map((campus) => campus.val));
        }
      }, []);
    }
    this.setState({ branchId, campusId });
  };

  initDict = (resolve) => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branchCampusTree'],
        callback() {
          resolve();
        },
      },
    });
  };

  updateBranch(val) {
    let payload = getRelationPayloadPlus('branchId', val, {
      type: branch,
      dictTree: this.props.branchCampusTree,
      state: this.state,
      map: this.branchTreeMap,
    });
    this.setState({
      ...payload,
      campusId: null,
    });
  }

  downloadExcel = async () => {
    const { branchId, campusId } = this.state;
    const currentTime = new Date().getTime();
    let lastClickTime = window.sessionStorage ? Number(window.sessionStorage.getItem('CLASSIN_SESSION_TIME')) : null;
    if (lastClickTime && currentTime - lastClickTime < 5 * 60 * 1000) {
      showMessage('刷新太频繁啦，系统限制刷新时间间隔为5分钟', 'error');
      return false;
    }
    if (!branchId || !campusId) {
      return showMessage('请先选择分公司及学习中心后再下载', 'error');
    }
    this.setState({ loading: true });
    await exportSyncFailedRecordExcelApi({ campusIds: campusId });
    this.props.dispatch({ type: 'main/exportNotice' });
    window.sessionStorage.setItem('CLASSIN_SESSION_TIME', String(currentTime));
    this.setState({ loading: false });
  };

  render() {
    const {
      loading,
      branchId,
      campusId,
      oneToOneLastUpdateTime,
      oneToManyLastUpdateTime,
      miniClassLastUpdateTime,
      twoTeacherLastUpdateTime,
    } = this.state;

    const branchCampusDict = getRelationsDictPlus(campus)(this.props.branchCampusTree, branchId);

    return (
      <Page title="classin同步失败记录下载" loading={loading}>
        <div style={{ color: '#f00' }}>
          {`注：本次下载的数据是上课日期在${startYear}年${startMonth}月${startDay}日-${endYear}年${endMonth}月${endDay}日之间，boss课程信息与classin不一致的课程数据`}
        </div>
        <Row className={styles.row}>
          <Col span={3} className={styles.col}>
            所属分公司:
          </Col>
          <Col span={6}>
            <MultiSelect showSearch value={branchId} dict={this.props.branchDict} onChange={this.updateBranch} />
          </Col>
        </Row>
        <Row className={styles.row}>
          <Col span={3} className={styles.col}>
            所属学习中心:
          </Col>
          <Col span={6}>
            <MultiSelect
              showSearch
              value={campusId}
              dict={branchCampusDict}
              onChange={(campusId) => this.setState({ campusId })}
            />
          </Col>
        </Row>
        <Row className={styles.row}>
          <Col span={2}></Col>
          <Col span={6}>
            <Button onClick={this.downloadExcel}>点击下载</Button>
          </Col>
        </Row>
        <hr className={styles.hr} />
        <Row>{oneToOneLastUpdateTime && `1对1课程：最近同步到classin  ${formatTimestamp(oneToOneLastUpdateTime)}`}</Row>
        <Row>
          {oneToManyLastUpdateTime && `1对多课程：最近同步到classin  ${formatTimestamp(oneToManyLastUpdateTime)}`}
        </Row>
        <Row>
          {miniClassLastUpdateTime && `小班课程：最近同步到classin  ${formatTimestamp(miniClassLastUpdateTime)}`}
        </Row>
        <Row>
          {twoTeacherLastUpdateTime && `双师课程：最近同步到classin  ${formatTimestamp(twoTeacherLastUpdateTime)}`}
        </Row>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const branchCampusTree = state.dict.branchCampusTree;
  return {
    // 字典
    branchDict: state.dict.branchCampusTree,
    branchCampusTree: state.dict.branchCampusTree,
  };
};

export default connect(mapStateToProps)(ClassinFailedSyncRecordDownload);
