/**
 * 编辑面板组件
 * @property {string} autoName 模板名称自动生成部分
 * @property {string} customName 模板名称后缀
 * @property {number} classCount 讲数
 * @property {string} year 产品年份
 * @property {string} seasonName 季度
 * @property {[string]} weekDay 上课星期
 * @property {string} phase 上课期数
 * @property {string} startTime 开始时间
 * @property {[{val, text}]} weekDict 上课星期字典
 * @property {[{val, text}]} phaseDict 上课期数字典
 * @property {string} seasonType 季节类型
 * @property {function(name: string, val: any)} onDataChange 模板字段改变时回调
 * @property {function} onDelete 删除模板回调
 * @property {function} onCancel 取消选中模板回调
 * @property {function} onSave 保存模板回调
 * @property {boolean} isNew 是否是添加模板
 */
import React from 'react';
import ReactDOM from 'react-dom';
import { Icon, Row, Col, Popconfirm } from 'antd';
import Popover from 'COMPONENT/Popover';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import InputNumber from 'COMPONENT/InputNumber';
import MultiSelect from 'COMPONENT/MultiSelect';
import TimePicker from 'COMPONENT/TimePicker';
import { TIME_FORMAT, momentToTime, timeToMoment } from 'COMPONENT/utils/form';
import styles from './styles.less';
import FormItem from '../FormItem';

/**
 * 在antd的Button组件基础上添加气泡信息提示
 * @property {any} ...props antd的Button组件所有属性
 * @property {string?} trigger 触发类型
 * @property {string|ReactNode?} content  信息内容
 * @property {function?} onClose 气泡关闭事件触发时的回调
 */
const ButtonWithPop = ({ trigger, icon, content, onClose, ...props }) => {
  let btn = <Button {...props} />;
  if (content) {
    if (trigger === 'click') {
      btn = (
        <Popover icon={icon} content={content} maxWidth={180} trigger="click" visible={true} onClose={onClose}>
          {btn}
        </Popover>
      );
    } else {
      btn = (
        <Popover icon={icon} content={content} maxWidth={180}>
          {btn}
        </Popover>
      );
    }
  }
  return btn;
};

class EditPanel extends React.Component {
  state = {
    $popupContainer: document, //承载弹出层的元素
  };

  componentDidMount() {
    this.setState({ $popupContainer: this.$wrap });
  }

  /**
   * 开始时间禁用的小时（禁用23:00-05:59）
   * @return {number[]}
   */
  startTimeDisabledHours = () => {
    return [23, 0, 1, 2, 3, 4, 5];
  };

  render() {
    const {
      autoName,
      customName,
      classCount,
      year,
      seasonName,
      weekDay,
      phase,
      dates,
      weekDict,
      phaseDict,
      seasonType,
      startTime,
      onDataChange,
      onDelete,
      onCancel,
      onSave,
      isNew,
      isDirty,
      saveBtnContent,
      saveBtnTrigger,
      saveBtnIcon,
      onSaveBtnPopClose,
      deleteBtnContent,
      deleteBtnTrigger,
      deleteBtnIcon,
      onDeleteBtnPopClose,
      id,
    } = this.props;
    const { $popupContainer } = this.state;
    return (
      <div className={styles.wrap} ref={(c) => (this.$wrap = ReactDOM.findDOMNode(c))}>
        <Row gutter={15}>
          <Col span={12}>
            <FormItem name="模板名称">
              <Col span={14}>
                <Input disabled value={autoName} title={autoName} />
              </Col>
              <Col span={2} style={{ textAlign: 'center' }}>
                ~
              </Col>
              <Col span={8}>
                <Input value={customName} onChange={(e) => onDataChange('customName', e.target.value)} />
              </Col>
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem disabled name="模板ID">
              <Input disabled value={id} />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="产品年份">
              <Input disabled value={year} />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="季度">
              <Input disabled value={seasonName} />
            </FormItem>
          </Col>
          {seasonType === 'hanOrShu' && (
            <Col span={12}>
              <FormItem required name="上课期数">
                <Select
                  dict={phaseDict}
                  value={phase}
                  onChange={(val) => onDataChange('phase', val)}
                  getPopupContainer={() => this.$wrap}
                />
              </FormItem>
            </Col>
          )}
          {seasonType === 'chunOrQiu' && (
            <Col span={12}>
              <FormItem required name="上课星期">
                <MultiSelect
                  dict={weekDict}
                  value={weekDay}
                  onChange={(val) => onDataChange('weekDay', val || [])}
                  getPopupContainer={() => this.$wrap}
                />
              </FormItem>
            </Col>
          )}
          <Col span={12}>
            <FormItem required name={`讲数（已排讲数：${dates.length}）`}>
              <InputNumber min={0} value={classCount} onChange={(val) => onDataChange('classCount', val)} />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem colon={false} name=" ">
              <div>（请在校历选择上课日期段）</div>
            </FormItem>
          </Col>
          <Col span={12}>
            {/* <FormItem name="开始时间">
              <TimePicker
                value={startTime}
                onChange={val => onDataChange('startTime', val)}
                disabledHours={this.startTimeDisabledHours}
                getPopupContainer={() => $popupContainer}
                disabled={true}
              />
            </FormItem> */}
          </Col>
        </Row>
        <div className={styles.footer}>
          {!isNew && (
            <ButtonWithPop
              onClick={onDelete}
              trigger={deleteBtnTrigger}
              icon={deleteBtnIcon}
              content={deleteBtnContent}
              onClose={onDeleteBtnPopClose}
            >
              删除
            </ButtonWithPop>
          )}
          <div className={styles.footerRight}>
            <span>
              {isDirty ? (
                <Popconfirm
                  title={<div style={{ maxWidth: 180 }}>取消保存，当前已修改的模板内容将不被保存</div>}
                  onConfirm={onSave}
                  onCancel={onCancel}
                  okText="保存"
                  cancelText="不保存"
                >
                  <Button>取消</Button>
                </Popconfirm>
              ) : (
                <Button onClick={onCancel}>取消</Button>
              )}
              <ButtonWithPop
                type="primary"
                style={{ marginLeft: 16 }}
                onClick={onSave}
                trigger={saveBtnTrigger}
                icon={saveBtnIcon}
                content={saveBtnContent}
                onClose={onSaveBtnPopClose}
              >
                保存
              </ButtonWithPop>
            </span>
          </div>
        </div>
      </div>
    );
  }
}

export default EditPanel;
