/**
 * 模板列表组件
 * @property {string} year 年份
 * @property {string} seasonId 季节ID
 * @property {string} seasonName 季节名称
 * @property {[object]} data 模板列表
 * @property {string} activeTemplateId 激活状态的模板ID
 * @property {object} activeTemplateInitData 激活状态模板修改之前的初始数据
 * @property {[object]} yearDict 年份字典
 * @property {[object]} weekDict 星期字典
 * @property {[object]} phaseDict 期数字典
 * @property {function} onAdd 添加模板回调
 * @property {function(id: string)} onSelect 选中模板回调
 * @property {function(name: string, val: any)} onDataChange 模板字段改变时回调
 * @property {function} onDelete 删除模板回调
 * @property {function} onCancel 取消选中模板回调
 * @property {function} onSave 保存模板回调
 * @property {boolean} isDirty 当前模板数据是否有改动
 */
import React from 'react';
import { Icon, Row, Col } from 'antd';
import getWindowSize from 'UTIL/getWindowSize';
import { getDateRange, getSeasonType, getDictText, getTemplateAutoName, getTemplateNameDate } from '../common';
import EditPanel from '../EditPanel';
import styles from './styles.less';

class TemplateList extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      winW: 0, //  number  窗口宽度
    };
  }

  componentDidMount() {
    this.updateWinW();
    window.addEventListener('resize', this.updateWinW, false);
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.updateWinW, false);
  }

  /**
   * 获取和更新窗口宽度状态
   */
  updateWinW = () => {
    this.setState({ winW: getWindowSize().width });
  };

  /**
   * 获取模板名称自动生成部分
   * @param  {object} data 模板数据
   * @return {string}
   */
  getAutoName = (data) => {
    const { year, seasonId, seasonName, phaseDict, weekDict } = this.props;
    const { weekDay, phase, startTime } = data;
    return getTemplateAutoName({ year, seasonId, seasonName, phaseDict, weekDict, weekDay, phase, startTime });
  };

  render() {
    const {
      year,
      seasonId,
      seasonName,
      data,
      activeTemplateId,
      activeTemplateInitData,
      yearDict,
      weekDict,
      phaseDict,
      onAdd,
      onSelect,
      onDataChange,
      onDelete,
      onCancel,
      onSave,
      isDirty,
      ...props
    } = this.props;
    const { winW } = this.state;

    //模板列表的列数，窗口宽度小于1366时为1列，否则2列
    const colCount = winW < 1366 ? 1 : 2;

    //非当前季节添加模板状态时，数据末尾加上一个代表新增按钮的项'newBtn'
    let dataWithNewBtn = [...data, 'newBtn'];
    if (activeTemplateId === 'new') {
      const { seasonId: activeSeasonId } = activeTemplateInitData;
      if (activeSeasonId === seasonId) {
        dataWithNewBtn = [...data];
      }
    }
    //数据按行分组
    let rows = [];
    let cols = [];
    dataWithNewBtn.forEach((t, i) => {
      if (cols.length > 0 && i % colCount === 0) {
        rows.push(cols);
        cols = [];
      }
      cols.push(t);
    });
    if (cols.length > 0) {
      rows.push(cols);
    }

    //生成模板按钮组件列表，激活状态模板按钮所在行下方添加编辑面板
    let rowComps = [];
    rows.forEach((cols, i) => {
      let activeTemplate = null;
      rowComps.push(
        <Row key={i} gutter={15} className={styles.row}>
          {cols.map((col, j) => {
            let isActive = false;
            let btn;
            if (col === 'newBtn') {
              btn = (
                <div className={styles.btn + ' ' + styles.add} onClick={onAdd}>
                  <Icon type="plus" style={{ marginRight: 4, fontSize: 12 }} />
                  <span>添加</span>
                </div>
              );
            } else {
              const { id, customName, classCount, weekDay, phase, dates } = col;
              isActive = activeTemplateId === id;
              if (isActive) {
                activeTemplate = col;
              }
              const fullName = this.getAutoName(col) + (customName || '') + getTemplateNameDate(col.dates);
              btn = (
                <div className={styles.btnParent}>
                  <div className={styles.templateId}>模板ID: {id}</div>
                  <div
                    className={
                      styles.btn +
                      (isActive ? ' ' + styles.active : '') +
                      (isActive && isDirty ? ' ' + styles.dirty : '')
                    }
                    onClick={() => onSelect(id)}
                    title={fullName}
                  >
                    <div className="text-ellipsis">{fullName}</div>
                  </div>
                </div>
              );
            }
            return (
              <Col key={j} span={24 / colCount}>
                {btn}
                {isActive && <div className={styles.arrow}></div>}
              </Col>
            );
          })}
        </Row>,
      );
      if (activeTemplate) {
        const { id, classCount, customName, weekDay, phase, dates, startTime } = activeTemplate;
        rowComps.push(
          <div key="panelRow" className={styles.panelRow}>
            <EditPanel
              autoName={this.getAutoName(activeTemplate)}
              customName={customName}
              classCount={classCount}
              year={year}
              seasonName={seasonName}
              weekDay={weekDay}
              phase={phase}
              dates={dates}
              startTime={startTime}
              weekDict={weekDict}
              phaseDict={phaseDict}
              seasonType={getSeasonType(seasonId)}
              onDataChange={onDataChange}
              onDelete={onDelete}
              onCancel={onCancel}
              onSave={onSave}
              isNew={id === 'new'}
              id={id}
              isDirty={isDirty}
              {...props}
            />
          </div>,
        );
      }
    });

    return (
      <div className={styles.wrap}>
        <div className={styles.title}>{`${seasonName}（${year}）`}</div>
        <div className={styles.content}>{rowComps}</div>
      </div>
    );
  }
}
export default TemplateList;
