import moment from 'moment';
import { DATE_FORMAT } from 'COMPONENT/utils/form';

/**
 * 获取日期范围
 * @param  {[string]} dates 日期数组
 * @return {{start: string, end: string}} 开始日期和结束日期组成的对象
 */
const getDateRange = (dates) => {
  if (!dates || dates.length === 0) {
    return { start: null, end: null };
  }
  const mDates = dates.map((d) => moment(d));
  return { start: moment.min(mDates).format(DATE_FORMAT), end: moment.max(mDates).format(DATE_FORMAT) };
};

/**
 * 获取季节类型，hanOrShu：寒暑季  chunOrQiu：春秋季
 * @param  {string} seasonId 季节ID
 * @return {string|null}
 */
const getSeasonType = (seasonId) => {
  if (seasonId === 'DAT0000000230' || seasonId === 'DAT0000000232') {
    return 'hanOrShu';
  } else if (seasonId === 'DAT0000000231' || seasonId === 'DAT0000000233') {
    return 'chunOrQiu';
  }
  return null;
};

/**
 * 获取字典项名称
 * @param  {{val: string, text: string}} dict 字典
 * @param  {string} val 值
 * @return {string|null}
 */
const getDictText = (dict, val) => {
  if (!dict) {
    return null;
  }
  const option = dict.find((d) => d.val === val);
  if (option) {
    return option.text;
  }
  return null;
};

/**
 * 获取模板名称自动生成部分
 * 1、秋季和春季，年份+季度+上课星期
 * 2、寒假和暑假，年份+季度+上课期数
 * @param  {string} year 年份
 * @param  {string} seasonId 季度ID
 * @param  {string} seasonName 季度名称
 * @param  {[object]} phaseDict 期字典
 * @param  {[object]} weekDict 星期字典
 * @param  {[string]} weekDay 星期
 * @param  {string} phase 期
 * @param  {string} startTime 开始时间
 * @return {string}
 */
const getTemplateAutoName = ({ year, seasonId, seasonName, phaseDict, weekDict, weekDay, phase, startTime }) => {
  const seasonType = getSeasonType(seasonId);
  let name = year + seasonName;
  let phaseOrWeekName;
  if (seasonType === 'hanOrShu') {
    phaseOrWeekName = getDictText(phaseDict, phase);
  } else if (seasonType === 'chunOrQiu') {
    if (weekDay) {
      const weekNames = weekDay.map((w, i) => {
        let name = getDictText(weekDict, w);
        if (i !== 0) {
          name = name.replace('星期', '');
        }
        return name;
      });
      phaseOrWeekName = weekNames.join('/');
    }
  }
  if (phaseOrWeekName) {
    name += phaseOrWeekName;
  }
  name += startTime || '';
  return name;
};

/**
 * 获取模板名称日期段部分
 * @param  {[string]} dates 日期
 * @return {string}
 */
const getTemplateNameDate = (dates) => {
  const { start: startDate, end: endDate } = getDateRange(dates);
  let name = '';
  if (startDate) {
    name += '（' + startDate;
  }
  if (endDate) {
    name += ' ~ ' + endDate;
  }
  if (startDate || endDate) {
    name += '）';
  }
  return name;
};

export default {
  getDateRange,
  getSeasonType,
  getDictText,
  getTemplateAutoName,
  getTemplateNameDate,
};
