import React from 'react';
import ReactDOM from 'react-dom';
import moment from 'moment';
import { Button, Icon } from 'antd';
import Page from 'COMPONENT/Page';
import Select from 'COMPONENT/Select';
import Modal from 'COMPONENT/Modal';
import SearchBar from 'COMPONENT/SearchBar';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import { loadCommonDict, loadBranchDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import { DATE_FORMAT } from 'COMPONENT/utils/form';
import { getSelectOption } from 'SERVICE/common';
import {
  brenchIncludeOtherCampus,
  getCourseModalList,
  deleteCourseModal,
  saveCourseModal,
  getClassDataByModal,
} from 'SERVICE/course';
import { getDateRange, getSeasonType, getDictText, getTemplateAutoName, getTemplateNameDate } from './common';
import MonthCalendar from './MonthCalendar';
import TemplateList from './TemplateList';
import styles from './styles.less';
import ModalUpdateClass from './ModalUpdateClass';

//默认年份
const defaultYearText = moment().year() + '';

class CourseTemplateManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // boolean 页面加载状态
      branchId: null, // string 查询条件，分公司ID
      year: null, // string 查询条件，年份
      currentDataParams: null, // object 查询出当前数据的参数
      currentDataYear: null, // string 当前数据的年份
      calendarExpand: false, // boolean 日历是否展开，默认显示12个月，展开显示15个月
      templates: [], // [object] 课程模板
      activeTemplateId: null, // string 当前激活状态的模板ID，新增时为'new'
      activeTemplateInitData: null, // object 当前激活状态模板修改之前的初始数据
      branchDict: [], // [{val, text}] 分公司字典
      yearDict: [], // [{val, text}] 年份字典
      seasonDict: [], // [{val, text}] 季节字典
      weekDict: [], // [{val, text}] 星期字典
      phaseDict: [], // [{val, text}] 期数字典
      saveBtnContent: null, // string 保存按钮提示信息内容
      saveBtnTrigger: 'click', // string 保存按钮提示信息触发方式 [ hover | click ]
      saveBtnIcon: 'error', // string 保存按钮提示信息图标类型 [ info | warning | error ]
      deleteBtnContent: null, // string 删除按钮提示信息内容
      deleteBtnTrigger: 'click', // string 删除按钮提示信息触发方式 [ hover | click ]
      deleteBtnIcon: 'error', // string 删除按钮提示信息图标类型 [ info | warning | error ]
      // 更新班级弹窗（课程模板下有课程关联时需要）
      toggleModalUpdateClass: false,
    };
    this._calendars = {};
    this.updateClassData = null;
    this.smallClassNum = 0;
    this.twoTeacherNum = 0;
    this.switchModalUpdateClass = this.switchModalUpdateClass.bind(this);
    this.checkBeforeSave = this.checkBeforeSave.bind(this);
    this.saveTemplate = this.saveTemplate.bind(this);
    this.submit = this.submit.bind(this);
  }

  componentDidMount() {
    this.initContent();
  }

  componentWillUnmount() {
    this.isUnmounted = true;
  }

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      if (!this.isUnmounted) {
        this.setState(newState, () => {
          resolve();
        });
      } else {
        resolve();
      }
    });
  };

  /**
   * 初始化内容
   */
  initContent = async () => {
    await this.updateState({ loading: true });
    const [yearDict, seasonDict, weekDict, phaseDict, branchDict] = await Promise.all([
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('WeekDay'),
      loadCommonDict('SMALL_CLASS_PHASE'),
      loadBranchDict(),
    ]);
    await this.updateState({ yearDict, seasonDict, weekDict, phaseDict, branchDict });
    await this.resetSearchKeys();
    this.loadTemplates();
  };

  /**
   * 查询
   */
  search = () => {
    const { branchId, year } = this.state;
    if (!branchId || !year) {
      return;
    }
    this.loadTemplates({ branchId, year }).then(() => {
      this.updateState({ activeTemplateId: null, activeTemplateInitData: null, calendarExpand: false });
      this._$leftSecContent.scrollTop = 0;
      this._$rightSecContent.scrollTop = 0;
    });
  };

  /**
   * 重置查询条件
   */
  resetSearchKeys = async () => {
    const { branchDict, yearDict } = this.state;
    const defaultYearOpt = yearDict.find((y) => y.text === defaultYearText);
    await this.updateState({
      branchId: branchDict[0].val,
      year: defaultYearOpt ? defaultYearOpt.val : null,
    });
    return true;
  };

  /**
   * 加载模板数据，并转换格式和更新状态，格式如下：
      [{
        id: '1',  // string 模板ID
        seasonId: 'DAT0000000231',  // string 季节ID
        customName: '名称后缀',  // string 模板名称附加信息
        classCount: 3,  // number 讲数
        weekDay: ['MON', 'FRI'],  //[string] 星期 春秋季有值
        phase: 'DAT0000000417',  // string 期数 寒暑季有值
        dates: ['2017-02-27', '2017-03-06', '2017-03-13'],  // [string] 上课日期
        startTime: '09:00',  // string 开始时间
      }]
   * @param  {Object} changedParams 改变的参数
   */
  loadTemplates = async (changedParams = {}) => {
    const { branchId, year, currentDataParams } = this.state;
    let params = currentDataParams || { branchId, year };
    params = { ...params, ...changedParams };
    await this.updateState({ loading: true });
    /*
[
   {
        "id": 10013,
        "modalName": "第2个模版我来搞23",
        "techNum": 111,
        "productYearId": "DAT0000000392",
        "productSeasonId": "DAT0000000230",
        "courseWeek": [
            "MON",
            "TUE",
            "WED",
            "THU"
        ],
        "coursePhaseId": "DAT0000000417",
        "branchId": "ORG0000000224",
        "productYearName": "2017",
        "productSeasonName": "寒假",
        "courseWeekName": null,
        "coursePhaseName": "第一期",
        "branchName": "长沙分公司",
        "version": 2,
        "courseDate": [
            "2017-06-01",
            "2017-06-02",
            "2017-07-01",
            "2017-08-01",
            "2017-09-01"
        ]
    }
]
 */
    const apiParams = {
      branchId: params.branchId,
      productYearId: params.year,
    };
    const apiData = await getCourseModalList(apiParams);
    const templates = apiData.map((t) => {
      const { id, suffixName, techNum, productSeasonId, courseWeek, coursePhaseId, courseDate, version, classTime } = t;
      return {
        id,
        seasonId: productSeasonId,
        customName: suffixName,
        classCount: techNum,
        weekDay: courseWeek,
        phase: coursePhaseId,
        dates: courseDate,
        startTime: classTime || null,
        version,
      };
    });
    await this.updateState({
      loading: false,
      currentDataYear: params.year,
      templates,
      currentDataParams: { branchId: params.branchId, year: params.year },
    });
    return templates;
  };

  /**
   * 检验是否可以选日期，异常情况弹框提示：没有选中模板、没有输入讲数
   * @return {boolean}
   */
  checkSelectDate = () => {
    const { templates, activeTemplateId } = this.state;
    if (!activeTemplateId) {
      showAlert('请先选择模板');
      return false;
    }
    const { classCount } = templates.find((t) => t.id === activeTemplateId);
    if (!classCount) {
      showAlert('请先输入讲数');
      return false;
    }
    return true;
  };

  /**
   * 选中日期，当前模板选中或取消选中日期
   * @param  {string} date 日期，格式为：YYYY-MM-DD
   */
  selectDate = (date) => {
    if (!this.checkSelectDate()) {
      return;
    }
    const { templates, activeTemplateId } = this.state;
    this.updateState({
      templates: templates.map((t) => {
        const { id, dates } = t;
        if (id === activeTemplateId) {
          let newDates;
          if (dates.indexOf(date) >= 0) {
            newDates = dates.filter((d) => d !== date);
          } else {
            newDates = [...dates, date];
          }
          return { ...t, dates: newDates };
        }
        return t;
      }),
    });
  };

  /**
   * 选中星期，当前模板选中特定星期的的日期，选中数目等于剩余讲数
   * @param  {number} year   年份
   * @param  {number} month   月份，0~11
   * @param  {number} weekDay 星期，0~6（星期日~星期六）
   */
  selectWeekDay = (year, month, weekDay) => {
    if (!this.checkSelectDate()) {
      return;
    }
    const { templates, activeTemplateId } = this.state;
    const { classCount, dates } = templates.find((t) => t.id === activeTemplateId);
    let availableCount = classCount;
    if (dates) {
      availableCount -= dates.length;
    }
    if (availableCount > 0) {
      let monthFormat = month + 1;
      monthFormat = monthFormat < 10 ? '0' + monthFormat : monthFormat;
      let baseDateMoment = moment(year + '-' + monthFormat + '-01');
      while (baseDateMoment.day() !== weekDay) {
        baseDateMoment.add(1, 'd');
      }
      let addDates = [];
      while (addDates.length < availableCount) {
        const mStr = baseDateMoment.format(DATE_FORMAT);
        if (dates.indexOf(mStr) < 0) {
          addDates.push(mStr);
        }
        baseDateMoment.add(7, 'd');
      }
      this.updateState({
        templates: templates.map((t) => {
          const { id, dates } = t;
          if (id === activeTemplateId) {
            return { ...t, dates: [...dates, ...addDates] };
          }
          return t;
        }),
      });
    }
  };

  /**
   * 检查是否有修改的模板未保存，并弹框提示
   * @return {boolean}
   */
  checkDirty = () => {
    const { activeTemplateInitData, templates, activeTemplateId } = this.state;
    if (activeTemplateId) {
      const activeTemplate = templates.find((t) => t.id === activeTemplateId);
      if (!objectDeepEquals(activeTemplateInitData, activeTemplate)) {
        return true;
      }
    }
    return false;
  };

  /**
   * 日历定位到月份
   * @param  {number} year 年份
   * @param  {number} month 月份
   */
  scrollToMonth = (year, month) => {
    let h = 0;
    Object.keys(this._calendars).forEach((key) => {
      const arry = key.split('-');
      const y = parseInt(arry[0]);
      const m = parseInt(arry[1]);
      const ref = this._calendars[key];
      if (ref && (y < year || (y === year && m < month))) {
        const $c = ReactDOM.findDOMNode(ref);
        h += $c.offsetHeight + 15;
      }
    });
    this._$leftSecContent.scrollTop = h;
  };

  /**
   * 添加模板
   * @param  {string} seasonId 季节ID
   */
  addTemplate = (seasonId) => {
    const { templates, currentDataYear } = this.state;
    if (this.checkDirty()) {
      showAlert('请保存正在编辑的模板，再添加模板');
      return;
    }
    const templatesWithoutNew = templates.filter((t) => t.id !== 'new');
    const newTemplate = {
      id: 'new',
      seasonId,
      customName: null,
      classCount: null,
      weekDay: [],
      phase: null,
      dates: [],
    };
    let changedState = {
      templates: [...templatesWithoutNew, newTemplate],
      activeTemplateId: 'new',
      activeTemplateInitData: { ...newTemplate },
    };
    let startMonth = 0;
    if (seasonId === 'DAT0000000231') {
      startMonth = 1;
    } else if (seasonId === 'DAT0000000232') {
      startMonth = 5;
    } else if (seasonId === 'DAT0000000233') {
      startMonth = 8;
      changedState = { ...changedState, calendarExpand: true };
    }
    this.updateState({ ...changedState });
    this.scrollToMonth(this.getYearNumberByValue(currentDataYear), startMonth);
  };

  /**
   * 选中模板
   * @param  {string} id 模板ID
   */
  selectTemplate = (id) => {
    const { templates, activeTemplateId } = this.state;
    if (id === activeTemplateId) {
      return;
    }
    if (this.checkDirty()) {
      showAlert('请保存正在编辑的模板，再选择其他模板');
      return;
    }
    const templatesWithoutNew = templates.filter((t) => t.id !== 'new');
    const activeTemplate = templates.find((t) => t.id === id);
    let changedState = {
      templates: templatesWithoutNew,
      activeTemplateId: id,
      activeTemplateInitData: { ...activeTemplate },
    };
    const { dates, seasonId } = activeTemplate;
    const startDate = getDateRange(dates).start;
    const startDateMoment = moment(startDate);
    const startYear = startDateMoment.year();
    const startMonth = startDateMoment.month();
    if (seasonId === 'DAT0000000233') {
      changedState = { ...changedState, calendarExpand: true };
    }
    this.updateState({ ...changedState }).then(() => {
      this.scrollToMonth(startYear, startMonth);
    });
  };

  /**
   * 处理模板数据改变
   * @param  {string} name 字段名
   * @param  {string} val 值
   */
  templateDataChange = (name, val) => {
    val = val === undefined || val === '' ? null : val;
    const { templates, activeTemplateId } = this.state;
    this.updateState({
      templates: templates.map((t) => {
        if (t.id === activeTemplateId) {
          return { ...t, [name]: val };
        }
        return t;
      }),
    });
  };

  /**
   * 删除模板
   */
  deleteTemplate = async () => {
    const { activeTemplateId } = this.state;
    const error = (text) => this.updateState({ deleteBtnContent: text });
    await this.updateState({ loading: true });
    try {
      //await getSelectOption({ xx: 11}, true)
      await deleteCourseModal({ modalId: activeTemplateId }, true);
      showMessage('删除成功', 'success');
      this.loadTemplates().then(() => {
        this.updateState({ activeTemplateId: null, activeTemplateInitData: null });
      });
    } catch (e) {
      if (e.text) {
        error(e.text);
      }
      showMessage('删除失败');
      this.updateState({ loading: false });
    }
  };

  /**
   * 取消选中模板
   */
  cancelSelectTemplate = () => {
    const { templates, activeTemplateId, activeTemplateInitData } = this.state;
    this.updateState({
      templates: templates
        .filter((t) => t.id !== 'new')
        .map((t) => {
          if (t.id === activeTemplateId) {
            return { ...activeTemplateInitData };
          }
          return t;
        }),
      activeTemplateId: null,
      activeTemplateInitData: null,
    });
  };

  /**
   * 根据年份值获取数值
   * @param  {string} yearVal 年份在字典项中的值
   * @return {number}
   */
  getYearNumberByValue = (yearVal) => {
    const { yearDict } = this.state;
    const year = yearDict.find((y) => y.val === yearVal);
    let yearNumber = null;
    if (year) {
      yearNumber = parseInt(year.text);
    }
    return yearNumber;
  };

  // 切换更新班级弹窗显示
  switchModalUpdateClass() {
    this.setState(
      (prevState) => ({
        toggleModalUpdateClass: !prevState.toggleModalUpdateClass,
      }),
      () => {
        if (!this.state.toggleModalUpdateClass) {
          this.smallClassNum = 0;
          this.twoTeacherNum = 0;
        }
      },
    );
  }
  // 保存模板
  async saveTemplate() {
    let check = this.checkBeforeSave();
    if (!check) {
      return;
    }
    if (this.state.activeTemplateId === 'new') {
      // 新增时，直接保存
      this.submit();
    } else {
      // 修改
      this.setState({ loading: true });
      // 调接口根据返回的数据判断当前模板是否有关联班级
      const rsp = await getClassDataByModal({ modalId: this.state.activeTemplateId }).catch((err) => {
        showMessage(err.text, 'error');
      });
      if (rsp) {
        if (rsp.smallClassCount > 0 || rsp.twoTeacherClassCount > 0) {
          // 有班级关联，弹出更新班级弹窗
          this.smallClassNum = rsp.smallClassCount;
          this.twoTeacherNum = rsp.twoTeacherClassCount;
          this.setState({ loading: false });
          this.switchModalUpdateClass();
        } else {
          // 没有班级关联，直接保存
          this.submit();
        }
      }
    }
  }
  // 提交前校验数据
  checkBeforeSave() {
    const { activeTemplateId, templates, currentDataYear, seasonDict, phaseDict, weekDict } = this.state;
    const { seasonId, customName, classCount, weekDay, phase, dates, startTime } = templates.find(
      (t) => t.id === activeTemplateId,
    );
    const seasonType = getSeasonType(seasonId);
    const dateRange = getDateRange(dates);
    if (!classCount) {
      this.setState({ saveBtnContent: '请填写讲数' });
      return false;
    }
    if (seasonType === 'hanOrShu' && !phase) {
      this.setState({ saveBtnContent: '请选择上课期数' });
      return false;
    }
    if (seasonType === 'chunOrQiu' && !weekDay) {
      this.setState({ saveBtnContent: '请选择上课星期' });
      return false;
    }
    if (dates.length === 0) {
      this.setState({ saveBtnContent: '请在校历上选择上课日期段' });
      return false;
    }
    const hasDuplicateName = templates
      .filter((t) => t.id !== activeTemplateId && t.seasonId === seasonId)
      .find((t) => {
        const range = getDateRange(t.dates);
        return (
          objectDeepEquals(t.weekDay, weekDay) &&
          t.phase === phase &&
          range.start === dateRange.start &&
          range.end === dateRange.end &&
          t.startTime === startTime &&
          t.customName === customName
        );
      });
    if (hasDuplicateName) {
      this.setState({ saveBtnContent: '本季度已有同名的模板名称，请在可选填中添加后缀区分' });
      return false;
    }
    if (dates.length > classCount) {
      this.setState({ saveBtnContent: '已排讲数超出计划讲数，请删减已排讲数' });
      return false;
    }
    if (dates.length < classCount) {
      this.setState({ saveBtnContent: '已排讲数少于计划讲数，请继续排课' });
      return false;
    }

    // 检验课程模板名称是否重复
    // const modalNameTemp = getTemplateAutoName({
    //   year: this.getYearNumberByValue(currentDataYear),
    //   seasonId,
    //   seasonName: seasonDict.find(s => s.val === seasonId).text,
    //   phaseDict,
    //   weekDict,
    //   weekDay,
    //   phase,
    //   startTime,
    // }) + (customName || '')
    // const oldTemplates = templates.filter(t => t.id !== activeTemplateId)
    // const repeatTemplate = oldTemplates.find(t => {
    //   const modalName = getTemplateAutoName({
    //     year: this.getYearNumberByValue(currentDataYear),
    //     seasonId: t.seasonId,
    //     seasonName: seasonDict.find(s => s.val === t.seasonId).text,
    //     phaseDict,
    //     weekDict,
    //     weekDay: t.weekDay,
    //     phase: t.phase,
    //     startTime: t.startTime
    //   }) + (t.customName || '')
    //   return modalName.trim() === modalNameTemp.trim()
    // })
    // if(repeatTemplate !== undefined){
    //   this.setState({ saveBtnContent: '模板名称在同个年份、分公司、季度、期数下必须唯一，不允许重复!' });
    //   return false;
    // }

    return true;
  }
  // 提交
  async submit(params) {
    let payload = { loading: true };
    if (this.state.toggleModalUpdateClass) payload.toggleModalUpdateClass = false;
    this.setState(payload);
    const {
      activeTemplateId,
      templates,
      currentDataParams: { branchId },
      currentDataYear,
      seasonDict,
      phaseDict,
      weekDict,
    } = this.state;
    const { id, seasonId, customName, classCount, weekDay, phase, dates, startTime, version } = templates.find(
      (t) => t.id === activeTemplateId,
    );
    const modalName =
      getTemplateAutoName({
        year: this.getYearNumberByValue(currentDataYear),
        seasonId,
        seasonName: seasonDict.find((s) => s.val === seasonId).text,
        phaseDict,
        weekDict,
        weekDay,
        phase,
        startTime,
      }) +
      (customName || '') +
      getTemplateNameDate(dates);
    // 拼参数
    let apiParams = {
      CourseModalVo: {
        branchId,
        courseDate: dates,
        coursePhaseId: phase,
        courseWeek: weekDay || [],
        modalName,
        productSeasonId: seasonId,
        productYearId: currentDataYear,
        techNum: classCount,
        suffixName: customName,
        classTime: startTime,
      },
      updateSmallClass: false,
      updateTwoTeacherClass: false,
    };
    if (activeTemplateId === 'new') {
      apiParams.CourseModalVo.version = 0;
    } else {
      apiParams.CourseModalVo.version = version;
      apiParams.CourseModalVo.id = id;
    }
    if (params) {
      // 需要关联班级
      if (params.smallClass) apiParams.updateSmallClass = true;
      if (params.twoTeacher) apiParams.updateTwoTeacherClass = true;
    }
    console.log('apiParams: ', apiParams);
    // 调用接口
    const rsp = await saveCourseModal(apiParams, true).catch((err) => {
      showMessage(err.text || '保存失败');
      this.setState({ loading: false });
    });
    if (rsp && rsp.resultCode === 0) {
      showMessage('保存成功', 'success');
      // 刷新课程模板列表
      const dateRange = getDateRange(dates);
      this.loadTemplates().then((newTemplates) => {
        let newActiveTemplateId = null;
        let newActiveTemplateInitData = null;
        newTemplates.forEach((t) => {
          if (t.seasonId === seasonId) {
            const range = getDateRange(t.dates);
            if (
              objectDeepEquals(t.weekDay, weekDay) &&
              t.phase === phase &&
              range.start === dateRange.start &&
              range.end === dateRange.end &&
              t.customName === customName
            ) {
              newActiveTemplateId = t.id;
              newActiveTemplateInitData = t;
            }
          }
        });
        this.updateState({ activeTemplateId: newActiveTemplateId, activeTemplateInitData: newActiveTemplateInitData });
        this.cancelSelectTemplate();
      });
    }
  }

  render() {
    const {
      loading,
      branchId,
      year,
      currentDataYear,
      calendarExpand,
      templates,
      activeTemplateId,
      activeTemplateInitData,
      branchDict,
      yearDict,
      seasonDict,
      weekDict,
      phaseDict,
      saveBtnContent,
      saveBtnTrigger,
      saveBtnIcon,
      cancelBtnContent,
      cancelBtnTrigger,
      cancelBtnIcon,
      deleteBtnContent,
      deleteBtnTrigger,
      deleteBtnIcon,
      toggleModalUpdateClass,
    } = this.state;
    const activeTemplate = templates.find((t) => t.id === activeTemplateId);
    let activeSeasonId;
    if (activeTemplate) {
      activeSeasonId = activeTemplate.seasonId;
    }

    let currentDataYearNumber;
    const calendars = [];
    if (currentDataYear) {
      currentDataYearNumber = this.getYearNumberByValue(currentDataYear);
      let selectedDates = [];
      let extraSelectedDates = [];
      if (activeTemplate) {
        const { dates, classCount } = activeTemplate;
        const maxSelectDates = classCount || 0;
        dates.forEach((d, i) => {
          if (i < classCount) {
            selectedDates.push(d);
          } else {
            extraSelectedDates.push(d);
          }
        });
      }
      for (let i = 0; i < 18; i++) {
        if (calendarExpand || i < 12) {
          const year = Math.floor(currentDataYearNumber + i / 12);
          const month = i % 12;
          calendars.push(
            <MonthCalendar
              key={i}
              year={year}
              month={month}
              selectedDates={selectedDates}
              extraSelectedDates={extraSelectedDates}
              onSelectDate={this.selectDate}
              onSelectWeekDay={(weekDay) => this.selectWeekDay(year, month, weekDay)}
              ref={(c) => (this._calendars[year + '-' + month] = c)}
            />,
          );
        }
      }
    }

    return (
      <Page fitWindowHeight={true} loading={loading} route={this.props.route}>
        <SearchBar onSearch={this.search} onReset={this.resetSearchKeys}>
          <Select
            placeholder="选择分公司"
            allowClear={false}
            dict={branchDict}
            value={branchId}
            onChange={(branchId) => this.updateState({ branchId })}
          />
          <Select
            placeholder="选择年份"
            allowClear={false}
            dict={yearDict}
            value={year}
            onChange={(year) => this.updateState({ year })}
          />
        </SearchBar>
        <div className={styles.section + ' ' + styles.sectionLeft}>
          <div className={styles.secTitle}>校历</div>
          <div className={styles.colorBar}>
            {/*<span className={styles.dot + ' ' + styles.yellow}></span>
            <span>假日排课</span>*/}
            <span className={styles.dot + ' ' + styles.green}></span>
            <span>正常排课</span>
            <span className={styles.dot + ' ' + styles.red}></span>
            <span>超额排课</span>
          </div>
          <div className={styles.secContent} ref={(c) => (this._$leftSecContent = ReactDOM.findDOMNode(c))}>
            {calendars}
            {currentDataYear && !calendarExpand && (
              <div className={styles.loadMoreBar}>
                <Button
                  size="large"
                  onClick={() => {
                    this.updateState({ calendarExpand: true });
                  }}
                >
                  加载更多日历
                </Button>
              </div>
            )}
          </div>
        </div>
        <div className={styles.section + ' ' + styles.sectionRight}>
          <div className={styles.secTitle}>课程模板设置</div>
          <div className={styles.secContent} ref={(c) => (this._$rightSecContent = ReactDOM.findDOMNode(c))}>
            {currentDataYear &&
              seasonDict
                .filter((d) => d.val && d.val !== 'DAT0000000245')
                .map((d, i) => {
                  return (
                    <TemplateList
                      key={i}
                      year={currentDataYearNumber + ''}
                      data={templates.filter((t) => t.seasonId === d.val)}
                      seasonId={d.val}
                      seasonName={d.text}
                      activeTemplateId={activeTemplateId}
                      activeTemplateInitData={activeTemplateInitData}
                      yearDict={yearDict}
                      seasonDict={seasonDict}
                      weekDict={weekDict}
                      phaseDict={phaseDict}
                      onAdd={() => this.addTemplate(d.val)}
                      onSelect={(id) => this.selectTemplate(id)}
                      onDataChange={this.templateDataChange}
                      onDelete={this.deleteTemplate}
                      onCancel={this.cancelSelectTemplate}
                      onSave={this.saveTemplate}
                      isDirty={this.checkDirty()}
                      saveBtnContent={saveBtnContent}
                      saveBtnTrigger={saveBtnTrigger}
                      saveBtnIcon={saveBtnIcon}
                      onSaveBtnPopClose={() => this.updateState({ saveBtnContent: null })}
                      deleteBtnContent={deleteBtnContent}
                      deleteBtnTrigger={deleteBtnTrigger}
                      deleteBtnIcon={deleteBtnIcon}
                      onDeleteBtnPopClose={() => this.updateState({ deleteBtnContent: null })}
                    />
                  );
                })}
          </div>
        </div>
        {toggleModalUpdateClass && (
          <ModalUpdateClass
            smallClassNum={this.smallClassNum}
            twoTeacherNum={this.twoTeacherNum}
            onConfirm={this.submit}
            onCancel={this.switchModalUpdateClass}
          />
        )}
      </Page>
    );
  }
}

export default CourseTemplateManage;
