/**
 * 精英班履行详情页面
 * @param {string} promiseStudentId 精英班学生ID
 * @param {string} contractProductId 合同产品ID
 * @param {number} totalCourseHours 总课时数
 * @param {boolean} canPlanCourse 是否可以跳转到课程规划详情
 */
import React from 'react';
import { connect } from 'dva';
import { browserHistory } from 'dva/router';
import Page from 'COMPONENT/Page';
import Block from 'COMPONENT/Block';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import DatePicker from 'COMPONENT/DatePicker';
import { getDictTextByVal, loadCommonDict } from 'UTIL/dict';
import { dateToMoment } from 'COMPONENT/utils/form';
import { getSelectOption } from 'SERVICE/common';
import {
  getEcsContractInfo,
  getEcsContractChargeInfo,
  findStudentInfoByPromiseStudentId,
  getEcsContractChargeList,
  getPromiseSubjectDetailList,
} from 'SERVICE/course';
import StudentInfo from './StudentInfo';
import ContractInfo from './ContractInfo';
import CourseHoursInfo from './CourseHoursInfo';
import FeeItem from './FeeItem';
import { formatMoney } from './common';
import styles from './styles.less';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

/**
 * 考勤状态字典
 */
const attendanceDict = [
  { val: 'NEW', text: '未上课' },
  { val: 'CONPELETE', text: '准时上课' },
  { val: 'LEAVE', text: '请假' },
  { val: 'ABSENT', text: '缺勤' },
  { val: 'LATE', text: '迟到' },
];

class PromiseClassDetail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      studentInfo: {}, // {object} 学生信息
      contractInfo: {}, // {object} 合同信息
      courseHoursInfo: {}, // {object} 课时信息
      searchExpand: false, // {boolean} 搜索栏展开状态
      startDate: null, // {string} 查询条件 开始时间
      endDate: null, // {string} 查询条件 结束时间
      quarter: null, // {string} 查询条件 季度
      miniClassName: null, // {string} 查询条件 小班名称
      attendance: null, // {string} 查询条件 考勤状态
      quarterDict: null, // {[object]} 字典 季度
      statusDict: null, // {[object]} 字典 学生精英班状态
      list: [], // {Array} 列表数据
      listLoading: false, // {false} 列表数据加载状态
      listSearchKeys: null, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 10, // {number} 每页记录数
      error: null, // {string} 数据加载失败的错误信息
    };
    const { location, params } = props;
    console.log(props, 'props');
    const mirrorId = location.query.mirrorId;
    if (mirrorId) {
      this.params = params.mirrors[mirrorId];
    } else {
      this.params = params;
    }
  }

  componentWillMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    const { promiseStudentId, contractProductId, totalCourseHours } = this.params;
    this.setState({ loading: true });
    Promise.all([
      this.loadStudentInfo(promiseStudentId),
      this.loadContractInfo(contractProductId),
      this.loadCourseHoursInfo(promiseStudentId, totalCourseHours),
      this.loadFeeDetails(contractProductId),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('StudentPromiseStatus'),
    ]).then(
      ([studentInfo, contractInfo, courseHoursInfo, feeDetails, quarterDict, statusDict]) => {
        const { promiseStudentStatus } = studentInfo;
        const { quarterFeeDetails } = feeDetails;
        const deductedVipMoney = this.compute(contractInfo.actualMoney, quarterFeeDetails).vipTotal;
        this.setState({
          loading: false,
          studentInfo: { ...studentInfo, promiseStudentStatusName: getDictTextByVal(statusDict, promiseStudentStatus) },
          contractInfo: { ...contractInfo, deductedVipMoney },
          courseHoursInfo,
          quarterDict,
          statusDict,
        });
      },
      (e) => {
        const error = e.text || '数据加载出错';
        this.setState({ loading: false, error });
      },
    );
    this.loadList();
  };

  /**
   * 加载学生信息，并转换格式，如下：
     {
      name: string  学生姓名
      cusName: string  客户姓名
      gradeName: string  学生年级
      contact: string  客户电话
      promiseStudentStatus: string  学生精英班状态
      promiseClassName: string  精英班名称
      signCourseName: string  签约课程
     }
   * @param  {string} promiseStudentId 精英班学生ID
   */
  loadStudentInfo = async (promiseStudentId) => {
    const apiData = await findStudentInfoByPromiseStudentId({ promiseStudentId }, true);
    const { studentName, customerContact, customerName, gradeName, courseStatus, promiseClassName, productName } =
      apiData;
    const data = {
      name: studentName,
      cusName: customerName,
      gradeName: gradeName,
      contact: customerContact,
      promiseStudentStatus: courseStatus,
      promiseClassName,
      signCourseName: productName,
    };
    /*
    const data = await new Promise(resolve => {
      setTimeout(() => {
        const fake = {
          name: '吴思', cusName: '吴伊琪', gradeName: '初三', contact: '18812345678', promiseStudentStatus: '1',
          promiseClassName: '2017初三精英班', signCourseName: '2017年暑假中考精英班',
        }
        resolve(fake)
      }, 500)
    })
*/
    return data;
  };

  /**
   * 加载合同信息，并转换格式，如下：
     {
      no: string  合同编号
      actualMoney: number  实收总额
      feeItems: Array  收费项目
        [{
            name: string  名称
            money: number  总金额
            discount: number  优惠金额
        }]
      deductedBaseMoney: number  已扣课时费
      }
   * @param  {string} contractProductId 合同产品ID
   */
  loadContractInfo = async (contractProductId) => {
    /*
0 courseAmount  number  true  课时费
0 courseProAmount number  true  课时优惠费
0 realPaidAmount  number  true  实收总额
0 vipAmount number  true  vip服务费
0 realPayAmount number  true  课时应收总额
0 vipProAmount  number  true  vip服务优惠费
0 courseName  string  true  签约课程名称
0 vipRealPayAmount  string  true  vip应收总额
0 consumeAmount string  true  已扣课时费
0 vipConsumeAmount  string  true  vip已扣课时费
 */

    const apiData = await getEcsContractInfo({ contractProductId }, true);
    const {
      contractId,
      courseAmount,
      courseProAmount,
      realPaidAmount,
      vipAmount,
      vipProAmount,
      courseName,
      consumeAmount,
      vipConsumeAmount,
      vipRealPayAmount,
      realPayAmount,
    } = apiData;
    const data = {
      no: contractId,
      actualMoney: realPaidAmount,
      feeItems: [
        { name: 'VIP服务费', money: vipRealPayAmount + vipProAmount, discount: vipProAmount },
        { name: courseName, money: realPayAmount + courseProAmount, discount: courseProAmount },
      ],
      deductedBaseMoney: consumeAmount,
    };

    /*
    const data = await new Promise(resolve => {
      setTimeout(() => {
        const fake = {
          no: 'CON0000301166',
          actualMoney: 35000,
          feeItems: [
            { name: 'VIP服务费', money: 8000, discount: 0 },
            { name: '2017初三精英班', money: 20000, discount: 1000 },
            { name: '2017年暑假中考精英班', money: 8000, discount: 0 },
          ],
          deductedBaseMoney: 2100,
        }
        resolve(fake)
      }, 500)
    })
*/
    return data;
  };

  /**
   * 加载课时分配信息，并转换格式，如下：
     {
      total: number  总课时数
      distributed: number  已分配课时数
      unDistributed: number  未分配课时数
      entered: number  已报小班课时数
      consumed: number  已上课时数
     }
   * @param  {string} promiseStudentId 精英班学生ID
   * @param  {number} totalCourseHours 总课时数
   */
  loadCourseHoursInfo = async (promiseStudentId, totalCourseHours) => {
    const apiData = await getPromiseSubjectDetailList({ promiseStudentId }, true);
    let distributed = 0;
    let entered = 0;
    let consumed = 0;
    apiData.forEach((quarter) => {
      const { quarterId, hasMiniClassNatureHour = 0, consumeNatureHour = 0, allNatureHour = 0 } = quarter;
      distributed += allNatureHour;
      entered += hasMiniClassNatureHour;
      consumed += consumeNatureHour;
    });
    const data = {
      total: totalCourseHours,
      distributed,
      unDistributed: totalCourseHours - distributed,
      entered,
      consumed,
    };
    /*
    const data = await new Promise(resolve => {
      setTimeout(() => {
        const fake = { total: 360, distributed: 360, unDistributed: 0, entered: 180, consumed: 21 }
        resolve(fake)
      }, 500)
    })
*/
    return data;
  };

  /**
   * 加载扣费明细，并转换格式，如下：
     {
      isBindVip: boolean  是否绑定VIP服务费
      quarterFeeDetails: Array  季度扣费明细
      [{
        id: string  季度ID
        base: object  基础扣费详情
        {
          regular: number  正价金额
          discount: number  优惠金额
          regularHours: number  正价课时数
          discountHours: number  优惠课时数
        }
        vip: object  VIP服务费详情
        {
          regular: number  正价金额
          discount: number  优惠金额
        }
     }]
     }
   * @param  {string} contractProductId 合同产品ID
   */
  loadFeeDetails = async (contractProductId) => {
    const apiData = await getEcsContractChargeInfo({ contractProductId }, true);
    const { chargeInfo, result } = apiData;
    const defaultQuarter = {
      base: { regular: 0, discount: 0, regularHours: 0, discountHours: 0 },
      vip: { regular: 0, discount: 0 },
    };
    let summer = { ...defaultQuarter, id: 'DAT0000000232' };
    let autumn = { ...defaultQuarter, id: 'DAT0000000233' };
    let winter = { ...defaultQuarter, id: 'DAT0000000230' };
    let spring = { ...defaultQuarter, id: 'DAT0000000231' };
    chargeInfo.forEach((f) => {
      const { realAmount, promotionQuantity, realQuantity, seasonId, promotionAmount, realVipAmount, proVipAmount } = f;
      const fee = {
        regular: realAmount,
        discount: promotionAmount,
        regularHours: realQuantity,
        discountHours: promotionQuantity,
      };
      const vipFee = {
        regular: realVipAmount,
        discount: proVipAmount,
      };
      if (seasonId === summer.id) {
        summer.base = fee;
        summer.vip = vipFee;
      } else if (seasonId === autumn.id) {
        autumn.base = fee;
        autumn.vip = vipFee;
      } else if (seasonId === winter.id) {
        winter.base = fee;
        winter.vip = vipFee;
      } else if (seasonId === spring.id) {
        spring.base = fee;
        spring.vip = vipFee;
      }
    });
    const data = {
      isBindVip: result,
      quarterFeeDetails: [summer, autumn, winter, spring],
    };
    return data;
  };

  /**
   * 计算金额
   * @param  {number} actualMoney    合同实收总额
   * @param  {object} quarterFeeDetails 各季度扣费明细
   * @return {object} 计算结果集合，字段如下：
      {
        baseRegularTotal: number 基础项目正价扣费合计
        baseDiscountTotal: number 基础项目优惠扣费合计
        baseRegularHoursTotal: number 基础项目正价课时数合计
        baseDiscountHoursTotal: number 基础项目优惠课时数合计
        vipRegularTotal: number VIP服务费项目正价扣费合计
        vipDiscountTotal: number VIP服务费项目优惠扣费合计
        baseTotal: number 基础项目扣费总合计
        baseHoursTotal: number 基础项目课时数总合计
        vipTotal: number VIP服务费项目扣费合计
        total: number 扣费总合计
        refundMoney: number 退费总额
      }
   */
  compute = (actualMoney, quarterFeeDetails) => {
    let baseRegularTotal = 0;
    let baseDiscountTotal = 0;
    let vipRegularTotal = 0;
    let vipDiscountTotal = 0;
    let baseRegularHoursTotal = 0;
    let baseDiscountHoursTotal = 0;
    quarterFeeDetails.forEach((quarterFeeDetail) => {
      const { base, vip } = quarterFeeDetail;
      baseRegularTotal += base.regular;
      baseDiscountTotal += base.discount;
      baseRegularHoursTotal += base.regularHours;
      baseDiscountHoursTotal += base.discountHours;
      vipRegularTotal += vip.regular;
      vipDiscountTotal += vip.discount;
    });
    const baseTotal = baseRegularTotal + baseDiscountTotal;
    const baseHoursTotal = baseRegularHoursTotal + baseDiscountHoursTotal;
    const vipTotal = vipRegularTotal + vipDiscountTotal;
    const total = baseTotal + vipTotal;
    let refundMoney = actualMoney - total;
    refundMoney = refundMoney < 0 ? 0 : refundMoney;
    return {
      baseRegularTotal,
      baseDiscountTotal,
      baseRegularHoursTotal,
      baseDiscountHoursTotal,
      vipRegularTotal,
      vipDiscountTotal,
      baseTotal,
      baseHoursTotal,
      vipTotal,
      total,
      refundMoney,
    };
  };

  /**
   * 加载列表数据，并转换格式
   * @param  {object} changedParams 要更新的列表数据参数，包括查询条件、页码状态
   */
  loadList = async (changedParams = {}) => {
    const { contractProductId } = this.params;
    const { startDate, endDate, quarter, miniClassName, attendance, currentPage, pageSize, listSearchKeys } =
      this.state;
    /*const defaultQueryStrings = {
      sidx: 'id',
      sord: 'desc',
    }*/
    const searchKeys = listSearchKeys || { startDate, endDate, quarter, miniClassName, attendance };
    const params = { currentPage, pageSize, ...searchKeys, ...changedParams };
    /*
    contractProductId true  请求URL string    合同产品ID
    startDate true  请求URL string    开始时间
    endDate true  请求URL string    结束时间
    season  true  请求URL string    季节
    miniClassName true  请求URL string    小班名称
    attendanceStatus  true  请求URL string    考勤状态
     */
    const apiParams = {
      page: params.currentPage,
      rows: params.pageSize,
      contractProductId,
      startDate: params.startDate,
      endDate: params.endDate,
      season: params.quarter,
      miniClassName: params.miniClassName,
      attendanceStatus: params.attendance,
    };
    this.setState({ listLoading: true });
    /*
    0 attendentStatus ? string:undefined ?  true  考勤状态
    0 realAmount  number  true  实收扣费金额
    0 courseNatureHour number  true  课时
    0 realHours number  true  实收扣费课时
    0 teacherName ? string:undefined ?  true  老师
    0 payTime string  true  扣费时间
    0 courseDate  string  true  课程日期
    0 promiseName ? string:undefined ?  true  精英班名字
    0 studyManagerName  ? string:undefined ?  true  学管名字
    0 proAmount number  true  优惠扣费金额
    0 proHours  number  true  优惠扣费课时
    0 courseTime  string  true  上课时间
     */

    const apiData = await getEcsContractChargeList(apiParams);
    const { records, rows } = apiData;
    const formatedRows = rows.map((row) => {
      const {
        attendentStatus,
        realAmount,
        courseHours,
        courseNatureHour = 0,
        realHours,
        teacherName,
        payTime,
        courseDate,
        promiseName,
        studyManagerName,
        proAmount,
        proHours,
        courseTime,
        price,
        proNatureHour,
        realNatureHour,
      } = row;

      return {
        miniClassName: promiseName,
        classDate: courseDate,
        classTime: courseTime,
        courseHours,
        courseNatureHour,
        teacherName,
        headMasterName: studyManagerName,
        attendance: attendentStatus,
        baseFee: price || 0,
        baseCount: realNatureHour || 0,
        discountFee: price || 0,
        discountCount: proNatureHour || 0,
        moneyDateTime: payTime,
      };
    });
    const data = {
      records,
      rows: formatedRows,
    };

    /*
    const data = await new Promise(resolve => {
      setTimeout(() => {
        const fake = {
          records: 12,
          rows: [
            { miniClassName: '小班1', classDate: '2017-05-02', classTime: '10:00', courseHours: 3, teacherName: '晓管理', headMasterName: '大管理',
              attendance: 'CONPELETE', baseFee: 100, baseCount: 2, discountFee: 100, discountCount: 1, moneyDateTime: '2017-05-02 18:00',
            },
          ],
        }
        resolve(fake)
      }, 1000)
    })
*/
    this.setState({
      listLoading: false,
      list: data.rows,
      totalRecords: data.records,
      currentPage: params.currentPage,
      pageSize: params.pageSize,
      listSearchKeys: searchKeys,
    });
    return true;
  };

  /**
   * 页码改变时重新加载列表数据
   * @param  {object} pagination 页码信息
   */
  paginationChange = (pagination) => {
    console.log(pagination);
    const { current: currentPage, pageSize } = pagination;
    this.loadList({ currentPage, pageSize });
  };

  /**
   * 查询
   */
  search = () => {
    const { startDate, endDate, quarter, miniClassName, attendance } = this.state;
    this.loadList({ startDate, endDate, quarter, miniClassName, attendance, currentPage: 1 });
  };

  /**
   * 重置查询条件
   */
  resetSearchKeys = () => {
    this.setState({
      startDate: null,
      endDate: null,
      quarter: null,
      miniClassName: null,
      attendance: null,
    });
  };

  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate = (startDate) => {
    const { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return dateToMoment(startDate).valueOf() > dateToMoment(endDate).valueOf();
  };

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return dateToMoment(endDate).valueOf() <= dateToMoment(startDate).valueOf();
  };

  /**
   * 打开标签页
   * @param  {string} path 路由
   * @param  {object} params 传入页面的参数
   */
  openTab = (path, params) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: { path, params, refresh: true },
    });
  };

  render() {
    const { promiseStudentId, canPlanCourse } = this.params;
    const {
      loading,
      studentInfo,
      contractInfo,
      courseHoursInfo,
      searchExpand,
      startDate,
      endDate,
      quarter,
      miniClassName,
      attendance,
      quarterDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
      listLoading,
      error,
    } = this.state;
    let deducted = 0;
    let consumed = 0;
    list.forEach((row) => {
      const { baseFee, baseCount, discountFee, discountCount } = row;
      deducted += baseFee * baseCount + discountFee * discountCount;
      consumed += baseCount + discountCount;
    });
    const columns = [
      { title: '小班名称', dataIndex: 'miniClassName' },
      { title: '上课日期', dataIndex: 'classDate' },
      { title: '上课时间', dataIndex: 'classTime' },
      { title: '小时', dataIndex: 'courseNatureHour', width: COL_WIDTH_S },
      { title: '老师', dataIndex: 'teacherName' },
      { title: '班主任', dataIndex: 'headMasterName' },
      { title: '考勤状态', dataIndex: 'attendance', dict: attendanceDict },
      // { title: '扣费金额', width: COL_WIDTH_XL, render: (text, record) => {
      //   const { baseFee, baseCount, discountFee, discountCount } = record
      //   return (
      //     <div className={styles.listFeeCell}>
      //       <div className={styles.listFeeItem}>
      //         <FeeItem name="实收" money={baseFee} count={baseCount} />
      //       </div>
      //       <div className={styles.listFeeItem}>
      //         <FeeItem name="优惠" money={discountFee} count={discountCount} type="discount" />
      //       </div>
      //     </div>
      //   )
      // } },
      // { title: '扣费时间', dataIndex: 'moneyDateTime', width: COL_WIDTH_L },
    ];
    return (
      <Page loading={loading} route={this.props.route} error={error}>
        <Block type="dark" title="学生信息">
          <StudentInfo data={studentInfo} />
        </Block>
        <Block type="dark" title="合同信息">
          <ContractInfo data={contractInfo} />
        </Block>
        <Block type="dark" title="课程信息">
          <CourseHoursInfo
            data={courseHoursInfo}
            canPlanCourse={canPlanCourse}
            onViewDetail={() => this.openTab('/PromiseClassPlan', { promiseStudentId })}
          />
        </Block>
        <Block type="dark" title="扣费明细">
          <SearchBar
            expand={searchExpand}
            onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
            onSearch={this.search}
            onReset={this.resetSearchKeys}
          >
            <DatePicker
              placeholder="开始时间"
              disabledDate={this.getDisabledStartDate}
              value={startDate}
              onChange={(val) => {
                this.setState({ startDate: val });
              }}
            />
            <DatePicker
              placeholder="结束时间"
              disabledDate={this.getDisabledEndDate}
              value={endDate}
              onChange={(val) => {
                this.setState({ endDate: val });
              }}
            />
            <Select
              placeholder="季度"
              dict={quarterDict}
              value={quarter}
              onChange={(val) => {
                this.setState({ quarter: val });
              }}
            />
            <Input
              placeholder="小班名称"
              value={miniClassName}
              onChange={(e) => {
                this.setState({ miniClassName: e.target.value });
              }}
            />
            <Select
              placeholder="考勤状态"
              dict={attendanceDict}
              value={attendance}
              onChange={(val) => {
                this.setState({ attendance: val });
              }}
            />
          </SearchBar>
          <div className={styles.summaryBar}>
            {/* <span className={styles.summaryItem}>{`已扣款金额：${formatMoney(deducted)}`}</span> */}
            <span className={styles.summaryItem}>{`已上小时数：${consumed}`}</span>
          </div>
          <div className={styles.detailList}>
            <List
              dataSource={list}
              columns={columns}
              loading={!loading && listLoading}
              pagination={{
                current: currentPage,
                total: totalRecords,
                pageSize,
              }}
              onChange={this.paginationChange}
            />
          </div>
        </Block>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/PromiseClassDetail'],
  };
};

export default connect(mapStateToProps)(PromiseClassDetail);
