import React from 'react';
import { connect } from 'dva';
import { Icon, Row, Col } from 'antd';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import Popover from 'COMPONENT/Popover';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadCommonDict, loadBranchDict, loadCampusDict } from 'UTIL/dict';
import courseApis from 'SERVICE/course';
import PromiseFeeDetailModal from '../modals/PromiseFeeDetail';
import PromiseFeeDetailOAModal from '../modals/PromiseFeeDetailOA';
import PromiseResultDetailModal from '../modals/PromiseResultDetail';
import PromiseResultDetailOAModal from '../modals/PromiseResultDetailOA';
import ParentConfirmModal from '../../approveManage/modals/AuditListModal'; // AuditListModal
import OfflineConfirmModal from '../../approveManage/modals/OnlineOffConfirm';
import CancelFlowModal from '../../approveManage/modals/CancelFlow'; // 撤销流程
import styles from './styles.less';
import { isPrdEnv } from 'CONST/config';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

/**
 * 审批状态字典
 */
const approvalStatusDict = [
  { val: 'UNAUDIT', text: '未审批' },
  { val: 'ACCOUNTANT_RAUDIT', text: '审批中' },
  { val: 'AUDIT_FAILED', text: '驳回' },
  { val: 'RAUDIT', text: '审批完成' },
];

// 确认方式
const confirmChannelDict = [
  { val: 'ONLINE', text: '家长线上确认' },
  { val: 'OFFLINE', text: '老师线下确认' },
];

// 确认状态
const confirmDict = [
  { val: '0', text: '无需确认' },
  { val: '1', text: '未确认' },
  { val: '2', text: '确认中' },
  { val: '3', text: '已确认' },
];

/**
 * 目标结果字典
 */
const resultDict = [
  { val: '0', text: '失败' },
  { val: '1', text: '成功' },
];

/**
 * 季度名简称
 */
const simpleQuarterNames = {
  DAT0000000232: '暑',
  DAT0000000233: '秋',
  DAT0000000230: '寒',
  DAT0000000231: '春',
};

/**
 * 规划完成状态单元格内容
 * @param  {object} record 内容数据
 * @return {ReactNode}
 */
const PlanCompleteCell = ({ record }) => {
  const { paidStatus, coursePlanInfo, status, abortClass } = record;
  if (!coursePlanInfo) {
    return null;
  }
  let colsInCell = [];
  let rowsInPop = [];
  let infoRed;
  coursePlanInfo.forEach((quarter, i) => {
    const { quarterId, distributed, signed, consumed, hasPlan } = quarter;
    const quarterName = simpleQuarterNames[quarterId];
    let dotStyle;
    if (!hasPlan) {
      dotStyle = styles.dotGray;
    } else if (distributed && signed) {
      dotStyle = styles.dotGreen;
      if (consumed === signed) {
        dotStyle = styles.dotGray;
      }
    } else if (!distributed || !signed) {
      dotStyle = styles.dotRed;
      infoRed = true;
    }
    colsInCell.push(
      <div key={i} className={styles.planCompleteItem + ' ' + styles.dot + ' ' + dotStyle}>
        {quarterName}
      </div>,
    );
    rowsInPop.push(
      <Row key={i} gutter={10} className={styles.popListRow}>
        <Col span={3} className={styles.dot + ' ' + dotStyle}>
          {quarterName}
        </Col>
        <Col span={7}>
          <span className={styles.label}>已分配</span>
          <span style={{ color: hasPlan && distributed === 0 ? 'red' : null }}>{distributed}</span>
        </Col>
        <Col span={7}>
          <span className={styles.label}>已报读</span>
          <span style={{ color: hasPlan && signed === 0 ? 'red' : null }}>{signed}</span>
        </Col>
        <Col span={7}>
          <span className={styles.label}>已上课</span>
          <span style={{ color: hasPlan && consumed === 0 ? 'red' : null }}>{consumed}</span>
        </Col>
      </Row>,
    );
  });
  return (
    <div className={styles.planCompleteCell}>
      {colsInCell}
      {status !== '3' && abortClass !== '1' && (
        <div className={styles.planCompleteItem}>
          <Popover
            trigger="click"
            placement="right"
            title={paidStatus === 'PAID' ? '全部缴费' : '部分缴费'}
            content={<div className={styles.popContent}>{rowsInPop}</div>}
          >
            <Icon
              type="info-circle"
              style={{ cursor: 'pointer', color: infoRed ? 'red' : '#108ee9' }}
              onClick={(e) => e.stopPropagation()}
            />
          </Popover>
        </div>
      )}
    </div>
  );
};

const PAGE_ID = '/CoursePromiseClassManage';

class PromiseClassManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态
      year: null, // {string} 查询条件 年份
      promiseClassName: null, // {string} 查询条件 精英班名称
      courseName: null, // {string} 查询条件 签约课程名称
      branch: null, // {string} 查询条件 分公司
      campus: null, // {string} 查询条件 学习中心
      studentName: null, // {string} 查询条件 学生姓名
      grade: null, // {string} 查询条件 年级
      status: ['1', '2', '4'], // {[string]} 查询条件 学生精英班状态
      result: null, // {string} 查询条件 目标结果
      confirmStatus: null, // 确认状态
      confirmChannel: null, // 确认方式
      approvalStatus: null, // {string} 查询条件 审批状态
      quitClassStatus: '0', // {string} 查询条件 是否退班
      yearDict: null, // {[object]} 字典 年份
      branchDict: null, // {[object]} 字典 分公司
      campusDict: null, // {[object]} 字典 学习中心
      defaultBranch: null, // {string} 默认查询条件 分公司
      defaultCampus: null, // {string} 默认查询条件 学习中心
      defaultStatus: ['1', '2', '4'], // {[string]} 默认查询条件 学生精英班状态
      defaultCampusDict: null, // {[object]} 默认字典 学习中心
      gradeDict: null, // {[object]} 字典 年级
      statusDict: null, // {[object]} 字典 学生精英班状态
      list: [], // {Array} 列表数据
      listLoading: false, // {false} 列表数据加载状态
      listSearchKeys: null, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
      feeModalVisible: false, // {boolean} 退费详情弹框显示状态
      feeOAModalVisible: false, // {boolean} OA退费详情弹框显示状态
      feeModalType: 1, // {number} 退费详情弹框显示类型
      currentRecord: null, // {object} 当前激活状态的列表行数

      resultModalVisible: false, //{boolean} 目标结果详情弹框显示状态
      resultOAModalVisible: false, //{boolean} OA目标结果详情弹框显示状态
      parentConfirmModalVisible: false, // 家长线上确认弹窗显示状态
      offlineConfirmModalVisible: false, // 线下确认弹窗显示状态
      resultModalType: 1, // {number} 目标结果详情弹框显示类型
      resultRecord: null, // {object} 当前激活状态的目标结果项
    };

    this.serverYear = new Date(props.user.timeStamp).getFullYear();
  }

  componentDidMount() {
    this.initContent();
    GlobalEvent.on('CrmNotification:resolveEcsClass', this.onResolveTodo);
  }

  componentWillUnmount() {
    this.isUnmounted = true;
    this.props.dispatch({
      type: 'main/deletePageParams',
      payload: { id: PAGE_ID },
    });
    GlobalEvent.off('CrmNotification:resolveEcsClass', this.onResolveTodo);
  }

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      if (!this.isUnmounted) {
        this.setState(newState, () => {
          resolve();
        });
      } else {
        resolve();
      }
    });
  };

  /**
   * 初始化内容
   */
  initContent = async () => {
    await this.updateState({ loading: true });
    const [yearDict, branchDict, gradeDict, statusDict] = await Promise.all([
      loadCommonDict('PRODUCT_VERSION'),
      loadBranchDict(),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('StudentPromiseStatus'),
    ]);

    let changedState = { loading: false, yearDict, branchDict, gradeDict, statusDict };
    if (branchDict.length === 1) {
      const branch = branchDict[0].val;
      changedState = { ...changedState, branch, defaultBranch: branch };
      const campusDict = await loadCampusDict(branch);
      changedState = { ...changedState, campusDict, defaultCampusDict: campusDict };
      if (campusDict.length === 1) {
        const campus = campusDict[0].val;
        changedState = { ...changedState, campus, defaultCampus: campus };
      }
    }

    if (this.props.pageParams && this.props.pageParams.isTodo) {
      changedState.approvalStatus = this.props.pageParams.todoParams.auditStatus;
      changedState.status = [];
    }
    await this.updateState({ ...changedState });
    this.search();
  };

  /**
   * 处理分公司值变化，清空学习中心值，重新加载学习中心选项
   * @param  {string} branch 分公司
   */
  branchChange = (branch) => {
    this.updateState({ branch, campus: null, campusDict: [] });
    if (branch) {
      loadCampusDict(branch).then((campusDict) => {
        this.updateState({ campusDict });
      });
    }
  };

  /**
   * 加载列表数据，并转换格式
   * @param  {object} changedParams 要更新的列表数据参数，包括查询条件、页码状态
   */
  loadList = async (changedParams = {}) => {
    const {
      year,
      promiseClassName,
      courseName,
      branch,
      campus,
      studentName,
      grade,
      status,
      result,
      approvalStatus,
      quitClassStatus,
      currentPage,
      pageSize,
      confirmStatus,
      confirmChannel,
      listSearchKeys,
    } = this.state;
    const searchKeys = listSearchKeys || {
      year,
      promiseClassName,
      courseName,
      branch,
      campus,
      studentName,
      grade,
      status,
      result,
      confirmChannel,
      confirm: confirmStatus,
      approvalStatus,
      quitClassStatus,
    };
    const params = { currentPage, pageSize, ...searchKeys, ...changedParams };
    /*
    rows  true  请求URL string    行数
    page  true  请求URL string    页码
    courseStatus  false 请求URL string    课程状态
    plSchoolId  false 请求URL string    学习中心ID
    pName false 请求URL string    精英班名称
    studentName false 请求URL string    学生名字
    resultStatus  false 请求URL string    结果
    year  true  请求URL string    年份
    brenchId  true  请求URL string    分公司
    gradeId true  请求URL string    年级
    auditStatus true  请求URL string    审批状态
    abortClass  true  请求URL string    是否退班
     */
    const apiParams = {
      page: params.currentPage,
      rows: params.pageSize,
      year: params.year,
      confirm: params.confirm,
      confirmChannel: params.confirmChannel,
      pName: params.promiseClassName,
      brenchId: params.branch,
      plSchoolId: params.campus,
      studentName: params.studentName,
      gradeId: params.grade,
      courseStatus: params.status ? params.status.join(',') : null,
      resultStatus: params.result,
      auditStatus: params.approvalStatus,
      abortClass: 0,
      productName: params.courseName,
    };
    await this.updateState({ listLoading: true });

    const apiData = await courseApis.getPromiseStudentList(apiParams);
    const { records, rows } = apiData;

    const formatRows = rows.map((row) => {
      const {
        studentId,
        subjectDetail,
        promiseClassName,
        productName,
        studentName,
        courseStatus,
        resultStatus,
        id,
        confirm,
        confirmChannel,
        studentGrade,
        auditStatus,
        paidStatus,
        natureHour = 0,
        contractProductId,
        abortClass,
        productVersion,
        workflowType,
        contractConfirmStatus,
        promiseClassCampusId,
        promiseClassGradeId,
      } = row;
      const defaultPlan = { distributed: 0, signed: 0, consumed: 0, hasPlan: false };
      let summerPlan = { ...defaultPlan, quarterId: 'DAT0000000232' };
      let autumnPlan = { ...defaultPlan, quarterId: 'DAT0000000233' };
      let winterPlan = { ...defaultPlan, quarterId: 'DAT0000000230' };
      let springPlan = { ...defaultPlan, quarterId: 'DAT0000000231' };
      subjectDetail.forEach((quarter) => {
        const { quarterId, hasMiniClassCount, consumeHours, allHours } = quarter;
        const plan = {
          quarterId,
          distributed: allHours,
          signed: hasMiniClassCount,
          consumed: consumeHours,
          hasPlan: true,
        };
        if (quarterId === summerPlan.quarterId) {
          summerPlan = plan;
        } else if (quarterId === autumnPlan.quarterId) {
          autumnPlan = plan;
        } else if (quarterId === winterPlan.quarterId) {
          winterPlan = plan;
        } else if (quarterId === springPlan.quarterId) {
          springPlan = plan;
        }
      });
      return {
        promiseClassName,
        studentName,
        studentId,
        gradeName: studentGrade,
        paidStatus,
        result: resultStatus,
        coursePlanInfo: [summerPlan, autumnPlan, winterPlan, springPlan],
        status: courseStatus,
        id,
        confirm,
        confirmChannel,
        approvalStatus: auditStatus,
        contractProductId,
        totalCourseHours: natureHour,
        abortClass,
        courseName: productName,
        productVersion: Number(productVersion),
        workflowType,
        contractConfirmStatus,
        promiseClassCampusId,
        promiseClassGradeId,
      };
    });
    const data = {
      records,
      rows: formatRows,
    };

    await this.updateState({
      listLoading: false,
      list: data.rows,
      totalRecords: data.records,
      currentPage: params.currentPage,
      pageSize: params.pageSize,
      listSearchKeys: {
        year: params.year,
        promiseClassName: params.promiseClassName,
        courseName: params.courseName,
        branch: params.branch,
        campus: params.campus,
        studentName: params.studentName,
        grade: params.grade,
        status: params.status,
        result: params.result,
        confirmStatus: params.confirm,
        confirmChannel: params.confirmChannel,
        approvalStatus: params.approvalStatus,
        quitClassStatus: params.quitClassStatus,
      },
    });
    return true;
  };

  /**
   * 查询
   */
  search = () => {
    const {
      year,
      promiseClassName,
      courseName,
      branch,
      campus,
      studentName,
      grade,
      status,
      result,
      confirmStatus,
      confirmChannel,
      approvalStatus,
      quitClassStatus,
    } = this.state;
    this.loadList({
      year,
      promiseClassName,
      courseName,
      branch,
      campus,
      studentName,
      grade,
      status,
      result,
      confirm: confirmStatus,
      confirmChannel,
      approvalStatus,
      quitClassStatus,
      currentPage: 1,
    });
  };

  /**
   * 重置查询条件
   */
  resetSearchKeys = () => {
    const { defaultBranch, defaultCampus, defaultCampusDict } = this.state;
    this.updateState({
      year: null,
      promiseClassName: null,
      courseName: null,
      branch: defaultBranch,
      campus: defaultCampus,
      campusDict: defaultCampusDict,
      studentName: null,
      grade: null,
      status: null,
      result: null,
      approvalStatus: null,
      quitClassStatus: null,
    });
  };

  /**
   * 更多菜单项点击处理
   * @param  {number} key 菜单唯一标识
   * @param  {object} record 列表行数据
   * @param  {number} index 列表行序号
   * workflowType 0 旧流程，1 OA 流程
   */
  moreMenuClick = (key, record, index) => {
    const isApply = key === 1 || key === 5;
    const isOA = record.workflowType === 1 || isApply;

    if ([1, 2, 3, 4, 41, 42].includes(key)) {
      this.setState({
        feeModalVisible: !isOA,
        feeOAModalVisible: isOA,
        feeModalType: parseInt(key),
        currentRecord: record,
      });
    } else {
      this.setState({
        resultModalVisible: !isOA,
        resultOAModalVisible: isOA,
        resultModalType: parseInt(key),
        resultRecord: record,
      });
    }
  };

  // 家长线上确认
  handleParentConfirm = (record) => {
    this.setState({
      currentRecord: record,
      parentConfirmModalVisible: true,
    });
  };

  // 线上确认
  handleOfflineConfirm = (record) => {
    this.setState({
      currentRecord: record,
      offlineConfirmModalVisible: true,
    });
  };

  // 撤销流程
  handleCancelFlow = (record) => {
    this.setState({
      currentRecord: record,
      cancelFlowModalVisible: true,
    });
  };

  /**
   * 申请退费/目标结果 菜单项点击处理
   * @param  {number} key 菜单唯一标识
   * @param  {object} record 列表行数据
   * @param  {number} index 列表行序号
   */
  oaApplyingMenuClick = async (key, record, index) => {
    // 1 申请退费
    // 5 目标结果
    const isApply = key === 1 || key === 5;

    if (!isApply) {
      this.moreMenuClick(key, record, index);

      return;
    }
    /**
     * @desc 合同状态为已确认和无需确认才能退费及目标结果操作
     */
    // if (![4, 16].includes(record.contractConfirmStatus)) {
    //   return showMessage('学生合同未确认，不能进行中途退费/目标结果', 'error')
    // }

    const mapping = {
      1: 'returnAudit',
      5: 'targetEnd',
    };

    try {
      const type = mapping[key];
      const res = await courseApis.isContrFundRecordUnaudited({
        contractProductId: record.contractProductId,
        checkType: type,
      });
      const { data } = res || {};

      if (!data) {
        throw new Error('服务器开小差了，暂时无法申请退费或处理目标结果');
      }

      const { flag, message } = data;

      if (flag) {
        showMessage(message, 'error');
      } else {
        this.moreMenuClick(key, record, index);
      }
    } catch (e) {
      showMessage(e.message || e.text, 'error');
    }
  };

  /**
   * 列表操作按钮点击处理
   * @param  {string} type 操作类型
   * @param  {object} record 列表行数据
   */
  listBtnClick = (type, record) => {
    // 合同确认状态 0~历史数据 -1~未知 1~待确认 2~确认中 4~已确认 8~已取消 16~无需确认 32~超期未确认
    const { id, contractProductId, totalCourseHours, status, promiseClassCampusId, promiseClassGradeId } = record;

    if (type === 'detail') {
      this.openTab(
        '/PromiseClassDetail',
        {
          promiseStudentId: id,
          contractProductId,
          totalCourseHours,
          canPlanCourse: status === '1',
          promiseClassCampusId,
          promiseClassGradeId,
        },
        id,
      );
    } else if (type === 'plan') {
      this.openTab('/PromiseClassPlan', {
        promiseStudentId: id,
        stuPromiseClassStatus: status,
        promiseClassCampusId,
        promiseClassGradeId,
      });
    }
  };

  /**
   * 打开标签页
   * @param  {string} path 路由
   * @param  {object} params 传入页面的参数
   * @param mirrorId {string?}
   */
  openTab = (path, params, mirrorId) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: { path, params, refresh: true, mirrorId },
    });
  };

  onResolveTodo = async (params) => {
    this.setState(
      {
        branch: this.state.defaultBranch,
        campus: this.state.defaultCampus,
        campusDict: this.state.defaultCampusDict,
        year: null,
        promiseClassName: null,
        courseName: null,
        studentName: null,
        grade: null,
        status: [],
        result: null,
        approvalStatus: params.auditStatus,
        quitClassStatus: null,
      },
      () => {
        this.search();
      },
    );
  };

  getColumns = () => {
    return [
      {
        title: '精英班名称',
        dataIndex: 'promiseClassName',
        key: 'promiseClassName',
        width: COL_WIDTH_XL,
        fixed: 'left',
      },
      { title: '学生编号', dataIndex: 'studentId', key: 'studentId', width: COL_WIDTH_L },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_L },
      { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: COL_WIDTH_S },
      { title: '签约课程名称', dataIndex: 'courseName', key: 'courseName', width: COL_WIDTH_XL },
      {
        title: '规划完成状态',
        key: 'planCompleteStatus',
        width: COL_WIDTH_XL,
        render: (text, record) => <PlanCompleteCell record={record} />,
      },
      {
        title: '学生精英班状态',
        dataIndex: 'status',
        key: 'status',
        width: COL_WIDTH_L,
        dict: this.state.statusDict,
        render: (text, record) => (record.abortClass === '1' ? null : text),
      },
      { title: '目标结果', dataIndex: 'result', key: 'result', width: COL_WIDTH_M, dict: resultDict },
      {
        title: '审批状态',
        dataIndex: 'approvalStatus',
        key: 'approvalStatus',
        width: COL_WIDTH_M,
        dict: approvalStatusDict,
      },
      {
        title: '确认状态',
        dataIndex: 'confirm',
        key: 'confirm',
        width: COL_WIDTH_M,
        dict: confirmDict,
      },
      {
        title: '确认方式',
        dataIndex: 'confirmChannel',
        key: 'confirmChannel',
        width: COL_WIDTH_M,
        dict: confirmChannelDict,
      },
      {
        title: '操作',
        key: 'action',
        fixed: 'right',
        width: COL_WIDTH_XL,
        getActionConfig: (text, record, index) => {
          const isOA = record.workflowType === 1;

          let actions = [
            {
              text: '详情',
              onClick: () => this.listBtnClick('detail', record, index),
              disabled: record.abortClass === '1',
            },
            {
              text: '课程规划',
              onClick: () => this.listBtnClick('plan', record, index),
              disabled: record.status === '2' || record.status === '3',
            },
          ];
          let menus = [];

          const midRefundBtn1 = { text: '申请退费', onClick: () => this.oaApplyingMenuClick(1, record, index) };
          const midRefundBtn2 = { text: '查看退费申请', onClick: () => this.moreMenuClick(2, record, index) };
          const midRefundBtn3 = { text: '退费审批', onClick: () => this.moreMenuClick(3, record, index) };
          const midRefundBtn4 = { text: '退费审批详情', onClick: () => this.moreMenuClick(4, record, index) };
          const midRefundBtn5 = { text: '退费审批', onClick: () => this.moreMenuClick(41, record, index) };
          const midRefundBtn6 = { text: '退费审批详情', onClick: () => this.moreMenuClick(42, record, index) };

          const endResultBtn1 = {
            text: '目标结果',
            onClick: () => this.oaApplyingMenuClick(5, record, index),
            disabled: !(
              record.status !== 1 &&
              record.paidStatus === 'PAID' &&
              (this.props.authIgnoreThisYear && !isPrdEnv
                ? true
                : // 限制只能操作本年产品
                  Number(record.productVersion) === this.serverYear)
            ),
          };
          const endResultBtn2 = { text: '查看目标结果', onClick: () => this.moreMenuClick(6, record, index) };
          const endResultBtn3 = { text: '目标结果审批', onClick: () => this.moreMenuClick(7, record, index) };
          const endResultBtn4 = { text: '目标结果审批详情', onClick: () => this.moreMenuClick(8, record, index) };
          const endResultBtn5 = { text: '目标结果审批', onClick: () => this.moreMenuClick(81, record, index) };
          const endResultBtn6 = { text: '目标结果审批详情', onClick: () => this.moreMenuClick(82, record, index) };

          switch (record.status) {
            // 进行中
            case '1':
              menus = [
                this.props.authMidRefundApply ? midRefundBtn1 : null,
                this.props.authEndRefundApply ? endResultBtn1 : null,
              ];
              break;
            // 退费中
            case '2':
              menus = [
                this.props.authMidRefundApply ||
                this.props.authMidRefundAccountingApprove ||
                this.props.authMidRefundCashierApprove
                  ? midRefundBtn2
                  : null,
                this.props.authMidRefundAccountingApprove && !isOA ? midRefundBtn3 : null,
              ];
              break;
            // 已退费
            case '3':
              menus = [
                this.props.authMidRefundApply ||
                this.props.authMidRefundAccountingApprove ||
                this.props.authMidRefundCashierApprove
                  ? midRefundBtn2
                  : null,
                ...(record.approvalStatus === 'RAUDIT'
                  ? [
                      this.props.authMidRefundApply ||
                      this.props.authMidRefundAccountingApprove ||
                      this.props.authMidRefundCashierApprove
                        ? midRefundBtn6
                        : null,
                    ]
                  : [
                      this.props.authMidRefundApply || this.props.authMidRefundAccountingApprove ? midRefundBtn4 : null,
                      this.props.authMidRefundCashierApprove && !isOA ? midRefundBtn5 : null,
                    ]),
              ];
              break;
            // 已结课
            case '4':
              menus = [
                this.props.authEndRefundApply ||
                this.props.authEndRefundAccountingApprove ||
                this.props.authEndRefundCashierApprove
                  ? endResultBtn2
                  : null,
                this.props.authEndRefundAccountingApprove && !isOA ? endResultBtn3 : null,
              ];
              break;
            // 已完结
            case '0':
              menus = [
                this.props.authEndRefundApply ||
                this.props.authEndRefundAccountingApprove ||
                this.props.authEndRefundCashierApprove
                  ? endResultBtn2
                  : null,
                ...(record.approvalStatus === 'RAUDIT'
                  ? [
                      record.result === '0' &&
                      (this.props.authEndRefundApply ||
                        this.props.authEndRefundAccountingApprove ||
                        this.props.authEndRefundCashierApprove)
                        ? endResultBtn6
                        : null,
                      record.result === '1' &&
                      (this.props.authEndRefundApply || this.props.authEndRefundAccountingApprove)
                        ? endResultBtn4
                        : null,
                    ]
                  : [
                      this.props.authEndRefundApply || this.props.authEndRefundAccountingApprove ? endResultBtn4 : null,
                      record.result === '0' && this.props.authEndRefundCashierApprove && !isOA ? endResultBtn5 : null,
                    ]),
              ];
              break;
            default:
              break;
          }

          // 审批状态 = 未审批 且 确认状态 = 确认中
          if (/*record.approvalStatus == 'UNAUDIT' && */ record.confirm == 2) {
            if (!!~this.props.btnAuths.indexOf('CLASS_MANAGE_PARENT_CONFIRM')) {
              menus.push({ text: '家长线上确认', onClick: () => this.handleParentConfirm(record) });
            }
            if (!!~this.props.btnAuths.indexOf('CLASS_MANAGE_OFFLINE_CONFIRM')) {
              menus.push({ text: '老师线下确认', onClick: () => this.handleOfflineConfirm(record) });
            }
            if (!!~this.props.btnAuths.indexOf('CLASS_MANAGE_CANCEL_FLOW')) {
              menus.push({ text: '撤销申请', onClick: () => this.handleCancelFlow(record) });
            }
          }

          menus = menus.filter((item) => item);

          if (menus.length > 0) {
            actions.push({ menus });
          }
          return actions;
        },
      },
    ];
  };

  render() {
    const {
      loading,
      searchExpand,
      year,
      promiseClassName,
      courseName,
      branch,
      campus,
      studentName,
      grade,
      status,
      result,
      approvalStatus,
      yearDict,
      branchDict,
      campusDict,
      gradeDict,
      statusDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
      listLoading,
    } = this.state;
    /*
    ING("1", "进行中"),//进行中    审批失败的时候传
    RETURN_ING("2", "退费中"),//退费中     申请退费的时候传
    RETURN("3", "已退费"),//已退费      退费成功的时候传
    END("4", "已结课"),//已结课  申请结课的时候传
    ENDED("0", "已完结"); // 已完结  结课成功的时候传

    UNAUDIT("UNAUDIT", "未审批"),//未审批
    RAUDIT("RAUDIT", "审批通过"),//审批通过
    AUDIT_FAILED("AUDIT_FAILED", "审批不通过"); // 审批不通过
     */

    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={
          <SearchBar
            expand={searchExpand}
            onToggleExpand={() => this.updateState({ searchExpand: !searchExpand })}
            onSearch={this.search}
            onReset={this.resetSearchKeys}
          >
            <Select
              placeholder="年份"
              dict={yearDict}
              value={year}
              onChange={(val) => {
                this.updateState({ year: val });
              }}
            />
            <Input
              placeholder="精英班名称"
              value={promiseClassName}
              onChange={(e) => {
                this.updateState({ promiseClassName: e.target.value });
              }}
            />
            <Input
              placeholder="签约课程名称"
              value={courseName}
              onChange={(e) => {
                this.updateState({ courseName: e.target.value });
              }}
            />
            <Select placeholder="分公司" dict={branchDict} value={branch} onChange={this.branchChange} />
            <Select
              placeholder="学习中心"
              dict={campusDict}
              value={campus}
              onChange={(val) => {
                this.updateState({ campus: val });
              }}
            />
            <Input
              placeholder="学生姓名或编号"
              value={studentName}
              onChange={(e) => {
                this.updateState({ studentName: e.target.value });
              }}
            />
            <Select
              placeholder="年级"
              dict={gradeDict}
              value={grade}
              onChange={(val) => {
                this.updateState({ grade: val });
              }}
            />
            <MultiSelect
              placeholder="学生精英班状态"
              dict={statusDict}
              value={status}
              onChange={(val) => {
                this.updateState({ status: val });
              }}
            />
            <Select
              placeholder="目标结果"
              dict={resultDict}
              value={result}
              onChange={(val) => {
                this.updateState({ result: val });
              }}
            />
            <Select
              placeholder="审批状态"
              dict={approvalStatusDict}
              value={approvalStatus}
              onChange={(val) => {
                this.updateState({ approvalStatus: val });
              }}
            />
            <Select
              placeholder="确认状态"
              dict={confirmDict}
              value={this.state.confirmStatus}
              onChange={(val) => {
                this.updateState({ confirmStatus: val });
              }}
            />
            <Select
              placeholder="确认方式"
              dict={confirmChannelDict}
              value={this.state.confirmChannel}
              onChange={(val) => {
                this.updateState({ confirmChannel: val });
              }}
            />
          </SearchBar>
        }
      >
        <List
          dataSource={list}
          columns={this.getColumns()}
          loading={!loading && listLoading}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadList({ currentPage: current, pageSize })}
        />

        {this.state.feeModalVisible && (
          <PromiseFeeDetailModal
            type={this.state.feeModalType}
            promiseStudentId={this.state.currentRecord.id}
            contractProductId={this.state.currentRecord.contractProductId}
            courseStatus={this.state.currentRecord.status}
            auditStatus={this.state.currentRecord.approvalStatus}
            onClose={() => this.updateState({ feeModalVisible: false })}
            onSave={() => {
              this.updateState({ feeModalVisible: false });
              this.loadList();
            }}
          />
        )}

        {this.state.feeOAModalVisible && (
          <PromiseFeeDetailOAModal
            type={this.state.feeModalType}
            promiseStudentId={this.state.currentRecord.id}
            contractProductId={this.state.currentRecord.contractProductId}
            courseStatus={this.state.currentRecord.status}
            auditStatus={this.state.currentRecord.approvalStatus}
            onClose={() => this.updateState({ feeOAModalVisible: false })}
            onSave={() => {
              this.updateState({ feeOAModalVisible: false });
              this.loadList();
            }}
          />
        )}

        {this.state.resultModalVisible && (
          <PromiseResultDetailModal
            type={this.state.resultModalType}
            promiseStudentId={this.state.resultRecord.id}
            contractProductId={this.state.resultRecord.contractProductId}
            courseStatus={this.state.resultRecord.status}
            auditStatus={this.state.resultRecord.approvalStatus}
            courseResult={this.state.resultRecord.result}
            campusDict={campusDict}
            onClose={() => this.updateState({ resultModalVisible: false })}
            onSave={() => {
              this.updateState({ resultModalVisible: false });
              this.loadList();
            }}
          />
        )}

        {this.state.resultOAModalVisible && (
          <PromiseResultDetailOAModal
            type={this.state.resultModalType}
            promiseStudentId={this.state.resultRecord.id}
            contractProductId={this.state.resultRecord.contractProductId}
            courseStatus={this.state.resultRecord.status}
            auditStatus={this.state.resultRecord.approvalStatus}
            courseResult={this.state.resultRecord.result}
            campusDict={campusDict}
            onClose={() => this.updateState({ resultOAModalVisible: false })}
            onSave={() => {
              this.updateState({ resultOAModalVisible: false });
              this.loadList();
            }}
          />
        )}

        {this.state.parentConfirmModalVisible && (
          <ParentConfirmModal
            contractInfo={{ refundWorkflowId: this.state?.currentRecord?.id, from: 'promiseStudent' }}
            onCancel={() => this.setState({ parentConfirmModalVisible: false })}
          />
        )}

        {this.state.offlineConfirmModalVisible && (
          <OfflineConfirmModal
            contractInfo={{ refundWorkflowId: this.state?.currentRecord?.id, from: 'promiseStudent' }}
            onCancel={() => this.setState({ offlineConfirmModalVisible: false })}
            onReload={() => {
              this.loadList();
            }}
          />
        )}

        {this.state.cancelFlowModalVisible && (
          <CancelFlowModal
            contractInfo={{ refundWorkflowId: this.state?.currentRecord?.id, from: 'promiseStudent' }}
            onCancel={() => this.setState({ cancelFlowModalVisible: false })}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    btnAuths: state.account.btnAuths,
    // 中途退费申请
    authMidRefundApply: state.account.btnAuths.indexOf('promiseMidReturn') > -1,
    // 中途退费会计审批
    authMidRefundAccountingApprove: state.account.btnAuths.indexOf('promiseMidReturnBtn') > -1,
    // 中途退费出纳审批
    authMidRefundCashierApprove: state.account.btnAuths.indexOf('ECS_MID_REFUND_CASHIER_APPROVE') > -1,
    // 目标完结申请
    authEndRefundApply: state.account.btnAuths.indexOf('targetEndApply') > -1,
    // 目标完结会计审批
    authEndRefundAccountingApprove: state.account.btnAuths.indexOf('promiseAuditEnd') > -1,
    // 目标完结出纳审批
    authEndRefundCashierApprove: state.account.btnAuths.indexOf('ECS_END_REFUND_CASHIER_APPROVE') > -1,
    // 目标结果忽略年份限制(注意：不可在生产环境生效!)
    authIgnoreThisYear: state.account.btnAuths.indexOf('ECS_END_REFUND_IGNORE_THIS_YEAR') > -1,

    user: state.account.user,
    pageParams: state.main.pageParams[PAGE_ID],
  };
};

export default connect(mapStateToProps)(PromiseClassManage);
