/**
 * 选择科目模态框
 */
import React from 'react';
import { Row, Col } from 'antd';
import Modal from 'COMPONENT/Modal';
import { loadSubjectAliasDict } from 'UTIL/dict';
import styles from './styles.less';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';

class ModalSelectSubject extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      visible: false, //模态框显示状态
      loading: false, //模态框加载状态
      subjects: [], //所有科目[{ id, name, fixSelected, selected }]
      blCampusId: null,
      gradeId: null,
    };

    this.subjectOptions = null; // 所有科目选项[{ id, name }]
  }

  componentWillMount() {
    //加载初始化数据
    this.setState({ loading: true });

    this.loadSelectOption().then(() => {
      this.setState({ loading: false });
    });
  }

  /**
   * 加载所有科目选项，并将接口获取的科目选项数据转换成前端使用的格式：
   * {'11': '语文', '22': '数学', ...}  ===>  [{ id, name }]
   * @return {Promise}
   */
  loadSelectOption = async (hasLoading) => {
    if (hasLoading) {
      this.setState({ loading: true });
    }

    const { blCampusId, gradeId } = this.props;
    const subjectAliasDict = await loadSubjectAliasDict({ orgId: blCampusId, gradeId });
    const options = subjectAliasDict.map((item) => {
      return {
        id: item.val,
        name: item.text,
      };
    });

    this.subjectOptions = options;

    if (hasLoading) {
      this.setState({ loading: false });
    }

    return true;
  };

  /**
   * 关闭弹框
   */
  close = () => {
    this.setState({ visible: false });
  };

  /**
   * 打开弹框
   * @param {function} onConfirm 选中后的回调
   * @param {[string]?} selectedIds 当季已选科目
   */
  open = (selectedIds, onConfirm, classId) => {
    this.selectedIds = selectedIds ? [selectedIds] : [];
    this.onConfirm = onConfirm;
    this.classId = classId;
    if (this.subjectOptions) {
      this.initContent();
    } else {
      this.loadSelectOption(true).then(() => {
        this.initContent();
      });
    }
    this.setState({ visible: true });
  };

  /**
   * 初始化弹框内容：科目选项按钮以及可选状态
   */
  initContent = () => {
    const { selectedIds } = this;
    const options = this.subjectOptions;
    let subjects;
    if (selectedIds && selectedIds.length > 0) {
      subjects = options.map((s) => {
        if (selectedIds.indexOf(s.id) >= 0) {
          return { ...s, selected: true };
        }
        return s;
      });
    } else {
      subjects = [...options];
    }
    this.setState({ subjects });
  };

  /**
   * 确定按钮点击处理
   */
  handleOk = () => {
    const { onConfirm } = this;
    const { subjects } = this.state;
    const selected = subjects.find((s) => s.selected);
    let id = null;
    let name = null;
    if (selected) {
      id = selected.id;
      name = selected.name;
    }
    if (this.classId && this.selectedIds.length > 0 && this.selectedIds[0].id !== id) {
      showAlert('更换科目，将同时删除已添加的小班', () => {
        onConfirm({ id, name, deleteClass: true });
        this.close();
      });
    } else {
      onConfirm({ id, name });
      this.close();
    }
  };

  toggle = (id, selected) => {
    const { subjects } = this.state;
    this.setState({
      subjects: subjects.map((s) => {
        if (selected && s.id === id) {
          return { ...s, selected: true };
        }
        return { ...s, selected: false };
      }),
    });
  };

  render() {
    const { selectedIds } = this;
    const { subjects, visible, loading } = this.state;
    const oldSelectedCount = selectedIds ? selectedIds.length : 0;
    const selectedCount = subjects.filter((subject) => subject.selected).length + oldSelectedCount;
    const leftCount = 8 - selectedCount;
    const modalOpts = {
      title: '选择科目（单选）',
      width: 600,
      visible,
      loading,
      onCancel: this.close,
      onOk: this.handleOk,
      toggle: true,
    };

    return (
      <Modal {...modalOpts}>
        <Row gutter={10}>
          {subjects.map((s, i) => {
            let btnClass = styles.btn;
            if (s.selected) {
              btnClass += ' ' + styles.btnSelected;
            }
            return (
              <Col span={4} key={i} style={{ marginTop: 10, textAlign: 'center' }}>
                <span onClick={() => this.toggle(s.id, !s.selected)} className={btnClass}>
                  {s.name}
                </span>
              </Col>
            );
          })}
        </Row>
      </Modal>
    );
  }
}

export default ModalSelectSubject;
