/**
 * 一个季度的科目课时展现
 * @property {object} data  数据结构如下：
 * { id, name: '暑假', subjects: [
        { id, name: '语文', hours: 48, classId: '123', className: '2017暑假初三精英课堂语文1班', campus: '柠溪学习中心',
          startTime: '2017-05-01 10:00', teacher: '高山', totalHours: 48, consume: 3, totalConsume: 15, status: '上课中',
        },
      ]
    }
    @property {function} onAddSubject 添加科目按钮点击回调
    @property {function} onHourChange 分配课时变化回调
    @property {function} onDeleteSubject 删除科目回调
    @property {function} onSelectClass 选择班级回调
    @property {function} onDeleteClass 删除班级回调
    @property {boolean?} signUpClass  区别“报班管理”和“科目规划”
 */
import React from 'react';
import { Row, Col, Icon } from 'antd';
import getWindowSize from 'UTIL/getWindowSize';
import Subject from '../Subject';
import styles from './styles.less';

const NewSubject = ({ short, onClick }) => {
  const h = short ? 140 : 355;
  return (
    <div className={styles.newSubject} style={{ height: h, lineHeight: h + 'px' }} onClick={onClick}>
      <Icon type="plus" style={{ fontSize: 20, marginRight: 5, verticalAlign: 'middle' }} />
      <span style={{ fontSize: 14, verticalAlign: 'middle' }}>点击这里新增本季度上课科目</span>
    </div>
  );
};

class Season extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      winW: 0, //  number  窗口宽度
    };
  }

  componentDidMount() {
    this.updateWinW();
    window.addEventListener('resize', this.updateWinW, false);
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.updateWinW, false);
  }

  /**
   * 获取和更新窗口宽度状态
   */
  updateWinW = () => {
    this.setState({ winW: getWindowSize().width });
  };

  render() {
    const {
      data: { name, subjects: subjectsProp, payed },
      onAddSubject,
      onHourChange,
      onDeleteSubject,
      onSelectClass,
      onDeleteClass,
      signUpClass,
      ...props
    } = this.props;
    const { winW } = this.state;

    let totalHours = 0;
    const subjectCount = subjectsProp.length;
    subjectsProp.forEach((s) => (totalHours += s.hours));
    //const subjects = !signUpClass ? [...subjectsProp, 'new'] : [...subjectsProp]
    const subjects = subjectsProp;

    let short = false;
    let titleShort = !signUpClass && (subjects.length === 1 || (subjects.length > 1 && !subjects[0].classId));

    const colCount = winW < 1660 ? (winW < 1350 ? (winW < 1070 ? 1 : 2) : 3) : 4;

    return (
      <div>
        <div style={{ float: 'left', width: 110, margin: '8px 0' }}>
          <div
            className={styles.titleBlock}
            style={{ height: titleShort ? 140 : 355, paddingTop: titleShort ? 20 : 105 }}
          >
            <div className={styles.title}>{name}</div>
            <div>总小时：{totalHours}</div>
            <div>已选科目：{subjectCount}科</div>
          </div>
        </div>
        <div style={{ marginLeft: 125 }}>
          <Row gutter={15}>
            {subjects.map((subject, i) => {
              const isNew = subject === 'new';
              if (i % colCount === 0 && !subject.classId) {
                short = true;
              }
              let content;
              if (isNew) {
                content = <NewSubject short={short} onClick={() => onAddSubject(-1)} />;
              } else {
                content = (
                  <Subject
                    data={subject}
                    short={short}
                    signUpClass={signUpClass}
                    payed={payed}
                    onHourChange={(val) => onHourChange(i, val)}
                    onDeleteSubject={() => onDeleteSubject(i)}
                    onSelectClass={(promiseSubjectId, leftHours, isTransfer) =>
                      onSelectClass(i, promiseSubjectId, leftHours, isTransfer)
                    }
                    onDeleteClass={() => onDeleteClass(i)}
                    onAddSubject={() => onAddSubject(i)}
                    {...props}
                  />
                );
              }
              return (
                <Col key={i} span={isNew && i === 0 ? 24 : 24 / colCount} style={{ margin: '8px 0' }}>
                  {content}
                </Col>
              );
            })}
          </Row>
        </div>
      </div>
    );
  }
}

export default Season;
