/**
 * 精英班学生课程规划页面
 * @param {string} promiseStudentId 小班学生ID
 */
import React from 'react';
import { connect } from 'dva';
import { Form, Row, Col, Input, Tabs, Icon } from 'antd';
import Page from 'COMPONENT/Page';
import Popover from 'COMPONENT/Popover';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import {
  getPromiseSubjectList,
  findStudentInfoByPromiseStudentId,
  savePromiseSubject,
  initPromiseClassSubject,
} from 'SERVICE/course';
import Season from './Season';
import Block from 'COMPONENT/Block';
import ModalSelectSubject from './ModalSelectSubject';
import ModalSelectClass from './ModalSelectClass';
import ModalCourseInfo from './ModalCourseInfo';
import styles from './styles.less';
import { includeQuarters } from 'PAGE/common/utils';
import NP from 'number-precision';

const FormItem = Form.Item;
const TabPane = Tabs.TabPane;
const formItemLayout = {
  labelCol: { span: 8 },
  wrapperCol: { span: 16 },
};
const colSpans = {
  sm: { span: 12 },
  lg: { span: 6 },
};

const TabTitle = ({ text, dirty, isActive }) => {
  if (isActive) {
    if (dirty) {
      return (
        <div className={styles.tabTitle}>
          <span>{text}</span>
          <span className={styles.redDot}></span>
        </div>
      );
    } else {
      return <div className={styles.tabTitle}>{text}</div>;
    }
  } else {
    if (dirty) {
      return (
        <Popover content={'需保存当前已修改的信息，才能进行' + text} icon="info">
          <div className={styles.tabTitle} style={{ cursor: 'not-allowed' }}>
            {text}
          </div>
        </Popover>
      );
    } else {
      return <div className={styles.tabTitle}>{text}</div>;
    }
  }
};

class PromiseClassPlan extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      detailData: {}, //学生详情（学生信息、总课时数、已上课时数）
      initSeasons: [], //初始季度数据
      seasons: [], //季度数据
      loading: false, //页面加载状态
      activeTab: '1', //当前标签页
      defaultCampus: null, //当前用户默认学习中心
      courseInfoModalVisible: false, // {boolean} 课程信息弹框显示状态
      currentMiniClassId: null, // {string} 当前小班ID
      checking: false, // {boolean} 校验状态是否开启
    };
    this.productYearId = null;
  }

  componentWillMount() {
    this.initContent();
  }

  componentDidMount() {
    //this.selectClass()
  }

  componentWillUnmount() {
    this.isUnmounted = true;
  }

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    if (!this.isUnmounted) {
      this.setState(newState);
    }
  };

  /**
   * 初始化内容
   */
  initContent = () => {
    const {
      params: { promiseStudentId },
    } = this.props;
    this.updateState({ loading: true });
    initPromiseClassSubject({ promiseStudentId }).then(
      () => {
        Promise.all([
          findStudentInfoByPromiseStudentId({ promiseStudentId }), //学生详情
          this.loadSeasonList(), //更新季节列表数据
        ]).then(
          ([detailData, seasonsAll]) => {
            const { productQuarterId, blcampusId } = detailData;
            const showQuarters = includeQuarters[productQuarterId];
            const seasons = showQuarters ? seasonsAll.filter((s) => showQuarters.indexOf(s.id) >= 0) : seasonsAll;
            this.updateState({ detailData, initSeasons: seasons, seasons, defaultCampus: blcampusId, loading: false });
          },
          (e) => {
            showMessage(e.text || '数据加载失败');
          },
        );
      },
      (e) => {
        showMessage(e.text || '数据加载失败');
      },
    );
  };

  /**
   * 加载科目列表
   * @return {Promise}
   */
  loadSeasonList = async () => {
    const {
      params: { promiseStudentId },
    } = this.props;
    const subjectListData = await getPromiseSubjectList({ promiseStudentId });
    const seasons = this.getSeasonsData(subjectListData.rows);
    return seasons;
  };

  /**
   * 将接口获取的科目列表数据转成前端使用的数据格式
   * @param  {[object]} subjects 科目列表，格式参见接口文档：
   * https://api.xuebangsoft.me/project.do#/f7525741-db26-4bb4-8475-7555d19dadcc/front/interfaceDetail/5de30175-e035-48c1-8ee4-f277d797b24f
   * @return {[object]}  转换后的数据格式，如下：
   * [
        { id: 'DAT0000000232', name: '暑假', payed: true, subjects: [
            { id: 'DAT0000000019', name: '语文', hours: 48, classId: '123', className: '2017暑假初三精英课堂语文1班', campus: '柠溪学习中心',
              startTime: '2017-05-01 10:00', teacher: '高山', totalHours: 48, consume: 3, totalConsume: 15,
              status: '上课中', statusCode: 'STARTED', consumeCourseHours: 0,
            },
          ]
        }
      ]
   */
  getSeasonsData = (subjects) => {
    const seasons = [
      { id: 'DAT0000000232', name: '暑假', payed: true, subjects: [] },
      { id: 'DAT0000000233', name: '秋季', payed: true, subjects: [] },
      { id: 'DAT0000000230', name: '寒假', payed: true, subjects: [] },
      { id: 'DAT0000000231', name: '春季', payed: true, subjects: [] },
    ];
    subjects.forEach((subject) => {
      const {
        id,
        quarterId,
        subjectId,
        subjectName,
        courseHours,
        courseNatureHours = 0,
        miniClass,
        consumeNatureHours = 0,
      } = subject;
      let subjectInfo = {
        id: subjectId,
        name: subjectName,
        hours: courseNatureHours,
        oldHours: courseHours,
        promiseSubjectId: id,
        totalConsume: consumeNatureHours || 0,
      };
      if (miniClass) {
        const {
          miniClassId,
          name,
          blCampusName,
          startDate,
          teacherName,
          totalClassNatureHours = 0,
          consume,
          status,
          statusName,
          consumeNatureHour = 0,
        } = miniClass;
        if (miniClassId) {
          subjectInfo = {
            ...subjectInfo,
            classId: miniClassId,
            className: name,
            campus: blCampusName,
            startTime: startDate,
            teacher: teacherName,
            totalHours: totalClassNatureHours,
            consume: consumeNatureHour,
            statusCode: status,
            status: statusName,
            consumeNatureHours: consumeNatureHours,
          };
        }
      }
      const quarter = seasons.find((s) => s.id === quarterId);
      if (quarter) {
        quarter.subjects.push(subjectInfo);
      }
    });
    seasons.map((season) => {
      const { subjects } = season;
      const hasClass = subjects.filter((s) => s.miniClassId);
      const noClass = subjects.filter((s) => !s.miniClassId);
      return {
        ...season,
        subjects: [...hasClass, ...noClass],
      };
    });
    return seasons;
  };

  /**
   * 保存，提交数据结构为：
   * [
   *   {"courseHours":"1","subjectId":"DAT0000000014","quarterId":"DAT0000000230",
   *     "miniClass":{"miniClassId":"2c90809e5c5db885015c5dbbf4a90009"},
   *     "promiseStudent":{"id":"8a9948d95aa34851015aa691a912004b"},
   *   }
   * ]
   */
  save = () => {
    const {
      params: { promiseStudentId },
    } = this.props;
    const { seasons } = this.state;
    let subjectVos = [];
    seasons.forEach((season) => {
      season.subjects.forEach((subject) => {
        const { oldHours, id, promiseSubjectId, classId } = subject;
        let d = {
          courseHours: oldHours,
          subjectId: id,
          quarterId: season.id,
          promiseStudent: { id: promiseStudentId },
        };
        if (classId) {
          d.miniClass = { miniClassId: classId };
        }
        if (promiseSubjectId) {
          d.id = promiseSubjectId;
        }
        subjectVos.push(d);
      });
    });
    const saveData = { promiseStudentId, subjectVos };
    this.updateState({ loading: true });
    savePromiseSubject(saveData).then(
      () => {
        this.updateState({ loading: false });
        showMessage('保存成功', 'success');
        this.initContent();
      },
      () => {
        this.updateState({ loading: false });
      },
    );
  };

  /**
   * 保存校验
   * 科目规划时：
   * 1、当前各季度各科目已分配课时数合计数大于产品总课时数
   * 2、当前各季度各科目已分配课时数合计数小于产品总课时数
   * 3、当前某一个季度或某些季度已选科目数为0
   * 4、可分配课时数小于等于0
   * 报班管理时：
   * 1、已报一个或一个以上小班的季度，是否存在某个或某些科目没报小班
   * @return {boolean} 校验是否通过
   */
  check = () => {
    const { seasons, activeTab } = this.state;
    let tips = null;
    if (activeTab === '1') {
      seasons.forEach((season) => {
        if (tips) {
          return false;
        }
        season.subjects.forEach((subject) => {
          if (!subject.name) {
            tips = '请选择科目';
            return false;
          }
        });
      });
      if (tips) {
        this._page.showOkTips(tips);
        return false;
      }
      const { unPlanHours } = this.getHoursReport();
      if (unPlanHours < 0) {
        tips = '已超出总小时，请减少科目或科目小时';
      } else if (unPlanHours > 0.1) {
        tips = '总小时未分配完，请继续添加科目或给科目分配小时';
      } else if (seasons.find((s) => s.subjects.length === 0)) {
        tips = '每个季度都需要添加科目，请给季度添加科目';
      } else {
        let invaildHours;
        seasons.forEach((s) => {
          if (s.subjects.find((subject) => subject.hours <= 0)) {
            invaildHours = true;
          }
        });
        if (invaildHours) {
          tips = '科目可分配小时数不能为0，请为科目分配小时数';
        }
      }
    } else {
      if (
        seasons.find((season) => {
          const { subjects } = season;
          const len = subjects.filter((subject) => subject.classId).length;
          return len !== 0 && len !== subjects.length;
        })
      ) {
        tips = '已报小班的季度需要报满所有科目才能保存，请报满后再保存';
      }
    }
    if (tips) {
      this._page.showOkTips(tips);
      return false;
    }
    return true;
  };

  /**
   * 保存按钮点击处理
   * 全部input都disable,为何还保留提交按钮？？22
   */
  handleOk = () => {
    const { checking } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    if (!this.check()) {
      return;
    }
    this.save();
  };

  /**
   * 更新某一个季节的数据状态
   * @param  {string} id   季节ID
   * @param  {object} data 要更新的字段
   */
  updateSeason = (id, data) => {
    const { seasons } = this.state;
    this.updateState({
      seasons: seasons.map((s) => {
        if (s.id === id) {
          return { ...s, ...data };
        }
        return s;
      }),
    });
  };

  /**
   * 打开选择科目弹框
   * @param {string} seasonId  季节ID
   * @param {number} subjectIdx  科目顺序号
   */
  addSubject = (seasonId, subjectIdx) => {
    const { seasons } = this.state;
    const subjects = seasons.find((s) => s.id === seasonId).subjects;
    this._modalSelectSubject.open(
      subjectIdx === -1 ? null : subjects[subjectIdx].id,
      ({ id, name, deleteClass }) => {
        if (subjectIdx === -1) {
          this.updateSeason(seasonId, {
            subjects: [...subjects, { id, name, hours: 0, promiseSubjectId: null, totalConsume: 0 }],
          });
        } else {
          this.updateSeason(seasonId, {
            subjects: subjects.map((s, i) => {
              if (i === subjectIdx) {
                let {
                  classId,
                  className,
                  campus,
                  startTime,
                  teacher,
                  totalHours,
                  consume,
                  statusCode,
                  status,
                  consumeCourseHours,
                  ...infosExceptClass
                } = s;
                if (deleteClass) {
                  return { ...infosExceptClass, id, name };
                } else {
                  return { ...s, id, name };
                }
              }
              return s;
            }),
          });
        }
      },
      subjectIdx === -1 ? null : subjects[subjectIdx].classId,
    );
  };

  /**
   * 可分配课时变化
   * @param {string} seasonId  季节ID
   * @param {number} subjectIdx  科目顺序号
   * @param {number} hours  课时数
   */
  hourChange = (seasonId, subjectIdx, hours) => {
    const { seasons } = this.state;
    const subjects = seasons.find((s) => s.id === seasonId).subjects;
    this.updateSeason(seasonId, {
      subjects: subjects.map((s, i) => {
        if (i === subjectIdx) {
          return { ...s, hours: parseInt(hours) > 0 ? parseInt(hours) : 0 };
        }
        return s;
      }),
    });
  };

  /**
   * 删除科目
   * @param {string} seasonId  季节ID
   * @param {number} subjectIdx  科目顺序号
   */
  deleteSubject = (seasonId, subjectIdx) => {
    const { seasons } = this.state;
    this.updateSeason(seasonId, {
      subjects: seasons.find((s) => s.id === seasonId).subjects.filter((s, i) => i !== subjectIdx),
    });
  };

  /**
   * 选班
   * @param {string} seasonId  季节ID
   * @param {number} subjectIdx  科目顺序号
   * @param {number} promiseSubjectId  小班科目ID
   * @param {number} leftHours  剩余课时数
   * @param {boolean} isTransfer  是否是转班
   */
  selectClass = (seasonId, subjectIdx, promiseSubjectId, leftHours, isTransfer) => {
    const { seasons, defaultCampus, detailData } = this.state;
    const subjects = seasons.find((s) => s.id === seasonId).subjects;
    this._modalSelectClass.open(
      promiseSubjectId,
      leftHours,
      defaultCampus,
      (classData) => {
        /*classData = {
          classId: '123', className: '2017暑假初三精英课堂语文1班', campus: '柠溪学习中心',
          startTime: '2017-05-01 10:00', teacher: '高山', totalHours: 48, consume: 3, status: '上课中', statusCode: 'STARTED',
        }*/
        /*classData = {
          classId: '123', className: '2017暑假初三精英课堂语文1班', campus: '柠溪学习中心',
          startTime: '2017-05-01 10:00', teacher: '高山', totalHours: 48, consume: 3, status: '未上课', statusCode: 'PENDDING_START',
        }*/
        this.updateSeason(seasonId, {
          subjects: subjects.map((s, i) => {
            if (i === subjectIdx) {
              const {
                miniClassId: classId,
                name: className,
                blCampusName: campus,
                startDate: startTime,
                teacherName: teacher,
                totalClassNatureHours: totalHours,
                consumeNatureHour: consume,
                statusName: status,
                status: statusCode,
              } = classData;
              return {
                ...s,
                classId,
                className,
                campus,
                startTime,
                teacher,
                totalHours,
                consume,
                status,
                statusCode,
              };
            }
            return s;
          }),
        });
      },
      isTransfer,
      detailData.productVersionId,
      seasonId,
    );
  };

  /**
   * 删除班级
   * @param {string} seasonId  季节ID
   * @param {number} subjectIdx  科目顺序号
   */
  deleteClass = (seasonId, subjectIdx) => {
    const { seasons } = this.state;
    const subjects = seasons.find((s) => s.id === seasonId).subjects;
    this.updateSeason(seasonId, {
      subjects: subjects.map((s, i) => {
        if (i === subjectIdx) {
          const { id, name, hours, oldHours, totalConsume, promiseSubjectId } = s;
          return { id, name, hours, oldHours, totalConsume, promiseSubjectId };
        }
        return s;
      }),
    });
  };

  /**
   * 切换标签页，当前标签页未保存时不能切换
   * @param  {string} tabKey 标签页唯一标识  '1': 科目规划  '2': 报班
   */
  tabChange = (tabKey) => {
    const { initSeasons, seasons } = this.state;
    const dirty = !objectDeepEquals(initSeasons, seasons);
    if (!dirty) {
      this.updateState({ activeTab: tabKey });
    }
  };

  /**
   * 计算课时汇总
   * @return {object}
   */
  getHoursReport = () => {
    const {
      detailData: { natureHour },
      seasons,
    } = this.state;

    let summerHours = 0;
    let autumnHours = 0;
    let winterHours = 0;
    let springHours = 0;
    seasons.forEach((season) => {
      season.subjects.forEach((subject) => {
        const hours = subject.hours ? subject.hours : 0;
        if (season.id === 'DAT0000000232') {
          summerHours += hours;
        } else if (season.id === 'DAT0000000233') {
          autumnHours += hours;
        } else if (season.id === 'DAT0000000230') {
          winterHours += hours;
        } else if (season.id === 'DAT0000000231') {
          springHours += hours;
        }
      });
    });
    const planedHours = NP.plus(summerHours, autumnHours, winterHours, springHours);
    const unPlanHours = Math.floor(NP.minus(natureHour || 0, planedHours));
    return { summerHours, autumnHours, winterHours, springHours, planedHours, unPlanHours };
  };

  /**
   * 弹出课程信息弹框
   * @param {string} miniClassId 小班ID
   */
  showCourseInfo = (miniClassId) => {
    this.setState({ courseInfoModalVisible: true, currentMiniClassId: miniClassId });
  };

  render() {
    const {
      params: { stuPromiseClassStatus },
    } = this.props;
    const {
      detailData: {
        studentId,
        studentName,
        customerName,
        gradeName,
        customerContact,
        productName,
        contractRemark,
        signStaffName,
        promiseClassName,
        sudyManagerName,
        natureHour,
        consumeNatureHour,
      },
      seasons,
      loading,
      initSeasons,
      activeTab,
      courseInfoModalVisible,
      currentMiniClassId,
      checking,
    } = this.state;

    const {
      summerHours,
      autumnHours,
      winterHours,
      springHours,
      planedHours,
      unPlanHours: _unPlanHours,
    } = this.getHoursReport();
    const unPlanHours = _unPlanHours > 0 ? _unPlanHours : 0;

    const dirty = !objectDeepEquals(initSeasons, seasons);

    return (
      <Page footer={true} onOk={this.handleOk} loading={loading} ref={(c) => (this._page = c)} route={this.props.route}>
        <Block type="dark" title="学生信息" className={styles.studentInfoBlock}>
          <Row>
            <Col {...colSpans}>
              <FormItem {...formItemLayout} label="学生姓名：">
                <Input value={studentName} disabled />
              </FormItem>
            </Col>
            <Col {...colSpans}>
              <FormItem {...formItemLayout} label="客户姓名：">
                <Input value={customerName} disabled />
              </FormItem>
            </Col>
            <Col {...colSpans}>
              <FormItem {...formItemLayout} label="学生年级：">
                <Input value={gradeName} disabled />
              </FormItem>
            </Col>
            <Col {...colSpans}>
              <FormItem {...formItemLayout} label="客户电话：">
                <Input value={customerContact} disabled />
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col {...colSpans}>
              <FormItem {...formItemLayout} label="签约课程名称：">
                <Input value={productName} disabled />
                {contractRemark && (
                  <div className={styles.remarkIcon}>
                    <Popover trigger="click" content={contractRemark}>
                      <Icon
                        type="info-circle"
                        style={{ cursor: 'pointer', color: '#108ee9' }}
                        onClick={(e) => e.stopPropagation()}
                      />
                    </Popover>
                  </div>
                )}
              </FormItem>
            </Col>
            <Col {...colSpans}>
              <FormItem {...formItemLayout} label="签约人：">
                <Input value={signStaffName} disabled />
              </FormItem>
            </Col>
            <Col {...colSpans}>
              <FormItem {...formItemLayout} label="精英班名称：">
                <Input value={promiseClassName} disabled />
              </FormItem>
            </Col>
            <Col {...colSpans}>
              <FormItem {...formItemLayout} label="班主任：">
                <Input value={sudyManagerName} disabled />
              </FormItem>
            </Col>
          </Row>
        </Block>
        <Block type="dark" title="课程信息">
          <Row gutter={15}>
            <Col {...colSpans}>
              <FormItem {...formItemLayout} label="总小时数：">
                <Input value={natureHour} disabled />
              </FormItem>
            </Col>
            <Col {...colSpans}>
              <FormItem {...formItemLayout} label="已分配小时数：">
                <Input value={planedHours} disabled />
              </FormItem>
            </Col>
            <Col {...colSpans}>
              <FormItem {...formItemLayout} label="未分配小时数：">
                <Input value={unPlanHours} disabled />
              </FormItem>
            </Col>
            <Col {...colSpans}>
              <FormItem {...formItemLayout} label="已上小时数：">
                <Input value={consumeNatureHour} disabled />
              </FormItem>
            </Col>
          </Row>
          <div>
            <table className={styles.summaryTable}>
              <tbody>
                <tr>
                  <th>暑假</th>
                  <th>秋季</th>
                  <th>寒假</th>
                  <th>春季</th>
                </tr>
                <tr>
                  <td>{summerHours}</td>
                  <td>{autumnHours}</td>
                  <td>{winterHours}</td>
                  <td>{springHours}</td>
                </tr>
              </tbody>
            </table>
          </div>
        </Block>
        <Block type="dark">
          <div className={styles.noPaddingTab}>
            <Tabs activeKey={activeTab} onChange={this.tabChange}>
              <TabPane tab={<TabTitle text="科目规划" dirty={dirty} isActive={activeTab === '1'} />} key="1">
                {seasons.map((data, i) => (
                  <Season
                    key={i}
                    data={data}
                    stuPromiseClassStatus={stuPromiseClassStatus}
                    onAddSubject={(subjectIdx) => this.addSubject(data.id, subjectIdx)}
                    onHourChange={(subjectIdx, val) => this.hourChange(data.id, subjectIdx, val)}
                    onDeleteSubject={(subjectIdx) => this.deleteSubject(data.id, subjectIdx)}
                    onShowCourseInfo={this.showCourseInfo}
                    checking={checking}
                  />
                ))}
              </TabPane>
              <TabPane tab={<TabTitle text="报班管理" dirty={dirty} isActive={activeTab === '2'} />} key="2">
                {seasons
                  .filter((s) => s.subjects && s.subjects.length > 0)
                  .map((data, i) => (
                    <Season
                      key={i}
                      data={data}
                      signUpClass
                      stuPromiseClassStatus={stuPromiseClassStatus}
                      onSelectClass={(subjectIdx, promiseSubjectId, leftHours, isTransfer) =>
                        this.selectClass(data.id, subjectIdx, promiseSubjectId, leftHours, isTransfer)
                      }
                      onDeleteClass={(subjectIdx) => this.deleteClass(data.id, subjectIdx)}
                      onShowCourseInfo={this.showCourseInfo}
                      checking={checking}
                    />
                  ))}
              </TabPane>
            </Tabs>
          </div>
        </Block>
        <ModalSelectSubject
          ref={(c) => (this._modalSelectSubject = c)}
          blCampusId={this.props.params.promiseClassCampusId}
          gradeId={this.props.params.promiseClassGradeId}
        />
        <ModalSelectClass ref={(c) => (this._modalSelectClass = c)} />
        {courseInfoModalVisible && (
          <ModalCourseInfo
            miniClassId={currentMiniClassId}
            studentId={studentId}
            onClose={() => this.setState({ courseInfoModalVisible: false })}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    // { promiseStudentId, promiseClassCampusId, promiseClassGradeId }
    params: state.main.pageParams['/PromiseClassPlan'],
  };
};

export default connect(mapStateToProps)(PromiseClassPlan);
