import React, { Component } from 'react';
import List from 'COMPONENT/List';
import Modal from 'COMPONENT/Modal';
import Block from 'COMPONENT/Block';
import Button from 'COMPONENT/Button';
import moment from 'moment';
import { loadCommonDictByGroup, loadBranchCampusDict, transformStringToCategoryArray } from 'UTIL/dict';
import SearchFeat from '../SearchFeat';
import { auxBuildQuarter, auxBuildGrade, auxBuildVersion } from '../util';

import Loader from '../loader';
import showAlert from '../../../../components/utils/globalTips/showAlert';
const TWO_TEACHER_VAL = 'TWO_TEACHER';
const SMALL_CLASS_VAL = 'SMALL_CLASS';

const { COL_WIDTH_S, COL_WIDTH_M } = List;

/**
 * 选择续班弹窗
 */
class AddClassModal extends Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      listLoading: false,
      searchKey: null,

      rows: [],
      page: 1,
      pageSize: 10,
      records: 0,

      campusDict: [],
      yearDict: [],
      quarterDict: [],
      gradeDict: [],
      subjectDict: [],
      classTypeDict: [],
      classLevelDict: [],
      mainTcDict: [],
      assistTcDict: [],
      selectedRowKeys: [],
      selectedRows: [],
      weekDayDict: [],
    };

    this.dictVersion = 1;
    this.searchRef = null;

    this.initContent = this.initContent.bind(this);
    this.initDict = this.initDict.bind(this);
    this.loadListData = this.loadListData.bind(this);
    this.formatApiParam = this.formatApiParam.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onPageChange = this.onPageChange.bind(this);
    this.getOriginClassName = this.getOriginClassName.bind(this);
    this.formatColumns = this.formatColumns.bind(this);
    this.onSearchRef = this.onSearchRef.bind(this);
    this.handleConfirm = this.handleConfirm.bind(this);
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化弹窗内容
   * 这里不再使用后端计算升年级处理
   */
  initContent() {
    this.initDict();
  }

  /**
   * 初始化字典
   */
  initDict() {
    const { branchId, classTypeId, productTypeId, propsDict } = this.props;

    const category = transformStringToCategoryArray('WeekDay,PRODUCT_VERSION,PRODUCT_QUARTER,STUDENT_GRADE,SUBJECT');
    category.push({ category: 'CLASS_LEVEL', parentId: classTypeId });

    const asyncCalls = [loadBranchCampusDict(), loadCommonDictByGroup(category)];

    // 教师字典
    let mainTcDict = null;
    let assistTcDict = null;
    if (productTypeId === SMALL_CLASS_VAL) {
      mainTcDict = propsDict.smallClassMainTcDict;
      assistTcDict = propsDict.smallClassAuxTcDict;
    } else {
      mainTcDict = propsDict.twoTcMainTcDict;
      assistTcDict = propsDict.twoTcAuxTcDict;
    }

    Promise.all(asyncCalls).then(([campusDict, groupDict]) => {
      this.setState(
        {
          campusDict: campusDict.find((branch) => branch.val === branchId).options,
          weekDayDict: groupDict['WeekDay'],
          yearDict: groupDict['PRODUCT_VERSION'],
          quarterDict: groupDict['PRODUCT_QUARTER'],
          gradeDict: groupDict['STUDENT_GRADE'],
          subjectDict: groupDict['SUBJECT'],
          classLevelDict: groupDict['CLASS_LEVEL'],
          mainTcDict,
          assistTcDict,
        },
        () => {
          this.setState({ searchKey: new Date().getTime() });
        },
      );
    });
  }

  /**
   * 获取列表
   * @param changeParam
   */
  async loadListData(changeParam) {
    if (this.listLoading) {
      return false;
    }

    this.setState({ listLoading: true });

    const param = this.formatApiParam(changeParam);
    const listData = await Loader.loadClasses(param);

    this.setState({
      listLoading: false,
      rows: listData.rows,
      records: listData.records,
      page: listData.page,
    });
  }

  /**
   * 格式化请求参数
   */
  formatApiParam(changeParam = {}) {
    const { productTypeId, originalClassId, classTypeId } = this.props;
    const { page, pageSize } = this.state;
    let param = { productTypeId, classId: originalClassId, classTypeId, page, pageSize };

    if (this.searchRef) {
      const searchParam = this.searchRef.formatParam();
      param = { ...param, ...searchParam };
    }

    param = { ...param, ...changeParam };

    return param;
  }

  /**
   * 搜索组件ref
   */
  onSearchRef(ref) {
    this.searchRef = ref;
  }

  /**
   * 搜索
   */
  onSearch() {
    this.setState(
      {
        page: 1,
      },
      () => {
        this.loadListData();
      },
    );
  }

  onPageChange({ current, pageSize }) {
    this.setState(
      {
        page: current,
        pageSize,
        selectedRowKeys: [],
        selectedRows: [],
      },
      () => {
        this.loadListData();
      },
    );
  }

  /**
   * 确认
   */
  handleConfirm() {
    const { selectedRows } = this.state;

    if (selectedRows.length <= 0) {
      showAlert('请先选择续班');
    } else {
      this.props.handleChose(selectedRows);
    }
  }

  /**
   * 获取原班课程名称
   */
  getOriginClassName() {
    const {
      className,
      blCampusName,
      mainTeacherName,
      classStartDate,
      classEndDate,
      assistTeacherName,
      classStartTime,
      classLength,
    } = this.props;
    return (
      className +
      ' / ' +
      blCampusName +
      ' / ' +
      (mainTeacherName || '') +
      ' ' +
      (assistTeacherName || '') +
      ' / ' +
      classStartDate +
      (classEndDate ? ' ~ ' + classEndDate : '') +
      ' / ' +
      classStartTime +
      ' ~ ' +
      moment(classStartTime, 'HH:mm').add(classLength, 'm').format('HH:mm')
    );
  }

  /**
   * 表头
   */
  formatColumns() {
    const { productTypeId } = this.props;

    let columns = null;
    if (productTypeId === TWO_TEACHER_VAL) {
      columns = [
        { title: '班课名称', dataIndex: 'className', key: 'className', width: COL_WIDTH_M },
        { title: '期数', dataIndex: 'phase', key: 'phase', width: COL_WIDTH_S, render: (data) => data || '无' },
        { title: '学习中心', dataIndex: 'blCampusName', key: 'blCampusName', width: COL_WIDTH_M },
        { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: COL_WIDTH_S },
        { title: '科目', dataIndex: 'subjectName', key: 'subjectName', width: COL_WIDTH_S },
        { title: '班级层次', dataIndex: 'classLevelName', key: 'classLevelName', width: COL_WIDTH_M },
        {
          title: '上课日期',
          dataIndex: 'classStartDate',
          key: 'classStartDate',
          width: 100,
          render: (data, record) => data + (record.classEndDate ? ' ~ ' + record.classEndDate : ''),
        },
        {
          title: '上课时间',
          dataIndex: 'classStartTime',
          key: 'classStartTime',
          width: COL_WIDTH_M,
          render: (data, record) => {
            const { classStartTime, classLength } = record;
            return classStartTime + ' ~ ' + moment(classStartTime, 'HH:mm').add(classLength, 'm').format('HH:mm');
          },
        },
        {
          title: productTypeId === TWO_TEACHER_VAL ? '主讲老师' : '授课老师',
          dataIndex: 'mainTeacherName',
          key: 'mainTeacherName',
          width: 85,
        },
      ];
    } else {
      columns = [
        { title: '班课名称', dataIndex: 'className', key: 'className', width: COL_WIDTH_M },
        { title: '期数', dataIndex: 'phaseName', key: 'phaseName', width: COL_WIDTH_S, render: (data) => data || '无' },
        { title: '学习中心', dataIndex: 'blCampusName', key: 'blCampusName', width: COL_WIDTH_M },
        { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: COL_WIDTH_S },
        { title: '科目', dataIndex: 'subjectName', key: 'subjectName', width: COL_WIDTH_S },
        { title: '班级层次', dataIndex: 'classLevelName', key: 'classLevelName', width: COL_WIDTH_M },
        {
          title: '上课日期',
          dataIndex: 'classStartDate',
          key: 'classStartDate',
          width: 100,
          render: (data, record) => {
            return data + (record.classEndDate ? ' ~ ' + record.classEndDate : '');
          },
        },
        {
          title: '上课时间',
          dataIndex: 'classStartTime',
          key: 'classStartTime',
          width: COL_WIDTH_M,
          render: (data, record) => {
            const { classStartTime, classLength } = record;
            return classStartTime + ' ~ ' + moment(classStartTime, 'HH:mm').add(classLength, 'm').format('HH:mm');
          },
        },
        {
          title: productTypeId === TWO_TEACHER_VAL ? '主讲老师' : '授课老师',
          dataIndex: 'mainTeacherName',
          key: 'mainTeacherName',
          width: 85,
        },
      ];
    }

    if (productTypeId === TWO_TEACHER_VAL) {
      columns.splice(columns.length - 1, 0, {
        title: '辅导老师',
        dataIndex: 'assistTeacherName',
        key: 'assistTeacherName',
        width: 85,
      });
    }

    return columns;
  }

  /**
   * 1、学习中心和季度支持多选
   * 2、年份、季度、年级默认选中项需要做 +1 处理
   * 3、班主任和老师默认不选中
   */
  render() {
    const {
      loading,
      listLoading,
      searchKey,
      rows,
      page,
      pageSize,
      records,
      mainTcDict,
      assistTcDict,
      campusDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjectDict,
      selectedRowKeys,
      classLevelDict,
    } = this.state;
    const { onCancel, productTypeId, quarterName } = this.props;
    const { blCampusId, yearId, gradeId, quarterId, subjectId, classLevel, mainTeacherId, assistTeacherId } =
      this.props;
    const selectTitle = productTypeId === SMALL_CLASS_VAL ? '小班' : '双师辅班';

    // 筛选条件
    const condition = {
      yearDict: yearDict,
      quarterDict: quarterDict,
      gradeDict: gradeDict,
      subjectDict: subjectDict,
      classLevelDict: classLevelDict,
      campusDict: campusDict,
      mainTeacherDict: mainTcDict,
      subTeacherDict: assistTcDict,
    };

    // 默认选中数据
    const selectedData = {
      yearId: (yearId && auxBuildVersion({ yearId, quarter: quarterName }, yearDict)) || null,
      quarterId: (quarterId && auxBuildQuarter({ quarter: quarterName }, quarterDict)) || null,
      gradeId: (gradeId && auxBuildGrade({ gradeId, quarter: quarterName }, gradeDict)) || null,
      subjectId,
      classLevel,
      blCampusId: (blCampusId && [blCampusId]) || null,
    };
    const isSmallClass = productTypeId === SMALL_CLASS_VAL;

    return (
      <Modal
        width={1150}
        closable={true}
        loading={loading}
        onCancel={onCancel}
        okBtnShow={false}
        cancelText="关闭"
        route={this.props.route}
        title={
          <p>
            <b>选择续报{selectTitle}&nbsp;&nbsp;</b>(原班:{this.getOriginClassName()})
          </p>
        }
        footer={
          <Button type="primary" onClick={this.handleConfirm}>
            确认选择
          </Button>
        }
      >
        <Block type="dark">
          <SearchFeat
            key={searchKey}
            condition={condition}
            selectedData={selectedData}
            isSmallClass={isSmallClass}
            handleSearch={this.onSearch}
            onRef={this.onSearchRef}
          />
        </Block>

        <List
          columns={this.formatColumns()}
          loading={listLoading}
          dataSource={rows}
          pagination={{
            current: page,
            pageSize,
            total: records,
          }}
          onChange={this.onPageChange}
          rowSelection={{
            selectedRowKeys,
            onChange: (keys, rows) => {
              this.setState({ selectedRowKeys: keys, selectedRows: rows });
            },
          }}
        />
      </Modal>
    );
  }
}

export default AddClassModal;
