import React from 'react';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Block from 'COMPONENT/Block';
import Page from 'COMPONENT/Page';
import Button from 'COMPONENT/Button';
import List from 'COMPONENT/List';
import Modal from 'COMPONENT/Modal';
import styles from './style.less';
import { urlBase } from 'CONST/config';
import moment from 'moment';
import { OSS_PRD_PREFIX } from 'CONST/ossConfig';

const DATEFORMAT = 'YYYY-MM-DD HH:mm';

const successAlert = Modal.success;

import Loaders from '../loader';
import utils from './util';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

export default class ImportLink extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // 页面加载状态
      // 列表数据
      listLoading: false,
      rows: [],
      currentPage: 1,
      pageSize: 20,
      totalRecords: 0,
    };
    // 防止多次点击
    this.limitClickTag = 1;
  }

  columns = [
    { title: '文件名称', dataIndex: 'realFileName', width: COL_WIDTH_XL },
    { title: '成功数量', dataIndex: 'importSuccessNum', width: COL_WIDTH_S },
    { title: '重复数量', dataIndex: 'importRepeatNum', width: COL_WIDTH_S },
    {
      title: '未匹配数量',
      dataIndex: 'importNotMatchNum',
      width: COL_WIDTH_S,
      render: (text, record) => {
        if (Number(text) && Number(text) > 0) {
          return (
            <div>
              <span>{text}</span>
              <span
                style={{ color: '#6d67ff', marginLeft: '5px', cursor: 'pointer' }}
                onClick={() => this.exportNotMatch(record)}
              >
                导出
              </span>
            </div>
          );
        }
        return text;
      },
    },
    { title: '上传者', dataIndex: 'createUser', width: COL_WIDTH_M },
    {
      title: '上传时间',
      dataIndex: 'createTime',
      width: COL_WIDTH_M,
      render: (text, record) => moment(record.createTime).format(DATEFORMAT),
    },
  ];

  componentDidMount() {
    this.loadListData();
  }

  loadListData = async () => {
    const { currentPage, pageSize } = this.state;
    this.updateState({ loading: true });
    const list = await Loaders.getImportLink({
      page: currentPage,
      rows: pageSize,
    });
    this.updateState({
      loading: false,
      rows: list.datas,
      totalRecords: list.rowCount,
    });
  };
  /**
   * 导入关联关系
   */
  importLink = async (event) => {
    let file = event.target.files[0];
    const name = file.name;
    const idx = name.lastIndexOf('.');
    const nameExtend = name.slice(idx + 1);
    if (nameExtend.toLowerCase() !== 'xlsx' && nameExtend.toLowerCase() !== 'xls') {
      return showMessage('文件格式不正确');
    }

    let formData = new FormData();
    formData.append('file', file);
    let xhr = new XMLHttpRequest();
    xhr.open('POST', urlBase + '/OriginalClassController/importOriginalSeriesClassInfo.do', true);
    xhr.upload.onloadstart = (file) => {
      this.setState({ loading: true });
    };
    xhr.onload = async (res) => {
      if (res.target.status === 200) {
        const response = JSON.parse(res.target.response);
        if (response.resultCode === 0) {
          successAlert({
            content: (
              <div>
                <p className={styles.mainInfo}>关联关系导入成功</p>
                <p className={styles.secInfo}>请核对未匹配的数据进行导出修改</p>
              </div>
            ),
          });
          await this.updateState({ currentPage: 1 });
          this.loadListData();
        } else {
          showMessage(response.resultMessage);
        }
      } else {
        showMessage(JSON.parse(res.target.response).resultMessage);
      }
      this.setState({ loading: false });
    };
    xhr.onerror = (e) => {
      showMessage('上传失败');
      this.setState({ loading: false });
    };
    xhr.send(formData);
  };

  /**
   * 下载模板
   * @returns {Promise<number>}
   */
  downloadDemo = async () => {
    if (this.limitClickTag === 1) {
      this.limitClickTag = 2;
      try {
        Loaders.exportDemo({
          href: `${OSS_PRD_PREFIX}/xx%e5%88%86%e5%85%ac%e5%8f%b8xx%e6%a0%a1%e5%8c%ba%e5%af%bc%e5%85%a5%e7%bb%ad%e6%8a%a5%e5%85%b3%e8%81%94Excel%e6%a8%a1%e6%9d%bf.xlsx`,
          download: '导入续报关联Excel模板.xlsx',
        });
      } catch (e) {
        showMessage(e.text);
        return (this.limitClickTag = 1);
      }
      this.limitClickTag = 1;
    }
  };

  /*
   *  导出未匹配列表
   * */
  exportNotMatch = async (record) => {
    const { realFileName, id } = record;
    const res = await Loaders.getUnMatchList({ importId: id });
    const href = res.data;
    const download = `${realFileName}-未匹配列表`;
    Loaders.exportDemo({ href, download });
  };

  /**
   * 导出excel
   * */
  async exportExcel(fetchUrl, fileOriginName, record) {
    await this.updateState({ loading: true });
    const fetchParams = {
      importId: record.id,
    };
    let fileName = `${fileOriginName}`;
    const _self = this;
    utils.exportExcel({
      apiUrl: fetchUrl,
      data: fetchParams,
      fileName: fileName,
      success: () => {
        _self.updateState({ loading: false });
        showMessage('导出成功！', 'success');
      },
      fail: (err) => {
        _self.updateState({ loading: false });
        showMessage(err.text, 'error');
      },
    });
  }

  /**
   * 翻页
   */
  pageChange = async ({ current, pageSize } = {}) => {
    await this.updateState({ pageSize, currentPage: current });
    this.loadListData();
  };

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  render() {
    const { loading, listLoading, rows, currentPage, pageSize, totalRecords } = this.state;
    return (
      <Page name="导入关联关系" loading={loading} route={this.props.route} ref={(c) => (this._page = c)}>
        <Block title="导入关联关系">
          <Button style={{ marginRight: '1em', position: 'relative' }}>
            导入Excel
            <input
              type="file"
              className={styles.importBtn}
              onClick={(event) => {
                event.target.value = null;
              }}
              onChange={this.importLink}
            />
          </Button>
          <Button onClick={this.downloadDemo}>模板下载</Button>
        </Block>
        <Block title="导入记录">
          <List
            columns={this.columns}
            loading={listLoading}
            dataSource={rows}
            pagination={{
              current: currentPage,
              pageSize,
              total: totalRecords,
              pageSizeOptions: ['10', '20', '50'],
            }}
            onChange={this.pageChange}
          />
        </Block>
      </Page>
    );
  }
}
