/**
 * 小班信息卡片
 * @property {[Object]} data 小班数据，格式如下：
 * {
    id: string  小班ID
    name: string  小班名称
    quarter: string  季度
    weekDay: [string]  星期
    phase: string  期
    statusName: string  小班状态名称
    scheduleStatus: string  排课状态
    scheduleMode: string  排课方式
    startTime: string  上课开始时间
    courseMinute: number 课程分钟数
    endTime: string  上课结束时间
    template: string  模板ID
    templateName: string  模板名称
    classroom: string  教室ID
    classroomName: string  教室名称
    teacher: string  老师ID
    teacherName: string  老师姓名
    totalCount: number  招生人数
    signedCount: number  已报班人数
    campus: string  学习中心
    campusName: string  学习中心名称
    year: string  年份
    classCount: number  讲数
    grade: string  年级
    subject: string  科目
    classTypeName: string  小班类型名称
    createTime: string  创建时间
  }
 * @property {boolean} classroomConflict  是否存在教室冲突
 * @property {boolean} teacherConflict  是否存在老师冲突
 * @property {string} branchName  分公司名称
 * @property {boolean} isActive  是否激活状态
 * @property {[object]} templateDict  当前编辑的小班的模板字典
 * @property {[object]} classroomDict  当前编辑的小班的教室字典
 * @property {[object]} teacherDict  当前编辑的小班的老师字典
 * @property {boolean} isDirty  是否有修改未保存
 * @property {boolean} isChunOrQiu  是否春秋季
 * @property {function(string)} onSelectClass  选中小班时的回调
 * @property {function(object)} onDataChange  当前编辑的小班数据改变时的回调
 * @property {function()} onDeleteSchedule  删除排课时的回调
 * @property {function()} onSaveSchedule  保存排课时的回调
 */
import React from 'react';
import ReactDOM from 'react-dom';
import { Row, Col, Popconfirm, Button, Icon } from 'antd';
import Popover from 'COMPONENT/Popover';
import Select from 'COMPONENT/Select';
import TimePicker from 'COMPONENT/TimePicker';
import { getDictTextByVal, getDictOptionByVal, formatGroupDict } from 'UTIL/dict';
import { momentToTime, timeToMoment } from 'COMPONENT/utils/form';
import { weekDayDict } from 'CONST/dicts/common';
import { classTypeDict, otherTeacherOption } from '../dicts';
import styles from './styles.less';
import mode1Img from '../imgs/moban.png';
import mode2Img from '../imgs/ziyou.png';
import status1Img from '../imgs/weipai.png';
import status2Img from '../imgs/yipai.png';
import conflictImg from '../imgs/chongtu.png';

/**
 * 按钮提示信息气泡的默认最大宽度
 */
const btnPopMaxW = 120;

class ClassCard extends React.Component {
  constructor(props) {
    super(props);
    const { template, templateDict } = this.props;
    let templateStartTime;
    if (template && templateDict) {
      const opt = templateDict.find((t) => t.val === template);
      if (opt) {
        templateStartTime = opt.startTime;
      }
    }
    this.state = {
      templateActive: false, //模板选框是否激活
      classroomActive: false, //教室选框是否激活
      teacherActive: false, //老师选框是否激活
      saveBtnError: null, //保存按钮的错误信息
      deleteBtnError: null, //删除按钮的错误信息
      saveBtnConfirmVisible: false, //保存按钮确认弹框的显示状态
      deleteBtnConfirmVisible: false, //删除按钮确认弹框的显示状态
      $popupContainer: document, //承载弹出层的元素
      startTimeEditable: templateStartTime ? false : true, //开始时间是否可编辑
    };
  }

  componentDidMount() {
    this.setState({ $popupContainer: this.$wrap });
  }

  shouldComponentUpdate(nextProps, nextState) {
    if (
      Object.keys(this.props).find((key) => this.props[key] !== nextProps[key]) ||
      Object.keys(this.state).find((key) => this.state[key] !== nextState[key])
    ) {
      return true;
    }
    return false;
  }

  componentDidUpdate(prevProps, prevState) {
    const isActivePrev = prevProps.isActive;
    if (isActivePrev && !this.props.isActive) {
      this.setState({ templateActive: false, classroomActive: false, teacherActive: false });
    }
  }

  /**
   * 保存排课按钮显示错误信息
   * @param  {string} text 错误信息内容
   */
  showSaveError = (text) => {
    this.setState({ saveBtnError: text });
  };

  /**
   * 删除排课按钮显示错误信息
   * @param  {string} text 错误信息内容
   */
  showDeleteError = (text) => {
    console.log('deleteBtnError: ', text);
    this.setState({ deleteBtnError: text });
  };

  /**
   * 开始时间禁用的小时（禁用23:00-05:59）
   * @return {number[]}
   */
  startTimeDisabledHours = () => {
    return [23, 0, 1, 2, 3, 4, 5];
  };

  /**
   * 获取结束时间
   * @param {string} startTime - 开始时间
   * @return {string}
   */
  getEndTime = (startTime) => {
    if (!startTime) {
      return null;
    }
    const {
      data: { courseMinute },
    } = this.props;
    return momentToTime(timeToMoment(startTime).add(courseMinute, 'm'));
  };

  render() {
    const {
      data: {
        id,
        name,
        weekDay,
        classTypeName,
        statusName,
        scheduleStatus,
        scheduleMode,
        startTime,
        endTime,
        template,
        templateName,
        classroom,
        classroomName,
        teacher,
        teacherName,
        totalCount,
        signedCount,
        campusName,
        productGroupId,
      },
      classroomConflict,
      teacherConflict,
      branchName,
      isActive,
      templateDict: templateDictProp,
      classroomDict: classroomDictProp,
      teacherDict: teacherDictProp,
      isDirty,
      isChunOrQiu,
      onSelectClass,
      onDataChange,
      onDeleteSchedule,
      onSaveSchedule,
    } = this.props;

    const {
      templateActive,
      classroomActive,
      teacherActive,
      saveBtnError,
      deleteBtnError,
      saveBtnConfirmVisible,
      deleteBtnConfirmVisible,
      $popupContainer,
      startTimeEditable,
    } = this.state;
    const title = `[小班]${name}`;
    const templateDict =
      isActive && templateActive && templateDictProp
        ? formatGroupDict(templateDictProp)
        : [{ val: template, text: templateName, title: templateName }];
    const classroomDict = isActive && classroomActive ? classroomDictProp : [{ val: classroom, text: classroomName }];
    const teacherDict =
      isActive && teacherActive && teacherDictProp
        ? teacherDictProp
        : [{ val: teacher, text: teacherName }, otherTeacherOption];

    const productGroupEnum = {
      ONLINE_CLASS: '本地网课',
      PRODUCT_GROUP_CLASSROOM_TEACHING: '面授',
      LIVE_CLASS: '在线A',
      VIDEO_CLASS: '在线B',
      FAQ_CLASS: '自习',
      PRODUCT_GROUP_HL: 'AI录播课',
    };
    const productGroupName = productGroupEnum[productGroupId] || '面授';
    let initialTeacher = teacher;
    if (isActive) {
      try {
        const allTeacherValList = (teacherDictProp || []).reduce(
          (pre, curr) => pre.concat(curr.options.map((r) => r.val)),
          [],
        );
        if (!allTeacherValList.includes(teacher)) {
          initialTeacher = '';
        }
      } catch (e) {}
    }

    const hasSaveConfirm = scheduleMode === '2';
    let saveBtn = (
      <Button
        type="primary"
        style={{ marginLeft: 16 }}
        onClick={hasSaveConfirm ? null : onSaveSchedule}
        disabled={
          (scheduleStatus === '2' && !isDirty) ||
          statusName === '上课中' ||
          (statusName === '已完成' && scheduleMode === '2')
        }
      >
        {scheduleStatus === '2' && isDirty ? '重新排课' : '排入课表'}
      </Button>
    );
    if (!saveBtnConfirmVisible) {
      if (saveBtnError) {
        saveBtn = (
          <Popover
            key="saveBtnErrorPop"
            icon="error"
            maxWidth={btnPopMaxW}
            trigger="click"
            visible={saveBtnError ? true : false}
            content={saveBtnError}
            onClose={() => this.setState({ saveBtnError: null })}
          >
            {saveBtn}
          </Popover>
        );
      } else {
        let saveBtnWarning;
        if (statusName === '上课中') {
          saveBtnWarning = '已上课小班，调课请到课程列表';
        }
        if (statusName === '已完成' && scheduleMode === '2') {
          saveBtnWarning = '已完成的小班，不能转模板排课';
        }
        if (saveBtnWarning) {
          saveBtn = (
            <Popover key="saveBtnWarningPop" icon="warning" content={saveBtnWarning} maxWidth={btnPopMaxW}>
              {saveBtn}
            </Popover>
          );
        }
      }
    }
    if (hasSaveConfirm) {
      saveBtn = (
        <Popconfirm
          title={<div style={{ maxWidth: btnPopMaxW }}>使用模板排课，已排的课程将覆盖，是否继续使用排课模板？</div>}
          onConfirm={onSaveSchedule}
          okText="确定"
          cancelText="取消"
          onVisibleChange={(v) => this.setState({ saveBtnConfirmVisible: v })}
        >
          {saveBtn}
        </Popconfirm>
      );
    }

    const hasDeleteConfirm = scheduleStatus === '2' && signedCount === 0;
    let deleteBtn = (
      <Button onClick={hasDeleteConfirm ? null : onDeleteSchedule} disabled={scheduleStatus === '1' || signedCount > 0}>
        删除排课
      </Button>
    );
    if (!deleteBtnConfirmVisible) {
      if (deleteBtnError) {
        deleteBtn = (
          <Popover
            key="deleteBtnErrorPop"
            icon="error"
            maxWidth={btnPopMaxW}
            trigger="click"
            visible={deleteBtnError ? true : false}
            content={deleteBtnError}
            onClose={() => this.setState({ deleteBtnError: null })}
          >
            {deleteBtn}
          </Popover>
        );
      } else {
        let deleteBtnWarning;
        if (scheduleStatus === '1') {
          deleteBtnWarning = '还没有排课';
        } else if (signedCount > 0) {
          deleteBtnWarning = '已有学生报名不能删除';
        }
        if (deleteBtnWarning) {
          deleteBtn = (
            <Popover key="deleteBtnWarningPop" icon="warning" content={deleteBtnWarning} maxWidth={btnPopMaxW}>
              {deleteBtn}
            </Popover>
          );
        }
      }
    }
    if (hasDeleteConfirm) {
      deleteBtn = (
        <Popconfirm
          title={<div style={{ maxWidth: btnPopMaxW }}>确定删除排课信息？删除后该小班将没有排课信息</div>}
          onConfirm={onDeleteSchedule}
          okText="删除"
          cancelText="取消"
        >
          {deleteBtn}
        </Popconfirm>
      );
    }

    let weekDayNames;
    if (isChunOrQiu) {
      weekDayNames = [];
      weekDayDict.forEach((d) => {
        if (weekDay.indexOf(d.val) >= 0) {
          weekDayNames.push(d.text);
        }
      });
      weekDayNames = weekDayNames.join('、');
    }

    let conflictNames = [];
    if (classroomConflict) {
      conflictNames.push('教室');
    }
    if (teacherConflict) {
      conflictNames.push('老师');
    }

    return (
      <div
        className={styles.wrap + (isActive ? ' ' + styles.active : '')}
        ref={(c) => (this.$wrap = ReactDOM.findDOMNode(c))}
      >
        {!isActive && <div className={styles.mask} onClick={() => onSelectClass(id)}></div>}
        <div className={styles.title} title={title}>
          {title}
        </div>
        {(classroomConflict || teacherConflict) && (
          <div className={styles.flag + ' ' + styles.flag0}>
            <img src={conflictImg} />
          </div>
        )}
        {scheduleStatus === '2' && (
          <div className={styles.flag + ' ' + styles.flag1}>
            <img src={scheduleMode === '1' ? mode1Img : mode2Img} />
          </div>
        )}
        <div className={styles.flag + ' ' + styles.flag2}>
          <img src={scheduleStatus === '1' ? status1Img : status2Img} />
        </div>
        <div className={styles.info}>{`已报人数 ${signedCount}/${totalCount}  ${statusName}`}</div>
        <div className={styles.tags}>
          <div className={styles.tag}>{branchName}</div>
          <div className={styles.tag}>{campusName}</div>
          <div className={styles.tag}>{classTypeName}</div>
          <div className={styles.tag}>{productGroupName}</div>
        </div>
        <div className={styles.inputs}>
          {/* <Row className={styles.inputRow}>
            <Col span={6} className={styles.label + ' '+ styles.required}>
              开始时间
            </Col>
            <Col span={18}>
              <TimePicker
                disabled={true}
                value={startTime}
                onChange={time => {
                  onDataChange({
                    startTime: time,
                    endTime: this.getEndTime(time),
                  })
                }}
                disabledHours={this.startTimeDisabledHours}
                getPopupContainer={() => $popupContainer}
                // disabled={!startTimeEditable}
              />
              {startTime &&
                <div className={styles.rowInfo}>
                  {`${startTime} ~ ${endTime}`}
                </div>
              }
            </Col>
          </Row> */}
          <Row className={styles.inputRow}>
            <Col span={6} className={styles.label + ' ' + styles.required}>
              课程模板
            </Col>
            <Col span={18}>
              <Select
                dict={templateDict}
                value={template}
                onChange={(val) => {
                  const opt = getDictOptionByVal(templateDict, val);
                  let changedData = { template: val };
                  if (opt) {
                    changedData.templateName = opt.text;
                    if (isChunOrQiu) {
                      changedData.weekDay = opt.weekDay;
                    } else {
                      changedData.phase = opt.phase;
                    }
                  } else {
                    changedData.templateName = null;
                    changedData.weekDay = [];
                    changedData.phase = null;
                  }
                  let startTimeEditable;
                  if (opt && opt.startTime) {
                    startTimeEditable = false;
                    changedData.startTime = opt.startTime;
                    changedData.endTime = this.getEndTime(opt.startTime);
                  } else {
                    startTimeEditable = true;
                  }
                  this.setState({ startTimeEditable });
                  onDataChange(changedData);
                }}
                onClick={() => this.setState({ templateActive: true })}
                getPopupContainer={() => $popupContainer}
                showSearch
                title={getDictTextByVal(templateDict, template)}
              />
              {isChunOrQiu && weekDayNames && <div className={styles.rowInfo}>{`上课星期：${weekDayNames}`}</div>}
            </Col>
          </Row>
          <Row className={styles.inputRow}>
            <Col span={6} className={styles.label + ' ' + styles.required}>
              教&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;室
            </Col>
            <Col span={18}>
              <Select
                dict={classroomDict}
                value={classroom}
                onChange={(val) => {
                  onDataChange({
                    classroom: val,
                    classroomName: getDictTextByVal(classroomDict, val),
                  });
                }}
                showSearch={true}
                onClick={() => this.setState({ classroomActive: true })}
                getPopupContainer={() => $popupContainer}
              />
            </Col>
          </Row>
          <Row className={styles.inputRow}>
            <Col span={6} className={styles.label + ' ' + styles.required}>
              老&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;师
            </Col>
            <Col span={18}>
              <Select
                dict={teacherDict}
                value={initialTeacher}
                onChange={(val) => {
                  onDataChange({
                    teacher: val,
                    teacherName: getDictTextByVal(teacherDict, val),
                  });
                }}
                showSearch={true}
                onClick={() => this.setState({ teacherActive: true })}
                getPopupContainer={() => $popupContainer}
              />
            </Col>
          </Row>
          {conflictNames.length > 0 && (
            <div className={styles.conflictTips}>{conflictNames.join('、') + '存在冲突'}</div>
          )}
        </div>
        <div className={styles.footer}>
          {deleteBtn}
          <div className={styles.footerRight}>{saveBtn}</div>
        </div>
      </div>
    );
  }
}

export default ClassCard;
