/**
 * 小班列表面板
 * @property {string} classTitle  小班标题输入值
 * @property {string} scheduleStatus  排课状态输入值
 * @property {string} scheduleMode  排课方式输入值
 * @property {[string]} visibleClassListIds  可见的小班列表ID
 * @property {boolean} filterBarExpand  筛选栏展开状态
 * @property {[object]} classList  小班列表
 * @property {object} editingClass  正在编辑的小班数据
 * @property {[object]} classListForTeacher  老师标签页下的结点使用的小班数据列表
 * @property {function(object)} onUpdateState  更新父组件状态时的回调
 * @property {string} branchName  分公司名称
 * @property {[object]} templateDict  当前编辑的小班的模板字典
 * @property {[object]} classroomDict  当前编辑的小班的教室字典
 * @property {[object]} teacherDict  当前编辑的小班的老师字典
 * @property {boolean} isDirty  是否有修改未保存
 * @property {boolean} isChunOrQiu  是否春秋季
 * @property {function(string)} onSelectClass  选中小班时的回调
 * @property {function(object)} onDataChange  当前编辑的小班数据改变时的回调
 * @property {function()} onDeleteSchedule  删除排课时的回调
 * @property {function()} onSaveSchedule  保存排课时的回调
 */
import React from 'react';
import ReactDOM from 'react-dom';
import { Input, Row, Col, Icon } from 'antd';
import Select from 'COMPONENT/Select';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import ClassCard from '../ClassCard';
import { getVisibleClassList, checkTimeRangeConflict, checkTemplateSchedule } from '../common';
import { scheduleStatusDict, scheduleModeDict } from '../dicts';
import styles from './styles.less';
import filterImg from '../imgs/filter.png';
import filterActiveImg from '../imgs/filter_active.png';

const Search = Input.Search;

class ClassPanel extends React.Component {
  constructor(props) {
    super(props);
    this.$classCards = [];
  }

  shouldComponentUpdate(nextProps, nextState) {
    if (Object.keys(this.props).find((key) => this.props[key] !== nextProps[key])) {
      return true;
    }
    return false;
  }

  /**
   * 定位小班列表面板滚动的位置
   * @param  {number} pos 滚动条距离顶端的位置
   */
  setScroll = (pos) => {
    this.$scroller.scrollTop = pos;
  };

  /**
   * 获取小班列表面板滚动的位置
   * @return {number}
   */
  getScroll = () => {
    return this.$scroller.scrollTop;
  };

  /**
   * 根据排课状态筛选
   * @param  {string} scheduleStatus 排课状态 [1 | 2]
   */
  filterByScheduleStatus = (scheduleStatus) => {
    const { classList, onUpdateState } = this.props;
    onUpdateState({ visibleClassListIds: getVisibleClassList(classList, { scheduleStatus }).map((c) => c.id) });
    this.setScroll(0);
  };

  /**
   * 根据冲突筛选
   * @param  {[string]} conflictIds 存在冲突的小班ID
   */
  filterByConflict = (conflictIds) => {
    const { classList, onUpdateState } = this.props;
    onUpdateState({ visibleClassListIds: classList.filter((c) => conflictIds.indexOf(c.id) >= 0).map((c) => c.id) });
    this.setScroll(0);
  };

  /**
   * 筛选
   */
  filter = () => {
    const { classList, onUpdateState, scheduleStatus, scheduleMode, classTitle } = this.props;
    onUpdateState({
      visibleClassListIds: getVisibleClassList(classList, { scheduleStatus, scheduleMode, classTitle }).map(
        (c) => c.id,
      ),
    });
    this.setScroll(0);
  };

  /**
   * 将指定小班定位到列表区域的最上方
   * @param  {string} id 小班ID
   */
  locateClass = (id) => {
    const { classList, visibleClassListIds } = this.props;
    const visibleClassList = classList.filter((c) => visibleClassListIds.indexOf(c.id) >= 0);
    let h = 0;
    for (let i = 0; i < visibleClassList.length; i++) {
      if (visibleClassList[i].id === id) {
        break;
      }
      h += this.$classCards[i].offsetHeight;
    }
    this.setScroll(h);
  };

  /**
   * 保存排课按钮显示错误信息
   * @param  {string} text 错误信息内容
   */
  showSaveError = (text) => {
    this._activeClassCard.showSaveError(text);
  };

  /**
   * 删除排课按钮显示错误信息
   * @param  {string} text 错误信息内容
   */
  showDeleteError = (text) => {
    this._activeClassCard.showDeleteError(text);
  };

  /**
   * 获得小班列表在另一个小板列表中存在冲突的所有小板ID
   * @param  {[object]} classList      小板列表
   * @param  {[object]} compareClassList 用于对比的小班列表
   * @param  {string} type 类型 [classroom | teacher]
   * @return {[string]}
   */
  getConflicts = (classList, compareClassList, type) => {
    let conflicts = [];
    classList.forEach((c) => {
      if (!checkTemplateSchedule(c)) {
        return;
      }
      if (conflicts.indexOf(c.id) < 0) {
        const hasConflict = compareClassList.find((cc) => {
          if (c.id === cc.id) {
            return false;
          }
          if (!checkTemplateSchedule(cc)) {
            return false;
          }
          if (c[type] !== cc[type]) {
            return false;
          }
          if (c.phase !== cc.phase) {
            return false;
          }
          if (c.weekDay) {
            if (!c.weekDay.find((w) => cc.weekDay.indexOf(w) >= 0)) {
              return false;
            }
          }
          if (!checkTimeRangeConflict(c, cc)) {
            return false;
          }
          return true;
        });
        if (hasConflict) {
          conflicts.push(c.id);
        }
      }
    });
    return conflicts;
  };

  render() {
    const {
      classTitle,
      scheduleStatus,
      scheduleMode,
      visibleClassListIds,
      filterBarExpand,
      classList: classListProp,
      editingClass,
      classListForTeacher,
      onUpdateState,
      ...props
    } = this.props;
    const editingClassId = editingClass ? editingClass.id : null;
    const classList = classListProp.map((c) => (c.id === editingClassId ? editingClass : c));
    const visibleClassList = classList.filter((c) => visibleClassListIds.indexOf(c.id) >= 0);
    const otherCampusClassList = classListForTeacher.filter((ct) => !classList.find((c) => c.id === ct.id));
    const classroomConflicts = this.getConflicts(classList, classList, 'classroom');
    const teacherConflicts = this.getConflicts(classList, [...classList, ...otherCampusClassList], 'teacher');
    const totalConflicts = [
      ...classroomConflicts,
      ...teacherConflicts.filter((c) => classroomConflicts.indexOf(c) < 0),
    ];

    const contentTop = filterBarExpand ? 142 : 89;

    return (
      <div className={styles.wrap}>
        <div className={styles.title}>
          <span>班课列表</span>
          <span className={styles.summary}>
            <span>（</span>
            <span className={styles.summaryItem}>
              <span>未排：</span>
              <a onClick={() => this.filterByScheduleStatus('1')}>
                {classList.filter((c) => c.scheduleStatus === '1').length}
              </a>
            </span>
            <span className={styles.summaryItem}>
              <span>已排：</span>
              <a onClick={() => this.filterByScheduleStatus('2')}>
                {classList.filter((c) => c.scheduleStatus === '2').length}
              </a>
            </span>
            <span className={styles.summaryItem}>
              <span>冲突：</span>
              <a onClick={() => this.filterByConflict(totalConflicts)}>{totalConflicts.length}</a>
            </span>
            <span>）</span>
          </span>
        </div>
        <div className={styles.filterBar}>
          <Row gutter={10}>
            <Col span={2}>
              <img
                className={styles.filterIcon}
                src={filterBarExpand ? filterActiveImg : filterImg}
                onClick={() => onUpdateState({ filterBarExpand: !filterBarExpand })}
              />
            </Col>
            <Col span={22}>
              <div className={styles.filterInput}>
                <Search
                  placeholder="请输入关键字"
                  onSearch={this.filter}
                  value={classTitle}
                  onChange={(e) => {
                    onUpdateState({ classTitle: e.target.value });
                  }}
                />
                <div className={styles.filterBtn} onClick={this.filter}>
                  <Icon type="search" />
                </div>
              </div>
            </Col>
          </Row>
          {filterBarExpand && (
            <div className={styles.filterMore}>
              <Row gutter={10}>
                <Col span={12}>
                  <Select
                    placeholder="排课状态"
                    dict={scheduleStatusDict}
                    value={scheduleStatus}
                    onChange={(val) => {
                      onUpdateState({ scheduleStatus: val });
                    }}
                  />
                </Col>
                <Col span={12}>
                  <Select
                    placeholder="排课方式"
                    dict={scheduleModeDict}
                    value={scheduleMode}
                    onChange={(val) => {
                      onUpdateState({ scheduleMode: val });
                    }}
                  />
                </Col>
              </Row>
            </div>
          )}
        </div>
        <div
          className={styles.classPanelContent}
          style={{ top: contentTop }}
          ref={(c) => (this.$scroller = ReactDOM.findDOMNode(c))}
        >
          {visibleClassList.length > 0 ? (
            visibleClassList.map((c, i) => {
              return (
                <ClassCard
                  key={c.id}
                  data={c}
                  classroomConflict={classroomConflicts.indexOf(c.id) >= 0}
                  teacherConflict={teacherConflicts.indexOf(c.id) >= 0}
                  isActive={editingClassId === c.id}
                  {...props}
                  ref={(comp) => {
                    if (editingClassId === c.id) {
                      this._activeClassCard = comp;
                    }
                    this.$classCards[i] = ReactDOM.findDOMNode(comp);
                  }}
                />
              );
            })
          ) : (
            <div className={styles.noDataTips}>
              与排课大表年份、季度、分公司、学习中心筛选项条件一致的小班才能被搜出，请修改筛选条件后再搜索
            </div>
          )}
        </div>
      </div>
    );
  }
}

export default ClassPanel;
