/**
 * 排课表
 * @property {string} tabKey 当前标签页 [ classroom | teacher ]]
 * @property {[object]} rowDict 行字典
 * @property {[object]} colDict 列字典
 * @property {[object]} nodeGroups 结点组列表
 * @property {boolean} hasAnimi 切换结点组的展开状态时是否有动画效果
 * @property {function(string)} onChangeTab 切换标签页时的回调，参数为标签页Key
 * @property {function(number)} onToggleNodeGroupExpand 切换结点组展开状态时的回调，参数为结点组ID
 * @property {string} activeClassId 当前激活状态的小班ID
 * @property {function(string)} onSelectNode 选中结点时的回调，参数为结点对应的小班ID
 */
import React from 'react';
import ReactDOM from 'react-dom';
import ScheduleBoardHeaderCell from '../ScheduleBoardHeaderCell';
import ScheduleBoardRow from '../ScheduleBoardRow';
import { dayW, checkIsChunOrQiu, getPxByClockTime, getIdxInDict } from '../common';
import styles from './styles.less';

class ScheduleBoard extends React.Component {
  componentDidMount() {
    this.$rightBottomScroller.addEventListener('scroll', this.handleScroll, false);
  }

  componentWillUnmount() {
    this.$rightBottomScroller.removeEventListener('scroll', this.handleScroll, false);
  }

  /**
   * 排课表主体区域滚动时，顶部和左侧的表头随之左右和上下滚动
   */
  handleScroll = () => {
    this.$rightTopScroller.style.left = -this.$rightBottomScroller.scrollLeft + 'px';
    this.$leftBottomScroller.style.top = -this.$rightBottomScroller.scrollTop + 'px';
  };

  /**
   * 将指定点定位到排课表的左上方
   * @param  {string} row 行
   * @param  {string} col 列
   * @param  {string} time 时间点
   */
  locate = (row, col, time) => {
    const { rowDict, colDict } = this.props;
    const left = getIdxInDict(col, colDict) * dayW + getPxByClockTime(time);
    const rowIdx = getIdxInDict(row, rowDict);
    let top = 0;
    for (let i = 0; i < rowIdx; i++) {
      top += this.rowHeights[i];
    }
    this.setVScroll(top);
    this.setHScroll(left);
  };

  /**
   * 定位排课表的纵向滚动的位置，要分别定位主体区域和左侧表头区域
   * @param  {number} pos 滚动条距离顶端的位置
   */
  setVScroll = (pos) => {
    this.$rightBottomScroller.scrollTop = pos;
    this.$leftBottomScroller.style.top = -pos + 'px';
  };

  /**
   * 定位排课表的横向滚动的位置，要分别定位主体区域和顶部表头区域
   * @param  {number} pos 滚动条距离左端的位置
   */
  setHScroll = (pos) => {
    this.$rightBottomScroller.scrollLeft = pos;
    this.$rightTopScroller.style.left = -pos + 'px';
  };

  /**
   * 获取纵向滚动条位置
   * @return {number}
   */
  getVScroll = () => {
    return this.$rightBottomScroller.scrollTop;
  };

  /**
   * 获取横向滚动条位置
   * @return {number}
   */
  getHScroll = () => {
    return this.$rightBottomScroller.scrollLeft;
  };

  render() {
    const { tabKey, rowDict, colDict, nodeGroups, onChangeTab, hasAnimi, ...props } = this.props;
    const totalW = dayW * colDict.length;
    const defaultRowH = 100;
    const defaultNodeH = 94;
    let rowHeights = rowDict.map((d) => {
      let expandNodeCount = 1;
      nodeGroups
        .filter((g) => g.row === d.val)
        .forEach(({ nodes, expand }) => {
          if (expand && nodes.length > expandNodeCount) {
            expandNodeCount = nodes.length;
          }
        });
      if (expandNodeCount === 1) {
        return defaultRowH;
      } else {
        return expandNodeCount * defaultNodeH + 29 + 5;
      }
    });
    this.rowHeights = rowHeights;

    return (
      <div className={styles.wrap}>
        <div className={styles.leftTop}>
          <span
            className={styles.tabBtn + (tabKey === 'teacher' ? ' ' + styles.tabBtnActive : '')}
            onClick={() => onChangeTab('teacher')}
          >
            老师
          </span>
          <span className={styles.tabBtnSplit}>|</span>
          <span
            className={styles.tabBtn + (tabKey === 'classroom' ? ' ' + styles.tabBtnActive : '')}
            onClick={() => onChangeTab('classroom')}
          >
            教室
          </span>
        </div>
        <div className={styles.rightTop}>
          <div
            className={styles.scroller}
            style={{ width: totalW }}
            ref={(c) => (this.$rightTopScroller = ReactDOM.findDOMNode(c))}
          >
            {colDict.map((d, i) => (
              <ScheduleBoardHeaderCell key={d.val} title={d.text} />
            ))}
          </div>
        </div>
        <div className={styles.leftBottom}>
          <div className={styles.scroller} ref={(c) => (this.$leftBottomScroller = ReactDOM.findDOMNode(c))}>
            {tabKey === 'teacher'
              ? rowDict.map((d, i) => {
                  const { text } = d;
                  return (
                    <div
                      key={d.val}
                      className={
                        styles.rowHeaderItem + ' ' + styles.rowHeaderItemTeacher + (hasAnimi ? ' ' + styles.animi : '')
                      }
                      style={{ height: rowHeights[i] }}
                    >
                      {text}
                    </div>
                  );
                })
              : rowDict.map((d, i) => {
                  const { text, campusName, volume } = d;
                  return (
                    <div
                      key={d.val}
                      className={
                        styles.rowHeaderItem +
                        ' ' +
                        styles.rowHeaderItemClassroom +
                        (hasAnimi ? ' ' + styles.animi : '')
                      }
                      style={{ height: rowHeights[i] }}
                    >
                      <div>{text}</div>
                      <div>{campusName}</div>
                      <div>{volume + '人'}</div>
                    </div>
                  );
                })}
          </div>
        </div>
        <div className={styles.rightBottom} ref={(c) => (this.$rightBottomScroller = ReactDOM.findDOMNode(c))}>
          <div className={styles.scroller} style={{ width: totalW }}>
            {rowDict.map((row, i) => {
              return (
                <ScheduleBoardRow
                  key={row.val}
                  nodeGroups={nodeGroups.filter((g) => g.row === row.val)}
                  height={rowHeights[i]}
                  colDict={colDict}
                  tabKey={tabKey}
                  hasAnimi={hasAnimi}
                  {...props}
                />
              );
            })}
          </div>
        </div>
      </div>
    );
  }
}

export default ScheduleBoard;
