import moment from 'moment';
import { TIME_FORMAT } from 'COMPONENT/utils/form';

/**
 * 时间轴上一分钟的长度（px）
 */
const minuteW = 1.5;

/**
 * 时间轴上一天开始时间的分钟数
 */
const dayStartMinute = 5.5 * 60;

/**
 * 时间轴上一天结束时间的分钟数
 */
const dayEndMintue = 24.5 * 60;

/**
 * 时间轴上一天的宽度（px）
 */
const dayW = minuteW * (dayEndMintue - dayStartMinute);

/**
 * 根据钟表时间获取在时间轴上的位置（px）
 * @param  {string} clockTime 钟表时间 格式为'hh:mm'
 * @return {number}
 */
const getPxByClockTime = (clockTime) => {
  const a = clockTime.split(':');
  return (parseInt(a[0]) * 60 + parseInt(a[1]) - dayStartMinute) * minuteW;
};

/**
 * 判断季度是否为春秋
 * @param  {string} quarter 季度ID
 * @return {boolean}
 */
const checkIsChunOrQiu = (quarter) => {
  return quarter === 'DAT0000000231' || quarter === 'DAT0000000233';
};

/**
 * 获取一个值在字典中的顺序号
 * @param  {string} val 要查找的值
 * @param  {[object]} dict 字典
 * @return {number}
 */
const getIdxInDict = (val, dict) => {
  let idx = -1;
  dict.forEach((d, i) => {
    if (d.val === val) {
      idx = i;
    }
  });
  return idx;
};

/**
 * 替换全部字符串
 * @param  {string} str 被操作的字符串
 * @param  {string} s1 要被替换的字符串
 * @param  {string} s2 要替换成的字符串
 * @return {string}
 */
const replaceAll = (str, s1, s2) => {
  return str.replace(new RegExp(s1, 'gm'), s2);
};

/**
 * 获取当前展现的小班列表
 * @param {[object]} classList  原小班列表
 * @param {object} filterKeys  筛选条件
 * @return {[object]}
 */
const getVisibleClassList = (classList, filterKeys) => {
  let list = [...classList];
  if (filterKeys) {
    const { scheduleStatus, scheduleMode, classTitle } = filterKeys;
    list = list.filter((c) => {
      if (classTitle && c.name.indexOf(classTitle) < 0) {
        return false;
      }
      if (scheduleStatus && c.scheduleStatus !== scheduleStatus) {
        return false;
      }
      if (scheduleMode && c.scheduleMode !== scheduleMode) {
        return false;
      }
      return true;
    });
  }
  return list;
};

/**
 * 检验两个时间段是否冲突，时间段格式为：
 * {
 *   startTime string 开始时间 'HH:mm'
 *   endTime string 结束时间 'HH:mm'
 * }
 * @param  {object} timeRange1 时间段1
 * @param  {object} timeRange2 时间段2
 * @return {boolean}
 */
const checkTimeRangeConflict = (timeRange1, timeRange2) => {
  const start1 = moment(timeRange1.startTime, TIME_FORMAT).valueOf();
  const start2 = moment(timeRange2.startTime, TIME_FORMAT).valueOf();
  const end1 = moment(timeRange1.endTime, TIME_FORMAT).valueOf();
  const end2 = moment(timeRange2.endTime, TIME_FORMAT).valueOf();
  return (
    (start1 >= start2 && start1 < end2) ||
    (end1 > start2 && end1 <= end2) ||
    (start2 >= start1 && start2 < end1) ||
    (end2 > start1 && end2 <= end1)
  );
};

/**
 * 检验是否模板排课，条件为：1、排课状态为已排课  2、排课方式为模板排课
 * @param  {object} classData 小班数据
 * @return {boolean}
 */
const checkTemplateSchedule = (classData) => {
  const { scheduleStatus, scheduleMode } = classData;
  return scheduleStatus === '2' && scheduleMode === '1';
};

export default {
  minuteW,
  dayW,
  getPxByClockTime,
  checkIsChunOrQiu,
  getIdxInDict,
  replaceAll,
  getVisibleClassList,
  checkTimeRangeConflict,
  checkTemplateSchedule,
};
