import moment from 'moment';
import { TIME_FORMAT } from 'COMPONENT/utils/form';
import { getSelectOption, selectOrgByIdAndTypeOption, getClassroomByCampu } from 'SERVICE/common';
import {
  getAllCurrentClassroom,
  loadSmallTcApi,
  getSmallClassListOnModal,
  getCourseModalList,
  getSmallClassListByTeacher,
} from 'SERVICE/course';
import { checkIsChunOrQiu } from './common';
import { otherTeacherOption } from './dicts';

/**
 * 加载小班列表数据，格式如下：
    [{
      id: string  小班ID
      name: string  小班名称
      quarter: string  季度
      weekDay: [string]  星期
      phase: string  期
      statusName: string  小班状态名称
      scheduleStatus: string  排课状态
      scheduleMode: string  排课方式
      startTime: string  上课开始时间
      courseMinute: number 课程分钟数
      endTime: string  上课结束时间
      template: string  模板ID
      templateName: string  模板名称
      classroom: string  教室ID
      classroomName: string  教室名称
      teacher: string  老师ID
      teacherName: string  老师姓名
      totalCount: number  招生人数
      signedCount: number  已报班人数
      campus: string  学习中心
      year: string  年份
      classCount: number  讲数
      grade: string  年级
      subject: string  科目
      classTypeName: string  小班类型名称
      createTime: string  创建时间
    }]
 * @param  {Object} searchKeys 查询条件键值对 { type, branch, campus, year, quarter, phase, classroom, teacher }
 */
const loadClassList = async ({ type, branch, campus, year, quarter, phase, classroom, teacher }) => {
  /*
    brenchId  true  请求URL string    分公司ID
    blCampusId  true  请求URL string    学习中心ID
    productVersionId  true  请求URL string    年份
    productQuarterId  true  请求URL string    季节
    phaseId true  请求URL string    期数
    teacherId true  请求URL string    老师
    classRoomId true  请求URL string    教室
    type  true  请求URL string    默认小班或者其他（暂时无用，随便传数）
    isModal true  请求URL string    排课类型
    classTypeId true  请求URL string    班级类型
    name  true  请求URL string    小班名称
   */
  const params = {
    brenchId: branch || '',
    blcampusIds: campus || [],
    productVersionId: year || '',
    productQuarterId: quarter || '',
    phaseId: phase || '',
    teacherId: teacher ? (teacher === otherTeacherOption.val ? '-1' : teacher) : '',
    classroomId: classroom || '',
    type,
  };
  const apiData = await getSmallClassListOnModal(params);
  const isChunOrQiu = checkIsChunOrQiu(quarter);
  const data = apiData.map((d) => {
    const startTime = d.classTime;
    const courseMinute = d.classTimeLength && d.everyCourseClassNum ? d.classTimeLength * d.everyCourseClassNum : 0;
    return {
      id: d.miniClassId,
      name: d.name,
      quarter: d.productQuarterId,
      weekDay: isChunOrQiu ? d.courseWeek || [] : null,
      phase: !isChunOrQiu && d.coursePhaseId ? d.coursePhaseId : null,
      statusName: d.statusName,
      scheduleStatus: d.isModal === '0' ? '1' : '2',
      scheduleMode: d.isModal === '1' ? '2' : d.isModal === '2' ? '1' : null,
      startTime,
      courseMinute,
      endTime: moment(startTime, TIME_FORMAT).add(courseMinute, 'm').format(TIME_FORMAT),
      template: d.modalId ? d.modalId + '' : null,
      templateName: d.modalName || null,
      classroom: d.classroomId,
      classroomName: d.classroomName || null,
      teacher: d.teacherId || otherTeacherOption.val,
      teacherName: d.teacherName || otherTeacherOption.text,
      totalCount: d.peopleQuantity,
      signedCount: d.miniClassPeopleNum,
      campus: d.blCampusId,
      year: d.productVersionId,
      classCount: Math.floor(d.totalClassHours / d.everyCourseClassNum),
      grade: d.gradeId,
      subject: d.subjectId,
      classTypeName: d.classTypeName,
      createTime: d.createTime,
      productGroupId: d.productGroupId,
    };
  });
  /*
  const data = await new Promise((resolve) => {
    setTimeout(() => {
      resolve(fakeClassList)
    }, 200)
  })

*/
  return data;
};

/**
 * 加载老师标签页下的结点使用的小班数据列表，格式如下：
    [{
      id: string  小班ID
      name: string  小班名称
      quarter: string  季度
      weekDay: [string]  星期
      phase: string  期
      statusName: string  小班状态名称
      scheduleStatus: string  排课状态
      scheduleMode: string  排课方式
      startTime: string  上课开始时间
      courseMinute: number 课程分钟数
      endTime: string  上课结束时间
      template: string  模板ID
      templateName: string  模板名称
      classroom: string  教室ID
      classroomName: string  教室名称
      teacher: string  老师ID
      teacherName: string  老师姓名
      totalCount: number  招生人数
      signedCount: number  已报班人数
      campus: string  学习中心
      year: string  年份
      classCount: number  讲数
      grade: string  年级
      subject: string  科目
      classTypeName: string  小班类型名称
      createTime: string  创建时间
    }]
 * @param  {Object} searchKeys 查询条件键值对 { year, quarter, teachers }
 */
const loadClassListForTeacher = async ({ branch, campus, year, quarter, teachers }) => {
  /*
    productVersionId  true  请求URL string    年份
    productQuarterId  true  请求URL string    季度
    teacherIds  true  请求URL string    数组格式[]老师ID
    isModal true  请求URL string    排课类型
   */
  const params = {
    brenchId: branch || '',
    blcampusIds: campus || [],
    productVersionId: year || '',
    productQuarterId: quarter || '',
    teacherIds: teachers || [],
    isModal: '2',
  };
  const apiData = await getSmallClassListByTeacher(params);
  const isChunOrQiu = checkIsChunOrQiu(quarter);
  const data = apiData.map((d) => {
    const startTime = d.classTime;
    const courseMinute = d.classTimeLength && d.everyCourseClassNum ? d.classTimeLength * d.everyCourseClassNum : 0;
    return {
      id: d.miniClassId,
      name: d.name,
      quarter: d.productQuarterId,
      weekDay: isChunOrQiu ? d.courseWeek || [] : null,
      phase: !isChunOrQiu && d.coursePhaseId ? d.coursePhaseId : null,
      statusName: d.statusName,
      scheduleStatus: d.isModal === '0' ? '1' : '2',
      scheduleMode: d.isModal === '1' ? '2' : d.isModal === '2' ? '1' : null,
      startTime,
      courseMinute,
      endTime: moment(startTime, TIME_FORMAT).add(courseMinute, 'm').format(TIME_FORMAT),
      template: d.modalId ? d.modalId + '' : '',
      templateName: d.modalName || null,
      classroom: d.classroomId,
      classroomName: d.classroomName || null,
      teacher: d.teacherId || otherTeacherOption.val,
      teacherName: d.teacherName || otherTeacherOption.text,
      totalCount: d.peopleQuantity,
      signedCount: d.miniClassPeopleNum,
      campus: d.blCampusId,
      year: d.productVersionId,
      classCount: Math.floor(d.totalClassHours / d.everyCourseClassNum),
      grade: d.gradeId,
      subject: d.subjectId,
      classTypeName: d.classTypeName,
      createTime: d.createTime,
    };
  });
  return data;
};

/**
 * 加载模板字典，格式如下：
    [{
      val: string  字典值
      text: string  字典显示值
      year: string  年份ID
      quarter: string  季度ID
      classCount: number  讲数
      weekDay: [string]  星期
      phase: string  期
    }]
 * @param  {Object} searchKeys 查询条件键值对 { branch, year, quarter, classCount }
 */
const loadTemplateDict = async ({ branch, year, quarter, classCount }) => {
  if (parseInt(classCount) !== classCount) {
    return [];
  }
  const params = {
    branchId: branch,
    productYearId: year,
    productSeasonId: quarter,
    techNum: classCount,
  };
  const apiData = await getCourseModalList(params);
  const isChunOrQiu = checkIsChunOrQiu(quarter);
  const data = apiData.map((d) => {
    return {
      val: d.id + '',
      text: d.modalName,
      title: d.modalName,
      year: d.productYearId,
      quarter: d.productSeasonId,
      classCount: d.techNum,
      weekDay: isChunOrQiu ? d.courseWeek || [] : null,
      phase: !isChunOrQiu && d.coursePhaseId ? d.coursePhaseId : null,
      startTime: d.classTime,
    };
  });
  /*
  const data = await new Promise((resolve) => {
    setTimeout(() => {
      resolve(fakeTemplateDict)
    }, 200)
  })
*/
  return data;
};

/**
 * 加载老师字典，格式如下：
    [{
      val: string  字典值
      text: string  字典显示值
      campus: string  学习中心
      campusName: string  学习中心名称
      grade: string  年级
      subject: string  科目
    }]
 * @param  {Object} searchKeys 查询条件键值对 { branch, campus, grade, subject }
 */
const loadTeacherDict = async ({ campus, grade, subject }) => {
  const apiData = await loadSmallTcApi({
    campusId: campus,
    gradeId: grade,
    subjectId: subject,
  });
  const data = apiData.map((d) => {
    return {
      val: d.userId,
      text: d.name,
      campus: d.campusId,
      campusName: d.campusName,
      grade: '',
      subject: '',
    };
  });
  const dataAll = [...data, { ...otherTeacherOption }];
  /*
  const data = await new Promise((resolve) => {
    setTimeout(() => {
      resolve(fakeTeacherDict)
    }, 200)
  })
*/
  return dataAll;
};

/**
 * 加载教室字典，格式如下：
    [{
      val: string  字典值
      text: string  字典显示值
      campus: string  学习中心
      campusName: string  学习中心名称
      volume: number  容量
    }]
 * @param  {Object} searchKeys 查询条件键值对 { branch }
 */
const loadClassroomDict = async ({ branch }) => {
  const apiData = await getAllCurrentClassroom({
    brenchId: branch,
  });
  const data = apiData.map((d) => {
    return {
      val: d.id,
      text: d.CLASS_ROOM,
      campus: '',
      campusName: d.campusName,
      volume: d.CLASS_MEMBER,
    };
  });
  /*
  const data = await new Promise((resolve) => {
    setTimeout(() => {
      resolve(fakeClassroomDict)
    }, 200)
  })
  */
  return data;
};

const fetchCampusClassroom = async (campusId, productGroupId) => {
  const isOnline = productGroupId === 'ONLINE_CLASS';

  return getClassroomByCampu({ campusId, isOnline }).then((res) => {
    return (res || []).map((room) => ({ val: room.value, text: room.name }));
  });
};

export default {
  loadClassList,
  loadClassListForTeacher,
  loadTemplateDict,
  loadTeacherDict,
  loadClassroomDict,
  fetchCampusClassroom,
};
