import React from 'react';
import { connect } from 'dva';
import ReactDOM from 'react-dom';
import moment from 'moment';
import { Icon } from 'antd';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import Modal from 'COMPONENT/Modal';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import ScheduleBoard from './ScheduleBoard';
import ClassPanel from './ClassPanel';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import { getDictTextByVal, loadCommonDict, loadBranchDict, loadCampusDict, formatGroupDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { TIME_FORMAT, DATE_FORMAT } from 'COMPONENT/utils/form';
import { errorsHandle } from 'SRC/utils/common';
import { weekDayDict } from 'CONST/dicts/common';
import { getSelectOption } from 'SERVICE/common';
import { clearAllClassCourse, saveEditClassModal } from 'SERVICE/course';
import { checkIsChunOrQiu, getPxByClockTime, checkTimeRangeConflict } from './common';
import { weekDayDictForSchedule, typeDict, otherTeacherOption } from './dicts';
import {
  loadClassList,
  loadTemplateDict,
  loadClassroomDict,
  loadTeacherDict,
  loadClassListForTeacher,
  fetchCampusClassroom,
} from './dataLoaders';
// import { loadSmallClassQuarterTeacherList } from '../../course/miniClassManage/MiniClassManage/Modal/ModifyCourse/loaders';
import styles from './styles.less';
import arrowImg from './imgs/arrow.png';

/**
 * 搜索栏的初始化状态
 */
const searchBarInitState = {
  searchExpand: false, // {boolean} 展开状态
  type: '1', // string 类型输入值
  branch: null, // string 分公司输入值
  campus: null, // [string] 学习中心输入值
  year: null, // string 年份输入值
  quarter: null, // string 季度输入值
  phase: null, // string 期输入值
  classroom: null, // string 教室输入值
  teacher: null, // string 老师输入值
  branchDict: [], // [object] 分公司字典
  campusDict: [], // [object] 学习中心字典
  yearDict: [], // [object] 年份字典
  quarterDict: [], // [object] 季度字典
  phaseDict: [], // [object] 期字典
  classroomDict: [], // [object] 教室字典
  teacherDict: [], // [object] 老师字典
  searchKeys: null, // object 搜索条件
  defaultType: '1', // string 默认类型输入值
  defaultBranch: null, // string 默认分公司输入值
  defaultYear: null, // string 默认年份输入值
  defaultQuarter: null, // string 默认季度输入值
  defaultCampusDict: [], // [object] 默认学习中心字典
  defaultClassroomDict: [], // [object] 默认教室字典
  defaultTeacherDict: [], // [object] 默认老师字典
};

/**
 * 排课表的初始化状态
 */
const scheduleScrollInitState = { classroom: 0, teacher: 0 };
const scheduleInitState = {
  tabKey: 'classroom', // string  当前标签页 [ classroom | teacher ]
  vScroll: scheduleScrollInitState, // object 教室和老师标签页的纵向滚动条距离顶端距离
  hScroll: scheduleScrollInitState, // object 教室和老师标签页的横向滚动条距离左端距离
  classroomList: [], // [object] 教室列表，用于左侧表头
  teacherList: [], // [object] 老师列表，用于左侧表头
  classListForTeacher: null, // [object] 老师标签页下的结点使用的小班数据列表
  classroomNodeGroups: null, // [object] 教室标签页下的课程结点组集合
  teacherNodeGroups: null, // [object] 老师标签页下的课程结点组集合
  hasAnimi: false, // boolean 切换结点组的展开状态时是否有动画效果
};

/**
 * 小班面板的初始化状态
 */
const classPanelInitState = {
  classPanelExpand: true, // boolean 展开状态
  filterBarExpand: false, // boolean 筛选栏展开状态
  classTitle: null, // string 小班标题输入值
  scheduleStatus: null, // string 排课状态输入值
  scheduleMode: null, // string 排课方式输入值
  visibleClassListIds: null, // [string] 可见的小班列表ID
};

const specialErrorCode = 6000001;

class ScheduleByCourseTemplate extends React.Component {
  constructor(props) {
    super(props);
    /** 小班带过来的参数 */
    const paramsFromOldPage = this.props.pageParams[`/${this.props.route.path}`];
    this.paramsFromOldPage = paramsFromOldPage;
    this.state = {
      loading: false, // boolean 页面加载状态
      classList: [], // [object] 小班列表
      editingClass: null, // object 正在编辑的小班数据
      templateDict: [], // [object] 模板字典
      warningModalVisible: false, // 提示信息
      warningTextContent: null,
      normalWarningText: '',
      classroomDictForEditClass: [], // [object] 编辑小班时使用的教室字典
      teacherDictForEditClass: [], // [object] 编辑小班时使用的老师字典
      ...searchBarInitState,
      ...scheduleInitState,
      ...classPanelInitState,
    };
    this.nodeGroupIdx = 0;
  }

  componentDidMount() {
    this.initContent();
    window.addEventListener('resize', this.setExpandBtnTop, false);
  }

  componentDidUpdate(prevProps, prevState) {
    const {
      searchKeys: searchKeysPrev,
      classroomNodeGroups: classroomNodeGroupsPrev,
      teacherNodeGroups: teacherNodeGroupsPrev,
    } = prevState;
    const { searchKeys, classroomNodeGroups, teacherNodeGroups } = this.state;
    if (
      searchKeys &&
      classroomNodeGroups &&
      teacherNodeGroups &&
      (!searchKeysPrev || !classroomNodeGroupsPrev || !teacherNodeGroupsPrev)
    ) {
      this.setExpandBtnTop();
    }
  }

  componentWillUnmount() {
    this.isUnmounted = true;
    window.removeEventListener('resize', this.setExpandBtnTop, false);
  }

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      if (!this.isUnmounted) {
        this.setState(newState, () => {
          resolve();
        });
      } else {
        resolve();
      }
    });
  };

  /**
   * 窗口大小改变时，调整小班列表面板展开按钮位置，使其相对小班面板垂直居中
   */
  setExpandBtnTop = () => {
    if (!this.$expandBtn || !this.$classPanel) {
      return;
    }
    this.$expandBtn.style.top = (this.$classPanel.offsetHeight - 80) / 2 + 'px';
  };

  /**
   * 切换标签页
   * @param  {string} nextTabKey 要切换到的标签页
   */
  changeTab = (nextTabKey) => {
    const { tabKey, vScroll, hScroll } = this.state;
    this.updateState({
      tabKey: nextTabKey,
      vScroll: { ...vScroll, [tabKey]: this._scheduleBoard.getVScroll() },
      hScroll: { ...hScroll, [tabKey]: this._scheduleBoard.getHScroll() },
    }).then(() => {
      this._scheduleBoard.setVScroll(vScroll[nextTabKey]);
      this._scheduleBoard.setHScroll(hScroll[nextTabKey]);
    });
  };

  /**
   * 初始化内容
   */
  initContent = async () => {
    const { uBranchId, uCampusId } = this.props;
    console.log('uBranchId, uCampusId: ', uBranchId, uCampusId);
    await this.updateState({ loading: true });
    const branchDict = await loadBranchDict();
    const notCampusUser = uBranchId === uCampusId;
    let branch = notCampusUser ? branchDict[0].val : uBranchId;
    // 如果从修改班级跳转 需覆盖默认参数
    if (this.paramsFromOldPage) {
      branch = this.paramsFromOldPage.branchId;
    }
    const [campusDict, yearDict, quarterDict, phaseDict, classroomDict, teacherDict] = await Promise.all([
      loadCampusDict(branch),
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('SMALL_CLASS_PHASE'),
      loadClassroomDict({ branch }),
      loadTeacherDict({ branch }),
    ]);
    let campus = notCampusUser ? [campusDict[0].val] : [uCampusId];
    // 如果从修改班级跳转 需覆盖默认参数
    if (this.paramsFromOldPage) {
      const campusId = this.paramsFromOldPage.campusId;
      campus = [campusId];
    }
    const now = new Date();
    const yearNumber = now.getFullYear();
    let year = yearDict.find((y) => y.text === yearNumber + '').val;
    const nowTime = now.getTime();
    const getTime = (month, date) => new Date(yearNumber, month, date).getTime();
    let quarter;
    if (nowTime >= getTime(0, 16) && nowTime < getTime(1, 16)) {
      quarter = 'DAT0000000230';
    } else if (nowTime < getTime(5, 1)) {
      quarter = 'DAT0000000231';
    } else if (nowTime < getTime(8, 1)) {
      quarter = 'DAT0000000232';
    } else {
      quarter = 'DAT0000000233';
    }
    // 如果从修改班级跳转 需覆盖默认参数
    if (this.paramsFromOldPage) {
      const { productQuarterId, productVersionId } = this.paramsFromOldPage;
      quarter = productQuarterId;
      year = productVersionId;
    }
    const searchKeys = { type: '1', branch, campus, year, quarter, phase: null, classroom: null, teacher: null };
    const [classList, classListForTeacher] = await Promise.all([
      loadClassList(searchKeys),
      loadClassListForTeacher({ ...searchKeys, year, quarter, teacherList: teacherDict }),
    ]);
    await this.updateState({
      loading: false,
      branch,
      year,
      quarter,
      campus,
      branchDict,
      campusDict,
      yearDict,
      quarterDict,
      phaseDict,
      classroomDict,
      teacherDict,
      defaultBranch: branch,
      defaultYear: year,
      defaultQuarter: quarter,
      defaultCampus: campus,
      defaultCampusDict: campusDict,
      defaultClassroomDict: classroomDict,
      defaultTeacherDict: teacherDict,
      classroomList: classroomDict,
      teacherList: teacherDict,
      searchKeys,
      classList: this.classListAddCampusName(classList, campusDict),
      classListForTeacher,
      visibleClassListIds: classList.map((c) => c.id),
    });
    this.buildNodeGroups();
    /* @desc 查询指定小班 */
    setTimeout(() => {
      if (this.paramsFromOldPage) {
        this.setState({ classTitle: this.paramsFromOldPage.miniClassName }, () => {
          this._classPanel.filter();
        });
      }
    }, 0);
  };

  classListAddCampusName = (classList, campusDict) => {
    return classList.map((c) => {
      return { ...c, campusName: getDictTextByVal(campusDict, c.campus) };
    });
  };

  /**
   * 搜索，根据查询条件重新加载数据：
   * 1、教室列表，并在前端根据教室搜索条件过滤
   * 2、老师列表，并在前端根据老师搜索条件过滤
   * 3、小班列表
   * 4、老师标签页的结点使用的小班列表
   * 搜索成功后刷新排课表结点，重置排课表的滚动条，重置小班面板的展开状态、筛选状态、滚动条状态，取消选中小班
   */
  search = async () => {
    const { type, branch, campus, year, quarter, phase, classroom, teacher, campusDict } = this.state;
    const searchKeys = { type, branch, campus, year, quarter, phase, classroom, teacher };
    await this.updateState({ loading: true });
    try {
      const [classroomList, teacherList] = await Promise.all([
        loadClassroomDict({ branch }),
        loadTeacherDict({ branch, campus }),
      ]);
      const newClassroomList = classroom ? classroomList.filter((c) => c.val === classroom) : classroomList;
      const newTeacherList = teacher ? teacherList.filter((c) => c.val === teacher) : teacherList;
      const [classList, classListForTeacher] = await Promise.all([
        loadClassList(searchKeys),
        loadClassListForTeacher({ ...searchKeys, year, quarter, teacherList: newTeacherList }),
      ]);
      await this.updateState({
        loading: false,
        classroomList: newClassroomList,
        teacherList: newTeacherList,
        searchKeys,
        classList: this.classListAddCampusName(classList, campusDict),
        classListForTeacher,
        visibleClassListIds: classList.map((c) => c.id),
        vScroll: scheduleScrollInitState,
        hScroll: scheduleScrollInitState,
        filterBarExpand: false,
        classTitle: null,
        scheduleStatus: null,
        scheduleMode: null,
        editingClass: null,
      });
      this.buildNodeGroups();
      this._scheduleBoard.setVScroll(0);
      this._scheduleBoard.setHScroll(0);
      this._classPanel.setScroll(0);
    } catch (e) {
      this.updateState({ loading: false });
    }
  };

  /**
   * 分别构建排课表中教室和老师标签页下的课程结点组，并更新状态，数据结构如下：
    [
      { 一个结点组
        id  number  结点组ID
        row  string  所在行，对应行字典的值
        col  string  所在列，对应列字典的值
        expand  boolean  展开状态
        nodes [
          {  一个结点
            startTime  string  开始时间，'HH:mm'
            endTime  string  结束时间，'HH:mm'
            startPx  number 开始位置（px，相对于所在单元格的开始位置的距离）
            endPx  number 结束位置（px，相对于所在单元格的开始位置的距离）
            width  number  宽度（px）
            classId  string  小班ID
            className  string  小班名称
            classStatusName  string  小班状态名称
            createTime  string  小班创建时间
            classCampusName  string  小班所属学习中心名称
            totalCount  number  招生人数
            signedCount  number  已报人数
            classroomName  string  教室名称
            classroomVolume  number  教室容量
            teacherName  string  老师姓名
          }
        ]
      }
    ]
   */
  buildNodeGroups = () => {
    const { classList, classListForTeacher } = this.state;
    let classroomNodeGroups = [];
    let teacherNodeGroups = [];
    classList.forEach((c) => {
      this.addNodesByClass('classroom', classroomNodeGroups, c);
    });
    classListForTeacher.forEach((c) => {
      this.addNodesByClass('teacher', teacherNodeGroups, c);
    });
    this.updateState({ classroomNodeGroups, teacherNodeGroups });
  };

  /**
   * 向一个结点组集合中添加一个小班中所包含的所有课程结点
   * 1、春秋季的列按星期划分，可能有多个结点
   * 2、寒暑季的列按期划分，只有一个结点
   * @param  {string} type 类型 [classroom | teacher]
   * @param  {object} nodeGroups 待操作的结点组集合
   * @param  {object} classData  小班数据
   * @return
   */
  addNodesByClass = (type, nodeGroups, classData) => {
    const {
      searchKeys: { quarter },
      classroomList,
      campusDict,
    } = this.state;
    const {
      id,
      weekDay,
      phase,
      classroom,
      startTime,
      endTime,
      scheduleStatus,
      scheduleMode,
      name,
      statusName,
      createTime,
      totalCount,
      signedCount,
      classroomName,
      teacherName,
      campus,
    } = classData;
    if (scheduleStatus === '2' && scheduleMode === '1') {
      const startPx = getPxByClockTime(startTime);
      const endPx = getPxByClockTime(endTime);
      const classroomData = classroomList.find((c) => c.val === classroom);
      const classroomVolume = classroomData ? classroomData.volume : null;
      const newNode = {
        startTime,
        endTime,
        startPx,
        endPx,
        width: endPx - startPx,
        classId: id,
        className: name,
        classStatusName: statusName,
        createTime,
        classCampusName: getDictTextByVal(campusDict, campus),
        totalCount,
        signedCount,
        classroomName,
        classroomVolume,
        teacherName,
      };
      let cols = [];
      if (checkIsChunOrQiu(quarter)) {
        cols = weekDay;
      } else {
        cols = [phase];
      }
      cols.forEach((col) => {
        this.addNode({ nodeGroups, row: classData[type], col, node: { ...newNode } });
      });
    }
  };

  /**
   * 向一个结点组集合中添加一个结点
   * 1、如果没有任何冲突，则新增一个只包含新结点的结点组
   * 2、如果新结点与集合中的一个或多个结点组存在冲突，则新结点和冲突的所有结点组融合成一个新的结点组，展开状态为，如果有一个结点组展开则展开，否则收起
   * @param  {object} nodeGroups 待操作的结点组集合
   * @param  {string} row  要添加到的行
   * @param  {string} col  要添加到的列
   * @param  {object} node  结点数据
   */
  addNode = ({ nodeGroups, row, col, node }) => {
    let conflictGroups = [];
    let notConflictGroups = [];
    let conflictIdxs = [];
    nodeGroups.forEach((g, i) => {
      if (g.row === row && g.col === col && g.nodes.find((n) => checkTimeRangeConflict(n, node))) {
        conflictGroups.push(g);
        conflictIdxs.push(i);
      } else {
        notConflictGroups.push(g);
      }
    });
    if (conflictGroups.length === 0) {
      nodeGroups.push({ id: this.nodeGroupIdx++, row, col, nodes: [node] });
    } else {
      let newGroupNodes = [node];
      let expand;
      conflictGroups.forEach((g, i) => {
        if (g.expand) {
          expand = true;
        }
        g.nodes.forEach((n) => {
          newGroupNodes.push(n);
        });
        newGroupNodes.sort((a, b) => {
          return moment(a.createTime, DATE_FORMAT).valueOf() < moment(b.createTime, DATE_FORMAT).valueOf();
        });
      });
      for (let i = conflictIdxs.length - 1; i >= 0; i--) {
        nodeGroups.splice(conflictIdxs[i], 1);
      }
      nodeGroups.push({ id: this.nodeGroupIdx++, row, col, nodes: newGroupNodes, expand });
    }
  };

  /**
   * 从一个结点组集合中删除一个小班所包含的课程结点
   * @param  {object} nodeGroups 待操作的结点组集合
   * @param  {object} classId  小班ID
   */
  deleteNodesByClass = (nodeGroups, classId) => {
    let emptyIdxs = [];
    nodeGroups.forEach((g, i) => {
      g.nodes = g.nodes.filter((n) => n.classId !== classId);
      if (g.nodes.length === 0) {
        emptyIdxs.push(i);
      }
    });
    for (let i = emptyIdxs.length - 1; i >= 0; i--) {
      nodeGroups.splice(emptyIdxs[i], 1);
    }
  };

  /**
   * 检查是否有修改小班信息保存
   * @return {boolean}
   */
  checkDirty = () => {
    const { editingClass, classList } = this.state;
    if (editingClass) {
      const editingClassOrigin = classList.find((c) => c.id === editingClass.id);
      if (!objectDeepEquals(editingClassOrigin, editingClass)) {
        return true;
      }
    }
    return false;
  };

  /**
   * 选中小班并在小班列表面板中定位该小班
   * @param  {string} id 小班ID
   */
  selectClassAndLocateClass = (id) => {
    const { classList, visibleClassListIds } = this.state;
    if (!classList.find((c) => c.id === id)) {
      return;
    }
    this.selectClass(id);
    if (visibleClassListIds.find((c) => c === id)) {
      this._classPanel.locateClass(id);
    } else {
      this.updateState({ visibleClassListIds: classList.map((c) => c.id) }).then(() => {
        this._classPanel.locateClass(id);
      });
    }
  };

  /**
   * 选中小班并在排课表定位该小班的第一个课程结点
   * @param  {string} id 小班ID
   */
  selectClassAndLocateNode = (id) => {
    const {
      classList,
      searchKeys: { quarter },
      tabKey,
    } = this.state;
    this.selectClass(id);
    const classData = classList.find((c) => c.id === id);
    const { phase, weekDay, scheduleStatus, scheduleMode, classroom, teacher, startTime } = classData;
    if (scheduleStatus !== '2' || scheduleMode !== '1') {
      return;
    }
    const row = tabKey === 'classroom' ? classroom : teacher;
    let col;
    if (checkIsChunOrQiu(quarter)) {
      let firstWeekDay;
      for (let i = 0; i < weekDayDictForSchedule.length; i++) {
        firstWeekDay = weekDayDictForSchedule[i].val;
        if (weekDay.indexOf(firstWeekDay) >= 0) {
          break;
        }
      }
      col = firstWeekDay;
    } else {
      col = phase;
    }
    this._scheduleBoard.locate(row, col, startTime);
  };

  /**
   * 选中小班
   * @param  {string} id 小班ID
   */
  selectClass = (id) => {
    const {
      classList,
      editingClass,
      searchKeys: { branch },
      phaseDict,
    } = this.state;
    if (editingClass && id === editingClass.id) {
      return;
    }
    const nextActiveClass = classList.find((c) => c.id === id);
    const { campus, year, quarter, classCount, grade, subject, productGroupId } = nextActiveClass;

    // const isOnLineClass = productGroupId === 'ONLINE_CLASS';

    const _selectClass = () => {
      this.updateState({ loading: true });
      Promise.all([
        loadTemplateDict({ branch, year, quarter, classCount }),
        fetchCampusClassroom(campus, productGroupId),
        //isOnLineClass ?
        loadTeacherDict({ branch, campus, grade, subject }) /* :
        loadSmallClassQuarterTeacherList({
          subjectId: subject,
          gradeId: grade,
          blCampusId: campus,
          yearId: year,
          quarterId: quarter,
          blBranchId: branch
        }),*/,
      ]).then(
        ([templateDictFromApi, classroomDictForEditClass, teacherDict]) => {
          // let teacherDictForEditClass = teacherDict || [];
          //if (isOnLineClass) {
          let teacherDictForEditClass = formatGroupDict(teacherDict || [], 'campusName');
          //}
          const templateDict = templateDictFromApi.map((t) => {
            const { weekDay, phase } = t;
            let groupName = '';
            let groupSort = 0;
            if (weekDay) {
              for (let i = 0; i < weekDayDict.length; i++) {
                const weekDayI = weekDayDict[i];
                if (weekDay.find((w) => w === weekDayI.val)) {
                  groupName = weekDayI.text;
                  groupSort = i;
                  break;
                }
              }
            } else {
              groupName = getDictTextByVal(phaseDict, phase);
            }
            return { ...t, groupName, groupSort };
          });
          this.updateState({
            loading: false,
            templateDict,
            classroomDictForEditClass,
            teacherDictForEditClass,
            editingClass: { ...nextActiveClass },
          });
        },
        () => {
          this.updateState({ loading: false });
        },
      );
    };
    if (this.checkDirty()) {
      Modal.confirm({
        content: '您正在编辑小班排课信息，现在切换小班将取消保存',
        onCancel: () => {
          this.cancelSelectClass().then(() => {
            _selectClass();
          });
        },
        okText: '继续编辑',
        cancelText: '取消保存',
        iconType: 'exclamation-circle',
      });
    } else {
      _selectClass();
    }
  };

  /**
   * 取消选中小班
   */
  cancelSelectClass = async () => {
    const { editingClass } = this.state;
    await this.updateState({ editingClass: null });
    return true;
  };

  /**
   * 更新正在编辑的小班数据
   * @param  {object} changedFields  要更新的字段键值对
   */
  updateEditingClass = (changedFields) => {
    const { editingClass } = this.state;
    this.updateState({
      editingClass: { ...editingClass, ...changedFields },
    });
  };

  generateWarningModal() {
    const props = {
      width: 430,
      okText: '确定',
      title: '温馨提示',
      onCancel: this.handleCloseWarningModal,
      footer: (
        <div>
          <Button type="primary" onClick={this.handleCloseWarningModal}>
            确定
          </Button>
        </div>
      ),
    };

    const { warningTextContent, normalWarningText, warningModalVisible } = this.state;
    const warningData = warningTextContent ? JSON.parse(warningTextContent) : null;

    return (
      warningModalVisible && (
        <Modal {...props}>
          <div className={styles.warningTextWrapper}>
            <div className={styles.warningTextContentWrapper}>
              <Icon type="exclamation-circle" style={{ color: '#ffbf00', fontSize: 22, marginRight: 5 }} />
              {warningData ? (
                <div className={styles.warningTextContent}>
                  <strong>{warningData.title}</strong>
                  <p>{warningData.subTitle}</p>
                  <ul className={styles.detail}>
                    {(warningData.studentList || []).map(({ courseInfo, studentName }, index) => {
                      return (
                        <li key={index}>
                          <span className={styles.studentName}>{studentName}</span>
                          {courseInfo}
                        </li>
                      );
                    })}
                  </ul>
                </div>
              ) : (
                <div className={styles.warningTextContent}>{normalWarningText}</div>
              )}
            </div>
          </div>
        </Modal>
      )
    );
  }

  handleCloseWarningModal = () => {
    this.setState({ warningModalVisible: false });
  };

  /**
   * 删除排课
   */
  deleteSchedule = async () => {
    const { editingClass, classList, classroomNodeGroups, teacherNodeGroups } = this.state;
    const id = editingClass.id;
    const editingClassOrigin = classList.find((t) => t.id === id);
    await this.updateState({ loading: true });
    try {
      await clearAllClassCourse({ miniClassId: id }, true);
      showMessage('删除成功', 'success');
      const newActiveClass = { ...editingClassOrigin, scheduleStatus: '1', scheduleMode: null };
      let newClassroomNodeGroups = [...classroomNodeGroups];
      let newTeacherNodeGroups = [...teacherNodeGroups];
      this.deleteNodesByClass(newClassroomNodeGroups, id);
      this.deleteNodesByClass(newTeacherNodeGroups, id);
      this.updateState({
        loading: false,
        classList: classList.map((c) => {
          if (c.id === id) {
            return newActiveClass;
          }
          return c;
        }),
        editingClass: { ...newActiveClass },
        classroomNodeGroups: newClassroomNodeGroups,
        teacherNodeGroups: newTeacherNodeGroups,
      });
    } catch (e) {
      if (e._data_ && e._data_.resultCode == specialErrorCode) {
        this.setState({ warningModalVisible: true, warningTextContent: e.text, normalWarningText: '' });
      } else {
        this.setState({ warningModalVisible: true, warningTextContent: null, normalWarningText: e.text || ' ' });
      }
      this.updateState({ loading: false });
    }
  };

  /**
   * 保存排课
   */
  saveSchedule = async () => {
    const { editingClass, classList, classroomNodeGroups, teacherNodeGroups } = this.state;
    const { id, startTime, template, classroom, teacher, courseMinute } = editingClass;
    const editingClassOrigin = classList.find((t) => t.id === id);
    const error = this._classPanel.showSaveError;
    if (!startTime) {
      error('请选择开始时间');
      return;
    }
    if (!template) {
      error('请选择课程模板');
      return;
    }
    if (!classroom) {
      error('请选择教室');
      return;
    }
    if (!teacher) {
      error('请选择老师');
      return;
    }
    const start = moment(startTime, TIME_FORMAT).valueOf();
    const end = moment(startTime, TIME_FORMAT).add(courseMinute, 'm').valueOf();
    const nextDayDisabledStart = moment('00:00', TIME_FORMAT).add(1, 'd').valueOf();
    const disabledEnd = moment('05:59', TIME_FORMAT).valueOf();
    if (start <= disabledEnd || end >= nextDayDisabledStart) {
      error('00:00-5:59不能上课，请修改开始时间或更换模板');
      return;
    }
    await this.updateState({ loading: true });
    /*
      miniClassId true  请求URL string    小班ID
      classTime true  请求URL string    上课时间
      modalId true  请求URL int   模板ID
      classRoomId true  请求URL string    教室ID
      teacherId true  请求URL string    老师ID
      classEndTime  true  请求URL string    上课结束时间
    */
    const saveData = {
      miniClassId: id,
      classTime: startTime,
      modalId: template,
      classRoomId: classroom,
      teacherId: teacher === otherTeacherOption.val ? '' : teacher,
      classEndTime: moment(startTime, TIME_FORMAT).add(courseMinute, 'm').format(TIME_FORMAT),
    };
    try {
      await saveEditClassModal(saveData, true);
      //await getSelectOption(saveData, true)
      showMessage('保存成功', 'success');
      const newActiveClass = { ...editingClass, scheduleStatus: '2', scheduleMode: '1' };

      let newClassroomNodeGroups = [...classroomNodeGroups];
      let newTeacherNodeGroups = [...teacherNodeGroups];

      this.deleteNodesByClass(newClassroomNodeGroups, id);
      this.deleteNodesByClass(newTeacherNodeGroups, id);

      this.addNodesByClass('classroom', newClassroomNodeGroups, newActiveClass);
      this.addNodesByClass('teacher', newTeacherNodeGroups, newActiveClass);

      this.updateState({
        loading: false,
        classList: classList.map((c) => {
          if (c.id === id) {
            return newActiveClass;
          }
          return c;
        }),
        editingClass: { ...newActiveClass },
        classroomNodeGroups: newClassroomNodeGroups,
        teacherNodeGroups: newTeacherNodeGroups,
      });
      this.selectClassAndLocateNode(id);
    } catch (e) {
      // if(e.text){
      //   error(e.text)
      // }
      // showMessage('保存失败'+ (e.text ? '：'+ e.text : ''))
      // this.updateState({ loading: false })
      if (e._data_ && e._data_.resultCode == specialErrorCode) {
        this.setState({ warningModalVisible: true, warningTextContent: e.text, normalWarningText: '' });
      } else {
        this.setState({ warningModalVisible: true, warningTextContent: null, normalWarningText: e.text || ' ' });
      }
      this.updateState({ loading: false });
    }
  };

  /**
   * 搜索栏分公司的值变化时的处理
   * 1、更新分公司状态
   * 2、清空学习中心、教室、老师的值和选项，重新加载学习中心、教室、老师选项
   * @param  {string} branch 分公司
   */
  branchChange = async (branch) => {
    this.updateState({
      branch,
      campus: null,
      classroom: null,
      teacher: null,
      campusDict: [],
      classroomDict: [],
      teacherDict: [],
    });
    const [campusDict, classroomDict, teacherDict] = await Promise.all([
      loadCampusDict(branch),
      loadClassroomDict({ branch }),
      loadTeacherDict({ branch }),
    ]);
    this.updateState({ campusDict, classroomDict, teacherDict });
  };

  /**
   * 搜索栏学习中心的值变化时的处理
   * 1、更新学习中心状态
   * 2、清空老师的值和选项，重新加载老师选项
   * @param  {string} campus 学习中心
   */
  campusChange = async (campus) => {
    const {
      searchKeys: { branch },
    } = this.state;
    this.updateState({ campus, teacher: null, teacherDict: [] });
    const [teacherDict] = await Promise.all([loadTeacherDict({ branch, campus })]);
    this.updateState({ teacherDict });
  };

  /**
   * 搜索栏季度的值变化时的处理
   * 1、更新季度状态
   * 2、当季度为春秋时，清空期的值
   * @param  {string} quarter 季度
   */
  quarterChange = (quarter) => {
    let changedState = { quarter };
    if (checkIsChunOrQiu(quarter)) {
      changedState = { ...changedState, phase: null };
    }
    this.updateState(changedState);
  };

  /**
   * 重置搜索栏的输入值
   */
  resetSearchBar = () => {
    const {
      defaultType,
      defaultBranch,
      defaultYear,
      defaultQuarter,
      defaultCampusDict,
      defaultClassroomDict,
      defaultTeacherDict,
    } = this.state;
    this.updateState({
      type: defaultType,
      branch: defaultBranch,
      campus: null,
      year: defaultYear,
      quarter: defaultQuarter,
      phase: null,
      classroom: null,
      teacher: null,
      campusDict: defaultCampusDict,
      classroomDict: defaultClassroomDict,
      teacherDict: defaultTeacherDict,
    });
  };

  /**
   * 排课表中结点组展开状态切换
   * @param  {number} id 结点组ID
   */
  toggleNodeGroupExpand = (id) => {
    const { classroomNodeGroups, teacherNodeGroups, tabKey } = this.state;
    let changedState = { hasAnimi: true };
    if (tabKey === 'classroom') {
      changedState = {
        ...changedState,
        classroomNodeGroups: classroomNodeGroups.map((g) => {
          if (g.id === id) {
            const expand = g.expand ? false : true;
            return { ...g, expand };
          }
          return g;
        }),
      };
    } else {
      changedState = {
        ...changedState,
        teacherNodeGroups: teacherNodeGroups.map((g) => {
          if (g.id === id) {
            const expand = g.expand ? false : true;
            return { ...g, expand };
          }
          return g;
        }),
      };
    }
    this.updateState(changedState).then(() => {
      setTimeout(() => {
        this.updateState({ hasAnimi: false });
      }, 600);
    });
  };

  render() {
    const {
      loading,
      classList,
      searchExpand,
      type,
      branch,
      campus,
      year,
      quarter,
      phase,
      classroom,
      teacher,
      branchDict,
      campusDict,
      yearDict,
      quarterDict,
      phaseDict,
      classroomDict,
      teacherDict,
      searchKeys,
      tabKey,
      classroomNodeGroups,
      teacherNodeGroups,
      hasAnimi,
      classroomList,
      teacherList,
      classListForTeacher,
      classPanelExpand,
      filterBarExpand,
      classTitle,
      scheduleStatus,
      scheduleMode,
      visibleClassListIds,
      templateDict,
      classroomDictForEditClass,
      teacherDictForEditClass,
      editingClass,
    } = this.state;
    const editingClassId = editingClass ? editingClass.id : null;
    return (
      <Page fitWindowHeight={true} loading={loading} route={this.props.route} className={styles.wrap}>
        <SearchBar
          expand={searchExpand}
          onToggleExpand={() => this.updateState({ searchExpand: !searchExpand })}
          onSearch={this.search}
          onReset={this.resetSearchBar}
        >
          <Select placeholder="类型" dict={typeDict} value={type} onChange={(val) => this.updateState({ type: val })} />
          <Select
            placeholder="分公司"
            dict={branchDict}
            value={branch}
            onChange={this.branchChange}
            allowClear={false}
          />
          <MultiSelect placeholder="学习中心" dict={campusDict} value={campus} onChange={this.campusChange} />
          <Select
            placeholder="年份"
            dict={yearDict}
            value={year}
            onChange={(val) => this.updateState({ year: val })}
            allowClear={false}
          />
          <Select
            placeholder="季度"
            dict={quarterDict}
            value={quarter}
            onChange={this.quarterChange}
            allowClear={false}
          />
          <Select
            placeholder="期"
            dict={phaseDict}
            value={phase}
            onChange={(val) => this.updateState({ phase: val })}
            disabled={checkIsChunOrQiu(quarter)}
          />
          {tabKey === 'classroom' ? (
            <Select
              placeholder="教室"
              dict={formatGroupDict(classroomDict, 'campusName')}
              value={classroom}
              onChange={(val) => this.updateState({ classroom: val })}
              showSearch={true}
            />
          ) : (
            <Select
              placeholder="老师"
              dict={formatGroupDict(teacherDict, 'campusName')}
              value={teacher}
              onChange={(val) => this.updateState({ teacher: val })}
              showSearch={true}
            />
          )}
        </SearchBar>
        {searchKeys && classroomNodeGroups && teacherNodeGroups && (
          <div className={styles.main + (classPanelExpand ? ' ' + styles.panelExpand : '')}>
            <div className={styles.left}>
              <ScheduleBoard
                tabKey={tabKey}
                rowDict={tabKey === 'classroom' ? classroomList : teacherList}
                colDict={checkIsChunOrQiu(searchKeys.quarter) ? weekDayDictForSchedule : phaseDict}
                nodeGroups={tabKey === 'classroom' ? classroomNodeGroups : teacherNodeGroups}
                activeClassId={editingClassId}
                hasAnimi={hasAnimi}
                onChangeTab={this.changeTab}
                onSelectNode={this.selectClassAndLocateClass}
                onToggleNodeGroupExpand={this.toggleNodeGroupExpand}
                ref={(c) => (this._scheduleBoard = c)}
              />
            </div>
            <div className={styles.right}>
              <ClassPanel
                classTitle={classTitle}
                scheduleStatus={scheduleStatus}
                scheduleMode={scheduleMode}
                visibleClassListIds={visibleClassListIds}
                filterBarExpand={filterBarExpand}
                classList={classList}
                editingClass={editingClass}
                classListForTeacher={classListForTeacher}
                branchName={getDictTextByVal(branchDict, searchKeys.branch)}
                templateDict={templateDict}
                classroomDict={classroomDictForEditClass}
                teacherDict={teacherDictForEditClass}
                isDirty={this.checkDirty()}
                isChunOrQiu={checkIsChunOrQiu(searchKeys.quarter)}
                onUpdateState={this.updateState}
                onSelectClass={this.selectClassAndLocateNode}
                onDataChange={this.updateEditingClass}
                onDeleteSchedule={this.deleteSchedule}
                onSaveSchedule={this.saveSchedule}
                ref={(c) => {
                  this._classPanel = c;
                  this.$classPanel = ReactDOM.findDOMNode(c);
                }}
              />
            </div>
            <div
              className={styles.expandBtn}
              onClick={() => {
                this.updateState({ classPanelExpand: !classPanelExpand });
              }}
              ref={(c) => (this.$expandBtn = ReactDOM.findDOMNode(c))}
            >
              <div className={styles.expandBtnInner}>
                <img src={arrowImg} />
              </div>
            </div>
          </div>
        )}
        {this.generateWarningModal()}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { brenchId, campusId } = state.account.user;
  return {
    uBranchId: brenchId,
    uCampusId: campusId,
    pageParams: state.main.pageParams,
  };
};

export default connect(mapStateToProps)(ScheduleByCourseTemplate);
