import React, { PropTypes } from 'react';
import { connect } from 'dva';
import SearchBar from 'COMPONENT/SearchBar';
import { Select, Input, DatePicker } from 'COMPONENT';
import { loadBranchCampusDict } from 'UTIL/dict';
import { getMiniClassTeacherListForClassChange } from 'SERVICE/common';
import _ from 'lodash';

import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const getSearchInitValue = () => {
  return {
    chargeCampusId: '',
    contractId: '',
    miniClassName: '',
    studentName: '',
    studyManagerId: '',
    teacherId: '',
    failTimeEnd: '',
    failTimeStart: '',
    courseDateEnd: '',
    courseDateStart: '',
  };
};

function formatTeachOptions(arr, result = []) {
  let teacherOptions = arr.map((cur) => {
    if (cur.children) {
      formatTeachOptions(cur.children, result);
    }
    return {
      label: cur.orgName,
      val: cur.orgId,
      options: (cur.teachers || []).map((t) => ({
        text: t.userName,
        val: t.userId,
      })),
    };
  });

  result.push(...teacherOptions);
  return result.filter((r) => r.options && r.options.length);
}

class Search extends React.Component {
  static propTypes = {
    onSearch: PropTypes.func.isRequired,
  };

  constructor(props) {
    super(props);
    this.state = {
      campusList: [],
      searchParams: getSearchInitValue(),
    };
  }

  componentDidMount() {
    this.fetchCampusOptions();
    this.initDictionary();
  }

  componentWillReceiveProps(nextProps) {
    if (
      nextProps.pageParams.contractId !== this.props.pageParams.contractId ||
      nextProps.pageParams.classCampusId !== this.props.pageParams.classCampusId
    ) {
      this.initSearchValue(nextProps);
    }
  }

  initSearchValue(props) {
    const {
      pageParams: { contractId, classCampusId },
    } = props;
    this.setSearchParams({ contractId, chargeCampusId: classCampusId }, null, this.prepareSearch);
  }

  initDictionary() {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['smallClassMainTc', 'branchSmallClassAuxTc'],
      },
    });

    this.fetchTeacherList();
  }

  fetchTeacherList = () => {
    getMiniClassTeacherListForClassChange().then((res) => {
      let teacherList = res.data || [];

      this.setState({
        teacherList: formatTeachOptions(teacherList),
      });
    });
  };

  fetchCampusOptions = async () => {
    try {
      const campusList = await loadBranchCampusDict();
      this.setState({ campusList });
    } catch (e) {
      showMessage('获取学习中心失败');
    }
  };

  setSearchParams = (key, value, callback) => {
    let newSearchParams = {};

    if (typeof key === 'string') {
      newSearchParams = { [key]: value };
    } else {
      _.forEach(key, (value, index) => {
        newSearchParams[index] = value;
      });
    }

    this.setState(
      {
        searchParams: {
          ...this.state.searchParams,
          ...newSearchParams,
        },
      },
      () => callback && callback(),
    );
  };

  resetSearchParams = () => {
    this.setState({
      searchParams: getSearchInitValue(),
    });
  };

  prepareSearch = () => {
    const searchParams = this.state.searchParams;
    const campus = searchParams.chargeCampusId;
    if (!campus) {
      return showMessage('请选择学习中心');
    }
    this.props.onSearch(searchParams);
  };

  render() {
    const {
      campusList,
      teacherList,
      searchParams: {
        chargeCampusId,
        contractId,
        miniClassName,
        studentName,
        studyManagerId,
        teacherId,
        failTimeEnd,
        failTimeStart,
        courseDateEnd,
        courseDateStart,
      },
    } = this.state;

    const { smallClassMainTc } = this.props;

    return (
      <SearchBar onSearch={this.prepareSearch} onReset={this.resetSearchParams}>
        <Select
          showSearch
          allowClear={false}
          placeholder="扣费学习中心"
          dict={campusList}
          value={chargeCampusId}
          onChange={(value) => {
            this.setSearchParams('chargeCampusId', value);
          }}
        />

        <Input
          placeholder="合同编号"
          value={contractId}
          onChange={(e) => {
            this.setSearchParams('contractId', e.target.value);
          }}
        />
        <Input
          placeholder="小班名称"
          value={miniClassName}
          onChange={(e) => {
            this.setSearchParams('miniClassName', e.target.value);
          }}
        />
        <Input
          placeholder="学生姓名"
          value={studentName}
          onChange={(e) => {
            this.setSearchParams('studentName', e.target.value);
          }}
        />

        <Select
          showSearch
          placeholder="老师姓名"
          dict={teacherList}
          value={teacherId}
          onChange={(value) => {
            this.setSearchParams('teacherId', value);
          }}
        />
        <Select
          showSearch
          placeholder="班级班主任姓名"
          dict={smallClassMainTc}
          value={studyManagerId}
          onChange={(value) => {
            this.setSearchParams('studyManagerId', value);
          }}
        />
        <DatePicker
          placeholder="最近扣费失败时间（起）"
          value={failTimeStart}
          onChange={(value) => {
            this.setSearchParams('failTimeStart', value);
          }}
        />
        <DatePicker
          placeholder="最近扣费失败时间（止）"
          disabledDate={(d) => {
            if (failTimeStart && d) {
              let start = new Date(failTimeStart).getTime();
              let end = new Date(d).getTime();
              return end < start;
            }
          }}
          value={failTimeEnd}
          onChange={(value) => {
            this.setSearchParams('failTimeEnd', value);
          }}
        />
        <DatePicker
          placeholder="上课日期（起）"
          value={courseDateStart}
          onChange={(value) => {
            this.setSearchParams('courseDateStart', value);
          }}
        />
        <DatePicker
          placeholder="上课日期（止）"
          disabledDate={(d) => {
            if (courseDateStart && d) {
              let start = new Date(courseDateStart).getTime();
              let end = new Date(d).getTime();
              return end < start;
            }
          }}
          value={courseDateEnd}
          onChange={(value) => {
            this.setSearchParams('courseDateEnd', value);
          }}
        />
      </SearchBar>
    );
  }
}

export default connect((state) => {
  return {
    smallClassMainTc: state.dict['smallClassMainTc'],
  };
})(Search);
