import React, { Component } from 'react';
import { connect } from 'dva';
import { Page, Modal, Input, Select, Button, SearchFilterForm, SearchFieldForm } from 'COMPONENT';
import { Icon, Table } from 'antd';
import { errorsHandle } from 'UTIL/common';
import {
  addClassCourseChangeGroup,
  deleteClassCourseChangeGroup,
  getClassCourseChangeGroupList,
} from 'SERVICE/smallClassCustomerTransfer';
import CreateCustomerTransferRuleModal from './CreateCustomerTransferRuleModal';
import CustomerTransferRuleDescriptionModal from './CustomerTransferRuleDescriptionModal';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

import styles from './styles.module.less';

const errorsHandler = errorsHandle();

class SmallClassCustomerTransfer extends Component {
  state = {
    list: [],

    total: 0,
    pageNum: 1,
    pageSize: 20,
    searchData: {},

    isCreateModalVisible: false,
    isConfirmDeleteModalVisible: false,
    isDescriptionModalVisible: false,

    loading: false,
    isDeleteLoading: false,
    tableFixedHeight: 100,
  };

  constructor(props) {
    super(props);
    this.currentId = null;
    this.searchRef = null;
  }

  componentWillMount() {
    this.init();
    this.initColumns();
    this.setTableFixedHeight();
    window.addEventListener('resize', this.setTableFixedHeight, false);
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.setTableFixedHeight, false);
  }

  init() {
    this.initDict();
    this.initData();
  }

  initColumns() {
    this.columns = [
      {
        title: '调转规则名称',
        dataIndex: 'name',
      },
      {
        width: 160,
        title: '班级数量',
        dataIndex: 'miniClassCount',
        render: (value) => value || '0',
      },
      {
        width: 180,
        title: '分公司',
        dataIndex: 'organizationName',
      },
      {
        width: 140,
        title: '年份',
        dataIndex: 'yearName',
      },
      {
        width: 170,
        title: '产品季度',
        dataIndex: 'quarterName',
      },
      {
        width: 170,
        title: '创建人',
        dataIndex: 'createUserName',
      },
      {
        width: 170,
        title: '最后更新人',
        dataIndex: 'updateUserName',
      },
      {
        width: 96,
        title: '操作',
        dataIndex: 'operation',
        render: (_, { id, name, yearId, quarterId, organizationId }) => {
          const params = {
            id,
            name,
            yearId,
            quarterId,
            organizationId,
          };
          return (
            <div>
              {this.props.btnAuth.includes('CLASS_CUST_TRAN_EDIT_RULE') && (
                <a style={{ marginRight: '10px' }} onClick={() => this.onViewDetail(params)}>
                  编辑
                </a>
              )}
              {this.props.btnAuth.includes('CLASS_CUST_TRAN_DELETE_RULE') && (
                <a onClick={(e) => this.onRemove(e, id)}>移除</a>
              )}
            </div>
          );
        },
      },
    ];
  }

  initDict() {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch', 'year', 'quarter'],
      },
    });
  }

  initData() {
    this.onSearch({});
  }

  async fetchList(apiParams) {
    const res = await getClassCourseChangeGroupList(apiParams).catch(errorsHandler);
    if (res && res.data) {
      this.setState({
        total: res.data.total,
        list: res.data.list.map((item, id) => ({ id, ...item })),
      });
    } else {
      this.setState({
        list: [],
        total: 0,
      });
    }
  }

  async loadList(apiParams) {
    this.setState({ loading: true });
    await this.fetchList(apiParams);
    this.setState({ loading: false });
  }

  setTableFixedHeight = () => {
    this.setState({
      tableFixedHeight: Math.max(window.document.body.clientHeight - 348, 100),
    });
  };

  onRemove = (e, id) => {
    e.preventDefault();
    this.currentId = id;
    this.setState({
      isConfirmDeleteModalVisible: true,
    });
  };

  // 创建规则
  onCreateRule = () => {
    this.setState({
      isCreateModalVisible: true,
    });
  };

  // 查看配置说明
  onViewRuleDescription = () => {
    this.setState({
      isDescriptionModalVisible: true,
    });
  };

  onViewDetail = (params) => {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        params,
        refresh: true,
        path: '/smallClassCustomerTransferDetail',
      },
    });
  };

  // 搜索
  onSearch = (searchData) => {
    this.setState(
      () => {
        return {
          searchData,
          pageNum: 1,
        };
      },
      () => {
        this.loadList({
          ...this.state.searchData,
          pageNum: this.state.pageNum,
          pageSize: this.state.pageSize,
        });
      },
    );
  };

  // 重置
  onReset = () => {
    this.setState({ searchData: {} }); // 清空搜索条件
  };

  // 分页
  onPageChange = ({ current: pageNum, pageSize }) => {
    this.setState(
      () => {
        return {
          pageNum,
          pageSize,
        };
      },
      () => {
        this.loadList({
          ...this.state.searchData,
          pageNum: this.state.pageNum,
          pageSize: this.state.pageSize,
        });
      },
    );
  };

  onSubmitRule = (values) => {
    return addClassCourseChangeGroup(values)
      .then((result) => {
        if (result.resultCode === 0) {
          this.initData();
          this.onCancelCreateRule();
          showMessage('创建成功', 'success');
          setTimeout(() => {
            this.props.dispatch({
              type: 'main/openPage',
              payload: {
                refresh: true,
                params: { ...values, id: result.data },
                path: '/smallClassCustomerTransferDetail',
              },
            });
          }, 1000);
        } else {
          showMessage(result.resultMessage || '创建失败，请稍后重试', 'error');
        }
      })
      .catch(errorsHandler);
  };

  onCancelCreateRule = () => {
    this.setState({
      isCreateModalVisible: false,
    });
  };

  handleDeleteRule = () => {
    this.setState({ isDeleteLoading: true });
    deleteClassCourseChangeGroup({ id: this.currentId })
      .then((result) => {
        if (result.resultCode === 0) {
          this.initData();
          this.handleCancelDeleteRule();
          showMessage('规则移除成功', 'success');
        } else {
          showMessage(result.resultMessage || '规则移除失败，请稍后重试', 'error');
        }
      })
      .catch(errorsHandler)
      .finally(() => {
        this.setState({ isDeleteLoading: false });
      });
  };

  handleCancelDeleteRule = () => {
    this.setState({
      isConfirmDeleteModalVisible: false,
    });
  };

  onCancelViewRule = () => {
    this.setState({
      isDescriptionModalVisible: false,
    });
  };

  generateCreateCustomerTransferRuleModal() {
    const props = {
      onSubmit: this.onSubmitRule,
      onCancel: this.onCancelCreateRule,
    };
    return this.state.isCreateModalVisible && <CreateCustomerTransferRuleModal {...props} />;
  }

  generateCustomerTransferRuleDescriptionModal() {
    const props = {
      onCancel: this.onCancelViewRule,
    };
    return this.state.isDescriptionModalVisible && <CustomerTransferRuleDescriptionModal {...props} />;
  }

  // 二次确认弹框
  generateConfirmDeleteModal() {
    const props = {
      widthSize: 's',
      title: '温馨提示',
      onOk: this.handleDeleteRule,
      loading: this.state.isDeleteLoading,
      onCancel: this.handleCancelDeleteRule,
    };

    return (
      this.state.isConfirmDeleteModalVisible && (
        <Modal {...props}>
          <div className={styles.confirmTextWrapper}>
            <Icon
              type="exclamation-circle"
              style={{ color: '#ffbf00', fontSize: 22, marginRight: 5, position: 'relative', top: 2 }}
            />
            确认要移除该条规则？
          </div>
        </Modal>
      )
    );
  }

  render() {
    const { state, props } = this;
    const { branchDict, yearDict, quarterDict } = props;
    return (
      <Page
        className={styles.smallClassCustomerTransferWrapper}
        route={props.route}
        loading={state.loading}
        subHeader={
          <div>
            <SearchFilterForm
              ref={(c) => (this.searchRef = c)}
              truncationNum={5}
              onSearch={this.onSearch}
              onReset={this.onReset}
              appendToolbarBody={
                props.btnAuth.includes('CLASS_CUST_TRAN_ADD_RULE') && (
                  <div style={{ position: 'absolute', right: '8px' }}>
                    <Button icon="plus" type="primary" onClick={this.onCreateRule}>
                      创建调转规则
                    </Button>
                  </div>
                )
              }
            >
              <SearchFieldForm name="name" span={2}>
                <Input placeholder="调转规则名称" maxLength={32} />
              </SearchFieldForm>
              <SearchFieldForm name="branchId">
                <Select showSearch dict={branchDict} placeholder="分公司" />
              </SearchFieldForm>
              <SearchFieldForm name="yearId">
                <Select showSearch dict={yearDict} placeholder="年份" />
              </SearchFieldForm>
              <SearchFieldForm name="quarterId">
                <Select showSearch dict={quarterDict} placeholder="产品季度" />
              </SearchFieldForm>
            </SearchFilterForm>
          </div>
        }
        headerToolBar={
          <div className={styles.pageTitle} onClick={this.onViewRuleDescription}>
            <Icon type="exclamation-circle" style={{ color: '#CCC', marginRight: '6px' }} />
            查看配置说明
          </div>
        }
        fitWindowHeight
      >
        <Table
          rowKey="id"
          dataSource={state.list}
          columns={this.columns}
          bordered={false}
          fixedHeader={false}
          onChange={this.onPageChange}
          pagination={{
            showQuickJumper: true,
            showSizeChanger: true,
            total: state.total,
            current: state.pageNum,
            pageSize: state.pageSize,
            defaultPageSize: 20,
            pageSizeOptions: ['20', '50', '100'],
            showTotal: (total) => `共 ${total} 条记录`,
          }}
          scroll={{ y: this.state.tableFixedHeight }}
        />
        {this.generateConfirmDeleteModal()}
        {this.generateCreateCustomerTransferRuleModal()}
        {this.generateCustomerTransferRuleDescriptionModal()}
      </Page>
    );
  }
}

const mapStoreToProps = (store) => {
  const {
    account: { btnAuths },
    dict: { branch: branchDict, year: yearDict, quarter: quarterDict },
  } = store;
  return {
    branchDict,
    quarterDict,
    btnAuth: btnAuths || [],
    yearDict: (yearDict || []).sort((a, b) => b.text - a.text),
  };
};

export default connect(mapStoreToProps)(SmallClassCustomerTransfer);
