import React, { Component } from 'react';
import { Tooltip } from 'antd';
import {
  Page,
  List,
  Input,
  Select,
  MultiSelect,
  Button,
  SearchFilterForm,
  SearchFieldForm,
  DateRangePicker,
} from 'COMPONENT';
import SearchStudentAutoComplete, { parseStudent } from 'BLOC/SearchStudentAutoComplete';

import SearchBox from 'SRC/BLoComponents/SearchBox';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import AttendanceModal from './components/AttendanceModal';
import Viewer from 'viewerjs';

import { errorsHandle } from 'UTIL/common';

import { connect } from 'dva';

import { listByQueryNameAndRoleCode } from 'SERVICE/common';
import * as attendanceMakeUpClassApi from 'SERVICE/attendanceMakeUpClass';
import * as broadcastCooperateBankStatementApi from 'SERVICE/broadcastCooperateBankStatement';

import styles from './styles.module.less';
import 'viewerjs/dist/viewer.css';

import moment from 'moment';
import file from 'UTIL/file';

const errorsHandler = errorsHandle();
const preMonthFistDate = moment().subtract(1, 'months').startOf('month').startOf('day');
const defaultDateRange = [preMonthFistDate, moment().endOf('day')];

const supplementStateDict = [
  {
    text: '已补',
    val: 'Y',
  },
  {
    text: '未补',
    val: 'N',
  },
];

class SmallClassCustomerTransfer extends Component {
  state = {
    list: [],

    total: 0,
    pageNum: 1,
    pageSize: 20,
    selectedRowKeys: [],
    selectedStudents: [],
    isExporting: false,
    searchData: {
      courseRangeDate: defaultDateRange,
    },

    campusDict: [],

    loading: false,
    tableFixedHeight: 100,

    isAttendanceModalVisible: false,
  };

  constructor(props) {
    super(props);
    this.currentId = null;
    this.searchRef = null;
  }

  componentWillMount() {
    this.init();
    this.initColumns();
    this.setTableFixedHeight();
    window.addEventListener('resize', this.setTableFixedHeight, false);
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.setTableFixedHeight, false);
  }

  componentDidMount() {
    this.searchRef?.form?.setFieldsValue({
      courseRangeDate: defaultDateRange,
    });
  }

  init() {
    this.initDict();
    this.initData();
  }

  initColumns() {
    this.columns = [
      {
        width: 150,
        title: '上课日期',
        dataIndex: 'courseDate',
      },
      {
        width: 120,
        title: '上课星期',
        dataIndex: 'dayOfWeek',
      },
      {
        width: 150,
        title: '上课时间',
        dataIndex: 'courseTime',
        render: (value, { courseEndTime }) => {
          if (value && courseEndTime) {
            return [value, courseEndTime].join(' ');
          } else {
            return '';
          }
        },
      },
      {
        width: 80,
        title: '小时',
        dataIndex: 'courseHours',
      },
      {
        width: 350,
        title: '小班名称',
        dataIndex: 'miniClassName',
      },
      {
        width: 200,
        title: '所属分公司',
        dataIndex: 'branchName',
      },
      {
        width: 200,
        title: '所属学习中心',
        dataIndex: 'campusName',
      },
      {
        width: 100,
        title: '年级',
        dataIndex: 'gradeName',
      },
      {
        width: 120,
        title: '科目',
        dataIndex: 'subjectName',
      },
      {
        width: 120,
        title: '老师',
        dataIndex: 'teacherName',
      },
      {
        width: 120,
        title: '班主任',
        dataIndex: 'studyManagerName',
      },
      {
        width: 120,
        title: '学生',
        dataIndex: 'studentName',
      },
      {
        width: 120,
        title: '学生上课状态',
        dataIndex: 'studentStatus_',
        render: () => '缺勤',
      },
      {
        width: 120,
        title: '补课状态',
        dataIndex: 'supplementSate',
        render: (_, record) => (record.supplementDate ? '已补课' : '未补课'),
      },
      {
        width: 120,
        title: '补课日期',
        dataIndex: 'supplementDate',
        render: (value) => value || '-',
      },
      {
        width: 250,
        title: '备注',
        dataIndex: 'supplementRemark',
        render: (value) => {
          return value ? (
            <Tooltip title={value}>
              <div className={styles.remark} title={value}>
                {value}
              </div>
            </Tooltip>
          ) : (
            '-'
          );
        },
      },
      {
        width: 80,
        title: '图片',
        className: styles.imageCell,
        dataIndex: 'supplementUrls',
        render: (value) => {
          return value ? (
            <img
              src={value}
              width="24"
              height="24"
              onClick={() => this.handleViewPicture(value)}
              className={styles.img}
            />
          ) : (
            '-'
          );
        },
      },
      {
        title: '操作',
        width: 90,
        fixed: 'right',
        key: 'action',
        getActionConfig: (_, record) => {
          return this.props?.btnAuth?.includes('ADD_MINI_CLASS_ATTE')
            ? [{ text: '补课登记', onClick: () => this.handleAddAttendance(record) }]
            : [];
        },
      },
    ];
  }

  initDict() {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch'],
      },
    });
  }

  initData() {
    this.onSearch(this.state.searchData);
  }

  reload() {
    this.onSearch(this.state.searchData);
  }

  setParams(apiParams) {
    if (apiParams.studentId) {
      const student = parseStudent(apiParams.studentId);
      if (!student || !student.studentId) {
        return showMessage('请输入正确的学生');
      }
      apiParams.studentId = student.studentId;
    }

    if (apiParams.courseRangeDate) {
      if (moment.isMoment(apiParams.courseRangeDate[0])) {
        apiParams.courseStartDate = apiParams.courseRangeDate[0].format('YYYY-MM-DD');
        apiParams.courseEndDate = apiParams.courseRangeDate[1].format('YYYY-MM-DD');
      } else {
        apiParams.courseStartDate = apiParams.courseRangeDate[0];
        apiParams.courseEndDate = apiParams.courseRangeDate[1];
      }

      delete apiParams.courseRangeDate;
    }
  }

  async fetchList(apiParams) {
    this.setParams(apiParams);
    const res = await attendanceMakeUpClassApi.getAttendanceMinClassList(apiParams).catch(errorsHandler);
    if (res && res.data) {
      this.setState({
        total: res.data.total,
        list: res.data.list.map((item, id) => ({ id, ...item })),
      });
    } else {
      this.setState({
        list: [],
        total: 0,
      });
    }
  }

  async loadList(apiParams) {
    this.setState({ loading: true });
    await this.fetchList(apiParams);
    this.setState({ loading: false });
  }

  setTableFixedHeight = () => {
    this.setState({
      tableFixedHeight: Math.max(window.document.body.clientHeight - 348, 100),
    });
  };

  // 搜索
  onSearch = (searchData) => {
    this.setState(
      () => {
        return {
          searchData,
          pageNum: 1,
        };
      },
      () => {
        this.loadList({
          ...this.state.searchData,
          pageNum: this.state.pageNum,
          pageSize: this.state.pageSize,
        });
      },
    );
  };

  // 重置
  onReset = () => {
    this.setState({
      searchData: {
        courseRangeDate: defaultDateRange,
      },
    });
    this.searchRef?.form?.setFieldsValue({
      courseRangeDate: defaultDateRange,
    });
  };

  // 分页
  onPageChange = ({ current: pageNum, pageSize }) => {
    this.setState(
      () => {
        return {
          pageNum,
          pageSize,
        };
      },
      () => {
        this.loadList({
          ...this.state.searchData,
          pageNum: this.state.pageNum,
          pageSize: this.state.pageSize,
        });
      },
    );
  };

  handleBranchIdChange = async (branchId) => {
    let campusDict = [];

    if (!branchId) {
      return this.setState({ branchId, campusDict, blCampusId: '', pageNum: 1 });
    }

    const result = await broadcastCooperateBankStatementApi
      .findOrganizationByParentId({ parentId: branchId, orgType: 'CAMPUS' })
      .catch(errorsHandle());
    if (result.resultCode === 0 && result.data && result.data.length) {
      campusDict = [...campusDict, ...result.data.map(({ name, id }) => ({ val: id, text: name }))];
    }

    this.setState({ branchId, campusDict, blCampusId: '', pageNum: 1 });
  };

  handleViewPicture = (url) => {
    const image = new Image();
    image.src = url;
    const viewer = new Viewer(image, {
      hidden: function () {
        viewer.destroy();
      },
    });
    viewer.show();
  };

  // 批量登记
  handleBatchAttend = () => {
    if (!this.state.selectedRowKeys.length) {
      return showMessage('请选择学生！');
    }
    this.setState({
      isAttendanceModalVisible: true,
      selectedStudents: this.state.list.filter((item) => this.state.selectedRowKeys?.includes(item.attendanceId)),
    });
  };

  // 导出
  handleExport = () => {
    if (!this.state.total) {
      return showMessage('当前查询数据为空');
    }
    this.setState({ isExporting: true });
    const apiParams = {
      ...this.state.searchData,
      pageNum: this.state.pageNum,
      pageSize: this.state.pageSize,
    };
    this.setParams(apiParams);
    attendanceMakeUpClassApi
      .exportAttendanceMinClassList(apiParams)
      .then((res) => {
        showMessage('导出Excel成功', 'success');
        file.downloadDirect(res.data, `小班缺勤学生列表_${Date.now()}`);
      })
      .catch((e) => {
        showMessage(e.resultMessage || e.text || '导出数据失败，请稍后重试', 'error');
      })
      .finally(() => {
        this.setState({ isExporting: false });
      });
  };

  handleAddAttendance = (student) => {
    this.setState({
      isAttendanceModalVisible: true,
      selectedStudents: [{ ...student }],
    });
  };

  handleSubmitAttendance = (isBatch) => {
    if (isBatch) {
      this.setState({
        selectedRowKeys: [],
        selectedStudents: [],
      });
    }
    this.reload();
    setTimeout(this.handleCloseAttendanceModal, 100);
  };

  handleCloseAttendanceModal = () => {
    this.setState({ isAttendanceModalVisible: false });
  };

  generateToolbar() {
    const hasBatchAuth = this.props?.btnAuth?.includes('BATCH_ADD_MINI_CLASS_ATTE');
    const hasAddAuth = this.props?.btnAuth?.includes('EXPORT_MINI_CLASS_ATTE');
    return (
      (hasBatchAuth || hasAddAuth) && (
        <div style={{ marginLeft: 7 }}>
          {hasBatchAuth && (
            <Button type="primary" style={{ marginRight: '14px' }} onClick={this.handleBatchAttend}>
              批量登记
            </Button>
          )}
          {hasAddAuth && (
            <Button type="primary" onClick={this.handleExport} loading={this.state.isExporting}>
              导出Excel
            </Button>
          )}
        </div>
      )
    );
  }

  generateList() {
    const { state } = this;
    const { selectedRowKeys } = state;
    const rowSelection = {
      selectedRowKeys,
      onChange: (selectedRowKeys) => {
        this.setState({ selectedRowKeys });
      },
    };

    return (
      <List
        rowKey="attendanceId"
        dataSource={state.list}
        columns={this.columns}
        bordered={false}
        fixedHeader={false}
        rowSelection={rowSelection}
        onChange={this.onPageChange}
        pagination={{
          showQuickJumper: true,
          showSizeChanger: true,
          total: state.total,
          current: state.pageNum,
          pageSize: state.pageSize,
          defaultPageSize: 20,
          pageSizeOptions: ['20', '50', '100'],
          showTotal: (total) => `共 ${total} 条记录`,
        }}
        scroll={{
          y: this.state.tableFixedHeight,
          x: _.reduce(this.columns, (sum, data) => (sum += data.width || 0), 0),
        }}
      />
    );
  }

  generateModal() {
    const { selectedStudents } = this.state;
    const props = {
      onSuccess: this.handleSubmitAttendance,
      onClose: this.handleCloseAttendanceModal,
      students: selectedStudents,
    };
    return <AttendanceModal {...props} />;
  }

  render() {
    const { state, props } = this;
    const { branchDict } = props;

    return (
      <Page
        className={styles.smallClassCustomerTransferWrapper}
        route={props.route}
        loading={state.loading}
        subHeader={
          <div>
            <SearchFilterForm
              ref={(c) => (this.searchRef = c)}
              truncationNum={8}
              onSearch={this.onSearch}
              onReset={this.onReset}
              appendToolbarBody={this.generateToolbar()}
            >
              <SearchFieldForm name="courseRangeDate">
                <DateRangePicker
                  placeholder={['开始日期', '结束日期']}
                  allowClear={false}
                  disabledDate={(currentDate) =>
                    currentDate.startOf('day').valueOf() < preMonthFistDate.startOf('day').valueOf()
                  }
                />
              </SearchFieldForm>
              <SearchFieldForm name="miniClassName">
                <Input placeholder="小班名称" maxLength={32} />
              </SearchFieldForm>
              <SearchFieldForm name="branchId">
                <Select dict={branchDict} placeholder="分公司" onChange={this.handleBranchIdChange} showSearch={true} />
              </SearchFieldForm>
              <SearchFieldForm name="campusIds">
                <MultiSelect mode="multiple" dict={state.campusDict} placeholder="学习中心" showSearch={true} />
              </SearchFieldForm>
              <SearchFieldForm name="teacherId" form={this.props.form}>
                <SearchBox
                  mode="edit"
                  size="large"
                  isMultiple={false}
                  paramName="queryName"
                  placeholder="任课老师"
                  isClearOptionsAfterSelect={true}
                  getData={listByQueryNameAndRoleCode}
                  extraParams={{ roleCode: 'TEATCHER', number: 100 }}
                  onLoaded={(list) =>
                    list.map(({ userId, name, employeeNo }) => ({
                      code: userId,
                      name: `${employeeNo ? `${name}（${employeeNo}）` : name}`,
                    }))
                  }
                />
              </SearchFieldForm>
              <SearchFieldForm name="studyManagerId" form={this.props.form}>
                <SearchBox
                  mode="edit"
                  size="large"
                  isMultiple={false}
                  paramName="queryName"
                  placeholder="班主任"
                  isClearOptionsAfterSelect={true}
                  getData={listByQueryNameAndRoleCode}
                  extraParams={{ roleCode: 'HOMEROOM_TEACHER', number: 100 }}
                  onLoaded={(list) =>
                    list.map(({ userId, name, employeeNo }) => ({
                      code: userId,
                      name: `${employeeNo ? `${name}（${employeeNo}）` : name}`,
                    }))
                  }
                />
              </SearchFieldForm>
              <SearchFieldForm name="studentId" form={this.props.form}>
                <SearchStudentAutoComplete width={250} />
              </SearchFieldForm>
              <SearchFieldForm name="supplementState">
                <Select dict={supplementStateDict} placeholder="补课状态" />
              </SearchFieldForm>
            </SearchFilterForm>
          </div>
        }
        fitWindowHeight
      >
        {/*this.generateToolbar()*/}
        {this.generateList()}
        {state.isAttendanceModalVisible && this.generateModal()}
      </Page>
    );
  }
}

const mapStoreToProps = (store) => {
  const {
    account: { btnAuths },
    dict: { branch: branchDict, year: yearDict, quarter: quarterDict },
  } = store;
  return {
    branchDict,
    quarterDict,
    btnAuth: btnAuths || [],
    yearDict: (yearDict || []).sort((a, b) => b.text - a.text),
  };
};

export default connect(mapStoreToProps)(SmallClassCustomerTransfer);
