import React from 'react';
import { connect } from 'dva';
import { SearchBar, Select, Input } from 'COMPONENT';
import { DatePicker } from 'antd';
import { getLevelCategory, getTeacherByCompany } from 'SERVICE/course';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import moment from 'moment';

import { loadCampusDict } from 'UTIL/dict';

const { RangePicker } = DatePicker;

const getSearchInitValue = () => {
  return {
    grade: '',
    branchId: '',
    subject: '',
    searchTime: null,
    courseStatus: '',
    teachingType: '',
    blCampusId: '',
    miniClassName: '',
  };
};

class Search extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态
      searchParams: getSearchInitValue(true),
      teacherList: [],
      courseStatusList: [],
      teachingTypeList: [
        { text: '面授', val: '1' },
        { text: '在线A', val: '2' },
        { text: '在线B', val: '3' },
        { text: '自习', val: '4' },
        { text: 'AI录播课', val: '5' },
      ],
      campusList: [],
    };
  }

  componentDidMount() {
    this.initDictionary();
    this.fetchCourseStatus();

    const els = document.querySelectorAll('.shouldResizeWidth');
    [...els].forEach((el) => {
      if (el.parentNode.classList.contains('SearchBar-control')) {
        el.parentNode.style.width = '33.33334%';
      }
    });
  }

  fetchTeach = (brenchId) => {
    getTeacherByCompany({ brenchId }).then((res) => {
      let teacher = res || [];
      let tempObj = {};
      teacher = teacher.reduce((pre, cur) => {
        if (!tempObj[cur.userId]) {
          tempObj[cur.userId] = true;
          pre.push({
            text: cur.name,
            val: cur.userId,
          });
        }
        return pre;
      }, []);

      this.setState(
        {
          teacherList: teacher,
        },
        () => {
          this.setSearchParams('teacherId', '');
        },
      );
    });
  };

  fetchCourseStatus = () => {
    getLevelCategory({
      category: 'MINI_CLASS_COURSE_STATUS',
    }).then((res) => {
      let courseStatusList = res.data || [];
      courseStatusList.forEach((course) => {
        course.text = course.name;
        course.val = course.value;
      });

      this.setState({
        courseStatusList,
      });
    });
  };

  fetchCampusByBranchId = (id) => {
    loadCampusDict(id).then((res) => {
      this.setState(
        {
          campusList: res || [],
        },
        () => {
          this.setSearchParams('blCampusId', '');
        },
      );
    });
  };

  initDictionary() {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch', 'grade', 'subject', 'branchAllCampus', 'productGroup'],
      },
    });
  }

  formatTime = (momentTime) => {
    return Number(momentTime.format('x'));
  };

  formatParams = (params) => {
    return Object.keys(params).reduce((obj, key) => {
      if (params[key]) {
        switch (key) {
          case 'searchTime':
            if (params[key].length > 0) {
              obj.startDate = params[key][0].format('YYYY-MM-DD');
              obj.endDate = params[key][1].format('YYYY-MM-DD');
            }
            break;
          default:
            obj[key] = params[key];
        }
      }

      return obj;
    }, {});
  };

  setSearchParams = (key, value, cb = function () {}) => {
    this.setState(
      {
        searchParams: {
          ...this.state.searchParams,
          [key]: value,
        },
      },
      cb,
    );
  };

  resetSearchParams = () => {
    this.setState({
      searchParams: getSearchInitValue(),
    });
  };

  render() {
    const { searchExpand, searchParams, teacherList, courseStatusList, teachingTypeList, campusList } = this.state;
    const { STUDENT_GRADE, COMPANY, SUBJECT, PRODUCT_GROUP } = this.props;
    const {
      subject,
      grade,
      branchId,
      searchTime,
      courseStatus,
      teachingType,
      teacherId,
      blCampusId,
      miniClassName,
      productGroupId,
    } = searchParams;

    return (
      <SearchBar
        expand={searchExpand}
        onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
        onSearch={() => this.props.onSearch(this.formatParams(searchParams))}
        onReset={this.resetSearchParams}
      >
        <RangePicker
          className="shouldResizeWidth"
          size="large"
          value={searchTime}
          placeholder={['课程日期开始时间', '课程日期结束时间']}
          onChange={(value) => {
            console.log({ value });
            const [startTime, endTime] = value;
            const diffDays = moment(endTime).diff(moment(startTime), 'days');
            console.log({ diffDays, searchTime });
            if (diffDays > 90) {
              return showMessage('时间跨度不能超过三个月');
            }
            this.setSearchParams('searchTime', value);
          }}
        />
        <Select
          showSearch
          placeholder="所属分公司"
          allowClear={false}
          dict={COMPANY}
          value={branchId}
          onChange={(value) => {
            this.setSearchParams('branchId', value);
            this.fetchCampusByBranchId(value);
            this.fetchTeach(value);
          }}
        />

        <Input
          placeholder="小班名称"
          value={miniClassName}
          onChange={(e) => {
            this.setSearchParams('miniClassName', e.target.value);
          }}
        />

        <Select
          placeholder="学习中心"
          dict={campusList}
          value={blCampusId}
          onChange={(value) => {
            this.setSearchParams('blCampusId', value);
          }}
          allowClear={true}
          showSearch={true}
        />

        <Select
          placeholder="产品组"
          dict={PRODUCT_GROUP}
          value={productGroupId}
          onChange={(value) => {
            this.setSearchParams('productGroupId', value);
          }}
          allowClear={true}
          showSearch={true}
        />

        <Select
          showSearch
          placeholder="年级"
          dict={STUDENT_GRADE}
          value={grade}
          onChange={(value) => {
            this.setSearchParams('grade', value);
          }}
        />

        <Select
          showSearch
          placeholder="科目"
          dict={SUBJECT}
          value={subject}
          onChange={(value) => {
            this.setSearchParams('subject', value);
          }}
        />

        <Select
          showSearch
          placeholder="老师"
          dict={teacherList}
          value={teacherId}
          onChange={(value) => {
            this.setSearchParams('teacherId', value);
          }}
        />

        <Select
          placeholder="状态"
          dict={courseStatusList}
          value={courseStatus}
          onChange={(value) => {
            this.setSearchParams('courseStatus', value);
          }}
        />

        <Select
          placeholder="-上课方式-"
          dict={teachingTypeList}
          value={teachingType}
          onChange={(value) => {
            this.setSearchParams('teachingType', value);
          }}
        />
      </SearchBar>
    );
  }
}

export default connect((state) => {
  return {
    STUDENT_GRADE: state.dict['grade'],
    COMPANY: state.dict['branch'],
    SUBJECT: state.dict['subject'],
    CAMPUS: state.dict['branchAllCampus'],
    PRODUCT_GROUP: state.dict['productGroup'],
  };
})(Search);
