import React from 'react';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import loaders from '../loaders';
import Select from 'COMPONENT/Select';
import DatePicker from 'COMPONENT/DatePicker';
import TimePicker from 'COMPONENT/TimePicker';
import styles from './index.less';
import moment from 'moment';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { getDictTextByVal } from 'UTIL/dict';
import { formatCourseToHour } from 'UTIL/common';
import { teachingTypeEnum, productGroupEnum } from 'CONST/dicts/course';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class EditCourse extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      miniClassDataSource: [],
      miniCourseDataSource: [],
      title: '',
      error: null,
      border: styles.border,
      conflictCourseNum: [], // 原班冲突讲数
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const { miniClassId } = this.props;
    this.setState({ loading: true });
    let [miniClassDataSource, miniCourseDataSource] = await Promise.all([
      loaders.loadConflictMiniClassInfo({ miniClassId }),
      loaders.loadConflictMiniCourseInfo({ miniClassId }),
    ]);
    this.setState({
      loading: false,
      miniClassDataSource,
      miniCourseDataSource: miniCourseDataSource.map((item) => ({ ...item, isDisabled: this.isDisabledRow(item) })),
      title: miniClassDataSource[0].miniClassName,
    });
  };

  isDisabledRow = (row) => {
    const isBefore = moment(`${row.courseDate} ${row.courseTime}`).isBefore(new Date());
    const isDisabledType = row.teachingType === teachingTypeEnum.LIVE || row.teachingType === teachingTypeEnum.RECORDED;

    return isBefore && isDisabledType;
  };

  upDateRow = (dataSource, index, changed) => {
    const { courseNum } = this.props;
    const currentCourseNum =
      dataSource === 'miniCourseDataSource' ? this.state.miniCourseDataSource[index].courseNum : null;
    if (dataSource === 'miniCourseDataSource' && courseNum === currentCourseNum) {
      this.setState({ border: '' });
    }
    this.setState({
      [dataSource]: this.state[dataSource].map((row, i) => {
        if (index === i) {
          return { ...row, ...changed };
        }
        return row;
      }),
      border: '',
    });
  };

  // 检出修改过后的课程(未提交修改)
  getCoursesOfChange = (typeValue) => {
    const { miniCourseDataSource } = this.state;
    const { type, miniClassId, closeModal, search } = this.props;
    let courses = miniCourseDataSource
      .map((course) => {
        const { courseId, courseNum, courseTime, courseDate, courseTimeLength, courseStatus, teachingType } = course;
        let hour = courseTime.slice(0, 2);
        let minute = courseTime.slice(3);
        let startTime = courseTime.replace(':', '-');
        let endTime = moment(courseDate)
          .set({ hour: hour, minute: minute })
          .add(courseTimeLength, 'minutes')
          .format()
          .slice(11, 16)
          .replace(':', '-');
        return {
          courseStatus,
          miniClassId,
          courseId,
          courseNum,
          endTime,
          personOrRoomeId: course[typeValue][typeValue + 'Id'],
          startDate: courseDate,
          startTime,
          type,
          teachingType,
        };
      })
      .filter((course) => {
        return course.courseStatus === 'NEW';
      });
    return courses;
  };

  onOk = async () => {
    const { type, miniClassId, closeModal, search } = this.props;

    if (!this.validateCourseTime()) {
      showMessage('“在线A”或“在线B”的课程上课时间不能选择历史时间~');
      return;
    }

    let params = type === 'teacher' ? this.getCoursesOfChange('teacher') : this.getCoursesOfChange('classroom');
    params = JSON.parse(JSON.stringify(params));

    this.setState({ loading: true });
    let conflictCourseNum = await loaders.loadChangetoGetConflictInfo(params);
    this.setState({ loading: false, conflictCourseNum, border: styles.border });
    if (conflictCourseNum.length === 0) {
      await loaders.changeClassRoomOrTeacherApi(JSON.parse(JSON.stringify(params)));
      closeModal();
      search();
    } else {
      let tipString =
        type === 'teacher' ? '该老师仍存在排课冲突，是否继续修改？' : '该教室仍存在排课冲突，是否继续修改？';
      showConfirm(tipString, async () => {
        await loaders.changeClassRoomOrTeacherApi(JSON.parse(JSON.stringify(params)));
        closeModal();
        search();
      });
    }
  };

  validateCourseTime = () => {
    const { miniCourseDataSource: courses, miniClassDataSource } = this.state;
    if (miniClassDataSource.length && miniClassDataSource[0].productGroupId === productGroupEnum.ONLINE_CLASS)
      return true;

    for (let i = 0; i < courses.length; i++) {
      const courseItem = courses[i];
      const isBefore = moment(`${courseItem.courseDate} ${courseItem.courseTime}`).isBefore(new Date());
      if (
        !courseItem.isDisabled &&
        (courseItem.teachingType === teachingTypeEnum.LIVE || courseItem.teachingType === teachingTypeEnum.RECORDED) &&
        isBefore
      ) {
        return false;
      }
    }

    return true;
  };

  // 生成课程的时间选择器
  renderTimePicker = (record, index, conflictCourseNum, border, courseNum, disable) => {
    if (conflictCourseNum.includes(record.courseNum)) {
      return (
        <TimePicker
          disabled={disable}
          className={border}
          value={record.courseTime}
          onChange={(value) => {
            this.upDateRow('miniCourseDataSource', index, {
              courseTime: value,
            });
          }}
        />
      );
    } else {
      return (
        <TimePicker
          disabled={disable}
          value={record.courseTime}
          onChange={(value) => {
            this.upDateRow('miniCourseDataSource', index, {
              courseTime: value,
            });
          }}
        />
      );
    }
  };

  /**
   * 改变任课老师或教室
   *
   * @param value 老师/教室ID
   * @param type 类型 teacher：老师，classroom：教室
   */
  changeClassTeacherOrClassRoom = (value, type) => {
    let { miniCourseDataSource } = this.state;
    const { teacherDict, classroomDict } = this.props;

    if (type === 'teacher') {
      const teacherName = getDictTextByVal(teacherDict, value);

      miniCourseDataSource = miniCourseDataSource.map((course) => {
        if (course.courseStatus === 'NEW' && !course.isDisabled) {
          return Object.assign(course, {
            teacher: {
              teacherName: teacherName,
              teacherId: value,
            },
          });
        } else {
          return course;
        }
      });
    } else {
      const classroomName = getDictTextByVal(classroomDict, value);

      miniCourseDataSource = miniCourseDataSource.map((course) => {
        if (course.courseStatus === 'NEW') {
          return Object.assign(course, {
            classroom: {
              classroomName: classroomName,
              classroomId: value,
            },
          });
        } else {
          return course;
        }
      });
    }

    this.setState(miniCourseDataSource);
  };

  render() {
    const { loading, miniClassDataSource, miniCourseDataSource, title, error, border, conflictCourseNum } = this.state;
    const { closeModal, type, classroomDict, teacherDict, courseNum } = this.props;
    let miniClassColumns = null;

    if (type === 'teacher') {
      miniClassColumns = [
        {
          title: '任课老师',
          dataIndex: 'teacher',
          render: (text, record, index) => {
            return (
              <Select
                allowClear={false}
                showSearch={true}
                dict={teacherDict}
                value={record.teacher.teacherId}
                onChange={(value) => {
                  this.changeClassTeacherOrClassRoom(value, type);
                  this.upDateRow('miniClassDataSource', index, {
                    teacher: {
                      teacherName: getDictTextByVal(teacherDict, value),
                      teacherId: value,
                    },
                  });
                }}
              />
            );
          },
        },
        { title: '教室', dataIndex: 'classroomName' },
        { title: '科目', dataIndex: 'subject' },
        { title: '年级', dataIndex: 'grade' },
        {
          title: '小时数/讲',
          dataIndex: 'everyCourseNum',
          render: (text, record) => {
            return formatCourseToHour(text, record.classTimeLength);
          },
        },
        // { title: "课时时长", dataIndex: "classTimeLength",width:100 }
      ];
    } else {
      miniClassColumns = [
        {
          title: '任课老师',
          dataIndex: 'teacherName',
        },
        {
          title: '教室',
          dataIndex: 'classroom',
          render: (text, record, index) => {
            return (
              <Select
                allowClear={false}
                showSearch={true}
                dict={classroomDict}
                value={record.classroom.classroomId}
                onChange={(value) => {
                  this.changeClassTeacherOrClassRoom(value, type);
                  this.upDateRow('miniClassDataSource', index, {
                    classroom: {
                      classroomName: getDictTextByVal(classroomDict, value),
                      classroomId: value,
                    },
                  });
                }}
              />
            );
          },
        },
        { title: '科目', dataIndex: 'subject' },
        { title: '年级', dataIndex: 'grade' },
        {
          title: '小时数/讲',
          dataIndex: 'everyCourseNum',
          render: (text, record) => {
            return formatCourseToHour(text, record.classTimeLength);
          },
        },
        // { title: "课时时长", dataIndex: "classTimeLength",width:100 }
      ];
    }

    let miniCourseColumns = null;
    if (type === 'teacher') {
      miniCourseColumns = [
        { title: '序号', dataIndex: 'courseNum', width: 60 },
        {
          title: '老师',
          dataIndex: 'teacher',
          render: (text, record, index) => {
            let teacherType = ['面授', '在线A', '在线B', '自习', 'AI录播课'];
            let disable = record.courseStatus !== 'NEW' || record.isDisabled;
            let teacherTypeStr = record.teachingType === null ? '' : teacherType[record.teachingType - 1];
            const isABType = ['在线A', '在线B'].includes(teacherTypeStr);
            if (isABType) {
              disable = false;
            }
            return (
              <Select
                disabled={disable}
                showSearch={true}
                allowClear={false}
                dict={teacherDict}
                value={record.teacher.teacherId}
                onChange={(value) => {
                  this.upDateRow('miniCourseDataSource', index, {
                    teacher: {
                      teacherName: getDictTextByVal(teacherDict, value),
                      teacherId: value,
                    },
                  });
                }}
              />
            );
          },
        },
        {
          title: '日期',
          dataIndex: 'courseDate',
          width: 180,
          render: (text, record, index) => {
            let teacherType = ['面授', '在线A', '在线B', '自习', 'AI录播课'];
            let disable = record.courseStatus !== 'NEW' || record.isDisabled;
            let teacherTypeStr = record.teachingType === null ? '' : teacherType[record.teachingType - 1];
            const isABType = ['在线A', '在线B'].includes(teacherTypeStr);
            if (isABType) {
              disable = false;
            }
            const disabledDate = (current) => {
              if (isABType) {
                return current && current < moment().endOf('day');
              }
              return true;
            };
            return (
              <DatePicker
                disabled={disable}
                value={record.courseDate}
                disabledDate={disabledDate}
                onChange={(value) => {
                  this.upDateRow('miniCourseDataSource', index, {
                    courseDate: value,
                  });
                }}
              />
            );
          },
        },
        {
          title: '时间',
          dataIndex: 'courseTime',
          render: (text, record, index) => {
            let disable = record.courseStatus !== 'NEW' || record.isDisabled;
            let teacherType = ['面授', '在线A', '在线B', '自习', 'AI录播课'];
            let teacherTypeStr = record.teachingType === null ? '' : teacherType[record.teachingType - 1];
            const isABType = ['在线A', '在线B'].includes(teacherTypeStr);
            if (isABType) {
              disable = false;
            }
            return this.renderTimePicker(record, index, conflictCourseNum, border, courseNum, disable);
          },
        },
        {
          title: '上课方式',
          dataIndex: 'teachingType',
          render: (text, record) => {
            let teacherType = ['面授', '在线A', '在线B', '自习', 'AI录播课'];
            return record.teachingType === null ? '' : teacherType[record.teachingType - 1];
          },
        },
      ];
    } else {
      miniCourseColumns = [
        { title: '序号', dataIndex: 'courseNum', width: 60 },
        {
          title: '教室',
          dataIndex: 'classroom',
          render: (text, record, index) => {
            let disable = record.courseStatus !== 'NEW' || record.isDisabled;
            let teacherType = ['面授', '在线A', '在线B', '自习', 'AI录播课'];
            let teacherTypeStr = record.teachingType === null ? '' : teacherType[record.teachingType - 1];
            const isABType = ['在线A', '在线B'].includes(teacherTypeStr);
            if (isABType) {
              disable = false;
            }
            return (
              <Select
                disabled={disable}
                showSearch={true}
                allowClear={false}
                dict={classroomDict}
                value={record.classroom.classroomId}
                onChange={(value) => {
                  this.upDateRow('miniCourseDataSource', index, {
                    classroom: {
                      classroomName: getDictTextByVal(classroomDict, value),
                      classroomId: value,
                    },
                  });
                }}
              />
            );
          },
        },
        {
          title: '日期',
          dataIndex: 'courseDate',
          render: (text, record, index) => {
            let teacherType = ['面授', '在线A', '在线B', '自习', 'AI录播课'];
            let teacherTypeStr = record.teachingType === null ? '' : teacherType[record.teachingType - 1];
            let disable = record.courseStatus !== 'NEW' || record.isDisabled;
            const isABType = ['在线A', '在线B'].includes(teacherTypeStr);
            if (isABType) {
              disable = false;
            }
            const disabledDate = (current) => {
              if (isABType) {
                return current && current < moment().endOf('day');
              }
              return true;
            };
            return (
              <DatePicker
                disabled={disable}
                allowClear={false}
                value={record.courseDate}
                disabledDate={disabledDate}
                onChange={(value) => {
                  this.upDateRow('miniCourseDataSource', index, {
                    courseDate: value,
                  });
                }}
              />
            );
          },
        },
        {
          title: '时间',
          dataIndex: 'courseTime',
          render: (text, record, index) => {
            let teacherType = ['面授', '在线A', '在线B', '自习', 'AI录播课'];
            let teacherTypeStr = record.teachingType === null ? '' : teacherType[record.teachingType - 1];
            let disable = record.courseStatus !== 'NEW' || record.isDisabled;
            const isABType = ['在线A', '在线B'].includes(teacherTypeStr);
            if (isABType) {
              disable = false;
            }
            return this.renderTimePicker(record, index, conflictCourseNum, border, courseNum, disable);
          },
        },
        {
          title: '上课方式',
          dataIndex: 'teachingType',
          render: (text, record) => {
            let teacherType = ['面授', '在线A', '在线B', '自习', 'AI录播课'];
            return record.teachingType === null ? '' : teacherType[record.teachingType - 1];
          },
        },
      ];
    }

    return (
      <Modal title={title} width="759" error={error} loading={loading} onCancel={closeModal} onOk={this.onOk}>
        <div>
          <List dataSource={miniClassDataSource} columns={miniClassColumns} />
          <List dataSource={miniCourseDataSource} columns={miniCourseColumns} />
        </div>
        <p style={{ marginTop: '20px', color: 'red' }}>课程为在线A或在线B时，不能把上课时间修改为历史时间。</p>
      </Modal>
    );
  }
}

export default EditCourse;
