import _ from 'lodash';
import moment from 'moment';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';

const START_HOUR = '06:00';
const CELL_HEIGHT = 70;
const WEEK_NAMES = ['周一', '周二', '周三', '周四', '周五', '周六', '周日'];
const TIME_RANGE = ['06:00', '08:00', '10:00', '12:00', '14:00', '16:00', '18:00', '20:00', '22:00'];
const TIME_RANGE_AREA = [
  '06:00-08:00',
  '08:00-10:00',
  '10:00-12:00',
  '12:00-14:00',
  '14:00-16:00',
  '16:00-18:00',
  '18:00-20:00',
  '20:00-22:00',
  '22:00-00:00',
];

const TIME_RANGE_AREA_MAP = {};
TIME_RANGE.forEach((time, index) => (TIME_RANGE_AREA_MAP[time] = TIME_RANGE_AREA[index]));

const Utils = {
  getTimeByMinutes(time) {
    const [hour, minute] = time.split(/:/).map((value) => parseInt(value));
    return hour * 60 + minute;
  },

  getHeightPerMinute() {
    return CELL_HEIGHT / (2 * 60);
  },

  getHeightByMinutes(hour) {
    return Utils.getHeightPerMinute() * (Utils.getTimeByMinutes(hour) - Utils.getTimeByMinutes(START_HOUR));
  },

  getTimeRangeAreaByTime(startTime) {
    const startTimeByMinutes = Utils.getTimeByMinutes(startTime);
    return TIME_RANGE_AREA.find((range) => {
      const [rangeStart, rangeEnd] = range.split(/-/).map((time) => Utils.getTimeByMinutes(time));
      return startTimeByMinutes >= rangeStart && startTimeByMinutes < rangeEnd;
    });
  },

  formatExportBodyData: function (roomList) {
    const list = [];
    const flattenRoomList = [];

    roomList.forEach(({ miniClassCourseList }) => {
      if (!miniClassCourseList.length) {
        return;
      }
      flattenRoomList.push(...miniClassCourseList);
    });

    const dataByDateMap = {};
    const dateList = _.sortBy(
      _.unionBy(flattenRoomList, 'courseDate').map(({ courseDate }) => courseDate),
      (courseDate) => new Date(courseDate).getTime(),
    ); // 获取日期

    flattenRoomList.forEach((room) => {
      if (!dataByDateMap[room.courseDate]) {
        dataByDateMap[room.courseDate] = [];
      }
      dataByDateMap[room.courseDate].push({ ...room });
    });

    dateList.forEach((date) => {
      const roomsByDate = _.sortBy(dataByDateMap[date], ({ courseTime }) => Utils.getTimeByMinutes(courseTime)); // 排序
      const dataTimeMap = {};

      roomsByDate.forEach((room) => {
        const timeRange = Utils.getTimeRangeAreaByTime(room.courseTime);
        if (!dataTimeMap[timeRange]) {
          dataTimeMap[timeRange] = [];
        }
        dataTimeMap[timeRange].push({ ...room });
      });

      TIME_RANGE_AREA.forEach((timeRange) => {
        const realRoom = {};
        const rooms = dataTimeMap[timeRange];

        if (!rooms) {
          return;
        }

        realRoom.date = date;
        realRoom.time = timeRange;

        rooms.forEach((room) => {
          const { classroomId } = room;
          if (!realRoom[classroomId]) {
            realRoom[classroomId] = [];
          }
          realRoom[classroomId].push({ ...room });
        });

        list.push({ ...realRoom });
      });
    });

    const markedMap = {};
    const listGroupByDate = _.groupBy(list, 'date');

    return list.map((item) => {
      const { date } = item;
      if (markedMap[date]) {
        return { ...item, rowSpan: 0 };
      } else {
        markedMap[date] = 1;
        return { ...item, rowSpan: listGroupByDate[date].length };
      }
    });
  },

  getExportHeaderList: function (roomHeaderList) {
    return [{ id: 'date', name: '日期', width: 120 }, { id: 'time', name: '上课时间', width: 120 }, ...roomHeaderList];
  },

  getExportBodyRowContent: function (row, column) {
    const { id, type } = column;
    if (type === 'room') {
      return (row[id] || [])
        .map((room) => {
          const {
            planNum,
            enrollNum,
            courseNum,
            courseTime,
            teacherName,
            miniClassName,
            courseEndTime,
            teachingTypeName,
          } = room;
          return `${[
            `<p>${miniClassName}【第${courseNum}讲-${teachingTypeName}】</p>`,
            `<strong>${courseTime}-${courseEndTime}</strong>`,
            `<p>${teacherName} ${enrollNum}/${planNum}</p>`,
          ].join('')}`;
        })
        .join('<br />');
    } else if (id === 'date') {
      const date = row[id];
      const day = moment(date).day();
      return `<strong>${WEEK_NAMES[day === 0 ? 6 : day - 1]}&nbsp;${date.split(/-/).slice(1).join('/')}</strong>`;
    } else if (id === 'time') {
      return `<strong><i>${row[id]}</i></strong>`;
    }
  },

  buildExportHeaderGroup: function (headerList) {
    const header = Utils.getExportHeaderList(headerList);
    return `<colgroup>${header.map(({ width }) => `<col width="${width}"></col>`).join('')}</colgroup>`;
  },

  buildExportBody: function (roomList, headerList) {
    const list = Utils.formatExportBodyData(roomList);
    const bodyTemplate = list.map((room) => Utils.buildExportBodyRow(room, headerList));
    return {
      data: list,
      template: `${bodyTemplate.join('')}`,
    };
  },

  buildExportBodyRow: function (row, headerList) {
    return `
      <tr>
        ${headerList
          .map((column) => {
            const { id, type } = column;
            if (id === 'date') {
              return row.rowSpan
                ? `<td class="text-center" rowSpan="${row.rowSpan}">${Utils.getExportBodyRowContent(row, column)}</td>`
                : '';
            } else {
              return `<td class="${type === 'room' ? 'text-left' : 'text-center'}">${Utils.getExportBodyRowContent(
                row,
                column,
              )}</td>`;
            }
          })
          .join('')}
      </tr>
    `;
  },

  buildExportHeader: function (roomHeaderList) {
    return `<tr>${this.getExportHeaderList(roomHeaderList)
      .map(({ name }) => `<td class="th"><strong>${name}</strong></td>`)
      .join('')}</tr>`;
  },

  buildTableDOM: function (roomList, title = '测试一下呀') {
    const roomHasDataList = roomList.filter(({ miniClassCourseList }) => !!miniClassCourseList.length);
    const roomHeaderList = [
      ...roomHasDataList.map(({ id, classRoom: name }) => ({ id, name, width: 300, type: 'room' })),
    ];
    const headerGroupTemplate = Utils.buildExportHeaderGroup(roomHeaderList); // 生成头部
    const headerTemplate = Utils.buildExportHeader(roomHeaderList); // 生成头部
    const { data, template: bodyTemplate } = Utils.buildExportBody(roomList, Utils.getExportHeaderList(roomHeaderList));
    return {
      data,
      template: `
        <html lang="zh-CN">
          <head>
            <title>${title}</title>
            <style>table {border-collapse: collapse;} p{margin: 0} th, td{font-size: 14px; vertical-align: middle; border: 1px solid #E9E9E9; background: #FFF; padding: 12px 10px;} .th{color: #FFF; text-align: center; background: #4DADA1; height: 50px; font-weight: bold; padding: 0;} .text-left{text-align: left} .text-center{text-align: center}</style>
          </head>
          <body>
            <table>
              ${headerGroupTemplate}
              <tbody>
                ${headerTemplate}
                ${bodyTemplate}
              </tbody>
            </table>
          </body>
        </html>
      `,
    };
  },

  exportExcel(roomList, fileName) {
    return new Promise((resolve, reject) => {
      const reader = new FileReader();
      const { data, template } = Utils.buildTableDOM(roomList);

      if (!data || !data.length) {
        return reject('当前无导出课程');
      }
      if (data.length > 1000) {
        return reject('限制最大导出课程数量为1000');
      }

      reader.addEventListener(
        'load',
        () => {
          let a = document.createElement('a');
          a.href = reader.result;
          a.download = `${fileName}.xls`;
          document.body.appendChild(a);
          a.click();
          a.remove();
          resolve();
        },
        false,
      );
      reader.addEventListener('error', () => {
        reject('导出Excel失败, 请稍后重试');
      });

      if (template) {
        reader.readAsDataURL(new Blob([template], { type: 'application/vnd.ms-excel' }));
      }
    });
  },
};

export default Utils;
