/**
 * 精英班中途退费模态框
 * @property {number} type 操作类型 1: 退费申请  2、查看退费申请  3、退费审批  4、查看退费审批
 * @property {string} promiseStudentId 精英班学生ID
 * @property {string} contractProductId 合同产品ID
 * @property {function} onClose 模态框关闭事件触发时的回调
 * @property {function?} onSave 模态框保存成功后的回调
 */
import React from 'react';
import { Row, Col, Form } from 'antd';
import Radio from 'COMPONENT/Radio';
import Button from 'COMPONENT/Button';
import Modal from 'COMPONENT/Modal';
import Block from 'COMPONENT/Block';
import Popover from 'COMPONENT/Popover';
import TextArea from 'COMPONENT/TextArea';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import { getDictTextByVal, loadCommonDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getSelectOption } from 'SERVICE/common';
import {
  getEcsContractInfo,
  getEcsContractChargeInfo,
  savaPromiseReturnAuditInfo,
  findStudentInfoByPromiseStudentId,
} from 'SERVICE/course';
import StudentInfo from './StudentInfo';
import ContractInfo from './ContractInfo';
import FeeItem from './FeeItem';
import { formatMoney } from './common';
import styles from './styles.less';
import EcsProcess from '../PromiseResultDetail/EcsProcess';
import { ecsProcessStep } from '../PromiseResultDetail/utils';
import styles_ from '../PromiseResultDetail/styles.less';
import UploadImg from '../UploadImg';

const RadioGroup = Radio.Group;

const accountNumberID = 'ACCOUNT_NUMBER_ID';

/**
 * 扣费明细表中，一个季度的一个扣费项目对应的单元格所显示的内容
 * @property {object} data 明细数据，格式如下：
   {
     regular: number 正价金额
     discount: number 优惠金额
     regularHours: number 正价课时数
     discountHours: number 优惠课时数
    }
 */
const FeeCell = ({ data }) => {
  const { regular, discount, regularHours, discountHours } = data || {};
  return (
    <div>
      <FeeItem name="正价扣费" money={regular} />
      {regularHours !== undefined && <div>{`（${regularHours}小时）`}</div>}
      <FeeItem type="discount" name="优惠扣费" money={discount} />
      {discountHours !== undefined && <div>{`（${discountHours}小时）`}</div>}
    </div>
  );
};

/**
 * 在antd的Button组件基础上添加气泡信息提示
 * @property {any} ...props antd的Button组件所有属性
 * @property {string?} info 鼠标悬停时的提示信息
 * @property {string?} error 错误信息
 * @property {string?} onErrorClose 错误信息关闭事件触发时的回调
 */
const ButtonWithPop = ({ info, error, onErrorClose, ...props }) => {
  let btn = <Button {...props} />;
  if (info) {
    btn = (
      <Popover icon="info" content={info} maxWidth={300}>
        {btn}
      </Popover>
    );
  } else if (error) {
    btn = (
      <Popover icon="error" content={error} maxWidth={300} trigger="click" visible={true} onClose={onErrorClose}>
        {btn}
      </Popover>
    );
  }
  btn = <span style={{ marginLeft: 8 }}>{btn}</span>;
  return btn;
};

class PromiseFeeDetail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      studentInfo: {}, // {object} 学生信息
      contractInfo: {}, // {object} 合同信息
      quarterFeeDetails: [], // {Array} 各季度扣费明细
      refundMode: null, // {string} 退费方式 [ RETURN_TO_ACCOUNT：退到电子账户 | RETURN_TO_CUSTOMER：直接退款 ]
      historyOpinions: [], // {Array} 历史审批意见
      opinion: '', // {string} 审批意见
      commitApplyBtnError: null, // {string} 提交申请按钮的错误信息
      cancelRefundBtnError: null, // {string} 撤销退费按钮的错误信息
      approvalPassBtnError: null, // {string} 审批通过按钮的错误信息
      totals: {}, // {object} 根据原始数据计算得到的各种合计数据
      statusDict: null, // {[object]} 字典 学生精英班状态
      isBindVip: null, // {boolean} 是否绑定VIP服务费
      error: null, // {string} 数据加载失败的错误信息
      cashierFile: null,
      cashierAuditRemark: '',
      applicationRefundFile: null,
      returnBankAccount: null, // 银行账号
      returnBankAccountOwner: null, // 银行账号名称
      returnBankName: null, // 银行名称
      returnApplyFileid: null, // 退费申请书文件id
    };

    this.process = ['申请退费', '会计审批', '出纳审批', '完成'];
    this.step = ecsProcessStep(props.courseStatus, props.auditStatus);
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    this.setState({ loading: true });
    Promise.all([
      this.loadStudentInfo(this.props.promiseStudentId),
      this.loadContractInfo(this.props.contractProductId),
      this.loadFeeDetails(this.props.contractProductId),
      loadCommonDict('StudentPromiseStatus'),
    ]).then(
      ([studentInfo, contractInfo, feeDetails, statusDict]) => {
        const { isBindVip, quarterFeeDetails } = feeDetails;
        const { refundMode, opinion, oldAuditRemark, promiseStudentStatus } = studentInfo;
        this.setState({
          loading: false,
          studentInfo: {
            ...studentInfo,
            promiseStudentStatusName: getDictTextByVal(statusDict, promiseStudentStatus),
          },
          contractInfo,
          quarterFeeDetails,
          refundMode: this.props.type === 1 ? null : refundMode,
          historyOpinions: oldAuditRemark ? [{ text: oldAuditRemark }] : [],
          opinion: this.props.type === 3 ? '' : opinion,
          totals: this.compute(contractInfo.actualMoney, quarterFeeDetails),
          statusDict,
          isBindVip,
          cashierFile: studentInfo.cashierFile,
          cashierAuditRemark: studentInfo.cashierAuditRemark,
          returnBankAccount: studentInfo.returnBankAccount,
          returnBankName: studentInfo.returnBankName,
          returnBankAccountOwner: studentInfo.returnBankAccountOwner,
          applicationRefundFile: studentInfo.applicationRefundFile,
        });
      },
      (e) => {
        const error = e.text || '数据加载出错';
        this.setState({ loading: false, error });
      },
    );
  };

  // 加载学生信息
  loadStudentInfo = async (promiseStudentId) => {
    const apiData = await findStudentInfoByPromiseStudentId({ promiseStudentId }, true);
    return {
      name: apiData.studentName,
      cusName: apiData.customerName,
      gradeName: apiData.gradeName,
      contact: apiData.customerContact,
      promiseStudentStatus: apiData.courseStatus,
      promiseClassName: apiData.promiseClassName,
      signCourseName: apiData.productName,
      refundMode: apiData.returnType,
      opinion: apiData.auditRemark,
      oldAuditRemark: apiData.oldAuditRemark,
      sudyManagerName: apiData.sudyManagerName,
      studentCampusName: apiData.studentCampusName,
      sudyManagerContact: apiData.sudyManagerContact,
      cashierFile: apiData.cashierAuditCertPath ? { src: apiData.cashierAuditCertPath } : null,
      cashierAuditRemark: apiData.cashierAuditRemark,
      returnBankAccount: apiData.returnBankAccount,
      returnBankName: apiData.returnBankName,
      returnBankAccountOwner: apiData.returnBankAccountOwner,
      applicationRefundFile: apiData.returnApplyFilePath ? { src: apiData.returnApplyFilePath } : null,
    };
  };

  // 加载合同信息
  loadContractInfo = async (contractProductId) => {
    const apiData = await getEcsContractInfo({ contractProductId }, true);
    return {
      no: apiData.contractId,
      actualMoney: apiData.realPaidAmount,
      feeItems: [
        {
          name: 'VIP服务费',
          money: apiData.vipRealPayAmount + apiData.vipProAmount,
          discount: apiData.vipProAmount,
        },
        {
          name: apiData.courseName,
          money: apiData.realPayAmount + apiData.courseProAmount,
          discount: apiData.courseProAmount,
        },
      ],
      deductedBaseMoney: apiData.consumeAmount,
      deductedVipMoney: apiData.vipConsumeAmount,
      after2018: apiData.after2018,
      availableAmount: apiData.availableAmount,
      paidStatus: apiData.paidStatus,
    };
  };

  // 加载扣费明细
  loadFeeDetails = async (contractProductId) => {
    const apiData = await getEcsContractChargeInfo({ contractProductId }, true);
    const { chargeInfo, result } = apiData;
    const defaultQuarter = {
      base: { regular: 0, discount: 0, regularHours: 0, discountHours: 0 },
      vip: { regular: 0, discount: 0 },
    };
    let summer = { ...defaultQuarter, id: 'DAT0000000232' };
    let autumn = { ...defaultQuarter, id: 'DAT0000000233' };
    let winter = { ...defaultQuarter, id: 'DAT0000000230' };
    let spring = { ...defaultQuarter, id: 'DAT0000000231' };
    chargeInfo.forEach((f) => {
      const {
        realAmount,
        promotionQuantity,
        realQuantity,
        seasonId,
        promotionAmount,
        realVipAmount,
        proVipAmount,
        realNatureHour,
        promotionNatureHour,
      } = f;
      const fee = {
        regular: realAmount,
        discount: promotionAmount,
        regularHours: realNatureHour,
        discountHours: promotionNatureHour,
      };
      const vipFee = {
        regular: realVipAmount,
        discount: proVipAmount,
      };
      if (seasonId === summer.id) {
        summer.base = fee;
        summer.vip = vipFee;
      } else if (seasonId === autumn.id) {
        autumn.base = fee;
        autumn.vip = vipFee;
      } else if (seasonId === winter.id) {
        winter.base = fee;
        winter.vip = vipFee;
      } else if (seasonId === spring.id) {
        spring.base = fee;
        spring.vip = vipFee;
      }
    });
    return {
      isBindVip: result,
      quarterFeeDetails: [summer, autumn, winter, spring],
    };
  };

  /**
   * 计算金额
   * @param  {number} actualMoney    合同实收总额
   * @param  {object} quarterFeeDetails 各季度扣费明细
   * @return {object} 计算结果集合，字段如下：
      {
        baseRegularTotal: number 基础项目正价扣费合计
        baseDiscountTotal: number 基础项目优惠扣费合计
        baseRegularHoursTotal: number 基础项目正价课时数合计
        baseDiscountHoursTotal: number 基础项目优惠课时数合计
        vipRegularTotal: number VIP服务费项目正价扣费合计
        vipDiscountTotal: number VIP服务费项目优惠扣费合计
        baseTotal: number 基础项目扣费总合计
        baseHoursTotal: number 基础项目课时数总合计
        vipTotal: number VIP服务费项目扣费合计
        total: number 扣费总合计
        refundMoney: number 退费总额
      }
   */
  compute = (actualMoney, quarterFeeDetails) => {
    let baseRegularTotal = 0;
    let baseDiscountTotal = 0;
    let vipRegularTotal = 0;
    let vipDiscountTotal = 0;
    let baseRegularHoursTotal = 0;
    let baseDiscountHoursTotal = 0;
    quarterFeeDetails.forEach((quarterFeeDetail) => {
      const { base, vip } = quarterFeeDetail;
      baseRegularTotal += base.regular;
      baseDiscountTotal += base.discount;
      baseRegularHoursTotal += base.regularHours;
      baseDiscountHoursTotal += base.discountHours;
      vipRegularTotal += vip.regular;
      vipDiscountTotal += vip.discount;
    });
    const baseTotal = baseRegularTotal + baseDiscountTotal;
    const baseHoursTotal = baseRegularHoursTotal + baseDiscountHoursTotal;
    const vipTotal = vipRegularTotal + vipDiscountTotal;
    const total = baseTotal + vipTotal;
    let refundMoney = actualMoney - baseRegularTotal - vipRegularTotal;
    refundMoney = refundMoney < 0 ? 0 : refundMoney;
    return {
      baseRegularTotal,
      baseDiscountTotal,
      baseRegularHoursTotal,
      baseDiscountHoursTotal,
      vipRegularTotal,
      vipDiscountTotal,
      baseTotal,
      baseHoursTotal,
      vipTotal,
      total,
      refundMoney,
    };
  };

  /**
   * 计算一个季度扣费总额
   * @param  {object} quarterFeeDetail  季度扣费明细数据
   * @return {number}
   */
  computeQuarterTotal = (quarterFeeDetail) => {
    const { base, vip } = quarterFeeDetail;
    if (!base || !vip) {
      return null;
    }
    return base.regular + base.discount + vip.regular + vip.discount;
  };

  /**
   * 提交申请，校验规则为：1、退费总额大于0  2、退款方式不为空
   */
  commitApply = () => {
    const { onSave, promiseStudentId } = this.props;
    const { refundMode, applicationRefundFile, opinion, returnBankAccount, returnBankAccountOwner, returnBankName } =
      this.state;

    const error = this.checkParams();
    if (error) {
      this.setState({ commitApplyBtnError: error });
      return;
    }

    let params = {
      id: promiseStudentId,
      auditStatus: 'UNAUDIT',
      returnType: refundMode,
      auditRemark: opinion,
      courseStatus: '2',
    };

    if (refundMode === 'RETURN_TO_CUSTOMER') {
      params = Object.assign({}, params, {
        returnBankAccount,
        returnBankAccountOwner,
        returnBankName,
        returnApplyFileid: applicationRefundFile.fileId,
      });
    }

    savaPromiseReturnAuditInfo(params, true).then(
      () => {
        showMessage('操作成功', 'success');
        onSave();
      },
      (e) => {
        showMessage('操作失败');
        if (e && e.text) {
          this.setState({ commitApplyBtnError: e.text });
        }
      },
    );
  };

  checkParams = () => {
    const {
      totals: { refundMoney },
      refundMode,
      applicationRefundFile,
      contractInfo: { after2018, availableAmount, paidStatus },
    } = this.state;
    let error = '';

    if (!refundMode) {
      error = '请选择退款方式';
    } else {
      if (after2018) {
        if (refundMoney < 0) {
          if (paidStatus === 'paid') {
            error = '当前合同欠费，不能申请退款，请联系精英班集团负责人处理';
          } else {
            if (availableAmount === 'unassigned ') {
              error = '请先前往合同管理处分配完已交金额才能申请退费';
            } else {
              error = '当前合同欠费，请前往合同管理缴费后再申请退款';
            }
          }
        }
      } else {
        if (refundMoney <= 0) {
          error = '合计退费总额为0，不能申请退费';
        }
      }
    }

    if (refundMode === 'RETURN_TO_CUSTOMER') {
      // 退费申请书
      !applicationRefundFile && !error && (error = '直接退费需要上传退费申请书');
      // 校验银行账号
      const checkAccountResult = this.checkAccount();
      checkAccountResult && !error && (error = checkAccountResult);
    }

    return error;
  };

  checkAccount = () => {
    const { returnBankAccount, returnBankName, returnBankAccountOwner } = this.state;
    if (!returnBankAccount) {
      return '请输入退费账号';
    }
    if (!returnBankAccountOwner) {
      return '请输入退费账号名';
    }
    if (!returnBankName) {
      return '请输入银行支行信息';
    }

    return false;
  };

  /**
   * 审批，校验规则为：1、审批意见不为空
   * @param {number} status 审批状态  1：通过  2：不通过
   */
  approval = (status) => {
    let error;
    if (this.props.courseStatus === '2') {
      if (!this.state.opinion) {
        error = '请填写审批意见';
      }
    } else if (this.props.courseStatus === '3' && this.props.auditStatus === 'ACCOUNTANT_RAUDIT') {
      if (!this.state.cashierFile) {
        error = '请上传转账凭证';
      }
    }
    if (error) {
      return this.setState({
        cancelRefundBtnError: status === 2 ? error : null,
        approvalPassBtnError: status === 1 ? error : null,
      });
    }

    let params = {};
    if (this.props.courseStatus === '2') {
      // 退电子账户则审批完成，auditStatus传`RAUDIT`
      // 直接退款则到出纳审批，auditStatus传`ACCOUNTANT_RAUDIT`
      const isRefundAccount = this.state.refundMode === 'RETURN_TO_ACCOUNT';
      const auditStatus = status === 1 ? (isRefundAccount ? 'RAUDIT' : 'ACCOUNTANT_RAUDIT') : 'AUDIT_FAILED';

      params = {
        returnType: this.state.refundMode,
        auditRemark: this.state.opinion,
        auditStatus,
        courseStatus: status === 1 ? '3' : '1',
      };
    } else if (this.props.courseStatus === '3') {
      params = {
        auditStatus: 'RAUDIT',
        cashierAuditCertFileid: this.state.cashierFile['x:fileid'],
        cashierAuditRemark: this.state.cashierAuditRemark,
      };
    }
    params.id = this.props.promiseStudentId;

    savaPromiseReturnAuditInfo(params, true).then(
      () => {
        showMessage('操作成功', 'success');
        this.props.onSave();
      },
      (e) => {
        showMessage('操作失败');
        if (e && e.text) {
          this.setState({
            cancelRefundBtnError: status === 2 ? e.text : null,
            approvalPassBtnError: status === 1 ? e.text : null,
          });
        }
      },
    );
  };

  handleUploadComplete = (file) => {
    this.setState({ cashierFile: file });
  };

  handleDeleteComplete = (id) => {
    this.setState({ cashierFile: null });
  };

  // ==================================================================================================================
  handleRefundFileUpload = (file) => {
    this.setState({ applicationRefundFile: file });
  };

  handleRefundFileDelete = (id) => {
    this.setState({ applicationRefundFile: null });
  };

  // 只能输入数字 去空格之后判断是否为纯数字即可赋值
  // formatter={value => String(value).replace(/\s/g, '').replace(/(\d{4})(?=\d)/g, "$1 ") }
  handleAcountNumberChange = (val) => {
    const value = val.replace(/\s/g, '');
    const maxLength = 26;

    const reg = /^[0-9]*$/;
    if ((reg.test(value) || value === '') && value.length <= maxLength) {
      this.setState({
        returnBankAccount: value,
      });
    }
  };

  // parser={value => String(value).replace(/\s/g, '')}
  formatAccountNumber = (val) => {
    let value = val || '';
    value = value.replace(/\s/g, '').replace(/(\d{4})(?=\d)/g, '$1 ');
    return value;
  };

  copyAccount = () => {
    const accountNumberInput = document.querySelector(`#${accountNumberID}`);
    const disabled = accountNumberInput.disabled;
    disabled && (accountNumberInput.disabled = !disabled);

    accountNumberInput && accountNumberInput.select();
    if (document.execCommand('copy')) {
      document.execCommand('copy');
      showMessage('复制成功', 'success');
    } else {
      showMessage('当前浏览器不支持复制功能', 'error');
    }

    accountNumberInput.disabled = disabled;
  };

  render() {
    const { onClose, type } = this.props;
    const {
      loading,
      studentInfo,
      contractInfo,
      quarterFeeDetails,
      refundMode,
      historyOpinions,
      opinion,
      applicationRefundFile,
      commitApplyBtnError,
      cancelRefundBtnError,
      approvalPassBtnError,
      totals,
      isBindVip,
      error,
      returnBankAccount,
      returnBankAccountOwner,
      returnBankName,
    } = this.state;
    const {
      baseRegularTotal,
      baseDiscountTotal,
      baseRegularHoursTotal,
      baseDiscountHoursTotal,
      vipRegularTotal,
      vipDiscountTotal,
      baseTotal,
      baseHoursTotal,
      vipTotal,
      total,
      refundMoney,
    } = totals;
    const summerFee = quarterFeeDetails.find((q) => q.id === 'DAT0000000232') || {};
    const autumnFee = quarterFeeDetails.find((q) => q.id === 'DAT0000000233') || {};
    const winterFee = quarterFeeDetails.find((q) => q.id === 'DAT0000000230') || {};
    const springFee = quarterFeeDetails.find((q) => q.id === 'DAT0000000231') || {};
    const isApproval = type === 3 || type === 4;
    let btnInfo;
    if (type === 2) {
      btnInfo = '已提交申请';
    } else if (type === 4) {
      btnInfo = '已审批';
    } else if (type === 1 && !isBindVip) {
      btnInfo =
        '该精英班合同关联的vip服务费产品超过1种或未绑定VIP服务费，暂不能通过本页面申请退费，请联系分公司精英班负责人，让其对接集团精英班项目部进行退费';
    }
    return (
      <div>
        <Modal
          loading={loading}
          onCancel={onClose}
          width={1100}
          error={error}
          title={
            <span>
              <span>精英班退费详情</span>
              <span style={{ color: 'red' }}>（中途退费）</span>
            </span>
          }
          footer={
            <div>
              <div className={styles.footerTips + ' ' + styles.required}>
                {isApproval
                  ? '审批通过后对该学生所有已报未完成小班进行退班；审核不通过则该学生可恢复上课'
                  : '如对退费金额有异议，请前往财务处申请课程冲销'}
              </div>
              {(type === 2 || type === 4 || type === 42) && (
                <Button size="large" onClick={onClose}>
                  关闭
                </Button>
              )}
              {type === 1 && (
                <ButtonWithPop
                  size="large"
                  type="primary"
                  onClick={this.commitApply}
                  disabled={loading || type === 2 || (type === 1 && !isBindVip)}
                  info={btnInfo}
                  error={commitApplyBtnError}
                  onErrorClose={() => this.setState({ commitApplyBtnError: null })}
                >
                  提交申请
                </ButtonWithPop>
              )}
              {type === 3 && (
                <ButtonWithPop
                  size="large"
                  type="danger"
                  onClick={() => this.approval(2)}
                  disabled={loading || type === 4}
                  info={btnInfo}
                  error={cancelRefundBtnError}
                  onErrorClose={() => this.setState({ cancelRefundBtnError: null })}
                >
                  审批不通过
                </ButtonWithPop>
              )}
              {(type === 3 || type === 41) && (
                <ButtonWithPop
                  size="large"
                  type="primary"
                  onClick={() => this.approval(1)}
                  disabled={loading || type === 4}
                  info={btnInfo}
                  error={approvalPassBtnError}
                  onErrorClose={() => this.setState({ approvalPassBtnError: null })}
                >
                  审批通过
                </ButtonWithPop>
              )}
            </div>
          }
        >
          <EcsProcess process={this.process} step={this.step} />
          <Block type="dark" title="学生信息" style={{ marginTop: -10 }}>
            <StudentInfo data={studentInfo} />
          </Block>
          <Block type="dark" title="合同信息">
            <ContractInfo data={contractInfo} />
          </Block>
          <Block type="dark" title="扣费明细">
            {
              <table className={styles.tableView}>
                <tbody>
                  <tr>
                    <th>扣费项目/季度</th>
                    <th>暑假</th>
                    <th>秋季</th>
                    <th>寒假</th>
                    <th>春季</th>
                    <th className={styles.bold}>小计</th>
                  </tr>
                  <tr>
                    <td className={styles.bold}>已扣款金额</td>
                    <td>
                      <FeeCell data={summerFee.base} />
                    </td>
                    <td>
                      <FeeCell data={autumnFee.base} />
                    </td>
                    <td>
                      <FeeCell data={winterFee.base} />
                    </td>
                    <td>
                      <FeeCell data={springFee.base} />
                    </td>
                    <td>{`${formatMoney(baseTotal)}（${baseHoursTotal}小时）`}</td>
                  </tr>
                  <tr>
                    <td className={styles.bold}>VIP服务费</td>
                    <td>
                      <FeeCell data={summerFee.vip} />
                    </td>
                    <td>
                      <FeeCell data={autumnFee.vip} />
                    </td>
                    <td>
                      <FeeCell data={winterFee.vip} />
                    </td>
                    <td>
                      <FeeCell data={springFee.vip} />
                    </td>
                    <td>{formatMoney(vipTotal)}</td>
                  </tr>
                  <tr>
                    <td className={styles.bold}>合计</td>
                    <td>{formatMoney(this.computeQuarterTotal(summerFee))}</td>
                    <td>{formatMoney(this.computeQuarterTotal(autumnFee))}</td>
                    <td>{formatMoney(this.computeQuarterTotal(winterFee))}</td>
                    <td>{formatMoney(this.computeQuarterTotal(springFee))}</td>
                    <td className={styles.bold}>{formatMoney(total)}</td>
                  </tr>
                </tbody>
              </table>
            }
            <div className={styles.summary}>
              <table>
                <tbody>
                  <tr>
                    <td>合同实收金额：</td>
                    <td>{formatMoney(contractInfo.actualMoney)}</td>
                  </tr>
                  <tr>
                    <td>
                      <span>合计扣取</span>
                      <span className={styles.highLight}>正价</span>
                      <span>小时费总额：</span>
                    </td>
                    <td>{'-' + formatMoney(baseRegularTotal)}</td>
                  </tr>
                  {/*<tr>
                  <td><span>合计扣取</span><span className={styles.highLight}>优惠</span><span>课时费总额：</span></td>
                  <td>{'-' + formatMoney(baseDiscountTotal)}</td>
                </tr>*/}
                  <tr>
                    <td>
                      <span>合计扣取</span>
                      <span className={styles.highLight}>正价</span>
                      <span>VIP服务费总额：</span>
                    </td>
                    <td>{'-' + formatMoney(vipRegularTotal)}</td>
                  </tr>
                  {/*<tr>
                  <td><span>合计扣取</span><span className={styles.highLight}>优惠</span><span>VIP服务费总额：</span></td>
                  <td>{'-' + formatMoney(vipDiscountTotal)}</td>
                </tr>*/}
                  <tr>
                    <td className={styles.bold + ' ' + styles.highLight2}>合计退费总额：</td>
                    <td className={styles.bold + ' ' + styles.highLight2}>{formatMoney(refundMoney)}</td>
                  </tr>
                </tbody>
              </table>
            </div>
          </Block>
          <Block type="dark" title={<span className={type === 1 && styles.required}>退费方式</span>}>
            <RadioGroup onChange={(e) => this.setState({ refundMode: e.target.value })} value={refundMode}>
              <Radio
                value="RETURN_TO_ACCOUNT"
                disabled={type !== 1}
                tip="部分退费、或想把钱退到电子账户先放着的时候采用"
              >
                退到电子账户
              </Radio>
              <Radio value="RETURN_TO_CUSTOMER" disabled={type !== 1} tip="全额退费时采用">
                直接退款
              </Radio>
            </RadioGroup>
            {/* 退费申请表 */}
            {refundMode === 'RETURN_TO_CUSTOMER' && (type === 1 || (type !== 1 && applicationRefundFile)) && (
              <div className={styles.applicationRefundImg}>
                <UploadImg
                  value={applicationRefundFile}
                  text="必填"
                  uploadComplete={(val) => this.handleRefundFileUpload(val)}
                  deleteComplete={(val) => this.handleRefundFileDelete(val)}
                  size={100}
                  viewOnly={type !== 1}
                />
                <div className={styles.applicationRefundText}>退费申请书</div>
              </div>
            )}
            {refundMode === 'RETURN_TO_CUSTOMER' && (type === 1 || (type !== 1 && returnBankAccount)) && (
              <Form className={styles.promiseFeeForm}>
                <Row>
                  <Col span={12}>
                    <FormItem name="退费账号" labelWidth={80} required>
                      <Input
                        id={accountNumberID}
                        style={{
                          display: 'inline-block',
                          width: 250,
                          marginRight: 30,
                        }}
                        placeholder="退费账号"
                        value={this.formatAccountNumber(returnBankAccount)}
                        onChange={(e) => this.handleAcountNumberChange(e.target.value)}
                        disabled={type !== 1}
                      />
                      <Button type="primary" size="small" onClick={this.copyAccount}>
                        复制
                      </Button>
                    </FormItem>
                  </Col>
                  <Col span={12}>
                    <FormItem name="退费账号名" labelWidth={100} required>
                      <Input
                        placeholder="退费账号名"
                        value={returnBankAccountOwner}
                        onChange={(e) =>
                          this.setState({
                            returnBankAccountOwner: e.target.value,
                          })
                        }
                        onBlur={() =>
                          this.setState({
                            returnBankAccountOwner: returnBankAccountOwner.trim(),
                          })
                        }
                        disabled={type !== 1}
                      />
                    </FormItem>
                  </Col>
                </Row>
                <Row>
                  <Col span={10}>
                    <FormItem name="银行支行信息" labelWidth={100} required>
                      <Input
                        placeholder="银行支行信息"
                        value={returnBankName}
                        onChange={(e) => this.setState({ returnBankName: e.target.value })}
                        onBlur={() =>
                          this.setState({
                            returnBankName: returnBankName.trim(),
                          })
                        }
                        disabled={type !== 1}
                      />
                    </FormItem>
                  </Col>
                </Row>
              </Form>
            )}
          </Block>
          {historyOpinions.length > 0 && (
            <Block type="dark" title="历史审批意见">
              {historyOpinions.map((his, i) => (
                <div key={i} className={styles.history}>
                  {his.text}
                </div>
              ))}
            </Block>
          )}
          {(this.step > 1 || type === 3) && (
            <Block type="dark" title={<span className={type === 3 && styles.required}>会计审批意见</span>}>
              <TextArea
                rows={3}
                value={opinion}
                onChange={(e) => this.setState({ opinion: e.target.value })}
                disabled={type !== 3}
              />
            </Block>
          )}
          {(this.step > 2 || type === 41) &&
            refundMode !== 'RETURN_TO_ACCOUNT' && [
              <Block
                key={1}
                type="dark"
                title={<span className={this.step === 2 && styles.required}>出纳上传凭证</span>}
              >
                <div className="hom-container">
                  <div className="clearfix">
                    <div className={styles_['hom-upload-block']}>
                      <UploadImg
                        value={this.state.cashierFile}
                        text="必填"
                        uploadComplete={(val) => this.handleUploadComplete(val)}
                        deleteComplete={(val) => this.handleDeleteComplete(val)}
                        size={160}
                        viewOnly={this.props.type !== 41}
                      />
                      <div className={styles_['hom-upload-block-text']}>转账凭证</div>
                    </div>
                  </div>
                </div>
              </Block>,
              <Block key={2} type="dark" title="出纳审批意见">
                <TextArea
                  rows={3}
                  value={this.state.cashierAuditRemark}
                  onChange={(e) => this.setState({ cashierAuditRemark: e.target.value })}
                  disabled={this.props.type !== 41}
                />
              </Block>,
            ]}
        </Modal>
      </div>
    );
  }
}

export default PromiseFeeDetail;
