/**
 * 精英班中途退费模态框
 * @property {number} type 操作类型 1: 退费申请  2、查看退费申请  3、退费审批  4、查看退费审批
 * @property {string} promiseStudentId 精英班学生ID
 * @property {string} contractProductId 合同产品ID
 * @property {function} onClose 模态框关闭事件触发时的回调
 * @property {function?} onSave 模态框保存成功后的回调
 */
import React from 'react';
import { Row, Col, Form, Tooltip, AutoComplete } from 'antd';
import { debounce } from 'lodash';
import Radio from 'COMPONENT/Radio';
import Button from 'COMPONENT/Button';
import Modal from 'COMPONENT/Modal';
import Block from 'COMPONENT/Block';
import IconButton from 'COMPONENT/IconButton';
import Popover from 'COMPONENT/Popover';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import StudentInfo from './StudentInfo';
import ContractInfo from './ContractInfo';
import ApprovalInfo from '../common/ApprovalInfo';
import ApprovalSelect from '../common/ApprovalSelect';
import FeeItem from './FeeItem';
import { getDictTextByVal, loadCommonDict } from 'UTIL/dict';
import { formatMoney } from './common';
import { ecsProcessStep } from '../PromiseResultDetail/utils';
import UploadImg from '../UploadImg';
import NP from 'number-precision';
import {
  getEcsContractInfo,
  getEcsContractChargeInfo,
  savaPromiseReturnAuditInfo,
  findStudentInfoByPromiseStudentId,
  getOaPromiseReturnAuditUserList,
  getDynamicVoByPromiseStudentId,
} from 'SERVICE/course';
import { getBankCardInfo } from 'SERVICE/common';
import { searchBankBranchLoader } from 'PAGE/common/loaders';
import styles from './styles.less';
import styles_ from '../PromiseResultDetail/styles.less';
import BankTooltip from 'BLOC/BankTooltip';

const RadioGroup = Radio.Group;
const Option = AutoComplete.Option;
const accountNumberID = 'ACCOUNT_NUMBER_ID';

/**
 * 扣费明细表中，一个季度的一个扣费项目对应的单元格所显示的内容
 * @property {object} data 明细数据，格式如下：
   {
     regular: number 正价金额
     discount: number 优惠金额
     regularHours: number 正价课时数
     discountHours: number 优惠课时数
    }
 */
const FeeCell = ({ data }) => {
  const { regular, discount, regularHours, discountHours } = data || {};
  return (
    <div>
      <FeeItem name="正价扣费" money={regular} />
      {regularHours !== undefined && <div>{`（${regularHours}小时）`}</div>}
      <FeeItem type="discount" name="优惠扣费" money={discount} />
      {discountHours !== undefined && <div>{`（${discountHours}小时）`}</div>}
    </div>
  );
};

/**
 * 在antd的Button组件基础上添加气泡信息提示
 * @property {any} ...props antd的Button组件所有属性
 * @property {string?} info 鼠标悬停时的提示信息
 * @property {string?} error 错误信息
 * @property {string?} onErrorClose 错误信息关闭事件触发时的回调
 */
const ButtonWithPop = ({ info, error, onErrorClose, ...props }) => {
  let btn = <Button {...props} />;
  if (info) {
    btn = (
      <Popover icon="info" content={info} maxWidth={300}>
        {btn}
      </Popover>
    );
  } else if (error) {
    btn = (
      <Popover icon="error" content={error} maxWidth={300} trigger="click" visible={true} onClose={onErrorClose}>
        {btn}
      </Popover>
    );
  }
  btn = <span style={{ marginLeft: 8 }}>{btn}</span>;
  return btn;
};

class PromiseFeeDetail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      chargeStrategy: '0', // 先实收后优惠
      studentInfo: {}, // {object} 学生信息
      contractInfo: {}, // {object} 合同信息
      quarterFeeDetails: [], // {Array} 各季度扣费明细
      refundMode: null, // {string} 退费方式 [ RETURN_TO_ACCOUNT：退到电子账户 | RETURN_TO_CUSTOMER：直接退款 ]
      historyOpinions: [], // {Array} 历史审批意见
      opinion: '', // {string} 审批意见
      commitApplyBtnError: null, // {string} 提交申请按钮的错误信息
      cancelRefundBtnError: null, // {string} 撤销退费按钮的错误信息
      approvalPassBtnError: null, // {string} 审批通过按钮的错误信息
      totals: {}, // {object} 根据原始数据计算得到的各种合计数据
      statusDict: null, // {[object]} 字典 学生精英班状态
      isBindVip: null, // {boolean} 是否绑定VIP服务费
      error: null, // {string} 数据加载失败的错误信息
      cashierFile: null,
      cashierAuditRemark: '',
      applicationRefundFile: null,
      returnBankAccount: null, // 银行账号
      returnBankAccountOwner: null, // 银行账号名称
      returnBankName: '', // 银行名称
      returnApplyFileid: null, // 退费申请书文件id,
      approvalSelectVisible: false,
      selectedApproval: '',
      approvalUsers: [],
      approvalDynamic: [],
      nodeId: null,
      branchCode: '', // 支行联行号
      branchBankList: [],
    };

    this.step = ecsProcessStep(props.courseStatus, props.auditStatus);
    this.getBankInfo = this.getBankInfo.bind(this);
    this.onBranchBankSelect = this.onBranchBankSelect.bind(this);
    this.searchBankInfo = debounce(this.searchBankInfo, 300).bind(this);
    this.checkBankSelect = this.checkBankSelect.bind(this);
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = (chargeStrategy, needSetChargeStrategy = true) => {
    // this.setState({ loading: true })

    // 非提交页面加载审批动态
    this.props.type !== 1 && this.loadApprovalDynamic(this.props.promiseStudentId);

    this.loadStudentInfo(this.props.promiseStudentId).then((studentInfo) => {
      // 加载数据信息
      Promise.all([
        // this.loadStudentInfo(this.props.promiseStudentId),
        this.loadContractInfo(this.props.contractProductId),
        this.loadFeeDetails(
          this.props.contractProductId,
          chargeStrategy == null ? studentInfo.chargeStrategy || 0 : chargeStrategy,
        ),
        loadCommonDict('StudentPromiseStatus'),
      ]).then(
        ([/*studentInfo, */ contractInfo, feeDetails, statusDict]) => {
          const { isBindVip, quarterFeeDetails, realChargeTotalAmount } = feeDetails;
          const { refundMode, opinion, oldAuditRemark, promiseStudentStatus } = studentInfo;
          let branchBankList = [];
          if (studentInfo.returnBankName && studentInfo.branchCode) {
            branchBankList = [
              {
                label: studentInfo.returnBankName,
                branchCode: studentInfo.branchCode,
              },
            ];
          }
          const newState = {
            loading: false,
            studentInfo: {
              ...studentInfo,
              promiseStudentStatusName: getDictTextByVal(statusDict, promiseStudentStatus),
            },
            contractInfo,
            quarterFeeDetails,
            refundMode: this.props.type === 1 ? null : refundMode,
            historyOpinions: oldAuditRemark ? [{ text: oldAuditRemark }] : [],
            opinion: this.props.type === 3 ? '' : opinion,
            totals: this.compute(contractInfo.actualMoney, quarterFeeDetails, realChargeTotalAmount),
            statusDict,
            isBindVip,
            cashierFile: studentInfo.cashierFile,
            cashierAuditRemark: studentInfo.cashierAuditRemark,
            returnBankAccount: studentInfo.returnBankAccount,
            returnBankName: studentInfo.returnBankName,
            branchCode: studentInfo.branchCode,
            branchBankList,
            returnBankAccountOwner: studentInfo.returnBankAccountOwner,
            applicationRefundFile: studentInfo.applicationRefundFile,
            isNeedShowChargeStrategy:
              parseInt(studentInfo.productVersionName || '0') >= 2024 &&
              studentInfo.ecsClassType === 'ESC_CLASS_TYPE_NORMAL', // 2024年及之后正规
          };
          if (needSetChargeStrategy) {
            newState.chargeStrategy = `${studentInfo.chargeStrategy || 0}`;
          }
          this.setState(newState);
        },
        (e) => {
          const error = e.text || '数据加载出错';
          this.setState({ loading: false, error });
        },
      );
    });
  };

  // 加载学生信息
  loadStudentInfo = async (promiseStudentId) => {
    const apiData = await findStudentInfoByPromiseStudentId({ promiseStudentId }, true);
    return {
      name: apiData.studentName,
      cusName: apiData.customerName,
      gradeName: apiData.gradeName,
      contact: apiData.customerContact,
      ecsClassType: apiData.ecsClassType,
      chargeStrategy: apiData.chargeStrategy,
      promiseStudentStatus: apiData.courseStatus,
      promiseClassName: apiData.promiseClassName,
      productVersionName: apiData.productVersionName,
      signCourseName: apiData.productName,
      refundMode: apiData.returnType,
      opinion: apiData.auditRemark,
      oldAuditRemark: apiData.oldAuditRemark,
      sudyManagerName: apiData.sudyManagerName,
      studentCampusName: apiData.studentCampusName,
      sudyManagerContact: apiData.sudyManagerContact,
      cashierFile: apiData.cashierAuditCertPath ? { src: apiData.cashierAuditCertPath } : null,
      cashierAuditRemark: apiData.cashierAuditRemark,
      returnBankAccount: apiData.returnBankAccount,
      returnBankName: apiData.returnBankName,
      branchCode: apiData.branchCode,
      returnBankAccountOwner: apiData.returnBankAccountOwner,
      applicationRefundFile: apiData.returnApplyFilePath ? { src: apiData.returnApplyFilePath } : null,
    };
  };

  // 加载合同信息
  loadContractInfo = async (contractProductId) => {
    const apiData = await getEcsContractInfo({ contractProductId }, true);
    return {
      no: apiData.contractId,
      actualMoney: apiData.realPaidAmount,
      feeItems: [
        {
          name: 'VIP服务费',
          money: NP.plus(apiData.vipRealPayAmount, apiData.vipProAmount),
          discount: apiData.vipProAmount,
        },
        {
          name: apiData.courseName,
          money: NP.plus(apiData.realPayAmount, apiData.courseProAmount),
          discount: apiData.courseProAmount,
        },
      ],
      deductedBaseMoney: apiData.consumeAmount,
      deductedVipMoney: apiData.vipConsumeAmount,
      after2018: apiData.after2018,
      availableAmount: apiData.availableAmount,
      paidStatus: apiData.paidStatus,
      vipPaidStatus: apiData.vipPaidStatus,
    };
  };

  // 加载扣费明细
  loadFeeDetails = async (contractProductId, chargeStrategy = this.state.chargeStrategy) => {
    const apiData = await getEcsContractChargeInfo(
      {
        contractProductId,
        chargeStrategy,
      },
      true,
    );
    const { chargeInfo, result, realChargeTotalAmount } = apiData;
    const defaultQuarter = {
      base: { regular: 0, discount: 0, regularHours: 0, discountHours: 0 },
      vip: { regular: 0, discount: 0 },
    };
    let summer = { ...defaultQuarter, id: 'DAT0000000232' };
    let autumn = { ...defaultQuarter, id: 'DAT0000000233' };
    let winter = { ...defaultQuarter, id: 'DAT0000000230' };
    let spring = { ...defaultQuarter, id: 'DAT0000000231' };
    chargeInfo.forEach((f) => {
      const {
        realAmount,
        promotionQuantity,
        realQuantity,
        seasonId,
        promotionAmount,
        realVipAmount,
        proVipAmount,
        realNatureHour,
        promotionNatureHour,
      } = f;
      const fee = {
        regular: realAmount,
        discount: promotionAmount,
        regularHours: realNatureHour,
        discountHours: promotionNatureHour,
      };
      const vipFee = {
        regular: realVipAmount,
        discount: proVipAmount,
      };
      if (seasonId === summer.id) {
        summer.base = fee;
        summer.vip = vipFee;
      } else if (seasonId === autumn.id) {
        autumn.base = fee;
        autumn.vip = vipFee;
      } else if (seasonId === winter.id) {
        winter.base = fee;
        winter.vip = vipFee;
      } else if (seasonId === spring.id) {
        spring.base = fee;
        spring.vip = vipFee;
      }
    });
    return {
      isBindVip: result,
      quarterFeeDetails: [summer, autumn, winter, spring],
      realChargeTotalAmount,
    };
  };

  // 加载审批动态
  loadApprovalDynamic = async (promiseStudentId) => {
    try {
      const data = await getDynamicVoByPromiseStudentId({ promiseStudentId }, true);
      this.setState({
        approvalDynamic: data || [],
      });
    } catch (error) {
      this.setState({
        loading: false,
      });
      showMessage(error.text);
    }
  };

  /**
   * 计算金额
   * @param  {number} actualMoney    合同实收总额
   * @param  {object} quarterFeeDetails 各季度扣费明细
   * @return {object} 计算结果集合，字段如下：
      {
        baseRegularTotal: number 基础项目正价扣费合计
        baseDiscountTotal: number 基础项目优惠扣费合计
        baseRegularHoursTotal: number 基础项目正价课时数合计
        baseDiscountHoursTotal: number 基础项目优惠课时数合计
        vipRegularTotal: number VIP服务费项目正价扣费合计
        vipDiscountTotal: number VIP服务费项目优惠扣费合计
        baseTotal: number 基础项目扣费总合计
        baseHoursTotal: number 基础项目课时数总合计
        vipTotal: number VIP服务费项目扣费合计
        total: number 扣费总合计
        refundMoney: number 退费总额
      }
   */
  compute = (actualMoney, quarterFeeDetails, realChargeTotalAmount) => {
    let baseRegularTotal = realChargeTotalAmount; // 合计扣取正价课时费总额 直接读取接口 字段
    let baseDiscountTotal = 0;
    let vipRegularTotal = 0;
    let vipDiscountTotal = 0;
    let baseRegularHoursTotal = 0;
    let baseDiscountHoursTotal = 0;
    quarterFeeDetails.forEach((quarterFeeDetail) => {
      const { base, vip } = quarterFeeDetail;
      baseDiscountTotal = NP.plus(baseDiscountTotal, base.discount);
      baseRegularHoursTotal = NP.plus(baseRegularHoursTotal, base.regularHours);
      baseDiscountHoursTotal = NP.plus(baseDiscountHoursTotal, base.discountHours);
      vipRegularTotal = NP.plus(vipRegularTotal, vip.regular);
      vipDiscountTotal = NP.plus(vipDiscountTotal, vip.discount);
    });
    const baseTotal = NP.plus(baseRegularTotal, baseDiscountTotal);
    const baseHoursTotal = NP.plus(baseRegularHoursTotal, baseDiscountHoursTotal);
    const vipTotal = NP.plus(vipRegularTotal, vipDiscountTotal);
    const total = NP.plus(baseTotal, vipTotal);
    let refundMoney = NP.minus(actualMoney, baseRegularTotal, vipRegularTotal);
    refundMoney = refundMoney < 0 ? 0 : refundMoney;
    return {
      baseRegularTotal,
      baseDiscountTotal,
      baseRegularHoursTotal,
      baseDiscountHoursTotal,
      vipRegularTotal,
      vipDiscountTotal,
      baseTotal,
      baseHoursTotal,
      vipTotal,
      total,
      refundMoney,
    };
  };

  /**
   * 计算一个季度扣费总额
   * @param  {object} quarterFeeDetail  季度扣费明细数据
   * @return {number}
   */
  computeQuarterTotal = (quarterFeeDetail) => {
    const { base, vip } = quarterFeeDetail;
    if (!base || !vip) {
      return null;
    }
    return NP.plus(base.regular, base.discount, vip.regular, vip.discount);
  };

  /**
   * 提交申请
   * 1. 查询审批人员，有多个人则进行选人审批操作
   * 2. 否者直接提交
   */
  commitApply = async () => {
    const error = this.checkParams();
    if (error) {
      this.setState({ commitApplyBtnError: error });
      return;
    }

    let params = this.getCommitApplyParams();
    await this.getAuditUserList(params);
    if (!this.state.approvalUsers.length) {
      return;
    }

    if (this.state.approvalUsers.length > 1) {
      this.setState({
        approvalSelectVisible: true,
      });
      return;
    } else {
      const user = this.state.approvalUsers[0];
      params.auditUserId = user.val;
      params.nodeId = this.state.nodeId;
      params.chargeStrategy = this.state.chargeStrategy;
      this.saveReturnAuditInfo(params);
    }
  };

  /**
   * 预申请：查询审核人
   */
  getAuditUserList = async (params) => {
    try {
      const res = await getOaPromiseReturnAuditUserList(params, true);
      const { approvalUsers = [], nodeId } = res.data;
      const users = approvalUsers.map((item) => {
        return {
          val: item.userId,
          text: item.userName,
        };
      });

      this.setState({
        approvalUsers: users,
        nodeId,
      });
    } catch (error) {
      this.setState({ commitApplyBtnError: error.text });
    }
  };

  /**
   * 提交申请
   */
  saveReturnAuditInfo = async (params) => {
    const { onSave } = this.props;

    try {
      await savaPromiseReturnAuditInfo(params, true);
      showMessage('操作成功', 'success');
      onSave();
    } catch (error) {
      showMessage('操作失败');
      if (error && error.text) {
        this.setState({ commitApplyBtnError: error.text });
      }
    }
  };

  /**
   * 获取提交参数
   * workflowType （流程类型 0：boss流程 1:OA类型）
   */
  getCommitApplyParams = () => {
    const { promiseStudentId } = this.props;
    const {
      refundMode,
      applicationRefundFile,
      opinion,
      returnBankAccount,
      returnBankAccountOwner,
      returnBankName,
      branchCode,
    } = this.state;

    let params = {
      id: promiseStudentId,
      auditStatus: 'UNAUDIT',
      returnType: refundMode,
      auditRemark: opinion,
      courseStatus: '2',
      workflowType: 1,
    };

    if (refundMode === 'RETURN_TO_CUSTOMER') {
      const { fileKey, mimeType, fileName, fileId } = applicationRefundFile || {};

      params = Object.assign({}, params, {
        returnBankAccount,
        returnBankAccountOwner,
        returnBankName,
        branchCode,
        returnApplyFileid: fileId,
        uploadFileRecordList: [
          {
            filePath: fileKey,
            mimeType,
            realFileName: fileName,
          },
        ],
      });
    }

    return params;
  };

  // * 提交申请，校验规则为：1、退费总额大于0  2、退款方式不为空
  checkParams = () => {
    const {
      totals: { refundMoney },
      refundMode,
      // applicationRefundFile, // 退费申请书
      contractInfo: { after2018, availableAmount, paidStatus },
    } = this.state;
    let error = '';

    if (!refundMode) {
      error = '请选择退款方式';
    } else {
      if (after2018) {
        if (refundMoney < 0) {
          if (paidStatus === 'paid') {
            error = '当前合同欠费，不能申请退款，请联系精英班集团负责人处理';
          } else {
            if (availableAmount === 'unassigned ') {
              error = '请先前往合同管理处分配完已交金额才能申请退费';
            } else {
              error = '当前合同欠费，请前往合同管理缴费后再申请退款';
            }
          }
        }
      } else {
        if (refundMoney <= 0) {
          error = '合计退费总额为0，不能申请退费';
        }
      }
    }

    if (refundMode === 'RETURN_TO_CUSTOMER') {
      // 20240402版本 【精英班学生管理（班主任）】中途退费，直接退款给家长优化：退费申请书/银行账号信息不用必填
      // 退费申请书
      // !applicationRefundFile && !error && (error = '直接退费需要上传退费申请书');
      // 校验银行账号
      // const checkAccountResult = this.checkAccount();
      // checkAccountResult && !error && (error = checkAccountResult);
    }

    return error;
  };

  checkAccount = () => {
    const { returnBankAccount, branchCode, returnBankAccountOwner } = this.state;
    if (!returnBankAccount) {
      return '请输入退费账号';
    }
    if (!returnBankAccountOwner) {
      return '请输入退费账号名';
    }
    if (!branchCode) {
      return '请输入搜索选择银行卡支行信息';
    }

    return false;
  };

  handleUploadComplete = (file) => {
    this.setState({ cashierFile: file });
  };

  handleDeleteComplete = (id) => {
    this.setState({ cashierFile: null });
  };

  handleRefundFileUpload = (file) => {
    this.setState({
      applicationRefundFile: file,
    });
  };

  handleRefundFileDelete = (id) => {
    this.setState({ applicationRefundFile: null });
  };

  // 只能输入数字 去空格之后判断是否为纯数字即可赋值
  // formatter={value => String(value).replace(/\s/g, '').replace(/(\d{4})(?=\d)/g, "$1 ") }
  handleAcountNumberChange = (val) => {
    const value = val.replace(/\s/g, '');
    const maxLength = 26;

    const reg = /^[0-9]*$/;
    if ((reg.test(value) || value === '') && value.length <= maxLength) {
      let payload = {};
      if (!value) {
        payload = { returnBankName: '', branchCode: '', branchBankList: [] };
      }
      this.setState({
        returnBankAccount: value,
        ...payload,
      });
    }
  };

  // parser={value => String(value).replace(/\s/g, '')}
  formatAccountNumber = (val) => {
    let value = val || '';
    value = value.replace(/\s/g, '').replace(/(\d{4})(?=\d)/g, '$1 ');
    return value;
  };

  copyAccount = () => {
    const accountNumberInput = document.querySelector(`#${accountNumberID}`);
    const disabled = accountNumberInput.disabled;
    disabled && (accountNumberInput.disabled = !disabled);

    accountNumberInput && accountNumberInput.select();
    if (document.execCommand('copy')) {
      document.execCommand('copy');
      showMessage('复制成功', 'success');
    } else {
      showMessage('当前浏览器不支持复制功能', 'error');
    }

    accountNumberInput.disabled = disabled;
  };

  approvalSelectClose = () => {
    this.setState({ approvalSelectVisible: false });
  };

  approvalSelectConfirm = (val) => {
    this.setState({ selectedApproval: val });
    this.approvalSelectClose();

    let params = this.getCommitApplyParams();
    // 获取审批人id
    params.auditUserId = val;
    params.nodeId = this.state.nodeId;
    params.chargeStrategy = this.state.chargeStrategy; // chargeStrategy 扣费策略。0-先实收后优惠，1-均摊。默认0
    this.saveReturnAuditInfo(params);
  };

  async getBankInfo() {
    const { returnBankAccount } = this.state;
    if (returnBankAccount) {
      let bankCardNo = returnBankAccount.trim().replace(/\s/g, '');
      try {
        const { resultCode, data } = await getBankCardInfo({
          bankCardNo,
          isFromAts: true /** 获取ATS数据源 */,
        });
        if (resultCode === 0) {
          const { openBranchCode, accountOpeningBranch } = data;
          let payload = {};
          if (openBranchCode && accountOpeningBranch) {
            payload = {
              returnBankName: accountOpeningBranch,
              branchCode: openBranchCode,
              branchBankList: [{ label: accountOpeningBranch, value: openBranchCode }],
            };
          }
          this.setState(payload);
        }
      } catch (e) {
        showMessage(e.resultMessage || '查询支行信息失败');
      }
    }
  }

  async searchBankInfo(query) {
    if (query.length >= 2) {
      try {
        const branchBankList = await searchBankBranchLoader(query);
        this.setState({ branchBankList });
      } catch (e) {
        showMessage(e, 'error');
      }
    }
  }

  onBranchBankSelect(label) {
    this.matchBank(label);
  }

  matchBank(text) {
    const { branchBankList } = this.state;
    let item;
    for (let i = 0, ii = branchBankList.length; i < ii; i++) {
      if (branchBankList[i].label === text) {
        item = branchBankList[i];
        break;
      }
    }
    this.setState({ branchCode: item ? item.value : '' });
  }

  checkBankSelect(e) {
    e.persist();
    let text = e.target.value.trim(/\s+/g, '');
    if (text) {
      this.matchBank(text);
    }
  }

  onChangeChargeStrategy = (e) => {
    const chargeStrategy = e.target.value;
    this.setState({ chargeStrategy });
    this.initContent(chargeStrategy, false);
  };

  generateBlockTitle() {
    const wrapperStyle = { position: 'relative' };
    const contentStyle = { top: '-4px', left: '90px', fontSize: '14px', position: 'absolute' };
    const titleStyle = { top: '2px', position: 'relative', marginRight: '4px' };
    const radioDisabled = ((this.state.contractInfo || {}).vipPaidStatus || '').toLowerCase() !== 'paid'; // 判断该合同VIP服务费是否付款完成，只有付款完成才可正常勾选
    const { type } = this.props;
    const isApply = type === 1;
    const title = (
      <div>
        <div style={{ marginBottom: '20px' }}>
          VIP服务费先实收后优惠扣费规则：
          <br />
          1）按先扣实收后扣优惠规则补扣已上季度服务费
          <br />
          2）剩余实收服务费退费
          <br />
          3）剩余优惠服务费划归收入
          <br />
        </div>
        <div>
          VIP服务费实收优惠均摊扣费规则：
          <br />
          1）按实收优惠均摊规则补扣已上季度服务费
          <br />
          2）剩余实收服务费退费
          <br />
          3）剩余优惠服务费划归收入
          <br />
        </div>
      </div>
    );

    return (
      <div style={wrapperStyle}>
        扣费明细
        {this.state.isNeedShowChargeStrategy && (
          <div style={contentStyle} className={styles.coursePromiseClassManageDescTipWrapper}>
            <Tooltip placement="top" title={title} overlayClassName={styles.tooltipWrapper}>
              <span className={styles.coursePromiseClassManageDescTip}>
                <IconButton type="question_circle" style={titleStyle} />
                VIP服务费扣费规则：
              </span>
            </Tooltip>
            <RadioGroup onChange={this.onChangeChargeStrategy} value={this.state.chargeStrategy}>
              <Radio value="0" disabled={!isApply} style={{ marginLeft: '10px' }}>
                先实收后优惠
              </Radio>
              <Radio value="1" disabled={!isApply || radioDisabled}>
                实收优惠均摊{isApply && radioDisabled && '（VIP服务未付完全款，优惠不生效)'}
              </Radio>
            </RadioGroup>
          </div>
        )}
      </div>
    );
  }

  render() {
    const { onClose, type } = this.props;
    const {
      loading,
      studentInfo,
      contractInfo,
      quarterFeeDetails,
      refundMode,
      applicationRefundFile,
      commitApplyBtnError,
      totals,
      isBindVip,
      error,
      returnBankAccount,
      branchBankList,
      returnBankAccountOwner,
      returnBankName,
      approvalDynamic,
      approvalSelectVisible,
      approvalUsers,
    } = this.state;
    const dataSource = branchBankList.map((item) => (
      <Option value={item.label} key={item.value}>
        {item.label}
      </Option>
    ));
    const { baseRegularTotal, vipRegularTotal, baseTotal, baseHoursTotal, vipTotal, total, refundMoney } = totals;
    const summerFee = quarterFeeDetails.find((q) => q.id === 'DAT0000000232') || {};
    const autumnFee = quarterFeeDetails.find((q) => q.id === 'DAT0000000233') || {};
    const winterFee = quarterFeeDetails.find((q) => q.id === 'DAT0000000230') || {};
    const springFee = quarterFeeDetails.find((q) => q.id === 'DAT0000000231') || {};
    const isApproval = type === 3 || type === 4;
    const isApply = type === 1;
    let btnInfo;

    if (type === 2) {
      btnInfo = '已提交申请';
    } else if (type === 4) {
      btnInfo = '已审批';
    } else if (type === 1 && !isBindVip) {
      btnInfo =
        '该精英班合同关联的vip服务费产品超过1种或未绑定VIP服务费，暂不能通过本页面申请退费，请联系分公司精英班负责人，让其对接集团精英班项目部进行退费';
    }

    return (
      <div>
        <Modal
          width={1100}
          loading={loading}
          onCancel={onClose}
          error={error}
          className={'promiseFeeDetailOA'}
          title={
            <span>
              <span>精英班退费详情</span>
              <span style={{ color: 'red' }}>（中途退费）</span>
            </span>
          }
          footer={
            <div>
              <div className={styles.footerTips + ' ' + styles.required}>
                {isApproval
                  ? '审批通过后对该学生所有已报未完成小班进行退班；审核不通过则该学生可恢复上课'
                  : '如对退费金额有异议，请前往财务处申请课程冲销'}
              </div>
              {type !== 1 && (
                <Button size="large" onClick={onClose}>
                  关闭
                </Button>
              )}
              {type === 1 && (
                <ButtonWithPop
                  size="large"
                  type="primary"
                  onClick={this.commitApply}
                  disabled={loading || type === 2 || (type === 1 && !isBindVip)}
                  info={btnInfo}
                  error={commitApplyBtnError}
                  onErrorClose={() => this.setState({ commitApplyBtnError: null })}
                >
                  提交申请
                </ButtonWithPop>
              )}
            </div>
          }
        >
          <Block type="dark" title="学生信息" style={{ marginTop: -10 }}>
            <StudentInfo data={studentInfo} />
          </Block>
          <Block type="dark" title="合同信息">
            <ContractInfo data={contractInfo} />
          </Block>
          <Block type="dark" title={this.generateBlockTitle()}>
            {
              <table className={styles.tableView}>
                <tbody>
                  <tr>
                    <th>扣费项目/季度</th>
                    <th>暑假</th>
                    <th>秋季</th>
                    <th>寒假</th>
                    <th>春季</th>
                    <th className={styles.bold}>小计</th>
                  </tr>
                  <tr>
                    <td className={styles.bold}>已扣款金额</td>
                    <td>
                      <FeeCell data={summerFee.base} />
                    </td>
                    <td>
                      <FeeCell data={autumnFee.base} />
                    </td>
                    <td>
                      <FeeCell data={winterFee.base} />
                    </td>
                    <td>
                      <FeeCell data={springFee.base} />
                    </td>
                    <td>{`${formatMoney(baseTotal)}（${baseHoursTotal}小时）`}</td>
                  </tr>
                  <tr>
                    <td className={styles.bold}>VIP服务费</td>
                    <td>
                      <FeeCell data={summerFee.vip} />
                    </td>
                    <td>
                      <FeeCell data={autumnFee.vip} />
                    </td>
                    <td>
                      <FeeCell data={winterFee.vip} />
                    </td>
                    <td>
                      <FeeCell data={springFee.vip} />
                    </td>
                    <td>{formatMoney(vipTotal)}</td>
                  </tr>
                  <tr>
                    <td className={styles.bold}>合计</td>
                    <td>{formatMoney(this.computeQuarterTotal(summerFee))}</td>
                    <td>{formatMoney(this.computeQuarterTotal(autumnFee))}</td>
                    <td>{formatMoney(this.computeQuarterTotal(winterFee))}</td>
                    <td>{formatMoney(this.computeQuarterTotal(springFee))}</td>
                    <td className={styles.bold}>{formatMoney(total)}</td>
                  </tr>
                </tbody>
              </table>
            }
            <div className={styles.summary}>
              <table>
                <tbody>
                  <tr>
                    <td>合同实收金额：</td>
                    <td>{formatMoney(contractInfo.actualMoney)}</td>
                  </tr>
                  <tr>
                    <td>
                      <span>合计扣取</span>
                      <span className={styles.highLight}>正价</span>
                      <span>小时费总额：</span>
                    </td>
                    <td>{'-' + formatMoney(baseRegularTotal)}</td>
                  </tr>
                  {/*<tr>
                  <td><span>合计扣取</span><span className={styles.highLight}>优惠</span><span>课时费总额：</span></td>
                  <td>{'-' + formatMoney(baseDiscountTotal)}</td>
                </tr>*/}
                  <tr>
                    <td>
                      <span>合计扣取</span>
                      <span className={styles.highLight}>正价</span>
                      <span>VIP服务费总额：</span>
                    </td>
                    <td>{'-' + formatMoney(vipRegularTotal)}</td>
                  </tr>
                  {/*<tr>
                  <td><span>合计扣取</span><span className={styles.highLight}>优惠</span><span>VIP服务费总额：</span></td>
                  <td>{'-' + formatMoney(vipDiscountTotal)}</td>
                </tr>*/}
                  <tr>
                    <td className={styles.bold + ' ' + styles.highLight2}>合计退费总额：</td>
                    <td className={styles.bold + ' ' + styles.highLight2}>{formatMoney(refundMoney)}</td>
                  </tr>
                </tbody>
              </table>
            </div>
          </Block>
          <Block type="dark" title={<span className={isApply && styles.required}>退费方式</span>}>
            <RadioGroup onChange={(e) => this.setState({ refundMode: e.target.value })} value={refundMode}>
              <Radio value="RETURN_TO_ACCOUNT" disabled={!isApply} tip="部分退费、或想把钱退到电子账户先放着的时候采用">
                退到电子账户
              </Radio>
              <Radio value="RETURN_TO_CUSTOMER" disabled={!isApply} tip="全额退费时采用">
                直接退款
              </Radio>
            </RadioGroup>
            {/* 退费申请表 */}
            {refundMode === 'RETURN_TO_CUSTOMER' && (isApply || (!isApply && applicationRefundFile)) && (
              <div className={styles.applicationRefundImg}>
                <UploadImg
                  value={applicationRefundFile}
                  text="选填"
                  uploadComplete={(val) => this.handleRefundFileUpload(val)}
                  deleteComplete={(val) => this.handleRefundFileDelete(val)}
                  size={100}
                  viewOnly={!isApply}
                />
                <div className={styles.applicationRefundText}>退费申请书</div>
              </div>
            )}
            {refundMode === 'RETURN_TO_CUSTOMER' && (isApply || (!isApply && returnBankAccount)) && (
              <Form className={styles.promiseFeeForm}>
                <Row>
                  <Col span={14}>
                    <p className={styles.promiseFeeFormTip}>
                      退回银行卡信息由家长操作确认时填写
                      {/*请注意！！！发起人一定要输入正确的退费账号和户名，否则会导致付款失败直接退单！*/}
                    </p>
                  </Col>
                </Row>
                <Row>
                  <Col span={10}>
                    <FormItem name="退费账号名" labelWidth={100}>
                      <Input
                        placeholder="退费账号名"
                        value={returnBankAccountOwner}
                        onChange={(e) =>
                          this.setState({
                            returnBankAccountOwner: e.target.value,
                          })
                        }
                        onBlur={() =>
                          this.setState({
                            returnBankAccountOwner: returnBankAccountOwner && returnBankAccountOwner.trim(),
                          })
                        }
                        disabled={/*!isApply*/ true}
                      />
                    </FormItem>
                  </Col>
                </Row>
                <Row align="middle">
                  <Col span={10}>
                    <FormItem name="退费账号" labelWidth={100}>
                      <Input
                        id={accountNumberID}
                        placeholder="退费账号"
                        value={this.formatAccountNumber(returnBankAccount)}
                        onChange={(e) => this.handleAcountNumberChange(e.target.value)}
                        onBlur={this.getBankInfo}
                        disabled={/*!isApply*/ true}
                      />
                    </FormItem>
                  </Col>
                  <Col span={2}>
                    <Button
                      type="primary"
                      size="small"
                      onClick={this.copyAccount}
                      style={{ margin: '5px 20px' }}
                      disabled
                    >
                      复制
                    </Button>
                  </Col>
                </Row>
                <Row align="middle">
                  <Col span={10}>
                    <FormItem name="银行支行信息" labelWidth={100}>
                      <AutoComplete
                        value={returnBankName}
                        onChange={(val) => this.setState({ returnBankName: val })}
                        dataSource={dataSource}
                        onSelect={this.onBranchBankSelect}
                        onSearch={this.searchBankInfo}
                        disabled={/*!isApply*/ true}
                      >
                        <Input size="large" onBlur={this.checkBankSelect} placeholder="请输入关键字搜索支行" />
                      </AutoComplete>
                    </FormItem>
                  </Col>
                  <Col span={2} className={styles.bankTooltip}>
                    <BankTooltip />
                  </Col>
                </Row>
              </Form>
            )}
          </Block>
          {!isApply && approvalDynamic.length > 0 && (
            <Block title={<span>审批动态</span>}>
              <ApprovalInfo data={approvalDynamic} />
            </Block>
          )}
          {this.step > 2 && refundMode !== 'RETURN_TO_ACCOUNT' && (
            <Block key={1} type="dark">
              <div className="hom-container">
                <div className="clearfix">
                  <div className={styles_['hom-upload-block']}>
                    <UploadImg
                      viewOnly
                      value={this.state.cashierFile}
                      text="必填"
                      uploadComplete={(val) => this.handleUploadComplete(val)}
                      deleteComplete={(val) => this.handleDeleteComplete(val)}
                      size={160}
                    />
                    <div className={styles_['hom-upload-block-text']}>转账凭证</div>
                  </div>
                </div>
              </div>
            </Block>
          )}
        </Modal>
        {/* 选择审批人 */}
        <ApprovalSelect
          visible={approvalSelectVisible}
          dict={approvalUsers}
          onClose={this.approvalSelectClose}
          onConfirm={this.approvalSelectConfirm}
        />
      </div>
    );
  }
}

export default PromiseFeeDetail;
