import React from 'react';
import styles from '../index.less';
import PropTypes from 'prop-types';
import { Table } from 'antd';
import Modal from 'COMPONENT/Modal';
import { formatCourseToHour } from 'UTIL/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { updateMiniClassCourseByCourseIdsApi, getMiniClassCourseDetailApi } from '../loaders';
import { teachingTypeEnum } from 'CONST/dicts/course';
import moment from 'moment';

class SinglePanel extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      loadingTip: null,
      dataSource: [],
      selectData: [],
      rowSelection: {
        onChange: (selectedRowKeys, selectedRows) => {
          this.setState({
            selectData: selectedRows,
          });
        },
        getCheckboxProps: (record) => {
          const disabled = this.isDisabledRow(record);
          return {
            disabled: record.courseStatus !== 'NEW' || disabled,
            defaultChecked: record.courseStatus === 'NEW' && !disabled,
          };
        },
      },
    };

    this.columns = [
      {
        title: '课程序号',
        dataIndex: 'courseNum',
        render: (data) => {
          return '第' + data + '讲';
        },
      },
      {
        title: '上课日期',
        dataIndex: 'courseDate',
      },
      {
        title: '上课时间',
        dataIndex: 'courseTime',
      },
      {
        title: '小时',
        dataIndex: 'courseHours',
        render: (text, row) => {
          return formatCourseToHour(text, row.courseMinutes);
        },
      },
      {
        title: '教室',
        dataIndex: 'classroom',
      },
      {
        title: '老师',
        dataIndex: 'teacherName',
      },
      {
        title: '班主任',
        dataIndex: 'studyManegerName',
      },
      {
        title: '科目',
        dataIndex: 'subject',
      },
      {
        title: '状态',
        dataIndex: 'courseStatusName',
      },
    ];
  }

  static PropTypes = {
    subjectName: PropTypes.string,
    classTime: PropTypes.string,
    teacherName: PropTypes.string,
    classroomName: PropTypes.string,
    studyManagerName: PropTypes.string,
  };

  isDisabledRow = (row) => {
    const isBefore = moment(`${row.courseDate} ${row.courseTime}`).isBefore(new Date());
    const isDisabledType = row.teachingType === teachingTypeEnum.LIVE || row.teachingType === teachingTypeEnum.RECORDED;

    return isBefore && isDisabledType;
  };

  // 获取表格数据
  getDataFunc = async () => {
    const { miniClassId } = this.props;
    const data = await getMiniClassCourseDetailApi({ miniClassId: miniClassId });
    this.setState({
      dataSource: data.rows,
      selectData: data.rows.filter((value) => {
        return value.courseStatus === 'NEW' && !this.isDisabledRow(value);
      }),
    });
  };

  // 提交
  submit = async () => {
    if (this.state.selectData.length === 0) {
      return showMessage('至少选择一个课程');
    }
    const { teacherId, subjectId, classTime, classroomId, studyManagerId, courseHours } = this.props;
    let params = {};
    if (teacherId) {
      params = { ...params, teacherId: teacherId };
    }
    if (subjectId) {
      params = { ...params, subjectId: subjectId };
    }
    if (classTime) {
      params = { ...params, courseTime: classTime };
    }
    if (classroomId) {
      params = { ...params, classRoomId: classroomId };
    }
    if (studyManagerId) {
      params = { ...params, studyManagerId: studyManagerId };
    }
    let tempData = this.state.selectData.map((value) => {
      return {
        ...params,
        courseHours,
        courseDate: value.courseDate,
        courseIds: [value.miniClassCourseId],
      };
    });
    try {
      const data = await updateMiniClassCourseByCourseIdsApi({
        miniClassCourseUpdateDtos: tempData || [],
      });
      if (data.data) {
        showMessage('同步成功！', 'success');
        this.props.onClose();
      } else {
        showMessage(data.resultMessage);
      }
    } catch (e) {
      showMessage(e.text);
    }
  };

  componentDidMount() {
    this.getDataFunc();
  }

  render() {
    let { loading, loadingTip, dataSource, rowSelection } = this.state;
    let {
      onClose,
      subjectId,
      subjectName,
      classTime,
      classroomId,
      teacherId,
      teacherName,
      classroomName,
      studyManagerId,
      studyManagerName,
      everyCourseClassNum,
      everyCourseClassNumMin,
    } = this.props;

    return (
      <Modal
        loading={loading}
        loadingTip={loadingTip}
        onCancel={onClose}
        okText="确认同步所选课程"
        cancelText="无需同步"
        title="修改成功"
        width={900}
        onOk={() => this.submit()}
        className={styles.syncConfirm}
      >
        <div className={styles.syncConfirmPanel}>
          <div className={styles.panelTop}>
            {everyCourseClassNum != null && (
              <p>
                <span>每讲时长：</span>
                <span>{`${everyCourseClassNum || 0}小时${everyCourseClassNumMin || 0}分钟`}</span>
              </p>
            )}
            {classroomId && (
              <p>
                <span>教室：</span>
                <span>{classroomName}</span>
              </p>
            )}
            {teacherId && (
              <p>
                <span>老师：</span>
                <span>{teacherName}</span>
              </p>
            )}
            {classTime && (
              <p>
                <span>上课时间：</span>
                <span>{classTime}</span>
              </p>
            )}
            {subjectId && (
              <p>
                <span>科目：</span>
                <span>{subjectName}</span>
              </p>
            )}
            {studyManagerId && (
              <p>
                <span>班主任：</span>
                <span>{studyManagerName}</span>
              </p>
            )}
          </div>
          <div className={styles.panelTable}>
            <div className={styles.panelHeader}>
              <p>是否需要将本次修改同步至班内所有未上课课程？</p>
            </div>
            <Table
              rowKey="miniClassCourseId"
              rowSelection={rowSelection}
              dataSource={dataSource}
              columns={this.columns}
            />
          </div>
          <div className={styles.panelFooter}>
            <p>1. 点击确认同步后，班内所有未上课课程将同步本次修改；</p>
            <p>2. 支持取消勾选本次不同步的课程。</p>
            {/* <p>3. 课程上课方式为“在线A”或“在线B”的课程，开课后将不会同步老师信息的修改。</p> */}
          </div>
        </div>
      </Modal>
    );
  }
}

export default SinglePanel;
