import React from 'react';
import './curr.less';
import Icon from 'COMPONENT/Icon';
import Popover from 'COMPONENT/Popover';
import QrcodeFileUpload from 'COMPONENT/QrcodeFileUpload';
import Upload from 'COMPONENT/Upload';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Modal as AntdModal } from 'antd';
import PreviewImgs from 'COMPONENT/PreviewImgs';
import Loading from 'COMPONENT/Loading';
import PropTypes from 'prop-types';

const validateFileTypes = ['png', 'jpg', 'jpeg', 'bmp', 'gif', 'webp'];

/**
 * @todo 1. 图片加载错误，拦截预览 一般情况下 不会出现类似问题
 * @todo 2. render props 自定义图片显示
 */

class UploadImg extends React.Component {
  static PropTypes = {
    max: PropTypes.number,
    hasDelete: PropTypes.boolean,
  };

  static defaultProps = {
    max: 1,
    hasDelete: true,
  };

  constructor(props) {
    super(props);
    this.state = {
      fileList: [].concat(props.value),
      max: props.max, // 最大上传个数
      previewImgSrc: '', // 图片预览
      isLoading: false,
    };
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.value) {
      let fileList = [].concat(nextProps.value);
      this.setState({ fileList });
    }
  }

  /**
   * 二维码传图回调
   *
   filePath:"阿里云oss"
   fileSize:"7467B"
   fileid:"UPL000000004645"
   realFileName:"mmexport1529651473907.jpg"
   type:""
   *
   * */
  get2dUploadImgInfo = (files) => {
    const { params, uploadComplete } = this.props;
    if (files.length > 0) {
      let fileList = files
        .filter((r) => r.filePath)
        .map((r) => {
          return {
            ...r,
            ...params,
            fileId: r.fileid,
            fileName: r.realFileName,
            src: r.filePath,
            size: r.fileSize,
          };
        });
      fileList = fileList.concat(this.state.fileList);
      let end = Math.min(this.props.max, fileList.length);
      // 处理最大上传数
      fileList = fileList.slice(0, end);
      this.setState({ fileList });
      uploadComplete(fileList);
    }
  };

  /**
   * 图片上传时不断调用
   * 当上传成功时，可获取到全部file数据
   * 调用父组件传递进来的callback
   * */
  onUploadChange = (file) => {
    const { params, uploadComplete } = this.props;
    let files = [].concat(file);
    let isFailed = files.find((r) => r.error === false);
    let isAllSuccess = files.every((r) => !!r.src);
    this.setState({ isLoading: true });
    if (isFailed) {
      showMessage('图片上传失败，请联系管理人员！', 'error');
      this.setState({ isLoading: false });
    } else if (isAllSuccess) {
      showMessage('图片上传成功！', 'success');
      this.setState({ isLoading: false });
      let fileList = files
        .filter((r) => r.src)
        .map((r) => {
          return {
            ...r,
            ...params,
            fileId: r['x:fileid'],
            fileName: r.name,
          };
        });
      fileList = fileList.concat(this.state.fileList);
      this.setState({ fileList });
      uploadComplete(fileList);
    }
  };

  /**
   * 图片上传前进行校验
   * 必需返回一个fileList [Array]
   * */
  checkLength = (files) => {
    if (files.length > 0) {
      // 是否超过最大限制
      const { max } = this.state;
      let newFileList = [...this.state.fileList, ...files];
      if (newFileList.length > max) {
        showMessage(`最多上传${max}张图片`, 'warning');
        return false;
      }
      // 文件格式限制
      let isAllImageType = files.every((r) => {
        let imageType = r.type.split('/')[1];
        return validateFileTypes.includes(imageType);
      });
      if (!isAllImageType) {
        showMessage('图片格式不正确！', 'warning');
        return false;
      }
      return files;
    }
  };

  /**
   * 预览
   * */
  previewImage(src) {
    this.setState({ previewImgSrc: src });
  }

  /**
   * 删除图片
   * */
  deleteImg(file) {
    const { deleteComplete } = this.props;
    const that = this;
    AntdModal.confirm({
      title: '是否删除此图片？',
      content: '',
      okType: 'danger',
      onOk() {
        that.setState((prevState) => {
          const fileList = prevState.fileList.filter((r) => r.src !== file.src);
          deleteComplete(fileList);
          return { fileList };
        });
      },
    });
  }

  renderImageList() {
    return this.state.fileList.length
      ? this.state.fileList.map((file, index) => (
          <div className="curr-upload-img-view" key={`${file.src}${index}`}>
            <div className="curr-upload-img-view-wrapper">
              <img className="curr-upload-img-view-img" src={file.src} />
            </div>

            <div className="curr-upload-img-cover">
              <div className="curr-upload-img-detail" onClick={this.previewImage.bind(this, file.src)} title="预览图片">
                <Icon type="visible_mc" color="#000000" size={{ width: 25, height: 25 }} />
              </div>
              {this.props.hasDelete && !this.props.viewOnly && (
                <div className="curr-upload-img-delete" onClick={this.deleteImg.bind(this, file)} title="删除图片">
                  <Icon type="delete_mc" color="#000000" size={{ width: 25, height: 25 }} />
                </div>
              )}
            </div>
          </div>
        ))
      : null;
  }

  renderQrCodePopOver() {
    const { disabled, viewOnly } = this.props;
    const { fileList, max } = this.state;
    if (disabled || viewOnly) return null;
    if (fileList.length === max) return null;
    return (
      <div className="curr-upload-img-qr" style={{ width: 25, height: 25 }}>
        <Popover
          placement="topRight"
          content={
            <div>
              <QrcodeFileUpload
                height={160}
                width={160}
                callback={this.get2dUploadImgInfo}
                multiple={this.props.multiple}
              />
              <div style={{ textAlign: 'center', marginTop: 10 }}>手机扫码上传图片</div>
            </div>
          }
        >
          <Icon type="2d_code_half_mc" size={{ width: 25, height: 25 }} />
        </Popover>
      </div>
    );
  }

  renderUploader() {
    const { max, disabled, viewOnly, text } = this.props;
    const { isLoading, fileList } = this.state;
    if (disabled || viewOnly) return null;
    if (fileList.length === max) return null;
    return (
      <div className="curr-upload-img-picker">
        <Upload
          multiple={this.props.multiple}
          hasExtension={true}
          onChange={this.onUploadChange}
          beforeUpload={this.checkLength}
        >
          <Loading size="default" loading={isLoading} />
          <div className="curr-upload-img-picker-btn">
            <div>
              <Icon type="plus" size={{ width: 25, height: 25 }} />
            </div>
            <div>{text}</div>
          </div>
        </Upload>
        {(disabled || viewOnly) && <div className="curr-upload-img-disabled"></div>}
      </div>
    );
  }

  render() {
    const { previewImgSrc, fileList } = this.state;

    return (
      <div className="curr-uploadImg">
        {this.renderImageList()}
        <div className="curr-upload-img">
          {this.renderQrCodePopOver()}
          {this.renderUploader()}
        </div>
        {previewImgSrc && (
          <div className="curr-upload-img-preview">
            <div className="curr-upload-img-preview-wrapper">
              <PreviewImgs list={fileList.map((r) => r.src)} activeSrc={previewImgSrc} showThumbnail={false} />
            </div>
            <div className="curr-upload-img-preview-close" onClick={this.previewImage.bind(this, '')}>
              <Icon type="cross_circle" size={32} />
            </div>
          </div>
        )}
      </div>
    );
  }
}

export default UploadImg;
