import React from 'react';
import ReactDOM from 'react-dom';
import { connect } from 'dva';
import { Button, Icon, Checkbox } from 'antd';
import Page from 'COMPONENT/Page';
import Popover from 'COMPONENT/Popover';
import SearchBar from 'COMPONENT/SearchBar';
import MultiSelect from 'COMPONENT/MultiSelect';
import Select from 'COMPONENT/Select';
import baiduMap from 'UTIL/thirdParty/baiduMap';
import { formateProvinceDict, formateCityDict } from 'UTIL/dict';
import { findOrganizationById, getCampusByRegionId, getStudentSchoolByRegionId, getRegionList } from 'SERVICE/common';
import { getStudentAddressAndLatLog } from 'SERVICE/student';
import styles from './styles.less';
import campusIcon from 'ASSET/imgs/location_blue.png';
import schoolIcon from 'ASSET/imgs/location_green.png';
import studentIcon from 'ASSET/imgs/waterdrop_red.png';

//学生类型字典
const studentTypeDict = [
  { val: 'ENROLLED', text: '正式学生' },
  { val: 'POTENTIAL', text: '潜在学生' },
];

const defaultStudentTypes = studentTypeDict.map((c) => c.val);

//默认地图等级
const defaultMapZoom = 11;

//显示麻点图或者热力图的地图等级临界点，大于等于该值显示麻点图
const mapLevelBreak = 14;

class CustomerMapAnalyze extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // boolean 页面加载状态
      mapCenter: null, // Point 地图当前的中心位置，Point类型参见百度地图API：http://lbsyun.baidu.com/cms/jsapi/reference/jsapi_reference.html#a1b0
      mapZoom: defaultMapZoom, //number 地图当前的等级
      studentList: null, // [object] 学生数据
      grades: [], // [object] 年级统计数据
      campuses: [], // [object] 学习中心数据
      schools: [], // [object] 公校数据
      userProvinceId: null, // string 当前登录用户所属省ID
      userCityId: null, // string 当前登录用户所属城市ID
      provinceId: null, // string 查询条件，省份ID
      cityId: null, // string 查询条件，城市ID
      studentTypes: defaultStudentTypes, // [string] 查询条件，学生类型
      region: null, // [object] 行政区划数据
      isCitySelectDisabled: false, // boolean 城市选框是否失效
    };
    this.map = null; // Map 地图实例，Map类型参见百度地图API：http://lbsyun.baidu.com/cms/jsapi/reference/jsapi_reference.html#a0b0
  }

  componentDidMount() {
    const { initialState } = this.props;
    this.initMap().then(() => {
      this.initContent();
    });
  }

  componentWillUnmount() {
    this.isUnmounted = true;
  }

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      if (!this.isUnmounted) {
        this.setState(newState, () => {
          resolve();
        });
      } else {
        resolve();
      }
    });
  };

  /**
   * 初始化地图
   */
  initMap = async () => {
    //加载百度地图API
    await baiduMap.init(['heatMap']);
    //创建地图实例
    const map = new BMap.Map(this._$map);
    //设置默认的中心点、缩放等级、最小缩放等级、允许滚轮
    map.centerAndZoom(new BMap.Point(113.32828752559, 23.134506560564), defaultMapZoom);
    map.setMinZoom(5);
    map.enableScrollWheelZoom(true);
    //添加地图比例尺、平移导航控件
    const anchor = new BMap.ScaleControl({ anchor: BMAP_ANCHOR_TOP_LEFT });
    const navigation = new BMap.NavigationControl();
    map.addControl(anchor);
    map.addControl(navigation);
    //监听缩放、拖拽事件
    map.addEventListener('zoomend', this.mapZoomEnd);
    map.addEventListener('moveend', this.mapMoveEnd);

    this.map = map;
    return true;
  };

  /**
   * 初始化内容
   */
  initContent = async () => {
    const {
      user: { organizationId, campusId, brenchId },
    } = this.props;
    await this.updateState({ loading: true });
    let provinceId;
    let cityId;
    if (organizationId === '000001') {
      provinceId = '440000';
      cityId = '441900';
    } else {
      const { city, province } = await findOrganizationById({ id: campusId || brenchId });
      provinceId = province.id;
      cityId = city.id;
    }
    /*provinceId = '440000'
    cityId = '441300'*/
    const region = await getRegionList();
    await this.updateState({ userProvinceId: provinceId, userCityId: cityId, provinceId, cityId, region });
    this.search();
  };

  /**
   * 查询并更新地图数据
   */
  search = async () => {
    const { provinceId, cityId, studentTypes, region } = this.state;
    await this.updateState({ loading: true });
    const [mapData, campuses, schools] = await Promise.all([
      this.loadMapData(provinceId, cityId, studentTypes && studentTypes.length === 1 ? studentTypes[0] : null),
      this.getCampuses(provinceId, cityId),
      this.getSchools(provinceId, cityId),
    ]);
    const { studentList, grades } = mapData;
    await this.updateState({ loading: false, studentList, grades, campuses, schools });
    let centerPointOrName;
    if (cityId) {
      centerPointOrName = formateCityDict(region, provinceId).find((c) => c.val === cityId).text;
    } else {
      if (provinceId === '810000') {
        centerPointOrName = '香港';
      } else if (provinceId === '820000') {
        centerPointOrName = '澳门';
      } else if (provinceId === '710000') {
        centerPointOrName = new BMap.Point(121.564002, 25.019392);
      } else {
        centerPointOrName = formateProvinceDict(region).find((c) => c.val === provinceId).text;
      }
    }
    this.refreshMap(centerPointOrName);
    return true;
  };

  /**
   * 加载地图数据，接口参见：https://api.xuebangsoft.me/index.do#/f7525741-db26-4bb4-8475-7555d19dadcc/front/interfaceDetail/806e6fd4-e636-4095-b0f0-4974ef534aa5
   * 并转换成前端使用的格式，如下：
     {
        studentList: [
          { lng: '114.413421', lat: '23.115822', name: '周幸',
            address: '广东省惠州市惠城区文昌二路10号', gradeId: '402881e647ce90330147ce94ef900004'
          },
        ],
        grades: [
          { id: 'xx1', name: '高一', count1: 1200, count2: 900, active: true },
          { id: 'xx2', name: '六年级', count1: 980, count2: 677, active: true },
          { id: 'xx3', name: '五年级', count1: 566, count2: 605, active: true },
        ]
     }
   * @param {string} provinceId 省份ID
   * @param {string} cityId 城市ID
   * @param {string?} studentType 学生类型
   */
  loadMapData = async (provinceId, cityId, studentType) => {
    const data = await getStudentAddressAndLatLog({ provinceId, cityId, studentType });
    let studentList = [];
    let grades = [];
    Object.keys(data).forEach((gradeId) => {
      const { studentAddressList, enrolledSize, potentialSize, gradeName } = data[gradeId];
      const list = studentAddressList.map((s) => {
        const { studentName, address, lat, log } = s;
        return { name: studentName, address, lat, lng: log, gradeId, gradeName };
      });
      studentList = [...studentList, ...list];
      grades = [...grades, { id: gradeId, name: gradeName, count1: enrolledSize, count2: potentialSize, active: true }];
    });
    grades.sort((a, b) => {
      return b.count1 + b.count2 - a.count1 - a.count2;
    });
    return { studentList, grades };
  };

  /**
   * 获取学习中心，接口参见https://api.xuebangsoft.me/index.do#/f7525741-db26-4bb4-8475-7555d19dadcc/front/interfaceDetail/fa55b9dd-8bcf-4e69-b816-f15709a719c1
   * 并转换为前端使用数据，格式如下：
   * [
      { id: 'xx1', name: '学习中心1', lng: 113.013183, lat: 28.192721, address: 'xx' },
    ]
   * @param  {string} provinceId 省份ID
   * @param  {string} cityId 城市ID
   */
  getCampuses = async (provinceId, cityId) => {
    const data = await getCampusByRegionId({ provinceId, cityId });
    return data.map((d) => {
      const { id, name, lon, lat, address } = d;
      return { id, name, lng: lon, lat, address };
    });
  };

  /**
   * 获取公校，接口参见https://api.xuebangsoft.me/index.do#/f7525741-db26-4bb4-8475-7555d19dadcc/front/interfaceDetail/37501d4a-cc33-4c01-b76e-6f4b6514a756
   * 并转换为前端使用数据，格式如下：
   * [
      { id: 'xx1', name: '公校1',  lng: 113.113183, lat: 28.292721, address: 'xx'  },
    ]
   * @param  {string} provinceId 省份ID
   * @param  {string} cityId 城市ID
   */
  getSchools = async (provinceId, cityId) => {
    const data = await getStudentSchoolByRegionId({ provinceId, cityId });
    return data.map((d) => {
      const { id, name, log, lat, address } = d;
      return { id, name, lng: log, lat, address };
    });
  };

  /**
   * 刷新地图，1、中心点和等级  2、覆盖物，展示学生数据的麻点图或热力图、展示学习中心和公校数据的标记
   * @param  {Point|string?} centerPointOrName 中心点对象或者地理位置名称
   */
  refreshMap = (centerPointOrName) => {
    const { mapCenter, mapZoom, studentList: studentListAll, grades, campuses, schools } = this.state;
    const activeGradeIds = grades.filter((g) => g.active).map((grade) => grade.id);
    const studentList = studentListAll.filter((s) => activeGradeIds.indexOf(s.gradeId) >= 0);
    this.map.centerAndZoom(centerPointOrName ? centerPointOrName : mapCenter, mapZoom);
    this.map.clearOverlays();
    if (mapZoom >= mapLevelBreak) {
      this.showDot(studentList);
    } else {
      this.showHeat(studentList);
    }
    campuses.forEach((campus) => {
      const { lng, lat, name, address } = campus;
      this.addMarker(lng, lat, name, address, campusIcon);
    });
    schools.forEach((school) => {
      const { lng, lat, name, address } = school;
      this.addMarker(lng, lat, name, address, schoolIcon);
    });
  };

  /**
   * 显示麻点图
   * @param {[object]} list 数据集合 [{ lng, lat, name, address }]
   */
  showDot = (list) => {
    const points = list.map((s) => {
      const { lng, lat, name, gradeName, address } = s;
      const point = new BMap.Point(lng, lat);
      point.name = name;
      point.gradeName = gradeName;
      point.address = address;
      return point;
    });
    const options = {
      size: BMAP_POINT_SIZE_SMALL,
      shape: BMAP_POINT_SHAPE_WATERDROP,
      color: '#5599F5',
    };
    const collection = new BMap.PointCollection(points, options);
    const infoWindowOpts = {
      width: 220,
    };
    collection.addEventListener('click', (e) => {
      const point = e.point;
      const { gradeName, address } = point;
      const infoWindow = new BMap.InfoWindow(gradeName + '<br />' + address, infoWindowOpts);
      this.map.openInfoWindow(infoWindow, point);
    });
    this.map.addOverlay(collection);
  };

  /**
   * 显示麻点图
   * @param {[object]} list 数据集合 [{ lng, lat, name, address }]
   */
  showHeat = (list) => {
    const points = list.map((s) => {
      const { lng, lat, name, address } = s;
      return { lng, lat, count: 1 };
    });
    const heatmapOverlay = new BMapLib.HeatmapOverlay({
      maxOpacity: 1,
      minOpacity: 0,
      blur: 0.75,
      radius: 20,
      /*{
        .01:'rgb(123, 240, 146)',
        .3:'rgb(212, 226, 89)',
        .6:'rgb(249, 212, 35)',
        .9:'rgb(248, 54, 0)',
      }*/
      gradient: {
        '.3': 'rgb(255, 0, 0)',
        '.6': 'rgb(255, 0, 0)',
        '.9': 'rgb(255, 255, 0)',
      },
    });
    this.map.addOverlay(heatmapOverlay);
    heatmapOverlay.setDataSet({ data: points, max: 100 });
  };

  /**
   * 添加标记
   * @param  {number} lng     经度
   * @param  {number} lat     纬度
   * @param  {string} name    名称
   * @param  {string} address 地址
   * @param  {string} iconUrl 图标地址
   */
  addMarker = (lng, lat, name, address, iconUrl) => {
    const point = new BMap.Point(lng, lat);
    const iconOpts = {
      anchor: new BMap.Size(12, 24),
    };
    const icon = new BMap.Icon(iconUrl, new BMap.Size(24, 24), iconOpts);
    const marker = new BMap.Marker(point, { icon });
    const opts = {
      width: 220,
      title: name,
    };
    const infoWindow = new BMap.InfoWindow(address, opts);
    marker.addEventListener('click', (e) => {
      this.map.openInfoWindow(infoWindow, point);
    });
    this.map.addOverlay(marker);
  };

  /**
   * 地图缩放完成事件处理
   */
  mapZoomEnd = () => {
    const { mapZoom } = this.state;
    const newMapZoom = this.map.getZoom();
    this.updateState({
      mapZoom: newMapZoom,
      mapCenter: this.map.getCenter(),
    }).then(() => {
      if (
        (mapZoom >= mapLevelBreak && newMapZoom < mapLevelBreak) ||
        (mapZoom < mapLevelBreak && newMapZoom >= mapLevelBreak)
      ) {
        this.refreshMap();
      }
    });
  };

  /**
   * 地图移动完成事件处理
   */
  mapMoveEnd = () => {
    this.updateState({ mapCenter: this.map.getCenter() });
  };

  /**
   * 重置查询条件
   */
  reset = () => {
    const { userProvinceId, userCityId } = this.state;
    this.updateState({
      provinceId: userProvinceId,
      cityId: userCityId,
      studentTypes: defaultStudentTypes,
      isCitySelectDisabled: this.checkIsSpecialProvince(userProvinceId),
    });
  };

  /**
   * 勾选年级事件处理
   * @param  {string} gradeId 年级ID
   * @param  {boolean} check 是否选中
   */
  gradeChange = (gradeId, check) => {
    const { grades } = this.state;
    this.updateState({
      grades: grades.map((grade) => {
        if (grade.id === gradeId) {
          return { ...grade, active: check };
        }
        return grade;
      }),
    }).then(() => {
      this.refreshMap();
    });
  };

  /**
   * 省份选框值改变时处理，直辖市或者港澳台时，城市选框失效
   * @param  {string} provinceId 省份ID
   */
  provinceChange = (provinceId) => {
    this.updateState({
      provinceId,
      cityId: null,
      isCitySelectDisabled: this.checkIsSpecialProvince(provinceId),
    });
  };

  /**
   * 判断是否直辖市或特别行政区
   * 直辖市：北京市110000、天津市120000 重庆市500000 上海市310000
   * 香港特别行政区810000、澳门特别行政区820000、台湾省710000
   * @param  {[type]} provinceId 省份ID
   * @return {boolean}
   */
  checkIsSpecialProvince = (provinceId) => {
    return ['110000', '120000', '500000', '310000', '810000', '820000', '710000'].indexOf(provinceId) >= 0;
  };

  render() {
    const { loading, grades, provinceId, cityId, studentTypes, region, isCitySelectDisabled } = this.state;
    const maxCount = grades.length > 0 ? grades[0].count1 + grades[0].count2 : 0;
    const barTotalWidth = 200;

    return (
      <Page loading={loading} route={this.props.route}>
        <SearchBar onSearch={this.search} onReset={this.reset}>
          <Select
            placeholder="选择省份"
            allowClear={false}
            dict={formateProvinceDict(region)}
            value={provinceId}
            onChange={this.provinceChange}
          />
          <Select
            placeholder="选择城市"
            dict={provinceId ? formateCityDict(region, provinceId) : null}
            value={cityId}
            onChange={(cityId) => this.updateState({ cityId })}
            disabled={isCitySelectDisabled}
          />
          <MultiSelect
            placeholder="学生类型"
            dict={studentTypeDict}
            value={studentTypes}
            onChange={(studentTypes) => this.updateState({ studentTypes })}
          />
        </SearchBar>
        <div style={{ clear: 'left' }}>
          <div className={styles.right}>
            <div className={styles.gradeBox}>
              <div className={styles.gradeBoxTitle}>学生年级分布</div>
              <div className={styles.gradeBoxContent}>
                {grades.map((grade, i) => {
                  const { id, name, count1, count2, active } = grade;
                  const width1 = (count1 / maxCount) * barTotalWidth;
                  const width2 = (count2 / maxCount) * barTotalWidth;
                  return (
                    <div key={i} className={styles.gradeWrap + (active ? ' ' + styles.active : '')}>
                      <span className={styles.checkWrap}>
                        <Checkbox checked={active} onChange={(e) => this.gradeChange(id, e.target.checked)}>
                          {name}
                        </Checkbox>
                      </span>
                      {(count1 || count2) && (
                        <Popover
                          content={
                            <span>
                              <span>{`正式学生：${count1}人`}</span>
                              <br />
                              <span>{` 潜在学生：${count2}人`}</span>
                            </span>
                          }
                        >
                          <span className={styles.barWrap}>
                            {count1 > 0 && (
                              <span className={styles.bar + ' ' + styles.studentTypeColor1} style={{ width: width1 }}>
                                {width1 > 20 ? count1 : ''}
                              </span>
                            )}
                            {count2 > 0 && (
                              <span className={styles.bar + ' ' + styles.studentTypeColor2} style={{ width: width2 }}>
                                {width2 > 20 ? count2 : ''}
                              </span>
                            )}
                          </span>
                        </Popover>
                      )}
                    </div>
                  );
                })}
                {grades.length === 0 && (
                  <div style={{ textAlign: 'center', marginTop: 20, color: '#ccc' }}>暂无学生数据</div>
                )}
              </div>
            </div>
            <div className={styles.tipsBar}>
              <div className={styles.tipsItem}>
                <span className={styles.tipsIcon + ' ' + styles.studentTypeColor1}></span>
                <span className={styles.tipsText}>正式学生</span>
              </div>
              <div className={styles.tipsItem}>
                <span className={styles.tipsIcon + ' ' + styles.studentTypeColor2}></span>
                <span className={styles.tipsText}>潜在学生</span>
              </div>
            </div>
          </div>
          <div className={styles.left}>
            <div className={styles.mapBox} ref={(c) => (this._$map = ReactDOM.findDOMNode(c))}></div>
            <div className={styles.tipsBar}>
              <div className={styles.tipsItem}>
                <span className={styles.tipsIcon}>
                  <img src={studentIcon} />
                </span>
                <span className={styles.tipsText}>学生位置</span>
              </div>
              <div className={styles.tipsItem}>
                <Icon className={styles.tipsIcon} type="environment" style={{ color: '#5599F5' }} />
                <span className={styles.tipsText}>学习中心位置</span>
              </div>
              <div className={styles.tipsItem}>
                <Icon className={styles.tipsIcon} type="environment" style={{ color: '#72D583' }} />
                <span className={styles.tipsText}>公校位置</span>
              </div>
            </div>
          </div>
        </div>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  console.log(state);
  return {
    user: state.account.user,
  };
};

export default connect(mapStateToProps)(CustomerMapAnalyze);
