import React from 'react';
import { connect } from 'dva';
import { DatePicker } from 'antd';
import moment from 'moment';
import { Page, SearchFilter, SearchField, Input, Select, Button, StaffSelect, List, ActionModal } from 'COMPONENT';
import AuditForm from './AuditForm';
import AssignForm from './AssignForm';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import {
  pageAuditPrimarySchoolResources,
  exportAuditPrimarySchoolResources,
  auditPrimarySchoolResource,
  assignPrimarySchoolResource,
} from 'SERVICE/customer';
import { download } from 'SRC/utils/common';

const DATE_FORMAT = 'YYYY-MM-DD';
const DATETIME_FORMAT = 'YYYY-MM-DD HH:mm:ss';

const AUDIT_STATUS_DICT = [
  { val: '0', text: '待审核' },
  { val: '1', text: '审核通过' },
  { val: '9', text: '审核不通过' },
];
const CONFIRM_STATUS_DICT = [
  { val: '0', text: '未确认' },
  { val: '1', text: '确认有效' },
  { val: '9', text: '确认无效' },
];
const DEFAULT_SEARCH_PARAMS = {
  studentName: '',
  customerName: '',
  contact: '',
  createUserId: '',
  createUserName: '',
  intentionCampusId: '',
  startTime: null,
  endTime: null,
  auditStatus: '',
  confirmStatus: '',
};
const DEFAULT_ACTION = {
  audit: false,
  assign: false,
  data: null,
};

class Main extends React.Component {
  state = {
    loading: false, // {boolean} 加载状态

    //search condition
    searchParams: { ...DEFAULT_SEARCH_PARAMS },

    //操作
    action: { ...DEFAULT_ACTION },

    pagination: {
      current: 1,
      pageSize: 50,
      total: 0,
    },

    data: [],
  };

  columns = [
    { title: '学生姓名', dataIndex: 'studentName2', key: 'studentName2', width: 80 },
    { title: '家长姓名', dataIndex: 'customerName2', key: 'customerName2', width: 80 },
    { title: '意向学习中心', dataIndex: 'intentionCampusName', key: 'intentionCampusName', width: 100 },
    { title: '联系方式', dataIndex: 'contact', key: 'contact', width: 80 },
    { title: '登记人', dataIndex: 'createUserFullName', key: 'createUserFullName', width: 100 },
    { title: '跟进人', dataIndex: 'followupUserFullName', key: 'followupUserFullName', width: 100 },
    {
      title: '登记时间',
      dataIndex: 'createTime',
      key: 'createTime',
      width: 120,
      render: (v) => moment(v).format(DATETIME_FORMAT),
    },
    { title: '外呼确认状态', dataIndex: 'confirmStatusDesc', key: 'confirmStatus', width: 80 },
    {
      title: '外呼确认备注',
      dataIndex: 'confirmRemark',
      key: 'confirmRemark',
      width: 120,
      render: (v) => (v ? v : '-'),
    },
    { title: '审核状态', dataIndex: 'auditStatusDesc', key: 'auditStatus', width: 80 },
    { title: '审核备注', dataIndex: 'auditRemark', key: 'auditRemark', width: 120, render: (v) => (v ? v : '-') },
    {
      title: '审核时间',
      dataIndex: 'auditTime',
      key: 'auditTime',
      width: 120,
      render: (v) => (v ? moment(v).format(DATETIME_FORMAT) : '-'),
    },
    {
      title: '操作',
      key: 'action',
      fixed: 'right',
      width: 60,
      getActionConfig: (v, row) => {
        const btns = [];
        if (row.auditStatus === 0) {
          btns.push({
            text: '审核',
            onClick: () => this.handleOpenModal('audit', row),
          });
        } else if (row.auditStatus === 1 && (!row.followupUserId || row.followupUserId === '')) {
          btns.push({
            text: '分配',
            onClick: () => this.handleOpenModal('assign', row),
          });
        }
        return btns;
      },
    },
  ];

  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
  }

  getCampusDict = (tree) => {
    if (!tree) {
      return [];
    }

    return tree.map((item) => ({
      label: item.text,
      options: item.campus,
      val: item.val,
    }));
  };

  componentDidMount() {
    this.init();

    this.loadPage();
  }

  async init() {
    this.setState({ loading: true });

    await new Promise(this.initDict);

    this.setState({
      loading: false,
    });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branchCampusTree'],
        callback: resolve,
      },
    });
  }

  handleSearchParamChangeEvent = (key) => {
    return function binder(e) {
      const update = {};
      update[key] = e ? (e.target ? e.target.value : e) : '';
      this.handleSearchParamChange(update);
    };
  };

  handleSearchParamChange = (changes) => {
    this.setState({ searchParams: { ...this.state.searchParams, ...changes } });
  };

  handleReset = () => {
    const { pagination } = this.state;
    this.setState(
      { searchParams: { ...DEFAULT_SEARCH_PARAMS }, pagination: { ...pagination, current: 1 } },
      //this.loadPage,
    );
  };

  handlePageChange = (pagination) => {
    const { pageSize, current } = pagination;
    this.setState({ pagination: { ...this.state.pagination, pageSize, current } }, this.loadPage);
  };

  handleSearch = () => {
    const { pagination } = this.state;
    this.setState({ pagination: { ...pagination, current: 1 } }, this.loadPage);
  };

  handleOpenModal = (actionKey, data) => {
    const { action } = this.state;
    action[actionKey] = true;
    action.data = data;
    this.setState({ action });
  };

  handleCloseModal = () => {
    this.setState({ action: { ...DEFAULT_ACTION } });
  };

  loadPage = async () => {
    this.setState({ loading: true });
    const { searchParams, pagination } = this.state;

    try {
      const res = await pageAuditPrimarySchoolResources({
        ...searchParams,
        pageNum: pagination.pageNum,
        pageSize: pagination.pageSize,
      });
      if (res.resultCode == 0) {
        const data = res.data;
        this.setState({
          data: Array.isArray(data.list) ? data.list : [],
          pagination: { ...pagination, total: data.total },
        });
      }
    } catch (e) {
      console.error(e);
      showMessage(e.text, 'error');
    } finally {
      this.setState({ loading: false });
    }
  };

  doAudit = async (form) => {
    const res = await auditPrimarySchoolResource({
      ...form,
      followupUserId: form.followupUser ? form.followupUser.val : null,
    });

    if (res.resultCode != 0) {
      return Promise.reject(res.resultMessage);
    } else {
      //重新加载页面刷新数据
      this.loadPage();
    }
  };

  doAssign = async (form) => {
    const res = await assignPrimarySchoolResource({
      id: form.id,
      followupUserId: form.followupUser.val,
    });

    if (res.resultCode != 0) {
      return Promise.reject(res.resultMessage);
    } else {
      //重新加载页面刷新数据
      this.loadPage();
    }
  };

  doExport = async () => {
    const { searchParams } = this.state;

    this.setState({ loading: true });
    try {
      const res = await exportAuditPrimarySchoolResources(searchParams);

      if (res && res.resultCode == 0 && res.data) {
        download(res.data, res.data.split(/\//).pop());
      }
    } catch (e) {
      console.error(e);
      showMessage(e.text, 'error');
    } finally {
      this.setState({ loading: false });
    }
  };

  render() {
    const { loading, searchParams, pagination, data, total, action } = this.state;
    const { exportAuth, branchCampusTree, brenchId } = this.props;
    // 非超管只显示主组织
    const usableCampusList = branchCampusTree.filter((b) => b.val === brenchId);

    return (
      <Page
        fitWindowHeight
        loadingTip="正在加载数据"
        loading={loading}
        route={this.props.route}
        headerToolBar={
          exportAuth && (
            <Button onClick={this.doExport} type="primary" disabled={total === 0}>
              导出
            </Button>
          )
        }
        subHeader={
          <SearchFilter truncationNum={9} onSearch={this.handleSearch} onReset={this.handleReset}>
            <SearchField>
              <Input
                value={searchParams.studentName}
                placeholder="学生姓名"
                allowClear
                maxLength={20}
                onChange={this.handleSearchParamChangeEvent('studentName').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Input
                value={searchParams.customerName}
                placeholder="家长姓名"
                allowClear
                maxLength={20}
                onChange={this.handleSearchParamChangeEvent('customerName').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Input
                value={searchParams.contact}
                placeholder="联系方式"
                allowClear
                maxLength={11}
                onChange={this.handleSearchParamChangeEvent('contact').bind(this)}
              />
            </SearchField>
            <SearchField>
              <StaffSelect
                placeholder="登记人"
                onChange={(selected) =>
                  this.handleSearchParamChange({ createUserId: selected.val, createUserName: selected.text })
                }
                initialValue={{
                  value: searchParams.createUserId,
                  text: searchParams.createUserName,
                }}
              />
            </SearchField>
            <SearchField>
              <Select
                value={searchParams.intentionCampusId}
                placeholder="意向学习中心"
                showSearch
                noOptionTips="暂无数据"
                dict={this.getCampusDict(usableCampusList)}
                onChange={this.handleSearchParamChangeEvent('intentionCampusId').bind(this)}
              />
            </SearchField>
            <SearchField>
              <DatePicker
                placeholder="登记开始时间"
                value={searchParams.startTime}
                onChange={(date, dateString) =>
                  this.handleSearchParamChange({ startTime: date ? moment(dateString + ' 00:00:00') : null })
                }
                allowClear
                size="large"
                format={DATE_FORMAT}
              />
            </SearchField>
            <SearchField>
              <DatePicker
                placeholder="登记结束时间"
                value={searchParams.endTime}
                onChange={(date, dateString) =>
                  this.handleSearchParamChange({ endTime: date ? moment(dateString + ' 23:59:59') : null })
                }
                allowClear
                size="large"
                format={DATE_FORMAT}
              />
            </SearchField>
            <SearchField>
              <Select
                value={searchParams.auditStatus}
                placeholder="审核状态"
                dict={AUDIT_STATUS_DICT}
                onChange={this.handleSearchParamChangeEvent('auditStatus').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Select
                value={searchParams.confirmStatus}
                placeholder="外呼确认状态"
                dict={CONFIRM_STATUS_DICT}
                onChange={this.handleSearchParamChangeEvent('confirmStatus').bind(this)}
              />
            </SearchField>
          </SearchFilter>
        }
      >
        <List
          rowKey="id"
          dataSource={data}
          columns={this.columns}
          pagination={pagination}
          onChange={this.handlePageChange}
        />

        {action.audit && (
          <ActionModal id={action.data.id} title="审核资源" onSubmit={this.doAudit} onClose={this.handleCloseModal}>
            <AuditForm />
          </ActionModal>
        )}

        {action.assign && (
          <ActionModal id={action.data.id} title="分配跟进人" onSubmit={this.doAssign} onClose={this.handleCloseModal}>
            <AssignForm />
          </ActionModal>
        )}
      </Page>
    );
  }
}

const mapStoreToProps = (store) => {
  const { user, btnAuths } = store.account;
  const { brenchId, onlineTime } = user;

  return {
    branchCampusTree: store.dict.branchCampusTree || [],
    exportAuth: btnAuths.indexOf('PrimarySchoolResourceAuditExport') > -1,
    brenchId,
    onlineTime,
  };
};

export default connect(mapStoreToProps)(Main);
