import React from 'react';
import { connect } from 'dva';
import PropTypes from 'prop-types';
import { Loading, Block, Button, Select, Icon } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import Page from 'COMPONENT/Page';
import { Form, Input, Row, Col } from 'antd';
import { loadCommonDictNew, loadAllBranchCampusDict } from 'UTIL/dict';
import SchoolModal from './SchoolModal';
import { runFunc } from 'UTIL/common';
import { registerPrimarySchoolResource } from 'SERVICE/customer';

const FormItem = Form.Item;
const genderDict = [
  { val: '1', text: '男' },
  { val: '2', text: '女' },
];
const formItemCol = {
  labelCol: { span: 6 },
  wrapperCol: { span: 16 },
};
const phoneRegExp = /^1\d{10}$/;

function extendFormItemCfg(origin, extend = {}) {
  for (let key in extend) {
    if (origin.hasOwnProperty(key)) {
      for (let subKey in extend[key]) {
        if (origin[key].hasOwnProperty(subKey)) {
          origin[key][subKey] = extend[key][subKey];
        }
      }
    }
  }
  return origin;
}

class Main extends React.Component {
  constructor(props) {
    super(props);

    this.formItemCfg = extendFormItemCfg(
      {
        // 学生姓名
        studentName: {
          initialValue: '',
          required: true,
          visible: true,
          disabled: false,
        },
        // 家长姓名
        customerName: {
          initialValue: '',
          required: true,
          visible: true,
          disabled: false,
        },
        // 联系方式
        contact: {
          initialValue: '',
          required: true,
          visible: true,
          disabled: false,
        },
        // 意向学习中心
        intentionCampusId: {
          initialValue: null,
          required: true,
          visible: true,
          disabled: false,
        },
        // 学生年级
        studentGrade: {
          initialValue: '',
          required: false,
          visible: true,
          disabled: false,
        },
        // 就读学校
        schoolName: {
          initialValue: '',
          required: false,
          visible: true,
          disabled: false,
        },
        // 性别
        gender: {
          initialValue: '',
          required: false,
          visible: true,
          disabled: false,
        },
        // 住址
        address: {
          initialValue: '',
          required: false,
          visible: true,
          disabled: false,
        },
      },
      props.formItemCfg,
    );

    this.dicts = {
      studentGradeDict: [],
      campusDict: [],
    };

    this.state = Object.assign(this.dicts, {
      loading: true,
      disabled: false,
      schoolModalVisible: false,
      intentionCampusDisabled: false,
    });

    this.defaultIntentionCampus = null;

    this.formRef = null;
  }

  static propTypes = {
    btnGroupVisible: PropTypes.bool,
    afterConfirm: PropTypes.func,
    formItemCfg: PropTypes.object,
  };

  static defaultProps = {
    btnGroupVisible: true,
    formItemCfg: {},
  };

  save = async (params) => {
    this.setState({ loading: true });
    try {
      const res = await registerPrimarySchoolResource({
        ...params,
      });
      if (res.resultCode != 0) {
        throw new Error(res.resultMessage);
      }
      showMessage(`${params.contact}提交审核成功`, 'success');

      if (this.props.afterConfirm) {
        // 仅用于Modal模式
        runFunc(this.props.afterConfirm);
      } else {
        this.onResetClick();
      }
    } catch (e) {
      console.error(e);
      showAlert(`提交审核失败：${e.text}`);
    } finally {
      this.setState({ loading: false });
    }
  };

  onFormSubmit = (e) => {
    e.preventDefault();
    this.props.form.validateFieldsAndScroll((err, values) => {
      if (err) {
        return showMessage('提交审核失败：请确认必填输入项是否填写完毕', 'error');
      }
      const params = Object.assign({}, values);
      delete params.schoolName;
      params.studentName = params.studentName.trim();
      params.customerName = params.customerName.trim();
      params.contact = params.contact.trim();
      params.address = params.address.trim();

      this.save(params);
    });
  };

  onSchoolModalCommit = (params) => {
    this.props.form.setFieldsValue({
      schoolName: params.schoolName,
      schoolId: params.schoolId,
    });
  };

  onResetClick = () => {
    this.props.form.resetFields();
  };

  onClearSchoolClick = () => {
    this.props.form.setFieldsValue({
      schoolName: '',
      schoolId: '',
    });
  };

  componentDidMount() {
    const { brenchId, campusId, organizationType } = this.props.user;

    Promise.all([loadCommonDictNew('STUDENT_GRADE'), loadAllBranchCampusDict()]).then(
      ([studentGradeDict, campusDict]) => {
        if (organizationType !== 'GROUNP') {
          campusDict = brenchId ? campusDict.filter((branch) => branch.val === brenchId) : [];
        }
        this.setState({
          studentGradeDict,
          campusDict,
          loading: false,
        });
        if (organizationType === 'CAMPUS') {
          this.defaultIntentionCampus = campusId;
          this.formItemCfg.intentionCampusId.initialValue = campusId;
        }
        this.setState({
          intentionCampusDisabled: !!this.defaultIntentionCampus,
        });
      },
    );
  }

  render() {
    const { organizationId } = this.props.user;
    const { getFieldDecorator, getFieldValue } = this.props.form;
    const { loading, studentGradeDict, campusDict, schoolModalVisible, disabled } = this.state;
    const formItemCfg = this.formItemCfg;
    return (
      <Page loading={loading} route={this.props.route} title={false}>
        <div>
          <Loading loading={loading} />
          <div>
            <Form ref={(c) => (this.formRef = c)} onSubmit={this.onFormSubmit}>
              <Block title="学生信息" key="basic">
                <Row type="flex" justify="start">
                  <Col span={8}>
                    <FormItem label="学生姓名" {...formItemCol}>
                      {getFieldDecorator('studentName', {
                        initialValue: formItemCfg.studentName.initialValue,
                        rules: [
                          { transform: (val) => (val ? val.trim() : val) },
                          { required: formItemCfg.studentName.required, message: '请输入学生姓名' },
                        ],
                      })(<Input disabled={formItemCfg.studentName.disabled} maxLength="20" />)}
                    </FormItem>
                  </Col>
                  <Col span={8}>
                    <FormItem label="家长姓名" {...formItemCol}>
                      {getFieldDecorator('customerName', {
                        initialValue: formItemCfg.customerName.initialValue,
                        rules: [
                          { transform: (val) => (val ? val.trim() : val) },
                          { required: formItemCfg.customerName.required, message: '请输入家长姓名' },
                        ],
                      })(<Input disabled={formItemCfg.customerName.disabled} maxLength="20" />)}
                    </FormItem>
                  </Col>
                </Row>

                <Row type="flex" justify="start">
                  <Col span={8}>
                    <FormItem label="联系方式" {...formItemCol}>
                      {getFieldDecorator('contact', {
                        initialValue: formItemCfg.contact.initialValue,
                        rules: [
                          {
                            required: formItemCfg.contact.required,
                            message: '请输入联系方式',
                          },
                          {
                            pattern: phoneRegExp,
                            message: '请填写正确的联系方式',
                          },
                        ],
                      })(<Input disabled={formItemCfg.contact.disabled} maxLength="11" />)}
                    </FormItem>
                  </Col>
                  <Col span={8}>
                    <FormItem label="意向学习中心" {...formItemCol}>
                      {getFieldDecorator('intentionCampusId', {
                        initialValue: formItemCfg.intentionCampusId.initialValue,
                        rules: [{ required: formItemCfg.intentionCampusId.required, message: '请选择意向学习中心' }],
                      })(
                        <Select
                          placeholder="请选择意向学习中心"
                          showSearch
                          dict={campusDict}
                          disabled={formItemCfg.intentionCampusId.disabled}
                        />,
                      )}
                    </FormItem>
                  </Col>
                </Row>

                <Row type="flex" justify="start">
                  <Col span={8}>
                    <FormItem label="学生年级" {...formItemCol}>
                      {getFieldDecorator('studentGrade', {
                        initialValue: formItemCfg.studentGrade.initialValue,
                        rules: [{ required: formItemCfg.studentGrade.required, message: '请选择学生年级' }],
                      })(
                        <Select
                          placeholder="请选择学生年级"
                          showSearch
                          dict={studentGradeDict}
                          disabled={formItemCfg.studentGrade.disabled}
                        />,
                      )}
                    </FormItem>
                  </Col>
                  <Col span={8}>
                    <FormItem label="就读学校" {...formItemCol}>
                      {getFieldDecorator('schoolName', {
                        initialValue: formItemCfg.schoolName.initialValue,
                        rules: [{ required: formItemCfg.schoolName.required, message: '请选择就读学校' }],
                      })(
                        <Input
                          autoComplete="off"
                          onClick={() => {
                            this.setState({
                              schoolModalVisible: true,
                            });
                          }}
                          suffix={
                            getFieldValue('schoolId') ? (
                              <Icon
                                type="cross_circle"
                                style={{
                                  color: '#6d67ff',
                                }}
                                onClick={this.onClearSchoolClick}
                              />
                            ) : null
                          }
                          style={{ caretColor: 'transparent' }}
                          disabled={formItemCfg.schoolName.disabled}
                        />,
                      )}
                    </FormItem>
                    <div style={{ display: 'none' }}>
                      <FormItem>{getFieldDecorator('schoolId', {})(<Input />)}</FormItem>
                    </div>
                  </Col>
                </Row>

                <Row type="flex" justify="start">
                  <Col span={8}>
                    <FormItem label="性别" {...formItemCol}>
                      {getFieldDecorator('gender', {
                        initialValue: formItemCfg.gender.initialValue,
                        rules: [
                          { transform: (val) => (val ? val.trim() : val) },
                          { required: formItemCfg.gender.required, message: '请选择性别' },
                        ],
                      })(<Select placeholder="请选择性别" dict={genderDict} />)}
                    </FormItem>
                  </Col>
                  <Col span={8}>
                    <FormItem label="住址" {...formItemCol}>
                      {getFieldDecorator('address', {
                        initialValue: formItemCfg.address.initialValue,
                        rules: [
                          { transform: (val) => (val ? val.trim() : val) },
                          { required: formItemCfg.address.required, message: '请输入住址' },
                        ],
                      })(<Input disabled={formItemCfg.address.disabled} maxLength="50" />)}
                    </FormItem>
                  </Col>
                </Row>
              </Block>

              <Row type="flex" justify="start" style={{ display: this.props.btnGroupVisible ? 'flex' : 'none' }}>
                <Col span={16}>
                  <div
                    style={{
                      display: 'flex',
                      justifyContent: 'center',
                    }}
                  >
                    <FormItem wrapperCol={{ span: 12, offset: 5 }}>
                      <Button type="primary" htmlType="submit" disabled={disabled} id="AddTransferCustomer-submit-btn">
                        提交审核
                      </Button>
                    </FormItem>
                    <FormItem
                      wrapperCol={{ span: 12, offset: 5 }}
                      style={{
                        marginLeft: '20px',
                      }}
                    >
                      <Button htmlType="button" onClick={this.onResetClick}>
                        重置
                      </Button>
                    </FormItem>
                  </div>
                </Col>
              </Row>
            </Form>
          </div>
          {schoolModalVisible && (
            <SchoolModal
              orgId={organizationId}
              onCommit={this.onSchoolModalCommit}
              onClose={() => this.setState({ schoolModalVisible: false })}
            />
          )}
        </div>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const data = state.main.pageParams['/PrimarySchoolResourceRegister'];
  return {
    user: state.account.user,
  };
};

export default connect(mapStateToProps)(Form.create()(Main));
