import React from 'react';
import { connect } from 'dva';
import { Page, SearchFilterForm, SearchFieldForm, Select, Button } from 'COMPONENT';
import { Table, DatePicker } from 'antd';
import moment from 'moment';
import { getInsightChargeStatistics, exportInsightChargeStatistics } from 'SERVICE/contract';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { errorsHandle } from 'UTIL/common';
import file from 'UTIL/file';

import styles from './styles.module.less';

const dayFormat = 'YYYY-MM-DD';
const errorsHandler = errorsHandle();

class PlatformCollectionStatistics extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initData = this.initData.bind(this);
    this.fetchList = this.fetchList.bind(this);
    this.loadList = this.loadList.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onPageChange = this.onPageChange.bind(this);
    this.onExport = this.onExport.bind(this);

    this.columns = [
      {
        title: '签单分公司',
        dataIndex: 'branchName',
        width: 180,
      },
      {
        title: '市场活动',
        width: 330,
        dataIndex: 'marketActivity',
        children: [
          {
            title: '收款金额',
            align: 'center',
            className: 'money-text-align',
            dataIndex: 'activityCashflow',
            width: 110,
          },
          {
            align: 'center',
            title: '退款金额',
            className: 'money-text-align',
            dataIndex: 'activityRefund',
            width: 110,
          },
          {
            align: 'center',
            title: '净额',
            className: 'money-text-align',
            dataIndex: 'activityNet',
            width: 110,
          },
        ],
      },
      {
        title: '引流活动',
        width: 330,
        dataIndex: 'market',
        children: [
          {
            title: '收款金额',
            className: 'money-text-align',
            dataIndex: 'marketCashflow',
            width: 110,
          },
          {
            title: '退款金额',
            className: 'money-text-align',
            dataIndex: 'marketRefund',
            width: 110,
          },
          {
            title: '净额',
            className: 'money-text-align',
            dataIndex: 'marketNet',
            width: 110,
          },
        ],
      },
      {
        title: '讲座订单',
        width: 330,
        dataIndex: 'lecture',
        children: [
          {
            title: '收款金额',
            className: 'money-text-align',
            dataIndex: 'lectureCashflow',
            width: 110,
          },
          {
            title: '退款金额',
            className: 'money-text-align',
            dataIndex: 'lectureRefund',
            width: 110,
          },
          {
            title: '净额',
            className: 'money-text-align',
            dataIndex: 'lectureNet',
            width: 110,
          },
        ],
      },
      {
        title: '合计',
        width: 330,
        dataIndex: 'all',
        children: [
          {
            title: '收款金额',
            className: 'money-text-align',
            dataIndex: 'allCashflow',
            width: 110,
          },
          {
            title: '退款金额',
            className: 'money-text-align',
            dataIndex: 'allRefund',
            width: 110,
          },
          {
            title: '净额',
            className: 'money-text-align',
            dataIndex: 'allNet',
            width: 110,
          },
        ],
      },
    ];
    this.searchRef = null;

    this.state = {
      list: [],
      total: 0,
      pageNum: 1,
      pageSize: 20,
      loading: false,
      searchData: {},
      rangeInitial: null,
      tableFixedHeight: 100,
    };
  }

  componentDidMount() {
    this.init();
    this.setTableFixedHeight();
    window.addEventListener('resize', this.setTableFixedHeight, false);
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.setTableFixedHeight, false);
  }

  async init() {
    this.initDict();
    this.initData();
  }

  initDict() {
    const { dispatch } = this.props;
    return new Promise((resolve, _) => {
      dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['branch'],
          callback() {
            resolve();
          },
        },
      });
    });
  }

  initData() {
    const endDate = moment().endOf('day');
    const beginDate = moment().startOf('year');
    const searchData = {
      endDate: endDate.format(dayFormat),
      beginDate: beginDate.format(dayFormat),
    };
    this.setState(
      {
        searchData,
        rangeInitial: [beginDate, endDate],
      },
      () => {
        this.onSearch(searchData);
      },
    );
  }

  setTableFixedHeight = () => {
    this.setState({
      tableFixedHeight: Math.max(window.document.body.clientHeight - 388, 100),
    });
  };

  async fetchList(apiParams) {
    if (apiParams.range && apiParams.range.length) {
      apiParams.endDate = apiParams.range[1].format(dayFormat);
      apiParams.beginDate = apiParams.range[0].format(dayFormat);
      delete apiParams.range;
    }
    if (
      moment(apiParams.endDate).endOf('day').valueOf() - moment(apiParams.beginDate).startOf('day').valueOf() >
      366 * 24 * 60 * 60 * 1000
    ) {
      showMessage('前后间隔最大366天（即一年）', 'error');
      return false;
    }
    const res = await getInsightChargeStatistics(apiParams).catch(errorsHandler);
    if (res && res.data) {
      this.setState({
        total: res.data.total,
        list: res.data.list.map((item, id) => ({ id, ...item })),
      });
    } else {
      this.setState({
        list: [],
        total: 0,
      });
    }
  }

  async loadList(apiParams) {
    this.setState({ loading: true });
    await this.fetchList(apiParams);
    this.setState({ loading: false });
  }

  onSearch(searchData) {
    let apiParams;
    this.setState(
      (state) => {
        apiParams = this.listParamsAdapter(searchData, 1, state.pageSize);
        return {
          searchData,
          pageNum: 1,
        };
      },
      () => {
        this.loadList(apiParams);
      },
    );
  }

  onReset = () => {
    this.setState({
      list: [],
      total: 0,
    });
  };

  onPageChange({ current: pageNum, pageSize }) {
    let apiParams;
    this.setState(
      (state) => {
        pageNum = state.pageSize === pageSize ? pageNum : 1;
        apiParams = this.listParamsAdapter(state.searchData, pageNum, pageSize);
        return {
          pageNum,
          pageSize,
        };
      },
      () => {
        this.loadList(apiParams);
      },
    );
  }

  async onExport() {
    if (!this.state.total) {
      return showMessage('当前查询数据为空');
    }
    const apiParams = this.searchRef.form.getFieldsValue() || this.state.searchData;
    if (apiParams.range && apiParams.range.length) {
      apiParams.endDate = apiParams.range[1].format(dayFormat);
      apiParams.beginDate = apiParams.range[0].format(dayFormat);
      delete apiParams.range;
    }
    if (
      moment(apiParams.endDate).endOf('day').valueOf() - moment(apiParams.beginDate).startOf('day').valueOf() >
      366 * 24 * 60 * 60 * 1000
    ) {
      showMessage('前后间隔最大366天（即一年）', 'error');
      return false;
    }
    this.setState({ loading: true });
    const res = await exportInsightChargeStatistics({
      ...apiParams,
      pageSize: this.state.total,
    }).catch(errorsHandler);
    if (res && res.data) {
      showMessage(res.resultMessage || '导出Excel成功', 'success');
      file.downloadDirect(res.data, `流量平台收款统计_${Date.now()}`);
    } else {
      showMessage(res.resultMessage || '导出数据失败，请稍后重试', 'error');
    }
    this.setState({ loading: false });
  }

  listParamsAdapter = (searchData, pageNum, pageSize) => {
    return {
      ...searchData,
      pageNum,
      pageSize,
    };
  };

  render() {
    const { state, props } = this;
    const { btnAuth, branchDict } = props;

    return (
      <Page
        className={styles.platformCollectionStatisticsWrapper}
        route={props.route}
        loading={state.loading}
        subHeader={
          <div>
            <SearchFilterForm
              ref={(c) => (this.searchRef = c)}
              truncationNum={5}
              onSearch={this.onSearch}
              onReset={this.onReset}
              expandToolbarBody={
                btnAuth.join(',').indexOf('PLATFORM_COLLECTION_STATISTICS_B') !== -1 && (
                  <Button onClick={this.onExport}>导出Excel</Button>
                )
              }
            >
              <SearchFieldForm name="range" span={2} initialValue={state.rangeInitial} required>
                <DatePicker.RangePicker
                  placeholder={['开始日期', '结束日期']}
                  format={dayFormat}
                  allowClear={false}
                  style={{ width: '100%' }}
                />
              </SearchFieldForm>
              <SearchFieldForm name="branchId">
                <Select showSearch dict={branchDict} placeholder="分公司" />
              </SearchFieldForm>
            </SearchFilterForm>
          </div>
        }
        fitWindowHeight
      >
        <Table
          rowKey="id"
          dataSource={state.list}
          total={8}
          columns={this.columns}
          bordered={true}
          fixedHeader={true}
          onChange={this.onPageChange}
          pagination={{
            total: state.total,
            current: state.pageNum,
            pageSize: state.pageSize,
            showQuickJumper: true,
            pageSizeOptions: ['10', '20', '50'],
            showSizeChanger: true,
            showTotal: (total) => `共 ${total} 条记录`,
          }}
          scroll={{ y: this.state.tableFixedHeight, x: this.columns.reduce((sum, item) => (sum += item.width), 0) }}
        />
      </Page>
    );
  }
}

const mapStoreToProps = (store) => {
  const { user } = store.account;
  const { roleCode, brenchId } = user;
  const isSuperAdmin = roleCode && roleCode.indexOf('SUPER_ADMIN') >= 0;

  return {
    brenchId,
    isSuperAdmin,
    branchDict: store.dict.branch,
    btnAuth: store.account.btnAuths || [],
  };
};

export default connect(mapStoreToProps)(PlatformCollectionStatistics);
