import React from 'react';
import ReactDOM from 'react-dom';
import { Table, Popconfirm, Form, message, Icon, Tooltip } from 'antd';
import Cell from './Cell';
import styles from '../style/index.less';
import { isEditing, isRenderScrollerButton, hasOverflowX, computeMaxScrollLeft, animateScrollLeft } from '../utils';
import { SCROLL_WIDTH_PER_TIME } from './width';

/**
 * 表格
 * props.list 注意添加list[]的key值
 * props.onSaveRow 保存行数据方法(行数据, 保存完成的回调函数)
 * props.columns 包含下面属性
 * columns[].dataIndex 必填
 * columns[].key 非必填(不填则取dataIndex值，若dataIndex存在重复字段，则需传唯一key)
 * columns[].align: 'left'|'center'|'right'
 * columns[].type: 'text'|'input'|'operation'
 * columns[].inputType: 'text'|'number'
 * columns[].showEdit: (record) => {} 是否显示编辑整行按钮(按钮权限)，从外部columns传入，type === 'operation' 时有效
 * columns[].canDoEdit: (record) => {} 能否编辑整行的首要条件，从外部columns传入，type === 'operation' 时有效
 * columns[].canBeEdit: (record) => {} 单元格能否被编辑，编辑时有效
 * columns[].validator: (record) => {} 表格的校验规则 从外部columns传入 (暂未封装)
 * columns[].style: object|(record) => {} 表格样式 对象或函数(仅当设置了type属性时有效) 没有设置type属性又想修改表格样式请在render(text, record)方法实现
 */
class EditableTable extends React.Component {
  static defaultProps = {
    loading: false,
    list: [],
    columns: [],
    defaultPagination: {
      pageSizeOptions: ['20', '50', '100'],
      showSizeChanger: true,
      showTotal: (total) => `共 ${total} 条记录`,
    },
    pagination: {},
    onPageChange: () => {},
    onSaveRow: (rowData, callback) => {
      if (callback && Object.prototype.toString.call(callback) === '[object Function]') {
        callback.call();
      }
    },
    footer: (pageData) => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      data: [],
      columns: [],
      columnsWidth: 0, // scrollX 列表总宽度
    };

    // 编辑时的临时表格数据
    this.cacheData = [];

    this.initEventListener = this.initEventListener.bind(this);
    this.initTable = this.initTable.bind(this);
    this.onScroller = this.onScroller.bind(this);
  }

  componentDidMount() {
    this.initTable();
    this.initEventListener();
  }

  /**
   * 监听列表的滚动事件
   * 主要用于有footer的时候, footer跟随body滚动
   */
  initEventListener() {
    const $tableNode = ReactDOM.findDOMNode(this);
    if (!$tableNode) {
      return;
    }

    // const $tableHead = $tableNode.querySelector('.ant-table-header');
    const $tableBody = $tableNode.querySelector('.ant-table-body');
    const $tableFoot = $tableNode.querySelector('.ant-table-footer');

    if ($tableBody && $tableFoot) {
      $tableBody.addEventListener('scroll', function (e) {
        $tableFoot.scrollLeft = e.target.scrollLeft;
      });
    }
  }

  /**
   * 初始化表格数据等
   */
  initTable() {
    const { list, columns } = this.props;
    const columnsResult = this.formatEditColumns(columns);
    const cacheData = list.map((item) => ({ ...item }));

    this.setState({
      data: list,
      columns: columnsResult.columns,
      columnsWidth: columnsResult.columnsWidth,
    });

    this.cacheData = cacheData;
  }

  /**
   * 格式化表格列及总宽度, 递归处理每一列
   * 能否编辑每一行权重为 column.showEdit > column.canDoEdit > record.editable
   * 能否编辑单个单元格用 column.canBeEdit 判断
   * columns[].type: 'text'|'input'|'operation'
   * columns[].showEdit: (record) => {} 是否显示编辑整行按钮(按钮权限)，从外部columns传入，type === 'operation' 时有效
   * columns[].canDoEdit: (record) => {} 能否编辑整行的首要条件，从外部columns传入，type === 'operation' 时有效
   * columns[].canBeEdit: (record) => {} 单元格能否被编辑，编辑时有效
   * columns[].validator: (record) => {} 表格的校验规则，从外部columns传入 (暂未封装)
   * @param columns
   */
  formatEditColumns(columns) {
    let resultObject = {
      columns: [],
      columnsWidth: 0,
    };

    if (!columns || columns.length <= 0) {
      return resultObject;
    }

    let columnsWidth = 0;

    // 根据权限等渲染编辑按钮
    const renderEdit = (col, record) => {
      // 编辑按钮权限
      const showEdit = !!col.showEdit;

      // 判断能否编辑
      let canDoEdit = true;
      if (col.canDoEdit !== undefined) {
        if (Object.prototype.toString.call(col.canDoEdit) === '[object Function]') {
          canDoEdit = col.canDoEdit(record);
        } else {
          canDoEdit = col.canDoEdit;
        }
      }

      // 判断是否正在编辑
      const { editable } = record;

      let child = '';
      if (showEdit) {
        if (canDoEdit) {
          if (editable) {
            child = (
              <span>
                <a style={{ marginRight: 10 }} onClick={() => this.onSave(record.key)}>
                  保存
                </a>

                <Popconfirm title="确定取消编辑?" onConfirm={() => this.onCancel(record.key)}>
                  <a>取消</a>
                </Popconfirm>
              </span>
            );
          } else {
            child = <a onClick={() => this.onEdit(record.key)}>编辑</a>;
          }
        } else {
          child = (
            <a onClick={() => {}} style={{ color: '#aaa' }} data-disabled={true}>
              编辑
            </a>
          );
        }
      }

      return child;
    };

    // 递归
    const formatChild = (cols) => {
      let result = [];

      cols.forEach((col) => {
        if (col.children && col.children.length > 0) {
          col.children = formatChild(col.children);
        } else {
          // 计算宽度
          columnsWidth += parseInt(col.width || 0);

          // 设置column的key(如果没有设置)
          col.key = col.key || col.dataIndex;

          if (col.type && col.type === 'input') {
            col.render = (text, record) => this.renderColumns(text, record, col.dataIndex, col);
          } else if (col.type && col.type === 'operation') {
            col.render = (text, record) => {
              return (
                <div className="editable-row-operations" style={{ textAlign: col.align || 'center' }}>
                  {renderEdit(col, record)}
                </div>
              );
            };
          }
        }

        result.push(col);
      });

      return result;
    };

    resultObject.columns = formatChild(columns);
    resultObject.columnsWidth = columnsWidth;

    return resultObject;
  }

  /**
   * 渲染表格的列数据
   * @param text 当前表格的值 record['dataIndex']
   * @param record 当前行数据
   * @param dataIndex
   * @param column 表头数据
   */
  renderColumns(text, record, dataIndex, column) {
    const { form } = this.props;

    return (
      <Cell
        form={form}
        editable={record.editable}
        value={text}
        dataIndex={dataIndex}
        record={record}
        column={column}
        onChange={(value) => this.onCellValueChange(value, record.key, dataIndex)}
      />
    );
  }

  /**
   * td数据修改/改变
   * @param value 当前表格值
   * @param key 当前行的key
   * @param column 当前表格的dataIndex
   */
  onCellValueChange(value, key, column) {
    const newData = [...this.state.data];
    const target = newData.filter((item) => key === item.key)[0];

    if (target) {
      target[column] = value;
      this.setState({ data: newData });
    }
  }

  /**
   * 编辑, target 为当前编辑行
   * 限制只能编辑一行
   */
  onEdit(key) {
    const newData = [...this.state.data];

    const editing = isEditing(newData);
    if (editing) {
      message.warning('为避免数据丢失，只支持单行数据编辑!');
      return false;
    }

    const target = newData.filter((item) => key === item.key)[0];
    if (target) {
      target.editable = true;
      this.setState({ data: newData });
    }
  }

  /**
   * 校验表单并保存, target 为当前编辑行
   * 将保存方法暴露给父组件进行ajax提交数据，提交成功更新表格
   * @param key 列表每一行的key
   */
  onSave(key) {
    const { form, onSaveRow } = this.props;

    form.validateFields((error, row) => {
      if (error) {
        return;
      }

      const newData = [...this.state.data];
      const target = newData.filter((item) => key === item.key)[0];
      if (target) {
        onSaveRow(target, () => {
          delete target.editable;
          this.setState({ data: newData });
          this.cacheData = newData.map((item) => ({ ...item }));
        });
      }
    });
  }

  /**
   * 取消编辑
   */
  onCancel(key) {
    const newData = [...this.state.data];
    const target = newData.filter((item) => key === item.key)[0];
    if (target) {
      Object.assign(target, this.cacheData.filter((item) => key === item.key)[0]);
      delete target.editable;
      this.setState({ data: newData });
    }
  }

  /**
   * 滚动条滚动
   * left: 边界值为 0
   * right: 边界值为 最大可滚动距离
   * @param scrollType 'left'|'right'
   */
  onScroller(scrollType) {
    const $tableNode = ReactDOM.findDOMNode(this);
    if (!$tableNode) {
      return;
    }

    const $tableBody = $tableNode.querySelector('.ant-table-body');
    if (!$tableBody) {
      return;
    }

    const hasOverflow = hasOverflowX($tableBody);
    if (!hasOverflow) {
      return;
    }

    const perTimeLeft = SCROLL_WIDTH_PER_TIME;
    const scrollLeft = $tableBody.scrollLeft;
    const maxLeft = computeMaxScrollLeft($tableBody);

    switch (scrollType) {
      case 'left':
        if (scrollLeft <= 0) {
          return;
        }

        let leftScroll = 0;
        if (scrollLeft - perTimeLeft > 0) {
          leftScroll = scrollLeft - perTimeLeft;
        }

        animateScrollLeft($tableBody, leftScroll, scrollType);

        break;
      case 'right':
        if (scrollLeft >= maxLeft) {
          return;
        }

        let rightScroll = maxLeft;
        if (scrollLeft + perTimeLeft < maxLeft) {
          rightScroll = scrollLeft + perTimeLeft;
        }

        animateScrollLeft($tableBody, rightScroll, scrollType);

        break;
      default:
        break;
    }
  }

  render() {
    const { data, columns, columnsWidth } = this.state;
    const { loading, defaultPagination, pagination, onPageChange, footer } = this.props;
    const paginationProp = { ...defaultPagination, ...pagination };
    const renderScrollerBtn = isRenderScrollerButton(data);

    return (
      <section className={styles.table_form}>
        <Table
          bordered
          scroll={{ x: columnsWidth, y: 1000 }}
          loading={loading}
          dataSource={data}
          columns={columns}
          pagination={paginationProp}
          onChange={({ current, pageSize }) => onPageChange({ current, pageSize })}
          footer={footer}
        />

        {renderScrollerBtn && (
          <div
            className={styles.scroller_left}
            onClick={(e) => {
              this.onScroller('left');
            }}
          >
            <Tooltip title="向左滚动">
              <Icon type="left" />
            </Tooltip>
          </div>
        )}

        {renderScrollerBtn && (
          <div
            className={styles.scroller_right}
            onClick={(e) => {
              this.onScroller('right');
            }}
          >
            <Tooltip title="向右滚动">
              <Icon type="right" />
            </Tooltip>
          </div>
        )}
      </section>
    );
  }
}

/**
 * 创建校验表单规则，数据同步将被 Form 接管
 * props => form
 */
const TableForm = Form.create()(EditableTable);

export default TableForm;
