/**
 * 将columns转成扁平化数组
 * @param columns
 */
export function transformColumnTreeToArray(columns = []) {
  let arr = [];

  const formatCols = (cols) => {
    cols.forEach((col) => {
      if (col.children && col.children.length > 0) {
        formatCols(col.children);
      } else {
        arr.push(col);
      }
    });
  };

  formatCols(columns);

  return arr;
}

/**
 * 计算fixed列的总宽度
 * @param cols
 * @return {number}
 */
export function countFixedWidth(cols) {
  let width = 0;

  cols.forEach((col) => {
    if (col.fixed !== undefined) {
      width += parseInt(col.width || 100);
    }
  });

  return width;
}

/**
 * 表格汇总行
 * @param columns
 * @param countRow 汇总行数据
 * @param domId
 */
export function renderTableFooter(columns, countRow, domId) {
  const columnsArr = transformColumnTreeToArray(columns);
  const fixedWidth = countFixedWidth(columnsArr);

  const renderTR = (cols, row) => {
    if (!row) {
      return '';
    }

    return (
      <tr>
        {cols.map((col, index) => {
          return (
            <td key={index}>
              <div style={{ width: (col.width || 100) + 'px', padding: '0 15px', boxSizing: 'border-box' }}>
                {row[col.dataIndex] !== undefined ? row[col.dataIndex] : ''}
              </div>
            </td>
          );
        })}
      </tr>
    );
  };

  return (
    (countRow && (
      <div {...(domId ? { id: domId } : {})} data-class="footer-table-wrap">
        <table className="table-footer-scroll">
          <tbody>{renderTR(columnsArr, countRow)}</tbody>
        </table>
        <table className="table-footer-fixed" data-class="count-fix-left" style={{ width: fixedWidth + 'px' }}>
          <tbody>{renderTR(columnsArr, countRow)}</tbody>
        </table>
      </div>
    )) ||
    ''
  );
}

/**
 * 判断表格是否正在编辑
 * @param data
 */
export function isEditing(data = []) {
  return data.some((item) => item.editable === true);
}

/**
 * 是否渲染左右滚动按钮
 * @param data
 * @return {*|boolean}
 */
export function isRenderScrollerButton(data) {
  return data && data.length > 0;
}

/**
 * 判断内容有无超出宽度
 * @param el
 * @return {boolean}
 */
export function hasOverflowX(el) {
  if (!el) {
    return false;
  }

  return el.scrollLeft > el.clientWidth || el.offsetWidth > el.clientWidth;
}

/**
 * 计算最大左右滚动距离
 * @param el
 */
export function computeMaxScrollLeft(el) {
  if (!el) {
    return 0;
  }

  let maxLeft = 0;
  if (hasOverflowX(el)) {
    maxLeft = el.scrollWidth - el.clientWidth;
  }

  return maxLeft;
}

/**
 * 滚动动画
 * @param el
 * @param goalScroll 目标滚动点
 * @param scrollType 'left'|'right'
 */
export function animateScrollLeft(el, goalScroll, scrollType) {
  let timer = null;
  let step = 0;
  let distance = 0;

  if (scrollType === 'left') {
    distance = el.scrollLeft - goalScroll;
    step = distance / 10;
  } else if (scrollType === 'right') {
    distance = goalScroll - el.scrollLeft;
    step = distance / 10;
  }

  clearInterval(timer);

  timer = setInterval(function () {
    if (scrollType === 'left') {
      if (el.scrollLeft <= goalScroll) {
        clearInterval(timer);
      }

      el.scrollLeft = el.scrollLeft - step;
    } else if (scrollType === 'right') {
      if (el.scrollLeft >= goalScroll) {
        clearInterval(timer);
      }

      el.scrollLeft = el.scrollLeft + step;
    } else {
      clearInterval(timer);
    }
  }, 50);
}
