import React from 'react';
import { connect } from 'dva';
import CommonSearch from '../../common/CommonSearch';
import TableForm, { ColWidth, renderTableFooter } from '../../common/TableForm';
import ImportTemplateModal from '../../common/ImportTemplateModal';
import { getGoalList, updateGoalValue } from 'SERVICE/goalManage';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import {
  formatCountRow,
  formatRecordToApiParam,
  formatRedStyle,
  renderRedText,
  renderNullPlanValue,
  formatYearText,
  isHistoryMonth,
  renderBranchText,
} from '../../util';
import TheWrapper from '../../common/TheWrapper';
import FooterTips from '../../common/FooterTips';
import YearExamGoalImg01 from '../../common/assets/YearExamGoalImg01.svg';
import YearExamGoalImg02 from '../../common/assets/YearExamGoalImg02.svg';

class YearExamGoal extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      searchParam: {}, // 搜索条件

      tableKey: '',
      listLoading: false,
      list: [], // 列表数据
      pageNo: 1, // 当前页码
      pageSize: 20, // 每页记录数
      totalRecords: 0, // 总记录数
      countRow: null, // 汇总行数据
      timeStamp: null, // 服务器时间戳

      showImportModal: false, // 显示导入弹窗
      importType: '', // 导入类型 'PLAN'|'GOAL'
      canImportHistory: false, // 能够导入历史数据
    };

    this.$ref = {
      searchRef: null,
    };

    this.onSearch = this.onSearch.bind(this);
    this.loadList = this.loadList.bind(this);
    this.formatParam = this.formatParam.bind(this);
    this.formatResponse = this.formatResponse.bind(this);
    this.setListData = this.setListData.bind(this);
    this.formatColumn = this.formatColumn.bind(this);
    this.formatMonthColumn = this.formatMonthColumn.bind(this);
    this.onSaveRow = this.onSaveRow.bind(this);
  }

  onSearch() {
    this.loadList({ pageNo: 1 });
  }

  loadList(changedParams = {}) {
    const param = this.formatParam(changedParams);

    this.setState({ listLoading: true });

    getGoalList(param)
      .then((response) => {
        let result = this.formatResponse(response);
        this.setListData(result, param);
      })
      .catch((e) => {
        let result = this.formatResponse({});
        this.setListData(result, param);
      });
  }

  formatParam(changeParam = {}) {
    const { pageNo, pageSize } = this.state;
    const searchParam = this.$ref.searchRef.getParam();
    const param = {
      pageNo,
      pageSize,
      getTotal: true,
      ...searchParam,
      ...changeParam,
    };

    param.page = param.pageNo;
    param.rows = param.pageSize;
    param.goalType = 'PROMISE_CLASS_STUDENT_NUMBER';

    this.setState({
      searchParam: { goalType: param.goalType, ...searchParam },
    });

    return param;
  }

  formatResponse(response = {}) {
    const { searchParam } = this.state;
    const { organizationType, goalTimeType } = searchParam;
    const showEditColumn = organizationType === 'CAMPUS' && goalTimeType === 'MONTH';
    const showCampusColumn = organizationType === 'CAMPUS';

    let data = response.data || [];
    let countRow = null;
    let countRowKey = '';

    if (showEditColumn) {
      countRowKey = 'operation';
    } else {
      countRowKey = showCampusColumn ? 'campusName' : 'branchName';
    }

    if (data.length > 0) {
      countRow = {};
      countRow[countRowKey] = '页面汇总';

      countRow = formatCountRow(data, countRow, countRowKey);
    }

    return {
      list: data,
      total: response.rowCount || 0,
      countRow: countRow,
      timeStamp: response.timeStamp || null,
    };
  }

  setListData(result = {}, param = {}) {
    this.setState({
      tableKey: new Date().getTime(),
      listLoading: false,
      list: result.list || [],
      totalRecords: result.total || 0,
      countRow: result.countRow,
      pageNo: param.pageNo,
      pageSize: param.pageSize,
      timeStamp: result.timeStamp,
    });
  }

  /**
   * 格式化表头, 仅当查看学习中心每月才允许编辑确认目标, 年份列表只有年度汇总
   */
  formatColumn() {
    const { btnAuth_edit, btnAuth_edit_limit } = this.props;
    const { searchParam } = this.state;
    const { yearText, organizationType, goalTimeType } = searchParam;
    const showEditColumn = organizationType === 'CAMPUS' && goalTimeType === 'MONTH';
    const editAuth = btnAuth_edit || btnAuth_edit_limit;
    const showCampusColumn = organizationType === 'CAMPUS';

    let leftColumn = this.formatLeftColumn(showEditColumn, editAuth, showCampusColumn);
    let centerColumn = this.formatMonthColumn(yearText);
    const rightColumn = this.formatRightColumn();

    if (goalTimeType === 'QUARTER') {
      centerColumn = this.formatQuarterColumn(yearText);
    }

    return leftColumn.concat(centerColumn, rightColumn);
  }

  /**
   * 左边表头
   * @param showEditColumn 是否显示操作列
   * @param editAuth 编辑按钮权限
   * @param showCampusColumn 是否显示学习中心列 仅当选择学习中心的时候显示
   */
  formatLeftColumn(showEditColumn = false, editAuth = false, showCampusColumn = false) {
    let column = [
      {
        title: '集团',
        dataIndex: 'groupName',
        width: ColWidth.WIDTH_XXS,
        fixed: 'left',
      },
      {
        title: '分公司',
        dataIndex: 'branchName',
        width: ColWidth.WIDTH_XXS,
        fixed: 'left',
        render: (text, record) => {
          return renderBranchText(text, record.hasVal);
        },
      },
    ];

    if (showCampusColumn) {
      column.push({
        title: '学习中心',
        dataIndex: 'campusName',
        width: ColWidth.WIDTH_XM,
        fixed: 'left',
      });
    }

    if (showEditColumn) {
      column.push({
        title: '操作',
        dataIndex: 'operation',
        width: 90,
        fixed: 'left',
        align: 'center',
        type: 'operation',
        showEdit: editAuth,
        canDoEdit: (record) => {
          return true;
        },
      });
    }

    return column;
  }

  /**
   * 季度表头
   * @param year 年份
   */
  formatQuarterColumn(year = '') {
    const { lastYear, thisYear } = formatYearText(year);

    return [
      {
        title: `${lastYear}秋季`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'autumnPlanValue',
            width: ColWidth.WIDTH_XXXM,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'autumnGoalValue',
            width: ColWidth.WIDTH_XXXM,
            render: (text, record) => {
              return renderRedText(text, record.autumnGoalValue, record.autumnPlanValue);
            },
          },
        ],
      },
      {
        title: `${thisYear}寒假`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'winterPlanValue',
            width: ColWidth.WIDTH_XXXM,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'winterGoalValue',
            width: ColWidth.WIDTH_XXXM,
            render: (text, record) => {
              return renderRedText(text, record.winterGoalValue, record.winterPlanValue);
            },
          },
        ],
      },
      {
        title: `${thisYear}春季`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'springPlanValue',
            width: ColWidth.WIDTH_XXXM,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'springGoalValue',
            width: ColWidth.WIDTH_XXXM,
            render: (text, record) => {
              return renderRedText(text, record.springGoalValue, record.springPlanValue);
            },
          },
        ],
      },
      {
        title: `${thisYear}暑假`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'summerPlanValue',
            width: ColWidth.WIDTH_XXXM,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'summerGoalValue',
            width: ColWidth.WIDTH_XXXM,
            render: (text, record) => {
              return renderRedText(text, record.summerGoalValue, record.summerPlanValue);
            },
          },
        ],
      },
    ];
  }

  /**
   * 月份表头
   * @param year
   */
  formatMonthColumn(year = '') {
    const { btnAuth_edit } = this.props;
    const { timeStamp } = this.state;
    const { lastYear, thisYear } = formatYearText(year);

    let column = [
      {
        title: `${lastYear}10月`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'octoberPlanValue',
            width: ColWidth.WIDTH_XXS,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'octoberGoalValue',
            width: ColWidth.WIDTH_XXS,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              const compareTime = parseInt(lastYear) + '-10-01';
              const isHistory = isHistoryMonth(compareTime, timeStamp);

              return !isHistory;
            },
            style: (record) => {
              return formatRedStyle(record.octoberGoalValue, record.octoberPlanValue);
            },
          },
        ],
      },
      {
        title: `${lastYear}11月`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'novemberPlanValue',
            width: ColWidth.WIDTH_XXS,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'novemberGoalValue',
            width: ColWidth.WIDTH_XXS,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              const compareTime = parseInt(lastYear) + '-11-01';
              const isHistory = isHistoryMonth(compareTime, timeStamp);

              return !isHistory;
            },
            style: (record) => {
              return formatRedStyle(record.novemberGoalValue, record.novemberPlanValue);
            },
          },
        ],
      },
      {
        title: `${lastYear}12月`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'decemberPlanValue',
            width: ColWidth.WIDTH_XXS,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'decemberGoalValue',
            width: ColWidth.WIDTH_XXS,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              const compareTime = parseInt(lastYear) + '-12-01';
              const isHistory = isHistoryMonth(compareTime, timeStamp);

              return !isHistory;
            },
            style: (record) => {
              return formatRedStyle(record.decemberGoalValue, record.decemberPlanValue);
            },
          },
        ],
      },
      {
        title: `${thisYear}1月`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'januaryPlanValue',
            width: ColWidth.WIDTH_XXS,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'januaryGoalValue',
            width: ColWidth.WIDTH_XXS,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              const compareTime = parseInt(thisYear) + '-01-01';
              const isHistory = isHistoryMonth(compareTime, timeStamp);

              return !isHistory;
            },
            style: (record) => {
              return formatRedStyle(record.januaryGoalValue, record.januaryPlanValue);
            },
          },
        ],
      },
      {
        title: `${thisYear}2月`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'februaryPlanValue',
            width: ColWidth.WIDTH_XXS,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'februaryGoalValue',
            width: ColWidth.WIDTH_XXS,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              const compareTime = parseInt(thisYear) + '-02-01';
              const isHistory = isHistoryMonth(compareTime, timeStamp);

              return !isHistory;
            },
            style: (record) => {
              return formatRedStyle(record.februaryGoalValue, record.februaryPlanValue);
            },
          },
        ],
      },
      {
        title: `${thisYear}3月`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'marchPlanValue',
            width: ColWidth.WIDTH_XXS,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'marchGoalValue',
            width: ColWidth.WIDTH_XXS,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              const compareTime = parseInt(thisYear) + '-03-01';
              const isHistory = isHistoryMonth(compareTime, timeStamp);

              return !isHistory;
            },
            style: (record) => {
              return formatRedStyle(record.marchGoalValue, record.marchPlanValue);
            },
          },
        ],
      },
      {
        title: `${thisYear}4月`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'aprilPlanValue',
            width: ColWidth.WIDTH_XXS,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'aprilGoalValue',
            width: ColWidth.WIDTH_XXS,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              const compareTime = parseInt(thisYear) + '-04-01';
              const isHistory = isHistoryMonth(compareTime, timeStamp);

              return !isHistory;
            },
            style: (record) => {
              return formatRedStyle(record.aprilGoalValue, record.aprilPlanValue);
            },
          },
        ],
      },
      {
        title: `${thisYear}5月`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'mayPlanValue',
            width: ColWidth.WIDTH_XXS,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'mayGoalValue',
            width: ColWidth.WIDTH_XXS,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              const compareTime = parseInt(thisYear) + '-05-01';
              const isHistory = isHistoryMonth(compareTime, timeStamp);

              return !isHistory;
            },
            style: (record) => {
              return formatRedStyle(record.mayGoalValue, record.mayPlanValue);
            },
          },
        ],
      },
      {
        title: `${thisYear}6月`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'junePlanValue',
            width: ColWidth.WIDTH_XXS,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'juneGoalValue',
            width: ColWidth.WIDTH_XXS,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              const compareTime = parseInt(thisYear) + '-06-01';
              const isHistory = isHistoryMonth(compareTime, timeStamp);

              return !isHistory;
            },
            style: (record) => {
              return formatRedStyle(record.juneGoalValue, record.junePlanValue);
            },
          },
        ],
      },
      {
        title: `${thisYear}7月`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'julyPlanValue',
            width: ColWidth.WIDTH_XXS,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'julyGoalValue',
            width: ColWidth.WIDTH_XXS,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              const compareTime = parseInt(thisYear) + '-07-01';
              const isHistory = isHistoryMonth(compareTime, timeStamp);

              return !isHistory;
            },
            style: (record) => {
              return formatRedStyle(record.julyGoalValue, record.julyPlanValue);
            },
          },
        ],
      },
      {
        title: `${thisYear}8月`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'augustPlanValue',
            width: ColWidth.WIDTH_XXS,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'augustGoalValue',
            width: ColWidth.WIDTH_XXS,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              const compareTime = parseInt(thisYear) + '-08-01';
              const isHistory = isHistoryMonth(compareTime, timeStamp);

              return !isHistory;
            },
            style: (record) => {
              return formatRedStyle(record.augustGoalValue, record.augustPlanValue);
            },
          },
        ],
      },
    ];

    if (year) {
      if (Number(year) > 2020) {
        column.unshift({
          title: `${lastYear}9月`,
          children: [
            {
              title: '规划目标',
              dataIndex: 'septemberPlanValue',
              width: ColWidth.WIDTH_XXS,
              render: (text, record) => {
                return renderNullPlanValue(text);
              },
            },
            {
              title: '确认目标',
              dataIndex: 'septemberGoalValue',
              width: ColWidth.WIDTH_XXS,
              type: 'input',
              inputType: 'number',
              canBeEdit: (record) => {
                if (btnAuth_edit) {
                  return true;
                }

                const compareTime = parseInt(lastYear) + '-09-01';
                const isHistory = isHistoryMonth(compareTime, timeStamp);

                return !isHistory;
              },
              style: (record) => {
                return formatRedStyle(record.septemberGoalValue, record.septemberPlanValue);
              },
            },
          ],
        });
      } else {
        column.push({
          title: `${thisYear}9月`,
          children: [
            {
              title: '规划目标',
              dataIndex: 'septemberPlanValue',
              width: ColWidth.WIDTH_XXS,
              render: (text, record) => {
                return renderNullPlanValue(text);
              },
            },
            {
              title: '确认目标',
              dataIndex: 'septemberGoalValue',
              width: ColWidth.WIDTH_XXS,
              type: 'input',
              inputType: 'number',
              canBeEdit: (record) => {
                if (btnAuth_edit) {
                  return true;
                }

                const compareTime = parseInt(thisYear) + '-09-01';
                const isHistory = isHistoryMonth(compareTime, timeStamp);

                return !isHistory;
              },
              style: (record) => {
                return formatRedStyle(record.septemberGoalValue, record.septemberPlanValue);
              },
            },
          ],
        });
      }
    } else {
      column.unshift({
        title: `9月`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'septemberPlanValue',
            width: ColWidth.WIDTH_XXS,
            render: (text, record) => {
              return renderNullPlanValue(text);
            },
          },
          {
            title: '确认目标',
            dataIndex: 'septemberGoalValue',
            width: ColWidth.WIDTH_XXS,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              const compareTime = parseInt(year) + '-09-01';
              const isHistory = isHistoryMonth(compareTime, timeStamp);

              return !isHistory;
            },
            style: (record) => {
              return formatRedStyle(record.septemberGoalValue, record.septemberPlanValue);
            },
          },
        ],
      });
    }

    return column;
  }

  /**
   * 右边汇总
   */
  formatRightColumn() {
    return [
      {
        title: '年度总目标',
        children: [
          {
            title: '规划目标',
            dataIndex: 'yearPlanValue',
            width: ColWidth.WIDTH_XXS,
          },
          {
            title: '确认目标',
            dataIndex: 'yearGoalValue',
            width: ColWidth.WIDTH_XXS,
            render: (text, record) => {
              return renderRedText(text, record.yearGoalValue, record.yearPlanValue);
            },
          },
        ],
      },
    ];
  }

  /**
   * 保存编辑
   */
  onSaveRow(rowData, callback) {
    const { btnAuth_edit } = this.props;
    const param = formatRecordToApiParam(rowData, 'PROMISE_CLASS_STUDENT_NUMBER', btnAuth_edit);

    this.setState({ listLoading: true });

    updateGoalValue(param)
      .then((response) => {
        showMessage('保存成功！', 'success');
        callback.call();
        this.setState({ listLoading: false });
        this.loadList();
      })
      .catch((e) => {
        showMessage('保存失败，请重试！');
        this.setState({ listLoading: false });
      });
  }

  render() {
    const { btnAuth_plan_import, btnAuth_plan_import_limit, btnAuth_goal_import, btnAuth_goal_import_limit } =
      this.props;
    const {
      tableKey,
      list,
      listLoading,
      pageNo,
      pageSize,
      totalRecords,
      countRow,
      showImportModal,
      importType,
      canImportHistory,
    } = this.state;
    const columns = this.formatColumn();

    return (
      <TheWrapper
        footer={
          <FooterTips title="温馨提示">
            <p>
              新财年全年中考精英班招生人数<span>规划目标</span>
              由集团统一导入，分公司无需再填写，导入后【作为全年目标考核】， 每月可微调<span>确认目标</span>
              ，全年目标（即规划目标）不可下调。相关报表会以<span>确认目标</span>算完成率
            </p>
            <h3>确认目标支持批量导入和单个录入：</h3>
            <p>
              1、个别目标修改：筛选项选择“学习中心，月份”，进行查询，点击<span>【编辑】</span>进行修改。
            </p>
            <img src={YearExamGoalImg01} alt="提示图片01" />
            <p>
              2、批量目标修改：点击“确认目标导入”按钮，然后下载导入模板，在模板填写各学习中心的全年中考精英班招生人数目标，
              然后选择导入的年份并上传文件，点击导入，导入成功后即可完成目标的批量修改。
            </p>
            <img src={YearExamGoalImg02} alt="提示图片02" />
          </FooterTips>
        }
      >
        <section className="tabPanelWrap">
          <CommonSearch
            planImportDisabled={!btnAuth_plan_import}
            planImportLimitDisabled={!btnAuth_plan_import_limit}
            goalImportDisabled={!btnAuth_goal_import}
            goalImportLimitDisabled={!btnAuth_goal_import_limit}
            onSearch={this.onSearch}
            onRefs={(ref) => {
              this.$ref.searchRef = ref;
            }}
            onShowImportModal={(type, history) => {
              this.setState({
                showImportModal: true,
                importType: type,
                canImportHistory: history,
              });
            }}
            {...this.props}
          />

          <TableForm
            key={tableKey}
            loading={listLoading}
            list={list}
            columns={columns}
            pagination={{
              current: pageNo,
              pageSize: pageSize,
              total: totalRecords,
            }}
            onPageChange={({ current, pageSize }) => this.loadList({ pageNo: current, pageSize })}
            onSaveRow={(rowData, callback) => {
              this.onSaveRow(rowData, callback);
            }}
            footer={() => {
              return renderTableFooter(columns, countRow);
            }}
          />

          {showImportModal && (
            <ImportTemplateModal
              pageType="YearExamGoal"
              pageImportType={importType}
              canImportHistory={canImportHistory}
              onClose={() => this.setState({ showImportModal: false })}
            />
          )}
        </section>
      </TheWrapper>
    );
  }
}

function mapStateToProps(state) {
  const { btnAuths } = state.account;

  const btnAuth_edit = btnAuths.indexOf('OPERATION_YEARGOAL_EDIT_FREE') > -1;
  const btnAuth_edit_limit = btnAuths.indexOf('OPERATION_YEARGOAL_EDIT') > -1;
  const btnAuth_plan_import = btnAuths.indexOf('OPERATION_YEAR_PLANIMPORT_FREE') > -1;
  const btnAuth_plan_import_limit = btnAuths.indexOf('OPERATION_YEARGOAL_PLANIMPORT') > -1;
  const btnAuth_goal_import = btnAuths.indexOf('OPERATION_YEAR_GOALIMPORT_FREE') > -1;
  const btnAuth_goal_import_limit = btnAuths.indexOf('OPERATION_YEARGOAL_GOALIMPORT') > -1;

  return {
    btnAuth_edit,
    btnAuth_edit_limit,
    btnAuth_plan_import,
    btnAuth_plan_import_limit,
    btnAuth_goal_import,
    btnAuth_goal_import_limit,
  };
}

export default connect(mapStateToProps)(YearExamGoal);
