/**
 * 季度月份区间
 * 年份 @2019
 * 秋季：2018.10~2018.12
 * 寒季：2019.01~2019.03
 * 秋季：2019.04~2019.06
 * 秋季：2019.07~2019.09
 */
export const QUARTER_MONTH = {
  AUTUMN: {
    min: 10,
    max: 12,
  },
  WINTER: {
    min: 1,
    max: 3,
  },
  SPRING: {
    min: 4,
    max: 6,
  },
  SUMMER: {
    min: 7,
    max: 9,
  },
};

/**
 * 获取季度区间
 * @param year 转换后的产品年份
 * @param quarterType
 * @return {{startTime: number, endTime: number}}
 */
export function formatQuarterMonth(year, quarterType) {
  const min = QUARTER_MONTH[quarterType].min;
  const max = QUARTER_MONTH[quarterType].max;

  return {
    startTime: new Date(`${year}-${min}-01`).getTime(),
    endTime: new Date(`${year}-${max}-01`).getTime(),
  };
}

/**
 * 保存每一行的确认目标, 将每行数据转成API参数
 * @param record 每行数据
 * @param goalType 目标类型 {小班科次: MINI_CLASS_SUBJECT_COUNT, 预收: CASH_FLOW, 小时消耗: COURSE_HOUR_CONSUME, 精英班招生人数: PROMISE_CLASS_STUDENT_NUMBER}
 * @param canEditHistoryData 是否允许编辑历史数据（根据用户权限判断）
 */
export function formatRecordToApiParam(record = {}, goalType = '', canEditHistoryData = false) {
  let param = {
    goalType: goalType,
    yearId: record.yearId,
    organizationId: record.campusId, // 学习中心ID
    classTypeId: record.classTypeId,
    gradeId: record.gradeId,
    subjectId: record.subjectId,
    productTypeId: record.productTypeId,
    cashFlowType: record.cashFlowType,
    canEditHistoryData: canEditHistoryData,
  };

  const keys = [
    'spring',
    'summer',
    'autumn',
    'winter',
    'january',
    'february',
    'march',
    'april',
    'may',
    'june',
    'july',
    'august',
    'september',
    'october',
    'november',
    'december',
  ];

  keys.forEach((k) => {
    const key = k + 'GoalValue';
    param[k] = record[key];
  });

  return param;
}

/**
 * 保留两位小数, 将浮点数四舍五入，取小数点后2位
 * @param x
 * @returns {number}
 */
export function toDecimal(x) {
  let f = parseFloat(x);

  if (isNaN(f)) {
    return 0;
  }

  f = Math.round(x * 100) / 100;

  return f;
}

/**
 * 获取当前年份的上一年
 * @param thisYear
 */
export function formatLastYear(thisYear = '') {
  if (!thisYear) {
    return '';
  }

  return Number(thisYear) - 1;
}

/**
 * 判断时间是否为历史时间，(待编辑的月份 ＜ 服务器时间，则认定为历史月份)
 * @param compareTime 需要比较的时间
 * @param serverTime 服务器时间戳
 * @return {boolean}
 */
export function isHistoryTime(compareTime, serverTime) {
  if (!compareTime || !serverTime) {
    return true;
  }

  return new Date(compareTime).getTime() < new Date(serverTime).getTime();
}

/**
 * 判断月份是否为历史月份
 * @Attention 月份`2019-01-01`与`2019-1-01`的时间戳结果不同
 * @param compareTime 时间或时间戳 `2019-01-01`|number
 * @param serverTime
 * @return {boolean}
 */
export function isHistoryMonth(compareTime, serverTime) {
  if (!compareTime || !serverTime) {
    return true;
  }

  if (Object.prototype.toString.call(compareTime) !== '[object Number]') {
    compareTime = new Date(compareTime).getTime();
  }

  const date = new Date(serverTime);
  const yy = date.getFullYear();
  const mm = date.getMonth() + 1;
  const serverMonth = `${yy}-${mm < 10 ? '0' + mm : mm}-01`;
  const trueTime = new Date(serverMonth).getTime();

  return compareTime < trueTime;
}

/**
 * 判断时间是否为历史季度
 * @param serverTime 服务器时间戳
 * @param year 年份(注意年份是转换后的产品年份，所以月份区间直接取QUARTER_MONTH的月份区间即可)
 * @param quarterType 取哪个季度 AUTUMN|WINTER|SPRING|SUMMER
 */
export function isHistoryQuarter(serverTime, year, quarterType) {
  if (!serverTime || !year) {
    return true;
  }

  const { endTime } = formatQuarterMonth(year, quarterType);

  const date = new Date(serverTime);
  const yy = date.getFullYear();
  const mm = date.getMonth() + 1;
  const trueDate = `${yy}-${mm}-01`;
  const trueTime = new Date(trueDate).getTime();

  return trueTime > endTime;
}

/**
 * 格式化年份
 * @param year
 * @return {*}
 */
export function formatYearText(year = '') {
  const lastYear = formatLastYear(year);

  return {
    lastYear: lastYear ? `${lastYear}年` : lastYear,
    thisYear: year ? `${year}年` : year,
  };
}

/**
 * 标红样式
 * @param goalValue 确认目标
 * @param planValue 规划目标
 */
export function formatRedStyle(goalValue, planValue) {
  if (Number(goalValue || 0) < Number(planValue || 0)) {
    return { color: 'red' };
  }

  return {};
}

/**
 * 页面汇总并将汇总数据保留两位小数
 * @param data 表格数据
 * @param countRow 汇总行
 * @param exceptKey 不需要保留小数的字段
 */
export function formatCountRow(data, countRow, exceptKey = 'operation') {
  data.forEach((row, index) => {
    row.key = index.toString();

    Object.keys(row).forEach((key) => {
      let lowerCaseKey = key.toLowerCase();

      if (lowerCaseKey.indexOf('value') > -1 && lowerCaseKey.indexOf('has') < 0) {
        if (countRow[key] !== undefined) {
          countRow[key] += Number(row[key] || 0);
        } else {
          countRow[key] = Number(row[key] || 0);
        }
      }
    });
  });

  Object.keys(countRow).forEach((key) => {
    if (key !== exceptKey) {
      countRow[key] = toDecimal(countRow[key]);
    }
  });

  return countRow;
}

/**
 * 渲染红色字体
 * @param text
 * @param goalValue 确认目标
 * @param planValue 规划目标
 */
export function renderRedText(text, goalValue, planValue) {
  let style = formatRedStyle(goalValue, planValue);
  return <span style={style}>{text}</span>;
}

/**
 * 渲染规划目标
 * @param text
 * @param noDataText
 */
export function renderNullPlanValue(text, noDataText = '无') {
  let str = !!text ? text : noDataText;
  return <span>{str}</span>;
}

/**
 * 渲染分公司名称
 * @param text 分公司名称
 * @param hasVal 是否已上传
 */
export function renderBranchText(text, hasVal) {
  return !hasVal ? (
    <div>
      <div>{text}</div>
      <div style={{ color: 'red' }}>(待上传)</div>
    </div>
  ) : (
    text
  );
}

/**
 * 将数组拼接成字符串
 * @param arr
 * @param splitStr
 */
export function transformArrayToString(arr = [], splitStr = ',') {
  if (!arr || arr.length <= 0) {
    return '';
  }

  return arr.join(splitStr);
}

/**
 * 计算数组里面有多少个true元素
 * @param boolArr
 * @return {number}
 */
export function computeTrueValueNumber(boolArr = []) {
  if (!boolArr) {
    return 0;
  }

  let num = 0;

  boolArr.forEach((value) => {
    if (value === true) {
      num++;
    }
  });

  return num;
}
