/**
 * 内容编辑主页面
 * @property {string} mode - 模式，'browser'（浏览模式）|'workbench'（工作台模式）
 * @property {string} userOrgId - 当前登录用户所属机构ID
 */
import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import DropdownMenu from 'COMPONENT/DropdownMenu';
import Button from 'COMPONENT/Button';
import styles from './styles.less';
import { getDictTextByVal } from 'UTIL/dict';
import { getAllBrenchAndGroup, getCmsMenuList } from 'SERVICE/ipadBackEnd';
import styleVars from 'COMPONENT/theme/variables';
import MenuTree from './MenuTree';
import PropTypes from 'prop-types';
import {
  TOP_ORG_ID,
  TOP_ORG_NAME,
  publishStatusDict,
  belongTypeDict,
  recommendDict,
  BELONG_TYPE_BRANCH,
  PERSONALINFO,
  MODE_WORKBENCH,
  BELONG_TYPE_PERSONAL,
  BELONG_TYPE_CAMPUS,
} from '../common';
import MenuContent from './MenuContent';
import getWrappedInstance from 'UTIL/getWrappedInstance';

class BrowserMainPage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 页面加载状态
      orgDict: [{ val: TOP_ORG_ID, text: TOP_ORG_NAME }], // {object[]} 字典，组织机构
      belongTypeDict, // {[object]} 字典 所属类型
      publishStatusDict, // {[object]} 字典 发布状态
      recommendDict, // {[object]} 字典 首页推荐
      orgId: props.userOrgId, // {string} 当前组织机构ID
      menuTree: [], // {object[]} 左侧菜单数据
      allMenuArr: [], // {object[]} 左侧菜单原始数据
      menuId: null, // {string} 当前菜单ID
      indexMenuId: null, // {string} 首页菜单ID
      defaultMenuId: null, // {string} 默认选中的菜单ID
      contentData: null, // {any} 右侧区域内容当前标签页下的数据
    };
  }

  /**
   * 嵌套实在太深了，加个context上下文
   * @returns {*}
   */
  getChildContext = () => {
    return {
      getActiveMenu: () => {
        return this.state.allMenuArr.find((menu) => menu.id === this.state.menuId);
      },
      //判断当前选中是否个性化资料模块
      isNowPersonal: this.isNowPersonal,
    };
  };

  isNowPersonal = () => {
    const { allMenuArr, menuId } = this.state;
    const nowMenu = allMenuArr.find((menu) => menu.id === menuId);
    if (nowMenu && nowMenu.name === PERSONALINFO) return true;
    const nowParent = allMenuArr.find((menu) => menu.id === nowMenu.parentId);
    if (nowParent && nowParent.name === PERSONALINFO) return true;
  };

  componentDidMount() {
    this.initContent();
  }

  initContent = () => {
    this.setState({ loading: true });
    const { orgId } = this.state;
    let calls = [this.updatePage(orgId)];
    if (this.isTopUser()) {
      calls.push(this.loadOrgDict());
    }
    Promise.all(calls).then(([a, orgDict]) => {
      let changed = { loading: false };
      if (orgDict) {
        changed = { ...changed, orgDict };
      }
      this.setState(changed);
    });
  };

  //主组织架构为集团的账号时
  isTopUser = () => {
    return this.props.userOrgId === TOP_ORG_ID;
  };

  //架构切换至星火集团时
  isTopOrg = () => {
    return this.state.orgId === TOP_ORG_ID;
  };

  /**
   * 目前只有编辑保存时用到
   * @returns {*}
   */
  getUserOrgType = () => {
    const orgType = this.props.userOrgType;
    //个性化资料下为个人类型
    if (this.isNowPersonal()) {
      return BELONG_TYPE_PERSONAL;
    }
    //主架构为学习中心，类型为分公司
    if (orgType === BELONG_TYPE_CAMPUS) {
      return BELONG_TYPE_BRANCH;
    }
    return orgType;
  };

  updatePage = async (orgId, menuId) => {
    const { menuTree, indexMenuId, defaultMenuId, allMenuArr } = await this.loadMenuTree(orgId);
    const nextMenuId = menuId || defaultMenuId;
    //setMenuId同时也要setmenuTree
    await new Promise((resolve, reject) => {
      this.setState({ menuId: nextMenuId, menuTree, indexMenuId, defaultMenuId, allMenuArr }, resolve);
    });
    const contentData = await this.menuContentRef.loadContentData(nextMenuId);
    this.setState({ contentData });
    return true;
  };

  reloadPage = () => {
    this.updatePage(this.state.orgId, this.state.menuId);
  };

  loadOrgDict = async () => {
    const data = await getAllBrenchAndGroup({}, true);
    //let dict = [{ val: TOP_ORG_ID, text: TOP_ORG_NAME }]
    let dict = [];
    data.forEach((d) => {
      dict.push({ val: d.id, text: d.name });
    });
    return dict;
  };

  loadMenuTree = async (orgId) => {
    const { mode, roleCode, btnAuths } = this.props;
    const apiData = await getCmsMenuList({ useType: mode, orgId }, true);
    let indexMenuId;
    let defaultMenuId;
    const dataLevel1 = apiData.filter((da) => da.level === '1');
    let data = dataLevel1.map((d) => {
      const children = apiData
        .filter((ch) => ch.parentId === d.id)
        .map((c) => {
          return { id: c.id, title: c.name, visible: c.status === '1', showContentNum: c.showContentNum };
        });

      return { id: d.id, title: d.name, children, visible: d.status === '1', showContentNum: d.showContentNum };
    });

    /**
     * 工作台模式内容模块需要在角色管理添加'查看内部资料'权限，否则只有个性化资料
     */
    if (mode === MODE_WORKBENCH && !btnAuths.includes('CHECK_INTERNAL_INFO')) {
      data = data.filter((d) => d.title === PERSONALINFO);
    }

    /**
     * 默认展开第一个一级菜单
     */
    indexMenuId = data.find((d) => d.parentId !== -1) || {};
    defaultMenuId = data.find((ele) => !!ele.visible || (!!!ele.visible && ele.children.length > 0));
    defaultMenuId = !!defaultMenuId.visible ? defaultMenuId.id : defaultMenuId.children.find((ele) => !!ele.visible).id;

    return { menuTree: data, indexMenuId: indexMenuId.id, defaultMenuId, allMenuArr: apiData };
  };

  onOrgChange = (orgId) => {
    this.setState({ loading: true, orgId, contentData: null }, () => {
      this.updatePage(orgId).then(() => {
        this.setState({ loading: false });
      });
    });
  };

  onMenuChange = (menuId) => {
    const { orgId } = this.state;
    this.setState({ loading: true, menuId, contentData: null }, () => {
      Promise.all([this.menuContentRef.loadContentData(menuId)]).then(([contentData]) => {
        this.setState({ loading: false, contentData });
      });
    });
  };

  getOrgSelect = () => {
    if (!this.isTopUser()) {
      return null;
    }
    const { orgDict, orgId } = this.state;
    const menus = orgDict.map((d) => {
      return { text: d.text, onClick: () => this.onOrgChange(d.val) };
    });
    return (
      <DropdownMenu
        menus={menus}
        color={styleVars.fontColor}
        hasArrow={true}
        text={orgId ? getDictTextByVal(orgDict, orgId) : TOP_ORG_NAME}
        dropdownClassName={styles.orgDropdown}
        placement="bottomCenter"
      />
    );
  };

  startLoading = () => {
    if (!this.state.loading) {
      this.setState({ loading: true });
    }
  };

  endLoading = () => {
    if (this.state.loading) {
      this.setState({ loading: false });
    }
  };

  openTagManagePage = () => {
    const { dispatch, mode, route } = this.props;
    const { orgId } = this.state;
    dispatch({
      type: 'main/openPage',
      payload: { path: `/${route.path}/TagManage`, params: { orgId, mode }, refresh: true },
    });
  };

  render() {
    const { route, mode } = this.props;
    const {
      loading,
      belongTypeDict,
      publishStatusDict,
      recommendDict,
      menuTree,
      allMenuArr,
      menuId,
      indexMenuId,
      orgId,
      orgDict,
      contentData,
    } = this.state;

    return (
      <Page
        loading={loading}
        route={route}
        headerToolBar={
          this.isTopOrg() ? (
            <Button onClick={this.openTagManagePage} style={{ marginRight: 20 }}>
              标签管理
            </Button>
          ) : null
        }
      >
        <div className={styles.orgSelectWrap}>{this.getOrgSelect()}</div>
        <div className={styles.wrap}>
          <div className={styles.left}>
            <MenuTree
              data={menuTree}
              selected={menuId + ''}
              onChange={(menuIdStr) => this.onMenuChange(parseInt(menuIdStr))}
              onLoadingStart={this.startLoading}
              onLoadingEnd={this.endLoading}
            />
          </div>
          <div className={styles.right}>
            <MenuContent
              belongTypeDict={belongTypeDict}
              publishStatusDict={publishStatusDict}
              recommendDict={recommendDict}
              contentData={contentData}
              onLoadingStart={this.startLoading}
              onLoadingEnd={this.endLoading}
              orgDict={orgDict}
              menuTree={menuTree}
              isTopUser={this.isTopUser()}
              isTopOrg={this.isTopOrg()}
              orgId={orgId}
              menuId={menuId}
              getUserOrgType={this.getUserOrgType}
              menuItem={allMenuArr.find((menu) => menu.id === menuId)}
              isIndex={menuId !== null && menuId === indexMenuId}
              ref={(c) => (this.menuContentRef = getWrappedInstance(c))}
              mode={mode}
              route={route}
              onReloadPage={this.reloadPage}
            />
          </div>
        </div>
      </Page>
    );
  }
}

BrowserMainPage.childContextTypes = {
  getActiveMenu: PropTypes.func,
  isNowPersonal: PropTypes.func,
};

const mapStateToProps = (state) => {
  return {
    userOrgId: state.account.user.brenchId,
    roleCode: state.account.user.roleCode,
    userOrgType: state.account.user.orgType,
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps)(BrowserMainPage);
