import React from 'react';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import DatePicker from 'COMPONENT/DatePicker';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import moment from 'moment';
import Loaders from './loaders';

const cmsOperateTypeDict = [
  { val: 'PUBLISH_NOW', text: '立即发布' },
  { val: 'PUBLISH_PRE', text: '预约发布' },
  { val: 'PUBLISH_CANCEL', text: '取消发布' },
];

class LogRecord extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      startDate: moment().day(1).format('YYYY-MM-DD'), // 开始日期
      endDate: moment().day(7).format('YYYY-MM-DD'), // 结束日期
      createUserName: null, // 操作人
      cmsOperateType: null, // 操作类型
      cmsContentTitle: null, // 操作对象

      listData: [], // 列表数据
      pagination: {
        current: 1,
        pageSize: 10,
        total: 0,
      }, // 列表分页
    };

    this.columns = [
      { title: '操作对象', dataIndex: 'cmsContentTitle' },
      { title: '操作类型', dataIndex: 'cmsOperateTypeName' },
      { title: '旧发布状态', dataIndex: 'cmsContentStatusOldName' },
      { title: '新发布状态', dataIndex: 'cmsContentStatusNewName' },
      { title: '操作人', dataIndex: 'createUserName' },
      { title: '操作时间', dataIndex: 'createTime' },
    ];
  }

  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate = (startDate) => {
    const { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  /**
   * 获取列表数据
   */
  loadList = async (pageNo = 1, pageSize = 10) => {
    const { startDate, endDate, createUserName, cmsOperateType, cmsContentTitle } = this.state;
    const { datas: listData, rowCount: total } = await Loaders.getPageCmsContentPublishLogList({
      startDate,
      endDate,
      createUserName,
      cmsOperateType,
      cmsContentTitle,
      pageNo,
      pageSize,
    });
    this.setState({ listData, pagination: Object.assign({}, this.state.pagination, { total }) });
  };

  /**
   * 搜索
   */
  onSearch = async () => {
    this.setState({ loading: true, pagination: { current: 1, pageSize: 10, total: 0 } });
    await this.loadList().then(null, () => {
      this.setState({ loading: false });
    });
    this.setState({ loading: false });
  };

  /**
   * 重置
   */
  onReset = () => {
    this.setState({
      startDate: moment().day(1).format('YYYY-MM-DD'), // 开始日期
      endDate: moment().day(7).format('YYYY-MM-DD'), // 结束日期
      createUserName: null, // 操作人
      cmsOperateType: null, // 操作类型
      cmsContentTitle: null, // 操作对象
    });
  };

  render() {
    const { loading, startDate, endDate, createUserName, cmsOperateType, cmsContentTitle, pagination, listData } =
      this.state;

    return (
      <Page route={this.props.route} loading={loading}>
        <SearchBar onSearch={this.onSearch} onReset={this.onReset}>
          <DatePicker
            placeholder="开始日期"
            value={startDate}
            onChange={(val) => this.setState({ startDate: val })}
            disabledDate={this.getDisabledStartDate}
          />
          <DatePicker
            placeholder="结束日期"
            value={endDate}
            onChange={(val) => this.setState({ endDate: val })}
            disabledDate={this.getDisabledEndDate}
          />
          <Input
            placeholder="操作人"
            value={createUserName}
            onChange={(e) => {
              const createUserName = e.currentTarget.value;
              this.setState({ createUserName });
            }}
          />
          <Select
            placeholder="操作类型"
            value={cmsOperateType}
            dict={cmsOperateTypeDict}
            onChange={(val) => this.setState({ cmsOperateType: val })}
          />
          <Input
            placeholder="操作对象"
            value={cmsContentTitle}
            onChange={(e) => {
              const cmsContentTitle = e.currentTarget.value;
              this.setState({ cmsContentTitle });
            }}
          />
        </SearchBar>
        <List
          columns={this.columns}
          pagination={pagination}
          dataSource={listData}
          onChange={(pagination) => {
            const { pageSize, current: pageNo, current } = pagination;
            this.setState({ pagination: Object.assign({}, this.state.pagination, { pageSize, current }) }, async () => {
              this.setState({ loading: true });
              await this.loadList(pageNo, pageSize).then(null, () => {
                this.setState({ loading: false });
              });
              this.setState({ loading: false });
            });
          }}
        />
      </Page>
    );
  }
}

export default LogRecord;
