import React from 'react';
import ReactDOM from 'react-dom';
import Tag from 'COMPONENT/Tag';
import TextLink from 'COMPONENT/TextLink';
import Icon from 'COMPONENT/Icon';
import IconButton from 'COMPONENT/IconButton';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import PopTip from 'COMPONENT/PopTip';
import styles from './styles.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Validator from 'COMPONENT/Validator';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import PopConfirm from 'COMPONENT/PopConfirm';

class TagCategory extends React.Component {
  constructor(props) {
    super(props);
    this.state = this.getInitStateFromProps(props);
    this.errors = [];
  }

  componentWillReceiveProps(nextProps) {
    if ((this.props.isEditing && !nextProps.isEditing) || !objectDeepEquals(this.props.data, nextProps.data)) {
      this.setState(this.getInitStateFromProps(nextProps));
    }
  }

  componentDidUpdate(prevProps, prevState) {
    if (!prevState.addEditing && this.state.addEditing) {
      ReactDOM.findDOMNode(this.addInputRef).querySelector('input').focus();
    }
  }

  getInitStateFromProps = (props) => {
    const { data } = props;
    return {
      titleValue: data.title, // {string} 标题文本框内容
      addValue: '', // {string} 添加标签文本框内容
      tags: data.tags, // {object[]}  标签集合
      addEditing: false, // {boolean} 添加按钮是否处于编辑状态
      tipsVisible: false, // {boolean} 保存提示显示状态
      checking: false, // {boolean} 开启校验
    };
  };

  save = () => {
    const { titleValue, tags, checking } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    if (Object.values(this.errors).find((v) => v)) {
      this.setState({ tipsVisible: true });
      return;
    }
    this.props.onSave({ title: titleValue, tags });
  };

  confirmAdd = () => {
    if (this.errors['add']) {
      return;
    }
    const { addValue, tags } = this.state;
    let changed = { addEditing: false };
    if (addValue) {
      changed = { ...changed, addValue: '', tags: [...tags, { text: addValue }] };
    }
    this.setState(changed);
  };

  deleteTag = (idx) => {
    this.setState({
      tags: this.state.tags.filter((t, i) => i !== idx),
    });
  };

  onAddInputKeyDown = (e) => {
    if (e.keyCode === 13) {
      this.confirmAdd();
    }
  };

  getAddBtn = () => {
    const { addValue, addEditing, checking } = this.state;
    if (addEditing) {
      return (
        <div style={{ width: 120, display: 'inline-block', margin: '0 20px 20px 0' }}>
          <Input
            value={addValue}
            onChange={(e) => this.setState({ addValue: e.target.value })}
            onKeyDown={this.onAddInputKeyDown}
            onBlur={this.confirmAdd}
            validator={{
              onErrorChange: (e) => this.onErrorChange('add', e),
              rules: [this.checkExistTag],
              keepHeight: true,
              silent: !checking,
            }}
            ref={(c) => (this.addInputRef = c)}
            maxLength={5}
            wordCount={true}
          />
        </div>
      );
    } else {
      return (
        <Button
          onClick={() => this.setState({ addEditing: true })}
          style={{ position: 'relative', top: -1, margin: '0 20px 20px 0' }}
        >
          <Icon type="plus" size={12} className={styles.addIcon} />
          <span className={styles.addText}>添加标签</span>
        </Button>
      );
    }
  };

  onErrorChange = (key, error) => {
    this.errors[key] = error;
  };

  checkExist = (value) => {
    const { existTitles } = this.props;
    if (existTitles.indexOf(value) >= 0) {
      return '分类已存在';
    }
    return null;
  };

  checkExistTag = (value) => {
    const { tags } = this.state;
    if (tags.some((t) => t.text === value)) {
      return '标签已存在';
    }
    return null;
  };

  onTipsVisibleChange = (visible) => {
    if (!visible) {
      this.setState({ tipsVisible: visible });
    }
  };

  checkTags = (tags) => {
    return null;
  };

  render() {
    const { isEditing, onEditStart, onDelete, onCancel, editable } = this.props;
    const { titleValue, tags, tipsVisible, checking } = this.state;
    return (
      <div className={styles.wrap}>
        <div className={styles.header}>
          <div className={styles.headerLeft}>
            {isEditing ? (
              <Input
                value={titleValue}
                onChange={(e) => this.setState({ titleValue: e.target.value })}
                style={{ width: 280 }}
                validator={{
                  onErrorChange: (e) => this.onErrorChange('title', e),
                  rules: ['required', this.checkExist],
                  silent: !checking,
                }}
                maxLength={6}
                wordCount={true}
              />
            ) : (
              <div className={styles.titleText}>{titleValue}</div>
            )}
          </div>
          {editable && (
            <div className={styles.headerRight}>
              {isEditing ? (
                <span>
                  <PopTip content="请正确填写信息" visible={tipsVisible} onVisibleChange={this.onTipsVisibleChange}>
                    <TextLink onClick={this.save}>保存</TextLink>
                  </PopTip>
                  <TextLink onClick={onCancel} style={{ marginLeft: 20 }}>
                    取消
                  </TextLink>
                </span>
              ) : (
                <span>
                  <IconButton type="edit_mc" onClick={onEditStart} style={{ marginRight: 20 }} />
                  <PopConfirm
                    content={`确定要删除此分类吗？`}
                    onOk={onDelete}
                    placement="topRight"
                    arrowPointAtCenter={true}
                  >
                    <IconButton type="delete_mc" />
                  </PopConfirm>
                </span>
              )}
            </div>
          )}
        </div>
        <div className={styles.tagsWrap}>
          <Validator
            rules={[this.checkTags]}
            onErrorChange={(e) => this.onErrorChange('tags', e)}
            silent={!checking}
            value={tags}
            keepHeight={true}
          >
            <div className={styles.tags}>
              {tags.map((t, i) => (
                <Tag key={t.text} closable={isEditing} onClose={isEditing ? () => this.deleteTag(i) : null}>
                  {t.text}
                </Tag>
              ))}
              {isEditing && tags.length < 20 && this.getAddBtn()}
            </div>
          </Validator>
        </div>
      </div>
    );
  }
}

export default TagCategory;
