import React from 'react';
import ReactDOM from 'react-dom';
import Page from 'COMPONENT/Page';
import Button from 'COMPONENT/Button';
import TagCategory from './TagCategory';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import styles from './styles.less';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import { editDataDict, getDataDictList } from 'SERVICE/system';
import { getCmsSignList, saveMultiCmsSign, deleteSignById } from 'SERVICE/ipadBackEnd';
import AddCategoryPopConfirm from './AddCategoryPopConfirm';
import { MODE_BROWSER } from 'PAGE/ipadBackEnd/articleEdit/common';

class TagManagePage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 页面加载状态
      data: [], // {object[]} 标签数据，
      /**
                  [
                    { id: '2', title: '老师分类',
                      tags: [
                        { id: '1', text: '普通' }, { id: '2', text: '高级' },
                      ],
                    },
                  ]
                  */
      editingCategoryId: null, // {string} 编辑状态的分类ID
    };
  }

  componentDidMount() {
    this.updataTags();
  }

  updataTags = () => {
    this.setState({ loading: true });
    Promise.all([this.loadCateList(), this.loadTags()]).then(([cateList, allTags]) => {
      const data = cateList.map((c) => {
        const { id, name } = c;
        return {
          id,
          title: name,
          tags: allTags
            .filter((t) => t.type === id)
            .map((ta) => {
              return { id: ta.id, text: ta.value };
            }),
        };
      });
      this.setState({ loading: false, data });
      this.cancelEdit();
    });
  };

  loadCateList = async () => {
    const apiData = await getDataDictList({ category: this.getCateApiType() });
    return apiData.rows;
  };

  loadTags = async () => {
    const { mode } = this.props;
    const apiParams = {
      useType: mode,
    };
    const apiData = await getCmsSignList(apiParams);
    return apiData;
  };

  getCateApiType = () => {
    const { mode } = this.props;
    return mode === MODE_BROWSER ? 'SIGN_TYPE' : 'SIGN_TYPE_WORK';
  };

  save = ({ id, data }) => {
    const { mode } = this.props;
    const { title, tags } = data;
    const signVos = tags.map((t) => {
      let params = {
        type: id,
        value: t.text,
        useType: mode,
      };
      if (t.id) {
        params.id = t.id;
      }
      return params;
    });
    this.setState({ loading: true });
    saveMultiCmsSign([{ type: id, signVos }], true).then(
      () => {
        const titleParams = {
          id,
          name: title,
          category: this.getCateApiType(),
          state: 0,
        };
        editDataDict(titleParams, true).then(
          () => {
            showMessage('保存成功', 'success');
            this.updataTags();
            this.setState({ loading: false });
          },
          (e) => {
            showMessage(e.text || '保存分类名称失败');
            this.updataTags();
            this.setState({ loading: false });
          },
        );
      },
      (e) => {
        showMessage(e.text || '保存标签失败');
        this.updataTags();
        this.setState({ loading: false });
      },
    );
  };

  cancelEdit = () => {
    this.setState({ editingCategoryId: null });
  };

  deleteCategory = (id) => {
    this.setState({ loading: true });
    saveMultiCmsSign([{ type: id, signVos: [] }], true).then(
      () => {
        deleteSignById({ id }, true).then(
          () => {
            showMessage('删除成功', 'success');
            this.updataTags();
          },
          (e) => {
            showMessage(e.text || '删除失败');
            this.setState({ loading: false });
            this.updataTags();
          },
        );
      },
      (e) => {
        showMessage(e.text || '删除失败');
        this.setState({ loading: false });
      },
    );
  };

  addCategory = (title) => {
    //name=%E6%A0%87%E7%AD%BE%E7%B1%BB%E5%9E%8B2&state=0&dictOrder=2&id=DAT000000000471&createTime=2017-12-06+18%3A04%3A16&createUserId=USE0000018100&category=SIGN_TYPE
    const params = {
      name: title,
      category: this.getCateApiType(),
      state: 0,
    };
    this.setState({ loading: true });
    editDataDict(params, true).then(
      () => {
        showMessage('添加成功', 'success');
        this.updataTags();
        this.setState({ loading: false });
      },
      (e) => {
        showMessage(e.text || '添加失败');
        this.setState({ loading: false });
      },
    );
  };

  render() {
    const { route } = this.props;
    const { loading, data, editingCategoryId } = this.state;
    return (
      <Page loading={loading} route={route}>
        <div className={styles.header} ref={(c) => (this.$header = ReactDOM.findDOMNode(c))}>
          <AddCategoryPopConfirm onOk={this.addCategory} getPopupContainer={() => this.$header}>
            <Button disabled={data.length >= 5}>添加分类</Button>
          </AddCategoryPopConfirm>
        </div>
        {data.map((d) => {
          const isEditing = editingCategoryId === d.id;
          return (
            <TagCategory
              key={d.id}
              data={d}
              isEditing={isEditing}
              onEditStart={() => this.setState({ editingCategoryId: d.id })}
              onDelete={() => this.deleteCategory(d.id)}
              onSave={(data) => this.save({ id: d.id, data })}
              onCancel={this.cancelEdit}
              existTitles={data.filter((da) => da.id !== d.id).map((dat) => dat.title)}
              editable={d.id !== 'signTypeCampus'}
            />
          );
        })}
      </Page>
    );
  }
}

export default TagManagePage;
