/**
 * 批量操作页面
 * @property {object[]} list - 文章列表数据
 * @property {function} onSaveSuccess - 保存成功时的回调
 * @property {function} onCancel - 取消时的回调
 */
import React from 'react';
import ReactDOM from 'react-dom';
import styles from './styles.less';
import Button from 'COMPONENT/Button';
import Checkbox from 'COMPONENT/Checkbox';
import ArticleCard from '../ArticleCard';
import { Row, Col } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import styleVars from 'COMPONENT/theme/variables';
import { publishContent } from 'SERVICE/ipadBackEnd';
import TransformModal from '../ArticleCard/TransformModal';
import CopyModal from '../ArticleCard/CopyModal';
import PublishPopConfirm from '../ArticleCard/PublishPopConfirm';
import {
  STATUS_UNPUBLISH,
  STATUS_PUBLISHED,
  STATUS_TOPUBLISH,
  MAX_CAROUSEL_COUNT,
  MAX_CAROUSEL_TIPS,
} from '../../common';
import PopTip from 'COMPONENT/PopTip';
import commonStyles from 'PAGE/ipadBackEnd/articleEdit/commonStyles.less';

class BatchPage extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      checkedItems: [], // 选中的文章ID集合
      transformModalVis: false, // 转移弹窗
      transformTipsVis: false, // 转移tips
      copyModalVis: false, // 复制弹窗
      copyTipsVis: false, // 复制tips
      publishTipsVis: false, // 发布tips
      unPublishTipsVis: false, // 取消发布tips
    };

    this.onCheckedChange = this.onCheckedChange.bind(this);
    this.onTransform = this.onTransform.bind(this);
    this.onTransformSuccess = this.onTransformSuccess.bind(this);
    this.copy = this.copy.bind(this);
    this.onCopySuccess = this.onCopySuccess.bind(this);
    this.publish = this.publish.bind(this);
    this.unPublish = this.unPublish.bind(this);
    this.tipsVisibleChange = this.tipsVisibleChange.bind(this);
  }

  /**
   * 选中
   * @param id
   * @param checked
   */
  onCheckedChange(id, checked) {
    const { checkedItems } = this.state;

    this.setState({
      checkedItems: checked ? [...checkedItems, id] : checkedItems.filter((c) => c !== id),
    });
  }

  /**
   * 转移
   */
  onTransform() {
    if (this.state.checkedItems.length === 0) {
      this.setState({ transformTipsVis: true });
      return;
    }

    this.setState({ transformModalVis: true });
  }

  /**
   * 转移成功
   */
  onTransformSuccess() {
    this.setState({ transformModalVis: false });
    this.props.onSaveSuccess();
  }

  /**
   * 复制
   */
  copy() {
    if (this.state.checkedItems.length === 0) {
      this.setState({ copyTipsVis: true });
      return;
    }

    this.setState({ copyModalVis: true });
  }

  /**
   * 复制成功
   */
  onCopySuccess() {
    this.setState({ copyModalVis: false });
    this.props.onSaveSuccess();
  }

  /**
   * 发布
   * @param date
   */
  publish(date) {
    const { onLoadingStart, onLoadingEnd, onSaveSuccess } = this.props;
    const { checkedItems } = this.state;
    const params = checkedItems.map((id) => {
      return {
        id,
        status: date ? STATUS_TOPUBLISH : STATUS_PUBLISHED,
        appointmentPublishTime: date ? `${date} 00:00:00` : null,
      };
    });

    onLoadingStart();
    publishContent(params, true).then(
      () => {
        onSaveSuccess();
        onLoadingEnd();
        showMessage('发布成功', 'success');
      },
      (e) => {
        showMessage(e.text || '发布失败');
        onLoadingEnd();
      },
    );
  }

  /**
   * 取消发布
   */
  unPublish() {
    const { onLoadingStart, onLoadingEnd, onSaveSuccess } = this.props;
    const { checkedItems } = this.state;
    const params = checkedItems.map((id) => {
      return { id, status: STATUS_UNPUBLISH };
    });

    onLoadingStart();
    publishContent(params, true).then(
      () => {
        onSaveSuccess();
        onLoadingEnd();
        showMessage('隐藏成功', 'success');
      },
      (e) => {
        showMessage(e.text || '隐藏失败');
        onLoadingEnd();
      },
    );
  }

  /**
   * 点击操作按钮
   * @param key
   * @param visible
   */
  tipsVisibleChange(key, visible) {
    if (!visible) {
      this.setState({ [key]: visible });
    }
  }

  render() {
    const {
      list,
      onCancel,
      isTopUser,
      orgDict,
      menuTree,
      orgId,
      columnType,
      publishCount,
      colSpan,
      isIndex,
      menuItem,
      selectedMenu,
    } = this.props;
    const {
      checkedItems,
      transformModalVis,
      transformTipsVis,
      copyModalVis,
      copyTipsVis,
      publishTipsVis,
      unPublishTipsVis,
    } = this.state;
    const hasCheck = checkedItems.length > 0;
    const noData = list.length === 0;
    const newPublishCount = list.filter(
      (li) => checkedItems.indexOf(li.id) >= 0 && li.publishStatus === STATUS_UNPUBLISH,
    ).length;
    const isOverPublish = publishCount + newPublishCount > MAX_CAROUSEL_COUNT;
    let publishBtn;

    // 是否为个性化目录
    const isPersonalMenu = selectedMenu && !!selectedMenu.isPersonalMenu;

    if (!hasCheck || isOverPublish) {
      publishBtn = (
        <PopTip
          content={!hasCheck ? '请选择要发布的图文' : MAX_CAROUSEL_TIPS}
          visible={publishTipsVis}
          onVisibleChange={(visible) => this.tipsVisibleChange('publishTipsVis', visible)}
        >
          <Button onClick={() => this.setState({ publishTipsVis: true })} disabled={noData}>
            发布
          </Button>
        </PopTip>
      );
    } else {
      publishBtn = (
        <PublishPopConfirm onOk={this.publish} getPopupContainer={() => this.$wrap}>
          <Button disabled={noData}>发布</Button>
        </PublishPopConfirm>
      );
    }

    return (
      <section ref={(c) => (this.$wrap = ReactDOM.findDOMNode(c))}>
        <div className={styles.header}>
          <div className={styles.headerLeft}>
            <Checkbox
              checked={checkedItems.length > 0 && list.length === checkedItems.length}
              indeterminate={checkedItems.length > 0 && list.length > checkedItems.length}
              onChange={(e) => this.setState({ checkedItems: e.target.checked ? list.map((d) => d.id) : [] })}
            />
            <span className={styles.tips}>
              已选择
              <span style={{ color: styleVars.highlightColor, margin: '0 5px' }}>{checkedItems.length}</span>
              项内容
            </span>
          </div>
          <div className={styles.headerRight}>
            <PopTip
              content="请选择要转移的图文"
              visible={transformTipsVis}
              onVisibleChange={(visible) => this.tipsVisibleChange('transformTipsVis', visible)}
            >
              <Button onClick={this.onTransform} disabled={noData || !isPersonalMenu}>
                转移
              </Button>
            </PopTip>

            <PopTip
              content="请选择要复制的图文"
              visible={copyTipsVis}
              onVisibleChange={(visible) => this.tipsVisibleChange('copyTipsVis', visible)}
            >
              <Button onClick={this.copy} disabled={noData}>
                复制
              </Button>
            </PopTip>

            {!isIndex && publishBtn}

            {hasCheck ? (
              <PublishPopConfirm type="cancel" onOk={this.unPublish} getPopupContainer={() => this.$wrap}>
                <Button>隐藏</Button>
              </PublishPopConfirm>
            ) : (
              <PopTip
                content="请选择要隐藏的图文"
                visible={unPublishTipsVis}
                onVisibleChange={(visible) => this.tipsVisibleChange('unPublishTipsVis', visible)}
              >
                <Button onClick={() => this.setState({ unPublishTipsVis: true })} disabled={noData}>
                  隐藏
                </Button>
              </PopTip>
            )}

            <Button onClick={onCancel}>取消</Button>
          </div>
        </div>

        {noData ? (
          <div className={commonStyles.noDataTips}>没有可批量操作的图文!</div>
        ) : (
          <Row style={{ margin: '-10px -10px 0 -10px' }}>
            {list.map((article) => {
              const { id } = article;

              return (
                <Col key={id} span={colSpan} style={{ padding: '10px 10px' }}>
                  <ArticleCard
                    data={article}
                    checkable={true}
                    checked={checkedItems.indexOf(id) >= 0}
                    onCheckedChange={(checked) => this.onCheckedChange(id, checked)}
                    isTopUser={isTopUser}
                    type="batch"
                    columnType={columnType}
                    menuItem={menuItem}
                    selectedMenu={selectedMenu}
                  />
                </Col>
              );
            })}
          </Row>
        )}

        {transformModalVis && (
          <TransformModal
            menuTree={menuTree}
            menuItem={menuItem}
            ids={checkedItems}
            onClose={() => this.setState({ transformModalVis: false })}
            onSaveSuccess={this.onTransformSuccess}
          />
        )}

        {copyModalVis && (
          <CopyModal
            canSelectOrg={isTopUser && !list.some((a) => a.isBelongTop)}
            orgDict={orgDict}
            menuTree={menuTree}
            onSaveSuccess={this.onCopySuccess}
            onClose={() => this.setState({ copyModalVis: false })}
            ids={checkedItems}
            orgId={orgId}
            isTopUser={isTopUser}
          />
        )}
      </section>
    );
  }
}

export default BatchPage;
