import React from 'react';
import { Tree, Tooltip } from 'antd';
import { connect } from 'dva';
import styles from './styles.less';
import Icon from 'COMPONENT/Icon';
import IconButton from 'COMPONENT/IconButton';
import Input from 'COMPONENT/Input';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import { setShowMenuContent, saveCmsMenu, deleteCmsMenu } from 'SERVICE/ipadBackEnd';
import classnames from 'classnames';
import { isPersonalContent } from '../../../config';
import { findTreeNodeById } from '../util';
import { MAX_MENU_COUNT } from '../../common';

const TreeNode = Tree.TreeNode;

class MenuTree extends React.Component {
  static defaultProps = {
    onLoadingStart: () => {},
    onLoadingEnd: () => {},
    onReloadCard: () => {}, // 刷新卡片列表
    onReloadPage: () => {}, // 刷新整个页面
  };

  constructor(props) {
    super(props);

    this.state = {
      data: [],
      expandedKeys: [],
      selectedKeys: [],

      isEditing: false, // 是否正在编辑
      showSave: false, // 是否显示保存按钮
      editingId: null, // 正在编辑的树节点
      editingTitle: '', // 正在编辑的树标题
    };

    this.onSelect = this.onSelect.bind(this);
    this.onToggleVisible = this.onToggleVisible.bind(this);
    this.toggleVisible = this.toggleVisible.bind(this);
    this.onEyeClick = this.onEyeClick.bind(this);
    this.renderNodeTitle = this.renderNodeTitle.bind(this);
    this.onEditClick = this.onEditClick.bind(this);
    this.onSaveClick = this.onSaveClick.bind(this);
    this.onDeleteClick = this.onDeleteClick.bind(this);
    this.doNormalDelete = this.doNormalDelete.bind(this);
    this.doAjaxDelete = this.doAjaxDelete.bind(this);
    this.onAddClick = this.onAddClick.bind(this);
  }

  componentWillReceiveProps(nextProps) {
    if (!objectDeepEquals(this.props.data, nextProps.data)) {
      const { data, selected: menuId } = nextProps;
      const selectedMenu = data.find((ele) => {
        return +ele.id === +menuId || (ele.children.length && !!ele.children.find((ele) => +ele.id === +menuId));
      });

      const expandedKeys = selectedMenu && !!!selectedMenu.visible ? ['' + selectedMenu.id] : [];
      const selectedKeys = menuId ? [menuId] : [];
      this.setState({ data, expandedKeys, selectedKeys });
    }
  }

  onSelect(keys, e) {
    const hasChild = !!e.node.props.children;
    const visible = !!e.node.props.visible;
    const canExpand = visible ? !visible : hasChild;
    if (canExpand) {
      const expandedKeys = keys.filter((ele) => this.state.expandedKeys.indexOf(ele) < 0);
      this.setState({ expandedKeys, selectedKeys: [] });
      return;
    } else if (hasChild) {
      this.setState({ expandedKeys: [] });
    }
    if (keys.length > 0) {
      this.setState({ selectedKeys: keys });
      this.props.onChange(keys[0]);
    }
  }

  onToggleVisible(id) {
    const { data } = this.state;
    let vis;
    let item;
    let parentItem;
    data.forEach((d, i) => {
      if (d.id === id) {
        item = d;
        vis = !d.visible;
      } else if (d.children) {
        d.children.forEach((c, j) => {
          if (c.id === id) {
            item = c;
            parentItem = d;
            vis = !c.visible;
          }
        });
      }
    });
    const parentId = parentItem && parentItem.id;
    let tips;
    const { children } = item;
    if (vis) {
      if (children && children.length > 0 && children.find((c) => c.visible)) {
        tips = '显示一级目录的图文将隐藏其下所有子目录的图文，确定吗？';
      }
    } else {
      if (children && children.length > 0 && children.find((c) => !c.visible)) {
        tips = '隐藏一级目录的图文将显示其下所有子目录的图文，确定吗？';
      }
    }
    if (tips) {
      showConfirm(tips, () => this.toggleVisible(item, vis, parentId));
    } else {
      this.toggleVisible(item, vis, parentId);
    }
  }

  toggleVisible(item, vis, parentId) {
    const { onLoadingStart, onLoadingEnd } = this.props;
    const { data } = this.state;

    let newData = data.map((d) => {
      const { id, children } = d;
      let newItem = { ...d };

      if (id === item.id) {
        newItem.visible = vis;
      }

      if (children && children.length > 0) {
        newItem.children = children.map((c) => {
          let newCItem = { ...c };

          if (c.id === item.id) {
            newCItem.visible = vis;
          }

          return newCItem;
        });
      }

      return newItem;
    });

    newData = newData.map((da) => {
      const { id, children } = da;
      let newItem = { ...da };

      if (parentId === id) {
        newItem.visible = !vis;
      }

      if (id === item.id && children && children.length > 0) {
        newItem.children = children.map((child) => {
          return { ...child, visible: !vis };
        });
      }

      return newItem;
    });

    this.setState({ data: newData, selectedKeys: [] });
    onLoadingStart();

    setShowMenuContent({ id: item.id, status: vis ? 1 : 0 }, true).then(
      () => {
        showMessage('操作成功', 'success');
        onLoadingEnd();
      },
      () => {
        showMessage('操作失败');
        this.setState({ data });
        onLoadingEnd();
      },
    );
  }

  /**
   * 点击眼睛
   * @param e
   * @param id
   */
  onEyeClick(e, id) {
    e.stopPropagation();
    this.onToggleVisible(id);
  }

  /**
   * 渲染树节点
   * @param node 树节点
   * @param level 树层级
   * @param editDisable 是否不允许编辑 工作台不允许编辑（只能由外面传入，通过父节点title判断）
   */
  renderNodeTitle(node, level, editDisable) {
    const { isEditing, showSave, editingId, editingTitle } = this.state;
    const { title, id, visible } = node;
    const isChildNode = level === 2;
    const btnGroupClass = {
      [styles.eye]: 1,
      [styles.eyeUnClickable]: isChildNode,
    };

    // 是否允许编辑
    const equalId = editingId === node.id;
    const canDoEdit = isEditing && equalId;
    const canShowSave = showSave && equalId;
    const isPersonal = node.menuType === 1;
    const canDoDelete = node.isAdded || isPersonal;

    // 判断字符是否超出
    const length = title.length || 0;
    const showTips = length >= 13;

    return (
      <div className={styles.content}>
        <div className={styles.title}>
          {canDoEdit ? (
            <div style={{ width: 100 }}>
              <Input
                placeholder="请输入"
                value={editingTitle}
                onChange={(e) => this.setState({ editingTitle: e.target.value })}
                onFocus={this.preventDefault}
                onClick={this.preventDefault}
              />
            </div>
          ) : showTips ? (
            <Tooltip placement="top" title={title}>
              <div className={styles.textWrap}>{title}</div>
            </Tooltip>
          ) : (
            <div className={styles.textWrap}>{title}</div>
          )}
        </div>

        <div className={classnames(btnGroupClass)} onMouseOver={this.preventDefault}>
          {isChildNode ? (
            <span>
              {!editDisable && (
                <span>
                  {canShowSave ? (
                    <IconButton
                      type="documents_mc"
                      style={{ marginRight: 8 }}
                      color={'#333'}
                      onClick={(e) => {
                        this.onSaveClick(e, node);
                      }}
                    />
                  ) : (
                    <IconButton
                      type="edit_mc"
                      style={{ marginRight: 8 }}
                      color={'#333'}
                      disabled={isEditing || !isPersonal}
                      onClick={(e) => {
                        this.onEditClick(e, node);
                      }}
                    />
                  )}

                  <IconButton
                    type="delete"
                    style={{ marginRight: 8 }}
                    color={'#333'}
                    disabled={!canDoDelete}
                    onClick={(e) => {
                      this.onDeleteClick(e, node);
                    }}
                  />
                </span>
              )}

              {!node.isAdded && !node.isPersonalMenu && (
                <Icon
                  type={visible ? 'visible_mc' : 'hide_mc'}
                  onClick={isChildNode ? null : (e) => this.onEyeClick(e, id)}
                />
              )}
            </span>
          ) : (
            <span>
              {!editDisable && (
                <IconButton
                  type="plus"
                  disabled={isEditing}
                  onClick={(e) => {
                    this.onAddClick(e, node);
                  }}
                />
              )}

              {!node.isPersonalMenu && (
                <IconButton
                  type={visible ? 'visible_mc' : 'hide_mc'}
                  onClick={isChildNode ? null : (e) => this.onEyeClick(e, id)}
                />
              )}
            </span>
          )}
        </div>
      </div>
    );
  }

  /**
   * 点击编辑
   * @param e
   * @param data
   */
  onEditClick(e, data) {
    e.stopPropagation();

    this.setState({
      isEditing: true,
      showSave: true,
      editingId: data.id,
      editingTitle: data.title,
    });
  }

  /**
   * 点击保存 需要通过node判断是新增还是修改
   * @param e
   * @param node
   */
  onSaveClick(e, node) {
    e.stopPropagation();

    const { onLoadingStart, onLoadingEnd, onReloadCard, mode, orgId, orgName } = this.props;
    const { editingTitle, data } = this.state;
    const { id, title } = node;
    const initState = {
      isEditing: false,
      showSave: false,
      editingId: null,
      editingTitle: '',
    };

    if (!editingTitle) {
      showMessage('请输入标题', 'error');
      return false;
    } else if (editingTitle === title) {
      this.setState(initState);
      return false;
    }

    const message = (node.isAdded && '新增') || '修改';
    let param = {
      name: editingTitle,
      useType: mode,
      level: 2,
      allCanSee: 0,
      showContentNum: 4,
      showType: 0,
      menuType: 1,
      status: 1,
      orgId: orgId,
      orgName: orgName,
    };

    if (node.isAdded) {
      param = { ...param, parentId: node.parentId };
    } else {
      const result = findTreeNodeById(data, id);
      param = {
        ...param,
        id,
        parentId: (result.parentNode && result.parentNode.id) || '',
        status: node.visible ? 1 : 0,
      };
    }

    onLoadingStart();

    saveCmsMenu(param)
      .then((response) => {
        showMessage(`${message}成功`, 'success');
        this.setState(initState);
        onLoadingEnd();
        onReloadCard();
      })
      .catch((e) => {
        showMessage(e.text || `${message}失败，请重试`, 'error');
        this.setState(initState);
        onLoadingEnd();

        if (node.isAdded) {
          this.doNormalDelete(node);
        }
      });
  }

  /**
   * 删除目录
   * @param e
   * @param node
   */
  onDeleteClick(e, node) {
    e.stopPropagation();

    if (node.isAdded) {
      this.doNormalDelete(node);
    } else {
      this.doAjaxDelete(node);
    }
  }

  /**
   * 删除本地新增的目录
   * @param node
   */
  doNormalDelete(node) {
    const { data } = this.state;
    let tempData = [...data];

    tempData = this.deleteTreeNodeById(tempData, node.id);

    this.setState({
      data: tempData,
      isEditing: false,
      showSave: false,
      editingId: null,
      editingTitle: '',
    });
  }

  /**
   * 删除远程目录
   * @param node 树节点
   */
  doAjaxDelete(node) {
    const { onLoadingStart, onLoadingEnd, onReloadPage } = this.props;

    showConfirm('是否确定删除？', () => {
      onLoadingStart();
      deleteCmsMenu({ id: node.id })
        .then((result) => {
          showMessage('删除成功', 'success');
          onLoadingEnd();
          onReloadPage();
        })
        .catch((e) => {
          showMessage(e.text || '删除失败，请重试', 'error');
          onLoadingEnd();
        });
    });
  }

  /**
   * 新增, 这里拼接的新节点加上 parentId 属性更佳
   * @param e
   * @param node
   */
  onAddClick(e, node) {
    e.stopPropagation();

    const { data } = this.state;
    let canDoAdd = true;
    let tempData = [...data];
    const newNode = { id: 'added', parentId: node.id, title: '', level: 2, isAdded: true, visible: true };
    let editingId = null;
    let editingTitle = '';

    tempData.forEach((item) => {
      if (item.id === node.id) {
        if (item.children) {
          if (item.children.length >= MAX_MENU_COUNT) {
            showMessage(`最多只能新增${MAX_MENU_COUNT}个目录`, 'error');
            canDoAdd = false;
          } else {
            item.children.push(newNode);
            editingId = newNode.id;
            editingTitle = newNode.title;
          }
        } else {
          item.children = [newNode];
          editingId = newNode.id;
          editingTitle = newNode.title;
        }

        return false;
      }
    });

    if (canDoAdd) {
      this.setState({
        data: tempData,
        isEditing: true,
        showSave: true,
        editingId: editingId,
        editingTitle: editingTitle,
      });
    }
  }

  /**
   * 删除某个树节点
   * @param data
   * @param id
   */
  deleteTreeNodeById(data, id) {
    let deleteNode = function (nodeList, nodeId) {
      for (let i = 0, len = nodeList.length; i < len; i++) {
        let item = nodeList[i];

        if (item.id === nodeId) {
          nodeList.splice(i, 1);
          break;
        } else if (item.children && item.children.length > 0) {
          item.children = deleteNode(item.children, nodeId);
        } else {
          continue;
        }
      }

      return nodeList;
    };

    return deleteNode(data, id);
  }

  /**
   * 阻止默认事件
   * @param e
   * @return {boolean}
   */
  preventDefault(e) {
    if (!e) {
      return false;
    }

    e.stopPropagation();
    e.preventDefault();
  }

  render() {
    const { data, expandedKeys, selectedKeys } = this.state;

    return (
      <div className={styles.wrap}>
        <Tree selectedKeys={selectedKeys} onSelect={this.onSelect} expandedKeys={expandedKeys}>
          {data &&
            data.map((item) => {
              const { title, children } = item;
              const editDisable = !isPersonalContent(title);

              return (
                <TreeNode key={item.id} visible={item.visible} title={this.renderNodeTitle(item, 1, editDisable)}>
                  {children &&
                    children.length > 0 &&
                    children.map((node) => (
                      <TreeNode
                        key={node.id}
                        visible={node.visible}
                        title={this.renderNodeTitle(node, 2, editDisable)}
                      />
                    ))}
                </TreeNode>
              );
            })}
        </Tree>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const { account } = state;

  return {
    orgId: account.user.brenchId,
    orgName: account.user.brenchName,
    nowRouter: state.main.activeTabId,
  };
};

export default connect(mapStateToProps)(MenuTree);
