/**
 * 内容编辑主页面
 * @property mode - 模式，'browser'（浏览模式）|'workbench'（工作台模式）
 * @property userOrgId - 当前登录用户所属机构ID
 */
import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import DropdownMenu from 'COMPONENT/DropdownMenu';
import Button from 'COMPONENT/Button';
import styles from './styles.less';
import { getDictTextByVal } from 'UTIL/dict';
import { getAllBrenchAndGroup, getCmsMenuList } from 'SERVICE/ipadBackEnd';
import styleVars from 'COMPONENT/theme/variables';
import MenuTree from './MenuTree';
import PropTypes from 'prop-types';
import {
  TOP_ORG_ID,
  TOP_ORG_NAME,
  publishStatusDict,
  belongTypeDict,
  recommendDict,
  BELONG_TYPE_BRANCH,
  PERSONALINFO,
  BELONG_TYPE_PERSONAL,
  BELONG_TYPE_CAMPUS,
  AUTH_INTERNAL,
  AUTH_PERSONAL,
} from '../common';
import MenuContent from './MenuContent';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { findTreeNodeById } from './util';

const hiddenStyle = { display: 'none' };

class WorkbenchMainPage extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false, // 页面加载状态
      orgDict: [{ val: TOP_ORG_ID, text: TOP_ORG_NAME }], // 字典，组织机构
      belongTypeDict, // 字典 所属类型
      publishStatusDict, // 字典 发布状态
      recommendDict, // 字典 首页推荐
      orgId: props.userOrgId, // 当前组织机构ID
      allMenuArr: [], // 左侧菜单原始数据
      menuTree: [], // 左侧菜单数据
      indexMenuId: null, // 首页菜单ID
      selectedMenu: null, // 当前选中的目录
      menuId: null, // 当前菜单ID (menuId? defaultMenuId?)
      defaultMenuId: null, // 默认选中的菜单ID
      contentData: null, // 右侧区域内容当前标签页下的数据
    };

    this.getChildContext = this.getChildContext.bind(this);
    this.isNowPersonal = this.isNowPersonal.bind(this);
    this.initContent = this.initContent.bind(this);
    this.isTopUser = this.isTopUser.bind(this);
    this.isTopOrg = this.isTopOrg.bind(this);
    this.getUserOrgType = this.getUserOrgType.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.reloadPage = this.reloadPage.bind(this);
    this.loadMenuTree = this.loadMenuTree.bind(this);
    this.onOrgChange = this.onOrgChange.bind(this);
    this.onMenuChange = this.onMenuChange.bind(this);
    this.getOrgSelect = this.getOrgSelect.bind(this);
    this.startLoading = this.startLoading.bind(this);
    this.endLoading = this.endLoading.bind(this);
    this.openTagManagePage = this.openTagManagePage.bind(this);
  }

  /**
   * 嵌套实在太深了，加个context上下文
   */
  getChildContext() {
    return {
      getActiveMenu: () => {
        return this.state.allMenuArr.find((menu) => menu.id === this.state.menuId);
      },
      //判断当前选中是否个性化资料模块
      isNowPersonal: this.isNowPersonal,
    };
  }

  isNowPersonal() {
    const { allMenuArr, menuId } = this.state;
    const nowMenu = allMenuArr.find((menu) => menu.id === menuId);
    if (nowMenu && nowMenu.name === PERSONALINFO) return true;
    const nowParent = allMenuArr.find((menu) => menu.id === nowMenu.parentId);
    if (nowParent && nowParent.name === PERSONALINFO) return true;
  }

  componentDidMount() {
    this.initContent();
  }

  initContent() {
    this.setState({ loading: true });
    const { orgId } = this.state;
    let calls = [this.updatePage(orgId)];

    if (this.isTopUser()) {
      calls.push(this.loadOrgDict());
    }

    Promise.all(calls).then(([a, orgDict]) => {
      let changed = { loading: false };

      if (orgDict) {
        changed = { ...changed, orgDict };
      }

      this.setState(changed);
    });
  }

  //主组织架构为集团的账号时
  isTopUser() {
    return this.props.userOrgId === TOP_ORG_ID;
  }

  //架构切换至星火集团时
  isTopOrg() {
    return this.state.orgId === TOP_ORG_ID;
  }

  /**
   * 目前只有编辑保存时用到
   */
  getUserOrgType() {
    const orgType = this.props.userOrgType;

    //个性化资料下为个人类型
    if (this.isNowPersonal()) {
      return BELONG_TYPE_PERSONAL;
    }

    //主架构为学习中心，类型为分公司
    if (orgType === BELONG_TYPE_CAMPUS) {
      return BELONG_TYPE_BRANCH;
    }

    return orgType;
  }

  async updatePage(orgId, menuId) {
    const { menuTree, indexMenuId, defaultMenuId, selectedMenu, allMenuArr } = await this.loadMenuTree(orgId);
    const nextMenuId = menuId || defaultMenuId;

    // setMenuId同时也要setmenuTree
    await new Promise((resolve, reject) => {
      this.setState({ menuId: nextMenuId, menuTree, indexMenuId, defaultMenuId, selectedMenu, allMenuArr }, resolve);
    });

    const contentData = await this.menuContentRef.loadContentData(nextMenuId);
    this.setState({ contentData });
  }

  reloadPage() {
    this.updatePage(this.state.orgId, this.state.menuId);
  }

  loadOrgDict = async () => {
    const data = await getAllBrenchAndGroup({}, true);
    let dict = [];

    data.forEach((d) => {
      dict.push({ val: d.id, text: d.name });
    });

    return dict;
  };

  /**
   * 加载左侧目录菜单
   * 权限判断修改，判断有无内部资料和个性化资料权限
   * 给个性化目录添加一个 isPersonalMenu 属性，便于后面判断使用
   * 保存默认选中的menu节点
   * @param orgId
   */
  async loadMenuTree(orgId) {
    const { mode, btnAuths } = this.props;

    let indexMenuId;
    let defaultMenuId;
    const apiData = await getCmsMenuList({ useType: mode, orgId }, true);
    const menuList = apiData.filter((da) => da.level === '1');

    let data = menuList.map((menu) => {
      const { id, name, status, showContentNum, menuType } = menu;

      const children = apiData
        .filter((child) => {
          return child.parentId === id;
        })
        .map((c) => {
          return {
            id: c.id,
            title: c.name,
            visible: c.status === '1',
            showContentNum: c.showContentNum,
            menuType: c.menuType,
          };
        });

      return { id, title: name, children, visible: status === '1', showContentNum, menuType };
    });

    // 权限判断
    const hasInternal = btnAuths.includes(AUTH_INTERNAL);
    const hasPersonal = btnAuths.includes(AUTH_PERSONAL);
    const hasBothAuth = hasInternal && hasPersonal;
    const hasNoAuth = !hasInternal && !hasPersonal;

    if (hasNoAuth) {
      data = [];
    } else if (!hasBothAuth) {
      if (hasInternal) {
        data = data.filter((d) => d.title !== PERSONALINFO);
      } else if (hasPersonal) {
        data = data.filter((d) => d.title === PERSONALINFO);
      }
    }

    // 给个性化目录添加isPersonalMenu属性
    data.forEach((item) => {
      if (item.title === PERSONALINFO) {
        item.isPersonalMenu = true;

        if (item.children && item.children.length > 0) {
          item.children.forEach((ch) => {
            ch.isPersonalMenu = true;
          });
        }
      }
    });

    // 默认展开第一个一级菜单
    indexMenuId = data.find((d) => d.parentId !== -1) || {};
    // 默认选中的目录ID
    defaultMenuId = data.find((ele) => !!ele.visible || (!!!ele.visible && ele.children.length > 0));
    defaultMenuId = !!defaultMenuId.visible ? defaultMenuId.id : defaultMenuId.children.find((ele) => !!ele.visible).id;

    // 默认选中的menu节点
    const nodeResult = findTreeNodeById(data, defaultMenuId);
    const selectedMenu = nodeResult.node;

    return { menuTree: data, indexMenuId: indexMenuId.id, defaultMenuId, selectedMenu, allMenuArr: apiData };
  }

  onOrgChange(orgId) {
    this.setState({ loading: true, orgId, contentData: null }, () => {
      this.updatePage(orgId).then(() => {
        this.setState({ loading: false });
      });
    });
  }

  /**
   * 选中目录
   * @param menuId
   */
  onMenuChange(menuId) {
    const { menuTree } = this.state;
    const nodeResult = findTreeNodeById(menuTree, menuId);
    const selectedMenu = nodeResult.node;

    this.setState(
      {
        loading: true,
        menuId,
        selectedMenu,
        contentData: null,
      },
      () => {
        Promise.all([this.menuContentRef.loadContentData(menuId)]).then(([contentData]) => {
          this.setState({ loading: false, contentData });
        });
      },
    );
  }

  /**
   * 获取选中的树节点
   */
  getOrgSelect() {
    if (!this.isTopUser()) {
      return null;
    }

    const { orgDict, orgId } = this.state;
    const menus = orgDict.map((d) => {
      return { text: d.text, onClick: () => this.onOrgChange(d.val) };
    });

    return (
      <DropdownMenu
        menus={menus}
        color={styleVars.fontColor}
        hasArrow={true}
        text={orgId ? getDictTextByVal(orgDict, orgId) : TOP_ORG_NAME}
        dropdownClassName={styles.orgDropdown}
        placement="bottomCenter"
      />
    );
  }

  startLoading() {
    if (!this.state.loading) {
      this.setState({ loading: true });
    }
  }

  endLoading() {
    if (this.state.loading) {
      this.setState({ loading: false });
    }
  }

  openTagManagePage() {
    const { dispatch, mode, route } = this.props;
    const { orgId } = this.state;
    dispatch({
      type: 'main/openPage',
      payload: { path: `/${route.path}/TagManage`, params: { orgId, mode }, refresh: true },
    });
  }

  render() {
    const { route, mode } = this.props;
    const {
      loading,
      belongTypeDict,
      publishStatusDict,
      recommendDict,
      menuTree,
      allMenuArr,
      menuId,
      selectedMenu,
      indexMenuId,
      orgId,
      orgDict,
      contentData,
    } = this.state;

    return (
      <Page
        loading={loading}
        route={route}
        headerToolBar={
          this.isTopOrg() ? (
            <Button onClick={this.openTagManagePage} style={{ marginRight: 20 }}>
              标签管理
            </Button>
          ) : null
        }
      >
        <div className={styles.orgSelectWrap} style={hiddenStyle}>
          {this.getOrgSelect()}
        </div>

        <div className={styles.wrap}>
          <div className={styles.left}>
            <MenuTree
              data={menuTree}
              selected={menuId + ''}
              mode={mode}
              onChange={(menuIdStr) => this.onMenuChange(parseInt(menuIdStr))}
              onLoadingStart={this.startLoading}
              onLoadingEnd={this.endLoading}
              onReloadCard={this.reloadPage}
              onReloadPage={this.initContent}
            />
          </div>

          <div className={styles.right}>
            <MenuContent
              belongTypeDict={belongTypeDict}
              publishStatusDict={publishStatusDict}
              recommendDict={recommendDict}
              contentData={contentData}
              onLoadingStart={this.startLoading}
              onLoadingEnd={this.endLoading}
              orgDict={orgDict}
              menuTree={menuTree}
              isTopUser={this.isTopUser()}
              isTopOrg={this.isTopOrg()}
              orgId={orgId}
              menuId={menuId}
              getUserOrgType={this.getUserOrgType}
              menuItem={allMenuArr.find((menu) => menu.id === menuId)}
              selectedMenu={selectedMenu}
              isIndex={menuId !== null && menuId === indexMenuId}
              ref={(c) => (this.menuContentRef = getWrappedInstance(c))}
              mode={mode}
              route={route}
              onReloadPage={this.reloadPage}
            />
          </div>
        </div>
      </Page>
    );
  }
}

WorkbenchMainPage.childContextTypes = {
  getActiveMenu: PropTypes.func,
  isNowPersonal: PropTypes.func,
};

const mapStateToProps = (state) => {
  return {
    userOrgId: state.account.user.brenchId,
    roleCode: state.account.user.roleCode,
    userOrgType: state.account.user.orgType,
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps)(WorkbenchMainPage);
