import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import SearchBar from './components/Search';
import RemarkModal from './components/RemarkModal';
import Player from './components/Player';
import { loadBranchDict, loadCampusDict } from 'UTIL/dict';
import { loadConsultantDict } from 'PAGE/common/loaders';
import commonStyles from './commonStyles/commonStyle.less';
import { dateToMoment } from 'COMPONENT/utils/form';
import { loadVoiceRecord, saveVoiceRemark, getVoiceRole, getOrganizationList } from './loader';
import showMessage from 'SRC/components/utils/globalTips/showMessage';
import moment from 'moment';
const FORMAT_VALUE = 'YYYY-MM-DD HH:mm:ss';

const recordingAuth = {
  // 查询
  inquire: 'BTN_RECORDING_INQUIRE',
  // 播放
  play: 'BTN_RECORDING_PLAY',
  // 下载
  download: 'BTN_RECORDING_DOWNLOAD',
  //备注
  remark: 'BTN_RECORDING_REMARK',
};
// 权限类型 集团 分公司 学习中心 个人
const authType = {
  org: 'org',
  com: 'com',
  cam: 'cam',
  per: 'per',
};
const currentDate = moment().format().slice(0, 10);
const defaultStartDate = moment().subtract(7, 'days').format().slice(0, 10);

class Recording extends React.Component {
  constructor(props) {
    super(props);
    const { btnAuth, user: { organizationType } = {} } = this.props;

    this.state = {
      loading: false, // boolean 页面加载状态
      showRemarkModal: false,
      selectRecordIndex: null, //选择的数据
      searchExpand: true,
      // 禁选
      isBranchDisabled: false,
      isCampusDisabled: false,
      isConsultDisabled: false,
      //查询条件
      //创建
      createTimeStart: defaultStartDate,
      createTimeEnd: currentDate,
      //上传
      uploadCompleteTimeStart: defaultStartDate,
      uploadCompleteTimeEnd: currentDate,
      branchId: null,
      blCampusId: null,
      consultantId: null,
      //列表数据'
      listLoading: false,
      rows: [],
      pageNo: 1,
      pageSize: 10,
      total: null,
      //数据字典数据
      branchDict: [],
      campusDict: [],
      consultantDict: [],
      audioData: {},
    };
  }

  async componentDidMount() {
    await this.loadRoleData();
    await this.initContent();
  }

  loadRoleData = async () => {
    const { user: { account, userId } = {} } = this.props;
    this.setState({ loading: true });
    try {
      const { data } = await getVoiceRole();
      const roleType = data.roleType;
      if (roleType === 'NORMAL') {
        // 如果是普通的只有当前账号的数据
        const normalState = {
          isBranchDisabled: true,
          isCampusDisabled: true,
          isConsultDisabled: true,
          consultantId: userId,
          consultantDict: [{ val: userId, text: account }],
          authType: authType.per,
        };
        this.setState(normalState);
        return roleType;
      }
    } catch (e) {
      showMessage(e.text);
      this.setState({ loading: false });
    }
  };
  loadListData = async () => {
    const {
      createTimeStart,
      createTimeEnd,
      uploadCompleteTimeStart,
      uploadCompleteTimeEnd,
      blCampusId,
      branchId,
      consultantId,
      pageNo,
      pageSize,
    } = this.state;
    let apiParams = {
      branchId,
      blCampusId,
      userId: consultantId || void 0,
      createTimeStart: createTimeStart + ' 00:00:00',
      createTimeEnd: createTimeEnd + ' 23:59:59',
      uploadCompleteTimeStart: uploadCompleteTimeStart + ' 00:00:00',
      uploadCompleteTimeEnd: uploadCompleteTimeEnd + ' 23:59:59',
      pageNo,
      pageSize,
    };

    try {
      return await loadVoiceRecord(apiParams);
    } catch (e) {
      console.log(e);
      showMessage(e.text);
      return [];
    }
  };

  initContent = async () => {
    const { branchId, isConsultDisabled } = this.state;
    this.setState({ loading: true });
    let asyncCalls = [getOrganizationList({ orgType: 'BRENCH' })];
    let [branchDict] = await Promise.all(asyncCalls);
    //获取列表数据
    // 集团权限需要自己选择公司之后才有数据 默认没有
    if (!branchId && !isConsultDisabled) {
      this.setState({
        loading: false,
        branchDict,
      });
      return false;
    }
    const list = await this.loadListData();
    this.setState({
      loading: false,
      rows: list.datas,
      total: list.rowCount,
      pageNo: list.pageNo,
      branchDict,
    });
  };

  dictToObj = (dict) => {
    return (dict || []).reduce((obj, ele) => {
      obj[ele.val] = ele.text ? ele.text : ele.name;
      return obj;
    }, {});
  };

  search = async () => {
    const { branchId, createTimeStart, createTimeEnd, uploadCompleteTimeStart, uploadCompleteTimeEnd, authType } =
      this.state;
    if (authType !== 'per' && !branchId) {
      showMessage('请选择分公司');
      return false;
    }
    if (!(createTimeStart && createTimeEnd && uploadCompleteTimeStart && uploadCompleteTimeEnd)) {
      showMessage('录音创建时间和上传时间不能为空');
      return false;
    }
    await this.updateState({ listLoading: true, pageNo: 1 });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      pageNo: list.pageNo,
    });
  };
  // 重新加载当前页面数据
  resetLoadData = async () => {
    await this.updateState({ listLoading: true });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      pageNo: list.pageNo,
    });
  };

  reset = async () => {
    const initParams = {};
    if (this.state.authType !== 'per') {
      initParams.branchId = null;
      initParams.blCampusId = null;
      initParams.consultantId = null;
      this.updateState(initParams);
    }
  };

  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ listLoading: true, pageSize, pageNo: current });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      pageNo: list.pageNo,
    });
  };

  updateRow = (index, changed) => {
    this.setState({
      rows: this.state.rows.map((row, i) => {
        if (i === index) {
          return { ...row, ...changed };
        }
        return row;
      }),
    });
  };

  /**
   * 获取要禁用的开始日期值
   * @param  {string} beginTime 过滤前的日期值
   * @param  {string} type 过滤的是上传的还是创建的时间
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate = (beginTime, type) => {
    const endTime = this.state[type];
    if (!beginTime || !endTime) {
      return false;
    }
    const startDate = dateToMoment(beginTime).valueOf();
    const endDate = dateToMoment(endTime).valueOf();
    if (startDate > endDate || endDate >= startDate + 1000 * 60 * 60 * 24 * 31) {
      return true;
    }
  };

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endTime   过滤前的日期值
   * @param  {string} type 过滤的是上传的还是创建的时间
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate = (endTime, type) => {
    const beginTime = this.state[type];
    if (!endTime || !beginTime) {
      return false;
    }
    const startDate = dateToMoment(beginTime).valueOf();
    const endDate = dateToMoment(endTime).valueOf();
    if (startDate > endDate || endDate >= startDate + 1000 * 60 * 60 * 24 * 31) {
      return true;
    }
  };

  /**
   * 更新state
   * */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  handleSetState = ({ type, val }) => {
    switch (type) {
      case 'branchId':
        //填写分公司 获取学习中心数据 并且清空学习中心和咨询师数据和字典
        this.setState({
          campusDict: [],
          consultantDict: [],
          blCampusId: null,
          consultantId: null,
        });
        getOrganizationList({ orgId: val, orgType: 'CAMPUS' }).then((data) => this.setState({ campusDict: data }));
        break;
      case 'blCampusId':
        //填写分公司 获取学习中心数据 并且清空学习中心和咨询师数据和字典
        this.setState({
          consultantDict: [],
          consultantId: null,
        });
        loadConsultantDict({ orgId: val }).then((data) => this.setState({ consultantDict: data }));
        break;
    }
    this.setState({
      [type]: val,
    });
  };

  /*
   * 提交备注
   * */

  onConfirm = async ({ value }) => {
    const { selectRecordIndex, rows } = this.state;
    const id = rows[selectRecordIndex].id;
    try {
      await saveVoiceRemark({ id, remark: value });
      this.updateRow(selectRecordIndex, { remark: value });
      showMessage('提交备注成功', 'success');
    } catch (e) {
      console.log(e);
      showMessage('提交备注失败');
    }
    this.setState({
      selectRecordIndex: null,
      showRemarkModal: false,
    });
  };
  // 下载录音文件
  exportExcel(data) {
    const { filePath, showName } = data;
    if (this.refA) {
      this.refA.href = filePath;
      this.refA.download = `${filePath}`;
      this.refA.click();
    }
  }

  renderRemark = (text) => {
    if (text && text.length > 20) {
      return text.slice(0, 19) + '...';
    }
    return text;
  };
  /**
   * 表头
   */
  columns = [
    { title: '序号', dataIndex: 'index', key: 'index', width: 60, render: (text, record, index) => index + 1 },
    { title: '文件名称', dataIndex: 'showName', key: 'showName', width: 150 },
    {
      title: '上传完成时间',
      dataIndex: 'uploadCompleteTime',
      key: 'uploadCompleteTime',
      width: 200,
      render: (text, record) => moment(record.uploadCompleteTime).format(FORMAT_VALUE),
    },
    { title: '文件大小(Mb)', dataIndex: 'fileSizeM', key: 'fileSizeM', width: 100 },
    { title: '备注内容', dataIndex: 'remark', key: 'remark', width: 200, render: this.renderRemark },
    {
      title: '操作',
      key: 'action',
      width: 180,
      getActionConfig: (text, record, index) => {
        const { btnAuth } = this.props;
        const btnArray = [];
        if (btnAuth.includes(recordingAuth.play)) {
          btnArray.push({
            text:
              this.state.audioData.id === record.id ? <span style={{ color: '#2ACF6D' }}>正在播放</span> : '在线播放',
            onClick: () => this.setState({ audioData: record }),
          });
        }
        if (btnAuth.includes(recordingAuth.download)) {
          btnArray.push({ text: '下载', onClick: () => this.exportExcel(record) });
        }
        if (btnAuth.includes(recordingAuth.remark)) {
          btnArray.push(
            record.remark
              ? { text: '查看备注', onClick: () => this.setState({ showRemarkModal: true, selectRecordIndex: index }) }
              : { text: '备注', onClick: () => this.setState({ showRemarkModal: true, selectRecordIndex: index }) },
          );
        }
        return btnArray;
      },
    },
  ];

  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      rows,
      pageNo,
      pageSize,
      total,
      selectRecordIndex,
      createTimeStart,
      createTimeEnd,
      uploadCompleteTimeStart,
      uploadCompleteTimeEnd,
      branchId,
      blCampusId,
      consultantId,
      branchDict,
      campusDict,
      consultantDict,
      showRemarkModal,
      isBranchDisabled,
      isCampusDisabled,
      isConsultDisabled,
      audioData,
    } = this.state;
    const valueMap = {
      createTimeStart,
      createTimeEnd,
      uploadCompleteTimeStart,
      uploadCompleteTimeEnd,
      branchId,
      blCampusId,
      consultantId,
      isBranchDisabled,
      isCampusDisabled,
      isConsultDisabled,
    };
    const dictMap = {
      branchDict,
      campusDict,
      consultantDict,
    };

    return (
      <Page
        loading={loading}
        route={this.props.route}
        fitWindowHeight
        subHeader={
          <SearchBar
            onSearch={this.search}
            onReset={this.reset}
            expand={searchExpand}
            onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
            handleKeyword={this.handleSetState}
            getDisabledStartDate={this.getDisabledStartDate}
            getDisabledEndDate={this.getDisabledEndDate}
            valueMap={valueMap}
            dict={dictMap}
          />
        }
      >
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows}
          pagination={{
            current: pageNo,
            pageSize,
            total: total,
          }}
          onChange={this.pageChange}
        />
        {showRemarkModal && (
          <RemarkModal
            value={rows[selectRecordIndex].remark}
            onConfirm={this.onConfirm}
            onClose={() => this.setState({ showRemarkModal: false, selectRecordIndex: null })}
          />
        )}
        <Player audioData={audioData} />
        <a ref={(refA) => (this.refA = refA)} style={{ opacity: 0 }} download={audioData.showName} />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
    btnAuth: state.account.btnAuths || [],
  };
};

export default connect(mapStateToProps)(Recording);
