import React from 'react';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import List from 'COMPONENT/List';
import TextLink from 'COMPONENT/TextLink';
import InputNumber from 'COMPONENT/InputNumber';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Styles from '../styles.less';

const { COL_WIDTH_M, COL_WIDTH_L } = List;

class PeopleLimitModal extends React.Component {
  static defaultProps = {
    campusDict: [],
  };
  constructor(props) {
    super(props);
    const {
      selectedProductRecord: { inventoryList, pepoleLimit, campusOrderStudents } = {},
      campusDict,
      editable,
    } = this.props;
    const inventoryListMap = this.campusDictToObject(inventoryList);
    const campusStudentNumberMap = this.campusStudentNumToObject(campusOrderStudents);
    const dataSource = campusDict.map(({ val, text }) => {
      // 新增的话需要默认空对象
      const limitMap = inventoryListMap[val] || {};
      const signNum = campusStudentNumberMap[val] || 0;
      return {
        campusName: text,
        campusNum: limitMap.pepoleLimit,
        signNum: signNum,
        campusId: val,
        id: limitMap.id,
      };
    });
    // 查看并且是-1的话就是无限制 那么初试人数为0 只用来查看已报名人数
    const defaultTotalPeopleNum = pepoleLimit === -1 ? (editable ? 1000 : 0) : pepoleLimit;
    this.state = {
      loading: false,
      totalPeopleNum: defaultTotalPeopleNum,
      dataSource,
    };
  }

  onConfirm = async () => {
    const { dataSource, totalPeopleNum } = this.state;
    const { selectedProductRecord: { inventoryList } = {}, onConfirm } = this.props;
    const inventoryListMap = this.campusDictToObject(inventoryList);
    // 已输入的人数
    let totalInputPeopleNum = 0;
    // 真正报名人数 可能包含未分配的人数
    dataSource.forEach(({ campusNum }) => {
      totalInputPeopleNum += campusNum || 0;
    });
    // 剩余未分配名额
    const remainPeopleNum = totalPeopleNum - totalInputPeopleNum;
    if (totalPeopleNum <= 0) {
      this.modal.showOkTips('产品总额需大于0');
      return false;
    }
    if (remainPeopleNum < 0) {
      this.modal.showOkTips('产品总额大于各学习中心名额之和，请调整后再保存');
      return false;
    }
    const inventoryListParams = dataSource
      .filter((campus) => campus.campusNum)
      .map(({ campusNum, campusId, id }) => ({ blCampusId: campusId, pepoleLimit: campusNum, id }));
    // 如果存在没有分配名额的学习中心 那么共用剩下的剩余名额
    if (inventoryListParams.length < dataSource.length) {
      // 编辑的话需要拿ALL的id
      const isAllId = inventoryListMap['ALL'] ? { id: inventoryListMap['ALL'].id } : {};
      inventoryListParams.push({ blCampusId: '', pepoleLimit: remainPeopleNum, ...isAllId });
    }
    onConfirm({ inventoryList: inventoryListParams, pepoleLimit: totalPeopleNum });
  };

  updateRow = (index, changed) => {
    this.setState({ loading: true });
    const dataSource = this.state.dataSource.map((row, i) => {
      if (i === index) {
        return { ...row, ...changed };
      }
      return row;
    });
    this.setState({
      loading: false,
      dataSource,
    });
    return { ...dataSource, ...changed };
  };

  // 一键均分
  averageNum = () => {
    const { totalPeopleNum, dataSource } = this.state;
    if (!dataSource.length) return false;
    if (totalPeopleNum < dataSource.length) {
      showMessage('产品名额小于学习中心数量，无法一键均分');
      return false;
    }
    const average = Math.floor(totalPeopleNum / dataSource.length);
    const newDataSource = dataSource.map((product) => ({ ...product, campusNum: average }));
    this.setState({
      dataSource: newDataSource,
    });
  };
  campusDictToObject = (dict) => {
    return dict.reduce((obj, cur) => {
      obj[cur.blCampusId] = cur;
      return obj;
    }, {});
  };

  campusStudentNumToObject = (dict) => {
    return dict.reduce((obj, cur) => {
      obj[cur.intentionCampusId] = cur.orderStudentNum;
      return obj;
    }, {});
  };

  columns = [
    { title: '活动学习中心名称', dataIndex: 'campusName', width: COL_WIDTH_M },
    {
      title: '学习中心名额',
      dataIndex: 'campusNum',
      width: COL_WIDTH_M,
      render: (text, record, index) => {
        const { editable } = this.props;
        return (
          <InputNumber
            disabled={!editable}
            placeholder="请输入学习中心名额"
            value={text}
            onChange={(val) => this.updateRow(index, { campusNum: val })}
            min={1}
            max={100000}
            step={1}
          />
        );
      },
      filterDropdownVisible: false,
      filterIcon: this.props.editable ? (
        <TextLink className="limitModalLink" defaultTitle="一键均分" onClick={() => this.averageNum()}>
          一键均分
        </TextLink>
      ) : (
        <span className="empty-content"> </span>
      ),
      filterDropdown: <span className="empty-content"> </span>,
    },
    { title: '已报人数', dataIndex: 'signNum', width: COL_WIDTH_M },
  ];

  render() {
    const { onClose, editable } = this.props;
    const { loading, dataSource, totalPeopleNum } = this.state;
    let totalInputPeopleNum = 0;
    let totalHasSignPeople = 0;
    dataSource.forEach(({ campusNum, signNum }) => {
      totalHasSignPeople += signNum;
      totalInputPeopleNum += campusNum || 0;
    });
    return (
      <div>
        <Modal
          title="人数限制"
          loading={loading}
          onCancel={onClose}
          okText="确定"
          onOk={this.onConfirm}
          okBtnShow={editable}
          ref={(c) => (this.modal = getWrappedInstance(c))}
          width={650}
        >
          <div className={Styles.peopleLimitModal}>
            <div className={Styles.topLimit}>
              <FormItem name="产品名额" required="true">
                <InputNumber
                  disabled={!editable}
                  placeholder="请输入产品名额"
                  value={totalPeopleNum}
                  min={1}
                  max={100000}
                  onChange={(val) => this.setState({ totalPeopleNum: val || 1 })}
                  step={1}
                />
              </FormItem>
              <FormItem name="未分配名额">
                <span>{totalPeopleNum - totalInputPeopleNum}</span>
              </FormItem>
              <FormItem name="已报人数">
                <span>{totalHasSignPeople}</span>
              </FormItem>
            </div>
            <div className="flexWrap">
              <div className="flexInnerFit">
                <List columns={this.columns} dataSource={dataSource} />
              </div>
            </div>
            <div className={Styles.limitModalBottom}>
              备注：未设置的学习中心将以未分配名额作为限制，如A和B学习中心未限制，未分配名额有20，则AB学习中心总的名额为20
            </div>
          </div>
        </Modal>
      </div>
    );
  }
}

export default PeopleLimitModal;
