import React from 'react';
import Modal from 'COMPONENT/Modal';
import Select from 'COMPONENT/Select';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Loading from 'COMPONENT/Loading';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Pagination, Card, Row, Col } from 'antd/lib/index';
import { loadCommonDict } from 'UTIL/dict';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import loader from '../loader';
import commonStyles from '../../common/commonStyle.less';
import 'COMPONENT/Page/styles.less';
import styles from '../styles.less';
import { SYSTEM_MARK } from '../../../../constants/config';
import { classTypeDict } from '../../classTypeDict';
// 是否双师
const isTwoTeacher = SYSTEM_MARK === 'TWO_TEACHER';

const pageSizeOptions = ['10', '20', '50'];
let defaultProductCharacter = '';

const defaultCategory = isTwoTeacher ? 'TWO_TEACHER' : 'SMALL_CLASS';
class ProductModal extends React.Component {
  constructor(props) {
    super(props);
    const { productList, yearDict } = this.props;
    const productListObj = (productList || []).reduce((obj, ele) => {
      obj[ele.id] = ele;
      return obj;
    }, {});
    const currentProductVersion = yearDict.find((y) => +y.text === new Date().getFullYear()).val;
    this.defaultValues = {
      category: defaultCategory,
      name: null,
      productCharacter: defaultProductCharacter,
      productQuarter: null,
      productVersion: currentProductVersion,
    };
    const initCategory = productList[0] ? productList[0].category : this.defaultValues.category;
    // 如果是小班产品 要限制季度
    let initProductQuarter = null;
    if (initCategory === 'SMALL_CLASS') {
      initProductQuarter = productList[0] ? productList[0].productQuarterId : this.defaultValues.productQuarter;
    }
    const initProductVersion = productList[0] ? productList[0].productVersionId : this.defaultValues.productVersion;

    this.state = {
      loading: true,
      searchExpand: false,
      listLoading: false,
      selectedList: productList || [],
      selectedListObj: productListObj,

      ...this.defaultValues,
      category: initCategory,
      productQuarter: initProductQuarter,
      productVersion: initProductVersion,

      list: [],
      pageNo: 1,
      pageSize: 10,
      total: null,

      yearDict,
      quarterDict: [],
      characterDict: [],
      classTypeDict: classTypeDict,
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    this.setState({ loading: true });
    const { selectedList, category } = this.state;
    const asyncCalls = [loadCommonDict('PRODUCT_QUARTER'), loadCommonDict('PRODUCT_CHARACTER')];
    const [quarterDict, characterDict] = await Promise.all(asyncCalls);
    defaultProductCharacter = characterDict.filter((character) => /引流/.test(character.text))[0].val;
    const limitQuarterArray = ['TWO_TEACHER', 'SMALL_CLASS'];
    if (limitQuarterArray.indexOf(category) > -1 && !selectedList.length) {
      // 双师或者小班默认季度并且没有选择过课程
      const productQuarter = Array.isArray(quarterDict) && quarterDict.length && quarterDict[0].val;
      this.setState({
        productCharacter: defaultProductCharacter,
        quarterDict,
        productQuarter,
      });
    } else {
      this.setState({
        productCharacter: defaultProductCharacter,
        quarterDict,
      });
    }

    const list = await this.loadListData();
    this.setState({
      characterDict,
      productCharacter: defaultProductCharacter,
      list: list.rows,
      total: list.records,
      pageNo: list.page,
      loading: false,
    });
  };

  loadListData = (params) => {
    const { category, name, productCharacter, productQuarter, productVersion, pageNo, pageSize } = this.state;
    const apiParams = {
      category,
      name,
      productCharacter,
      'productQuarter.id': productQuarter,
      'productVersion.id': productVersion,
      page: pageNo,
      rows: pageSize,
      ...params,
      'organization.id': this.props.branchId || '',
      sidx: 'listAll',
      proStatus: 0,
    };
    // 其他类型需多加参数（限制为考试费）
    if (category === 'OTHERS') {
      apiParams.otherCategory = 'DAT0000000383';
    }
    try {
      return loader.loadProductList(apiParams);
    } catch (e) {
      console.log(e);
      ShowMessage(e.text);
      return [];
    }
  };

  search = async () => {
    this.setState({ listLoading: true });
    const list = await this.loadListData({
      page: 1,
    });
    this.setState({
      listLoading: false,
      list: list.rows,
      total: list.records,
      pageNo: list.page,
    });
  };

  pageChange = async (current, pageSize) => {
    this.setState({ listLoading: true });
    const list = await this.loadListData({
      rows: pageSize,
      page: current,
    });
    this.setState({
      listLoading: false,
      list: list.rows,
      total: list.records,
      pageNo: list.page,
      pageSize,
    });
  };

  reset = () => {
    const { selectedList } = this.state;
    const value = {
      ...this.defaultValues,
    };
    if (selectedList.length) {
      value['category'] = selectedList[0].category;
      value['productVersion'] = selectedList[0].productVersionId;
      value['productQuarter'] = selectedList[0].productQuarterId;
    }
    this.setState(value);
  };

  addProduct = (product) => {
    const { selectedList, selectedListObj } = this.state;
    // 如果是讲座或者其他则不用判断年级
    if (product.category === 'ONE_ON_ONE_COURSE' || product.category === 'ONE_ON_MANY') {
      const repeatList = selectedList.filter((ele) => ele.gradeId === product.gradeId);
      if (repeatList.length) {
        this.modal.showOkTips('同一个年级只允许选择一个产品');
        return;
      }
    }
    const newSelectedList = [...selectedList, product];
    const { category, productVersionId, productQuarterId } = newSelectedList[0];

    let apiParams = {
      selectedList: newSelectedList,
      selectedListObj: {
        ...selectedListObj,
        [product.id]: product,
      },
      category: category,
      productVersion: productVersionId,
    };
    if (category === 'SMALL_CLASS') {
      apiParams.productQuarter = productQuarterId;
    }
    this.setState(apiParams);
  };

  removeProduct = (product) => {
    const { selectedList, selectedListObj } = this.state;
    const newSelectedList = selectedList.filter((ele) => ele.id !== product.id);
    delete selectedListObj[product.id];
    this.setState({
      selectedList: newSelectedList,
    });
  };

  onCommit = async () => {
    this.setState({ loading: true });
    try {
      const { onClose, onConfirm } = this.props;
      const { selectedList } = this.state;
      if (selectedList.length) {
        const productIds = selectedList.map((ele) => ele.id).join(',');
        const activityProductList = await loader.loadActivityProductList({ productIds: productIds });
        await onConfirm(activityProductList);
      } else {
        await onConfirm([]);
      }
      onClose();
    } catch (e) {
      console.log(e);
      this.setState({ loading: false });
    }
  };

  selectCardRender = (selectProduct) => {
    const { gradeName, productVersionName, productQuarterName, productCategoryName } = selectProduct;
    return (
      `${gradeName ? gradeName + '·' : ''}` +
      `${productVersionName}年·` +
      `${productQuarterName ? productQuarterName + '·' : ''}` +
      `${productCategoryName}`
    );
  };

  handleChangeType = (category) => {
    const limitQuarterArray = ['TWO_TEACHER', 'SMALL_CLASS'];
    if (limitQuarterArray.indexOf(category) === -1) {
      this.setState({ category, productQuarter: null });
    } else {
      this.setState({ category });
    }
  };
  render() {
    const { onClose, title } = this.props;
    const {
      loading,
      searchExpand,
      category,
      name,
      productCharacter,
      productQuarter,
      productVersion,
      yearDict,
      quarterDict,
      characterDict,
      classTypeDict,
      list,
      pageNo,
      pageSize,
      total,
      listLoading,
      selectedList,
      selectedListObj,
    } = this.state;
    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        okText="保存"
        cancelText="关闭"
        onOk={this.onCommit}
        className="auto-fit-modal"
        themeConfig={{
          bgc: 'gray',
          footerTopShadow: true,
        }}
        widthSize="XL"
        ref={(c) => (this.modal = getWrappedInstance(c))}
        className="custom-modal"
      >
        <div className={commonStyles.wrap + ' ' + styles.container}>
          <div className={'flexWrap' + ' ' + styles.content}>
            <h2 style={{ padding: '15px 0px 15px', fontSize: '18px' }}>{title}</h2>
            <div className="Page-scroller" style={{ margin: '0', width: 'calc(100% + 28px)', paddingRight: '18px' }}>
              <div className={'custom-SearchBar ' + styles.searchBar} style={{ marginBottom: '5px' }}>
                <SearchBar
                  onSearch={this.search}
                  onReset={this.reset}
                  expand={searchExpand}
                  onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
                >
                  <Input
                    placeholder="产品名称"
                    value={name}
                    onChange={(e) => this.setState({ name: e.target.value })}
                  />
                  <Select
                    placeholder="年份"
                    dict={yearDict}
                    value={productVersion}
                    onChange={(productVersion) => this.setState({ productVersion })}
                    disabled={!!selectedList.length}
                    allowClear={false}
                  />
                  <Select
                    placeholder="季度"
                    dict={quarterDict}
                    value={productQuarter}
                    onChange={(productQuarter) => this.setState({ productQuarter })}
                    disabled={!!selectedList.length}
                  />
                  <Select
                    placeholder="产品性质"
                    dict={characterDict}
                    value={productCharacter}
                    onChange={(productCharacter) => this.setState({ productCharacter })}
                    disabled={true}
                  />
                  <Select
                    placeholder="产品类型"
                    dict={classTypeDict}
                    value={category}
                    allowClear={false}
                    onChange={(category) => this.handleChangeType(category)}
                    disabled={!!selectedList.length}
                  />
                </SearchBar>
              </div>
              {listLoading ? (
                <div className="custom-loading">
                  <Loading loading={true} />
                </div>
              ) : (
                <div className={'flexInnerFit'}>
                  {list.map((ele) => {
                    const {
                      categoryName,
                      name,
                      gradeName,
                      productVersionName,
                      productQuarterName,
                      productCategoryName,
                    } = ele;
                    return (
                      <div key={ele.id} className={styles.card + ' custom-card'}>
                        <Card style={{ width: '100%' }} bodyStyle={{ padding: '22px 20px', display: 'flex' }}>
                          <div className={styles.cardContent}>
                            <div className={styles.cardHeader}>
                              <span className={styles.cardMarkup}>{categoryName}</span>
                              <span className={styles.cardTitle}>{name}</span>
                            </div>
                            <div className="custom-row">
                              <Row>
                                <Col span={4}>
                                  <span className={styles.cardContext}>年级：{gradeName || '无'}</span>
                                </Col>
                                <Col span={4}>
                                  <span className={styles.cardContext}>年份：{productVersionName}年</span>
                                </Col>
                                <Col span={4}>
                                  <span className={styles.cardContext}>季度：{productQuarterName || '无'}</span>
                                </Col>
                                <Col span={5}>
                                  <span className={styles.cardContext}>产品类型：{productCategoryName}</span>
                                </Col>
                              </Row>
                            </div>
                          </div>
                          <div className={styles.cardAction}>
                            {selectedListObj[ele.id] ? (
                              <div className={styles.cardButton + ' ' + styles.disabled}>已选</div>
                            ) : (
                              <div
                                className={styles.cardButton}
                                onClick={() => {
                                  this.addProduct(ele);
                                }}
                              >
                                选择
                              </div>
                            )}
                          </div>
                        </Card>
                      </div>
                    );
                  })}
                  {!list.length ? (
                    ''
                  ) : (
                    <div style={{ marginRight: '30px' }}>
                      <Pagination
                        current={pageNo}
                        total={total}
                        onChange={this.pageChange}
                        showSizeChanger
                        onShowSizeChange={this.pageChange}
                        pageSizeOptions={pageSizeOptions}
                        pageSize={pageSize}
                        showTotal={(total) => `共 ${total} 条`}
                      />
                    </div>
                  )}
                </div>
              )}
            </div>
          </div>
          <div className={styles.vrContainer}>
            <div className={styles.vr}></div>
          </div>
          <div className={'flexWrap' + ' ' + styles.content}>
            <h2 style={{ padding: '15px 0px', fontSize: '18px' }}>已选产品（{selectedList.length}）</h2>
            <div className={'flexInnerFit'}>
              <div className="Page-scroller" style={{ margin: '0', width: 'calc(100% + 20px)', paddingRight: '10px' }}>
                <div className={'flexInnerFit'}>
                  {selectedList.map((ele) => {
                    const { categoryName, name } = ele;
                    return (
                      <div key={ele.id} className={styles.card + ' custom-card'}>
                        <Card style={{ width: '100%' }} bodyStyle={{ padding: '22px 20px', display: 'flex' }}>
                          <div className={styles.cardContent}>
                            <div className={styles.cardHeader}>
                              <span className={styles.cardMarkup}>{categoryName}</span>
                              <span className={styles.cardTitle}>{name}</span>
                            </div>
                            <div className={'custom-row ' + styles.cardContext}>{this.selectCardRender(ele)}</div>
                          </div>
                          <div
                            className={styles.deleteBtn}
                            onClick={() => {
                              this.removeProduct(ele);
                            }}
                          ></div>
                        </Card>
                      </div>
                    );
                  })}
                </div>
              </div>
            </div>
          </div>
        </div>
      </Modal>
    );
  }
}

export default ProductModal;
