import React from 'react';
import Modal from 'COMPONENT/Modal';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { loadCommonDict, loadBranchCampusDict, loadClassLevelDict, getDictTextByVal } from 'UTIL/dict';
import loader from '../loader';
import commonStyles from '../../common/commonStyle.less';
import Styles from '../styles.less';
import { SYSTEM_MARK } from '../../../../constants/config';
// 是否双师
const isTwoTeacher = SYSTEM_MARK === 'TWO_TEACHER';

const { COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

class ClassModal extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: true, // boolean 页面加载状态
      searchExpand: false,

      blCampusId: null,
      classLevel: null,
      className: null,
      subjectId: null,
      phase: null,
      productType: null,
      campusDict: [],
      subjectDict: [],
      classLevelDict: [],
      dictQuarter: [], // 字典 季度
      dictPhaseWeekDay: [], // 字典 期数/星期数(实际使用)
      dictClassPhase: [], // 字典 期数
      dictWeekDay: [], // 字典 星期数
      listLoading: false,
      rows: [],
      pageNo: 1,
      pageSize: 20,
      total: null,

      selectedRowKeys: [],
    };

    this.defaultBlCampus = '';
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    this.setState({ loading: true });
    const { product, branchId, defaultBlCampus } = this.props;
    try {
      const asyncCalls = [
        loadBranchCampusDict(),
        loadCommonDict('SUBJECT'),
        loadClassLevelDict(product.classTypeId),
        loadCommonDict('PRODUCT_QUARTER'),
        loadCommonDict('SMALL_CLASS_PHASE'),
        loadCommonDict('WeekDay'),
      ];
      let [campusDict, subjectDict, classLevelDict, mapQuarter, mapClassPhase, mapWeekDay] = await Promise.all(
        asyncCalls,
      );
      //获取列表数据
      // campusDict = campusDict.filter((ele) => ele.val === branchId);
      this.defaultBlCampus =
        defaultBlCampus == null
          ? campusDict[0] && campusDict[0]['options'][0]
            ? campusDict[0]['options'][0]['val']
            : ''
          : defaultBlCampus;
      const isContractCampusIdExited = !!(campusDict || []).find(
        ({ options }) => !!(options || []).find(({ val }) => val === defaultBlCampus),
      );
      if (!isContractCampusIdExited) {
        campusDict.unshift({
          val: '-1',
          label: '学生学籍/意向学习中心',
          options: [{ val: defaultBlCampus, text: this.props.defaultBlCampusName }],
        });
      }
      this.setState({
        campusDict,
        subjectDict,
        classLevelDict,
        blCampusId: this.defaultBlCampus,
        dictQuarter: mapQuarter.filter((item) => item.text !== '全年'),
        dictClassPhase: mapClassPhase,
        dictWeekDay: mapWeekDay,
      });
      const res = await this.loadListData({
        blCampusId: this.defaultBlCampus,
      });
      this.setState({
        rows: res.rows,
        total: res.records,
        pageNo: res.page,
      });
    } catch (e) {
      console.log(e);
    }
    this.setState({ loading: false });
  };

  loadListData = (params) => {
    const { product } = this.props;
    const { blCampusId, classLevel, className, subjectId, phase, quarter, pageNo: page, pageSize: rows } = this.state;
    const apiParams = {
      blCampusId,
      classLevel,
      className,
      page,
      rows,
      phase,
      subjectId,
      ...params,
    };
    let extraParams = {};
    //处理双师的数据
    if (isTwoTeacher) {
      extraParams.productQuarter = quarter;
    } else {
      extraParams.quarter = quarter;
    }
    apiParams['productId'] = product.id;
    apiParams['classTypeId'] = product.classTypeId;
    switch (product.category) {
      case 'SMALL_CLASS':
        return loader.loadMiniClassList(apiParams);
      case 'TWO_TEACHER':
        return loader.loadTwoTeacherClassList(apiParams);
      default:
        throw new Error(`invalid product type`);
    }
  };

  search = async () => {
    await this.setState({ listLoading: true, pageNo: 1 });
    const res = await this.loadListData();
    this.setState({
      listLoading: false,
      rows: res.rows,
      total: res.records,
      pageNo: res.page,
      selectedRowKeys: [],
    });
  };

  reset = async () => {
    this.setState({
      blCampusId: this.defaultBlCampus,
      classLevel: null,
      className: null,
      subjectId: null,
      phase: null,
      quarter: null,
    });
  };

  pageChange = async ({ current, pageSize }) => {
    await this.setState({ listLoading: true, pageSize, pageNo: current });
    const res = await this.loadListData();
    this.setState({
      listLoading: false,
      rows: res.rows,
      total: res.records,
      pageNo: res.page,
      pageSize,
      selectedRowKeys: [],
    });
  };

  onConfirm = async () => {
    const { onConfirm, product, order } = this.props;
    const { selectedRowKeys, rows } = this.state;
    if (!selectedRowKeys.length) {
      this.modal.showOkTips('请选择班级');
      return;
    }
    const selectedClass = rows[selectedRowKeys[0]];
    // 获取开始上课时间
    const { classTime, classTimelength, everyCourseClassNum = 20 } = selectedClass;
    selectedClass.firstClassTime = this.calCourseTime(classTime, classTimelength, everyCourseClassNum);
    try {
      switch (product.category) {
        case 'SMALL_CLASS':
          await loader.checkAllowAddStudent4MiniClass(
            {
              studentIds: order.studentNo,
              smallClassId: selectedClass.classId,
            },
            true,
          );
          break;
        case 'TWO_TEACHER':
          await loader.checkAllowAddStudent4TwoTeacherClass(
            {
              studentIds: order.studentNo,
              classTwoId: selectedClass.classTwoId,
            },
            true,
          );
          break;
        default:
          throw new Error(`invalid product type`);
      }
    } catch (err) {
      this.modal.showOkTips(err.message || err.text);
      return;
    }
    // 外层都是使用classId 这里对双师的classTwoId做处理
    if (product.category === 'TWO_TEACHER') {
      selectedClass.classId = selectedClass.classTwoId;
    }
    onConfirm({
      selectedClass,
    });
  };

  listRowClick = (record, index, event) => {
    this.setState({ selectedRowKeys: [index] });
  };
  /*      -----------------------一下为新增产品季度以及星期数加的代码---------------*/

  buildPhaseDict(quarterVal) {
    const { dictQuarter, dictClassPhase, dictWeekDay } = this.state;
    let quarterText = getDictTextByVal(dictQuarter, quarterVal);
    if (quarterText === '暑假' || quarterText === '寒假') {
      return dictClassPhase;
    } else if (quarterText === '春季' || quarterText === '秋季') {
      return dictWeekDay;
    } else {
      return null;
    }
  }

  /**
   * 更新搜索关键词
   * */
  updateKeyword = (params) => {
    const { type, val } = params;
    switch (type) {
      case 'quarter':
        if (!val) {
          this.setState({ phase: null });
        } else {
          this.setState({ dictPhaseWeekDay: this.buildPhaseDict(val) });
        }
        break;
      default:
      // ...
    }
    this.updateState({ [type]: val });
  };

  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };

  /**
   * 计算上课时间
   * @param courseTimeStart
   * @param timeLong
   * @param planHours
   * @returns {string}
   */
  calCourseTime = (courseTimeStart, timeLong, planHours) => {
    let courseTimeValues = courseTimeStart.split(':');
    let d = new Date();
    d.setHours(courseTimeValues[0]);
    d.setMinutes(courseTimeValues[1]);
    d.setMinutes(d.getMinutes() + timeLong * planHours);
    return `${courseTimeStart} - ${d.getHours()}:${d.getMinutes() >= 10 ? d.getMinutes() : `0${d.getMinutes()}`}`;
  };
  render() {
    const { onClose, product } = this.props;
    const {
      loading,
      searchExpand,
      blCampusId,
      classLevel,
      className,
      subjectId,
      phase,
      quarter,
      campusDict,
      subjectDict,
      classLevelDict,
      dictQuarter,
      dictPhaseWeekDay,
      listLoading,
      rows,
      pageNo,
      pageSize,
      total,
      selectedRowKeys,
    } = this.state;

    const isTwoTeacherType = product.category === 'TWO_TEACHER';
    let columns = [
      {
        title: '班级名称',
        dataIndex: 'className',
        key: 'className',
        width: COL_WIDTH_XL,
        render: (text, record) => record[isTwoTeacherType ? 'classTwoName' : 'className'],
      },
      {
        title: isTwoTeacherType ? '辅导老师' : '任课老师',
        dataIndex: 'teacherName',
        key: 'teacherName',
        width: COL_WIDTH_M,
      },
      {
        title: '开始上课日期',
        dataIndex: 'startDate',
        key: 'startDate',
        width: COL_WIDTH_L,
      },
      {
        title: '上课时间',
        dataIndex: 'classTime',
        key: 'classTime',
        width: COL_WIDTH_L,
        render: (text, record) => {
          const { classTime, classTimelength, everyCourseClassNum = 20, classTimeLength } = record;
          const classTimeLengthData = isTwoTeacherType ? classTimeLength : classTimelength;
          return this.calCourseTime(classTime, classTimeLengthData, everyCourseClassNum);
        },
      },
      {
        title: '报名情况',
        key: 'regInfo',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { peopleQuantity, enrolledNum } = record;
          return `${enrolledNum || 0}/${peopleQuantity || 0}`;
        },
      },
      {
        title: '课时单价',
        dataIndex: 'unitPrice',
        key: 'unitPrice',
        width: COL_WIDTH_M,
      },
      {
        title: '已上讲数',
        dataIndex: 'chargedCourseNum',
        key: 'chargedCourseNum',
        width: COL_WIDTH_M,
      },
      {
        title: '未上讲数',
        dataIndex: 'unChargedCourseNum',
        key: 'unChargedCourseNum',
        width: COL_WIDTH_M,
      },
    ];
    if (isTwoTeacherType) {
      columns.splice(2, 0, {
        title: '主讲老师',
        dataIndex: 'mainTeacherName',
        key: 'mainTeacherName',
        width: COL_WIDTH_M,
      });
    }

    return (
      <Modal
        loading={loading}
        title={`报班（产品：${product.name}）`}
        onCancel={onClose}
        okText="确定"
        onOk={this.onConfirm}
        ref={(c) => (this.modal = getWrappedInstance(c))}
        width={1200}
      >
        <div className={commonStyles.wrap}>
          <div className="flexWrap">
            <div className={Styles.activitySearchBar}>
              <SearchBar
                onSearch={this.search}
                onReset={this.reset}
                expand={searchExpand}
                onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
              >
                <Input
                  placeholder="班级名称"
                  value={className}
                  onChange={(e) => this.setState({ className: e.target.value })}
                />
                <Select
                  size="large"
                  placeholder="校区"
                  dict={campusDict}
                  value={blCampusId}
                  allowClear={false}
                  showSearch={true}
                  onChange={(blCampusId) =>
                    this.setState({ blCampusId }, () => {
                      this.search();
                    })
                  }
                />
                <Select
                  placeholder="科目"
                  showSearch={true}
                  value={subjectId}
                  dict={subjectDict}
                  allowClear={true}
                  onChange={(subjectId) => this.setState({ subjectId })}
                />
                <Select
                  placeholder="班级层次"
                  dict={classLevelDict}
                  value={classLevel}
                  allowClear={true}
                  onChange={(classLevel) => this.setState({ classLevel })}
                />
                <Select
                  placeholder="产品季度"
                  dict={dictQuarter}
                  value={quarter}
                  onChange={(val) => this.updateKeyword({ type: 'quarter', val: val })}
                />
                <Select
                  placeholder="期数/星期数"
                  dict={dictPhaseWeekDay}
                  value={phase}
                  onChange={(val) => this.updateKeyword({ type: 'phase', val: val })}
                  disabled={!quarter}
                />
              </SearchBar>
            </div>
            <div className="flexInnerFit">
              <List
                columns={columns}
                loading={listLoading}
                dataSource={rows}
                pagination={{
                  current: pageNo,
                  pageSize,
                  total: total == null ? 0 : total,
                }}
                onChange={this.pageChange}
                rowSelection={{
                  type: 'radio',
                  selectedRowKeys,
                  onChange: (keys) => {
                    this.setState({ selectedRowKeys: keys });
                  },
                }}
                onRowClick={this.listRowClick}
              />
            </div>
          </div>
        </div>
      </Modal>
    );
  }
}

export default ClassModal;
