import React from 'react';
import moment from 'moment';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import TextArea from 'COMPONENT/TextArea';
import DatePicker from 'COMPONENT/DatePicker';
import Select from 'COMPONENT/Select';
import TimePicker from 'COMPONENT/TimePicker';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { loadCommonDict, loadAllBranchCampusDict } from 'UTIL/dict';
import loaders from './loaders';

const formDict = {
  satisficing: '客户意向度',
  appointCampusId: '预约上门学习中心',
  appointDate: '预约上门时间',
  appointDatetime: '预约上门时间',
  remark: '备注',
};

class AppointmentModal extends React.Component {
  constructor(props) {
    super(props);
    this.defaultDicts = {
      customerSatisficingDict: [],
      campusDict: [],
    };

    this.state = Object.assign(
      {
        loading: true,
        satisficing: '',
        appointCampusId: '',
        appointDate: '',
        appointDatetime: '',
        remark: '',
        customerId: '',
        appointmentId: '',
        disabled: true,
      },
      this.defaultDicts,
    );
  }

  getDisabledDate = (date) => {
    if (!date) {
      return false;
    }
    return moment(date).valueOf() < moment(new Date()).valueOf();
  };

  onCommit = async () => {
    const { onClose, onCommit } = this.props;
    const appointmentSetting = {
      appointmentId: this.state.appointmentId,
      customerId: this.state.customerId,
      satisficing: this.state.satisficing,
      appointCampusId: this.state.appointCampusId,
      appointDate: this.state.appointDate,
      appointDatetime: this.state.appointDatetime,
      remark: this.state.remark ? this.state.remark.trim() : '',
      id: this.state.id,
      appointmentType: 'APPOINTMENT',
      meetingTime: this.state.appointDate + ' ' + this.state.appointDatetime,
    };

    const appointmentSettingKeys = Object.keys(formDict);
    for (let i = 0; i < appointmentSettingKeys.length; i++) {
      if (!!!appointmentSetting[appointmentSettingKeys[i]]) {
        this._modal.showOkTips(`请补充${formDict[appointmentSettingKeys[i]]}`);
        return;
      }
    }

    this.setState({ loading: true });
    const res = await loaders.updateAppointmentRecord(appointmentSetting);
    if (res.resultCode == 0) {
      onCommit();
      onClose();
      showMessage('操作成功', 'success');
    } else {
      showMessage('操作失败：' + res.resultMessage, 'error');
    }
  };

  componentDidMount() {
    this.setState({ loading: true });
    Promise.all([
      loadCommonDict('CUSTOMER_SATISFICING'),
      loadAllBranchCampusDict(),
      loaders.customerMeetingConfirm({ customerId: this.props.data.cusId }),
    ]).then(([customerSatisficingDict, campusDict, customerMeetingConfirm]) => {
      this.setState({
        customerSatisficingDict,
        campusDict,
      });
      if (customerMeetingConfirm.resultCode == 0) {
        loaders.lastAppointmentRecord({ customerId: this.props.data.cusId }).then((res) => {
          this.setState({
            loading: false,
            appointmentId: res.id,
            customerId: res.customerId,
            satisficing: res.satisficing,
            appointCampusId: res.appointCampusId,
            appointDate: res.meetingTime.substring(0, 10),
            appointDatetime: res.meetingTime.substring(11, 16),
            remark: res.remark,
            id: res.id,
            disabled: false,
          });
        });
      } else {
        this.setState({
          loading: false,
          disabled: true,
        });
      }
    });
  }

  render() {
    const { onClose } = this.props;
    const {
      satisficing,
      appointCampusId,
      appointDate,
      appointDatetime,
      remark,
      loading,
      disabled,
      customerSatisficingDict,
      campusDict,
    } = this.state;
    return (
      <Modal
        loading={loading}
        title="预约上门"
        onCancel={onClose}
        okText="保存"
        onOk={this.onCommit}
        cancelText={disabled ? '关闭' : '取消'}
        okBtnShow={!disabled}
        ref={(c) => (this._modal = getWrappedInstance(c))}
      >
        <FormItem name="客户意向度" required="true">
          <Select
            placeholder="客户意向度"
            dict={customerSatisficingDict}
            value={satisficing}
            onChange={(val) => this.setState({ satisficing: val })}
            disabled={disabled}
          />
        </FormItem>
        <FormItem name="预约上门学习中心" required="true">
          <Select
            placeholder="请选择学习中心"
            showSearch
            dict={campusDict}
            value={appointCampusId}
            onChange={(val) => this.setState({ appointCampusId: val })}
            disabled={disabled}
          />
        </FormItem>
        <FormItem name="预约上门时间" required="true">
          <DatePicker
            placeholder={moment(appointDate).valueOf() < moment(new Date()).valueOf() ? appointDate : '请选择日期'}
            value={appointDate}
            onChange={(val) => this.setState({ appointDate: val })}
            disabledDate={this.getDisabledDate}
            disabled={disabled}
          />
          <TimePicker
            value={appointDatetime}
            onChange={(val) => this.setState({ appointDatetime: val })}
            disabled={disabled}
          />
        </FormItem>
        <FormItem name="备注" required="true">
          <TextArea value={remark} onChange={(e) => this.setState({ remark: e.target.value })} disabled={disabled} />
        </FormItem>
      </Modal>
    );
  }
}
export default AppointmentModal;
