import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import { isIdCard } from 'UTIL/idCardValidation';
import { Button, Section, PopTip, Block, FormItem, Input, Select, DatePicker, TextArea, Loading } from 'COMPONENT';
import { getCommonErrorTips, getFieldWithFiller } from 'COMPONENT/utils/form';
import CustomerInfo from '../components/CustomerInfo';
import { getCustomerDetail, updateCustomer } from 'SERVICE/outcallWorkbench';
import { loadSourceDetailDict } from 'PAGE/common/loaders';
import { loadCommonDictNew } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import styles from '../styles.less';

const formItemLayout = {
  labelWidth: '100px',
  style: {
    padding: 0,
    marginBottom: 0,
  },
};

const statusMap = {
  STAY_FOLLOW: '待跟进',
  FOLLOWING: '跟进中',
};

class CustomerPanel extends React.Component {
  constructor(props) {
    super(props);
    const { pageParams } = props;
    this.params = pageParams[`/${props.route.path}`];
    this.errors = {};
    this.state = {
      isEdit: false,
      hasError: false,
      gradeDict: [],
      sourceDetailDict: [],
      customer: {},
      isChecking: false,
      errorTip: '',
    };
  }

  componentDidMount() {
    this.initDict();
  }

  initDict = () => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch' /*,"grade"*/, 'campus', 'networkCustomerAllocateStatus', 'resEntrance', 'resType'],
        callback: this.getCustomerDetail,
      },
    });
    loadCommonDictNew('STUDENT_GRADE').then((gradeDict) => {
      this.setState({ gradeDict });
    });
  };

  checkIsIdCard = (val) => {
    if (val) {
      return isIdCard(val) ? null : '身份证输入有误';
    } else {
      return null;
    }
  };

  checkIsPhone = (val) => {
    if (val) {
      return /^1[3-9][0-9]{9}$/.test(val) ? null : '电话号码有误';
    } else {
      return null;
    }
  };

  loadSourceDetailDict = async (val) => {
    const sourceDetailDict = await loadSourceDetailDict(val);
    if (sourceDetailDict) {
      this.setState({ sourceDetailDict });
    }
  };

  getCustomerDetail = async () => {
    this.setState({ loading: true });
    // 资源类型 1：外呼资源 2：普通资源(Integer)
    const { resourceType = 1, customerId = '', studentId = '', deliverTarget = '' } = this.params;
    const { data } = await getCustomerDetail({
      studentId,
      customerId,
      resourceType,
      deliverTarget,
    });
    if (data) {
      this.loadSourceDetailDict(data.cusType);
      this.setState({ customer: data });
      this.props.getCustomerInfo(data);
    }
    this.setState({ loading: false });
  };

  toggleEdit = () => {
    this.setState((preState) => ({ isEdit: !preState.isEdit, isChecking: false }));
  };

  onEdit = (key) => {
    return function handleEdit(e) {
      let { customer } = this.state;
      let update = {};
      update[key] = e ? (e.target ? e.target.value : e) : '';
      this.setState({ customer: { ...customer, ...update } });
      // 级联查询
      if (key === 'cusType') {
        this.setState({ customer: { ...customer, cusOrg: '', cusType: update[key] } });
        let value = e ? (e.target ? e.target.value : e) : '';
        if (value) {
          this.loadSourceDetailDict(value);
        }
      }
    };
  };

  errorChange = (key, e) => {
    this.errors[key] = e;
  };

  onSave = () => {
    this.setState({ isChecking: true }, () => {
      let errorTip = getCommonErrorTips(this.errors);
      if (errorTip) {
        this.setState({ errorTip, hasError: true });
      } else {
        this.handleUpdate();
      }
    });
  };

  handleUpdate = async () => {
    this.setState({ loading: true });
    try {
      const { resourceType = 1, customerId = '', studentId = '', deliverTarget = '' } = this.params;
      const { resultCode } = await updateCustomer({
        studentId,
        customerId,
        resourceType,
        deliverTarget,
        ...this.state.customer,
      });
      if (resultCode == '0') {
        showMessage('更新成功', 'success');
        this.toggleEdit();
        setTimeout(() => {
          this.getCustomerDetail();
        }, 200);
      }
    } finally {
      this.setState({ loading: false });
    }
  };

  renderFormContent() {
    const { customer, loading, isChecking } = this.state;
    return (
      <Block>
        <Loading loading={loading} />
        <Row className={styles.edit_form_item} gutter={20}>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="学生姓名：">
              <Input
                value={customer.studentName}
                placeholder="请输入学生姓名"
                onChange={this.onEdit('studentName').bind(this)}
                validator={{
                  rules: ['required'],
                  silent: !isChecking,
                  onErrorChange: (e) => this.errorChange('studentName', e),
                }}
              />
            </FormItem>
          </Col>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="年级：">
              <Select
                showSearch={true}
                value={customer.gradeId}
                dict={this.state.gradeDict}
                onChange={this.onEdit('gradeId').bind(this)}
                placeholder="请选择年级"
              />
            </FormItem>
          </Col>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="学校：">
              <Input
                value={customer.schoolName}
                onChange={this.onEdit('schoolName').bind(this)}
                placeholder="请输入学校"
              />
            </FormItem>
          </Col>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="学生电话：">
              <Input
                value={customer.studentContact}
                onChange={this.onEdit('studentContact').bind(this)}
                placeholder="请输入电话"
                validator={{
                  rules: [this.checkIsPhone],
                  silent: !isChecking,
                  onErrorChange: (e) => this.errorChange('studentContact', e),
                }}
              />
            </FormItem>
          </Col>
        </Row>
        <Row className={styles.edit_form_item} gutter={20}>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="客户姓名：">
              <Input
                value={customer.customerName}
                onChange={this.onEdit('customerName').bind(this)}
                placeholder="请输入客户姓名"
                validator={{
                  rules: ['required'],
                  silent: !isChecking,
                  onErrorChange: (e) => this.errorChange('customerName', e),
                }}
              />
            </FormItem>
          </Col>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="客户电话：">
              <span>{getFieldWithFiller(customer.customerContact)}</span>
            </FormItem>
          </Col>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="出生日期：">
              <DatePicker
                value={customer.birthday}
                onChange={this.onEdit('birthday').bind(this)}
                placeholder="请选择客户出生日期"
              />
            </FormItem>
          </Col>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="客户身份证：">
              <Input
                value={customer.idCardNo}
                onChange={this.onEdit('idCardNo').bind(this)}
                placeholder="请输入客户身份证"
                validator={{
                  rules: [this.checkIsIdCard],
                  silent: !isChecking,
                  onErrorChange: (e) => this.errorChange('idCardNo', e),
                }}
              />
            </FormItem>
          </Col>
        </Row>
        <Row className={styles.edit_form_item} gutter={20}>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="父亲姓名：">
              <Input
                value={customer.fatherName}
                onChange={this.onEdit('fatherName').bind(this)}
                placeholder="请输入学生父亲姓名"
              />
            </FormItem>
          </Col>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="父亲电话：">
              <Input
                value={customer.fatherPhone}
                onChange={this.onEdit('fatherPhone').bind(this)}
                placeholder="请输入学生父亲电话"
                validator={{
                  rules: [this.checkIsPhone],
                  silent: !isChecking,
                  onErrorChange: (e) => this.errorChange('fatherPhone', e),
                }}
              />
            </FormItem>
          </Col>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="母亲姓名：">
              <Input
                value={customer.motherName}
                onChange={this.onEdit('motherName').bind(this)}
                placeholder="请输入学生父亲姓名"
              />
            </FormItem>
          </Col>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="母亲电话：">
              <Input
                value={customer.motherPhone}
                onChange={this.onEdit('motherPhone').bind(this)}
                placeholder="请输入学生母亲电话"
                validator={{
                  rules: [this.checkIsPhone],
                  silent: !isChecking,
                  onErrorChange: (e) => this.errorChange('motherPhone', e),
                }}
              />
            </FormItem>
          </Col>
        </Row>
        <Row className={styles.edit_form_item} gutter={20}>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="家庭住址：">
              <Input
                value={customer.address}
                onChange={this.onEdit('address').bind(this)}
                placeholder="请输入学生家庭住址"
              />
            </FormItem>
          </Col>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="备注：">
              <TextArea
                maxLength={150}
                value={customer.remark}
                onChange={this.onEdit('remark').bind(this)}
                placeholder="备注"
              />
            </FormItem>
          </Col>
        </Row>
        <Row className={styles.edit_form_item} gutter={20}>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="所属组织：">
              <span>{getFieldWithFiller(customer.blOrganizationName)}</span>
            </FormItem>
          </Col>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="分配人：">
              <span>{getFieldWithFiller(customer.deliverOperateUserName)}</span>
            </FormItem>
          </Col>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="跟进人：">
              <span>{getFieldWithFiller(customer.deliverTargetName)}</span>
            </FormItem>
          </Col>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="跟进状态：">
              <span>{statusMap[customer.dealStatus]}</span>
            </FormItem>
          </Col>
        </Row>
        <Row className={styles.edit_form_item} gutter={20}>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="资源入口：">
              <span>{getFieldWithFiller(customer.resEntranceName)}</span>
            </FormItem>
          </Col>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="资源来源：">
              <Select
                allowClear={true}
                showSearch={true}
                dict={this.props.resTypeDict}
                value={customer.cusType}
                onChange={this.onEdit('cusType').bind(this)}
                placeholder="请选择资源来源"
                validator={{
                  rules: ['required'],
                  silent: !isChecking,
                  onErrorChange: (e) => this.errorChange('cusType', e),
                }}
              />
            </FormItem>
          </Col>
          <Col className={styles.form_column} span={6}>
            <FormItem {...formItemLayout} name="来源细分：">
              <Select
                allowClear={true}
                showSearch={true}
                dict={this.state.sourceDetailDict}
                value={customer.cusOrg}
                onChange={this.onEdit('cusOrg').bind(this)}
                placeholder="请选择来源细分"
                validator={{
                  rules: ['required'],
                  silent: !isChecking,
                  onErrorChange: (e) => this.errorChange('cusOrg', e),
                }}
              />
            </FormItem>
          </Col>
        </Row>
      </Block>
    );
  }

  render() {
    const { canEdit } = this.params;
    const { isEdit, hasError, customer, errorTip } = this.state;
    return (
      <Section
        title="基本信息"
        toolBar={
          canEdit ? (
            isEdit ? (
              [
                <Button onClick={this.toggleEdit} style={{ marginRight: '10px' }} key="cancel">
                  取消
                </Button>,
                <PopTip
                  key="save"
                  visible={hasError}
                  content={errorTip}
                  onClose={() => this.setState({ hasError: false })}
                >
                  <Button type="primary" onClick={this.onSave}>
                    保存
                  </Button>
                </PopTip>,
              ]
            ) : (
              <Button type="primary" onClick={this.toggleEdit}>
                编辑
              </Button>
            )
          ) : null
        }
      >
        {isEdit ? this.renderFormContent() : <CustomerInfo customer={customer} />}
      </Section>
    );
  }
}

export default connect((state) => {
  return {
    // gradeDict: state.dict["grade"],
    resTypeDict: state.dict['resType'],
    pageParams: state.main.pageParams,
  };
})(CustomerPanel);
