import React from 'react';
import { connect } from 'dva';
import styles from './styles.less';
import { Row, Col, Tooltip } from 'antd';
import { Input, Select, Checkbox, FormItem, TextArea, Block, Button, TextLink } from 'COMPONENT';
import NewStudent from './NewStudent';
import loaders from './loaders';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import { loadSourceDetailDict, loadConsultantDict } from 'PAGE/common/loaders';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { checkPhone } from 'COMPONENT/Validator/checkers';
import {
  addCusFormCfg,
  addCusModalRowStyle,
  noop,
  calcWorkbenchReception,
  formatCustomerFormItemConf,
  formatCustomerInitState,
} from './utils';
import { loadCommonDict, loadCommonDictNew, loadAllBranchCampusDict } from 'UTIL/dict';
import { loginUser, isIdCard } from 'UTIL/common';
import CusInfoLayout from './CusInfoLayout';
import { entranceDict, entranceDict2, ENTRANCE_DIRECT, ENTRANCE_PULL, CERTIFICATE_TYPE_ID_CARD } from './dicts';

const linkFormCfg = {
  labelWidth: 40,
  style: { padding: '0 10px', marginBottom: 10, lineHeight: '24px' },
};

/**
 * 录入客户表单
 *
 * @param props.workBench {string} 工作台 `WORKBENCH_CALL_OUT|WORKBENCH_RECEPTION`
 * @param props.callType {string} `FRONT_DESK|CONSULTANT`
 * @param props.onLoadingStart {function}
 * @param props.onLoadingEnd {function}
 * @param props.onSaveSuccess {function}
 * @param props.onError {function}
 * @param props.customerFormItemConf {object} 客户信息表单项配置,根据workBench参数生成不同配置
 * @param props.studentFormItemConf {object} 学生信息表单项配置(未完成)
 * @param props.btnAddStudentVisible {boolean} 是否显示添加学生按钮 默认true
 */
class AddCustomerMain extends React.Component {
  static defaultProps = {
    workBench: '',
    callType: '',
    onLoadingStart: noop,
    onLoadingEnd: noop,
    onSaveSuccess: noop,
    onError: noop,
    customerFormItemConf: null,
    studentFormItemConf: null,
    btnAddStudentVisible: true,
    isConsultant: false,
  };

  constructor(props) {
    super(props);

    const campusId = props.userCampusId || null;
    const initCustomerState = formatCustomerInitState(props.customerFormItemConf);

    this.customerFormItemConf = formatCustomerFormItemConf(props.workBench, props.customerFormItemConf);
    this.initStates = {
      checking: false, // {boolean} 校验开启状态

      customerId: null, // {string} 客户ID
      name: null, // {string} 客户姓名
      phone: null, // {string} 客户电话
      entrance: null, // {string} 资源入口
      source: null, // {string} 资源来源
      sourceDetail: null, // {string} 来源细分
      campus: campusId, // {string} 意向学习中心
      follower: null, // {string} 分配跟进人
      customerSign: null, // 客户意向产品
      remark: '', // {string} 备注
      cusIdCardNo: '', // {string} 客户身份证号码
      cusIdCardType: null, // 证件类型

      sourceDetailDict: [], // 字典，来源细分
      linkStudents: [], // 关联学生 [{ id, name, grade, schoolName, follower,gradeID, schoolId }]
      checkedLinkStudents: [], // 选中的关联学生ID [{canFollow, follower, gradeId, schoolId, gradeName, id, name, schoolName}]
      newStudents: [{ id: '1' }], // 添加的学生 [{ name, grade, cls, school, schoolName, sex, birthday, contract, address, idCardNo }]

      ...initCustomerState,
    };
    this.state = {
      ...this.initStates,
      userProvince: null, // {string} 当前登录用户所属省份
      userCity: null, // {string} 当前登录用户所属城市
      gradeDict: [], // {object[]} 字典，年级
      sourceDict: [], // {object[]} 字典，资源来源
      campusDict: [], // {object[]} 字典，意向学习中心
      followerDict: [], // {object[]} 字典，分配跟进人
      customerSignDict: [],
      certificateTypeDict: [], // 字典，证件类型
    };
    this.errors = {};
    this.newStudentIdx = 2;
  }

  componentDidMount() {
    const { onLoadingStart, onLoadingEnd } = this.props;
    onLoadingStart();

    Promise.all([
      loadCommonDictNew('STUDENT_GRADE'),
      loadCommonDict('RES_TYPE'),
      loadAllBranchCampusDict(),
      loadConsultantDict(),
      loaders.loadLoginUserProvinceCity(),
      loadCommonDict('CustomerSignType'),
      loadCommonDict('CERTIFICATE_TYPE'),
    ]).then(
      ([gradeDict, sourceDict, campusDict, followerDict, provinceCity, customerSignDict, certificateTypeDict]) => {
        onLoadingEnd();
        const { province, city } = provinceCity;
        this.setState({
          gradeDict,
          sourceDict,
          campusDict,
          followerDict,
          userProvince: province,
          userCity: city,
          customerSignDict,
          certificateTypeDict,
        });
      },
    );

    this.initContent();
  }

  /**
   * 初始化的一些动作
   * 1、判断有无初始客户电话，查询关联学生
   */
  initContent = () => {
    const { phone } = this.customerFormItemConf;

    if (phone && phone.initialValue) {
      this.phoneChange(phone.initialValue);
    }
    this.canWriteIdNumber();
  };

  reset = () => {
    this.setState(this.initStates);
    this.newStudentIdx = 2;
  };

  onOk = () => {
    const { onSaveSuccess, onError, workBench, onLoadingStart, onLoadingEnd } = this.props;
    const {
      customerId,
      name,
      phone,
      entrance,
      source,
      sourceDetail,
      campus,
      follower,
      remark,
      cusIdCardNo,
      cusIdCardType,
      linkStudents,
      checkedLinkStudents,
      newStudents: newStudentsState,
      checking,
      customerSign,
    } = this.state;

    if (!checking) {
      this.setState({ checking: true });
    }

    const errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      onError(errorTips);
      return;
    }

    const shouldReturn = (cusIdCardType && !cusIdCardNo) || (!cusIdCardType && cusIdCardNo);
    if (shouldReturn) {
      const msg = !cusIdCardType ? '请选择证件类型！' : '请填写证件号码！';
      return onError(msg);
    }

    // 校验身份证号码
    if (cusIdCardType) {
      if (cusIdCardType === CERTIFICATE_TYPE_ID_CARD && !isIdCard(cusIdCardNo)) {
        return onError('客户身份证格式错误');
      } else if (!cusIdCardNo.trim()) {
        return onError('客户证件号码错误');
      }
    }

    const newStudents = newStudentsState.filter((s) => s.name && s.name.trim());
    if (customerId && newStudents.length === 0 && checkedLinkStudents.length === 0) {
      onError('请至少选择/填写一个学生姓名');
      return;
    }

    if (newStudents.length !== 0 && (entrance === 'DIRECT_VISIT' || entrance === 'OUTSTANDING_VISIT')) {
      const gradeIds = newStudents.filter((student) => {
        return student.grade;
      });
      if (gradeIds.length !== newStudents.length) {
        onError('新录入学生信息中,学生年级不能为空!');
        return;
      }
    }

    if (newStudents.length) {
      let status = newStudents.every((value) => !value.idCardNo || isIdCard(value.idCardNo));
      if (!status) {
        return onError('学生身份证格式错误');
      }
    }

    let stuNames = [...newStudents.map((s) => s.name.trim()), ...linkStudents.map((s) => s.name.trim())];
    if (Array.from(new Set(stuNames)).length < stuNames.length) {
      onError('学生姓名不能重名');
      return;
    }
    const saveData = {
      workBench,
      customerId,
      name,
      phone,
      entrance,
      source,
      sourceDetail,
      campus,
      follower,
      remark,
      newStudents,
      checkedLinkStudents,
      customerSign,
      cusIdCardType,
    };
    if (loginUser.isXingHuoSym()) {
      saveData.cusIdCardNo = cusIdCardNo;
    }
    if (this.props.callType) {
      saveData['callType'] = this.props.callType;
    }
    onLoadingStart();
    loaders
      .addCustomer(saveData)
      .then(
        () => {
          showMessage('保存成功', 'success');
          onLoadingEnd();
          onSaveSuccess();
        },
        (e) => {
          if (e.text) {
            onError(e.text);
          } else {
            showMessage('保存失败');
          }
          onLoadingEnd();
        },
      )
      .catch(() => {
        onLoadingEnd();
      });
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  checkAllLinkStudentChange = (e) => {
    const { linkStudents } = this.state;
    const checked = e.target.checked;
    let checkedLinkStudents;

    if (checked) {
      checkedLinkStudents = linkStudents.filter((s) => s.canFollow);
    } else {
      checkedLinkStudents = [];
    }

    this.setState({ checkedLinkStudents });
  };

  checkLinkStudentChange = (e, stu) => {
    const { checkedLinkStudents } = this.state;
    const checked = e.target.checked;
    let newCheckedLinkStudents;

    if (checked) {
      if (checkedLinkStudents.indexOf(stu) < 0) {
        newCheckedLinkStudents = [...checkedLinkStudents, stu];
      }
    } else {
      newCheckedLinkStudents = checkedLinkStudents.filter((c) => c !== stu);
    }

    this.setState({ checkedLinkStudents: newCheckedLinkStudents });
  };

  getLinkStudentsToolBar = () => {
    const { linkStudents, checkedLinkStudents } = this.state;
    const canFollowStus = linkStudents.filter((s) => s.canFollow);
    const checked = canFollowStus.length > 0 && canFollowStus.length === checkedLinkStudents.length;

    return (
      <Checkbox checked={checked} onChange={this.checkAllLinkStudentChange} disabled={canFollowStus.length === 0}>
        全部跟进
      </Checkbox>
    );
  };

  newStudentChange = (id, data) => {
    const newStudents = this.state.newStudents.map((stu) => {
      if (stu.id === id) {
        return data;
      }

      return stu;
    });

    this.setState({ newStudents });
  };

  deleteNewStudent = (id) => {
    this.setState({
      newStudents: this.state.newStudents.filter((stu) => stu.id !== id),
    });

    let newErrors = {};
    Object.keys(this.errors).forEach((key) => {
      if (key.indexOf(`newStu_${id}`) < 0) {
        newErrors[key] = this.errors[key];
      }
    });
    this.errors = newErrors;
  };

  addNewStudent = () => {
    this.setState({
      newStudents: [...this.state.newStudents, { id: '' + this.newStudentIdx++ }],
    });
  };

  checkAddNewStudent = () => {
    if (this.state.newStudents.length === 0) {
      this.addNewStudent();
    }
  };

  /**
   * 客户电话切换，查询关联学生
   *
   * @param val {string} 客户电话
   */
  phoneChange = (val) => {
    const { onLoadingStart, onLoadingEnd } = this.props;
    const value = val ? val.trim() : null;

    this.setState({ phone: value });
    const phoneValidate = checkPhone(val) === null;
    if (phoneValidate) {
      onLoadingStart();
      loaders.loadLinkStudents({ phone: val }).then(
        ({ list, customerId }) => {
          onLoadingEnd();
          this.setState({ linkStudents: list || [], checkedLinkStudents: [], customerId: customerId || null });
          if (!list || list.length === 0) {
            this.checkAddNewStudent();
          }
        },
        () => {
          onLoadingEnd();
          this.setState({ linkStudents: [], checkedLinkStudents: [], customerId: null });
          this.checkAddNewStudent();
        },
      );
    } else {
      this.setState({ linkStudents: [], checkedLinkStudents: [], customerId: null });
      this.checkAddNewStudent();
    }
  };

  sourceChange = (val) => {
    const { onLoadingStart, onLoadingEnd } = this.props;

    if (val) {
      onLoadingStart();
      loadSourceDetailDict(val).then(
        (dict) => {
          onLoadingEnd();
          this.setState({ source: val, sourceDetail: null, sourceDetailDict: dict });
        },
        () => {
          onLoadingEnd();
        },
      );
    } else {
      this.setState({ source: null, sourceDetail: null, sourceDetailDict: [] });
    }
  };

  // 是否满足如下路由，满足了才能展示录入身份证
  canWriteIdNumber = () => {
    let tempArr = ['/ReceptionAddCustomer', '/ConsultantFollowCustomer', '/OutcallCustomerFollow'];
    return tempArr.includes(this.props.routingPathName);
  };

  idNumberErrorEle = () => {
    return (
      <span
        style={{
          color: '#ff5b5b',
          display: 'block',
          lineHeight: '18px',
          fontSize: '12px',
          height: '18px',
        }}
      >
        身份证格式不对
      </span>
    );
  };

  getToolTipText = ({ status, finishDay, studentName, blCampusName }) => {
    return {
      CLASSING: `【学生：${studentName}/学籍：${blCampusName}/状态：上课中】不满足激活条件，请到【学生列表】操作签单`,
      STOP_CLASS: `【学生：${studentName}/学籍：${blCampusName}/状态：停课${
        finishDay || 0
      }天】不满足激活条件，请到【学生列表】操作签单`,
      FINISH_CLASS: `【学生：${studentName}/学籍：${blCampusName}/状态：结课${
        finishDay || 0
      }天】不满足激活条件，请到【学生列表】操作签单`,
    }[status];
  };

  render() {
    const { workBench, btnAddStudentVisible } = this.props;
    const {
      name,
      phone,
      entrance,
      source,
      sourceDetail,
      campus,
      follower,
      remark,
      cusIdCardNo,
      linkStudents,
      checkedLinkStudents,
      newStudents,
      checking,
      sourceDetailDict,
      gradeDict,
      sourceDict,
      campusDict,
      followerDict,
      userProvince,
      userCity,
      customerSign,
      customerSignDict,
    } = this.state;
    const customerFormConf = this.customerFormItemConf;
    const isReception = calcWorkbenchReception(workBench);

    return (
      <div className={styles.wrap}>
        <Block title="客户信息" style={{ marginTop: 20 }}>
          <CusInfoLayout>
            <FormItem {...addCusFormCfg} name="客户姓名" required={customerFormConf.name.required}>
              <Input
                value={name}
                onChange={(e) => this.setState({ name: e.target.value ? e.target.value.trim() : null })}
                validator={{
                  silent: !checking,
                  onErrorChange: (e) => this.errorChange('name', e),
                }}
                disabled={customerFormConf.name.disabled}
              />
            </FormItem>
            <FormItem {...addCusFormCfg} name="客户电话" required={customerFormConf.phone.required}>
              <Input
                value={phone}
                onChange={(e) => this.phoneChange(e.target.value)}
                validator={{
                  rules: ['required', 'phone'],
                  silent: !checking,
                  onErrorChange: (e) => this.errorChange('phone', e),
                }}
                disabled={customerFormConf.phone.disabled}
              />
            </FormItem>
            {isReception && (
              <FormItem {...addCusFormCfg} name="资源入口" required={customerFormConf.entrance.required}>
                <Select
                  dict={this.props.isConsultant ? entranceDict2 : entranceDict}
                  value={entrance}
                  onChange={(val) => {
                    if ([ENTRANCE_DIRECT, ENTRANCE_PULL].indexOf(val) < 0) {
                      delete this.errors['customerSign'];
                    }

                    this.setState({
                      entrance: val,
                      customerSign: [ENTRANCE_DIRECT, ENTRANCE_PULL].indexOf(val) >= 0 ? customerSign : null,
                    });
                  }}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('entrance', e),
                  }}
                  disabled={customerFormConf.entrance.disabled}
                />
              </FormItem>
            )}
            <FormItem {...addCusFormCfg} name="资源来源" required={customerFormConf.source.required}>
              <Select
                dict={sourceDict}
                value={source}
                onChange={this.sourceChange}
                validator={{
                  rules: ['required'],
                  silent: !checking,
                  onErrorChange: (e) => this.errorChange('source', e),
                }}
                disabled={customerFormConf.source.disabled}
              />
            </FormItem>
            <FormItem {...addCusFormCfg} name="来源细分" required={customerFormConf.sourceDetail.required}>
              <Select
                dict={sourceDetailDict}
                value={sourceDetail}
                onChange={(val) => this.setState({ sourceDetail: val })}
                validator={{
                  rules: ['required'],
                  silent: !checking,
                  onErrorChange: (e) => this.errorChange('sourceDetail', e),
                }}
                disabled={customerFormConf.sourceDetail.disabled}
              />
            </FormItem>
            {isReception && (
              <FormItem {...addCusFormCfg} name="意向学习中心" required={customerFormConf.campus.required}>
                <Select
                  dict={campusDict}
                  value={campus}
                  onChange={(val) => this.setState({ campus: val })}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('campus', e),
                  }}
                  disabled={customerFormConf.campus.disabled}
                />
              </FormItem>
            )}
            {isReception && (
              <FormItem {...addCusFormCfg} name="分配跟进人" required={customerFormConf.follower.required}>
                <Select
                  dict={followerDict}
                  value={follower}
                  onChange={(val) => this.setState({ follower: val })}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('follower', e),
                  }}
                  disabled={customerFormConf.follower.disabled}
                />
              </FormItem>
            )}
            {isReception && [ENTRANCE_DIRECT, ENTRANCE_PULL].indexOf(entrance) >= 0 ? (
              <FormItem {...addCusFormCfg} name="客户意向产品" required={customerFormConf.customerSign.required}>
                <Select
                  dict={customerSignDict}
                  value={customerSign}
                  onChange={(val) => this.setState({ customerSign: val })}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('customerSign', e),
                  }}
                  disabled={customerFormConf.customerSign.disabled}
                />
              </FormItem>
            ) : (
              ''
            )}
          </CusInfoLayout>
          {loginUser.isXingHuoSym() && this.canWriteIdNumber() && (
            <Row>
              <Col span={12}>
                <FormItem {...addCusFormCfg} name="证件类型">
                  <Select
                    dict={this.state.certificateTypeDict}
                    value={this.state.cusIdCardType}
                    onChange={(val) => {
                      this.setState({ cusIdCardType: val });
                    }}
                  />
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem {...addCusFormCfg} name="证件号码">
                  <Input
                    value={cusIdCardNo}
                    onChange={(e) => this.setState({ cusIdCardNo: e.target.value ? e.target.value.trim() : null })}
                  />
                </FormItem>
              </Col>
            </Row>
          )}
          <Row style={addCusModalRowStyle}>
            <Col span={24}>
              <FormItem {...addCusFormCfg} name="备注" required={customerFormConf.remark.required}>
                <TextArea
                  rows={3}
                  value={remark}
                  wordCount
                  maxLength={300}
                  onChange={(e) => this.setState({ remark: e.target.value })}
                  disabled={customerFormConf.remark.disabled}
                />
              </FormItem>
            </Col>
          </Row>
        </Block>

        {linkStudents.length > 0 && (
          <Block title="关联学生" headerToolBar={this.getLinkStudentsToolBar()}>
            <div className={styles.tipInfo}>
              <span>同一学生，请勾选关联并保存即可</span>
            </div>
            {linkStudents.map((stu) => {
              const toolTipText = this.getToolTipText(stu);
              const checked = checkedLinkStudents.indexOf(stu) >= 0;
              const tag = stu.studentType === 'ENROLLED' && !stu.canFollow && '在读学生';
              const checkboxItem = (
                <Checkbox
                  checked={checked}
                  onChange={(e) => this.checkLinkStudentChange(e, stu)}
                  disabled={!stu.canFollow}
                />
              );

              return (
                <div key={stu.id}>
                  <Row style={{ marginLeft: 20 }}>
                    <Col span={1} className={styles.xsCheckboxCol}>
                      {stu.canFollow ? (
                        checkboxItem
                      ) : toolTipText ? (
                        <Tooltip placement="right" title={toolTipText} overlayClassName={styles.toolTipWrapper}>
                          {checkboxItem}
                        </Tooltip>
                      ) : (
                        checkboxItem
                      )}
                    </Col>
                    <Col span={8}>
                      <FormItem {...linkFormCfg} name="学生">
                        <div
                          style={{
                            wordBreak: 'break-word',
                            display: 'flex',
                            justifyContent: 'center',
                            alignItems: 'baseline',
                          }}
                        >
                          <span style={{ wordBreak: 'break-word', flex: '1' }}>
                            {stu.name}
                            {stu.haveLabel && <i style={{ color: 'red' }}>*</i>}
                          </span>
                          {tag && (
                            <span
                              style={{
                                marginLeft: '10px',
                                border: '1px #ff8080 solid',
                                padding: '0 4px',
                                borderRadius: '4px',
                                color: '#ff8080',
                                display: 'inline-block',
                                lineHeight: '20px',
                                fontSize: '12px',
                              }}
                            >
                              {tag}
                            </span>
                          )}
                        </div>
                      </FormItem>
                    </Col>
                    <Col span={4}>
                      <FormItem {...linkFormCfg} name="年级">
                        {stu.gradeName}
                      </FormItem>
                    </Col>
                    <Col span={11}>
                      <FormItem {...linkFormCfg} name="学校">
                        {stu.schoolName}
                      </FormItem>
                    </Col>
                  </Row>
                </div>
              );
            })}
          </Block>
        )}

        <Block title="学生信息">
          {newStudents.map((stu, idx) => {
            return (
              <div key={stu.id}>
                {btnAddStudentVisible && (
                  <div className={styles.newStuHeader}>
                    <div className={styles.newStuTitle}>{`学生${idx + 1}`}</div>
                    {(linkStudents.length > 0 || newStudents.length > 1) && (
                      <div className={styles.newStuToolBar}>
                        <TextLink onClick={() => this.deleteNewStudent(stu.id)}>删除</TextLink>
                      </div>
                    )}
                  </div>
                )}
                <NewStudent
                  key={stu.id}
                  data={stu}
                  onChange={(data) => this.newStudentChange(stu.id, data)}
                  onErrorChange={(fieldName, e) => this.errorChange(`newStu_${stu.id}_${fieldName}`, e)}
                  checking={checking}
                  gradeDict={gradeDict}
                  userProvince={userProvince}
                  userCity={userCity}
                  idNumberErrorEle={this.idNumberErrorEle}
                  canShowIdNumber={this.canWriteIdNumber()}
                />
              </div>
            );
          })}
          {btnAddStudentVisible && (
            <div className={styles.newStudentsFooter}>
              <Button onClick={() => this.addNewStudent()}>添加学员</Button>
            </div>
          )}
        </Block>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    userCampusId: state.account.user.campusId,
    routingPathName: state.routing.locationBeforeTransitions.pathname,
  };
};

export default connect(mapStateToProps, null, null, { withRef: true })(AddCustomerMain);
