import React from 'react';
import PropTypes from 'prop-types';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { FormItem, TextArea, Select, Button, Input, Loading } from 'COMPONENT';
import { Modal } from 'antd';
import { followCustomer, disabledCustomer } from 'SERVICE/outcallWorkbench';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import { loadCommonDictByGroup } from 'UTIL/dict';

// 资源类型，1：外呼；2：常规
const outcallType = 1;

const formCfg = {
  labelWidth: 110,
};

const NO_INTENTION_ID = 'DAT0000000349';

class AddFollowRecord extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      // 跟进记录表单数据
      recordDetail: {
        content: '',
        intention: '',
        product: '',
        invalidTag: '',
      },
      dictionaryMap: {
        INVALID_SIGN: [], // {[object]} 字典 无效标签
        CustomerSignType: [], // {[object]} 字典 意向产品
        CUSTOMER_SATISFICING: [], // {[object]} 字典 意向度
      },
      checking: false,
      loading: false,
      dialogLoading: true,
    };
    this.errors = {};
  }

  static propTypes = {
    customerName: PropTypes.string,
    customerId: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
    studentId: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
    onSuccess: PropTypes.func,
    onCancel: PropTypes.func,
  };

  static defaultProps = {
    onSuccess: function () {},
    onCancel: function () {},
  };

  componentDidMount() {
    this.initDictionary();
  }

  componentWillReceiveProps(props) {
    if (props.visible === false) {
      this.resetForm();
    }
  }

  // 初始化下拉框
  initDictionary() {
    loadCommonDictByGroup('CUSTOMER_SATISFICING,CustomerSignType,INVALID_SIGN')
      .then((res) => {
        this.setState({
          dictionaryMap: res,
          dialogLoading: false,
        });
      })
      .catch(() => {
        this.setState({
          dialogLoading: false,
        });
      });
  }

  // 设置单个表单值
  setFormValue = (key, value) => {
    this.setState({
      recordDetail: {
        ...this.state.recordDetail,
        [key]: value,
      },
    });
  };

  resetForm() {
    this.setState({
      recordDetail: {
        content: '',
        intention: '',
        product: '',
        invalidTag: '',
      },
      loading: false,
      checking: false,
    });
  }

  onIntentionChange = (value) => {
    this.setFormValue('intention', value);
  };

  checkIsIntentionValid(value) {
    return value !== NO_INTENTION_ID;
  }

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  onSubmit = () => {
    const { content, intention, product, invalidTag } = this.state.recordDetail;
    const { customerId, studentId, onSuccess } = this.props;
    const errors = { ...this.errors };
    const errorTips = getCommonErrorTips(errors);

    this.setState({ checking: true });

    if (errorTips) {
      return;
    }

    let params = {
      customerId: customerId,
      followStuId: studentId,
      remark: content,
      resourceType: outcallType,
    };

    if (intention === NO_INTENTION_ID) {
      params.invalidSignId = invalidTag;
    } else {
      params.satisficing = intention;
      params.customerSign = product;
    }

    this.setState({ loading: true });

    let request = intention === NO_INTENTION_ID ? disabledCustomer(params) : followCustomer(params);

    request
      .then(() => {
        this.setState({ loading: false });
        showMessage('保存成功', 'success');
        onSuccess();
      })
      .catch(() => {
        this.setState({ loading: false });
      });
  };

  onCancel = () => {
    this.props.onCancel();
  };

  render() {
    const { customerName, visible, onCancel } = this.props;
    const { checking, loading, dialogLoading } = this.state;
    const { content, intention, product, invalidTag } = this.state.recordDetail;
    const { INVALID_SIGN, CustomerSignType, CUSTOMER_SATISFICING } = this.state.dictionaryMap;

    const renderIntentionSelect = () => {
      if (!intention) return null;

      if (this.checkIsIntentionValid(intention)) {
        return (
          <FormItem {...formCfg} name="客户意向产品">
            <Select dict={CustomerSignType} value={product} onChange={(val) => this.setFormValue('product', val)} />
          </FormItem>
        );
      } else {
        return (
          <FormItem {...formCfg} name="无效标签" required>
            <Select
              dict={INVALID_SIGN}
              value={invalidTag}
              onChange={(val) => this.setFormValue('invalidTag', val)}
              validator={{
                rules: ['required'],
                silent: !checking,
                onErrorChange: (e) => this.errorChange('invalidTag', e),
              }}
            />
          </FormItem>
        );
      }
    };

    return (
      <Modal
        title="添加跟进记录"
        width={800}
        maskClosable={false}
        onCancel={onCancel}
        footer={
          <div>
            <Button type="primary" loading={loading} onClick={this.onSubmit}>
              提交
            </Button>
            <Button onClick={this.onCancel}>取消</Button>
          </div>
        }
        visible={visible}
      >
        <Loading loading={dialogLoading} />
        <FormItem {...formCfg} name="客户姓名">
          <Input value={customerName} disabled />
        </FormItem>

        <FormItem {...formCfg} name="跟进内容" required>
          <TextArea
            rows={3}
            wordCount
            maxLength={1000}
            value={content}
            onChange={(e) => this.setFormValue('content', e.target.value)}
            validator={{
              rules: ['required'],
              silent: !checking,
              onErrorChange: (e) => this.errorChange('content', e),
            }}
          />
        </FormItem>
        <FormItem {...formCfg} name="意向度" required>
          <Select
            dict={CUSTOMER_SATISFICING}
            value={intention}
            onChange={(value) => {
              this.setFormValue('intention', value);
            }}
            validator={{
              rules: ['required'],
              silent: !checking,
              onErrorChange: (e) => this.errorChange('intention', e),
            }}
          />
        </FormItem>
        {renderIntentionSelect()}
      </Modal>
    );
  }
}

export default AddFollowRecord;
