import React from 'react';
import Modal from 'COMPONENT/Modal';
import noop from 'lodash/noop';
import { Select, FormItem, Upload, FileList } from 'COMPONENT';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { Col, Row, Select as AntdSelect, TreeSelect } from 'antd';
import { loadProvinceCityDict } from 'UTIL/dict';
import style from './style.less';
import { importCustomerData, outCallResourcePools } from 'SERVICE/outcallWorkbench';
import PopTip from 'COMPONENT/PopTip';
import Button from 'COMPONENT/Button';
import { OSS_BASE_PREFIX } from 'CONST/ossConfig';

const { Option } = AntdSelect;
const EXCEL_URL = `${OSS_BASE_PREFIX}/新的客户资源导入模板.xlsx`;

const ModalFooterBtn = (props) => {
  const { content, popTipVisible, onClose, onOk, onCancel, confirmBtnDisabled } = props;
  return (
    <div style={{ display: 'flex', justifyContent: 'flex-end' }}>
      <PopTip icon="error" content={content} visible={popTipVisible} onClose={onClose} maxWidth={200}>
        <Button type="primary" onClick={onOk} disabled={confirmBtnDisabled}>
          提交
        </Button>
      </PopTip>
      <Button onClick={onCancel}>取消</Button>
    </div>
  );
};

function formatNodesToParentChildren(nodes = [], parentId = '') {
  if (nodes.length === 0) {
    return [];
  }

  const parents = [];
  const rest = [];
  for (let node of nodes) {
    node.parentId === parentId ? parents.push(node) : rest.push(node);
  }

  for (let parent of parents) {
    parent.children = formatNodesToParentChildren(rest, parent.nodeId);
  }

  return parents;
}

class ImportModal extends React.Component {
  static defaultProps = {
    onClose: noop,
    onSave: noop,
  };

  state = {
    provinceDict: [],
    cityDict: [],
    province: '',
    schoolCityId: '',
    loading: false,
    fileList: [],
    treeDict: [],
    deliverTarget: '',
    treeDefaultExpandedKeys: [],
  };

  constructor(props) {
    super(props);
    this.modal = null;
  }

  componentDidMount() {
    this.init();
    this.getDistributePoolData();
  }

  init = async () => {
    this.setState({
      loading: true,
    });

    try {
      this.state.provinceDict = await loadProvinceCityDict(false);
    } finally {
      this.setState({
        loading: false,
      });
    }
  };

  getDistributePoolData = async () => {
    const { data } = await outCallResourcePools({
      /* 外呼资源池树 */ resourceType: 1,
      withUser: true,
      withCapacity: true,
    });
    // get nodeIds map
    const nodeIdsMap = {};
    for (let node of data) {
      const { nodeId } = node;
      nodeIdsMap[nodeId] = node;
    }

    // get distributableNodes' parents
    const parents = new Set();
    const distributableNodes = data.filter((item) => item.distributable);
    for (let { parentId } of distributableNodes) {
      if (nodeIdsMap[parentId]) {
        parents.add(nodeIdsMap[parentId]);
      }
    }

    // remove duplicate
    const nodesSet = new Set(distributableNodes.concat([...parents]));

    // map nodes to TreeSelect format
    const nodes = [...nodesSet].map((node) => ({
      ...node,
      label: `${node.nodeName}（${node.usedCapacity || 0}/${node.capacity || 0}）`,
      value: node.nodeId,
      key: node.nodeId,
    }));
    const parentNode = nodes.find((node) => node.orgType === 'GROUNP');
    parentNode.children = formatNodesToParentChildren([...nodes], parentNode.nodeId);
    parentNode.disabled = true;

    this.setState({
      treeDict: [parentNode],
      treeDefaultExpandedKeys: [parentNode.nodeId],
    });
  };

  onCommit = async () => {
    this.setState({
      loading: true,
    });
    // const user = this.state.treeDict.find(r => r.val === this.state.deliverTarget);

    try {
      const { resultCode } = await importCustomerData({
        fileKey: this.state.fileList[0].fileKey,
        type: 1,
        deliverTarget: this.state.deliverTarget,
        schoolCityId: this.state.schoolCityId,
      });
      if (resultCode == '0') {
        setTimeout(() => {
          this.onClose(true);
        }, 10);
      }
    } finally {
      this.setState({
        loading: false,
      });
    }
  };

  onClose = (reload = false) => {
    this.props.onClose(reload);
  };

  handleProvinceSelect = (val) => {
    if (!val) {
      this.setState({
        cityDict: [],
        province: val,
        schoolCityId: '',
      });
      return;
    }

    const province = this.state.provinceDict.find((province) => province.val == val);
    this.setState({
      cityDict: province.options,
      province: val,
      schoolCityId: '',
    });
  };

  handleCitySelect = (schoolCityId) => {
    this.setState({
      schoolCityId,
    });
  };

  onUploadChange = (file) => {
    file.id = file.fileKey;
    this.setState({
      fileList: [file],
    });
  };

  beforeUpload = (fileList) => {
    for (const file of fileList) {
      file['fileKey'] += '.' + file['name'].split('.').pop();
    }
    return fileList;
  };

  onTreeSelectChange = (deliverTarget) => {
    this.setState({ deliverTarget });
  };

  userFilter = (searchKey, option) => {
    return option.text.indexOf(searchKey) >= 0;
  };

  render() {
    const { loading, provinceDict, cityDict, treeDict, deliverTarget, treeDefaultExpandedKeys } = this.state;

    return (
      <Modal
        loading={loading}
        title="资源导入"
        ref={(c) => (this.modal = getWrappedInstance(c))}
        width={750}
        onCancel={this.onClose}
        footer={
          <ModalFooterBtn
            content=""
            popTipVisible={false}
            onClose={this.closeTips}
            onOk={this.onCommit}
            onCancel={this.onClose}
            confirmBtnDisabled={this.state.fileList.length === 0 || !this.state.schoolCityId}
          />
        }
      >
        <FormItem name="资源模板下载">
          <a className={style.download} href={EXCEL_URL} download>
            模板下载
          </a>
          <div className={style.label}>先下载导入模板，再根据模板要求填写好文件</div>
        </FormItem>
        <FormItem name="资源上传">
          {this.state.fileList.length === 0 ? (
            <Upload
              multiple={false}
              // limit={{type: 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'}}
              onChange={this.onUploadChange}
              beforeUpload={this.beforeUpload}
            >
              <span className={style.upload}>上传文件</span>
              <div className={style.label}>上传文件下载模板 文件单次只能导入5000条</div>
            </Upload>
          ) : (
            <FileList data={this.state.fileList} onChange={(fileList) => this.setState({ fileList })} />
          )}
        </FormItem>
        <FormItem name="外呼资源分配" required="true">
          <TreeSelect
            size="large"
            allowClear
            value={deliverTarget}
            dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
            treeData={this.state.treeDict}
            placeholder="请选择"
            treeDefaultExpandedKeys={treeDefaultExpandedKeys}
            onChange={this.onTreeSelectChange}
            showSearch
            treeNodeFilterProp="label"
          />
        </FormItem>
        <FormItem name="请选择学校所在市">
          <Row gutter={8}>
            <Col span={12}>
              <AntdSelect
                value={this.state.province}
                placeholder="省"
                // showSearch={true}
                noOptionTips="暂无数据"
                size="large"
                allowClear={true}
                onChange={this.handleProvinceSelect}
              >
                {provinceDict.map((item) => {
                  const { val, label } = item;
                  return (
                    <Option value={val} key={val}>
                      {label}
                    </Option>
                  );
                })}
              </AntdSelect>
            </Col>
            <Col span={12}>
              <AntdSelect
                value={this.state.schoolCityId}
                placeholder="市"
                // showSearch={true}
                noOptionTips="暂无数据"
                size="large"
                allowClear={true}
                onChange={this.handleCitySelect}
              >
                {cityDict.map((item) => {
                  const { val, text } = item;
                  return (
                    <Option value={val} key={val}>
                      {text}
                    </Option>
                  );
                })}
              </AntdSelect>
            </Col>
          </Row>
        </FormItem>
      </Modal>
    );
  }
}

export default ImportModal;
