import React from 'react';
import { DatePicker } from 'antd';
import { connect } from 'dva';
import { Page, Input, Select, List, SearchFilter, SearchField, Button } from 'COMPONENT';
import { loadCommonDictNew } from 'UTIL/dict';
import { desensitizationPhoneNumber } from 'UTIL/common';
import moment from 'moment';
import DistributeModal from './components/distributeModal';
import Index from './components/importModal';
import { distributeCustomerList, distributeCustomer } from 'SERVICE/outcallWorkbench';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadSourceDetailDict } from 'PAGE/common/loaders';
import SchoolButton from '../components/SchoolButton';

const title = '分配客户管理';

const { RangePicker } = DatePicker;

const dayRange = [moment().startOf('day'), moment().endOf('day')];

// 初始化搜索参数
const defaultParams = {
  studentName: '',
  customerName: '',
  gradeId: '',
  resEntrance: '',
  dealStatus: '',
  deliverType: '',
  deliverTime: [],
  createTime: dayRange,
  schoolId: null,
  cusType: '',
  cusOrg: '',
};

/** 客户状态字典 */
const dealStatusDict = [
  {
    val: 'STAY_FOLLOW',
    text: '待跟进',
  },
  {
    val: 'FOLLOWING',
    text: '跟进中',
  },
];

/** 客户分配状态字典  */
const DistributeDict = [
  {
    val: 'PERSONAL_POOL',
    text: '已分配',
  },
  {
    val: 'CUSTOMER_RESOURCE_POOL',
    text: '未分配',
  },
];

const DeliverTypeMap = {
  PERSONAL_POOL: '已分配',
  CUSTOMER_RESOURCE_POOL: '未分配',
};
class DistributeCustomerManage extends React.Component {
  columns = [
    {
      title: '客户姓名',
      dataIndex: 'customerName',
      key: 'customerName',
      width: 150,
    },
    {
      title: '学生姓名',
      dataIndex: 'studentName',
      key: 'studentName',
      width: 150,
    },
    {
      title: '客户电话',
      dataIndex: 'customerContact',
      key: 'customerContact',
      width: 120,
      render: function (text) {
        return desensitizationPhoneNumber(text);
      },
    },
    {
      title: '资源入口',
      dataIndex: 'resEntranceName',
      key: 'resEntranceName',
      width: 120,
    },
    { title: '学校', dataIndex: 'schoolName', key: 'schoolName', width: 120 },
    { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: 120 },
    {
      title: '跟进人',
      dataIndex: 'deliverTargetName',
      key: 'deliverTargetName',
      width: 120,
    },
    {
      title: '客户状态',
      dataIndex: 'dealStatusName',
      key: 'dealStatusName',
      width: 120,
    },
    {
      title: '分配人',
      dataIndex: 'deliverOperateUserName',
      key: 'deliverOperateUserName',
      width: 120,
      render: (val, row) => {
        if (+row.isImport) {
          return '';
        }
        return val;
      },
    },
    {
      title: '分配状态',
      width: 120,
      render: function (record) {
        return DeliverTypeMap[record.deliverType];
      },
    },
    {
      title: '最新分配时间',
      dataIndex: 'deliverTime',
      key: 'deliverTime',
      width: 160,
      render: function (text) {
        return text ? moment(text).format('YYYY-MM-DD HH:mm:ss') : '';
      },
    },
    {
      title: '创建时间',
      dataIndex: 'createTime',
      key: 'createTime',
      width: 160,
      render: function (text) {
        return text ? moment(text).format('YYYY-MM-DD HH:mm:ss') : '';
      },
    },
  ];

  state = {
    loading: false,
    gradeDict: [],
    dataList: [],
    selections: [],
    selectedRowKeys: [],
    searchParams: defaultParams,
    schoolName: null,
    showDistributeModal: false,
    showImportModal: false,
    pagination: {
      total: 0,
      current: 1,
      pageSize: 20,
    },
  };

  constructor(props) {
    super(props);

    this.onSchoolChange = this.onSchoolChange.bind(this);
  }

  componentDidMount() {
    this.initDict();
  }

  initDict = () => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: [/*"grade", */ 'resEntrance', 'resType'],
        callback: this.getCustomerList,
      },
    });
    loadCommonDictNew('STUDENT_GRADE').then((gradeDict) => {
      this.setState({ gradeDict });
    });
  };

  transformParams = () => {
    const {
      searchParams,
      pagination: { pageSize, current },
    } = this.state;
    const params = { ...searchParams, pageSize, pageNum: current };
    if (params.deliverTime.length) {
      const [deliverTimeStart, deliverTimeEnd] = params.deliverTime;
      params.deliverTimeStart = deliverTimeStart.startOf('day').valueOf();
      params.deliverTimeEnd = deliverTimeEnd.endOf('day').valueOf();
    }
    delete params.deliverTime;
    if (params.createTime.length) {
      const [beginCreateTime, endCreateTime] = params.createTime;
      params.beginCreateTime = beginCreateTime.startOf('day').valueOf();
      params.endCreateTime = endCreateTime.endOf('day').valueOf();
    }
    delete params.createTime;
    return params;
  };

  getCustomerList = async () => {
    this.setState({ loading: true });
    try {
      const { pagination } = this.state;
      const { data } = await distributeCustomerList(this.transformParams());
      const { list, total } = data;
      this.setState({ dataList: list, pagination: { ...pagination, total } });
    } finally {
      this.setState({ loading: false });
    }
  };

  handleSearch = () => {
    let { pagination } = this.state;
    this.setState({ pagination: { ...pagination, current: 1 } }, this.getCustomerList);
  };

  handleReset = () => {
    let { pagination } = this.state;
    this.setState(
      {
        searchParams: { ...defaultParams, createTime: [] },
        schoolName: null,
        pagination: { ...pagination, current: 1 },
        selections: [],
        selectedRowKeys: [],
        sourceDetailDict: [],
      },
      this.getCustomerList,
    );
  };

  handlePageChange = (pagination) => {
    const { pageSize, current } = pagination;
    this.setState({ pagination: { ...this.state.pagination, pageSize, current } }, this.getCustomerList);
  };

  onTableSelectChange = (selectedRowKeys, selections) => {
    this.setState({ selections, selectedRowKeys });
  };

  handleSearchParamChange = (key) => {
    return function handleChange(e) {
      let { searchParams } = this.state;
      const update = {};
      update[key] = e ? (e.target ? e.target.value : e) : '';
      // 级联资源细分
      if (key === 'cusType') {
        update.cusOrg = '';
        if (update[key]) {
          loadSourceDetailDict(update[key]).then((dict) => {
            this.setState({
              sourceDetailDict: dict,
            });
          });
        } else {
          this.setState({
            sourceDetailDict: [],
          });
        }
      }
      this.setState({ searchParams: { ...searchParams, ...update } });
    };
  };

  toggleDistributeModal = () => {
    this.setState((pre) => {
      const updated = {};
      updated.showDistributeModal = !pre.showDistributeModal;
      if (pre.showDistributeModal) {
        updated.selections = [];
        updated.selectedRowKeys = [];
      }
      return updated;
    });
  };

  closeDistributeModal = (reload) => {
    this.toggleDistributeModal();
    if (reload) {
      this.getCustomerList();
    }
  };

  toggleImportModal = () => {
    this.setState((pre) => {
      return { showImportModal: !pre.showImportModal };
    });
  };

  closeImportModal = (reload) => {
    this.toggleImportModal();
    if (reload) {
      this.getCustomerList();
    }
  };

  distributeCustomer = async (user) => {
    const { selections } = this.state;
    const { capacity = 0, usedCapacity = 0, val: userId } = user;
    const usableCapacity = capacity - usedCapacity;
    if (usableCapacity === 0) {
      return showMessage('该用户资源容量已达上限！', 'error');
    }
    if (selections.length > usableCapacity) {
      // 超出分配配额
      return showMessage('超出该用户可分配资源容量', 'error');
    }
    try {
      await distributeCustomer({
        /* 资源类型  外呼 */
        resourceType: 1,
        deliverTarget: userId,
        customerStudentResourceIds: selections.map((r) => r.id),
      });
      showMessage('分配成功', 'success');
      this.closeDistributeModal(true);
    } catch (e) {
      throw e;
    }
  };

  onSchoolChange(schoolId, schoolName) {
    this.setState((state) => ({
      searchParams: {
        ...state.searchParams,
        schoolId,
      },
      schoolName,
    }));
  }

  render() {
    const {
      loading,
      dataList,
      pagination,
      searchParams,
      selections,
      selectedRowKeys,
      showDistributeModal,
      showImportModal,
    } = this.state;
    const selected = selections.filter((r) => r.dealStatus === 'FOLLOWING').length;
    const tipText = `已选${selections.length}个，其中${selected}个已有跟进人`;

    return (
      <Page
        name={title}
        fitWindowHeight
        loadingTip="正在加载数据"
        loading={loading}
        route={this.props.route}
        headerToolBar={
          <div>
            {this.props.hasDistributeAuth && (
              <Button type="primary" onClick={this.toggleDistributeModal} disabled={!selections.length}>
                资源分配
              </Button>
            )}
            {this.props.hasImportAuth && (
              <Button type="primary" style={{ marginLeft: '20px' }} onClick={this.toggleImportModal}>
                资源导入
              </Button>
            )}
          </div>
        }
        subHeader={
          <SearchFilter truncationNum={4} onSearch={this.handleSearch} onReset={this.handleReset}>
            <SearchField style={{ flex: '0 0 20%' }}>
              <RangePicker
                value={searchParams.createTime}
                placeholder={['创建时间起', '创建时间起止']}
                size="large"
                allowClear
                onChange={this.handleSearchParamChange('createTime').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Input
                value={searchParams.customerName}
                placeholder="客户姓名"
                allowClear
                maxLength={50}
                onChange={this.handleSearchParamChange('customerName').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Input
                value={searchParams.studentName}
                placeholder="学生姓名"
                allowClear
                maxLength={50}
                onChange={this.handleSearchParamChange('studentName').bind(this)}
              />
            </SearchField>
            <SearchField>
              <SchoolButton
                id={this.props.userCampusId}
                text={this.state.schoolName}
                value={searchParams.schoolId}
                onChange={this.onSchoolChange}
              />
            </SearchField>
            <SearchField>
              <Select
                value={searchParams.gradeId}
                placeholder="年级"
                showSearch={true}
                noOptionTips="暂无数据"
                dict={this.state.gradeDict}
                onChange={this.handleSearchParamChange('gradeId').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Select
                showSearch={true}
                value={searchParams.resEntrance}
                placeholder="资源入口"
                noOptionTips="暂无数据"
                dict={this.props.resEntranceDict}
                onChange={this.handleSearchParamChange('resEntrance').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Select
                showSearch={true}
                value={searchParams.deliverType}
                placeholder="分配状态"
                dict={DistributeDict}
                noOptionTips="暂无数据"
                onChange={this.handleSearchParamChange('deliverType').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Select
                showSearch={true}
                value={searchParams.dealStatus}
                placeholder="客户状态"
                dict={dealStatusDict}
                noOptionTips="暂无数据"
                onChange={this.handleSearchParamChange('dealStatus').bind(this)}
              />
            </SearchField>
            <SearchField style={{ flex: '0 0 20%' }}>
              <RangePicker
                value={searchParams.deliverTime}
                placeholder={['分配时间起', '分配时间止']}
                size="large"
                allowClear
                onChange={this.handleSearchParamChange('deliverTime').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Select
                showSearch={true}
                value={searchParams.cusType}
                placeholder="资源来源"
                noOptionTips="暂无数据"
                dict={this.props.resTypeDict}
                onChange={this.handleSearchParamChange('cusType').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Select
                showSearch={true}
                value={searchParams.cusOrg}
                placeholder="资源细分"
                noOptionTips="暂无数据"
                dict={this.state.sourceDetailDict}
                onChange={this.handleSearchParamChange('cusOrg').bind(this)}
              />
            </SearchField>
          </SearchFilter>
        }
      >
        <List
          rowKey="id"
          rowSelection={{
            selectedRowKeys,
            onChange: this.onTableSelectChange,
          }}
          dataSource={dataList}
          columns={this.columns}
          pagination={pagination}
          onChange={this.handlePageChange}
        />
        {showDistributeModal && (
          <DistributeModal tipText={tipText} onSave={this.distributeCustomer} onClose={this.closeDistributeModal} />
        )}
        {showImportModal && <Index onClose={this.closeImportModal} />}
      </Page>
    );
  }
}

export default connect((state) => {
  const { btnAuths } = state.account;
  // 是否有分配资源权限
  const hasImportAuth = btnAuths.indexOf('OUTCALL_RESOURCE_IMPORT') > -1;
  // 是否有资源导入权限
  const hasDistributeAuth = btnAuths.indexOf('OUTCALL_RESOURCE_DISTRIBUTE') > -1;
  return {
    userCampusId: state.account.user.campusId,
    gradeDict: state.dict.grade,
    resEntranceDict: state.dict.resEntrance,
    resTypeDict: state.dict.resType,
    hasDistributeAuth,
    hasImportAuth,
  };
})(DistributeCustomerManage);
