import React from 'react';
import Modal from 'COMPONENT/Modal';
import Wrapper from 'COMPONENT/Wrapper';
import style from './index.less';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import List from 'COMPONENT/List';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import TempStorage from 'COMPONENT/TempStorage';
import { getPromotionList } from 'SERVICE/preferential';
import { activityStatusDict, MAX_PREFERENTIAL, activityTypeDict } from '../../config';
import { renderActivityDate } from '../util';
import { getUseCondition } from '../../../../utils';

const { COL_WIDTH_S, COL_WIDTH_L, COL_WIDTH_XL } = List;

export default class PreferentialModal extends React.Component {
  static defaultProps = {
    branchIdArr: [], // 分公司
    defaultSelectedRows: [], // 默认选中的优惠方案，当修改优惠方案时使用到
    onClose: () => {},
    refreshList: () => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      searchName: null, // 方案名称
      searchType: null, // 种类
      searchStatus: null, // 状态

      listLoading: false,
      list: [], // 列表数据
      totalRecords: 0, // 总记录数
      pageNo: 1, // 当前页码
      pageSize: 20, // 每页记录数
      selectedRows: [], // 所有分页选中的列表(由于分页，选中的列与当前页的选中key并无绝对的联系)
      pageSelectedKeys: [], // 当前页选中列表的key

      activityTypeDict: null, // 种类字典
    };

    this.initSearchParam = {
      searchName: null,
      searchType: null,
      searchStatus: null,
    };

    this.initDict = this.initDict.bind(this);
    this.initPage = this.initPage.bind(this);
    this.loadList = this.loadList.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);
    this.onCommit = this.onCommit.bind(this);
    this.formatColumn = this.formatColumn.bind(this);
    this.onRowSelect = this.onRowSelect.bind(this);
    this.onSelectedDelete = this.onSelectedDelete.bind(this);
    this.updateSelectedKeys = this.updateSelectedKeys.bind(this);
    this.getSelectedRows = this.getSelectedRows.bind(this);
  }

  componentDidMount() {
    this.initDict();
    this.initPage();
  }

  /**
   * 加载字典
   * 目前只显示(满减促销、打折促销)两个大类
   */
  initDict() {
    this.setState({ activityTypeDict: activityTypeDict });
  }

  /**
   * 初始化并带入默认选中方案
   */
  initPage() {
    const { defaultSelectedRows } = this.props;

    this.setState(
      {
        selectedRows: defaultSelectedRows || [],
        pageSelectedKeys: [],
      },
      () => {
        this.onSearch();
      },
    );
  }

  /**
   * 获取方案列表
   */
  async loadList(changedParams = {}) {
    const { searchName, searchType, searchStatus, pageNo, pageSize } = this.state;
    const tempParam = {
      searchName,
      searchType,
      searchStatus,
      pageNo,
      pageSize,
      ...changedParams,
    };
    const param = this.formatApiParam(tempParam);

    this.setState({ listLoading: true });

    // Attention!!! 傻屌api 第一页默认从0开始
    let apiParam = { ...param };
    apiParam.pageNo = apiParam.pageNo - 1;
    const response = await getPromotionList(apiParam);
    const result = this.formatResponse(response);
    apiParam = null;

    this.setState(
      {
        listLoading: false,
        list: result.data,
        totalRecords: result.rowCount,
        pageNo: param.pageNo,
        pageSize: param.pageSize,
      },
      () => {
        this.updateSelectedKeys();
      },
    );
  }

  /**
   * 格式化请求参数
   * param.searchType 默认全选
   * @param param
   */
  formatApiParam(param) {
    const { branchIdArr } = this.props;

    let promotionLittleType = param.searchType;
    if (!param.searchType) {
      const typeArr = activityTypeDict.map((item) => {
        return item.val;
      });

      promotionLittleType = typeArr.join(',');
    }

    return {
      branchId: branchIdArr.join(','),
      promotionName: param.searchName,
      promotionLittleTypeId: promotionLittleType,
      status: param.searchStatus,
      remark: 'productNums',
      pageNo: param.pageNo,
      pageSize: param.pageSize,
    };
  }

  /**
   * 格式化方案列表
   * @param response
   */
  formatResponse(response = {}) {
    let data = response.datas || [];
    const rowCount = response.rowCount || 0;

    data.forEach((item, index) => {
      data.key = index;
    });

    return {
      data: data,
      rowCount: rowCount,
    };
  }

  onSearch() {
    this.loadList({ pageNo: 1 });
  }

  onReset() {
    this.setState(this.initSearchParam);
  }

  /**
   * 保存选中方案
   */
  onCommit() {
    const { selectedRows } = this.state;

    if (!selectedRows || selectedRows.length < 2) {
      showMessage('请选择两条优惠方案!', 'error');
      return false;
    }

    this.props.onClose(selectedRows);
  }

  /**
   * 表格列
   * @return {*[]}
   */
  formatColumn() {
    return [
      { title: '优惠方案', dataIndex: 'promotionName', key: 'promotionName', width: COL_WIDTH_L, lines: 2 },
      {
        title: '使用条件',
        dataIndex: 'conditions',
        key: 'conditions',
        width: COL_WIDTH_XL,
        lines: 2,
        render: (text, record, index) => {
          return getUseCondition(record);
        },
      },
      {
        title: '活动时间',
        dataIndex: 'date',
        key: 'date',
        width: 240,
        render: (text, record, index) => {
          return renderActivityDate(record);
        },
      },
      { title: '状态', dataIndex: 'status', key: 'status', width: COL_WIDTH_S, dict: activityStatusDict },
    ];
  }

  /**
   * 选择列表
   * @Attention 由于分页，所有分页选中的列与当前页的选中key并无绝对的联系
   * @param row 当前列
   * @param isSelected 是否选中
   * @param checkedRows 当前页选中的所有列
   */
  onRowSelect(row, isSelected, checkedRows) {
    const { selectedRows } = this.state;
    let tempSelectedRows = [...selectedRows];
    let tempPageSelectedKeys = [];

    if (isSelected) {
      if (selectedRows.length >= MAX_PREFERENTIAL) {
        showMessage(`目前只支持${MAX_PREFERENTIAL}个优惠方案叠加!`, 'error');
        return false;
      }

      // 先判断是否在所有选中列表中存在
      let hasSelected = false;
      selectedRows.forEach((item) => {
        if (item.id === row.id) {
          hasSelected = true;
          return false;
        }
      });

      if (!hasSelected) {
        tempSelectedRows.push(row);
        checkedRows.forEach((item) => {
          tempPageSelectedKeys.push(item.key);
        });
      } else {
        checkedRows.forEach((item) => {
          tempPageSelectedKeys.push(item.key);
        });
      }
    } else {
      tempSelectedRows.forEach((tempRow, index) => {
        if (tempRow.id === row.id) {
          tempSelectedRows.splice(index, 1);
          return false;
        }
      });

      checkedRows.forEach((item) => {
        tempPageSelectedKeys.push(item.key);
      });
    }

    this.setState({
      selectedRows: tempSelectedRows,
      pageSelectedKeys: tempPageSelectedKeys,
    });
  }

  /**
   * 删除选中的某一条记录
   * @Attention 新增叠加优惠和修改叠加优惠的逻辑不一样
   * @param record selectedRows[i] 需要删除的记录
   * @param index selectedRows.index 不等于key及list.index
   */
  onSelectedDelete(record, index) {
    const { pageSelectedKeys, selectedRows, list } = this.state;
    let keysArr = [...pageSelectedKeys];
    let rowsArr = [...selectedRows];

    // 1. 找到list中对应的key（判断删除的记录在list中是否存在）
    let keyIndex = list.findIndex((item) => {
      return item.id === record.id;
    });

    // 2. 如果存在则删除对应key
    if (keyIndex > -1) {
      keysArr.forEach((key, i) => {
        if (key === keyIndex) {
          keysArr.splice(i, 1);
          return false;
        }
      });
    }

    // 3. 不管存在与否都删除对应row 刷新列表会重新计算选中key
    rowsArr.splice(index, 1);

    this.setState({
      selectedRows: rowsArr,
      pageSelectedKeys: keysArr,
    });
  }

  /**
   * 每次查询数据时需要重新更新选中的数据
   * 列表的index作为选中的key，判断当前选中的列是否在list中存在，再更新selectedRowKeys
   */
  updateSelectedKeys() {
    const { selectedRows, list } = this.state;
    let tempPageSelectedKeys = [];

    selectedRows.forEach((row) => {
      let listIndex = 0;
      let hasThisRow = false;

      list.forEach((item, index) => {
        if (item.id === row.id) {
          listIndex = index;
          hasThisRow = true;
          return false;
        }
      });

      if (hasThisRow) {
        tempPageSelectedKeys.push(listIndex);
      }
    });

    this.setState({
      pageSelectedKeys: tempPageSelectedKeys,
    });
  }

  /**
   * 获取选中的优惠方案
   */
  getSelectedRows() {
    return this.state.selectedRows;
  }

  render() {
    const {
      searchName,
      searchType,
      searchStatus,
      activityTypeDict,
      listLoading,
      list,
      pageNo,
      pageSize,
      totalRecords,
    } = this.state;
    const { pageSelectedKeys, selectedRows } = this.state;
    const columns = this.formatColumn();

    return (
      <Modal
        loading={false}
        onCancel={this.props.onClose}
        title={
          <h5>
            <span className={style.modal__title}>添加优惠方案</span>
            <span className={style.modal__subTitle} style={{ display: 'none' }}>
              (按顺序计算优惠金额)
            </span>
          </h5>
        }
        width={960}
        height={700}
        cancelText="关闭"
        okBtnShow={true}
        onOk={this.onCommit}
        okBtnDisabled={false}
        className={style.modalWrap}
      >
        <Wrapper
          bodyFitHeight
          className={style.container}
          header={
            <section>
              <SearchBar onSearch={this.onSearch} onReset={this.onReset}>
                <Input
                  placeholder="方案名称"
                  value={searchName}
                  onChange={(e) => this.setState({ searchName: e.target.value })}
                />
                <Select
                  placeholder="种类"
                  showSearch={true}
                  dict={activityTypeDict}
                  value={searchType}
                  onChange={(val) => this.setState({ searchType: val })}
                />
                <Select
                  placeholder="是否有效"
                  dict={activityStatusDict}
                  value={searchStatus}
                  onChange={(val) => this.setState({ searchStatus: val })}
                />
              </SearchBar>

              <TempStorage
                className={style.selectedList}
                label="已选方案"
                isLabelCount
                optionTextKey="promotionName"
                source={selectedRows}
                onDelete={this.onSelectedDelete}
              />
            </section>
          }
        >
          <List
            dataSource={list}
            columns={columns}
            loading={listLoading}
            pagination={{
              current: pageNo,
              total: totalRecords,
              pageSize,
            }}
            rowSelection={{
              selectedRowKeys: pageSelectedKeys,
              onSelect: (record, selected, selectedRows) => {
                this.onRowSelect(record, selected, selectedRows);
              },
              onChange: (keys, rows) => {},
            }}
            onChange={({ current, pageSize }) => this.loadList({ pageNo: current, pageSize })}
          />
        </Wrapper>
      </Modal>
    );
  }
}
