import moment from 'moment';
import NP from 'number-precision';
import { isNumber } from 'UTIL/common';
import { ENG_CHARSET, createRandomStr } from 'UTIL/uid';
import {
  rangeFormat,
  STUDENT,
  NONE,
  MONEY,
  PRODUCT_NUMS,
  COURSE_HOURS,
  COUPON,
  COUPON_DEPOSIT,
  PERIOD_VALID_REGULAR,
  PERIOD_VALID_RELATIVE,
  RECEIVE_LIMIT_FOR_STUD,
  AUDIT_UNREVIEW,
} from '../config';
import { COURSE_HOURS_TRANSFORM_MULTIPLE } from '../store/constant';

/**
 * 创建 uid
 * @param {string} suffix 补充字符串
 */
export function createUid(suffix = '') {
  const prefix = createRandomStr(16, ENG_CHARSET);

  return prefix + String(suffix);
}

/**
 * 处理代金券提交数据
 * @param {Object} params 待提交数据
 * @param {Object} originData 上次保存的详情数据
 * @return {Object}
 */
export function submitDataAdapter(params, originData = {}) {
  const modelWithoutRules = Object.keys(params)
    .filter((name) => !/rules_/.test(name))
    .reduce((prev, name) => {
      prev[name] = params[name];

      return prev;
    }, {});

  const originResult = {
    id: null,
    // 优惠类型
    promotionKindId: COUPON,
    // 优惠小类
    promotionLittleTypeId: 'COUPON_FULL_SALE',
    // 优惠小小类
    promotionTypeId: 'COUPON_FULL_SALE_NORMAL',
    // 券类型 - 无门槛优惠券
    promotionNature: COUPON_DEPOSIT,
    // 合同条件
    contractCondition: 'ALL',
    // 用户条件
    // customerCondition: originData.customerCondition,
    // (暂0)
    firstLevel: 0,
    // 优惠码(暂空)
    promotionCode: '',
    // 备注
    remark: '',
    // 发布数量
    publishNum: originData.publishNum || 0,
    // 使用数量
    useNum: originData.useNum || 0,
    // 状态
    status: originData.status || 0,
    // 优惠产品
    productVos: [],
    // 发放对象
    publishObject: STUDENT,
    // 是否需要领取 - 不需领取
    needToRecive: 0,
    // 领取限制
    receiveLimit: RECEIVE_LIMIT_FOR_STUD,
    // userRange 为 2，指定对象
    userRange: 2,
    // 优惠规则
    ruleVos: [],
    // 券的购买信息
    goodsVo: originData.goodsVo || {},
    // 审核状态，新增券时状态为 AUDIT_UNREVIEW : '0'
    auditStatus: originData.auditStatus || AUDIT_UNREVIEW,
    ...modelWithoutRules,
  };

  const {
    branchId = '',
    productVos = [],
    periodType,
    validDateRange,
    buyingDateRange,
    price,
    ...result
  } = originResult;

  // 分公司
  result.branchVos = branchId
    ? [
        {
          branchId,
          id: 0,
          promotionId: 0,
        },
      ]
    : [];

  const { productVos: originProductVos = [] } = originData;

  // 已选产品
  result.productVos = productVos.map((item) => {
    const keyName = 'productId';
    let tag = originProductVos.find((oriPrd) => oriPrd[keyName] === item[keyName]);

    tag = tag || item;

    return {
      id: tag.id || null,
      productId: tag.productId || null,
      activityId: tag.activityId || null,
    };
  });

  // 固定有效期
  if (periodType === PERIOD_VALID_REGULAR) {
    result.startDate = validDateRange[0].format(rangeFormat);
    result.endDate = validDateRange[1].format(rangeFormat);
  }

  // 是否连报
  const isContinue = result.continuityType !== NONE;

  // 连报条件
  if (result.continuityType) {
    result.continuitySubjectType = isContinue ? 1 : 0;
  }

  // 优惠规则
  const ruleFieldNames = Object.keys(params).filter((name) => /rules_/.test(name) && Array.isArray(params[name]));

  // 原始优惠使用条件
  const originRules = originData.ruleVos || [];

  result.ruleVos = ruleFieldNames.map((ruleName) => {
    const currentRuleModel = params[ruleName];
    const [promotionRule, quarterPromotionA = 0, quarterPromotionB = 0] = currentRuleModel;

    // 优惠规则默认配置项
    const ruleRowId = ruleName.replace('rules_', '');
    const [ruleItem = {}] = originRules.filter((rule) => rule.ruleRowId === ruleRowId);

    return {
      promotionRule: promotionRule,
      promotion: NP.plus(quarterPromotionA, quarterPromotionB),
      quarterPromotionA: isContinue ? quarterPromotionA : 0,
      quarterPromotionB: isContinue ? quarterPromotionB : 0,
      ruleUnit: PRODUCT_NUMS,
      unit: MONEY,
      id: ruleItem.id || null,
      promotionId: ruleItem.promotionId || null,
      useNum: ruleItem.useNum || 0,
      promotionIds: ruleItem.promotionIds || 0,
      giftVos: ruleItem.giftVos || [],
      ruleName: ruleItem.ruleName || '',
      needToRecive: result.needToRecive,
    };
  });

  const rulePromotions = result.ruleVos.map((el) => el.promotion);

  // 买券信息
  result.goodsVo = {
    ...result.goodsVo,
    beginTime: buyingDateRange[0].format(rangeFormat),
    endTime: buyingDateRange[1].format(rangeFormat),
    price,
    discount: Math.max(...rulePromotions),
  };

  return result;
}

/**
 * 处理从远端加载的抵扣券详情（除优惠规则）
 * @param {Object} data 抵扣券详情
 * @return {Object}
 */
export function fieldDataAdapter(data) {
  const [{ branchId = null } = {}] = data.branchVos || [];
  const goodsVo = data.goodsVo || {};

  return {
    promotionName: data.promotionName,
    branchId: branchId,
    showScope: data.showScope,
    campusIds: (data.promotionCampusVos || []).map(({ id }) => id),
    limitScope: isNumber(data.limitScope) ? data.limitScope.toString() : data.limitScope || null,
    receiveNumLimit: data.receiveNumLimit,
    superPosition: isNumber(data.superPosition) ? data.superPosition.toString() : data.superPosition || null,
    periodType: isNumber(data.relativePeriod) && data.relativePeriod > 0 ? PERIOD_VALID_RELATIVE : PERIOD_VALID_REGULAR,
    validDateRange: data.startDate && data.endDate ? [moment(data.startDate), moment(data.endDate)] : null,
    relativePeriod: data.relativePeriod,
    buyingDateRange: goodsVo.beginTime && goodsVo.endTime ? [moment(goodsVo.beginTime), moment(goodsVo.endTime)] : null,
    continuityType: data.continuityType,
    price: goodsVo.price,
    promotionComment: data.promotionComment,
    useCase: data.useCase,
  };
}

/**
 * 处理从远端加载的优惠规则
 * @param {Object} ruleVos 优惠规则列表
 * @return {Object}
 */
export function fieldRuleAdapter(ruleVos, type) {
  const ret = ruleVos.reduce((prev, rule) => {
    const { ruleRowId } = rule;
    const ruleName = `rules_${ruleRowId}`;

    let promotionRule = rule.promotionRule;
    if (promotionRule && type !== 0) {
      // 仅1对1 1对多需要换算课时为小时
      promotionRule = NP.divide(promotionRule, COURSE_HOURS_TRANSFORM_MULTIPLE).toFixed(2);
    }

    prev[ruleName] = [
      // 优惠规则基本数值
      promotionRule,
      // 季度连报优惠规则数值 A
      rule.quarterPromotionB ? rule.quarterPromotionA : rule.promotion,
      // 季度连报优惠规则数值 B
      rule.quarterPromotionB,
    ];

    return prev;
  }, {});

  return ret;
}

/**
 * @description 判断数组是否递增
 */
export function isIncreaseArray(arr) {
  if (arr.length === 0) return false;
  if (arr.length === 1) return true;
  let [item] = arr;
  let result = true;
  for (let i = 1; i < arr.length; i++) {
    if (item < arr[i]) {
      item = arr[i];
    } else {
      result = false;
    }
  }
  return result;
}

/**
 * @description 增强处理
 */
export function submitDataAdapterExtension(params, originData = {}) {
  const isClassType = params.type === 0; // 是否为班课
  delete params.type;
  const processData = submitDataAdapter(params, originData);
  const final = { ...processData };
  final.ruleVos = final.ruleVos.map((rule) => {
    return {
      ...rule,
      promotionRule: isClassType ? rule.promotionRule : NP.times(rule.promotionRule, COURSE_HOURS_TRANSFORM_MULTIPLE),
      ruleUnit: isClassType ? PRODUCT_NUMS : COURSE_HOURS, // 小班1对1 1对多 以课时产品为单位
    };
  });
  return final;
}
