import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'dva';
import moment from 'moment';
import { DatePicker, Input, Select as AntdSelect } from 'antd';
import { Select, SearchFilterForm as SearchForm, SearchFieldForm as SearchFormItem } from 'COMPONENT';
import { MISC_FIELD_DEFAULT_TYPE, miscFieldOptions, usedStatusDict } from '../config';

import styles from './styles.module.less';

const InputGroup = Input.Group;
const Option = AntdSelect.Option;

/**
 * 混合输入框
 * @description 客户号码/学生姓名/学生编号/券编号/订单编号/合同编号
 * @description 默认客户号码
 */
class MiscFieldInput extends React.Component {
  constructor(props) {
    super(props);
  }

  static propTypes = {
    defaultSelectValue: PropTypes.string,
    selectOptions: PropTypes.array,
    onSelectChange: PropTypes.func,
    value: PropTypes.string,
    onChange: PropTypes.func,
  };

  render() {
    const { defaultSelectValue, selectOptions, onSelectChange, value, onChange } = this.props;

    return (
      <InputGroup className={styles.inputGroup} compact size="large">
        <AntdSelect
          className={styles.antdSelect}
          size="large"
          defaultValue={defaultSelectValue}
          onChange={onSelectChange}
        >
          {selectOptions.map((option) => {
            return (
              <Option value={option.value} key={option.value}>
                {option.label}
              </Option>
            );
          })}
        </AntdSelect>
        <Input className={styles.antdInput} placeholder="请输入" value={value} onChange={onChange} />
      </InputGroup>
    );
  }
}

class Search extends React.Component {
  constructor(props) {
    super(props);

    this.getDisabledStartDate = this.getDisabledStartDate.bind(this);
    this.getDisabledEndDate = this.getDisabledEndDate.bind(this);

    this.handleMiscFieldSelect = this.handleMiscFieldSelect.bind(this);
    this.handleStartChange = this.handleStartChange.bind(this);
    this.handleEndChange = this.handleEndChange.bind(this);
    this.handleBranchChange = this.handleBranchChange.bind(this);
    this.getCampusDict = this.getCampusDict.bind(this);
    this.search = this.search.bind(this);

    this.searchRef = null;

    this.state = {
      miscFieldType: MISC_FIELD_DEFAULT_TYPE,
      startDate: null,
      endDate: null,
      branchId: null,
    };
  }

  static propTypes = {
    handleSearch: PropTypes.func,
  };

  /**
   * 起始日期限制
   * @param {moment} startDate
   * @returns {boolean}
   */
  getDisabledStartDate(startDate) {
    const { endDate } = this.state;

    if (!startDate || !endDate) {
      return false;
    }

    return startDate.valueOf() > endDate.valueOf();
  }

  /**
   * 结束日期限制
   * @param {moment} endDate
   * @returns {boolean}
   */
  getDisabledEndDate(endDate) {
    const { startDate } = this.state;

    if (!endDate || !startDate) {
      return false;
    }

    return endDate.valueOf() <= startDate.valueOf();
  }

  /**
   * 处理混合字段输入框类型变化
   * @param {string} value 选中的类型值
   */
  handleMiscFieldSelect(value) {
    this.setState({
      miscFieldType: value,
    });
  }

  /**
   * 处理开始日期变化
   * @param {moment} date
   */
  handleStartChange(date) {
    this.setState({
      startDate: date,
    });
  }

  /**
   * 处理结束日期变化
   * @param {moment} date
   */
  handleEndChange(date) {
    this.setState({
      endDate: date,
    });
  }

  /**
   * 处理分公司变化
   * @param {string} branchId 分公司 id
   */
  handleBranchChange(branchId) {
    this.setState({
      branchId,
    });
    this.searchRef.setFieldsValue({
      campusId: null,
    });
  }

  /**
   * 计算获得学习中心字典
   */
  getCampusDict() {
    const { branchId } = this.state;
    const { branchCampusDict } = this.props;

    return branchId ? branchCampusDict.filter((item) => item.val === branchId) : branchCampusDict;
  }

  /**
   * 执行查询操作
   */
  search() {
    const model = this.searchRef.getFieldsValue();

    const { miscInputField: miscFieldValue, startTime, endTime, ...restModel } = model;
    const { miscFieldType } = this.state;

    const payload = {
      ...restModel,
      startTime: startTime ? startTime.format('YYYY-MM-DD HH:mm:ss') : undefined,
      endTime: endTime ? endTime.format('YYYY-MM-DD HH:mm:ss') : undefined,
      [miscFieldType]: miscFieldValue ? miscFieldValue.trim() : undefined,
    };

    this.props.handleSearch(payload);
  }

  render() {
    const { branchDict } = this.props;

    return (
      <SearchForm truncationNum={5} onSearch={this.search} ref={(c) => (this.searchRef = c)}>
        <SearchFormItem className={styles.searchField} name="miscInputField" span={2}>
          <MiscFieldInput
            defaultSelectValue={MISC_FIELD_DEFAULT_TYPE}
            selectOptions={miscFieldOptions}
            onSelectChange={this.handleMiscFieldSelect}
          />
        </SearchFormItem>
        <SearchFormItem name="startTime" className={styles.datePickerItem}>
          <DatePicker
            placeholder="起始日期"
            disabledDate={this.getDisabledStartDate}
            onChange={this.handleStartChange}
          />
        </SearchFormItem>
        <SearchFormItem name="endTime" className={styles.datePickerItem}>
          <DatePicker placeholder="截止日期" disabledDate={this.getDisabledEndDate} onChange={this.handleEndChange} />
        </SearchFormItem>
        <SearchFormItem name="branchId">
          <Select placeholder="分公司" showSearch dict={branchDict} onChange={this.handleBranchChange} />
        </SearchFormItem>
        {/*<SearchFormItem name="campusId">*/}
        {/*  <Select placeholder="学习中心" showSearch dict={this.getCampusDict()} />*/}
        {/*</SearchFormItem>*/}
        <SearchFormItem name="status">
          <Select placeholder="抵扣券状态" dict={usedStatusDict} />
        </SearchFormItem>
      </SearchForm>
    );
  }
}

const mapStateToProps = (state) => {
  const { branch: branchDict, branchCampus: branchCampusDict } = state.dict;

  return {
    branchDict,
    branchCampusDict,
  };
};

export default connect(mapStateToProps)(Search);
