import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import { Input, Button, List, Modal, DropdownMenu, TextArea } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getCustInfoForSendingCoupon } from 'SERVICE/preferential';
import { storeName } from '../../store/constant.js';
import { STUDENT } from '../../config';

import styles from './styles.module.less';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;

const ContentItem = (props) => {
  const { label, children } = props;

  return (
    <div className={styles.contentItem}>
      <span className={styles.contentItemLabel}>{label}</span>
      <span>{children}</span>
    </div>
  );
};

const Search = (props) => {
  const { inputVal, onChange, onSearch } = props;

  return (
    <Row className="Search" style={{ marginBottom: 20 }}>
      <Col span={8}>
        <Input placeholder="输入客户手机号码" value={inputVal} onChange={onChange} />
      </Col>
      <Col span={4}>
        <Button type="primary" onClick={onSearch} style={{ marginLeft: 20 }}>
          查询
        </Button>
      </Col>
    </Row>
  );
};

const dataColumns = [
  {
    title: '学生姓名',
    dataIndex: 'studentName',
    key: 'studentName',
    width: COL_WIDTH_M,
  },
  {
    title: '年级',
    dataIndex: 'gradeName',
    key: 'gradeName',
    width: COL_WIDTH_M,
  },
  {
    title: '分公司',
    dataIndex: 'branchName',
    key: 'branchName',
    width: COL_WIDTH_L,
  },
  {
    title: '学籍学习中心',
    dataIndex: 'blCampusName',
    key: 'blCampusName',
    width: COL_WIDTH_L,
  },
];

class SendingToPhoneModal extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      name: '发券',
      modalLoading: false,
      custInfoLoading: false,
      // 客户手机号码输入值
      contactNoInputVal: '',
      // 客户手机号码
      contactNo: '',
      // 客户手机号码查询提示
      custInfoTips: '',
      // 学生列表
      studList: [],
      // 短信模板
      promotionOrderSmsTemp: null,
      // 短信code
      promotionOrderSmsCode: null,
    };

    this.initMsgTemplMenus = this.initMsgTemplMenus.bind(this);
    this.buildMsgTemplMenus = this.buildMsgTemplMenus.bind(this);
    this.loadCustInfo = this.loadCustInfo.bind(this);
    this.search = this.search.bind(this);
    this.handleSearchInputChange = this.handleSearchInputChange.bind(this);
    this.confirm = this.confirm.bind(this);
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ modalLoading: true });

    await new Promise(this.initMsgTemplMenus);

    this.setState({ modalLoading: false });
  }

  /**
   * 初始化短信模板列表
   * @param {Function} resolve promise resolve
   */
  initMsgTemplMenus(resolve) {
    this.props.dispatch({
      type: `${storeName}/initMsgTempMenus`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  /**
   * 构建短信模板列表
   * @param {Array} menus
   */
  buildMsgTemplMenus(menus) {
    if (menus && menus.length) {
      return menus.map((item) => ({
        ...item,
        onClick: () =>
          this.setState({
            promotionOrderSmsTemp: item.text,
            promotionOrderSmsCode: item.code,
          }),
      }));
    }

    return [];
  }

  /**
   * 根据手机号码搜索客户信息和学生信息
   */
  search() {
    const inputVal = String(this.state.contactNoInputVal);

    if (this.isContactNoValid(inputVal)) {
      this.loadCustInfo(inputVal.trim());
    } else {
      showMessage('请输入有效的手机号码后再查询！', 'error');
    }
  }

  /**
   * 验证客户手机号码有效性
   * @param {string} contactNo 客户手机号码
   */
  isContactNoValid(contactNo) {
    const str = contactNo.trim();

    return /^1\d{10}$/.test(str);
  }

  /**
   * 加载客户信息和相关学生列表
   * @param {string} contactNo 客户手机号码
   */
  async loadCustInfo(contactNo) {
    this.setState({ custInfoLoading: true });

    try {
      const resp = await getCustInfoForSendingCoupon({
        customerContact: contactNo,
      });

      const studList = resp && resp.data && Array.isArray(resp.data) ? resp.data : [];
      const tips =
        studList.length > 0
          ? '请先确认客户信息准确无误，再点击“确认发送”'
          : '该号码未录入系统，请先确认手机号码，再点击“确定发送”';

      this.setState({
        custInfoLoading: false,
        contactNo,
        custInfoTips: tips,
        studList,
      });
    } catch (e) {
      this.setState({
        custInfoLoading: false,
        contactNo: '',
        custInfoTips: '查询该手机号码失败，无法发券',
        studList: [],
      });
    }
  }

  /**
   * 响应客户手机号码搜索框输入变化
   * @param {Object} event
   */
  handleSearchInputChange(event) {
    this.setState({
      contactNoInputVal: event.target.value,
    });
  }

  /**
   * 响应确定按钮点击
   */
  confirm() {
    const { contactNo, promotionOrderSmsCode } = this.state;
    const { onFinish, data } = this.props;

    this.setState({
      modalLoading: true,
    });

    this.props.dispatch({
      type: `${storeName}/publishCoupon`,
      payload: {
        couponId: data.id,
        msgCode: promotionOrderSmsCode,
        phone: contactNo,
        type: 'saleCoupon',
        callback: (rsp) => {
          this.setState({
            modalLoading: false,
          });
          if (rsp) {
            // 成功
            onFinish && onFinish();
          }
        },
      },
    });
  }

  render() {
    const { onCancel, msgTmplMenus, data } = this.props;
    const {
      name,
      modalLoading,
      custInfoLoading,
      contactNoInputVal,
      contactNo,
      custInfoTips,
      studList,
      promotionOrderSmsTemp,
    } = this.state;

    // 临时：发券给家长时屏蔽短信
    const isSMSEnabled = data.publishObject === STUDENT;

    return (
      <Modal
        className="SendingToPhoneModal"
        title={name}
        loading={modalLoading}
        width={960}
        okText="确认发送"
        onOk={this.confirm}
        okBtnDisabled={!this.isContactNoValid(contactNo)}
        onCancel={onCancel}
      >
        <Search inputVal={contactNoInputVal} onChange={this.handleSearchInputChange} onSearch={this.search} />
        {contactNo && (
          <div className={styles.custInfoWrapper}>
            <span className={styles.contactLabel}>家长号码：</span>
            <span className={styles.contactNo}>{contactNo}</span>
            {custInfoTips && <span className={styles.custInfoTips}>（{custInfoTips}）</span>}
          </div>
        )}
        {studList.length > 0 && (
          <div style={{ height: '300px' }}>
            <List dataSource={studList} columns={dataColumns} loading={!modalLoading && custInfoLoading} />
          </div>
        )}
        {isSMSEnabled && (
          <div style={{ marginTop: 10, marginBottom: 5 }}>
            <Row>
              <Col span={24}>
                <ContentItem label="短信通知">
                  <span style={{ color: '#999' }}>不选择模板即视为不发短信通知</span>
                </ContentItem>
              </Col>
            </Row>
            <Row>
              <TextArea placeholder="请点击下方选择框选择短信模板" value={promotionOrderSmsTemp} rows={3} disabled />
              <DropdownMenu
                style={{ margin: '10px 0 5px', display: 'block' }}
                text="短信模板"
                menus={this.buildMsgTemplMenus(msgTmplMenus)}
                hasArrow={true}
              />
            </Row>
          </div>
        )}
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { msgTempMenus } = state[storeName];

  return {
    msgTmplMenus: msgTempMenus,
  };
};

export default connect(mapStateToProps)(SendingToPhoneModal);
