import { runFunc } from '../../utils/common';
import {
  NONE,
  PROMOTION_FULL_SALE_NORMAL,
  PROMOTION_FULL_SALE_EVERY,
  PROMOTION_DISCOUNT_CASH,
  PROMOTION_DISCOUNT_COURSE,
  PROMOTION_DISCOUNT_PRODUCT,
  PROMOTION_GIVE_HOURS,
  PRODUCT_NUMS,
  FullReducePromotion,
  DiscountPromotion,
  FullGivePromotion,
  conditionUnitCfg,
  continuityPromotionsTypeCfg,
  promotionsPrefixCfg,
  promotionsTypeCfg,
  promotionsUnitCfg,
  needRuleUnitList,
} from './config';
import NP from 'number-precision';

/**
 * 连报 且是 满减+产品数 或 产品折扣时，运行函数
 *
 * @param continuityType {string} required 连报条件
 * @param promotionType {string} required 优惠类型
 * @param ruleUnit {string} required 满减条件类型
 * @param func1 {function=} true时执行
 * @param func2 {function=} false时执行
 *
 * @return {function,boolean}
 * */
function checkContinuePromotion({ continuityType, promotionType, ruleUnit }, func1, func2) {
  if (
    continuityType !== NONE &&
    ((promotionType === PROMOTION_FULL_SALE_NORMAL && ruleUnit === PRODUCT_NUMS) ||
      promotionType === PROMOTION_DISCOUNT_PRODUCT)
  ) {
    return (func1 && runFunc(func1)) || true;
  } else {
    return (func2 && runFunc(func2)) || false;
  }
}

/**
 * 根据优惠类型获取连报条件文字描述配置
 *
 * @param promotionType {string} 优惠类型
 *
 * @return {Object} 文字描述配置
 * */
function getContinuityPromotionsType(promotionType) {
  let result;
  switch (promotionType) {
    case PROMOTION_FULL_SALE_NORMAL:
    case PROMOTION_FULL_SALE_EVERY:
      result = continuityPromotionsTypeCfg[FullReducePromotion];
      break;
    case PROMOTION_DISCOUNT_CASH:
    case PROMOTION_DISCOUNT_COURSE:
    case PROMOTION_DISCOUNT_PRODUCT:
      result = continuityPromotionsTypeCfg[DiscountPromotion];
      break;
    case PROMOTION_GIVE_HOURS:
      result = continuityPromotionsTypeCfg[FullGivePromotion];
      break;
    default:
      result = continuityPromotionsTypeCfg['undefined'];
  }
  return result;
}

/**
 * 根据优惠数据拼装使用条件文字
 *
 * @param data {Object} 优惠数据
 *
 * @return {string} 使用条件
 * */
function getUseCondition(data) {
  let { ruleVos, promotionTypeId: promotionType, continuityType, hightAmount } = data;
  const prefixText = promotionsPrefixCfg[promotionType];
  let prefixUnit = conditionUnitCfg[promotionType];
  let resultText = promotionsTypeCfg[promotionType];
  let resultUnit = promotionsUnitCfg[promotionType];

  if (promotionType === PROMOTION_FULL_SALE_EVERY) {
    return ruleVos
      .map((item) => {
        return `${prefixText}${item.promotionRule}${prefixUnit[item.ruleUnit]}${resultText}${
          item.promotion
        }${resultUnit}，最高减${hightAmount}${resultUnit}`;
      })
      .join('；');
  } else {
    return ruleVos
      .map((item) => {
        let _prefixUnit;
        if (needRuleUnitList.includes(promotionType)) {
          _prefixUnit = prefixUnit[item.ruleUnit];
        } else {
          _prefixUnit = prefixUnit;
        }
        if (typeof _prefixUnit !== 'string') {
          _prefixUnit = '';
        }

        let firstStr = `${prefixText}${item.promotionRule}${_prefixUnit}`;
        let lastStr = '';

        if (
          checkContinuePromotion({ continuityType, promotionType, ruleUnit: item.ruleUnit }) &&
          Array.isArray(item.detailList) &&
          !item.promotion
        ) {
          lastStr = item.detailList
            .map((detail, index) => {
              return `${getContinuityPromotionsType(promotionType)[continuityType][index]}${
                detail.promotion
              }${resultUnit}`;
            })
            .join('');
        } else {
          lastStr = `${resultText}${item.promotion}${resultUnit}`;
        }
        return firstStr + lastStr;
      })
      .join('；');
  }
}

/**
 * 课时数转换小时数
 * tips: 仅`满赠促销`需要转换，即：pageType='FullGivePromotion'
 *
 * @param ruleVos {Array}
 * @param pageType {string< 'FullGivePromotion'|'DiscountPromotion'|'FullReducePromotion' >}
 * @return {(*|{promotionRule: number, promotion: number})[]}
 */
function transformRuleVosToHours(ruleVos = [], pageType = '') {
  if (pageType !== 'FullGivePromotion') {
    return ruleVos;
  }

  return ruleVos.map((item) => {
    let result = { ...item };
    if (item.promotion) {
      let promotion = NP.divide(item.promotion, 1.5);
      promotion = NP.round(promotion, 2);
      result.promotion = promotion;
    }

    if (item.promotionRule) {
      let promotionRule = NP.divide(item.promotionRule, 1.5);
      promotionRule = NP.round(promotionRule, 2);
      result.promotionRule = promotionRule;
    }

    return result;
  });
}

/**
 * 小时数转换课时数
 * tips: 仅`满赠促销`需要转换，即：pageType='FullGivePromotion'
 *
 * @param ruleVos {Array}
 * @param pageType {string< 'FullGivePromotion'|'DiscountPromotion'|'FullReducePromotion' >}
 * @return {(*|{promotionRule: number, promotion: number})[]}
 */
function transformRuleVosToCourse(ruleVos = [], pageType = '') {
  if (pageType !== 'FullGivePromotion') {
    return ruleVos;
  }

  return ruleVos.map((item) => {
    let result = { ...item };
    if (item.promotion) {
      let promotion = NP.times(item.promotion, 1.5);
      promotion = NP.round(promotion, 2);
      result.promotion = promotion;
    }

    if (item.promotionRule) {
      let promotionRule = NP.times(item.promotionRule, 1.5);
      promotionRule = NP.round(promotionRule, 2);
      result.promotionRule = promotionRule;
    }

    return result;
  });
}

export { checkContinuePromotion, getUseCondition, transformRuleVosToHours, transformRuleVosToCourse };
