import React from 'react';
import styles from './styles.less';
import { getDictTextByVal } from 'UTIL/dict';
import classnames from 'classnames';
import { getSummaryBySubjectGroupList } from '../../utils';
import InputNumber from 'COMPONENT/InputNumber';
import Block from 'COMPONENT/Block';
import Button from 'COMPONENT/Button';
import TextLink from 'COMPONENT/TextLink';
import FormItem from 'COMPONENT/FormItem';
import { Row, Col } from 'antd';
import { checkNumber } from 'COMPONENT/Validator/checkers';

const newSubject = { courseHours: null, subjectNums: null, teachNums: null };
const formCfg = {
  labelWidth: 55,
};

class QuaterSubjectList extends React.Component {
  constructor(props) {
    super(props);
    this.keyIdx = 1;
    const subjectList = props.data.subjects.map((s) => {
      return { ...s, key: '' + this.keyIdx++ };
    });
    this.state = {
      quarterId: props.data.quarterId, // {string} 季度ID
      subjectList, // {object[]} 科目列表
    };
    this.errors = {};
  }

  getToolBar = () => {
    return (
      <Button iconType="plus" size="small" onClick={this.add}>
        添加
      </Button>
    );
  };

  add = () => {
    const { subjectList } = this.state;
    const newSubjectList = [
      ...subjectList,
      { courseHours: 3, subjectNums: null, teachNums: null, key: '' + this.keyIdx++ },
    ];
    this.fireChange(newSubjectList);
  };

  delete = (key) => {
    const { subjectList } = this.state;
    const newSubjectList = subjectList.filter((s) => s.key !== key);
    this.fireChange(newSubjectList);
    let newErrors = {};
    Object.keys(this.errors).forEach((k) => {
      if (k.split('_')[0] !== key) {
        newErrors[k] = this.errors[k];
      }
    });
    this.errors = newErrors;
    this.props.onErrorChange(this.errors);
  };

  fireChange = (newSubjectList) => {
    const { quarterId } = this.state;
    this.setState({ subjectList: newSubjectList });
    this.props.onChange({
      quarterId,
      subjects: this.subjectsClearKey(newSubjectList),
    });
  };

  subjectsClearKey = (subjectList) => {
    return subjectList.map((s) => {
      const { key, ...infos } = s;
      return infos;
    });
  };

  onChange = (k, fieldName, val) => {
    const { subjectList } = this.state;
    const newSubjectList = subjectList.map((s) => {
      if (k === s.key) {
        return { ...s, [fieldName]: val };
      }
      return s;
    });
    this.fireChange(newSubjectList);
  };

  errorChange = (key, fieldName, e) => {
    this.errors[key + '_' + fieldName] = e;
    this.props.onErrorChange(this.errors);
  };

  render() {
    const { checking, quarterDict } = this.props;
    const { quarterId, subjectList } = this.state;
    const modalCls = {
      [styles.wrap]: 1,
      [styles.modal]: 1,
    };
    const summary = getSummaryBySubjectGroupList([{ subjects: subjectList }]);
    const editable = !subjectList.some((s) => s.classNums > 0 || s.chargeNums > 0);
    return (
      <Block
        title={getDictTextByVal(quarterDict, quarterId)}
        headerToolBar={editable ? this.getToolBar() : null}
        className={styles.wrap}
      >
        <Row className={styles.summaryWrap}>
          <Col span={8}>
            <FormItem {...formCfg} name="科次数">
              {summary.totalSubjectNums}
            </FormItem>
          </Col>
          <Col span={8}>
            <FormItem {...formCfg} name="讲数">
              {summary.totalTeachNums}
            </FormItem>
          </Col>
          <Col span={8}>
            <FormItem {...formCfg} name="课时数">
              {summary.totalHours}
            </FormItem>
          </Col>
        </Row>
        <div className={styles.table}>
          <div className={styles.tableHeader}>
            <div className={styles.tr}>
              <div className={styles.td}>科次数</div>
              <div className={styles.td}>每讲课时数</div>
              <div className={styles.td}>讲数</div>
              <div className={styles.td}>操作</div>
            </div>
          </div>
          <div className={styles.tableBody}>
            {subjectList.map((s) => {
              /*
                courseHours	课时	string
                    subjectNums	科目数	string
                    teachNums 讲数 string
                * */
              return (
                <div key={s.key} className={styles.tr}>
                  <div className={styles.td}>
                    {editable ? (
                      <InputNumber
                        size="small"
                        precision={0}
                        value={s.subjectNums}
                        onChange={(val) => {
                          this.onChange(s.key, 'subjectNums', val);
                        }}
                        min={1}
                        validator={{
                          onErrorChange: (e) => this.errorChange(s.key, 'subjectNums', e),
                          silent: !checking,
                          rules: ['required', (v) => checkNumber(v, { excludeMin: 0, int: true })],
                        }}
                      />
                    ) : (
                      <div>{s.subjectNums}</div>
                    )}
                  </div>
                  <div className={styles.td}>
                    {editable ? (
                      <InputNumber
                        size="small"
                        precision={1}
                        value={s.courseHours}
                        onChange={(val) => {
                          this.onChange(s.key, 'courseHours', val);
                        }}
                        min={0.1}
                        validator={{
                          onErrorChange: (e) => this.errorChange(s.key, 'courseHours', e),
                          silent: !checking,
                          rules: ['required', (v) => checkNumber(v, { excludeMin: 0 })],
                        }}
                      />
                    ) : (
                      <div>{s.courseHours}</div>
                    )}
                  </div>
                  <div className={styles.td}>
                    {editable ? (
                      <InputNumber
                        size="small"
                        precision={0}
                        value={s.teachNums}
                        onChange={(val) => this.onChange(s.key, 'teachNums', val)}
                        min={1}
                        validator={{
                          onErrorChange: (e) => this.errorChange(s.key, 'teachNums', e),
                          silent: !checking,
                          rules: ['required', (v) => checkNumber(v, { excludeMin: 0, int: true })],
                        }}
                      />
                    ) : (
                      <div>{s.teachNums}</div>
                    )}
                  </div>
                  <div className={styles.td}>
                    {editable && subjectList.length > 1 ? (
                      <TextLink onClick={() => this.delete(s.key)}>删除</TextLink>
                    ) : (
                      <span>-</span>
                    )}
                  </div>
                </div>
              );
            })}
          </div>
        </div>
      </Block>
    );
  }
}

export default QuaterSubjectList;
