/**
 * 产品复制编辑页面
 * @param {[string]} productIds 产品ID
 */
import React from 'react';
import { connect } from 'dva';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import DatePicker from 'COMPONENT/DatePicker';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import { cutText } from 'COMPONENT/utils/basic/str';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { dateToMoment } from 'COMPONENT/utils/form';
import { getDataDictBySonId } from 'SERVICE/system';
import { getDictTextByVal, loadCommonDict, loadAllBranchDict, formatGroupDict } from 'UTIL/dict';
import { getProductByIds, saveOrUpdateMultiProduct } from 'SERVICE/product';
import { loadLinkProductsDict, gradeLevelDict, getGradeLevel, currentYearText } from '../../common';
import { productTypeDict, feelClassDict } from 'CONST/dicts/product';
import styles from 'PAGE/product/copyPage.less';
import TextLink from 'COMPONENT/TextLink';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const productTypeVal = 'LECTURE';
const productType = productTypeDict.find((p) => p.val === productTypeVal);

class CopyEditPage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      list: [], // {[object]} 列表数据
      yearDict: null, // {[object]} 字典 产品年份
      branchsDict: null, // {[object]} 字典 分公司
      productName: '',
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = async () => {
    const {
      params: { productIds },
    } = this.props;
    this.setState({ loading: true });
    const [list, yearDict, branchsDict, quarterDict, classTypeDict, gradeDict] = await Promise.all([
      this.loadProductDetailList(productIds),
      loadCommonDict('PRODUCT_VERSION'),
      loadAllBranchDict(),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('CLASS_TYPE', 'LECTURE'),
      loadCommonDict('STUDENT_GRADE'),
    ]);
    this.setState({ loading: false, list, yearDict, branchsDict, quarterDict, classTypeDict, gradeDict });
  };

  /**
   * 加载产品详情列表，并转换格式
   * @param  {[string]} productIds 产品ID集合
   */
  loadProductDetailList = async (productIds) => {
    const apiDatas = await getProductByIds({ productIds: productIds.join(',') });
    const data = apiDatas.map((apiData) => {
      const otherProducts = apiData.otherProducts || [];
      const linkProductsDict = otherProducts.map((product) => {
        return { val: product.id, text: product.name, branchName: product.branchName, status: product.status };
      });
      const d = {
        additionName: null,
        year: apiData.productVersionId,
        gradeId: apiData.gradeId,
        serise: apiData.courseSeriesId,
        group: apiData.productGroupId,
        classTimeLen: apiData.classTimeLength,
        price: apiData.price,
        startDate: apiData.validStartDate,
        endDate: apiData.validEndDate,
        feeRule: apiData.chargeRuleId,
        feelClass: apiData.experienceLessonId,
        branchs: apiData.branchId,
        linkSubjects: apiData.subjectIdSet,
        linkProducts: apiData.sonProduct.map((p) => p.id),
        editable: apiData.canEdit === 1 ? false : true,
        remark: apiData.remark || '',
        gradeName: apiData.gradeName,
        linkProductsDict,
        quarter: apiData.productQuarterName,
        quarterId: apiData.productQuarterId,
        miniClassTotalhours: apiData.miniClassTotalhours,
        everyCourseHours: apiData.everyCourseHours,
        classTypeName: apiData.classTypeName,
        classTypeId: apiData.classTypeId,
        productVersionName: apiData.productVersionName,
        categoryName: apiData.categoryName,
        gradeSegmentId: apiData.gradeSegmentId,
      };
      return {
        ...d,
        initData: { ...d },
      };
    });
    return data;
  };

  gradeChange = (idx, grade) => {
    if (grade) {
      getDataDictBySonId({ sonId: grade }, true).then((item) => {
        this.updateRow(idx, { gradeSegmentId: item.id, gradeId: grade });
      });
    }
  };

  /**
   * 修改行数据
   * @param  {number} idx 行序号
   * @param  {object} changed 要修改的数据键值对
   */
  updateRow = (idx, changed) => {
    this.setState({
      list: this.state.list.map((row, i) => {
        if (i === idx) {
          return { ...row, ...changed };
        }
        return row;
      }),
    });
  };

  /**
   * 分公司改变时加载其他关联产品选项
   * @param  {number} idx 行序号
   * @param  {string} branchs 分公司
   */
  branchChange = (idx, branchs) => {
    this.updateRow(idx, { branchs, linkProducts: null, linkProductsDict: null });
    loadLinkProductsDict(branchs, productTypeVal).then((linkProductsDict) => {
      this.updateRow(idx, { linkProductsDict });
    });
  };

  /**
   * 年份改变时同步上架日期和下架日期的年份
   * @param  {number} idx 行序号
   * @param  {string} year 年份
   */
  yearChange = (idx, year) => {
    const { list, yearDict } = this.state;
    let changed = { year };
    if (year) {
      const { startDate, endDate } = list[idx];
      const yearText = getDictTextByVal(yearDict, year);
      changed = { year, startDate: yearText + startDate.slice(4), endDate: yearText + endDate.slice(4) };
    }
    this.updateRow(idx, changed);
  };

  /**
   * 获取产品名称自动生成部分
   * @param {string} year 年份ID
   * @param {string} gradeName 年级名称
   */
  getAutoName = (year, classType) => {
    const { yearDict, quarterDict, classTypeDict } = this.state;
    return (getDictTextByVal(yearDict, year) || '') + (getDictTextByVal(classTypeDict, classType) || '');
  };

  /**
   * 保存按钮点击处理
   */
  handleOk = () => {
    const {
      params: { onSaveSuccess },
    } = this.props;
    const { list, yearDict } = this.state;
    const error = (text) => this._page.showOkTips(text);
    for (let i = 0; i < list.length; i++) {
      const {
        additionName,
        year,
        grade,
        serise,
        group,
        classTimeLen,
        price,
        startDate,
        endDate,
        feeRule,
        feelClass,
        branchs,
        linkSubjects,
        linkProducts,
        editable,
        remark,
        gradeName,
        linkProductsDict,
        miniClassTotalhours,
        everyCourseHours,
        gradeId,
        classTypeId,
      } = list[i];
      if (!year) {
        error(`第 ${i + 1} 行年份未填写`);
        return;
      }
      if (!startDate) {
        error(`第 ${i + 1} 行上架日期未填写`);
        return;
      }
      if (!endDate) {
        error(`第 ${i + 1} 行下架日期未填写`);
        return;
      }
      if (!price) {
        error(`第 ${i + 1} 行单价未填写`);
        return;
      }
      /*      if (!miniClassTotalhours) {
        error(`第 ${i + 1} 行总课时数未填写`)
        return
      }
      if (!everyCourseHours) {
        error(`第 ${i + 1} 行课时数/(次) 未填写`)
        return
      }*/
      if (!classTimeLen) {
        error(`第 ${i + 1} 行课时时长未填写`);
        return;
      }
      if (!gradeId) {
        error(`第 ${i + 1} 行年级未填写`);
        return;
      }
      if (!classTypeId) {
        error(`第 ${i + 1} 行讲座类型未填写`);
        return;
      }
      if (!branchs || branchs.length === 0) {
        error(`第 ${i + 1} 行所属分公司未填写`);
        return;
      }
      /*      if (miniClassTotalhours % everyCourseHours !== 0) {
        error(`第 ${i + 1} 行 课时数/次，必须能被总课时数整除`)
        return
      }*/
      if (
        list.find(
          (row, idx) =>
            i !== idx &&
            year === row.year &&
            gradeName === row.gradeName &&
            additionName === row.additionName &&
            branchs.find((b) => row.branchs.indexOf(b) >= 0),
        )
      ) {
        error(`第 ${i + 1} 行产品名称有重复，请修改后再提交`);
        return;
      }
      if (parseInt(endDate.slice(0, 4)) < parseInt(yearDict.find((d) => d.val === year).text)) {
        error(`第 ${i + 1} 行下架日期的年份小于产品年份，请修改后再提交`);
        return;
      }
      if (linkProducts && linkProducts.find((p) => linkProductsDict.find((d) => d.val === p).status === '1')) {
        error(`第 ${i + 1} 行关联产品已失效，请修改后再提交`);
        return;
      }
    }

    let saveData = list.map((p) => {
      let {
        additionName,
        year,
        gradeId,
        serise,
        group,
        classTimeLen,
        price,
        startDate,
        endDate,
        feeRule,
        feelClass,
        branchs,
        linkSubjects,
        linkProducts,
        editable,
        remark,
        linkProductsDict,
        quarterId,
        classTypeId,
        miniClassTotalhours,
        everyCourseHours,
        gradeSegmentId,
      } = p;
      additionName = additionName ? additionName : '';
      let autoName = this.getAutoName(year, classTypeId);
      return {
        name: autoName + additionName,
        shortname: additionName,
        productVersionId: year,
        gradeId: gradeId,
        courseSeriesId: serise,
        productGroupId: group,
        classTimeLength: classTimeLen,
        price,
        validStartDate: startDate,
        validEndDate: endDate,
        chargeRuleId: feeRule,
        experienceLessonId: feelClass,
        branchId: branchs,
        subjectId: linkSubjects,
        sonProductId: linkProducts,
        canEdit: editable ? 0 : 1,
        remark,
        category: productTypeVal,
        productCategoryId: productType.id,
        productQuarterId: quarterId,
        classTypeId,
        miniClassTotalhours,
        everyCourseHours,
        gradeSegmentId,
      };
    });
    this.setState({ loading: true });
    saveOrUpdateMultiProduct(saveData, true).then(
      () => {
        showMessage('保存成功', 'success');
        this.setState({ loading: false });
        onSaveSuccess();
      },
      (e) => {
        if (e.text) {
          error(e.text);
        }
        showMessage('保存失败');
        this.setState({ loading: false });
      },
    );
  };

  /**
   * 移除行
   * @param  {object} record 行数据
   * @param  {number} index  行序号
   */
  removeRow = (record, index) => {
    this.setState({
      list: this.state.list.filter((row, i) => i !== index),
    });
  };

  /**
   * 添加行
   * @param  {object} record 行数据
   * @param  {number} index  行序号
   */
  addRow = (record, index) => {
    const { list } = this.state;
    const { initData, ...props } = list[index];
    const copyRow = { ...props, initData: { ...props } };
    let newList = [...list];
    newList.splice(index + 1, 0, copyRow);
    this.setState({ list: newList });
  };

  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @param  {number} index  行序号
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate = (startDate, index) => {
    const { list } = this.state;
    const endDate = list[index].endDate;
    if (!startDate || !endDate) {
      return false;
    }
    return dateToMoment(startDate).valueOf() > dateToMoment(endDate).valueOf();
  };

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @param  {number} index  行序号
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate = (endDate, index) => {
    const { list } = this.state;
    const startDate = list[index].startDate;
    if (!endDate || !startDate) {
      return false;
    }
    return dateToMoment(endDate).valueOf() <= dateToMoment(startDate).valueOf();
  };

  /**
   * 检验是否有改动
   * @param  {object} record    行记录
   * @param  {string} fieldName 字段名
   * @return {boolean}
   */
  checkDirty = (record, fieldName) => {
    const { initData } = record;
    return initData && !objectDeepEquals(record[fieldName], initData[fieldName]) ? true : false;
  };

  render() {
    const {
      params: { productIds },
    } = this.props;
    const { loading, list, yearDict, branchsDict, quarterDict, classTypeDict, gradeDict } = this.state;
    const columns = [
      {
        title: '产品名称',
        key: 'productName',
        width: COL_WIDTH_L,
        fixed: 'left',
        render: (text, record) => {
          const { year, classTypeId } = record;
          const dirty = this.checkDirty(record, this.getAutoName(year, classTypeId));
          return <div className={dirty ? styles.dirty : styles.clean}>{this.getAutoName(year, classTypeId)}</div>;
        },
      },
      {
        title: '附加名称',
        key: 'additionName',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { additionName } = record;
          const dirty = this.checkDirty(record, 'additionName');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Input
                value={additionName}
                onChange={(e) => this.updateRow(index, { additionName: cutText(e.target.value, 25) })}
              />
            </div>
          );
        },
      },
      {
        title: '年份',
        key: 'year',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { year } = record;
          const dirty = this.checkDirty(record, 'year');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Select dict={yearDict} value={year} onChange={(val) => this.yearChange(index, val)} />
            </div>
          );
        },
      },

      {
        title: '上架日期',
        key: 'startDate',
        width: 150,
        render: (text, record, index) => {
          const { startDate } = record;
          const dirty = this.checkDirty(record, 'startDate');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <DatePicker
                value={startDate}
                onChange={(val) => this.updateRow(index, { startDate: val })}
                disabledDate={(val) => this.getDisabledStartDate(val, index)}
              />
            </div>
          );
        },
      },
      {
        title: '下架日期',
        key: 'endDate',
        width: 150,
        render: (text, record, index) => {
          const { endDate } = record;
          const dirty = this.checkDirty(record, 'endDate');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <DatePicker
                value={endDate}
                onChange={(val) => this.updateRow(index, { endDate: val })}
                disabledDate={(val) => this.getDisabledEndDate(val, index)}
              />
            </div>
          );
        },
      },
      // {
      //     title: '季度', key: 'quarter', width: COL_WIDTH_M, render: (text, record, index) => {
      //     const {quarterId} = record
      //     const dirty = this.checkDirty(record, 'quarterId')
      //     return (
      //         <div className={dirty ? styles.dirty : styles.clean}>
      //             <Select dict={quarterDict} value={quarterId}
      //                     onChange={val => this.updateRow(index, {quarterId: val})}/>
      //         </div>
      //     )
      // }
      // },

      {
        title: '产品单价',
        key: 'price',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { price } = record;
          const dirty = this.checkDirty(record, 'price');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <InputNumber min={0} value={price} onChange={(val) => this.updateRow(index, { price: val })} />
            </div>
          );
        },
      },
      /*      {
        title: '总课时数', key: 'miniClassTotalhours', width: COL_WIDTH_M, render: (text, record, index) => {
        const {miniClassTotalhours} = record
        const dirty = this.checkDirty(record, 'miniClassTotalhours')
        return (
          <div className={dirty ? styles.dirty : styles.clean}>
            <InputNumber min={0} value={miniClassTotalhours}
                         onChange={val => this.updateRow(index, {miniClassTotalhours: val})}/>
          </div>
        )
      }
      },
      {
        title: '课时数/(次)', key: 'everyCourseHours', width: COL_WIDTH_M, render: (text, record, index) => {
          const {everyCourseHours} = record
          const dirty = this.checkDirty(record, 'everyCourseHours')
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <InputNumber min={0} value={everyCourseHours}
                           onChange={val => this.updateRow(index, {everyCourseHours: val})}/>
            </div>
          )
        }
      },*/
      {
        title: '课时时长',
        key: 'classTimeLen',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { classTimeLen } = record;
          const dirty = this.checkDirty(record, 'classTimeLen');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <InputNumber
                min={0}
                value={classTimeLen}
                onChange={(val) => this.updateRow(index, { classTimeLen: val })}
              />
            </div>
          );
        },
      },
      {
        title: '年级',
        key: 'gradeId',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { gradeId } = record;
          const dirty = this.checkDirty(record, 'gradeId');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Select dict={gradeDict} value={gradeId} onChange={(val) => this.gradeChange(index, val)} />
            </div>
          );
        },
      },

      {
        title: '班级类型',
        key: 'classTypeId',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { classTypeId } = record;
          const dirty = this.checkDirty(record, 'classTypeId');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Select
                dict={classTypeDict}
                value={classTypeId}
                onChange={(val) => this.updateRow(index, { classTypeId: val })}
              />
            </div>
          );
        },
      },
      {
        title: '关联其他产品',
        key: 'linkProducts',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          const { linkProducts, linkProductsDict } = record;
          const dirty = this.checkDirty(record, 'linkProducts');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <MultiSelect
                dict={formatGroupDict(linkProductsDict, 'branchName')}
                value={linkProducts}
                onChange={(val) => this.updateRow(index, { linkProducts: val })}
              />
            </div>
          );
        },
      },
      {
        title: '所属分公司',
        key: 'branchs',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          const { branchs } = record;
          const dirty = this.checkDirty(record, 'branchs');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <MultiSelect dict={branchsDict} value={branchs} onChange={(val) => this.branchChange(index, val)} />
            </div>
          );
        },
      },
      // {
      //     title: '体验课设置', key: 'feelClass', width: COL_WIDTH_M, render: (text, record, index) => {
      //     const {feelClass} = record
      //     const dirty = this.checkDirty(record, 'feelClass')
      //     return (
      //         <div className={dirty ? styles.dirty : styles.clean}>
      //             <Select dict={feelClassDict} value={feelClass}
      //                     onChange={val => this.updateRow(index, {feelClass: val})}/>
      //         </div>
      //     )
      // }
      // },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          return (
            <span>
              <TextLink className="list-btn" onClick={() => this.removeRow(record, index)}>
                移除
              </TextLink>
              <TextLink className="list-btn" onClick={() => this.addRow(record, index)}>
                添加
              </TextLink>
            </span>
          );
        },
      },
    ];

    const tips = <div className={styles.tips}>{`当前已选择${productIds.length}条数据`}</div>;

    return (
      <Page
        footer={true}
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        onOk={this.handleOk}
        headerToolBar={tips}
        className={styles.wrap}
        ref={(c) => (this._page = c)}
      >
        <List dataSource={list} columns={columns} pagination={false} />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    // idsArray: state.main.idsArray,
    params: state.main.pageParams['/ProductLecture/CopyEdit'],
  };
};

export default connect(mapStateToProps)(CopyEditPage);
