/**
 * 编辑产品页面，仅修改
 * @param {string} productId 产品ID
 * @param {function} onSaveSucess 保存成功时的回调
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import Page from 'COMPONENT/Page';
import Select from 'COMPONENT/Select';
import TagSelect from 'COMPONENT/TagSelect';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadCommonDict, loadAllBranchDict, getDictTextByVal } from 'UTIL/dict';
import { getProductById, saveOrUpdateProduct } from 'SERVICE/product';
import SelectBookModal from '../../modals/SelectBook';
import { productTypeDict } from 'CONST/dicts/product';
import styles from 'PAGE/product/editPage.less';

const productTypeVal = 'LIVE';
const productType = productTypeDict.find((p) => p.val === productTypeVal);

const FormItemConfig = {
  size: 'small',
  labelWidth: 130,
};

class EditPage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态

      name: null, // {string} 产品名称
      teacher: null, // {string} 主讲老师
      grade: null, // {string} 年级
      subject: null, // {string} 科目
      classType: null, // {string} 类型
      level: null, // {[string]} 层次
      quarter: null, // {string} 季度
      gradeLevel: null, // {string} 年级段
      classTimeLength: null, // {string} 课时时长
      price: null, // {string} 单价（元）
      totalHours: null, // {string} 总课时数
      totalMoney: null, // {string} 总价（元）
      startDate: null, // {string} 上架日期
      endDate: null, // {string} 下架日期
      branchs: null, // {[string]} 所属分公司ID集合
      linkProducts: null, // {string} 其他关联产品
      editable: null, // {string} 关联产品允许编辑
      classBook: null, // {string} 教材
      feeRule: 'chargeCosumeNow', // {string} 扣费规则
      remark: '', // {string} 备注

      subjectDict: null, // {[object]} 字典 科目
      branchsDict: null, // {[object]} 字典 分公司

      detailApiData: null, // {object} 产品详情API返回的数据
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = async () => {
    const {
      params: { productId },
    } = this.props;
    this.setState({ loading: true });
    const [subjectDict, branchsDict, productDetail] = await Promise.all([
      loadCommonDict('SUBJECT'),
      loadAllBranchDict(),
      this.loadProductDetail(productId),
    ]);
    const { subjects, ...detail } = productDetail;
    const subject = subjects ? subjects.map((s) => getDictTextByVal(subjectDict, s)).join(', ') : '';
    this.setState({ loading: false, subjectDict, branchsDict, subject, ...detail });
  };

  /**
   * 加载产品详情，并转换格式
   * @param  {string} productId 产品ID
   */
  loadProductDetail = async (productId) => {
    const apiData = await getProductById({ productId });
    const data = {
      name: apiData.name,
      teacher: apiData.teacherName,
      grade: apiData.gradeName,
      subjects: apiData.subjectIdSet,
      classType: apiData.classTypeName,
      level: apiData.classTypeName,
      quarter: apiData.productQuarterName,
      gradeLevel: apiData.gradeSegmentName,
      classTimeLength: apiData.classTimeLength,
      totalHours: apiData.miniClassTotalhours,
      totalMoney: apiData.totalAmount,
      startDate: apiData.validStartDate,
      endDate: apiData.validEndDate,
      price: apiData.price,
      branchs: apiData.branchId,
      linkProducts: apiData.sonProduct.map((p) => p.name).join(', '),
      editable: apiData.canEdit === 1 ? '否' : '是',
      classBook: apiData.textBookName,
      feeRule: apiData.chargeRuleName,
      remark: apiData.remark,
      detailApiData: apiData,
    };
    return data;
  };

  /**
   * 保存按钮点击处理
   */
  handleOk = () => {
    const {
      params: { productId, onSaveSuccess },
    } = this.props;
    const { branchs, detailApiData } = this.state;
    const error = (text) => this._page.showOkTips(text);
    if (!branchs || branchs.length === 0) {
      error('请填写分公司');
      return;
    }
    const {
      id,
      productVersionId,
      productQuarterId,
      gradeId,
      gradeSegmentId,
      subjectId,
      courseSeriesId,
      productSystemId,
      classTypeId,
      shortname,
      feeTypeId,
      classModalId,
      productGroupId,
      classTimeLength,
      price,
      miniClassTotalhours,
      everyCourseHours,
      validStartDate,
      validEndDate,
      sonProduct,
      canEdit,
      textBookId,
      textBookName,
      chargeRuleId,
      remark,
      category,
      productCategoryId,
      name,
    } = detailApiData;
    let saveData = {
      id,
      productVersionId,
      productQuarterId,
      gradeId,
      gradeSegmentId,
      subjectId,
      courseSeriesId,
      productSystemId,
      classTypeId,
      shortname,
      feeTypeId,
      classModalId,
      productGroupId,
      classTimeLength,
      price,
      miniClassTotalhours,
      everyCourseHours,
      validStartDate,
      validEndDate,
      sonProductId: sonProduct ? sonProduct.map((s) => s.id) : null,
      canEdit,
      textBookId,
      textBookName,
      chargeRuleId,
      remark,
      category,
      productCategoryId,
      name,
      branchId: branchs,
    };
    console.log('保存：', saveData);
    console.log(JSON.stringify(saveData));
    this.setState({ loading: true });
    saveOrUpdateProduct(saveData, true).then(
      () => {
        showMessage('保存成功', 'success');
        this.setState({ loading: false });
        onSaveSuccess();
      },
      (e) => {
        if (e.text) {
          error(e.text);
        }
        showMessage('保存失败');
        this.setState({ loading: false });
      },
    );
  };

  render() {
    const {
      params: { sellCount, productId },
    } = this.props;
    const {
      loading,
      name,
      teacher,
      grade,
      subject,
      classType,
      level,
      quarter,
      gradeLevel,
      classTimeLength,
      price,
      totalHours,
      totalMoney,
      startDate,
      endDate,
      branchs,
      linkProducts,
      editable,
      classBook,
      feeRule,
      remark,
      branchsDict,
    } = this.state;
    return (
      <Page
        name={productId ? '编辑产品' : '新增产品'}
        footer={true}
        loading={loading}
        route={this.props.route}
        onOk={this.handleOk}
        ref={(c) => (this._page = c)}
      >
        <Block title="产品信息" style={{ marginTop: -5, marginLeft: 45 }} className={styles.wrap}>
          <Row style={{ marginLeft: 20, marginTop: 5 }}>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="产品名称">
                {name}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="主讲老师">
                {teacher}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="年级">
                {grade}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="年级段">
                {gradeLevel}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="课时时长（分）">
                {classTimeLength}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="单价（元）">
                {price}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="总课时数">
                {totalHours}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="总价（元）">
                {totalMoney}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="季度">
                {quarter}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="层次">
                {level}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="类型">
                {classType}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="扣费规则">
                {feeRule}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="上架日期">
                {startDate}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="下架日期">
                {endDate}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="教材">
                {classBook}
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem {...FormItemConfig} name="所属分公司" required={true}>
                <TagSelect dict={branchsDict} value={branchs} onChange={(val) => this.setState({ branchs: val })} />
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem {...FormItemConfig} name="关联科目">
                {subject}
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem {...FormItemConfig} name="关联其他产品">
                {linkProducts}
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem {...FormItemConfig} name="关联产品允许编辑">
                {editable}
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem {...FormItemConfig} name="备注">
                {remark}
              </FormItem>
            </Col>
          </Row>
        </Block>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/ProductLive/Edit'],
  };
};

export default connect(mapStateToProps)(EditPage);
