/**
 * 产品复制编辑页面
 * @param {[string]} productIds 产品ID
 */
import React from 'react';
import { connect } from 'dva';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import DatePicker from 'COMPONENT/DatePicker';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import { cutText } from 'COMPONENT/utils/basic/str';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { dateToMoment } from 'COMPONENT/utils/form';
import { getDictTextByVal, loadCommonDict, formatGroupDict, dictAddOptionTitle } from 'UTIL/dict';
import { getDataDictBySonId } from 'SERVICE/system';
import { getProductByIds, saveOrUpdateMultiProduct } from 'SERVICE/product';
import SelectBookModal from '../../modals/SelectBook';
import { loadLinkProductsDict, currentYearText } from '../../common';
import { productTypeDict } from 'CONST/dicts/product';
import styles from 'PAGE/product/copyPage.less';
import TextLink from 'COMPONENT/TextLink';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const productTypeVal = 'SMALL_CLASS';
const productType = productTypeDict.find((p) => p.val === productTypeVal);

class CopyEditPage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      list: [], // {[object]} 列表数据
      yearDict: null, // {[object]} 字典 产品年份
      quarterDict: null, // {[object]} 字典 季度
      gradeDict: null, // {[object]} 字典 年级
      subjectDict: null, // {[object]} 字典 科目
      seriseDict: null, // {[object]} 字典 产品系列
      classTypeDict: null, // {[object]} 字典 班次
      selectBookModalVisible: false, // {boolean} 选择教材弹框显示状态
      currentRowIndex: null, // {number} 当前行序号
    };
  }

  componentDidMount() {
    this.initContent();
  }

  componentWillUnmount() {
    this.isUnmounted = true;
  }

  /**
   * 以Promise的方式更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      if (!this.isUnmounted) {
        this.setState(newState, () => {
          resolve();
        });
      } else {
        resolve();
      }
    });
  };

  /**
   * 初始化内容
   */
  initContent = async () => {
    const {
      params: { productIds },
    } = this.props;
    this.setState({ loading: true });
    const [yearDict, quarterDict, gradeDict, subjectDict, seriseDict, classTypeDict] = await Promise.all([
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
      loadCommonDict('COURSE_SERIES'),
      loadCommonDict('CLASS_TYPE'),
    ]);
    await this.updateState({ yearDict, quarterDict, gradeDict, subjectDict, seriseDict, classTypeDict });
    const list = await this.loadProductDetailList(productIds);
    this.setState({ loading: false, list });
  };

  /**
   * 加载产品详情列表，并转换格式
   * @param  {[string]} productIds 产品ID集合
   */
  loadProductDetailList = async (productIds) => {
    const apiDatas = await getProductByIds({ productIds: productIds.join(',') });
    const data = apiDatas.map((apiData) => {
      const otherProducts = apiData.otherProducts || [];
      const linkProductsDict = otherProducts.map((product) => {
        return { val: product.id, text: product.name, branchName: product.branchName, status: product.status };
      });
      let d = {
        year: apiData.productVersionId,
        quarter: apiData.productQuarterId,
        grade: apiData.gradeId,
        gradeLevel: apiData.gradeSegmentId,
        gradeLevelName: apiData.gradeSegmentName,
        subject: apiData.subjectIdSet,
        serise: apiData.courseSeriesId,
        system: apiData.productSystemId,
        systemName: apiData.productSystemName,
        classType: apiData.classTypeId,
        additionName: null,
        feeType: apiData.feeTypeId,
        classLenType: apiData.classModalId,
        group: apiData.productGroupId,
        classTimeLen: apiData.classTimeLength,
        price: apiData.price,
        totalHours: apiData.miniClassTotalhours,
        totalHoursCount: apiData.everyCourseHours,
        startDate: apiData.validStartDate,
        endDate: apiData.validEndDate,
        branchs: apiData.branchId,
        linkProducts: apiData.sonProduct.map((p) => p.id),
        editable: apiData.canEdit === 1 ? false : true,
        classBook: apiData.textbookList.map((book) => {
          return { val: book.textBookId, text: book.textBookName };
        }),
        feeRule: apiData.chargeRuleId,
        remark: apiData.remark || '',
        linkProductsDict,
      };
      d.autoName = this.getAutoName(d);
      return {
        ...d,
        initData: { ...d },
      };
    });
    return data;
  };

  /**
   * 修改行数据
   * @param  {number} idx 行序号
   * @param  {object} changed 要修改的数据键值对
   * @param  {function?} callback 设置状态完成后的回调
   */
  updateRow = (idx, changed, callback) => {
    this.setState(
      {
        list: this.state.list.map((row, i) => {
          if (i === idx) {
            return { ...row, ...changed };
          }
          return row;
        }),
      },
      callback,
    );
  };

  /**
   * 获取产品名称自动生成部分
   * @param {string} year 年份ID
   * @param {string} quarter 季度ID
   * @param {string} grade 年级ID
   * @param {string} subject 科目ID
   * @param {string} serise 产品系列ID
   * @param {string} classType 班次ID
   */
  getAutoName = ({ year, quarter, grade, subject, serise, classType }) => {
    const { yearDict, quarterDict, gradeDict, subjectDict, seriseDict, classTypeDict } = this.state;
    const subjectNames = subject ? subject.map((s) => getDictTextByVal(subjectDict, s)).join('') : '';
    return (
      (getDictTextByVal(yearDict, year) || '') +
      (getDictTextByVal(quarterDict, quarter) || '') +
      (getDictTextByVal(gradeDict, grade) || '') +
      (subjectNames || '') +
      (getDictTextByVal(seriseDict, serise) || '') +
      (getDictTextByVal(classTypeDict, classType) || '') +
      productType.text
    );
  };

  /**
   * 影响产品名称自动生成部分的字段（年份、季度、年级、科目、产品系列、班次）改变时的处理
   * @param  {number} idx 行序号
   * @param {object?} changed 改变的字段键值对
   * @param  {function?} callback 设置状态完成后的回调
   */
  autoNameFieldsChange = (idx, changed = {}, callback) => {
    const { list } = this.state;
    const { year, quarter, grade, subject, serise, classType } = list[idx];
    this.updateRow(
      idx,
      {
        ...changed,
        autoName: this.getAutoName({ year, quarter, grade, subject, serise, classType, ...changed }),
      },
      callback,
    );
  };

  /**
   * 年份改变时，1、更新年份值  2、更新产品名称自动拼凑部分 3、同步上架日期和下架日期的年份
   * @param  {number} idx 行序号
   * @param  {string} year 年份
   */
  yearChange = (idx, year) => {
    this.autoNameFieldsChange(idx, { year }, () => {
      if (year) {
        const { list, yearDict } = this.state;
        const { startDate, endDate } = list[idx];
        const yearText = getDictTextByVal(yearDict, year);
        this.updateRow(idx, {
          year,
          startDate: startDate ? yearText + startDate.slice(4) : null,
          endDate: endDate ? yearText + endDate.slice(4) : null,
        });
      }
    });
  };

  /**
   * 年级值变化时，1、更新年级值  2、更新产品名称自动拼凑部分  3、更新年级段
   * @param  {number} idx 行序号
   * @param {string} grade 年级ID
   */
  gradeChange = (idx, grade) => {
    this.autoNameFieldsChange(idx, { grade }, () => {
      if (grade) {
        getDataDictBySonId({ sonId: grade }, true).then((item) => {
          this.updateRow(idx, { gradeLevel: item.id, gradeLevelName: item.name });
        });
      }
    });
  };

  /**
   * 产品系列值变化时，1、更新产品系列值  2、更新产品名称自动拼凑部分  3、更新产品体系
   * @param  {number} idx 行序号
   * @param {string} grade 年级ID
   */
  seriseChange = (idx, serise) => {
    this.autoNameFieldsChange(idx, { serise }, () => {
      if (serise) {
        getDataDictBySonId({ sonId: serise }, true).then((item) => {
          this.updateRow(idx, { system: item.id, systemName: item.name });
        });
      }
    });
  };

  /**
   * 保存按钮点击处理
   */
  handleOk = () => {
    const {
      params: { onSaveSuccess },
    } = this.props;
    const { list, yearDict } = this.state;
    const error = (text) => this._page.showOkTips(text);
    for (let i = 0; i < list.length; i++) {
      const {
        year,
        quarter,
        grade,
        gradeLevel,
        subject,
        serise,
        system,
        classType,
        additionName,
        feeType,
        classLenType,
        group,
        classTimeLen,
        price,
        totalHours,
        totalHoursCount,
        startDate,
        endDate,
        branchs,
        linkProducts,
        editable,
        classBook,
        feeRule,
        remark,
        autoName,
        linkProductsDict,
      } = list[i];
      if (!year) {
        error(`第 ${i + 1} 行年份未填写`);
        return;
      }
      if (!startDate) {
        error(`第 ${i + 1} 行上架日期未填写`);
        return;
      }
      if (!endDate) {
        error(`第 ${i + 1} 行下架日期未填写`);
        return;
      }
      if (!quarter) {
        error(`第 ${i + 1} 行季度未填写`);
        return;
      }
      if (!price) {
        error(`第 ${i + 1} 行产品单价未填写`);
        return;
      }
      if (!totalHours) {
        error(`第 ${i + 1} 行课时数未填写`);
        return;
      }
      if (!totalHoursCount) {
        error(`第 ${i + 1} 行课时数/次未填写`);
        return;
      }
      if (!classTimeLen) {
        error(`第 ${i + 1} 行课时时长未填写`);
        return;
      }
      if (!grade) {
        error(`第 ${i + 1} 行年级未填写`);
        return;
      }
      /*      if(!subject || subject.length === 0){
        error(`第 ${i + 1} 行科目未填写`)
        return
      }*/
      if (!serise) {
        error(`第 ${i + 1} 行产品系列未填写`);
        return;
      }
      if (!classType) {
        error(`第 ${i + 1} 行班次未填写`);
        return;
      }
      /*      if(!classBook){
        error(`第 ${i + 1} 行关联教材未填写`)
        return
      }*/
      if (
        list.find(
          (row, idx) =>
            i !== idx &&
            autoName === row.autoName &&
            additionName === row.additionName &&
            branchs.find((b) => row.branchs.indexOf(b) >= 0),
        )
      ) {
        error(`第 ${i + 1} 行产品名称（产品名称 + 附加名称）有重复，请修改后再提交`);
        return;
      }
      if (parseInt(endDate.slice(0, 4)) < parseInt(yearDict.find((d) => d.val === year).text)) {
        error(`第 ${i + 1} 行下架日期的年份小于产品年份，请修改后再提交`);
        return;
      }
      if (totalHours % totalHoursCount !== 0) {
        error(`第 ${i + 1} 行课时数/次，必须能被总课时数整除，请修改后再提交`);
        return;
      }
      if (linkProducts && linkProducts.find((p) => linkProductsDict.find((d) => d.val === p).status === '1')) {
        error(`第 ${i + 1} 行关联产品已失效，请修改后再提交`);
        return;
      }
    }

    let saveData = list.map((p) => {
      const {
        year,
        quarter,
        grade,
        gradeLevel,
        subject,
        serise,
        system,
        classType,
        additionName,
        feeType,
        classLenType,
        group,
        classTimeLen,
        price,
        totalHours,
        totalHoursCount,
        startDate,
        endDate,
        branchs,
        linkProducts,
        editable,
        classBook,
        feeRule,
        remark,
        autoName,
        linkProductsDict,
      } = p;
      return {
        productVersionId: year,
        productQuarterId: quarter,
        gradeId: grade,
        gradeSegmentId: gradeLevel,
        subjectId: subject,
        courseSeriesId: serise,
        productSystemId: system,
        classTypeId: classType,
        shortname: additionName || '',
        feeTypeId: feeType,
        classModalId: classLenType,
        productGroupId: group,
        classTimeLength: classTimeLen,
        price,
        miniClassTotalhours: totalHours,
        everyCourseHours: totalHoursCount,
        validStartDate: startDate,
        validEndDate: endDate,
        branchId: branchs,
        sonProductId: linkProducts,
        canEdit: editable ? 0 : 1,
        textBooks: classBook.map((book) => {
          return { textBookId: book.val, textBookName: book.text };
        }),
        chargeRuleId: feeRule,
        remark,
        category: productTypeVal,
        productCategoryId: productType.id,
        name: (autoName || '') + (additionName || ''),
      };
    });
    console.log('保存：', saveData);
    console.log(JSON.stringify(saveData));
    this.setState({ loading: true });
    saveOrUpdateMultiProduct(saveData, true).then(
      () => {
        showMessage('保存成功', 'success');
        this.setState({ loading: false });
        onSaveSuccess();
      },
      (e) => {
        if (e.text) {
          error(e.text);
        }
        showMessage('保存失败');
        this.setState({ loading: false });
      },
    );
  };

  /**
   * 移除行
   * @param  {object} record 行数据
   * @param  {number} index  行序号
   */
  removeRow = (record, index) => {
    this.setState({
      list: this.state.list.filter((row, i) => i !== index),
    });
  };

  /**
   * 添加行
   * @param  {object} record 行数据
   * @param  {number} index  行序号
   */
  addRow = (record, index) => {
    const { list } = this.state;
    const { initData, ...props } = list[index];
    const copyRow = { ...props, initData: { ...props } };
    let newList = [...list];
    newList.splice(index + 1, 0, copyRow);
    this.setState({ list: newList });
  };

  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @param  {number} index  行序号
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate = (startDate, index) => {
    const { list } = this.state;
    const endDate = list[index].endDate;
    if (!startDate || !endDate) {
      return false;
    }
    return dateToMoment(startDate).valueOf() > dateToMoment(endDate).valueOf();
  };

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @param  {number} index  行序号
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate = (endDate, index) => {
    const { list } = this.state;
    const startDate = list[index].startDate;
    if (!endDate || !startDate) {
      return false;
    }
    return dateToMoment(endDate).valueOf() <= dateToMoment(startDate).valueOf();
  };

  /**
   * 检验是否有改动
   * @param  {object} record    行记录
   * @param  {string} fieldName 字段名
   * @return {boolean}
   */
  checkDirty = (record, fieldName) => {
    const { initData } = record;
    return initData && !objectDeepEquals(record[fieldName], initData[fieldName]) ? true : false;
  };

  /**
   * 打开选择教材弹框
   * @param  {number} idx 行号
   */
  selectBook = (idx) => {
    this.setState({ selectBookModalVisible: true, currentRowIndex: idx });
  };

  classBookSelect = (idx, books) => {
    console.log(idx, books);
    const { list } = this.state;
    const classBook = list[idx].classBook;
    let newClassBook = [...classBook];
    if (books) {
      books.forEach((b) => {
        if (!classBook || !classBook.find((c) => c.val === b.val)) {
          newClassBook.push(b);
        }
      });
    }
    this.updateRow(idx, { classBook: newClassBook });
    this.setState({ selectBookModalVisible: false });
  };

  render() {
    const {
      params: { productIds },
    } = this.props;
    const {
      loading,
      selectBookModalVisible,
      currentRowIndex,
      list,
      yearDict,
      quarterDict,
      gradeDict,
      subjectDict,
      seriseDict,
      classTypeDict,
    } = this.state;

    const columns = [
      {
        title: '产品名称',
        key: 'productName',
        width: COL_WIDTH_XL,
        fixed: 'left',
        render: (text, record, index) => {
          const { autoName } = record;
          const dirty = this.checkDirty(record, 'autoName');
          return (
            <div className={dirty ? styles.dirty : styles.clean} title={autoName}>
              {autoName}
            </div>
          );
        },
      },
      {
        title: '附加名称',
        key: 'additionName',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { additionName } = record;
          const dirty = this.checkDirty(record, 'additionName');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Input
                value={additionName}
                onChange={(e) => this.updateRow(index, { additionName: cutText(e.target.value, 25) })}
              />
            </div>
          );
        },
      },
      {
        title: '年份',
        key: 'year',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { year } = record;
          const dirty = this.checkDirty(record, 'year');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Select dict={yearDict} value={year} onChange={(val) => this.yearChange(index, val)} />
            </div>
          );
        },
      },
      {
        title: '上架日期',
        key: 'startDate',
        width: 150,
        render: (text, record, index) => {
          const { startDate } = record;
          const dirty = this.checkDirty(record, 'startDate');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <DatePicker
                value={startDate}
                onChange={(val) => this.updateRow(index, { startDate: val })}
                disabledDate={(val) => this.getDisabledStartDate(val, index)}
              />
            </div>
          );
        },
      },
      {
        title: '下架日期',
        key: 'endDate',
        width: 150,
        render: (text, record, index) => {
          const { endDate } = record;
          const dirty = this.checkDirty(record, 'endDate');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <DatePicker
                value={endDate}
                onChange={(val) => this.updateRow(index, { endDate: val })}
                disabledDate={(val) => this.getDisabledEndDate(val, index)}
              />
            </div>
          );
        },
      },
      {
        title: '季度',
        key: 'quarter',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { quarter } = record;
          const dirty = this.checkDirty(record, 'quarter');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Select
                dict={quarterDict}
                value={quarter}
                onChange={(val) => this.autoNameFieldsChange(index, { quarter: val })}
              />
            </div>
          );
        },
      },
      {
        title: '产品单价',
        key: 'price',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { price } = record;
          const dirty = this.checkDirty(record, 'price');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <InputNumber min={0} value={price} onChange={(val) => this.updateRow(index, { price: val })} />
            </div>
          );
        },
      },
      {
        title: '课时数',
        key: 'totalHours',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { totalHours } = record;
          const dirty = this.checkDirty(record, 'totalHours');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <InputNumber min={0} value={totalHours} onChange={(val) => this.updateRow(index, { totalHours: val })} />
            </div>
          );
        },
      },
      {
        title: '课时数/次',
        key: 'totalHoursCount',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { totalHoursCount } = record;
          const dirty = this.checkDirty(record, 'totalHoursCount');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <InputNumber
                min={0}
                value={totalHoursCount}
                onChange={(val) => this.updateRow(index, { totalHoursCount: val })}
              />
            </div>
          );
        },
      },
      {
        title: '课时时长',
        key: 'classTimeLen',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { classTimeLen } = record;
          const dirty = this.checkDirty(record, 'classTimeLen');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <InputNumber
                min={0}
                value={classTimeLen}
                onChange={(val) => this.updateRow(index, { classTimeLen: val })}
              />
            </div>
          );
        },
      },
      {
        title: '年级',
        key: 'grade',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { grade } = record;
          const dirty = this.checkDirty(record, 'grade');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Select dict={gradeDict} value={grade} onChange={(val) => this.gradeChange(index, val)} />
            </div>
          );
        },
      },
      {
        title: '科目',
        key: 'subject',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          const { subject } = record;
          const dirty = this.checkDirty(record, 'subject');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <MultiSelect
                dict={subjectDict}
                value={subject}
                onChange={(val) => this.autoNameFieldsChange(index, { subject: val })}
              />
            </div>
          );
        },
      },
      {
        title: '产品系列',
        key: 'serise',
        width: COL_WIDTH_XL,
        render: (text, record, index) => {
          const { serise } = record;
          const dirty = this.checkDirty(record, 'serise');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Select
                showSearch
                dict={dictAddOptionTitle(seriseDict)}
                value={serise}
                onChange={(val) => this.seriseChange(index, val)}
              />
            </div>
          );
        },
      },
      {
        title: '班次',
        key: 'classType',
        width: COL_WIDTH_XL,
        render: (text, record, index) => {
          const { classType } = record;
          const dirty = this.checkDirty(record, 'classType');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Select
                showSearch
                dict={dictAddOptionTitle(classTypeDict)}
                value={classType}
                onChange={(val) => this.autoNameFieldsChange(index, { classType: val })}
              />
            </div>
          );
        },
      },
      {
        title: '关联产品',
        key: 'linkProducts',
        width: COL_WIDTH_XL,
        render: (text, record, index) => {
          const { linkProducts, linkProductsDict } = record;
          const dirty = this.checkDirty(record, 'linkProducts');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <MultiSelect
                dict={formatGroupDict(linkProductsDict, 'branchName')}
                value={linkProducts}
                onChange={(val) => this.updateRow(index, { linkProducts: val })}
              />
            </div>
          );
        },
      },
      {
        title: '关联教材',
        key: 'classBook',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          const { classBook } = record;
          const dirty = this.checkDirty(record, 'classBook');
          const names = classBook.map((b) => b.text).join('、');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Input title={names} value={names} readOnly={true} onClick={() => this.selectBook(index)} />
            </div>
          );
        },
      },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          return (
            <span>
              <TextLink className="list-btn" onClick={() => this.removeRow(record, index)}>
                移除
              </TextLink>
              <TextLink className="list-btn" onClick={() => this.addRow(record, index)}>
                添加
              </TextLink>
            </span>
          );
        },
      },
    ];

    const tips = <div className={styles.tips}>{`当前已选择${productIds.length}条数据`}</div>;

    return (
      <Page
        footer={true}
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        onOk={this.handleOk}
        headerToolBar={tips}
        className={styles.wrap}
        ref={(c) => (this._page = c)}
      >
        <List dataSource={list} columns={columns} pagination={false} />
        {selectBookModalVisible && (
          <SelectBookModal
            onSelect={(books) => this.classBookSelect(currentRowIndex, books)}
            onClose={() => this.setState({ selectBookModalVisible: false })}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/ProductMiniClass/CopyEdit'],
  };
};

export default connect(mapStateToProps)(CopyEditPage);
