/**
 * 编辑产品页面，新增或修改
 * @param {string?} productId 产品ID，新增时为空
 * @param {function} onSaveSucess 保存成功时的回调
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import Radio from 'COMPONENT/Radio';
import Page from 'COMPONENT/Page';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Checkbox from 'COMPONENT/Checkbox';
import TagSelect from 'COMPONENT/TagSelect';
import DatePicker from 'COMPONENT/DatePicker';
import TextArea from 'COMPONENT/TextArea';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import { cutText } from 'COMPONENT/utils/basic/str';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { dateToMoment } from 'COMPONENT/utils/form';
import { getDictTextByVal, loadCommonDict, loadBranchDict, loadAllBranchDict, formatGroupDict } from 'UTIL/dict';
import { getDataDictBySonId } from 'SERVICE/system';
import { getProductById, saveOrUpdateProduct } from 'SERVICE/product';
import { loadLinkProductsDict, currentYearText } from '../../common';
import { productTypeDict } from 'CONST/dicts/product';
import styles from 'PAGE/product/editPage.less';
import { formatCurrency } from 'COMPONENT/utils/basic/num';

const RadioGroup = Radio.Group;

const productTypeVal = 'ONE_ON_ONE_COURSE';
const productType = productTypeDict.find((p) => p.val === productTypeVal);

class EditPage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态

      autoName: null, // {string} 产品名称自动生成部分
      additionName: null, // {string} 附加名称
      year: null, // {string} 产品年份
      grade: null, // {string} 年级
      gradeLevel: null, // {string} 年级段
      gradeLevelName: null, // {string} 年级段名称
      serise: null, // {string} 产品系列
      group: 'DAT0000000207', // {string} 产品组
      classTimeLen: 40, // {number} 课时时长
      price: null, // {number} 单价
      startDate: null, // {string} 上架日期
      endDate: null, // {string} 下架日期
      feeRule: 'chargeCosumeNow', // {string} 扣费规则
      feelClass: 'noneLessons', // {string} 体验课设置
      branchs: null, // {[string]} 所属分公司
      linkSubjects: null, // {[string]} 关联科目
      linkProducts: null, // {[string]} 其他关联产品
      editable: true, // {boolean} 关联产品允许编辑
      remark: '', // {string} 备注
      sellCount: null, // {number} 销售数量

      yearDict: null, // {[object]} 字典 产品年份
      gradeDict: null, // {[object]} 字典 年级
      seriseDict: null, // {[object]} 字典 产品系列
      groupDict: null, // {[object]} 字典 产品组
      feeRuleDict: null, // {[object]} 字典 扣费规则
      branchsDict: null, // {[object]} 字典 分公司
      linkSubjectsDict: null, // {[object]} 字典 关联科目
      linkProductsDict: null, // {[object]} 字典 其他关联产品
      feelClassDict: null, // {[object]} 字典 体验课
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = async () => {
    const {
      params: { productId },
    } = this.props;
    this.setState({ loading: true });
    let calls = [
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('COURSE_SERIES'),
      loadCommonDict('PRODUCT_GROUP'),
      loadCommonDict('PRODUCT_CHARGE_RULE'),
      loadAllBranchDict(),
      loadCommonDict('SUBJECT'),
      loadCommonDict('PRODUCT_EXPERIENCE_LESSON'),
    ];
    if (productId) {
      calls.push(this.loadProductDetail(productId));
    }
    const [
      yearDict,
      gradeDict,
      seriseDict,
      groupDict,
      feeRuleDict,
      branchsDict,
      linkSubjectsDict,
      feelClassDict,
      productDetail,
    ] = await Promise.all(calls);
    let changed = {
      yearDict,
      gradeDict,
      seriseDict,
      groupDict,
      feeRuleDict,
      branchsDict,
      linkSubjectsDict,
      feelClassDict,
    };
    if (productDetail) {
      const linkProductsDict = await loadLinkProductsDict(productDetail.branchs, productTypeVal);
      changed = { ...changed, ...productDetail, linkProductsDict };
    } else {
      const yearOpt = yearDict.find((y) => y.text === currentYearText);
      changed = { ...changed, year: yearOpt ? yearOpt.val : null };
    }
    this.setState({ loading: false, ...changed }, this.autoNameFieldsChange);
  };

  /**
   * 加载产品详情，并转换格式
   * @param  {string} productId 产品ID
   */
  loadProductDetail = async (productId) => {
    const apiData = await getProductById({ productId });
    const data = {
      additionName: apiData.shortname,
      year: apiData.productVersionId,
      grade: apiData.gradeId,
      gradeLevel: apiData.gradeSegmentId,
      gradeLevelName: apiData.gradeSegmentName,
      serise: apiData.courseSeriesId,
      group: apiData.productGroupId,
      classTimeLen: apiData.classTimeLength,
      price: apiData.price,
      startDate: apiData.validStartDate,
      endDate: apiData.validEndDate,
      feeRule: apiData.chargeRuleId,
      feelClass: apiData.experienceLessonId,
      branchs: apiData.branchId,
      linkSubjects: apiData.subjectIdSet,
      linkProducts: apiData.sonProduct.map((p) => p.id),
      editable: apiData.canEdit === 1 ? false : true,
      remark: apiData.remark || '',
      sellCount: apiData.sellCount,
    };
    return data;
  };

  /**
   * 分公司改变时加载其他关联产品选项
   * @param  {string} branchs 分公司
   */
  branchChange = (branchs) => {
    this.setState({ branchs, linkProducts: null, linkProductsDict: null });
    loadLinkProductsDict(branchs, productTypeVal).then((linkProductsDict) => {
      this.setState({ linkProductsDict });
    });
  };

  /**
   * 获取产品名称自动生成部分
   * @param {string} year 年份ID
   * @param {string} grade 年级ID
   */
  getAutoName = ({ year, grade }) => {
    const { yearDict, gradeDict } = this.state;
    return (getDictTextByVal(yearDict, year) || '') + (getDictTextByVal(gradeDict, grade) || '') + productType.text;
  };

  /**
   * 影响产品名称自动生成部分的字段（年份、年级）改变时的处理
   * @param {object?} changed 改变的字段键值对
   */
  autoNameFieldsChange = (changed = {}) => {
    const { year, grade } = this.state;
    this.setState({
      ...changed,
      autoName: this.getAutoName({ year, grade, ...changed }),
    });
  };

  /**
   * 年级值变化时，1、更新年级值  2、更新产品名称自动拼凑部分  3、更新年级段
   * @param {string} grade 年级ID
   */
  gradeChange = (grade) => {
    this.autoNameFieldsChange({ grade });
    this.setState({ gradeLevelName: '' });
    if (grade) {
      getDataDictBySonId({ sonId: grade }, true).then((item) => {
        this.setState({ gradeLevel: item.id, gradeLevelName: item.name });
      });
    }
  };

  /**
   * 保存按钮点击处理
   */
  handleOk = () => {
    const {
      params: { productId, onSaveSuccess },
    } = this.props;
    const {
      autoName,
      additionName,
      year,
      grade,
      gradeLevel,
      serise,
      group,
      classTimeLen,
      price,
      startDate,
      endDate,
      feeRule,
      feelClass,
      branchs,
      linkSubjects,
      linkProducts,
      editable,
      remark,
      yearDict,
    } = this.state;
    const error = (text) => this._page.showOkTips(text);
    if (!year) {
      error('请填写产品年份');
      return;
    }
    if (!grade) {
      error('请填写年级');
      return;
    }
    if (!classTimeLen) {
      error('请填写课时时长');
      return;
    }
    if (!price) {
      error('请填写单价');
      return;
    }
    if (!startDate) {
      error('请填写上架日期');
      return;
    }
    if (!endDate) {
      error('请填写下架日期');
      return;
    }
    if (!branchs || branchs.length === 0) {
      error('请填写所属分公司');
      return;
    }
    if (!feeRule) {
      error('请填写扣费规则');
      return;
    }
    if (!feelClass) {
      error('请填写体验课设置');
      return;
    }
    if (!linkSubjects || linkSubjects.length === 0) {
      error('请填写关联科目');
      return;
    }
    if (parseInt(endDate.slice(0, 4)) < parseInt(yearDict.find((d) => d.val === year).text)) {
      error('下架日期的年份不能小于产品年份');
      return;
    }
    let saveData = {
      name: autoName + (additionName || ''),
      shortname: additionName,
      productVersionId: year,
      gradeId: grade,
      gradeSegmentId: gradeLevel,
      courseSeriesId: serise,
      productGroupId: group,
      classTimeLength: classTimeLen,
      price,
      validStartDate: startDate,
      validEndDate: endDate,
      chargeRuleId: feeRule,
      experienceLessonId: feelClass,
      branchId: branchs,
      subjectId: linkSubjects,
      sonProductId: linkProducts,
      canEdit: editable ? 0 : 1,
      remark,
      category: productTypeVal,
      productCategoryId: productType.id,
    };
    if (productId) {
      saveData.id = productId;
    }
    console.log('保存：', saveData);
    console.log(JSON.stringify(saveData));
    this.setState({ loading: true });
    saveOrUpdateProduct(saveData, true).then(
      () => {
        showMessage('保存成功', 'success');
        this.setState({ loading: false });
        onSaveSuccess();
      },
      (e) => {
        if (e.text) {
          error(e.text);
        }
        showMessage('保存失败');
        this.setState({ loading: false });
      },
    );
  };

  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate = (startDate) => {
    const { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return dateToMoment(startDate).valueOf() > dateToMoment(endDate).valueOf();
  };

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return dateToMoment(endDate).valueOf() <= dateToMoment(startDate).valueOf();
  };

  render() {
    const {
      params: { sellCount, productId },
    } = this.props;
    const {
      loading,
      autoName,
      additionName,
      year,
      grade,
      gradeLevelName,
      serise,
      group,
      classTimeLen,
      price,
      startDate,
      endDate,
      feeRule,
      feelClass,
      branchs,
      linkSubjects,
      linkProducts,
      yearDict,
      gradeDict,
      seriseDict,
      groupDict,
      feeRuleDict,
      branchsDict,
      linkSubjectsDict,
      linkProductsDict,
      editable,
      remark,
      feelClassDict,
    } = this.state;
    const partEdit = sellCount && sellCount > 0 ? true : false;
    return (
      <Page
        name={productId ? '编辑产品' : '新增产品'}
        footer={true}
        loading={loading}
        route={this.props.route}
        onOk={this.handleOk}
        ref={(c) => (this._page = c)}
      >
        <Block title="产品信息" style={{ marginTop: -5, marginLeft: 45 }} className={styles.wrap}>
          <Row style={{ marginLeft: -26, marginTop: 5 }}>
            <Col span={12}>
              <FormItem name="产品名称">
                <Row gutter={10}>
                  <Col span={14} className="text-ellipsis">
                    {autoName}
                  </Col>
                  <Col span={10}>
                    <Input
                      placeholder="附加名称"
                      maxLength="10"
                      value={additionName}
                      onChange={(e) => this.setState({ additionName: cutText(e.target.value, 25) })}
                      disabled={partEdit}
                    />
                  </Col>
                </Row>
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="产品年份" required={true}>
                <Select
                  dict={yearDict}
                  value={year}
                  onChange={(val) => this.autoNameFieldsChange({ year: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="年级" required={true}>
                <Select dict={gradeDict} value={grade} onChange={this.gradeChange} disabled={partEdit} />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="年级段">{gradeLevelName}</FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="产品系列">
                <Select
                  dict={seriseDict}
                  value={serise}
                  onChange={(val) => this.setState({ serise: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="产品组">
                <Select
                  dict={groupDict}
                  value={group}
                  onChange={(val) => this.setState({ group: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="课时时长（分）" required={true}>
                <InputNumber
                  min={1}
                  precision={0}
                  value={classTimeLen}
                  onChange={(val) => this.setState({ classTimeLen: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="单价（元）" required={true}>
                <InputNumber
                  min={0.01}
                  precision={2}
                  value={price}
                  onChange={(val) => this.setState({ price: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="上架日期" required={true}>
                <DatePicker
                  value={startDate}
                  onChange={(val) => this.setState({ startDate: val })}
                  disabled={partEdit}
                  disabledDate={(val) => this.getDisabledStartDate(val)}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="下架日期" required={true}>
                <DatePicker
                  value={endDate}
                  onChange={(val) => this.setState({ endDate: val })}
                  disabledDate={(val) => this.getDisabledEndDate(val)}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="扣费规则" required={true}>
                <Select
                  dict={feeRuleDict}
                  value={feeRule}
                  onChange={(val) => this.setState({ feeRule: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem name="体验课设置" required={true}>
                <RadioGroup
                  value={feelClass}
                  onChange={(e) => this.setState({ feelClass: e.target.value })}
                  disabled={partEdit}
                >
                  {feelClassDict &&
                    feelClassDict.map((item) => (
                      <Radio key={item.val} value={item.val}>
                        {item.text}
                      </Radio>
                    ))}
                </RadioGroup>
              </FormItem>
            </Col>
            <Col span={20}>
              <FormItem name="所属分公司" required={true}>
                <TagSelect dict={branchsDict} value={branchs} onChange={this.branchChange} disabled={partEdit} />
              </FormItem>
            </Col>
            <Col span={20}>
              <FormItem name="关联科目" required>
                <TagSelect
                  dict={linkSubjectsDict}
                  value={linkSubjects}
                  onChange={(val) => this.setState({ linkSubjects: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={20}>
              <FormItem name="关联其他产品">
                <TagSelect
                  dict={formatGroupDict(linkProductsDict, 'branchName')}
                  value={linkProducts}
                  onChange={(val) => {
                    console.log(val);
                    this.setState({ linkProducts: val });
                  }}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={4}>
              <FormItem labelWidth={0} style={{ paddingLeft: 0, paddingRight: 5, marginLeft: -30 }}>
                <Checkbox
                  checked={editable}
                  onChange={(e) => this.setState({ editable: e.target.checked })}
                  disabled={partEdit}
                >
                  关联产品允许编辑
                </Checkbox>
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem name="备注" style={{ marginTop: 20, lineHeight: '36px' }}>
                <TextArea
                  rows={3}
                  value={remark}
                  onChange={(e) => this.setState({ remark: cutText(e.target.value, 50) })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
          </Row>
        </Block>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/ProductOneToOne/Edit'],
  };
};

export default connect(mapStateToProps)(EditPage);
