/**
 * 产品详情查看模态框
 * @property {string} productId 产品ID
 * @property {function} onClose 关闭弹框事件触发时的回调
 */
import React from 'react';
import { Row, Col, Button } from 'antd';
import Modal from 'COMPONENT/Modal';
import { getDictTextByVal } from 'UTIL/dict';
import { getProductById } from 'SERVICE/product';
import { loadCommonDict, loadAllBranchDict } from 'UTIL/dict';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import { statusDict } from 'CONST/dicts/product';

const FormItemConfig = {
  size: 'small',
  labelWidth: 130,
};

class ModalDetailView extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      productDetail: {}, // {object} 产品详情
      subjectDict: null, // {[object]} 字典 科目
      branchsDict: null, // {[object]} 字典 分公司
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    const { productId } = this.props;
    this.setState({ loading: true });
    Promise.all([this.loadProductDetail(productId), loadAllBranchDict(), loadCommonDict('SUBJECT')]).then(
      ([productDetail, branchsDict, subjectDict]) => {
        this.setState({ loading: false, productDetail, branchsDict, subjectDict });
      },
    );
  };

  /**
   * 加载产品详情，并转换格式
   * @param  {string} productId 产品ID
   */
  loadProductDetail = async (productId) => {
    const apiData = await getProductById({ productId });
    const linkProductObjs = apiData.sonProduct || [];
    const data = {
      name: apiData.name,
      year: apiData.productVersionName,
      grade: apiData.gradeName,
      gradeLevel: apiData.gradeSegmentName,
      serise: apiData.courseSeriesName,
      group: apiData.productGroupName,
      classTimeLength: apiData.classTimeLength,
      price: apiData.price,
      startDate: apiData.validStartDate,
      endDate: apiData.validEndDate,
      feeRule: apiData.chargeRuleName,
      feelClass: apiData.experienceLessonName,
      branchIds: apiData.branchId,
      subjectIds: apiData.subjectIdSet,
      linkProducts: linkProductObjs.map((product) => product.name).join('、'),
      linkProductsEditable: apiData.canEdit === 1 ? '否' : '是',
      remark: apiData.remark,
    };
    return data;
  };

  render() {
    const { onClose } = this.props;
    const {
      loading,
      productDetail: {
        name,
        year,
        grade,
        gradeLevel,
        serise,
        group,
        classTimeLength,
        price,
        startDate,
        endDate,
        feeRule,
        feelClass,
        branchIds,
        subjectIds,
        linkProducts,
        linkProductsEditable,
        remark,
      },
      branchsDict,
      subjectDict,
    } = this.state;
    return (
      <Modal loading={loading} onCancel={onClose} title="查看信息" width={1012} okBtnShow={false} cancelText="关闭">
        <Block title={name} style={{ marginTop: -10 }}>
          <Row gutter={10} style={{ paddingLeft: 20 }}>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="产品年份">
                {year}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="年级">
                {grade}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="年级段">
                {gradeLevel}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="产品系列">
                {serise}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="产品组">
                {group}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="课时时长（分）">
                {classTimeLength}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="单价（元）">
                {price}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="产品有效期">{`${startDate} 至 ${endDate}`}</FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="扣费规则">
                {feeRule}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="体验课设置">
                {feelClass}
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem {...FormItemConfig} name="所属分公司">
                {branchIds ? branchIds.map((b) => getDictTextByVal(branchsDict, b)).join('、') : ''}
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem {...FormItemConfig} name="关联科目">
                {subjectIds ? subjectIds.map((b) => getDictTextByVal(subjectDict, b)).join('、') : ''}
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem {...FormItemConfig} name="其他关联产品">
                {linkProducts}
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem {...FormItemConfig} name="关联产品允许编辑">
                {linkProductsEditable}
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem {...FormItemConfig} name="备注">
                {remark}
              </FormItem>
            </Col>
          </Row>
        </Block>
      </Modal>
    );
  }
}

export default ModalDetailView;
