/**
 * 产品复制编辑页面
 * @param {[string]} productIds 产品ID
 */
import React from 'react';
import { connect } from 'dva';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import DatePicker from 'COMPONENT/DatePicker';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import { cutText } from 'COMPONENT/utils/basic/str';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { dateToMoment } from 'COMPONENT/utils/form';
import { getDictTextByVal, loadCommonDict, loadAllBranchDict, dictAddOptionTitle } from 'UTIL/dict';
import { getDataDictBySonId } from 'SERVICE/system';
import { getProductByIds, saveOrUpdateMultiProduct } from 'SERVICE/product';
import { currentYearText } from '../../common';
import { productTypeDict } from 'CONST/dicts/product';
import styles from 'PAGE/product/copyPage.less';
import TextLink from 'COMPONENT/TextLink';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const productTypeVal = 'OTHERS';
const productType = productTypeDict.find((p) => p.val === productTypeVal);

class CopyEditPage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      list: [], // {[object]} 列表数据
      yearDict: null, // {[object]} 字典 产品年份
      gradeDict: null, // {[object]} 字典 年级
      belongProductTypeDict: null, // {[object]} 字典 所属产品类型
      feeTypeDict: null, // {[object]} 字典 费用类型
      linkStuffDict: null, // {[object]} 字典 关联物料
      branchsDict: null, // {[object]} 字典 分公司
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = async () => {
    const {
      params: { productIds },
    } = this.props;
    this.setState({ loading: true });
    const [list, yearDict, gradeDict, belongProductTypeDict, feeTypeDict, branchsDict] = await Promise.all([
      this.loadProductDetailList(productIds),
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('OTHER_OF_CATEGORY'),
      loadCommonDict('PRODUCT_FEE_TYPE'),
      loadAllBranchDict(),
    ]);
    this.setState({ loading: false, list, yearDict, gradeDict, belongProductTypeDict, feeTypeDict, branchsDict });
  };

  /**
   * 加载产品详情列表，并转换格式
   * @param  {[string]} productIds 产品ID集合
   */
  loadProductDetailList = async (productIds) => {
    const apiDatas = await getProductByIds({ productIds: productIds.join(',') });
    const data = apiDatas.map((apiData) => {
      const d = {
        name: apiData.name,
        year: apiData.productVersionId,
        grade: apiData.gradeId,
        gradeLevel: apiData.gradeSegmentId,
        gradeLevelName: apiData.gradeSegmentName,
        belongProductType: apiData.otherOfCategoryId,
        feeType: apiData.feeTypeId,
        price: apiData.price,
        startDate: apiData.validStartDate,
        endDate: apiData.validEndDate,
        feeRule: apiData.chargeRuleId,
        branchs: apiData.branchId,
        linkStuff: null,
        remark: apiData.remark || '',
      };
      return {
        ...d,
        initData: { ...d },
      };
    });
    return data;
  };

  /**
   * 修改行数据
   * @param  {number} idx 行序号
   * @param  {object} changed 要修改的数据键值对
   * @param  {function?} callback 设置状态完成后的回调
   */
  updateRow = (idx, changed, callback) => {
    this.setState(
      {
        list: this.state.list.map((row, i) => {
          if (i === idx) {
            return { ...row, ...changed };
          }
          return row;
        }),
      },
      callback,
    );
  };

  /**
   * 年份改变时同步上架日期和下架日期的年份
   * @param  {number} idx 行序号
   * @param  {string} year 年份
   */
  yearChange = (idx, year) => {
    const { list, yearDict } = this.state;
    let changed = { year };
    if (year) {
      const { startDate, endDate } = list[idx];
      const yearText = getDictTextByVal(yearDict, year);
      changed = {
        year,
        startDate: startDate ? yearText + startDate.slice(4) : null,
        endDate: endDate ? yearText + endDate.slice(4) : null,
      };
    }
    this.updateRow(idx, changed);
  };

  /**
   * 年级值变化时，1、更新年级值  2、更新产品名称自动拼凑部分  3、更新年级段
   * @param  {number} idx 行序号
   * @param {string} grade 年级ID
   */
  gradeChange = (idx, grade) => {
    this.updateRow(idx, { grade }, () => {
      if (grade) {
        getDataDictBySonId({ sonId: grade }, true).then((item) => {
          this.updateRow(idx, { gradeLevel: item.id, gradeLevelName: item.name });
        });
      }
    });
  };

  /**
   * 保存按钮点击处理
   */
  handleOk = () => {
    const {
      params: { onSaveSuccess },
    } = this.props;
    const { list, yearDict } = this.state;
    const error = (text) => this._page.showOkTips(text);
    for (let i = 0; i < list.length; i++) {
      const { name, year, grade, startDate, endDate, price, belongProductType, feeType, linkStuff, branchs } = list[i];
      if (!name) {
        error(`第 ${i + 1} 行产品名称未填写`);
        return;
      }
      if (!year) {
        error(`第 ${i + 1} 行年份未填写`);
        return;
      }
      if (!grade) {
        error(`第 ${i + 1} 行年级未填写`);
        return;
      }
      if (!belongProductType) {
        error(`第 ${i + 1} 行所属产品类型未填写`);
        return;
      }
      if (!feeType) {
        error(`第 ${i + 1} 行费用类型未填写`);
        return;
      }
      if (!price) {
        error(`第 ${i + 1} 行单价未填写`);
        return;
      }
      if (!startDate) {
        error(`第 ${i + 1} 行上架日期未填写`);
        return;
      }
      if (!endDate) {
        error(`第 ${i + 1} 行下架日期未填写`);
        return;
      }
      if (!branchs || branchs.length === 0) {
        error(`第 ${i + 1} 行所属分公司未填写`);
        return;
      }
      if (list.find((row, idx) => i !== idx && name === row.name && branchs.find((b) => row.branchs.indexOf(b) >= 0))) {
        error(`第 ${i + 1} 行产品名称有重复，请修改后再提交`);
        return;
      }
      if (parseInt(endDate.slice(0, 4)) < parseInt(yearDict.find((d) => d.val === year).text)) {
        error(`第 ${i + 1} 行下架日期的年份小于产品年份，请修改后再提交`);
        return;
      }
    }

    let saveData = list.map((p) => {
      const {
        name,
        year,
        grade,
        startDate,
        endDate,
        price,
        belongProductType,
        feeType,
        linkStuff,
        branchs,
        gradeLevel,
        feeRule,
        remark,
      } = p;
      return {
        name,
        productVersionId: year,
        gradeId: grade,
        gradeSegmentId: gradeLevel,
        otherOfCategoryId: belongProductType,
        feeTypeId: feeType,
        price,
        validStartDate: startDate,
        validEndDate: endDate,
        branchId: branchs,
        //xxx: linkStuff,
        remark,
        category: productTypeVal,
        productCategoryId: productType.id,
      };
    });
    console.log('保存：', saveData);
    console.log(JSON.stringify(saveData));
    this.setState({ loading: true });
    saveOrUpdateMultiProduct(saveData, true).then(
      () => {
        showMessage('保存成功', 'success');
        this.setState({ loading: false });
        onSaveSuccess();
      },
      (e) => {
        if (e.text) {
          error(e.text);
        }
        showMessage('保存失败');
        this.setState({ loading: false });
      },
    );
  };

  /**
   * 移除行
   * @param  {object} record 行数据
   * @param  {number} index  行序号
   */
  removeRow = (record, index) => {
    this.setState({
      list: this.state.list.filter((row, i) => i !== index),
    });
  };

  /**
   * 添加行
   * @param  {object} record 行数据
   * @param  {number} index  行序号
   */
  addRow = (record, index) => {
    const { list } = this.state;
    const { initData, ...props } = list[index];
    const copyRow = { ...props, initData: { ...props } };
    let newList = [...list];
    newList.splice(index + 1, 0, copyRow);
    this.setState({ list: newList });
  };

  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @param  {number} index  行序号
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate = (startDate, index) => {
    const { list } = this.state;
    const endDate = list[index].endDate;
    if (!startDate || !endDate) {
      return false;
    }
    return dateToMoment(startDate).valueOf() > dateToMoment(endDate).valueOf();
  };

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @param  {number} index  行序号
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate = (endDate, index) => {
    const { list } = this.state;
    const startDate = list[index].startDate;
    if (!endDate || !startDate) {
      return false;
    }
    return dateToMoment(endDate).valueOf() <= dateToMoment(startDate).valueOf();
  };

  /**
   * 检验是否有改动
   * @param  {object} record    行记录
   * @param  {string} fieldName 字段名
   * @return {boolean}
   */
  checkDirty = (record, fieldName) => {
    const { initData } = record;
    return initData && !objectDeepEquals(record[fieldName], initData[fieldName]) ? true : false;
  };

  render() {
    const {
      params: { productIds },
    } = this.props;
    const { loading, list, yearDict, gradeDict, belongProductTypeDict, feeTypeDict, branchsDict, linkStuffDict } =
      this.state;

    const columns = [
      {
        title: '产品名称',
        key: 'name',
        width: COL_WIDTH_XL,
        render: (text, record, index) => {
          const { name } = record;
          const dirty = this.checkDirty(record, 'name');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Input value={name} onChange={(e) => this.updateRow(index, { name: cutText(e.target.value, 25) })} />
            </div>
          );
        },
      },
      {
        title: '年份',
        key: 'year',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { year } = record;
          const dirty = this.checkDirty(record, 'year');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Select dict={yearDict} value={year} onChange={(val) => this.yearChange(index, val)} />
            </div>
          );
        },
      },
      {
        title: '年级',
        key: 'grade',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { grade } = record;
          const dirty = this.checkDirty(record, 'grade');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Select dict={gradeDict} value={grade} onChange={(val) => this.gradeChange(index, val)} />
            </div>
          );
        },
      },
      {
        title: '上架日期',
        key: 'startDate',
        width: 150,
        render: (text, record, index) => {
          const { startDate } = record;
          const dirty = this.checkDirty(record, 'startDate');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <DatePicker
                value={startDate}
                onChange={(val) => this.updateRow(index, { startDate: val })}
                disabledDate={(val) => this.getDisabledStartDate(val, index)}
              />
            </div>
          );
        },
      },
      {
        title: '下架日期',
        key: 'endDate',
        width: 150,
        render: (text, record, index) => {
          const { endDate } = record;
          const dirty = this.checkDirty(record, 'endDate');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <DatePicker
                value={endDate}
                onChange={(val) => this.updateRow(index, { endDate: val })}
                disabledDate={(val) => this.getDisabledEndDate(val, index)}
              />
            </div>
          );
        },
      },
      {
        title: '产品单价',
        key: 'price',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { price } = record;
          const dirty = this.checkDirty(record, 'price');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <InputNumber min={0} value={price} onChange={(val) => this.updateRow(index, { price: val })} />
            </div>
          );
        },
      },
      {
        title: '所属产品类型',
        key: 'belongProductType',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          const { belongProductType } = record;
          const dirty = this.checkDirty(record, 'belongProductType');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Select
                dict={belongProductTypeDict}
                value={belongProductType}
                onChange={(val) => this.updateRow(index, { belongProductType: val })}
              />
            </div>
          );
        },
      },
      {
        title: '费用类型',
        key: 'feeType',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const { feeType } = record;
          const dirty = this.checkDirty(record, 'feeType');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Select dict={feeTypeDict} value={feeType} onChange={(val) => this.updateRow(index, { feeType: val })} />
            </div>
          );
        },
      },
      {
        title: '关联物料',
        key: 'linkStuff',
        width: COL_WIDTH_XL,
        render: (text, record, index) => {
          const { linkStuff } = record;
          const dirty = this.checkDirty(record, 'linkStuff');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Select
                showSearch
                dict={dictAddOptionTitle(linkStuffDict)}
                value={linkStuff}
                onChange={(val) => this.updateRow(index, { linkStuff: val })}
              />
            </div>
          );
        },
      },
      {
        title: '所属分公司',
        key: 'branchs',
        width: COL_WIDTH_XL,
        render: (text, record, index) => {
          const { branchs } = record;
          const dirty = this.checkDirty(record, 'branchs');
          return (
            <div className={dirty ? styles.dirty : styles.clean}>
              <Select
                dict={dictAddOptionTitle(branchsDict)}
                value={branchs ? branchs[0] : null}
                showSearch
                onChange={(val) => this.updateRow(index, { branchs: val ? val[0] : null })}
              />
            </div>
          );
        },
      },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          return (
            <span>
              <TextLink className="list-btn" onClick={() => this.removeRow(record, index)}>
                移除
              </TextLink>
              <TextLink className="list-btn" onClick={() => this.addRow(record, index)}>
                添加
              </TextLink>
            </span>
          );
        },
      },
    ];

    const tips = <div className={styles.tips}>{`当前已选择${productIds.length}条数据`}</div>;

    return (
      <Page
        footer={true}
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        onOk={this.handleOk}
        headerToolBar={tips}
        className={styles.wrap}
        ref={(c) => (this._page = c)}
      >
        <List dataSource={list} columns={columns} pagination={false} />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/ProductOthers/CopyEdit'],
  };
};

export default connect(mapStateToProps)(CopyEditPage);
