/**
 * 编辑产品页面，新增或修改
 * @param {string?} productId 产品ID，新增时为空
 * @param {function} onSaveSucess 保存成功时的回调
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Page from 'COMPONENT/Page';
import Select from 'COMPONENT/Select';
import DatePicker from 'COMPONENT/DatePicker';
import TextArea from 'COMPONENT/TextArea';
import Block from 'COMPONENT/Block';
import TagSelect from 'COMPONENT/TagSelect';
import FormItem from 'COMPONENT/FormItem';
import { cutText } from 'COMPONENT/utils/basic/str';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { dateToMoment } from 'COMPONENT/utils/form';
import { getDictTextByVal, loadCommonDict, loadAllBranchDict } from 'UTIL/dict';
import { getDataDictBySonId } from 'SERVICE/system';
import { getProductById, saveOrUpdateProduct } from 'SERVICE/product';
import { currentYearText } from '../../common';
import { productTypeDict } from 'CONST/dicts/product';
import styles from 'PAGE/product/editPage.less';
import { formatCurrency } from 'COMPONENT/utils/basic/num';

const productTypeVal = 'OTHERS';
const productType = productTypeDict.find((p) => p.val === productTypeVal);

class EditPage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态

      name: null, // {string} 产品名称
      year: null, // {string} 产品年份
      grade: null, // {string} 年级
      gradeLevel: null, // {string} 年级段
      gradeLevelName: null, // {string} 年级段名称
      belongProductType: null, // {string} 所属产品类型
      feeType: null, // {string} 费用类型
      price: null, // {number} 单价
      startDate: null, // {string} 上架日期
      endDate: null, // {string} 下架日期
      branchs: null, // {[string]} 所属分公司
      linkStuff: null, // {string} 关联物料
      remark: '', // {string} 备注
      sellCount: '', // {number} 销售数量

      yearDict: null, // {[object]} 字典 产品年份
      gradeDict: null, // {[object]} 字典 年级
      belongProductTypeDict: null, // {[object]} 字典 所属产品类型
      feeTypeDict: null, // {[object]} 字典 费用类型
      branchsDict: null, // {[object]} 字典 所属分公司
      linkStuffDict: null, // {[object]} 字典 关联物料
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = async () => {
    const {
      params: { productId },
    } = this.props;
    this.setState({ loading: true });
    let calls = [
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('OTHER_OF_CATEGORY'),
      loadCommonDict('PRODUCT_FEE_TYPE'),
      loadAllBranchDict(),
    ];
    if (productId) {
      calls.push(this.loadProductDetail(productId));
    }
    const [yearDict, gradeDict, belongProductTypeDict, feeTypeDict, branchsDict, productDetail] = await Promise.all(
      calls,
    );
    let changed = { yearDict, gradeDict, belongProductTypeDict, feeTypeDict, branchsDict };
    if (productDetail) {
      const linkStuffDict = await this.loadlinkStuffDict(productDetail.branchs, productTypeVal);
      changed = { ...changed, ...productDetail, linkStuffDict };
    } else {
      const yearOpt = yearDict.find((y) => y.text === currentYearText);
      changed = { ...changed, year: yearOpt ? yearOpt.val : null };
    }
    this.setState({ loading: false, ...changed });
  };

  /**
   * 加载产品详情，并转换格式
   * @param  {string} productId 产品ID
   */
  loadProductDetail = async (productId) => {
    const apiData = await getProductById({ productId });
    const data = {
      name: apiData.name,
      year: apiData.productVersionId,
      grade: apiData.gradeId,
      gradeLevel: apiData.gradeSegmentId,
      gradeLevelName: apiData.gradeSegmentName,
      belongProductType: apiData.otherOfCategoryId,
      feeType: apiData.feeTypeId,
      price: apiData.price,
      startDate: apiData.validStartDate,
      endDate: apiData.validEndDate,
      feeRule: apiData.chargeRuleId,
      branchs: apiData.branchId,
      linkStuff: null,
      remark: apiData.remark || '',
      sellCount: apiData.sellCount,
    };
    return data;
  };

  /**
   * 加载关联物料字典
   */
  loadlinkStuffDict = async () => {
    return [];
  };

  /**
   * 年级值变化时，1、更新年级值  2、更新产品名称自动拼凑部分  3、更新年级段
   * @param {string} grade 年级ID
   */
  gradeChange = (grade) => {
    this.setState({ grade, gradeLevelName: '' });
    if (grade) {
      getDataDictBySonId({ sonId: grade }, true).then((item) => {
        this.setState({ gradeLevel: item.id, gradeLevelName: item.name });
      });
    }
  };

  /**
   * 保存按钮点击处理
   */
  handleOk = () => {
    const {
      params: { productId, onSaveSuccess },
    } = this.props;
    const {
      name,
      year,
      grade,
      gradeLevel,
      belongProductType,
      feeType,
      price,
      startDate,
      endDate,
      branchs,
      linkStuff,
      remark,
      yearDict,
    } = this.state;
    const error = (text) => this._page.showOkTips(text);
    if (!name) {
      error('请填写产品名称');
      return;
    }
    if (!year) {
      error('请填写产品年份');
      return;
    }
    if (!grade) {
      error('请填写年级');
      return;
    }
    if (!belongProductType) {
      error('请填写所属产品类型');
      return;
    }
    if (!feeType) {
      error('请填写费用类型');
      return;
    }
    if (!price) {
      error('请填写单价');
      return;
    }
    if (!startDate) {
      error('请填写上架日期');
      return;
    }
    if (!endDate) {
      error('请填写下架日期');
      return;
    }
    if (!branchs || branchs.length === 0) {
      error('请填写所属分公司');
      return;
    }
    if (parseInt(endDate.slice(0, 4)) < parseInt(yearDict.find((d) => d.val === year).text)) {
      error('下架日期的年份不能小于产品年份');
      return;
    }
    let saveData = {
      name,
      productVersionId: year,
      gradeId: grade,
      gradeSegmentId: gradeLevel,
      otherOfCategoryId: belongProductType,
      feeTypeId: feeType,
      price,
      validStartDate: startDate,
      validEndDate: endDate,
      branchId: branchs,
      //xxx: linkStuff,
      remark,
      category: productTypeVal,
      productCategoryId: productType.id,
    };
    if (productId) {
      saveData.id = productId;
    }
    console.log('保存：', saveData);
    console.log(JSON.stringify(saveData));
    this.setState({ loading: true });
    saveOrUpdateProduct(saveData, true).then(
      () => {
        showMessage('保存成功', 'success');
        this.setState({ loading: false });
        onSaveSuccess();
      },
      (e) => {
        if (e.text) {
          error(e.text);
        }
        showMessage('保存失败');
        this.setState({ loading: false });
      },
    );
  };

  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate = (startDate) => {
    const { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return dateToMoment(startDate).valueOf() > dateToMoment(endDate).valueOf();
  };

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return dateToMoment(endDate).valueOf() <= dateToMoment(startDate).valueOf();
  };

  render() {
    const {
      params: { sellCount, productId },
    } = this.props;
    const {
      loading,
      name,
      year,
      grade,
      gradeLevel,
      gradeLevelName,
      belongProductType,
      feeType,
      price,
      startDate,
      endDate,
      branchs,
      linkStuff,
      remark,
      yearDict,
      gradeDict,
      belongProductTypeDict,
      feeTypeDict,
      branchsDict,
    } = this.state;
    const partEdit = sellCount && sellCount > 0 ? true : false;
    return (
      <Page
        name={productId ? '编辑产品' : '新增产品'}
        footer={true}
        loading={loading}
        route={this.props.route}
        onOk={this.handleOk}
        ref={(c) => (this._page = c)}
      >
        <Block title="产品信息" style={{ marginTop: -5, marginLeft: 45 }} className={styles.wrap}>
          <Row style={{ marginLeft: -26, marginTop: 5 }}>
            <Col span={12}>
              <FormItem name="产品名称" required={true}>
                <Input
                  value={name}
                  onChange={(e) => this.setState({ name: cutText(e.target.value, 25) })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="产品年份" required={true}>
                <Select
                  dict={yearDict}
                  value={year}
                  onChange={(val) => this.setState({ year: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="年级" required={true}>
                <Select dict={gradeDict} value={grade} onChange={this.gradeChange} disabled={partEdit} />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="年级段">{gradeLevelName}</FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="单价（元）" required={true}>
                <InputNumber
                  min={0.01}
                  precision={2}
                  value={price}
                  onChange={(val) => this.setState({ price: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="所属产品类型" required={true}>
                <Select
                  dict={belongProductTypeDict}
                  value={belongProductType}
                  onChange={(val) => this.setState({ belongProductType: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="费用类型" required={true}>
                <Select
                  dict={feeTypeDict}
                  value={feeType}
                  onChange={(val) => this.setState({ feeType: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="关联物料">
                <Select
                  dict={null}
                  value={linkStuff}
                  onChange={(val) => this.setState({ linkStuff: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="上架日期" required={true}>
                <DatePicker
                  value={startDate}
                  onChange={(val) => this.setState({ startDate: val })}
                  disabled={partEdit}
                  disabledDate={(val) => this.getDisabledStartDate(val)}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="下架日期" required={true}>
                <DatePicker
                  value={endDate}
                  onChange={(val) => this.setState({ endDate: val })}
                  disabledDate={(val) => this.getDisabledEndDate(val)}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="所属分公司" required={true}>
                <TagSelect
                  dict={branchsDict}
                  value={branchs}
                  onChange={(branchs) => this.setState({ branchs })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem name="备注" style={{ marginTop: 20, lineHeight: '36px' }}>
                <TextArea
                  rows={3}
                  value={remark}
                  onChange={(e) => this.setState({ remark: cutText(e.target.value, 50) })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
          </Row>
        </Block>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/ProductOthers/Edit'],
  };
};

export default connect(mapStateToProps)(EditPage);
