/**
 * 产品详情查看模态框
 * @property {string} productId 产品ID
 * @property {function} onClose 关闭弹框事件触发时的回调
 */
import React from 'react';
import { Row, Col, Button } from 'antd';
import Modal from 'COMPONENT/Modal';
import { getDictTextByVal } from 'UTIL/dict';
import { getProductById } from 'SERVICE/product';
import { loadCommonDict, loadAllBranchDict } from 'UTIL/dict';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import { statusDict } from 'CONST/dicts/product';

const FormItemConfig = {
  size: 'small',
  labelWidth: 130,
};

class ModalDetailView extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      productDetail: {}, // {object} 产品详情
      belongProductTypeDict: null, // {[object]} 字典 所属产品类型
      branchsDict: null, // {[object]} 字典 分公司
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    const { productId } = this.props;
    this.setState({ loading: true });
    Promise.all([this.loadProductDetail(productId), loadAllBranchDict(), loadCommonDict('OTHER_OF_CATEGORY')]).then(
      ([productDetail, branchsDict, belongProductTypeDict]) => {
        this.setState({ loading: false, productDetail, branchsDict, belongProductTypeDict });
      },
    );
  };

  /**
   * 加载产品详情，并转换格式
   * @param  {string} productId 产品ID
   */
  loadProductDetail = async (productId) => {
    const apiData = await getProductById({ productId });
    const linkProductObjs = apiData.sonProduct || [];
    const data = {
      name: apiData.name,
      year: apiData.productVersionName,
      grade: apiData.gradeName,
      gradeLevel: apiData.gradeSegmentName,
      belongProductTypeId: apiData.otherOfCategoryId,
      feeType: apiData.feeTypeName,
      price: apiData.price,
      linkStuff: null,
      startDate: apiData.validStartDate,
      endDate: apiData.validEndDate,
      branchIds: apiData.branchId,
      remark: apiData.remark,
    };
    return data;
  };

  render() {
    const { onClose } = this.props;
    const {
      loading,
      productDetail: {
        name,
        year,
        grade,
        gradeLevel,
        belongProductTypeId,
        feeType,
        price,
        linkStuff,
        startDate,
        endDate,
        branchIds,
        remark,
      },
      branchsDict,
      belongProductTypeDict,
    } = this.state;
    return (
      <Modal loading={loading} onCancel={onClose} title="查看信息" width={1012} okBtnShow={false} cancelText="关闭">
        <Block title={name} style={{ marginTop: -10 }}>
          <Row gutter={10} style={{ paddingLeft: 20 }}>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="产品年份">
                {year}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="年级">
                {grade}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="年级段">
                {gradeLevel}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="单价（元）">
                {price}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="所属产品类型">
                {getDictTextByVal(belongProductTypeDict, belongProductTypeId)}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="费用类型">
                {feeType}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="关联物料">
                {linkStuff}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="产品有效期">{`${startDate} 至 ${endDate}`}</FormItem>
            </Col>
            <Col span={24}>
              <FormItem {...FormItemConfig} name="所属分公司">
                {branchIds ? branchIds.map((b) => getDictTextByVal(branchsDict, b)).join('、') : ''}
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem {...FormItemConfig} name="备注">
                {remark}
              </FormItem>
            </Col>
          </Row>
        </Block>
      </Modal>
    );
  }
}

export default ModalDetailView;
