import React from 'react';
import moment from 'moment';
import { Input, Radio, Select, InputNumber } from 'COMPONENT';
import { Row, Col, DatePicker, Form } from 'antd';
import Tip from '../Tip';
import { PeriodTypeDict, SINGLE_AUTUMN, SINGLE_SPRING, PeriodTextMap } from '../../const';
import { QuarterTextMap, QuarterRenderMap } from './utils';
import { getUserBranchList } from 'SERVICE/product';
import style from './style.module.less';
import { runFunc } from 'COMPONENT/utils/common';

const FormItem = Form.Item;
const RadioGroup = Radio.Group;
const { RangePicker } = DatePicker;
const formItemLayout = {
  labelCol: {
    xs: { span: 24 },
    sm: { span: 2 },
  },
  wrapperCol: {
    xs: { span: 24 },
    sm: { span: 12 },
  },
};

class AddForm extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      // 组织列表
      branchDict: [],
    };
  }

  async componentDidMount() {
    await this.loadBranchDict();
  }

  async loadBranchDict() {
    const { resultCode, data } = await getUserBranchList({ resourceId: 'PERIOD_CHARGE' });
    if (resultCode === 0) {
      this.setState({ branchDict: data.map((r) => ({ val: r.id, text: r.name })) });
    }
  }

  /**
   * @desc 渲染季度类型
   */
  renderQuarterContent() {
    const { upNumber, downNumber, upClick, downClick, isEdit, organizationId, type } = this.props;
    const render = QuarterRenderMap[type];
    const btnDisabled = isEdit ? false : !organizationId;
    return render
      ? render(
          isEdit ? '增加' : '选择',
          {
            upNumber,
            downNumber,
          },
          btnDisabled,
          {
            onClickUp() {
              runFunc(upClick);
            },
            onClickDown() {
              runFunc(downClick);
            },
          },
        )
      : null;
  }

  /**
   * @desc 渲染相应文字
   */
  renderText() {
    return QuarterTextMap[this.props.type];
  }

  isNotSingleQuarter() {
    return [SINGLE_SPRING, SINGLE_AUTUMN].indexOf(this.props.type) === -1;
  }

  getSingleTipText() {
    const type = PeriodTextMap.get(this.props.type);
    const quarter = QuarterTextMap[this.props.type];
    return this.isNotSingleQuarter()
      ? `设置分段收费后${type}时，${quarter}班只收取已设置的收费小时数`
      : `设置分段收费后，${quarter}只能收取已设置的收费小时数`;
  }

  render() {
    const { form, isEdit } = this.props;
    const { getFieldDecorator } = form;

    return (
      <Form className={style.formWrap}>
        <FormItem {...formItemLayout} required label="分段收费名称">
          <Row type="flex" align="middle">
            <Col lg={8} md={14} xs={20}>
              {getFieldDecorator('name', {
                rules: [
                  {
                    required: true,
                    message: '请输入分段收费名称',
                  },
                ],
              })(<Input maxLength={30} disabled={isEdit} placeholder="分段收费名称" />)}
            </Col>
          </Row>
        </FormItem>

        <FormItem {...formItemLayout} required label="所属分公司">
          <Row>
            <Col lg={8} md={14} xs={20}>
              {getFieldDecorator('organizationId', {
                rules: [
                  {
                    required: true,
                    message: '请选择分公司',
                  },
                ],
              })(
                <Select
                  showSearch
                  dict={this.state.branchDict}
                  allowClear={false}
                  placeholder="请选择分公司"
                  disabled={isEdit}
                />,
              )}
            </Col>
            {!isEdit && (
              <Col lg={2} md={10} xs={4}>
                <div className={style.rowTip}>
                  <Tip title={'请先选择分公司后再选择对应分段的产品'} />
                </div>
              </Col>
            )}
          </Row>
        </FormItem>

        <FormItem {...formItemLayout} required label="连报条件">
          {getFieldDecorator('type', {
            rules: [
              {
                required: true,
              },
            ],
            initialValue: this.props.type,
          })(
            <RadioGroup disabled={isEdit}>
              {PeriodTypeDict.map((item) => {
                return (
                  <Radio value={item.val} key={item.val}>
                    {item.text}
                  </Radio>
                );
              })}
            </RadioGroup>,
          )}
        </FormItem>

        <FormItem {...formItemLayout} required label="有效期">
          {getFieldDecorator('validateTime', {
            rules: [
              {
                required: true,
                message: '有效期不能为空',
              },
            ],
          })(
            <RangePicker
              showTime
              style={{ width: 360 }}
              format="YYYY-MM-DD HH:mm:ss"
              placeholder={['开始时间', '结束时间']}
            ></RangePicker>,
          )}
        </FormItem>

        <p className={style.title}>设置分段</p>

        <Row gutter={20}>
          <Col span={3} className={style.rowHeader}>
            <span className={style.text}>分段</span>
          </Col>
          <Col span={4} className={style.rowHeader}>
            季度
          </Col>
          <Col span={6} className={style.rowHeader}>
            收费小时数
          </Col>
          <Col span={5} className={style.rowHeader}>
            分段优惠比例上限
          </Col>
          <Col span={6} className={style.rowHeader}>
            开始收费时间
          </Col>
        </Row>

        <Row gutter={20}>
          <Col span={3} className={style.rowItem}>
            <p className={style.first}>阶段一</p>
          </Col>
          {this.renderQuarterContent()}
          <Col span={6} className={style.rowContent}>
            {this.isNotSingleQuarter() && <p className={style.row}>默认全部（不可修改）</p>}
            <Row>
              <Col span={7}>
                <FormItem>
                  {getFieldDecorator('hours', {
                    rules: [
                      {
                        required: true,
                        message: '小时数不能为空',
                      },
                    ],
                  })(<InputNumber precision={0} min={0} placeholder="小时数" />)}
                </FormItem>
              </Col>
              <Col span={3} style={{ padding: '0 5px', lineHeight: '32px' }}>
                小时
              </Col>
              <Col span={7}>
                <FormItem>
                  {getFieldDecorator('minutes', {
                    rules: [
                      {
                        required: true,
                        message: '分钟数不能为空',
                      },
                    ],
                  })(<InputNumber precision={0} min={0} max={59} placeholder="分钟数" />)}
                </FormItem>
              </Col>
              <Col span={3} style={{ paddingLeft: 10, lineHeight: '32px' }}>
                分钟
              </Col>
              <Col span={4}>
                <div className={style.rowTip}>
                  <Tip title={this.getSingleTipText()} />
                </div>
              </Col>
            </Row>
          </Col>
          <Col span={5} className={style.rowContent}>
            {this.isNotSingleQuarter() && (
              <Row>
                <Col span={12}>
                  <FormItem>
                    {getFieldDecorator('upFirstQuarterPromotionScaleLimit', {
                      rules: [
                        {
                          required: true,
                          message: '阶段一优惠比例不能为空',
                        },
                      ],
                    })(<InputNumber precision={0} min={0} max={100} placeholder="优惠比例上限" />)}
                  </FormItem>
                </Col>
                <Col span={2} style={{ paddingLeft: 10, lineHeight: '32px' }}>
                  %
                </Col>
                <Col span={3}>
                  <div className={style.rowTip}>
                    <Tip title="仅影响阶段一收款时的金额计算，班级扣费时仍会享受优惠" />
                  </div>
                </Col>
              </Row>
            )}
            <Row>
              <Col span={12}>
                <FormItem>
                  {getFieldDecorator('upSecondQuarterPromotionScaleLimit', {
                    rules: [
                      {
                        required: true,
                        message: '阶段一优惠比例不能为空',
                      },
                    ],
                  })(<InputNumber precision={0} min={0} max={100} placeholder="优惠比例上限" />)}
                </FormItem>
              </Col>
              <Col span={2} style={{ paddingLeft: 10, lineHeight: '32px' }}>
                %
              </Col>
              <Col span={3}>
                <div className={style.rowTip}>
                  <Tip title="仅影响阶段一收款时的金额计算，班级扣费时仍会享受优惠" />
                </div>
              </Col>
            </Row>
          </Col>
          <Col span={6} className={style.rowContent}>
            <FormItem>
              <Row>
                <Col span={10}>
                  {getFieldDecorator('upStartTime', {
                    rules: [
                      {
                        required: true,
                        message: '阶段一开始收费时间不能为空',
                      },
                    ],
                    initialValue: moment(),
                  })(<DatePicker size="large" placeholder="开始收费时间" />)}
                </Col>
                <Col span={2}>
                  <div className={style.rowTip}>
                    <Tip title="从所选日期的0点开始允许收费" />
                  </div>
                </Col>
              </Row>
            </FormItem>
          </Col>
        </Row>

        <Row gutter={20} className={style.line}>
          <Col span={3} className={style.rowItem}>
            <p className={style.first}>阶段二</p>
          </Col>
          <Col span={4} className={style.rowContent}>
            <p className={style.row}>{this.renderText()}</p>
          </Col>
          <Col span={6} className={style.rowContent}>
            <p className={style.row}>
              {this.renderText()}
              <span>班剩余小时</span>
            </p>
          </Col>
          <Col span={5} className={style.rowContent}>
            <p className={style.row}>剩余所有优惠</p>
          </Col>
          <Col span={6} className={style.rowContent}>
            <FormItem>
              <Row>
                <Col span={10}>
                  {getFieldDecorator('downStartTime', {
                    rules: [
                      {
                        required: true,
                        message: '阶段二开始收费时间不能为空',
                      },
                    ],
                  })(<DatePicker size="large" placeholder="开始收费时间" />)}
                </Col>
                <Col span={2}>
                  <div className={style.rowTip}>
                    <Tip title="从所选日期的0点开始允许收费" />
                  </div>
                </Col>
              </Row>
            </FormItem>
          </Col>
        </Row>
      </Form>
    );
  }
}

export default Form.create({
  onFieldsChange(props, fields) {
    runFunc(props.onFieldsChange, fields);
  },
})(AddForm);
