import React from 'react';
import { List, SearchFilterForm, SearchFieldForm, Input, Select, Wrapper, Deck } from 'COMPONENT';
import { getPeriodChargeProductList } from 'SERVICE/product';
import { runFunc } from 'UTIL/common';
import SelectedCard from '../SelectedCard';
import { Modal } from 'COMPONENT/ShoppingCart';
import { ProductSegType, SegTypeDict } from '../../const';
import style from '../SelectProductModal/style.module.less';

const { COL_WIDTH_M } = List;

// 已选产品render
function selectedCardRender({ onDelete }) {
  return (props, index) => {
    return <SelectedCard {...props} onDelete={onDelete} index={index} key={index} />;
  };
}

export default class ProductListDelModal extends React.Component {
  columns = [
    {
      title: '产品名称',
      dataIndex: 'productName',
      key: 'productName',
      width: COL_WIDTH_M,
    },
    {
      title: '分段类型',
      dataIndex: 'segType',
      key: 'segType',
      width: COL_WIDTH_M,
      render: function (type) {
        return type ? ProductSegType[type] : '';
      },
    },
    {
      title: '产品系列',
      dataIndex: 'courseSeriesName',
      key: 'courseSeriesName',
      width: COL_WIDTH_M,
    },
    {
      title: '班级类型',
      dataIndex: 'classTypeName',
      key: 'classTypeName',
      width: COL_WIDTH_M,
    },
    {
      title: '产品季度',
      dataIndex: 'quarterName',
      key: 'quarterName',
      width: COL_WIDTH_M,
    },
    {
      title: '年级',
      dataIndex: 'gradeName',
      key: 'gradeName',
      width: COL_WIDTH_M,
    },
  ];

  constructor(props) {
    super(props);

    const { selectedList = [] } = props;

    const selectedRowKeys = selectedList.map((r) => r.productId);

    this.state = {
      loading: false,
      list: [],
      total: 0,
      pageNum: 1,
      pageSize: 10,
      selectedList,
      selectedRowKeys,
      productName: '',
      segType: '',
    };

    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.onSelect = this.onSelect.bind(this);
    this.onSelectAll = this.onSelectAll.bind(this);
    this.onSelectChange = this.onSelectChange.bind(this);
    this.handleDelete = this.handleDelete.bind(this);
    this.onClose = this.onClose.bind(this);
    this.onOk = this.onOk.bind(this);
  }

  componentDidMount() {
    this.loadList();
  }

  async loadList() {
    try {
      this.setState({ loading: true });
      const { pageSize, productName, pageNum, segType } = this.state;
      const { resultCode, data } = await getPeriodChargeProductList({
        productName,
        segId: this.props.id,
        segType,
        pageSize,
        pageNum,
      });
      if (resultCode === 0) {
        const { total, list } = data;
        this.setState({
          list,
          total,
        });
      }
    } finally {
      this.setState({ loading: false });
    }
  }

  changePage({ pageSize, current }) {
    this.setState({ pageSize, pageNum: current }, this.loadList);
  }

  search(params) {
    const { productName = '', segType } = params;
    this.setState(
      {
        segType,
        productName,
        pageNum: 1,
      },
      this.loadList,
    );
  }

  onSelectChange(keys) {
    this.setState({ selectedRowKeys: keys });
  }

  onSelect(record, selected) {
    this.setState((pre) => {
      const { selectedList } = pre;
      if (selected) {
        selectedList.push(record);
      } else {
        const idx = selectedList.findIndex((item) => item.id === record.productId);
        if (idx > -1) {
          selectedList.splice(idx, 1);
        }
      }
      return { selectedList };
    });
  }

  onSelectAll(selected, rows, changeRows) {
    this.setState((pre) => {
      let { selectedList } = pre;
      if (selected) {
        selectedList = selectedList.concat(changeRows);
      } else {
        changeRows.forEach((row) => {
          const idx = selectedList.findIndex((item) => item.id === row.productId);
          if (idx > -1) {
            selectedList.splice(idx, 1);
          }
        });
      }
      return { selectedList };
    });
  }

  handleDelete(index) {
    const { selectedList, selectedRowKeys } = this.state;
    const { productId: id } = selectedList[index];
    let idx = selectedRowKeys.indexOf(id);
    if (idx > -1) {
      selectedRowKeys.splice(idx, 1);
    }
    selectedList.splice(index, 1);
    this.setState({ selectedList, selectedRowKeys });
  }

  onClose() {
    runFunc(this.props.onChange, []);
  }

  onOk() {
    runFunc(this.props.onChange, this.state.selectedList);
  }

  render() {
    const { loading } = this.state;
    return (
      <Modal
        width={1200}
        title="选择产品"
        loading={loading}
        okText="保存"
        onOk={this.onOk}
        onClose={this.onClose}
        contentOptions={{
          title: <span style={{ fontSize: 18 }}>删除产品</span>,
          children: (
            <Wrapper
              header={
                <div className={style.wrapper}>
                  <SearchFilterForm onSearch={this.search} truncationNum={4}>
                    <SearchFieldForm name="productName" span={2}>
                      <Input placeholder="请输入产品名称" />
                    </SearchFieldForm>
                    <SearchFieldForm name="segType">
                      <Select placeholder="分段类型" dict={SegTypeDict} />
                    </SearchFieldForm>
                  </SearchFilterForm>
                </div>
              }
            >
              <div className={style.wrapper}>
                <List
                  rowKey="productId"
                  columns={this.columns}
                  dataSource={this.state.list}
                  onChange={this.changePage}
                  pagination={{
                    total: this.state.total,
                    current: this.state.pageNum,
                    pageSize: this.state.pageSize,
                  }}
                  rowSelection={{
                    selectedRowKeys: this.state.selectedRowKeys,
                    onSelect: this.onSelect,
                    onChange: this.onSelectChange,
                    onSelectAll: this.onSelectAll,
                  }}
                  className={style.ToBeSelectCard}
                />
              </div>
            </Wrapper>
          ),
        }}
        sideOptions={{
          title: '已选产品',
          children: (
            <Deck
              dataSource={this.state.selectedList}
              render={selectedCardRender({
                onDelete: this.handleDelete,
              })}
              className={style.SelectedCard}
            />
          ),
        }}
      />
    );
  }
}
