import React from 'react';
import { connect } from 'dva';
import { noop } from 'lodash';
import { Wrapper, Select, Input, SearchFilterForm, SearchFieldForm, List, Deck } from 'COMPONENT';
import { Modal } from 'COMPONENT/ShoppingCart';
import SelectedCard from '../SelectedCard';
import { searchPeriodChargeProductList } from 'SERVICE/product';
import { Columns } from './const';
import { runFunc } from 'UTIL/common';
import { loadCommonDict } from 'UTIL/dict';
import style from './style.module.less';

// 已选产品render
function selectedCardRender({ onDelete }) {
  return (props, index) => {
    return <SelectedCard {...props} onDelete={onDelete} index={index} key={index} />;
  };
}

class ProductListModal extends React.Component {
  constructor(props) {
    super(props);
    const { organizationId, selectedList = [], disabledIds = [] } = props;

    const selectedRowKeys = selectedList.map((r) => r.id);

    this.state = {
      loading: false,
      list: [],
      total: 0,
      pageNum: 1,
      pageSize: 10,
      selectedList,
      disabledIds,
      selectedRowKeys,
      classTypeDict: [],
      courseSeriesDict: [],
      params: {
        organizationId,
        name: '',
        // 默认查小班
        category: 'SMALL_CLASS',
        classTypeId: '',
        gradeId: '',
        productQuarterId: '',
        productVersionId: '',
        courseSeriesId: '',
      },
    };

    this.onClose = this.onClose.bind(this);
    this.onSelect = this.onSelect.bind(this);
    this.onOk = this.onOk.bind(this);
    this.onSelectAll = this.onSelectAll.bind(this);
    this.onSelectChange = this.onSelectChange.bind(this);
    this.handleDelete = this.handleDelete.bind(this);
    this.disabledItem = this.disabledItem.bind(this);
    this.search = this.search.bind(this);
    this.changePage = this.changePage.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initSearchParams = this.initSearchParams.bind(this);
    this.loadList = this.loadList.bind(this);
    this.initClassTypeDict = this.initClassTypeDict.bind(this);
  }

  async componentDidMount() {
    this.setState({ loading: true });
    await new Promise(this.initClassTypeDict);
    await new Promise(this.initDict);
    await new Promise(this.initSearchParams);
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  async initClassTypeDict(resolve) {
    try {
      // 加载小班的班级类型
      const classTypeDict = await loadCommonDict('CLASS_TYPE', 'SMALL_CLASS');
      // 加载小班产品系列
      const courseSeriesDict = await loadCommonDict('COURSE_SERIES', 'SMALL_CLASS');
      this.setState({
        classTypeDict,
        courseSeriesDict,
      });
    } finally {
      resolve();
    }
  }

  async initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['grade', 'year', 'quarter', 'productCharacter'],
        callback: () => {
          resolve();
        },
      },
    });
  }

  async initSearchParams(resolve) {
    const { quarterKey, quarterDict, yearDict } = this.props;
    // 初始化当前年份
    const currYear = new Date().getFullYear();
    const productVersionId = (yearDict.find((item) => item.text === String(currYear)) || {}).val;
    // 查询相应季度
    const productQuarterId = (quarterDict.find((item) => item.text.startsWith(quarterKey)) || {}).val;
    const payload = { productVersionId, productQuarterId };
    this.setState(
      {
        params: {
          ...this.state.params,
          ...payload,
        },
      },
      resolve,
    );
  }

  async loadList(resolve = noop) {
    try {
      this.setState({ loading: true });
      const { params, pageNum, pageSize } = this.state;
      const { resultCode, data } = await searchPeriodChargeProductList({
        ...params,
        pageNum,
        pageSize,
      });
      if (resultCode === 0) {
        const { list, total } = data;
        this.setState({ list, total });
      }
    } finally {
      this.setState({ loading: false });
      resolve();
    }
  }

  changePage({ pageSize, current }) {
    this.setState({ pageSize, pageNum: current }, this.loadList);
  }

  search(searchParams) {
    const { params } = this.state;
    this.setState(
      {
        params: {
          ...params,
          ...searchParams,
          pageNum: 1,
        },
      },
      this.loadList,
    );
  }

  onSelectChange(keys) {
    this.setState({ selectedRowKeys: keys });
  }

  onSelect(record, selected) {
    this.setState((pre) => {
      const { selectedList } = pre;
      if (selected) {
        selectedList.push(record);
      } else {
        const idx = selectedList.findIndex((item) => item.id === record.id);
        if (idx > -1) {
          selectedList.splice(idx, 1);
        }
      }
      return { selectedList };
    });
  }

  onSelectAll(selected, rows, changeRows) {
    this.setState((pre) => {
      let { selectedList } = pre;
      if (selected) {
        selectedList = selectedList.concat(changeRows);
      } else {
        changeRows.forEach((row) => {
          const idx = selectedList.findIndex((item) => item.id === row.id);
          if (idx > -1) {
            selectedList.splice(idx, 1);
          }
        });
      }
      return { selectedList };
    });
  }

  disabledItem(record) {
    const { id } = record;
    return { disabled: this.state.disabledIds.includes(id) };
  }

  handleDelete(index) {
    const { selectedList, selectedRowKeys } = this.state;
    const { id } = selectedList[index];
    let idx = selectedRowKeys.indexOf(id);
    if (idx > -1) {
      selectedRowKeys.splice(idx, 1);
    }
    selectedList.splice(index, 1);
    this.setState({ selectedList, selectedRowKeys });
  }

  onClose() {
    runFunc(this.props.onChange, []);
  }

  onOk() {
    runFunc(this.props.onChange, this.state.selectedList);
  }

  render() {
    const { loading } = this.state;

    return (
      <Modal
        width={1200}
        title="选择产品"
        loading={loading}
        okText="保存"
        onOk={this.onOk}
        onClose={this.onClose}
        contentOptions={{
          title: <span style={{ fontSize: 18 }}>选择产品</span>,
          children: (
            <Wrapper
              header={
                <div className={style.wrapper}>
                  <SearchFilterForm onSearch={this.search} truncationNum={5}>
                    <SearchFieldForm name="name">
                      <Input placeholder="产品名称" />
                    </SearchFieldForm>
                    <SearchFieldForm name="classTypeId">
                      <Select placeholder="班级类型" dict={this.state.classTypeDict} allowClear={false} />
                    </SearchFieldForm>
                    <SearchFieldForm name="gradeId">
                      <Select placeholder="年级" dict={this.props.gradeDict} />
                    </SearchFieldForm>
                    <SearchFieldForm name="productQuarterId" initialValue={this.state.params.productQuarterId}>
                      <Select placeholder="季度" dict={this.props.quarterDict} />
                    </SearchFieldForm>

                    <SearchFieldForm name="productVersionId" initialValue={this.state.params.productVersionId}>
                      <Select placeholder="年份" dict={this.props.yearDict} />
                    </SearchFieldForm>

                    <SearchFieldForm name="courseSeriesId">
                      <Select placeholder="产品系列" dict={this.state.courseSeriesDict} />
                    </SearchFieldForm>
                  </SearchFilterForm>
                </div>
              }
            >
              <div className={style.wrapper}>
                <List
                  rowKey="id"
                  columns={Columns}
                  dataSource={this.state.list}
                  onChange={this.changePage}
                  pagination={{
                    total: this.state.total,
                    current: this.state.pageNum,
                    pageSize: this.state.pageSize,
                  }}
                  rowSelection={{
                    selectedRowKeys: this.state.selectedRowKeys,
                    onSelect: this.onSelect,
                    onChange: this.onSelectChange,
                    onSelectAll: this.onSelectAll,
                    getCheckboxProps: this.disabledItem,
                  }}
                  className={style.ToBeSelectCard}
                />
              </div>
            </Wrapper>
          ),
        }}
        sideOptions={{
          title: '已选产品',
          children: (
            <Deck
              dataSource={this.state.selectedList}
              render={selectedCardRender({
                onDelete: this.handleDelete,
              })}
              className={style.SelectedCard}
            />
          ),
        }}
      />
    );
  }
}

const mapStateToProps = (state) => {
  return {
    gradeDict: state.dict.grade,
    yearDict: state.dict.year,
    quarterDict: state.dict.quarter,
    productCharacterDict: state.dict.productCharacter,
  };
};

export default connect(mapStateToProps)(ProductListModal);
