import React from 'react';
import { connect } from 'dva';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import { productStatusDict, GROUP_ID } from '../../../../config';
import { STORE_NAME } from '../../../../store/config';
import { getRelationsDictPlus, getRelationPayloadPlus } from 'MODEL/dict/utils/index';
import { productType, classType } from 'MODEL/dict/utils/config';

class SearchBlock extends React.Component {
  static defaultProps = {
    showSearchStatus: false,
  };

  constructor(props) {
    super(props);

    // 产品类型关联关系type、key对应关系配置
    this.productTypeTreeMap = {
      [productType]: 'val_productType',
      [classType]: 'val_classType',
    };

    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);
    this.onUpdateStore = this.onUpdateStore.bind(this);
    this.onUpdateClassType = this.onUpdateClassType.bind(this);
  }

  /**
   * 搜索列表
   * 传入paneKey用于判断搜索哪个tab
   */
  onSearch() {
    const { paneKey, dispatch } = this.props;

    dispatch({
      type: `${STORE_NAME.tab}/onTabSearch`,
      payload: {
        paneKey,
      },
    });
  }

  onReset() {
    const { paneKey, dispatch } = this.props;

    dispatch({
      type: `${STORE_NAME.tab}/resetTabSearch`,
      payload: {
        paneKey,
      },
    });
  }

  /**
   * 更新store，组装成 {tab_one_search: {val_yearId: 'xxx'}}
   * @param key 键
   * @param value 值
   */
  onUpdateStore(key, value) {
    const { paneKey, dispatch } = this.props;
    const storeKey = (paneKey === '1' && 'tab_one') || 'tab_two';
    const oldSearch = this.props[storeKey];

    let payload = {};
    payload[storeKey] = {
      ...oldSearch,
      [key]: value,
    };

    dispatch({
      type: `${STORE_NAME.tab}/updateState`,
      payload: { ...payload },
    });
  }

  /**
   * 单独更新班级类型
   * @param value
   */
  onUpdateClassType(value) {
    const { paneKey, dispatch, productTypeDict } = this.props;
    const storeKey = (paneKey === '1' && 'tab_one') || 'tab_two';
    const oldSearch = this.props[storeKey];

    const result = getRelationPayloadPlus('val_classType', value, {
      type: classType,
      dictTree: productTypeDict,
      state: oldSearch,
      map: this.productTypeTreeMap,
    });

    let payload = {};
    payload[storeKey] = {
      ...oldSearch,
      ...result,
    };

    dispatch({
      type: `${STORE_NAME.tab}/updateState`,
      payload: { ...payload },
    });
  }

  render() {
    const { paneKey, showSearchStatus } = this.props;
    const {
      branchDict,
      yearDict,
      quarterDict,
      productTypeDict,
      productCharacterDict,
      subjectDict,
      courseSeriesDict,
      gradeDict,
      branchStore,
      tab_one,
      tab_two,
    } = this.props;
    const tabSearch = paneKey === '1' ? tab_one : tab_two;
    const {
      val_yearId,
      val_quarterId,
      val_productType,
      val_productCharacter,
      val_subjectId,
      val_courseSeries,
      val_classType,
      val_gradeId,
      val_productName,
      val_isValid,
      val_productBranchIds,
    } = tabSearch;
    // 筛选班级类型字典
    const classTypeDict = getRelationsDictPlus(classType)(productTypeDict, val_productType);
    const { val_branchId } = branchStore;
    const showProductBranch = val_branchId === GROUP_ID;

    return (
      <SearchBar onSearch={this.onSearch} onReset={this.onReset}>
        {showProductBranch && (
          <MultiSelect
            placeholder="分公司"
            dict={branchDict}
            value={val_productBranchIds}
            onChange={(val) => {
              this.onUpdateStore('val_productBranchIds', val);
            }}
          />
        )}
        <Select
          placeholder="年份"
          dict={yearDict}
          value={val_yearId}
          onChange={(val) => {
            this.onUpdateStore('val_yearId', val);
          }}
          allowClear={true}
        />

        <Select
          placeholder="季度"
          dict={quarterDict}
          value={val_quarterId}
          onChange={(val) => {
            this.onUpdateStore('val_quarterId', val);
          }}
          allowClear={true}
        />

        <Select
          placeholder="产品类型"
          dict={productTypeDict}
          value={val_productType}
          onChange={(val) => {
            this.onUpdateStore('val_productType', val);
          }}
          allowClear={true}
        />

        <Select
          placeholder="产品性质"
          dict={productCharacterDict}
          value={val_productCharacter}
          onChange={(val) => {
            this.onUpdateStore('val_productCharacter', val);
          }}
          allowClear={true}
        />

        <Select
          placeholder="科目"
          dict={subjectDict}
          value={val_subjectId}
          onChange={(val) => {
            this.onUpdateStore('val_subjectId', val);
          }}
          allowClear={true}
        />

        <Select
          placeholder="产品系列"
          dict={courseSeriesDict}
          value={val_courseSeries}
          onChange={(val) => {
            this.onUpdateStore('val_courseSeries', val);
          }}
          allowClear={true}
        />

        <Select
          placeholder="班级类型"
          dict={classTypeDict}
          value={val_classType}
          onChange={this.onUpdateClassType}
          allowClear={true}
        />

        <Select
          placeholder="年级"
          dict={gradeDict}
          value={val_gradeId}
          onChange={(val) => {
            this.onUpdateStore('val_gradeId', val);
          }}
          allowClear={true}
        />

        <Input
          placeholder="产品名称"
          value={val_productName}
          onChange={(e) => {
            this.onUpdateStore('val_productName', e.target.value);
          }}
        />

        {showSearchStatus && (
          <Select
            placeholder="有效期"
            dict={productStatusDict}
            value={val_isValid}
            onChange={(val) => {
              this.onUpdateStore('val_isValid', val);
            }}
            allowClear={false}
          />
        )}
      </SearchBar>
    );
  }
}

const mapStateToProps = (state) => {
  const { branch, year, quarter, productCharacter, subject, courseSeries, grade, productTypeTree } = state.dict;
  const branchStore = state[STORE_NAME.branch];
  const tabStore = state[STORE_NAME.tab];
  const { tab_one, tab_two } = tabStore;

  return {
    branchDict: branch,
    yearDict: year,
    quarterDict: quarter,
    productTypeDict: productTypeTree || null,
    productCharacterDict: productCharacter,
    subjectDict: subject,
    courseSeriesDict: courseSeries,
    gradeDict: grade,
    branchStore,
    tab_one,
    tab_two,
  };
};

export default connect(mapStateToProps)(SearchBlock);
