import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import { Row, Col, Checkbox, Pagination, Tooltip } from 'antd';
import Loading from 'COMPONENT/Loading';
import NoData from 'COMPONENT/NoData';
import { STORE_NAME } from '../../../../store/config';

class ListBlock extends React.Component {
  constructor(props) {
    super(props);

    this.pageSizeOptions = ['10', '20', '50'];

    this.loadList = this.loadList.bind(this);
    this.renderCheckboxList = this.renderCheckboxList.bind(this);
    this.onCheckedChange = this.onCheckedChange.bind(this);
    this.onPageChange = this.onPageChange.bind(this);
    this.onPageSizeChange = this.onPageSizeChange.bind(this);
  }

  /**
   * 请求列表
   * @Attention 把pageNo改为page，pageSize改为rows
   * @param changeParam
   * @return {boolean}
   */
  loadList(changeParam = {}) {
    this.props.dispatch({
      type: `${STORE_NAME.promotion}/loadList`,
      payload: { ...changeParam },
    });
  }

  /**
   * 优惠券列表
   * props.data 优惠券列表
   * props.checkedPromotion 选中的优惠券 从store获取
   * @param props
   */
  renderCheckboxList(props) {
    const { data, checkedPromotion } = props;
    const checkedValues =
      (checkedPromotion &&
        checkedPromotion.map((item) => {
          return item.id;
        })) ||
      [];

    return (
      <Checkbox.Group onChange={this.onCheckedChange} value={checkedValues}>
        <Row style={{ minHeight: 60 }}>
          {data &&
            data.length &&
            data.map((item, index) => {
              const { id, name, promotionCampusName } = item;

              let campusName = '';
              if (promotionCampusName) {
                campusName = promotionCampusName.split(',').join('、');
              }

              const wholeName = promotionCampusName ? `${name} (${promotionCampusName})` : name;

              return (
                <Col span={6} key={index}>
                  <Checkbox value={id}>
                    <Tooltip placement="topRight" title={wholeName}>
                      {campusName ? (
                        <span>
                          {name}
                          <i>({campusName})</i>
                        </span>
                      ) : (
                        name
                      )}
                    </Tooltip>
                  </Checkbox>
                </Col>
              );
            })}
        </Row>
      </Checkbox.Group>
    );
  }

  /**
   * 选择优惠，效果与PanelHear删除优惠类似
   * @Attention 将修改状态放在 store/promotion 确认弹窗里面，先判断是否存在产品列表或选中产品，有则需要清空
   * @Attention 因confirm弹窗回调不允许触发action，因此只能放在dispatch的回调方法实现
   * @param checkedValues [] 当前还选中的ID集合
   */
  onCheckedChange(checkedValues) {
    const { dispatch, list, checkedPromotion } = this.props;
    let checkedResult = [];

    // 先判断store中的checkedPromotion是否为空
    if (!checkedPromotion || checkedPromotion.length <= 0) {
      if (checkedValues && checkedValues.length > 0) {
        checkedValues.forEach((id) => {
          list.forEach((item) => {
            if (item.id === id) {
              checkedResult.push(item);
              return false;
            }
          });
        });
      }
    } else {
      checkedValues.forEach((id) => {
        // 优先判断该id是否在选中的列表里面, 否则在当前分页list里面查找
        const inPromotion = checkedPromotion.some((item) => item.id === id);
        let packList = [];

        if (inPromotion) {
          packList = [...checkedPromotion];
        } else {
          packList = [...list];
        }

        packList.forEach((item) => {
          if (item.id === id) {
            checkedResult.push(item);
            return false;
          }
        });
      });
    }

    dispatch({
      type: `${STORE_NAME.promotion}/onCheckedChange`,
      payload: {},
      doUpdateState: () => {
        dispatch({
          type: `${STORE_NAME.promotion}/updateState`,
          payload: { allPageChecked: checkedResult },
        });
      },
      doResetTabData: () => {
        dispatch({ type: `${STORE_NAME.tab}/resetTabListAndChecked` });
      },
    });
  }

  /**
   * 翻页
   * @param pageNo
   * @param pageSize
   */
  onPageChange(pageNo, pageSize) {
    const changeParam = { pageNo, pageSize };
    this.loadList(changeParam);
  }

  /**
   * 每页数量切换
   * @param current
   * @param size
   */
  onPageSizeChange(current, size) {
    const changeParam = { pageNo: 1, pageSize: size };
    this.loadList(changeParam);
  }

  /**
   * 显示总数
   * @param total
   * @return {string}
   */
  showTotal(total) {
    return `共 ${total} 条记录`;
  }

  render() {
    const { listLoading, list, total, pageNo, pageSize, checkedPromotion } = this.props;
    const CheckboxList = this.renderCheckboxList;
    const noData = !list || list.length <= 0;

    return (
      <div className={style.container}>
        <Loading loading={listLoading} />

        {!noData && <CheckboxList data={list} checkedPromotion={checkedPromotion} />}

        {!noData && (
          <Pagination
            defaultCurrent={1}
            current={pageNo}
            pageSize={pageSize}
            pageSizeOptions={this.pageSizeOptions}
            total={total}
            showSizeChanger={true}
            showTotal={this.showTotal}
            onChange={this.onPageChange}
            onShowSizeChange={this.onPageSizeChange}
          />
        )}

        {noData && <NoData />}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const { listLoading, list, total, pageNo, pageSize, allPageChecked } = state[STORE_NAME.promotion];

  return {
    listLoading,
    list,
    total,
    pageNo,
    pageSize,
    checkedPromotion: allPageChecked,
  };
};

export default connect(mapStateToProps)(ListBlock);
