import { clone } from 'lodash';
import { defaultState, defaultTabSearch, defaultTabList, defaultTabChecked } from './state';
import { STORE_NAME } from '../config';
import { loadProductList } from './loader';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getRelevanceType, GROUP_ID } from '../../config';

export default {
  namespace: STORE_NAME.tab,
  state: {
    ...clone(defaultState),
  },
  reducers: {
    /**
     * @Attention 因为tab有多个，这里需要深度拷贝(深度为2)
     * @param state
     * @param payload
     */
    updateState(state, { payload }) {
      let newState = { ...state };

      if (payload) {
        for (let i in payload) {
          let item = payload[i];
          if (item && Object.prototype.toString.call(item) === '[object Object]') {
            newState[i] = { ...state[i], ...item };
          } else {
            newState[i] = item;
          }
        }
      }

      return {
        ...newState,
      };
    },
  },
  effects: {
    *resetPage(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState),
        },
      });
    },

    /**
     * 重置所有tab的选中和列表
     * @param action
     * @param put
     * @return {IterableIterator<*>}
     */
    *resetTabListAndChecked(action, { put }) {
      const initData = { ...defaultTabList, ...defaultTabChecked };

      yield put({
        type: 'updateState',
        payload: {
          tab_one: { ...initData },
          tab_two: { ...initData },
        },
      });
    },

    /**
     * 重置tab的搜索结果，组装成 {tab_one: {...}}
     * @param payload JSON {paneKey: '1'}
     * @param put
     * @param select
     * @return {IterableIterator<*>}
     */
    *resetTabSearch({ payload }, { put, select }) {
      const { paneKey } = payload;
      const storeKey = paneKey === '1' ? 'tab_one' : 'tab_two';
      const tabStore = yield select((state) => {
        return state[STORE_NAME.tab][storeKey];
      });

      let search = {};
      search[storeKey] = { ...tabStore, ...defaultTabSearch };

      yield put({
        type: 'updateState',
        payload: search,
      });
    },

    /**
     * 清除当前tab所有选中产品列表
     * @param payload JSON {paneKey: '1'}
     * @param put
     * @param select
     * @return {IterableIterator<*>}
     */
    *resetTabChecked({ payload }, { put, select }) {
      const { paneKey } = payload;
      const storeKey = paneKey === '1' ? 'tab_one' : 'tab_two';

      let store = {};
      store[storeKey] = { ...defaultTabChecked };

      yield put({
        type: 'updateState',
        payload: store,
      });
    },

    /**
     * 清除所有tab的选中产品列表
     * @param action
     * @param put
     * @return {IterableIterator<*>}
     */
    *resetAllChecked(action, { put }) {
      yield put({
        type: 'resetTabChecked',
        payload: { paneKey: '1' },
      });

      yield put({
        type: 'resetTabChecked',
        payload: { paneKey: '2' },
      });
    },

    /**
     * 刷新产品模块
     * @param action
     * @param put
     * @return {IterableIterator<*>}
     */
    *onRefresh(action, { put }) {
      yield put({ type: 'resetAllChecked' });
      yield put({ type: 'onSearch' });
    },

    /**
     * 搜索两个tab的列表，分公司切换时使用
     * @param action
     * @param put
     * @return {IterableIterator<*>}
     */
    *onSearch(action, { put }) {
      yield put({
        type: 'onTabSearch',
        payload: { paneKey: '1' },
      });

      yield put({
        type: 'onTabSearch',
        payload: { paneKey: '2' },
      });
    },

    /**
     * 单个tab搜索
     * @param JSON payload {paneKey: '选项卡对应key'}
     * @param put
     * @return {IterableIterator<*>}
     */
    *onTabSearch({ payload }, { put }) {
      const { paneKey } = payload;
      const changeParam = { paneKey, pageNo: 1 };

      yield put({
        type: 'loadList',
        payload: { ...changeParam },
      });
    },

    /**
     * 获取产品列表, 不同tab对应参数不同，根据传入的paneKey判断
     * @param payload
     * @param put
     * @param select
     * @return {IterableIterator<*>}
     */
    *loadList({ payload }, { put, select }) {
      const { paneKey } = payload;
      const tabKey = paneKey === '1' ? 'tab_one' : 'tab_two';
      const tabStore = yield select((state) => {
        return state[STORE_NAME.tab][tabKey];
      });

      if (tabStore.listLoading) {
        return false;
      }

      // 组装搜索条件
      const searchParam = yield select((state) => {
        const { val_branchId } = state[STORE_NAME.branch];
        const { allPageChecked } = state[STORE_NAME.promotion];
        const checkedPromotion = allPageChecked || [];

        let promotionIds = [];
        if (checkedPromotion && checkedPromotion.length > 0) {
          checkedPromotion.forEach((item) => {
            promotionIds.push(item.promotionId);
          });
        }

        const {
          val_productBranchIds,
          val_yearId,
          val_quarterId,
          val_productType,
          val_productCharacter,
          val_subjectId,
          val_courseSeries,
          val_classType,
          val_gradeId,
          val_productName,
          val_isValid,
          pageNo,
          pageSize,
        } = tabStore || {};

        const productBranchIds = val_branchId === GROUP_ID ? val_productBranchIds : [];

        return {
          branchId: val_branchId,
          productBranchIds: productBranchIds,
          promotions: promotionIds,
          productVersionId: val_yearId,
          productQuarterId: val_quarterId,
          productType: val_productType,
          productCharacterId: val_productCharacter,
          subjectId: val_subjectId,
          courseSeriesId: val_courseSeries,
          classTypeId: val_classType,
          gradeId: val_gradeId,
          productName: val_productName,
          isValid: Number(val_isValid) || 0,
          relevanceType: getRelevanceType(paneKey),
          pageNo,
          pageSize,
        };
      });
      const param = { ...searchParam, ...payload };

      if (!param.branchId) {
        showMessage('请选择分公司!');
        return false;
      }

      // 转换为api参数，把pageNo改为page，pageSize改为rows
      let apiParam = { ...param };
      apiParam.page = param.pageNo;
      apiParam.rows = param.pageSize;
      delete apiParam.pageNo;
      delete apiParam.pageSize;

      if (apiParam.paneKey) {
        delete apiParam.paneKey;
      }

      // 组装store
      let newStore = {};
      newStore[tabKey] = {
        listLoading: true,
      };

      yield put({
        type: 'updateState',
        payload: newStore,
      });

      const result = yield loadProductList(apiParam, true);

      newStore[tabKey] = {
        ...result,
        pageNo: param.pageNo,
        pageSize: param.pageSize,
        listLoading: false,
      };

      yield put({
        type: 'updateState',
        payload: newStore,
      });
    },
  },
};
