/**
 * 产品详情查看模态框
 * @property {string} productId 产品ID
 * @property {function} onClose 关闭弹框事件触发时的回调
 */
import React from 'react';
import { Row, Col, Button } from 'antd';
import Modal from 'COMPONENT/Modal';
import { getDictTextByVal } from 'UTIL/dict';
import { getProductById } from 'SERVICE/product';
import { loadCommonDict, loadAllBranchDict } from 'UTIL/dict';
import Block from '../../Block';
import FormItem from 'COMPONENT/FormItem';
import { statusDict } from 'CONST/dicts/product';
import styles from './styles.less';

class ProductDetailView extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      productDetail: {}, // {object} 产品详情
      subjectDict: null, // {[object]} 字典 科目
      branchsDict: null, // {[object]} 字典 分公司
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    const { productId } = this.props;
    this.setState({ loading: true });
    Promise.all([this.loadProductDetail(productId), loadCommonDict('SUBJECT'), loadAllBranchDict()]).then(
      ([productDetail, subjectDict, branchsDict]) => {
        const { branchIds, subjectIds, ...detail } = productDetail;
        const newDetail = {
          branchs: branchIds.map((id) => getDictTextByVal(branchsDict, id)).join('，'),
          subjects: subjectIds.map((id) => getDictTextByVal(subjectDict, id)).join('，'),
          ...detail,
        };
        this.setState({
          loading: false,
          productDetail: newDetail,
          subjectDict,
          branchsDict,
        });
      },
    );
  };

  /**
   * 加载产品详情，并转换格式
   * @param  {string} productId 产品ID
   */
  loadProductDetail = async (productId) => {
    const apiData = await getProductById({ productId });
    const {
      name,
      proStatus,
      productVersionName,
      gradeName,
      courseSeriesName,
      gradeSegmentName,
      productGroupName,
      classTimeLength,
      price,
      validStartDate,
      validEndDate,
      branchId,
      subjectIdSet,
      sonProduct,
      chargeRuleName,
      experienceLessonName,
      remark,
    } = apiData;

    const data = {
      name,
      status: getDictTextByVal(statusDict, proStatus),
      year: productVersionName,
      grade: gradeName,
      series: courseSeriesName,
      gradeLevel: gradeSegmentName,
      group: productGroupName,
      courseTimeLength: classTimeLength,
      price,
      dateRange: validStartDate + '至' + validEndDate,
      branchIds: branchId,
      subjectIds: subjectIdSet || [],
      otherProducts: sonProduct.map((product) => product.name).join('，'),
      rule: chargeRuleName,
      feelCourse: experienceLessonName,
      remark,
    };
    return data;
  };

  render() {
    const { onClose } = this.props;
    const {
      loading,
      productDetail: {
        name,
        status,
        year,
        grade,
        series,
        gradeLevel,
        group,
        courseTimeLength,
        price,
        dateRange,
        branchs,
        subjects,
        otherProducts,
        rule,
        feelCourse,
        remark,
      },
    } = this.state;

    return (
      <Modal loading={loading} onCancel={onClose} title="查看信息" width={1012} okBtnShow={false} cancelText="关闭">
        <Block title={name} style={{ marginTop: -10 }} className={styles.wrap}>
          <Row gutter={10} style={{ paddingLeft: 20 }}>
            <Col span={8}>
              <FormItem name="产品年份">{year}</FormItem>
            </Col>
            <Col span={8}>
              <FormItem name="年级">{grade}</FormItem>
            </Col>
            <Col span={8}>
              <FormItem name="年级段">{gradeLevel}</FormItem>
            </Col>
            <Col span={8}>
              <FormItem name="产品系列">{series}</FormItem>
            </Col>
            <Col span={8}>
              <FormItem name="产品组">{group}</FormItem>
            </Col>
            <Col span={8}>
              <FormItem name="状态">{status}</FormItem>
            </Col>
            <Col span={8}>
              <FormItem name="课时时长">{courseTimeLength}</FormItem>
            </Col>
            <Col span={8}>
              <FormItem name="单价">{price}</FormItem>
            </Col>
            <Col span={8}>
              <FormItem name="产品有效期">{dateRange}</FormItem>
            </Col>
            <Col span={8}>
              <FormItem name="扣费规则">{rule}</FormItem>
            </Col>
            <Col span={8}>
              <FormItem name="关联产品">{otherProducts}</FormItem>
            </Col>
            <Col span={8}>
              <FormItem name="体验课">{feelCourse}</FormItem>
            </Col>
            <Col span={24}>
              <FormItem name="关联科目">{subjects}</FormItem>
            </Col>
            <Col span={24}>
              <FormItem name="所属分公司">{branchs}</FormItem>
            </Col>
            <Col span={24}>
              <FormItem name="备注">{remark}</FormItem>
            </Col>
          </Row>
        </Block>
      </Modal>
    );
  }
}

export default ProductDetailView;
