/**
 * 选择教材模态框
 * @property {function([object])} onSelect 选中教材时触发时的回调，参数为教材ID和名称的集合，格式如[{ val, text }]
 * @property {function} onClose 关闭弹框事件触发时的回调
 */
import React from 'react';
import { Row, Col, Pagination } from 'antd';
import Button from 'COMPONENT/Button';
import Input from 'COMPONENT/Input';
import Checkbox from 'COMPONENT/Checkbox';
import Modal from 'COMPONENT/Modal';
import TagSelect from 'COMPONENT/TagSelect';
import { loadCommonDict, loadAllBranchDict } from 'UTIL/dict';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import { getTextBookInfoList } from 'SERVICE/product';
import CollapsableSearchBox from './CollapsableSearchBox';
import styles from './styles.less';
import getWrappedInstance from 'UTIL/getWrappedInstance';

/**
 * 列表信息项目组件
 * @param  {string} title  标题
 * @param  {string} content 内容
 * @return {ReactNode}
 */
const ListInfo = ({ title, content }) => {
  return (
    <div className={styles.listInfo}>
      <div className={styles.listInfoTitle}>{title}</div>
      <div className={styles.listInfoContent}>{content}</div>
    </div>
  );
};

class SelectBook extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态

      name: null, // {string} 查询条件 名称
      branch: null, // {string} 查询条件 使用范围
      year: null, // {string} 查询条件 年份
      quarter: null, // {string} 查询条件 季度
      serise: null, // {string} 查询条件 产品系列
      grade: null, // {string} 查询条件 年级
      subject: null, // {string} 查询条件 科目
      bookType: null, // {string} 查询条件 教材类型

      branchDict: null, // {[object]} 字典 使用范围
      yearDict: null, // {[object]} 字典 年份
      quarterDict: null, // {[object]} 字典 季度
      seriseDict: null, // {[object]} 字典 产品系列
      gradeDict: null, // {[object]} 字典 年级
      subjectDict: null, // {[object]} 字典 科目
      bookTypeDict: null, // {[object]} 字典 教材类型

      list: [], // {[object]} 列表数据
      currentSearchKeys: {}, // {object} 当前的查询条件
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 5, // {number} 每页记录数
      selectedBooks: [], // {[object]} 选中的教材ID和名称的集合，格式如[{ val, text }]
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    this.setState({ loading: true });
    Promise.all([
      loadAllBranchDict(),
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('COURSE_SERIES'),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
    ]).then(([branchDict, yearDict, quarterDict, seriseDict, gradeDict, subjectDict]) => {
      this.setState({
        loading: false,
        branchDict,
        yearDict,
        quarterDict,
        seriseDict,
        gradeDict,
        subjectDict,
      });
    });
    this.search();
  };

  /**
   * 加载列表数据，并转换格式
   * @param  {object} changedParams 要更新的列表数据参数，包括查询条件、页码状态
   */
  loadList = async (changedParams = {}) => {
    const { currentPage, pageSize } = this.state;
    const params = { currentPage, pageSize, ...changedParams };
    const apiParams = {
      page: params.currentPage,
      rows: params.pageSize,
      name: params.name,
      organizationId: params.branch,
      productVersionId: params.year,
      productQuarterId: params.quarter,
      courseSeriesId: params.serise,
      gradeId: params.grade,
      subjectId: params.subject,
    };
    console.log(apiParams, JSON.stringify(apiParams));
    this.setState({ loading: true });
    const apiData = await getTextBookInfoList(apiParams);
    const { records, rows } = apiData;
    const formatedRows = (rows || []).map((row) => {
      return {
        id: row.id,
        name: row.name,
        createrName: row.createByName,
        branchName: row.organizationNames,
        statusName: row.state == '1' ? '已发布' : '已下架',
        classTypeName: row.categoryName,
        yearName: row.productVersionName,
        quarterName: row.productQuarterName,
        gradeName: row.gradeName,
        subjectName: row.subjectName,
        bookTypeName: row.wareTypeName,
      };
    });
    const data = {
      records,
      rows: formatedRows,
    };
    console.log(data);
    const { currentPage: newCurrentPage, pageSize: newPageSize, ...currentSearchKeys } = params;
    this.setState({
      loading: false,
      list: data.rows,
      totalRecords: data.records,
      currentPage: newCurrentPage,
      pageSize: newPageSize,
      currentSearchKeys,
    });
    return true;
  };

  /**
   * 根据名称查询
   */
  searchByName = () => {
    const { name } = this.state;
    this.loadList({ name, currentPage: 1 });
  };

  /**
   * 根据选择条件查询
   * @param  {object} searchKeys 查询条件键值对
   */
  search = (searchKeys = {}) => {
    const { branch, year, quarter, serise, grade, subject, bookType } = this.state;
    const newSearchKeys = { branch, year, quarter, serise, grade, subject, bookType, ...searchKeys };
    this.setState({ ...newSearchKeys });
    this.loadList({ ...newSearchKeys, currentPage: 1 });
  };

  /**
   * 页码信息改变时根据当前列表的查询条件重新加载数据
   * @param  {number} currentPage 当前页码
   * @param  {number} pageSize    每页条数
   */
  pageChange = (currentPage, pageSize) => {
    const { currentSearchKeys } = this.state;
    this.loadList({ currentPage, pageSize, ...currentSearchKeys });
  };

  /**
   * 选中的教材值变化时的处理
   * @param  {[string]} val 值
   */
  selectedBooksChange = (val) => {
    const { selectedBooks } = this.state;
    const newClassBook = selectedBooks.filter((c) => val && val.find((v) => v === c.val));
    this.setState({ selectedBooks: newClassBook });
  };

  /**
   * 某一教材选中状态改变
   * @param  {boolean} checked 是否选中
   * @param  {string} id     教材ID
   * @param  {string} name   教材名称
   */
  checkBook = (checked, id, name) => {
    const { selectedBooks } = this.state;
    if (selectedBooks && selectedBooks.length === 10 && checked) {
      showAlert('当前选择不能超过10条');
      return;
    }
    let newBooks;
    if (checked) {
      newBooks = [...selectedBooks, { val: id, text: name }];
    } else {
      newBooks = selectedBooks.filter((s) => s.val !== id);
    }
    this.setState({ selectedBooks: newBooks });
  };

  render() {
    const { onClose, onSelect } = this.props;
    const {
      loading,
      name,
      branch,
      year,
      quarter,
      serise,
      grade,
      subject,
      bookType,
      branchDict,
      yearDict,
      quarterDict,
      seriseDict,
      gradeDict,
      subjectDict,
      bookTypeDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
      selectedBooks,
    } = this.state;

    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        width={960}
        title="选择教材"
        onOk={() => {
          if (selectedBooks.length === 0) {
            this._modal.showOkTips('还没有选择教材');
            return;
          }
          onSelect(selectedBooks);
        }}
        onCancel={onClose}
        ref={(c) => (this._modal = getWrappedInstance(c))}
      >
        <div className={styles.wrap}>
          <div className={styles.nameSearchSec}>
            <div className={styles.nameSearchInput}>
              <Input
                placeholder="请输入名称进行搜索"
                value={name}
                onChange={(e) => this.setState({ name: e.target.value })}
                onKeyDown={(e) => {
                  if (e.key === 'Enter') {
                    this.searchByName();
                  }
                }}
              />
            </div>
            <div className={styles.nameSearchBtn}>
              <Button type="primary" onClick={this.searchByName}>
                搜索
              </Button>
            </div>
          </div>
          <div className={styles.selectSearchSec}>
            <CollapsableSearchBox
              title="使用范围"
              dict={branchDict}
              value={branch}
              onChange={(val) => this.search({ branch: val })}
            />
            <CollapsableSearchBox
              title="年份"
              dict={yearDict}
              value={year}
              onChange={(val) => this.search({ year: val })}
            />
            <CollapsableSearchBox
              title="季度"
              dict={quarterDict}
              value={quarter}
              onChange={(val) => this.search({ quarter: val })}
            />
            <CollapsableSearchBox
              title="产品系列"
              dict={seriseDict}
              value={serise}
              onChange={(val) => this.search({ serise: val })}
            />
            <CollapsableSearchBox
              title="年级"
              dict={gradeDict}
              value={grade}
              onChange={(val) => this.search({ grade: val })}
            />
            <CollapsableSearchBox
              title="科目"
              dict={subjectDict}
              value={subject}
              onChange={(val) => this.search({ subject: val })}
            />
          </div>
          <div className={styles.selectedBooks}>
            <div className={styles.selectedBooksLabel}>已选择（最多10条）</div>
            <div className={styles.selectedBooksContent}>
              <TagSelect
                selectable={false}
                allowClear={false}
                dict={selectedBooks}
                value={selectedBooks.map((c) => c.val)}
                onChange={this.selectedBooksChange}
              />
            </div>
          </div>
          <div className={styles.totalTips}>
            <span>共搜索到</span>
            <span style={{ color: 'red' }}>{totalRecords}</span>
            <span>条数据</span>
          </div>
          <div className={styles.list}>
            {list && list.length > 0 ? (
              list.map((d) => {
                const {
                  id,
                  name,
                  createrName,
                  branchName,
                  statusName,
                  classTypeName,
                  yearName,
                  quarterName,
                  gradeName,
                  subjectName,
                  bookTypeName,
                } = d;
                let categoryNames = [];
                if (classTypeName) {
                  categoryNames.push(classTypeName);
                }
                if (yearName) {
                  categoryNames.push(yearName);
                }
                if (quarterName) {
                  categoryNames.push(quarterName);
                }
                if (gradeName) {
                  categoryNames.push(gradeName);
                }
                if (subjectName) {
                  categoryNames.push(subjectName);
                }
                if (bookTypeName) {
                  categoryNames.push(bookTypeName);
                }
                return (
                  <div key={id} className={styles.listItem}>
                    <div className={styles.listItemCheckbox}>
                      <Checkbox
                        checked={selectedBooks && selectedBooks.find((s) => s.val === id) ? true : false}
                        onChange={(checked) => this.checkBook(checked, id, name)}
                      />
                    </div>
                    <div className={styles.listItemContent}>
                      <div className={styles.listItemTitle}>{name}</div>
                      <div className={styles.listItemInfos}>
                        <ListInfo title="上传者" content={createrName} />
                        <ListInfo title="使用范围" content={branchName} />
                        <ListInfo title="状态" content={statusName} />
                        <ListInfo title="分类" content={categoryNames.join('; ')} />
                      </div>
                    </div>
                  </div>
                );
              })
            ) : (
              <div className={styles.noDataTips}>暂无数据</div>
            )}
          </div>
          <Pagination
            current={currentPage}
            pageSize={pageSize}
            total={totalRecords}
            pageSizeOptions={['5', '10', '20']}
            showSizeChanger
            showQuickJumper
            onChange={(currentPage, pageSize) => this.pageChange(currentPage, pageSize)}
            onShowSizeChange={(currentPage, pageSize) => this.pageChange(currentPage, pageSize)}
          />
        </div>
      </Modal>
    );
  }
}

export default SelectBook;
