import React from 'react';
import { connect } from 'dva';
import { STORE_NAME, formatTabAttribute } from '../../store/config';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import DatePicker from 'COMPONENT/DatePicker';
import { dateToMoment } from 'COMPONENT/utils/form';
import { getRelationsDictPlus, getRelationPayloadPlus } from 'MODEL/dict/utils/index';
import { branch, campus } from 'MODEL/dict/utils/config';

class SearchBlock extends React.Component {
  constructor(props) {
    super(props);

    // 分公司关联关系type、key对应关系配置
    this.branchTreeMap = {
      [branch]: 'val_branchId',
      [campus]: 'val_campusId',
    };

    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);
    this.onUpdateStore = this.onUpdateStore.bind(this);
    this.onBranchChange = this.onBranchChange.bind(this);
  }

  /**
   * 搜索列表
   * 传入paneKey用于判断搜索哪个tab
   */
  onSearch() {
    const { paneKey, dispatch } = this.props;

    dispatch({
      type: `${STORE_NAME}/onTabSearch`,
      payload: { paneKey },
    });
  }

  onReset() {
    const { paneKey, dispatch } = this.props;

    dispatch({
      type: `${STORE_NAME}/resetTabSearch`,
      payload: { paneKey },
    });
  }

  /**
   * 更新store，组装成 {tab_one: {val_yearId: 'xxx'}}
   * @param key 键
   * @param value 值
   */
  onUpdateStore(key, value) {
    const { paneKey, dispatch } = this.props;
    const { storeKey } = formatTabAttribute(paneKey);
    const oldSearch = this.props[storeKey];

    let payload = {};
    payload[storeKey] = {
      ...oldSearch,
      [key]: value,
    };

    dispatch({
      type: `${STORE_NAME}/updateState`,
      payload: { ...payload },
    });
  }

  /**
   * 选择分公司
   * 同时清空咨询师、获取咨询师
   * @param val
   */
  onBranchChange(val) {
    const { paneKey, dispatch, branchDict } = this.props;
    const { storeKey } = formatTabAttribute(paneKey);
    const oldStore = this.props[storeKey];

    const result = getRelationPayloadPlus('val_branchId', val, {
      type: branch,
      dictTree: branchDict,
      state: oldStore,
      map: this.branchTreeMap,
    });

    let payload = {};
    payload[storeKey] = {
      ...oldStore,
      ...result,
      val_counselor: null,
    };

    dispatch({
      type: `${STORE_NAME}/updateState`,
      payload: { ...payload },
    });

    dispatch({
      type: `${STORE_NAME}/loadDict`,
      payload: { paneKey },
    });
  }

  /**
   * 获取要禁用的开始日期值
   * @param date 过滤前的日期值
   * @param endDate
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate(date, endDate) {
    if (!date || !endDate) {
      return false;
    }

    return dateToMoment(date).valueOf() > dateToMoment(endDate).valueOf();
  }

  /**
   * 获取要禁用的结束日期值
   * @param date 过滤前的日期值
   * @param startDate
   * @return {boolean} 是否要禁用
   */
  getDisabledEndDate(date, startDate) {
    if (!date || !startDate) {
      return false;
    }

    return dateToMoment(date).valueOf() < dateToMoment(startDate).valueOf();
  }

  render() {
    const { paneKey, branchDict } = this.props;

    const { storeKey, showCounselor, counselorName } = formatTabAttribute(paneKey);
    const paneSearch = this.props[storeKey];
    const { val_branchId, val_campusId, val_counselor, val_startTime, val_endTime, disable_branch, counselorDict } =
      paneSearch;
    const branchCampusDict = getRelationsDictPlus(campus)(branchDict, val_branchId);

    return (
      <SearchBar onSearch={this.onSearch} onReset={this.onReset}>
        <Select
          placeholder="分公司"
          dict={branchDict}
          value={val_branchId}
          onChange={this.onBranchChange}
          allowClear={false}
          showSearch={true}
          disabled={disable_branch}
        />

        <Select
          placeholder="学习中心"
          dict={branchCampusDict}
          value={val_campusId}
          onChange={(val) => {
            this.onUpdateStore('val_campusId', val);
          }}
          allowClear={true}
          showSearch={true}
        />

        {showCounselor && (
          <Select
            placeholder={counselorName}
            value={val_counselor}
            dict={counselorDict}
            onChange={(val) => {
              this.onUpdateStore('val_counselor', val);
            }}
            allowClear={true}
            showSearch={true}
          />
        )}

        <DatePicker
          placeholder="起始日期"
          value={val_startTime}
          disabledDate={(date) => this.getDisabledStartDate(date, val_endTime)}
          onChange={(val) => {
            this.onUpdateStore('val_startTime', val);
          }}
          allowClear={true}
        />

        <DatePicker
          placeholder="结束日期"
          value={val_endTime}
          disabledDate={(date) => this.getDisabledEndDate(date, val_startTime)}
          onChange={(val) => {
            this.onUpdateStore('val_endTime', val);
          }}
          allowClear={true}
        />
      </SearchBar>
    );
  }
}

const mapStateToProps = (state) => {
  const { branchCampusTree } = state.dict;
  const tabStore = state[STORE_NAME];
  const { tab_one, tab_two, tab_three } = tabStore;

  return {
    branchDict: branchCampusTree,
    tab_one,
    tab_two,
    tab_three,
  };
};

export default connect(mapStateToProps)(SearchBlock);
