import { clone } from 'lodash';
import { STORE_NAME, TABS, formatTabAttribute } from './config';
import { defaultState, defaultTabSearch, formatInitValue } from './state';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import {
  loadCounselorDict,
  loadCounselorList,
  loadCashList,
  formatCounselorParam,
  formatCashParam,
  exportCounselorReport,
  exportCashReport,
} from './loader';

export default {
  namespace: STORE_NAME,
  state: {
    ...clone(defaultState),
  },
  reducers: {
    /**
     * @Attention 因为tab有多个，这里需要深度拷贝(深度为2)
     * @param state
     * @param payload
     */
    updateState(state, { payload }) {
      let newState = { ...state };

      if (payload) {
        for (let i in payload) {
          let item = payload[i];
          if (item && Object.prototype.toString.call(item) === '[object Object]') {
            newState[i] = { ...state[i], ...item };
          } else {
            newState[i] = item;
          }
        }
      }

      return {
        ...newState,
      };
    },
  },
  effects: {
    /**
     * 初始化用户权限tab
     * @param payload
     * @param put
     * @param select
     * @return {IterableIterator<*>}
     */
    *initTabs({ payload }, { put, select }) {
      let tab_key = '';
      const tabs = yield select((state) => {
        const { btnAuths } = state.account;
        let result = [];

        TABS.forEach((item) => {
          if (btnAuths.includes(item.auth)) {
            result.push(item);
          }
        });

        return result;
      });

      if (tabs && tabs.length > 0) {
        tab_key = tabs[0].key;
      }

      yield put({
        type: 'updateState',
        payload: {
          tabs,
          tab_key,
        },
      });
    },

    /**
     * 初始化所有tab搜索条件默认选中项(最好是能够根据用户权限tab进行初始化参数)
     * @param payload
     * @param put
     * @param select
     * @return {IterableIterator<*>}
     */
    *initPageSearch({ payload }, { put, select }) {
      const initValue = yield select((state) => {
        return formatInitValue(state);
      });

      const store = {
        tab_one: initValue,
        tab_two: initValue,
        tab_three: initValue,
      };

      yield put({
        type: 'updateState',
        payload: store,
      });

      yield put({
        type: 'loadDict',
        payload: { paneKey: '1' },
      });

      yield put({
        type: 'loadDict',
        payload: { paneKey: '2' },
      });
    },

    /**
     * 初始化对应tab的搜索条件默认选中项
     * @param payload
     * @param put
     * @param select
     * @return {IterableIterator<*>}
     */
    *initTabSearch({ payload }, { put, select }) {
      const { paneKey } = payload;
      const { storeKey } = formatTabAttribute(paneKey);
      const initValue = yield select((state) => {
        return formatInitValue(state);
      });

      let store = {};
      store[storeKey] = { ...initValue };

      yield put({
        type: 'updateState',
        payload: store,
      });

      yield put({
        type: 'loadDict',
        payload: { paneKey },
      });
    },

    /**
     * 重置页面同时重置tabs
     * @param action
     * @param put
     * @return {IterableIterator<*>}
     */
    *resetPage(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState),
        },
      });

      yield put({ type: 'initTabs' });
    },

    /**
     * 重置tab的搜索结果，组装成 {tab_one: {...}}
     * @param payload JSON {paneKey: '1'}
     * @param put
     * @param select
     * @return {IterableIterator<*>}
     */
    *resetTabSearch({ payload }, { put, select }) {
      const { paneKey } = payload;
      const { storeKey } = formatTabAttribute(paneKey);
      const tabStore = yield select((state) => {
        return state[STORE_NAME][storeKey];
      });

      let search = {};
      search[storeKey] = { ...tabStore, ...defaultTabSearch };

      yield put({
        type: 'updateState',
        payload: search,
      });

      yield put({
        type: 'initTabSearch',
        payload: { paneKey },
      });
    },

    /**
     * 单个tab搜索
     * @param JSON payload {paneKey: '选项卡对应key'}
     * @param put
     * @return {IterableIterator<*>}
     */
    *onTabSearch({ payload }, { put }) {
      const { paneKey } = payload;
      const changeParam = { paneKey, pageNo: 1 };

      yield put({
        type: 'loadList',
        payload: { ...changeParam },
      });
    },

    /**
     * 根据tab选中的分公司获取相应tab的咨询师/学管师
     * @param payload
     * @param put
     * @param select
     * @return {IterableIterator<*>}
     */
    *loadDict({ payload }, { put, select }) {
      const { paneKey } = payload;
      const { storeKey } = formatTabAttribute(paneKey);
      const tabStore = yield select((state) => {
        return state[STORE_NAME][storeKey];
      });

      if (paneKey === '3') {
        return false;
      }

      let param = { orgId: tabStore.val_branchId };
      if (paneKey === '1') {
        param.job = ['ZXS', 'ZXZG'];
      } else {
        param.job = ['XGS', 'XGZG'];
      }

      const result = yield loadCounselorDict(param, true);
      let newStore = {};
      newStore[storeKey] = { counselorDict: result };

      yield put({ type: 'updateState', payload: newStore });
    },

    /**
     * 获取产品列表, 不同tab对应参数不同，根据传入的paneKey判断
     * @param payload
     * @param put
     * @param select
     * @return {IterableIterator<*>}
     */
    *loadList({ payload }, { put, select }) {
      const { paneKey } = payload;
      const { storeKey } = formatTabAttribute(paneKey);
      const tabStore = yield select((state) => {
        return state[STORE_NAME][storeKey];
      });

      if (tabStore.listLoading) {
        return false;
      }

      let loadFunction = loadCounselorList;
      let param = formatCounselorParam(tabStore, payload);

      if (paneKey === '3') {
        loadFunction = loadCashList;
        param = formatCashParam(tabStore, payload);
      }

      if (!param.branchId) {
        showMessage('请选择分公司!');
        return false;
      }

      if (!param.startDate || !param.endDate) {
        showMessage('请选择时间段!');
        return false;
      }

      if (param.paneKey) {
        delete param.paneKey;
      }

      // 组装store
      let newStore = {};
      newStore[storeKey] = { listLoading: true };

      yield put({ type: 'updateState', payload: newStore });

      const result = yield loadFunction(param, true);

      newStore[storeKey] = {
        ...result,
        pageNo: param.pageNo,
        pageSize: param.pageSize,
        listLoading: false,
      };

      yield put({ type: 'updateState', payload: newStore });
    },

    /**
     * 导出报表
     * @param payload
     * @param put
     * @param select
     * @return {IterableIterator<*>}
     */
    *exportExcel({ payload }, { put, select }) {
      const paneKey = yield select((state) => {
        return state[STORE_NAME].tab_key;
      });
      const { storeKey } = formatTabAttribute(paneKey);
      const tabStore = yield select((state) => {
        return state[STORE_NAME][storeKey];
      });

      let exportFunction = exportCounselorReport;
      let param = formatCounselorParam(tabStore, { paneKey });

      if (paneKey === '3') {
        exportFunction = exportCashReport;
        param = formatCashParam(tabStore, { paneKey });
      }

      if (!param.branchId) {
        showMessage('请选择分公司!');
        return false;
      }

      if (!param.startDate || !param.endDate) {
        showMessage('请选择时间段!');
        return false;
      }

      const result = yield exportFunction(param, true);
      if (result && result.resultCode === 0) {
        yield put({ type: 'main/exportNotice' });
      }
    },
  },
};
