import React, { PropTypes } from 'react';
import { connect } from 'dva';
import style from './index.less';
import { Icon as AntdIcon } from 'antd';
import List from 'COMPONENT/List';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import file from 'UTIL/file';
import { exportStatus } from 'CONST/exportCenter';
import { Popconfirm } from 'antd';
import { getExportRecordList, deleteExportRecord } from 'SERVICE/exportFile';
import Wrapper from 'COMPONENT/Wrapper';

const Header = (props) => {
  const prefix = 'Header';
  return (
    <div className={style[prefix]}>
      <div className={style[`${prefix}-title`]}>{props.title}</div>
      <div className={style[`${prefix}-tool`]}>{props.toolNode}</div>
    </div>
  );
};
Header.propTypes = {
  title: PropTypes.oneOfType([PropTypes.string, PropTypes.node]).isRequired,
  toolNode: PropTypes.node,
};
const CleanBtn = (props) => {
  const prefix = 'CleanBtn';
  return (
    <div className={style[prefix]} onClick={props.onClick}>
      <AntdIcon type="delete" />
      <span className={style[`${prefix}-text`]}>清空</span>
    </div>
  );
};
CleanBtn.propTypes = {
  onClick: PropTypes.func.isRequired,
};

// 导出结果
let _exportResult = {};
Object.values(exportStatus).forEach((item) => {
  _exportResult[item.val.toString()] = item.text;
});

class ExportHistory extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 20,
    };
    this.columns = [
      { title: '', dataIndex: 'number', key: 'number', width: 46 },
      { title: '数据类型', dataIndex: 'dataTypeName', key: 'dataTypeName', width: 88 },
      { title: '导出文件', dataIndex: 'file', key: 'file', width: 230 },
      { title: '导出人', dataIndex: 'exportPerson', key: 'exportPerson', width: 108 },
      { title: '导出日期', dataIndex: 'exportDate', key: 'exportDate', width: 156 },
      { title: '导出结果', dataIndex: 'exportResult', key: 'exportResult', width: 88 },
      {
        title: '操作',
        key: 'action',
        width: 58,
        getActionConfig: (text, record, index) => {
          let actionArr = [];
          if (record.status === exportStatus.fail.val) {
            actionArr.push({
              text: '重试',
              onClick: () => {
                this.moreAction(record);
              },
            });
          } else {
            actionArr.push({
              text: '下载',
              onClick: () => {
                this.moreAction(record);
              },
              disabled: record.status !== exportStatus.success.val,
            });
          }
          return actionArr;
        },
      },
    ];
    this.cleanHistory = this.cleanHistory.bind(this);
    this.moreAction = this.moreAction.bind(this);
    this.initList = this.initList.bind(this);
    this.loadList = this.loadList.bind(this);
    this.buildList = this.buildList.bind(this);
    this.changePage = this.changePage.bind(this);
  }
  static classPrefix = 'ExportHistory';
  componentDidMount() {
    GlobalEvent.on('ExportHistory:init_list', this.initList);
  }
  componentWillUnmount() {
    GlobalEvent.off('ExportHistory:init_list', this.initList);
  }
  // 清除导出历史
  async cleanHistory() {
    this.setState({ loading: true });
    const rsp = await deleteExportRecord({ recordId: null, deleteType: 1 }).catch((err) => {
      showMessage(err.text, 'error');
    });
    if (rsp && rsp.resultCode === 0) {
      showMessage('清空成功！', 'success');
      this.setState({
        loading: false,
        list: [],
        total: 0,
        pageCurrent: 1,
      });
    }
  }
  // 列表更多操作
  moreAction(record) {
    if (record.status === exportStatus.success.val) {
      // 下载文件
      file.download({
        url: record.filePath,
        fileName: record.name,
        format: record.format,
      });
    } else if (record.status === exportStatus.fail.val) {
      // 重试导出
      this.setState({ loading: true });
      const _self = this;
      this.props.dispatch({
        type: 'main/reExportFile',
        payload: {
          id: record.id,
          callback() {
            _self.setState({ loading: false });
            _self.loadList(_self.state.pageCurrent, _self.state.pageSize);
          },
        },
      });
    }
  }
  // 初始化列表数据
  initList() {
    this.loadList(1, this.state.pageSize);
  }
  // 加载列表数据
  async loadList(pageCurrent, pageSize) {
    this.setState({ loading: true });
    let apiParams = {
      page: pageCurrent,
      rows: pageSize,
      exportType: 1,
    };
    const rsp = await getExportRecordList(apiParams, true).catch((err) => {
      showMessage(err.text, 'error');
    });
    let list = [];
    let total = 0;
    if (rsp && rsp.resultCode === 0) {
      list = this.buildList(rsp.datas, pageCurrent, pageSize);
      total = rsp.rowCount;
    }
    this.setState({
      loading: false,
      list,
      total,
    });
  }
  // 构建list数据
  buildList(origin, pageNum, pageSize) {
    return origin.map((item, index) => ({
      id: item.id,
      name: item.excelName,
      format: item.extName,
      status: item.status,
      filePath: item.excelPath,
      dataTypeName: item.excelTypeName,
      exportPerson: item.createUser,
      date: item.createTime,
      number: pageSize * (pageNum - 1) + index + 1,
      file: `${item.excelName}.${item.extName}`,
      exportDate: item.createTime,
      exportResult: _exportResult[item.status.toString()],
    }));
  }
  // 翻页
  changePage({ current, pageSize }) {
    this.setState(
      {
        pageCurrent: current,
        pageSize,
      },
      () => {
        this.loadList(current, pageSize);
      },
    );
  }

  render() {
    const prefix = ExportHistory.classPrefix;

    return (
      <div className={style[prefix]}>
        <div className={style[`${prefix}-wrap`]}>
          <Wrapper
            bodyFitHeight
            header={
              <Header
                title="导出历史"
                toolNode={
                  <Popconfirm placement="bottomRight" title="确定清空所有导出记录吗？" onConfirm={this.cleanHistory}>
                    <CleanBtn onClick={() => {}} />
                  </Popconfirm>
                }
              />
            }
          >
            <List
              dataSource={this.state.list}
              columns={this.columns}
              loading={this.state.loading}
              pagination={{
                total: this.state.total,
                current: this.state.pageCurrent,
                pageSize: this.state.pageSize,
              }}
              onChange={this.changePage}
            />
          </Wrapper>
        </div>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(ExportHistory);
