import React from 'react';
import ReactDOM from 'react-dom';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import DatePicker from 'COMPONENT/DatePicker';
import Radio from 'COMPONENT/Radio';
import { Cascader } from 'antd';
import { loadBranchDict, loadCampusDict, dictAddOptionTitle } from 'UTIL/dict';
import { loadConsultantDict } from 'PAGE/common/loaders';
import loader from './loader';
import moment from 'moment';
import styles from './styles.less';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const COL_WIDTH_N = 80;

const resEntranceObj = {
  CALL_OUT: '陌拜',
  DIRECT_VISIT: '直访',
  CALL_IN: '呼入',
  ON_LINE: '网络',
  TRANSFER: '转介绍',
  OUTSTANDING_VISIT: '拉访',
  OUTCALL: 'TMK',
  GROUNDPROMOTION: '地推',
  NEW_MEDIA: '新媒体',
};
const resEntrance = Object.keys(resEntranceObj).map((ele) => {
  return {
    val: ele,
    text: resEntranceObj[ele],
  };
});

class FollowupCustomerStat extends React.Component {
  constructor(props) {
    super(props);

    const { grounpId, grounpName, brenchId, brenchName, campusId, campusName, organizationType } = this.props.user;

    this.branchObj = {};
    this.campusObj = {};
    this.deliverTargetObj = {};

    this.table = null;
    this.currentDate = moment().format('YYYY-MM-DD');
    this.listData = {};

    this.cascaderSettings = {
      GROUNP: {
        defaultSearchVal: [grounpId],
        configs: [
          this.getGroupDictFactory(grounpId, grounpName),
          this.getBranchDictFactory(),
          this.getCampusDictFactory(),
          this.getDeliverTargetDictFactory(),
        ],
      },
      BRENCH: {
        defaultSearchVal: [grounpId, brenchId],
        configs: [
          this.getBranchDictFactory(brenchId, brenchName),
          this.getCampusDictFactory(),
          this.getDeliverTargetDictFactory(),
        ],
      },
      CAMPUS: {
        defaultSearchVal: [grounpId, brenchId, campusId],
        configs: [this.getCampusDictFactory(campusId, campusName), this.getDeliverTargetDictFactory()],
      },
    };

    this.cascaderSetting = this.cascaderSettings[organizationType];

    this.state = {
      loading: false, // boolean 页面加载状态
      searchExpand: false,
      position: null,

      cascaderOptions: [],
      cascaderVal: [],

      listLoading: false,
      rows: [],
      pageNo: 1,
      pageSize: 20,
      total: 0,
    };
  }

  componentDidMount() {
    this.init();
  }

  loadListData = async (params = {}, forceRequest) => {
    try {
      const { cascaderVal } = this.state;
      const defaultSearchVal = this.cascaderSetting.defaultSearchVal;
      const currentDate = this.currentDate;
      const pageNo = params['pageNo'] || this.state.pageNo;
      const pageSize = params['pageSize'] || this.state.pageSize;
      const [grounpId, branchId, campusId, deliverTarget] = !cascaderVal.length
        ? defaultSearchVal
        : [...defaultSearchVal.slice(0, defaultSearchVal.length - 1), ...cascaderVal];

      const apiParams = {
        startDate: currentDate,
        endDate: currentDate,
      };
      if (branchId !== undefined) {
        apiParams['branchId'] = branchId;
      }
      if (campusId !== undefined) {
        apiParams['campusId'] = campusId;
      }
      if (deliverTarget !== undefined) {
        apiParams['deliverTarget'] = deliverTarget;
      }

      let listData = this.listData[`${branchId}.${campusId}.${deliverTarget}`];
      if (forceRequest || !listData) {
        const res = await loader.getFollowupCustomerStatistics(apiParams);
        listData = this.formatListData(res);
        this.listData = {};
        this.listData[`${branchId}.${campusId}.${deliverTarget}`] = listData;
      }

      return {
        rows: listData.slice((pageNo - 1) * pageSize, pageNo * pageSize),
        total: listData.length,
        pageNo: pageNo,
        pageSize: pageSize,
      };
    } catch (e) {
      console.log(e);
      return {
        rows: [],
        total: 0,
        pageNo: pageNo,
        pageSize: pageSize,
      };
    }
  };

  formatListData = (data) => {
    return data.reduce((arr, column) => {
      const columnName = column.blockIdentity || '';
      column.rows.map((row, i) => {
        arr[i] = arr[i] || {
          branchId: row.branchId,
          branchName: this.branchObj[row.branchId] || '',
          campusId: row.campusId,
          campusName: this.campusObj[row.campusId] || '',
          deliverTarget: row.deliverTarget,
          deliverTargetName: this.deliverTargetObj[row.deliverTarget] || '',
        };
        arr[i][`${columnName}.appointCount`] = row.appointCount || 0;
        arr[i][`${columnName}.followupCount`] = row.followupCount || 0;
        arr[i][`${columnName}.visitCountApt`] = row.visitCountApt || 0;
        arr[i][`${columnName}.visitCountUnApt`] = row.visitCountUnApt || 0;
      });
      return arr;
    }, []);
  };

  init = async () => {
    const { organizationType } = this.props.user;
    this.setState({ loading: true });
    if (!this.cascaderSetting) {
      throw new Error(`invalid organization type`);
    }

    const defaultSearchVal = this.cascaderSetting.defaultSearchVal;
    const cascaderOptions = await this.cascaderSetting.configs[0]();
    const list = await this.loadListData();
    this.setState({
      cascaderOptions,
      cascaderVal: [defaultSearchVal[defaultSearchVal.length - 1]],
      rows: list.rows,
      total: list.total,
      pageNo: list.pageNo,
      pageSize: list.pageSize,
    });

    this.setState({ loading: false });
  };

  getGroupDictFactory = (grounpId, grounpName) => {
    if (grounpId) {
      this.branchObj[grounpId] = grounpName;
    }
    return async () => {
      return [
        {
          value: grounpId,
          label: grounpName,
          isLeaf: false,
        },
      ];
    };
  };

  getBranchDictFactory = (branchId, branchName) => {
    if (branchId) {
      this.branchObj[branchId] = branchName;
    }
    return async (groupId) => {
      if (branchId) {
        return [
          {
            value: branchId,
            label: branchName,
            isLeaf: false,
          },
        ];
      }
      const dict = [
        {
          value: '',
          label: '全部分公司',
        },
      ].concat(
        ((await loadBranchDict()) || []).map((ele) => {
          this.branchObj[ele.val] = ele.text;
          return {
            value: ele.val,
            label: ele.text,
            isLeaf: false,
          };
        }),
      );
      return dict;
    };
  };

  getCampusDictFactory = (campusId, campusName) => {
    if (campusId) {
      this.campusObj[campusId] = campusName;
    }
    return async (branchId) => {
      if (campusId) {
        return [
          {
            value: campusId,
            label: campusName,
            isLeaf: false,
          },
        ];
      }

      const dict = [
        {
          value: '',
          label: '全部学习中心',
        },
      ].concat(
        ((await loadCampusDict(branchId)) || []).map((ele) => {
          this.campusObj[ele.val] = ele.text;
          return {
            value: ele.val,
            label: ele.text,
            isLeaf: false,
          };
        }),
      );
      return dict;
    };
  };

  getDeliverTargetDictFactory = () => {
    return async (campusId) => {
      const dict = [
        {
          value: '',
          label: '全部咨询师',
        },
      ].concat(
        dictAddOptionTitle((await loadConsultantDict({ orgId: campusId, includeLeft: true })) || []).map((ele) => {
          this.deliverTargetObj[ele.val] = ele.text;
          return {
            value: ele.val,
            label: ele.text,
          };
        }),
      );
      return dict;
    };
  };

  onSearch = async () => {
    this.setState({ listLoading: true });
    const { cascaderVal } = this.state;
    const list = await this.loadListData({ pageNo: 1 }, true);
    if (!cascaderVal.length) {
      this.reset();
    }

    this.setState({
      listLoading: false,
      rows: list.rows,
      total: list.total,
      pageNo: list.pageNo,
      pageSize: list.pageSize,
    });
  };

  onReset = () => {
    const defaultSearchVal = this.cascaderSetting.defaultSearchVal;
    this.setState({
      cascaderVal: defaultSearchVal.slice(defaultSearchVal.length - 1),
    });
  };

  onPageChange = async ({ current, pageSize }) => {
    this.setState({ listLoading: true });
    const list = await this.loadListData({ pageSize, pageNo: current });
    this.setState({
      listLoading: false,
      rows: list.rows,
      total: list.total,
      pageNo: list.pageNo,
      pageSize: list.pageSize,
    });
  };

  loadCascaderData = async (selectedOptions) => {
    const targetOption = selectedOptions[selectedOptions.length - 1];
    targetOption.loading = true;
    if (!targetOption.children) {
      targetOption.children = await this.cascaderSetting.configs[selectedOptions.length](targetOption.value);
    }
    targetOption.loading = false;
    this.setState({
      cascaderOptions: [...this.state.cascaderOptions],
    });
  };

  jumpToHeader = (i) => {
    const tableDOM = ReactDOM.findDOMNode(this.table).querySelector('.ant-table-body');
    const offset = (i || 0) * (3 * COL_WIDTH_N);
    tableDOM.scrollTo(offset, 0);
  };

  childrenColumns = (dataIndexPrefix) => {
    return [
      {
        title: '跟进学生',
        dataIndex: `${dataIndexPrefix}.followupCount`,
        width: COL_WIDTH_N,
        render: (val) => val || 0,
      },
      {
        title: '预约学生',
        dataIndex: `${dataIndexPrefix}.appointCount`,
        width: COL_WIDTH_N,
        render: (val) => val || 0,
      },
      {
        title: '预约后上门学生',
        dataIndex: `${dataIndexPrefix}.visitCountApt`,
        width: COL_WIDTH_N,
        render: (val) => val || 0,
      },
      {
        title: '无预约上门学生',
        dataIndex: `${dataIndexPrefix}.visitCountUnApt`,
        width: COL_WIDTH_N,
        render: (val) => val || 0,
      },
    ];
  };

  columns = [
    {
      title: '所属分公司',
      dataIndex: 'branchName',
      fixed: 'left',
      width: COL_WIDTH_M,
    },
    {
      title: '所属学习中心',
      dataIndex: 'campusName',
      fixed: 'left',
      width: COL_WIDTH_M,
    },
    {
      title: '咨询师',
      dataIndex: 'deliverTargetName',
      fixed: 'left',
      width: COL_WIDTH_N,
    },
    {
      title: '总资源',
      width: 4 * COL_WIDTH_N,
      children: this.childrenColumns('TOTAL'),
    },
    ...resEntrance.map((ele) => {
      return {
        title: ele.text,
        width: 4 * COL_WIDTH_N,
        children: this.childrenColumns(ele.val),
      };
    }),
  ];

  render() {
    const {
      loading,
      searchExpand,
      position,
      listLoading,
      cascaderOptions,
      cascaderVal,
      rows,
      pageNo,
      pageSize,
      total,
    } = this.state;
    return (
      <Page
        loading={loading}
        route={this.props.route}
        subHeader={
          <div>
            <div className="custom-serchbar">
              <SearchBar
                onSearch={this.onSearch}
                onReset={this.onReset}
                expand={searchExpand}
                onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
              >
                <DatePicker value={this.currentDate} disabled={true} />
                <Cascader
                  allowClear={false}
                  options={cascaderOptions}
                  value={cascaderVal}
                  onChange={(val) => this.setState({ cascaderVal: val })}
                  changeOnSelect
                  size="large"
                  loadData={this.loadCascaderData}
                  style={{ width: '400px' }}
                  popupClassName="custom-cascader"
                />
              </SearchBar>
            </div>
            <label>
              报表快速定位
              <span>
                <Radio
                  key={''}
                  checked={position === ''}
                  onChange={() => {
                    this.setState({ position: '' });
                    this.jumpToHeader(0);
                  }}
                  style={{ marginLeft: 20 }}
                >
                  总资源
                </Radio>
                {resEntrance.map((ele, i) => {
                  return (
                    <Radio
                      key={ele.val}
                      checked={position === ele.val}
                      onChange={() => {
                        this.setState({ position: ele.val });
                        this.jumpToHeader(i + 1);
                      }}
                      style={{ marginLeft: 20 }}
                    >
                      {ele.text}
                    </Radio>
                  );
                })}
              </span>
            </label>
          </div>
        }
      >
        <div className={'custom-list ' + (rows.length === 0 && 'custom-list-empty')}>
          <List
            dataSource={rows}
            columns={this.columns}
            pagination={{
              pageNo,
              total,
              pageSize,
            }}
            loading={listLoading}
            bordered={true}
            onChange={this.onPageChange}
            ref={(c) => (this.table = c)}
          />
        </div>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
    btnAuth: state.account.btnAuths || [],
  };
};
export default connect(mapStateToProps)(FollowupCustomerStat);
