import React from 'react';
import { connect } from 'dva';

import { Table } from 'antd';
import { formatCurrency } from 'COMPONENT/utils/basic/num';
import { Page, List, Button, Wrapper, Select, TextLink } from 'COMPONENT';

import * as ecsClassIncomeStatisticsApi from 'SERVICE/ecsClassIncomeStatistics';
import showMessage from 'SRC/components/utils/globalTips/showMessage';
import { errorsHandle } from 'SRC/utils/common';

import styles from './index.less';
import _ from 'lodash';
import moment from 'moment';

const { COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const MONTH_MAP = {
  一月: '01',
  二月: '02',
  三月: '03',
  四月: '04',
  五月: '05',
  六月: '06',
  七月: '07',
  八月: '08',
  九月: '09',
  十月: '10',
  十一月: '11',
  十二月: '12',
};
const QUERY_LEVEL_TYPE_GROUP = 'GROUP'; // 集團
const QUERY_LEVEL_TYPE_BRANCH = 'BRANCH'; // 分公司
const QUERY_LEVEL_TYPE_CAMPUS = 'CAMPUS'; // 校区

const Utils = {
  downloadFile(downFileUrl, fileName) {
    const a = document.createElement('a');

    console.log(fileName);
    a.href = downFileUrl;
    a.setAttribute('download', fileName);
    a.style.display = 'none';

    document.body.appendChild(a);
    a.addEventListener('click', () => {
      a.parentNode.removeChild(a);
    });
    a.click();
  },

  formatNumber(number) {
    return number < 10 ? `0${number}` : `${number}`;
  },

  getQueryLevelType(branchId, campusId) {
    return campusId ? QUERY_LEVEL_TYPE_CAMPUS : branchId ? QUERY_LEVEL_TYPE_BRANCH : QUERY_LEVEL_TYPE_GROUP;
  },

  computedTotal(data = []) {
    if (!data.length) {
      return [];
    }

    const numFields = [
      /**
       * 常规型
       */
      // 实收金额
      'normalRealAmount',
      // 优惠金额
      'normalPromotionAmount',
      // 实收冲销金额
      'normalRealWashAmount',
      // 优惠冲销金额
      'normalPromotionWashAmount',
      // 小计
      'normalTotalAmount',

      /**
       * 跟读型
       */
      // 实收金额
      'followRealAmount',
      // 优惠金额
      'followPromotionAmount',
      // 实收冲销金额
      'followRealWashAmount',
      // 优惠冲销金额
      'followPromotionWashAmount',
      // 小计
      'followTotalAmount',

      /**
       * 汇总
       */
      // 实收金额
      'totalRealAmount',
      // 优惠金额
      'totalPromotionAmount',
      // 实收冲销金额
      'totalRealWashAmount',
      // 优惠冲销金额
      'totalPromotionWashAmount',
      // 小计
      'totalAmount',
    ];

    const totalRow = data.reduce(
      (prev, row) => {
        return numFields.reduce((totalRow, field) => {
          if (totalRow[field] === undefined) {
            totalRow[field] = 0;
          }

          totalRow[field] += row[field];

          return totalRow;
        }, prev);
      },
      {
        id: -1,
        studentName: '合计: ',
      },
    );

    const newData = data.concat(totalRow);

    return newData;
  },
};

class EcsClassIncomeStatistics extends React.Component {
  state = {
    loading: false,
    total: 0,
    pageNum: 1,
    pageSize: 999,
    branchId: '',
    campusId: '',
    campusDict: [],
    dataSource: [],
    isLinkSearch: false,
    productVersionId: '',
    productVersionDict: [],
    queryLevelType: QUERY_LEVEL_TYPE_BRANCH,
    countYear: `${new Date().getFullYear()}`,
    countMonth: `${Utils.formatNumber(new Date().getMonth() + 1)}`,
  };

  constructor(props) {
    super(props);

    this.columns = [
      {
        title: '集团',
        dataIndex: 'groupName',
        key: 'groupName',
        width: COL_WIDTH_XL,
        render: (groupName) => (
          <TextLink onClick={() => this.getStatisticsList({ queryLevelType: QUERY_LEVEL_TYPE_GROUP })}>
            {groupName}
          </TextLink>
        ),
      },
      {
        title: '分公司',
        dataIndex: 'branchName',
        key: 'branchName',
        width: COL_WIDTH_XL,
        render: (branchName, { branchId }) => (
          <TextLink onClick={() => this.getStatisticsList({ branchId, queryLevelType: QUERY_LEVEL_TYPE_BRANCH })}>
            {branchName}
          </TextLink>
        ),
      },
      {
        title: '学习中心',
        dataIndex: 'campusName',
        key: 'campusName',
        width: COL_WIDTH_XL,
        render: (campusName, { branchId, campusId }) => (
          <TextLink
            onClick={() => this.getStatisticsList({ branchId, campusId, queryLevelType: QUERY_LEVEL_TYPE_CAMPUS })}
          >
            {campusName}
          </TextLink>
        ),
      },
      {
        title: '学生',
        dataIndex: 'studentName',
        key: 'studentName',
        width: COL_WIDTH_M,
      },
      {
        title: '产品年份',
        dataIndex: 'productVersionName',
        key: 'productVersionName',
        width: COL_WIDTH_M,
      },
      {
        title: '目标班（常规）',
        children: [
          {
            title: '实收资金',
            dataIndex: 'normalRealAmount',
            key: 'normalRealAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'normalPromotionAmount',
            key: 'normalPromotionAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'normalRealWashAmount',
            key: 'normalRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'normalPromotionWashAmount',
            key: 'normalPromotionWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'normalTotalAmount',
            key: 'normalTotalAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '目标班（跟读）',
        children: [
          {
            title: '实收资金',
            dataIndex: 'followRealAmount',
            key: 'followRealAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'followPromotionAmount',
            key: 'followPromotionAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'followRealWashAmount',
            key: 'followRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'followPromotionWashAmount',
            key: 'followPromotionWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'followTotalAmount',
            key: 'followTotalAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '汇总',
        children: [
          {
            title: '实收资金',
            dataIndex: 'totalRealAmount',
            key: 'totalRealAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'totalPromotionAmount',
            key: 'totalPromotionAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'totalRealWashAmount',
            key: 'totalRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'totalPromotionWashAmount',
            key: 'totalPromotionWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'totalAmount',
            key: 'totalAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
        ],
      },
    ];
  }

  componentDidMount() {
    const { dispatch } = this.props;
    dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['year', 'month', 'branch', 'branchCampus'],
      },
    });
    this.initContent();
  }

  initContent() {
    // this.loadDict();
  }

  loadDict = async () => {
    const result = await ecsClassIncomeStatisticsApi.getProductVersionsDict().catch(errorsHandle());
    if (!result || result.resultCode !== 0) {
      return this.setState({ productVersionDict: [] });
    }
    const productVersionDict = [];
    _.forEach(result.value, (value, key) => {
      productVersionDict.push({ val: key, text: value });
    });
    this.setState({ productVersionDict });
  };

  loadData = async () => {
    this.setState({ loading: true });

    const {
      pageNum,
      branchId,
      pageSize,
      campusId,
      countMonth,
      countYear,
      isLinkSearch,
      queryLevelType,
      productVersionId,
      linkSearchParams,
    } = this.state;

    const params = isLinkSearch ? { ...linkSearchParams } : { campusId, branchId, queryLevelType };
    const result = await ecsClassIncomeStatisticsApi
      .getEsClassIncomeStatisticsReport({
        pageNum,
        pageSize,
        ...params,
        productVersionId,
        countMonth: [countYear, countMonth].join(''),
      })
      .catch(errorsHandle());

    if (!result || !result.data.list.length) {
      this.setState({ total: 0, dataSource: [], loading: false });
    } else {
      this.setState({ total: result.data.total, dataSource: Utils.computedTotal(result.data.list), loading: false });
    }
  };

  getStatisticsList = ({ branchId, campusId, queryLevelType }) => {
    const state = {
      ...this.state,
      pageNum: 1,
      dataSource: [],
      isLinkSearch: true,
    };
    const params = {
      branchId: '',
      campusId: '',
    };

    if (queryLevelType === QUERY_LEVEL_TYPE_GROUP) {
      params.queryLevelType = QUERY_LEVEL_TYPE_GROUP;
    } else if (queryLevelType === QUERY_LEVEL_TYPE_BRANCH) {
      params.queryLevelType = QUERY_LEVEL_TYPE_BRANCH;
      params.branchId = branchId;
    } else if (queryLevelType === QUERY_LEVEL_TYPE_CAMPUS) {
      params.queryLevelType = QUERY_LEVEL_TYPE_CAMPUS;
      params.branchId = branchId;
      params.campusId = campusId;
    }

    this.setState(
      {
        ...state,
        linkSearchParams: { ...params },
      },
      () => {
        this.loadData();
      },
    );
  };

  handleCountYearChange = (countYear) => {
    this.setState({ countYear, pageNum: 1 });
  };

  handleCountMonthChange = (countMonth) => {
    this.setState({ countMonth, pageNum: 1 });
  };

  handleBranchIdChange = (branchId) => {
    this.setState({
      branchId,
      pageNum: 1,
      campusDict: ((this.props.campusDict || []).find(({ val }) => val === branchId) || {}).options || [],
    });
  };

  handleCampusIdChange = (campusId) => {
    this.setState({ campusId, pageNum: 1 });
  };

  handleProductVersionIdChange = (productVersionId) => {
    this.setState({ productVersionId, pageNum: 1 });
  };

  handleChangePage = ({ current, pageSize }) => {
    this.setState({ pageNum: current, pageSize }, () => {
      this.loadData();
    });
  };

  handleReset = () => {
    this.setState(
      {
        pageNum: 1,
        branchId: '',
        campusId: '',
        pageSize: 999,
        campusDict: [],
        linkSearchParams: {},
        productVersionId: '',
        isLinkSearch: false,
        queryLevelType: QUERY_LEVEL_TYPE_BRANCH,
        countYear: `${new Date().getFullYear()}`,
        countMonth: `${Utils.formatNumber(new Date().getMonth() + 1)}`,
      },
      () => {
        this.loadData();
      },
    );
  };

  handleExport = async () => {
    const {
      total,
      branchId,
      campusId,
      countYear,
      countMonth,
      isLinkSearch,
      queryLevelType,
      linkSearchParams,
      productVersionId,
    } = this.state;

    if (total <= 0) {
      return showMessage('当前查询数据为空');
    }

    this.setState({ loading: true });
    const params = isLinkSearch ? { ...linkSearchParams } : { campusId, branchId, queryLevelType };
    const result = await ecsClassIncomeStatisticsApi
      .exportEsClassIncomeStatisticsReport({
        ...params,
        pageNum: 1,
        pageSize: 5000,
        productVersionId,
        countMonth: [countYear, countMonth].join(''),
      })
      .catch(errorsHandle());

    if (result && result.resultCode === 0 && result.data) {
      Utils.downloadFile(result.data, result.data.split(/\//).pop());
    }
    this.setState({ loading: false });
  };

  handleSearch = () => {
    this.setState(
      {
        ...this.state,
        isLinkSearch: false,
        linkSearchParams: {},
      },
      () => {
        this.loadData();
      },
    );
  };

  generateExportBtn() {
    const props = {
      type: 'primary',
      onClick: this.handleExport,
    };
    return <Button {...props}>导出Excel</Button>;
  }

  generateSearchBtn() {
    const props = {
      type: 'primary',
      onClick: this.handleSearch,
      style: { marginRight: '10px' },
    };
    return <Button {...props}>查询</Button>;
  }

  generateResetBtn() {
    const props = {
      onClick: this.handleReset,
      style: { marginRight: '10px' },
    };
    return <Button {...props}>重置</Button>;
  }

  generateSearchBox() {
    const { campusId, branchId, countYear, countMonth, campusDict, productVersionId } = this.state;
    const { yearDict, monthDict, branchDict } = this.props;

    return (
      <div className={styles.searchWrapper}>
        <div className={styles.searchFields}>
          <Select
            allowClear={false}
            showSearch={true}
            value={countYear}
            placeholder="凭证年份"
            style={{ width: 130 }}
            onChange={this.handleCountYearChange}
            dict={yearDict.map(({ text }) => ({ text, val: text }))}
          />
          <Select
            allowClear={false}
            value={countMonth}
            placeholder="月份"
            showSearch={true}
            style={{ width: 130 }}
            onChange={this.handleCountMonthChange}
            dict={monthDict.map(({ text }) => ({ text, val: MONTH_MAP[text] }))}
          />
          <Select
            dict={branchDict}
            value={branchId}
            placeholder="分公司"
            style={{ width: 200 }}
            onChange={this.handleBranchIdChange}
            showSearch={true}
          />
          <Select
            showSearch={true}
            dict={campusDict}
            value={campusId}
            placeholder="学习中心"
            style={{ width: 200 }}
            onChange={this.handleCampusIdChange}
          />
          <Select
            showSearch={true}
            dict={yearDict}
            value={productVersionId}
            placeholder="产品年份"
            style={{ width: 130 }}
            onChange={this.handleProductVersionIdChange}
          />
          {this.generateSearchBtn()}
          {this.generateResetBtn()}
          {this.generateExportBtn()}
        </div>
      </div>
    );
  }

  render() {
    const { total, loading, pageNum, pageSize, dataSource } = this.state;

    return (
      <Page
        title="精英班营收凭证"
        loading={loading}
        route={this.props.route}
        fitWindowHeight={true}
        className={styles.ecsClassIncomeWrapper}
      >
        <Wrapper header={this.generateSearchBox()}>
          <div style={{ margin: '10px 0' }}>
            最近更新时间：
            <span style={{ color: 'red' }}>{`${
              dataSource.length
                ? dataSource[0].updateTime
                  ? moment(dataSource[0].updateTime).format('YYYY-MM-DD HH:mm:ss')
                  : ''
                : ''
            }`}</span>
          </div>
          <Table
            bordered
            columns={this.columns}
            dataSource={dataSource}
            locale={{ emptyText: '请查询数据' }}
            onChange={this.handleChangePage}
            rowKey={(record) => record.id}
            scroll={{ x: 2550, y: document.body.clientHeight - 440 }}
            pagination={{
              total,
              pageSize,
              current: pageNum,
              showSizeChanger: true,
              showQuickJumper: true,
              pageSizeOptions: ['999'],
              showTotal: (total) => `共 ${total} 条记录`,
              pageSize: dataSource.length > 999 ? pageSize + 1 : pageSize,
            }}
          />
        </Wrapper>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const {
    dict: { year, month, branch = [], branchCampus = [] },
  } = state;
  return {
    branchDict: branch,
    yearDict: year || [],
    monthDict: month || [],
    campusDict: branchCampus,
  };
};

export default connect(mapStateToProps)(EcsClassIncomeStatistics);
