import React from 'react';
import { connect } from 'dva';
import SearchPage from './search/index.js';
import { Table } from 'antd';
import { errorsHandle } from 'SRC/utils/common';
import showMessage from 'SRC/components/utils/globalTips/showMessage';
import { formatCurrency } from 'COMPONENT/utils/basic/num';
import { Page, List, TextLink, IconButton } from 'COMPONENT';
import {
  getIncomeStatisticsByGrade,
  getMaxUpdateTimeByGradeOrSegment,
  getIncomeStatisticsByGradeSegment,
  exportIncomeStatisticsByGrade,
  exportIncomeStatisticsByGradeSegment,
} from 'SERVICE/report';
import { computedTotal } from './utils.js';
import styles from './index.less';
import moment from 'moment';

const { COL_WIDTH_S, COL_WIDTH_M } = List;

const Utils = {
  downloadFile(downFileUrl, fileName) {
    const a = document.createElement('a');
    console.log(fileName);
    a.href = downFileUrl;
    a.setAttribute('download', fileName);
    a.style.display = 'none';
    document.body.appendChild(a);
    a.addEventListener('click', () => {
      a.parentNode.removeChild(a);
    });
    a.click();
  },
};

class FinancialVoucherForGrade extends React.Component {
  state = {
    list: [],
    total: 0,
    count_date: '',
    loading: false,
    isExporting: false,
    pagination: {
      current: 1,
      pageSize: 999,
    },
    branchId: '',
    campusId: '',
    searchData: {},
    queryLevelType: 'GROUP',
  };

  componentDidMount() {
    const { dispatch } = this.props;
    // 请求产品类型枚举
    dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['year', 'month', 'branch', 'branchCampus'],
      },
    });
    this.tipContent = (
      <ul>
        <li>1）报表只统计实收金额</li>
        <li>
          <div>2）学段：</div>
          <ul style={{ marginLeft: 5 }}>
            <li>①小学：包含一年级、二年级、三年级、四年级、五年级、六年级</li>
            <li>②初中：包含初一、初二、初三、高一年级</li>
            <li>③高中：包含高一、高二、高三</li>
            <li>④其他学段：除去以上十三个年级外的其他年级数据</li>
          </ul>
        </li>
        <li>
          <div>3）年级：</div>
          <ul style={{ marginLeft: 5 }}>
            <li>①其他年级：除去常规十三个年级外的其他年级数据</li>
          </ul>
        </li>
      </ul>
    );
  }

  componentWillUnmount() {}

  /**
   * 查询现金流列表
   * @param params: 参数列表
   * @memberof financialVoucherForGrade
   */
  getReceiptMonthlyStatisticsList = (params = {}) => {
    const {
      pagination: { current, pageSize },
      searchData,
      queryLevelType,
    } = this.state;
    const _queryLevelType = params.queryLevelType || queryLevelType;
    const _branchId = params.branchId;
    const _campusId = params.campusId;
    const data = {
      ...searchData,
      pageNum: current,
      pageSize: pageSize,
      ...params,
      branchId: _branchId,
      campusId: _campusId,
      queryLevelType: _queryLevelType,
    };
    this.setState({ loading: true, branchId: _branchId, campusId: _campusId, queryLevelType: _queryLevelType });
    (data.searchType === '1' ? getIncomeStatisticsByGradeSegment : getIncomeStatisticsByGrade)(data)
      .then((res) => {
        const { data } = res;
        const { total, list } = data;
        this.setState({
          list,
          total,
        });
      })
      .catch(() => {
        this.setState({
          list: [],
          total: 0,
        });
      })
      .finally(() => {
        this.setState({ loading: false });
      })
      .finally(() => {
        getMaxUpdateTimeByGradeOrSegment(data)
          .then((res) => {
            this.setState({
              count_date: res?.data ? moment(res?.data).format('YYYY-MM-DD HH:mm:ss') : '',
            });
          })
          .catch(() => {
            this.setState({
              count_date: '',
            });
          });
      });
  };

  /**
   * 查询事件回调
   * @param searchData 查询数据
   * @memberof financialVoucherForGrade
   */
  handleSearch = (searchData) => {
    this.setState({ searchData });
    this.getReceiptMonthlyStatisticsList(searchData);
  };

  /**
   * 导出回调
   * @memberof financialVoucherForGrade
   */
  onExport = async () => {
    const {
      searchData,
      queryLevelType,
      pagination: { current, pageSize },
      branchId,
      campusId,
    } = this.state;
    const params = {
      ...searchData,
      branchId,
      campusId,
      queryLevelType,
      pageNum: current,
      pageSize: pageSize,
    };
    if (!this.state.total) {
      return showMessage('当前查询数据为空');
    }
    this.setState({ isExporting: true });
    const { data, resultCode } = await (searchData.searchType === '1'
      ? exportIncomeStatisticsByGradeSegment
      : exportIncomeStatisticsByGrade)(params).catch(errorsHandle());
    this.setState({ isExporting: false });
    if (resultCode !== 0 || !data) {
      return showMessage('导出数据失败，请稍后重试');
    }
    Utils.downloadFile(data, data.split(/\//).pop());
  };

  /**
   * 分页 回调
   * @param current 页数
   * @param pageSize 页码
   * @memberof financialVoucherForGrade
   */
  changePage = ({ current, pageSize }) => {
    const { campusId, branchId } = this.state;
    this.setState({ pagination: { current, pageSize } });
    this.getReceiptMonthlyStatisticsList({ page: current, rows: pageSize, campusId, branchId });
  };

  /**
   * 格式化表格列
   *
   * @return {*[]}
   */
  formatColumn = () => {
    const _column =
      this.state.searchData.searchType === '1'
        ? [
            {
              title: '1对1',
              children: [
                {
                  title: '小学',
                  dataIndex: 'otoPrimaryGradeSegment',
                  key: 'otoPrimaryGradeSegment',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '初中',
                  dataIndex: 'otoMiddleGradeSegment',
                  key: 'otoMiddleGradeSegment',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '高中',
                  dataIndex: 'otoHighGradeSegment',
                  key: 'otoHighGradeSegment',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '其他学段',
                  dataIndex: 'otoOtherGradeSegment',
                  key: 'otoOtherGradeSegment',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '小计',
                  dataIndex: 'otoTotal',
                  key: 'otoTotal',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
              ],
            },
            {
              title: '1对多',
              children: [
                {
                  title: '小学',
                  dataIndex: 'otmPrimaryGradeSegment',
                  key: 'otmPrimaryGradeSegment',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '初中',
                  dataIndex: 'otmMiddleGradeSegment',
                  key: 'otmMiddleGradeSegment',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '高中',
                  dataIndex: 'otmHighGradeSegment',
                  key: 'otmHighGradeSegment',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '其他学段',
                  dataIndex: 'otmOtherGradeSegment',
                  key: 'otmOtherGradeSegment',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '小计',
                  dataIndex: 'otmTotal',
                  key: 'otmTotal',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
              ],
            },
            {
              title: '小班',
              children: [
                {
                  title: '小学',
                  dataIndex: 'smallPrimaryGradeSegment',
                  key: 'smallPrimaryGradeSegment',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '初中',
                  dataIndex: 'smallMiddleGradeSegment',
                  key: 'smallMiddleGradeSegment',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '高中',
                  dataIndex: 'smallHighGradeSegment',
                  key: 'smallHighGradeSegment',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '其他学段',
                  dataIndex: 'smallOtherGradeSegment',
                  key: 'smallOtherGradeSegment',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '小计',
                  dataIndex: 'smallTotal',
                  key: 'smallTotal',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
              ],
            },
          ]
        : [
            {
              title: '1对1',
              children: [
                {
                  title: '一年级',
                  dataIndex: 'otoGrade1',
                  key: 'otoGrade1',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '二年级',
                  dataIndex: 'otoGrade2',
                  key: 'otoGrade2',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '三年级',
                  dataIndex: 'otoGrade3',
                  key: 'otoGrade3',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '四年级',
                  dataIndex: 'otoGrade4',
                  key: 'otoGrade4',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '五年级',
                  dataIndex: 'otoGrade5',
                  key: 'otoGrade5',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '六年级',
                  dataIndex: 'otoGrade6',
                  key: 'otoGrade6',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '初一',
                  dataIndex: 'otoGrade7',
                  key: 'otoGrade7',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '初二',
                  dataIndex: 'otoGrade8',
                  key: 'otoGrade8',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '初三',
                  dataIndex: 'otoGrade9',
                  key: 'otoGrade9',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '高一年级',
                  dataIndex: 'otoGrade9Alias',
                  key: 'otoGrade9Alias',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '高一',
                  dataIndex: 'otoGrade10',
                  key: 'otoGrade10',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '高二',
                  dataIndex: 'otoGrade11',
                  key: 'otoGrade11',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '高三',
                  dataIndex: 'otoGrade12',
                  key: 'otoGrade12',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '其他年级',
                  dataIndex: 'otoGradeOther',
                  key: 'otoGradeOther',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '小计',
                  dataIndex: 'otoTotal',
                  key: 'otoTotal',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
              ],
            },
            {
              title: '1对多',
              children: [
                {
                  title: '一年级',
                  dataIndex: 'otmGrade1',
                  key: 'otmGrade1',
                  width: COL_WIDTH_S,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '二年级',
                  dataIndex: 'otmGrade2',
                  key: 'otmGrade2',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '三年级',
                  dataIndex: 'otmGrade3',
                  key: 'otmGrade3',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '四年级',
                  dataIndex: 'otmGrade4',
                  key: 'otmGrade4',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '五年级',
                  dataIndex: 'otmGrade5',
                  key: 'otmGrade5',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '六年级',
                  dataIndex: 'otmGrade6',
                  key: 'otmGrade6',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '初一',
                  dataIndex: 'otmGrade7',
                  key: 'otmGrade7',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '初二',
                  dataIndex: 'otmGrade8',
                  key: 'otmGrade8',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '初三',
                  dataIndex: 'otmGrade9',
                  key: 'otmGrade9',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '高一年级',
                  dataIndex: 'otmGrade9Alias',
                  key: 'otmGrade9Alias',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '高一',
                  dataIndex: 'otmGrade10',
                  key: 'otmGrade10',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '高二',
                  dataIndex: 'otmGrade11',
                  key: 'otmGrade11',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '高三',
                  dataIndex: 'otmGrade12',
                  key: 'otmGrade12',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '其他年级',
                  dataIndex: 'otmGradeOther',
                  key: 'otmGradeOther',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '小计',
                  dataIndex: 'otmTotal',
                  key: 'otmTotal',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
              ],
            },
            {
              title: '小班',
              children: [
                {
                  title: '一年级',
                  dataIndex: 'smallGrade1',
                  key: 'smallGrade1',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '二年级',
                  dataIndex: 'smallGrade2',
                  key: 'smallGrade2',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '三年级',
                  dataIndex: 'smallGrade3',
                  key: 'smallGrade3',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '四年级',
                  dataIndex: 'smallGrade4',
                  key: 'smallGrade4',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '五年级',
                  dataIndex: 'smallGrade5',
                  key: 'smallGrade5',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '六年级',
                  dataIndex: 'smallGrade6',
                  key: 'smallGrade6',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '初一',
                  dataIndex: 'smallGrade7',
                  key: 'smallGrade7',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '初二',
                  dataIndex: 'smallGrade8',
                  key: 'smallGrade8',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '初三',
                  dataIndex: 'smallGrade9',
                  key: 'smallGrade9',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '高一年级',
                  dataIndex: 'smallGrade9Alias',
                  key: 'smallGrade9Alias',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '高一',
                  dataIndex: 'smallGrade10',
                  key: 'smallGrade10',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '高二',
                  dataIndex: 'smallGrade11',
                  key: 'smallGrade11',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '高三',
                  dataIndex: 'smallGrade12',
                  key: 'smallGrade12',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '其他年级',
                  dataIndex: 'smallGradeOther',
                  key: 'smallGradeOther',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
                {
                  title: '小计',
                  dataIndex: 'smallTotal',
                  key: 'smallTotal',
                  width: COL_WIDTH_M,
                  render: (t) => formatCurrency(t),
                },
              ],
            },
          ];
    const column = [
      {
        title: '集团',
        dataIndex: 'groupName',
        key: 'groupName',
        width: COL_WIDTH_S,
        render: (t) => (
          <TextLink
            onClick={() =>
              this.getReceiptMonthlyStatisticsList({
                queryLevelType: 'GROUP',
                branchId: undefined,
                campusId: undefined,
              })
            }
          >
            {t}
          </TextLink>
        ),
      },
      {
        title: '分公司',
        dataIndex: 'branchName',
        key: 'branchName',
        width: COL_WIDTH_S,
        render: (t, r) => (
          <TextLink
            onClick={() =>
              this.getReceiptMonthlyStatisticsList({
                queryLevelType: 'BRANCH',
                branchId: r.branchId,
                campusId: undefined,
              })
            }
          >
            {t}
          </TextLink>
        ),
      },
      {
        title: '学习中心',
        dataIndex: 'campusName',
        key: 'campusName',
        width: COL_WIDTH_M,
        render: (t, r) => (
          <TextLink
            onClick={() => {
              r.id !== -1 &&
                this.getReceiptMonthlyStatisticsList({
                  queryLevelType: 'CAMPUS',
                  campusId: r.campusId,
                  branchId: r.branchId,
                });
            }}
          >
            {t}
          </TextLink>
        ),
      },
      { title: '学生', dataIndex: 'studentName', width: COL_WIDTH_S, key: 'studentName' },
      ..._column,
      {
        title: '精英班',
        dataIndex: 'ecsTotal',
        key: 'ecsTotal',
        width: COL_WIDTH_S,
        render: (t) => formatCurrency(t),
      },
      {
        title: '讲座',
        dataIndex: 'lectureTotal',
        key: 'lectureTotal',
        width: COL_WIDTH_S,
        render: (t) => formatCurrency(t),
      },
      {
        title: '其他',
        dataIndex: 'otherTotal',
        key: 'otherTotal',
        width: COL_WIDTH_M,
        render: (t) => formatCurrency(t),
      },
      {
        title: '非经营',
        dataIndex: 'isNormalTotal',
        key: 'isNormalTotal',
        width: COL_WIDTH_M,
        render: (t) => formatCurrency(t),
      },
      {
        title: '汇总',
        dataIndex: 'totalAmount',
        key: 'totalAmount',
        width: COL_WIDTH_M,
        render: (t) => formatCurrency(t),
      },
    ];
    return column;
  };

  render() {
    const { route } = this.props;
    const { list, total, count_date } = this.state;
    const {
      loading,
      pagination: { current, pageSize },
    } = this.state;
    const columns = this.formatColumn();
    return (
      <Page
        fitWindowHeight={true}
        route={route}
        title={
          <div>
            营收凭证（学段年级）
            <IconButton type="question_circle" title={{ title: this.tipContent }} />
          </div>
        }
        className={styles.finan_flow_voucher}
        subHeader={
          <SearchPage onSearch={this.handleSearch} onExport={this.onExport} loading={this.state.isExporting} />
        }
      >
        <div style={{ marginTop: '10px' }}>
          <span style={{ color: 'red' }}>{`最近更新时间：${count_date || ''}`}</span>
        </div>
        <Table
          dataSource={computedTotal(list, columns.slice(4))}
          columns={columns}
          loading={loading}
          bordered
          pagination={{
            total,
            current,
            showSizeChanger: true,
            showQuickJumper: true,
            showTotal: (total) => `共 ${total} 条记录`,
            pageSize: list.length > 999 ? pageSize + 1 : pageSize,
            pageSizeOptions: ['999'],
          }}
          rowKey={(record) => record.id}
          onChange={this.changePage}
          scroll={{ x: this.state.searchData.searchType === '1' ? 4000 : 8000, y: document.body.clientHeight - 440 }}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(FinancialVoucherForGrade);
