import React from 'react';

import { DatePicker } from 'antd';
import { Page, List, Input, Button, Select } from 'COMPONENT';

import showMessage from 'SRC/components/utils/globalTips/showMessage';

import * as techStatisticReportApi from 'SERVICE/techStatisticReport';
import * as industrialRelConfigApi from 'SERVICE/industrialRelConfig';

import moment from 'moment';

const { RangePicker } = DatePicker;
const Utils = {
  downloadFile(downFileUrl, fileName) {
    const a = document.createElement('a');

    console.log(fileName);
    a.href = downFileUrl;
    a.setAttribute('download', fileName);
    a.style.display = 'none';

    document.body.appendChild(a);
    a.addEventListener('click', () => {
      a.parentNode.removeChild(a);
    });
    a.click();
  },
};

import styles from './styles.module.less';

export default class TechStatisticReport extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      list: [],
      orgCode: null,
      payMerName: null,
      campusDict: [],
      loading: false,
      currentDate: [moment().startOf('month'), moment()],
    };
  }

  componentDidMount() {
    this.init();
  }

  init() {
    this.loadDicts();
    this.loadData();
  }

  async loadData() {
    const { orgCode, payMerName, currentDate } = this.state;

    this.setState({ loading: true });
    const result = await techStatisticReportApi
      .getTechStatisticReport({
        orgCode,
        payMerName,
        report: 'xhboss.pay.category.report',
        endTime: currentDate[1].format('YYYY-MM-DD HH:mm:ss'),
        startTime: currentDate[0].format('YYYY-MM-DD HH:mm:ss'),
      })
      .catch(() => {
        this.setState({ loading: false });
      });
    this.setState({
      loading: false,
      list: !result || result.resultCode !== 0 || !result.data.length ? [] : result.data,
    });
  }

  async loadDicts() {
    const result = await industrialRelConfigApi.listUserAllBranchByResourceId({
      resourceId: 'techStatisticReport',
    });
    if (!result || result.resultCode !== 0 || !result.data.length) {
      return;
    }
    this.setState({
      campusDict: result.data.map(({ id, name }) => ({ val: id, text: name })),
    });
  }

  handleExport = async () => {
    const { list, orgCode, payMerName, currentDate } = this.state;

    if (!list.length) {
      showMessage('当前查询数据为空');
      return;
    }

    this.setState({ loading: true });
    const fileName = `${currentDate[0].format('YYYY-MM-DD')}至${currentDate[1].format(
      'YYYY-MM-DD',
    )}_科技主体收款统计_${moment().format('YYYY-MM-DD')}`;
    const result = await techStatisticReportApi
      .exportTechStatisticReport({
        orgCode,
        fileName,
        payMerName,
        columns: ['org', 'name', 'amount'],
        report: 'xhboss.pay.category.report',
        headers: ['分公司', '商户名称', '收款金额（元）'],
        endTime: currentDate[1].format('YYYY-MM-DD HH:mm:ss'),
        startTime: currentDate[0].format('YYYY-MM-DD HH:mm:ss'),
      })
      .catch(() => {
        this.setState({ loading: false });
      });

    // “开始查询日期至结束查询日期_科技主体收款统计_导出时间”
    if (result && result.resultCode === 0 && result.data) {
      Utils.downloadFile(result.data, `${fileName}.xlsx`);
    }

    this.setState({ loading: false });
  };

  handleSearch = () => {
    this.loadData();
  };

  handleReset = () => {
    this.setState(
      {
        orgCode: null,
        payMerName: null,
        currentDate: [moment().startOf('month'), moment()],
      },
      () => {
        this.handleSearch();
      },
    );
  };

  handleChangeOrgCode = (orgCode) => {
    this.setState({ orgCode });
  };

  handleChangePayMerName = (e) => {
    this.setState({ payMerName: e.target.value });
  };

  handleChangeDate = (currentDate) => {
    this.setState({ currentDate });
  };

  generateDate() {
    const props = {
      size: 'large',
      allowClear: false,
      style: { width: 300 },
      format: 'YYYY-MM-DD',
      value: this.state.currentDate,
      onChange: this.handleChangeDate,
      placeholder: ['开始时间', '结束时间'],
      disabledDate: (current) => {
        return current && (current < moment('2021-10-01') || current > moment().endOf('day'));
      },
    };
    return <RangePicker {...props} />;
  }

  generateOrgCodeSelect() {
    const props = {
      showSearch: true,
      style: { width: 200 },
      placeholder: '分公司',
      value: this.state.orgCode,
      dict: this.state.campusDict,
      onChange: this.handleChangeOrgCode,
    };
    return <Select {...props} />;
  }

  generatePayMerName() {
    const props = {
      style: { width: 300 },
      placeholder: '商户名称',
      value: this.state.payMerName,
      onChange: this.handleChangePayMerName,
    };

    return <Input {...props} />;
  }

  generateSearchBtn() {
    const props = {
      type: 'primary',
      onClick: this.handleSearch,
      style: { marginRight: '10px' },
    };
    return <Button {...props}>查询</Button>;
  }

  generateResetBtn() {
    const props = {
      onClick: this.handleReset,
      style: { marginRight: '10px' },
    };
    return <Button {...props}>重置</Button>;
  }

  generateExportBtn() {
    const props = {
      type: 'primary',
      onClick: this.handleExport,
    };
    return <Button {...props}>导出Excel</Button>;
  }

  generateSearchBox() {
    return (
      <div className={styles.searchWrapper}>
        <div className={styles.searchFields}>
          {this.generateDate()}
          {this.generateOrgCodeSelect()}
          {this.generatePayMerName()}
          {this.generateSearchBtn()}
          {this.generateResetBtn()}
          {this.generateExportBtn()}
        </div>
      </div>
    );
  }

  generateList() {
    const { list } = this.state;

    return <List pagination={null} dataSource={list} columns={this.getColumns()} />;
  }

  getColumns() {
    return [
      { title: '分公司', dataIndex: 'org' },
      { title: '商户名称', dataIndex: 'name' },
      { title: '收款金额（元）', dataIndex: 'amount' },
    ];
  }

  render() {
    return (
      <Page
        title="科技主体收款统计"
        loading={this.state.loading}
        footer={false}
        route={this.props.route}
        fitWindowHeight={true}
      >
        <div className={styles.listContentWrapper}>
          {this.generateSearchBox()}
          {this.generateList()}
        </div>
      </Page>
    );
  }
}
