import React from 'react';
import { connect } from 'dva';
import { SearchButton, DatePicker, Input, Select } from 'COMPONENT';
import moment from 'moment';
import { buildBranchCampusDict } from 'MODEL/dict/utils';
import { loginUser } from 'UTIL/common';
import { SMALL_CLASS_VAL } from './store/config';

// 产品组选项开关，现关闭，后续再打开
const SHOW_PRODUCT_GROUP_SELECT = false;

const productTypeDict = [{ val: SMALL_CLASS_VAL, text: '小班' }];

const operationTypeDict = [
  { val: 'DROP_CLASS', text: '退班' },
  { val: 'CHANGE_CLASS', text: '转班' },
  { val: 'CHANGE_COURSE', text: '调课' },
];

const defaultState = {
  startDate: moment().startOf('month').format('YYYY-MM-DD'),
  endDate: moment().format('YYYY-MM-DD'),
  operationType: '',
  branchId: '',
  campusId: '',
  studentName: '',
  productGroupId: null,
};

class Search extends React.Component {
  constructor(props) {
    super(props);

    this.isTwoTeacher = loginUser.isTwoTeacherSym();
    this.state = {
      ...defaultState,
      productType: SMALL_CLASS_VAL,
    };

    this.getDisabledEndDate = this.getDisabledEndDate.bind(this);
    this.getDisabledStartDate = this.getDisabledStartDate.bind(this);
    this.onBranchIdChange = this.onBranchIdChange.bind(this);
    this.onReset = this.onReset.bind(this);
  }

  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate(startDate) {
    const { endDate } = this.state;

    if (!startDate || !endDate) {
      return false;
    }

    return moment(startDate).valueOf() > moment(endDate).valueOf();
  }

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate(endDate) {
    const { startDate } = this.state;

    if (!endDate || !startDate) {
      return false;
    }

    return moment(endDate).valueOf() < moment(startDate).valueOf();
  }

  onBranchIdChange(branchId) {
    this.setState({ branchId, campusId: null });
    this.props.onSearchOptionsChang({ branchId, campusId: null });
  }

  onReset() {
    this.setState({ ...defaultState, productType: SMALL_CLASS_VAL });
    this.props.onSearchOptionsChang();
  }

  /**
   * 产品类型修改
   */
  onProductTypeChange = (val) => {
    let state = { productType: val };

    if (this.isTwoTeacher && val !== SMALL_CLASS_VAL) {
      state.productGroupId = null;
    }

    this.setState(state);
    this.props.onSearchOptionsChang(state);
  };

  render() {
    const { startDate, endDate, productType, productGroupId, operationType, branchId, campusId, studentName } =
      this.state;
    const { branchDict, productGroupDict, onSearchOptionsChang } = this.props;

    return (
      <SearchButton onReset={this.onReset} onSearch={this.props.onSearch}>
        <DatePicker
          placeholder="开始日期"
          value={startDate}
          onChange={(startDate) => {
            this.setState({ startDate });
            onSearchOptionsChang({ startDate });
          }}
          disabledDate={this.getDisabledStartDate}
          validator={{
            showStatus: this.props.isError && startDate === null,
            showTips: false,
          }}
        />
        <DatePicker
          placeholder="结束日期"
          value={endDate}
          onChange={(endDate) => {
            this.setState({ endDate });
            onSearchOptionsChang({ endDate });
          }}
          disabledDate={this.getDisabledEndDate}
          validator={{
            showStatus: this.props.isError && endDate === null,
            showTips: false,
          }}
        />
        <Select
          placeholder="产品类型"
          value={productType}
          dict={productTypeDict}
          onChange={this.onProductTypeChange}
          allowClear={false}
        />
        {this.isTwoTeacher && SHOW_PRODUCT_GROUP_SELECT && (
          <Select
            placeholder="产品组"
            dict={productGroupDict}
            value={productGroupId}
            onChange={(val) => {
              this.setState({ productGroupId: val });
              onSearchOptionsChang({ productGroupId: val });
            }}
            disabled={productType !== SMALL_CLASS_VAL}
            allowClear={true}
          />
        )}
        <Select
          placeholder="调转退类型"
          value={operationType}
          dict={operationTypeDict}
          onChange={(operationType) => {
            this.setState({ operationType });
            onSearchOptionsChang({ operationType });
          }}
        />
        <Select
          placeholder="选择分公司"
          value={branchId}
          dict={branchDict}
          onChange={this.onBranchIdChange}
          showSearch={true}
        />
        <Select
          placeholder="选择学习中心"
          value={campusId}
          dict={branchId === '' ? [] : buildBranchCampusDict([branchId], this.props.branchCampusDict)}
          onChange={(campusId) => {
            this.setState({ campusId });
            onSearchOptionsChang({ campusId });
          }}
          showSearch={true}
        />
        <Input
          placeholder="学生姓名或编号"
          value={studentName}
          onChange={(event) => {
            this.setState({ studentName: event.currentTarget.value });
            onSearchOptionsChang({ studentName: event.currentTarget.value });
          }}
        />
      </SearchButton>
    );
  }
}

const mapStateToProps = (state) => {
  const { productGroup } = state.dict;
  const { institution } = state.account.user;
  const { branch, branchCampus } = state.dict;

  return {
    institution,
    branchDict: branch,
    branchCampusDict: branchCampus,
    productGroupDict: productGroup ? productGroup[SMALL_CLASS_VAL] : null,
  };
};

export default connect(mapStateToProps)(Search);
