import React, { Component } from 'react';
import Modal from 'COMPONENT/Modal';
import { connect } from 'dva';
import { Row, Col, DatePicker, Icon as AntIcon } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Loading from 'COMPONENT/Loading';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import Block from 'COMPONENT/Block';
import TextLink from 'COMPONENT/TextLink';
import TextArea from 'COMPONENT/TextArea';
import { Tree } from 'antd';
import Icon from 'COMPONENT/Icon';
import moment from 'moment';
import { loadCommonDict, loadBranchDict, getDictTextByVal, loadCampusDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import styVars from 'COMPONENT/theme/variables';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import { debounce } from 'lodash';
import Loaders from '../loader';
import commonStyles from '../../common/commonStyle.less';
import { PROJECT_NAME } from '../../../../constants/config';

const TreeNode = Tree.TreeNode;
const spingId = 'DAT0000000231'; //春季id
const autumnId = 'DAT0000000233'; //秋季id
const ORIGINMODE = 'origin'; //原班续报阶段
const ACROSSMODE = 'across'; //跨报阶段
const OriginalEnrollStartTime = 'originalEnrollStartTime';
const OriginalEnrollEndTime = 'originalEnrollEndTime';
const AcrossEnrollStartTime = 'acrossEnrollStartTime';
const AcrossEnrollEndTime = 'acrossEnrollEndTime';
const DATEFORMAT = 'YYYY-MM-DD HH:mm:ss'; //日期格式
const productTypeMap = {
  TWO_TEACHER: 'TWO_TEACHER',
  SMALL_CLASS: 'SMALL_CLASS',
};

/**
 * 去掉padding的FormItem
 * @param props
 * @returns {*}
 * @constructor
 */
function CustomFormItem(props) {
  const { children, style, ...lastProps } = props;
  return (
    <FormItem style={{ padding: 0, ...style }} {...lastProps}>
      {children}
    </FormItem>
  );
}

/**
 * 数组转换为字符串，如果入参为空则返回undefinded，否则拼接为字符串
 */
function arrToStr(arr) {
  if (arr && arr.length > 0) {
    return arr.join(',');
  } else {
    return undefined;
  }
}

/**
 * 字符串转换数组,如果入参为空则返回[]，否则转换为数组
 */
function strToArr(str) {
  if (str) {
    return str.split(',');
  } else {
    return [];
  }
}

/**
 * 日期格式化，如果入参为空，则返回undefined，否则格式化为
 */
function formatDate(date) {
  if (date) {
    return date.format(DATEFORMAT);
  } else {
    return undefined;
  }
}

/**
 * 过滤班课树
 * @param classTree
 * @param key
 */
function filterClassTree(classTree, key) {
  return classTree.map((campus) => {
    const newCampus = { ...campus };
    if (campus.classes) {
      newCampus.classes = campus.classes.filter((classItem) => classItem.className.indexOf(key) > -1);
    }
    return newCampus;
  });
}

class EditModal extends Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      //表单数据
      acrossEnrollEndTime: null,
      acrossEnrollStartTime: null,
      branchId: [],
      classIds: [],
      classTree: [],
      gradeId: null,
      name: '',
      originalEnrollEndTime: null,
      originalEnrollStartTime: null,
      phaseOrWeekId: null,
      productTypeId: PROJECT_NAME === 'twoteacher' ? productTypeMap.TWO_TEACHER : productTypeMap.SMALL_CLASS,
      quarterId: null,
      remark: null,
      subjectId: null,
      yearId: null,
      originalDiscountIds: [],
      acrossDiscountIds: [],
      //数据字典
      branchDict: null,
      yearDict: null,
      productTypeDict: [
        { text: '双师', val: 'TWO_TEACHER' },
        { text: '小班', val: 'SMALL_CLASS' },
      ],
      productCharacterId: null,
      characterDict: [],
      weekDict: [],
      quarterDict: [],
      gradeDict: [],
      subjectDict: [],
      //展开班课设置
      expand: props.editId,
      classInfo: '',
      originalEnrollStartTimeError: null,
      originalEnrollEndTimeError: null,
      acrossEnrollStartTimeError: null,
      acrossEnrollEndTimeError: null,
      freeEnrollStartTimeError: null,
      freeEnrollEndTimeError: null,
    };
    this.checking = false; //是否显示校验提示
    this.errors = {}; //校验错误信息
    this.oldClassIds = []; //原本的班
    this.originClassTree = []; //从服务器获取到的未经过滤的原始classTree
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化数据
   */
  initContent = async () => {
    const { editId } = this.props;
    const asyncCalls = [
      loadBranchDict(),
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
      loadCommonDict('SMALL_CLASS_PHASE'),
      loadCommonDict('PRODUCT_CHARACTER'),
      loadCommonDict('WeekDay'),
    ];
    if (editId) {
      asyncCalls.push(Loaders.loadDetail({ id: editId }));
    }
    Promise.all(asyncCalls).then(
      async ([
        branchDict,
        yearDict,
        quarterDict,
        gradeDict,
        subjectDict,
        phaseDict,
        characterDict,
        weekDict,
        detailData,
      ]) => {
        // 产品性质默认全选中
        this.setState({
          branchDict,
          yearDict,
          quarterDict,
          gradeDict,
          subjectDict,
          phaseDict,
          characterDict,
          weekDict,
          productCharacterId: characterDict.map((character) => character.val),
        });
        if (!detailData) {
          this.setState({
            loading: false,
          });
        } else {
          const d = detailData.data;
          //todo class-type
          Promise.all([
            Loaders.getClassTree({
              productTypeId: d.productTypeId,
              yearId: d.yearId,
              phaseId: d.phaseId,
              branchId: d.branchId,
              gradeId: d.gradeId,
              subjectId: d.subjectId,
              quarterId: d.quarterId,
            }),
          ]).then(([result]) => {
            /**
             * 详情数据
             */
            const phaseOrWeekDict = d.quarterId
              ? d.quarterId === autumnId || d.quarterId === spingId
                ? weekDict
                : phaseDict
              : [];
            this.originClassTree = [...result.data]; //先保存原始的classTree
            this.setState({
              classTree: filterClassTree(result.data, this.state.classInfo),
              loading: false,
              ...d,
              originalDiscountIds: strToArr(d.originalDiscountIds),
              acrossDiscountIds: strToArr(d.acrossDiscountIds),
              productTypeId: d.productTypeId,
              quarterId: d.quarterId,
              yearId: d.yearId,
              phaseOrWeekId: d.phaseId,
              branchId: strToArr(d.branchId),
              gradeId: strToArr(d.gradeId),
              subjectId: strToArr(d.subjectId),
              classIds: strToArr(d.classIds),
              productCharacterId: strToArr(this.props.editCharacterId),
              characterDict,
              phaseOrWeekDict,
              //时间
              acrossEnrollEndTime: d.acrossEnrollEndTime ? moment(d.acrossEnrollEndTime) : null,
              acrossEnrollStartTime: d.acrossEnrollStartTime ? moment(d.acrossEnrollStartTime) : null,
              originalEnrollEndTime: d.originalEnrollEndTime ? moment(d.originalEnrollEndTime) : null,
              originalEnrollStartTime: d.originalEnrollStartTime ? moment(d.originalEnrollStartTime) : null,
            });
          });
        }
      },
    );
  };

  /**
   * 保存
   */
  saveDetail = async (params) => {
    const { editId } = this.props;
    if (editId) {
      return await Loaders.edit(params);
    } else {
      return await Loaders.add(params);
    }
  };

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  /**
   * 自动生成周期的名称
   * @param key
   * @param id
   */
  createPeriodName = (key, id) => {
    //需要拼接的字段的key
    const pieces = ['year', 'branch', 'productType', 'quarter', 'phaseOrWeek', 'grade', 'subject'];

    const name = pieces.reduce((acc, cur) => {
      let Id, //当前字段的下拉框选中值val
        pieceName; //当前字段的下拉框选中值Text

      if (cur === key) {
        //key为当前字段，则取变化后的val
        Id = id;
      } else {
        //key为其它字段，取之前的val
        Id = this.state[cur + 'Id'];
      }
      //多选不拼接名称
      if (typeof Id === 'string') {
        pieceName = getDictTextByVal(this.state[cur + 'Dict'], Id);
      } else if (Id && Id.length === 1) {
        pieceName = getDictTextByVal(this.state[cur + 'Dict'], Id[0]);
      } else {
        pieceName = '';
      }
      return acc + pieceName;
    }, '');

    return name;
  };

  /**
   * 根据产品类型获取班级类型
   */
  getClassTypeByProduct = (id) => {
    loadCommonDict('CLASS_TYPE', id).then((classTypeDict) => {
      this.setState({
        classTypeDict,
      });
    });
  };

  /**
   *下拉框变化还要设置名字和加载tree数据
   */
  handleChangeSelect = async (key, id) => {
    const { weekDict, phaseDict } = this.state;
    if (key === 'quarter') {
      let phaseOrWeekDict;
      if (id) {
        if (id === autumnId || id === spingId) {
          phaseOrWeekDict = weekDict;
        } else {
          phaseOrWeekDict = phaseDict;
        }
      } else {
        phaseOrWeekDict = [];
      }
      await this.updateState({
        phaseOrWeekId: null,
        phaseOrWeekDict,
      });
    }

    /* 班级类型改成了产品性质
    //如果是产品类型要重新设置班级类型
    if (key === 'productType') {
      this.getClassTypeByProduct(id)
      await this.updateState({
        classTypeId: null
      })
    }
*/

    //如果是分公司要重置优惠，注意updateState方法合并了旧state，react会自动合并，不应该手动合并，以后改
    if (key === 'branch') {
      await this.updateState({
        originalDiscountIds: [],
        acrossDiscountIds: [],
      });
    }

    //先更新对应的id和周期名称
    await this.updateState({
      [key + 'Id']: id,
      name: this.createPeriodName(key, id),
    });

    //取更新state后的id
    const { productTypeId, yearId, quarterId, gradeId, subjectId, branchId, phaseOrWeekId, productCharacterId } =
      this.state;

    //如果已选产品类型，分公司则加载classTree数据
    if (productTypeId && branchId && branchId.length > 0 && yearId && quarterId) {
      await this.updateState({
        loading: true,
      });
      Loaders.getClassTree({
        branchId,
        productTypeId,
        yearId,
        quarterId,
        gradeId: gradeId || undefined,
        subjectId: subjectId || undefined,
        phaseId: phaseOrWeekId || undefined,
        productCharacterId: productCharacterId || undefined,
      }).then(
        (result) => {
          this.originClassTree = [...result.data]; //先保存原始的classTree
          const filterTree = filterClassTree(result.data, this.state.classInfo);

          const newClassIds = [];
          //默认选择全部
          filterTree.forEach((campus) => {
            // newClassIds.push(campus.campusId)
            campus.classes.forEach((classItem) => {
              if (!classItem.periodId || this.oldClassIds.includes(classItem.classId)) {
                newClassIds.push(classItem.classId);
              }
            });
          });

          this.setState({
            classTree: filterTree,
            classIds: newClassIds,
            loading: false,
            // expand: true
          });
        },
        (err) => {
          this.setState({ loading: false });
        },
      );
    }
  };

  /**
   * 保存校验信息
   */
  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  /**
   * 根据输入搜索班课
   * @param e
   */
  onSearchTree = (e) => {
    const { classIds } = this.state;
    const newClassTree = filterClassTree(this.originClassTree, e.target.value);
    console.log(this.originClassTree, newClassTree);
    console.log(classIds);
    const newClassIds = classIds.filter((classId) => {
      return newClassTree.indexOf((treeNode) => treeNode.classId === classId) >= 0;
    });
    this.setState({
      classInfo: e.target.value,
      classTree: newClassTree,
      classIds: newClassIds,
    });
  };

  /**
   * 获取班课范围树
   */
  getTree = () => {
    const { classIds, classTree, productTypeId, branchId, classInfo } = this.state;
    const { editId } = this.props;

    /**
     * 判断是否禁用checkbox
     */
    const isCheckDisable = (classItem) => {
      if (editId) {
        return classItem.periodId && +classItem.periodId !== +editId;
      } else {
        return !!classItem.periodId;
      }
    };

    /**
     * 渲染树
     * @returns {*}
     */
    const renderTree = () => {
      return (
        <Tree showLine checkable checkedKeys={classIds || []} onCheck={this.onCheck}>
          {
            /**
             * 遍历学习中心
             */
            classTree.map((campus) => {
              return (
                <TreeNode
                  selectable={false}
                  data-type="campus"
                  disableCheckbox={campus.classes.every((classItem) => isCheckDisable(classItem))}
                  title={campus.campusName}
                  key={campus.campusId}
                >
                  {
                    /**
                     * 遍历班
                     */
                    campus.classes.map((classItem) => {
                      return (
                        <TreeNode
                          selectable={false}
                          key={classItem.classId}
                          disableCheckbox={isCheckDisable(classItem)}
                          title={
                            classItem.className +
                            (classItem.periodName ? '  (已关联:' + classItem.periodName + ')' : '')
                          }
                        />
                      );
                    })
                  }
                </TreeNode>
              );
            })
          }
        </Tree>
      );
    };

    /**
     * 渲染班课区域
     */
    if (productTypeId && branchId) {
      return (
        <div style={{ marginTop: '1em' }}>
          {/* <Row style={{paddingBottom: '.5em'}}>
            <Col span={6}>
              <Input size="small" placeholder="请输入班课名称" value={classInfo} onChange={this.onSearchTree} />
            </Col>
          </Row> */}

          <div className={`${commonStyles.treeArea} ${commonStyles.customScrollbar}`}>
            {classTree.length > 0 ? renderTree() : <div style={{ color: '#aaa', paddingTop: '1em' }}>暂无数据</div>}
          </div>
        </div>
      );
    } else {
      return <div style={{ color: '#aaa', paddingTop: '1em' }}>请先选择产品类型、年份、季度、分公司</div>;
    }
  };

  /**
   * 树节点check
   */
  onCheck = (checkedKeys, info) => {
    this.setState({ classIds: checkedKeys });
  };

  /**
   * 禁用日期
   */
  setDisableDate = (key, moment) => {
    switch (key) {
      case 'originalEnrollStartTime': {
        const { originalEnrollEndTime } = this.state;
        if (!moment || !originalEnrollEndTime) {
          return false;
        }
        return moment.valueOf() > originalEnrollEndTime.valueOf();
      }
      case 'originalEnrollEndTime': {
        const { originalEnrollStartTime } = this.state;
        if (!moment || !originalEnrollStartTime) {
          return false;
        }
        return moment.valueOf() <= originalEnrollStartTime.valueOf();
      }
      case 'acrossEnrollStartTime': {
        const { acrossEnrollEndTime, originalEnrollEndTime } = this.state;
        if (!moment) {
          return false;
        }
        if (acrossEnrollEndTime) {
          if (moment.valueOf() > acrossEnrollEndTime.valueOf()) {
            return true;
          }
        }
        if (originalEnrollEndTime) {
          if (moment.valueOf() < originalEnrollEndTime.valueOf()) {
            return true;
          }
        }
        return false;
      }
      case 'acrossEnrollEndTime': {
        const { acrossEnrollStartTime } = this.state;
        if (!moment || !acrossEnrollStartTime) {
          return false;
        }
        return moment.valueOf() <= acrossEnrollStartTime.valueOf();
      }
    }
  };

  /**
   * 检测日期范围
   */
  checkDate = () => {
    const { acrossEnrollEndTime, acrossEnrollStartTime, originalEnrollEndTime, originalEnrollStartTime } = this.state;
    try {
      this.checkOriginalStart(originalEnrollStartTime);
      this.checkOriginalEnd(originalEnrollEndTime);
      this.checkAcrossStart(acrossEnrollStartTime);
      this.checkAcrossEnd(acrossEnrollEndTime);
    } catch (e) {
      this.modal.showOkTips(e.message);
      return false;
    }

    return true;
  };

  /**
   * 获取模态框标题
   */
  getTitle = () => {
    const { editId, editMode } = this.props;
    if (!editId) {
      return '新增续报活动';
    } else {
      if (editMode === 'detail') {
        return '续报活动详情';
      } else {
        return '编辑续报活动';
      }
    }
  };

  /**
   * 原班续报起始日期校验
   */
  checkOriginalStart = (date) => {
    const { originalEnrollEndTime } = this.state;
    if (originalEnrollEndTime && !date) {
      this.setState({ originalEnrollStartTimeError: true });
      throw new Error('请选择原班续报起始时间');
    }
    if (originalEnrollEndTime && date && date.valueOf() >= originalEnrollEndTime.valueOf()) {
      this.setState({ originalEnrollStartTimeError: true });
      throw new Error('原班续报起始时间需小于截止时间');
    }
    this.setState({ originalEnrollStartTimeError: false });
  };

  /**
   * 原班续报截止日期校验
   */
  checkOriginalEnd = (date) => {
    const { originalEnrollStartTime } = this.state;
    if (originalEnrollStartTime && !date) {
      this.setState({ originalEnrollEndTimeError: true });
      throw new Error('请选择原班续报截止时间');
    }
    if (originalEnrollStartTime && date && date.valueOf() <= originalEnrollStartTime.valueOf()) {
      this.setState({ originalEnrollStartTimeError: true });
      throw new Error('原班续报起始时间需小于截止时间');
    }
    this.setState({ originalEnrollEndTimeError: false });
  };

  /**
   * 跨班续报起始日期校验
   */
  checkAcrossStart = (date) => {
    const { acrossEnrollEndTime, originalEnrollEndTime } = this.state;
    if (acrossEnrollEndTime && !date) {
      this.setState({ acrossEnrollStartTimeError: true });
      throw new Error('请选择跨班续报起始时间');
    }
    if (acrossEnrollEndTime && date && date.valueOf() >= acrossEnrollEndTime.valueOf()) {
      this.setState({ acrossEnrollStartTimeError: true });
      throw new Error('跨班续报起始时间需小于截止时间');
    }
    if (originalEnrollEndTime && date && date.valueOf() < originalEnrollEndTime.valueOf()) {
      this.setState({ acrossEnrollStartTimeError: true });
      throw new Error('跨班续报起始时间需大于或等于原班续报截止时间');
    }
    this.setState({ acrossEnrollStartTimeError: false });
  };

  /**
   * 跨班续报截止日期校验
   */
  checkAcrossEnd = (date) => {
    const { acrossEnrollStartTime } = this.state;
    if (acrossEnrollStartTime && !date) {
      this.setState({ acrossEnrollEndTimeError: true });
      throw new Error('请选择跨班续报截止时间');
    }
    if (acrossEnrollStartTime && date && date.valueOf() <= acrossEnrollStartTime.valueOf()) {
      this.setState({ acrossEnrollEndTimeError: true });
      throw new Error('跨班续报起始时间需小于截止时间');
    }
    this.setState({ acrossEnrollEndTimeError: false });
  };

  /**
   * 重置优惠
   * @param key
   */
  resetDiscount = (key) => {
    if (key === OriginalEnrollStartTime || key === OriginalEnrollEndTime) {
      this.setState({
        originalDiscountIds: [],
      });
    } else {
      this.setState({
        acrossDiscountIds: [],
      });
    }
  };

  /**
   * 设置和校验日期
   */
  handleDateChange = async (date, key) => {
    if (date) {
      date = moment(date.format(DATEFORMAT), DATEFORMAT);
    }
    await this.setState({ [key]: date });
    if (key === 'originalEnrollEndTime') {
      this.setState({ acrossEnrollStartTime: date });
    }

    //重置优惠
    this.resetDiscount(key);
    //校验
    const { acrossEnrollEndTime, acrossEnrollStartTime, originalEnrollEndTime, originalEnrollStartTime } = this.state;
    try {
      this.checkOriginalStart(originalEnrollStartTime);
      this.checkOriginalEnd(originalEnrollEndTime);
      this.checkAcrossStart(acrossEnrollStartTime);
      this.checkAcrossEnd(acrossEnrollEndTime);
    } catch (e) {}
  };

  /**
   * 如果是原班续报则修改originalDiscountIds，否则为跨班续报
   * @param ids
   * @param mode
   */
  updateDiscount = (ids, mode) => {
    if (mode === ORIGINMODE) {
      this.setState({
        originalDiscountIds: ids,
      });
    } else {
      this.setState({
        acrossDiscountIds: ids,
      });
    }
  };

  onTogglePreferential = (discountIds, mode, startTime, endTime) => {
    const { branchId } = this.state;
    if (startTime && endTime) {
      this.props.toggleSelectPreferen(
        discountIds,
        mode,
        startTime.format(DATEFORMAT),
        endTime.format(DATEFORMAT),
        branchId,
      );
    } else {
      showMessage('请先设置日期');
    }
  };

  /**
   * 确认按钮
   */
  handleOk = async () => {
    const {
      checking,
      acrossEnrollEndTime,
      acrossEnrollStartTime,
      branchId,
      classIds,
      gradeId,
      name,
      originalEnrollEndTime,
      originalEnrollStartTime,
      productTypeId,
      quarterId,
      remark,
      subjectId,
      yearId,
      phaseOrWeekId,
      originalDiscountIds,
      acrossDiscountIds,
      productCharacterId,
    } = this.state;
    const classes = classIds.filter((classId) => classId.startsWith('class@'));
    if (!checking) {
      this.setState({ checking: true });
    }
    let errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      this.modal.showOkTips(errorTips);
      return;
    }
    if (classes.length <= 0) {
      this.modal.showOkTips('请设置班课范围');
      return;
    }
    if (!this.checkDate()) {
      return;
    }
    const { editId } = this.props;
    this.setState({ loading: true });

    const requestJson = {
      id: editId || undefined,
      acrossEnrollEndTime: formatDate(acrossEnrollEndTime),
      acrossEnrollStartTime: formatDate(acrossEnrollStartTime),
      originalEnrollEndTime: formatDate(originalEnrollEndTime),
      originalEnrollStartTime: formatDate(originalEnrollStartTime),
      originalDiscountIds: arrToStr(originalDiscountIds),
      acrossDiscountIds: arrToStr(acrossDiscountIds),
      remark,
      name,
      productTypeId,
      quarterId,
      yearId,
      phaseId: phaseOrWeekId,
      subjectId: arrToStr(subjectId),
      branchId: arrToStr(branchId),
      gradeId: arrToStr(gradeId),
      productCharacterId: arrToStr(productCharacterId),
    };
    //新增还需要这些参数
    // if (!editId) {
    Object.assign(requestJson, {
      classIds: classes.map((classId) => classId.substring(6)).join(','),
    });
    // }
    this.saveDetail(requestJson).then(
      () => {
        this.setState({ loading: false });
        //保存后的回掉，关闭模态框,更新列表等
        this.props.onOk();
        showMessage('保存成功', 'success');
      },
      (e) => {
        this.setState({ loading: false });
        if (e.text) {
          this.modal.showOkTips(e.text);
        } else {
          showMessage('保存失败');
        }
      },
    );
  };

  /**
   * 渲染报读周期日期选择
   * @returns {*}
   */
  renderDatePicker = () => {
    const {
      originalEnrollStartTimeError,
      checking,
      originalEnrollEndTimeError,
      acrossEnrollStartTimeError,
      acrossEnrollEndTimeError,
      originalEnrollEndTime,
      originalEnrollStartTime,
      acrossEnrollEndTime,
      acrossEnrollStartTime,
      originalDiscountIds,
      acrossDiscountIds,
    } = this.state;
    const { editMode } = this.props;
    return (
      <div className={commonStyles.grid}>
        <div className={commonStyles.row}>
          <div className={commonStyles['col-left']}>原班续报</div>
          <div className={commonStyles['col-middle']}>
            <div className={commonStyles.beginTime}>
              <DatePicker
                disabled={editMode === 'detail'}
                showTime={{ defaultValue: moment('00:00:00', 'HH:mm:ss') }}
                className={originalEnrollStartTimeError && checking ? commonStyles.errorDate : null}
                disabledDate={(moment) => this.setDisableDate('originalEnrollStartTime', moment)}
                size="large"
                style={{ width: '270px' }}
                value={originalEnrollStartTime}
                format={DATEFORMAT}
                placeholder="起始时间"
                onChange={(originalEnrollStartTime) =>
                  this.handleDateChange(originalEnrollStartTime, OriginalEnrollStartTime)
                }
              />
            </div>
            <div className={commonStyles.timeToTime}>至</div>
            <div className={commonStyles.endTime}>
              <DatePicker
                disabled={editMode === 'detail'}
                showTime={{ defaultValue: moment('23:59:59', 'HH:mm:ss') }}
                className={originalEnrollEndTimeError && checking ? commonStyles.errorDate : null}
                disabledDate={(moment) => this.setDisableDate('originalEnrollEndTime', moment)}
                size="large"
                style={{ width: '270px' }}
                value={originalEnrollEndTime}
                format={DATEFORMAT}
                placeholder="截止时间"
                onChange={(originalEnrollEndTime) =>
                  this.handleDateChange(originalEnrollEndTime, OriginalEnrollEndTime)
                }
              />
            </div>
          </div>
          {originalDiscountIds.length > 0 && (
            <div className={commonStyles['col-right']}>
              <TextLink
                onClick={() =>
                  this.onTogglePreferential(
                    originalDiscountIds,
                    ORIGINMODE,
                    originalEnrollStartTime,
                    originalEnrollEndTime,
                  )
                }
              >
                {'优惠（' + originalDiscountIds.length + '）'}
              </TextLink>
            </div>
          )}
        </div>
        <div className={commonStyles.row}>
          <div className={commonStyles['col-left']}>跨班续报</div>
          <div className={commonStyles['col-middle']}>
            <div className={commonStyles.beginTime}>
              <DatePicker
                disabled={editMode === 'detail'}
                showTime={{ defaultValue: moment('00:00:00', 'HH:mm:ss') }}
                className={acrossEnrollStartTimeError && checking ? commonStyles.errorDate : null}
                disabledDate={(moment) => this.setDisableDate('acrossEnrollStartTime', moment)}
                size="large"
                style={{ width: '270px' }}
                value={acrossEnrollStartTime}
                format={DATEFORMAT}
                placeholder="起始时间"
                onChange={(acrossEnrollStartTime) =>
                  this.handleDateChange(acrossEnrollStartTime, AcrossEnrollStartTime)
                }
              />
            </div>
            <div className={commonStyles.timeToTime}>至</div>
            <div className={commonStyles.endTime}>
              <DatePicker
                disabled={editMode === 'detail'}
                showTime={{ defaultValue: moment('23:59:59', 'HH:mm:ss') }}
                className={acrossEnrollEndTimeError && checking ? commonStyles.errorDate : null}
                disabledDate={(moment) => this.setDisableDate('acrossEnrollEndTime', moment)}
                size="large"
                style={{ width: '270px' }}
                value={acrossEnrollEndTime}
                format={DATEFORMAT}
                placeholder="截止时间"
                onChange={(acrossEnrollEndTime) => this.handleDateChange(acrossEnrollEndTime, AcrossEnrollEndTime)}
              />
            </div>
          </div>
          {acrossDiscountIds.length > 0 && (
            <div className={commonStyles['col-right']}>
              <TextLink
                onClick={() =>
                  this.onTogglePreferential(acrossDiscountIds, ORIGINMODE, acrossEnrollStartTime, acrossEnrollEndTime)
                }
              >
                {'优惠（' + acrossDiscountIds.length + '）'}
              </TextLink>
            </div>
          )}
        </div>
      </div>
    );
  };

  render() {
    const {
      checking,
      loading,
      expand,
      productCharacterId,
      branchId,
      gradeId,
      name,
      productTypeId,
      quarterId,
      remark,
      subjectId,
      yearId,
      branchDict,
      yearDict,
      classIds,
      productTypeDict,
      quarterDict,
      gradeDict,
      subjectDict,
      phaseDict,
      characterDict,
      weekDict,
      phaseOrWeekDict,
      phaseOrWeekId,
    } = this.state;
    const { editMode, editId } = this.props;

    return (
      <Modal
        title={this.getTitle()}
        loading={loading}
        okBtnShow={editMode !== 'detail'}
        okText="保存"
        closable
        cancelText={editMode === 'detail' ? '关闭' : '取消'}
        onOk={this.handleOk}
        onCancel={this.props.onCancel}
        className={commonStyles.detailModal}
        ref={(c) => (this.modal = getWrappedInstance(c))}
        width={900}
      >
        <Block title="基础信息">
          <Row>
            <Col span={24}>
              <CustomFormItem name="续报活动名称:" labelWidth={110}>
                <Input
                  value={name}
                  disabled={editMode === 'detail'}
                  onChange={(e) => this.setState({ name: e.target.value })}
                  placeholder="自动生成，可编辑"
                />
              </CustomFormItem>
            </Col>
          </Row>
          <Row>
            <Col span={11}>
              <CustomFormItem name="产品类型:" labelWidth={110} required>
                <Select
                  value={productTypeId}
                  disabled={editMode === 'detail'}
                  dict={productTypeDict}
                  allowClear={false}
                  onChange={(productTypeId) => this.handleChangeSelect('productType', productTypeId)}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('productTypeId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
            <Col span={11} offset={2}>
              <CustomFormItem name="分公司:" labelWidth={110} required>
                <MultiSelect
                  value={branchId}
                  disabled={editMode === 'detail'}
                  dict={branchDict}
                  allowClear={false}
                  onChange={(branchId) => this.handleChangeSelect('branch', branchId)}
                  showSearch={true}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('branchId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
          </Row>
          <Row>
            <Col span={11}>
              <CustomFormItem name="年份:" labelWidth={110} required>
                <Select
                  value={yearId}
                  disabled={editMode === 'detail'}
                  dict={yearDict}
                  allowClear={false}
                  showSearch={true}
                  onChange={(yearId) => this.handleChangeSelect('year', yearId)}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('yearId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
            <Col span={11} offset={2}>
              <CustomFormItem name="季度:" labelWidth={110} required>
                <Select
                  value={quarterId}
                  disabled={editMode === 'detail'}
                  dict={quarterDict}
                  allowClear={false}
                  onChange={(quarterId) => this.handleChangeSelect('quarter', quarterId)}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('quarterId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
          </Row>
          <Row>
            <Col span={11}>
              <CustomFormItem name="年级:" labelWidth={110}>
                <MultiSelect
                  value={gradeId}
                  disabled={editMode === 'detail'}
                  dict={gradeDict}
                  showSearch={true}
                  onChange={(gradeId) => this.handleChangeSelect('grade', gradeId)}
                />
              </CustomFormItem>
            </Col>
            <Col span={11} offset={2}>
              <CustomFormItem name="科目:" labelWidth={110}>
                <MultiSelect
                  value={subjectId}
                  disabled={editMode === 'detail'}
                  dict={subjectDict}
                  showSearch={true}
                  onChange={(subjectId) => this.handleChangeSelect('subject', subjectId)}
                />
              </CustomFormItem>
            </Col>
          </Row>
          <Row>
            <Col span={11}>
              <CustomFormItem name="期数/星期数:" labelWidth={110}>
                <Select
                  placeholder={quarterId ? '请选择' : '请先选择季度'}
                  value={phaseOrWeekId}
                  dict={phaseOrWeekDict}
                  disabled={editMode === 'detail' || !quarterId}
                  onChange={(phaseOrWeekId) => this.handleChangeSelect('phaseOrWeek', phaseOrWeekId)}
                />
              </CustomFormItem>
            </Col>
            <Col span={11} offset={2}>
              <CustomFormItem name="产品性质:" labelWidth={110} required>
                <MultiSelect
                  disabled={editMode === 'detail'}
                  dict={characterDict}
                  value={productCharacterId}
                  showSearch={false}
                  onChange={(productCharacterId) => this.handleChangeSelect('productCharacter', productCharacterId)}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('productCharacterId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
          </Row>
          <div>
            <span style={{ fontColor: '#666', fontWeight: 'bold' }}>
              设置班课范围（已选 {classIds.filter((classId) => classId.startsWith('class@')).length} 个）
            </span>
            <TextLink
              onClick={() => {
                this.setState({ expand: !expand });
              }}
              style={{ float: 'right' }}
            >
              {expand ? '收起' : '展开'}
              <Icon
                type={expand ? 'triangle_up' : 'triangle_down'}
                color={styVars.primaryColor}
                style={{ position: 'relative', top: 1, left: -1, marginLeft: 5 }}
              />
            </TextLink>
            {expand && this.getTree()}
          </div>
        </Block>

        <Block title="续报活动" style={{ marginTop: '30px' }}>
          {this.renderDatePicker()}
        </Block>

        <div style={{ margin: '10px' }}>
          <p style={{ fontColor: '#666', fontWeight: 'bold', paddingBottom: '20px' }}>备注</p>
          <TextArea
            disabled={editMode === 'detail'}
            rows={3}
            maxLength={250}
            placeholder="请输入备注"
            value={remark}
            onChange={(e) => this.setState({ remark: e.target.value })}
          />
        </div>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    monitoringUserId: state.account.user.userId,
  };
};

export default connect(mapStateToProps, null, null, { withRef: true })(EditModal);
