import React from 'react';
import { connect } from 'dva';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import EditModal from './modal/index';
import ShowConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadCommonDict, loadBranchDict, loadBranchCampusDict } from 'UTIL/dict';
import SelectPreferential from '../common/SelectPreferential/index';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import Loader from './loader';
import moment from 'moment';
import { PROJECT_NAME } from 'CONST/config';

import { renderMultiLine } from '../common';
import commonStyles from '../common/commonStyle.less';
import ShareModal from './modal/ShareModal';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL, createSetWidth } = List;

const setWidth = new createSetWidth();

let defaultBranch = null; //默认第一个学习中心
const campusTempDict = [{ text: '请先选择分公司', val: '', disabled: true }];
const TWO_TEACHER_VAL = 'TWO_TEACHER';
const SMALL_CLASS_VAL = 'SMALL_CLASS';
const productTypeDict = [
  { text: '双师', val: TWO_TEACHER_VAL },
  { text: '小班', val: SMALL_CLASS_VAL },
];
const statusDict = [
  { text: '有效', val: '1' },
  { text: '无效', val: '2' },
];
const getValByText = (dict, text) => {
  const item = dict.find((item) => {
    return item.text === text;
  });
  if (item) {
    return item.val;
  } else {
    return null;
  }
};

class FreeEnroll extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true, // boolean 页面加载状态
      //查询条件
      branchId: null,
      blCampusId: null,
      productTypeId: PROJECT_NAME === 'twoteacher' ? TWO_TEACHER_VAL : SMALL_CLASS_VAL, //培优默认小班，其它双师
      yearId: null,
      quarterId: null,
      //列表数据
      listLoading: false,
      rows: [],
      page: 1,
      pageSize: 20,
      total: null,
      //数据字典数据
      branchDict: null,
      yearDict: null,
      quarterDict: null,
      //其他
      editQuliShow: false,
      editModalShow: false,
      editId: null,
      editMode: null,
      selectModalShow: false,
      discountIds: [],
      discountMode: null,
      startTime: null,
      endTime: null,
      discountBranch: null,
      status: '1',
      editCharacterId: null,
      characterMap: {},
      shareActivity: null, //分享的报读数据
      shareModalVisible: false, // 分享模态框
      activityName: '', // 分享报读名称
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  /**
   * 获取列表数据
   */
  loadListData = async (params) => {
    const { branchId, blCampusId, productTypeId, yearId, quarterId, status, pageSize, page } = this.state;
    const apiParams = {
      branchId,
      blCampusId,
      productTypeId,
      yearId,
      quarterId,
      status,
      pageSize,
      page,
      pageNo: page,
      ...params,
    };
    try {
      return await Loader.loadList(apiParams);
    } catch (e) {
      console.log(e);
      ShowMessage(e.text);
      return [];
    }
  };

  /**
   * 初始化内容
   */
  initContent = async () => {
    this.setState({ loading: true });
    const asyncCalls = [
      loadBranchDict(),
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadBranchCampusDict(),
      loadCommonDict('PRODUCT_CHARACTER'),
    ];
    Promise.all(asyncCalls).then(([branchDict, yearDict, quarterDict, campusDict, characterDict]) => {
      //默认第一个分公司，当前年份
      defaultBranch = branchDict[0] ? branchDict[0].val : null;
      const defaultYear = getValByText(yearDict, new Date().getFullYear().toString()) || null;
      const characterMap = characterDict.reduce((prev, obj) => {
        prev[obj.val] = obj.text;
        return prev;
      }, {});

      //获取列表数据
      this.loadListData({ yearId: defaultYear, branchId: defaultBranch }).then((list) => {
        this.setState({
          yearId: defaultYear,
          branchId: defaultBranch,
          loading: false,
          rows: list.datas,
          total: list.rowCount,
          page: list.pageNo,
          branchDict,
          yearDict,
          quarterDict,
          campusDict,
          characterMap,
        });
      });
    });
  };

  /**
   * 查询列表
   */
  search = async () => {
    await this.updateState({ listLoading: true, page: 1 });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      page: list.pageNo,
    });
  };

  /**
   * 重置搜索表单
   */
  reset = async () => {
    this.updateState({
      branchId: defaultBranch,
      productTypeId: null,
      yearId: null,
      quarterId: null,
      blCampusId: null,
      status: '1',
    });
  };

  /**
   * 翻页
   */

  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ listLoading: true, pageSize, page: current });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      page: list.pageNo,
    });
  };

  /**
   * 显示编辑模态框
   */
  showEditModal = (record, mode) => {
    this.setState({
      editMode: mode,
      editId: record.id,
      editModalShow: true,
      editCharacterId: record.productCharacterId,
    });
  };

  /**
   * 打开详情页
   */
  openDetailPage = (record) => {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/FreeEnroll/FreeEnrollDetail',
        params: {
          ...record,
        },
        refresh: true,
      },
    });
  };

  /**
   * 关闭模态框的回调
   */
  handleModalOk = () => {
    this.setState({
      editId: null,
      editMode: null,
      editModalShow: false,
      editCharacterId: null,
    });
    this.search();
  };

  /**
   * 改变无效和有效状态
   */
  changeStatus = (id, status) => {
    let newStatus;
    if (String(status) === '2') {
      newStatus = 1;
    } else {
      newStatus = 2;
    }
    ShowConfirm('确定改变当前周期状态？', async () => {
      await this.updateState({ loading: true });
      try {
        const data = await Loader.changeStatus({ id, status: newStatus });
        ShowMessage(data.resultMessage, 'success');
        await this.updateState({ loading: false });
        this.search();
      } catch (e) {
        ShowMessage(e.text);
        this.setState({
          loading: false,
        });
      }
    });
  };

  /**
   * 显示隐藏选择优惠
   */
  toggleSelectPreferen = (discountIds, mode, startTime, endTime, discountBranch) => {
    this.setState({
      selectModalShow: !this.state.selectModalShow,
      discountIds,
      discountMode: mode,
      startTime,
      endTime,
      discountBranch,
    });
  };

  /**
   * 保存优惠
   */
  onSavePreferen = (ids) => {
    this.editRef.updateDiscount(ids, this.state.discountMode);
    this.toggleSelectPreferen();
  };

  disableBtn = (record) => {
    const periodOneStarttime = moment(record.periodOneStarttime || Date.now()).valueOf(),
      now = moment().valueOf(),
      periodTwoEndtime = moment(record.periodTwoEndtime || Date.now()).valueOf();

    return now < periodOneStarttime || now > periodTwoEndtime;
  };

  // 分享
  shareModal = (enrollData) => {
    const { branchId } = this.state;
    this.setState({
      shareActivity: { ...enrollData, branchId },
      shareModalVisible: true,
      enrollName: enrollData.name,
    });
  };

  renderStatus = (detail, rowData, index) => {
    detail = String(detail);
    return (
      <p key="asd" style={{ display: 'flex' }}>
        <b style={{ fontSize: '25px', color: detail === '1' ? '#FFCC49' : '#AAA', marginRight: '3px' }}>·</b>
        {detail === '1' ? '有效' : '无效'}
      </p>
    );
  };

  /**
   * 渲染续报活动明细
   */
  renderDetail = (detail, rowData) => {
    const { periodOneStarttime, periodOneEndtime, periodTwoStarttime, periodTwoEndtime } = rowData;
    return (
      <div>
        <p key="asd">
          {periodOneStarttime ? (
            <span>第一期&nbsp;&nbsp;&nbsp;{periodOneStarttime + ' ~ ' + periodOneEndtime}</span>
          ) : (
            <span>第一期&nbsp;&nbsp;&nbsp;暂无</span>
          )}
        </p>
        <p key="zxc">
          {periodTwoStarttime ? (
            <span>第二期&nbsp;&nbsp;&nbsp;{periodTwoStarttime + ' ~ ' + periodTwoEndtime}</span>
          ) : (
            <span>第二期&nbsp;&nbsp;&nbsp;暂无</span>
          )}
        </p>
      </div>
    );
  };

  renderCharacter = (detail, rowData, index) => {
    if (!detail) {
      return '';
    }
    const characterString = detail
      .split(',')
      .map((id) => this.state.characterMap[id])
      .join(',');
    return renderMultiLine(characterString);
  };

  /**
   * 表头
   */
  columns = [
    {
      title: '序号',
      dataIndex: 'index',
      width: setWidth(2),
      fixed: 'left',
      render: (text, record, index) => index + 1,
    },
    { title: '分公司', dataIndex: 'branchName', width: COL_WIDTH_S },
    { title: '学习中心', dataIndex: 'blCampusName', width: COL_WIDTH_S, render: renderMultiLine },
    {
      title: '产品类型',
      dataIndex: 'productTypeId',
      width: setWidth(4),
      render: (detail) => productTypeDict.find((dict) => dict.val === detail).text,
    },
    { title: '产品年份', dataIndex: 'yearName', width: setWidth(4) },
    {
      title: '季度',
      dataIndex: 'quarterId',
      width: COL_WIDTH_S,
      render: (text) => {
        const { quarterDict } = this.state;
        const quarterArray = text.split(',');
        const quarterName = quarterDict
          .filter((quarter) => quarterArray.indexOf(quarter.val) > -1)
          .map((quarter) => quarter.text)
          .join(',');
        return renderMultiLine(quarterName);
      },
    },
    { title: '年级', dataIndex: 'gradeName', width: COL_WIDTH_S, render: renderMultiLine },
    { title: '产品性质', dataIndex: 'productCharacterId', width: COL_WIDTH_M, render: this.renderCharacter },
    { title: '报读期明细', dataIndex: 'periodOneStarttime', width: 300, render: this.renderDetail },
    { title: '状态', dataIndex: 'status', width: COL_WIDTH_S, render: this.renderStatus },
    {
      title: '满班数/总班数',
      dataIndex: 'fullNums',
      width: COL_WIDTH_M,
      render: (detail, record) => detail + ' / ' + record['allNums'],
    },
    {
      title: '操作',
      key: 'action',
      width: COL_WIDTH_L,
      fixed: 'right',
      getActionConfig: (text, record, index) => {
        const { status } = record;
        const btns = [{ text: '班课列表', onClick: () => this.openDetailPage(record) }];
        if (this.props.btnAuths.indexOf('BTN_FREEENROLL_EDIT') >= 0) {
          btns.unshift({ text: '修改', onClick: () => this.showEditModal(record) });
        }
        if (this.props.btnAuths.indexOf('BTN_FREEENROLL_SHARE') >= 0 && String(status) === '1') {
          btns.push({ text: '分享', onClick: () => this.shareModal(record) });
        }
        if (this.props.btnAuths.indexOf('BTN_FREEENROLL_TOGGLE') >= 0) {
          btns.push({
            text: String(status) === '2' ? '有效' : '无效',
            onClick: () => this.changeStatus(record.id, status),
            disabled: this.disableBtn(record),
          });
        }
        return btns;
      },
    },
  ];

  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      rows,
      page,
      pageSize,
      total,
      branchId,
      blCampusId,
      productTypeId,
      yearId,
      quarterId,
      branchDict,
      yearDict,
      quarterDict,
      startTime,
      endTime,
      status,
      campusDict,
      editModalShow,
      editId,
      editMode,
      selectModalShow,
      discountIds,
      discountBranch,
      editCharacterId,
      shareActivity,
      shareModalVisible,
      activityName,
    } = this.state;
    return (
      <Page
        loading={loading}
        route={this.props.route}
        subHeader={
          <div className={commonStyles.searchBar} style={{ paddingRight: '29px' }}>
            {this.props.btnAuths.indexOf('BTN_FREEENROLL_ADD') >= 0 && (
              <Button className={commonStyles.addQuli} onClick={() => this.setState({ editModalShow: true })}>
                新增
              </Button>
            )}
            <SearchBar
              onSearch={this.search}
              onReset={this.reset}
              expand={searchExpand}
              onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
            >
              <Select
                placeholder="分公司"
                dict={branchDict}
                value={branchId}
                allowClear={false}
                showSearch={true}
                onChange={(branchId) => this.setState({ branchId, blCampusId: null })}
              />
              <Select
                placeholder="学习中心"
                dict={branchId ? campusDict.filter((branch) => branch.val === branchId) : campusTempDict}
                value={blCampusId}
                allowClear={true}
                showSearch={true}
                onChange={(blCampusId) => this.setState({ blCampusId: blCampusId })}
              />
              <Select
                placeholder="产品类型"
                dict={productTypeDict}
                value={productTypeId}
                allowClear={true}
                onChange={(productTypeId) => this.setState({ productTypeId: productTypeId })}
              />
              <Select
                placeholder="年份"
                dict={yearDict}
                value={yearId}
                allowClear={true}
                onChange={(yearId) => this.setState({ yearId })}
              />
              <Select
                placeholder="季度"
                dict={quarterDict}
                value={quarterId}
                allowClear={true}
                onChange={(quarterId) => this.setState({ quarterId })}
              />
              <Select
                placeholder="状态"
                dict={statusDict}
                value={status}
                allowClear={true}
                onChange={(status) => this.setState({ status })}
              />
            </SearchBar>
          </div>
        }
        fitWindowHeight
      >
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows}
          pagination={{
            current: page,
            pageSize,
            total: total,
          }}
          onChange={this.pageChange}
        />
        {editModalShow && (
          <EditModal
            editId={editId}
            editMode={editMode}
            editCharacterId={editCharacterId}
            ref={(e) => {
              this.editRef = getWrappedInstance(e);
            }}
            onOk={this.handleModalOk}
            toggleSelectPreferen={this.toggleSelectPreferen}
            onCancel={() => {
              this.setState({ editModalShow: false, editId: null, editMode: null, editCharacterId: null });
            }}
          />
        )}
        {selectModalShow && (
          <SelectPreferential
            editMode={editMode}
            startTime={startTime}
            endTime={endTime}
            discountBranch={discountBranch}
            discountIds={discountIds}
            onCancel={this.toggleSelectPreferen}
            onOk={this.onSavePreferen}
          />
        )}
        {shareModalVisible && (
          <ShareModal
            activityName={activityName}
            shareActivity={shareActivity}
            onClose={() => {
              this.setState({ shareModalVisible: false, activityName: '' });
            }}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProp = (state) => {
  return {
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProp, null)(FreeEnroll);
