import React, { Component } from 'react';
import Modal from 'COMPONENT/Modal';
import { connect } from 'dva';
import { Row, Col, DatePicker } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Block from 'COMPONENT/Block';
import TextLink from 'COMPONENT/TextLink';
import moment from 'moment';
import { loadCommonDict, loadBranchDict, getDictTextByVal, loadBranchCampusDict, loadCampusDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import { debounce } from 'lodash';
import Loaders from '../loader';
import { CAMPUS, BRENCH } from 'CONST/userInfo';
import commonStyles from '../../common/commonStyle.less';

const ORIGINMODE = 'origin'; //原班续报阶段
const ACROSSMODE = 'across'; //跨报阶段
const OriginalEnrollStartTime = 'originalEnrollStartTime';
const OriginalEnrollEndTime = 'originalEnrollEndTime';
const AcrossEnrollStartTime = 'acrossEnrollStartTime';
const AcrossEnrollEndTime = 'acrossEnrollEndTime';
const DATEFORMAT = 'YYYY-MM-DD HH:mm:ss'; //日期格式

const campusTempDict = [{ text: '请先选择分公司', val: '', disabled: true }];

/**
 * 去掉padding的FormItem
 * @param props
 * @returns {*}
 * @constructor
 */
function CustomFormItem(props) {
  const { children, style, ...lastProps } = props;
  return (
    <FormItem style={{ padding: 0, ...style }} {...lastProps}>
      {children}
    </FormItem>
  );
}

/**
 * 数组转换为字符串，如果入参为空则返回undefinded，否则拼接为字符串
 */
function arrToStr(arr) {
  if (arr && arr.length > 0) {
    return arr.join(',');
  } else {
    return undefined;
  }
}

/**
 * 字符串转换数组,如果入参为空则返回[]，否则转换为数组
 */
function strToArr(str) {
  if (str) {
    return str.split(',');
  } else {
    return [];
  }
}

/**
 * 日期格式化，如果入参为空，则返回undefined，否则格式化为
 */
function formatDate(date) {
  if (date) {
    return date.format(DATEFORMAT);
  } else {
    return undefined;
  }
}

class EditModal extends Component {
  constructor(props) {
    super(props);
    let defaultBranch = null,
      defaultCampus = [];
    //取当前用户组织架构的分公司或学习中心
    if (props.organizationType === CAMPUS) {
      defaultBranch = props.userBrenchId;
      defaultCampus = [props.userCampusId];
    } else if (props.organizationType === BRENCH) {
      defaultBranch = props.userBrenchId;
    }
    this.state = {
      loading: true,
      //表单数据
      acrossEnrollEndTime: null,
      acrossEnrollStartTime: null,
      branchId: defaultBranch,
      originalEnrollEndTime: null,
      originalEnrollStartTime: null,
      productTypeId: null,
      quarterId: [],
      yearId: null,
      blCampusId: defaultCampus,
      originalDiscountIds: [],
      acrossDiscountIds: [],
      gradeId: [],
      productCharacterId: null,
      //数据字典
      characterDict: [],
      branchDict: null,
      yearDict: null,
      productTypeDict: [
        { text: '双师', val: 'TWO_TEACHER' },
        { text: '小班', val: 'SMALL_CLASS' },
      ],
      quarterDict: null,
      campusDict: [],
      studentGradeDict: [],
      originalEnrollStartTimeError: null,
      originalEnrollEndTimeError: null,
      acrossEnrollStartTimeError: null,
      acrossEnrollEndTimeError: null,
      freeEnrollStartTimeError: null,
      freeEnrollEndTimeError: null,
    };
    this.checking = false; //是否显示校验提示
    this.errors = {}; //校验错误信息
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化数据
   */
  initContent = async () => {
    const { editId } = this.props;
    const asyncCalls = [
      loadBranchDict(),
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadBranchCampusDict(),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('PRODUCT_CHARACTER'),
    ];
    if (editId) {
      asyncCalls.push(Loaders.loadDetail({ id: editId }));
    }
    Promise.all(asyncCalls).then(
      async ([branchDict, yearDict, quarterDict, campusDict, studentGradeDict, characterDict, detailData]) => {
        /**
         * 字典数据
         */
        this.setState({
          branchDict,
          yearDict,
          quarterDict,
          campusDict,
          studentGradeDict,
          characterDict,
          productCharacterId: characterDict.map((character) => character.val),
        });
        if (!detailData) {
          this.setState({
            loading: false,
          });
        } else {
          const d = detailData.data;
          this.originData = { ...d };
          /**
           * 详情数据
           */
          this.setState({
            loading: false,
            ...d,
            originalDiscountIds: strToArr(d.periodOneDiscountIds),
            acrossDiscountIds: strToArr(d.periodTwoDiscountIds),
            productTypeId: d.productTypeId,
            quarterId: strToArr(d.quarterId),
            yearId: d.yearId,
            branchId: d.branchId,
            blCampusId: strToArr(d.blCampusId),
            gradeId: strToArr(d.gradeId),
            productCharacterId: strToArr(this.props.editCharacterId),
            characterDict,
            //时间
            acrossEnrollEndTime: d.periodTwoEndtime ? moment(d.periodTwoEndtime) : null,
            acrossEnrollStartTime: d.periodTwoStarttime ? moment(d.periodTwoStarttime) : null,
            originalEnrollEndTime: d.periodOneEndtime ? moment(d.periodOneEndtime) : null,
            originalEnrollStartTime: d.periodOneStarttime ? moment(d.periodOneStarttime) : null,
          });
        }
      },
    );
  };

  /**
   * 保存
   */
  saveDetail = async (params) => {
    const { editId } = this.props;
    if (editId) {
      return await Loaders.edit(params);
    } else {
      return await Loaders.add(params);
    }
  };

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  /**
   * 保存校验信息
   */
  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  /**
   * 禁用日期
   */
  setDisableDate = (key, moment) => {
    switch (key) {
      case 'originalEnrollStartTime': {
        const { originalEnrollEndTime } = this.state;
        if (!moment || !originalEnrollEndTime) {
          return false;
        }
        return moment.valueOf() > originalEnrollEndTime.valueOf();
      }
      case 'originalEnrollEndTime': {
        const { originalEnrollStartTime } = this.state;
        if (!moment || !originalEnrollStartTime) {
          return false;
        }
        return moment.valueOf() <= originalEnrollStartTime.valueOf();
      }
      case 'acrossEnrollStartTime': {
        const { acrossEnrollEndTime, originalEnrollEndTime } = this.state;
        if (!moment) {
          return false;
        }
        if (acrossEnrollEndTime) {
          if (moment.valueOf() > acrossEnrollEndTime.valueOf()) {
            return true;
          }
        }
        if (originalEnrollEndTime) {
          if (moment.valueOf() < originalEnrollEndTime.valueOf()) {
            return true;
          }
        }
        return false;
      }
      case 'acrossEnrollEndTime': {
        const { acrossEnrollStartTime } = this.state;
        if (!moment || !acrossEnrollStartTime) {
          return false;
        }
        return moment.valueOf() <= acrossEnrollStartTime.valueOf();
      }
    }
  };

  /**
   * 检测日期范围
   */
  checkDate = () => {
    const { acrossEnrollEndTime, acrossEnrollStartTime, originalEnrollEndTime, originalEnrollStartTime } = this.state;
    try {
      this.checkOriginalStart(originalEnrollStartTime);
      this.checkOriginalEnd(originalEnrollEndTime);
      this.checkAcrossStart(acrossEnrollStartTime);
      this.checkAcrossEnd(acrossEnrollEndTime);
    } catch (e) {
      this.modal.showOkTips(e.message);
      return false;
    }

    return true;
  };

  /**
   * 获取模态框标题
   */
  getTitle = () => {
    const { editId, editMode } = this.props;
    if (!editId) {
      return '新增自由报读周期';
    } else {
      if (editMode === 'detail') {
        return '自由报读周期详情';
      } else {
        return '编辑自由报读周期';
      }
    }
  };

  /**
   * 原班续报起始日期校验
   */
  checkOriginalStart = (date) => {
    const { originalEnrollEndTime } = this.state;
    if (originalEnrollEndTime && !date) {
      this.setState({ originalEnrollStartTimeError: true });
      throw new Error('请选择第一期起始时间');
    }
    if (originalEnrollEndTime && date && date.valueOf() >= originalEnrollEndTime.valueOf()) {
      this.setState({ originalEnrollStartTimeError: true });
      throw new Error('第一期起始时间需小于截止时间');
    }
    this.setState({ originalEnrollStartTimeError: false });
  };

  /**
   * 原班续报截止日期校验
   */
  checkOriginalEnd = (date) => {
    const { originalEnrollStartTime } = this.state;
    if (originalEnrollStartTime && !date) {
      this.setState({ originalEnrollEndTimeError: true });
      throw new Error('请选择第一期截止时间');
    }
    if (originalEnrollStartTime && date && date.valueOf() <= originalEnrollStartTime.valueOf()) {
      this.setState({ originalEnrollStartTimeError: true });
      throw new Error('第一期起始时间需小于截止时间');
    }
    this.setState({ originalEnrollEndTimeError: false });
  };

  /**
   * 跨班续报起始日期校验
   */
  checkAcrossStart = (date) => {
    const { acrossEnrollEndTime, originalEnrollEndTime } = this.state;
    if (acrossEnrollEndTime && !date) {
      this.setState({ acrossEnrollStartTimeError: true });
      throw new Error('请选择第二期起始时间');
    }
    if (acrossEnrollEndTime && date && date.valueOf() >= acrossEnrollEndTime.valueOf()) {
      this.setState({ acrossEnrollStartTimeError: true });
      throw new Error('第二期起始时间需小于截止时间');
    }
    if (originalEnrollEndTime && date && date.valueOf() < originalEnrollEndTime.valueOf()) {
      this.setState({ acrossEnrollStartTimeError: true });
      throw new Error('第二期起始时间需大于或等于第一期截止时间');
    }
    this.setState({ acrossEnrollStartTimeError: false });
  };

  /**
   * 跨班续报截止日期校验
   */
  checkAcrossEnd = (date) => {
    const { acrossEnrollStartTime } = this.state;
    if (acrossEnrollStartTime && !date) {
      this.setState({ acrossEnrollEndTimeError: true });
      throw new Error('请选择第二期截止时间');
    }
    if (acrossEnrollStartTime && date && date.valueOf() <= acrossEnrollStartTime.valueOf()) {
      this.setState({ acrossEnrollEndTimeError: true });
      throw new Error('第二期起始时间需小于截止时间');
    }
    this.setState({ acrossEnrollEndTimeError: false });
  };

  /**
   * 重置优惠
   * @param key
   */
  resetDiscount = (key) => {
    if (key === OriginalEnrollStartTime || key === OriginalEnrollEndTime) {
      this.setState({
        originalDiscountIds: [],
      });
    } else {
      this.setState({
        acrossDiscountIds: [],
      });
    }
  };

  /**
   * 设置和校验日期
   */
  handleDateChange = async (date, key) => {
    if (date) {
      date = moment(date.format(DATEFORMAT), DATEFORMAT);
    }
    await this.setState({ [key]: date });
    if (key === 'originalEnrollEndTime') {
      this.setState({ acrossEnrollStartTime: date });
    }

    //重置优惠
    this.resetDiscount(key);
    //校验
    const { acrossEnrollEndTime, acrossEnrollStartTime, originalEnrollEndTime, originalEnrollStartTime } = this.state;
    try {
      this.checkOriginalStart(originalEnrollStartTime);
      this.checkOriginalEnd(originalEnrollEndTime);
      this.checkAcrossStart(acrossEnrollStartTime);
      this.checkAcrossEnd(acrossEnrollEndTime);
    } catch (e) {}
  };

  /**
   * 如果是原班续报则修改originalDiscountIds，否则为跨班续报
   * @param ids
   * @param mode
   */
  updateDiscount = (ids, mode) => {
    if (mode === ORIGINMODE) {
      this.setState({
        originalDiscountIds: ids,
      });
    } else {
      this.setState({
        acrossDiscountIds: ids,
      });
    }
  };

  onBranchChange = (branchId) => {
    this.setState({ branchId });
    this.resetDiscount(OriginalEnrollStartTime);
    this.resetDiscount(AcrossEnrollStartTime);
  };

  onTogglePreferential = (discountIds, mode, startTime, endTime) => {
    const { branchId } = this.state;
    if (startTime && endTime) {
      this.props.toggleSelectPreferen(
        discountIds,
        mode,
        startTime.format(DATEFORMAT),
        endTime.format(DATEFORMAT),
        branchId,
      );
    } else {
      showMessage('请先设置日期');
    }
  };

  /**
   * 确认按钮
   */
  handleOk = async () => {
    const {
      checking,
      acrossEnrollEndTime,
      acrossEnrollStartTime,
      branchId,
      originalEnrollEndTime,
      originalEnrollStartTime,
      productTypeId,
      quarterId,
      yearId,
      blCampusId,
      gradeId,
      originalDiscountIds,
      acrossDiscountIds,
      productCharacterId,
    } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    let errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      this.modal.showOkTips(errorTips);
      return;
    }
    if (!this.checkDate()) {
      return;
    }
    const { editId } = this.props;
    this.setState({ loading: true });

    const requestJson = {
      ...this.originData,
      id: editId || undefined,
      periodTwoEndtime: formatDate(acrossEnrollEndTime),
      periodTwoStarttime: formatDate(acrossEnrollStartTime),
      periodOneEndtime: formatDate(originalEnrollEndTime),
      periodOneStarttime: formatDate(originalEnrollStartTime),
      periodOneDiscountIds: arrToStr(originalDiscountIds),
      periodTwoDiscountIds: arrToStr(acrossDiscountIds),
      productTypeId,
      quarterId: arrToStr(quarterId),
      yearId,
      branchId: branchId,
      blCampusId: arrToStr(blCampusId),
      gradeId: arrToStr(gradeId),
      productCharacterId: arrToStr(productCharacterId),
    };
    this.saveDetail(requestJson).then(
      () => {
        this.setState({ loading: false });
        //保存后的回掉，关闭模态框,更新列表等
        this.props.onOk();
        showMessage('保存成功', 'success');
      },
      (e) => {
        this.setState({ loading: false });
        if (e.text) {
          this.modal.showOkTips(e.text);
        } else {
          showMessage('保存失败');
        }
      },
    );
  };

  /**
   * 渲染报读周期日期选择
   * @returns {*}
   */
  renderDatePicker = () => {
    const {
      originalEnrollStartTimeError,
      checking,
      originalEnrollEndTimeError,
      acrossEnrollStartTimeError,
      acrossEnrollEndTimeError,
      originalEnrollEndTime,
      originalEnrollStartTime,
      acrossEnrollEndTime,
      acrossEnrollStartTime,
      originalDiscountIds,
      acrossDiscountIds,
      productCharacterId,
    } = this.state;
    const { editMode } = this.props;
    return (
      <div className={commonStyles.grid}>
        <div className={commonStyles.row}>
          <div className={commonStyles['col-left']}>第一期</div>
          <div className={commonStyles['col-middle']}>
            <div className={commonStyles.beginTime}>
              <DatePicker
                disabled={editMode === 'detail'}
                showTime={{ defaultValue: moment('00:00:00', 'HH:mm:ss') }}
                className={originalEnrollStartTimeError && checking ? commonStyles.errorDate : null}
                disabledDate={(moment) => this.setDisableDate('originalEnrollStartTime', moment)}
                size="large"
                style={{ width: '270px' }}
                value={originalEnrollStartTime}
                format={DATEFORMAT}
                placeholder="起始时间"
                onChange={(originalEnrollStartTime) =>
                  this.handleDateChange(originalEnrollStartTime, OriginalEnrollStartTime)
                }
              />
            </div>
            <div className={commonStyles.timeToTime}>至</div>
            <div className={commonStyles.endTime}>
              <DatePicker
                disabled={editMode === 'detail'}
                showTime={{ defaultValue: moment('00:00:00', 'HH:mm:ss') }}
                className={originalEnrollEndTimeError && checking ? commonStyles.errorDate : null}
                disabledDate={(moment) => this.setDisableDate('originalEnrollEndTime', moment)}
                size="large"
                style={{ width: '270px' }}
                value={originalEnrollEndTime}
                format={DATEFORMAT}
                placeholder="截止时间"
                onChange={(originalEnrollEndTime) =>
                  this.handleDateChange(originalEnrollEndTime, OriginalEnrollEndTime)
                }
              />
            </div>
          </div>
          {originalDiscountIds.length > 0 && (
            <div className={commonStyles['col-right']}>
              <TextLink
                onClick={() =>
                  this.onTogglePreferential(
                    originalDiscountIds,
                    ORIGINMODE,
                    originalEnrollStartTime,
                    originalEnrollEndTime,
                  )
                }
              >
                {'优惠（' + originalDiscountIds.length + '）'}
              </TextLink>
            </div>
          )}
        </div>
        <div className={commonStyles.row}>
          <div className={commonStyles['col-left']}>第二期</div>
          <div className={commonStyles['col-middle']}>
            <div className={commonStyles.beginTime}>
              <DatePicker
                disabled={editMode === 'detail'}
                showTime={{ defaultValue: moment('00:00:00', 'HH:mm:ss') }}
                className={acrossEnrollStartTimeError && checking ? commonStyles.errorDate : null}
                disabledDate={(moment) => this.setDisableDate('acrossEnrollStartTime', moment)}
                size="large"
                style={{ width: '270px' }}
                value={acrossEnrollStartTime}
                format={DATEFORMAT}
                placeholder="起始时间"
                onChange={(acrossEnrollStartTime) =>
                  this.handleDateChange(acrossEnrollStartTime, AcrossEnrollStartTime)
                }
              />
            </div>
            <div className={commonStyles.timeToTime}>至</div>
            <div className={commonStyles.endTime}>
              <DatePicker
                disabled={editMode === 'detail'}
                showTime={{ defaultValue: moment('00:00:00', 'HH:mm:ss') }}
                className={acrossEnrollEndTimeError && checking ? commonStyles.errorDate : null}
                disabledDate={(moment) => this.setDisableDate('acrossEnrollEndTime', moment)}
                size="large"
                style={{ width: '270px' }}
                value={acrossEnrollEndTime}
                format={DATEFORMAT}
                placeholder="截止时间"
                onChange={(acrossEnrollEndTime) => this.handleDateChange(acrossEnrollEndTime, AcrossEnrollEndTime)}
              />
            </div>
          </div>
          {acrossDiscountIds.length > 0 && (
            <div className={commonStyles['col-right']}>
              <TextLink
                onClick={() =>
                  this.onTogglePreferential(acrossDiscountIds, ORIGINMODE, acrossEnrollStartTime, acrossEnrollEndTime)
                }
              >
                {'优惠（' + acrossDiscountIds.length + '）'}
              </TextLink>
            </div>
          )}
        </div>
      </div>
    );
  };

  render() {
    const {
      checking,
      loading,
      blCampusId,
      branchId,
      productTypeId,
      quarterId,
      yearId,
      branchDict,
      yearDict,
      campusDict,
      gradeId,
      productTypeDict,
      quarterDict,
      studentGradeDict,
      characterDict,
      productCharacterId,
    } = this.state;
    const { editMode, editId } = this.props;
    return (
      <Modal
        title={this.getTitle()}
        loading={loading}
        okBtnShow={editMode !== 'detail'}
        okText="保存"
        closable
        cancelText={editMode === 'detail' ? '关闭' : '取消'}
        onOk={this.handleOk}
        onCancel={this.props.onCancel}
        className={commonStyles.detailModal}
        ref={(c) => (this.modal = getWrappedInstance(c))}
        width={900}
      >
        <Block title="基础信息">
          <Row>
            <Col span={11}>
              <CustomFormItem name="分公司:" labelWidth={110} required>
                <Select
                  value={branchId}
                  dict={branchDict}
                  // disabled={!!editId}
                  allowClear={false}
                  onChange={this.onBranchChange}
                  showSearch={true}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('branchId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
            <Col span={11} offset={2}>
              <CustomFormItem name="学习中心:" labelWidth={110} required>
                <MultiSelect
                  value={blCampusId}
                  dict={branchId ? campusDict.filter((branch) => branch.val === branchId) : campusTempDict}
                  // disabled={!!editId}
                  allowClear={false}
                  onChange={(blCampusId) => this.setState({ blCampusId })}
                  showSearch={true}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('blCampusId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
          </Row>
          <Row>
            <Col span={11}>
              <CustomFormItem name="产品类型:" labelWidth={110} required>
                <Select
                  value={productTypeId}
                  dict={productTypeDict}
                  // disabled={!!editId}
                  allowClear={false}
                  onChange={(productTypeId) => this.setState({ productTypeId })}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('productTypeId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
            <Col span={11} offset={2}>
              <CustomFormItem name="年份:" labelWidth={110} required>
                <Select
                  value={yearId}
                  dict={yearDict}
                  // disabled={!!editId}
                  allowClear={false}
                  showSearch={true}
                  onChange={(yearId) => this.setState({ yearId })}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('yearId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
          </Row>
          <Row>
            <Col span={11}>
              <CustomFormItem name="季度:" labelWidth={110} required>
                <MultiSelect
                  value={quarterId}
                  dict={quarterDict}
                  // disabled={!!editId}
                  allowClear={false}
                  onChange={(quarterId) => this.setState({ quarterId })}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('quarterId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
            <Col span={11} offset={2}>
              <CustomFormItem name="年级:" labelWidth={110} required>
                <MultiSelect
                  value={gradeId}
                  dict={studentGradeDict}
                  // disabled={!!editId}
                  allowClear={true}
                  showSearch={true}
                  onChange={(gradeId) => this.setState({ gradeId })}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('gradeId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
          </Row>
          <Row>
            <Col span={11}>
              <CustomFormItem name="产品性质:" labelWidth={110} required>
                <MultiSelect
                  dict={characterDict}
                  value={productCharacterId}
                  showSearch={false}
                  onChange={(productCharacterId) => this.setState({ productCharacterId })}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('productCharacterId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
          </Row>
        </Block>

        <Block title="自由报读周期" style={{ marginTop: '30px' }}>
          {this.renderDatePicker()}
        </Block>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    organizationType: state.account.user.organizationType,
    userBrenchId: state.account.user.brenchId,
    userCampusId: state.account.user.campusId,
  };
};

export default connect(mapStateToProps, null, null, { withRef: true })(EditModal);
