import React, { Component } from 'react';
import Modal from 'COMPONENT/Modal';
import { connect } from 'dva';
import { Row, Col, Radio, DatePicker } from 'antd';
import List from 'COMPONENT/List';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import Validator from 'COMPONENT/Validator';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Block from 'COMPONENT/Block';
import moment from 'moment';
import { loadCommonDict, loadBranchDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import Loaders from '../loader';
import commonStyles from '../../common/commonStyle.less';
import { PROJECT_NAME } from '../../../../constants/config';
import styles from './style.less';
const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;
import { dateToMoment } from 'COMPONENT/utils/form';

const DATEFORMAT = 'YYYY-MM-DD HH:mm:ss'; //日期格式
const productTypeMap = {
  TWO_TEACHER: 'TWO_TEACHER',
  SMALL_CLASS: 'SMALL_CLASS',
};
const { RangePicker } = DatePicker;

// 列表默认的两个数据（第一段和第二段）
const defaultRows = [
  { index: '第一段收费', firstPaymentHours: 1, acrossEnrollStartTime: null, acrossEnrollEndTime: null },
  { index: '第二段收费', originalEnrollEndTime: null, originalEnrollStartTime: null },
];

/**
 * 去掉padding的FormItem
 * @param props
 * @returns {*}
 * @constructor
 */
function CustomFormItem(props) {
  const { children, style, ...lastProps } = props;
  return (
    <FormItem style={{ padding: 0, ...style }} {...lastProps}>
      {children}
    </FormItem>
  );
}

/**
 * 数组转换为字符串，如果入参为空则返回undefinded，否则拼接为字符串
 */
function arrToStr(arr) {
  if (arr && arr.length > 0) {
    return arr.join(',');
  } else {
    return undefined;
  }
}

/**
 * 字符串转换数组,如果入参为空则返回[]，否则转换为数组
 */
function strToArr(str) {
  if (str) {
    return str.split(',');
  } else {
    return [];
  }
}

const getValByText = (dict, text) => {
  const item = dict.find((item) => {
    return item.text === text;
  });
  if (item) {
    return item.val;
  } else {
    return null;
  }
};

/**
 * 日期格式化，如果入参为空，则返回undefined，否则格式化为
 */
function formatDate(date) {
  if (date) {
    return date.format(DATEFORMAT);
  } else {
    return undefined;
  }
}

class EditModal extends Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      // 第一段时间的结束日期 用于禁选第二段的其实日期
      disabledEndDate: null,
      // 第二段时间的其实日期 用于第一段时间禁选
      disabledStartDate: null,
      //表单数据
      branchId: null,
      name: '',
      productTypeId: PROJECT_NAME === 'twoteacher' ? productTypeMap.TWO_TEACHER : productTypeMap.SMALL_CLASS,
      quarterId: null,
      yearId: null,
      isContinue: true,
      rows: defaultRows,
      checking: false, //是否显示校验提示
      //数据字典
      branchDict: null,
      yearDict: null,
      productTypeDict: [
        { text: '双师', val: 'TWO_TEACHER' },
        { text: '小班', val: 'SMALL_CLASS' },
      ],
      quarterDict: [
        { text: '春季', val: 'DAT0000000231' },
        { text: '秋季', val: 'DAT0000000233' },
      ],
      gradeDict: [],
    };
    this.errors = {}; //校验错误信息
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化数据
   */
  initContent = async () => {
    const { editRecord } = this.props;
    const asyncCalls = [loadBranchDict(), loadCommonDict('PRODUCT_VERSION')];
    try {
      const [branchDict, yearDict] = await Promise.all(asyncCalls);
      /**
       * 字典数据
       */
      // 产品性质默认全选中
      this.setState({
        branchDict,
        yearDict,
      });

      if (!editRecord) {
        const defaultYear = getValByText(yearDict, new Date().getFullYear().toString()) || null;

        this.setState({
          loading: false,
          yearId: defaultYear,
        });
      } else {
        const { productTypeId, isContinue, quarterId, yearId, branchId, periods, name } = editRecord;
        const [firstSection, secondSection] = periods;
        const rows = [
          {
            ...firstSection,
            index: '第一段收费',
            firstPaymentHours: firstSection.hours,
            acrossEnrollStartTime: moment(firstSection.chargeBeginTime),
            acrossEnrollEndTime: moment(firstSection.chargeEndTime),
          },
          {
            ...secondSection,
            index: '第二段收费',
            originalEnrollEndTime: moment(secondSection.chargeEndTime),
            originalEnrollStartTime: moment(secondSection.chargeBeginTime),
          },
        ];
        /**
         * 详情数据
         */
        this.setState({
          loading: false,
          name,
          productTypeId: productTypeId,
          isContinue: !!isContinue,
          quarterId: quarterId,
          yearId: yearId,
          branchId: branchId,
          rows,
          disabledEndDate: moment(firstSection.chargeEndTime),
          disabledStartDate: moment(secondSection.chargeEndTime),
        });
      }
    } catch (e) {
      console.log(e);
      showMessage('加载数据错误', 'error');
    }
  };

  /**
   * 保存
   */
  saveDetail = async (params) => {
    const { editRecord } = this.props;
    if (editRecord) {
      return await Loaders.edit(params);
    } else {
      return await Loaders.add(params);
    }
  };

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  /**
   *  更新列表数据
   * @param index
   * @param changed
   */
  updateRow = (index, changed) => {
    this.setState({ listLoading: true });
    const rows = this.state.rows.map((row, i) => {
      if (i === index) {
        return { ...row, ...changed };
      }
      return row;
    });
    this.setState({
      listLoading: false,
      rows,
    });
  };

  /**
   *更新数据
   */
  handleChangeSelect = (key, id) => {
    this.setState({
      [key]: id,
    });
  };

  /**
   * 保存校验信息
   */
  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  /**
   * 获取模态框标题
   */
  getTitle = () => {
    const { editRecord, editMode } = this.props;
    if (!editRecord) {
      return '新增分段收费';
    } else {
      if (editMode === 'detail') {
        return '分段收费详情';
      } else {
        return '编辑分段收费';
      }
    }
  };

  // 更新日期范围
  updateDateRange(moments, index) {
    const [startDate, endDate] = moments;
    const updateRow =
      index === 0
        ? { acrossEnrollStartTime: startDate, acrossEnrollEndTime: endDate }
        : { originalEnrollStartTime: startDate, originalEnrollEndTime: endDate };
    this.updateRow(index, updateRow);
    if (index === 0) {
      this.setState({
        disabledEndDate: endDate,
      });
    } else {
      this.setState({
        disabledStartDate: startDate,
      });
    }
  }
  /**
   * 确认按钮
   */
  handleOk = async () => {
    const { branchId, name, checking, productTypeId, quarterId, yearId, isContinue, rows } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    let errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      this.modal.showOkTips(errorTips);
      return;
    }
    const { editRecord } = this.props;
    this.setState({ loading: true });
    const sectionChargePeriodVoList = rows.map((sectionTime, index) => {
      const {
        chargeNo,
        firstPaymentHours,
        id,
        acrossEnrollEndTime,
        acrossEnrollStartTime,
        originalEnrollEndTime,
        originalEnrollStartTime,
      } = sectionTime;
      const chargeBeginTime = acrossEnrollStartTime
        ? formatDate(acrossEnrollStartTime)
        : formatDate(originalEnrollStartTime);
      const chargeEndTime = acrossEnrollEndTime ? formatDate(acrossEnrollEndTime) : formatDate(originalEnrollEndTime);
      const currentChargeNo = chargeNo || index + 1;
      return {
        productTypeId,
        chargeNo: currentChargeNo,
        hours: firstPaymentHours,
        id,
        chargeBeginTime,
        chargeEndTime,
      };
    });
    const requestJson = {
      id: editRecord ? editRecord.id : undefined,
      name,
      productTypeId,
      quarterId,
      yearId,
      branchId: branchId,
      isContinue: isContinue ? 1 : 0,
      sectionChargePeriodVoList,
    };
    this.saveDetail(requestJson).then(
      () => {
        this.setState({ loading: false });
        //保存后的回掉，关闭模态框,更新列表等
        this.props.onOk();
        showMessage('保存成功', 'success');
      },
      (e) => {
        this.setState({ loading: false });
        if (e.text) {
          this.modal.showOkTips(e.text);
        } else {
          showMessage('保存失败');
        }
      },
    );
  };

  /**
   * 获取要禁用的日期值
   * @param  {string} value 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  getDisabledDate = (value, index) => {
    const { disabledEndDate, disabledStartDate } = this.state;
    if (index === 0) {
      // 禁选第一段时间日期
      if (!value || !disabledStartDate) {
        return false;
      }
      return value.valueOf() >= disabledStartDate.valueOf();
    } else {
      // 禁选第二段
      if (!value || !disabledEndDate) {
        return false;
      }
      return value.valueOf() <= disabledEndDate.valueOf();
    }
  };

  columns = [
    { title: '分段', dataIndex: 'index', width: 80 },
    {
      title: '收费课时数',
      dataIndex: 'paymentHours',
      width: 200,
      render: (text, record, index) => {
        const { firstPaymentHours = 1 } = record;
        return index === 1 ? (
          '产品总课时数-第一段收费课时数'
        ) : (
          <InputNumber
            disabled={this.props.editMode === 'detail'}
            placeholder="请输入课时数"
            value={firstPaymentHours}
            onChange={(val) => this.updateRow(index, { firstPaymentHours: val })}
            onBlur={(e) => this.updateRow(index, { firstPaymentHours: Math.floor(e.target.value) || 1 })}
            min={1}
            max={1000}
            step={1}
          />
        );
      },
    },
    {
      title: '收费时间',
      dataIndex: 'paymentTime',
      width: 400,
      render: (text, record, index) => {
        const { acrossEnrollStartTime, acrossEnrollEndTime, originalEnrollEndTime, originalEnrollStartTime } = record;
        // 等于0的话就是第一阶段收费时间
        const momentRange =
          index === 0 ? [acrossEnrollStartTime, acrossEnrollEndTime] : [originalEnrollStartTime, originalEnrollEndTime];
        const errorType = index === 0 ? 'firstRangeType' : 'secondRangeType';
        const isCheck = Boolean(momentRange[0] && momentRange[1]);
        const checking = this.state.checking;
        return (
          <Validator
            value={isCheck}
            rules={['required']}
            silent={!checking}
            showStatus={!isCheck}
            showTips={false}
            onErrorChange={(val) => {
              this.errorChange(errorType, val);
            }}
          >
            <RangePicker
              disabled={this.props.editMode === 'detail'}
              value={momentRange}
              onChange={(moments) => this.updateDateRange(moments, index)}
              format="YYYY-MM-DD HH:mm"
              showTime={{
                hideDisabledOptions: true,
                defaultValue: [moment('00:00:00', 'HH:mm:ss'), moment('23:59:59', 'HH:mm:ss')],
              }}
              placeholder={['开始时间', '结束时间']}
              disabledDate={(val) => this.getDisabledDate(val, index)}
            />
          </Validator>
        );
      },
    },
  ];

  render() {
    const {
      checking,
      loading,
      branchId,
      name,
      productTypeId,
      quarterId,
      isContinue,
      rows,
      yearId,
      branchDict,
      yearDict,
      productTypeDict,
      quarterDict,
    } = this.state;
    const { editMode, editRecord } = this.props;
    const isDetail = editMode === 'detail';
    return (
      <Modal
        title={this.getTitle()}
        loading={loading}
        okBtnShow={!isDetail}
        okText="保存"
        closable
        cancelText={isDetail ? '关闭' : '取消'}
        onOk={this.handleOk}
        onCancel={this.props.onCancel}
        className={commonStyles.detailModal}
        ref={(c) => (this.modal = getWrappedInstance(c))}
        width={900}
      >
        <Block title="基础信息">
          <Row>
            <Col span={24}>
              <CustomFormItem required name="分段收费名称:" labelWidth={110}>
                <Input
                  disabled={isDetail}
                  value={name}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('name', e),
                  }}
                  onChange={(e) => this.setState({ name: e.target.value })}
                  placeholder="请输入分段收费名称"
                />
              </CustomFormItem>
            </Col>
          </Row>
          <Row>
            <Col span={11}>
              <CustomFormItem name="分公司:" labelWidth={110} required>
                <Select
                  disabled={isDetail}
                  value={branchId}
                  dict={branchDict}
                  allowClear={false}
                  onChange={(branchId) => this.handleChangeSelect('branchId', branchId)}
                  showSearch={true}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('branchId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
            <Col span={11} offset={2}>
              <CustomFormItem name="产品类型:" labelWidth={110} required>
                <Select
                  disabled={isDetail}
                  value={productTypeId}
                  dict={productTypeDict}
                  allowClear={false}
                  onChange={(productTypeId) => this.handleChangeSelect('productTypeId', productTypeId)}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('productTypeId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
          </Row>
          <Row>
            <Col span={11}>
              <CustomFormItem name="产品年份:" labelWidth={110} required>
                <Select
                  disabled={isDetail}
                  value={yearId}
                  dict={yearDict}
                  allowClear={false}
                  showSearch={true}
                  onChange={(yearId) => this.handleChangeSelect('yearId', yearId)}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('yearId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
            <Col span={11} offset={2}>
              <CustomFormItem name="季度:" labelWidth={110} required>
                <Select
                  disabled={isDetail}
                  value={quarterId}
                  dict={quarterDict}
                  allowClear={false}
                  onChange={(quarterId) => this.handleChangeSelect('quarterId', quarterId)}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('quarterId', e),
                  }}
                />
              </CustomFormItem>
            </Col>
          </Row>
          <Row>
            <CustomFormItem name="是否连报" labelWidth={110} required>
              <Radio disabled={isDetail} checked={isContinue} onChange={() => this.setState({ isContinue: true })}>
                是
              </Radio>
              <Radio
                disabled={isDetail}
                checked={!isContinue}
                onChange={() => this.setState({ isContinue: false })}
                style={{ marginLeft: 50 }}
              >
                否
              </Radio>
            </CustomFormItem>
          </Row>
        </Block>

        <Block
          title={
            <div className={styles.header}>
              分段收费<span className={styles.notes}>（不限制则代表必须收取全款）</span>
            </div>
          }
          style={{ marginTop: '30px' }}
        >
          <List columns={this.columns} dataSource={rows} />
        </Block>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    monitoringUserId: state.account.user.userId,
  };
};

export default connect(mapStateToProps, null, null, { withRef: true })(EditModal);
