import React from 'react';
import { connect } from 'dva';
import { Page, List, SearchBar, Popover, Icon, Input } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import EditCityModal from './modal/EditCityModal';
import loader from './loader';
import { SYSTEM_MARK } from 'CONST/config';
const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const sys = SYSTEM_MARK;
class SightUpCustomer extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // boolean 页面加载状态
      //查询条件
      name: null,
      studentName: null,
      editCityModal: false,
      selectedRecord: {},
      //列表数据
      listLoading: false,
      rows: [],
      pageNo: 1,
      pageSize: 10,
      total: null,
    };
  }

  componentDidMount() {
    this.initContent();
  }

  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  loadListData = async (params = {}) => {
    const { name, studentName, pageNo, pageSize } = this.state;
    const apiParams = {
      orgSysType: sys,
      mobileStudentName: studentName,
      bossStudentName: studentName,
      pageNo,
      pageSize,
      ...params,
    };
    if (name) {
      if (name.length === 11) {
        apiParams.phone = name;
      } else {
        apiParams.customerName = name;
      }
    }
    try {
      await this.updateState({ listLoading: true });
      const list = await loader.loadStudentList(apiParams);
      this.updateState({
        listLoading: false,
        rows: list.records,
        total: list.total,
        pageNo: apiParams.pageNo,
        pageSize: apiParams.pageSize,
      });
    } catch (e) {
      showMessage(e.text);
      return [];
    }
  };

  initContent = async () => {
    //获取列表数据
    return this.loadListData();
  };

  search = async () => {
    return this.loadListData({
      pageNo: 1,
    });
  };

  reset = async () => {
    this.updateState({
      name: null,
      studentName: null,
    });
  };

  pageChange = async ({ current, pageSize }) => {
    return this.loadListData({ pageNo: current, pageSize });
  };

  /**
   * 修改学生城市
   * @param branchId 分公司id
   * @param text 城市名称
   * @param val城市id
   * @param loginMarkId
   */
  onEditCity = async ({ branchId, text, val, loginMarkId }) => {
    try {
      await loader.editStdCity({
        branchId,
        cityId: val,
        cityName: text,
        id: loginMarkId,
        orgSysType: sys,
      });
      this.setState({
        editCityModal: false,
        selectedRecord: {},
      });
      return this.search();
    } catch (e) {
      showMessage(e.text, 'error');
    }
  };
  /**
   * 表头
   */
  columns = [
    {
      title: '序号',
      dataIndex: 'index',
      width: 70,
      fixed: 'left',
      render: (text, record, index) => index + 1,
    },
    { title: '客户姓名', dataIndex: 'customerName', width: COL_WIDTH_L },
    { title: '客户手机号码', dataIndex: 'phone', width: COL_WIDTH_M },
    {
      title: '报读城市',
      dataIndex: 'cityName',
      width: 90,
      render: (text, record) => (record.city || {}).name,
    },
    {
      title: '学生boss名称',
      dataIndex: 'bossStudentName',
      width: COL_WIDTH_M,
      filterDropdownVisible: false,
      filterIcon: (
        <Popover overlayClassName="sectionPayment-list-popover" content="在boss中学生的名字">
          <Icon type="question_circle" size={12} style={{ marginLeft: 4 }} />
        </Popover>
      ),
      filterDropdown: <span className="empty-content"> </span>,
    },
    {
      title: '学生移动端名称',
      dataIndex: 'xiaoStudentName',
      width: 150,
      filterDropdownVisible: false,
      filterIcon: (
        <Popover overlayClassName="sectionPayment-list-popover" content="在app、报读H5中家长填写的名字，可由家长编辑">
          <Icon type="question_circle" size={12} style={{ marginLeft: 4 }} />
        </Popover>
      ),
      filterDropdown: <span className="empty-content"> </span>,
    },
    {
      title: '学籍学习中心',
      dataIndex: 'blCampusName',
      width: 90,
      render: (text, record) => (record.blCampus || {}).name,
    },
    {
      title: '操作',
      key: 'action',
      width: COL_WIDTH_XL,
      fixed: 'right',
      getActionConfig: (text, record) => {
        return [
          {
            text: '修改报读城市',
            onClick: () => {
              this.setState({ editCityModal: true, selectedRecord: record });
            },
          },
        ];
      },
    },
  ];

  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      rows,
      pageNo,
      pageSize,
      total,
      name,
      studentName,
      editCityModal,
      selectedRecord,
    } = this.state;
    const { route } = this.props;
    return (
      <Page
        loading={loading}
        route={route}
        fitWindowHeight={true}
        subHeader={
          <div style={{ paddingRight: '29px' }}>
            <SearchBar
              childWidth={[1, 1]}
              onSearch={this.search}
              onReset={this.reset}
              expand={searchExpand}
              onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
            >
              <Input
                placeholder="客户手机号码/客户名称"
                value={name}
                onChange={(e) =>
                  this.setState({
                    name: e.target.value.replace(/(^\s*)|(\s*$)/g, ''),
                  })
                }
              />
              <Input
                placeholder="学生boss名称/学生移动端名称"
                value={studentName}
                onChange={(e) =>
                  this.setState({
                    studentName: e.target.value.replace(/(^\s*)|(\s*$)/g, ''),
                  })
                }
              />
            </SearchBar>
          </div>
        }
      >
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows || []}
          pagination={{
            current: pageNo,
            pageSize,
            total: total,
          }}
          onChange={this.pageChange}
        />
        {editCityModal && (
          <EditCityModal
            studentData={selectedRecord}
            onConfirm={this.onEditCity}
            onClose={() => this.setState({ editCityModal: false, selectedRecord: {} })}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
    btnAuth: state.account.btnAuths || [],
  };
};
export default connect(mapStateToProps)(SightUpCustomer);
