import React from 'react';
import PropTypes from 'prop-types';
import styles from './styles.module.less';
import { Form, Input, Button, Radio } from 'COMPONENT';
import Select from '../components/Select';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { runFunc, errorsHandle } from 'UTIL/common';
import { throttle } from 'lodash';
import { addCustomerStudent } from 'SERVICE/student';
import { getAutoCompleteByCustomer, findCustomerById } from 'SERVICE/customer';

const FormItem = Form.Item;
const RadioGroup = Radio.Group;
const errorsHandler = errorsHandle();

class AddRelationForm extends React.Component {
  static propTypes = {
    studentId: PropTypes.string.isRequired,
    setLoading: PropTypes.func,
    onAdd: PropTypes.func,
  };

  constructor(props) {
    super(props);

    this.fetchCustomer = throttle(this.fetchCustomer.bind(this), 1000, { leading: false });
    this.handleFieldsChange = this.handleFieldsChange.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
    this.submit = this.submit.bind(this);

    this.state = {
      customerDict: [],
      customerIdStorage: null,
    };

    this.formRef = null;
    this.selectRef = null;
  }

  async fetchCustomer(val) {
    let dict = [];
    if (val) {
      dict = await getAutoCompleteByCustomer({ term: val }).catch(errorsHandler);
    }
    if (Array.isArray(dict)) {
      this.setState((prevState) => {
        if (prevState.customerIdStorage) {
          if (dict.findIndex((item) => item.value === prevState.customerIdStorage) === -1) {
            const tagIndex = prevState.customerDict.findIndex((item) => item.value === prevState.customerIdStorage);
            if (tagIndex > -1) {
              dict.unshift(prevState.customerDict[tagIndex]);
            }
          }
        }
        return { customerDict: dict };
      });
    }
  }

  handleFieldsChange(fields) {
    if (fields.customerId) {
      this.setState({
        customerIdStorage: fields.customerId.value || null,
      });
    }
  }

  handleSubmit() {
    runFunc(this.formRef.submitFields);
  }

  async submit(values) {
    runFunc(this.props.setLoading, true);
    const apiParams = {
      ...values,
      studentId: this.props.studentId,
    };
    const res = await addCustomerStudent(apiParams).catch(errorsHandler);
    if (res && res.resultCode === 0) {
      showMessage('提交成功！', 'success');
      let data = await this.getCustomerById(apiParams.customerId);
      data.id = res.data;
      this.setState({
        customerDict: [],
        customerIdStorage: null,
      });
      runFunc(this.formRef.resetFields);
      runFunc(this.selectRef.refresh);
      runFunc(this.props.onAdd, data);
    }
    runFunc(this.props.setLoading, false);
  }

  submitFail = () => {
    showMessage('请完善好所有信息后再提交！', 'error');
  };

  getCustomerById = async (id) => {
    let data = {};
    const res = await findCustomerById({ id }).catch(errorsHandler);
    if (res) {
      data.customerName = res.name;
      data.phoneNumber = res.contact;
    }
    return data;
  };

  render() {
    return (
      <Form
        className={styles.formWrap}
        ref={(c) => (this.formRef = c)}
        onSubmit={this.submit}
        onSubmitFail={this.submitFail}
        onFieldsChange={this.handleFieldsChange}
        style={{ width: 440 }}
      >
        <FormItem
          wrapClassName={styles.formItem}
          name="customerId"
          rules={[{ required: true, message: '客户信息不能为空' }]}
          formItemProps={{
            label: '客户信息',
            colon: false,
            labelCol: { span: 7 },
            wrapperCol: { span: 17 },
          }}
        >
          <Select
            ref={(c) => (this.selectRef = c)}
            placeholder="客户关键字"
            onSearch={this.fetchCustomer}
            dict={this.state.customerDict}
          />
        </FormItem>

        <FormItem
          wrapClassName={styles.formItem}
          name="relation"
          rules={[{ required: true, message: '关系不能为空' }]}
          formItemProps={{
            label: '关系',
            colon: false,
            labelCol: { span: 7 },
            wrapperCol: { span: 17 },
          }}
        >
          <Input placeholder="客户与学生关系(最多6个字)" />
        </FormItem>

        <FormItem
          wrapClassName={styles.formItem}
          name="confirmCustomer"
          initialValue="1"
          formItemProps={{
            label: '设置为信息确认家长',
            colon: false,
            labelCol: { span: 15 },
            wrapperCol: { span: 9 },
          }}
        >
          <RadioGroup>
            <Radio value="1">是</Radio>
            <Radio value="0">否</Radio>
          </RadioGroup>
        </FormItem>

        <Button className={styles.submitBtn} onClick={this.handleSubmit} type="primary">
          提交
        </Button>
      </Form>
    );
  }
}

export default AddRelationForm;
