import React from 'react';
import PropTypes from 'prop-types';
import { List, ListAction, PopConfirm } from 'COMPONENT';
import { Button, Input } from 'antd';
import { errorsHandle, runFunc } from 'UTIL/common';
import { getCustomerStudentList, deleteCustomerStudent, setRelation, setConfirmCustomer } from 'SERVICE/student';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const setWidth = List.createSetWidth();
const errorsHandler = errorsHandle();

class ParentList extends React.Component {
  static propTypes = {
    studentId: PropTypes.string.isRequired,
    onDelete: PropTypes.func,
    onModify: PropTypes.func,
  };

  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.loadData = this.loadData.bind(this);
    this.refresh = this.refresh.bind(this);
    this.onRelationChange = this.onRelationChange.bind(this);
    this.editRelation = this.editRelation.bind(this);
    this.saveRelation = this.saveRelation.bind(this);
    this.cancelEditRelation = this.cancelEditRelation.bind(this);
    this.deleteRecord = this.deleteRecord.bind(this);
    this.modifyDefault = this.modifyDefault.bind(this);

    this.state = {
      loading: false,
      list: [],
    };

    this.columns = [
      { title: '客户姓名', dataIndex: 'customerName', key: 'customerName', width: setWidth(6) },
      { title: '客户电话', dataIndex: 'customerContact', key: 'customerContact', width: setWidth(6) },
      {
        title: '关系',
        dataIndex: 'relation',
        key: 'relation',
        width: setWidth(8),
        render: (text, record) =>
          record._isEdit ? (
            <Input
              placeholder="客户与学生关系(最多6个字)"
              value={text}
              onChange={(e) => {
                this.onRelationChange(e.target.value, record);
              }}
              key={record.customerId}
            />
          ) : (
            <span>{text || ''}</span>
          ),
      },
      {
        title: '信息确认家长',
        dataIndex: 'confirmCustomer',
        key: 'confirmCustomer',
        width: setWidth(6),
        render: (text, record) => {
          return (
            <PopConfirm
              content={text ? '是否取消该家长作为信息确认家长？' : '是否设置该家长作为信息确认家长？'}
              onOk={() => {
                this.modifyDefault(text ? 0 : 1, record);
              }}
            >
              {text ? <Button type="default">已设置</Button> : <Button type="dashed">未设置</Button>}
            </PopConfirm>
          );
        },
      },
      {
        title: '操作',
        dataIndex: 'customerId',
        key: 'actionPlus',
        width: setWidth(6),
        render: (text, record) => (
          <ListAction
            config={
              record._isEdit
                ? [
                    {
                      text: '保存',
                      onClick: () => {
                        this.saveRelation(record);
                      },
                    },
                    {
                      text: '取消',
                      onClick: () => {
                        this.cancelEditRelation(record);
                      },
                    },
                  ]
                : [
                    {
                      text: '编辑',
                      onClick: () => {
                        this.editRelation(record);
                      },
                    },
                    {
                      text: '删除',
                      onClick: () => {
                        this.deleteRecord(record);
                      },
                      needConfirm: true,
                      confirmTips: '确定删除吗？',
                    },
                  ]
            }
          />
        ),
      },
    ];
  }

  componentDidMount() {
    this.init();
  }

  init() {
    if (this.props.studentId) {
      this.loadData();
    }
  }

  async loadData() {
    this.setState({ loading: true });
    let apiParams = {
      studentId: this.props.studentId,
    };
    const res = await getCustomerStudentList(apiParams).catch(errorsHandler);
    if (res && res.resultCode === 0) {
      this.setState({
        list: res.rows || [],
      });
    }
    this.setState({ loading: false });
  }

  refresh() {
    this.loadData();
  }

  onRelationChange(val, record) {
    // if (val.length > 6) {
    //   val = val.slice(0, 6);
    // }
    this.setState((state) => {
      const index = state.list.findIndex((item) => item.id === record.id);
      if (index > -1) {
        state.list[index].relation = val;
      }
      return state;
    });
  }

  editRelation(record) {
    this.setState((state) => {
      const index = state.list.findIndex((item) => item.id === record.id);
      if (index > -1) {
        if (state.list[index].relation) {
          state.list[index]._relationStorage = state.list[index].relation;
        }
        state.list[index]._isEdit = true;
      }
      return state;
    });
  }

  async saveRelation(record) {
    if (!record.relation) {
      return showMessage('关系不能为空！');
    }
    this.setState({ loading: true });
    const apiParams = {
      relationId: record.id,
      relation: record.relation,
    };
    const res = await setRelation(apiParams).catch(errorsHandler);
    if (res && res.resultCode === 0) {
      showMessage('保存成功！', 'success');
      this.setState((state) => {
        const index = state.list.findIndex((item) => item.id === record.id);
        if (index > -1) {
          delete state.list[index]._relationStorage;
          state.list[index]._isEdit = false;
        }
        return state;
      });
    }
    this.setState({ loading: false });
  }

  cancelEditRelation(record) {
    this.setState((state) => {
      const index = state.list.findIndex((item) => item.id === record.id);
      if (index > -1) {
        state.list[index].relation = state.list[index]._relationStorage || null;
        delete state.list[index]._relationStorage;
        state.list[index]._isEdit = false;
      }
      return state;
    });
  }

  async deleteRecord(record) {
    this.setState({ loading: true });
    const apiParams = {
      relationId: record.id,
    };
    const res = await deleteCustomerStudent(apiParams).catch(errorsHandler);
    if (res && res.resultCode === 0) {
      showMessage('删除成功！', 'success');
      this.loadData();
      runFunc(this.props.onDelete);
    } else {
      this.setState({ loading: false });
    }
  }

  async modifyDefault(val, record) {
    this.setState({ loading: true });
    const apiParams = {
      relationId: record.id,
      confirmCustomer: val,
    };
    const res = await setConfirmCustomer(apiParams).catch(errorsHandler);
    if (res && res.resultCode === 0) {
      showMessage('设置成功！', 'success');
      this.loadData();
      runFunc(this.props.onModify);
    } else {
      this.setState({ loading: false });
    }
  }

  render() {
    return <List loading={this.state.loading} columns={this.columns} dataSource={this.state.list} />;
  }
}

export default ParentList;
