import React, { Component } from 'react';
import { connect } from 'dva';
import { Row, Col, Switch, Checkbox } from 'antd';
import { FormItem, Select, Modal } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import Loaders from '../loader';
import styles from './styles.less';

class EditSightUpLimitModal extends Component {
  constructor(props) {
    super(props);
    const { branchDict = [] } = props.dict || {};
    const defaultBranchId = branchDict.length ? branchDict[0].val : null;
    this.state = {
      loading: false,
      classLevelCompatible: {},
      branchId: defaultBranchId,
      allBranchClassCompatibleData: {},
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化数据
   */
  initContent = async () => {
    const { branchId, allBranchClassCompatibleData } = this.state;
    let limitBranchDetail;

    this.setState({
      loading: true,
    });
    try {
      if (allBranchClassCompatibleData[branchId]) {
        limitBranchDetail = allBranchClassCompatibleData[branchId];
      } else {
        limitBranchDetail = await Loaders.getQualificationLimitData({ branchId });
      }

      // 初始化拿第一个分公司的数据就可以了
      const classLevelCompatible = this.getClassLevelCompatibleMap(limitBranchDetail);
      this.setState({
        loading: false,
        classLevelCompatible,
        allBranchClassCompatibleData: {
          ...allBranchClassCompatibleData,
          [branchId]: limitBranchDetail,
        },
      });
    } catch (e) {
      this.setState({
        loading: false,
      });
      showMessage(e.text, 'error');
    }
  };

  /**
   * 处理资格限制的数据
   * @param qualificationVos 资格限制数据
   * @returns {*} map
   */
  getClassLevelCompatibleMap = (qualificationVos) => {
    return qualificationVos.reduce((obj, cur) => {
      obj[cur.tagType] = {
        key: cur.tagType,
        checked: !!cur.valid,
        canCompatible: !!cur.compatible,
        id: cur.id,
      };
      return obj;
    }, {});
  };
  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  /**
   * 分公司改变获取对应的学习中心字典
   */
  handleBranchChange = async (branchId) => {
    await this.updateState({ branchId });
    this.initContent();
  };
  /**
   * 确认按钮
   */
  handleOk = async () => {
    const { branchId, classLevelCompatible } = this.state;

    const { onOk } = this.props;
    // 存在填写了的限制
    const isCompatibleValid = Object.values(classLevelCompatible).some((val) => val.checked);
    if (!branchId) {
      this.modal.showOkTips('请选择分公司');
      return false;
    }
    if (!isCompatibleValid) {
      this.modal.showOkTips('资格限制不能为空');
      return false;
    }
    this.setState({ loading: true });
    // 更新限制
    const qualificationVos = Object.keys(classLevelCompatible).map((compatibleKey) => {
      const { key, checked, canCompatible, id } = classLevelCompatible[compatibleKey];
      return {
        tagType: key,
        valid: checked ? 1 : 0,
        compatible: canCompatible ? 1 : 0,
        id,
        branchId,
      };
    });
    try {
      await Loaders.updateQualificationLimitData(qualificationVos);
      showMessage('保存成功', 'success');
      onOk && onOk();
    } catch (e) {
      showMessage('保存失败:' + e.text, 'error');
    } finally {
      this.setState({ loading: false });
    }
  };
  /**
   * 取消按钮
   */
  handleCancel = () => {
    this.props.onCancel();
  };

  //设置报读资格
  handleLimitVal = (params) => {
    const { classLevelCompatible } = this.state;
    const { key, type, val } = params;

    // 修改报读资格限制对象的值
    const newClassLevelCompatible = {
      ...classLevelCompatible,
      [key]: {
        ...classLevelCompatible[key],
        [type]: val,
      },
    };
    this.setState({
      classLevelCompatible: newClassLevelCompatible,
    });
  };

  render() {
    const { loading, branchId, classLevelCompatible } = this.state;
    const { branchDict, tagTypeDict } = this.props.dict || {};
    return (
      <Modal
        title="设置报读资格限制"
        loading={loading}
        onOk={this.handleOk}
        onCancel={this.handleCancel}
        ref={(c) => (this.modal = getWrappedInstance(c))}
        width={600}
      >
        <div>
          <Row>
            <Col span={24}>
              <FormItem name="分公司" labelWidth={90} required>
                <Select
                  value={branchId}
                  dict={branchDict}
                  allowClear={false}
                  onChange={this.handleBranchChange}
                  showSearch={true}
                />
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={24}>
              {Object.keys(classLevelCompatible).length ? (
                <FormItem name="资格限制" labelWidth={90} required>
                  {tagTypeDict.map((tag) => {
                    const { key, checked, canCompatible } = classLevelCompatible[tag.val];
                    return (
                      <div className={styles.limitWrapper} key={key}>
                        <Checkbox
                          checked={checked}
                          onChange={(e) => this.handleLimitVal({ key, type: 'checked', val: e.target.checked })}
                          className={styles.limitCheckbox}
                        >
                          {tag.text}
                        </Checkbox>
                        <span>允许班级层次向下兼容</span>
                        <Switch
                          disabled={!checked}
                          checked={canCompatible}
                          onChange={(compatibleVal) =>
                            this.handleLimitVal({ key, type: 'canCompatible', val: !!compatibleVal })
                          }
                        />
                      </div>
                    );
                  })}
                </FormItem>
              ) : null}
            </Col>
          </Row>
        </div>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { userId, brenchId } = state.account.user;
  return {
    monitoringUserId: userId,
    branchId: brenchId,
  };
};
export default connect(mapStateToProps)(EditSightUpLimitModal);
