import React from 'react';
import Page from 'COMPONENT/Page';
import styles from './styles.less';
import vars from 'COMPONENT/theme/variables';
import DatePicker from 'COMPONENT/DatePicker';
import GradeTemplate from './GradeTemplate';
import { loadTemplateDetail } from '../common';
import { saveFullAchievementTemplate, listTemplateEffectiveDateByCityId } from 'SERVICE/student';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';

class EditPage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      startDate: null, // {string} 起始生效日期
      checking: false, // {boolean} 校验开启状态
      existDates: [], // {string[]} 当前城市已存在的模板日期
    };
    this.errors = {}; //{boolean|string} 校验错误信息
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    const {
      record: { templateId, template, startDate },
    } = this.props;

    let changed = { loading: false };
    let calls = [this.loadExistDates()];

    if (templateId) {
      //带有templateId为编辑
      if (template) {
        changed = { ...changed, data: template, initData: template }; //前面的列表中已加载了模板数据，直接使用
      } else {
        calls.push(loadTemplateDetail(templateId));
      }
    } else {
      // 若没有id值则为新增模板，使用新的年级数据
      const { gradeDict } = this.props;
      const defaultData = gradeDict.map((g) => {
        return { gradeId: g.val, types: [], subjects: [] };
      });
      this.setState({
        data: defaultData, // {Array} 模板数据，结构如下：
        initData: defaultData, // {Array} 备份初始数据，用于判断是否修改
      });
    }
    this.setState({ loading: true });
    Promise.all(calls).then(([existDatesApi, data]) => {
      let existDates = existDatesApi;

      if (startDate) {
        changed = { ...changed, startDate };
        existDates = existDatesApi.filter((d) => d !== startDate);
      }
      changed = { ...changed, existDates };

      if (data) {
        changed = { ...changed, data, initData: data };
      }
      this.setState(changed);
    });
  };

  /**
   * 加载当前城市已有的模板日期
   */
  loadExistDates = async () => {
    const {
      record: { cityId },
    } = this.props;
    const apiData = await listTemplateEffectiveDateByCityId({ cityId });
    return apiData;
  };

  gradeDataChange = (gradeData) => {
    const { data } = this.state;
    this.setState(
      {
        data: data.map((d) => {
          if (d.gradeId === gradeData.gradeId) {
            return gradeData;
          } else {
            return d;
          }
        }),
      },
      this.check,
    );
  };

  back = () => {
    this.page.back();
  };

  save = () => {
    const { checking } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }

    if (Object.values(this.errors).find((v) => v)) {
      this.page.showOkTips('请正确填写信息');
      return;
    }

    const {
      onSaveSuccess,
      record: { cityId, templateId, version },
    } = this.props;
    const { data, startDate } = this.state;
    let saveData = {
      cityId,
      effectiveDate: startDate,
      grades: data.map((d) => {
        return {
          achievementTypeList: d.types.join(','),
          gradeId: d.gradeId,
          subjects: d.subjects.map((s) => {
            return {
              subjectId: s.id,
              totalScore: s.totalScore,
            };
          }),
        };
      }),
    };
    if (templateId) {
      saveData.id = templateId;
      saveData.version = version;
    } else {
      saveData.version = 0;
    }

    saveFullAchievementTemplate(saveData, true).then(
      () => {
        showMessage('保存成功', 'success');
        this.setState({ loading: false });
        onSaveSuccess();
      },
      (e) => {
        if (e.text) {
          this.page.showOkTips(e.text);
        } else {
          showMessage('保存失败');
        }
        this.setState({ loading: false });
      },
    );
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  getDisabledDate = (currentDate) => {
    if (this.state.existDates.indexOf(currentDate) >= 0) {
      return true;
    }
    return false;
  };

  render() {
    const {
      gradeDict,
      subjectDict,
      scoreTypeDict,
      record: { cityName },
    } = this.props;
    const { loading, data, startDate, checking, initData } = this.state;

    return (
      <Page
        loading={loading}
        route={this.props.route}
        footer={true}
        cancelBtnShow={true}
        cancelConfirm={
          objectDeepEquals(data, initData)
            ? null
            : { content: '取消保存本次修改？', okBtnText: '是', cancelBtnText: '否' }
        }
        onOk={this.save}
        onCancel={this.back}
        ref={(c) => (this.page = c)}
      >
        <div className={styles.wrap}>
          <div className={styles.header} style={{ background: vars.basicColor }}>
            <div className={styles.headerColFix}>城市</div>
            <div className={styles.colFlex}>起始生效日期</div>
          </div>
          <div className={styles.main}>
            <div className={styles.header}>
              <div className={styles.headerColFix}>{cityName}</div>
              <div style={{ width: 130 }}>
                <DatePicker
                  size="small"
                  value={startDate}
                  onChange={(val) => this.setState({ startDate: val }, this.check)}
                  validator={{
                    onErrorChange: (e) => this.errorChange('startDate', e),
                    silent: !checking,
                  }}
                  disabledDate={this.getDisabledDate}
                />
              </div>
            </div>
            <div className={styles.content}>
              {data &&
                data.map((d, i) => (
                  <GradeTemplate
                    key={d.gradeId}
                    gradeDict={gradeDict}
                    subjectDict={subjectDict}
                    scoreTypeDict={scoreTypeDict}
                    data={d}
                    onChange={this.gradeDataChange}
                    onScoreTypeErrorChange={(e) => this.errorChange('scoreType_' + d.gradeId, e)}
                    onSubjectsErrorChange={(e) => this.errorChange('subjects_' + d.gradeId, e)}
                    onTotalScoreErrorChange={(subjectId, e) =>
                      this.errorChange('totalScore_' + d.gradeId + '_' + subjectId, e)
                    }
                    checking={checking}
                  />
                ))}
            </div>
          </div>
        </div>
      </Page>
    );
  }
}

export default EditPage;
