/**
 * 学生类别设置模态框
 * @property {function} onClose 关闭弹框的回调
 */
import React from 'react';
import classnames from 'classnames';
import Modal from 'COMPONENT/Modal';
import InputNumber from 'COMPONENT/InputNumber';
import IconButton from 'COMPONENT/IconButton';
import DotButton from 'COMPONENT/DotButton';
import styles from './styles.less';
import { getDictTextByVal } from 'UTIL/dict';
import { listAllAchievementCategory, updateAchievementCategory } from 'SERVICE/student';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import vars from 'COMPONENT/theme/variables';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import { checkNumber } from 'COMPONENT/Validator/checkers';
import getWrappedInstance from 'UTIL/getWrappedInstance';

const defaultScoreRange = {
  1: { min: 90, max: 999 },
  2: { min: 70, max: 90 },
  3: { min: 50, max: 70 },
  4: { min: 40, max: 55 },
  5: { min: 0, max: 40 },
};

class StudentTypeModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 等待状态
      data: null, // {object[]} 类别设置数据，格式如下：
      /**
                  [
                    {
                      id: 'a', - 类别ID
                      level: 'a', - 评级
                      settingsByScore: [ - 按分数规则设置（多条）
                        {
                          total: 100, - 总分
                          min: 90, - 分数下限
                          max: 999 - 分数上限
                        }
                      ],
                      settingByClassRank: { - 按班级排名规则设置
                        min: 0, - 排名上限
                        max: 5  - 排名下限
                      },
                      settingByGradeRank: { - 按年级排名规则设置
                        min: 0, - 排名上限
                        max: 30 - 排名下限
                      },
                    }
                  ]
                  */
      initData: null, // {Array} 备份初始数据，用于判断是否修改
      checking: false, //校验开启状态
    };
    this.errors = {}; //校验错误
    this.rowIdx = 1; //用于生成唯一的行key
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    this.setState({ loading: true });
    Promise.all([this.loadData()]).then(([data]) => {
      this.setState({
        loading: false,
        data,
        initData: data,
      });
    });
  };

  /**
   * 加载设置数据
   */
  loadData = async () => {
    const apiData = await listAllAchievementCategory();
    const data = apiData.map((d) => {
      let fixedSetting;
      let otherSettings = [];
      d.scoreSet.forEach((s) => {
        const obj = {
          id: s.id,
          total: s.totalScore,
          min: s.scoreStart,
          max: s.scoreEnd,
          rowId: this.rowIdx++,
        };
        if (s.totalScore === 100) {
          fixedSetting = obj;
        } else {
          otherSettings.push(obj);
        }
      });
      otherSettings = otherSettings.sort((a, b) => a.total > b.total);

      return {
        id: d.id,
        name: d.name,
        level: d.evaluationList,
        settingsByScore: [fixedSetting, ...otherSettings],
        settingByClassRank: {
          min: d.classRankingStart,
          max: d.classRankingEnd,
          rowId: this.rowIdx++,
        },
        settingByGradeRank: {
          min: d.gradeRankingStart,
          max: d.gradeRankingEnd,
          rowId: this.rowIdx++,
        },
        version: d.version,
      };
    });
    return data;
  };

  onOk = () => {
    const { checking } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    if (Object.values(this.errors).find((v) => v)) {
      this.modal.showOkTips('请正确填写信息');
      return;
    }
    const { onClose } = this.props;
    const { data } = this.state;
    const saveData = {
      achievementCategorys: data.map((d) => {
        return {
          classRankingEnd: d.settingByClassRank.max,
          classRankingStart: d.settingByClassRank.min,
          evaluationList: d.level,
          gradeRankingEnd: d.settingByGradeRank.max,
          gradeRankingStart: d.settingByGradeRank.min,
          id: d.id,
          name: d.name,
          scoreSet: d.settingsByScore.map((s) => {
            let x = {
              scoreEnd: s.max,
              scoreStart: s.min,
              totalScore: s.total,
            };
            if (s.id) {
              x.id = s.id;
            }
            return x;
          }),
          version: d.version,
        };
      }),
    };
    //console.log('saveData:', saveData, JSON.stringify(saveData))
    updateAchievementCategory(saveData, true).then(
      () => {
        showMessage('保存成功', 'success');
        this.setState({ loading: false });
        onClose();
      },
      (e) => {
        if (e.text) {
          this.modal.showOkTips(e.text);
        } else {
          showMessage('保存失败');
        }
        this.setState({ loading: false });
      },
    );
  };

  updateTypeData = (typeId, changed) => {
    const { data } = this.state;
    const newData = data.map((d) => {
      if (d.id === typeId) {
        return { ...d, ...changed };
      }
      return d;
    });
    this.setState({ data: newData });
  };

  addSettingByScore = (typeId) => {
    const { data } = this.state;
    const newData = data.map((d) => {
      if (d.id === typeId) {
        return {
          ...d,
          settingsByScore: [...d.settingsByScore, { rowId: this.rowIdx++ }],
        };
      }
      return d;
    });
    this.setState({ data: newData });
  };

  removeSettingByScore = (typeId, index, rowId) => {
    let newErrors = {};
    Object.keys(this.errors).forEach((key) => {
      const a = key.split('_');
      if (typeId + '' !== a[0] || rowId + '' !== a[1]) {
        newErrors[key] = this.errors[key];
      }
    });
    this.errors = newErrors;
    const { data } = this.state;
    const newData = data.map((d) => {
      if (d.id === typeId) {
        return {
          ...d,
          settingsByScore: d.settingsByScore.filter((s, i) => i !== index),
        };
      }
      return d;
    });
    this.setState({ data: newData });
  };

  valueChange = (typeId, index, settingType, changed) => {
    const { data } = this.state;
    const newData = data.map((d) => {
      if (d.id === typeId) {
        if (settingType === 'score') {
          const newSettingsByScore = d.settingsByScore.map((s, i) => {
            if (i === index) {
              return { ...s, ...changed };
            }
            return s;
          });
          return { ...d, settingsByScore: newSettingsByScore };
        } else if (settingType === 'class') {
          return { ...d, settingByClassRank: { ...d.settingByClassRank, ...changed } };
        } else if (settingType === 'grade') {
          return { ...d, settingByGradeRank: { ...d.settingByGradeRank, ...changed } };
        }
      }
      return d;
    });
    this.setState({ data: newData });
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  render() {
    const { onClose, studentLevelDict } = this.props;
    const { loading, data, initData, checking } = this.state;
    return (
      <Modal
        className={styles.modalWrap}
        loading={loading}
        title={
          <span>
            <span>学生类别设置</span>
            <IconButton
              type="question_mc"
              title={{ title: '未单独设置规则的总分分制将统一换算成百分制', placement: 'right' }}
              style={{ marginLeft: 10 }}
            />
          </span>
        }
        width={860}
        onOk={this.onOk}
        onCancel={onClose}
        cancelConfirm={
          objectDeepEquals(data, initData)
            ? null
            : { content: '取消保存本次修改？', okBtnText: '是', cancelBtnText: '否' }
        }
        noLine
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        <div className={styles.wrap}>
          {data &&
            data.map((d, i) => {
              const rows = [
                ...d.settingsByScore.map((setting) => {
                  return { ...setting, label: '分数', type: 'score' };
                }),
                { ...d.settingByClassRank, label: '班级排名', type: 'class' },
                { ...d.settingByGradeRank, label: '年级排名', type: 'grade' },
              ];
              return (
                <div key={i} className={styles.type}>
                  <div className={styles.typeTitle}>{d.name}</div>
                  <div className={styles.setting}>
                    {rows.map((row, j) => {
                      const { rowId } = row;
                      return (
                        <div key={rowId} className={styles.row}>
                          <div className={styles.label}>{row.label}</div>
                          <div className={styles.total}>
                            {row.type === 'score' ? (
                              <span>
                                <span className={styles.label2}>总分</span>
                                {j === 0 ? (
                                  <span>{row.total}</span>
                                ) : (
                                  <div className={styles.input}>
                                    <InputNumber
                                      min={0}
                                      value={row.total}
                                      onChange={(val) => this.valueChange(d.id, j, row.type, { total: val })}
                                      validator={{
                                        onErrorChange: (e) => this.errorChange(d.id + '_' + rowId + '_total', e),
                                        silent: !checking,
                                        rules: ['required', (v) => checkNumber(v, { min: 0, max: 999 })],
                                      }}
                                    />
                                  </div>
                                )}
                              </span>
                            ) : (
                              '-'
                            )}
                          </div>
                          <div className={styles.min}>
                            <span className={styles.sign}>{row.type === 'score' ? `≥` : `>`}</span>
                            <div className={styles.input}>
                              <InputNumber
                                min={0}
                                value={row.min}
                                onChange={(val) => this.valueChange(d.id, j, row.type, { min: val })}
                                validator={{
                                  onErrorChange: (e) => this.errorChange(d.id + '_' + rowId + '_min', e),
                                  silent: !checking,
                                  rules: [
                                    'required',
                                    (v) =>
                                      checkNumber(v, {
                                        min: 0,
                                        max: row.type === 'grade' ? 9999 : 999,
                                        int: row.type === 'score' ? false : true,
                                      }),
                                  ],
                                }}
                              />
                            </div>
                          </div>
                          <div className={styles.max}>
                            <span className={styles.sign}>{row.type === 'score' ? `<` : `≤`}</span>
                            <div className={styles.input}>
                              <InputNumber
                                min={0}
                                value={row.max}
                                onChange={(val) => this.valueChange(d.id, j, row.type, { max: val })}
                                validator={{
                                  onErrorChange: (e) => this.errorChange(d.id + '_' + rowId + '_max', e),
                                  silent: !checking,
                                  rules: [
                                    'required',
                                    (v) =>
                                      checkNumber(v, {
                                        min: 0,
                                        max: row.type === 'grade' ? 9999 : 999,
                                        int: row.type === 'score' ? false : true,
                                      }),
                                  ],
                                }}
                              />
                            </div>
                          </div>
                          {row.type === 'score' && (
                            <div className={styles.btns}>
                              {j === d.settingsByScore.length - 1 && (
                                <IconButton
                                  type="plus_circle"
                                  color={vars.fontColor}
                                  size={20}
                                  className={styles.btn}
                                  onClick={() => this.addSettingByScore(d.id)}
                                />
                              )}
                              {j > 0 && (
                                <IconButton
                                  type="minus_circle"
                                  color={vars.fontColor}
                                  size={20}
                                  className={styles.btn}
                                  onClick={() => this.removeSettingByScore(d.id, j, rowId)}
                                />
                              )}
                            </div>
                          )}
                        </div>
                      );
                    })}
                    <div key="levelRow" className={styles.levelRow}>
                      <div className={styles.label}>评级</div>
                      <div className={styles.levels}>
                        {studentLevelDict.map((s, n) => {
                          const isActive = d.level === s.val;
                          return (
                            <DotButton
                              key={n}
                              size={36}
                              shape="square"
                              className={styles.levelBtn}
                              type={isActive ? 'primary' : null}
                              onClick={isActive ? null : () => this.updateTypeData(d.id, { level: s.val })}
                            >
                              {s.text}
                            </DotButton>
                          );
                        })}
                      </div>
                    </div>
                  </div>
                </div>
              );
            })}
        </div>
      </Modal>
    );
  }
}

export default StudentTypeModal;
