import React from 'react';
import { Radio, Button, Icon, Progress } from 'antd';
import { Modal, Upload, Loading } from 'COMPONENT';
import { importStudentAchievement } from 'SERVICE/scoreManage';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import moment from 'moment';
import classNames from 'classnames';
import downloadFile from 'COMPONENT/utils/file/downloadFile';

const TEMPLATE_URL = {
  GRADE:
    'https://xhboss-test.oss-cn-shenzhen.aliyuncs.com/template/%E5%AF%BC%E5%85%A5%E6%88%90%E7%BB%A9-%E7%AD%89%E7%BA%A7%E6%A8%A1%E6%9D%BF.xlsx',
  SCORE_RANKING:
    'https://xhboss-test.oss-cn-shenzhen.aliyuncs.com/template/%E5%AF%BC%E5%85%A5%E6%88%90%E7%BB%A9-%E5%88%86%E6%95%B0%E6%A8%A1%E6%9D%BF.xlsx',
};
const TEMPLATE_NAME = {
  GRADE: '导入成绩-等级模板.xlsx',
  SCORE_RANKING: '导入成绩-分数模板.xlsx',
};

import styles from './styles.module.less';
export default class ImportScoreModal extends React.Component {
  state = {
    type: null,
    percent: 0,
    isImporting: false,
    isLoading: false,
    currentFile: null,
    resultInfo: {},
    isResultModalVisible: false,
    isImportModalVisible: true,
  };

  constructor(props) {
    super(props);
    this.percent = 0;
    this.timer = null;
  }

  componentWillUnmount() {
    this.clearTimer();
  }

  clearTimer = () => {
    if (this.timer) {
      clearTimeout(this.timer);
      this.timer = null;
    }
  };

  onOk = async () => {
    const { type, currentFile } = this.state;
    if (!type) {
      return showMessage('请选择成绩形式', 'warning');
    }
    if (!currentFile?.fileKey) {
      return showMessage('请上传Excel导入学校成绩', 'warning');
    }
    this.clearTimer();
    this.timer = setInterval(() => {
      if (this.percent >= 90) {
        this.clearTimer();
      }
      this.percent += 10;
      this.setState((prevState) => ({ percent: prevState.percent + 10 }));
    }, 20);
    this.setState({
      isImporting: true,
      isImportModalVisible: false,
    });
    try {
      const res = await importStudentAchievement({
        achievementType: this.state.type,
        filePath: currentFile?.fileKey,
      });
      this.clearTimer();
      this.percent = 100;
      this.setState({
        percent: 100,
        resultInfo: res.data,
        isImporting: false,
        isResultModalVisible: true,
        isImportModalVisible: false,
      });
    } catch (e) {
      this.onCancel();
      this.setState({
        percent: 0,
        isImporting: false,
      });
      showMessage('批量导入成绩失败，请稍后重试');
    }
  };

  onDownloadFile = () => {
    if (!this.state.resultInfo?.resultFilePath) {
      return;
    }
    downloadFile(
      this.state.resultInfo?.resultFilePath,
      `批量导入成绩失败明细_${moment().format('YYYY-MM-DD_hh_mm_ss')}.xlsx`,
    );
  };

  onCancel = () => {
    this.clearTimer();
    this.props.onCancel();
  };

  onChangeType = (e) => {
    this.setState({ type: e.target.value });
  };

  onRemoveFile = () => {
    this.setState({ currentFile: null });
  };

  onUploadChange = (file) => {
    this.setState({ isLoading: true });
    if (file?.error === false) {
      this.setState({ isLoading: false });
      return showMessage('上传失败，请稍后重试', 'error');
    } else if (file?.src) {
      console.log(file);
      this.setState({ isLoading: false, currentFile: file });
    }
  };

  onBeforeUpload = (files) => {
    const [file] = files;
    this.setState({ currentFile: null });
    if (!['xlsx', 'xls'].includes(file?.name?.split(/\./)?.pop()?.toLowerCase())) {
      showMessage('只支持xls，xlsx文件类型进行导入', 'warning');
      return false;
    }
    if (file?.size > 10 * 1024 * 1024) {
      showMessage('文件超过10M，无法上传', 'warning');
      return false;
    }
    return files;
  };

  onDownloadTemplate = () => {
    downloadFile(TEMPLATE_URL[this.state.type], TEMPLATE_NAME[this.state.type]);
  };

  generateImportModal = () => {
    const props = {
      width: 494,
      onOk: this.onOk,
      cancelText: '取消',
      okText: '确认提交',
      title: '批量导入成绩',
      onCancel: this.onCancel,
    };
    return (
      this.state.isImportModalVisible && (
        <Modal {...props}>
          <div className={styles.wrapper}>
            <div className={styles.section}>
              <div className={styles.title}>第1步：选择考试成绩形式</div>
              <div className={styles.content}>
                <Radio.Group onChange={this.onChangeType}>
                  <div className={styles.radioGroup}>
                    <div
                      className={classNames({
                        [styles.radioGroupItemWrapper]: 1,
                        [styles.selected]: this.state.type === 'SCORE_RANKING',
                      })}
                    >
                      <div className={styles.radioGroupItem}>
                        <Radio value="SCORE_RANKING">
                          分数<span className={styles.radioTip}>（比如90、80、70）</span>
                        </Radio>
                      </div>
                    </div>
                    <div
                      className={classNames({
                        [styles.radioGroupItemWrapper]: 1,
                        [styles.selected]: this.state.type === 'GRADE',
                      })}
                    >
                      <div className={styles.radioGroupItem}>
                        <Radio value="GRADE">
                          等级<span className={styles.radioTip}>（比如A、B、C、D）</span>
                        </Radio>
                      </div>
                    </div>
                  </div>
                </Radio.Group>
              </div>
            </div>
            <div className={styles.section}>
              <div className={styles.title}>第2步：下载成绩模板并填写</div>
              <div className={styles.content}>
                <Button disabled={!this.state.type} onClick={this.onDownloadTemplate}>
                  下载填写模版
                </Button>
              </div>
            </div>
            <div className={styles.section}>
              <div className={styles.title}>第3步：上传Excel导入考试成绩</div>
              <div className={styles.tip}>提示：只支持xls，xlsx文件类型进行导入</div>
              <div className={styles.content}>
                <Upload
                  multiple={false}
                  hasExtension={true}
                  ignoreExpires={true}
                  onChange={this.onUploadChange}
                  beforeUpload={this.onBeforeUpload}
                  limit={{ type: 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' }}
                >
                  <Loading size="default" loading={this.state.isLoading} />
                  <div className={styles.uploadWrapper}>
                    <div>
                      <Icon type="plus" size={{ width: 25, height: 25, color: '#AAA' }} />
                    </div>
                    <div>上传成绩</div>
                  </div>
                </Upload>
                {this.state.currentFile?.name && (
                  <div className={styles.fileNameWrapper}>
                    <Icon type="paper-clip" />
                    <div className={styles.fileName}>{this.state.currentFile?.name}</div>
                    <Icon type="delete" onClick={this.onRemoveFile} />
                  </div>
                )}
              </div>
            </div>
          </div>
        </Modal>
      )
    );
  };

  generateResultModal = () => {
    const props = {
      width: 520,
      title: '导入结果',
      onCancel: this.onCancel,
      footer: (
        <div className={styles.resultModalFooter}>
          <Button size="large" type="primary" onClick={this.onCancel}>
            知道了
          </Button>
        </div>
      ),
    };
    return (
      this.state.isResultModalVisible && (
        <Modal {...props}>
          <div className={styles.resultContent}>
            <p>
              本次导入学生分数{this.state.resultInfo?.total}条，导入成功
              <span className={styles.resultSuccess}>{this.state.resultInfo?.success}</span>条 ，导入失败
              <span className={styles.resultFail}>{this.state.resultInfo?.fail}</span>条。{' '}
            </p>
            <p>具体失败原因请下载下方【结果excel】进行查看</p>
            <p>
              <a onClick={this.onDownloadFile}>点击下载：批量导入成绩失败明细</a>
            </p>
          </div>
        </Modal>
      )
    );
  };

  generateProgress = () => {
    return (
      this.state.isImporting && (
        <div className={styles.importProgressMask}>
          <div className={styles.importProgressWrapper}>
            <div className={styles.importProgressContent}>
              <Progress type="circle" percent={this.percent} width={80} />
              <div className={styles.importText}>导入中</div>
            </div>
          </div>
        </div>
      )
    );
  };

  render() {
    return (
      <div>
        {this.generateImportModal()}
        {this.generateResultModal()}
        {this.generateProgress()}
      </div>
    );
  }
}
