import React from 'react';
import { Row, Col, Tooltip, Icon as AntIcon } from 'antd';
import { List, Icon, Modal, Upload, Button, TextArea, Popover, Checkbox, PreviewImgs } from 'COMPONENT';
import _ from 'lodash';
import moment from 'moment';
import * as studentApiService from 'SERVICE/student';
import { errorsHandle } from 'SRC/utils/common';

import showMessage from 'SRC/components/utils/globalTips/showMessage';

import style from './styles.module.less';

const { Group: CheckboxGroup } = Checkbox;

// 添加学生标签相关组件
const StudentLabel = (props) => {
  const { id, name, onRemove } = props;
  return (
    <div className="student-label" key={id}>
      <label>{name}</label>
      <Icon size="small" color="#9B97FF" type="cross_circle" onClick={() => onRemove(id)}></Icon>
    </div>
  );
};
const AddLabelPopoverTitle = ({ onClose }) => {
  return (
    <div className="add-label-popover-title">
      <label>选择标签</label>
      <Icon size="small" type="cross" onClick={onClose}></Icon>
    </div>
  );
};
const AddLabelPopoverContent = ({ labelList = [], onAddLabel, onSelect }) => {
  return (
    <div>
      <CheckboxGroup onChange={onSelect}>
        {labelList.map(({ id, name }) => (
          <Checkbox value={id} key={id}>
            {name}
          </Checkbox>
        ))}
      </CheckboxGroup>
      <div className="add-label-button-wrapper">
        <Button type="primary" onClick={onAddLabel}>
          确定添加
        </Button>
      </div>
    </div>
  );
};

// 图片列表
const PictureList = ({ onUpload, onRemove, fileList, onBeforeUpload }) => {
  const uploadProps = {
    multiple: false,
    onChange: onUpload,
    beforeUpload: onBeforeUpload,
  };
  return (
    <div className="picture-list-wrapper">
      <ul className="picture-list">
        {fileList.map(({ id, src, fileName, showProgress }, index) => {
          return (
            <li key={id} style={{ marginRight: index === 6 ? 0 : 16 }} className="uploaded-picture">
              {showProgress ? <div className="uploading-picture-tip">上传中</div> : <img src={src} alt={fileName} />}
              <Icon type="delete" color="#FFF" onClick={() => onRemove(index)} />
              <div className="picture-mask"></div>
            </li>
          );
        })}
        {fileList.length < 9 && (
          <li className="upload-picture-btn-item">
            <div className="upload-picture-btn-wrapper">
              <Upload {...uploadProps}>
                <Icon type="plus" color="#333" />
                <p>上传图片</p>
              </Upload>
            </div>
            <p className="upload-picture-btn-tip">最多上传9张</p>
          </li>
        )}
      </ul>
    </div>
  );
};

const ViewPicturePopoverTitle = ({ id, onClose }) => {
  return (
    <div className="view-picture-popover-title">
      <label>查看图片</label>
      <Icon type="cross" size="small" onClick={() => onClose(id)}></Icon>
    </div>
  );
};
const ViewPicturePopoverContent = ({ id, onView, pictureList }) => {
  return (
    <ul className="view-picture-popover-content-wrapper">
      {(pictureList || []).map((picture, i) => {
        return (
          <li key={i} style={{ marginRight: (i + 1) % 3 ? 8 : 0 }}>
            <img src={picture} />
            <AntIcon type="search" style={{ color: '#FFF' }} onClick={() => onView(picture, pictureList, id)} />
            <div className="picture-mask"></div>
          </li>
        );
      })}
    </ul>
  );
};

export default class EditStudentLabelModal extends React.Component {
  state = {
    detail: {},
    list: [],
    labelList: [],
    popOverKey: 1,
    fileList: [],
    listLoading: false,
    modalLoading: false,
    selectedLabelList: [],
    previewImg: null,
    previewImgList: [],
    viewPicturePopOverKey: 1,
    addLabelPopVisible: false,
    previewImgVisible: false,
  };

  constructor(props) {
    super(props);
    this.columns = [
      {
        title: '操作时间',
        dataIndex: 'createTime',
        key: 'createTime',
        width: 140,
        render: (value) => moment(value).format('YYYY-MM-DD hh:mm:ss'),
      },
      { title: '操作人', dataIndex: 'createUserName', key: 'createUserName', width: 90 },
      {
        title: '学生标签',
        dataIndex: 'currentLabelList',
        key: 'currentLabelList',
        width: 160,
        render: this.generateCurrentLabel,
      },
      { title: '详细信息', dataIndex: 'detail', key: 'detail', width: 300, render: this.generateDescWrapper },
      { title: '操作', dataIndex: 'operation', key: 'operation', width: 100, render: this.generateOperation },
    ];
  }

  componentDidMount() {
    this.init();
  }

  init() {
    this.loadData();
    this.loadRecordList();
    this.loadLabelList();
  }

  async loadData() {
    const result = await studentApiService
      .getStudentLabel({
        studentId: this.props.studentId,
      })
      .catch(errorsHandle());

    if (!result || result.resultCode !== 0 || !result.data) {
      return this.setState({ detail: {} });
    }

    this.setState({ detail: { ...result.data } });
  }

  async loadRecordList() {
    this.setState({ listLoading: true });
    const result = await studentApiService
      .pageStudentLabelOperationLogList({
        pageNum: 1,
        pageSize: 999,
        studentId: this.props.studentId,
      })
      .catch(errorsHandle());
    this.setState({ listLoading: false });

    if (!result || result.resultCode !== 0 || !result.data) {
      return this.setState({ list: {} });
    }
    this.setState({
      list: result.data.list.map((item) => ({ ...item, key: item.id })),
    });
  }

  async loadLabelList() {
    const result = await studentApiService.getAllLabel().catch(errorsHandle());

    if (!result || result.resultCode !== 0 || !result.data) {
      return this.setState({ labelList: [] });
    }

    this.setState({ labelList: result.data });
  }

  get labelSelectList() {
    const {
      labelList,
      detail: { studentLabelList = [] },
    } = this.state;
    return studentLabelList.length
      ? labelList.filter(({ id }) => !studentLabelList.find((label) => label.id === id))
      : labelList;
  }

  handleSubmitMainModal = async () => {
    const {
      fileList,
      detail: { lastUpdateDetail = '', studentLabelList },
    } = this.state;
    const data = {
      detail: lastUpdateDetail.trim(),
      studentId: this.props.studentId,
      photoUrlList: fileList.map(({ src }) => src),
      labelIdList: studentLabelList.map(({ id }) => id),
    };

    this.setState({ modalLoading: true });
    const result = await studentApiService.saveStudentLabel(data).catch(errorsHandle());
    if (!result || result.resultCode !== 0) {
      return;
    }
    try {
      GlobalEvent.emit('REFRESH_STUDENT_MANAGEMENT_LIST');
    } catch (e) {
      console.warn(e);
    }
    showMessage(`修改学生标签成功`, 'success');
    setTimeout(this.handleCancelMainModal, 1000);
  };

  handleCancelMainModal = () => {
    this.props.onClose();
  };

  handleSetListLoading = (listLoading) => {
    this.setState({ listLoading });
  };

  handleSetModalLoading = (modalLoading) => {
    this.setState({ modalLoading });
  };

  handleChangeDesc = (e) => {
    const { detail } = this.state;
    this.setState({
      detail: {
        ...detail,
        lastUpdateDetail: e.target.value,
      },
    });
  };

  handleRemoveLabel = (id) => {
    const { detail } = this.state;
    const selected = detail.studentLabelList.filter((label) => label.id !== id);
    this.setState({
      selectedLabelList: [...selected],
      detail: { ...detail, studentLabelList: [...selected] },
    });
  };

  handleAddLabel = () => {
    const { detail, selectedLabelList } = this.state;
    this.setState({
      addLabelPopVisible: false,
      detail: {
        ...detail,
        studentLabelList: [
          ...detail.studentLabelList,
          ...this.state.labelList.filter(({ id }) => selectedLabelList.includes(id)),
        ],
      },
    });
  };

  handleSetAddLabelPopVisible = (addLabelPopVisible) => {
    const {
      popOverKey,
      detail: { studentLabelList },
    } = this.state;
    const state = { addLabelPopVisible };
    if (addLabelPopVisible) {
      state.selectedLabelList = [...studentLabelList];
      state.popOverKey = popOverKey + 1;
    }
    this.setState({ ...state });
  };

  handleCloseAddLabelPopover = () => {
    this.handleSetAddLabelPopVisible(false);
  };

  handleCloseViewPicturePopover = (id) => {
    this.setState({ [`${id}_viewPicturePopVisible`]: false });
  };

  handleSelectedLabel = (selectedLabelList) => {
    this.setState({ selectedLabelList });
  };

  // 上传图片
  handleUploadPicture = (file) => {
    file.id = file.fileKey;
    if (!file.src) {
      return;
    }
    this.setState({ fileList: _.uniqBy([...this.state.fileList, file], 'id') });
  };

  // 删除图片
  handleRemovePicture = (index) => {
    this.setState({
      fileList: this.state.fileList.filter((_, i) => i !== index),
    });
  };

  // 上传图片前
  handleBeforeUploadPicture = (fileList) => {
    if (fileList.filter((file) => !~['jpg', 'jpeg', 'png', 'gif'].indexOf(file.name.split('.').pop())).length) {
      showMessage('文件格式有误，请重新上传', 'error');
      return false;
    }
    fileList.forEach((file) => (file.fileKey += `.${file.name.split('.').pop()}`));
    return fileList;
  };

  // 查看图片
  handleViewPicture = (id) => {
    this.setState({ [`${id}_previewImgVisible`]: true });
  };

  handleViewPictureItem = (previewImg, previewImgList, id) => {
    this.setState({
      previewImg,
      previewImgList,
      previewImgVisible: true,
      [`${id}_viewPicturePopVisible`]: false,
    });
  };

  // 主体弹框
  generateMainModal() {
    const props = {
      widthSize: 'l',
      okText: '保存修改',
      cancelText: '取消',
      title: '标记特殊学生',
      visible: this.props.visible,
      loading: this.state.modalLoading,
      onOk: this.handleSubmitMainModal,
      onCancel: this.handleCancelMainModal,
      className: 'editStudentLabelModal',
    };

    return (
      <Modal {...props}>
        {this.generateTag()}
        {this.generateDesc()}
        {this.generatePictures()}
        {this.generateRecordList()}
        {this.generatePictureViewer()}
      </Modal>
    );
  }

  // 学生标签
  generateTag() {
    const {
      popOverKey,
      addLabelPopVisible,
      detail: { studentLabelList },
    } = this.state;
    const popoverProps = {
      key: popOverKey,
      trigger: 'click',
      placement: 'bottom',
      visible: addLabelPopVisible,
      overlayClassName: style.addLabelPopOver,
      onVisibleChange: this.handleSetAddLabelPopVisible,
      title: AddLabelPopoverTitle({ onClose: this.handleCloseAddLabelPopover }),
      content: AddLabelPopoverContent({
        labelList: this.labelSelectList,
        onAddLabel: this.handleAddLabel,
        onSelect: this.handleSelectedLabel,
      }),
    };

    return (
      <Row gutter={16}>
        <Col span={2}>
          <span style={{ position: 'relative', top: 4 }}>学生标签</span>
        </Col>
        <Col span={22}>
          {(studentLabelList || []).map(({ name, id }) => StudentLabel({ name, id, onRemove: this.handleRemoveLabel }))}
          {!!this.labelSelectList.length && (
            <Popover {...popoverProps}>
              <Button iconType="plus">点击添加标签</Button>
            </Popover>
          )}
        </Col>
      </Row>
    );
  }

  // 详细备注
  generateDesc() {
    const props = {
      rows: 5,
      wordCount: true,
      maxLength: 1000,
      placeholder: '请输入',
      onChange: this.handleChangeDesc,
      value: this.state.detail.lastUpdateDetail || '',
      onFocus: () => this.setState({ isDescTextAreaFocus: true }),
      onBlur: () => this.setState({ isDescTextAreaFocus: false }),
      className: this.state.isDescTextAreaFocus ? 'desc-textarea-focus' : '',
    };
    return (
      <Row gutter={16}>
        <Col span={2}>
          <span style={{ position: 'relative', top: 8 }}>详细备注</span>
        </Col>
        <Col span={22}>
          <TextArea {...props} />
        </Col>
      </Row>
    );
  }

  // 上传图片
  generatePictures() {
    const props = {
      fileList: this.state.fileList,
      onUpload: this.handleUploadPicture,
      onRemove: this.handleRemovePicture,
      onBeforeUpload: this.handleBeforeUploadPicture,
    };
    return (
      <Row gutter={16}>
        <Col span={2}>
          <span style={{ position: 'relative', top: 8 }}>上传图片</span>
        </Col>
        <Col span={22}>
          <PictureList {...props} />
        </Col>
      </Row>
    );
  }

  generateRecordList() {
    const props = {
      columns: this.columns,
      dataSource: this.state.list,
      loading: this.state.listLoading,
    };
    return (
      <div className="record-list-wrapper">
        <p className="record-list-title">操作记录</p>
        <List {...props}></List>
      </div>
    );
  }

  generateCurrentLabel = (value) => {
    value = (value || []).filter((v) => !_.isEmpty(v));
    return (
      !!value.length && (
        <ul className="current-label-item-wrapper">
          {value.map((label, i) => (
            <li key={i}>{label}</li>
          ))}
        </ul>
      )
    );
  };

  generateDescWrapper = (value) => {
    value = (value || '').trim();
    const desc = <div className="current-label-desc-wrapper">{value}</div>;
    return value.length > 50 ? (
      <Tooltip placement="top" title={value} overlayClassName="current-label-desc-tooltip-wrapper">
        {desc}
      </Tooltip>
    ) : (
      desc
    );
  };

  generateOperation = (_, item) => {
    const popoverProps = {
      trigger: 'click',
      placement: 'left',
      overlayClassName: style.viewPicturePopOver,
      visible: this.state[`${item.id}_viewPicturePopVisible`],
      title: ViewPicturePopoverTitle({ onClose: this.handleCloseViewPicturePopover, id: item.id }),
      onVisibleChange: (visible) => this.setState({ [`${item.id}_viewPicturePopVisible`]: visible }),
      content: ViewPicturePopoverContent({
        id: item.id,
        pictureList: item.photoUrlList,
        onView: this.handleViewPictureItem,
      }),
    };

    return (
      <div>
        {(item.photoUrlList || []).length ? (
          <Popover {...popoverProps}>
            <a style={{ color: '#6D67FF' }} onClick={() => this.handleViewPicture(item.id)}>
              点击查看图片
            </a>
          </Popover>
        ) : (
          <a style={{ color: '#AAAAAA', cursor: 'not-allowed' }}>点击查看图片</a>
        )}
      </div>
    );
  };

  generatePictureViewer = () => {
    const { previewImg, previewImgList, previewImgVisible } = this.state;
    const modalProps = {
      width: 900,
      height: 600,
      closable: true,
      okBtnShow: false,
      cancelBtnShow: false,
      onClose: () => this.setState({ previewImgVisible: false }),
    };
    return (
      previewImgVisible && (
        <Modal {...modalProps}>
          <PreviewImgs list={previewImgList} activeSrc={previewImg} showThumbnail={false} />
        </Modal>
      )
    );
  };

  render() {
    return <div>{this.generateMainModal()}</div>;
  }
}
