import React from 'react';

import { connect } from 'dva';
import { Tooltip, Switch } from 'antd';

import { Page, Select, List, Icon, Modal, Button, ListAction, SearchFilterForm, SearchFieldForm } from 'COMPONENT';

import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';

import { errorsHandle, loginUser } from 'UTIL/common';
import {
  getSubjectAlias,
  getSelectOption,
  incrementSaveSwitchConfig,
  listOrgRankingDisplaySwitch,
  listUserAllBranchByResourceId,
  deleteStudentAchievementTemplate,
  getStudentAchievementTemplateList,
} from 'SERVICE/scoreOrderTemplateManagement';

import moment from 'moment';

import styles from './styles.less';

const setWidth = List.createSetWidth();
const formatDict = (dict) => {
  const result = Object.keys(dict).map((val) => ({ text: dict[val], val }));
  return [...result.filter(({ val }) => val)];
};

class ScoreOrderTemplateManagement extends React.Component {
  state = {
    title: '成绩单模板管理',

    loading: false,
    searchKey: null,
    defaultSKey: {},
    isRankModalVisible: false,

    list: [],
    total: 0,
    pageCurrent: 1,
    pageSize: 50,

    rankList: [],

    yearDict: [],
    gradeDict: [],
    branchDict: [],
    subjectDict: [],
    semesterDict: [],
    examinationTypeDict: [],
  };

  constructor(props) {
    super(props);

    this.columns = [
      { title: '所属分公司', dataIndex: 'branchName', key: 'branchName', width: 120 },
      { title: '学年', dataIndex: 'schoolYearName', key: 'schoolYearName', width: 80 },
      {
        title: '学期',
        dataIndex: 'semester',
        key: 'semester',
        width: 60,
        render: (value) => (value ? this.state.semesterDict?.find((item) => item.val == value)?.text || '' : ''),
      },
      { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: 80 },
      { title: '科目', dataIndex: 'subjectName', key: 'subjectName', width: 80 },
      {
        title: '考试时间区间',
        dataIndex: 'rangeTime',
        key: 'rangeTime',
        width: 160,
        render: (_, record) =>
          record.examinationDateStart && record.examinationDateEnd
            ? [record.examinationDateStart, record.examinationDateEnd].join('~')
            : '',
      },
      { title: '创建人', dataIndex: 'createUserName', key: 'createUserName', width: 80 },
      {
        title: '创建时间',
        dataIndex: 'createTime',
        key: 'createTime',
        width: 100,
        render: (value) => (value ? moment(value).format('YYYY-MM-DD HH:mm:ss') : ''),
      },
      { title: '最近编辑人', dataIndex: 'modifyUserName', key: 'modifyUserName', width: 80 },
      {
        title: '最近编辑时间',
        dataIndex: 'modifyTime',
        key: 'modifyTime',
        width: 100,
        render: (value) => (value ? moment(value).format('YYYY-MM-DD HH:mm:ss') : ''),
      },
      {
        title: '操作',
        dataIndex: 'id',
        key: 'actionPlus',
        render: (_, record) => {
          const config = [
            {
              text: '编辑',
              onClick: () => {
                this.handleEdit(record);
              },
            },
            {
              text: '删除',
              onClick: () => {
                this.handleDelete(record);
              },
            },
          ];

          return <ListAction config={config} />;
        },
      },
    ];
  }

  componentDidMount() {
    this.init();
  }

  init = () => {
    this.initDict();
    this.loadList();
    this.loadRankList();
  };

  initDict = () => {
    Promise.all([
      listUserAllBranchByResourceId({ resourceId: 'scoreOrderTemplateManagement' }),
      getSelectOption({ selectOptionCategory: 'SCHOOL_YEAR' }),
      getSelectOption({ selectOptionCategory: 'Semester' }),
      getSelectOption({ selectOptionCategory: 'EXAMINATION_TYPE' }),
      getSelectOption({ selectOptionCategory: 'STUDENT_GRADE' }),
    ]).then((result) => {
      let [
        { data: branchDict },
        { value: yearDict },
        { value: semesterDict },
        { value: examinationTypeDict },
        { value: gradeDict },
      ] = result;

      this.setState({
        yearDict: formatDict(yearDict),
        gradeDict: formatDict(gradeDict),
        semesterDict: formatDict(semesterDict),
        examinationTypeDict: formatDict(examinationTypeDict),
        branchDict: branchDict.map(({ id, name }) => ({ text: name, val: id })),
      });
    });
  };

  loadRankList = () => {
    listOrgRankingDisplaySwitch()
      .then((res) => {
        this.setState({ rankList: res?.data || [] });
      })
      .catch(() => {
        this.setState({ rankList: [] });
      });
  };

  handleDelete = (record) => {
    const { gradeName, schoolYearName, branchName, semester, examinationType, subjectName } = record;
    const semesterText = this.state.semesterDict?.find(({ val }) => val == semester)?.text || '';
    const examinationTypeText = this.state.examinationTypeDict?.find(({ val }) => val == examinationType)?.text || '';

    showConfirm(
      `您本次操作将删除【${branchName}+${schoolYearName}+${semesterText}+${examinationTypeText}+${gradeName}+${subjectName}】的成绩单模板，请问是否确认删除，删除后无法恢复，只能重新创建模板，请谨慎操作`,
      () => {
        deleteStudentAchievementTemplate({ id: record.id })
          .then(() => {
            const { pageSize, pageCurrent } = this.state;
            this.changePage(pageCurrent, pageSize);
            showMessage('删除成功', 'success');
          })
          .catch((e) => {
            showMessage(e?.text || '删除失败，请稍后重试', 'error');
          });
      },
    );
  };

  handleSaveSuccess = () => {
    const { pageSize, pageCurrent } = this.state;
    this.changePage(pageCurrent, pageSize);
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        refresh: false,
        path: `/scoreOrderTemplateManagement`,
      },
    });
  };

  handleAdd = () => {
    let { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        refresh: true,
        path: `/scoreOrderTemplateManagement/Add`,
        params: { title: '新增模板', onSaveSuccess: this.handleSaveSuccess },
      },
    });
  };

  handleEdit = (record) => {
    let { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        refresh: true,
        path: `/scoreOrderTemplateManagement/Add`,
        params: { title: '编辑', id: record.id, onSaveSuccess: this.handleSaveSuccess },
      },
    });
  };

  handleSetRank = () => {
    this.setState({ isRankModalVisible: true });
  };

  handleCloseModal = () => {
    this.setState({ isRankModalVisible: false });
  };

  getListParams = () => {
    const searchKey = {
      pageNum: this.state.pageCurrent,
      pageSize: this.state.pageSize,
    };

    for (let key in this.state.searchKey) {
      if (key === 'branchIds') {
        searchKey[key] = this.state.searchKey[key] ? [this.state.searchKey[key]] : null;
      } else {
        searchKey[key] = this.state.searchKey[key];
      }
    }

    return searchKey;
  };

  loadList = async () => {
    this.setState({ loading: true });

    const apiParams = this.getListParams();
    const rsp = await getStudentAchievementTemplateList(apiParams).catch(errorsHandle());
    if (rsp?.resultCode === 0 && rsp?.data) {
      this.setState({
        list: rsp.data.list || [],
        total: rsp.data.total,
      });
    } else {
      this.setState({
        list: [],
        total: 0,
      });
    }

    this.setState({ loading: false });
  };

  search = (params) => {
    this.setState(
      {
        searchKey: { ...params },
      },
      () => {
        this.changePage({ current: 1, pageSize: this.state.pageSize });
      },
    );
  };

  updatePage = (pageCurrent, pageSize) => {
    return new Promise((resolve) => {
      this.setState(
        (prevState) => {
          let payload = {};
          if (pageSize && pageSize !== prevState.pageSize) {
            payload.pageCurrent = 1;
            payload.pageSize = pageSize;
          } else {
            payload.pageCurrent = pageCurrent || 1;
          }
          return payload;
        },
        () => {
          resolve();
        },
      );
    });
  };

  changePage = async ({ current, pageSize }) => {
    await this.updatePage(current, pageSize);
    await new Promise(this.loadList);
  };

  loadSubjectAlias = (params) => {
    if (params.orgId && params.gradeId) {
      getSubjectAlias(params)
        .then((res) => {
          this.setState({
            subjectDict: res?.data.map((item) => ({ val: item.value, text: item.aliasName || item.name })),
          });
        })
        .catch(() => {
          this.setState({ subjectDict: [] });
        })
        .finally(() => {
          this.searchFilterForm.setFieldsValue({ subjectId: null });
        });
    } else {
      this.setState({ subjectDict: [] });
      this.searchFilterForm.setFieldsValue({ subjectId: null });
    }
  };

  handleChangeRank = (id, checked) => {
    incrementSaveSwitchConfig({
      campusIds: [id],
      switchFlag: checked ? 'OPEN' : 'CLOSE',
      configs: ['SC_BRANCH_ACHIEVEMENT_DISPLAY'],
    })
      .then(() => {
        this.loadRankList();
      })
      .catch((e) => {
        showMessage(e?.text || '设置星火排名显示失败，请稍后重试', 'error');
      });
  };

  handleLoadSubjectByBranchId = (orgId) => {
    const params = {
      orgId,
      gradeId: this.searchFilterForm?.form?.getFieldValue('gradeId'),
    };
    this.loadSubjectAlias(params);
  };

  handleLoadSubjectByGradeId = (gradeId) => {
    const params = {
      gradeId,
      orgId: this.searchFilterForm?.form?.getFieldValue('branchIds'),
    };
    this.loadSubjectAlias(params);
  };

  render() {
    console.log(this.state.rankList);
    const fieldsValue = this.searchFilterForm?.form?.getFieldsValue();
    return (
      <Page
        title={this.state.title}
        loading={this.state.loading || this.state.reportLoading}
        subHeader={
          <div className={styles.wrap}>
            <div className={styles.leftWrap}>
              <SearchFilterForm onSearch={this.search} truncationNum={5} ref={(el) => (this.searchFilterForm = el)}>
                <SearchFieldForm name="branchIds" initialValue={this.state.defaultSKey.branchIds}>
                  <Select
                    placeholder="分公司"
                    dict={this.state.branchDict}
                    allowClear={false}
                    onChange={this.handleLoadSubjectByBranchId}
                  />
                </SearchFieldForm>
                <SearchFieldForm name="schoolYear" initialValue={this.state.defaultSKey.schoolYear}>
                  <Select placeholder="学年" dict={this.state.yearDict} showSearch={true} />
                </SearchFieldForm>
                <SearchFieldForm name="semester">
                  <Select placeholder="学期" dict={this.state.semesterDict} showSearch={true} />
                </SearchFieldForm>
                <SearchFieldForm name="gradeId">
                  <Select
                    placeholder="年级"
                    dict={this.state.gradeDict}
                    showSearch={true}
                    onChange={this.handleLoadSubjectByGradeId}
                  />
                </SearchFieldForm>
                <SearchFieldForm name="subjectId">
                  <Select
                    placeholder="科目"
                    dict={this.state.subjectDict}
                    showSearch={true}
                    disabled={!fieldsValue?.branchIds || !fieldsValue?.gradeId}
                  />
                </SearchFieldForm>
              </SearchFilterForm>
            </div>
            <div className={styles.rightWrap}>
              <Button type="primary" onClick={this.handleAdd} className={styles.marginRight}>
                新增模板
              </Button>
              <Button type="primary" onClick={this.handleSetRank} className={styles.marginRight}>
                设置星火排名显示
              </Button>
              <Tooltip title="分公司总部管理人员可在本页面设置对应【学年+学期+考试类型+科目】的成绩单模板，设置模板后，家长登录成绩单查询链接查询对应【学年+学期+考试类型+科目】的成绩单时，即可看到分公司总部录入的相关分析内容，有助于家长充分了解学生的学习情况以及进步曲线。">
                <Icon type="question_circle" />
              </Tooltip>
            </div>
          </div>
        }
      >
        <List
          columns={this.columns}
          dataSource={this.state.list}
          pagination={{
            total: this.state.total,
            current: this.state.pageCurrent,
            pageSize: this.state.pageSize,
            pageSizeOptions: ['50', '100'],
          }}
          onChange={this.changePage}
        />
        {this.state.isRankModalVisible && (
          <Modal width={400} footer={null} title="设置星火排名显示" onCancel={this.handleCloseModal}>
            <ul className={styles.rankLisWrap}>
              {this.state.rankList.map(({ id, name, display }) => {
                return (
                  <li key={id} className={styles.rankList}>
                    <span className={styles.rankName}>{name}</span>
                    <Switch checked={display} onChange={() => this.handleChangeRank(id, !display)} />
                  </li>
                );
              })}
            </ul>
          </Modal>
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(ScoreOrderTemplateManagement);
