import {
  getBossOrganizationList,
  getOrganizationById,
  getOrganizationAppendInfoById,
  findResourcePoolByIdNew,
  getResourcePoolRoleList,
  getAllSwitchByCampusId,
  incrementSaveSwitchConfig,
} from 'SERVICE/system';
import dicts from './common';
import { PROJECT_NAME } from 'CONST/config';

let SPARK_ID = '';
switch (PROJECT_NAME) {
  case 'peiyou':
    SPARK_ID = '15460101177476096';
    break;
  case 'twoteacher':
    SPARK_ID = '15460184888443904';
    break;
  default:
    SPARK_ID = '15460069240996864';
}

const isBoss = (type) => {
  return type === 'BOSS';
};
const isValid = (status) => {
  return status === 0;
};
const loadInitOrgList = async (type) => {
  let params;
  if (isBoss(type)) {
    params = { type, level: 2 };
  } else {
    params = { level: 3, id: SPARK_ID, parentId: SPARK_ID };
  }
  const apiData = await getBossOrganizationList(params, true);
  let checkedKeys = [];
  const platData = apiData.data.map((d) => {
    const isChecked = d.bossUse === 0;
    const id = d.hrms_id;
    if (isChecked) {
      checkedKeys.push(id);
    }
    return {
      id,
      bossId: d.bossId,
      name: d.name,
      valid: isValid(d.status),
      hasChildren: d.hasChildren > 0 ? true : false,
      isRoot: !isBoss(type) && d.id === SPARK_ID ? true : d.parentId ? false : true,
      orgType: d.orgType,
      isChecked,
    };
  });
  const root = platData.find((d) => d.isRoot);
  const children = platData.filter((d) => !d.isRoot);
  return {
    data: [{ ...root, children, hasChildren: root.hasChildren > 0 ? true : false }],
    checkedKeys,
    platData,
  };
};

const loadOrgListByParentId = async ({ parentId, type }) => {
  const params = { parentId, type };
  const apiData = await getBossOrganizationList(params, true);
  let checkedKeys = [];
  const data = apiData.data.map((d) => {
    const isChecked = d.bossUse === 0;
    const id = d.hrms_id;
    if (isChecked) {
      checkedKeys.push(id);
    }
    return {
      id,
      bossId: d.bossId,
      name: d.name,
      valid: isValid(d.status),
      hasChildren: d.hasChildren > 0 ? true : false,
      orgType: d.orgType,
      isChecked,
    };
  });
  return { data, checkedKeys };
};

const loadOrgListByName = async ({ name }) => {
  const params = { name };
  const apiData = await getBossOrganizationList(params, true);
  let checkedKeys = [];
  const data = apiData.data.map((d) => {
    const isChecked = d.bossUse === 0;
    const id = d.hrms_id;
    const bossId = d.bossId;
    if (isChecked) {
      checkedKeys.push(id);
    }
    return { id, bossId, name: d.name, valid: isValid(d.status), hasChildren: false, orgType: d.orgType, isChecked };
  });
  return { data, checkedKeys };
};

const loadBasicInfo = async ({ id }) => {
  try {
    const params = { id };
    const { data } = await getOrganizationById(params, true);
    return {
      id: data.hrmsId,
      bossId: data.bossId,
      oldName: data.hrmsName,
      name: data.bossName,
      belong: data.parentName,
      sourceProtectiveDay: data.sourceProtectiveDay,
      type: data.orgType,
      subType: data.subOrgType + '',
      valid: isValid(data.status) ? dicts.STATUS_VALID : dicts.STATUS_INVALID,
      isBoss: data.bossUse === 0 ? dicts.YES_NO_TRUE : dicts.YES_NO_FALSE,
      remark: data.remark,
      validBoolean: isValid(data.status),
      isChecked: data.bossUse === 0,
      bossOrgType: data.bossOrgType,
    };
  } catch (e) {
    return {};
  }
};

const loadContractInfo = async ({ id }) => {
  try {
    const params = { id };
    const { data } = await getOrganizationAppendInfoById(params, true);
    const imgList = data.fileList.map((d) => {
      return {
        id: d.id,
        src: d.filePath,
      };
    });
    return {
      active: data.hasAppend === 1 ? false : true,
      province: data.province,
      city: data.cityId,
      phone: data.contact,
      headPerson: data.contactUser,
      address: data.address,
      lon: data.lon,
      lat: data.lat,
      traffic: data.trafficInfo,
      imgList,
    };
  } catch (e) {
    return {};
  }
};

const loadPoolInfo = async ({ bossId }) => {
  if (!bossId) {
    return {};
  }

  try {
    const [poolInfoApi, list] = await Promise.all([
      findResourcePoolByIdNew({ organizationId: bossId }, true),
      getResourcePoolRoleList({ organizationId: bossId }, true),
    ]);
    const roleList = list.data.map((li) => {
      return { name: li.roleId, max: li.oneTimeResource };
    });
    const poolInfo = poolInfoApi.data;
    return {
      active: poolInfo.status === 'VALID' ? true : false,
      poolName: poolInfo.name,
      volume: poolInfo.capacity,
      circType: poolInfo.cycleType,
      backflowNode: poolInfo.returnNote,
      cycle: poolInfo.recoveyPeriod,
      roleList,
    };
  } catch (e) {
    return {};
  }
};

const loadDetail = async ({ id, bossId }) => {
  const [basicData, contractData, poolData] = await Promise.all([
    loadBasicInfo({ id }),
    loadContractInfo({ id }),
    loadPoolInfo({ bossId }),
  ]);
  return { basicData, contractData, poolData };
};

const loadCollectionAllSwitch = async ({ campusId }) => {
  const res = await getAllSwitchByCampusId({ campusId });
  const pickData = res.data || [];
  let result = [];

  pickData.forEach(function (collection) {
    if (collection.items)
      collection.items.forEach((item) => {
        item.type = collection.type;
      });
    result = result.concat(collection.items);
  });

  // 默认不显示以下开关(待废弃)
  // '电子账户A', '银行汇款(A)', '支付宝支付(A)', '微信支付(A)', '切换电子账户类型'
  const hideList = [
    'SC_CONF_POS_MACHINE_TYPE_A',
    'SC_BANK_TRANSFER_A',
    'SC_ALI_PAY_A',
    'SC_WEB_CHART_PAY_A',
    'SC_CAN_SW_ELECTRONIC_ACC',
  ];

  return result.filter((item) => {
    return !hideList.includes(item.config);
  });
};

const saveIncrementSaveSwitchConfig = async (data) => {
  const res = await incrementSaveSwitchConfig(data);
  return res;
};

export default {
  loadInitOrgList,
  loadOrgListByParentId,
  loadOrgListByName,
  loadDetail,
  loadCollectionAllSwitch,
  saveIncrementSaveSwitchConfig,
};
