import React from 'react';
import ReactDOM from 'react-dom';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import TextArea from 'COMPONENT/TextArea';
import Select from 'COMPONENT/Select';
import Button from 'COMPONENT/Button';
import styles from './styles.less';
import dicts from '../common';
import { getDictTextByVal } from 'UTIL/dict';
import ImageList from '../ImageList';
import { getCityDictByProvince } from 'UTIL/dict';
import baiduMap from 'UTIL/thirdParty/baiduMap';
import styleVars from 'COMPONENT/theme/variables';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import PopTip from 'COMPONENT/PopTip';
import { FIELD_FILLER, checkFieldEmpty, getFieldWithFiller } from 'COMPONENT/utils/form';

class ContractContent extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      province: props.data.province,
      cityDict: this.getCityDict(props),
      searchTipsVisible: false,

      provinceStorage: null,
      cityStorage: null,
    };
    this.map = null; // Map 地图实例，Map类型参见百度地图API：http://lbsyun.baidu.com/cms/jsapi/reference/jsapi_reference.html#a0b0
    this.geo = null;
  }

  componentDidMount() {
    this.initMap().then(() => {
      const {
        data: { lon, lat },
      } = this.props;
      this.showPop(lon, lat);
    });
  }

  componentWillReceiveProps(nextProps) {
    if (this.props.data.province !== nextProps.data.province) {
      this.setState({ province: nextProps.data.province, cityDict: this.getCityDict(nextProps) });
    }
    if (this.map) {
      if (this.props.data.address !== nextProps.data.address) {
        const {
          data: { lon, lat },
        } = nextProps;
        this.showPop(lon, lat);
      }
      if (!this.props.editing && nextProps.editing) {
        this.map.enableScrollWheelZoom();
      } else if (this.props.editing && !nextProps.editing) {
        this.map.disableScrollWheelZoom();
      }
    }
  }

  initMap = async () => {
    //加载百度地图API
    await baiduMap.init();
    //创建地图实例
    const map = new BMap.Map(this.$map);
    //设置默认的中心点、缩放等级、最小缩放等级、允许滚轮
    map.centerAndZoom(new BMap.Point(113.30765, 23.120049), 14);
    map.setMinZoom(5);
    //监听点击事件
    map.addEventListener('click', this.mapClick);
    this.map = map;
    this.geo = new BMap.Geocoder();
    return true;
  };

  mapClick = (e) => {
    const lon = e.point.lng;
    const lat = e.point.lat;
    this.showPop(lon, lat);
    this.geo.getLocation(e.point, (rs) => {
      const addComp = rs.addressComponents;
      const address = addComp.province + addComp.city + addComp.district + addComp.street + addComp.streetNumber;
      this.onChange({ address, lon, lat });
    });
  };

  searchLonLat = () => {
    const {
      data: { address, city },
    } = this.props;
    const { cityDict } = this.state;
    let lon = null;
    let lat = null;
    if (address) {
      this.geo.getPoint(address, (point) => {
        if (point) {
          lon = point.lng;
          lat = point.lat;
          this.showPop(lon, lat);
        } else {
          this.setState({ searchTipsVisible: true });
        }
        this.onChange({ lon, lat });
      });
    } else {
      this.onChange({ lon, lat });
    }
  };

  showPop = (lon, lat) => {
    this.map.clearOverlays();
    if (lon && lat) {
      const point = new BMap.Point(lon, lat);
      const marker = new BMap.Marker(point);
      this.map.addOverlay(marker);
      const label = new BMap.Label(`经度：${lon}<br/>纬度：${lat}`, { offset: new BMap.Size(22, -32) });
      label.setStyle({
        padding: '5px 8px',
        borderRadius: styleVars.borderRadius,
        boxShadow: styleVars.shadowPopup,
        background: '#fff',
        border: '1px solid ' + styleVars.basicColorBorder,
        color: styleVars.fontColor,
      });
      marker.setLabel(label);
      this.map.centerAndZoom(point, 16);
    }
  };

  getCityDict = (props) => {
    const provinceObj = props.provinceCityDict.find((item) => item.val === props.data.province);

    return provinceObj === undefined
      ? []
      : Array.isArray(provinceObj.options) && provinceObj.options.length > 0
      ? provinceObj.options
      : [{ text: provinceObj.label || provinceObj.text, val: provinceObj.val }];
  };

  onChange = (changed) => {
    const { data, onChange } = this.props;
    const newData = { ...data, ...changed };
    if (!objectDeepEquals(data, newData)) {
      onChange(newData);
    }
  };

  provinceChange = (val) => {
    this.onChange({ province: val, city: null });
  };

  render() {
    const { editing, data, checking, onErrorChange, provinceDict, provinceCityDict } = this.props;
    const { cityDict, searchTipsVisible } = this.state;
    const { province, city, phone, headPerson, address, lon, lat, traffic, imgList } = data;
    const formCfg = {
      labelWidth: 80,
      size: editing ? null : 'small',
      style: { padding: '0 5px' },
    };

    return (
      <div className={styles.wrap + (editing ? ' ' + styles.wrapEditing : '')}>
        <Row style={{ marginLeft: -5 }} gutter={50}>
          <Col span={12}>
            <FormItem {...formCfg} name="省份" required={editing ? true : null}>
              {editing ? (
                <Select
                  dict={provinceDict}
                  value={province}
                  onChange={this.provinceChange}
                  validator={{
                    onErrorChange: (e) => onErrorChange('province', e),
                    silent: !checking,
                  }}
                />
              ) : (
                <div>{getDictTextByVal(provinceDict, province, true)}</div>
              )}
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem {...formCfg} name="城市" required={editing ? true : null}>
              {editing ? (
                <Select
                  dict={cityDict}
                  value={city}
                  onChange={(val) => this.onChange({ city: val })}
                  validator={{
                    onErrorChange: (e) => onErrorChange('city', e),
                    silent: !checking,
                  }}
                />
              ) : (
                <div>{getDictTextByVal(cityDict, city, true)}</div>
              )}
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem {...formCfg} name="电话" required={editing ? true : null}>
              {editing ? (
                <Input
                  value={phone}
                  onChange={(e) => this.onChange({ phone: e.target.value })}
                  validator={{
                    onErrorChange: (e) => onErrorChange('phone', e),
                    silent: !checking,
                  }}
                />
              ) : (
                <div>{getFieldWithFiller(phone)}</div>
              )}
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem {...formCfg} name="负责人">
              {editing ? (
                <Input value={headPerson} onChange={(e) => this.onChange({ headPerson: e.target.value })} />
              ) : (
                <div>{getFieldWithFiller(headPerson)}</div>
              )}
            </FormItem>
          </Col>
          <Col span={24}>
            <FormItem {...formCfg} name="地址">
              {editing ? (
                <div className={styles.addressWrap}>
                  <div className={styles.addressInputWrap}>
                    <Input
                      value={address}
                      onChange={(e) => this.onChange({ address: e.target.value, lon: null, lat: null })}
                    />
                  </div>
                  <PopTip
                    visible={searchTipsVisible}
                    content="没有查询到经纬度，请完善地址信息"
                    onClose={() => this.setState({ searchTipsVisible: false })}
                    getPopupContainer={(ele) => ele.parentNode}
                  >
                    <Button style={{ position: 'relative', top: 2 }} onClick={this.searchLonLat}>
                      查询经纬度
                    </Button>
                  </PopTip>
                </div>
              ) : (
                <div>{getFieldWithFiller(address)}</div>
              )}
            </FormItem>
          </Col>
          <Col span={24}>
            <FormItem {...formCfg} name="实时地图">
              <div className={styles.mapWrap} ref={(c) => (this.$map = ReactDOM.findDOMNode(c))}>
                {' '}
              </div>
            </FormItem>
          </Col>
          <Col span={24}>
            <FormItem {...formCfg} name="交通指南">
              {editing ? (
                <TextArea rows={3} value={traffic || ''} onChange={(e) => this.onChange({ traffic: e.target.value })} />
              ) : (
                <div>{getFieldWithFiller(traffic)}</div>
              )}
            </FormItem>
          </Col>
          <Col span={24}>
            <FormItem {...formCfg} name="实景图">
              {(imgList && imgList.length > 0) || editing ? (
                <ImageList data={imgList} onChange={(val) => this.onChange({ imgList: val })} editing={editing} />
              ) : (
                <div>{FIELD_FILLER}</div>
              )}
            </FormItem>
          </Col>
        </Row>
      </div>
    );
  }
}

export default ContractContent;
