import React from 'react';
import ReactDOM from 'react-dom';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Select from 'COMPONENT/Select';
import styles from './styles.less';
import dicts from '../common';
import { getDictTextByVal } from 'UTIL/dict';
import styleVars from 'COMPONENT/theme/variables';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import { FIELD_FILLER, checkFieldEmpty, getFieldWithFiller } from 'COMPONENT/utils/form';
import { checkNumber } from 'COMPONENT/Validator/checkers';
import RoleList from '../RoleList';

class PoolContent extends React.Component {
  onChange = (changed) => {
    const { data, onChange } = this.props;
    const newData = { ...data, ...changed };
    if (!objectDeepEquals(data, newData)) {
      onChange(newData);
    }
  };

  circTypeChange = (val) => {
    this.onChange({ circType: val });
    if (val !== dicts.CIRC_TYPE_1) {
      this.props.onErrorChange('backflowNode', null);
      this.props.onErrorChange('cycle', null);
    }
  };

  render() {
    const { editing, data, checking, onErrorChange, backflowNodeDict, roleDict, bossOrgType } = this.props;
    const { active, poolName, volume, circType, backflowNode, cycle, roleList } = data;
    const formCfg = {
      labelWidth: 130,
      size: editing ? null : 'small',
      style: { padding: '0 5px' },
    };

    return (
      <div className={styles.wrap + (editing ? ' ' + styles.wrapEditing : '')}>
        <div className={styles.title}>基本信息</div>
        <Row style={{ marginLeft: -5 }} gutter={50}>
          <Col span={12}>
            <FormItem {...formCfg} name="资源池名称" required={editing ? true : null}>
              {editing ? (
                <Input
                  value={poolName}
                  onChange={(e) => this.onChange({ poolName: e.target.value })}
                  validator={{
                    onErrorChange: (e) => onErrorChange('poolName', e),
                    silent: !checking,
                  }}
                  maxLength={15}
                  wordCount={true}
                />
              ) : (
                <div>{getFieldWithFiller(poolName)}</div>
              )}
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem {...formCfg} name="资源池容量（个）" required={editing ? true : null}>
              {editing ? (
                <InputNumber
                  value={volume}
                  onChange={(val) => this.onChange({ volume: val })}
                  validator={{
                    onErrorChange: (e) => onErrorChange('volume', e),
                    silent: !checking,
                    rules: ['required', (v) => checkNumber(v, { excludeMin: 0, int: true })],
                  }}
                />
              ) : (
                <div>{getFieldWithFiller(volume)}</div>
              )}
            </FormItem>
          </Col>
          <Col span={12} style={{ clear: 'left' }}>
            <FormItem {...formCfg} name="流转方式" required={editing ? true : null}>
              {editing ? (
                <Select
                  dict={dicts.circTypeDict}
                  value={bossOrgType === dicts.BOSS_ORG_TYPE_BRANCH ? dicts.CIRC_TYPE_2 : circType}
                  onChange={this.circTypeChange}
                  validator={{
                    onErrorChange: (e) => onErrorChange('circType', e),
                    silent: !checking,
                  }}
                  disabled={bossOrgType === dicts.BOSS_ORG_TYPE_BRANCH ? true : false}
                />
              ) : (
                <div>{getDictTextByVal(dicts.circTypeDict, circType, true)}</div>
              )}
            </FormItem>
          </Col>
          {circType === dicts.CIRC_TYPE_1 && (
            <Col span={12}>
              <FormItem {...formCfg} name="回流节点" required={editing ? true : null}>
                {editing ? (
                  <Select
                    dict={backflowNodeDict}
                    value={backflowNode}
                    onChange={(val) => this.onChange({ backflowNode: val })}
                    validator={{
                      onErrorChange: (e) => onErrorChange('backflowNode', e),
                      silent: !checking,
                    }}
                  />
                ) : (
                  <div>{getDictTextByVal(backflowNodeDict, backflowNode, true)}</div>
                )}
              </FormItem>
            </Col>
          )}
          {circType === dicts.CIRC_TYPE_1 && (
            <Col span={12}>
              <FormItem {...formCfg} name="回收周期（天）" required={editing ? true : null}>
                {editing ? (
                  <InputNumber
                    value={cycle}
                    onChange={(val) => this.onChange({ cycle: val })}
                    validator={{
                      onErrorChange: (e) => onErrorChange('cycle', e),
                      silent: !checking,
                      rules: ['required', (v) => checkNumber(v, { excludeMin: 0, int: true })],
                    }}
                  />
                ) : (
                  <div>{getFieldWithFiller(cycle)}</div>
                )}
              </FormItem>
            </Col>
          )}
        </Row>
        <RoleList
          data={roleList}
          onChange={(val) => this.onChange({ roleList: val })}
          roleDict={roleDict}
          editing={editing}
        />
      </div>
    );
  }
}

export default PoolContent;
