import React from 'react';
import ReactDOM from 'react-dom';
import InputNumber from 'COMPONENT/InputNumber';
import Select from 'COMPONENT/Select';
import styles from './styles.less';
import styleVars from 'COMPONENT/theme/variables';
import { FIELD_FILLER, checkFieldEmpty, getFieldWithFiller, getCommonErrorTips } from 'COMPONENT/utils/form';
import Button from 'COMPONENT/Button';
import List from 'COMPONENT/List';
import TextLink from 'COMPONENT/TextLink';
import { getDictTextByVal } from 'UTIL/dict';
import PopTip from 'COMPONENT/PopTip';
import classnames from 'classnames';
import PropTypes from 'prop-types';
import { checkNumber } from 'COMPONENT/Validator/checkers';

class RoleList extends React.Component {
  static defaultProps = {
    data: [],
  };
  constructor(props) {
    super(props);
    this.state = {
      editingRowIdx: null,
      editingName: null,
      editingMax: null,
      isEditingNew: false,
      saveTipsVis: false,
      saveTips: null,
      rowChecking: false,
    };
    this.errors = {};
  }

  componentDidMount() {
    this.$wrap = ReactDOM.findDOMNode(this.wrapRef);
  }

  componentWillReceiveProps = (nextProps) => {
    if (!nextProps.editing) {
      this.cancel();
    }
  };

  save = () => {
    const { onChange, data } = this.props;
    const { isEditingNew, editingName, editingMax, editingRowIdx, rowChecking } = this.state;
    if (!rowChecking) {
      this.setState({ rowChecking: true });
    }
    const errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      this.setState({ saveTipsVis: true, saveTips: errorTips });
      return;
    }
    const newRow = { name: editingName, max: editingMax };
    let newData;
    if (isEditingNew) {
      newData = [newRow, ...data];
    } else {
      newData = data.map((row, idx) => {
        if (idx === editingRowIdx) {
          return newRow;
        }
        return row;
      });
    }
    onChange(newData);
    this.cancel();
  };

  cancel = () => {
    this.setState({
      editingRowIdx: null,
      editingName: null,
      editingMax: null,
      isEditingNew: false,
      rowChecking: false,
    });
  };

  edit = (idx, record) => {
    if (this.state.isEditingNew) {
      idx -= 1;
    }
    this.setState({
      editingRowIdx: idx,
      editingName: record.name,
      editingMax: record.max,
      isEditingNew: false,
      saveTipsVis: false,
      saveTips: null,
      rowChecking: false,
    });
    this.errors = {};
  };

  delete = (idx) => {
    const { onChange, data } = this.props;
    onChange(data.filter((da, i) => (this.state.isEditingNew ? i + 1 : i) !== idx));
  };

  add = () => {
    this.setState({ editingRowIdx: 0, editingName: null, editingMax: null, isEditingNew: true });
  };

  render() {
    const { data, roleDict, editing } = this.props;
    const { isEditingNew, editingName, editingMax, editingRowIdx, saveTipsVis, saveTips, rowChecking } = this.state;
    const activeRoleDict = roleDict.map((opt) => {
      if (
        data
          .filter((d, idx) => isEditingNew || idx !== editingRowIdx)
          .some((da) => da.name === opt.val && editingName !== opt.val)
      ) {
        return { ...opt, disabled: true };
      }
      return opt;
    });
    let columns = [
      {
        title: '角色',
        dataIndex: 'name',
        render: (text, record, index) => {
          if (editingRowIdx === index) {
            return (
              <Select
                dict={activeRoleDict}
                value={editingName}
                onChange={(val) => this.setState({ editingName: val })}
                validator={{
                  onErrorChange: (e) => (this.errors['name'] = e),
                  silent: !rowChecking,
                }}
                showSearch
              />
            );
          }
          return getDictTextByVal(roleDict, text, true);
        },
      },
      {
        title: '一次性可获取资源量上限',
        dataIndex: 'max',
        render: (text, record, index) => {
          if (editingRowIdx === index) {
            return (
              <InputNumber
                value={editingMax}
                onChange={(val) => this.setState({ editingMax: val })}
                validator={{
                  onErrorChange: (e) => (this.errors['max'] = e),
                  silent: !rowChecking,
                  rules: ['required', (v) => checkNumber(v, { excludeMin: 0, int: true })],
                }}
              />
            );
          }
          return text;
        },
      },
      {
        title: '操作',
        key: 'action',
        render: (text, record, index) => {
          if (!editing) {
            return FIELD_FILLER;
          }
          if (editingRowIdx === index) {
            return (
              <span className={styles.editingBtnWrap}>
                <PopTip
                  visible={saveTipsVis}
                  content={saveTips}
                  onClose={() => this.setState({ saveTipsVis: false })}
                  getPopupContainer={() => this.$wrap}
                >
                  <TextLink className="list-btn" onClick={this.save}>
                    保存
                  </TextLink>
                </PopTip>
                <TextLink className="list-btn" onClick={this.cancel}>
                  取消
                </TextLink>
              </span>
            );
          }
          return (
            <span>
              <TextLink className="list-btn" onClick={() => this.edit(index, record)}>
                修改
              </TextLink>
              <TextLink className="list-btn" onClick={() => this.delete(index)}>
                删除
              </TextLink>
            </span>
          );
        },
      },
    ];
    if (editing) {
      columns.push();
    }
    const dataSource = isEditingNew ? [{ name: editingName, max: editingMax }, ...data] : data;
    const cls = {
      [styles.wrap]: 1,
      [styles.wrapEditing]: editing,
    };

    return (
      <div className={classnames(cls)} ref={(c) => (this.wrapRef = c)}>
        <div className={styles.header}>
          <div className={styles.headerLeft}>资源池可见角色</div>
          {editing && (
            <div className={styles.headerRight}>
              <Button onClick={this.add}>添加</Button>
            </div>
          )}
        </div>
        <List dataSource={dataSource} columns={columns} />
      </div>
    );
  }
}

export default RoleList;
