import React from 'react';
import ReactDOM from 'react-dom';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import InputNumber from 'COMPONENT/InputNumber';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import styles from './styles.less';
import dicts from '../dicts';
import { getDictTextByVal } from 'UTIL/dict';
import styleVars from 'COMPONENT/theme/variables';
import { FIELD_FILLER, checkFieldEmpty, getFieldWithFiller } from 'COMPONENT/utils/form';
import { checkNumber } from 'COMPONENT/Validator/checkers';
import { Switch } from 'antd';
import classnames from 'classnames';
import PopConfirm from 'COMPONENT/PopConfirm';

const rowCfg = {
  style: { margin: '35px -45px 0 -8px' },
  gutter: 50,
};

class CustomerInfo extends React.Component {
  onChange = (changed) => {
    const { data, onChange } = this.props;
    const newData = { ...data, ...changed };
    onChange(newData);
  };

  circTypeChange = (val) => {
    this.onChange({ circType: val });
    if (val !== dicts.CIRC_TYPE_1) {
      this.props.onErrorChange('cycle', null);
    }
  };

  render() {
    const { editable, data, checking, onErrorChange, roleDict, onActiveChange } = this.props;
    const { active, volume, circType, cycle, auths } = data;
    const formCfg = {
      labelWidth: 130,
      size: editable ? null : 'small',
      style: { padding: '0 5px' },
    };

    const cls = {
      [styles.wrap]: 1,
      [styles.wrapEditable]: editable,
    };

    let switchBtn = (
      <span>
        <Switch size="small" checked={active} disabled={!editable} onChange={active ? () => {} : onActiveChange} />
      </span>
    );
    if (active && editable) {
      switchBtn = (
        <PopConfirm
          content="关闭后，该角色客户资源设置信息将失效，是否确定关闭？"
          onOk={() => onActiveChange(false)}
          getPopupContainer={() => this.$wrap}
        >
          {switchBtn}
        </PopConfirm>
      );
    }

    return (
      <div className={classnames(cls)} ref={(c) => (this.$wrap = ReactDOM.findDOMNode(c))}>
        <div className={styles.title}>
          <span className={styles.title}>客户资源跟进角色</span>
          <span className={styles.switchBtn}>{switchBtn}</span>
        </div>
        {active && (
          <div>
            <Row {...rowCfg}>
              <Col span={12}>
                <FormItem {...formCfg} name="资源池容量（个）" required={editable ? true : null}>
                  {editable ? (
                    <InputNumber
                      value={volume}
                      onChange={(val) => this.onChange({ volume: val })}
                      validator={{
                        onErrorChange: (e) => onErrorChange('volume', e),
                        silent: !checking,
                        rules: ['required', (v) => checkNumber(v, { excludeMin: 0, int: true })],
                      }}
                    />
                  ) : (
                    <div>{getFieldWithFiller(volume)}</div>
                  )}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem {...formCfg} name="流转方式" required={editable ? true : null}>
                  {editable ? (
                    <Select
                      dict={dicts.circTypeDict}
                      value={circType}
                      onChange={this.circTypeChange}
                      validator={{
                        onErrorChange: (e) => onErrorChange('circType', e),
                        silent: !checking,
                      }}
                    />
                  ) : (
                    <div>{getDictTextByVal(dicts.circTypeDict, circType, true)}</div>
                  )}
                </FormItem>
              </Col>
            </Row>
            <Row {...rowCfg}>
              {circType === dicts.CIRC_TYPE_1 && (
                <Col span={12}>
                  <FormItem {...formCfg} name="回收周期（天）" required={editable ? true : null}>
                    {editable ? (
                      <InputNumber
                        value={cycle}
                        onChange={(val) => this.onChange({ cycle: val })}
                        validator={{
                          onErrorChange: (e) => onErrorChange('cycle', e),
                          silent: !checking,
                          rules: ['required', (v) => checkNumber(v, { excludeMin: 0, int: true })],
                        }}
                      />
                    ) : (
                      <div>{getFieldWithFiller(cycle)}</div>
                    )}
                  </FormItem>
                </Col>
              )}
              <Col span={12}>
                <FormItem {...formCfg} name="资源分配权限">
                  {editable ? (
                    <MultiSelect dict={roleDict} value={auths} onChange={(val) => this.onChange({ auths: val })} />
                  ) : (
                    <div>
                      {auths && auths.length > 0
                        ? roleDict
                            .filter((ro) => auths.indexOf(ro.val) >= 0)
                            .map((rol) => rol.text)
                            .join('、')
                        : FIELD_FILLER}
                    </div>
                  )}
                </FormItem>
              </Col>
            </Row>
          </div>
        )}
      </div>
    );
  }
}

export default CustomerInfo;
