import React from 'react';
import { connect } from 'dva';
import Input from 'COMPONENT/Input';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import styles from './styles.less';
import loaders from './loaders';
import dicts from './dicts';
import Button from 'COMPONENT/Button';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const defaultSearchKeys = { name: null, type: null, level: null, status: dicts.STATUS_VAILD };

class RoleManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态

      name: null, // {string} 查询条件 角色名称
      type: null, // {string} 查询条件 角色类型
      level: null, // {string} 查询条件 角色等级
      status: dicts.STATUS_VAILD, // {string} 查询条件 状态

      typeDict: dicts.typeDict, // {[object]} 字典 角色类型
      levelDict: dicts.levelDict, // {[object]} 字典 角色等级
      statusDict: dicts.statusDict, // {[object]} 字典 状态
      roleDict: [], // {[object]} 字典 角色

      list: [], // {Array} 列表数据
      listSearchKeys: defaultSearchKeys, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数

      treeData: [], //  {object[]} BOSS权限树
      handTreeData: [], //  {object[]} 掌上BOSS权限树
      /**
        treeData、handTreeData结构如下：
        [
          { id: '1', name: '客户管理', parentId: null },
          { id: '2', name: '学生管理', parentId: null },
          { id: '1-1', name: '客户列表', parentId: '1'},
          { id: '1-2', name: '客户统计', parentId: '1' },
          { id: '1-1-1', name: '新增按钮', parentId: '1-1' },
        ]
      */
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = () => {
    this.setState({ loading: true });
    Promise.all([
      loaders.loadList(this.getListParams()),
      loaders.loadTreeData(),
      loaders.loadHandTreeData(),
      loaders.loadRoleDict(),
    ]).then(([listData, treeData, handTreeData, roleDict]) => {
      const { list, totalRecords } = listData;
      this.setState({ loading: false, list, totalRecords, treeData, handTreeData, roleDict });
    });
  };

  getListParams = () => {
    const {
      listSearchKeys: { name, type, level, status },
      currentPage,
      pageSize,
    } = this.state;
    return { name, type, level, status, currentPage, pageSize };
  };

  search = () => {
    const { name, type, level, status } = this.state;
    const listSearchKeys = { name, type, level, status };
    this.setState({ loading: true });
    loaders.loadList({ ...this.getListParams(), ...listSearchKeys, currentPage: 1 }).then(({ list, totalRecords }) => {
      this.setState({ loading: false, listSearchKeys, currentPage: 1, list, totalRecords });
    });
  };

  reset = () => {
    this.setState(defaultSearchKeys);
  };

  goDetail = (id, mode) => {
    const { dispatch } = this.props;
    const { typeDict, levelDict, statusDict, roleDict, treeData, handTreeData } = this.state;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: `/SystemRoleManage/${mode}Role`,
        params: {
          mode,
          id,
          typeDict,
          levelDict,
          statusDict,
          roleDict,
          treeData,
          handTreeData,
        },
        refresh: true,
      },
    });
  };

  onPaginationChange = ({ current, pageSize }) => {
    this.setState({ loading: true });
    const changed = {
      currentPage: current,
      pageSize,
    };
    this.setState({ loading: true });
    loaders.loadList({ ...this.getListParams(), ...changed }).then(
      ({ list, totalRecords }) => {
        this.setState({ loading: false, ...changed, list, totalRecords });
      },
      () => {
        this.setState({ loading: false });
      },
    );
  };

  setRoleStatus = (id, status) => {
    this.setState({ loading: true });
    loaders.setRoleStatus(id, status).then(
      () => {
        showMessage('操作成功', 'success');
        const newList = this.state.list.map((li) => {
          if (li.id === id) {
            return { ...li, status };
          }
          return li;
        });
        this.setState({ loading: false, list: newList });
      },
      (e) => {
        this.setState({ loading: false });
        showMessage(e.text || '操作失败');
      },
    );
  };

  render() {
    const {
      loading,
      searchExpand,
      name,
      type,
      level,
      status,
      typeDict,
      levelDict,
      statusDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
    } = this.state;

    const columns = [
      { title: '角色名称', dataIndex: 'name', key: 'name', fixed: 'left' },
      { title: '角色类型', dataIndex: 'type', key: 'type', dict: typeDict },
      { title: '角色等级', dataIndex: 'level', key: 'level', width: COL_WIDTH_S, dict: levelDict },
      { title: '有效用户数', dataIndex: 'userCount', key: 'userCount', width: COL_WIDTH_S },
      { title: '状态', dataIndex: 'status', key: 'status', width: COL_WIDTH_S, dict: statusDict },
      {
        title: '操作',
        key: 'action',
        getActionConfig: (text, record, index) => {
          let tempStatus = false; // 操作者和被操作者是否都是超级管理员
          const { id, status, sysRoleCode } = record;
          // 当前row是超级管理员，但是登陆者没有超级管理员身份，不可操作
          if (sysRoleCode === 'SUPER_ADMIN' && this.props.roleCode.indexOf(sysRoleCode) === -1) {
            tempStatus = true;
          }
          let actions = [{ text: '复制', onClick: () => this.goDetail(id, 'Copy') }];
          if (status === dicts.STATUS_VAILD) {
            actions.push({ text: '修改', onClick: () => this.goDetail(id, 'Modify'), disabled: tempStatus });
            actions.push({ text: '无效', onClick: () => this.setRoleStatus(id, dicts.STATUS_INVAILD) });
          } else {
            actions.push({ text: '详情', onClick: () => this.goDetail(id, 'View') });
            actions.push({ text: '启用', onClick: () => this.setRoleStatus(id, dicts.STATUS_VAILD) });
          }
          return actions;
        },
      },
    ];

    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={
          <div className={styles.wrap}>
            <SearchBar
              expand={searchExpand}
              onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
              onSearch={this.search}
              onReset={this.reset}
            >
              <Input placeholder="角色名称" value={name} onChange={(e) => this.setState({ name: e.target.value })} />
              <Select
                placeholder="角色类型"
                dict={typeDict}
                value={type}
                onChange={(val) => this.setState({ type: val })}
              />
              <Select
                placeholder="角色等级"
                dict={levelDict}
                value={level}
                onChange={(val) => this.setState({ level: val })}
              />
              <Select
                placeholder="状态"
                dict={statusDict}
                value={status}
                onChange={(val) => this.setState({ status: val })}
              />
            </SearchBar>
            <div className={styles.rightBtnWrap}>
              <Button onClick={() => this.goDetail(null, 'Add')}>新增角色</Button>
            </div>
          </div>
        }
      >
        <List
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    roleCode: state.account.user.roleCode,
  };
};

export default connect(mapStateToProps, null)(RoleManage);
