import { getRoleList, getResourceList, findRoleById, editRoleNew, updateRoleStatusById } from 'SERVICE/system';
import { getRoleForSelection } from 'SERVICE/common';
import { formateDict } from 'UTIL/dict';
import dicts from './dicts';
import { indexOf } from 'lodash';

const loadList = async (params) => {
  //{ name, type, level, status, currentPage, pageSize }
  /*
name	角色名称	string
page	页码	number
roleLevel	角色等级	number
roleStatus	角色状态	string
roleType	角色类型	string
rows	每页数量	number
sidx	排序字段	string
sord
   */

  const apiParams = {
    name: params.name,
    page: params.currentPage - 1,
    roleLevel: params.level ? parseInt(params.level) : null,
    roleStatus: params.status,
    roleType: params.type,
    rows: params.pageSize,
  };
  const { rows, records } = await getRoleList(apiParams, true);
  /*
  name	角色名称	string	@mock=$order('分公司外呼组人员','分公司教务主管','集团教学管理部教研组人员','掌上星火大区咨询负责人','学习中心管理学管','学习中心管理主任','分公司管理经理')
  realCount	有效用户数	number	@mock=$order(13,29,1,17,1,0,0)
  roleId	角色id	string	@mock=$order('ROL0000000096','ROL0000000101','ROL0000000112','ROL0000000137','specialRole','specialRole1','specialRole2')
  roleLevel	角色等级	number	@mock=$order(4,4,2,2,5,5,5)
  roleStatus	VALID-有效，INVALID-无效	string	@mock=$order('VALID','null','null','null','null','null','null')
  roleType	MANAGER-管理角色，NORMAL-普通角色	string	@mock=$order('NORMAL','null','null','null','null','null','null')
  userAmount
  */
  const list = rows.map((d) => {
    return {
      id: d.roleId,
      name: d.name,
      type: d.roleType,
      level: d.roleLevel ? d.roleLevel + '' : null,
      userCount: d.realCount,
      status: d.roleStatus,
      sysRoleCode: d.roleCode,
    };
  });
  const rtn = {
    list,
    totalRecords: records,
  };
  return rtn;
};

const loadTreeData = async (type) => {
  let apiParams = {
    rows: 2000,
  };

  if (type === 'hand') {
    apiParams = {
      rtype: 'APP',
    };
  }

  const { rows } = await getResourceList(apiParams, true);
  let data = [];

  if (rows && rows.length > 0) {
    data = rows
      .filter((ro) => ro.id !== ro.parentId)
      .map((row) => {
        return {
          id: row.id,
          name: row.rname,
          parentId: row.parentId === '-1' ? null : row.parentId,
          isOff: row.goAlive === 'N',
        };
      });
  }

  return data;
};

const loadHandTreeData = async () => {
  const data = await loadTreeData('hand');
  return data;
};

const loadDetail = async (id) => {
  /*
  const [basicData, contractData, poolData] = await Promise.all([
    loadBasicInfo({ id }),
    loadContractInfo({ id }),
    loadPoolInfo({ bossId }),
  ])
  return { basicData, contractData, poolData } */

  const apiParams = {
    roleId: id,
  };
  const data = await findRoleById(apiParams, true);

  const basicInfo = {
    name: data.name,
    type: data.roleType,
    level: data.roleLevel ? data.roleLevel + '' : null,
    status: data.roleStatus,
    remark: data.remark,
  };
  let treeChecked = [];
  let handTreeChecked = [];
  data.resources.forEach((re) => {
    if (re.rtype === 'APP') {
      handTreeChecked.push(re.id);
    } else {
      treeChecked.push(re.id);
    }
  });
  const customerInfo = {
    active: data.resourceStatus === 'VALID' ? true : false,
    volume: data.resourceNum,
    circType: data.cycleType === 0 ? '0' : data.cycleType === 1 ? '1' : null,
    cycle: data.returnCycle,
    auths: data.relateRoleIds,
  };
  // 日志数据
  const logs = data.logs;
  return {
    basicInfo,
    treeChecked,
    handTreeChecked,
    customerInfo,
    logs,
  };
};

const saveDetail = async (data) => {
  /*
{ id, mode, treeChecked, handTreeChecked, name, type, level, status, remark, customerInfo }

cycleType	回收类型 0:不回收，1：可回收;	number	@mock=1
name	角色名称	string	@mock=咨询主任
relateRoleIds	资源分配权限（角色id，数组）	array<string>	@mock=$order('4028818d46e1abf10146e1abf3fd0001','ROL000000000184','ROL0000000015','ROL0000000168')
remark	备注
resourceNum	资源容量	number	@mock=10
resourceStatus	客户资源状态VALID-有效，INVALID-无效	string	@mock=INVALID
returnCycle	回收周期	number	@mock=30
roleId	角色id（新增时不填）	string	@mock=ROL000000000184
roleLevel	角色等级	number	@mock=3
roleSign	角色sign（新增时为角色名称首字母小写，修改时不变）	string	@mock=zxzr
roleStatus	角色状态VALID-有效，INVALID-无效	string	@mock=VALID
roleType	角色类型	string	@mock=NORMAL
selectedResIds	资源id（逗号隔开）	string	@mock=2,getCustomerByContact_CALL_OUT,appointmentCustomers_CALL_OUT,distributeCustomers_CALL_OUT,followupCustomers_CALL_OUT,inputCustomer_CALL_OUT
  */
  try {
    const { active, volume, circType, cycle, auths } = data.customerInfo;

    let params = {
      cycleType: circType ? parseInt(circType) : null,
      name: data.name,
      relateRoleIds: auths,
      remark: data.remark,
      resourceNum: volume,
      resourceStatus: active ? 'VALID' : 'INVALID',
      returnCycle: cycle,
      roleLevel: data.level ? parseInt(data.level) : null,
      //    roleSign: data.xxx,
      roleStatus: data.status,
      roleType: data.type,
      opReason: data.opReason,
      selectedResIds: [...data.treeChecked, ...data.handTreeChecked].join(','),
    };
    if (data.id) {
      params['roleId'] = data.id;
    }

    const apiData = await editRoleNew(params, true);
    return apiData.data;
  } catch (e) {
    console.log(e);
    throw e;
  }
};

const setRoleStatus = async (id, status) => {
  const params = {
    roleId: id,
    roleStatus: status,
    type: 'role',
  };
  try {
    await updateRoleStatusById(params, true);
    return true;
  } catch (e) {
    throw e;
  }
};

const setCustomerStatus = async (id, active) => {
  try {
    const params = {
      roleId: id,
      roleStatus: active ? 'VALID' : 'INVALID',
      type: 'pool',
    };
    await updateRoleStatusById(params, true);
    return true;
  } catch (e) {
    throw e;
  }
};

const loadRoleDict = async () => {
  const data = await getRoleForSelection({}, true);
  return formateDict(data);
};

export default {
  loadList,
  saveDetail,
  loadRoleDict,
  setRoleStatus,
  setCustomerStatus,
  loadTreeData,
  loadHandTreeData,
  loadDetail,
};
