import React from 'react';
import Modal from 'COMPONENT/Modal';
import styles from './styles.less';
import Button from 'COMPONENT/Button';
import Block from 'COMPONENT/Block';
import AuthBlock from '../AuthBlock';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import getWrappedInstance from 'UTIL/getWrappedInstance';

class AuthSettingModal extends React.Component {
  constructor(props) {
    super(props);
    this.errors = {}; //校验错误
    this.rowIdx = 1; //用于生成唯一的行key

    const { initOrgDict } = this.props;
    this.initOrgDictObj = initOrgDict.reduce((obj, ele) => {
      obj[ele.val] = ele.text;
      return obj;
    }, {});
    const data = props.data.map((ele) => {
      return {
        primary: ele.primary,
        org: ele.orgType,
        role: ele.roleId,
        orgName: this.initOrgDictObj[ele.orgType] || ele.orgType,
        roleName: ele.roleName,
      };
    });
    const mainAuth = data.find((d) => d.primary) || {
      primary: true,
      org: '',
      role: '',
      orgName: '',
      roleName: '',
    };
    const otherAuths = data
      .filter((d) => !d.primary)
      .map((da) => {
        return { ...da, id: '' + this.rowIdx++ };
      });
    this.state = {
      loading: false, // {boolean} 等待状态
      mainAuth, // {object} 主组织架构
      otherAuths, // {[{ org, role }]} 其他组织架构

      checking: false, //校验开启状态
    };
  }

  onOk = () => {
    const { checking } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    const errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      this.modal.showOkTips(errorTips);
      return;
    }
    if (this.checkDup()) {
      this.modal.showOkTips('当前存在相同的组织架构及角色设置');
      return;
    }
    const { mainAuth, otherAuths } = this.state;
    const data = [
      { primary: true, orgType: mainAuth.org, roleId: mainAuth.role, roleName: mainAuth.roleName },
      ...otherAuths.map((a) => {
        return { primary: false, orgType: a.org, roleId: a.role, roleName: a.roleName };
      }),
    ];
    this.props.onOk(data);
  };

  onErrorChange = (key, error) => {
    this.errors[key] = error;
  };

  onMainChange = (changedKv) => {
    this.setState({ mainAuth: { ...this.state.mainAuth, ...changedKv } });
  };

  onOtherChange = (id, changedKv) => {
    const otherAuths = this.state.otherAuths;
    this.setState({
      otherAuths: otherAuths.map((a) => {
        if (a.id === id) {
          return { ...a, ...changedKv };
        }
        return a;
      }),
    });
  };

  onAdd = () => {
    const newOtherAuths = [
      { org: null, role: null, orgName: null, roleName: null, id: '' + this.rowIdx++ },
      ...this.state.otherAuths,
    ];
    this.setState({ otherAuths: newOtherAuths });
  };

  onDelete = (id) => {
    let newErrors = {};
    Object.keys(this.errors).forEach((key) => {
      const a = key.split('_');
      if (a[1] !== id) {
        newErrors[key] = this.errors[key];
      }
    });
    this.errors = newErrors;
    this.setState({ otherAuths: this.state.otherAuths.filter((a) => a.id !== id) });
  };

  checkDup = () => {
    const { mainAuth, otherAuths } = this.state;
    const auths = [mainAuth, ...otherAuths].filter((a) => a.org && a.role);
    return auths.some((a) =>
      auths.filter((au) => au.id !== a.id).some((aut) => aut.org === a.org && aut.role === a.role),
    );
  };

  render() {
    const { onCancel, initOrgDict, sysRoleDict } = this.props;
    const { loading, mainAuth, otherAuths, checking } = this.state;

    return (
      <Modal
        className={styles.wrap}
        loading={loading}
        title="组织架构及角色设置"
        width={920}
        onOk={this.onOk}
        onCancel={onCancel}
        noLine
        fixedLoading
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        <Block title="主组织">
          <AuthBlock
            title="组织架构"
            value={mainAuth}
            onChange={this.onMainChange}
            checking={checking}
            onErrorChange={(field, e) => this.onErrorChange(`${field}_main`, e)}
            initOrgDict={initOrgDict}
            sysRoleDict={sysRoleDict}
          />
        </Block>
        <Block title="其他组织" headerToolBar={<Button onClick={this.onAdd}>添加其他组织</Button>}>
          {otherAuths.map((a, i) => {
            return (
              <AuthBlock
                key={a.id}
                title={`组织架构${i + 1}`}
                value={a}
                onChange={(changedKv) => this.onOtherChange(a.id, changedKv)}
                checking={checking}
                onErrorChange={(field, e) => this.onErrorChange(`${field}_${a.id}`, e)}
                onDelete={() => this.onDelete(a.id)}
                initOrgDict={initOrgDict}
                sysRoleDict={sysRoleDict}
              />
            );
          })}
        </Block>
      </Modal>
    );
  }
}

export default AuthSettingModal;
