import React from 'react';
import { connect } from 'dva';
import Input from 'COMPONENT/Input';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import loaders from './loaders';
import jslog from 'SRC/jslog';
import { loadCommonDict } from 'UTIL/dict';

const configuredDict = [
  { val: true, text: '是' },
  { val: false, text: '否' },
];

class RoleTemplateManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态

      stationName: null,
      includedRole: null,
      configured: null,

      sysRoleDict: null, // {[object]} 字典 系统角色
      initOrgDict: null, // {[object]} 字典 初始显示的组织架构

      list: [], // {Array} 列表数据
      listSearchKeys: {
        stationName: null,
        includedRole: null,
        configured: null,
      }, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = () => {
    this.setState({ loading: true });
    Promise.all([
      loadCommonDict('OrganizationType'),
      loaders.loadSysRoleDict(),
      loaders.loadList(this.getListParams()),
    ]).then(([initOrgDict, sysRoleDict, listData]) => {
      const validStr = ['DEPARTMENT', 'BRENCH', 'GROUNP', 'CAMPUS'].join(',');
      let orgDict = [];
      initOrgDict.forEach((item) => {
        if (validStr.indexOf(item.val) > -1) {
          if (item.val === 'CAMPUS') {
            item.text = '学习中心';
          }
          orgDict.push(item);
        }
      });
      const { list, totalRecords } = listData;
      this.setState({ loading: false, initOrgDict: orgDict, sysRoleDict, list, totalRecords });
    });
  };

  getListParams = () => {
    const {
      listSearchKeys: { stationName, includedRole, configured },
      currentPage,
      pageSize,
    } = this.state;
    return { stationName, includedRole, configured, currentPage, pageSize };
  };

  search = () => {
    const { stationName, includedRole, configured } = this.state;
    const listSearchKeys = { stationName, includedRole, configured };
    this.setState({ loading: true });
    loaders.loadList({ ...this.getListParams(), ...listSearchKeys, currentPage: 1 }).then(({ list, totalRecords }) => {
      jslog.report('query', {
        function_name: '系统管理>人事岗位角色配置>查询',
        page: 1,
        total: totalRecords,
        size: list ? list.length : 0,
      });
      this.setState({ loading: false, listSearchKeys, currentPage: 1, list, totalRecords });
    });
  };

  reset = () => {
    this.setState({ stationName: null, includedRole: null, configured: null });
  };

  editSaveSuccess = () => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: { path: '/RoleTemplateManage' },
    });
    this.search();
  };

  goDetail = (id, type) => {
    const { dispatch } = this.props;
    const { initOrgDict, sysRoleDict } = this.state;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/RoleTemplateManage/Detail',
        params: {
          type,
          id,
          onSaveSuccess: this.editSaveSuccess,
          initOrgDict,
          sysRoleDict,
        },
        refresh: true,
      },
    });
  };

  onPaginationChange = ({ current, pageSize }) => {
    this.setState({ loading: true });
    const changed = {
      currentPage: current,
      pageSize,
    };
    this.setState({ loading: true });
    loaders.loadList({ ...this.getListParams(), ...changed }).then(({ list, totalRecords }) => {
      this.setState({ loading: false, ...changed, list, totalRecords });
    });
  };

  render() {
    const {
      loading,
      searchExpand,
      stationName,
      includedRole,
      configured,
      sysRoleDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
    } = this.state;

    const columns = [
      { title: '人事岗位', key: 'stationName', dataIndex: 'stationName' },
      { title: '是否已配置', key: 'configured', dataIndex: 'configured' },
      { title: '角色配置', key: 'roleName', dataIndex: 'roleName', lines: 3 },
      {
        title: '操作',
        key: 'action',
        getActionConfig: (text, record, index) => {
          const { stationId } = record;
          return [
            { text: '修改', onClick: () => this.goDetail(stationId) },
            { text: '查看详情', onClick: () => this.goDetail(stationId, 'view') },
          ];
        },
      },
    ];

    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={
          <SearchBar
            expand={searchExpand}
            onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
            onSearch={this.search}
            onReset={this.reset}
          >
            <Input
              placeholder="人事岗位名称"
              value={stationName}
              onChange={(e) => this.setState({ stationName: e.target.value })}
            />
            <Select
              placeholder="是否已配置"
              dict={configuredDict}
              value={configured}
              onChange={(val) => this.setState({ configured: val })}
            />
            <Select
              placeholder="系统角色"
              dict={sysRoleDict}
              showSearch
              value={includedRole}
              onChange={(val) => this.setState({ includedRole: val })}
            />
          </SearchBar>
        }
      >
        <List
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
        />
      </Page>
    );
  }
}

export default connect()(RoleTemplateManage);
