import React from 'react';
import { Button, Row, Col } from 'antd';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Modal from 'COMPONENT/Modal';
import getWindowSize from 'UTIL/getWindowSize';
import { getAllBranch, getDeptAndCampusByBranchId } from 'SERVICE/common';
import { findOrganization, updateOrgStatusById, updateBatchOrgStatusById } from 'SERVICE/system';
import styles from './styles.less';
import Loading from 'COMPONENT/Loading';
import loaders from './loaders';
import Icon from 'COMPONENT/Icon';
import IconButton from 'COMPONENT/IconButton';
import AddUserModal from './AddUserModal';

//状态字典
const statusDict = [
  { val: 'NORMAL', text: '正常' },
  { val: 'EMERGENCY', text: '紧急' },
];

class SwitchEmergencyConfig extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // boolean 页面加载状态
      branchId: null, // string 查询条件，分公司ID
      campusId: null, // string 查询条件，学习中心ID
      status: null, // string 查询条件，状态
      branchDict: [], // [{val, text}] 分公司字典
      campusDict: [], // [{val, text}] 学习中心字典
      list: null, // 列表数据
      detailModalVisible: false, // boolean 批量操作弹框显示状态
      activeBranchId: null, // string 当前正在操作的分公司ID
      activeBranchName: null, // string 当前正在操作的分公司名称
      activeCampusId: null,
      activeCampusName: null,
      branchsUsers: {},
      addModalVisible: false,
      winW: 0, //  number  窗口宽度
      modalLoading: false,
    };
  }

  componentWillMount() {
    this.initContent();
    this.updateWinW();
    window.addEventListener('resize', this.updateWinW, false);
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.updateWinW, false);
    this.isUnmounted = true;
  }

  /**
   * 获取和更新窗口宽度状态
   */
  updateWinW = () => {
    this.updateState({ winW: getWindowSize().width });
  };

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      if (!this.isUnmounted) {
        this.setState(newState, () => {
          resolve();
        });
      } else {
        resolve();
      }
    });
  };

  /**
   * 初始化内容
   */
  initContent = async () => {
    await this.updateState({ loading: true });
    const [branchDict] = await Promise.all([this.loadBranchDict()]);
    await this.updateState({ branchDict });
    await this.search();
  };

  /**
   * 加载分公司字典
   */
  loadBranchDict = async () => {
    const data = await getAllBranch();
    return data.map((b) => {
      return { val: b.organizationId, text: b.organizationName };
    });
  };

  /**
   * 加载学习中心字典
   * @param {string} branchId 分公司ID
   */
  loadCampusDict = async (branchId) => {
    const data = await getDeptAndCampusByBranchId({ organizationId: branchId });
    return data.map((b) => {
      return { val: b.organizationId, text: b.organizationName };
    });
  };

  /**
   * 查询并更新数据
   */
  search = async () => {
    const { branchId, campusId, status } = this.state;
    await this.updateState({ loading: true });
    const list = await loaders.loadList(branchId, campusId, status);
    await this.updateState({
      loading: false,
      list,
      branchsUsers: {},
    });
    return true;
  };

  /**
   * 重置查询条件
   */
  resetSearchKeys = async () => {
    await this.updateState({
      branchId: null,
      campusId: null,
      campusDict: [],
      status: null,
    });
    return true;
  };

  /**
   * 处理分公司值变化，清空学习中心值，重新加载学习中心选项
   * @param  {string} branchId 分公司ID
   */
  branchChange = (branchId) => {
    this.updateState({ branchId, campusId: null, campusDict: [] });
    if (branchId) {
      this.loadCampusDict(branchId).then((campusDict) => {
        this.updateState({ campusDict });
      });
    }
  };

  /**
   * 点击列表中分公司标题时，切换展开状态
   * @param  {string} branchId 分公司ID
   */
  branchTitleClick = async (branchId, i) => {
    const { list } = this.state;
    const { expand, loading } = list[i];
    list[i] = { ...list[i], expand: !expand, loading: !loading };
    await this.updateState({
      list: list,
    });
    await this.loadBranchUsers(branchId);
    list[i]['loading'] = !list[i]['loading'];
    await this.updateState({
      list: list,
    });
  };

  loadBranchUsers = async (branchId) => {
    const { branchsUsers } = this.state;
    if (branchsUsers[branchId]) {
      return;
    }
    const branchUsers = await loaders.loadBranchUsers({ brenchId: branchId });
    branchsUsers[branchId] = branchUsers;
    return this.updateState({
      branchsUsers: branchsUsers,
    });
  };

  showDetailModal = (params) => {
    this.updateState({
      detailModalVisible: true,
      activeBranchId: params.branchId,
      activeBranchName: params.branchName,
      activeCampusId: params.campusId,
      activeCampusName: params.campusName,
    });
  };

  /**
   * 关闭批量操作弹框
   */
  closeDetailModal = () => {
    this.updateState({
      detailModalVisible: false,
      activeBranchId: null,
      activeBranchName: null,
      activeCampusId: null,
      activeCampusName: null,
    });
  };

  onIconBtnClick = async ({ orgId, userId }) => {
    const { branchsUsers, activeBranchId, activeCampusId } = this.state;
    this.setState({ modalLoading: true });
    let users = branchsUsers[activeBranchId][activeCampusId];

    const updateUsers = users.filter((ele) => ele.userId !== userId);
    const orgIds = [];
    const userIds = [];
    console.log(users);
    console.log(updateUsers);
    updateUsers.map((ele) => {
      orgIds.push(ele.orgId);
      userIds.push(ele.userId);
    });
    if (!userIds.length) {
      orgIds.push(orgId);
    }
    await loaders.updateUsers({ orgIds: orgIds.join(','), userIds: userIds.join(',') });

    branchsUsers[activeBranchId][activeCampusId] = [...updateUsers];
    return this.updateState({
      branchsUsers: branchsUsers,
      modalLoading: false,
    });
  };

  onAddUserOk = async (addedUsers) => {
    const { branchsUsers, activeBranchId, activeCampusId } = this.state;
    this.setState({ modalLoading: true });
    if (!branchsUsers[activeBranchId]) {
      branchsUsers[activeBranchId] = {};
    }
    if (!branchsUsers[activeBranchId][activeCampusId]) {
      branchsUsers[activeBranchId][activeCampusId] = [];
    }
    let users = branchsUsers[activeBranchId][activeCampusId];

    const updateUsers = [...users, ...addedUsers];
    const orgIds = [];
    const userIds = [];
    updateUsers.map((ele) => {
      orgIds.push(activeCampusId);
      userIds.push(ele.userId);
    });
    await loaders.updateUsers({ orgIds: orgIds.join(','), userIds: userIds.join(',') });

    branchsUsers[activeBranchId][activeCampusId] = [...updateUsers];
    return this.updateState({
      branchsUsers: branchsUsers,
      modalLoading: false,
    });
  };

  render() {
    const {
      loading,
      branchId,
      campusId,
      status,
      branchDict,
      campusDict,
      list,
      detailModalVisible,
      branchsUsers,
      activeBranchId,
      activeBranchName,
      activeCampusId,
      activeCampusName,
      addModalVisible,
      winW,
      modalLoading,
    } = this.state;
    const colCount = winW < 1500 ? 3 : winW < 1700 ? 4 : 6;

    return (
      <Page loading={loading} route={this.props.route}>
        <SearchBar onSearch={this.search} onReset={this.resetSearchKeys}>
          <Select showSearch placeholder="分公司" dict={branchDict} value={branchId} onChange={this.branchChange} />
          <Select
            showSearch
            placeholder="分公司部门或学习中心"
            dict={campusDict}
            value={campusId}
            onChange={(campusId) => this.updateState({ campusId })}
          />
          <Select
            placeholder="状态"
            dict={statusDict}
            value={status}
            onChange={(status) => this.updateState({ status })}
          />
        </SearchBar>
        <div>
          {list &&
            list.map((b, i) => {
              const { id: branchId, name, children, expand, loading } = b;
              return (
                <div
                  key={branchId}
                  className={styles.branchItem + (expand ? ' ' + styles.expand : '')}
                  style={{ height: expand ? 'auto' : 51 }}
                >
                  <div className={styles.branchTitle} onClick={() => this.branchTitleClick(branchId, i)}>
                    <span className={styles.branchName}>{name}</span>
                  </div>
                  <div className={styles.branchContent}>
                    {loading ? (
                      <div style={{ width: '100%', height: 80, position: 'relative' }}>
                        <Loading loading={true} />
                      </div>
                    ) : (
                      children &&
                      children.map((c, j) => {
                        const brancsUsers = branchsUsers[branchId] ? branchsUsers[branchId][c.id] || [] : [];
                        return (
                          <div key={c.id} className={styles.row}>
                            <Row>
                              <Col span={1}>
                                <div className={styles.dashedDot}></div>
                                {j !== children.length - 1 && <div className={styles.dashedLine}></div>}
                              </Col>
                              <Col span={23}>
                                <span className={styles.blockTitle}>{c.name}</span>
                              </Col>
                            </Row>
                            <Row>
                              <div className={styles.lastRow}>
                                <Col offset={1} span={22}>
                                  {brancsUsers.slice(0, 5).map((ele) => {
                                    return (
                                      <Col key={ele.userId} span={4}>
                                        <span>{`${ele.userName}(${ele.employeeNo})`}</span>
                                      </Col>
                                    );
                                  })}
                                  {brancsUsers.length > 5 && (
                                    <Col key="etc" span={4}>
                                      <span>…</span>
                                    </Col>
                                  )}
                                </Col>
                                <Col span={1}>
                                  <a
                                    onClick={() =>
                                      this.showDetailModal({
                                        branchId,
                                        campusId: c.id,
                                        branchName: name,
                                        campusName: c.name,
                                      })
                                    }
                                  >
                                    编辑
                                  </a>
                                </Col>
                              </div>
                            </Row>
                          </div>
                        );
                      })
                    )}
                  </div>
                </div>
              );
            })}
        </div>
        <Modal
          key="detailModal"
          toggle={true}
          visible={detailModalVisible}
          loading={modalLoading}
          okBtnShow={false}
          cancelText="关闭"
          onCancel={this.closeDetailModal}
          width={1366}
          title={activeCampusName}
        >
          <Row>
            <Col span={21}>
              <span style={{ margin: '10px 0', display: 'block', fontSize: '16px' }}>已配置账户</span>
            </Col>
            <Col span={3}>
              <div style={{ margin: '10px 0', display: 'flex' }}>
                <span className={styles.square}>
                  <span className={styles.block + ' ' + styles.active}></span>
                  <span>在职</span>
                </span>
                <span className={styles.square}>
                  <span className={styles.block + ' ' + styles.inactive}></span>
                  <span>离职</span>
                </span>
              </div>
            </Col>
          </Row>
          <Row>
            <Col span={24}>
              <Button
                type="primary"
                style={{ display: 'flex', margin: '5px 0 25px' }}
                onClick={() => {
                  this.setState({ addModalVisible: true });
                }}
              >
                <Icon type="plus" size={13} color="white" />
                <span style={{ marginLeft: '6px' }}>新增</span>
              </Button>
            </Col>
          </Row>
          <Row gutter={40}>
            {!branchsUsers[activeBranchId] || !branchsUsers[activeBranchId][activeCampusId] ? (
              <Col span={24}>
                <div>暂无用户</div>
              </Col>
            ) : (
              branchsUsers[activeBranchId][activeCampusId].map((ele) => {
                return (
                  <Col key={ele.userId} span={24 / colCount}>
                    <div className={styles.campusItem + ' ' + (ele.employeeStatu === 0 && styles.campusItemActive)}>
                      <span
                        className={styles.campusTitle}
                        title={`${ele.userName}(${ele.employeeNo})`}
                      >{`${ele.userName}(${ele.employeeNo})`}</span>
                      <div className={styles.campusStatus}>
                        {ele.employeeStatu === 0 ? (
                          <IconButton
                            type={'cross_circle'}
                            color={'#8d88ff'}
                            onClick={() =>
                              this.onIconBtnClick({
                                orgId: activeCampusId,
                                userId: ele.userId,
                              })
                            }
                          />
                        ) : (
                          <IconButton
                            type={'cross_circle'}
                            onClick={() =>
                              this.onIconBtnClick({
                                orgId: activeCampusId,
                                userId: ele.userId,
                              })
                            }
                          />
                        )}
                      </div>
                    </div>
                  </Col>
                );
              })
            )}
          </Row>
        </Modal>
        {addModalVisible && (
          <AddUserModal
            key="addUserModal"
            onCancel={() => {
              this.setState({ addModalVisible: false });
            }}
            campusId={activeCampusId}
            onOk={this.onAddUserOk}
          />
        )}
      </Page>
    );
  }
}

export default SwitchEmergencyConfig;
