import React from 'react';
import { Page, SearchBar, Select } from 'COMPONENT';
import { STATUS_NORMAL, statusDict } from './dict';
import ResourceList from './ResourceList';
import * as loaders from './loader';
import * as utils from './util';

class SwitchEmergencyResource extends React.Component {
  constructor(props) {
    super(props);

    this.initState = {
      branchId: null,
      status: null,
    };
    this.state = {
      loading: false,
      list: null, // 紧急资源列表
      branchId: null, // 分公司ID
      status: null, // 紧急状态
      branchDict: [], // 分公司字典
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   * tips: 需求需要，分公司字典用首次加载的列表的组织结构
   */
  initContent = async () => {
    this.loadList((list) => {
      this.setState({
        branchDict: utils.formatResourceBranchDict(list),
      });
    });
  };

  /**
   * 获取紧急资源列表
   */
  loadList = (callback) => {
    const params = this.formatSearchParam();

    this.toggleLoading();
    loaders
      .loadResourceList(params)
      .then((list) => {
        this.toggleLoading();
        this.setState({ list });

        if (callback) {
          callback(list);
        }
      })
      .catch(() => this.toggleLoading());
  };

  /**
   * 格式化请求参数
   * tips: emergencyStatus {Boolean} 紧急资源状态：false正常，true紧急
   *
   * @return {{branchId: (null|*), emergencyStatus: (null|*)}}
   */
  formatSearchParam = () => {
    const { branchId, status } = this.state;

    return { branchId, emergencyStatus: utils.formatApiStatus(status) };
  };

  /**
   * 搜索
   */
  onSearch = () => {
    this.loadList();
  };

  /**
   * 重置
   */
  onReset = () => {
    this.setState(this.initState);
  };

  /**
   * 分公司选中
   *
   * @param val {string}
   */
  onBranchChange = (val) => {
    this.setState({ branchId: val });
  };

  /**
   * 切换状态之前(2.X switch组件不支持loading属性)
   *
   * @param id {string} 分公司资源列表组织ID
   */
  onBeforeSwitch = (id) => {
    const list = this.state.list.map((item) => {
      if (item.id !== id) {
        return item;
      }

      return {
        ...item,
        loading: true,
      };
    });

    this.setState({ list });
  };

  /**
   * 完成切换
   *
   * @param id {string} 分公司资源列表组织ID
   * @param type {string} 是否切换成功
   */
  onAfterSwitch = (id, type) => {
    const isSuccess = type === 'success';

    const branch = this.state.list.find((item) => item.id === id);
    const status = branch.status || STATUS_NORMAL;
    const nextStatus = statusDict.find((s) => s.val !== status);
    const newStatus = isSuccess ? nextStatus.val : status;

    const list = this.state.list.map((item) => {
      if (item.id !== id) {
        return item;
      }

      return {
        ...item,
        loading: false,
        status: newStatus,
      };
    });

    this.setState({ list });
  };

  toggleLoading = () => {
    this.setState((prevState) => {
      return {
        loading: !prevState.loading,
      };
    });
  };

  render() {
    return (
      <Page
        loading={this.state.loading}
        route={this.props.route}
        subHeader={
          <SearchBar onSearch={this.onSearch} onReset={this.onReset}>
            <Select
              showSearch
              placeholder="分公司"
              dict={this.state.branchDict}
              value={this.state.branchId}
              onChange={this.onBranchChange}
            />
            <Select
              placeholder="状态"
              dict={statusDict}
              value={this.state.status}
              onChange={(val) => this.setState({ status: val })}
            />
          </SearchBar>
        }
      >
        <ResourceList
          list={this.state.list}
          beforeSwitch={this.toggleLoading}
          afterSwitch={(id, type) => {
            this.onAfterSwitch(id, type);
            this.toggleLoading();
          }}
        />
      </Page>
    );
  }
}

export default SwitchEmergencyResource;
