import React from 'react';
import { connect } from 'dva';
import { Row, Col, Select, Collapse } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

import { getTeacherCourseSchedule, api } from 'SERVICE/teacherPlan';
import { getCampusByUserId } from 'SERVICE/common';
import { Button, FormItem, InputNumber } from 'COMPONENT';

import style from './style.less';

const Option = Select.Option;
const OptGroup = Select.OptGroup;
const Panel = Collapse.Panel;
const { addTeacherCourseSchedule, updateTeacherCourseSchedule } = api;

function formatCampus(data) {
  const dict = data.map((branchCampus) => {
    const label = branchCampus[0].name;
    const val = branchCampus[0].id;
    branchCampus.shift();
    const options = branchCampus.map((campus) => {
      return {
        val: campus.id,
        text: campus.name,
      };
    });
    return {
      label,
      val,
      options,
    };
  });
  return dict;
}

class IntelligentSchedule extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      isLimitRegular: false,
      showRegularModal: false,
      loading: false,
      form: {
        campusIds: [],
        maxSubjectNumber: '',
        maxGradeNumber: '',
        maxClassLevelNumber: '',
      },
      detail: null,
      teacherBelongCapmus: [],
    };
  }

  componentDidMount() {
    if (this.props.teacherId) {
      this.fetchSchedule(this.props.teacherId);
    }
  }

  componentWillReceiveProps(props, preProps) {
    if (props.teacherId !== preProps.teacherId) {
      this.fetchSchedule(props.teacherId);
    }
  }

  onFieldChange = (key, value) => {
    this.setState({
      form: {
        ...this.state.form,
        [key]: typeof value == 'string' ? value.replace(/[^\d]/g, '') : value,
      },
    });
  };

  fetchTeacherBelongCampus = (teacherId) => {
    getCampusByUserId({
      userId: teacherId,
    }).then((res) => {
      this.setState({
        teacherBelongCapmus: formatCampus(res || []),
      });
    });
  };

  fetchSchedule = (teacherId) => {
    this.fetchTeacherBelongCampus(teacherId);
    getTeacherCourseSchedule(teacherId || this.props.teacherId)().then((res) => {
      let detail = res.data;
      if (res.data) {
        this.setState({
          detail,
          form: {
            campusIds: detail.campusList.map((c) => c.id),
            maxSubjectNumber: detail.maxSubjectNumber,
            maxGradeNumber: detail.maxGradeNumber,
            maxClassLevelNumber: detail.maxClassLevelNumber,
          },
        });
      } else {
        this.setState({
          detail: null,
          form: {
            campusIds: [],
            maxSubjectNumber: '2',
            maxGradeNumber: '2',
            maxClassLevelNumber: '2',
          },
        });
      }
    });
  };

  getFormValue = () => {
    const { form } = this.state;
    const params = {
      ...form,
      teacherId: this.props.teacherId,
    };
    return params;
  };

  save = () => {
    let rqeuest;
    this.setState({
      loading: true,
    });

    if (!this.state.detail) {
      rqeuest = this.onAdd();
    } else {
      rqeuest = this.onUpdate();
    }

    rqeuest
      .then(() => {
        this.fetchSchedule();
        showMessage('保存成功', 'success');
      })
      .finally(() => {
        this.setState({
          loading: false,
        });
      });
  };

  onAdd = () => {
    return addTeacherCourseSchedule(this.getFormValue());
  };

  onUpdate = () => {
    return updateTeacherCourseSchedule({
      ...this.getFormValue(),
      id: this.state.detail.id,
    });
  };

  render() {
    const { hasSaveIntelligentRegularPermission } = this.props;
    const { form, loading, teacherBelongCapmus } = this.state;

    return (
      <Collapse>
        <Panel header="智能排课规则" key="1">
          <Row>
            <Col span={8}>
              <FormItem style={{ paddingLeft: 0 }} labelWidth={140} name="可选学习中心（多选）" required={true}>
                <Select
                  placeholder="可选学习中心"
                  mode="multiple"
                  value={form.campusIds}
                  disabled={!hasSaveIntelligentRegularPermission}
                  onChange={(e) => {
                    this.onFieldChange('campusIds', e);
                  }}
                  filterOption={function (val, option) {
                    return option.props.children.match(val);
                  }}
                >
                  {teacherBelongCapmus.map((c) => {
                    return (
                      <OptGroup key={c.val} label={c.label}>
                        {c.options.map((o) => {
                          return (
                            <Option key={o.val} value={o.val}>
                              {o.text}
                            </Option>
                          );
                        })}
                      </OptGroup>
                    );
                  })}
                </Select>
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={8}>
              <FormItem style={{ paddingLeft: 0 }} labelWidth={120} name="最多带科目数：" required={true}>
                <InputNumber
                  placeholder="最多带科目数"
                  min={1}
                  value={form.maxSubjectNumber}
                  disabled={!hasSaveIntelligentRegularPermission}
                  onChange={(e) => {
                    this.onFieldChange('maxSubjectNumber', e);
                  }}
                />
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem style={{ paddingLeft: 0 }} labelWidth={120} name="最多带层级数：" required={true}>
                <InputNumber
                  placeholder="最多带层级数"
                  min={1}
                  value={form.maxClassLevelNumber}
                  disabled={!hasSaveIntelligentRegularPermission}
                  onChange={(e) => {
                    this.onFieldChange('maxClassLevelNumber', e);
                  }}
                />
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem style={{ paddingLeft: 0 }} labelWidth={120} name="最多带年级数：" required={true}>
                <InputNumber
                  placeholder="最多带年级数"
                  min={1}
                  value={form.maxGradeNumber}
                  disabled={!hasSaveIntelligentRegularPermission}
                  onChange={(e) => {
                    this.onFieldChange('maxGradeNumber', e);
                  }}
                />
              </FormItem>
            </Col>
          </Row>
          <div>
            {hasSaveIntelligentRegularPermission && (
              <Button type="primary" onClick={this.save} loading={loading}>
                保存
              </Button>
            )}
          </div>
        </Panel>
      </Collapse>
    );
  }
}

export default connect((state) => {
  const { btnAuths } = state.account;
  const hasSaveIntelligentRegularPermission = btnAuths.indexOf('SAVE_INTELLIGENT_SCHEDULE') > -1;

  return {
    CAMPUS: state.dict['branchAllCampus'],
    hasSaveIntelligentRegularPermission,
  };
})(IntelligentSchedule);
