import React from 'react';
import { connect } from 'dva';
import { Modal, Row, Col, Checkbox } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import {
  getConfigRegularList,
  addTeacherRegular,
  updateTeacherRegular,
  deleteTeacherRegular,
  getTeacherScheduleDetail,
} from 'SERVICE/teacherSchedule';
import { FormItem, InputNumber, Loading, Select } from 'COMPONENT';

import style from './style.less';

const seasonMap = {
  spring: 'DAT0000000231',
  summer: 'DAT0000000232',
  autumn: 'DAT0000000233',
  winter: 'DAT0000000230',
};

// 按季度格式化规则用于渲染使用
function formatRegular(regular = [], system = []) {
  // 最大值不能超过已配置的系统规则值
  regular.forEach((cur) => {
    if (system && system.length) {
      system.forEach((s) => {
        if (s.termNumber == cur.termNumber && s.quarterId == cur.quarterId) {
          cur.max = s.maxClassNumber;
        }
      });
    } else {
      cur.max = cur.maxClassNumber;
    }
  });

  const data = regular.reduce((pre, cur) => {
    if (!pre[cur.quarterId]) {
      pre[cur.quarterId] = {
        quarterName: cur.quarterName,
        quarterId: cur.quarterId,
        children: [cur],
      };
    } else {
      pre[cur.quarterId].children.push(cur);
    }

    return pre;
  }, {});

  return Object.keys(data).map((k) => data[k]);
}

// 格式化规则成请求接口时需要的格式
function formatRegularForRequest(regularList) {
  return regularList.reduce((pre, cur) => {
    return pre.concat(
      cur.children.map((r) => {
        return {
          quarterId: r.quarterId,
          termNumber: r.termNumber,
          maxClassNumber: r.maxClassNumber,
          salaryCoefficient: r.salaryCoefficient,
        };
      }),
    );
  }, []);
}

function isSummerOrWinter(quarterId) {
  return quarterId === seasonMap.summer || quarterId === seasonMap.winter;
}

class LimitRegularModal extends React.Component {
  constructor(props) {
    super(props);
    const currentYear = new Date().getFullYear();
    const currentYearData = props.YEAR.find((y) => y.text == currentYear);

    this.state = {
      loading: false,
      // 可选系统规则列表
      regularMap: {},
      systemRegular: [],
      // 表单相关字段
      isLimitRegular: false,
      currentSelectedRegular: '',
      salaryCoefficient: '',
      yearId: currentYearData ? currentYearData.val : '',
      // 用于填充展示的详细规则
      regularList: [],
      detail: null,
    };
  }

  async componentDidMount() {
    await this.fetchConfigRegular();
    this.fetchSchedule();
  }
  // 获取可用系统规则
  fetchConfigRegular() {
    return getConfigRegularList({
      type: 1,
      status: 1,
      pageSize: 999,
      pageNum: 1,
      withDetails: true,
    }).then((res) => {
      const data = res.data.list || [];
      const systemRegular = data.map((r) => ({
        val: String(r.id),
        text: r.name,
      }));
      const regularMap = data.reduce((pre, cur) => {
        pre[cur.id] = cur;
        return pre;
      }, {});

      this.setState({
        systemRegular: systemRegular,
        regularMap: regularMap,
      });
    });
  }

  // 获取已设置的规则详情
  fetchSchedule = () => {
    this.setState({
      loading: true,
    });
    return getTeacherScheduleDetail({
      type: 99,
      objectId: this.props.teacherId,
      yearId: this.state.yearId,
    })
      .then((res) => {
        const detail = res.data;
        if (detail) {
          const system = this.state.regularMap[detail.parentId] || {};
          this.setState({
            detail,
            isLimitRegular: true,
            salaryCoefficient: detail.salaryCoefficient,
            currentSelectedRegular: String(detail.parentId),
            regularList: formatRegular(detail.mcNumberLimitRuleDetailVos, system.mcNumberLimitRuleDetailVos),
          });
        } else {
          this.setState({
            detail: null,
            isLimitRegular: false,
            salaryCoefficient: '',
            currentSelectedRegular: '',
            regularList: [],
          });
        }
      })
      .finally(() => {
        this.setState({
          loading: false,
        });
      });
  };

  // 更换系统规则
  onSystemRegularChange = (id) => {
    const selected = this.state.regularMap[id];
    this.setState({
      regularList: formatRegular(selected.mcNumberLimitRuleDetailVos),
      currentSelectedRegular: id,
      salaryCoefficient: selected.salaryCoefficient,
    });
  };

  onYearChange = (yearId) => {
    this.setState(
      {
        yearId,
      },
      this.fetchSchedule,
    );
  };

  // 课程系数修改，子规则的值联动
  onCoefficientChange = (val) => {
    const newRegularList = this.state.regularList.map((r) => {
      r.children = r.children.map((item) => {
        return {
          ...item,
          salaryCoefficient: val,
        };
      });
      return r;
    });

    this.setState({
      regularList: val ? newRegularList : this.state.regularList,
      salaryCoefficient: val,
    });
  };

  validate = () => {
    const { regularList, currentSelectedRegular, salaryCoefficient, regularMap } = this.state;
    const selectRegular = regularMap[currentSelectedRegular];

    return new Promise((resolve, reject) => {
      if (!currentSelectedRegular) {
        showMessage('请选择规则');
        return reject();
      }

      if (!salaryCoefficient) {
        showMessage('请填写规则系数');
        return reject();
      }

      const maxSalary = selectRegular.salaryCoefficient;
      if (salaryCoefficient > maxSalary) {
        showMessage(`课酬补贴系数不得大于${maxSalary}`);
        return reject();
      }

      // 检测不能有空的值
      const noEmpty = regularList.every((r) => {
        return r.children.every((c) => {
          return c.maxClassNumber && c.salaryCoefficient;
        });
      });

      if (!noEmpty) {
        showMessage('请检查是否填写完整');
        return reject();
      }

      const invalidList = [];

      regularList.forEach((r) => {
        r.children.forEach((cur) => {
          const type = isSummerOrWinter(cur.quarterId)
            ? `${cur.quarterName}第${cur.termNumber}期`
            : `${cur.quarterName}`;

          if (cur.salaryCoefficient > salaryCoefficient) {
            invalidList.push(`${type}课程系数不能大于${salaryCoefficient}`);
          }

          if (cur.maxClassNumber > cur.max) {
            invalidList.push(`${type}最大带班数量不能大于${cur.max}`);
          }
        });

        if (r.children.length <= 1) {
          return;
        }

        const [first, ...children] = r.children;

        children.reduce((pre, cur) => {
          if (cur.maxClassNumber < Math.max(pre.maxClassNumber, first.maxClassNumber)) {
            invalidList.push(`${cur.quarterName}第${cur.termNumber}
              期最大带班数量不得小于往期`);
          }

          return cur;
        }, first);
      });

      if (invalidList.length > 0) {
        Modal.error({
          title: '提示',
          content: invalidList.map((invalItem) => {
            return <p key={invalItem}>{invalItem}</p>;
          }),
        });
        return reject();
      }

      resolve();
    });
  };

  onSave = async () => {
    const { detail, isLimitRegular } = this.state;
    let request;

    if (isLimitRegular) {
      await this.validate();
      if (detail) {
        request = this.onUpdate();
      } else {
        request = this.onAdd();
      }
    } else {
      if (detail) {
        request = this.onDelete();
      } else {
        request = Promise.resolve();
      }
    }

    request.then(() => {
      showMessage('保存成功', 'success');
      this.props.onSuccess();
    });
  };

  onAdd = async () => {
    const { teacherId } = this.props;
    const { regularMap, currentSelectedRegular, salaryCoefficient, yearId, regularList } = this.state;
    const selectRegular = regularMap[currentSelectedRegular];
    const regular = formatRegularForRequest(regularList);

    return addTeacherRegular({
      type: 99,
      objectId: teacherId,
      name: selectRegular.name,
      parentId: selectRegular.id,
      salaryCoefficient: salaryCoefficient,
      yearId,
      mcNumberLimitRuleDetailSaveDtos: regular,
    });
  };

  onUpdate = () => {
    const { detail, regularList, salaryCoefficient, currentSelectedRegular, regularMap } = this.state;
    const selectRegular = regularMap[currentSelectedRegular];
    const regular = formatRegularForRequest(regularList);

    return updateTeacherRegular({
      ruleId: detail.id,
      parentId: currentSelectedRegular,
      name: selectRegular.name,
      salaryCoefficient,
      mcNumberLimitRuleDetailSaveDtos: regular,
    });
  };

  onDelete = () => {
    const { detail } = this.state;

    return deleteTeacherRegular({
      ruleId: detail.id,
    });
  };

  onWhichMaxClassChange = (season, phase, value) => {
    phase.maxClassNumber = String(value).replace(/[^\d]/g, '');
    this.setState({
      season,
    });
  };

  onWhichCoefficientChange = (season, phase, value) => {
    phase.salaryCoefficient = value;
    this.setState({
      season,
    });
  };

  render() {
    const {
      regularList,
      loading,
      isLimitRegular,
      systemRegular,
      currentSelectedRegular,
      salaryCoefficient,
      yearId,
      detail,
    } = this.state;

    const { YEAR } = this.props;
    const avalidYear = YEAR.filter((y) => {
      return y.text >= new Date().getFullYear();
    });
    return (
      <Modal title="限课规则" width={700} visible={true} onCancel={this.props.onClose} onOk={this.onSave}>
        <div>
          <Loading loading={loading} />
          <Row>
            <FormItem style={{ paddingLeft: 0 }} labelWidth={100} name="选择年份：">
              <Select
                style={{ width: '184px' }}
                value={yearId}
                placeholder="请选择年份"
                dict={avalidYear}
                onChange={this.onYearChange}
              />
            </FormItem>
          </Row>
          <Row>
            <FormItem style={{ paddingLeft: 0 }} labelWidth={100} name="是否限课：">
              <Checkbox
                checked={isLimitRegular}
                onChange={(e) => {
                  const checked = e.target.checked;

                  this.setState({
                    isLimitRegular: checked,
                  });
                }}
              >
                限课
              </Checkbox>
            </FormItem>
          </Row>

          <div style={{ display: isLimitRegular ? 'block' : 'none' }}>
            <Row>
              <Col span={12}>
                <FormItem style={{ paddingLeft: 0 }} labelWidth={100} name="规则名称">
                  <Select
                    value={currentSelectedRegular}
                    placeholder="请选择规则"
                    dict={systemRegular}
                    onChange={this.onSystemRegularChange}
                  />
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem style={{ paddingLeft: 0 }} labelWidth={100} name="课酬补贴系数">
                  <InputNumber
                    value={salaryCoefficient}
                    placeholder="请输入课酬补贴系数"
                    min={0}
                    onChange={this.onCoefficientChange}
                  />
                </FormItem>
              </Col>
            </Row>

            <Row>
              {regularList.map((season) => {
                return (
                  <div className={style.regularContainer} key={season.quarterId}>
                    <span className={style.regularType}>{season.quarterName}最大带班数</span>
                    <ul className={style.regularContent}>
                      {season.children.map((phase) => (
                        <li className={style.regularItem} key={phase.id}>
                          <span className={style.regularPhase}>
                            {isSummerOrWinter(season.quarterId) ? `${phase.termNumber}期` : null}
                          </span>
                          <InputNumber
                            min={0}
                            value={phase.maxClassNumber}
                            onChange={(value) => this.onWhichMaxClassChange(season, phase, value)}
                          />
                          <span className={style.regularLabel}>课酬补贴系数</span>
                          <InputNumber
                            min={0}
                            value={phase.salaryCoefficient}
                            onChange={(value) => this.onWhichCoefficientChange(season, phase, value)}
                          />
                        </li>
                      ))}
                    </ul>
                  </div>
                );
              })}
            </Row>
          </div>
        </div>
      </Modal>
    );
  }
}

export default connect((state) => {
  return {
    YEAR: state.dict['year'] || [],
  };
})(LimitRegularModal);
