import React from 'react';
import TextLink from 'COMPONENT/TextLink';
import Select from 'COMPONENT/Select';
import DatePicker from 'COMPONENT/DatePicker';
import TreeSelect from 'COMPONENT/TreeSelect';
import SubBlock from 'COMPONENT/Block/SubBlock';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import { getTreeNode } from '../common';
import styles from './styles.less';
import { getDictTextByVal } from 'UTIL/dict';
import loaders from '../loaders';
import { debounce } from 'lodash';

const formItemCfg = {
  size: 'small',
  style: { padding: '0 50px 0 0' },
  required: true,
  labelWidth: 85,
};

class AuthBlock extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      orgDict: props.initOrgDict,
      orgExpandedKeys: [props.initOrgDict[0].val],
    };
    this.orgDictVersion = 1;
  }

  onChange = (field, val) => {
    let changed;
    let isOrgChange = false;
    if (field === 'org') {
      val = val || {};
      isOrgChange = true;
      changed = { org: val.val, orgName: val.text };
    } else if (field === 'role') {
      val = val || {};
      changed = { role: val.val, roleName: val.text };
    } else {
      changed = { [field]: val };
    }
    this.props.onChange(changed, isOrgChange);
  };

  onLoadOrgTreeData = (treeNode) => {
    return new Promise((resolve) => {
      if (treeNode.props.children) {
        resolve();
        return;
      }
      const key = treeNode.props.eventKey;
      loaders.loadOrgDictByParentId(key).then((data) => {
        this.setState({
          orgDict: [...this.state.orgDict, ...data],
          orgExpandedKeys: [...this.state.orgExpandedKeys, key],
        });
        resolve();
      });
    });
  };

  onOrgTreeSearch = (searchKey) => {
    debounce(this._onOrgTreeSearch, 300)(searchKey);
  };

  _onOrgTreeSearch = (searchKey) => {
    if (searchKey) {
      this.orgDictVersion++;
      loaders.loadOrgDictByName(searchKey, this.orgDictVersion).then(({ dict: orgDict, version }) => {
        if (this.orgDictVersion === version) {
          this.setState({ orgDict, orgExpandedKeys: [] });
        }
      });
    } else {
      this.orgDictVersion++;
      this.setState({ orgDict: this.props.initOrgDict, orgExpandedKeys: [this.props.initOrgDict[0].val] });
    }
  };

  getDisabledDate = (date) => {
    let Day = new Date();
    let DayTime = `${Day.getFullYear()}-${Day.getMonth() + 1}-${Day.getDate()}`;
    let currentDate = new Date(DayTime).getTime();

    return new Date(date).getTime() < currentDate;
  };

  render() {
    const { title, value, onChange, checking, onErrorChange, onDelete, sysRoleDict, isMain } = this.props;
    const { orgDict, orgExpandedKeys } = this.state;

    if (isMain && !value?.validityPeriod) {
      this.onChange('validityPeriod', '2099-01-01');
    }

    return (
      <SubBlock
        title={title}
        headerToolBar={onDelete ? <TextLink onClick={onDelete}>删除</TextLink> : null}
        className={styles.wrap}
      >
        <Row style={{ marginRight: -70 }}>
          <Col span={8}>
            <FormItem {...formItemCfg} name="组织架构">
              <TreeSelect
                value={value?.org ? { val: value?.org, text: value?.orgName } : null}
                onChange={(val) => this.onChange('org', val)}
                validator={{
                  onErrorChange: (e) => onErrorChange('org', e),
                  silent: !checking,
                }}
                textInValue
                onLoadChildren={this.onLoadOrgTreeData}
                onSearchKeyChange={this.onOrgTreeSearch}
                expandedKeys={orgExpandedKeys}
                onExpandChange={(orgExpandedKeys) => this.setState({ orgExpandedKeys })}
                dict={orgDict}
              />
            </FormItem>
          </Col>
          <Col span={8}>
            <FormItem {...formItemCfg} name="角色">
              <Select
                dict={sysRoleDict}
                value={value?.role ? { val: value?.role, text: value?.roleName } : null}
                onChange={(val) => this.onChange('role', val)}
                showSearch
                validator={{
                  onErrorChange: (e) => onErrorChange('role', e),
                  silent: !checking,
                }}
                textInValue
              />
            </FormItem>
          </Col>
          <Col span={8}>
            <FormItem {...formItemCfg} name="截止有效期">
              <DatePicker
                disabled={isMain}
                value={value?.validityPeriod ? value?.validityPeriod : '2099-01-01'}
                onChange={(val) => this.onChange('validityPeriod', val)}
                validator={{
                  onErrorChange: (e) => onErrorChange('validityPeriod', e),
                  silent: !checking,
                }}
                disabledDate={(val) => this.getDisabledDate(val)}
              />
            </FormItem>
          </Col>
        </Row>
      </SubBlock>
    );
  }
}

export default AuthBlock;
