import React from 'react';
import classnames from 'classnames';
import Modal from 'COMPONENT/Modal';
import styles from './styles.less';
import { getDictTextByVal } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import vars from 'COMPONENT/theme/variables';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import Button from 'COMPONENT/Button';
import Block from 'COMPONENT/Block';
import SubBlock from 'COMPONENT/Block/SubBlock';
import AuthBlock from '../AuthBlock';
import Validator from 'COMPONENT/Validator';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { checkUserOrgCanModify } from 'SERVICE/system';

class AuthSettingModal extends React.Component {
  constructor(props) {
    super(props);
    this.errors = {}; //校验错误
    this.rowIdx = 1; //用于生成唯一的行key
    const { data } = props;
    const mainAuth = data.find((d) => d.isMain);
    const otherAuths = data
      .filter((d) => !d.isMain)
      .map((da) => {
        return { ...da, id: '' + this.rowIdx++ };
      });

    this.state = {
      loading: false, // {boolean} 等待状态
      mainAuth, // {object} 主组织架构
      otherAuths, // {[{ org, role }]} 其他组织架构

      checking: false, //校验开启状态
    };
  }

  onOk = () => {
    const { checking } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    const errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      this.modal.showOkTips(errorTips);
      return;
    }
    if (this.checkDup()) {
      this.modal.showOkTips('当前存在相同的组织架构及角色设置');
      return;
    }
    const { mainAuth, otherAuths } = this.state;
    const data = [
      {
        isMain: true,
        org: mainAuth.org,
        role: mainAuth.role,
        orgName: mainAuth.orgName,
        roleName: mainAuth.roleName,
        validityPeriod: mainAuth.validityPeriod,
      },
      ...otherAuths.map((a) => {
        return {
          isMain: false,
          org: a.org,
          role: a.role,
          orgName: a.orgName,
          roleName: a.roleName,
          validityPeriod: a.validityPeriod,
        };
      }),
    ];
    this.props.onOk(data);
  };

  onErrorChange = (key, error) => {
    this.errors[key] = error;
  };

  checkCanModify = async (data) => {
    this.setState({ loading: true });
    try {
      const res = await checkUserOrgCanModify(data, true);
    } catch (e) {
      this.setState({ loading: false });
      showAlert(e.text || '出错了');
      throw e;
    }
    this.setState({ loading: false });
    return true;
  };

  onMainChange = (changedKv, isOrgChange) => {
    const info = this.state.mainAuth?.info;
    if (info) {
      this.checkCanModify(info).then(
        () => {
          this.setState({ mainAuth: { ...this.state.mainAuth, ...changedKv, info: null } });
        },
        () => {},
      );
    } else {
      this.setState({ mainAuth: { ...this.state.mainAuth, ...changedKv } });
    }
  };

  onOtherChange = (id, info, changedKv, isOrgChange) => {
    const otherAuths = this.state.otherAuths;
    //let info = otherAuths.find(a => a.id === id).info
    if (info) {
      this.checkCanModify(info).then(
        () => {
          this.setState({
            otherAuths: otherAuths.map((a) => {
              if (a.id === id) {
                return { ...a, ...changedKv, info: null };
              }
              return a;
            }),
          });
        },
        () => {},
      );
    } else {
      this.setState({
        otherAuths: otherAuths.map((a) => {
          if (a.id === id) {
            return { ...a, ...changedKv };
          }
          return a;
        }),
      });
    }
  };

  onAdd = () => {
    const newOtherAuths = [
      { org: null, role: null, orgName: null, roleName: null, id: '' + this.rowIdx++, validityPeriod: '2099-01-01' },
      ...this.state.otherAuths,
    ];
    this.setState({ otherAuths: newOtherAuths });
  };

  onDelete = (id, info) => {
    const doDelete = () => {
      let newErrors = {};
      Object.keys(this.errors).forEach((key) => {
        const a = key.split('_');
        if (a[1] !== id) {
          newErrors[key] = this.errors[key];
        }
      });
      this.errors = newErrors;
      this.setState({ otherAuths: this.state.otherAuths.filter((a) => a.id !== id) });
    };
    const otherAuths = this.state.otherAuths;
    //let info = otherAuths.find(a => a.id === id).info
    if (info) {
      this.checkCanModify(info).then(doDelete, () => {});
    } else {
      doDelete();
    }
  };

  checkDup = () => {
    const { mainAuth, otherAuths } = this.state;
    const auths = [mainAuth, ...otherAuths].filter((a) => a.org && a.role);
    return auths.some((a) =>
      auths.filter((au) => au.id !== a.id).some((aut) => aut.org === a.org && aut.role === a.role),
    );
  };

  render() {
    const { onCancel, initOrgDict, sysRoleDict } = this.props;
    const { loading, mainAuth, otherAuths, checking } = this.state;

    return (
      <Modal
        className={styles.wrap}
        loading={loading}
        title="组织架构及角色设置"
        width={950}
        onOk={this.onOk}
        onCancel={onCancel}
        noLine
        fixedLoading
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        <Block title="主组织">
          <AuthBlock
            isMain={true}
            title="组织架构"
            value={mainAuth}
            onChange={this.onMainChange}
            checking={checking}
            onErrorChange={(field, e) => this.onErrorChange(`${field}_main`, e)}
            initOrgDict={initOrgDict}
            sysRoleDict={sysRoleDict}
          />
        </Block>
        <Block title="其他组织" headerToolBar={<Button onClick={this.onAdd}>添加其他组织</Button>}>
          {otherAuths.map((a, i) => {
            return (
              <AuthBlock
                key={a.id}
                title={`组织架构${i + 1}`}
                value={a}
                onChange={(changedKv, isOrgChange) => this.onOtherChange(a.id, a.info, changedKv, isOrgChange)}
                checking={checking}
                onErrorChange={(field, e) => this.onErrorChange(`${field}_${a.id}`, e)}
                onDelete={() => this.onDelete(a.id, a.info)}
                initOrgDict={initOrgDict}
                sysRoleDict={sysRoleDict}
              />
            );
          })}
        </Block>
      </Modal>
    );
  }
}

export default AuthSettingModal;
