import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import loaders from '../loaders';
import DropdownMenu from 'COMPONENT/DropdownMenu';
import Block from 'COMPONENT/Block';
import SubBlock from 'COMPONENT/Block/SubBlock';
import Page from 'COMPONENT/Page';
import FormItem from 'COMPONENT/FormItem';
import TextArea from 'COMPONENT/TextArea';
import styleVars from 'COMPONENT/theme/variables';
import List from 'COMPONENT/List';
import styles from './styles.less';
import Select from 'COMPONENT/Select';
import Button from 'COMPONENT/Button';
import { getDictTextByVal } from 'UTIL/dict';
import AuthSettingModal from '../AuthSettingModal';
import Validator from 'COMPONENT/Validator';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { FIELD_FILLER, checkFieldEmpty, getFieldWithFiller } from 'COMPONENT/utils/form';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const formItemCfg = {
  size: 'small',
  style: { padding: '0 50px 0 0' },
};
const blockStyle = {
  margin: ' -5px 20px 10px 60px',
};

class Detail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态

      realName: null, // {string} 真实姓名
      workNum: null, // {string} 工号
      name: null, // {string} 用户名
      account: null, // {string} 登录账号
      contract: null, // {string} 联系方式
      sex: null, // {string} 性别
      status: null, // {string} 状态
      workType: null, // {string} 用工类型

      postInfo: [], // {[dep, post]} 岗位信息
      inDate: null, // {string} 入职日期
      outDate: null, // {string} 离职日期
      userProp: null, // {string} 用户属性
      authInfo: [], // {[org, role]} 权限信息
      exAuthInfo: [],

      list: [], // {object[]} 操作流水列表数据
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 5, // {number} 每页记录数

      authSettingModalVis: false, // {boolean} 权限设置弹框显示状态

      checking: false, // {boolean}  校验开启状态
      adjustReason: '', // 调整原因
    };
    this.errors = {}; //{boolean|string} 校验错误信息
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = () => {
    const {
      params: { id },
    } = this.props;
    this.setState({ loading: true });
    Promise.all([loaders.loadUserDetail(id)]).then(([userDetail]) => {
      this.setState({ loading: false, ...userDetail, totalRecords: userDetail.list.length });
    });
  };

  getDropdown = (title, data) => {
    return (
      <DropdownMenu
        text={`${title}（${data.length}个）`}
        menus={data}
        hasArrow={true}
        color={styleVars.fontColor}
        activeColor={styleVars.primaryColor}
      />
    );
  };

  getPostDropdown = (data) => {
    return this.getDropdown(
      '更多职能',
      data.map((d) => {
        return { text: `${d.dep}-${d.post}` };
      }),
    );
  };

  getAuthDropdown = (data) => {
    const {
      params: { sysRoleDict },
    } = this.props;
    return this.getDropdown(
      '更多权限',
      data.map((d) => {
        return { text: `${d.orgName + (d.isMain ? '(主)' : '')}-${d.roleName}` };
      }),
    );
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  onOk = () => {
    const {
      params: { id, onSaveSuccess },
    } = this.props;
    const { checking, adjustReason } = this.state;
    const opReason = adjustReason.trim();

    if (!checking) {
      this.setState({ checking: true });
    }
    if (Object.values(this.errors).find((v) => v)) {
      this.pageRef.showOkTips('请正确填写信息');
      return;
    }
    if (!opReason || opReason.length > 200) {
      return showMessage('请输入调整原因，不超过200个字符');
    }

    const { userProp, authInfo, exAuthInfo } = this.state;
    this.setState({ loading: true });
    const totalAuthInfo = [...authInfo, ...exAuthInfo];
    loaders
      .saveDetail({ id, userProp, authInfo: totalAuthInfo, opReason })
      .then(
        () => {
          showMessage('保存成功', 'success');
          onSaveSuccess();
          this.setState({
            adjustReason: '',
          });
        },
        (e) => {
          showMessage(e.text || '保存失败');
        },
      )
      .finally(() => {
        this.setState({ loading: false });
      });
  };

  render() {
    const {
      params: { type, userPropDict, initOrgDict, sysRoleDict },
    } = this.props;
    const {
      loading,
      realName,
      workNum,
      name,
      account,
      contract,
      sex,
      status,
      workType,
      postInfo,
      inDate,
      outDate,
      userProp,
      authInfo,
      list,
      totalRecords,
      currentPage,
      pageSize,
      authSettingModalVis,
      checking,
      adjustReason,
    } = this.state;

    const columns = [
      { title: '时间', dataIndex: 'datetime' },
      { title: '操作详情', dataIndex: 'detail', width: COL_WIDTH_XL },
      { title: '调整原因', dataIndex: 'opReason', width: COL_WIDTH_XL },
      { title: '操作人', dataIndex: 'user' },
    ];

    const isView = type === 'view';
    const renderFooter = (
      <div className={styles.footer}>
        <FormItem name="调整原因" required className={styles.reason}>
          <TextArea
            placeholder="请输入调整原因"
            value={adjustReason}
            rows={2}
            wordCount
            maxLength={200}
            onChange={(e) => {
              this.setState({ adjustReason: e.target.value });
            }}
          />
        </FormItem>
        <div className={styles.operation}>
          <Button type="primary" onClick={this.onOk}>
            确定
          </Button>
          <Button onClick={() => history.back()}>取消</Button>
        </div>
      </div>
    );

    return (
      <Page
        className={styles.pageContainer}
        loading={loading}
        route={{ ...this.props.route, name: isView ? '用户详情' : '用户修改' }}
        footer={isView ? null : renderFooter}
        onOk={this.onOk}
        ref={(c) => (this.pageRef = c)}
      >
        <Block title="基本信息" style={blockStyle}>
          <SubBlock title="个人信息">
            <Row>
              <Col span={12}>
                <FormItem {...formItemCfg} name="真实姓名">
                  {getFieldWithFiller(realName)}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem {...formItemCfg} name="工号">
                  {getFieldWithFiller(workNum)}
                </FormItem>
              </Col>
              {/*              <Col span={12}>
                <FormItem {...formItemCfg} name="用户名">
                  {name}
                </FormItem>
              </Col>*/}
              <Col span={12}>
                <FormItem {...formItemCfg} name="登录账号">
                  {getFieldWithFiller(account)}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem {...formItemCfg} name="联系方式">
                  {getFieldWithFiller(contract)}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem {...formItemCfg} name="性别">
                  {getFieldWithFiller(sex)}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem {...formItemCfg} name="状态">
                  {getFieldWithFiller(status)}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem {...formItemCfg} name="用工类型">
                  {getFieldWithFiller(workType)}
                </FormItem>
              </Col>
            </Row>
          </SubBlock>
          <SubBlock title="岗位信息">
            <Row>
              <Col span={24}>
                <FormItem {...formItemCfg} name="部门-岗位">
                  {postInfo.slice(0, 3).map((po, i) => {
                    return <div key={i}>{`${po.dep}-${po.post}`}</div>;
                  })}
                  {postInfo.length > 3 && this.getPostDropdown(postInfo.slice(3))}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem {...formItemCfg} name="入职日期">
                  {getFieldWithFiller(inDate)}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem {...formItemCfg} name="离职日期">
                  {getFieldWithFiller(outDate)}
                </FormItem>
              </Col>
            </Row>
          </SubBlock>
        </Block>
        <Block title="权限信息" style={blockStyle}>
          <SubBlock title="用户设置">
            <Row>
              <Col span={12}>
                <FormItem {...formItemCfg} name="用户属性" required={isView ? null : true}>
                  {isView ? (
                    <div>{getDictTextByVal(userPropDict, userProp)}</div>
                  ) : (
                    <Select
                      dict={userPropDict}
                      value={userProp}
                      onChange={(val) => this.setState({ userProp: val })}
                      validator={{
                        onErrorChange: (e) => this.errorChange('userProp', e),
                        silent: !checking,
                      }}
                    />
                  )}
                </FormItem>
              </Col>
            </Row>
          </SubBlock>
          <SubBlock title="权限设置">
            <Row>
              <Col span={24}>
                <FormItem {...formItemCfg} name="权限-角色" required={isView ? null : true}>
                  <Validator
                    rules={[{ required: '请选择权限' }]}
                    onErrorChange={(e) => this.errorChange('authInfo', e)}
                    silent={!checking}
                    value={authInfo}
                  >
                    <div className={styles.authWrap}>
                      <div className={styles.authWrapLeft}>
                        {(authInfo || []).slice(0, 3).map((po, i) => {
                          return <div key={i}>{`${po.orgName + (po.isMain ? '(主)' : '')}-${po.roleName}`}</div>;
                        })}
                        {(authInfo || []).length > 3 && this.getAuthDropdown(authInfo.slice(3))}
                      </div>
                      {!isView && (
                        <div className={styles.authWrapRight}>
                          <Button onClick={() => this.setState({ authSettingModalVis: true })}>选择</Button>
                        </div>
                      )}
                    </div>
                  </Validator>
                </FormItem>
              </Col>
            </Row>
          </SubBlock>
          <SubBlock title="操作流水">
            <div style={{ paddingTop: 10 }}>
              {list.length > 0 ? (
                <List
                  dataSource={list.slice((currentPage - 1) * pageSize, currentPage * pageSize)}
                  columns={columns}
                  pagination={{
                    current: currentPage,
                    total: totalRecords,
                    pageSize,
                    pageSizeOptions: ['5', '10', '20'],
                  }}
                  onChange={({ current, pageSize }) => this.setState({ currentPage: current, pageSize })}
                />
              ) : (
                <div className={styles.tips}>暂无操作流水信息</div>
              )}
            </div>
          </SubBlock>
        </Block>
        {authSettingModalVis && (
          <AuthSettingModal
            data={authInfo}
            onOk={(data) => this.setState({ authSettingModalVis: false, authInfo: data })}
            onCancel={() => this.setState({ authSettingModalVis: false })}
            initOrgDict={initOrgDict}
            sysRoleDict={sysRoleDict}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/SystemUserManage/Detail'],
  };
};

export default connect(mapStateToProps)(Detail);
