import React from 'react';
import { Button, Icon, Switch, Row, Col } from 'antd';
import Modal from 'COMPONENT/Modal';
import getWindowSize from 'UTIL/getWindowSize';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { findOrganizationNew, updateOrgStatusByIdNew, updateBatchOrgStatusByIdNew } from 'SERVICE/system';
import styles from './styles.less';
import 'COMPONENT/Page/styles.less';

//状态字典
const statusDict = [
  { val: 'NORMAL', text: '正常' },
  { val: 'EMERGENCY', text: '紧急' },
];

class MultiSwitchModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // boolean 页面加载状态
      list: null, // 列表数据
      batchModalVisible: false, // boolean 批量操作弹框显示状态
      activeBranchId: null, // string 当前正在操作的分公司ID
      activeBranchName: null, // string 当前正在操作的分公司名称
      winW: 0, //  number  窗口宽度
    };
  }

  componentWillMount() {
    this.initContent();
    this.updateWinW();
    window.addEventListener('resize', this.updateWinW, false);
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.updateWinW, false);
    this.isUnmounted = true;
  }

  /**
   * 获取和更新窗口宽度状态
   */
  updateWinW = () => {
    this.updateState({ winW: getWindowSize().width });
  };

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      if (!this.isUnmounted) {
        this.setState(newState, () => {
          resolve();
        });
      } else {
        resolve();
      }
    });
  };

  /**
   * 初始化内容
   */
  initContent = async () => {
    this.setState({ loading: true });
    const { userId, isAdmin } = this.props;
    const params = isAdmin ? {} : { userId };
    const list = (await this.loadList(params)) || [];
    await this.updateState({ list });
    this.setState({ loading: false });
  };

  /**
   * 加载列表数据，接口参见https://api.xuebangsoft.me/index.do#/f7525741-db26-4bb4-8475-7555d19dadcc/front/interfaceDetail/d901ddaf-7ec8-4d78-a4c9-f6c8370c9556
   * 并转换格式，如下：
      [{
        id: '2',
        name: '杭州分公司',
        expand: false,
        children: [
          { id: '1', name: '杭州分公司部门', status: 'EMERGENCY' },
          { id: '2', name: '解放路学习中心', status: 'EMERGENCY' },
          { id: '3', name: '城西学习中心', status: 'NORMAL' },
        ],
      }]
   * @param  {string} branchId 分公司ID
   * @param  {string} campusId 学习中心ID
   * @param  {string} status 状态
   */
  loadList = async (params) => {
    /*const list = await new Promise((resolve) => {
      setTimeout(() => {
        resolve([
          {
            id: '1',
            name: '东莞分公司',
            expand: true,
            children: [
              { id: '1', name: '东莞分公司部门', status: 'NORMAL' },
              { id: '2', name: '虎门学习中心', status: 'NORMAL' },
              { id: '3', name: '南城学习中心', status: 'EMERGENCY' },
              { id: '4', name: '常平学习中心', status: 'EMERGENCY' },
              { id: '5', name: '樟木头学习中心', status: 'NORMAL' },
              { id: '6', name: '虎门学习中心', status: 'NORMAL' },
              { id: '7', name: '虎门学习中心', status: 'NORMAL' },
              { id: '8', name: '虎门学习中心', status: 'NORMAL' },
              { id: '9', name: '虎门学习中心', status: 'NORMAL' },
              { id: '10', name: '虎门学习中心', status: 'NORMAL' },
              { id: '11', name: '虎门学习中心', status: 'NORMAL' },
              { id: '12', name: '虎门学习中心', status: 'NORMAL' },
              { id: '13', name: '虎门学习中心', status: 'NORMAL' },
              { id: '14', name: '虎门学习中心', status: 'NORMAL' },
            ],
          },
          {
            id: '2',
            name: '杭州分公司',
            expand: false,
            children: [
              { id: '1', name: '杭州分公司部门', status: 'EMERGENCY' },
              { id: '2', name: '解放路学习中心', status: 'EMERGENCY' },
              { id: '3', name: '城西学习中心', status: 'NORMAL' },
            ],
          },
        ])
      }, 200)
    })*/
    const data = await findOrganizationNew(params);
    const list = data.map((branch) => {
      const { branchId, branchName, organizations } = branch;
      return {
        id: branchId,
        name: branchName,
        children: organizations.map((campus) => {
          const { organizationId, organizationName, stateOfEmergency } = campus;
          return { id: organizationId, name: organizationName, status: stateOfEmergency };
        }),
      };
    });
    return list;
  };

  /**
   * 点击列表中分公司标题时，切换展开状态
   * @param  {string} branchId 分公司ID
   */
  branchTitleClick = (branchId) => {
    const { list } = this.state;
    this.updateState({
      list: list.map((b) => {
        const { id, expand } = b;
        if (id === branchId) {
          return { ...b, expand: !expand };
        }
        return b;
      }),
    });
  };

  /**
   * 点击学习中心项目时切换学习中心状态，切换前弹框确认
   * @param  {string} campusId  学习中心ID
   * @param  {string} name  学习中心名称
   * @param  {string} status 当前的状态
   */
  campusItemClick = (campusId, name, status) => {
    const nextStatus = statusDict.find((s) => s.val !== status);
    showConfirm(name + '将切换为' + nextStatus.text + '状态，请确认是否切换？', () => {
      this.updateState({ loading: true });
      const statu = nextStatus.val === 'EMERGENCY' ? 1 : 0;
      this.requestSwitchStatus(campusId, statu).then(() => {
        const { list } = this.state;
        this.updateState({
          loading: false,
          list: list.map((b) => {
            const { children } = b;
            const newChildren = children.map((c) => {
              if (c.id === campusId) {
                return { ...c, status: nextStatus.val };
              }
              return c;
            });
            return { ...b, children: newChildren };
          }),
        });
      });
    });
  };

  /**
   * 打开批量操作弹框，选择操作类型
   * @param  {string} branchId 分公司ID
   * @param  {string} branchName 分公司名称
   */
  showBatchModal = (branchId, branchName) => {
    this.updateState({
      batchModalVisible: true,
      activeBranchId: branchId,
      activeBranchName: branchName,
    });
  };

  /**
   * 关闭批量操作弹框
   */
  closeBatchModal = () => {
    this.setState({
      batchModalVisible: false,
      activeBranchId: null,
      activeBranchName: null,
    });
  };

  /**
   * 批量切换状态
   * @param  {string} status 要切换的状态
   */
  batchSwitch = (status) => {
    const { activeBranchId, activeBranchName, branchDict, list } = this.state;
    const nextStatusText = statusDict.find((s) => s.val === status).text;
    this.updateState({ batchModalVisible: false });
    showConfirm(activeBranchName + '及下属所有学习中心将切换为' + nextStatusText + '状态，请确认是否切换？', () => {
      this.requestSwitchStatusBatch(activeBranchId, status).then(() => {
        this.updateState({
          list: list.map((b) => {
            const { id, children } = b;
            if (id === activeBranchId) {
              const newChildren = children.map((c) => {
                return { ...c, status };
              });
              return { ...b, children: newChildren };
            }
            return b;
          }),
        });
      });
    });
  };

  /**
   * 请求切换状态
   * @param  {string} campusId 学习中心ID
   */
  requestSwitchStatus = async (campusId, statu) => {
    await updateOrgStatusByIdNew({ organizationId: campusId, statu });
    return true;
  };

  /**
   * 请求批量切换状态
   * @param  {string} branchId 分公司ID
   * @param  {string} status 要切换的状态
   */
  requestSwitchStatusBatch = async (branchId, status) => {
    await updateBatchOrgStatusByIdNew({ branchId, stateOfEmergency: status });
    return true;
  };

  render() {
    const { onClose } = this.props;
    const { loading, list, batchModalVisible, activeBranchName, winW } = this.state;
    const colCount = winW < 1500 ? 2 : winW < 1700 ? 2 : 3;

    return (
      <Modal
        loading={loading}
        okBtnShow={false}
        cancelText="关闭"
        onCancel={onClose}
        width={1366}
        title="切换状态"
        widthSize="XL"
      >
        <div className="Page-scroller" style={{ maxHeight: '76vh' }}>
          {list &&
            list.map((b, i) => {
              const { id, name, children, expand } = b;
              const normalCount = children.filter((c) => c.status === 'NORMAL').length;
              const dangerCount = children.filter((c) => c.status === 'EMERGENCY').length;
              const rowCount = Math.ceil(children.length / colCount);
              const h = 55 * rowCount + 77;
              return (
                <div
                  key={i}
                  className={styles.branchItem + (expand ? ' ' + styles.expand : '')}
                  style={{ height: expand ? h : 51 }}
                >
                  <div className={styles.branchTitle} onClick={() => this.branchTitleClick(id)}>
                    <span className={styles.branchName}>{name}</span>
                    <span className={styles.statusItem}>
                      <span className={styles.dot + ' ' + styles.green}></span>
                      <span>{'正常：' + normalCount}</span>
                    </span>
                    <span className={styles.statusItem}>
                      <span className={styles.dot + ' ' + styles.red}></span>
                      <span>{'紧急：' + dangerCount}</span>
                    </span>
                  </div>
                  <div className={styles.branchContent}>
                    <div className={styles.campusWrap}>
                      <Row gutter={40}>
                        {children &&
                          children.map((c, j) => {
                            const { id, name, status } = c;
                            return (
                              <Col key={j} span={24 / colCount}>
                                <div className={styles.campusItem}>
                                  <span className={styles.campusTitle}>{name}</span>
                                  <div className={styles.campusStatus}>
                                    <Switch
                                      checkedChildren="紧急"
                                      unCheckedChildren="正常"
                                      checked={status === 'EMERGENCY'}
                                      onChange={() => this.campusItemClick(id, name, status)}
                                    />
                                  </div>
                                </div>
                              </Col>
                            );
                          })}
                      </Row>
                    </div>
                    <div className={styles.batchBtnWrap}>
                      <Button type="primary" size="large" onClick={() => this.showBatchModal(id, name)}>
                        批量操作
                      </Button>
                    </div>
                  </div>
                </div>
              );
            })}
        </div>
        <Modal
          toggle={true}
          visible={batchModalVisible}
          footer={null}
          closable={false}
          widthSize="M"
          className={styles.batchModalWrap}
          onCancel={() => this.setState({ batchModalVisible: false })}
        >
          <div className={styles.batchModal}>
            <div className={styles.content}>
              <div className={styles.left}>
                <Icon type="exclamation-circle" />
              </div>
              <div className={styles.right}>
                <div className={styles.title}>提示</div>
                <div>{`该操作将影响${activeBranchName}的所有部门和所有学习中心，请谨慎操作`}</div>
              </div>
            </div>
            <div className={styles.footer}>
              <Button size="large" onClick={() => this.batchSwitch('EMERGENCY')} className={styles.btn}>
                批量紧急
              </Button>
              <Button size="large" onClick={() => this.batchSwitch('NORMAL')} className={styles.btn}>
                批量正常
              </Button>
              <Button size="large" onClick={this.closeBatchModal} className={styles.btn}>
                取消
              </Button>
            </div>
          </div>
        </Modal>
      </Modal>
    );
  }
}

export default MultiSwitchModal;
