import React from 'react';
import PropTypes from 'prop-types';
import styles from './styles.module.less';
import { List, ListAction } from 'COMPONENT';
import Modal from 'COMPONENT/Modal/Modal';
import { Row, Col, Modal as antModal, TreeSelect } from 'antd';
import { runFunc, errorsHandle } from 'UTIL/common';
import { getBossOrganizationList } from 'SERVICE/system';
import { assignAgents } from 'SERVICE/callCenter';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { orgDictAdapter } from './utils';

const setWidth = List.createSetWidth();
const errorsHandler = errorsHandle();
const DICT_NAME = 'modalAgentAssign_OrgDict';

class ModalAgentAssign extends React.Component {
  static propTypes = {
    data: PropTypes.arrayOf(
      PropTypes.shape({
        agentId: PropTypes.string,
        agentName: PropTypes.string,
        agentStatus: PropTypes.string,
      }),
    ).isRequired,
    onCancel: PropTypes.func.isRequired,
    onSuccess: PropTypes.func,
    dispatch: PropTypes.func,
    storeDict: PropTypes.object,
  };

  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.onDelete = this.onDelete.bind(this);
    this.onSelect = this.onSelect.bind(this);
    this.submit = this.submit.bind(this);
    this.close = this.close.bind(this);

    let organizationId = undefined;
    if (props.data.length > 0 && new Set(props.data.map((item) => item.organizationId)).size === 1) {
      organizationId = props.data[0].organizationId;
    }

    this.state = {
      list: props.data,
      loading: false,
      dict: props.storeDict[DICT_NAME] || [],
      organizationId,
    };

    this.columns = [
      { title: '坐席ID', dataIndex: 'agentId', key: 'agentId', width: setWidth(12) },
      { title: '坐席名称', dataIndex: 'agentName', key: 'agentName', width: setWidth(4) },
      {
        title: '分配状态',
        dataIndex: 'organizationId',
        key: 'organizationId',
        width: setWidth(4),
        render: (text) => (text ? '已分配' : '未分配'),
      },
      {
        title: '操作',
        dataIndex: 'agentId',
        key: 'actionPlus',
        width: setWidth(2),
        fixed: 'right',
        render: (text) => (
          <ListAction
            config={[
              {
                text: '移除',
                onClick: () => {
                  this.onDelete(text);
                },
              },
            ]}
          />
        ),
      },
    ];

    this.modalRef = null;
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    if (this.state.dict.length === 0) {
      await new Promise(this.initDict);
    }
    this.setState({ loading: false });
  }

  async initDict(resolve) {
    const apiParams = {
      type: 'BOSS',
      orgTypes: 'GROUNP,BRENCH,CAMPUS,DEPARTMENT',
    };
    const res = await getBossOrganizationList(apiParams).catch(errorsHandler);
    if (res && res.resultCode === 0) {
      const dict = orgDictAdapter(res.data);
      this.props.dispatch({
        type: 'dict/updateState',
        payload: {
          [DICT_NAME]: dict,
        },
      });
      this.setState({ dict });
    }
    resolve();
  }

  onDelete(agentId) {
    this.setState(
      (prevState) => {
        const index = prevState.list.findIndex((item) => item.agentId === agentId);
        if (index > -1) {
          prevState.list.splice(index, 1);
        }
        return prevState;
      },
      () => {
        if (this.state.list.length === 0) {
          antModal.warning({
            content: '已无选中坐席，请返回重新选择！',
            okText: '返回',
            onOk: this.close,
          });
        }
      },
    );
  }

  onSelect(val) {
    this.setState({ organizationId: val });
  }

  async submit() {
    if (!this.state.organizationId) {
      return runFunc(this.modalRef.showOkTips, '请先选择归属组织');
    }
    this.setState({ loading: true });
    const apiParams = {
      agentIds: this.state.list.map((item) => item.agentId),
      organizationId: this.state.organizationId,
    };
    const res = await assignAgents(apiParams).catch(errorsHandler);
    if (res && res.resultCode === 0) {
      showMessage('分配成功！', 'success');
      this.close();
      return runFunc(this.props.onSuccess);
    }
    this.setState({ loading: false });
  }

  close() {
    runFunc(this.props.onCancel);
  }

  render() {
    return (
      <Modal
        ref={(c) => (this.modalRef = c)}
        title="分配所属部门"
        loading={this.state.loading}
        onCancel={this.close}
        onOk={this.submit}
        widthSize="M"
      >
        <Row className={styles.form}>
          <Col className={styles.label} span={3}>
            <span className={styles.required} />
            <span>归属组织</span>
          </Col>
          <Col span={21}>
            <TreeSelect
              placeholder="请选择"
              searchPlaceholder="请输入内容搜索"
              treeData={this.state.dict}
              value={this.state.organizationId}
              onChange={this.onSelect}
              treeNodeFilterProp="title"
              showSearch={true}
              allowClear={true}
              treeDefaultExpandAll={false}
              size="large"
            />
          </Col>
        </Row>

        <List columns={this.columns} dataSource={this.state.list} />
      </Modal>
    );
  }
}

export default ModalAgentAssign;
