import React from 'react';
import PropTypes from 'prop-types';
import style from './styles.module.less';
import { Modal, Wrapper, SearchFilterForm, SearchFieldForm, List, Input, TextLink } from 'COMPONENT';
import { Layout, Icon, Popconfirm } from 'antd';
import TextContent from './components/TextContent';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { clone, trim } from 'lodash';
import { errorsHandle, runFunc } from 'UTIL/common';
import { listUserAgentsByAgentId, listCampusUsersByAgentId, bindUsers } from 'SERVICE/callCenter';

const { Sider, Content } = Layout;

const setWidth = List.createSetWidth();

// 匹配已选数据的key值
const matchKey = 'userId';

const SelectedUser = (props) => {
  return (
    <div className={style.selectedUser}>
      <div className={style.selectedUserContent}>
        <span>{props.userName}</span>
      </div>
      <span
        className={style.selectedUserAction}
        onClick={() => {
          runFunc(props.onDelete, props.index);
        }}
      >
        <Icon type="close-circle" />
      </span>
    </div>
  );
};

SelectedUser.propTypes = {
  userId: PropTypes.string.isRequired,
  userName: PropTypes.string.isRequired,
  index: PropTypes.number.isRequired,
  onDelete: PropTypes.func,
  disabled: PropTypes.bool,
};

class ModalAgentBind extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.loadData = this.loadData.bind(this);
    this.loadList = this.loadList.bind(this);
    this.loadSearchResult = this.loadSearchResult.bind(this);
    this.search = this.search.bind(this);
    this.updateSelectedRowKeys = this.updateSelectedRowKeys.bind(this);
    this.changeSelectedRowKeys = this.changeSelectedRowKeys.bind(this);
    this.selectRowKeys = this.selectRowKeys.bind(this);
    this.selectAllRowKeys = this.selectAllRowKeys.bind(this);
    this.delete = this.delete.bind(this);
    this.deleteAll = this.deleteAll.bind(this);
    this.submit = this.submit.bind(this);

    this.state = {
      loading: false,
      searchKey: null,
      list: [],
      selectedRowKeys: [],
      data: [],
    };

    this.columns = [
      { title: '工号', dataIndex: 'employeeNo', key: 'employeeNo', width: setWidth(4) },
      { title: '姓名', dataIndex: 'userName', key: 'userName', width: setWidth(4) },
      { title: '登录账号', dataIndex: 'userAccount', key: 'userAccount', width: setWidth(5) },
      {
        title: '角色',
        dataIndex: 'roleVos',
        key: 'roleVos',
        width: setWidth(10),
        render: (text) => (
          <TextContent data={(text || []).map((item) => item.name)} truncationNum={2} joinSymbol="，" />
        ),
      },
    ];
  }

  static propTypes = {
    data: PropTypes.shape({
      agentId: PropTypes.string,
    }),
    onClose: PropTypes.func.isRequired,
    afterSubmit: PropTypes.func,
  };

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await Promise.all([new Promise(this.loadData), new Promise(this.loadList)]);
    await new Promise(this.updateSelectedRowKeys);
    this.setState({ loading: false });
  }

  async loadData(resolve) {
    const apiParams = { agentId: this.props.data.agentId };
    const res = await listUserAgentsByAgentId(apiParams).catch(errorsHandle());
    if (res && res.data) {
      this.setState({ data: res.data });
    }
    resolve();
  }

  async loadList(resolve) {
    const apiParams = {
      agentId: this.props.data.agentId,
      ...this.state.searchKey,
    };
    const res = await listCampusUsersByAgentId(apiParams).catch(errorsHandle());
    if (res && res.data) {
      this.setState({ list: res.data });
    }
    resolve();
  }

  async loadSearchResult() {
    this.setState({ loading: true });
    await new Promise(this.loadList);
    await new Promise(this.updateSelectedRowKeys);
    this.setState({ loading: false });
  }

  search(params) {
    this.setState(
      {
        searchKey: { ...this.searchKeyAdapter(params) },
      },
      () => {
        this.loadSearchResult();
      },
    );
  }

  searchKeyAdapter = (params) => {
    let sKey = clone(params) || {};
    for (const key in sKey) {
      sKey[key] = sKey[key] ? trim(sKey[key]) : sKey[key];
    }
    return sKey;
  };

  updateSelectedRowKeys(resolve) {
    let selectedRowKeys = [];
    const { list, data } = this.state;
    if (data.length > 0 && list.length > 0) {
      list.forEach((listItem, index) => {
        if (data.findIndex((dataItem) => dataItem[matchKey] === listItem[matchKey]) > -1) {
          selectedRowKeys.push(index);
        }
      });
    }
    this.setState({ selectedRowKeys });
    runFunc(resolve);
  }

  changeSelectedRowKeys(keys) {
    this.setState({ selectedRowKeys: keys });
  }

  selectRowKeys(record, selected) {
    this.setState((prevState) => {
      if (selected) {
        prevState.data.push(record);
      } else {
        const index = prevState.data.findIndex((item) => item[matchKey] === record[matchKey]);
        if (index > -1) {
          prevState.data.splice(index, 1);
        }
      }
      return { data: prevState.data };
    });
  }

  selectAllRowKeys(selected, noNeedUse, changeRows) {
    this.setState((prevState) => {
      if (selected) {
        prevState.data = prevState.data.concat(changeRows);
      } else {
        changeRows.forEach((rowItem) => {
          const index = prevState.data.findIndex((item) => item[matchKey] === rowItem[matchKey]);
          if (index > -1) {
            prevState.data.splice(index, 1);
          }
        });
      }
      return { data: prevState.data };
    });
  }

  delete(index) {
    this.setState(
      (prevState) => {
        prevState.data.splice(index, 1);
        return { data: prevState.data };
      },
      () => {
        this.updateSelectedRowKeys();
      },
    );
  }

  deleteAll() {
    this.setState({
      data: [],
      selectedRowKeys: [],
    });
  }

  async submit() {
    this.setState({ loading: true });
    const apiParams = {
      agentId: this.props.data.agentId,
      userIds: this.state.data.map((item) => item.userId),
    };
    const res = await bindUsers(apiParams).catch(errorsHandle());
    if (res && res.resultCode === 0) {
      showMessage('保存成功！', 'success');
      runFunc(this.props.afterSubmit);
      this.props.onClose();
      return;
    }
    this.setState({ loading: false });
  }

  render() {
    const selectedNum = this.state.data.length;

    return (
      <Modal
        className={style.ModalAgentBind}
        title="坐席绑定/解绑"
        loading={this.state.loading}
        onCancel={this.props.onClose}
        onOk={this.submit}
        widthSize="L"
      >
        <Wrapper
          header={
            <SearchFilterForm onSearch={this.search} truncationNum={1}>
              <SearchFieldForm name="userName" span={2}>
                <Input placeholder="输入用户姓名搜索" />
              </SearchFieldForm>
            </SearchFilterForm>
          }
          bodyFitHeight
        >
          <Layout className={style.Layout} style={{ height: 524 }}>
            <Content className={style.Content}>
              <List
                columns={this.columns}
                dataSource={this.state.list}
                rowSelection={{
                  selectedRowKeys: this.state.selectedRowKeys,
                  onChange: this.changeSelectedRowKeys,
                  onSelect: this.selectRowKeys,
                  onSelectAll: this.selectAllRowKeys,
                }}
              />
            </Content>
            <Sider className={style.Sider} width={300}>
              <Wrapper
                header={
                  <div className={style.title}>
                    <div className={style.titleContent}>
                      已选<span className={style.titleLight}>{selectedNum}</span>个用户
                    </div>
                    <div className={style.titleContent}>
                      <Popconfirm title="确认全部移除吗？" onConfirm={this.deleteAll}>
                        <TextLink disabled={selectedNum === 0}>全部移除</TextLink>
                      </Popconfirm>
                    </div>
                  </div>
                }
                bodyFitHeight
              >
                <div className={style.wrapperBody}>
                  {this.state.data.map((item, index) => (
                    <SelectedUser {...item} index={index} onDelete={this.delete} key={item.userId} />
                  ))}
                </div>
              </Wrapper>
            </Sider>
          </Layout>
        </Wrapper>
      </Modal>
    );
  }
}

export default ModalAgentBind;
