import React from 'react';
import PropTypes from 'prop-types';
import style from './styles.module.less';
import { Modal, Wrapper, Title, List } from 'COMPONENT';
import { Layout, Row, Col, Form } from 'antd';
import { callAgentStatusCfg } from 'CONST/callConfig';
import { errorsHandle } from 'UTIL/common';
import {
  getAgentByAgentId,
  pageUserAgentRelationLogsByAgentId,
  getRenewRecord, // 续费记录
} from 'SERVICE/callCenter';
import moment from 'moment';
import { bindOperationCfg } from '../config';

const { Sider, Content } = Layout;
const FormItem = Form.Item;
const setWidth = List.createSetWidth();

const labelColSpan = 8;
const wrapperColSpan = 24 - labelColSpan;

const momentFormat = 'YYYY-MM-DD HH:mm:ss';

class ModalAgentDetail extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.loadData = this.loadData.bind(this);
    this.loadList = this.loadList.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.initRenewRecord = this.initRenewRecord.bind(this);
    this.changeRecordPage = this.changeRecordPage.bind(this);

    this.state = {
      loading: false,
      data: {},
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 20,
      renewList: [],
      recordTotal: 0,
      recordSize: 10,
      recordCurrent: 1,
    };

    this.recordColumns = [
      {
        title: '操作时间',
        dataIndex: 'createTime',
        key: 'createTime',
        width: setWidth(7),
        render: (text) => moment(text).format(momentFormat),
      },
      {
        title: '操作类型',
        dataIndex: 'operation',
        key: 'operation',
        width: setWidth(4),
        render: (text) => bindOperationCfg[text] || '',
      },
      { title: '用户名', dataIndex: 'userName', key: 'userName', width: setWidth(4) },
    ];

    this.renewCols = [
      { title: '序号', key: 'aa', render: (text, row, index) => +index + 1, width: setWidth(3) },
      {
        title: '开始日期',
        dataIndex: 'startTime',
        render: (time) => moment(time).format('YYYY-MM-DD'),
        width: setWidth(7),
      },
      { title: '续费周期', dataIndex: 'cycle', render: (text) => (text == 1 ? '半年' : '一年'), width: setWidth(4) },
      {
        title: '到期日期',
        dataIndex: 'expireTime',
        render: (time) => moment(time).format('YYYY-MM-DD'),
        width: setWidth(7),
      },
      {
        title: '操作时间',
        dataIndex: 'updateTime',
        render: (time) => moment(time).format('YYYY-MM-DD hh:mm:ss'),
        width: setWidth(10),
      },
    ];
  }

  static propTypes = {
    data: PropTypes.shape({
      agentId: PropTypes.string,
    }),
    onClose: PropTypes.func.isRequired,
    hideRenewRecord: PropTypes.bool, // 是否需要隐藏续费记录
  };

  static defaultTypes = {
    hideRenewRecord: false,
  };

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await Promise.all([new Promise(this.loadData), new Promise(this.loadList), new Promise(this.initRenewRecord)]);
    this.setState({ loading: false });
  }

  async loadData(resolve) {
    const apiParams = { agentId: this.props.data.agentId };
    const res = await getAgentByAgentId(apiParams).catch(errorsHandle());
    if (res && res.data) {
      this.setState({ data: res.data });
    }
    resolve();
  }

  async loadList(resolve) {
    const apiParams = {
      agentId: this.props.data.agentId,
      // 操作类型(1: 绑定; 2: 解绑; 3: 修改手机号;)
      operations: [1, 2],
      pageNum: this.state.pageCurrent,
      pageSize: this.state.pageSize,
    };
    const res = await pageUserAgentRelationLogsByAgentId(apiParams).catch(errorsHandle());
    if (res && res.data) {
      this.setState({
        list: res.data.list,
        total: res.data.total,
      });
    }
    resolve();
  }

  updatePage(pageCurrent, pageSize) {
    return new Promise((resolve) => {
      this.setState(
        (prevState) => {
          let payload = {};
          if (pageSize && pageSize !== prevState.pageSize) {
            payload.pageCurrent = 1;
            payload.pageSize = pageSize;
          } else {
            payload.pageCurrent = pageCurrent;
          }
          return payload;
        },
        () => {
          resolve();
        },
      );
    });
  }

  async changePage({ current, pageSize }) {
    await this.updatePage(current, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  // 续费记录
  async initRenewRecord(resolve) {
    const { resultCode, data } = await getRenewRecord({
      agentId: this.props.data.agentId,
      pageSize: this.state.recordSize,
      pageNum: this.state.recordCurrent,
    });
    if (resultCode === 0) {
      this.setState({
        renewList: data.list,
        recordCurrent: data.pageNum,
        recordSize: data.pageSize,
        recordTotal: data.total,
      });
    }
    resolve();
  }

  changeRecordPage({ current, pageSize }) {
    this.setState({ loading: true, recordCurrent: current, recordSize: pageSize }, async () => {
      await new Promise(this.initRenewRecord);
      this.setState({ loading: false });
    });
  }

  render() {
    return (
      <Modal
        className={style.ModalAgentDetail}
        title="坐席详情"
        loading={this.state.loading}
        onCancel={this.props.onClose}
        okBtnShow={false}
        widthSize="L"
      >
        <Layout className={style.Layout} style={{ height: this.props.hideRenewRecord ? 600 : 900 }}>
          <Content className={style.Content}>
            <Wrapper bodyFitHeight>
              <Title>基本信息</Title>
              <Row className={style.blockBody}>
                <Col span={12}>
                  <FormItem
                    label="坐席ID"
                    colon={false}
                    labelCol={{ span: labelColSpan }}
                    wrapperCol={{ span: wrapperColSpan }}
                  >
                    <span title={this.state.data.agentId}>{this.state.data.agentId}</span>
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem
                    label="坐席名称"
                    colon={false}
                    labelCol={{ span: labelColSpan }}
                    wrapperCol={{ span: wrapperColSpan }}
                  >
                    {this.state.data.agentName}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem
                    label="邮箱账号"
                    colon={false}
                    labelCol={{ span: labelColSpan }}
                    wrapperCol={{ span: wrapperColSpan }}
                  >
                    <span title={this.state.data.agentEmail}>{this.state.data.agentEmail}</span>
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem
                    label="坐席昵称"
                    colon={false}
                    labelCol={{ span: labelColSpan }}
                    wrapperCol={{ span: wrapperColSpan }}
                  >
                    {this.state.data.agentNick}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem
                    label="坐席角色"
                    colon={false}
                    labelCol={{ span: labelColSpan }}
                    wrapperCol={{ span: wrapperColSpan }}
                  >
                    {this.state.data.agentRoleName}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem
                    label="坐席状态"
                    colon={false}
                    labelCol={{ span: labelColSpan }}
                    wrapperCol={{ span: wrapperColSpan }}
                  >
                    {callAgentStatusCfg[this.state.data.agentStatus]}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem
                    label="SIP话机账号"
                    colon={false}
                    labelCol={{ span: labelColSpan }}
                    wrapperCol={{ span: wrapperColSpan }}
                  >
                    {this.state.data.voipAccount}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem
                    label="SIP话机密码"
                    colon={false}
                    labelCol={{ span: labelColSpan }}
                    wrapperCol={{ span: wrapperColSpan }}
                  >
                    {this.state.data.voipPwd}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem
                    label="创建日期"
                    colon={false}
                    labelCol={{ span: labelColSpan }}
                    wrapperCol={{ span: wrapperColSpan }}
                  >
                    {moment(this.state.data.createTime).format(momentFormat)}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem
                    label="到期日期"
                    colon={false}
                    labelCol={{ span: labelColSpan }}
                    wrapperCol={{ span: wrapperColSpan }}
                  >
                    {moment(this.state.data.expireTime).format('YYYY-MM-DD')}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem
                    label="到期状态"
                    colon={false}
                    labelCol={{ span: labelColSpan }}
                    wrapperCol={{ span: wrapperColSpan }}
                  >
                    {this.state.data.expireStatus == 1 ? '未到期' : '已到期'}
                  </FormItem>
                </Col>
              </Row>

              <Title>归属信息</Title>
              <Row className={style.blockBody}>
                <Col span={12}>
                  <FormItem
                    label="所属学习中心"
                    colon={false}
                    labelCol={{ span: labelColSpan }}
                    wrapperCol={{ span: wrapperColSpan }}
                  >
                    {this.state.data.organizationName}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem
                    label="绑定员工数"
                    colon={false}
                    labelCol={{ span: labelColSpan }}
                    wrapperCol={{ span: wrapperColSpan }}
                  >
                    {this.state.data.bindNumber}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem
                    label="分配时间"
                    colon={false}
                    labelCol={{ span: labelColSpan }}
                    wrapperCol={{ span: wrapperColSpan }}
                  >
                    {moment(this.state.data.assignTime).format(momentFormat)}
                  </FormItem>
                </Col>
              </Row>

              {this.props.hideRenewRecord ? null : <Title>续费记录</Title>}
              {this.props.hideRenewRecord ? null : (
                <Row className={style.blockBody}>
                  <Col span={24}>
                    <List
                      style={{ height: '260px' }}
                      columns={this.renewCols}
                      dataSource={this.state.renewList}
                      pagination={{
                        total: this.state.recordTotal,
                        current: this.state.recordCurrent,
                        pageSize: this.state.recordSize,
                      }}
                      onChange={this.changeRecordPage}
                    />
                  </Col>
                </Row>
              )}
            </Wrapper>
          </Content>
          <Sider className={style.Sider} width={360}>
            <Wrapper header={<Title>用户绑定记录</Title>} bodyFitHeight>
              <List
                columns={this.recordColumns}
                dataSource={this.state.list}
                pagination={{
                  total: this.state.total,
                  pageSize: this.state.pageSize,
                  current: this.state.pageCurrent,
                  showSizeChanger: false,
                  showQuickJumper: false,
                }}
                onChange={this.changePage}
              />
            </Wrapper>
          </Sider>
        </Layout>
      </Modal>
    );
  }
}

export default ModalAgentDetail;
