import {
  getSelectOption,
  getSelectOptionGroup,
  selectOrgByIdAndTypeOption,
  getBrenchForSelection,
  getRegionList,
  getCampusByLoginUser,
  getAllCampusForSelection,
  findClassLevelByClassType,
  getMainClassTcDict,
  getAuxClassTcDict,
  getSmallClassTcDict,
  findClassLevelTree,
  getSelectOptionNew,
  getUserByRoldCodesSelection,
  getAllMainClassTcDict,
  findOrganizationById,
  getSchoolByBlCampus,
  getClassroomByCampu,
  getLoginUserTwoTeacherListGroupByArea,
  findMiniClassTeacherForSelectGroupByArea,
  getReceptionistGroupByCampus,
  getCampusAll,
  getCampusByUserBelongBranch,
  getMiniClassTeacherListForClassChange,
  getTaggedSelectOption,
  getDistinctUserGroupByCampus,
  getCityTree,
  getQuarterIfCanEditList,
  getSubjectAlias,
} from 'SERVICE/common';
import { getDataDictList } from 'SERVICE/system';
import { getJxDict } from 'SERVICE/entranceExam';
import { FIELD_FILLER, checkFieldEmpty } from 'COMPONENT/utils/form';
import { uniqBy } from 'lodash';

/**
 * 格式化普通字典
 */
export const formateDict = ({ value }) => {
  let dict = [];
  Object.keys(value).forEach((key) => {
    if (key) {
      dict.push({ val: key, text: value[key] });
    }
  });
  return dict;
};

export const formatDictNew = ({ data }, noFilter) => {
  return (
    noFilter
      ? data
      : data.filter(({ tag }) => {
          return tag && JSON.parse(tag).selectShow;
        })
  )
    .map(({ name, value, tag }) => {
      return { val: value, text: name, disabled: !tag || !JSON.parse(tag).selectShow };
    })
    .sort((a, b) => {
      return (b.disabled ? 0 : 1) - (a.disabled ? 0 : 1);
    });
};

/**
 * 根据format数据模版转换数据
 * */
export const transformArrayToSelect = (data = [], format) => {
  const { valName = 'val', textName = 'text', optionsName = 'options' } = format;
  const newData = [];
  data.map((item) => {
    const _item = {};
    if (item[optionsName] && item[optionsName].length > 0) {
      _item.options = transformArrayToSelect(item[optionsName], format);
      _item.label = item[textName];
    } else {
      _item.text = item[textName];
    }
    _item.val = item[valName];
    newData.push(_item);
  });
  return newData;
};

/**
 * 格式化批量字典
 *
 * @param data 接口返回数据
 * @param categoryArr 字典类型数组
 */
export const formatDictByGroup = (data = [], categoryArr = []) => {
  let dict = {};

  if (!data) {
    return dict;
  }

  if (data && data.length > 0) {
    data.forEach((item) => {
      dict[item.category] = formateDict({ value: item.value });
    });
  } else if (categoryArr) {
    categoryArr.forEach((item) => {
      dict[item.category] = [];
    });
  }

  return dict;
};

/**
 * 将普通字典格式化为分组字典
 * @param  {[object]} dict 将被分组的字典，格式如下：
  [{
    val: string  选项值
    text: string  选项文本
    [groupKeyName]: string  用于分组的字段，默认为'groupName'
    groupSort: number  分组的顺序
  }]
 * @param  {string} groupKeyName 用于分组的字段名
 * @return {[object]} 分组后的字典，格式如下：
   [{
    label: string  组名
    options:  一个组下面的选项
    [{
      val: string  选项值
      text: string  选项文本
    }]
   }]

 */
export const formatGroupDict = (dict, groupKeyName) => {
  groupKeyName = groupKeyName || 'groupName';
  let groupDict = [];
  if (dict) {
    dict.forEach((d) => {
      const newOption = { ...d };
      const label = d[groupKeyName];
      let group = groupDict.find((g) => g.label === label);
      if (group) {
        group.options = [...group.options, newOption];
      } else {
        groupDict.push({
          label,
          options: [newOption],
          groupSort: d.groupSort,
        });
      }
    });
  }
  groupDict.sort((a, b) => a.groupSort > b.groupSort);
  return groupDict;
};

/**
 * 将行政区划数据格式化为省份字典
 */
export const formateProvinceDict = (data) => {
  if (!data) {
    return [];
  }
  const dict = data.map((item) => {
    return { val: item.id, text: item.name };
  });
  return dict;
};

/**
 * 获取业务年份
 * @param yearDict {[object]} 年份字典
 * @param date {Date=} 要转换的年份(可选，默认为当前年份)
 * @param options {object}
 * */
export const getBLoYearId = (yearDict, date, options) => {
  // TODO: 是否把默认值改为8(即9月1日后生年份)
  let bLoMount = 10;
  let needGoUp = true;
  if (options) {
    if (options.hasOwnProperty('mount') && typeof options.mount === 'number') {
      bLoMount = options.mount - 1;
    }
    if (options.hasOwnProperty('goUp') && typeof options.goUp === 'boolean') {
      needGoUp = options.goUp;
    }
  }

  const _date = date || new Date();
  let y = _date.getFullYear();
  const m = _date.getMonth();
  if (m >= bLoMount && needGoUp) {
    y++;
  }
  return getDictValByText(yearDict, y.toString());
};

/**
 * 将行政区划数据格式化为城市字典
 */
export const formateCityDict = (data, provinceId) => {
  if (!data || !provinceId) {
    return [];
  }
  const province = data.filter((item) => item.id === provinceId)[0];
  let dict = [];
  if (province && province.children) {
    dict = province.children.map((item) => {
      return { val: item.id, text: item.name };
    });
  }
  return dict;
};

/**
 * 加载按省份分组的城市字典
 * @param  {boolean} isFull  - 是否完整版，即包含直辖市、直辖县、港澳台
 * @return {object[]}
 */
export const loadProvinceCityDict = async (isFull) => {
  let directCity = []; //直辖市
  let directCounty = []; //直辖县
  let special = []; //港澳台
  if (!isFull) {
    directCity = ['110000', '120000', '310000', '500000'];
    directCounty = ['139000', '419000', '429000', '469000', '659000'];
    special = ['710000', '810000', '820000'];
  }
  const data = await getRegionList({}, true);
  const dict = data.map((province) => {
    let citys = [];
    if (province.children) {
      province.children.forEach((city) => {
        if (directCounty.indexOf(city.id) < 0) {
          citys.push({ val: city.id, text: city.name });
        }
      });
    }
    if ([...directCity, ...special].indexOf(province.id) >= 0) {
      return { val: province.id, text: province.name };
    } else {
      return { val: province.id, label: province.name, options: citys };
    }
  });
  return dict;
};

/**
 * 根据省份获取城市字典
 * @param  {object[]} groupDict - 省份城市分组字典
 * @param  {string} provinceId  - 省份ID
 * @return {object[]}
 */
export const getCityDictByProvince = (groupDict, provinceId) => {
  let dict = [];
  if (groupDict && provinceId) {
    const group = groupDict.find((g) => g.val === provinceId);
    if (group) {
      dict = group.options;
    }
  }
  return dict;
};

/**
 * 根据字典值获取显示文字
 */
export const getDictTextByVal = (dict, val, checkEmpty) => {
  if (checkEmpty && checkFieldEmpty(val)) {
    return FIELD_FILLER;
  }
  if (!dict || dict.length === 0) {
    return checkEmpty ? FIELD_FILLER : null;
  }
  let option;
  if (dict[0].label) {
    dict.forEach((item) => {
      if (option) {
        return;
      }
      item.options.forEach((opt) => {
        if (opt.val === val) {
          option = opt;
          return;
        }
      });
    });
  } else {
    option = dict.find((item) => item.val === val);
  }
  return option ? option.text : checkEmpty ? FIELD_FILLER : null;
};

/**
 * 根据字典文字获取其值
 */
export const getDictValByText = (dict, text, checkEmpty) => {
  if (checkEmpty && checkFieldEmpty(text)) {
    return FIELD_FILLER;
  }
  if (!dict || dict.length === 0) {
    return checkEmpty ? FIELD_FILLER : null;
  }
  let option;
  if (dict[0].label) {
    dict.forEach((item) => {
      if (option) {
        return;
      }
      item.options.forEach((opt) => {
        if (opt.text === text) {
          option = opt;
          return;
        }
      });
    });
  } else {
    option = dict.find((item) => item.text === text);
  }
  return option ? option.val : checkEmpty ? FIELD_FILLER : null;
};

/**
 * 根据字典值获取选项对象
 */
export const getDictOptionByVal = (dict, val) => {
  if (!dict || !val || dict.length === 0) {
    return null;
  }
  let option;
  if (dict[0].label) {
    dict.forEach((item) => {
      if (option) {
        return;
      }
      item.options.forEach((opt) => {
        if (opt.val === val) {
          option = opt;
          return;
        }
      });
    });
  } else {
    option = dict.find((item) => item.val === val);
  }
  return option || null;
};

/**
 * 通过值查找某个字典匹配的选项的值
 * 例如：通过分公司的值查找对应学习中心字典的选项值 getOptionValueInFormatterDict(branchCampusDict, branchVal)
 * @param dict 已经格式化过的字典 [label: '', val: '', options: [{val: '', text: ''}]]
 * @param value 需要匹配的值
 * @return {null|*}
 */
export const getOptionValueInFormatterDict = function (dict = [], value = '') {
  if (!dict || dict.length <= 0 || !value) {
    return null;
  }

  const item = dict.find((item) => item.val === value) || { options: [] };

  return item.options[0] && item.options[0].val;
};

/**
 * 将字符串的category转换为批量获取数组的参数
 * @param str 'STUDENT_GRADE,SUBJECT'
 * @return {Array}
 */
export const transformStringToCategoryArray = function (str = '') {
  if (!str) {
    return [];
  }

  let category = [];
  const arr = str.split(',');

  arr.forEach((key) => {
    category.push({ category: key, parentId: '' });
  });

  return category;
};

/**
 * 加载公共字典
 * @param {string} name 字典名称
 */
export const loadCommonDict = async (name, type) => {
  try {
    const data = await getSelectOption({ selectOptionCategory: name, parentId: type }, true);
    return formateDict(data);
  } catch (e) {
    console.error(e);
    return [];
  }
};

export const loadCommonDictNew = async (category, parentId, noFilter = false) => {
  try {
    const data = await getSelectOptionNew({ category, parentId });
    return formatDictNew(data, noFilter);
  } catch (e) {
    return [];
  }
};

/**
 * 批量获取字典,支持多种传参方式
 * eg: loadCommonDictByGroup('STUDENT_GRADE,SUBJECT')
 * eg: loadCommonDictByGroup([{category: 'SUBJECT'}, {category: 'STUDENT_GRADE', parentId: '123'}])
 * eg: loadCommonDictByGroup(['SUBJECT', {category: 'STUDENT_GRADE', parentId: '123'}])
 *
 * @param {string|array} category 字典类型
 * @return {object} {STUDENT_GRADE: [], SUBJECT: []}
 */
export const loadCommonDictByGroup = async (category) => {
  const isString = Object.prototype.toString.call(category) === '[object String]';
  const isArray = Object.prototype.toString.call(category) === '[object Array]';

  if (!isString && !isArray) {
    console.warn('TypeError: 批量获取字典调用参数错误');
    return {};
  }

  let categoryArr = [];
  if (isString) {
    let arr = category.split(',');
    arr.forEach((value) => {
      if (value) {
        categoryArr.push({ category: value.trim(), parentId: '' });
      }
    });
  } else {
    category.forEach((value) => {
      let item = {};

      if (Object.prototype.toString.call(value) === '[object String]') {
        item = { category: value, parentId: '' };
      } else {
        item = { category: value.category || '', parentId: value.parentId || '' };
      }

      categoryArr.push(item);
    });
  }

  try {
    const response = await getSelectOptionGroup(categoryArr);

    let dict = {};
    if (response && response.data) {
      dict = formatDictByGroup(response.data, categoryArr);
    } else {
      dict = formatDictByGroup([], categoryArr);
    }

    return dict;
  } catch (e) {
    return formatDictByGroup([], categoryArr);
  }
};

/**
 * 加载当前用户所属分公司字典
 */
export const loadBranchDict = async () => {
  const data = await selectOrgByIdAndTypeOption({ orgType: 'BRENCH' }, true);
  return formateDict(data);
};

/** 新
 * 加载当前用户所属分公司字典 包括子学习中心
 */
export const loadUserBranchDict = async () => {
  const data = await selectOrgByIdAndTypeOption({ orgType: 'BRENCH' }, true);
  return formateDict(data);
};

/**
 * 加载学习中心字典
 * @param {string} branch 分公司
 */
export const loadCampusDict = async (branch) => {
  const data = await selectOrgByIdAndTypeOption({ orgType: 'CAMPUS', orgId: branch });
  return formateDict(data);
};

/**
 * 加载用户学习中心字典
 */
export const loadUserCampusDict = async () => {
  const data = await selectOrgByIdAndTypeOption({ orgType: 'CAMPUS' });
  return formateDict(data);
};

/**
 * 加载所有分公司字典
 */
export const loadAllBranchDict = async () => {
  const data = await getBrenchForSelection({}, true);
  return formateDict(data);
};

/**
 * 加载当前用户所属学习中心按分公司分组字典
 */
export const loadBranchCampusDict = async () => {
  const data = await getCampusByLoginUser({}, true);
  return data.map((group) => {
    const branch = group[0] || {};
    const campuses = group && group.length > 1 ? group.slice(1) : [];
    return {
      val: branch.id,
      label: branch.name,
      options: campuses.map((d) => {
        return { val: d.id, text: d.name };
      }),
    };
  });
};

/**
 * 处理单个分公司所属学习中心
 */
export const dealCampus = (campusDict, branchId) => {
  const dict = campusDict.filter((campus) => {
    return campus.val === branchId;
  });
  return dict.length === 0 ? dict : dict[0].options;
};

/**
 * 加载所有学习中心按分公司分组字典
 */
export const loadAllBranchCampusDict = async () => {
  const data = await getAllCampusForSelection({}, true);
  let dict = [];
  data.forEach((d) => {
    const opt = { val: d.orgId, text: d.orgName };
    const branchId = d.orgParentId;
    let group = dict.find((v) => v.val === branchId);
    if (!!group) {
      group.options.push(opt);
    } else {
      dict.push({ val: branchId, label: d.orgParentName, options: [opt] });
    }
  });
  return dict;
};

/**
 * 将后台返回的数据转换为符合select组件的数据格式
 * @param data
 * @returns {Array}
 */
export const mapDataToDict = (data) => {
  let dict = Object.values(data).map((v, k) => {
    return {
      text: v,
      val: Object.keys(data)[k],
    };
  });
  let cleacDict = dict.filter((v, k) => {
    return v.val !== '';
  });
  return cleacDict;
};

/**
 * 字典选项添加标题
 */
export const dictAddOptionTitle = (dict) => {
  if (!dict) {
    return null;
  }
  return dict.map((d) => {
    if (d.label) {
      return {
        ...d,
        title: d.text,
        options: d.options
          ? d.options.map((opt) => {
              return { ...opt, title: opt.text };
            })
          : null,
      };
    } else {
      return { ...d, title: d.text };
    }
  });
};

/**
 * 班级层次字典
 * */
export const loadClassLevelDict = async (classType) => {
  const { data } = await findClassLevelByClassType({ classTypeId: classType });
  return data.map((item) => ({ val: item.ID, text: item.NAME }));
};

/**
 * 加载双师主班老师字典(当前登录用户所属学习中心)
 * */
export const loadMainClassTcDict = async (params) => {
  const data = await getMainClassTcDict(params);
  const teachers = data.map((tc) => {
    return {
      text: tc.name,
      val: tc.userId,
    };
  });
  teachers.unshift({ text: '其他', val: 'other' });
  return teachers;
};
/**
 * 加载双师辅班老师字典(当前登录用户所属学习中心)
 */
export const loadAuxClassTcDict = async (params) => {
  const data = await getAuxClassTcDict(params);
  return data.map((tc) => {
    return {
      text: tc.name,
      val: tc.userId,
    };
  });
};
/**
 * 加载小班授课老师数据
 */
export const loadSmallClassTcDict = async (params) => {
  const data = await getSmallClassTcDict(params);
  const teachers = data.map((tc) => {
    return {
      text: tc.name,
      val: tc.userId,
    };
  });
  teachers.unshift({ text: '其他', val: 'other' });
  return uniqBy(teachers, 'val');
};

/**
 * 加载小班班主任数据(学管师)
 * */
export const loadSmallMainTc = async (params) => {
  const userData = Index.getLoginUser();
  let fetchParams = {
    roleCode: userData.roleCode,
    organizationId: userData.organizationId,
    monitoringUserId: userData.userId,
    monitoringUserName: userData.name,
  };
  const rsp = await getUserByRoldCodesSelection(fetchParams, true);
  let dictKey = Object.keys(rsp.value);
  let dict = [];
  dictKey.forEach((item) => {
    dict.push({
      text: rsp.value[item],
      val: item,
    });
  });
  return dict;
};
/**
 * 加载客户类型
 * 优惠用到其他地方不清楚
 * */
export const loadClientTypeDict = async () => {
  const { data } = await getSelectOptionNew({ category: 'PromotionCusCondition' });
  return data.map((v, k) => {
    return { val: v.value, text: v.name };
  });
};
/**
 * 根据父类型获取字典
 * */
export const loadDictByFather = async (father, params) => {
  const rsp = await getDataDictList({
    category: father,
    'parentDataDict.id': params,
  });
  return rsp.rows.map((item) => ({ text: item.name, val: item.id }));
};
// 加载所有双师主班老师字典
export const loadAllMainClassTcDict = async (params) => {
  const data = await getAllMainClassTcDict(params);
  const teachers = data.map((tc) => {
    return {
      text: tc.name,
      val: tc.userId,
    };
  });
  teachers.unshift({ text: '其他', val: 'other' });
  return teachers;
};
// 基础状态
export const loadBaseStatusDict = async () => {
  try {
    const rsp = await getSelectOption({ selectOptionCategory: 'BaseStatus' });
    return formateDict(rsp);
  } catch (e) {
    return [];
  }
};
// 获取学习中心对应城市的学校
export const getSchoolByCampus = async (campusId) => {
  const rspCity = await findOrganizationById({ id: campusId }, true);
  if (rspCity && rspCity.cityId) {
    const rspSchool = await getSchoolByBlCampus({ blCampus: rspCity.cityId });
    return formateDict(rspSchool);
  }
  return [];
};

// 获取 产品类型->班级类型->班级层次 树
export const loadProductTypeTree = async (isCascade) => {
  try {
    const rsp = await findClassLevelTree();
    return buildProductTypeTree(rsp.data);
  } catch (e) {
    return [];
  }
};
// 格式化loadProductTypeTree返回的数据
function buildProductTypeTree(data, needLabel) {
  return data.map((item) => {
    let _item = {
      text: item.name,
      val: item.id,
    };

    if (item.classTypes) {
      _item.classTypes = buildProductTypeTree(item.classTypes);
    }
    if (item.courseSeries) {
      _item.courseSeries = buildProductTypeTree(item.courseSeries);
    }
    if (item.classLevels) {
      _item.classLevels = buildProductTypeTree(item.classLevels);
    }
    if (item.productGroup) {
      _item.productGroup = buildProductTypeTree(item.productGroup);
    }

    return _item;
  });
}

// 处理多选框传参
export const joinArrToStr = (param) => {
  return param === null ? null : param.join(',');
};

export const loadClassRoomsByCampusId = async (campusId) => {
  const data = await getClassroomByCampu({ campusId });
  return data.map((campus) => {
    return {
      text: campus.name,
      val: campus.value,
    };
  });
};

// 获取`双师辅导老师`(所属分公司下所有学习中心，按学习中心分组)
export const loadBranchAuxClassTcDict = async () => {
  try {
    const rsp = await getLoginUserTwoTeacherListGroupByArea();
    return formatAuxTcTree(rsp.data);
  } catch (e) {
    return [];
  }
};
// 获取`小班任课老师`(所属分公司下所有学习中心，按学习中心分组)
export const loadBranchSmallClassAuxTc = async () => {
  try {
    const rsp = await findMiniClassTeacherForSelectGroupByArea();
    return formatAuxTcTree(rsp.data);
  } catch (e) {
    return [];
  }
};
// 获取小班任课老师(有权限分公司下所有学习中心,按学习中心分组)
export const loadMiniClassTeacherListForClassChange = async () => {
  try {
    const rsp = await getMiniClassTeacherListForClassChange();
    return formatAuxTcTree(rsp.data);
  } catch (e) {
    return [];
  }
};

// 格式化`双师辅导老师`与`小班任课老师`返回数据
function formatAuxTcTree(data) {
  return data.map((item) => {
    let option = {
      label: item.orgName,
      val: item.orgId,
      options: [],
    };
    if (item.children && item.children.length > 0) {
      option.options = formatAuxTcTree(item.children);
    }
    if (item.teachers && item.teachers.length > 0) {
      if (item.orgType === 'BRENCH') {
        let otherOptions = {
          label: item.orgName,
          val: item.orgId,
          options: formatTeacher(item.teachers),
        };
        option.options.unshift(otherOptions);
      }
      if (item.orgType === 'CAMPUS') {
        option.options = formatTeacher(item.teachers);
      }
    }
    return option;
  });
}
function formatTeacher(data) {
  return data.map((teacher) => ({
    text: teacher.userName,
    val: teacher.userId,
    _origin: teacher,
  }));
}

/**
 * 加载当前用户所属分公司学习中心树
 */
export const loadBranchCampusTree = async () => {
  const data = await getCampusByLoginUser({}, true);
  return data.map((group) => ({
    text: group[0].name,
    val: group[0].id,
    campus: (group.slice(1) || []).map((campus) => ({
      text: campus.name,
      val: campus.id,
    })),
  }));
};

/**
 * 加载所有分公司学习中心树
 */
export const loadAllBranchCampusTree = async () => {
  const data = await getCampusAll({}, true);
  return data.map((group) => ({
    text: group[0].name,
    val: group[0].id,
    campus: (group.slice(1) || []).map((campus) => ({
      text: campus.name,
      val: campus.id,
    })),
  }));
};

/**
 * 获取跟进人字典
 *
 * showBelong:
 * 1 分公司
 * 2 学习中心
 * jobs:
 * WLZY("WLZY","网络专员"),
 * WLZG("WLZG","网络主管"),
 * XGZG("XGZG","学管主管"),
 * LS("LS","老师"),
 * WHZG("WHZG","外呼主管"),//TMK主管改为外呼主管
 * WHZY("WHZY", "外呼专员"),//TMK专员 改为外呼专员
 * ZXS("ZXS", "咨询师"),//咨询师
 * XQZR("XQZR","学习中心主任"),
 * QT("QT","前台"),
 * YYJL("YYJL","营运经理"),//新增营运经理  20170220
 * YYZJ("YYZJ","营运总监"),
 * XQYYFZR("XQYYFZR","学习中心营运主任"),
 * ZXZG("ZXZG","咨询主管"),
 * WLYXZY("WLYXZY","网络营销专员"),
 * XGS("XGS","学管师"),
 * SCJL("SCJL","市场经理"),
 * YYZR("YYZR","运营主任"),
 * SCZY("SCZY","市场专员");
 * */
export const getFollowUpPersonDict = async (params = {}) => {
  const { showBelong, jobs } = params;
  let urlParams = 'showBelong=';
  if (showBelong) {
    urlParams += showBelong;
  }
  if (Array.isArray(jobs) && jobs.length > 0) {
    urlParams += jobs.reduce((total, current) => {
      return total + `&job[]=${current}`;
    }, '');
  }
  const origin = await getReceptionistGroupByCampus(urlParams, true);
  let diffStorage = {};
  let sortByKey = {};
  origin.forEach((item) => {
    if (diffStorage[item.userId]) {
      return;
    }
    diffStorage[item.userId] = true;
    if (!sortByKey[item.orgId]) {
      sortByKey[item.orgId] = [];
    }
    sortByKey[item.orgId].push(item);
  });
  return Object.values(sortByKey).map((options) => ({
    label: options[0].orgName,
    val: options[0].orgId,
    options: options.map((opt) => ({
      text: opt.userName,
      val: opt.userId,
    })),
  }));
};

export const getCampusByUserBelongBranchApi = async () => {
  const data = await getCampusByUserBelongBranch();
  const dict = data.map((branchCampus) => {
    const label = branchCampus[0].name;
    const val = branchCampus[0].id;
    branchCampus.shift();
    const options = branchCampus.map((campus) => {
      return {
        val: campus.id,
        text: campus.name,
      };
    });
    return {
      label,
      val,
      options,
    };
  });
  return dict;
};

// 获取资源入口字典
export const getEntranceDict = async (selectOptionCategory, tags) => {
  const rsp = await getTaggedSelectOption({ selectOptionCategory, tags }, true);
  if (rsp && rsp.data) {
    return (rsp.data || []).map((item) => ({
      text: item.name,
      val: item.value,
    }));
  }
  return [];
};

// 获取教学数据字典
export const getJxDictByType = async (dictType) => {
  try {
    const rsp = await getJxDict({ dictType }, true);

    if (rsp && Array.isArray(rsp.data)) {
      return rsp.data.map((item) => ({
        text: item.codeValue,
        val: item.code,
      }));
    }

    return [];
  } catch (e) {
    return [];
  }
};

export const getOutcallUser = async () => {
  try {
    const rsp = await getDistinctUserGroupByCampus({ showBelong: 1, job: ['WHZY', 'WHZG'] });

    if (rsp && Array.isArray(rsp)) {
      return rsp.map((item) => ({
        text: item.userName,
        val: item.userId,
      }));
    }

    return [];
  } catch (e) {
    return [];
  }
};

export const getCityTreeDict = async () => {
  try {
    const resp = await getCityTree();
    return resp;
  } catch (e) {
    return [];
  }
};

// 小班签单季度
export const getQuarterTypeDict = async (apiParams) => {
  try {
    const rsp = await getQuarterIfCanEditList(apiParams);

    return rsp.data.map((item) => ({
      val: item.quarterTypeValue,
      text: item.quarterTypeName,
      disabled: !item.canEdit,
    }));
  } catch (e) {
    return [];
  }
};

/**
 * 获取科目别名字典
 *
 * @param params {Object<{ orgId: string, gradeId: string }>}
 * @return {Promise<{val: *, text: ts.SymbolDisplayPartKind.aliasName | number | *}[]|Array>}
 */
export async function loadSubjectAliasDict(params = {}) {
  const { orgId = '', gradeId = '' } = params || {};

  try {
    const { data } = await getSubjectAlias({ orgId, gradeId });

    return (data || []).map((item) => {
      return {
        text: item.aliasName || item.name,
        val: item.value,
      };
    });
  } catch (e) {
    return [];
  }
}
