import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'dva';
import { isFunction } from 'UTIL/lang/data-type';
import {
  CALL_LOGIN_STATUS_RINGING,
  CALL_LOGIN_STATUS_CALLING,
  callLoginStatusCfg,
  CALL_DIRECT_ANSWER,
  CALL_DIRECT_CALL,
  CALL_WAY_WEB,
  callWayCfg,
} from 'CONST/callConfig';
import { DefaultPhoto } from 'COMPONENT';
import CallButton from 'BLOC/CallOutSider/components/CallButton';

import styles from './styles.module.less';

// 底部状态栏
const statusRenderTable = [
  {
    agentLoginStatus: CALL_LOGIN_STATUS_RINGING,
    direction: CALL_DIRECT_ANSWER,
    render: (callWay) => {
      const { [callWay]: callWayLabel } = callWayCfg;

      return `请在${callWayLabel}上接听来电`;
    },
  },
  {
    agentLoginStatus: CALL_LOGIN_STATUS_RINGING,
    direction: CALL_DIRECT_CALL,
    render: () => '对方接通中...',
  },
  {
    agentLoginStatus: CALL_LOGIN_STATUS_CALLING,
    direction: CALL_DIRECT_ANSWER,
    render: (callWay) => {
      const { [callWay]: callWayLabel } = callWayCfg;

      return `${callWayLabel}接听中...`;
    },
  },
  {
    agentLoginStatus: CALL_LOGIN_STATUS_CALLING,
    direction: CALL_DIRECT_CALL,
    render: () => '对方接听中...',
  },
];

// 底部操作按钮
const actionRenderTable = [
  {
    agentLoginStatus: CALL_LOGIN_STATUS_RINGING,
    direction: [CALL_DIRECT_ANSWER],
    render: (props = {}) => {
      // 呼入，接通中
      const { answer, hangUp, answerEnabled, hangUpEnabled } = props;

      return (
        <div className={styles.callActionWrapper}>
          <CallButton className={styles.callHangUpBtn} type="HAND_UP" disabled={!hangUpEnabled} onClick={hangUp} />
          <CallButton className={styles.callAnswerBtn} type="ANSWER" disabled={!answerEnabled} onClick={answer} />
        </div>
      );
    },
  },
  {
    agentLoginStatus: CALL_LOGIN_STATUS_CALLING,
    direction: [CALL_DIRECT_CALL, CALL_DIRECT_ANSWER],
    render: (props = {}) => {
      // 通话中
      const { hangUp, hangUpEnabled } = props;

      return (
        <div className={styles.callActionWrapper}>
          <CallButton type="HAND_UP" disabled={!hangUpEnabled} onClick={hangUp} />
        </div>
      );
    },
  },
];

class CallingPanel extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      answerEnabled: true,
      hangUpEnabled: true,
    };

    this.answer = this.answer.bind(this);
    this.hangUp = this.hangUp.bind(this);
  }

  static propTypes = {
    phoneNum: PropTypes.oneOfType([PropTypes.number, PropTypes.string]).isRequired,
    name: PropTypes.string.isRequired,
    agentLoginStatus: PropTypes.number,
    direction: PropTypes.number,
    callWay: PropTypes.number,
  };

  /**
   * 接听网路电话
   */
  async answer() {
    this.setState({
      answerEnabled: false,
      hangUpEnabled: false,
    });

    try {
      await new Promise((resolve, reject) => {
        this.props.dispatch({
          type: 'callCenter/answerCall',
          payload: {
            resolve,
            reject,
          },
        });
      });

      setTimeout(() => {
        this.setState({
          answerEnabled: true,
          hangUpEnabled: true,
        });
      }, 3000);
    } catch (e) {
      this.setState({
        answerEnabled: true,
        hangUpEnabled: true,
      });
    }
  }

  /**
   * 挂断网路电话
   */
  async hangUp() {
    this.setState({
      answerEnabled: false,
      hangUpEnabled: false,
    });

    try {
      await new Promise((resolve, reject) => {
        this.props.dispatch({
          type: 'callCenter/hangUp',
          payload: {
            resolve,
            reject,
          },
        });
      });

      setTimeout(() => {
        this.setState({
          answerEnabled: true,
          hangUpEnabled: true,
        });
      }, 3000);
    } catch (e) {
      this.setState({
        answerEnabled: true,
        hangUpEnabled: true,
      });
    }
  }

  renderActionBar() {
    const { agentLoginStatus, direction } = this.props;

    const [{ render } = {}] = actionRenderTable.filter((item) => {
      return item.direction.indexOf(direction) > -1 && agentLoginStatus === item.agentLoginStatus;
    });

    const { answerEnabled, hangUpEnabled } = this.state;

    const actionProps = {
      answerEnabled,
      hangUpEnabled,
      answer: this.answer,
      hangUp: this.hangUp,
    };

    return isFunction(render) ? render(actionProps) : '';
  }

  renderStatusTips() {
    const { agentLoginStatus, direction, callWay } = this.props;

    const [{ render } = {}] = statusRenderTable.filter((item) => {
      return direction === item.direction && agentLoginStatus === item.agentLoginStatus;
    });

    return isFunction(render) ? render(callWay) : '';
  }

  render() {
    const { phoneNum, name, agentLoginStatus, callWay } = this.props;
    const isVertoMode = callWay === CALL_WAY_WEB;

    return (
      <div className={styles.callingPanel}>
        <div className={styles.phoneNum}>{phoneNum || '未知号码'}</div>

        <div className={styles.photoWrapper}>
          <DefaultPhoto />
        </div>

        <div className={styles.personName}>{name || '未知客户'}</div>

        <div className={styles.callStatus}>{callLoginStatusCfg[agentLoginStatus]}</div>

        <div className={styles.callFooter}>
          {isVertoMode ? (
            this.renderActionBar()
          ) : (
            <div className={styles.callStatusTips}>{this.renderStatusTips()}</div>
          )}
        </div>
      </div>
    );
  }
}

export default connect()(CallingPanel);
