import React from 'react';
import { connect } from 'dva';
import { Menu, Dropdown, Button, Alert, Icon as AntdIcon } from 'antd';
import { Icon, Loading, NoDataPlus } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { isNumber, isFunction } from 'UTIL/lang/data-type';
import {
  CALL_LOGIN_STATUS_OFFLINE,
  CALL_LOGIN_STATUS_LEISURE,
  CALL_LOGIN_STATUS_RINGING,
  CALL_LOGIN_STATUS_CALLING,
  CALL_WAY_WEB,
  CALL_WAY_SIP,
  CALL_WAY_PHONE,
  CALL_DIRECT_ANSWER,
} from 'CONST/callConfig';
import CallWayPanel from '../CallWayPanel';
import PhonePanel from '../PhonePanel';
import CallingPanel from '../CallingPanel';

import styles from './styles.module.less';
import moment from 'moment';
class CallCenterPanel extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      // 初始化
      callWay: null,
      callWayPanelVisible: true,
      phonePanelVisible: false,
    };

    this.handleMenuClick = this.handleMenuClick.bind(this);
    this.handleOnlineClick = this.handleOnlineClick.bind(this);
    this.handleCallWayChange = this.handleCallWayChange.bind(this);
    this.handlePhoneUpdate = this.handlePhoneUpdate.bind(this);
    this.changePhonePanelVisible = this.changePhonePanelVisible.bind(this);
    this.renderIcon = this.renderIcon.bind(this);
    this.renderPanel = this.renderPanel.bind(this);
  }

  componentDidMount() {
    const { callWay: currentCallWay } = this.state;
    const { callWay: prevCallWay } = this.props;

    if (!currentCallWay && isNumber(prevCallWay)) {
      this.setState({
        callWay: prevCallWay,
      });
    }
  }

  componentDidUpdate(prevProps, prevState) {
    if (prevProps.callWay === this.props.callWay) {
      return;
    }

    const { callWay: currentCallWay } = this.state;
    const { callWay: outerCallWay } = this.props;

    if (!currentCallWay && isNumber(outerCallWay)) {
      this.setState({
        callWay: outerCallWay,
      });
    }
  }

  /**
   * 切换上下线菜单选项
   * @param {Object} param0
   * @param {string} param0.key 选中 key 值
   */
  handleMenuClick({ key }) {
    switch (Number(key)) {
      case CALL_LOGIN_STATUS_LEISURE:
        this.handleOnlineClick();
        break;
      case CALL_LOGIN_STATUS_OFFLINE:
        this.props.onOfflineClick();
        break;
    }
  }

  /**
   * 点击上线回调
   */
  handleOnlineClick() {
    const { phoneNumber } = this.props;
    const { callWay } = this.state;

    if (callWay === CALL_WAY_PHONE && !phoneNumber) {
      showMessage('需设置手机号码后才能上线', 'error');

      return;
    }

    this.props.onOnlineClick({ callWay });
  }

  /**
   * 响应呼叫方式更改
   * @param {number} callWay
   */
  handleCallWayChange(callWay) {
    this.setState({
      callWay,
    });
  }

  /**
   * 修改手机号码设置面板可见性
   * @param {boolean} visible
   */
  changePhonePanelVisible(visible = false) {
    this.setState({
      phonePanelVisible: visible,
    });
  }

  /**
   * 响应手机号码修改
   * @param {Object} payload
   */
  async handlePhoneUpdate(payload) {
    const successful = await this.props.onPhoneUpdate(payload);

    if (successful) {
      this.changePhonePanelVisible();
    }
  }

  /**
   * 渲染图标
   */
  renderIcon() {
    const { callWay } = this.props;

    const iconRenderTable = [
      {
        callWay: CALL_WAY_WEB,
        render: () => <AntdIcon className={styles.ccPanelHeaderTitleIcon} type="phone" title="网络电话" />,
      },
      {
        callWay: CALL_WAY_SIP,
        render: () => <Icon className={styles.ccPanelHeaderTitleIcon} type="sip-phone" title="SIP话机" />,
      },
      {
        callWay: CALL_WAY_PHONE,
        render: () => <AntdIcon className={styles.ccPanelHeaderTitleIcon} type="mobile" title="手机" />,
      },
    ];

    const [{ render } = {}] = iconRenderTable.filter((item) => item.callWay === callWay);

    return isFunction(render) ? render() : null;
  }

  /**
   * 渲染面板内容
   */
  renderPanel() {
    const { isOnline, agentLoginStatus, callMsgData } = this.props;

    const callingPanelVisible =
      isNumber(callMsgData.direction) &&
      [CALL_LOGIN_STATUS_RINGING, CALL_LOGIN_STATUS_CALLING].some((state) => state === agentLoginStatus);

    const { phonePanelVisible } = this.state;

    // 所有可能的面板
    const panelRenderTable = [
      {
        online: false,
        phonePanelVisible: false,
        callingPanelVisible: false,
        render: () => {
          const { phoneNumber } = this.props;
          const { callWay } = this.state;

          return (
            <CallWayPanel
              callWay={callWay}
              phoneNumber={phoneNumber}
              onChange={this.handleCallWayChange}
              onPhoneSetting={() => this.changePhonePanelVisible(true)}
            />
          );
        },
      },
      {
        online: false,
        phonePanelVisible: true,
        callingPanelVisible: false,
        render: () => {
          return <PhonePanel onConfirm={this.handlePhoneUpdate} onCancel={() => this.changePhonePanelVisible(false)} />;
        },
      },
      {
        online: true,
        phonePanelVisible: false,
        callingPanelVisible: true,
        render: () => {
          const {
            callWay,
            callMsgData: { direction, callerPhone, callerName, calleePhone, calleeName },
          } = this.props;

          const isCallingIn = direction === CALL_DIRECT_ANSWER;
          const custPhone = isCallingIn ? callerPhone : calleePhone;
          const custName = isCallingIn ? callerName : calleeName;

          return (
            <CallingPanel
              phoneNum={custPhone}
              name={custName}
              callWay={callWay}
              direction={direction}
              agentLoginStatus={agentLoginStatus}
            />
          );
        },
      },
      {
        online: true,
        phonePanelVisible: false,
        callingPanelVisible: false,
        render: () => <NoDataPlus imgClassName={styles.ccPanelBodyEmptyImg} showText={false} />,
      },
    ];

    const [{ render } = {}] = panelRenderTable.filter((item) => {
      return (
        isOnline === item.online &&
        phonePanelVisible === item.phonePanelVisible &&
        callingPanelVisible === item.callingPanelVisible
      );
    });

    return isFunction(render) ? render() : null;
  }

  render() {
    const { loading, isOnline, available, occupantName, expireTime } = this.props;

    const { phonePanelVisible } = this.state;

    const alertVisible = !available && !isOnline && !phonePanelVisible;

    return (
      <div className={styles.ccPanel} onClick={(e) => e.stopPropagation()}>
        <div className={styles.ccPanelHeader}>
          <div className={styles.ccPanelHeaderTop}>
            <div className={styles.ccPanelHeaderTitle}>
              <span className={styles.ccPanelHeaderTitleText}>呼叫服务</span>
              {this.renderIcon()}
            </div>
            <div className={styles.ccPanelHeaderTools}>
              <Dropdown
                trigger={['click']}
                overlay={
                  <Menu onClick={this.handleMenuClick}>
                    <Menu.Item key={CALL_LOGIN_STATUS_LEISURE}>在线</Menu.Item>
                    <Menu.Item key={CALL_LOGIN_STATUS_OFFLINE}>离线</Menu.Item>
                  </Menu>
                }
                disabled={phonePanelVisible}
              >
                <Button>
                  {isOnline ? '在线' : '离线'} <AntdIcon type="down" />
                </Button>
              </Dropdown>
            </div>
          </div>
          <div style={{ color: expireTime >= Date.now() ? '#04D500' : '#BC2200' }}>
            到期日期：{moment(expireTime).format('YYYY-MM-DD')}
          </div>
        </div>
        <div className={styles.ccPanelBody}>
          {alertVisible && (
            <Alert className={styles.ccPanelBodyAlert} message={`${occupantName}在线中`} type="warning" showIcon />
          )}
          {this.renderPanel()}
        </div>
        <Loading loading={loading} />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const { enabled, available, occupantName, agentLoginStatus, callWay, phoneNumber, expireTime } = state.callCenter;

  return {
    enabled,
    available,
    occupantName,
    phoneNumber,
    callWay,
    agentLoginStatus,
    expireTime,
    isOnline: agentLoginStatus !== CALL_LOGIN_STATUS_OFFLINE,
  };
};

export default connect(mapStateToProps)(CallCenterPanel);
