import React from 'react';
import { Form, Input, Button } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import smsApi from 'SERVICE/sms';

import styles from './styles.module.less';

const DEFAULT_COUNTDOWN = 60;

const FormItem = Form.Item;

class VerificationCodeRow extends React.Component {
  constructor(props) {
    super(props);
  }

  render() {
    const { countdown, btnDisabled, onBtnClick, ...restProps } = this.props;

    return (
      <div>
        <Input className={styles.codeInput} maxLength={8} allowClear placeholder="验证码" {...restProps} />
        <Button className={styles.codeBtn} disabled={btnDisabled} onClick={onBtnClick}>
          {btnDisabled ? countdown : '获取验证码'}
        </Button>
      </div>
    );
  }
}

class PhoneSettingPanel extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      countdown: 60,
      timer: 0,
      codeBtnDisabled: false,
    };

    this.formRef = null;

    this.phoneNumberValidator = this.phoneNumberValidator.bind(this);
    this.verificationCodeValidator = this.verificationCodeValidator.bind(this);
    this.handleConfirm = this.handleConfirm.bind(this);
    this.handleCancel = this.handleCancel.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
    this.handleSubmitFailed = this.handleSubmitFailed.bind(this);
    this.isPhoneNoValid = this.isPhoneNoValid.bind(this);
    this.handleGetCode = this.handleGetCode.bind(this);
    this.setCountdown = this.setCountdown.bind(this);
  }

  /**
   * 设置倒计时
   * @param {boolean} initial 是否第一次执行
   */
  setCountdown(initial = false) {
    const { countdown, timer } = this.state;

    if (initial) {
      this.setState({
        countdown: DEFAULT_COUNTDOWN,
        codeBtnDisabled: true,
      });
    } else if (countdown === 0) {
      clearTimeout(timer);

      this.setState({
        timer: 0,
        codeBtnDisabled: false,
      });

      return;
    }

    const nextTimer = setTimeout(() => {
      this.setState(
        (prevState) => {
          const nextCountdown = prevState.countdown - 1;

          return {
            countdown: nextCountdown,
            timer: nextTimer,
          };
        },
        () => {
          this.setCountdown();
        },
      );
    }, 1000);
  }

  /**
   * 发送验证码
   */
  async handleGetCode() {
    const rawPhoneNumber = this.formRef.getFieldValue('phoneNumber');
    const phoneNumber = String(rawPhoneNumber).trim();

    if (!this.isPhoneNoValid(phoneNumber)) {
      showMessage('请填写有效的手机号码', 'error');

      return;
    }

    try {
      const response = await smsApi.sendVerificationCode({
        phone: phoneNumber,
      });

      if (response && response.data) {
        this.setCountdown(true);
        showMessage('发送验证码成功，请查看短信', 'success');
      } else {
        showMessage('发送验证码失败', 'error');
      }
    } catch (e) {
      showMessage('发送验证码失败', 'error');
    }
  }

  /**
   * 验证手机号码
   * @param {string} value
   */
  isPhoneNoValid(value) {
    return /^1\d{10}$/.test(value);
  }

  /**
   * 表单手机号码校验
   * @param {Object} rule
   * @param {String} value
   * @param {Function} callback
   */
  phoneNumberValidator(rule, value, callback) {
    const phoneNumber = String(value).trim();

    if (!this.isPhoneNoValid(phoneNumber)) {
      return callback('请输入正确的手机号码');
    }

    callback();
  }

  /**
   * 表单验证码校验
   * @param {Object} rule
   * @param {String} value
   * @param {Function} callback
   */
  verificationCodeValidator(rule, value, callback) {
    const verificationCode = String(value).trim();

    if (!/^\d+$/.test(verificationCode)) {
      return callback('请输入正确的验证码');
    }

    callback();
  }

  handleConfirm() {
    this.formRef.submitFields();
  }

  handleCancel() {
    this.props.onCancel();
  }

  handleSubmitFailed(error) {
    showMessage('请填写有效的手机号码和验证码', 'error');
  }

  /**
   * 表单提交回调
   * @param {Object} fieldMap 字段 hashmap
   */
  handleSubmit(fieldMap) {
    const { phoneNumber, verificationCode } = fieldMap;

    this.props.onConfirm({
      phoneNumber: String(phoneNumber).trim(),
      verificationCode: String(verificationCode).trim(),
    });
  }

  render() {
    const { codeBtnDisabled, countdown } = this.state;

    return (
      <div className={styles.phonePanel}>
        <div className={styles.phonePanelTitle}>设置手机号码</div>
        <Form ref={(el) => (this.formRef = el)} onSubmit={this.handleSubmit} onSubmitFail={this.handleSubmitFailed}>
          <FormItem name="phoneNumber" rules={[{ required: true, validator: this.phoneNumberValidator }]}>
            <Input maxLength={11} allowClear placeholder="手机号码" />
          </FormItem>
          <FormItem name="verificationCode" rules={[{ required: true, validator: this.verificationCodeValidator }]}>
            <VerificationCodeRow countdown={countdown} btnDisabled={codeBtnDisabled} onBtnClick={this.handleGetCode} />
          </FormItem>
        </Form>
        <Button className={styles.phonePanelConfirmBtn} onClick={this.handleConfirm} type="primary">
          确定
        </Button>
        <Button className={styles.phonePanelCancelBtn} onClick={this.handleCancel}>
          取消
        </Button>
      </div>
    );
  }
}

export default PhoneSettingPanel;
