const webpack = require('webpack');
const path = require('path');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const TerserPlugin = require('terser-webpack-plugin');
const CssMinimizerWebpackPlugin = require("css-minimizer-webpack-plugin");
const SimpleProgressWebpackPlugin = require('simple-progress-webpack-plugin');
const theme = require('./src/components/theme/override_antd.js')();
const rootPath = path.resolve(__dirname); // 项目根目录
const srcPath = path.join(rootPath, 'src'); // 开发源码目录

module.exports = (env) => {
  require('dotenv').config({ path: `./env.${env.node_env}` });

  const isDev = env.node_env === 'development'; // 是否开发环境

  return {
    entry: path.join(srcPath, 'index.js'),
    devServer: isDev ? {
      static: {
        directory: path.join(__dirname, 'public'),
      },
      host: 'localhost', // 主机地址
      port: 8000, // 端口号
      open: ['/Dashboard'],
      devMiddleware: {
        publicPath: "/"
      },
      historyApiFallback: true,
      client: {
        overlay: {
          errors: true,
        }
      },
      proxy: {
        //临时对接本地后端接口
        // "/eduboss/boss-xinghuo-api/crm/primary-school-resource/": {
        //   "target": "http://localhost:9000/",
        //   "changeOrigin": true,
        //   "pathRewrite": {
        //     '^/eduboss/boss-xinghuo-api': ''
        //   }
        // },
        "/eduboss": {
          "target": "https://xhboss-test.xiaojiaoyu100.com",
          "changeOrigin": true
        },
        "/cmall": {
          "target": "https://xhboss-test.xiaojiaoyu100.com",
          "changeOrigin": true
        },
        "/xhmall": {
          "target": "https://xhboss-test.xiaojiaoyu100.com",
          "changeOrigin": true
        },
        "/webapp": {
          "target": process.env.PROXY_OLD_BOSS_URL,
          "changeOrigin": true
        },
        "/xh-extra": {
          "target": "https://xhboss-test.xiaojiaoyu100.com",
          "changeOrigin": true
        }
      }
    } : undefined,
    output: {
      clean: true,
      filename: '[name].js',
      path: path.resolve(__dirname, 'dist'),
      publicPath: '/',
      chunkFilename: '[name].async.js',
    },
    resolve: {
      alias: {
        SRC: srcPath,
        ASSET: path.join(srcPath, 'assets'),
        COMPONENT: path.join(srcPath, 'components'),
        MODEL: path.join(srcPath, 'models'),
        PAGE: path.join(srcPath, 'pages'),
        SERVICE: path.join(srcPath, 'services'),
        UTIL: path.join(srcPath, 'utils'),
        VIEW: path.join(srcPath, 'views'),
        CONST: path.join(srcPath, 'constants'),
        DOC: path.join(srcPath, 'documents'),
        BLOC: path.join(srcPath, 'BLoComponents'),
      },
      extensions: [
        '.js', '.json', '.jsx', '.ts', '.tsx'
      ],
      fallback: {
        fs: false,
        module: false,
        url: false
      }
    },
    resolveLoader: {
      modules: [ 'node_modules' ],
      extensions: [ '.js', '.json' ],
      mainFields: [ 'loader', 'main' ]
    },
    module: {
      rules: [
        {
          // 字体文件等
          test: /\.(woff|woff2|eot|ttf|otf)$/i,
          type: 'asset/resource',
          exclude: /node_modules/,
          parser: {
            dataUrlCondition: {
              maxSize: 1024 * 20, // 超过20kb以文件形式引入，反之以base64形式引入
            }
          },
          generator: {
            // 基础路径
            publicPath: '/',
            // 文件路径
            filename: 'static/[name].[hash:8].[ext]'
          },
        },
        { // 图片的转化
          test: /\.(jpe?g|png|gif|bmp|svg)$/i,
          type: 'asset',
          include: [path.resolve(__dirname, 'src')],
          parser: {
            dataUrlCondition: {
              maxSize: 1024 * 15,
            }
          },
          generator: {
            publicPath: '/',
            filename: 'static/[name].[hash:8].[ext]',
          },
        },
        {
          test: /\.(js|tsx|jsx)$/,
          include: [path.resolve(__dirname, 'src')],
          use: [
            'thread-loader',
            {
              loader: 'babel-loader',
              options: {
                cacheDirectory: true
              }
            }
          ]
        },
        {
          test: /\.css$/,
          use: [
            'css-hot-loader',
            'style-loader',
            {
              loader: 'css-loader',
              options: {
                importLoaders: 1,
              },
            },
          ],
        },
        {
          test: /\.less$/,
          use: [
            'css-hot-loader',
            'style-loader',
            'thread-loader',
            {
              loader: 'css-loader',
              options: {
                importLoaders: 1,
                modules: true,
                localIdentName: '[name]_[local]-[hash:base64:5]',
              },
            },
            {
              loader: 'less-loader',
              options: {
                javascriptEnabled: true,
                modifyVars: theme,
              },
            },
          ],
          exclude: /node_modules/,
        },
        {
          test: /\.less$/,
          use: [
            'style-loader',
            'thread-loader',
            {
              loader: 'css-loader',
              options: {
                importLoaders: 1,
              },
            },
            {
              loader: 'less-loader',
              options: {
                javascriptEnabled: true,
                modifyVars: theme,
              },
            },
          ],
          exclude: /src/,
        },
        {
          test: /\.html$/i,
          loader: "html-loader",
        }
      ],
    },
    devtool: isDev ? 'source-map' : false,
    optimization: {
      usedExports: true, // 配合实现 Tree Shaking，，默认 true 开启
      concatenateModules: !isDev, // 公共代码整合，生产环境下被启用
      minimize: !isDev,
      minimizer: isDev ? undefined : [
        new CssMinimizerWebpackPlugin({
          parallel: true,
          minimizerOptions: {
            preset: [
              "default",
              {
                discardComments: { removeAll: true }, // 删除注释
              },
            ],
          },
        }),
        new TerserPlugin({
          parallel: true,
          extractComments: false
        })
      ],
      splitChunks: {
        chunks: 'async',
        minSize:  40000,
        maxAsyncRequests: 5, // 最大异步请求数
        maxInitialRequests: 4, // 页面初始化最大异步请求数
        automaticNameDelimiter: '~', // 解决命名冲突
        cacheGroups: {
          baseCommon: {
            name: 'baseCommon',
            chunks: 'all',
            test: /[\\/]node_modules[\\/](react|react-dom|react-router|redux-saga|dva|react-router-dom|draft-js\/lib|core-js|moment|@antv\/data-set\/build|)[\\/]/,
            priority: -10,
          },
          UI: {
            name: 'UI',
            chunks: 'all',
            test: /[\\/]node_modules[\\/](@ant-design|antd|immutable\/dist|rc-calendar\/es|braft-finder\/dist|lodash|rc-tree\/es)[\\/]/,
            priority: -11,
          },
          someUsed: {
            name: 'someUsed',
            chunks: 'all',
            test: /[\\/]node_modules[\\/](bizcharts\/umd|echarts|print-js|html2canvas|react-color\/lib|zrender\/lib|braft-editor\/dist|rc-table|rc-animate|react-amap\/lib|rc-menu\/es|rc-select\/es|rc-trigger\/es)[\\/]/,
            priority: -12,
          },
        },
      },
    },
    plugins: [
      ...(isDev ? [new SimpleProgressWebpackPlugin()] : []),
      new webpack.IgnorePlugin({
        resourceRegExp: /^\.\/locale$/,
        contextRegExp: /moment$/,
      }),
      new webpack.DefinePlugin({
        "process.env": {
          "PROXY_OLD_BOSS": process.env.PROXY_OLD_BOSS,
          "PROXY_OLD_BOSS_URL": JSON.stringify(process.env.PROXY_OLD_BOSS_URL),
          "BUILD_ENV": JSON.stringify(process.env.BUILD_ENV)
        }
      }),
      new webpack.ProvidePlugin({
        "React": "react"
      }),
      new HtmlWebpackPlugin({
        template: './src/index.ejs', // 模板
        hash: 6,
        inject: 'body', // 在body标签下方填入标签
        // 压缩
        minify: {
          removeComments: true,    //移除HTML中的注释
          collapseWhitespace: true    //删除空白符与换行符
        },
        chunksSortMode: 'auto' // srcipt标签插入  auto
      }),
      new CopyWebpackPlugin({
        patterns: [{
          from: path.resolve(__dirname, 'public'),
          to: path.resolve(__dirname, 'dist')
        }]
      }),
      ...(isDev ? [new webpack.HotModuleReplacementPlugin()] : [])
    ],
  }
};